#include <iostream>
#include <string>

#include "GrLocalizationPose.h"
#include "emscripten/bind.h"

#include <stdio.h>
#include <cereal/types/unordered_map.hpp>
#include <cereal/types/memory.hpp>
#include <cereal/archives/binary.hpp>
#include <fstream>
#include <interfaces/baseTypes/include/Header.h>
#include <interfaces/LocalizationPose/LocalizationPose.h>

using namespace emscripten;
using namespace std;

GrLocalizationPose::GrLocalizationPose(string p_name): m_name(p_name) {}

string GrLocalizationPose::getGreeting () {
  return "Hello from Blob: Greetings " + m_name + "!";
}

bool GrLocalizationPose::readBlob(std::string p_blob, bool p_printRecord)
{
    const auto HEADER_SIZE = interfaces::baseTypes::getInterfaceHeaderSize(OTTO_INTERFACE_HEADER_VERSION);

    std::string header = p_blob.substr(0, HEADER_SIZE);
    memset(static_cast<void*>(&m_localizationPose), 0, sizeof(LocalizationPose));
    aptv::deserializeODatum(m_localizationPose, p_blob);
    
    if (p_printRecord) {
      cout << m_localizationPose.timestamp << ",";
      cout << m_localizationPose.pose << ",";
      cout << m_localizationPose.velocity << ",";
      cout << m_localizationPose.angular_rate << ",";
      cout << m_localizationPose.acceleration << ",";
      cout << m_localizationPose.epsg << ",";
      cout << m_localizationPose.health;
      cout << " <---> " << endl;
    }

    return true;
}

double GrLocalizationPose::posePositionX() { return m_localizationPose.pose.position.x; }
double GrLocalizationPose::posePositionY() { return m_localizationPose.pose.position.y; }
double GrLocalizationPose::posePositionZ() { return m_localizationPose.pose.position.z; }

double GrLocalizationPose::poseQuaternionX() { return m_localizationPose.pose.rotation.x; }
double GrLocalizationPose::poseQuaternionY() { return m_localizationPose.pose.rotation.y; }
double GrLocalizationPose::poseQuaternionZ() { return m_localizationPose.pose.rotation.z; }
double GrLocalizationPose::poseQuaternionW() { return m_localizationPose.pose.rotation.w; }

EMSCRIPTEN_BINDINGS (c) {
  class_<av::geometry::Vector3d>("Vector3d")
    .constructor<>()
    .property("x", &av::geometry::Vector3d::x)
    .property("y", &av::geometry::Vector3d::y)
    .property("z", &av::geometry::Vector3d::z)
    ;

  class_<av::geometry::Quaterniond>("Quaterniond")
    .constructor<>()
    .property("x", &av::geometry::Quaterniond::x)
    .property("y", &av::geometry::Quaterniond::y)
    .property("z", &av::geometry::Quaterniond::z)
    .property("w", &av::geometry::Quaterniond::w)
    ;

  class_<av::geometry::Pose>("Pose")
    .constructor<>()
    .property("position", &av::geometry::Pose::position)
    .property("rotation", &av::geometry::Pose::rotation)
    ;

  class_<LocalizationPose>("LocalizationPose")
    .constructor<>()
    .property("pose", &LocalizationPose::pose)
    ;

  class_<GrLocalizationPose>("GrLocalizationPose")
    .constructor<string>()
    .function("readBlob", &GrLocalizationPose::readBlob)
    .function("posePositionX", &GrLocalizationPose::posePositionX)
    .function("posePositionY", &GrLocalizationPose::posePositionY)
    .function("posePositionZ", &GrLocalizationPose::posePositionZ)
    .function("poseQuaternionX", &GrLocalizationPose::poseQuaternionX)
    .function("poseQuaternionY", &GrLocalizationPose::poseQuaternionY)
    .function("poseQuaternionZ", &GrLocalizationPose::poseQuaternionZ)
    .function("poseQuaternionW", &GrLocalizationPose::poseQuaternionW)
    .function("getGreeting", &GrLocalizationPose::getGreeting)
    .property("m_localizationPose", &GrLocalizationPose::m_localizationPose);

}
