#include <iostream>
#include <string>

#include "TestCereal.h"
#include "emscripten/bind.h"

#include <stdio.h>
#include <cereal/types/unordered_map.hpp>
#include <cereal/types/memory.hpp>
#include <cereal/archives/binary.hpp>
#include <fstream>
//#include <interfaces/baseTypes/include/Header.h>
#include <interfaces/LocalizationPose/LocalizationPose.h>
   
struct MyRecord
{
  uint8_t x, y;
  float z;
  
  template <class Archive>
  void serialize( Archive & ar )
  {
    ar( x, y, z );
  }
};
    
struct SomeData
{
  int32_t id;
  //std::shared_ptr<std::unordered_map<uint32_t, MyRecord>> data;
  MyRecord data[2];
  
  template <class Archive>
  void save( Archive & ar ) const
  {
    ar( data );
  }
      
  template <class Archive>
  void load( Archive & ar )
  {
    static int32_t idGen = 0;
    id = idGen++;
    ar( data );
  }
};

using namespace emscripten;
using namespace std;

TestCereal::TestCereal(string p_name): m_name(p_name) {}

bool TestCereal::testLib () {
  std::stringstream os;
  cereal::BinaryOutputArchive archive( os );

  SomeData myData;
  myData.id = 1;

  MyRecord a;
  a.x = 1;
  a.y = 2;
  a.z = 3;
  myData.data[0] = a;

  MyRecord b;
  b.x = 51;
  b.y = 52;
  b.z = 53;
  myData.data[1] = b;

  archive( myData ); // this archive data can be store in some binary file.

  // Now use the same stream ('os') and extract and new data structure.
  // In real application we will read the IDL cereal file and extract it into std::stringstream and .
  cereal::BinaryInputArchive iarchive(os); // Create an input archive
  SomeData myData2;

  iarchive( myData2 );
  bool result = false;
  
  if ((myData2.data[0].x == myData.data[0].x) && (myData2.data[0].y == myData.data[0].y) && (myData2.data[0].z == myData.data[0].z)) 
  {
	  if ((myData2.data[1].x == myData.data[1].x) && (myData2.data[1].y == myData.data[1].y) && (myData2.data[1].z == myData.data[1].z)) 
	  {
		  return true;
	  }
  }

  std::cout << result ? "C++ cereal lib SUCCESS!" : "C++ cereal lib FAIL!";

  return result;
}

bool TestCereal::testLocalizationPose() {
  std::stringstream os;
  cereal::BinaryOutputArchive archive( os );

  LocalizationPose poses;
  poses.timestamp = 269472;

  archive(poses); // this archive data can be store in some binary file.

  // Now use the same stream ('os') and extract and new data structure.
  // In real application we will read the IDL cereal file and extract it into std::stringstream and .
  cereal::BinaryInputArchive iarchive(os); // Create an input archive
  LocalizationPose poses2;

  iarchive(poses2);
  
  cout << "Old: " << poses.timestamp << "New: " << poses2.timestamp << endl;
  bool result = false;
  if (poses.timestamp == poses2.timestamp) 
  {
    result = true;
  }

  return result;
}

EMSCRIPTEN_BINDINGS (c) {
  class_<TestCereal>("TestCereal")
    .constructor<string>()
    .function("testLib", &TestCereal::testLib)
    .function("testLocalizationPose", &TestCereal::testLocalizationPose);
}
