/**
 * \copyright
 * Copyright (c) 2020- Aptiv. All rights reserved.
 * NOTICE: Removing this heading is prohibited.
 * All information contained herein is, and remains the property of Aptiv and its suppliers, if any.
 * The intellectual and technical concepts contained herein are proprietary to Aptiv and its suppliers and may be
 * covered by U.S. and Foreign Patents, patents in process, and are protected by trade secret or copyright law.
 * Dissemination of this information or reproduction of this material is strictly forbidden unless prior written
 * permission is obtained from Aptiv.
 *
 * \brief Classification of BlackChannel metadata.
 */

#ifndef APTIVWARECORE_BLACKCHANNELMETADATATYPE_H
#define APTIVWARECORE_BLACKCHANNELMETADATATYPE_H

#include "BlackChannel/BlackChannelKey.h"

#include <memory>

namespace bc {

/**
 * \brief An enumeration representing a black channel metadata type. When converted to a string, each metadata
 *        encoding will be prefixed with the corresponding character from this enumeration.
 */
enum class BlackChannelMetadataType : char
{
    Unknown = '\0',
    Salt = 'S',
    SaltUpdate = 'U'
};

/**
 * \brief Decode the metadata type for an input string. This checks the encoding type prefix, as well as validating the
 *        format of the string.
 * \param s The string to decode.
 * \returns A valid metadata type, or \a Unknown if the input string cannot be decoded.
 */
BlackChannelMetadataType getBlackChannelMetadataType(const std::string& s);

} // namespace bc

#endif // APTIVWARECORE_BLACKCHANNELMETADATATYPE_H
