/**
 * \copyright
 * Copyright (c) 2020- Aptiv. All rights reserved.
 * NOTICE: Removing this heading is prohibited.
 * All information contained herein is, and remains the property of Aptiv and its suppliers, if any.
 * The intellectual and technical concepts contained herein are proprietary to Aptiv and its suppliers and may be
 * covered by U.S. and Foreign Patents, patents in process, and are protected by trade secret or copyright law.
 * Dissemination of this information or reproduction of this material is strictly forbidden unless prior written
 * permission is obtained from Aptiv.
 *
 * \brief Class representing a Black Channel Salt message.
 */

#include "BlackChannel/BlackChannelSaltMetadata.h"

#include "BlackChannelMetadataType.h"

#include <algorithm>
#include <cctype>
#include <sstream>

namespace bc {

constexpr char saltPrefix = static_cast<char>(BlackChannelMetadataType::Salt);

std::string BlackChannelSaltMetadata::toString() const
{
    std::stringstream ss;
    ss << saltPrefix;
    ss << key_.toString();
    return ss.str();
}

bool BlackChannelSaltMetadata::isSaltMetadata(const std::string& s)
{
    static constexpr std::size_t expectedStringSize = sizeof(saltPrefix) + HEX_DIGITS_PER_KEY;

    return ((s.size() == expectedStringSize) && (s.front() == saltPrefix) &&
            std::all_of(s.begin() + sizeof(saltPrefix), s.end(), [](char digit) { return std::isxdigit(digit); }));
}

BlackChannelSaltMetadata BlackChannelSaltMetadata::fromString(const std::string& s)
{
    if (!isSaltMetadata(s)) {
        throw std::runtime_error("Invalid Salt metadata message");
    }

    const auto key = BlackChannelKey::fromString(s.substr(sizeof(saltPrefix)));
    return BlackChannelSaltMetadata{key};
}

} // namespace bc
