/**
 * \copyright
 * Copyright (c) 2020- Aptiv. All rights reserved.
 * NOTICE: Removing this heading is prohibited.
 * All information contained herein is, and remains the property of Aptiv and its suppliers, if any.
 * The intellectual and technical concepts contained herein are proprietary to Aptiv and its suppliers and may be
 * covered by U.S. and Foreign Patents, patents in process, and are protected by trade secret or copyright law.
 * Dissemination of this information or reproduction of this material is strictly forbidden unless prior written
 * permission is obtained from Aptiv.
 *
 * \brief Class representing a Black Channel Salt Update message.
 */

#include "BlackChannel/BlackChannelSaltUpdateMetadata.h"

#include "BlackChannelMetadataType.h"

#include <algorithm>
#include <cctype>
#include <iomanip>
#include <sstream>

namespace bc {

constexpr char saltUpdatePrefix = static_cast<char>(BlackChannelMetadataType::SaltUpdate);

std::string BlackChannelSaltUpdateMetadata::toString() const
{
    std::stringstream ss;
    ss << saltUpdatePrefix;
    ss << std::hex << std::setfill('0') << std::setw(HEX_DIGITS_PER_UINT32) << validThrough_;
    ss << key_.toString();
    return ss.str();
}

bool BlackChannelSaltUpdateMetadata::isSaltUpdateMetadata(const std::string& s)
{
    static constexpr std::size_t expectedStringSize =
        sizeof(saltUpdatePrefix) + HEX_DIGITS_PER_UINT32 + HEX_DIGITS_PER_KEY;

    return (
        (s.size() == expectedStringSize) && (s.front() == saltUpdatePrefix) &&
        std::all_of(s.begin() + sizeof(saltUpdatePrefix), s.end(), [](char digit) { return std::isxdigit(digit); }));
}

BlackChannelSaltUpdateMetadata BlackChannelSaltUpdateMetadata::fromString(const std::string& s)
{
    if (!isSaltUpdateMetadata(s)) {
        throw std::runtime_error("Invalid Salt Update metadata message");
    }

    const auto validThroughDigits = s.substr(sizeof(saltUpdatePrefix), HEX_DIGITS_PER_UINT32);
    const auto validThrough = static_cast<std::uint32_t>(std::stoul(validThroughDigits, nullptr, BASE_16));

    const auto keyDigits = s.substr(sizeof(saltUpdatePrefix) + HEX_DIGITS_PER_UINT32);
    const auto key = BlackChannelKey::fromString(keyDigits);

    return BlackChannelSaltUpdateMetadata{validThrough, key};
}

} // namespace bc
