/**
 * \copyright
 * Copyright (c) 2020- Aptiv. All rights reserved.
 * NOTICE: Removing this heading is prohibited.
 * All information contained herein is, and remains the property of Aptiv and its suppliers, if any.
 * The intellectual and technical concepts contained herein are proprietary to Aptiv and its suppliers and may be
 * covered by U.S. and Foreign Patents, patents in process, and are protected by trade secret or copyright law.
 * Dissemination of this information or reproduction of this material is strictly forbidden unless prior written
 * permission is obtained from Aptiv.
 *
 * \brief a simple BlackChannelKey class implementation, salt is an example of BlackChannelKey
 */

#ifndef APTIVWARECORE_BLACKCHANNELKEY_H
#define APTIVWARECORE_BLACKCHANNELKEY_H

#include "BlackChannelConstants.h"

#include <array>
#include <cstdint>

namespace bc {
using BCByteArray = std::array<uint8_t, BC_KEY_SIZE>;
/**
 * @brief BlackChannelKey class
 * - a helper class to provide APIs for salt related operations
 */
class BlackChannelKey final
{
public:
    /**
     * @brief default ctor
     */
    BlackChannelKey() = default;

    /**
     * @brief default dtor
     */
    ~BlackChannelKey() = default;

    /**
     * @brief ctor for BlackChannelKey, it basically basically converts a BCByteArray to BlackChannelKey, and marks it
     * valid
     *
     * @param key - BCByteArray
     */
    explicit BlackChannelKey(const BCByteArray& key) noexcept;

    /**
     * @brief getter function for BlackChannelKey
     *
     * @return BCByteArray
     */
    BCByteArray getBCKey() const noexcept { return m_key; };

    /**
     * @brief setter function for BlackChannelKey
     *
     * @param key - BCByteArray array to update the salt
     */
    void setBCKey(const BCByteArray& key) { m_key = key; };

    /**
     * @brief check if the BlackChannelKey is valid
     *
     * @return true if valid, false otherwise
     */
    bool isBCKeyValid() const noexcept { return m_validness; };

    /**
     * @brief returns string representation of the BlackChannelKey
     *
     * @return string
     */
    std::string toString() const;

    /**
     * @brief convert a string to the BlackChannelKey
     *
     * @param s - string to convert
     * @return BlackChannelKey
     */
    static BlackChannelKey fromString(const std::string& s);

    /**
     * @brief operator == for BlackChannelKey
     *
     * @param other - the other BlackChannelKey to compare
     * @return true if they're equal, false otherwise
     */
    bool operator==(const BlackChannelKey& other) const noexcept;

    /**
     * @brief operator != for BlackChannelKey
     *
     * @param other - the other BlackChannelKey to compare
     * @return true if they're not equal, false otherwise
     */
    bool operator!=(const BlackChannelKey& other) const noexcept;

private:
    BCByteArray m_key{};     ///< internal array of the BlackChannelKey
    bool m_validness{false}; ///< validness of the key
};
} // namespace bc

#endif // APTIVWARECORE_BLACKCHANNELKEY_H
