/**
 * \copyright
 * Copyright (c) 2020- Aptiv. All rights reserved.
 * NOTICE: Removing this heading is prohibited.
 * All information contained herein is, and remains the property of Aptiv and its suppliers, if any.
 * The intellectual and technical concepts contained herein are proprietary to Aptiv and its suppliers and may be
 * covered by U.S. and Foreign Patents, patents in process, and are protected by trade secret or copyright law.
 * Dissemination of this information or reproduction of this material is strictly forbidden unless prior written
 * permission is obtained from Aptiv.
 *
 * \brief Class representing a Black Channel Salt message.
 */

#ifndef APTIVWARECORE_BLACKCHANNELSALTMETADATA_H
#define APTIVWARECORE_BLACKCHANNELSALTMETADATA_H

#include "BlackChannel/BlackChannelKey.h"

namespace bc {

/**
 * \brief Class representing a Black Channel Salt message.
 *
 * Salt Message {
 *     uint8 salt[SALT_SZ]
 * }
 */
class BlackChannelSaltMetadata final
{
public:
    /**
     * \brief Construct a BlackChannelSaltMetadata instance.
     * \param key The salt.
     */
    explicit BlackChannelSaltMetadata(const BlackChannelKey& key)
        : key_{key}
    {}

    /**
     * \brief Destructor.
     */
    ~BlackChannelSaltMetadata() = default;

    /**
     * \brief Accessor for our key (salt).
     * \returns The salt value.
     */
    const BlackChannelKey& key() const noexcept { return key_; }

    /**
     * \brief Encode a string represention of this instance.
     * \returns The encoded string.
     */
    std::string toString() const;

    /**
     * \brief Determine whether the given string represents encoded Salt metadata.
     * \param s The string to classify.
     * \returns True if the string, \a s, represents Salt metadata,false otherwise.
     */
    static bool isSaltMetadata(const std::string& s);

    /**
     * \brief Decode a string representation of Salt metadata.
     * \param s The string to decode.
     * \returns A new BlackChannelSaltMetadata instance.
     */
    static BlackChannelSaltMetadata fromString(const std::string& s);

private:
    BlackChannelKey key_{};
};

} // namespace bc

#endif // APTIVWARECORE_BLACKCHANNELSALTMETADATA_H
