/// @copyright Copyright (c) 2021- Motional. All rights reserved.
///
/// NOTICE: Removing this heading is prohibited.
///
/// All information contained herein is, and remains the property of Motional,
/// its affiliates, and its suppliers (if any). The intellectual and technical
/// concepts contained herein are proprietary to Motional, its affiliates, and
/// its suppliers (if any) and may be covered by U.S. and Foreign Patents,
/// patents in process, and are protected by trade secrets or copyright law.
/// Dissemination of this information or reproduction of this material is
/// strictly forbidden unless prior written permission is obtained from Motional.
///
/// @file
/// @brief Short description of the contents of the file.

#include "platform/middleware/libs/BlackChannel/include/BlackChannel/BlackChannelMsg.h"
#include <gtest/gtest.h>
#include <stdexcept>
#include <string>

using namespace bc;

TEST(BlackChannelMsgTest, constructor)
{
    // Verify exception thrown for unsupported SHA2 types.
    EXPECT_THROW(BlackChannelMsg("", "", MESSAGE_TYPE::METADATA, SHA2Type::SHA512), std::runtime_error);
    EXPECT_THROW(BlackChannelMsg("", "", MESSAGE_TYPE::METADATA, SHA2Type::UNKNOWN), std::runtime_error);

    // Verify exception thrown if senderId length is invalid.
    EXPECT_THROW(
        BlackChannelMsg("", std::string(BC_SENDER_UUID_SIZE - 1, '0'), MESSAGE_TYPE::METADATA, SHA2Type::SHA256),
        std::runtime_error);
    EXPECT_THROW(
        BlackChannelMsg("", std::string(BC_SENDER_UUID_SIZE + 1, '0'), MESSAGE_TYPE::METADATA, SHA2Type::SHA256),
        std::runtime_error);

    // Verify exception thrown for unsupported message types
    EXPECT_THROW(BlackChannelMsg("", std::string(BC_SENDER_UUID_SIZE, '0'), MESSAGE_TYPE::INVALID, SHA2Type::SHA256),
                 std::runtime_error);

    EXPECT_NO_THROW(
        BlackChannelMsg("", std::string(BC_SENDER_UUID_SIZE, '0'), MESSAGE_TYPE::METADATA, SHA2Type::SHA256));
    EXPECT_NO_THROW(
        BlackChannelMsg("", std::string(BC_SENDER_UUID_SIZE, '0'), MESSAGE_TYPE::MESSAGE, SHA2Type::SHA256));
}

TEST(BlackChannelMsgTest, getMsgBuf)
{
    // Verify getMsgBuf returns generated BC message based on constructor inputs.
    EXPECT_EQ(BlackChannelMsg("meta", std::string(BC_SENDER_UUID_SIZE, '0'), MESSAGE_TYPE::METADATA, SHA2Type::SHA256)
                  .getMsgBuf()
                  .substr(HEADER_SIZE_SHA256_FULL),
              std::string(BC_SENDER_UUID_SIZE, '0') + '#' + "meta");
    EXPECT_EQ(BlackChannelMsg("message", std::string(BC_SENDER_UUID_SIZE, '1'), MESSAGE_TYPE::MESSAGE, SHA2Type::SHA256)
                  .getMsgBuf()
                  .substr(HEADER_SIZE_SHA256_FULL),
              std::string(BC_SENDER_UUID_SIZE, '1') + '&' + "message");
}
