/// @copyright Copyright (c) 2021- Motional. All rights reserved.
///
/// NOTICE: Removing this heading is prohibited.
///
/// All information contained herein is, and remains the property of Motional,
/// its affiliates, and its suppliers (if any). The intellectual and technical
/// concepts contained herein are proprietary to Motional, its affiliates, and
/// its suppliers (if any) and may be covered by U.S. and Foreign Patents,
/// patents in process, and are protected by trade secrets or copyright law.
/// Dissemination of this information or reproduction of this material is
/// strictly forbidden unless prior written permission is obtained from Motional.
///
/// @file
/// @brief Short description of the contents of the file.

#include "platform/middleware/libs/BlackChannel/include/BlackChannel/BlackChannelSaltMetadata.h"
#include <gtest/gtest.h>

using namespace bc;

static BlackChannelKey createKey(std::uint8_t fillValue)
{
    BCByteArray key{};
    key.fill(fillValue);
    return BlackChannelKey(key);
}

TEST(BlackChannelSaltMetadataTest, constructor)
{
    EXPECT_NO_THROW(BlackChannelSaltMetadata({}));
}

TEST(BlackChannelSaltMetadataTest, key)
{
    // Verify key() returns key passed during construction.
    EXPECT_EQ(BlackChannelSaltMetadata({}).key(), BlackChannelKey());
    EXPECT_EQ(BlackChannelSaltMetadata(createKey(0x55)).key(), createKey(0x55));
}

TEST(BlackChannelSaltMetadataTest, fromStringToString)
{
    const std::string validKey = "0123456789abcdef0123456789abcdef0123456789abcdef0123456789abcdef";
    const std::string tooSmallKey = "0123456789abcdef0123456789abcdef0123456789abcdef0123456789abcde";
    const std::string tooLargeKey = "0123456789abcdef0123456789abcdef0123456789abcdef0123456789abcdef0";
    const std::string invalidCharKey = "0123456789abcdef0123456789abcdef0123456789abcdef0123456789abcdeg";

    for (const std::string& key : {validKey, "S" + tooSmallKey, "S" + tooLargeKey, "S" + invalidCharKey}) {
        EXPECT_THROW(BlackChannelSaltMetadata::fromString(key), std::runtime_error);
    }
    EXPECT_THROW(BlackChannelSaltMetadata::fromString("U" + validKey), std::runtime_error);
    EXPECT_NO_THROW(BlackChannelSaltMetadata::fromString("S" + validKey).toString());

    EXPECT_EQ(BlackChannelSaltMetadata::fromString("S" + validKey).toString(), "S" + validKey);
}
