/// @copyright Copyright (c) 2021- Motional. All rights reserved.
///
/// NOTICE: Removing this heading is prohibited.
///
/// All information contained herein is, and remains the property of Motional,
/// its affiliates, and its suppliers (if any). The intellectual and technical
/// concepts contained herein are proprietary to Motional, its affiliates, and
/// its suppliers (if any) and may be covered by U.S. and Foreign Patents,
/// patents in process, and are protected by trade secrets or copyright law.
/// Dissemination of this information or reproduction of this material is
/// strictly forbidden unless prior written permission is obtained from Motional.
///
/// @file
/// @brief Short description of the contents of the file.

#include "platform/middleware/libs/BlackChannel/include/BlackChannel/BlackChannelSaltUpdateMetadata.h"
#include <gtest/gtest.h>

using namespace bc;

static BlackChannelKey createKey(std::uint8_t fillValue)
{
    BCByteArray key{};
    key.fill(fillValue);
    return BlackChannelKey(key);
}

TEST(BlackChannelSaltUpdateMetadataTest, constructor)
{
    EXPECT_NO_THROW(BlackChannelSaltUpdateMetadata(0U, {}));
}

TEST(BlackChannelSaltUpdateMetadataTest, key)
{
    // Verify key() returns key passed during construction.
    EXPECT_EQ(BlackChannelSaltUpdateMetadata(0, {}).key(), BlackChannelKey());
    EXPECT_EQ(BlackChannelSaltUpdateMetadata(0, createKey(0x55)).key(), createKey(0x55));
}

TEST(BlackChannelSaltUpdateMetadataTest, validThrough)
{
    // Verify key() returns key passed during construction.
    EXPECT_EQ(BlackChannelSaltUpdateMetadata(0, {}).validThrough(), 0);
    EXPECT_EQ(BlackChannelSaltUpdateMetadata(std::numeric_limits<std::uint32_t>::max(), {}).validThrough(),
              std::numeric_limits<std::uint32_t>::max());
}

TEST(BlackChannelSaltUpdateMetadataTest, fromStringToString)
{
    const std::string validThroughDigits = "00000000";
    const std::string validKey = "0123456789abcdef0123456789abcdef0123456789abcdef0123456789abcdef";
    const std::string tooSmallKey = "0123456789abcdef0123456789abcdef0123456789abcdef0123456789abcde";
    const std::string tooLargeKey = "0123456789abcdef0123456789abcdef0123456789abcdef0123456789abcdef0";
    const std::string invalidCharKey = "0123456789abcdef0123456789abcdef0123456789abcdef0123456789abcdeg";

    for (const std::string& key : {"U" + validKey, "U" + tooSmallKey, "U" + tooLargeKey, "U" + invalidCharKey}) {
        EXPECT_THROW(BlackChannelSaltUpdateMetadata::fromString(key), std::runtime_error);
    }
    EXPECT_THROW(BlackChannelSaltUpdateMetadata::fromString("S" + validThroughDigits + validKey), std::runtime_error);
    EXPECT_NO_THROW(BlackChannelSaltUpdateMetadata::fromString("U" + validThroughDigits + validKey).toString());

    EXPECT_EQ(BlackChannelSaltUpdateMetadata::fromString("U" + validThroughDigits + validKey).toString(),
              "U" + validThroughDigits + validKey);
}
