/**
 * \copyright
 * Copyright (c) 2021- Motional. All rights reserved.
 * NOTICE: Removing this heading is prohibited.
 * All information contained herein is, and remains the property of Motional, its affiliates, and its suppliers (if
 * any). The intellectual and technical concepts contained herein are proprietary to Motional, its affiliates, and its
 * suppliers (if any) and may be covered by U.S. and Foreign Patents, patents in process, and are protected by trade
 * secrets or copyright law. Dissemination of this information or reproduction of this material is strictly forbidden
 * unless prior written permission is obtained from Motional.
 *
 * \file
 * \brief Defines a common interface to interact with two types of clock. A generic Clock that simply returns time,
 * and a clock which uses the PlaybackClockParams type and provides an interface to get its parameters.
 */

#ifndef CLOCK_SOURCE_H
#define CLOCK_SOURCE_H

#include <TimeStamp/otime.h>
#include <interfaces/Clock/Playback/PlaybackClockParams.h>

#include <stdint.h>

#include <optional>

/**
 * \brief An abstract clock object that will return the current time.
 */
class Clock
{
public:
    /**
     * \brief Default destructor
     */
    virtual ~Clock() = default;

    /**
     * \brief Return the current time.
     *
     * \return Current time as otimePoint
     */
    virtual ottotime::otimePoint getCurrentTime() = 0;
};

/**
 * \brief A clock that uses the PlaybackClockParams and will make those params available.
 */
class ClockParamsClock : public Clock
{
public:
    /**
     * \brief Default destructor
     */
    virtual ~ClockParamsClock() = default;

    /**
     * \brief Return the current clock parameters
     *
     * \return The current clock parameters
     */
    virtual std::optional<PlaybackClockParams> getParams() = 0;
};

#endif
