/**
 * \copyright
 * Copyright (c) 2021- Motional. All rights reserved.
 * NOTICE: Removing this heading is prohibited.
 * All information contained herein is, and remains the property of Motional, its affiliates, and its suppliers (if
 * any). The intellectual and technical concepts contained herein are proprietary to Motional, its affiliates, and its
 * suppliers (if any) and may be covered by U.S. and Foreign Patents, patents in process, and are protected by trade
 * secrets or copyright law. Dissemination of this information or reproduction of this material is strictly forbidden
 * unless prior written permission is obtained from Motional.
 *
 * \file
 * \brief An external clock implementation that can be given an interface for PlaybackClockParams and a base Clock.
 *
 * Parameters from the PlaybackClockParams are used to transform the base Clock. This is intended to replace the
 * ClockPlaybackClient interface.
 */

#include <Comms/CommsPlaybackClock.h>

CommsPlaybackClock::CommsPlaybackClock(ParamsInputIntf* interface, Clock* baseClock)
    : m_paramsInput{interface}
    , m_baseClock{baseClock}
{}

ottotime::otimePoint CommsPlaybackClock::getCurrentTime()
{
    otimePoint baseTime{};
    otimePoint result{};

    fetchParams();

    if (m_baseClock != nullptr && m_params.has_value()) {
        baseTime = m_baseClock->getCurrentTime();
        result = (*m_params).apply(baseTime);
    }

    return result;
}

std::optional<PlaybackClockParams> CommsPlaybackClock::getParams()
{
    fetchParams();

    return m_params;
}

void CommsPlaybackClock::fetchParams()
{
    if (m_paramsInput != nullptr) {
        bool hasParams{false};
        PlaybackClockParams params{};

        while (m_paramsInput->get(params)) {
            // Read until the latest message
            hasParams = true;
        }

        if (hasParams) {
            m_params = params;
        }
    }
}
