#include <Comms/OttoConfig.h>

OttoConfig::OttoConfig(const std::string& configFile)
{
    m_configFilePath = configFile;
    // Parse the config file to get all of the requested interfaces
    if (!m_interfaceParser.parseFile(m_configFilePath)) {
        throw std::runtime_error("OttoComms: Failed to parse file at: " + m_configFilePath);
    }
    for (const ConfigSection& cfg : m_interfaceParser.getAllReadOnlyInterfaces()) {
        m_readOnlyInterfaces.emplace(cfg.getName(), cfg);
    }
    for (const ConfigSection& cfg : m_interfaceParser.getAllWriteOnlyInterfaces()) {
        m_writeOnlyInterfaces.emplace(cfg.getName(), cfg);
    }
}

std::vector<std::string> OttoConfig::getChannels(const std::string& parameter)
{
    ConfigSection section;
    if (!m_interfaceParser.getSection("Config", section)) {
        throw std::runtime_error("OttoConfig: couldn't get section for: 'Config'");
    }
    if (!section.hasParameter(parameter)) {
        std::ostringstream oss;
        oss << "OttoConfig: Could not get parameter \"" << parameter << "\" in Config of \"" << m_configFilePath
            << "\"";
        throw std::runtime_error(oss.str());
    }
    std::vector<std::string> channels;
    section.getArray(parameter, channels);
    return channels;
}

std::optional<ConfigSection> OttoConfig::getReadOnlyInterfaceConfig(const std::string& key) const
{
    const auto itr = m_readOnlyInterfaces.find("ReadOnlyInterfaces." + key);
    return itr != m_readOnlyInterfaces.cend() ? std::optional<ConfigSection>{itr->second} : std::nullopt;
}

std::optional<ConfigSection> OttoConfig::getWriteOnlyInterfaceConfig(const std::string& key) const
{
    const auto itr = m_writeOnlyInterfaces.find("WriteOnlyInterfaces." + key);
    return itr != m_writeOnlyInterfaces.cend() ? std::optional<ConfigSection>{itr->second} : std::nullopt;
}

ConfigSection OttoConfig::getSection(const std::string& name) const
{
    ConfigSection section;
    if (!m_interfaceParser.getSection(name, section)) {
        throw std::runtime_error("OttoComms: couldn't get section for: " + name);
    }
    return section;
}
