/*!
 ***********************************************************
 * @file InterfaceDb.h
 * @brief A file defining the InterfaceDb class
 *
 * @section License
 *
 * Copyright (c) 2015- Delphi Automotive. All rights
 * reserved.
 *
 * @section changeLogs Change Logs
 *
 ***********************************************************
 */

/**********************************************************
  Copyright (c) 2014-2015 Ottomatika, Inc.
 **********************************************************/

/**********************************************************
  Copyright (c) 2010-2013 Carnegie Mellon University,
  All rights reserved.

  This source code was developed in part with support from
  sponsors including General Motors, the National Science
  Foundation and the US Department of Transportation.

  Use of this software is strictly governed by agreements
  between Carnegie Mellon University and various sponsors.

  Users of this software must also be fully aware and agree
  that Carnegie Mellon does not guarantee the correct
  functioning of this software in any system. Carnegie
  Mellon or any of its affiliates will not be liable for any
  damage and/or penalties resulting from the use of this
  software. Any user of this software takes complete
  responsibility for the use of any software or design.
 **********************************************************/

/*
 ***********************************************************
 * InterfaceDb.h
 * Bryan Salesky
 * 09/02/2006
 *
 * Junsung Kim
 * 12/16/2012
 * A function chmod() is added to support fault-tolerance.
 *
 * Copyright (C) 2006 CMU. All rights reserved.
 ***********************************************************
 */
#ifndef INTERFACE_DB_H
#define INTERFACE_DB_H

#include "Comms/Access.h"
#include <map>
#include <memory>

#include "lib/avlog/avlog.hpp"

namespace task {

/*!
 * @class IDbObject
 *
 * @brief This class shall be designed as an abstract class from which all the classes
 * \n in InterfaceDb must be inherited.
 */
class IDbObject
{
public:
    /*!
     * @brief Default constructor.
     */
    IDbObject() {}

    /*!
     * Default virtual destructor.
     */
    virtual ~IDbObject() {}

    /*!
     * @brief This method shall initialize the interface.
     *
     * @return bool
     */
    virtual bool initialize(void) = 0;
    /*!
     * @brief This method shall clear the state of the channel.
     *
     * @return void
     */
    virtual void cleanup(void) = 0;
    /*!
     * @brief This method shall change the access mode of the interface.
     *
     * @param[in] acc as Access enum : The new mode to be changed [Read/Write/ReadWrite/Standby].
     *
     * @return void
     */
    virtual void chmod([[maybe_unused]] const task::Access& acc) { return; }

protected:
private:
};

} // namespace task

#endif // INTERFACE_DB_H
