/**
 * \copyright
 * Copyright (c) 2021- Motional. All rights reserved.
 * NOTICE: Removing this heading is prohibited.
 * All information contained herein is, and remains the property of Motional, its affiliates, and its suppliers (if
 * any). The intellectual and technical concepts contained herein are proprietary to Motional, its affiliates, and its
 * suppliers (if any) and may be covered by U.S. and Foreign Patents, patents in process, and are protected by trade
 * secrets or copyright law. Dissemination of this information or reproduction of this material is strictly forbidden
 * unless prior written permission is obtained from Motional.
 *
 * \file
 * \brief DDS Quality of Service Helper
 */
#pragma once

#include "lib/avlog/avlog.hpp"
#include <dds/dds.hpp>

#include <optional>
#include <string>

namespace DDSComms {

/// Name used for logging purposes when we do not know what QoS profile is active.
static const std::string unknownProfileName{"<UNKNOWN>"};

class DDSConfigurator
{

public:
    DDSConfigurator() = default;
    virtual ~DDSConfigurator() = default;

    /// Create a DataReader QoS profile.
    /// the QoS will be adjusted such that it's history queue depth is receiveCapacityOpt if it is provided.
    /// If history queue depth is adjusted, sample resource levels and history kind will also be adjusted
    /// conservatively.
    /// @param receiveCapacityOpt Depth of history receive queue. Must be <= than numeric_limits<int>::max()
    /// @param channelName the name of the channel this QoS is for
    virtual dds::sub::qos::DataReaderQos createReaderQos(
        const std::optional<uint32_t>& receiveCapacityOpt = std::nullopt,
        const std::optional<std::string>& channelName = std::nullopt) const = 0;

    /// create a DataWriter Qos profile.
    /// @param channelName the name of the channel this QoS is for
    virtual dds::pub::qos::DataWriterQos createWriterQos(
        const std::optional<std::string>& channelName = std::nullopt) const = 0;

    /// create a DomainParticipant Qos profile.
    virtual dds::domain::qos::DomainParticipantQos createParticipantQos() const = 0;
};

void applyDataReaderQueueDepth(uint32_t receiveCapacityOpt, dds::sub::qos::DataReaderQos& qos);

/// Get the contents of the OTTO_DIR environment variable
/// @return The contents of OTTO_DIR if set, otherwise '.'
inline std::string getOttoDirectory()
{
    auto buf = getenv("OTTO_DIR"); // NOLINT(cppcoreguidelines-pro-type-vararg)
    if (buf != nullptr) {
        return buf;
    }
    return ".";
}

} // namespace DDSComms
