/**
 * \copyright
 * Copyright (c) 2021- Motional. All rights reserved.
 * NOTICE: Removing this heading is prohibited.
 * All information contained herein is, and remains the property of Motional, its affiliates, and its suppliers (if
 * any). The intellectual and technical concepts contained herein are proprietary to Motional, its affiliates, and its
 * suppliers (if any) and may be covered by U.S. and Foreign Patents, patents in process, and are protected by trade
 * secrets or copyright law. Dissemination of this information or reproduction of this material is strictly forbidden
 * unless prior written permission is obtained from Motional.
 *
 * \file
 * \brief abstracts the compile-time differences of configuring DDS implementations
 */
#pragma once

#include <dds/dds.hpp>

#ifdef DDS_VERSION_RTI_DDS_CPP
#include "RtiProConfigurator.hpp"
#include "UserBytesTopicType.hpp"
#else
#include "CycloneConfigurator.hpp"
#include "UserBytesTopicType.hpp"
#endif

namespace DDSComms {

using PayloadType = BytesMsgType;
using BytesTopicType = BytesMsgType;

#ifdef DDS_VERSION_RTI_DDS_CPP
using DDSConfiguratorType = RtiProConfigurator;
using DDSOptionsType = RtiProOptions;
using GenericTopicType = dds::core::BytesTopicType;
#else
using DDSConfiguratorType = CycloneConfigurator;
using DDSOptionsType = CycloneOptions;
using GenericTopicType = BytesTopicType;
#endif

/// Create a new BytesTopicType from a string payload holding serialized bytes
BytesTopicType createMsg(const std::string& sendMsg,
                         const uint32_t& sequenceNumber = 0,
                         const std::string& origin = "");

/// Create a new string holding serialized bytes from a BytesTopicType
std::string getMsgData(const BytesTopicType& sample);

/// set the sequence number on the BytesTopicType.
void setSequenceNumber(BytesTopicType& msg, const uint32_t& sequenceNumber);

} // namespace DDSComms