/**
 * @copyright
 * Copyright (c) 2022- Motional. All rights reserved.
 * NOTICE: Removing this heading is prohibited.
 * All information contained herein is, and remains the property of Motional, its affiliates, and its suppliers (if
 * any). The intellectual and technical concepts contained herein are proprietary to Motional, its affiliates, and its
 * suppliers (if any) and may be covered by U.S. and Foreign Patents, patents in process, and are protected by trade
 * secrets or copyright law. Dissemination of this information or reproduction of this material is strictly forbidden
 * unless prior written permission is obtained from Motional.
 *
 * @file DDSConfiguratorOptions.h
 * @brief DDS DDS Configurator Options
 */
#pragma once

#include <memory>
#include <optional>
#include <string>
#include <unordered_map>

namespace DDSComms {

/// Configuration options for Cyclone DDS
struct CycloneOptions
{
    /// Filepath for the cyclone config file
    /// This may ne absolute or in relation to OTTO_DIR
    std::string configFileName{"not supported yet"};
    /// profile name to use for the domain participant, if not given, default will be used
    std::optional<std::string> domainParticipantProfile{std::nullopt};
    /// default profile to be used for creating data readers.
    std::optional<std::string> defaultReadChannelProfile{"AVQosLib::Generic.KeepLast50Reliable"};
    /// default profile to be used for creating data readers.
    std::optional<std::string> defaultWriteChannelProfile{"AVQosLib::Generic.KeepLast25Reliable"};
    /// A mapping of read channel names to the QoS profile name that channel is to use.
    /// Falls back to defaultReadChannelProfile if set, otherwise uses DDS default
    std::unordered_map<std::string, std::string> readChannelProfileMap{};
    /// A mapping of write channel names to the QoS profile name that channel is to use.
    /// Falls back to defaultWriteChannelProfile if set, otherwise uses DDS default
    std::unordered_map<std::string, std::string> writeChannelProfileMap{};
};

struct RtiProOptions
{
    /// Filepath for the RTI DDS QoS XML config file
    /// This may ne absolute or in relation to OTTO_DIR
    std::string configFileName{"config/common/USER_QOS_PROFILES.xml"};
    /// profile name to use for the domain participant, if not given, default will be used
    std::optional<std::string> domainParticipantProfile{std::nullopt};
    /// default profile to be used for creating data readers.
    std::optional<std::string> defaultReadChannelProfile{"AVQosLib::Generic.KeepLast50Reliable"};
    /// default profile to be used for creating data readers.
    std::optional<std::string> defaultWriteChannelProfile{"AVQosLib::Generic.KeepLast25Reliable"};
    /// A mapping of read channel names to the QoS profile name that channel is to use.
    /// Falls back to defaultReadChannelProfile if set, otherwise uses DDS default
    std::unordered_map<std::string, std::string> readChannelProfileMap{};
    /// A mapping of write channel names to the QoS profile name that channel is to use.
    /// Falls back to defaultWriteChannelProfile if set, otherwise uses DDS default
    std::unordered_map<std::string, std::string> writeChannelProfileMap{};
};

#ifdef DDS_VERSION_RTI_DDS_CPP
using DDSOptionsType = RtiProOptions;
#else
using DDSOptionsType = CycloneOptions;
#endif

} // namespace DDSComms
