/**
 * \copyright
 * Copyright (c) 2021- Motional. All rights reserved.
 * NOTICE: Removing this heading is prohibited.
 * All information contained herein is, and remains the property of Motional, its affiliates, and its suppliers (if
 * any). The intellectual and technical concepts contained herein are proprietary to Motional, its affiliates, and its
 * suppliers (if any) and may be covered by U.S. and Foreign Patents, patents in process, and are protected by trade
 * secrets or copyright law. Dissemination of this information or reproduction of this material is strictly forbidden
 * unless prior written permission is obtained from Motional.
 *
 * \file
 * \brief DDS Quality of Service Helper
 */
#pragma once

#include <string>

#include "lib/avlog/avlog.hpp"
#include <dds/dds.hpp>

namespace DDSComms {

template<typename T>
class DataWriterListener : public dds::pub::NoOpDataWriterListener<T>
{
public:
    DataWriterListener(std::string topicName)
        : topicName_(std::move(topicName))
    {}

    void on_publication_matched([[maybe_unused]] dds::pub::DataWriter<T>& writer,
                                const dds::core::status::PublicationMatchedStatus& status) override
    {
        AVLOG_INFO("on_publication_matched callback: writer {}; total_count = {}; total_count_change = {}; "
                   "current_count = {}; current_count_change = {}; ",
                   topicName_,
                   status.total_count(),
                   status.total_count_change(),
                   status.current_count(),
                   status.current_count_change());
    }
    void on_offered_incompatible_qos([[maybe_unused]] dds::pub::DataWriter<T>& writer,
                                     const dds::core::status::OfferedIncompatibleQosStatus& status) override
    {
        AVLOG_ERROR("on_offered_incompatible_qos callback: writer {}; total_count = {}; total_count_change = {}; "
                    "last_policy_id = {};  ",
                    topicName_,
                    status.total_count(),
                    status.total_count_change(),
                    status.last_policy_id());

        dds::core::policy::QosPolicyCountSeq qos_seq = status.policies();
        if (!qos_seq.empty()) {
            AVLOG_ERROR("policy_id of one first incompatible Qos policy = {}", qos_seq[0].policy_id());
        }
    }
    void on_offered_deadline_missed([[maybe_unused]] dds::pub::DataWriter<T>& writer,
                                    const dds::core::status::OfferedDeadlineMissedStatus& status) override
    {
        AVLOG_WARN("on_offered_deadline_missed callback: writer {}; total_count = {}; total_count_change = {}  ",
                   topicName_,
                   status.total_count(),
                   status.total_count_change());
    }
    void on_liveliness_lost([[maybe_unused]] dds::pub::DataWriter<T>& writer,
                            const dds::core::status::LivelinessLostStatus& status) override
    {
        AVLOG_WARN("on_liveliness_lost callback: writer {}; total_count = {}; total_count_change = {}; ",
                   topicName_,
                   status.total_count(),
                   status.total_count_change());
    }

private:
    std::string topicName_{};
};

} // namespace DDSComms
