#include <gtest/gtest.h>
#include <iostream>
#include <string>

#include "DDSConfiguratorHelper.hpp"
#include <dds/dds.hpp>

#include "platform/middleware/libs/DDSComms/DataReaderListener.hpp"
#include "platform/middleware/libs/DDSComms/DataWriterListener.hpp"

using namespace std::placeholders;

namespace {

using namespace DDSComms;

const std::string testChannelName{"ListenerTestChannel"};

//! @brief Test DDS Writer Listener
class DDSWriterListenerTest : public ::testing::Test
{

protected:
    using PayloadType = GenericTopicType;
    DataWriterListener<PayloadType> listener{testChannelName};
    dds::domain::DomainParticipant dp{std::stoi(std::getenv("DDS_DOMAIN_ID"))};
    dds::pub::Publisher publisher{dp};
    dds::topic::Topic<PayloadType> topic{dp, testChannelName};
    dds::pub::DataWriter<PayloadType> writer{publisher, topic};

    void SetUp() override {}

    void TearDown() override {}
};

TEST_F(DDSWriterListenerTest, on_publication_matched)
{
    EXPECT_NO_THROW(dds::core::status::PublicationMatchedStatus status{};
                    listener.on_publication_matched(writer, status););
}

TEST_F(DDSWriterListenerTest, on_offered_incompatible_qos)
{
    EXPECT_NO_THROW(dds::core::status::OfferedIncompatibleQosStatus status{};
                    listener.on_offered_incompatible_qos(writer, status););
}

TEST_F(DDSWriterListenerTest, on_offered_deadline_missed)
{
    EXPECT_NO_THROW(dds::core::status::OfferedDeadlineMissedStatus status{};
                    listener.on_offered_deadline_missed(writer, status););
}

TEST_F(DDSWriterListenerTest, on_liveliness_lost)
{
    EXPECT_NO_THROW(dds::core::status::LivelinessLostStatus status{}; listener.on_liveliness_lost(writer, status););
}

//! @brief Test DDS Reader Listener
class DDSReaderListenerTest : public ::testing::Test
{

protected:
    DataReaderListener<PayloadType> listener{testChannelName};

    dds::domain::DomainParticipant dp{std::stoi(std::getenv("DDS_DOMAIN_ID"))};
    dds::sub::Subscriber publisher{dp};
    dds::topic::Topic<PayloadType> topic{dp, testChannelName};
    dds::sub::DataReader<PayloadType> reader{publisher, topic};

    void SetUp() override {}

    void TearDown() override {}
};

TEST_F(DDSReaderListenerTest, on_sample_lost)
{
    EXPECT_NO_THROW(dds::core::status::SampleLostStatus status{}; listener.on_sample_lost(reader, status););
}

TEST_F(DDSReaderListenerTest, on_subscription_matched)
{
    EXPECT_NO_THROW(dds::core::status::SubscriptionMatchedStatus status{};
                    listener.on_subscription_matched(reader, status););
}

TEST_F(DDSReaderListenerTest, on_sample_rejected)
{
    EXPECT_NO_THROW(dds::core::status::SampleRejectedStatus status{}; listener.on_sample_rejected(reader, status););
}

TEST_F(DDSReaderListenerTest, on_requested_incompatible_qos)
{
    EXPECT_NO_THROW(dds::core::status::RequestedIncompatibleQosStatus status{};
                    listener.on_requested_incompatible_qos(reader, status););
}

TEST_F(DDSReaderListenerTest, on_requested_deadline_missed)
{
    EXPECT_NO_THROW(dds::core::status::RequestedDeadlineMissedStatus status{};
                    listener.on_requested_deadline_missed(reader, status););
}

TEST_F(DDSReaderListenerTest, on_liveliness_changed)
{
    EXPECT_NO_THROW(dds::core::status::LivelinessChangedStatus status{};
                    listener.on_liveliness_changed(reader, status););
}

TEST_F(DDSReaderListenerTest, on_data_available)
{
    EXPECT_NO_THROW(listener.on_data_available(reader););
}

} // namespace