/**
 * \copyright
 * Copyright (c) 2021- Motional. All rights reserved.
 * NOTICE: Removing this heading is prohibited.
 * All information contained herein is, and remains the property of Motional, its affiliates, and its suppliers (if
 * any). The intellectual and technical concepts contained herein are proprietary to Motional, its affiliates, and its
 * suppliers (if any) and may be covered by U.S. and Foreign Patents, patents in process, and are protected by trade
 * secrets or copyright law. Dissemination of this information or reproduction of this material is strictly forbidden
 * unless prior written permission is obtained from Motional.
 *
 * \file
 * \brief Helpers for Testing with DDS
 */
#pragma once

#include "platform/middleware/libs/DDSComms/test/DDSTestPathsHelper.hpp"

#ifdef DDS_VERSION_RTI_DDS_CPP
#include "RtiProConfigurator.hpp"
#else
#include "CycloneConfigurator.hpp"
#endif

namespace DDSComms::testing {

#ifdef DDS_VERSION_RTI_DDS_CPP
using DDSConfiguratorType = RtiProConfigurator;
constexpr auto reliableProf = "AVQosLib::Generic.StrictReliable";
const RtiProOptions defaultTestingOptions{configFile, reliableProf, reliableProf, reliableProf};
#else
using DDSConfiguratorType = CycloneConfigurator;
const CycloneOptions defaultTestingOptions{configFile};
#endif

/// Lazily create and then return a DDConfigurator singleton, setup for unit testing
/// We can reuse the same configurator across tests, but we cannot staticly initialize it
/// because AVLOG may not yet be initialized.
auto getTestCfg()
{
    static std::shared_ptr<DDSConfiguratorType> configuratorSingleton{nullptr};

    if (!configuratorSingleton) {
        configuratorSingleton = std::make_shared<DDSConfiguratorType>(defaultTestingOptions);
    }
    return configuratorSingleton;
}

} // namespace DDSComms::testing