/**
 * \copyright
 * Copyright (c) 2021- Motional. All rights reserved.
 * NOTICE: Removing this heading is prohibited.
 * All information contained herein is, and remains the property of Motional, its affiliates, and its suppliers (if
 * any). The intellectual and technical concepts contained herein are proprietary to Motional, its affiliates, and its
 * suppliers (if any) and may be covered by U.S. and Foreign Patents, patents in process, and are protected by trade
 * secrets or copyright law. Dissemination of this information or reproduction of this material is strictly forbidden
 * unless prior written permission is obtained from Motional.
 *
 * \file
 * \brief Definition of a fake receiver class.
 */
#pragma once

#include <condition_variable>
#include <functional>
#include <mutex>
#include <vector>

#include "SimpleComms/Client.h"
#include "lib/avlog/avlog.hpp"

using namespace std::placeholders;
using namespace std::chrono_literals;

struct FakeReceiver
{
    std::vector<std::string> received;
    std::mutex m_mutex;
    std::condition_variable m_condition;
    uint32_t m_numExpectedMsgs{1};

    ~FakeReceiver() { AVLOG_DEBUG("FakeReceiver: received: #{} msgs", received.size()); }

    void call(const std::string& channel, const std::string& data, const SimpleComms::Client::MessageKey& msgKey)
    {
        AVLOG_DEBUG("FakeReceiver: CB Msg: {}; {}; {}; {}", channel, data, msgKey.first, msgKey.second);
        {
            std::unique_lock<std::mutex> lock(m_mutex);
            received.push_back(data);
        }
        m_condition.notify_all();
    }

    SimpleComms::Client::Callback getBoundCallback() { return std::bind(&FakeReceiver::call, this, _1, _2, _3); }

    uint32_t getNumReceivedData() const noexcept { return static_cast<uint32_t>(received.size()); }

    void setExpectMsgs(uint32_t numMsgs) { m_numExpectedMsgs = numMsgs; }

    bool wait(const std::chrono::milliseconds& timeout = 5000ms)
    {
        std::unique_lock<std::mutex> lock(m_mutex);
        return m_condition.wait_for(
            lock, timeout, [this] { return static_cast<uint32_t>(received.size()) >= m_numExpectedMsgs; });
    }
};
