#include <gtest/gtest.h>

#include <iostream>
#include <string>
#include <vector>

#include <dds/dds.hpp>
#include <rti/core/SequenceNumber.hpp>

using SequenceNumber = rti::core::SequenceNumber;

namespace {

/// The intention of these tests is to demonstrate the behavior of the RTI
/// SequenceNumber whenever we roll over maximum sizes of integer types.
/// This shows that we can expect sequence IDs up to the uint32 max to be
/// represented in the `low()` portion of the structure, as expected.

TEST(RtiDdsTest_MsgKeyConversion, test_1)
{
    SequenceNumber n(0, 1);

    EXPECT_EQ(n.high(), 0);
    EXPECT_EQ(n.low(), 1);
}

TEST(RtiDdsTest_MsgKeyConversion, test_2)
{
    SequenceNumber n(1);

    EXPECT_EQ(n.high(), 0);
    EXPECT_EQ(n.low(), 1);
}

TEST(RtiDdsTest_MsgKeyConversion, test_3)
{
    SequenceNumber n(255);

    EXPECT_EQ(n.high(), 0);
    EXPECT_EQ(n.low(), 255);
}

TEST(RtiDdsTest_MsgKeyConversion, test_4)
{
    SequenceNumber n(2147483647);

    EXPECT_EQ(n.high(), 0);
    EXPECT_EQ(n.low(), 2147483647);
}

TEST(RtiDdsTest_MsgKeyConversion, test_5)
{
    SequenceNumber n(2147483648);

    EXPECT_EQ(n.high(), 0);
    EXPECT_EQ(n.low(), 2147483648);
}

TEST(RtiDdsTest_MsgKeyConversion, test_6)
{
    SequenceNumber n(2147483649);

    EXPECT_EQ(n.high(), 0);
    EXPECT_EQ(n.low(), 2147483649);
}

TEST(RtiDdsTest_MsgKeyConversion, test_7)
{
    // largest number we can fit in a uint32
    SequenceNumber n(4294967295);

    EXPECT_EQ(n.high(), 0);
    EXPECT_EQ(n.low(), 4294967295);
}

TEST(RtiDdsTest_MsgKeyConversion, test_8)
{
    SequenceNumber n(4294967296);

    EXPECT_EQ(n.high(), 1);
    EXPECT_EQ(n.low(), 0);
}

TEST(RtiDdsTest_MsgKeyConversion, test_9)
{
    SequenceNumber n(4294967297);

    EXPECT_EQ(n.high(), 1);
    EXPECT_EQ(n.low(), 1);
}

TEST(RtiDdsTest_MsgKeyConversion, test_10)
{
    // largest positive number which can be rep in signed int 64
    SequenceNumber n(9223372036854775807);

    EXPECT_EQ(n.high(), 2147483647);
    EXPECT_EQ(n.low(), 4294967295);
}

TEST(RtiDdsTest_MsgKeyConversion, test_11)
{
    // overflow 9223372036854775808
    SequenceNumber n(static_cast<int64_t>(9223372036854775808ULL));

    EXPECT_EQ(n.high(), -2147483648);
    EXPECT_EQ(n.low(), 0);
}

TEST(RtiDdsTest_MsgKeyConversion, test_12)
{
    // overflow 9223372036854775809
    SequenceNumber n(-9223372036854775807LL);

    EXPECT_EQ(n.high(), -2147483647);
    EXPECT_EQ(n.low(), 1);
}

TEST(RtiDdsTest_MsgKeyConversion, test_13)
{
    // overflow 9223372036854775810
    SequenceNumber n(-9223372036854775806LL);

    EXPECT_EQ(n.high(), -2147483647);
    EXPECT_EQ(n.low(), 2);
}

} // namespace