// DiagHelper.cc

#include <DiagHelper/DiagHelper.h>

std::string DiagHelper::convertToText(int i)
{
    switch (i) {
        case 0:
            return "one";
        case 1:
            return "two";
        case 2:
            return "three";
        case 3:
            return "four";
        default:
            return "";
    }
}

// handle this in better way
std::string DiagHelper::getHostIP(std::string name)
{
    if (std::string::npos != name.find("one"))
        return "192.168.1.110";
    else if (std::string::npos != name.find("two"))
        return "192.168.1.120";
    else if (std::string::npos != name.find("hmi"))
        return "192.168.1.150";
    else if (std::string::npos != name.find("log"))
        return "192.168.1.160";

    return std::string();
}

std::string DiagHelper::getHostVehicleName()
{
    std::string vehicleID;
    try {
        auto vehicleIDFile = std::string(getenv("ACTIVE_CONFIG_DIR")) + "/config";
        std::ifstream file(vehicleIDFile);
        if (file) {
            std::getline(file, vehicleID);
        } else {
            std::cerr << "Cannot read vehicle ID from " << vehicleIDFile << " (${ACTIVE_CONFIG_DIR}/config)"
                      << std::endl;
        }
    } catch (...) {
        std::cerr << "Fail to read vehicle ID" << std::endl;
        return std::string();
    }
    return vehicleID;
}

void DiagHelper::readConfigFile(libconfig::Config& cfg)
{
    // handle reading config file
    std::string configDir = std::string(getenv("OTTO_CONFIG_DIR")) + "/vehicles/";
    std::string host = DiagHelper::getHostVehicleName();
    configDir += host.substr(0, 3); // "bmw", "pac", etc.
    std::string filename = std::string("/diagmanager_") + host + std::string(".cfg");
    std::string config = configDir + "/" + host + filename;
    std::string config_file(config);

    try {
        cfg.readFile(config_file.c_str());
    } catch (const libconfig::FileIOException& fioex) {
        std::cerr << "I/O error while reading file" << config << std::endl;
    } catch (const libconfig::ParseException& parex) {
        std::cerr << "Parse error at " << parex.getFile() << parex.getLine() << " - " << parex.getError() << std::endl;
    }
}

std::vector<Device> DiagHelper::parseConfigFile(libconfig::Config& cfg)
{
    std::vector<Device> devicelist;

    int num_of_computers = cfg.lookup("num_of_computers");
    if (num_of_computers == -1) {
        std::cerr << "Could not find num_of_computers" << std::endl;
        return devicelist;
    }

    libconfig::Setting& computerNameSettings = cfg.lookup("computer_names");
    std::vector<std::string> computerNames;
    for (auto ii = 0; ii < computerNameSettings.getLength(); ii++) {
        computerNames.push_back(computerNameSettings[ii]);
    }

    for (unsigned i = 0; i < computerNames.size(); i++) {
        libconfig::Setting& root = cfg.lookup(computerNames.at(i));

        for (int j = 0; j < root.getLength(); j++) {
            libconfig::Setting& child = root[j];
            for (int k = 0; k < child.getLength(); k++) {
                Device dev;

                std::string intf = child.getPath();
                libconfig::Setting& subchild = child[k];

                if (subchild.getLength() > 0) {

                    if (!subchild.lookupValue("device", dev.name)) {
                        std::cout << "Error looking up for device name" << std::endl;
                    }

                    if (!subchild.lookupValue("type", dev.type)) {
                        std::cout << "Error looking up for device type" << std::endl;
                    }

                    if (cfg.exists(std::string(subchild.getPath()) + ".IP")) {
                        subchild.lookupValue("IP", dev.addr);
                    } else if (cfg.exists(std::string(subchild.getPath()) + ".ID")) {
                        uint32_t id;
                        subchild.lookupValue("ID", id);
                        dev.addr = std::to_string(id);
                    }

                    dev.host = getHostIP(root.getName()).c_str();

                    devicelist.push_back(dev);
                }
            }
        }
    }
    return devicelist;
}

std::vector<Device> DiagHelper::getDeviceInfo()
{
    libconfig::Config cfg;
    std::vector<Device> devicelist;

    try {
        readConfigFile(cfg);
        devicelist = parseConfigFile(cfg);
    } catch (const libconfig::SettingNotFoundException& setex) {
        std::cerr << "Setting not found - " << setex.getPath() << std::endl;
    } catch (...) {
        std::cerr << "Failed to get device info" << std::endl;
        return std::vector<Device>();
    }
    return devicelist;
}
