/**
 * \copyright
 * Copyright (c) 2020- Motional. All rights reserved.
 * NOTICE: Removing this heading is prohibited.
 * All information contained herein is, and remains the property of Motional and its suppliers, if any.
 * The intellectual and technical concepts contained herein are proprietary to Motional and its suppliers and may be
 * covered by U.S. and Foreign Patents, patents in process, and are protected by trade secret or copyright law.
 * Dissemination of this information or reproduction of this material is strictly forbidden unless prior written
 * permission is obtained from Motional.
 *
 * \brief Defines unit tests for DiagHelper
 */

#include "DiagHelper/DiagHelper.h"

#include <gmock/gmock.h>
#include <gtest/gtest.h>

TEST(DiagHelperTest, normalCase4convertToText)
{
    int input = 0;
    std::string output = DiagHelper::convertToText(input);
    EXPECT_EQ("one", output);

    input = 1;
    output = DiagHelper::convertToText(input);
    EXPECT_EQ("two", output);

    input = 2;
    output = DiagHelper::convertToText(input);
    EXPECT_EQ("three", output);

    input = 3;
    output = DiagHelper::convertToText(input);
    EXPECT_EQ("four", output);
}

TEST(DiagHelperTest, edgeCase4convertToText)
{
    int input = -1;
    std::string output = DiagHelper::convertToText(input);
    EXPECT_EQ("", output);

    input = 12;
    output = DiagHelper::convertToText(input);
    EXPECT_EQ("", output);

    input = 31;
    output = DiagHelper::convertToText(input);
    EXPECT_EQ("", output);

    input = 123456;
    output = DiagHelper::convertToText(input);
    EXPECT_EQ("", output);
}

TEST(DiagHelperTest, normalCase4getHostIP)
{
    std::string input = "one";
    std::string output = DiagHelper::getHostIP(input);
    EXPECT_EQ("192.168.1.110", output);

    input = "two";
    output = DiagHelper::getHostIP(input);
    EXPECT_EQ("192.168.1.120", output);

    input = "hmi";
    output = DiagHelper::getHostIP(input);
    EXPECT_EQ("192.168.1.150", output);

    input = "log";
    output = DiagHelper::getHostIP(input);
    EXPECT_EQ("192.168.1.160", output);
}

TEST(DiagHelperTest, edgeCase4getHostIP)
{
    std::string input = "oneVehicle";
    std::string output = DiagHelper::getHostIP(input);
    EXPECT_EQ("192.168.1.110", output);

    input = "tw";
    output = DiagHelper::getHostIP(input);
    EXPECT_EQ(std::string(), output);

    input = "hmiVehicleCar";
    output = DiagHelper::getHostIP(input);
    EXPECT_EQ("192.168.1.150", output);

    input = "loggggggggggg";
    output = DiagHelper::getHostIP(input);
    EXPECT_EQ("192.168.1.160", output);
}

TEST(DiagHelperTest, testGetHostVehicleName)
{
    std::string output = DiagHelper::getHostVehicleName();
    EXPECT_EQ("", output);
}

TEST(DiagHelperTest, testGetDeviceInfo)
{
    std::vector<Device> output = DiagHelper::getDeviceInfo();
    EXPECT_EQ(0, output.size());
}

TEST(DiagHelperTest, testParseConfigFileEmptyCase)
{
    std::string config{"num_of_computers = -1"};

    libconfig::Config cfg;
    cfg.readString(config);
    auto output = DiagHelper::parseConfigFile(cfg);
}

TEST(DiagHelperTest, testParseNormalConfigFileIP)
{
    std::string config{"num_of_computers = 1;"
                       "computer_names = [\"mvp-one\"];"
                       "mvp-one = {"
                       "ethernet_interfaces = [ \"SWITCH1\" ];"
                       "SWITCH1 = ( "
                       "{ device = \"mvp-two\";"
                       "IP = \"192.168.1.210\";"
                       "type = \"PC\";"
                       "});"
                       "};"};

    libconfig::Config cfg;
    cfg.readString(config);
    auto output = DiagHelper::parseConfigFile(cfg);
}

TEST(DiagHelperTest, testParseNormalConfigFileWithID)
{
    std::string config{"num_of_computers = 1;"
                       "computer_names = [\"mvp-one\"];"
                       "mvp-one = {"
                       "ethernet_interfaces = [ \"SWITCH1\" ];"
                       "SWITCH1 = ( "
                       "{ device = \"mvp-two\";"
                       "ID = \"192.168.1.210\";"
                       "type = \"PC\";"
                       "});"
                       "};"};

    libconfig::Config cfg;
    cfg.readString(config);
    auto output = DiagHelper::parseConfigFile(cfg);
}

TEST(DiagHelperTest, testParseIncompletedConfigFile)
{
    std::string config{"num_of_computers = 1;"
                       "computer_names = [\"mvp-one\"];"
                       "mvp-one = {"
                       "ethernet_interfaces = [ \"SWITCH1\" ];"
                       "SWITCH1 = ( "
                       "{"
                       "aaa = \"PC\";"
                       "});"
                       "};"};

    libconfig::Config cfg;
    cfg.readString(config);
    auto output = DiagHelper::parseConfigFile(cfg);
}
