/*!
 ***********************************************************************************************************************
 * MOTIONAL PRIVILEGED AND CONFIDENTIAL
 *
 * @brief EventLoggerMessage helper functions implementation
 *
 * @section License
 *
 * Copyright (c) 2022- Aptiv. All rights reserved.
 * NOTE: Removing this heading is prohibited.
 *
 * @section Open Source License List
 *
 * This is a list of any open-source project used by this file.
 ***********************************************************************************************************************
 */

#pragma once

#include "lib/base/enum/Trait.hpp"
#include "lib/base/enum/Utility.hpp"
#include <interfaces/EventLoggerMessageList/EventLoggerMessageList.h>
#include <iomanip>
#include <recgeometry/recGeometry.h>
#include <vector>

namespace base {
/// Specialization of EnumTrait for EventType
template<>
struct EnumTrait<av::msg::EventType>
{
    static constexpr std::array namedEnumerators = {
        Enumerator{av::msg::EventType::DISENGAGEMENT, "Disengagement"},
        Enumerator{av::msg::EventType::BRAKE_OVERRIDE, "Brake Override"},
        Enumerator{av::msg::EventType::STEERING_OVERRIDE, "Steering Override"},
        Enumerator{av::msg::EventType::THROTTLE_OVERRIDE, "Throttle Override"},
        Enumerator{av::msg::EventType::MANUAL_INCIDENT, "Manual Incident"},
        Enumerator{av::msg::EventType::AUTO_INCIDENT, "Auto Incident"},
        Enumerator{av::msg::EventType::OVERRIDE, "Override"},
        Enumerator{av::msg::EventType::ENGAGEMENT, "Engagement"},
        Enumerator{av::msg::EventType::SYSTEM_STATUS, "System Status"},
        Enumerator{av::msg::EventType::NOTABLE_EVENT, "Notable Event"},
        Enumerator{av::msg::EventType::UNKNOWN_TYPE, ""},
    };
};
/// Specialization of EnumTrait for EventCause
template<>
struct EnumTrait<av::msg::EventCause>
{
    static constexpr std::array namedEnumerators = {
        Enumerator{av::msg::EventCause::NO_CAUSE, "NoCause"},
        Enumerator{av::msg::EventCause::BRAKE, "Brake"},
        Enumerator{av::msg::EventCause::STEERING, "Steering"},
        Enumerator{av::msg::EventCause::THROTTLE, "Throttle"},
        Enumerator{av::msg::EventCause::MANUAL_CAUSE, "Manual"},
        Enumerator{av::msg::EventCause::AUTO_CAUSE, "Automatic"},
        Enumerator{av::msg::EventCause::MACHINE_FAIL, "MachineFail"},
        Enumerator{av::msg::EventCause::SENSOR_FAIL, "SensorFail"},
        Enumerator{av::msg::EventCause::ACTUATOR_FAIL, "ActuatorFail"},
        Enumerator{av::msg::EventCause::UNKNOWN_CAUSE, ""},
    };
};
} // namespace base

/**
 * @brief get string equivalent of enum type
 * @param etype enum type
 * @return string equivalent of enum type
 */
inline std::string getStringValue(av::msg::EventType etype)
{
    return std::string(*base::enumeratorName(etype));
}

/**
 * @brief get string equivalent of enum type
 * @param EventCause enum type
 * @return string equivalent of enum type
 */
inline std::string getStringValue(av::msg::EventCause eCause)
{
    return std::string(*base::enumeratorName(eCause));
}

inline av::msg::EventType convertToEventType(std::string eTypeString)
{
    av::msg::EventType returnValue{av::msg::EventType::UNKNOWN_TYPE};
    const auto eventType = base::enumeratorValue<av::msg::EventType>(eTypeString);
    if (eventType) {
        returnValue = eventType.value();
    }
    return returnValue;
}

/**
 * @brief get time in HH:MM:SS format from provided double value
 * @param timeElapsed time elapsed in seconds
 * @return string representing time in required format
 */
inline std::string getTimeElapsedInStringFormat(double timeElapsed)
{
    std::stringstream ss;
    int hour = static_cast<int>(timeElapsed) / 3600;
    int min = (static_cast<int>(timeElapsed) % 3600) / 60;
    int sec = (static_cast<int>(timeElapsed) % 3600) % 60;
    ss << std::setfill('0') << std::setw(2) << hour;
    ss << ":";
    ss << std::setfill('0') << std::setw(2) << min;
    ss << ":";
    ss << std::setfill('0') << std::setw(2) << sec;
    return ss.str();
}
