/**
 * \copyright
 * Copyright (c) 2021- Motional. All rights reserved.
 * NOTICE: Removing this heading is prohibited.
 * All information contained herein is, and remains the property of Motional, its affiliates, and its suppliers (if
 * any). The intellectual and technical concepts contained herein are proprietary to Motional, its affiliates, and its
 * suppliers (if any) and may be covered by U.S. and Foreign Patents, patents in process, and are protected by trade
 * secrets or copyright law. Dissemination of this information or reproduction of this material is strictly forbidden
 * unless prior written permission is obtained from Motional.
 */

#pragma once

#include <interfaces/KingResponse/KingResponse.h>

#include <unistd.h>

namespace av {
namespace msg {
namespace utils {

inline void setSenderAsLocalHost(av::msg::KingResponse& msg)
{
    char name[1024];
    gethostname(name, 1023);
    msg.sender_.assign(name);
}

inline std::map<std::string, std::vector<std::string>> getConfigDirectories(const av::msg::KingResponse& msg)
{
    ///< a map representing tree structure of scenarios in config directory
    ///< Sometimes, a directory representing scenario folder might have multiple sub-directories. This data structure
    ///< represents it.
    std::map<std::string, std::vector<std::string>> configs;
    for (auto item : msg.configDir_) {
        configs[item.config_] = item.scenarios_;
    }
    return configs;
}

inline void addConfig(av::msg::KingResponse& msg, std::string config, std::string scenario)
{
    auto itr = std::find_if(msg.configDir_.begin(), msg.configDir_.end(), [&config](av::msg::ConfigDirectory& dir) {
        if (config == dir.config_) {
            return true;
        }
        return false;
    });
    if (itr != msg.configDir_.end()) {
        itr->scenarios_.push_back(scenario);
    } else {
        msg.configDir_.push_back(av::msg::ConfigDirectory(config, std::vector<std::string>(1, scenario)));
    }
}

} // namespace utils
} // namespace msg
} // namespace av
