/*!
 ***********************************************************************************************************************
 * APTIV PRIVILEGED AND CONFIDENTIAL
 *
 * @brief Common functions for IdlHelper
 *
 * @section License
 *
 * Copyright (c) 2019- Aptiv. All rights reserved.
 * NOTE: Removing this heading is prohibited.
 *
 * @section Open Source License List
 *
 * This is a list of any open-source project used by this file.
 ***********************************************************************************************************************
 */

#ifndef _IDLHELPER_COMMON_HPP
#define _IDLHELPER_COMMON_HPP

#include <interfaces/rec_geometry_avt/rec_geometry_avt.h>
#include <recgeometry/recGeometry.h>

namespace idlhelper {
inline RecRadians getRecRadians(const av::geometry::RecRadians& r)
{
    return {r.rad};
}

inline RecPoint2D getRecPoint2D(const av::geometry::RecPoint2D& p)
{
    return {p.x, p.y};
}

inline RecPoint3D getRecPoint3D(const av::geometry::RecPoint3D& p)
{
    return {p.x, p.y, p.z};
}

inline RecVector2D getRecVec2D(const av::geometry::RecVector2D& vec)
{
    return {vec.x, vec.y};
}

inline RecVector3D getRecVector3D(const av::geometry::RecVector3D& vec)
{
    return {vec.x, vec.y, vec.z};
}

inline RecPose2D getRecPose2D(const av::geometry::RecPose2D& p)
{
    return {p.x, p.y, p.rotZ.rad};
}

inline av::geometry::RecPose2D getRecPose2DMsg(const RecPose2D& p)
{
    return av::geometry::RecPose2D(p.x, p.y, av::geometry::RecRadians(p.rotZ));
}

inline RecDifferentialPose2D getRecDifferentialPose2D(const av::geometry::RecDifferentialPose2D& p)
{
    return {p.inx, p.iny, p.inw.rad};
}

inline av::geometry::RecDifferentialPose2D getRecDifferentialPose2DMsg(const RecDifferentialPose2D& p)
{
    return av::geometry::RecDifferentialPose2D(p.x, p.y, av::geometry::RecRadians(p.rotZ));
}

inline RecPose3D getRecPose3D(const av::geometry::RecPose3D& p)
{
    return {p.x, p.y, p.z, p.rot1.rad, p.rot2.rad, p.rot3.rad, static_cast<RecPose3D::EulerAngleOrder>(p.angleOrder)};
}

inline av::geometry::RecPose3D getRecPose3DMsg(const RecPose3D& p)
{
    return av::geometry::RecPose3D(p.x,
                                   p.y,
                                   p.z,
                                   av::geometry::RecRadians(p.rot1),
                                   av::geometry::RecRadians(p.rot2),
                                   av::geometry::RecRadians(p.rot3),
                                   static_cast<av::geometry::EULER_ANGLE_ORDER>(p.angleOrder));
}

inline RecDifferentialPose3D getRecDifferentialPose3D(const av::geometry::RecDifferentialPose3D& p)
{
    return {p.x, p.y, p.z, p.rot1.rad, p.rot2.rad, p.rot3.rad};
}

inline av::geometry::RecDifferentialPose3D getRecDifferentialPose3DMsg(const RecDifferentialPose3D& p)
{
    return av::geometry::RecDifferentialPose3D(p.x,
                                               p.y,
                                               p.z,
                                               av::geometry::RecRadians(p.rot1),
                                               av::geometry::RecRadians(p.rot2),
                                               av::geometry::RecRadians(p.rot3));
}
} // namespace idlhelper
#endif // _IDLHELPER_COMMON_HPP
