/// @copyright Copyright (c) 2022- Motional. All rights reserved.
///
/// NOTICE: Removing this heading is prohibited.
///
/// All information contained herein is, and remains the property of Motional,
/// its affiliates, and its suppliers (if any). The intellectual and technical
/// concepts contained herein are proprietary to Motional, its affiliates, and
/// its suppliers (if any) and may be covered by U.S. and Foreign Patents,
/// patents in process, and are protected by trade secrets or copyright law.
/// Dissemination of this information or reproduction of this material is
/// strictly forbidden unless prior written permission is obtained from Motional.

#include "IdlHelper/EventLoggerMessageHelper.h"
#include <gtest/gtest.h>
#include <interfaces/EventLoggerMessageList/EventLoggerMessageList.h>
#include <string>

namespace {
const std::vector<std::pair<av::msg::EventType, std::string>> eventTypes{
    std::make_pair(av::msg::EventType::DISENGAGEMENT, std::string("Disengagement")),
    std::make_pair(av::msg::EventType::BRAKE_OVERRIDE, std::string("Brake Override")),
    std::make_pair(av::msg::EventType::STEERING_OVERRIDE, std::string("Steering Override")),
    std::make_pair(av::msg::EventType::THROTTLE_OVERRIDE, std::string("Throttle Override")),
    std::make_pair(av::msg::EventType::MANUAL_INCIDENT, std::string("Manual Incident")),
    std::make_pair(av::msg::EventType::AUTO_INCIDENT, std::string("Auto Incident")),
    std::make_pair(av::msg::EventType::OVERRIDE, std::string("Override")),
    std::make_pair(av::msg::EventType::ENGAGEMENT, std::string("Engagement")),
    std::make_pair(av::msg::EventType::SYSTEM_STATUS, std::string("System Status")),
    std::make_pair(av::msg::EventType::NOTABLE_EVENT, std::string("Notable Event")),
    std::make_pair(av::msg::EventType::UNKNOWN_TYPE, std::string(""))};

const std::vector<std::pair<av::msg::EventCause, std::string>> eventCauses{
    std::make_pair(av::msg::EventCause::NO_CAUSE, std::string("NoCause")),
    std::make_pair(av::msg::EventCause::BRAKE, std::string("Brake")),
    std::make_pair(av::msg::EventCause::STEERING, std::string("Steering")),
    std::make_pair(av::msg::EventCause::THROTTLE, std::string("Throttle")),
    std::make_pair(av::msg::EventCause::MANUAL_CAUSE, std::string("Manual")),
    std::make_pair(av::msg::EventCause::AUTO_CAUSE, std::string("Automatic")),
    std::make_pair(av::msg::EventCause::MACHINE_FAIL, std::string("MachineFail")),
    std::make_pair(av::msg::EventCause::SENSOR_FAIL, std::string("SensorFail")),
    std::make_pair(av::msg::EventCause::ACTUATOR_FAIL, std::string("ActuatorFail")),
    std::make_pair(av::msg::EventCause::UNKNOWN_CAUSE, std::string(""))};
} // namespace

// EventType --> std::string
class EventTypeToStringFixture : public ::testing::TestWithParam<std::pair<av::msg::EventType, std::string>>
{};

TEST_P(EventTypeToStringFixture, testGetStringValue)
{
    const auto [eventType, expectedStr] = GetParam();
    EXPECT_EQ(getStringValue(eventType), expectedStr);
}
INSTANTIATE_TEST_SUITE_P(EventLoggerMessageTest, EventTypeToStringFixture, ::testing::ValuesIn(eventTypes));

// std::string --> EventType
class StringToEventTypeFixture : public ::testing::TestWithParam<std::pair<av::msg::EventType, std::string>>
{};
TEST_P(StringToEventTypeFixture, testConvertToEventType)
{
    const auto [expectedEventType, eventStr] = GetParam();
    EXPECT_EQ(convertToEventType(eventStr), expectedEventType);
}
INSTANTIATE_TEST_SUITE_P(EventLoggerMessageTest, StringToEventTypeFixture, ::testing::ValuesIn(eventTypes));

// EventCause --> std::string
class EventCauseToStringFixture : public ::testing::TestWithParam<std::pair<av::msg::EventCause, std::string>>
{};

TEST_P(EventCauseToStringFixture, testGetStringValue)
{
    const auto [eventCause, expectedStr] = GetParam();
    EXPECT_EQ(getStringValue(eventCause), expectedStr);
}
INSTANTIATE_TEST_SUITE_P(EventLoggerMessageTest, EventCauseToStringFixture, ::testing::ValuesIn(eventCauses));

// std::string --> EventCause
class StringToEventCauseFixture : public ::testing::TestWithParam<std::pair<av::msg::EventCause, std::string>>
{};

TEST_P(StringToEventCauseFixture, testGetStringValue)
{
    const auto [expectedCause, eventStr] = GetParam();
    auto eventCause = base::enumeratorValue<av::msg::EventCause>(eventStr);
    EXPECT_EQ(*eventCause, expectedCause);
}
INSTANTIATE_TEST_SUITE_P(EventLoggerMessageTest, StringToEventCauseFixture, ::testing::ValuesIn(eventCauses));
