/// @copyright
/// Copyright (c) 2020- Motional. All rights reserved.
/// NOTICE: Removing this heading is prohibited.
/// All information contained herein is, and remains the property of Motional, its affiliates, and its suppliers (if
/// any). The intellectual and technical concepts contained herein are proprietary to Motional, its affiliates, and its
/// suppliers (if any) and may be covered by U.S. and Foreign Patents, patents in process, and are protected by trade
/// secrets or copyright law. Dissemination of this information or reproduction of this material is strictly forbidden
/// unless prior written permission is obtained from Motional.
///
/// @brief Conversion tests for LaneConnectorTrafficLightTest

#include "lib/maps/src/lib/avmap/BaseEdge.hpp"
#include "lib/maps/src/lib/avmap/Enums.hpp"
#include "lib/maps/src/lib/avmap/Intersection.hpp"
#include "lib/maps/src/lib/avmap/Lane.hpp"
#include "lib/maps/src/lib/avmap/LaneConnector.hpp"
#include "lib/maps/src/lib/avmap/LinearObject.hpp"
#include "lib/maps/src/lib/avmap/RoadBlock.hpp"
#include "lib/maps/src/lib/avmap/RoadBlockConnector.hpp"
#include "lib/maps/src/lib/avmap/RoadSegment.hpp"
#include "lib/maps/src/lib/avmap/RoadSegmentConnector.hpp"
#include "lib/maps/src/lib/avmap/SimplePolygonalObject.hpp"

#include <IdlHelper/LaneConnectorTrafficLightState.hpp>
#include <interfaces/IntersectionTLStatus/IntersectionTLStatus.h>
#include <interfaces/MultiIntersectionTLStatus/MultiIntersectionTLStatus.h>

#include <gtest/gtest.h>

class LaneConnectorTrafficLightStateTest : public ::testing::Test
{
public:
    LaneConnectorTrafficLightStateTest()
        : intersection_{createReferenceId(), createPolygon(), av::map::IntersectionType::trafficLight, false}
        , laneDivider_{createReferenceId(), createLinearObject()}
        , edge_{createReferenceId(), createLinearObject()}
        , roadSegment_{createReferenceId(),
                       createPolygon(),
                       123,
                       laneDivider_,
                       laneDivider_,
                       edge_,
                       edge_,
                       std::vector<av::map::ID>{}}
        , roadSegmentConnector_{createReferenceId(),
                                createPolygon(),
                                laneDivider_,
                                laneDivider_,
                                edge_,
                                edge_,
                                std::vector<RoadSegment*>{}}
        , directionalRoadSegmentConnector_{createReferenceId(), roadSegmentConnector_}
        , laneGroupConnector_{createReferenceId(),
                              intersection_,
                              directionalRoadSegmentConnector_,
                              laneDivider_,
                              laneDivider_}
        , laneConnector_{av::map::ID(1),
                         laneGroupConnector_,
                         createPolygon(),
                         edge_,
                         edge_,
                         {},
                         {},
                         true,
                         false,
                         createPolygon(),
                         laneDivider_,
                         laneDivider_,
                         createOverlapSection(),
                         {},
                         boost::uuids::nil_uuid(),
                         av::map::TurnType::straight}
        , fromLaneGroup_{av::map::ID(incomingLaneGroupId_), roadSegment_, createPolygon(), laneDivider_, laneDivider_}
        , toLaneGroup_{av::map::ID(outgoingLaneGroupId_), roadSegment_, createPolygon(), laneDivider_, laneDivider_}
        , fromLane_{createReferenceId(),
                    av::map::LaneType::car,
                    fromLaneGroup_,
                    createPolygon(),
                    edge_,
                    edge_,
                    {},
                    {},
                    true,
                    false,
                    10,
                    laneDivider_,
                    laneDivider_,
                    1}
        , toLane_{createReferenceId(),
                  av::map::LaneType::car,
                  toLaneGroup_,
                  createPolygon(),
                  edge_,
                  edge_,
                  {},
                  {},
                  true,
                  false,
                  10,
                  laneDivider_,
                  laneDivider_,
                  1}
    {
        laneConnector_.addIncoming(fromLane_);
        fromLane_.addOutgoing(laneConnector_);
        laneConnector_.addOutgoing(toLane_);
        toLane_.addIncoming(laneConnector_);
    }

    ~LaneConnectorTrafficLightStateTest() override = default;

    void SetUp() override {}
    void TearDown() override {}

    av::msg::MultiIntersectionTLStatus createMultiIntersectionTLStatus(av::msg::FOV_STATE fovState,
                                                                       av::msg::LIGHT_SHAPE shape,
                                                                       av::msg::LIGHT_COLOR color,
                                                                       av::msg::LIGHT_STATE lightState,
                                                                       int64_t incomingID = incomingLaneGroupId_,
                                                                       int64_t outgoingID = outgoingLaneGroupId_)
    {
        av::msg::EdgeTLStatus edgeTLStatus{fovState, color, shape, lightState, {}, 0, 0};
        av::msg::EdgeTLStatusMapping edgeTlStatusMapping{std::to_string(outgoingID), edgeTLStatus};
        av::msg::IntersectionTLStatus intersectionTLStatus{
            0,
            intersection_.id().toString(),
            std::to_string(incomingID),
            std::vector<av::msg::EdgeTLStatusMapping>{edgeTlStatusMapping}};
        av::msg::MultiIntersectionTLStatus multiIntersectionTLStatus{
            0, std::vector<av::msg::IntersectionTLStatus>{intersectionTLStatus}};
        return multiIntersectionTLStatus;
    }

private:
    av::map::ID createReferenceId()
    {
        static av::map::ID id{654321};
        return id;
    }

    av::map::SimplePolygonalObject createPolygon()
    {
        const av::map::Point pt1{-1, -9};
        const av::map::Point pt2{1, -9};
        const av::map::Point pt3{1, 9};
        const av::map::Point pt4{-1, 9};
        const std::vector<av::map::Point> points{pt1, pt2, pt3, pt4};
        return av::map::SimplePolygonalObject{points};
    }

    av::map::LinearObject createLinearObject()
    {
        const av::map::Point pt1{0, -10};
        const av::map::Point pt2{0, 10};
        return av::map::LinearObject{std::vector<av::map::Point>{pt1, pt2}};
    }

    av::map::OverlapSection createOverlapSection() { return av::map::OverlapSection{0, 10}; }

protected:
    static const int64_t incomingLaneGroupId_ = 111;
    static const int64_t outgoingLaneGroupId_ = 222;

    av::map::Intersection intersection_;
    av::map::LaneDivider laneDivider_;
    av::map::BaseEdge edge_;
    av::map::RoadSegment roadSegment_;
    av::map::RoadSegmentConnector roadSegmentConnector_;
    av::map::DirectionalRoadSegmentConnector directionalRoadSegmentConnector_;
    av::map::RoadBlockConnector laneGroupConnector_;
    av::map::LaneConnector laneConnector_;
    av::map::RoadBlock fromLaneGroup_;
    av::map::RoadBlock toLaneGroup_;
    av::map::Lane fromLane_;
    av::map::Lane toLane_;
};

TEST_F(LaneConnectorTrafficLightStateTest, ConvertMessageGreen)
{
    av::msg::MultiIntersectionTLStatus msgGreen = createMultiIntersectionTLStatus(av::msg::FOV_STATE::IN_FOV,
                                                                                  av::msg::LIGHT_SHAPE::UNKNOWN,
                                                                                  av::msg::LIGHT_COLOR::GREEN,
                                                                                  av::msg::LIGHT_STATE::UNKNOWN);

    auto laneConnectorTrafficLightState =
        av::msg::utils::createLaneConnectorTrafficLightStateFromMessage(laneConnector_, msgGreen);

    EXPECT_EQ(laneConnectorTrafficLightState.color_, TrafficLightItemColor::green);
}

TEST_F(LaneConnectorTrafficLightStateTest, ConvertMessageYellow)
{
    av::msg::MultiIntersectionTLStatus msgYellow = createMultiIntersectionTLStatus(av::msg::FOV_STATE::IN_FOV,
                                                                                   av::msg::LIGHT_SHAPE::UNKNOWN,
                                                                                   av::msg::LIGHT_COLOR::YELLOW,
                                                                                   av::msg::LIGHT_STATE::UNKNOWN);

    auto laneConnectorTrafficLightState =
        av::msg::utils::createLaneConnectorTrafficLightStateFromMessage(laneConnector_, msgYellow);

    EXPECT_EQ(laneConnectorTrafficLightState.color_, TrafficLightItemColor::yellow);
}

TEST_F(LaneConnectorTrafficLightStateTest, ConvertMessageRed)
{
    av::msg::MultiIntersectionTLStatus msgRed = createMultiIntersectionTLStatus(av::msg::FOV_STATE::IN_FOV,
                                                                                av::msg::LIGHT_SHAPE::UNKNOWN,
                                                                                av::msg::LIGHT_COLOR::RED,
                                                                                av::msg::LIGHT_STATE::UNKNOWN);

    auto laneConnectorTrafficLightState =
        av::msg::utils::createLaneConnectorTrafficLightStateFromMessage(laneConnector_, msgRed);

    EXPECT_EQ(laneConnectorTrafficLightState.color_, TrafficLightItemColor::red);
}

TEST_F(LaneConnectorTrafficLightStateTest, ConvertMessageFlashing)
{
    av::msg::MultiIntersectionTLStatus msgFlashing = createMultiIntersectionTLStatus(av::msg::FOV_STATE::IN_FOV,
                                                                                     av::msg::LIGHT_SHAPE::UNKNOWN,
                                                                                     av::msg::LIGHT_COLOR::UNKNOWN,
                                                                                     av::msg::LIGHT_STATE::FLASHING);

    auto laneConnectorTrafficLightState =
        av::msg::utils::createLaneConnectorTrafficLightStateFromMessage(laneConnector_, msgFlashing);

    EXPECT_EQ(laneConnectorTrafficLightState.state_, TrafficLightFlashingState::flashing);
}

TEST_F(LaneConnectorTrafficLightStateTest, ConvertMessageSteady)
{
    av::msg::MultiIntersectionTLStatus msgSteady = createMultiIntersectionTLStatus(av::msg::FOV_STATE::IN_FOV,
                                                                                   av::msg::LIGHT_SHAPE::UNKNOWN,
                                                                                   av::msg::LIGHT_COLOR::UNKNOWN,
                                                                                   av::msg::LIGHT_STATE::STEADY);

    auto laneConnectorTrafficLightState =
        av::msg::utils::createLaneConnectorTrafficLightStateFromMessage(laneConnector_, msgSteady);

    EXPECT_EQ(laneConnectorTrafficLightState.state_, TrafficLightFlashingState::steady);
}

TEST_F(LaneConnectorTrafficLightStateTest, ConvertMessageInFov)
{
    av::msg::MultiIntersectionTLStatus msgInFov = createMultiIntersectionTLStatus(av::msg::FOV_STATE::IN_FOV,
                                                                                  av::msg::LIGHT_SHAPE::UNKNOWN,
                                                                                  av::msg::LIGHT_COLOR::UNKNOWN,
                                                                                  av::msg::LIGHT_STATE::UNKNOWN);

    auto laneConnectorTrafficLightState =
        av::msg::utils::createLaneConnectorTrafficLightStateFromMessage(laneConnector_, msgInFov);

    EXPECT_EQ(laneConnectorTrafficLightState.fov_, TrafficLightFov::inFov);
}

TEST_F(LaneConnectorTrafficLightStateTest, ConvertMessageNotInFov)
{
    av::msg::MultiIntersectionTLStatus msgNotInFov = createMultiIntersectionTLStatus(av::msg::FOV_STATE::NOT_IN_FOV,
                                                                                     av::msg::LIGHT_SHAPE::UNKNOWN,
                                                                                     av::msg::LIGHT_COLOR::UNKNOWN,
                                                                                     av::msg::LIGHT_STATE::UNKNOWN);

    auto laneConnectorTrafficLightState =
        av::msg::utils::createLaneConnectorTrafficLightStateFromMessage(laneConnector_, msgNotInFov);

    EXPECT_EQ(laneConnectorTrafficLightState.fov_, TrafficLightFov::notInFov);
}

TEST_F(LaneConnectorTrafficLightStateTest, ConvertMessageCircle)
{
    av::msg::MultiIntersectionTLStatus msgCircle = createMultiIntersectionTLStatus(av::msg::FOV_STATE::IN_FOV,
                                                                                   av::msg::LIGHT_SHAPE::CIRCLE,
                                                                                   av::msg::LIGHT_COLOR::UNKNOWN,
                                                                                   av::msg::LIGHT_STATE::UNKNOWN);

    auto laneConnectorTrafficLightState =
        av::msg::utils::createLaneConnectorTrafficLightStateFromMessage(laneConnector_, msgCircle);

    EXPECT_EQ(laneConnectorTrafficLightState.shape_, TrafficLightItemShape::circle);
}

TEST_F(LaneConnectorTrafficLightStateTest, ConvertMessageLeftArrow)
{
    av::msg::MultiIntersectionTLStatus msgLeftArrow = createMultiIntersectionTLStatus(av::msg::FOV_STATE::IN_FOV,
                                                                                      av::msg::LIGHT_SHAPE::LEFT_ARROW,
                                                                                      av::msg::LIGHT_COLOR::UNKNOWN,
                                                                                      av::msg::LIGHT_STATE::UNKNOWN);

    auto laneConnectorTrafficLightState =
        av::msg::utils::createLaneConnectorTrafficLightStateFromMessage(laneConnector_, msgLeftArrow);

    EXPECT_EQ(laneConnectorTrafficLightState.shape_, TrafficLightItemShape::left);
}

TEST_F(LaneConnectorTrafficLightStateTest, ConvertMessageRightArrow)
{
    av::msg::MultiIntersectionTLStatus msgRightArrow =
        createMultiIntersectionTLStatus(av::msg::FOV_STATE::IN_FOV,
                                        av::msg::LIGHT_SHAPE::RIGHT_ARROW,
                                        av::msg::LIGHT_COLOR::UNKNOWN,
                                        av::msg::LIGHT_STATE::UNKNOWN);

    auto laneConnectorTrafficLightState =
        av::msg::utils::createLaneConnectorTrafficLightStateFromMessage(laneConnector_, msgRightArrow);

    EXPECT_EQ(laneConnectorTrafficLightState.shape_, TrafficLightItemShape::right);
}

TEST_F(LaneConnectorTrafficLightStateTest, ConvertMessageStraightArrow)
{
    av::msg::MultiIntersectionTLStatus msgStraightArrow =
        createMultiIntersectionTLStatus(av::msg::FOV_STATE::IN_FOV,
                                        av::msg::LIGHT_SHAPE::STRAIGHT_ARROW,
                                        av::msg::LIGHT_COLOR::UNKNOWN,
                                        av::msg::LIGHT_STATE::UNKNOWN);

    auto laneConnectorTrafficLightState =
        av::msg::utils::createLaneConnectorTrafficLightStateFromMessage(laneConnector_, msgStraightArrow);

    EXPECT_EQ(laneConnectorTrafficLightState.shape_, TrafficLightItemShape::up);
}

TEST_F(LaneConnectorTrafficLightStateTest, ConvertMessageUnrelated)
{
    // Green signal provided to some other turn.
    av::msg::MultiIntersectionTLStatus msgUnrelated = createMultiIntersectionTLStatus(av::msg::FOV_STATE::IN_FOV,
                                                                                      av::msg::LIGHT_SHAPE::CIRCLE,
                                                                                      av::msg::LIGHT_COLOR::GREEN,
                                                                                      av::msg::LIGHT_STATE::STEADY,
                                                                                      987654321,
                                                                                      123456789);

    auto laneConnectorTrafficLightState =
        av::msg::utils::createLaneConnectorTrafficLightStateFromMessage(laneConnector_, msgUnrelated);

    EXPECT_EQ(laneConnectorTrafficLightState.color_, TrafficLightItemColor::unknown);
    EXPECT_EQ(laneConnectorTrafficLightState.shape_, TrafficLightItemShape::unknown);
    EXPECT_EQ(laneConnectorTrafficLightState.state_, TrafficLightFlashingState::unknown);
    EXPECT_EQ(laneConnectorTrafficLightState.fov_, TrafficLightFov::notInFov);
}

TEST_F(LaneConnectorTrafficLightStateTest, ConvertMessageUnknown)
{
    av::msg::MultiIntersectionTLStatus msgUnknown = createMultiIntersectionTLStatus(av::msg::FOV_STATE::NOT_IN_FOV,
                                                                                    av::msg::LIGHT_SHAPE::UNKNOWN,
                                                                                    av::msg::LIGHT_COLOR::UNKNOWN,
                                                                                    av::msg::LIGHT_STATE::UNKNOWN,
                                                                                    incomingLaneGroupId_,
                                                                                    outgoingLaneGroupId_);

    auto laneConnectorTrafficLightState =
        av::msg::utils::createLaneConnectorTrafficLightStateFromMessage(laneConnector_, msgUnknown);

    EXPECT_EQ(laneConnectorTrafficLightState.color_, TrafficLightItemColor::unknown);
    EXPECT_EQ(laneConnectorTrafficLightState.shape_, TrafficLightItemShape::unknown);
    EXPECT_EQ(laneConnectorTrafficLightState.state_, TrafficLightFlashingState::unknown);
    EXPECT_EQ(laneConnectorTrafficLightState.fov_, TrafficLightFov::notInFov);
}
