/**
 * \copyright
 * Copyright (c) 2020- Motional. All rights reserved.
 * NOTICE: Removing this heading is prohibited.
 * All information contained herein is, and remains the property of Motional, its affiliates, and its suppliers (if
 * any). The intellectual and technical concepts contained herein are proprietary to Motional, its affiliates, and its
 * suppliers (if any) and may be covered by U.S. and Foreign Patents, patents in process, and are protected by trade
 * secrets or copyright law. Dissemination of this information or reproduction of this material is strictly forbidden
 * unless prior written permission is obtained from Motional.
 */

#include <gtest/gtest.h>

#include "IdlHelper/LocalizationPoseHelper.h"

using namespace idlhelper;

TEST(LocalizationPoseHelperTest, getPosition)
{
    LocalizationPoseHelper lph;
    lph.pose.position.x = 1;
    lph.pose.position.y = 2;
    lph.pose.position.z = 3;
    Eigen::Vector3d result = lph.getPosition();
    EXPECT_EQ(result(0), 1);
    EXPECT_EQ(result(1), 2);
    EXPECT_EQ(result(2), 3);
}

TEST(LocalizationPoseHelperTest, getQuaternion)
{
    LocalizationPoseHelper lph;
    lph.pose.rotation.x = 1;
    lph.pose.rotation.y = 1;
    lph.pose.rotation.z = 1;
    Eigen::Quaterniond result = lph.getQuaternion();
    EXPECT_EQ(result.squaredNorm(), 3);
}

TEST(LocalizationPoseHelperTest, getRollPitchYaw)
{
    LocalizationPoseHelper lph;
    lph.pose.rotation.x = 1;
    lph.pose.rotation.y = 1;
    lph.pose.rotation.z = 1;
    Eigen::Vector3d result = lph.getRollPitchYaw();
    EXPECT_DOUBLE_EQ(result(0), 2.5535900500422257);
    EXPECT_DOUBLE_EQ(result(1), -0.50644464341350037);
    EXPECT_DOUBLE_EQ(result(2), 1.179722649198429);
}

TEST(LocalizationPoseHelperTest, getVelocity)
{
    LocalizationPoseHelper lph;
    lph.velocity.x = 1;
    lph.velocity.y = 2;
    lph.velocity.z = 3;
    Eigen::Vector3d result = lph.getVelocity();
    EXPECT_EQ(result(0), 1);
    EXPECT_EQ(result(1), 2);
    EXPECT_EQ(result(2), 3);
}

TEST(LocalizationPoseHelperTest, getAngularRate)
{
    LocalizationPoseHelper lph;
    lph.angular_rate.x = 1;
    lph.angular_rate.y = 2;
    lph.angular_rate.z = 3;
    Eigen::Vector3d result = lph.getAngularRate();
    EXPECT_EQ(result(0), 1);
    EXPECT_EQ(result(1), 2);
    EXPECT_EQ(result(2), 3);
}

TEST(LocalizationPoseHelperTest, getAcceleration)
{
    LocalizationPoseHelper lph;
    lph.acceleration.x = 1;
    lph.acceleration.y = 2;
    lph.acceleration.z = 3;
    Eigen::Vector3d result = lph.getAcceleration();
    EXPECT_EQ(result(0), 1);
    EXPECT_EQ(result(1), 2);
    EXPECT_EQ(result(2), 3);
}

TEST(LocalizationPoseHelperTest, getTimestamp)
{
    LocalizationPoseHelper lph;
    lph.timestamp = 1000000;
    std::chrono::system_clock::time_point result = lph.getTimestamp();
    EXPECT_EQ(result.time_since_epoch().count(), 1000000000);
}

TEST(LocalizationPoseHelperTest, getTimePoint)
{
    LocalizationPoseHelper lph;
    lph.timestamp = 1000000;
    ottotime::otimePoint result = lph.getTimePoint();
    EXPECT_EQ(result.to_timeval().tv_sec, 1);
}

TEST(LocalizationPoseHelperTest, constructFromLocalizationPose)
{
    LocalizationPose lp;
    lp.pose.position.x = 1;
    lp.pose.position.y = 2;
    lp.pose.position.z = 3;
    lp.pose.rotation.x = 1;
    lp.pose.rotation.y = 1;
    lp.pose.rotation.z = 1;
    lp.velocity.x = 1;
    lp.velocity.y = 2;
    lp.velocity.z = 3;
    lp.angular_rate.x = 1;
    lp.angular_rate.y = 2;
    lp.angular_rate.z = 3;
    lp.acceleration.x = 1;
    lp.acceleration.y = 2;
    lp.acceleration.z = 3;
    lp.timestamp = 1000000;

    LocalizationPoseHelper lph{lp};

    EXPECT_EQ(lph.pose.position.x, lp.pose.position.x);
    EXPECT_EQ(lph.pose.position.y, lp.pose.position.y);
    EXPECT_EQ(lph.pose.position.z, lp.pose.position.z);
    EXPECT_EQ(lph.pose.rotation.x, lp.pose.rotation.x);
    EXPECT_EQ(lph.pose.rotation.y, lp.pose.rotation.y);
    EXPECT_EQ(lph.pose.rotation.z, lp.pose.rotation.z);
    EXPECT_EQ(lph.velocity.x, lp.velocity.x);
    EXPECT_EQ(lph.velocity.y, lp.velocity.y);
    EXPECT_EQ(lph.velocity.z, lp.velocity.z);
    EXPECT_EQ(lph.angular_rate.x, lp.angular_rate.x);
    EXPECT_EQ(lph.angular_rate.y, lp.angular_rate.y);
    EXPECT_EQ(lph.angular_rate.z, lp.angular_rate.z);
    EXPECT_EQ(lph.acceleration.x, lp.acceleration.x);
    EXPECT_EQ(lph.acceleration.y, lp.acceleration.y);
    EXPECT_EQ(lph.acceleration.z, lp.acceleration.z);
    EXPECT_EQ(lph.timestamp, lp.timestamp);
}