/**
 * \copyright
 * Copyright (c) 2020- Motional. All rights reserved.
 * NOTICE: Removing this heading is prohibited.
 * All information contained herein is, and remains the property of Motional, its affiliates, and its suppliers (if
 * any). The intellectual and technical concepts contained herein are proprietary to Motional, its affiliates, and its
 * suppliers (if any) and may be covered by U.S. and Foreign Patents, patents in process, and are protected by trade
 * secrets or copyright law. Dissemination of this information or reproduction of this material is strictly forbidden
 * unless prior written permission is obtained from Motional.
 */

#include <gtest/gtest.h>

#include "IdlHelper/recgeometryHelper.h"

using namespace idlhelper;

TEST(RecgeometryHelperTest, convertRecRadians)
{
    RecRadians r{1.0};
    av::geometry::RecRadians rMsg{1.0};

    // Convert av::geometry::RecRadians => RecRadians
    EXPECT_EQ(getRecRadians(rMsg), r);

    // Convert RecRadians => av::geometry::RecRadians
    EXPECT_EQ(av::geometry::RecRadians(r), rMsg);
}

TEST(RecgeometryHelperTest, convertRecPoint2D)
{
    RecPoint2D p{1.0, 2.0};
    av::geometry::RecPoint2D pMsg{1.0, 2.0};

    // Convert av::geometry::RecPoint2D => RecPoint2D
    EXPECT_EQ(getRecPoint2D(pMsg), p);

    // Convert RecPoint2D => av::geometry::RecPoint2D
    EXPECT_EQ(av::geometry::RecPoint2D(p.x, p.y), pMsg);
}

TEST(RecgeometryHelperTest, convertRecPoint3D)
{
    RecPoint3D p{1.0, 2.0, 3.0};
    av::geometry::RecPoint3D pMsg{1.0, 2.0, 3.0};

    // Convert av::geometry::RecPoint3D => RecPoint3D
    EXPECT_EQ(getRecPoint3D(pMsg), p);

    // Convert RecPoint3D => av::geometry::RecPoint3D
    EXPECT_EQ(av::geometry::RecPoint3D(p.x, p.y, p.z), pMsg);
}

TEST(RecgeometryHelperTest, convertRecVector2D)
{
    RecVector2D vec{1.0, 2.0};
    av::geometry::RecVector2D vecMsg{1.0, 2.0};

    // Convert av::geometry::RecVector2D => RecVector2D
    EXPECT_EQ(getRecVec2D(vecMsg), vec);

    // Convert RecVector2D => av::geometry::RecVector2D
    EXPECT_EQ(av::geometry::RecVector2D(vec.x, vec.y), vecMsg);
}

TEST(RecgeometryHelperTest, convertRecVector3D)
{
    RecVector3D vec{1.0, 2.0, 3.0};
    av::geometry::RecVector3D vecMsg{1.0, 2.0, 3.0};

    // Convert av::geometry::RecVector3D => RecVector3D
    EXPECT_EQ(getRecVector3D(vecMsg), vec);

    // Convert RecVector3D => av::geometry::RecVector3D
    EXPECT_EQ(av::geometry::RecVector3D(vec.x, vec.y, vec.z), vecMsg);
}

TEST(RecgeometryHelperTest, convertRecPose2D)
{
    RecPose2D p{1.0, 2.0, 3.0};
    av::geometry::RecPose2D pMsg{1.0, 2.0, 3.0};

    // Convert av::geometry::RecPose2D => RecPose2D
    EXPECT_EQ(getRecPose2D(pMsg), p);

    // Convert RecPose2D => av::geometry::RecPose2D
    EXPECT_EQ(getRecPose2DMsg(p), pMsg);
}

TEST(RecgeometryHelperTest, convertRecDifferentialPose2D)
{
    RecDifferentialPose2D p{1.0, 2.0, 3.0};
    av::geometry::RecDifferentialPose2D pMsg{1.0, 2.0, 3.0};

    // Convert av::geometry::RecDifferentialPose2D => RecDifferentialPose2D
    EXPECT_EQ(getRecDifferentialPose2D(pMsg), p);

    // Convert RecDifferentialPose2D => av::geometry::RecDifferentialPose2D
    EXPECT_EQ(getRecDifferentialPose2DMsg(p), pMsg);
}

TEST(RecgeometryHelperTest, convertRecPose3D)
{
    RecPose3D p{1.0, 2.0, 3.0, 1.0, 2.0, 3.0};
    av::geometry::RecPose3D pMsg{1.0, 2.0, 3.0, 1.0, 2.0, 3.0, av::geometry::EULER_ANGLE_ORDER::ZYX};

    // Convert av::geometry::RecPose3D => RecPose3D
    EXPECT_EQ(getRecPose3D(pMsg), p);

    // Convert RecPose3D => av::geometry::RecPose3D
    EXPECT_EQ(getRecPose3DMsg(p), pMsg);
}

TEST(RecgeometryHelperTest, convertRecDifferentialPose3D)
{
    RecDifferentialPose3D p{1.0, 2.0, 3.0, 1.0, 2.0, 3.0};
    av::geometry::RecDifferentialPose3D pMsg{1.0, 2.0, 3.0, 1.0, 2.0, 3.0};

    // Convert av::geometry::RecDifferentialPose3D => RecDifferentialPose3D
    EXPECT_EQ(getRecDifferentialPose3D(pMsg), p);

    // Convert RecDifferentialPose3D => av::geometry::RecDifferentialPose3D
    EXPECT_EQ(getRecDifferentialPose3DMsg(p), pMsg);
}
