#include <gtest/gtest.h>

#include <avtime/Time.hpp>
#include <interfaces/Duration/Duration.h>
#include <interfaces/TimePoint/TimePoint.h>

TEST(TimeHelperTest, Duration)
{
    // av::Duration 5 nanoseconds
    const av::Duration durationIn(5);

    // Convert to av::msg::Duration (int64 over the wire)
    const av::msg::Duration durationMsg(durationIn);
    EXPECT_EQ(durationMsg.privateData, 5);
    // Test assignment operator works
    const av::msg::Duration testAssignment = durationIn;
    EXPECT_EQ(testAssignment.privateData, 5);

    // Convert back to av::Duration
    // Implicit conversion function defined for av::msg::Duration:
    //      operator av::Duration() const
    const av::Duration durationOut = durationMsg;
    EXPECT_EQ(typeid(av::Duration), typeid(durationOut));
    EXPECT_EQ(durationIn, durationOut);
    // Note: Using auto will not convert to av::Duration
    const auto durationMsgCopy = durationMsg;
    EXPECT_EQ(typeid(av::msg::Duration), typeid(durationMsgCopy));
}

TEST(TimeHelperTest, TimePoint)
{
    // av::TimePoint
    const av::TimePoint timepointIn(av::AvClock::now());

    // Convert to av::msg::TimePoint (int64 over the wire)
    const av::msg::TimePoint timepointMsg(timepointIn);
    EXPECT_EQ(timepointMsg.privateData, timepointIn.time_since_epoch().count());
    // Test assignment operator works
    const av::msg::TimePoint testAssignment = timepointIn;
    EXPECT_EQ(testAssignment.privateData, timepointIn.time_since_epoch().count());

    // Convert back to av::TimePoint
    const av::TimePoint timepointOut = timepointMsg;
    EXPECT_EQ(timepointIn, timepointOut);
    // Note: Using auto will not convert to av::TimePoint
    const auto timepointMsgCopy = timepointMsg;
    EXPECT_EQ(typeid(av::msg::TimePoint), typeid(timepointMsgCopy));
}
