/*!
 ***********************************************************************************************************************
 * APTIV PRIVILEGED AND CONFIDENTIAL
 *
 * @file ImageUtility/ImageEncodeDecode.h
 * @product <Subsystem name>
 * @version <Release Number>
 * @released <date>
 * @updated <date>
 * @brief Utility functions for encode and decode images
 *
 * @sa Optional Reference
 *
 * @section License
 *
 * Copyright (c) 2019- Aptiv. All rights reserved.
 * NOTE: Removing this heading is prohibited.
 *
 * @section Open Source License List
 *
 * This is a list of any open-source project used by this file.
 ***********************************************************************************************************************
 */

#include "ImageUtility/ImageEncodeDecode.h"

#include <turbojpeg.h>

namespace image_encoder {

bool compressMatBgrToJpeg(const cv::Mat& mat, std::vector<uint8_t>& data, const int jpegQuality)
{
    tjhandle tjCompressHandler = tjInitCompress();

    ulong jpegBufferSize{0};
    uint8_t* jpegBuffer{nullptr};
    int status = tjCompress2(tjCompressHandler,
                             mat.data,
                             mat.cols,
                             static_cast<int>(mat.step[0]),
                             mat.rows,
                             TJPF_BGR,
                             &jpegBuffer,
                             &jpegBufferSize,
                             TJSAMP_422,
                             jpegQuality,
                             TJFLAG_FASTDCT);

    if (status == 0) {
        data.resize(jpegBufferSize);
        std::copy(jpegBuffer, jpegBuffer + jpegBufferSize, data.begin());
    }

    tjDestroy(tjCompressHandler);

    if (jpegBuffer != nullptr) {
        tjFree(jpegBuffer);
    }

    return (status == 0);
}

} // namespace image_encoder

namespace image_decoder {

bool decompressJpegToMat(const std::vector<uint8_t>& data, cv::Mat& mat, bool isRGB)
{
    int width, height, jpegSubsamp;
    tjhandle tjDecompressHandler = tjInitDecompress();
    auto status = tjDecompressHeader2(tjDecompressHandler,
                                      const_cast<unsigned char*>(data.data()),
                                      static_cast<ulong>(data.size()),
                                      &width,
                                      &height,
                                      &jpegSubsamp);

    if (status != 0) {
        tjDestroy(tjDecompressHandler);
        return false;
    }

    mat.create(height, width, CV_8UC3);
    uint8_t* bgr8Buffer = mat.data;

    const auto colorArrangement = isRGB ? TJPF_RGB : TJPF_BGR;
    status = tjDecompress2(tjDecompressHandler,
                           const_cast<unsigned char*>(data.data()),
                           static_cast<ulong>(data.size()),
                           bgr8Buffer,
                           width,
                           0,
                           height,
                           colorArrangement,
                           TJFLAG_FASTDCT);

    tjDestroy(tjDecompressHandler);

    return (status == 0);
}

} // namespace image_decoder
