/*!
 ***********************************************************************************************************************
 * APTIV PRIVILEGED AND CONFIDENTIAL
 *
 * @file ImageUtility/ImageEncodeDecode.h
 * @product <Subsystem name>
 * @version <Release Number>
 * @released <date>
 * @updated <date>
 * @brief Utility for image encode and decode
 *
 * This is a detailed description of the file that spans several lines and characterizes the basic functionality
 * of the class.
 *
 * @section License
 *
 * Copyright (c) 2019- Aptiv. All rights reserved.
 * NOTE: Removing this heading is prohibited.
 *
 * This is a list of any open-source project used by this file.
 ***********************************************************************************************************************
 */

#include <opencv2/core/core.hpp>

/**
 * @brief Utility functions to encode/compress an image.
 */
namespace image_encoder {

/**
 * @brief Compress Cv::Mat data to jpeg message
 * @param mat[in]: raw image data in Cv::Mat BGR format
 * @param data[out]: compressed data
 * @param jpegQuality[in]: the image quality of the generated JPEG image (1 = worst,
 *                         100 = best)
 */
bool compressMatBgrToJpeg(const cv::Mat& mat, std::vector<uint8_t>& data, const int jpegQuality = 75);

} // namespace image_encoder

/**
 * @brief Utility functions to decode/decompress an image.
 */
namespace image_decoder {

/**
 * @brief Decompress jpeg message to Cv::Mat data
 * @param data[in]: compressed data
 * @param mat[out]: raw image data in Cv::Mat format
 * @param isRGB[in]: true if RGB; false if BGR
 */
bool decompressJpegToMat(const std::vector<uint8_t>& data, cv::Mat& mat, bool isRGB = false);

} // namespace image_decoder
