/*!
 ***********************************************************************************************************************
 * APTIV PRIVILEGED AND CONFIDENTIAL
 *
 * @file epsg.cc
 * @product <Subsystem name>
 * @version <Release Number>
 * @released 03/29/2019
 * @updated <date>
 * @brief Implements the epsg.
 *
 * @section License
 *
 * Copyright (c) 2019- Aptiv. All rights reserved.
 * NOTICE: Removing this heading is prohibited.
 * All information contained herein is, and remains the property of Aptiv and its suppliers, if any.
 * The intellectual and technical concepts contained herein are proprietary to Aptiv and its suppliers and may be
 * covered by U.S. and Foreign Patents, patents in process, and are protected by trade secret or copyright law.
 * Dissemination of this information or reproduction of this material is strictly forbidden unless prior written
 * permission is obtained from Aptiv.
 *
 * @section Open Source License List
 *
 * This is a list of any open-source project used by this file.
 ***********************************************************************************************************************
 */

#include "lib/geospatial/CoordinateProjection.hpp"
#include "lib/geospatial/EpsgDef.hpp"
#include <LLUTMConvLibrary/epsg.h>
#include <iostream>
#include <proj_api.h>

namespace ProjectionEPSG {

static std::unique_ptr<geospatial::CoordinateProjection> proj_;
static int initializedEpsgZone_{-1};

bool need_init()
{
    if (proj_) {
        return false;
    }

    return true;
}

void init_proj(unsigned int epsg)
{
    proj_ = std::make_unique<geospatial::CoordinateProjection>(epsg);
}

void validateInitialization(double lat, double lon, int utm_zone)
{
    if (utm_zone != initializedEpsgZone_) {
        proj_.reset();
    }
    if (need_init()) {
        unsigned int epsg_zone = 0;
        initializedEpsgZone_ = utm_zone;
        if (utm_zone < 0) {
            epsg_zone = static_cast<unsigned int>(
                geospatial::CoordinateProjection::getEpsgUsingUtm(geospatial::getGeoPosFromLatLon(lat, lon)));
        } else if (utm_zone <= 60) {
            epsg_zone = static_cast<unsigned int>(geospatial::UTM_NORTH_TO_EPSG_OFFSET + utm_zone);
        } else { // this is an invalid UTM zone
            throw(std::runtime_error("Invalid UTM zone"));
        }
        init_proj(epsg_zone);
    }
}

void LatLonToXY(const double& lat, const double& lon, double& northing, double& easting, const int& zone)
{
    validateInitialization(lat, lon, zone);

    const auto localPt = proj_->getXYFromLatLon(geospatial::getGeoPosFromLatLon(lat, lon));

    easting = localPt.x();
    northing = localPt.y();
}

void XYToLatLon(double& lat, double& lon, const double& northing, const double& easting, const int& zone)
{
    validateInitialization(lat, lon, zone);

    const auto deg = proj_->getLatLonFromXY(Eigen::Vector2d({easting, northing}));

    lat = deg.lat().value();
    lon = deg.lon().value();
}

int getUtmZone(const double& lat, const double& lon)
{
    int epsg = geospatial::CoordinateProjection::getEpsgUsingUtm(geospatial::getGeoPosFromLatLon(lat, lon));
    int utmZone = epsg - geospatial::UTM_NORTH_TO_EPSG_OFFSET;
    if (utmZone > geospatial::UTM_MAX_ZONE_NUMBER) {
        utmZone = epsg - geospatial::UTM_SOUTH_TO_EPSG_OFFSET;
    }
    return utmZone;
}

} // namespace ProjectionEPSG
