#include <gtest/gtest.h>

#include <LLUTMConvLibrary/epsg.h>
#include <LLUTMConvLibrary/utm.h>

constexpr double abs_error = 0.000001;

TEST(LLUTMConvLibraryTests, TestGetUtmZone)
{
    EXPECT_EQ(31, ProjectionEPSG::getUtmZone(0.0, 0.0));
}

TEST(LLUTMConvLibraryTests, TestLatLonToXY)
{
    double northing{0.0};
    double easting{0.0};
    ProjectionEPSG::LatLonToXY(0.0, 0.0, northing, easting, -1);
    EXPECT_NEAR(0, northing, abs_error);
    EXPECT_NEAR(166021.44308054051, easting, abs_error);
}

TEST(LLUTMConvLibraryTests, TestXYToLatLon)
{
    double lat{0.0};
    double lon{0.0};
    ProjectionEPSG::XYToLatLon(lat, lon, 0.0, 0.0, -1);
    EXPECT_NEAR(0, lat, abs_error);
    EXPECT_NEAR(-1.4887438843872076, lon, abs_error);
}

TEST(LLUTMConvLibraryTests, TestLatLonToUtmWGS84)
{
    double northing{0.0};
    double easting{0.0};
    LatLonToUtmWGS84(40.497547, -79.870165, northing, easting, 17);
    EXPECT_NEAR(4483595.5162085453, northing, abs_error);
    EXPECT_NEAR(595739.74843331426, easting, abs_error);
}

TEST(LLUTMConvLibraryTests, TestUtmToLatLonWGS84)
{
    double lat{0.0};
    double lon{0.0};
    UtmToLatLonWGS84(lat, lon, 40.499000, -79.870426, 17);
    EXPECT_NEAR(0.00036527534032090696, lat, abs_error);
    EXPECT_NEAR(-85.489459443006226, lon, abs_error);
}

TEST(LLUTMConvLibraryTests, TestGetUTMZone2)
{
    EXPECT_EQ(31, getUTMZone(0.0, 0.0));
}

TEST(LLUTMConvLibraryTests, TestGetUTMZone3)
{
    EXPECT_EQ(0, getUTMZone(""));
    EXPECT_EQ(17, getUTMZone("EPSG:32617"));
}

GTEST_API_ int main(int argc, char** argv)
{
    testing::InitGoogleTest(&argc, argv);
    return RUN_ALL_TESTS();
}
