// Copyright (c) 2021- Motional. All rights reserved.
//
// NOTICE: Removing this heading is prohibited.
//
// All information contained herein is, and remains the property of Motional, its affiliates, and
// its suppliers (if any). The intellectual and technical concepts contained herein are proprietary
// to Motional, its affiliates, and its suppliers (if any) and may be covered by U.S. and Foreign
// Patents, patents in process, and are protected by trade secrets or copyright law. Dissemination
// of this information or reproduction of this material is strictly forbidden unless prior written
// permission is obtained from Motional.

#include "platform/middleware/libs/LatencyLogger/include/LatencyLogger/DataLoggerWriter.hpp"

namespace av {
namespace latency {

DataLoggerWriter::DataLoggerWriter(const std::string& logPath, bool async)
    : DataLoggerWriter(avlog::DataLogger(logPath, async))
{
    // Intentionally blank
}

DataLoggerWriter::DataLoggerWriter(avlog::DataLogger&& dl)
    : dl_(std::move(dl))
{
    // Add relevant flattens for latency data
    dl_.addFlatten("lla", {"checkpoint", "id", "timestamp"}, {0});
    dl_.addFlatten("llb", {"checkpoint", "id", "timestamp", "predecessors"}, {0});
}

DataLoggerWriter DataLoggerWriter::UsingLoggerName(const std::string& loggerName)
{
    return DataLoggerWriter(avlog::DataLogger::UsingLoggerName(loggerName));
}

void DataLoggerWriter::operator()(const std::string& /* loggingName */,
                                  const std::string& checkpointName,
                                  const ottotime::otimePoint& timestamp,
                                  av::latency::EventID eventID,
                                  std::vector<av::latency::Pred>&& predecessors)
{
    nlohmann::json event;
    event["checkpoint"] = checkpointName;
    event["id"] = eventID;
    event["timestamp"] = timestamp.toMicrosecondsSinceEpoch();
    if (!predecessors.empty()) {
        event["predecessors"] = predecessors;
    }
    dl_.log(event);
}

void DataLoggerWriter::flush()
{
    dl_.flush();
}

} // namespace latency
} // namespace av
