/**
 * \copyright
 * Copyright (c) 2021 Motional. All rights reserved.
 * NOTICE: Removing this heading is prohibited.
 *
 * All information contained herein is, and remains the property of
 * Motional and its suppliers, if any.  The intellectual and technical
 * concepts contained herein are proprietary to Motional and its
 * suppliers and may be covered by U.S. and Foreign Patents, patents
 * in process, and are protected by trade secret or copyright law.
 * Dissemination of this information or reproduction of this material
 * is strictly forbidden unless prior written permission is obtained
 * from Motional.
 */

#include "platform/middleware/libs/LatencyLogger/include/LatencyLogger/JSONWriter.hpp"

namespace av {

namespace latency {

void writeDataToStream(std::ostream& stream,
                       const std::string& checkpointName,
                       const ottotime::otimePoint& timestamp,
                       av::latency::EventID eventID,
                       std::vector<av::latency::Pred>&& predecessors)
{
    nlohmann::json event;
    event["checkpoint"] = checkpointName;
    event["id"] = eventID;
    event["timestamp"] = timestamp.toMicrosecondsSinceEpoch();
    if (!predecessors.empty()) {
        event["predecessors"] = predecessors;
    }
    stream << event << '\n';
}

JSONWriter::JSONWriter(const std::string& loggingName, const std::string& subFolder)
{
    outFile_.open(subFolder + "/" + loggingName + ".json");
}

void JSONWriter::operator()(const std::string& /* loggingName */,
                            const std::string& checkpointName,
                            const ottotime::otimePoint& timestamp,
                            av::latency::EventID eventID,
                            std::vector<av::latency::Pred>&& predecessors)
{
    writeDataToStream(outFile_, checkpointName, timestamp, eventID, std::move(predecessors));
}

} // namespace latency
} // namespace av
