// Copyright (c) 2021- Motional. All rights reserved.
//
// NOTICE: Removing this heading is prohibited.
//
// All information contained herein is, and remains the property of Motional, its affiliates, and
// its suppliers (if any). The intellectual and technical concepts contained herein are proprietary
// to Motional, its affiliates, and its suppliers (if any) and may be covered by U.S. and Foreign
// Patents, patents in process, and are protected by trade secrets or copyright law. Dissemination
// of this information or reproduction of this material is strictly forbidden unless prior written
// permission is obtained from Motional.

#include "platform/middleware/libs/LatencyLogger/include/LatencyLogger/DataLoggerWriter.hpp"

#include <experimental/filesystem>

#include "lib/avlog/DataLogReader.hpp"
#include <gtest/gtest.h>

using DataLoggerWriter = av::latency::DataLoggerWriter;
namespace fs = std::experimental::filesystem;

namespace av {

namespace {

class DataLoggerWriterTest : public ::testing::Test
{
public:
    DataLoggerWriterTest() {}
};

TEST_F(DataLoggerWriterTest, DataLoggerWriter)
{
    std::string filePath = std::getenv("TEST_TMPDIR") + std::string("/DataLoggerWriterTest.log");
    {
        DataLoggerWriter writer(filePath, false);

        av::latency::EventID liftoffEvent = 1234;
        struct tm timeParts0
        {
            7, 28, 13, 12, 3, 121, 1, 101, 0, 0, "GMT"
        };
        const ottotime::otimePoint t0(timeParts0);
        writer("DataLoggerWriterTest", "Liftoff", t0, liftoffEvent, {});

        av::latency::EventID sendEvent = 4567;
        struct tm timeParts1
        {
            8, 27, 12, 9, 3, 115, 1, 99, 0, 1, "GMT"
        };
        const ottotime::otimePoint t1(timeParts1);
        writer("DataLoggerWriterTest", "send:Channel", t1, sendEvent, {liftoffEvent});

        av::latency::EventID recvEvent = 6789;
        struct tm timeParts2
        {
            6, 25, 12, 11, 3, 118, 1, 100, 0, 2, "GMT"
        };
        const ottotime::otimePoint t2(timeParts2);
        writer("DataLoggerWriterTest",
               "recv:Channel",
               t2,
               recvEvent,
               {static_cast<av::latency::EventID>(t1.toMicrosecondsSinceEpoch())});

        // Force flush for testing purposes
        writer.flush();
    }

    // Make sure the file exists
    ASSERT_TRUE(fs::exists(filePath));
    ASSERT_NE(fs::file_size(filePath), 0);

    avlog::DataLogReader dlr{filePath};
    nlohmann::json jdata;
    jdata = dlr.read();
    ASSERT_FALSE(jdata.is_discarded());
    EXPECT_EQ(jdata["checkpoint"], "Liftoff");
    EXPECT_EQ(jdata["id"], 1234);
    EXPECT_EQ(jdata["timestamp"], 1618234087000000);

    jdata = dlr.read();
    ASSERT_FALSE(jdata.is_discarded());
    EXPECT_EQ(jdata["checkpoint"], "send:Channel");
    EXPECT_EQ(jdata["id"], 4567);
    EXPECT_EQ(jdata["timestamp"], 1428582428000000);
    EXPECT_EQ(jdata["predecessors"].size(), 1);
    EXPECT_EQ(jdata["predecessors"][0], 1234);

    jdata = dlr.read();
    ASSERT_FALSE(jdata.is_discarded());
    EXPECT_EQ(jdata["checkpoint"], "recv:Channel");
    EXPECT_EQ(jdata["id"], 6789);
    EXPECT_EQ(jdata["timestamp"], 1523449506000000);
    EXPECT_EQ(jdata["predecessors"].size(), 1);
    EXPECT_EQ(jdata["predecessors"][0], 1428582428000000);

    jdata = dlr.read();
    ASSERT_TRUE(jdata.is_discarded());
}

} // namespace
} // namespace av
