/*!
 ***********************************************************************************************************************
 * MOTIONAL PRIVILEGED AND CONFIDENTIAL
 *
 * @file LatencyLoggerTest.cc
 * @brief This file implements tests for the LatencyLogger class.
 *
 * @section License
 *
 * Copyright (c) 2021- Motional. All rights reserved.
 * NOTE: Removing this heading is prohibited.
 *
 * @section Open Source License List
 *
 * This is a list of any open-source project used by this file.
 ***********************************************************************************************************************
 */

#include "platform/middleware/libs/LatencyLogger/test/TestWriter.hpp"
#include "platform/middleware/libs/config/include/config/ConfigParser.h"

namespace av {

TEST(InitializeMidwayTest, InitializeMidway)
{
    EXPECT_NO_THROW(LatencyLogger::getLogger());
    EXPECT_FALSE(LatencyLogger::isInitialized());

    LatencyLogger& logger = LatencyLogger::getLogger();

    const av::latency::EventID event1 = logger.RegularEvent("Event1");
    const av::latency::EventID event2 = logger.RegularEvent("Event2", {event1});

    TestWriter testWriter;
    EXPECT_NO_THROW(LatencyLogger::initialize(true, "InitilizeMidwayTest", std::ref(testWriter)));
    EXPECT_TRUE(LatencyLogger::isInitialized());

    LatencyLogger& logger1 = LatencyLogger::getLogger();
    const av::latency::EventID event3 = logger1.RegularEvent("Event3", {event2});
    const av::latency::EventID event4 = logger1.RegularEvent("Event4", {event3});

    EXPECT_NE(event4, av::latency::EventID{0}); // logger was initialized only after the first getLogger

    const auto& entries = testWriter.getEntries();
    ASSERT_EQ(2, entries.size());
}

} // namespace av
