/*!
 ***********************************************************************************************************************
 * MOTIONAL PRIVILEGED AND CONFIDENTIAL
 *
 * @file JSONWriterTest.cpp
 * @brief This file implements tests for the LatencyLogger class.
 *
 * @section License
 *
 * Copyright (c) 2021- Motional. All rights reserved.
 * NOTE: Removing this heading is prohibited.
 *
 * @section Open Source License List
 *
 * This is a list of any open-source project used by this file.
 ***********************************************************************************************************************
 */

#include <LatencyLogger/JSONWriter.hpp>
#include <gtest/gtest.h>

namespace av {

namespace {

class JSONWriterTest : public ::testing::Test
{
public:
    JSONWriterTest() {}
};

TEST_F(JSONWriterTest, JSONWriter)
{
    {
        av::latency::JSONWriter writer("JSONWriterTest", std::getenv("TEST_TMPDIR"));

        av::latency::EventID liftoffEvent = 1234;
        struct tm timeParts0
        {
            7, 28, 13, 12, 3, 121, 1, 101, 0, 0, "GMT"
        };
        const ottotime::otimePoint t0(timeParts0);
        writer("JSONWriterTest", "Liftoff", t0, liftoffEvent, {});

        av::latency::EventID sendEvent = 4567;
        struct tm timeParts1
        {
            8, 27, 12, 9, 3, 115, 1, 99, 0, 1, "GMT"
        };
        const ottotime::otimePoint t1(timeParts1);
        writer("JSONWriterTest", "send:Channel", t1, sendEvent, {liftoffEvent});

        av::latency::EventID recvEvent = 6789;
        struct tm timeParts2
        {
            6, 25, 12, 11, 3, 118, 1, 100, 0, 2, "GMT"
        };
        const ottotime::otimePoint t2(timeParts2);
        writer("JSONWriterTest",
               "recv:Channel",
               t2,
               recvEvent,
               {static_cast<av::latency::EventID>(t1.toMicrosecondsSinceEpoch())});
    }

    std::ifstream f(std::getenv("TEST_TMPDIR") + std::string("/JSONWriterTest.json"));
    std::string line;
    f >> line;
    EXPECT_EQ(line, "{\"checkpoint\":\"Liftoff\",\"id\":1234,\"timestamp\":1618234087000000}");
    f >> line;
    EXPECT_EQ(line,
              "{\"checkpoint\":\"send:Channel\",\"id\":4567,\"predecessors\":[1234],"
              "\"timestamp\":1428582428000000}");
    f >> line;
    EXPECT_EQ(line,
              "{\"checkpoint\":\"recv:Channel\",\"id\":6789,\"predecessors\":[1428582428000000],"
              "\"timestamp\":1523449506000000}");
    f >> line;
    EXPECT_TRUE(f.eof());
}

} // namespace
} // namespace av
