/// @copyright Copyright (c) 2021- Motional. All rights reserved.
///
/// NOTICE: Removing this heading is prohibited.
///
/// All information contained herein is, and remains the property of Motional,
/// its affiliates, and its suppliers (if any). The intellectual and technical
/// concepts contained herein are proprietary to Motional, its affiliates, and
/// its suppliers (if any) and may be covered by U.S. and Foreign Patents,
/// patents in process, and are protected by trade secrets or copyright law.
/// Dissemination of this information or reproduction of this material is
/// strictly forbidden unless prior written permission is obtained from Motional.
///
/// @file TestWriter
/// @brief Writer used for unit-testing LatencyLogger.

#include "platform/middleware/libs/LatencyLogger/include/LatencyLogger/LatencyLogger.h"
#include "platform/middleware/libs/TimeStamp/include/TimeStamp/otime.h"
#include <gtest/gtest.h>

namespace av {

struct LogEntry
{
    LogEntry(std::string loggingNameInput,
             std::string checkpointNameInput,
             ottotime::otimePoint timestampInput,
             av::latency::EventID eventIDInput,
             std::vector<av::latency::EventID>&& predecessorsInput)
        : loggingName(std::move(loggingNameInput))
        , checkpointName(std::move(checkpointNameInput))
        , timestamp(std::move(timestampInput))
        , eventID(eventIDInput)
        , predecessors(std::move(predecessorsInput))
    {}

    LogEntry(const LogEntry& other) = default;
    LogEntry(LogEntry&& other) = default;
    LogEntry& operator=(const LogEntry& other) = default;
    LogEntry& operator=(LogEntry&& other) = default;

    std::string loggingName;
    std::string checkpointName;
    ottotime::otimePoint timestamp;
    av::latency::EventID eventID;
    std::vector<av::latency::EventID> predecessors;
};

class TestWriter
{
public:
    TestWriter() {}

    const std::vector<LogEntry>& getEntries() { return entries_; }

    void operator()(const std::string& loggingName,
                    const std::string& checkpointName,
                    const ottotime::otimePoint& timestamp,
                    av::latency::EventID eventID,
                    std::vector<av::latency::Pred>&& predecessors)
    {
        LogEntry entry{loggingName, checkpointName, timestamp, eventID, std::move(predecessors)};
        entries_.push_back(entry);
    }

    void clear() { entries_.clear(); }

private:
    std::vector<LogEntry> entries_{};
};

} // namespace av
