/*!
 ***********************************************************************************************************************
 * MOTIONAL PRIVILEGED AND CONFIDENTIAL
 *
 * @file TwoInitializesTest.cpp
 * @brief This file implements tests for the LatencyLogger class.
 *
 * @section License
 *
 * Copyright (c) 2021- Motional. All rights reserved.
 * NOTE: Removing this heading is prohibited.
 *
 * @section Open Source License List
 *
 * This is a list of any open-source project used by this file.
 ***********************************************************************************************************************
 */

#include "platform/middleware/libs/LatencyLogger/test/TestWriter.hpp"
#include "platform/middleware/libs/config/include/config/ConfigParser.h"

namespace av {

TEST(TwoInitializesTest, TwoInitializes)
{
    ConfigParser parser;
    ConfigSection config;
    parser.parseString("Parameters = {\n"
                       "  loggingName = \"TwoInitializesTest\";\n"
                       "  enabled = True;\n"
                       "}\n");
    parser.getSection("Parameters", config);

    TestWriter testWriter;
    EXPECT_NO_THROW(LatencyLogger::initialize(config, std::ref(testWriter)));
    EXPECT_NO_THROW(LatencyLogger::getLogger());

    LatencyLogger& logger = LatencyLogger::getLogger();

    const av::latency::EventID liftoff = logger.RegularEvent("Liftoff");

    av::latency::EventID splashdown = logger.RegularEvent("Splashdown", {liftoff});

    EXPECT_NE(splashdown, av::latency::EventID{0});
    EXPECT_NE(splashdown, liftoff);

    const auto& entries = testWriter.getEntries();
    ASSERT_EQ(2, entries.size());

    EXPECT_NO_THROW(LatencyLogger::initialize(config, std::ref(testWriter)));
}

} // namespace av
