/// @copyright Copyright (c) 2021- Motional. All rights reserved.
///
/// NOTICE: Removing this heading is prohibited.
///
/// All information contained herein is, and remains the property of Motional,
/// its affiliates, and its suppliers (if any). The intellectual and technical
/// concepts contained herein are proprietary to Motional, its affiliates, and
/// its suppliers (if any) and may be covered by U.S. and Foreign Patents,
/// patents in process, and are protected by trade secrets or copyright law.
/// Dissemination of this information or reproduction of this material is
/// strictly forbidden unless prior written permission is obtained from Motional.
///
/// @file
/// @brief status reporting wrapper minimally implements the MonitoringStatusProvider interface

#ifndef MONITORING_NOTIFIER_HPP
#define MONITORING_NOTIFIER_HPP

#include <task/MonitoringStatusProvider.hpp>
#include <task/Task.h>

namespace av {

class MonitoringNotifier
{
public:
    /// Constructor
    /// @param[in] provider - interface object to MonitoringStatusProviderInterface
    MonitoringNotifier(std::shared_ptr<MonitoringStatusProvider> provider);

    virtual ~MonitoringNotifier() = default;

    /// Wrapper for monitorProvider_->updateStatus()
    /// @param[in] currentStatus - TaskState one of { DOWN, DEGRADED, UP, UNRECOVERABLE, UNINITIALIZED }
    /// @param[in] source - faultSource FaultSource::INTERNAL, FaultSource::DEPENDENCY
    /// @param[in] description - A human friendly contextually descriptive string
    virtual void notify(av::msg::TaskState currentState,
                        av::msg::FaultSource faultSource,
                        const std::string& description);

    ///
    /// wrapper to reset water mark to TaskState::UP and related on each executive()
    ///
    virtual void resetStatus();

private:
    std::shared_ptr<MonitoringStatusProvider> monitorProvider_; ///< report status to Gen2 Monitoring System

protected:
    ///
    /// mock class can construct the base class without pesky parameters.
    ///
    inline MonitoringNotifier() = default;
};

} // namespace av

#endif