/// @copyright Copyright (c) 2021- Motional. All rights reserved.
///
/// NOTICE: Removing this heading is prohibited.
///
/// All information contained herein is, and remains the property of Motional,
/// its affiliates, and its suppliers (if any). The intellectual and technical
/// concepts contained herein are proprietary to Motional, its affiliates, and
/// its suppliers (if any) and may be covered by U.S. and Foreign Patents,
/// patents in process, and are protected by trade secrets or copyright law.
/// Dissemination of this information or reproduction of this material is
/// strictly forbidden unless prior written permission is obtained from Motional.
///
/// @file
/// @brief unit tests for MonitoringStatusProvider

#include "platform/middleware/libs/task/include/task/MonitoringStatusProvider.hpp"
#include <MonitoringNotifier/MonitoringNotifier.hpp>
#include <gmock/gmock.h>
#include <gtest/gtest.h>

class MockMonitoringStatusProvider : public MonitoringStatusProvider
{
public:
    MockMonitoringStatusProvider()
        : MonitoringStatusProvider("foo"){};

    MOCK_METHOD4(updateStatus, bool(int64_t, av::msg::TaskState, av::msg::FaultSource, const std::string&));
    MOCK_METHOD0(resetStatus, void());
};

class MonitoringStatusProviderTest : public ::testing::Test
{
public:
    MonitoringStatusProviderTest() {}

protected:
    void SetUp() override {}
    void TearDown() override {}
};

TEST_F(MonitoringStatusProviderTest, notifyTest)
{
    auto provider = std::make_shared<MockMonitoringStatusProvider>();

    auto notifier = std::make_shared<av::MonitoringNotifier>(av::MonitoringNotifier(provider));

    EXPECT_NO_THROW(notifier->notify(av::msg::TaskState::UNRECOVERABLE, av::msg::FaultSource::INTERNAL, "unit test"));
}

TEST_F(MonitoringStatusProviderTest, resetStatusTest)
{
    auto provider = std::make_shared<MockMonitoringStatusProvider>();

    auto notifier = std::make_shared<av::MonitoringNotifier>(av::MonitoringNotifier(provider));

    EXPECT_NO_THROW(notifier->resetStatus());
}

int main(int argc, char** argv)
{
    ::testing::InitGoogleTest(&argc, argv);
    return RUN_ALL_TESTS();
}