#pragma once
#include <mutex>

namespace testing {
class CustomClock : public Clock
{
public:
    CustomClock() { initCurrentTime(); }

    av::TimePoint now() const
    {
        std::scoped_lock lock(mtx_);
        return curTime_;
    }

    ottotime::otimePoint getCurrentTime() override
    {
        const ottotime::otimeSpec spec{0, now().time_since_epoch() / av::Duration(1)};
        return ottotime::otimePoint(spec);
    }
    void setCurrentTime(av::TimePoint t)
    {
        std::scoped_lock lock(mtx_);
        curTime_ = t;
    }
    void advanceCurrentTime(int64_t secs) { setCurrentTime(curTime_ + av::Duration(secs * 1000000000)); }
    void initCurrentTime() { setCurrentTime(av::TimePoint(av::Duration{65})); }

private:
    av::TimePoint curTime_{};
    mutable std::mutex mtx_;
};
} // namespace testing
