/// @copyright Copyright (c) 2021- Motional. All rights reserved.
///
/// NOTICE: Removing this heading is prohibited.
///
/// All information contained herein is, and remains the property of Motional,
/// its affiliates, and its suppliers (if any). The intellectual and technical
/// concepts contained herein are proprietary to Motional, its affiliates, and
/// its suppliers (if any) and may be covered by U.S. and Foreign Patents,
/// patents in process, and are protected by trade secrets or copyright law.
/// Dissemination of this information or reproduction of this material is
/// strictly forbidden unless prior written permission is obtained from Motional.
///
/// @file
/// @brief Utility functions for deserializing ODatum types

#include <cereal/archives/json.hpp>

#include <ODatum/Deserializer.h>
#include <ODatum/IntrospectionArchive.h>
#include <ODatum/ODatum.h>

#include "TypeList.h"

namespace aptv {
/*
 * @brief FieldInfo printer helper
 *
 * Recursively prints a FieldInfo tree
 *
 * @param info: FieldInfo to print
 * @param indent: Indent level
 */
inline void printFieldInfo(const FieldInfoPtr info, size_t indent)
{
    std::cout << std::string(indent * 2, ' ') << "[" << info->type << "]"
              << " " << info->name;

    if (!info->value.empty()) {
        std::cout << " : " << info->value << std::endl;
    } else {
        std::cout << std::endl;
    }

    // Child nodes are optional
    if (info->children) {
        for (auto const c : *(info->children)) {
            printFieldInfo(c, indent + 1);
        }
    }
}

/*
 * @brief FieldInfo printer
 *
 * Recursively prints a FieldInfo tree
 *
 * @param info: FieldInfo to print
 */
inline void printFieldInfo(const FieldInfoPtr info)
{
    std::cout << "[" << info->type << "]"
              << " " << info->name;

    if (!info->value.empty()) {
        std::cout << " : " << info->value << std::endl;
    } else {
        std::cout << std::endl;
    }

    if (info->children) {
        // Recurse
        for (auto const c : *(info->children)) {
            printFieldInfo(c, 1);
        }
    }
}

} // namespace aptv

namespace odatum {
namespace deserializer {

aptv::FieldInfoPtr getFieldInfo(std::string& blob)
{
    aptv::ODatumPtr value = std::make_shared<aptv::ODatum>();
    aptv::deserializeODatum(value, blob);
    cereal::IntrospectionArchive ia;
    ia(value);
    return ia.getInfo();
}

void printFieldInfo(std::string& blob)
{
    aptv::printFieldInfo(getFieldInfo(blob));
}

std::string toJson(std::string& blob)
{
    aptv::ODatumPtr value = std::make_shared<aptv::ODatum>();
    aptv::deserializeODatum(value, blob);
    std::stringstream ss;
    // this scope is needed to force flushing the string buffer in cereal
    {
        cereal::JSONOutputArchive ar(ss);
        ar(CEREAL_NVP(value));
    }
    return ss.str();
}

} // namespace deserializer
} // namespace odatum
