#ifndef _ODAT_TYPE_SUPPORT_H_
#define _ODAT_TYPE_SUPPORT_H_

#include <array>
#include <deque>
#include <map>
#include <memory>
#include <queue>
#include <set>
#include <string>
#include <type_traits>
#include <typeinfo>
#include <unordered_map>
#include <unordered_set>
#include <vector>

// For __cxa_demangle
#include <cxxabi.h>

// For BinaryData<T>
#include <cereal/details/helpers.hpp>

//
// Pulled from:
// https://stackoverflow.com/questions/281818/unmangling-the-result-of-stdtype-infoname
//

namespace TypeSupport {
/*
 * @brief Demangle a mangled C++ type name
 */
inline std::string demangle(const std::string& mangled)
{
    int status = -1;

    std::unique_ptr<char, void (*)(void*)> res{abi::__cxa_demangle(mangled.c_str(), nullptr, nullptr, &status),
                                               std::free};

    std::string demangled(res.get());

    return (status == 0) ? demangled : mangled;
}

/*
 * @brief Get the type name for a type
 *
 * This handles most of the arithmetic types, but we need to specialize for std::string and
 * templated containers.  String is actually a templated basic_string with a long name and templated
 * types are unnecessarily verbose.
 */
template<typename T>
inline std::string name(const T& t)
{
    return demangle(typeid(t).name());
}

inline std::string name(const std::string& t)
{
    return "string";
}

template<typename T, size_t N>
inline std::string name(const std::array<T, N>& t)
{
    return "array";
}

template<typename T, typename Alloc>
inline std::string name(const std::vector<T, Alloc>& t)
{
    return "vector";
}

template<typename K, typename V>
inline std::string name(const std::map<K, V>& t)
{
    return "map";
}

template<typename K, typename V>
inline std::string name(const std::unordered_map<K, V>& t)
{
    return "unordered_map";
}

template<typename F, typename S>
inline std::string name(const std::pair<F, S>& t)
{
    return "pair";
}

template<typename T>
inline std::string name(const cereal::BinaryData<T>& b)
{
    return "binary";
}

template<typename T, typename Compare, typename Alloc>
inline std::string name(const std::set<T, Compare, Alloc>& s)
{
    return "set";
}

template<typename T, typename Compare, typename Alloc>
inline std::string name(const std::unordered_set<T, Compare, Alloc>& s)
{
    return "unordered_set";
}

} // namespace TypeSupport

#endif // _ODAT_TYPE_SUPPORT_H_
