/*!
 **********************************************************************************************************************
 * APTIV PRIVILEGED AND CONFIDENTIAL
 *
 * @file ODatumUnitTests.cc
 * @product Messaging Infrastructure
 * @version
 * @released
 * @updated
 * @brief Base OttoWare Datum (ODatum) unit tests
 *
 * Unit tests for our base datum type
 *
 * @section License
 *
 * Copyright (c) 2018- Aptiv. All rights reserved.
 * NOTE: Removing this heading is prohibited.
 *
 * @section Open Source License List
 * - Cereal
 *
 * This is a list of any open-source project used by this file.
 ***********************************************************************************************************************
 */

#include <cereal/archives/json.hpp>
#include <cereal/cereal.hpp>
#include <cereal/types/vector.hpp>

#include <cstdint>
#include <sstream>
#include <string>

#include <gtest/gtest.h>

TEST(JSONTests, CArraySupport)
{
    std::string expected = R"V0G0N({
    "value0": {
        "value0": 1,
        "value1": 2,
        "value2": 3
    }
})V0G0N";

    int array[] = {1, 2, 3};

    std::stringstream ss;
    {
        // Scoped to force a flush.  This is just how cereal's json archives work
        cereal::JSONOutputArchive ar(ss);
        ar(array);
    }

    auto actual = ss.str();

    EXPECT_EQ(expected, actual);
}

TEST(JSONTests, BinaryDataSupport)
{
    std::string expected = R"V0G0N({
    "value0": {
        "value0": "AQAAAAIAAAADAAAA"
    }
})V0G0N";

    int array[] = {1, 2, 3};

    std::stringstream ss;
    {
        // Scoped to force a flush.  This is just how cereal's json archives work
        cereal::JSONOutputArchive ar(ss);
        ar(cereal::binary_data(array, sizeof(array)));
    }

    auto actual = ss.str();

    EXPECT_EQ(expected, actual);
}

TEST(JSONTests, VectorSupport)
{
    std::string expected = R"V0G0N({
    "value0": [
        1,
        2,
        3
    ]
})V0G0N";

    std::vector<uint8_t> vec = {1, 2, 3};

    std::stringstream ss;
    {
        // Scoped to force a flush.  This is just how cereal's json archives work
        cereal::JSONOutputArchive ar(ss);
        ar(vec);
    }

    auto actual = ss.str();

    EXPECT_EQ(expected, actual);
}
