/*!
***********************************************************************************************************************
* APTIV PRIVILEGED AND CONFIDENTIAL
*
* @file HashUnitTests.cc
* @product Utility
* @version
* @released
* @updated
* @brief Hash unit tests
*
* Unit tests for hashing messages
*
* @section License
*
* Copyright (c) 2018- Aptiv. All rights reserved.
* NOTE: Removing this heading is prohibited.
*
* @section Open Source License List
* - Cereal
*
* This is a list of any open-source project used by this file.
***********************************************************************************************************************
*/
#include <ODatum/Hash.h> // hashVector

#include <gtest/gtest.h>

#include <array>
#include <iostream>
#include <string>
#include <vector>

struct CustomHashObject
{
    int a;
    bool b;
    char c;
    double d;
};

//
// Simple hash function for CustomHashObject
//
template<>
struct std::hash<CustomHashObject>
{
    std::size_t operator()(CustomHashObject const& obj) const noexcept
    {
        std::size_t result = 17;
        result = result * 31 + std::hash<int>()(obj.a);
        result = result * 31 + std::hash<bool>()(obj.b);
        result = result * 31 + std::hash<char>()(obj.c);
        result = result * 31 + std::hash<double>()(obj.d);
        return result;
    }
};

//
// Test that whenever we serialize a vector of int,
// we get back the same result
//
TEST(HashTests, HashVectorInt)
{
    // Testing a vector of ints
    std::vector<int> numbers = {4, 2, 0};

    std::size_t h = hashVector(numbers);

    EXPECT_EQ(h, 11093822180230);
}

//
// Test that whenever we serialize a vector of string,
// we get back the same result
//
TEST(HashTests, HashVectorString)
{
    // Testing a vector of strings
    std::vector<std::string> strings = {"test_0", "test_1", "test_2"};

    std::size_t h = hashVector(strings);

    EXPECT_EQ(h, 4229785794322581878);
}

//
// Test that whenever we serialize a vector of objects,
// we get back the same result
//
TEST(HashTests, HashVectorObjects)
{
    CustomHashObject a{4, true, 'a', 5.0};
    CustomHashObject b{2, true, 'b', 5.0};
    CustomHashObject c{0, false, 'c', 15.0};

    // Testing a vector of objects
    std::vector<CustomHashObject> objects = {a, b, c};

    std::size_t h = hashVector(objects);

    EXPECT_EQ(h, 5256639124347560770);
}
