#ifndef _ODAT_TEST_DATUMS_H_
#define _ODAT_TEST_DATUMS_H_

#include <ODatum/ODatum.h>

#include <cereal/types/array.hpp>
#include <cereal/types/map.hpp>
#include <cereal/types/set.hpp>
#include <cereal/types/string.hpp>
#include <cereal/types/unordered_map.hpp>
#include <cereal/types/unordered_set.hpp>
#include <cereal/types/utility.hpp>
#include <cereal/types/vector.hpp>

#include <array>
#include <map>
#include <set>
#include <string>
#include <unordered_map>
#include <unordered_set>
#include <utility>
#include <vector>

//
// Test datum for a pre-ODatum string wrapper
//
struct OldStringWrapper
{
public:
    std::string data;

    OldStringWrapper() {}
    explicit OldStringWrapper(std::string input)
        : data(input)
    {}

    ~OldStringWrapper() {}

    template<class Archive>
    void serialize(Archive& ar, unsigned int version)
    {
        if (version >= 1) {
            ar& cereal::make_nvp("data", data);
        }
    }
};

struct StringWrapper : public aptv::ODatum
{
public:
    std::string data;

    StringWrapper() {}
    explicit StringWrapper(std::string input)
        : data(input)
    {}

    ~StringWrapper() {}

    template<class Archive>
    void serialize(Archive& ar, unsigned int version)
    {
        if (version >= 1) {
            ar& cereal::make_nvp("data", data);
        }
    }

    virtual std::string str() const { return data; }

    bool operator==(const StringWrapper& other) const { return this->data == other.data; }

    bool operator!=(const StringWrapper& other) const { return !(*this == other); }
};

struct SizeTWrapper : public aptv::ODatum
{
public:
    std::size_t data;

    ~SizeTWrapper() {}

    template<class Archive>
    void serialize(Archive& ar, unsigned int version)
    {
        if (version >= 1) {
            ar& cereal::make_nvp("data", data);
        }
    }

    virtual std::string str() const { return std::to_string(data); }

    bool operator==(const SizeTWrapper& other) const { return this->data == other.data; }

    bool operator!=(const SizeTWrapper& other) const { return !(*this == other); }
};

struct NestedDatum : public aptv::ODatum
{
public:
    SizeTWrapper size;
    StringWrapper str;

    template<class Archive>
    void serialize(Archive& ar, unsigned int version)
    {
        if (version >= 1) {
            ar& cereal::make_nvp("size", size) & cereal::make_nvp("str", str);
        }
    }
};

struct VectorDatum : public aptv::ODatum
{
    std::vector<uint8_t> vec;

    template<class Archive>
    void serialize(Archive& ar, unsigned int version)
    {
        if (version >= 1) {
            ar& cereal::make_nvp("vec", vec);
        }
    }
};

struct ArrayDatum : public aptv::ODatum
{
    std::array<uint8_t, 8> arr;

    template<class Archive>
    void serialize(Archive& ar, unsigned int version)
    {
        if (version >= 1) {
            ar& cereal::make_nvp("arr", arr);
        }
    }
};

struct MapDatum : public aptv::ODatum
{
    std::map<std::string, uint8_t> m;

    template<class Archive>
    void serialize(Archive& ar, unsigned int version)
    {
        if (version >= 1) {
            ar& cereal::make_nvp("map", m);
        }
    }
};

struct UnorderedMapDatum : public aptv::ODatum
{
    std::unordered_map<std::string, uint8_t> m;

    template<class Archive>
    void serialize(Archive& ar, unsigned int version)
    {
        if (version >= 1) {
            ar& cereal::make_nvp("map", m);
        }
    }
};

struct PairDatum : public aptv::ODatum
{
    std::pair<std::string, uint8_t> p;

    template<class Archive>
    void serialize(Archive& ar, unsigned int version)
    {
        if (version >= 1) {
            ar& cereal::make_nvp("pair", p);
        }
    }
};

struct BinaryDataDatum : public aptv::ODatum
{
    std::array<uint8_t, 8> b;

    template<class Archive>
    void serialize(Archive& ar, unsigned int version)
    {
        if (version >= 1) {
            ar& cereal::binary_data(&b[0], b.size());
        }
    }
};

struct SetDatum : public aptv::ODatum
{
    std::set<uint8_t> s;

    template<class Archive>
    void serialize(Archive& ar, unsigned int version)
    {
        if (version >= 1) {
            ar& cereal::make_nvp("set", s);
        }
    }
};

struct UnorderedSetDatum : public aptv::ODatum
{
    std::unordered_set<uint8_t> s;

    template<class Archive>
    void serialize(Archive& ar, unsigned int version)
    {
        if (version >= 1) {
            ar& cereal::make_nvp("unordered_set", s);
        }
    }
};

struct NestedVectorDatum : public aptv::ODatum
{
    std::vector<VectorDatum> vec;

    template<class Archive>
    void serialize(Archive& ar, unsigned int version)
    {
        if (version >= 1) {
            ar& cereal::make_nvp("vec", vec);
        }
    }
};

CEREAL_CLASS_VERSION(OldStringWrapper, 5)

CEREAL_CLASS_VERSION(StringWrapper, 5)
CEREAL_REGISTER_TYPE(StringWrapper)

CEREAL_CLASS_VERSION(SizeTWrapper, 1)
CEREAL_REGISTER_TYPE(SizeTWrapper)

CEREAL_CLASS_VERSION(NestedDatum, 2)
CEREAL_REGISTER_TYPE(NestedDatum)

CEREAL_CLASS_VERSION(VectorDatum, 1)
CEREAL_REGISTER_TYPE(VectorDatum)

CEREAL_CLASS_VERSION(ArrayDatum, 1)
CEREAL_REGISTER_TYPE(ArrayDatum)

CEREAL_CLASS_VERSION(MapDatum, 1)
CEREAL_REGISTER_TYPE(MapDatum)

CEREAL_CLASS_VERSION(UnorderedMapDatum, 1)
CEREAL_REGISTER_TYPE(UnorderedMapDatum)

CEREAL_CLASS_VERSION(PairDatum, 1)
CEREAL_REGISTER_TYPE(PairDatum)

CEREAL_CLASS_VERSION(BinaryDataDatum, 1)
CEREAL_REGISTER_TYPE(BinaryDataDatum)

CEREAL_CLASS_VERSION(SetDatum, 1)
CEREAL_REGISTER_TYPE(SetDatum)

CEREAL_CLASS_VERSION(UnorderedSetDatum, 1)
CEREAL_REGISTER_TYPE(UnorderedSetDatum)

CEREAL_CLASS_VERSION(NestedVectorDatum, 1)
CEREAL_REGISTER_TYPE(NestedVectorDatum)

#endif
