/*!
 ***********************************************************************************************************************
 * APTIV PRIVILEGED AND CONFIDENTIAL
 *
 * @file LogReaderException.h
 * @product ReSimulation
 * @version TODO <Release Number>
 * @released TODO <date>
 * @updated Dec 12, 2018
 * @brief This file defines an exception class thrown by OfflineLogReader and OfflineLogWriter
 *
 * @sa https://conf.ptc.delphiauto.net/confluence/display/SI/Offline+Log+Utilities
 *
 * @section License
 *
 * Copyright (c) 2018- Aptiv. All rights reserved.
 * NOTE: Removing this heading is prohibited.
 ***********************************************************************************************************************
 */
#ifndef OFFLINELOGEXCEPTION_H_
#define OFFLINELOGEXCEPTION_H_

#include <exception>
#include <string>

/*!
 * @class OfflineLogException
 *
 * @brief Exception thrown by OfflineLogReader or OfflineLogWriter
 *
 * @sa OfflineLogReader.h
 * @sa OfflineLogWriter.h
 */
class OfflineLogException : virtual public std::exception
{
public:
    /*!
     * @enum Cause
     *
     * @brief Describes the underlying cause of the exception.
     */
    enum class Cause
    {
        SQL_ERROR,               ///< Set when the error was caused by the SQL interface
        MSG_TYPE_ERROR,          ///< Set when the error was caused by an invalid message type was passed for the log
        MSG_CONTENT_ERROR,       ///< Set when an invalid message was retrieved from the log
        LOG_MULTIPLE_WRITE_OPEN, ///< Set when the same log file is opened for writing concurrently multiple times
        ENTRY_INVALID_TIME,      ///< Set when a TIME in the SQL entry is an invalid value
        UNKNOWN                  ///< Set when the error was caused by unknown reasons
    };

    /*!
     * @brief Default constructor for OfflineLogException
     *
     * @return An instance of OfflineLogException
     */
    OfflineLogException()
        : exceptionCause_(Cause::UNKNOWN)
    { /* Nothing to do here */
    }

    /*!
     * @brief Copy constructor for OfflineLogException
     *
     * @return An instance of OfflineLogException
     */
    OfflineLogException(const OfflineLogException& other)
        : exceptionCause_(other.exceptionCause_)
        , whatStr_(other.whatStr_)
    { /* Nothing to do here */
    }

    OfflineLogException(Cause cause, std::string const& what)
        : exceptionCause_(cause)
        , whatStr_(what)
    {}

    /*!
     * @brief Destructor for LogReaderException
     */
    virtual ~OfflineLogException() {}

    /*!
     * @brief Exception what() interface; returns human-readable error specification.
     *
     * @return char* : C-str error
     */
    virtual const char* what() const noexcept { return whatStr_.c_str(); }

    Cause exceptionCause_; ///< Reason for the exception
    std::string whatStr_;  ///< String encapsulating the specified error
};

#endif /* OFFLINELOGEXCEPTION_H_ */
