#include <pybind11/pybind11.h>
#include <pybind11/stl.h>

#include <OfflineLogUtils/OfflineLogReader.h>
#include <OfflineLogUtils/OfflineLogWriter.h>

#include <interfaces/CompressedImageMessage/CompressedImageMessage.h>
#include <interfaces/D3Image/D3Image.h>
#include <interfaces/D3ImageDebugInfo/D3ImageDebugInfo.h>
#include <interfaces/LidarPointCloud/LidarPointCloud.h>
#include <interfaces/LocalizationHealth/LocalizationHealth.h>
#include <interfaces/LocalizationPose/LocalizationPose.h>
#include <interfaces/Pose/Pose.h>
#include <interfaces/Quaterniond/Quaterniond.h>
#include <interfaces/Vector3d/Vector3d.h>

using namespace av::geometry;
using namespace d3;
using namespace localization;
using namespace av::msg;

#define PY_OFFLINELOGUTILS_READER(Type)                                                                                \
    pybind11::class_<OfflineLogReader<Type>>(pyMod, "OfflineLogReader" #Type)                                          \
        .def(pybind11::init<const std::string&>())                                                                     \
        .def("hasMoreData", &OfflineLogReader<Type>::hasMoreData)                                                      \
        .def("get", &OfflineLogReader<Type>::get)                                                                      \
        .def("getWithTime", &OfflineLogReader<Type>::getWithTime)                                                      \
        .def("getAll", &OfflineLogReader<Type>::getAll)                                                                \
        .def("reset", &OfflineLogReader<Type>::reset)                                                                  \
        .def("getStartTime", &OfflineLogReader<Type>::getStartTime)                                                    \
        .def("getEndTime", &OfflineLogReader<Type>::getEndTime)                                                        \
        .def("finalize", &OfflineLogReader<Type>::finalize)                                                            \
        .def("__enter__", [](OfflineLogReader<Type>& obj) { return obj; })                                             \
        .def("__exit__", [](OfflineLogReader<Type>& obj) { obj.finalize(); })

#define PY_OFFLINELOGUTILS_WRITER(Type)                                                                                \
    pybind11::class_<OfflineLogWriter<Type>>(pyMod, "OfflineLogWriter" #Type)                                          \
        .def(pybind11::init<const std::string&>())                                                                     \
        .def("publish", &OfflineLogWriter<Type>::publish)                                                              \
        .def(                                                                                                          \
            "publishAtTime",                                                                                           \
            pybind11::overload_cast<const Type&, const ottotime::otimePoint&>(&OfflineLogWriter<Type>::publishAtTime)) \
        .def("publishAtTime",                                                                                          \
             pybind11::overload_cast<const Type&, const av::TimePoint&>(&OfflineLogWriter<Type>::publishAtTime))       \
        .def("finalize", &OfflineLogWriter<Type>::finalize)                                                            \
        .def("__enter__", [](OfflineLogWriter<Type>& obj) { return obj; })                                             \
        .def("__exit__", [](OfflineLogWriter<Type>& obj) { obj.finalize(); })

#define PY_OFFLINELOGUTILS(Type)                                                                                       \
    PY_OFFLINELOGUTILS_READER(Type);                                                                                   \
    PY_OFFLINELOGUTILS_WRITER(Type)

PYBIND11_MODULE(PyOfflineLogUtils, pyMod) // NOLINT
{
    pyMod.doc() = "pybind11 wrapper for OfflineLogUtils";
    PY_OFFLINELOGUTILS(Vector3d);
    PY_OFFLINELOGUTILS(Quaterniond);
    PY_OFFLINELOGUTILS(Pose);
    PY_OFFLINELOGUTILS(CompressedImageMessage);
    PY_OFFLINELOGUTILS(D3Image);
    PY_OFFLINELOGUTILS(D3ImageDebugInfo);
    PY_OFFLINELOGUTILS(LocalizationHealth);
    PY_OFFLINELOGUTILS(LocalizationPose);
    PY_OFFLINELOGUTILS(LidarPointCloud);
}
