#include <assert.h>
#include <cmath>
#include <mutex>

#include <OttoProfiler/OttoProfiler.h>

// See example in test OttoProfiler.cc
namespace ottoProfiler {

constexpr double NANOSEC_TO_MILISEC = 1e6;

std::mutex m;

void WelfordsAlgorithm::update(const double value)
{
    ++count_;
    const double oldmean{mean_};
    mean_ += (value - mean_) / static_cast<double>(count_);
    intermediate_ += ((value - mean_) * (value - oldmean));
    last_ = value;
}

ottoProfiler::ottoProfiler(const std::string& profiledName)
    : profiledName_(profiledName)
{
    consTime_ = std::chrono::high_resolution_clock::now();
}

mapType getTimeMap()
{
    return NameTimeMap;
}

ottoProfiler::~ottoProfiler()
{
    const my_clock endTime = std::chrono::high_resolution_clock::now();
    const my_time totalTime = std::chrono::duration_cast<std::chrono::nanoseconds>(endTime - consTime_);

    m.lock();

    NameTimeMap[profiledName_].update(totalTime.count());

    m.unlock();
}

void displayProfiler()
{
    std::cout << "==============================OTTO PROFILER DISPLAY==================================" << std::endl;
    for (const auto& keyVal : NameTimeMap) {
        std::string scopeName = keyVal.first;
        const uint64_t numCalled = keyVal.second.getCount();

        double meanTimeMs = keyVal.second.getMean() / NANOSEC_TO_MILISEC;

        double stdd = keyVal.second.getStandardDeviation() / NANOSEC_TO_MILISEC;

        // TODO: More fancy
        std::cout << " Scope Name: " << std::setw(30) << scopeName << " Average time(ms): " << std::setw(10)
                  << meanTimeMs << " STD: " << std::setw(10) << stdd << " Number called: " << numCalled << std::endl;
    }

    std::cout << "=====================================================================================" << std::endl;
}

void showLastRecorded()
{
    std::cout << "==============================LAST RECORDED TIME=====================================" << std::endl;
    for (const auto& keyVal : NameTimeMap) {
        std::string scopeName = keyVal.first;
        double timeMs = keyVal.second.getLast() / NANOSEC_TO_MILISEC;

        // TODO: More fancy
        std::cout << " Scope Name: " << std::setw(30) << scopeName << " Runing time(ms): " << timeMs << std::endl;
    }

    std::cout << "=====================================================================================" << std::endl;
}

} // namespace ottoProfiler
