#include <OttoProfiler/OttoProfiler.h>
#include <gtest/gtest.h>
#include <math.h>

using ottoProfiler::WelfordsAlgorithm;

TEST(WelfordAlgorithmTests, TestInitialConstruction)
{
    WelfordsAlgorithm welford{};

    EXPECT_EQ(welford.getCount(), 0);
    EXPECT_DOUBLE_EQ(welford.getMean(), 0);
    EXPECT_DOUBLE_EQ(welford.getStandardDeviation(), 0);
    EXPECT_DOUBLE_EQ(welford.getVariance(), 0);
    EXPECT_DOUBLE_EQ(welford.getLast(), 0);
}

TEST(WelfordAlgorithmTests, TestSingleUpdate)
{
    WelfordsAlgorithm welford{};

    welford.update(1.5);

    EXPECT_EQ(welford.getCount(), 1);
    EXPECT_DOUBLE_EQ(welford.getMean(), 1.5);
    EXPECT_DOUBLE_EQ(welford.getStandardDeviation(), 0);
    EXPECT_DOUBLE_EQ(welford.getVariance(), 0);
    EXPECT_DOUBLE_EQ(welford.getLast(), 1.5);
}

TEST(WelfordAlgorithmTests, TestMoreUpdates)
{
    WelfordsAlgorithm welford{};

    welford.update(1.0);
    welford.update(2.0);
    welford.update(3.0);
    welford.update(4.0);
    welford.update(5.0);

    EXPECT_EQ(welford.getCount(), 5);
    EXPECT_DOUBLE_EQ(welford.getMean(), 3);
    EXPECT_DOUBLE_EQ(welford.getStandardDeviation(), sqrt(2.5));
    EXPECT_DOUBLE_EQ(welford.getVariance(), 2.5);
    EXPECT_DOUBLE_EQ(welford.getLast(), 5.0);
}

GTEST_API_ int main(int argc, char** argv)
{
    testing::InitGoogleTest(&argc, argv);
    return RUN_ALL_TESTS();
}
