#include <gtest/gtest.h>

#include <Clock/SystemClock.h>
#include <ProcessLauncher/processUtil.h>

using namespace std;

TEST(ProcessLauncherTest, DISABLED_processUtil)
{
    // executeCommand
    EXPECT_FALSE(ProcessUtil::executeCommand("wrong_command"));
    EXPECT_TRUE(ProcessUtil::executeCommand("echo TEST"));

    // getTimeStamp
    ottotime::otimePoint curTime(ottotime::SystemClock::getSystemTime());
    if (!curTime.isTimeValid()) {
        curTime.getCurrentWallClockTime();
    }
    EXPECT_EQ(ProcessUtil::getTimeStamp(true), ProcessUtil::getTimeStamp(true, curTime));
    EXPECT_EQ(ProcessUtil::getTimeStamp(false), ProcessUtil::getTimeStamp(false, curTime));

    ottotime::otimePoint refTime(ottotime::SystemClock::getSystemTime());
    refTime = refTime.fromSecondsSinceEpochInt(100);
    EXPECT_EQ(ProcessUtil::getTimeStamp(true, refTime), "00:01:40");
    EXPECT_EQ(ProcessUtil::getTimeStamp(false, refTime), "1970.01.01.00.01.40");

    auto pbnTest1 = ProcessUtil::processesByName("ProcessLauncherTest", true);
    EXPECT_TRUE(pbnTest1.size() == 1);
    auto pbnTest2 = ProcessUtil::processesByName("ProcessLauncherTest", false);
    EXPECT_TRUE(pbnTest1.size() > 0);
    auto pbnTest3 = ProcessUtil::processesByName("ProcessDoesNotExist", false);
    EXPECT_TRUE(pbnTest3.empty());

    // Tasks
    EXPECT_TRUE(ProcessUtil::isAliveTaskByName("ProcessLauncherTest"));
    EXPECT_FALSE(ProcessUtil::isAliveTaskByName("ProcessDoesNotExist"));
}
