/**
 * \copyright
 * Copyright (c) 2021- Motional. All rights reserved.
 * NOTICE: Removing this heading is prohibited.
 * All information contained herein is, and remains the property of Motional, its affiliates, and its suppliers (if
 * any). The intellectual and technical concepts contained herein are proprietary to Motional, its affiliates, and its
 * suppliers (if any) and may be covered by U.S. and Foreign Patents, patents in process, and are protected by trade
 * secrets or copyright law. Dissemination of this information or reproduction of this material is strictly forbidden
 * unless prior written permission is obtained from Motional.
 *
 * \file
 * \brief File implementing the progress::report() API.
 */
#include "lib/avlog/avlog.hpp"
#include <ProgressReporter/ProgressReporter.h>
#include <ProgressReporter/ProgressReporterDiagnosticsMsg.hpp>
#include <utility/utility.h>

namespace progress {
bool Reporter::s_enabled{false};

namespace {
/// Returns a fixed string, a name for the given type of
/// progress. Should be the same as the enum, so that, if you see a
/// progress report, you know what to grep for to find it in the code.
///
/// \progressType The type of progress whose name is needed.
const char* progressTypeName(ProgressType progressType)
{
    switch (progressType) {
        case ProgressType::OS_BOOT_COMPLETE:
            return PROGRESS_MSG_OS_BOOT_COMPLETE;
        case ProgressType::STACK_BOOT_COMPLETE:
            return PROGRESS_MSG_STACK_BOOT_COMPLETE;
        case ProgressType::TASK_INITIALIZED:
            return PROGRESS_MSG_TASK_INITIALIZED;
        case ProgressType::TASK_INITIALIZATION_FAILED:
            return PROGRESS_MSG_TASK_INITIALIZATION_FAILED;
        case ProgressType::TASK_STARTED:
            return PROGRESS_MSG_TASK_STARTED;
        case ProgressType::TASK_EXITED:
            return PROGRESS_MSG_TASK_EXITED;
        case ProgressType::PROCESS_STARTED:
            return PROGRESS_MSG_PROCESS_STARTED;
        case ProgressType::SERVICE_AVAILABLE:
            return PROGRESS_MSG_SERVICE_AVAILABLE;
        case ProgressType::LATENCY_EVENT:
            return PROGRESS_MSG_LATENCY_EVENT;

            // Do not add a default case; if any new ProgressType enums
            // are added, we want the compiler to flag them.
    }

    // This shouldn't be reachable, but the compiler doesn't seem to
    // realize that, and complains "control reaches end of non-void
    // function".
    return "Unknown progress type";
}

} // namespace

void Reporter::initialize(ConfigSection& config)
{
    /// Once it's enabled, it never gets disabled.
    if (!s_enabled) {
        bool progressEnabled{false};
        if (config.get("progressEnabled", progressEnabled, false)) {
            s_enabled = progressEnabled;
        }
    }
} // namespace progress

void Reporter::reportInner(const ProgressType progressType, const std::string& parameter)
{
    AVLOG_INFO("@Progress@: {}: {}", progressTypeName(progressType), escapeForCSV(parameter));
}
} // namespace progress
