/**
 * \copyright
 * Copyright (c) 2019- Aptiv. All rights reserved.
 * NOTICE: Removing this heading is prohibited.
 * All information contained herein is, and remains the property of Aptiv and its suppliers, if any.
 * The intellectual and technical concepts contained herein are proprietary to Aptiv and its suppliers and may be
 * covered by U.S. and Foreign Patents, patents in process, and are protected by trade secret or copyright law.
 * Dissemination of this information or reproduction of this material is strictly forbidden unless prior written
 * permission is obtained from Aptiv.
 *
 * \brief Additional discovery functionality
 */

#include "Discovery.h"

#include "lib/avlog/avlog.hpp"

namespace SimpleComms {
void buildDiscoveryFilterLists(ConfigSection& routes,
                               const std::string& defaultInterface,
                               std::unordered_set<std::string>& blacklist,
                               std::unordered_map<std::string, std::unordered_set<std::string>>& whitelists)
{
    // We have routes, let's parse them into lists for discovery
    auto channels = routes.getParameterNames();
    for (const auto& channel : channels) {
        // The channel could be requested on multiple interfaces, in which case it is an array
        std::vector<std::string> interfaces;
        if (!routes.getArray(channel, interfaces)) {
            // If the config section value is not an array, it is a single string
            std::string interface;
            if (routes.get(channel, interface) == ConfigSection::GS_OK) {
                interfaces.push_back(interface);
            }
        }

        if (!interfaces.empty()) {
            // Loop over the interfaces and put the corresponding channel in the appropriate lists
            for (auto const& interface : interfaces) {
                if (interface == defaultInterface) {
                    // This is the default interface, nothing to do
                    continue;
                }

                auto auxIntf = whitelists.find(interface);
                if (auxIntf == whitelists.end()) {
                    AVLOG_WARN("{} is not a configured auxiliary interface for scs, ignoring", interface);
                } else {
                    // Always prevent this channel from being published on the default interface
                    blacklist.insert(channel);

                    // But only whitelist it on the appropriate auxiliary interface
                    auxIntf->second.insert(channel);
                }
            }
        } else {
            AVLOG_ERROR("No routes for this host");
        }
    }
}
} // namespace SimpleComms
