/*!
 ***********************************************************
 * @file Discovery.h
 * @brief A file defining the Discovery class.
 *
 * @section License
 *
 * Copyright (c) 2015- Delphi Automotive. All rights
 * reserved.
 *
 * @section changeLogs Change Logs
 *
 * @author Vasudeva Pai (vasudeva.pai@delphi.com)
 * @date 03/23/2018
 * @remark
 * removing boost dependency
 * @n
 ***********************************************************
 */

/**********************************************************
  Copyright (c) 2014-2015 Ottomatika, Inc.
 **********************************************************/

/**********************************************************
  Simple Comms
  Copyright (c) 2010-2013 Carnegie Mellon University,
  All rights reserved.

  This source code was developed in part with support from
  sponsors including General Motors, the National Science
  Foundation and the US Department of Transportation.

  Use of this software is strictly governed by agreements
  between Carnegie Mellon University and various sponsors.

  Users of this software must also be fully aware and agree
  that Carnegie Mellon does not guarantee the correct
  functioning of this software in any system. Carnegie
  Mellon or any of its affiliates will not be liable for any
  damage and/or penalties resulting from the use of this
  software. Any user of this software takes complete
  responsibility for the use of any software or design.
 **********************************************************/

/*
 * Discovery.h
 * Tugrul Galatali
 * 02/10/2007
 *
 * Copyright (C) 2007
 * Carnegie Mellon University
 * All rights reserved
 */
#ifndef _SIMPLECOMMS_DISCOVERY_H_
#define _SIMPLECOMMS_DISCOVERY_H_

#include <string>
#include <unordered_map>
#include <unordered_set>
#include <vector>

#include <config/ConfigSection.h>

#include "Peer.h"
#include <SimpleComms/Status.h>

namespace SimpleComms {
/*
 * @brief Resolve discovery interface filter lists from the configured values
 *
 * @param routes Configuration section of the channel to interface routes
 * @param defaultInterface Name of the default physical network interface
 * @param blacklist Returned default interface channel blacklist
 * @param whitelists Returned auxiliary interface whitelists
 */
void buildDiscoveryFilterLists(ConfigSection& routes,
                               const std::string& defaultInterface,
                               std::unordered_set<std::string>& blacklist,
                               std::unordered_map<std::string, std::unordered_set<std::string>>& whitelists);

class Discovery
{
public:
    virtual ~Discovery() {}

    virtual Status::States initialize() = 0;
    virtual Status::States isActive() = 0;
    virtual Status::States terminate() = 0;

    virtual Status::States registerLocalPeeringMethod(const PeerPtr localPeer) = 0;
    virtual Status::States unregisterLocalPeeringMethod(const PeerPtr localPeer) = 0;
    virtual Status::States registerRemotePeeringMethod(const PeerPtr remotePeer) = 0;
    virtual Status::States unregisterRemotePeeringMethod(const PeerPtr remotePeer) = 0;
};

typedef std::shared_ptr<Discovery> DiscoveryPtr;

} // namespace SimpleComms

#endif
