/*!
 ***********************************************************
 * @file NetworkIntfUtils.h
 * @brief A file defining the network interface utility
 * functions.
 *
 * @section License
 *
 * Copyright (c) 2015- Delphi Automotive. All rights
 * reserved.
 *
 * @section changeLogs Change Logs
 *
 ***********************************************************
 */

/**********************************************************
  Copyright (c) 2014-2015 Ottomatika, Inc.
 **********************************************************/

/**********************************************************
  Simple Comms
  Copyright (c) 2010-2013 Carnegie Mellon University,
  All rights reserved.

  This source code was developed in part with support from
  sponsors including General Motors, the National Science
  Foundation and the US Department of Transportation.

  Use of this software is strictly governed by agreements
  between Carnegie Mellon University and various sponsors.

  Users of this software must also be fully aware and agree
  that Carnegie Mellon does not guarantee the correct
  functioning of this software in any system. Carnegie
  Mellon or any of its affiliates will not be liable for any
  damage and/or penalties resulting from the use of this
  software. Any user of this software takes complete
  responsibility for the use of any software or design.
 **********************************************************/

/*
 * NetworkIntfUtils.h
 * Tugrul Galatali
 * 03/10/2007
 *
 * Copyright (c) 2006
 * Carnegie Mellon University
 * All rights reserved.
 */
#ifndef _NETWORKINTFUTILS_H_
#define _NETWORKINTFUTILS_H_

#include <map>
#include <stdint.h>
#include <string.h>
#include <string>

namespace NetworkIntfUtils {

typedef std::map<std::string, uint32_t> IntfNameAddrMap;

bool GetInterfaceAddresses(IntfNameAddrMap& intfs);

enum ParameterTypes
{
    Address,
    Netmask,
    Broadcast
};

bool GetInterfaceParameter(const std::string& name, const ParameterTypes& type, unsigned int& value);

// the kernel will double the requested size for internal use
// (so the default allocation will actually be 16MiB)
constexpr const std::size_t defaultBufferSize = 1024 * 1024 * 8;
int setSocketBufferSize(const int sockfd, const std::size_t bufSize = defaultBufferSize);

} // namespace NetworkIntfUtils

#endif
