/*!
 ***********************************************************
 * @file Server.h
 * @brief A file defining the Server class.
 *
 * @section License
 *
 * Copyright (c) 2015- Delphi Automotive. All rights
 * reserved.
 *
 * @section changeLogs Change Logs
 *
 * @author Vasudeva Pai (vasudeva.pai@delphi.com)
 * @date 03/23/2018
 * @remark
 * removing boost dependency
 * @n
 ***********************************************************
 */

/**********************************************************
  Copyright (c) 2014-2015 Ottomatika, Inc.
 **********************************************************/

/**********************************************************
  Simple Comms
  Copyright (c) 2010-2013 Carnegie Mellon University,
  All rights reserved.

  This source code was developed in part with support from
  sponsors including General Motors, the National Science
  Foundation and the US Department of Transportation.

  Use of this software is strictly governed by agreements
  between Carnegie Mellon University and various sponsors.

  Users of this software must also be fully aware and agree
  that Carnegie Mellon does not guarantee the correct
  functioning of this software in any system. Carnegie
  Mellon or any of its affiliates will not be liable for any
  damage and/or penalties resulting from the use of this
  software. Any user of this software takes complete
  responsibility for the use of any software or design.
 **********************************************************/

/*
 * Server.h
 * Tugrul Galatali
 * 02/10/2007
 *
 * Copyright (C) 2007
 * Carnegie Mellon University
 * All rights reserved
 */
#ifndef _SIMPLECOMMS_SERVER_H_
#define _SIMPLECOMMS_SERVER_H_

#include "MessageControl.h"
#include "Peer.h"

#include <config/ConfigSection.h>

#include <pthread.h>
#include <sys/poll.h>

#include <deque>
#include <list>
#include <string>
#include <unordered_set>

namespace SimpleComms {

class Server
{
public:
    Server(const PeerPtr local,
           const std::unordered_set<PeerPtr>& remotes,
           const std::string& logPath,
           MessageControl::Mode mode,
           ConfigSection& params);

    Server(const Server&) = delete;
    Server& operator=(Server const&) = delete;

    ~Server();

    bool start(const uint32_t numberOfOutputWorkers);
    void stop();

private:
    const PeerPtr local_;
    const std::unordered_set<PeerPtr> remotes_;
    MessageControl messageControl_;

    pthread_t inThread_;
    uint32_t numberOfOutputWorkers_{};
    std::unique_ptr<pthread_t[]> outThreads_;
    int inToOutPipe_[2]{-1, -1};
    bool running_{false};

    pthread_mutex_t writeQueuesMutex_;
    std::list<Peer::BaseClientPtr> writeWaiting_;
    std::deque<Peer::BaseClientPtr> writeReady_;
    pthread_cond_t writeQueuesNotEmpty_;

    pthread_mutex_t writeWaitingMutex_;
    uint32_t maxWriteHandles_{};
    std::unique_ptr<pollfd[]> writefds_;

    // Global server statistics
    unsigned reads_{};
    unsigned localReadBytes_{};
    unsigned remoteReadBytes_{};
    unsigned writes_{};

    void initialize();

    static void* inThreadMainLauncher(void* arg);
    static void* outThreadMainLauncher(void* arg);

    void inThreadMain();
    // Queue a message for all subscribers, local and remote, and inform the out thread.
    // This is a helper function for the input and scheduler threads.
    bool queueMsg(const MessagePtr& msg, Peer* currentHandlePeer = nullptr);
    void outThreadMain();

    void doPoll();
};

} // namespace SimpleComms

#endif
