/*!
 ***********************************************************
 * @file TcpPeer.h
 * @brief A file defining the TcpPeer class
 *
 * @section License
 *
 * Copyright (c) 2015- Delphi Automotive. All rights
 * reserved.
 *
 * @section changeLogs Change Logs
 *
 * @author Vasudeva Pai (vasudeva.pai@delphi.com)
 * @date 03/23/2018
 * @remark
 * removing boost dependency
 * @n
 *
 * @author Junsung Kim
 * @date 03/09/2016
 * @remark
 * Added the cross-platform support
 *  -- 32-bit and big-endian
 * @n
 *
 ***********************************************************
 */

/**********************************************************
  Copyright (c) 2014-2015 Ottomatika, Inc.
 **********************************************************/

/**********************************************************
  Simple Comms
  Copyright (c) 2010-2013 Carnegie Mellon University,
  All rights reserved.

  This source code was developed in part with support from
  sponsors including General Motors, the National Science
  Foundation and the US Department of Transportation.

  Use of this software is strictly governed by agreements
  between Carnegie Mellon University and various sponsors.

  Users of this software must also be fully aware and agree
  that Carnegie Mellon does not guarantee the correct
  functioning of this software in any system. Carnegie
  Mellon or any of its affiliates will not be liable for any
  damage and/or penalties resulting from the use of this
  software. Any user of this software takes complete
  responsibility for the use of any software or design.
 **********************************************************/

/*
 * TcpPeer.h
 * Tugrul Galatali
 * 02/10/2007
 *
 * Copyright (C) 2007
 * Carnegie Mellon University
 * All rights reserved
 */
#ifndef _SIMPLECOMMS_TCPPEER_H_
#define _SIMPLECOMMS_TCPPEER_H_

#include "BasePeer.h"

namespace SimpleComms {

class TcpPeer : public BasePeer
{
public:
    TcpPeer(const std::string& advertisedIntf,
            const uint16_t port,
            const int backLog,
            const TimeStamp& staleFragmentPruningThreshold);
    virtual ~TcpPeer();

    virtual Status::States connect();
    virtual Status::States disconnect();

    virtual Status::States getConnectionInfo(Message& msg);

    virtual Status::States receiveMsg(const BaseClientPtr& clientHandle, MessagePtr& msg);

    virtual Status::States getNamedReadHandles(BaseClientList& readHandles);

    virtual Status::States handleErrors(BaseClientList& errorHandles);

protected:
    const std::string& advertisedIntf_;
    uint16_t port_;
    int backLog_;

    class InboundClientDataTCP : public BaseClientType
    {
    public:
        InboundClientDataTCP(BasePeer* _bp, const int _fd)
            : fd(_fd)
            , msgBufIndex(0)
            , bp_(_bp)
        {}
        virtual ~InboundClientDataTCP() {}

        virtual Peer* getParentPeer() { return bp_; }
        virtual std::string getName() { return ""; }
        virtual int getReadHandle() { return fd; }
        virtual int getWriteHandle() { return -1; }
        virtual bool isReadyToWrite() { return false; }
        virtual void setReadyToWrite() {}
        virtual bool hasDataToWrite() { return false; }
        virtual void setOnWriteQueue() {}
        virtual void setOffWriteQueue() {}
        virtual bool isOnWriteQueue() { return false; }

        int fd;

        // MessageStub ends with a 0-size array, which g++ doesn't like.
#pragma GCC diagnostic push
#pragma GCC diagnostic ignored "-Wpedantic"
        MessageStub msgHeaderBuf;
#pragma GCC diagnostic pop

        MessagePtr msg;
        uint32_t msgBufIndex;

    private:
        BasePeer* bp_;
    };

    typedef std::shared_ptr<InboundClientDataTCP> InboundClientDataTCPPtr;
    typedef std::vector<InboundClientDataTCPPtr> InboundClientList;

    InboundClientList inboundClients_;
    void inboundClientReset(const InboundClientDataTCPPtr& inboundClient);

    virtual bool clientConnect(const ClientDataPtr& namedClient, const ConnectionMessage* connMsg);
};

} // namespace SimpleComms

#endif
