/*!
 ***********************************************************
 * @file Message.h
 * @brief A file defining general messages.
 *
 * @section License
 *
 * Copyright (c) 2015- Delphi Automotive. All rights
 * reserved.
 *
 * @section changeLogs Change Logs
 *
 * @author Vasudeva Pai (vasudeva.pai@delphi.com)
 * @date 03/23/2018
 * @remark
 * removing boost dependency
 * @n
 *
 * @author Junsung Kim
 * @date 03/09/2016
 * @remark
 * Added the cross-platform support
 *  -- 32-bit and big-endian
 * @n
 *
 ***********************************************************
 */

/**********************************************************
  Copyright (c) 2014-2015 Ottomatika, Inc.
 **********************************************************/

/**********************************************************
  Simple Comms
  Copyright (c) 2010-2013 Carnegie Mellon University,
  All rights reserved.

  This source code was developed in part with support from
  sponsors including General Motors, the National Science
  Foundation and the US Department of Transportation.

  Use of this software is strictly governed by agreements
  between Carnegie Mellon University and various sponsors.

  Users of this software must also be fully aware and agree
  that Carnegie Mellon does not guarantee the correct
  functioning of this software in any system. Carnegie
  Mellon or any of its affiliates will not be liable for any
  damage and/or penalties resulting from the use of this
  software. Any user of this software takes complete
  responsibility for the use of any software or design.
 **********************************************************/

/*
 * Message.h
 * Tugrul Galatali
 * 02/10/2007
 *
 * Copyright (C) 2007
 * Carnegie Mellon University
 * All rights reserved
 */
#ifndef _SIMPLECOMMS_MESSAGE_H_
#define _SIMPLECOMMS_MESSAGE_H_

#include <netinet/in.h>
#include <stdint.h>
#include <string.h>
#include <zlib.h>

#include <cassert>
#include <iostream>
#include <memory>

#include "Constants.h"
#include "TimeStamp.h"

namespace SimpleComms {

struct MessageHeader
{
    MessageHeader() {}

    char origin[MAX_NULL_TERM_CLIENT_NAME];
    char channel[MAX_NULL_TERM_CHANNEL_NAME];
    struct TimeStamp sendMsgTime;
    uint32_t length;
    uint32_t rollingMessageID;
    uint32_t totalPages;

    uint32_t pageNumber;
    uint32_t dataLength;
    uint32_t headerChecksum; // Checking header or data checksums is deprecated.
    uint32_t dataChecksum;   // Checking header or data checksums is deprecated.

    uint32_t MessageLength() const { return static_cast<uint32_t>(sizeof(MessageHeader)) + ntohl(dataLength); }
} __attribute__((packed));

struct Message : MessageHeader
{
    Message() {}

    unsigned char data[MAX_FRAGMENT_PAYLOAD];
} __attribute__((packed));

struct MessageBase : MessageHeader
{
    MessageBase() {}

    unsigned char* data;
};

struct MessageStub : MessageHeader
{
    MessageStub() {}
    MessageStub(const Message& source) { memcpy(this, reinterpret_cast<const void*>(&source), source.MessageLength()); }

    unsigned char data[0];

    void* operator new(const size_t size) { return ::operator new(size); }

    void* operator new(const size_t size, const unsigned int dataLength) { return ::operator new(size + dataLength); }

    void operator delete(void* retired) { ::operator delete(retired); }
};

typedef std::shared_ptr<MessageStub> MessagePtr;

#include "Message.def.h"

} // namespace SimpleComms

#endif
