/*!
 ***********************************************************************************************************************
 * APTIV PRIVILEGED AND CONFIDENTIAL
 *
 * @file MockClient.h
 * @brief Mocks out the Client class for using it in tests.
 *
 * @section License
 *
 * Copyright (c) 2018- Aptiv. All rights reserved.
 * NOTE: Removing this heading is prohibited.
 *
 * @section Open Source License List
 *
 * This is a list of any open-source project used by this file.
 ***********************************************************************************************************************
 */

#ifndef _SIMPLECOMMS_CLIENT_MOCK_H_
#define _SIMPLECOMMS_CLIENT_MOCK_H_

#include <gmock/gmock.h>
#include <utility>

#include <SimpleComms/Client.h>
#include <mutex>

namespace SimpleComms {

class MockClient : public Client
{
public:
    MOCK_METHOD(Status::States, connect, ());
    MOCK_METHOD(Status::States, disconnect, ());
    MOCK_METHOD(Status::States, declare, (const std::string&, ChannelType, uint32_t));
    MOCK_METHOD(Status::States, undeclare, (const std::string&, ChannelType));

    MOCK_METHOD(Status::States, sendMsgV, (const std::string&, const std::string&, std::vector<av::latency::Pred>&&));
    MOCK_METHOD(void, hasMsg_, (), (const));
    MOCK_METHOD(Status::States, receiveMsg, (const std::string&, std::string&, MessageInfo&));
    MOCK_METHOD(void, allowThrottling, (std::function<void(bool)> callback));
    MOCK_METHOD(std::vector<std::string>, getPublishListMessage, ());

    bool hasMsg() const override
    {
        std::lock_guard<std::mutex> lock(mtx_hasMessages);

        hasMsg_();

        return hasMessages;
    }

    void setHasMessages(bool value)
    {
        std::lock_guard<std::mutex> lock(mtx_hasMessages);
        hasMessages = value;
    }

    mutable std::mutex mtx_hasMessages;
    bool hasMessages{false};
};

} // namespace SimpleComms

#endif /* _SIMPLECOMMS_CLIENT_MOCK_H_ */
