/**
 * \copyright
 * Copyright (c) 2019- Aptiv. All rights reserved.
 * NOTICE: Removing this heading is prohibited.
 * All information contained herein is, and remains the property of Aptiv and its suppliers, if any.
 * The intellectual and technical concepts contained herein are proprietary to Aptiv and its suppliers and may be
 * covered by U.S. and Foreign Patents, patents in process, and are protected by trade secret or copyright law.
 * Dissemination of this information or reproduction of this material is strictly forbidden unless prior written
 * permission is obtained from Aptiv.
 *
 * \brief Tests for support functionality for the main SimpleCoimms server program
 */

#include <ServerMainSupport.h>
#include <config/ConfigParser.h>

#include <gtest/gtest.h>

#include <string>
#include <unordered_map>
#include <unordered_set>
#include <vector>

const std::string TEST_RESOURCES = std::getenv("TEST_RESOURCES");

using namespace SimpleComms;

TEST(ParseInterfaceListsTests, emptyConfig)
{
    auto configFilePath = TEST_RESOURCES + "/EmptyConfig.cfg";

    std::vector<std::string> auxIntfs;
    std::unordered_set<std::string> defaultIntfBlacklist;
    std::unordered_map<std::string, std::unordered_set<std::string>> auxIntfWhitelists;

    ConfigParser cp;
    ASSERT_TRUE(cp.parseFile(configFilePath));

    ConfigSection emptyConfig;
    parseInterfaceLists(emptyConfig, "eth0", auxIntfs, defaultIntfBlacklist, auxIntfWhitelists);

    EXPECT_TRUE(auxIntfs.empty());
    EXPECT_TRUE(defaultIntfBlacklist.empty());
    EXPECT_TRUE(auxIntfWhitelists.empty());
}

TEST(ParseInterfaceListsTests, basicAuxIntfTest)
{
    auto configFilePath = TEST_RESOURCES + "/TestConfig.cfg";

    std::vector<std::string> auxIntfs;
    std::unordered_set<std::string> defaultIntfBlacklist;
    std::unordered_map<std::string, std::unordered_set<std::string>> auxIntfWhitelists;

    ConfigParser cp;
    ASSERT_TRUE(cp.parseFile(configFilePath));

    ConfigSection hostConfig;
    ASSERT_TRUE(cp.getSection("pc-one", hostConfig));

    parseInterfaceLists(hostConfig, "eth0", auxIntfs, defaultIntfBlacklist, auxIntfWhitelists);

    EXPECT_EQ(auxIntfs, std::vector<std::string>{"bond0"});
    EXPECT_EQ(defaultIntfBlacklist, std::unordered_set<std::string>{"LidarData"});

    std::unordered_map<std::string, std::unordered_set<std::string>> expectedWhitelists{{"bond0", {"LidarData"}}};
    EXPECT_EQ(auxIntfWhitelists, expectedWhitelists);
}

TEST(ParseInterfaceListsTests, sameChannelTwoAux)
{
    auto configFilePath = TEST_RESOURCES + "/TestConfig.cfg";

    std::vector<std::string> auxIntfs;
    std::unordered_set<std::string> defaultIntfBlacklist;
    std::unordered_map<std::string, std::unordered_set<std::string>> auxIntfWhitelists;

    ConfigParser cp;
    ASSERT_TRUE(cp.parseFile(configFilePath));

    ConfigSection hostConfig;
    ASSERT_TRUE(cp.getSection("pc-log", hostConfig));

    parseInterfaceLists(hostConfig, "eth0", auxIntfs, defaultIntfBlacklist, auxIntfWhitelists);

    std::vector<std::string> expectedAuxIntfs{"bond0", "bond1"};
    EXPECT_EQ(auxIntfs, expectedAuxIntfs);

    EXPECT_EQ(defaultIntfBlacklist, std::unordered_set<std::string>{"NetworkStatistics"});

    std::unordered_map<std::string, std::unordered_set<std::string>> expectedWhitelists{
        {"bond0", {"NetworkStatistics"}}, {"bond1", {"NetworkStatistics"}}};

    EXPECT_EQ(auxIntfWhitelists, expectedWhitelists);
}

int main(int argc, char** argv)
{
    ::testing::InitGoogleTest(&argc, argv);
    return RUN_ALL_TESTS();
}
