/*!
 ***********************************************************************************************************************
 * APTIV PRIVILEGED AND CONFIDENTIAL
 *
 * @file TestingBasePeer.h
 * @brief This file allows access to protected members of BasePeer for testing purposes.
 *
 * @section License
 *
 * Copyright (c) 2018- Aptiv. All rights reserved.
 * NOTE: Removing this heading is prohibited.
 *
 * @section Open Source License List
 *
 * This is a list of any open-source project used by this file.
 ***********************************************************************************************************************
 */

#ifndef _SIMPLECOMMS_TEST_TESTING_BASE_PEER_H_
#define _SIMPLECOMMS_TEST_TESTING_BASE_PEER_H_

#include <memory>
#include <string>
#include <vector>

#include <BasePeer.h>

namespace SimpleComms {

class TestingBasePeer : public BasePeer
{
public:
    void setChannels(std::vector<ChannelData>& channels) { channels_ = channels; }
    std::vector<ChannelData> getChannels() { return channels_; }

    TestingBasePeer(bool supportsPartialWrites = false)
        : BasePeer(Transports::TransportTypes::UnixDomainSocket, supportsPartialWrites, TimeStamp(1))
    {
        fd_ = 0;
    }

    size_t numChannels() { return channels_.size(); }

    // Trivially implement the BasePeer pure virtual functions so we can actually instantiate

    virtual Status::States connect() { return Status::States::Error; }

    virtual Status::States disconnect() { return Status::States::Error; }

    virtual Status::States getConnectionInfo(SimpleComms::Message& /*msg*/) { return Status::States::Error; }

    virtual bool clientConnect(const ClientDataPtr& /*namedClient*/, const ConnectionMessage* /*connMsg*/)
    {
        return false;
    }

    virtual bool clientConnect(const ClientDataPtr& /*namedClient*/, const bool /*retireEntry*/ = false)
    {
        return false;
    }

    virtual void resetPassthrough(const ClientDataPtr& namedClient, const bool retireEntry = false)
    {
        clientReset(namedClient, retireEntry);
    }

    virtual bool clientLookupPassthrough(const std::string& name,
                                         ClientList::iterator& namedClient,
                                         const bool create = false)
    {
        return clientLookup(name, namedClient, create);
    }
};

} // namespace SimpleComms
#endif
