#!/usr/bin/python

import os
import subprocess
import sys
import threading
import time

TX_PATH = os.path.join(
    os.environ["OTTO_DIR"], "../ottomatika/src/infrastructure/simplecomms/benchmarking/obj/multiSender"
)
RX_PATH = os.path.join(
    os.environ["OTTO_DIR"], "../ottomatika/src/infrastructure/simplecomms/benchmarking/obj/multiReceiver"
)
TX_CONFIG = os.path.join(os.environ["OTTO_DIR"], "../ottomatika/src/infrastructure/simplecomms/benchmarking/tx.config")
RX_CONFIG = os.path.join(os.environ["OTTO_DIR"], "../ottomatika/src/infrastructure/simplecomms/benchmarking/rx.config")

DURATION = 30
SIZE_RANGE = (32, 1048576)
SIZE_STEP = 2

OUT_FILE = "scs_results.csv"

pid = None


def run_once(datum_size):
    def _rx_proc():
        output = subprocess.check_output([RX_PATH, RX_CONFIG, pid])

        results = output.split("\n")[-5:]
        results = results[: len(results) - 1]

        with open("scs_results.csv", "a") as f:
            f.write("{},".format(datum_size))
            f.write(results[-1])
            f.write("\n")

    t = threading.Thread(target=_rx_proc)
    t.start()

    # give things some time to configure
    time.sleep(3)

    with open("/dev/null", "w"):
        # Send data and terminate the receiver when done
        subprocess.call([TX_PATH, TX_CONFIG])

    t.join()
    time.sleep(3)


def benchmark():
    datum_size = SIZE_RANGE[0]
    while datum_size <= SIZE_RANGE[1]:
        lines = []
        with open(TX_CONFIG, "r") as f:
            lines = f.readlines()
            lines[0] = str(datum_size) + "\n"
            lines[1] = str(DURATION) + "\n"

        with open(TX_CONFIG, "w") as f:
            for line in lines:
                f.write(line)

        run_once(datum_size)

        datum_size *= SIZE_STEP


if __name__ == "__main__":
    if len(sys.argv) != 2:
        sys.exit()

    pid = sys.argv[1]
    benchmark()
