#include <SparseMap/SparseMap.h>
#include <ctime>   // clock
#include <fstream> // ifstream
#include <gridMapCore/GridMap.hpp>
#include <gtest/gtest.h>

#include "lib/avlog/avlog.hpp"

// define the source dir so that the test can find it's data
#ifndef TESTDIR
#define TESTDIR "platform/middleware/libs/SparseMap/test/"
#endif

using namespace std;
using namespace grid_map;

const double thres = 0.0001;
const string layer = "cost";
const bool test_large = false; // Data files aren't present for these tests

const string config_path = string(TESTDIR);

const string sparseMapSmallEvenData = config_path + "data/sparseMapSmallEvenData.txt";

const string sparseMapSmallEvenConfig = config_path + "data/sparseMapSmallEvenConfig.json";

const string genSparseMapSmallEvenData = config_path + "data/genSparseMapSmallEvenData.txt";

const string genSparseMapSmallEvenConfig = config_path + "data/genSparseMapSmallEvenConfig.json";

const string sparseMapSmallOddData = config_path + "data/sparseMapSmallOddData.txt";

const string sparseMapSmallOddConfig = config_path + "data/sparseMapSmallOddConfig.json";

const string genSparseMapSmallOddData = config_path + "data/genSparseMapSmallOddData.txt";

const string genSparseMapSmallOddConfig = config_path + "data/genSparseMapSmallOddConfig.json";

const string genSparseMapLargeData = config_path + "data/genSparseMapLargeData.txt";

const string genSparseMapLargeConfig = config_path + "data/genSparseMapLargeConfig.json";

const string sparseMapConfigFileLargeData = config_path + "data/radarMap_REM.json";

const string gridMapLargeData = config_path + "data/radarMap_REM.bin";

const string JH_data = config_path + "data/test_JH/lidarMap_demo_Pacifica.bin";

const string JH_config = config_path + "data/test_JH/lidarMap_demo_Pacifica.json";

struct subMapParams
{
    subMapParams(Position pos, Length len)
    {
        position = pos;
        length = len;
    }
    Position position;
    Length length;
};

void iniSparseMapSmallOddData(SparseMapStructure& oriSparseMap)
{
    /*
        oriSparseMap[0] = {make_pair(2, 2.46), make_pair(7, 1.27), make_pair(8, 1.32)};
        oriSparseMap[2] = {make_pair(2, 1.2), make_pair(4, 5.6), make_pair(6, 1.3),
                           make_pair(7, 123.56)};
        oriSparseMap[3] = {make_pair(0, 1.34), make_pair(4, 35), make_pair(5, 1.2),
                           make_pair(7, 45.34), make_pair(8, 12.56)};
        oriSparseMap[4] = {make_pair(0, 0.98), make_pair(3, 5.6), make_pair(4, 7),
                           make_pair(7, 2), make_pair(8, 6.7)};
        oriSparseMap[5] = {make_pair(3, 2), make_pair(5, 2.5), make_pair(8, 5.8)};
        oriSparseMap[7] = {make_pair(0, 4.5), make_pair(2, 1.23), make_pair(3, 0.46),
                           make_pair(5, 2.3), make_pair(6, 9.45), make_pair(8, 3)};
        oriSparseMap[8] = {make_pair(2, 23.1), make_pair(4, 35), make_pair(8, 4)};
    */

    std::map<int, std::vector<float>> data_v;
    std::map<int, std::vector<int>> index_v;
    data_v[0] = {2.46, 1.27, 1.32};
    index_v[0] = {2, 7, 8};
    data_v[2] = {1.2, 5.6, 1.3, 123.56};
    index_v[2] = {2, 4, 6, 7};
    data_v[3] = {1.34, 35, 1.2, 45.34, 12.56};
    index_v[3] = {0, 4, 5, 7, 8};
    data_v[4] = {0.98, 5.6, 7, 2, 6.7};
    index_v[4] = {0, 3, 4, 7, 8};
    data_v[5] = {2, 2.5, 5.8};
    index_v[5] = {3, 5, 8};
    data_v[7] = {4.5, 1.23, 0.46, 2.3, 9.45, 3};
    index_v[7] = {0, 2, 3, 5, 6, 8};
    data_v[8] = {23.1, 35, 4};
    index_v[8] = {2, 4, 8};
    oriSparseMap.SparseMapData = data_v;
    oriSparseMap.SparseMapIndex = index_v;
    /* oriSparseMap[0][2] = 2.46; oriSparseMap[0][7] = 1.27; oriSparseMap[0][8] = 1.32;

    oriSparseMap[2][2] = 1.2; oriSparseMap[2][4] = 5.6; oriSparseMap[2][6] = 1.3;
    oriSparseMap[2][7] = 123.56;

    oriSparseMap[3][0] = 1.34; oriSparseMap[3][4] = 35; oriSparseMap[3][5] = 1.2;
    oriSparseMap[3][7] = 45.34; oriSparseMap[3][8] = 12.56;

    oriSparseMap[4][0] = 0.98; oriSparseMap[4][3] = 5.6; oriSparseMap[4][4] = 7;
    oriSparseMap[4][7] = 2; oriSparseMap[4][8] = 6.7;

    oriSparseMap[5][3] = 2; oriSparseMap[5][5] = 2.5; oriSparseMap[5][8] = 5.8;

    oriSparseMap[7][0] = 4.5; oriSparseMap[7][2] = 1.23; oriSparseMap[7][3] = 0.46;
    oriSparseMap[7][5] = 2.3; oriSparseMap[7][6] = 9.45; oriSparseMap[7][8] = 3;

    oriSparseMap[8][2] = 23.1; oriSparseMap[8][4] = 35; oriSparseMap[8][8] = 4;  */
}

void iniSparseMapSmallEvenData(SparseMapStructure& oriSparseMap)
{
    /*     oriSparseMap[0] = {make_pair(2, 2.46), make_pair(7, 1.27), make_pair(8, 1.32)};
        oriSparseMap[2] = {make_pair(2, 1.2), make_pair(4, 5.6), make_pair(6, 1.3),
                           make_pair(7, 123.56)};
        oriSparseMap[3] = {make_pair(0, 1.34), make_pair(4, 35), make_pair(5, 1.2),
                           make_pair(7, 45.34), make_pair(8, 12.56), make_pair(9, 1)};
        oriSparseMap[4] = {make_pair(0, 0.98), make_pair(3, 5.6), make_pair(4, 7),
                           make_pair(7, 2), make_pair(8, 6.7), make_pair(9, 2)};
        oriSparseMap[5] = {make_pair(3, 2), make_pair(5, 2.5), make_pair(8, 5.8)};
        oriSparseMap[7] = {make_pair(0, 4.5), make_pair(2, 1.23), make_pair(3, 0.46),
                           make_pair(5, 2.3), make_pair(6, 9.45), make_pair(8, 3),
                           make_pair(9, 3)};
        oriSparseMap[8] = {make_pair(2, 23.1), make_pair(4, 35), make_pair(8, 4),
                           make_pair(9, 40)}; */
    std::map<int, std::vector<float>> data_v;
    std::map<int, std::vector<int>> index_v;
    data_v[0] = {2.46, 1.27, 1.32};
    index_v[0] = {2, 7, 8};
    data_v[2] = {1.2, 5.6, 1.3, 123.56};
    index_v[2] = {2, 4, 6, 7};
    data_v[3] = {1.34, 35, 1.2, 45.34, 12.56, 1};
    index_v[3] = {0, 4, 5, 7, 8, 9};
    data_v[4] = {0.98, 5.6, 7, 2, 6.7, 2};
    index_v[4] = {0, 3, 4, 7, 8, 9};
    data_v[5] = {2, 2.5, 5.8};
    index_v[5] = {3, 5, 8};
    data_v[7] = {4.5, 1.23, 0.46, 2.3, 9.45, 3, 3};
    index_v[7] = {0, 2, 3, 5, 6, 8, 9};
    data_v[8] = {23.1, 35, 4, 40};
    index_v[8] = {2, 4, 8, 9};
    oriSparseMap.SparseMapData = data_v;
    oriSparseMap.SparseMapIndex = index_v;

    /* oriSparseMap[0][2] = 2.46; oriSparseMap[0][7] = 1.27; oriSparseMap[0][8] = 1.32;

    oriSparseMap[2][2] = 1.2; oriSparseMap[2][4] = 5.6; oriSparseMap[2][6] = 1.3;
    oriSparseMap[2][7] = 123.56;

    oriSparseMap[3][0] = 1.34; oriSparseMap[3][4] = 35; oriSparseMap[3][5] = 1.2;
    oriSparseMap[3][7] = 45.34; oriSparseMap[3][8] = 12.56; oriSparseMap[3][9] = 1;

    oriSparseMap[4][0] = 0.98; oriSparseMap[4][3] = 5.6; oriSparseMap[4][4] = 7;
    oriSparseMap[4][7] = 2; oriSparseMap[4][8] = 6.7; oriSparseMap[4][9] = 2;

    oriSparseMap[5][3] = 2; oriSparseMap[5][5] = 2.5; oriSparseMap[5][8] = 5.8;

    oriSparseMap[7][0] = 4.5; oriSparseMap[7][2] = 1.23; oriSparseMap[7][3] = 0.46;
    oriSparseMap[7][5] = 2.3; oriSparseMap[7][6] = 9.45; oriSparseMap[7][8] = 3;
    oriSparseMap[7][9] = 3;

    oriSparseMap[8][2] = 23.1; oriSparseMap[8][4] = 35; oriSparseMap[8][8] = 4;
    oriSparseMap[8][9] = 40; */
}

void iniGridMapSmallOddData(Matrix& mat)
{
    mat.resize(9, 9);
    mat << 0, 0, 2.46f, 0, 0, 0, 0, 1.27f, 1.32f, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1.2f, 0, 5.6f, 0, 1.3f, 123.56f, 0,
        1.34f, 0, 0, 0, 35, 1.2f, 0, 45.34f, 12.56f, 0.98f, 0, 0, 5.6f, 7, 0, 0, 2, 6.7f, 0, 0, 0, 2, 0, 2.5f, 0, 0,
        5.8f, 0, 0, 0, 0, 0, 0, 0, 0, 0, 4.5f, 0, 1.23f, 0.46f, 0, 2.3f, 9.45f, 0, 3, 0, 0, 23.1f, 0, 35, 0, 0, 0, 4;
}

void iniGridMapSmallEvenData(Matrix& mat)
{
    mat.resize(10, 10);
    mat << 0, 0, 2.46f, 0, 0, 0, 0, 1.27f, 1.32f, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1.2f, 0, 5.6f, 0, 1.3f,
        123.56f, 0, 0, 1.34f, 0, 0, 0, 35, 1.2f, 0, 45.34f, 12.56f, 1, 0.98f, 0, 0, 5.6f, 7, 0, 0, 2, 6.7f, 2, 0, 0, 0,
        2, 0, 2.5f, 0, 0, 5.8f, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 4.5f, 0, 1.23f, 0.46f, 0, 2.3f, 9.45f, 0, 3, 3, 0, 0,
        23.1f, 0, 35, 0, 0, 0, 4, 40, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0;
}

void readGridMapFromBinFile(const char* filename, Matrix& matrix)
{
    ifstream in(filename, ios::in | ios::binary);
    typename Matrix::Index rows = 0, cols = 0;
    in.read(reinterpret_cast<char*>(&rows), sizeof(typename Matrix::Index));
    in.read(reinterpret_cast<char*>(&cols), sizeof(typename Matrix::Index));
    matrix.resize(rows, cols);
    in.read(reinterpret_cast<char*>(matrix.data()),
            rows * cols * static_cast<int64_t>(sizeof(typename Matrix::Scalar)));
    in.close();
}

bool compareSparseMaps(const SparseMapStructure& map1, SparseMapStructure& map2)
{
    if ((map1.SparseMapData).size() != (map2.SparseMapData).size()) {
        return false;
    }
    for (const auto& iter1 : (map1.SparseMapData)) {
        vector<float> v1 = iter1.second;
        vector<float> v2 = (map2.SparseMapData)[iter1.first];
        if (v1.size() != v2.size()) {
            return false;
        }
        for (unsigned i = 0; i < v1.size(); i++) {
            if (static_cast<double>(abs(v1[i] - v2[i])) > thres) {
                return false;
            }
        }
    }
    return true;
}

bool compareSparseMapWithGridMap(SparseMapStructure& sparseMap, const GridMap& gridMap)
{
    Matrix mat = gridMap[layer];
    auto row = mat.rows(), col = mat.cols();
    float sum_gridmap = 0;
    for (int i = 0; i < row; ++i) {
        for (int j = 0; j < col; j++) {
            sum_gridmap += mat(i, j);
        }
    }

    float sum_sparsemap = 0;
    for (const auto& iter1 : sparseMap.SparseMapData) {
        int ind = iter1.first;
        vector<float> data_v = iter1.second;
        vector<int> index_v = (sparseMap.SparseMapIndex)[ind];

        for (unsigned i = 0; i < data_v.size(); i++) {
            sum_sparsemap += data_v[i];
            if (static_cast<double>(abs(data_v[i] - mat(ind, index_v[i]))) > thres) {
                return false;
            }
        }
    }
    if (static_cast<double>(abs(sum_gridmap - sum_sparsemap)) > thres)
        return false;
    return true;
}

bool compareGridMaps(const GridMap& gridMap1, const GridMap& gridMap2)
{
    Matrix mat1 = gridMap1[layer], mat2 = gridMap2[layer];
    if (mat1.rows() != mat2.rows() || mat1.cols() != mat2.cols()) {
        AVLOG_ERROR("compareGridMap error: size not agree");
        AVLOG_ERROR("size 1: {} {}, size 2: {} {}", mat1.rows(), mat1.cols(), mat2.rows(), mat2.cols());
        return false;
    }
    auto row = mat1.rows(), col = mat1.cols();
    for (int i = 0; i < row; ++i) {
        for (int j = 0; j < col; ++j) {
            if (static_cast<double>(abs(mat1(i, j) - mat2(i, j))) > thres) {
                AVLOG_ERROR("compareGridMap error: value not agree");
                AVLOG_ERROR(
                    "({}, {}): {} and {}", i, j, static_cast<float>(mat1(i, j)), static_cast<float>(mat2(i, j)));
                return false;
            }
        }
    }
    return true;
}

bool compareGridMapLists(const vector<GridMap>& gridMapList1, const vector<GridMap>& gridMapList2)
{
    for (unsigned i = 0; i < gridMapList1.size(); ++i) {
        if (!compareGridMaps(gridMapList1[i], gridMapList2[i])) {
            return false;
        }
    }
    return true;
}

void setupTestConstrWithGridMap(SparseMapStructure& oriSparseMap, SparseMapStructure& genSparseMap, GridMap& gridMap)
{
    SparseMapParams spmPara = readSparseMapConfig(sparseMapSmallEvenConfig);
    Matrix mat;
    iniGridMapSmallEvenData(mat);
    gridMap.setGeometry(Length(spmPara.mapSizeX_m_, spmPara.mapSizeY_m_),
                        spmPara.mapCellSize_m_,
                        Position(spmPara.centerX_m_, spmPara.centerY_m_));
    gridMap.add(layer, mat);
    SparseMap sparseMap = SparseMap(gridMap, layer, spmPara);
    genSparseMap = sparseMap.getGlobalSparseMap();
    iniSparseMapSmallEvenData(oriSparseMap);
}

void setupTestConstrWithSparseFile(SparseMapStructure& genSparseMap, SparseMapStructure& oriSparseMap)
{
    SparseMapParams spmPara;
    SparseMap sparseMap = SparseMap(sparseMapSmallEvenData, spmPara);
    genSparseMap = sparseMap.getGlobalSparseMap();
    iniSparseMapSmallEvenData(oriSparseMap);
}

void setupWriteSparseMapToFileLargeData(SparseMapStructure& map_ori, SparseMapStructure& map_gen)
{
    SparseMapParams spmPara = readSparseMapConfig(sparseMapConfigFileLargeData);
    GridMap gridMap;
    Matrix mat;
    readGridMapFromBinFile(gridMapLargeData.c_str(), mat);
    gridMap.setGeometry(Length(spmPara.mapSizeX_m_, spmPara.mapSizeY_m_),
                        spmPara.mapCellSize_m_,
                        Position(spmPara.centerX_m_, spmPara.centerY_m_));
    gridMap.add(layer, mat);
    SparseMap sparseMap = SparseMap(gridMap, layer, spmPara);
    sparseMap.writeSparseMapToFile(genSparseMapLargeData, genSparseMapLargeConfig);
    SparseMap genSparseMap = SparseMap(genSparseMapLargeData, spmPara);
    map_ori = sparseMap.getGlobalSparseMap();
    map_gen = genSparseMap.getGlobalSparseMap();
}

void setupWriteSparseMapToFileSmallEven(SparseMapStructure& map_ori, SparseMapStructure& map_gen)
{
    SparseMapParams spmPara = readSparseMapConfig(sparseMapSmallEvenConfig);
    SparseMap sparseMap = SparseMap(sparseMapSmallEvenData, spmPara);
    sparseMap.writeSparseMapToFile(genSparseMapSmallEvenData, genSparseMapSmallEvenConfig);
    SparseMap genSparseMap = SparseMap(genSparseMapSmallEvenData, spmPara);
    map_ori = sparseMap.getGlobalSparseMap();
    map_gen = genSparseMap.getGlobalSparseMap();
}

void setupWriteSparseMapToFileSmallOdd(SparseMapStructure& map_ori, SparseMapStructure& map_gen)
{
    SparseMapParams spmPara = readSparseMapConfig(sparseMapSmallOddConfig);
    SparseMap sparseMap = SparseMap(sparseMapSmallOddData, spmPara);
    sparseMap.writeSparseMapToFile(genSparseMapSmallOddData, genSparseMapSmallOddConfig);
    SparseMap genSparseMap = SparseMap(genSparseMapSmallOddData, spmPara);
    map_ori = sparseMap.getGlobalSparseMap();
    map_gen = genSparseMap.getGlobalSparseMap();
}

TEST(TestSparseMap, TestConstructor)
{
    SparseMapStructure oriSparseMapWithFile, genSparseMapWithFile;
    setupTestConstrWithSparseFile(oriSparseMapWithFile, genSparseMapWithFile);
    EXPECT_EQ(true, compareSparseMaps(genSparseMapWithFile, oriSparseMapWithFile));

    SparseMapStructure oriSparseMapWithMap, genSparseMapWithMap;
    GridMap gridMap;
    setupTestConstrWithGridMap(oriSparseMapWithMap, genSparseMapWithMap, gridMap);
    EXPECT_EQ(true, compareSparseMaps(genSparseMapWithMap, oriSparseMapWithMap));
    EXPECT_EQ(true, compareSparseMapWithGridMap(genSparseMapWithMap, gridMap));
}

TEST(TestSparseMap, TestWriteSparseMapToFile)
{
    if (test_large) {
        SparseMapStructure map_ori, map_gen;
        setupWriteSparseMapToFileLargeData(map_ori, map_gen);
        EXPECT_EQ(true, compareSparseMaps(map_ori, map_gen));
    }

    SparseMapStructure map_ori_even, map_gen_even;
    setupWriteSparseMapToFileSmallEven(map_ori_even, map_gen_even);
    EXPECT_EQ(true, compareSparseMaps(map_ori_even, map_gen_even));

    SparseMapStructure map_ori_odd, map_gen_odd;
    setupWriteSparseMapToFileSmallOdd(map_ori_odd, map_gen_odd);
    EXPECT_EQ(true, compareSparseMaps(map_ori_odd, map_gen_odd));
}

void setupTestGetGridMapEven(GridMap& gridmap_ori, GridMap& gridmap_gen)
{
    SparseMapParams spmPara = readSparseMapConfig(sparseMapSmallEvenConfig);
    GridMap gridMap;
    Matrix mat;
    iniGridMapSmallEvenData(mat);
    gridmap_ori.setGeometry(Length(spmPara.mapSizeX_m_, spmPara.mapSizeY_m_),
                            spmPara.mapCellSize_m_,
                            Position(spmPara.centerX_m_, spmPara.centerY_m_));

    gridmap_ori.add(layer, mat);

    SparseMap sparseMap = SparseMap(gridmap_ori, layer, spmPara);

    sparseMap.getGridMap(gridmap_gen, layer);
}

void setupTestGetGridMapOdd(GridMap& gridmap_ori, GridMap& gridmap_gen)
{
    SparseMapParams spmPara = readSparseMapConfig(sparseMapSmallOddConfig);
    GridMap gridMap;
    Matrix mat;
    iniGridMapSmallOddData(mat);
    gridmap_ori.setGeometry(Length(spmPara.mapSizeX_m_, spmPara.mapSizeY_m_),
                            spmPara.mapCellSize_m_,
                            Position(spmPara.centerX_m_, spmPara.centerY_m_));
    gridmap_ori.add(layer, mat);
    SparseMap sparseMap = SparseMap(gridmap_ori, layer, spmPara);
    sparseMap.getGridMap(gridmap_gen, layer);
}

TEST(TestSparseMap, TestGetGridMap)
{
    GridMap gridmap_ori_even, gridmap_gen_even;
    setupTestGetGridMapEven(gridmap_ori_even, gridmap_gen_even);
    EXPECT_EQ(true, compareGridMaps(gridmap_ori_even, gridmap_gen_even));

    GridMap gridmap_ori_odd, gridmap_gen_odd;
    setupTestGetGridMapOdd(gridmap_ori_odd, gridmap_gen_odd);
    EXPECT_EQ(true, compareGridMaps(gridmap_ori_odd, gridmap_gen_odd));
}

TEST(TestSparseMap, TestGetIndex)
{
    SparseMapParams spmPara = readSparseMapConfig(sparseMapSmallOddConfig);
    GridMap gridMap;
    Matrix mat;
    iniGridMapSmallOddData(mat);
    GridMap gridmap_ori;
    gridmap_ori.setGeometry(Length(spmPara.mapSizeX_m_, spmPara.mapSizeY_m_),
                            spmPara.mapCellSize_m_,
                            Position(spmPara.centerX_m_, spmPara.centerY_m_));
    gridmap_ori.add(layer, mat);
    SparseMap sparseMap = SparseMap(gridmap_ori, layer, spmPara);

    Index index0;
    sparseMap.getIndex(RecPoint2D(5, 5), index0);
    EXPECT_EQ(4, index0(0));
    EXPECT_EQ(4, index0(1));

    Index index1;
    sparseMap.getIndex(RecPoint2D(7, 5), index1);
    EXPECT_EQ(0, index1(0));
    EXPECT_EQ(4, index1(1));

    Index index2;
    sparseMap.getIndex(RecPoint2D(5, 7), index2);
    EXPECT_EQ(4, index2(0));
    EXPECT_EQ(0, index2(1));
}

TEST(TestSparseMap, TestGetVal)
{
    SparseMapParams spmPara = readSparseMapConfig(sparseMapSmallOddConfig);
    GridMap gridMap;
    Matrix mat;
    iniGridMapSmallOddData(mat);
    GridMap gridmap_ori;
    gridmap_ori.setGeometry(Length(spmPara.mapSizeX_m_, spmPara.mapSizeY_m_),
                            spmPara.mapCellSize_m_,
                            Position(spmPara.centerX_m_, spmPara.centerY_m_));
    gridmap_ori.add(layer, mat);
    SparseMap sparseMap = SparseMap(gridmap_ori, layer, spmPara);

    float v0 = 0;
    sparseMap.getVal(RecPoint2D(5, 5), v0);
    EXPECT_EQ(7, v0);

    float v1 = 0;
    sparseMap.getVal(RecPoint2D(6, 5), v1);
    EXPECT_EQ(float(5.6), v1);

    float v2 = 0;
    sparseMap.getVal(RecPoint2D(2.75, 2.75), v2);
    EXPECT_EQ(4, v2);

    float v3 = 0;
    sparseMap.getVal(Index(4, 4), v3);
    EXPECT_EQ(7, v3);

    float v4 = 0;
    sparseMap.getVal(Index(0, 2), v4);
    EXPECT_EQ(float(2.46), v4);
}

TEST(TestSparseMap, TestCheckBoundary)
{
    SparseMapParams spmPara = readSparseMapConfig(sparseMapSmallOddConfig);
    GridMap gridMap;
    Matrix mat;
    iniGridMapSmallOddData(mat);
    GridMap gridmap_ori;
    gridmap_ori.setGeometry(Length(spmPara.mapSizeX_m_, spmPara.mapSizeY_m_),
                            spmPara.mapCellSize_m_,
                            Position(spmPara.centerX_m_, spmPara.centerY_m_));
    gridmap_ori.add(layer, mat);
    SparseMap sparseMap = SparseMap(gridmap_ori, layer, spmPara);
    EXPECT_EQ(true, sparseMap.checkBoundary(Index(0, 1)));
    EXPECT_EQ(true, sparseMap.checkBoundary(Index(0, 8)));
    EXPECT_EQ(true, sparseMap.checkBoundary(Index(8, 8)));
    EXPECT_EQ(true, sparseMap.checkBoundary(Index(3, 5)));
    EXPECT_EQ(false, sparseMap.checkBoundary(Index(9, 9)));
    EXPECT_EQ(false, sparseMap.checkBoundary(Index(4, 9)));
    EXPECT_EQ(false, sparseMap.checkBoundary(Index(1, 10)));
    EXPECT_EQ(false, sparseMap.checkBoundary(Index(-1, 8)));
    EXPECT_EQ(false, sparseMap.checkBoundary(Index(8, 12)));
    EXPECT_EQ(false, sparseMap.checkBoundary(Index(12, 10)));

    EXPECT_EQ(true, sparseMap.checkBoundary(RecPoint2D(5, 5)));
    EXPECT_EQ(true, sparseMap.checkBoundary(RecPoint2D(7.25, 7.25)));
    EXPECT_EQ(true, sparseMap.checkBoundary(RecPoint2D(2.75, 7.25)));
    EXPECT_EQ(false, sparseMap.checkBoundary(RecPoint2D(2.74, 7.25)));
    EXPECT_EQ(false, sparseMap.checkBoundary(RecPoint2D(0, 0)));
    EXPECT_EQ(false, sparseMap.checkBoundary(RecPoint2D(5, 7.5)));
    EXPECT_EQ(false, sparseMap.checkBoundary(RecPoint2D(4, 2.5)));
}

void setupGetSubMapGeneralTestCases(vector<subMapParams>& paras, SparseMapParams& spmPara)
{
    double centerX = spmPara.centerX_m_;
    double centerY = spmPara.centerY_m_;
    double mapSizeX = spmPara.mapSizeX_m_;
    double mapSizeY = spmPara.mapSizeY_m_;
    double mapCellSize = spmPara.mapCellSize_m_;
    auto cellNumX = mapSizeX / mapCellSize;
    auto cellNumY = mapSizeY / mapCellSize;

    paras = {
        subMapParams(Position(centerX, centerY), Length(mapSizeX, mapSizeY)),
        subMapParams(Position(centerX, centerY), Length(mapCellSize, mapCellSize)),
        subMapParams(Position(centerX, centerY), Length(51 * mapCellSize, 50 * mapCellSize)),
        subMapParams(Position(centerX - 5 * mapCellSize, centerY + 10 * mapCellSize),
                     Length(34 * mapCellSize, 35 * mapCellSize)),

        subMapParams(Position(centerX - 10 * mapCellSize, centerY + 5 * mapCellSize),
                     Length(35 * mapCellSize, 35 * mapCellSize)),
        subMapParams(Position(centerX - int(cellNumX / 4) * mapCellSize, centerY - int(cellNumY / 4) * mapCellSize),
                     Length(int(cellNumX / 3) * mapCellSize, int(cellNumY / 3) * mapCellSize)),
        subMapParams(Position(centerX - int(cellNumX / 4) * mapCellSize, centerY + int(cellNumY / 4) * mapCellSize),
                     Length(int(cellNumX / 3) * mapCellSize, int(cellNumY / 3) * mapCellSize)),
        subMapParams(Position(centerX + int(cellNumX / 4) * mapCellSize, centerY + int(cellNumY / 4) * mapCellSize),
                     Length(int(cellNumX / 4) * mapCellSize, int(cellNumY / 3) * mapCellSize)),
        subMapParams(Position(centerX + int(cellNumX / 6) * mapCellSize, centerY + int(cellNumY / 3) * mapCellSize),
                     Length(int(cellNumX / 2) * mapCellSize, int(cellNumY / 5) * mapCellSize)),
        subMapParams(Position(centerX + int(cellNumX / 6) * mapCellSize, centerY + int(cellNumY / 3) * mapCellSize),
                     Length(int(cellNumX / 3) * mapCellSize, int(cellNumY / 4) * mapCellSize)),
        subMapParams(Position(centerX + int(cellNumX / 6) * mapCellSize, centerY + int(cellNumY / 3) * mapCellSize),
                     Length(int(cellNumX / 3) * mapCellSize, int(cellNumY / 3) * mapCellSize)),
        subMapParams(Position(centerX + 2 * mapCellSize, centerY - 3 * mapCellSize),
                     Length(int(cellNumX / 3) * mapCellSize, int(cellNumY / 3) * mapCellSize)),

        subMapParams(Position(centerX + mapCellSize / 2, centerY + mapCellSize / 2), Length(mapSizeX, mapSizeY)),
        subMapParams(Position(centerX + mapCellSize / 2, centerY - mapCellSize / 2), Length(mapCellSize, mapCellSize)),
        subMapParams(Position(centerX - mapCellSize / 2, centerY + mapCellSize / 2),
                     Length(51 * mapCellSize, 50 * mapCellSize)),
        subMapParams(
            Position(centerX - 5 * mapCellSize + mapCellSize / 2, centerY + 10 * mapCellSize + mapCellSize / 2),
            Length(34 * mapCellSize, 35 * mapCellSize)),
        subMapParams(
            Position(centerX - 10 * mapCellSize - mapCellSize / 2, centerY + 5 * mapCellSize - mapCellSize / 2),
            Length(35 * mapCellSize, 35 * mapCellSize)),
        subMapParams(Position(centerX - int(cellNumX / 4) * mapCellSize - mapCellSize / 2,
                              centerY - int(cellNumY / 4) * mapCellSize - mapCellSize / 2),
                     Length(int(cellNumX / 3) * mapCellSize, int(cellNumY / 3) * mapCellSize)),
        subMapParams(Position(centerX - int(cellNumX / 4) * mapCellSize + mapCellSize / 2,
                              centerY + int(cellNumY / 4) * mapCellSize - mapCellSize / 2),
                     Length(int(cellNumX / 3) * mapCellSize, int(cellNumY / 3) * mapCellSize)),
        subMapParams(Position(centerX + int(cellNumX / 4) * mapCellSize - mapCellSize / 2,
                              centerY + int(cellNumY / 4) * mapCellSize + mapCellSize / 2),
                     Length(int(cellNumX / 4) * mapCellSize, int(cellNumY / 3) * mapCellSize)),
        subMapParams(Position(centerX + int(cellNumX / 6) * mapCellSize + mapCellSize / 2,
                              centerY + int(cellNumY / 3) * mapCellSize - mapCellSize / 2),
                     Length(int(cellNumX / 2) * mapCellSize, int(cellNumY / 5) * mapCellSize)),
        subMapParams(Position(centerX + int(cellNumX / 6) * mapCellSize - mapCellSize / 2,
                              centerY + int(cellNumY / 3) * mapCellSize - mapCellSize / 2),
                     Length(int(cellNumX / 3) * mapCellSize, int(cellNumY / 4) * mapCellSize)),
        subMapParams(Position(centerX + int(cellNumX / 6) * mapCellSize + mapCellSize / 2,
                              centerY + int(cellNumY / 3) * mapCellSize + mapCellSize / 2),
                     Length(int(cellNumX / 3) * mapCellSize, int(cellNumY / 3) * mapCellSize)),
        subMapParams(Position(centerX + 2 * mapCellSize + mapCellSize / 2, centerY - 3 * mapCellSize + mapCellSize / 2),
                     Length(int(cellNumX / 3) * mapCellSize, int(cellNumY / 3) * mapCellSize))};
}

void setupTestGetSubGridMapLargeDataGeneral(vector<GridMap>& v_sparse, vector<GridMap>& v_grid)
{

    SparseMapParams spmPara = readSparseMapConfig(sparseMapConfigFileLargeData);
    GridMap gridMap;
    Matrix mat;
    readGridMapFromBinFile(gridMapLargeData.c_str(), mat);
    gridMap.setGeometry(Length(spmPara.mapSizeX_m_, spmPara.mapSizeY_m_),
                        spmPara.mapCellSize_m_,
                        Position(spmPara.centerX_m_, spmPara.centerY_m_));
    gridMap.add(layer, mat);
    SparseMap sparseMap = SparseMap(gridMap, layer, spmPara);

    vector<subMapParams> paras;
    setupGetSubMapGeneralTestCases(paras, spmPara);
    for (const auto& p : paras) {
        Index refIndex;
        sparseMap.getSubGridMap(gridMap, layer, p.position, p.length, refIndex);
        v_sparse.emplace_back(gridMap);

        bool is_succ_grid = false;
        GridMap subGridMap = gridMap.getSubmap(p.position, p.length, is_succ_grid);
        v_grid.emplace_back(subGridMap);
    }
    EXPECT_EQ(true, compareGridMapLists(v_sparse, v_grid));
}

void setupTestGetSubGridMapSmallEvenGeneral(vector<GridMap>& v_sparse, vector<GridMap>& v_grid)
{
    SparseMapParams spmPara = readSparseMapConfig(sparseMapSmallEvenConfig);
    SparseMap sparseMap = SparseMap(sparseMapSmallEvenData, spmPara);

    vector<subMapParams> paras;
    setupGetSubMapGeneralTestCases(paras, spmPara);

    GridMap gridMap;
    gridMap.setGeometry(Length(spmPara.mapSizeX_m_, spmPara.mapSizeY_m_),
                        spmPara.mapCellSize_m_,
                        Position(spmPara.centerX_m_, spmPara.centerY_m_));

    Matrix mat;
    iniGridMapSmallEvenData(mat);
    gridMap.add(layer, mat);

    for (const auto& p : paras) {
        Index refIndex;
        sparseMap.getSubGridMap(gridMap, layer, p.position, p.length, refIndex);
        v_sparse.emplace_back(gridMap);

        bool is_succ_grid = false;
        GridMap subGridMap = gridMap.getSubmap(p.position, p.length, is_succ_grid);
        v_grid.emplace_back(subGridMap);
    }
}

void setupTestGetSubGridMapSmallOddGeneral(vector<GridMap>& v_sparse, vector<GridMap>& v_grid)
{
    SparseMapParams spmPara = readSparseMapConfig(sparseMapSmallOddConfig);
    ;
    SparseMap sparseMap = SparseMap(sparseMapSmallEvenData, spmPara);

    vector<subMapParams> paras;
    setupGetSubMapGeneralTestCases(paras, spmPara);

    GridMap gridMap;
    gridMap.setGeometry(Length(spmPara.mapSizeX_m_, spmPara.mapSizeY_m_),
                        spmPara.mapCellSize_m_,
                        Position(spmPara.centerX_m_, spmPara.centerY_m_));

    Matrix mat;
    iniGridMapSmallOddData(mat);
    gridMap.add(layer, mat);

    for (const auto& p : paras) {
        Index refIndex;
        sparseMap.getSubGridMap(gridMap, layer, p.position, p.length, refIndex);
        v_sparse.emplace_back(gridMap);

        bool is_succ_grid = false;
        GridMap subGridMap = gridMap.getSubmap(p.position, p.length, is_succ_grid);
        v_grid.emplace_back(subGridMap);
    }
}

TEST(TestSparseMap, TestGetSubGridMapGeneral)
{
    vector<GridMap> v_sparse_even;
    vector<GridMap> v_grid_even;
    setupTestGetSubGridMapSmallEvenGeneral(v_sparse_even, v_grid_even);
    EXPECT_EQ(true, compareGridMapLists(v_sparse_even, v_grid_even));

    vector<GridMap> v_sparse_odd;
    vector<GridMap> v_grid_odd;
    setupTestGetSubGridMapSmallOddGeneral(v_sparse_odd, v_grid_odd);
    EXPECT_EQ(true, compareGridMapLists(v_sparse_odd, v_grid_odd));

    if (test_large) {
        vector<GridMap> v_sparse_large;
        vector<GridMap> v_grid_large;
        setupTestGetSubGridMapLargeDataGeneral(v_sparse_large, v_grid_large);
        EXPECT_EQ(true, compareGridMapLists(v_sparse_large, v_grid_large));
    }
}

void setupGetSubMapOutOfBoundsTestCases(vector<subMapParams>& paras, SparseMapParams& spmPara)
{
    double centerX = spmPara.centerX_m_;
    double centerY = spmPara.centerY_m_;
    double mapSizeX = spmPara.mapSizeX_m_;
    double mapSizeY = spmPara.mapSizeY_m_;
    double mapCellSize = spmPara.mapCellSize_m_;
    auto cellNumX = mapSizeX / mapCellSize;
    auto cellNumY = mapSizeY / mapCellSize;

    paras = {
        subMapParams(Position(centerX, centerY), Length(cellNumX * 2 * mapCellSize, cellNumY * 2 * mapCellSize)),
        subMapParams(Position(centerX, centerY), Length(cellNumX * 0.5 * mapCellSize, cellNumY * 2 * mapCellSize)),
        subMapParams(Position(centerX, centerY), Length(cellNumX * 2 * mapCellSize, cellNumY * 0.5 * mapCellSize)),
        subMapParams(Position(centerX - int(cellNumX / 4) * mapCellSize, centerY - int(cellNumY / 4) * mapCellSize),
                     Length(mapSizeX, mapSizeY)),
        subMapParams(Position(centerX - mapSizeX, centerY - mapSizeY), Length(mapSizeX, mapSizeY)),
        subMapParams(Position(centerX - mapSizeX, centerY - mapSizeY), Length(3 * mapSizeX, 3 * mapSizeY))};
}

void setupTestGetSubGridMapSmallEvenOutOfBounds(vector<GridMap>& v_sparse, vector<GridMap>& v_grid)
{
    SparseMapParams spmPara = readSparseMapConfig(sparseMapSmallEvenConfig);
    SparseMap sparseMap = SparseMap(sparseMapSmallEvenData, spmPara);

    vector<subMapParams> paras;
    setupGetSubMapOutOfBoundsTestCases(paras, spmPara);

    GridMap gridMap;
    gridMap.setGeometry(Length(spmPara.mapSizeX_m_, spmPara.mapSizeY_m_),
                        spmPara.mapCellSize_m_,
                        Position(spmPara.centerX_m_, spmPara.centerY_m_));

    Matrix mat;
    iniGridMapSmallEvenData(mat);
    gridMap.add(layer, mat);

    for (const auto& p : paras) {
        Index refIndex;
        sparseMap.getSubGridMap(gridMap, layer, p.position, p.length, refIndex);
        v_sparse.emplace_back(gridMap);

        bool is_succ_grid = false;
        GridMap subGridMap = gridMap.getSubmap(p.position, p.length, is_succ_grid);
        v_grid.emplace_back(subGridMap);
    }
}

void setupTestGetSubGridMapSmallOddOutOfBounds(vector<GridMap>& v_sparse, vector<GridMap>& v_grid)
{
    SparseMapParams spmPara = readSparseMapConfig(sparseMapSmallOddConfig);

    SparseMap sparseMap = SparseMap(sparseMapSmallOddData, spmPara);

    vector<subMapParams> paras;
    setupGetSubMapOutOfBoundsTestCases(paras, spmPara);

    GridMap gridMap;
    gridMap.setGeometry(Length(spmPara.mapSizeX_m_, spmPara.mapSizeY_m_),
                        spmPara.mapCellSize_m_,
                        Position(spmPara.centerX_m_, spmPara.centerY_m_));

    Matrix mat;
    iniGridMapSmallOddData(mat);
    gridMap.add(layer, mat);

    for (const auto& p : paras) {
        Index refIndex;
        sparseMap.getSubGridMap(gridMap, layer, p.position, p.length, refIndex);
        v_sparse.emplace_back(gridMap);

        bool is_succ_grid = false;
        GridMap subGridMap = gridMap.getSubmap(p.position, p.length, is_succ_grid);
        v_grid.emplace_back(subGridMap);
    }
}

void setupTestGetSubGridMapLargeDataOutOfBounds(vector<GridMap>& v_sparse, vector<GridMap>& v_grid)
{
    SparseMapParams spmPara = readSparseMapConfig(sparseMapConfigFileLargeData);
    GridMap gridMap;
    gridMap.setGeometry(Length(spmPara.mapSizeX_m_, spmPara.mapSizeY_m_),
                        spmPara.mapCellSize_m_,
                        Position(spmPara.centerX_m_, spmPara.centerY_m_));
    Matrix mat;
    readGridMapFromBinFile(gridMapLargeData.c_str(), mat);
    gridMap.add(layer, mat);

    SparseMap sparseMap = SparseMap(gridMap, layer, spmPara);

    vector<subMapParams> paras;
    setupGetSubMapOutOfBoundsTestCases(paras, spmPara);

    for (const auto& p : paras) {
        Index refIndex;
        sparseMap.getSubGridMap(gridMap, layer, p.position, p.length, refIndex);
        // v_sparse.emplace_back(gridMap);
        v_sparse.emplace_back(gridMap);

        bool is_succ_grid = false;
        GridMap subGridMap = gridMap.getSubmap(p.position, p.length, is_succ_grid);
        v_grid.emplace_back(subGridMap);
    }
    EXPECT_EQ(true, compareGridMapLists(v_sparse, v_grid));
}

TEST(TestSparseMap, TestGetSubGridMapOutOfBounds)
{
    vector<GridMap> v_sparse_even;
    vector<GridMap> v_grid_even;
    setupTestGetSubGridMapSmallEvenOutOfBounds(v_sparse_even, v_grid_even);
    EXPECT_EQ(true, compareGridMapLists(v_sparse_even, v_grid_even));

    vector<GridMap> v_sparse_odd;
    vector<GridMap> v_grid_odd;
    setupTestGetSubGridMapSmallOddOutOfBounds(v_sparse_odd, v_grid_odd);
    EXPECT_EQ(true, compareGridMapLists(v_sparse_odd, v_grid_odd));

    if (test_large) {
        vector<GridMap> v_sparse_large;
        vector<GridMap> v_grid_large;
        setupTestGetSubGridMapLargeDataOutOfBounds(v_sparse_large, v_grid_large);
        EXPECT_EQ(true, compareGridMapLists(v_sparse_large, v_grid_large));
    }
}

void setupTestGetSubGridMapEfficiencyCases(vector<subMapParams>& paras, SparseMapParams& spmPara)
{
    double centerX = spmPara.centerX_m_;      // 4483696.2107791668
    double centerY = spmPara.centerY_m_;      // 596126.7098418640
    double cellSize = spmPara.mapCellSize_m_; // 0.5
    double mapSizeX = spmPara.mapSizeX_m_;    // 7000
    double mapSizeY = spmPara.mapSizeY_m_;    // 7000
    double currPartSizeX = 200;
    double currPartSizeY = 200;
    auto cellNumX = mapSizeX / cellSize; // 14000
    auto cellNumY = mapSizeY / cellSize; // 14000
    paras = {
        subMapParams(Position(centerX, centerY), Length(currPartSizeX, currPartSizeY)),
        subMapParams(Position(centerX, centerY), Length(currPartSizeX, currPartSizeY)),
        subMapParams(Position(centerX + 50 * cellSize, centerY + 50 * cellSize), Length(currPartSizeX, currPartSizeY)),
        subMapParams(Position(centerX + 50 * cellSize, centerY - 50 * cellSize), Length(currPartSizeX, currPartSizeY)),
        subMapParams(Position(centerX - int(cellNumX / 2) * cellSize, centerY + int(cellNumY / 2) * cellSize),
                     Length(currPartSizeX, currPartSizeY)),
        subMapParams(Position(centerX - int(cellNumX / 3) * cellSize, centerY + int(cellNumY / 3) * cellSize),
                     Length(currPartSizeX, currPartSizeY)),
        subMapParams(Position(centerX + int(cellNumX / 4) * cellSize, centerY + int(cellNumY / 4) * cellSize),
                     Length(currPartSizeX, currPartSizeY)),
        subMapParams(Position(centerX - int(cellNumX / 2) * cellSize, centerY - int(cellNumY / 3) * cellSize),
                     Length(currPartSizeX, currPartSizeY)),
        subMapParams(Position(centerX - int(cellNumX / 3) * cellSize, centerY + int(cellNumY / 2) * cellSize),
                     Length(currPartSizeX, currPartSizeY)),
        subMapParams(Position(centerX + int(cellNumX / 2) * cellSize, centerY + int(cellNumY / 4) * cellSize),
                     Length(currPartSizeX, currPartSizeY)),
        subMapParams(Position(centerX + int(cellNumX / 3) * cellSize, centerY + int(cellNumY / 4) * cellSize),
                     Length(currPartSizeX, currPartSizeY)),
        subMapParams(Position(centerX - int(cellNumX / 3) * cellSize, centerY - int(cellNumY / 3) * cellSize),
                     Length(currPartSizeX, currPartSizeY)),
        subMapParams(Position(centerX, centerY), Length(currPartSizeX, currPartSizeY))};
}

double setupTestGetSubGridMapEfficiency()
{
    SparseMapParams spmPara = readSparseMapConfig(JH_config);
    GridMap gridMap;
    gridMap.setGeometry(Length(spmPara.mapSizeX_m_, spmPara.mapSizeY_m_),
                        spmPara.mapCellSize_m_,
                        Position(spmPara.centerX_m_, spmPara.centerY_m_));
    Matrix mat;
    readGridMapFromBinFile(JH_data.c_str(), mat);
    gridMap.add(layer, mat);
    SparseMap sparseMap = SparseMap(gridMap, layer, spmPara);
    SparseMapStructure spMap = sparseMap.getGlobalSparseMap();

    vector<subMapParams> paras;
    setupTestGetSubGridMapEfficiencyCases(paras, spmPara);

    double max_time_diff = 0;
    for (const auto& p : paras) {
        auto time_start = clock();
        GridMap subGridMap;
        Index refIndex;
        sparseMap.getSubGridMap(subGridMap, layer, p.position, p.length, refIndex);
        auto time_diff = (clock() - time_start) / CLOCKS_PER_SEC;
        max_time_diff = max(max_time_diff, static_cast<double>(time_diff));
        AVLOG_DEBUG("getSubGridMap time_diff: {}s", time_diff);
    }
    return max_time_diff;
}

TEST(TestSparseMap, TestGetSubGridMapEfficiency)
{
    if (test_large) {
        EXPECT_GE(0.01, setupTestGetSubGridMapEfficiency());
    }
}

TEST(TestSparseMap, TestAddPoint)
{
    SparseMapParams spmPara = readSparseMapConfig(sparseMapSmallOddConfig);
    GridMap gridMap;
    Matrix mat;
    iniGridMapSmallOddData(mat);
    GridMap gridmap_ori;
    gridmap_ori.setGeometry(Length(spmPara.mapSizeX_m_, spmPara.mapSizeY_m_),
                            spmPara.mapCellSize_m_,
                            Position(spmPara.centerX_m_, spmPara.centerY_m_));
    gridmap_ori.add(layer, mat);
    SparseMap sparseMap = SparseMap(gridmap_ori, layer, spmPara);

    float v0 = 0;
    sparseMap.getVal(RecPoint2D(5.5, 5.5), v0);
    EXPECT_EQ(0, v0);

    sparseMap.addPoint(RecPoint2D(5.5, 5.5), 21);
    float v1 = 0;
    sparseMap.getVal(RecPoint2D(5.5, 5.5), v1);
    EXPECT_EQ(21, v1);

    float v2 = 0;
    sparseMap.getVal(RecPoint2D(7.25, 7.25), v2);
    EXPECT_EQ(0, v2);

    sparseMap.addPoint(RecPoint2D(7.25, 7.25), 45.6f);
    float v3 = 0;
    sparseMap.getVal(RecPoint2D(7.25, 7.25), v3);
    EXPECT_EQ(float(45.6), v3);

    float v4 = 0;
    sparseMap.getVal(Index(0, 1), v4);
    EXPECT_EQ(0, v4);

    sparseMap.addPoint(Index(0, 1), 1.234f);
    float v5 = 0;
    sparseMap.getVal(Index(0, 1), v5);
    EXPECT_EQ(float(1.234), v5);

    sparseMap.addPoint(RecPoint2D(7.25, 7.25), 4.1f);
    float v6 = 0;
    sparseMap.getVal(RecPoint2D(7.25, 7.25), v6);
    EXPECT_EQ(float(4.1), v6);

    float v7 = 0;
    sparseMap.getVal(Index(0, 3), v7);
    EXPECT_EQ(0, v7);
}

GTEST_API_ int main(int argc, char** argv)
{
    avlog::setGlobalLogLevel(avlog::LogLevel::debug);
    testing::InitGoogleTest(&argc, argv);
    return RUN_ALL_TESTS();
}
