/**
 * \copyright
 * Copyright (c) 2021- Motional. All rights reserved.
 * NOTICE: Removing this heading is prohibited.
 * All information contained herein is, and remains the property of Motional, its affiliates, and its suppliers (if
 * any). The intellectual and technical concepts contained herein are proprietary to Motional, its affiliates, and its
 * suppliers (if any) and may be covered by U.S. and Foreign Patents, patents in process, and are protected by trade
 * secrets or copyright law. Dissemination of this information or reproduction of this material is strictly forbidden
 * unless prior written permission is obtained from Motional.
 *
 * \file
 * \brief Defines an interface for statistic collection
 */

#ifndef SERIES_STATISTICS_INTERFACE_HPP
#define SERIES_STATISTICS_INTERFACE_HPP

#include <nlohmann/json.hpp>

namespace statistics {

/**
 * \brief A pure virtual interface for calculating statistics on a series of measurements.
 */
template<typename T>
class SeriesStatisticsInterface
{
public:
    /// \brief Used so that templates can learn the T. "value_type" is
    /// the convention from STL collections.
    using value_type = T;

    /**
     * \brief Update state based on one new sample
     * \return True if updated successfully
     */
    virtual bool addSample(const T& sample) = 0;

    virtual uint64_t getCount() const = 0;

    /**
     * \brief Returns JSON representation of the output
     * \return Statistics as JSON
     */
    virtual nlohmann::json getJSON() const = 0;

    /**
     * \brief Default virtual destructor
     */
    virtual ~SeriesStatisticsInterface() = default;
};

} // namespace statistics
#endif
