/**
 * \copyright
 * Copyright (c) 2021- Motional. All rights reserved.
 * NOTICE: Removing this heading is prohibited.
 * All information contained herein is, and remains the property of Motional, its affiliates, and its suppliers (if
 * any). The intellectual and technical concepts contained herein are proprietary to Motional, its affiliates, and its
 * suppliers (if any) and may be covered by U.S. and Foreign Patents, patents in process, and are protected by trade
 * secrets or copyright law. Dissemination of this information or reproduction of this material is strictly forbidden
 * unless prior written permission is obtained from Motional.
 *
 * \file
 * \brief Tests for Mean/Variance collector
 */

#include <Statistics/MeanVariance.hpp>
#include <gtest/gtest.h>
#include <math.h>

using json = nlohmann::json;

namespace statistics {

#define EXPECT_OPTIONAL_FLOAT_EQ(opt, val)                                                                             \
    {                                                                                                                  \
        const auto optCache{opt};                                                                                      \
        EXPECT_TRUE(optCache.has_value());                                                                             \
        EXPECT_FLOAT_EQ(*optCache, (val));                                                                             \
    }

TEST(MeanVarianceTests, TestInitialConstruction)
{
    MeanVariance<float> meanvariance{};

    EXPECT_EQ(meanvariance.getCount(), 0);
    EXPECT_FALSE(meanvariance.getMean().has_value());
    EXPECT_FALSE(meanvariance.getStandardDeviation().has_value());
    EXPECT_FALSE(meanvariance.getVariance().has_value());

    json j = meanvariance.getJSON().at("meanvariance");
    EXPECT_EQ(j.at("count"), 0);
    EXPECT_TRUE(j.at("mean").is_null());
    EXPECT_TRUE(j.at("variance").is_null());
}

TEST(MeanVarianceTests, TestSingleUpdate)
{
    MeanVariance<float> meanvariance{};

    meanvariance.addSample(1.5);

    EXPECT_EQ(meanvariance.getCount(), 1);
    EXPECT_OPTIONAL_FLOAT_EQ(meanvariance.getMean(), 1.5);
    EXPECT_OPTIONAL_FLOAT_EQ(meanvariance.getStandardDeviation(), 0);
    EXPECT_OPTIONAL_FLOAT_EQ(meanvariance.getVariance(), 0);

    json j = meanvariance.getJSON().at("meanvariance");
    EXPECT_EQ(j.at("count"), 1);
    EXPECT_FLOAT_EQ(j.at("mean"), 1.5);
    EXPECT_FLOAT_EQ(j.at("variance"), 0);
}

TEST(MeanVarianceTests, TestMoreUpdates)
{
    MeanVariance<float> meanvariance{};

    meanvariance.addSample(1.0);
    meanvariance.addSample(2.0);
    meanvariance.addSample(3.0);
    meanvariance.addSample(4.0);
    meanvariance.addSample(5.0);

    EXPECT_EQ(meanvariance.getCount(), 5);
    EXPECT_OPTIONAL_FLOAT_EQ(meanvariance.getMean(), 3);
    EXPECT_OPTIONAL_FLOAT_EQ(meanvariance.getStandardDeviation(), sqrt(2.5f));
    EXPECT_OPTIONAL_FLOAT_EQ(meanvariance.getVariance(), 2.5);

    json j = meanvariance.getJSON().at("meanvariance");
    EXPECT_EQ(j.at("count"), 5);
    EXPECT_FLOAT_EQ(j.at("mean"), 3);
    EXPECT_FLOAT_EQ(j.at("variance"), 2.5);
}
} // namespace statistics
