/**
 * \copyright
 * Copyright (c) 2021- Motional. All rights reserved.
 * NOTICE: Removing this heading is prohibited.
 * All information contained herein is, and remains the property of Motional, its affiliates, and its suppliers (if
 * any). The intellectual and technical concepts contained herein are proprietary to Motional, its affiliates, and its
 * suppliers (if any) and may be covered by U.S. and Foreign Patents, patents in process, and are protected by trade
 * secrets or copyright law. Dissemination of this information or reproduction of this material is strictly forbidden
 * unless prior written permission is obtained from Motional.
 *
 * \file
 * \brief Tests for MinMax
 */

#include <Statistics/MinMax.hpp>
#include <gtest/gtest.h>
#include <math.h>

using json = nlohmann::json;

#define EXPECT_OPTIONAL_DOUBLE_EQ(opt, val)                                                                            \
    {                                                                                                                  \
        const auto optCache{opt};                                                                                      \
        EXPECT_TRUE(optCache.has_value());                                                                             \
        EXPECT_DOUBLE_EQ(*optCache, (val));                                                                            \
    }

namespace statistics {
TEST(MinMaxTests, TestInitialConstruction)
{
    MinMax<float> mm{};

    EXPECT_FALSE(mm.getMin().has_value());
    EXPECT_FALSE(mm.getMax().has_value());

    json j = mm.getJSON().at("minmax");
    EXPECT_TRUE(j.at("min").is_null());
    EXPECT_TRUE(j.at("max").is_null());
}

TEST(MinMaxTests, TestSingleUpdate)
{
    MinMax<float> mm{};

    EXPECT_TRUE(mm.addSample(1.5));

    EXPECT_OPTIONAL_DOUBLE_EQ(mm.getMin(), 1.5);
    EXPECT_OPTIONAL_DOUBLE_EQ(mm.getMax(), 1.5);

    json j = mm.getJSON().at("minmax");
    EXPECT_DOUBLE_EQ(j.at("min"), 1.5);
    EXPECT_DOUBLE_EQ(j.at("max"), 1.5);
}

TEST(MinMaxTests, TestMoreUpdates)
{
    MinMax<float> mm{};

    EXPECT_TRUE(mm.addSample(1.0));
    EXPECT_TRUE(mm.addSample(2.0));
    EXPECT_TRUE(mm.addSample(-3.0));
    EXPECT_TRUE(mm.addSample(5.0));
    EXPECT_TRUE(mm.addSample(4.0));

    EXPECT_OPTIONAL_DOUBLE_EQ(mm.getMin(), -3);
    EXPECT_OPTIONAL_DOUBLE_EQ(mm.getMax(), 5);

    json j = mm.getJSON().at("minmax");
    EXPECT_DOUBLE_EQ(j.at("min"), -3);
    EXPECT_DOUBLE_EQ(j.at("max"), 5);
}

TEST(MinMaxTests, TestOneValLimit)
{
    // Since uint32_t is unsigned, its ::lowest value will be zero.
    // Previously, the MinMax class may have incorrectly reported that it had no value.
    MinMax<uint32_t> mm{};

    EXPECT_TRUE(mm.addSample(0));

    EXPECT_EQ(mm.getMin(), 0);
    EXPECT_EQ(mm.getMax(), 0);
}

} // namespace statistics
