/**
 * \copyright
 * Copyright (c) 2021- Motional. All rights reserved.
 * NOTICE: Removing this heading is prohibited.
 * All information contained herein is, and remains the property of Motional, its affiliates, and its suppliers (if
 * any). The intellectual and technical concepts contained herein are proprietary to Motional, its affiliates, and its
 * suppliers (if any) and may be covered by U.S. and Foreign Patents, patents in process, and are protected by trade
 * secrets or copyright law. Dissemination of this information or reproduction of this material is strictly forbidden
 * unless prior written permission is obtained from Motional.
 *
 * \file
 * \brief Tests for TimeWindowedSeriesStatistics.
 */
#include <Statistics/MinMax.hpp>
#include <Statistics/TimeWindowedSeriesStatistics.hpp>

#include <gtest/gtest.h>

using namespace statistics;
using namespace std::chrono_literals;

namespace {
using TWSS = TimeWindowedSeriesStatistics<int, MinMax<int>, av::AvClock>;

TEST(TestWindowedMinMax, OneWindow)
{
    struct WindowResult
    {
        av::TimePoint start;
        std::optional<int> min;
        std::optional<int> max;

        WindowResult(av::TimePoint _start, std::optional<int> _min, std::optional<int> _max)
            : start(_start)
            , min(_min)
            , max(_max)
        {}
    };

    std::vector<WindowResult> results;

    TWSS::Callback callback{[&results](const MinMax<int>& series, av::TimePoint start) {
        results.emplace_back(start, series.getMin(), series.getMax());
    }};
    TWSS windowed{callback, 60s, 3, av::TimePoint(0s)};

    // First minute
    windowed.addSample(63, av::TimePoint(0s));
    windowed.addSample(-200, av::TimePoint(10s));
    windowed.addSample(17, av::TimePoint(20s));

    EXPECT_EQ(results.size(), 0);

    // Second minute
    windowed.addSample(12, av::TimePoint(60s));
    windowed.addSample(-12, av::TimePoint(70s));
    windowed.addSample(24, av::TimePoint(80s));

    EXPECT_EQ(results.size(), 0);

    // Third minute
    windowed.addSample(52, av::TimePoint(120s));
    windowed.addSample(-2, av::TimePoint(130s));
    windowed.addSample(15, av::TimePoint(140s));

    EXPECT_EQ(results.size(), 0);

    // Fourth minute
    windowed.addSample(52, av::TimePoint(180s));
    windowed.addSample(-2, av::TimePoint(190s));
    windowed.addSample(15, av::TimePoint(200s));

    EXPECT_EQ(results.size(), 1);
    ASSERT_GE(results.size(), 1);

    EXPECT_EQ(results[0].start, av::TimePoint(0s));
    EXPECT_EQ(results[0].min, -200);
    EXPECT_EQ(results[0].max, 63);

    // Fifth minute
    windowed.addSample(59, av::TimePoint(240s));
    windowed.addSample(-100, av::TimePoint(250s));
    windowed.addSample(15, av::TimePoint(260s));

    EXPECT_EQ(results.size(), 2);
    ASSERT_GE(results.size(), 2);

    EXPECT_EQ(results[0].start, av::TimePoint(0s));
    EXPECT_EQ(results[0].min, -200);
    EXPECT_EQ(results[0].max, 63);
    EXPECT_EQ(results[1].start, av::TimePoint(60s));
    EXPECT_EQ(results[1].min, -12);
    EXPECT_EQ(results[1].max, 52);
}
} // namespace
