/*!
 ***********************************************************************************************************************
 * APTIV PRIVILEGED AND CONFIDENTIAL
 *
 * @file StringUtils.h
 * @product common
 * @version TODO <Release Number>
 * @released TODO <date>
 * @updated September 25, 2018
 * @brief This defines a string utlities class.
 *
 * @section License
 *
 * Copyright (c) 2018- Aptiv. All rights reserved.
 * NOTE: Removing this heading is prohibited.
 *
 ***********************************************************************************************************************
 */

#ifndef STRINGUTILS_H
#define STRINGUTILS_H

#include <cstdint>
#include <string>
#include <vector>

namespace string_utils {
/*!
 * @brief This method shall return a vector of possibly empty strings after breaking the given string by the specified
 * delimiter The output shall meet the following post-conditions: output_vector.size() == input_string.count(delimiter)
 * + 1  -- i.e. shall include both the (possibly empty) strings before and after each delimiter all(delimiter not in s
 * for s in output_vector)             -- i.e. no delimiter shall be included in the output
 *     delimiter.join(output_vector) == input_string              -- i.e. the ordering, and every non-delimeter
 * character, shall be preserved NOTE: Behavior should be consistent with Python's input_string.split(delim)
 * implementation
 *
 * @param[in] str : input string
 * @param[in] delimiter : the string splits at this specified delimiter.
 *
 * @return
 *   vector of strings if the specified delimiter is found in the input string
 *   vector with input string if it doesn't find the specified delimiter
 */
std::vector<std::string> splitString(const std::string& str, const char delimiter);

/*!
 * @brief This method shall convert a string to a 16-bit unsigned integer.
 * \n It throws range_error exception if converted value is outside the range of uint16_t[0,+65535]
 * \n It generates below exceptions as well so caller of this method shall be able to handle these exceptions.
 * \n out_of_range     - if the converted value outside the range of [-2147483648,+2147483647]
 * \n invalid_argument - if the input string is invalid.for example string contains only spaces
 *
 * @param[in] str : input string
 *
 * @throw range_error if the value is negative or wider than a 16-bit unsigned integer
 *
 * @return uint16_t value of the string
 */
uint16_t toShort(const std::string& str);
} // namespace string_utils

#endif // end of STRINGUTILS_H
