/**
 * \copyright
 * Copyright (c) 2020- Motional. All rights reserved.
 * NOTICE: Removing this heading is prohibited.
 * All information contained herein is, and remains the property of Motional, its affiliates, and its suppliers (if
 * any). The intellectual and technical concepts contained herein are proprietary to Motional, its affiliates, and its
 * suppliers (if any) and may be covered by U.S. and Foreign Patents, patents in process, and are protected by trade
 * secrets or copyright law. Dissemination of this information or reproduction of this material is strictly forbidden
 * unless prior written permission is obtained from Motional.
 *
 * \file
 * \brief Utilities for setting up task framework tests
 */

#ifndef TASK_FRAMEWORK_TESTUTILS_COMMON_H
#define TASK_FRAMEWORK_TESTUTILS_COMMON_H

#ifndef SOURCE_DIR
#define SOURCE_DIR "."
#endif

#ifndef APTIVWARE_CORE_BUILD_DIR
#define APTIVWARE_CORE_BUILD_DIR "."
#endif

#include <ProcessLauncher/processUtil.h>
#include <memory>
#include <string>

namespace TestUtils {
const std::string configFilePath{"./middleware/src/task/test/TaskTest/config"}; ///< The config file path
const std::string sourceDir{SOURCE_DIR};
const std::string aptivwareCoreBuildDir{APTIVWARE_CORE_BUILD_DIR};

/**
 * \brief Get the testing data path such as config files
 *
 * \param[in] relativePath as string: The path relative to the root of the si-middleware package. Starts with
 * "/aptivware-core", "/conan-aptivware", "/safer", etc.
 *
 * \param[in] useBuildDir as bool: When true, path represents location in the build tree. When false, source tree.
 *
 * \return string: The absolute path
 */
std::string getTestingDataPath(const std::string& relativePath = configFilePath, const bool useBuildDir = false)
{
    char buff[PATH_MAX + 1];
    std::string prefix{""};
    if (getcwd(buff, sizeof(buff)) != nullptr) {
        const std::string cwd{buff};
        if (!useBuildDir) {
            prefix = sourceDir;
        } else {
            prefix = aptivwareCoreBuildDir + "/";
        }
    }
    return (prefix + relativePath);
}

/**
 * \brief Copy the text log file to the given destination on the log machine
 *
 * \param[in] srcFilepath as string: The path to the log file
 * \param[in] destDir as string: The destination directory
 * \param[in] fileName as string: The name of the copied log file on the destination machine
 *
 * \return True if the copying was fine
 */
bool copyFile(const std::string& srcFilePath, const std::string& destDir, const std::string& fileName)
{
    bool result{false};

    const std::string copyCommand{"/usr/bin/rsync -qdcIL --exclude=*~ --port=10101"}; ///< The rsync copy command
    const std::string command = copyCommand + " " + srcFilePath + "/" + fileName + " " + destDir + "/" + fileName;
    if (ProcessUtil::executeCommand(command)) {
        result = true;
    }
    return result;
}

} // namespace TestUtils

#endif // TASK_FRAMEWORK_TESTUTILS_COPYINTERFACES_H
