/**
 * \copyright
 * Copyright (c) 2020- Motional. All rights reserved.
 * NOTICE: Removing this heading is prohibited.
 * All information contained herein is, and remains the property of Motional, its affiliates, and its suppliers (if
 * any). The intellectual and technical concepts contained herein are proprietary to Motional, its affiliates, and its
 * suppliers (if any) and may be covered by U.S. and Foreign Patents, patents in process, and are protected by trade
 * secrets or copyright law. Dissemination of this information or reproduction of this material is strictly forbidden
 * unless prior written permission is obtained from Motional.
 *
 * \file
 * \brief Temporarily change working directory, and return to original directory when done
 */

#ifndef TASK_FRAMEWORK_TESTUTILS_CWDBACKUP_H
#define TASK_FRAMEWORK_TESTUTILS_CWDBACKUP_H

#include <iostream>
#include <limits.h>
#include <string.h>
#include <string>
#include <unistd.h>

namespace TestUtils {
class CwdBackup
{
public:
    explicit CwdBackup(std::string newDir)
        : m_buff("")
        , m_oldDir(getcwd(m_buff, sizeof(m_buff)))
        , m_constructorErrno(0)
    {
        if (chdir(newDir.c_str()) < 0) {
            char buff[1024];
            m_constructorErrno = errno;
            std::cerr << "Can't set working directory: " << strerror_r(m_constructorErrno, buff, sizeof(buff));
        }
    }

    virtual ~CwdBackup()
    {
        if (m_constructorErrno == 0) {
            if (chdir(m_oldDir.c_str()) < 0) {
                char buff[1024];
                const int tmp = errno;
                std::cerr << "Can't restore working directory: " << strerror_r(tmp, buff, sizeof(buff));
            }
        }
    }

    int getConstructorErrno() const { return m_constructorErrno; }

private:
    char m_buff[PATH_MAX + 1];
    std::string m_oldDir;
    int m_constructorErrno;
};
} // namespace TestUtils
#endif // TASK_FRAMEWORK_TESTUTILS_CWDBACKUP_H
