/**
 * \copyright
 * Copyright (c) 2020- Motional. All rights reserved.
 * NOTICE: Removing this heading is prohibited.
 * All information contained herein is, and remains the property of Motional and its suppliers, if any.
 * The intellectual and technical concepts contained herein are proprietary to Motional and its suppliers and may be
 * covered by U.S. and Foreign Patents, patents in process, and are protected by trade secret or copyright law.
 * Dissemination of this information or reproduction of this material is strictly forbidden unless prior written
 * permission is obtained from Motional.
 *
 * \brief Defines some mock classes for interfaces for writing unit tests
 */

#pragma once

#include <interfaces/Generic/Input/Abstract.h>
#include <interfaces/Generic/Output/Abstract.h>

#include <gmock/gmock.h>

namespace TestUtils {

/**
 * \brief This class defines a fully mocked GenericInputIntf.
 */
template<typename TStruct>
class MockGenericInput : public GenericInputIntf<TStruct>
{

public:
    MockGenericInput() = default;
    virtual ~MockGenericInput() = default;
    MOCK_METHOD2_T(get, bool(TStruct& val, bool blocking));
    MOCK_METHOD0(unblock, void());
    MOCK_METHOD0(clear, void());
    MOCK_METHOD0(initialize, bool());
    MOCK_METHOD0(cleanup, void());
};

/**
 * \brief This class defines a fully mocked GenericOutputIntf.
 */
template<typename TStruct>
class MockGenericOutput : public GenericOutputIntf<TStruct>
{

public:
    MockGenericOutput() = default;
    virtual ~MockGenericOutput() = default;
    MOCK_METHOD2_T(publishV, bool(const TStruct& val, std::vector<av::latency::Pred>&& predecessors));
    MOCK_METHOD0(initialize, bool());
    MOCK_METHOD0(cleanup, void());
};

} // namespace TestUtils
