/**
 * \copyright
 * Copyright (c) 2020- Motional. All rights reserved.
 * NOTICE: Removing this heading is prohibited.
 * All information contained herein is, and remains the property of Motional, its affiliates, and its suppliers (if
 * any). The intellectual and technical concepts contained herein are proprietary to Motional, its affiliates, and its
 * suppliers (if any) and may be covered by U.S. and Foreign Patents, patents in process, and are protected by trade
 * secrets or copyright law. Dissemination of this information or reproduction of this material is strictly forbidden
 * unless prior written permission is obtained from Motional.
 *
 * \file
 * \brief A settable clock for use in tests.
 */

#ifndef TESTUTILS_SETTABLE_CLOCK_H
#define TESTUTILS_SETTABLE_CLOCK_H

#include <Clock/ClockSource.h>

namespace TestUtils {

class SettableClock : public Clock
{
public:
    SettableClock() {}
    explicit SettableClock(ottotime::otimePoint current)
        : currentTime_(std::move(current))
    {}
    virtual ~SettableClock() {}

    virtual ottotime::otimePoint getCurrentTime() override { return currentTime_; }

    void setCurrentTime(const ottotime::otimePoint& newTime) { currentTime_ = newTime; }

private:
    ottotime::otimePoint currentTime_;
};

} // namespace TestUtils

#endif // TESTUTILS_SETTABLE_CLOCK_H
