#ifndef _APTIVWARE_CORE_SRC_TESTUTILS_SYSTEMCLOCKSCOPEDINITIALIZER_HPP_
#define _APTIVWARE_CORE_SRC_TESTUTILS_SYSTEMCLOCKSCOPEDINITIALIZER_HPP_

/**
 * \copyright
 * Copyright (c) 2020- Motional. All rights reserved.
 * NOTICE: Removing this heading is prohibited.
 * All information contained herein is, and remains the property of Motional and its suppliers, if any.
 * The intellectual and technical concepts contained herein are proprietary to Motional and its suppliers and may be
 * covered by U.S. and Foreign Patents, patents in process, and are protected by trade secret or copyright law.
 * Dissemination of this information or reproduction of this material is strictly forbidden unless prior written
 * permission is obtained from Motional.
 *
 * \brief Declare SystemClock's friend, SystemClockScopedInitializer
 */

#include <Clock/SystemClock.h>

namespace ottotime {

/*!
 * @brief SystemClock's unit testing friend class definition
 */
class SystemClockScopedInitializer final
{
public:
    SystemClockScopedInitializer() = delete;
    explicit SystemClockScopedInitializer(SystemClock::SystemTimeMode mode, Clock* externalClock = nullptr)
        : initStatus_(SystemClock::initialize(mode, externalClock))
    {}
    ~SystemClockScopedInitializer() { uninitialize(); }

    static void uninitialize() { SystemClock::uninitializeSystemClock(); }

    bool getInitStatus() const { return initStatus_; }

private:
    const bool initStatus_;
};

} // namespace ottotime

#endif // _APTIVWARE_CORE_SRC_TESTUTILS_SYSTEMCLOCKSCOPEDINITIALIZER_HPP_
