/**
 * \copyright
 * Copyright (c) 2020- Motional. All rights reserved.
 * NOTICE: Removing this heading is prohibited.
 * All information contained herein is, and remains the property of Motional and its suppliers, if any.
 * The intellectual and technical concepts contained herein are proprietary to Motional and its suppliers and may be
 * covered by U.S. and Foreign Patents, patents in process, and are protected by trade secret or copyright law.
 * Dissemination of this information or reproduction of this material is strictly forbidden unless prior written
 * permission is obtained from Motional.
 *
 * \brief unit tests for CwdBackup.
 */

#include <TestUtils/CwdBackup.hpp>
#include <gtest/gtest.h>
#include <unistd.h>

using TestUtils::CwdBackup;

namespace {
std::string getCwd()
{
    char buff[PATH_MAX + 1];
    const char* const cwd{getcwd(buff, sizeof(buff))};
    return cwd;
}
} // namespace

TEST(CwdBackup, backUp)
{
    const std::string dir{"/"};
    const std::string previous{getCwd()};

    ASSERT_NE(dir, previous);

    {
        CwdBackup cwb(dir);
        EXPECT_EQ(dir, getCwd());
    }

    // Make sure that it was restored to the correct value.
    EXPECT_EQ(previous, getCwd());
}

TEST(CwdBackup, backUpInvalid)
{
    const std::string dir{"/no/such/directory"};
    const std::string previous{getCwd()};

    ASSERT_NE(dir, previous);

    {
        CwdBackup cwb(dir);
        EXPECT_NE(dir, getCwd());
    }

    // Make sure that it was restored to the correct value.
    EXPECT_EQ(previous, getCwd());
}
