/**
 * \copyright
 * Copyright (c) 2021- Motional. All rights reserved.
 * NOTICE: Removing this heading is prohibited.
 * All information contained herein is, and remains the property of Motional and its suppliers, if any.
 * The intellectual and technical concepts contained herein are proprietary to Motional and its suppliers and may be
 * covered by U.S. and Foreign Patents, patents in process, and are protected by trade secret or copyright law.
 * Dissemination of this information or reproduction of this material is strictly forbidden unless prior written
 * permission is obtained from Motional
 */

#include "platform/middleware/libs/TestUtils/include/TestUtils/TmpDir.hpp"
#include "platform/middleware/libs/TestUtils/include/TestUtils/EnvBackup.hpp"
#include "platform/middleware/libs/utility/include/utility/utility.h"

#include <experimental/filesystem>
#include <gtest/gtest.h>
#include <string>

namespace fs = std::experimental::filesystem;

TEST(TmpDirTests, VarExists)
{
    const std::string path{"/tmp/nonexistentpath"};

    // set the env to a nonsensical path
    TestUtils::EnvBackup env("TEST_TMPDIR", path);
    {
        TestUtils::TestTmpDirectory theDirectory{};
        EXPECT_EQ(theDirectory.getPath().string(), path);

        // We should NOT have made this directory, it is expected to be externally managed.
        EXPECT_FALSE(fs::exists(path));
    }

    EXPECT_FALSE(fs::exists(path));
}

TEST(TmpDirTests, VarDoesNotExist)
{
    // Make sure the env is unset, so we will create our own directory
    TestUtils::EnvBackup env("TEST_TMPDIR", "");

    fs::path thePath{};

    {
        const std::string prefix{"/tmp/test_"};

        TestUtils::TestTmpDirectory theDirectory{};
        thePath = theDirectory.getPath();
        EXPECT_TRUE(startsWith(thePath.string(), prefix));
        EXPECT_EQ(thePath.string().size(), 36 + prefix.size());

        // We should have created this directory
        EXPECT_TRUE(fs::exists(thePath));
    }

    // The path should have been eliminated after tearing down the directory
    EXPECT_FALSE(fs::exists(thePath));
}