/*!
 ***********************************************************
 * @file otimeChrono.cc
 * @brief Implements functions for getting local timers.
 *
 * @section License
 *
 * Copyright (c) 2016- Delphi Automotive. All rights
 * reserved.
 *
 * @section changeLogs Change Logs
 *
 * @author Junsung Kim (junsung.kim@delphi.com)
 * @date 01/15/2020
 * refactored, now has multiple timers correlating with std::chrono.
 * @n
 *
 ***********************************************************
 */

#ifndef OTTOTIMECHRONO_CC
#define OTTOTIMECHRONO_CC

#include <TimeStamp/otime.h>
#include <TimeStamp/otimeConversions.h>

namespace ottotime {

template<class Clock>
otimePoint buildotimePointFromChronoTime(const std::chrono::time_point<Clock>& chronoTime)
{
    return ottotime::otimePoint{av::otimeDurationFromDuration(chronoTime.time_since_epoch())};
}

void otimePoint::getCurrentWallClockTime()
{
    *this = buildCurrentWallClockTime();
}
/* static */ otimePoint otimePoint::buildCurrentWallClockTime()
{
    return buildotimePointFromChronoTime<std::chrono::system_clock>(std::chrono::system_clock::now());
}

void otimePoint::getCurrentSteadyTime()
{
    *this = buildCurrentSteadyTime();
}
/* static */ otimePoint otimePoint::buildCurrentSteadyTime()
{
    return buildotimePointFromChronoTime<std::chrono::steady_clock>(std::chrono::steady_clock::now());
}

void otimePoint::getCurrentHighResolutionTime()
{
    *this = buildCurrentHighResolutionTime();
}
/* static */ otimePoint otimePoint::buildCurrentHighResolutionTime()
{
    return buildotimePointFromChronoTime<std::chrono::high_resolution_clock>(std::chrono::high_resolution_clock::now());
}

} // end namespace ottotime

#endif // OTTOTIMECHRONO_CC
