/// @copyright Copyright (c) 2022 - Motional. All rights reserved.
///
/// NOTICE: Removing this heading is prohibited.
///
/// All information contained herein is, and remains the property of Motional,
/// its affiliates, and its suppliers (if any). The intellectual and technical
/// concepts contained herein are proprietary to Motional, its affiliates, and
/// its suppliers (if any) and may be covered by U.S. and Foreign Patents,
/// patents in process, and are protected by trade secrets or copyright law.
/// Dissemination of this information or reproduction of this material is
/// strictly forbidden unless prior written permission is obtained from Motional.
///
/// @file
/// brief This is a brief (1-line) description of the file.
/// This file is generated with dbc2cpp script version : 3.1.0
/// This file was generated from the following DBC file:
/// https://gitlab.ci.motional.com/systems/av-netwrok-infrastructure/Gen2-AVT-Sample-B/-/blob/dd539e5d752bcae0408ed2b8f29b270cc0db624e/Gen2_AVT_B_Control_CAN1.dbc

#include "VehicleCANLibrary/g2h/av_proto/VIC_Dynamic_Error.hpp"
#include <algorithm>
#include <string.h>

using namespace av::can::g2h::avproto;

VIC_Dynamic_Error::VIC_Dynamic_Error(const uint32_t id)
{
    canPacket_.id = id;
    canPacket_.length = VIC_Dynamic_Error_CONSTS::VIC_DYNAMIC_ERROR_PACKET_LEN;
}

VIC_Dynamic_Error::VIC_Dynamic_Error(const uint32_t id, uint8_t* const payload, const uint8_t size)
{
    canPacket_.id = id;

    if ((size <= sizeof(canPacket_.data)) && payload) {
        canPacket_.length = size;
        std::copy_n(payload, size, canPacket_.data);
    }
}

const CanFdPacket& VIC_Dynamic_Error::getCanPacket() const
{
    return canPacket_;
}

double VIC_Dynamic_Error::getALIV_Dynamic_Error() const noexcept
{
    uint64_t bitBuffer{0};

    bitBuffer |= static_cast<uint64_t>(canPacket_.data[2]); // Bytes containing signal
    bitBuffer &= UINT64_C(0xff);                            // Bitmask for signal data
    const double output = static_cast<double>(bitBuffer);   // Unsigned value

    return output;
}

double VIC_Dynamic_Error::getCRC_Dynamic_Error() const noexcept
{
    uint64_t bitBuffer{0};

    bitBuffer |= static_cast<uint64_t>(canPacket_.data[0]);      // Bytes containing signal
    bitBuffer |= static_cast<uint64_t>(canPacket_.data[1]) << 8; // Bytes containing signal
    bitBuffer &= UINT64_C(0xffff);                               // Bitmask for signal data
    const double output = static_cast<double>(bitBuffer);        // Unsigned value

    return output;
}

double VIC_Dynamic_Error::getL_CrossTrackError_10ms() const noexcept
{
    uint64_t bitBuffer{0};

    bitBuffer |= static_cast<uint64_t>(canPacket_.data[3]);      // Bytes containing signal
    bitBuffer |= static_cast<uint64_t>(canPacket_.data[4]) << 8; // Bytes containing signal
    bitBuffer &= UINT64_C(0xffff);                               // Bitmask for signal data
    double output = static_cast<double>(bitBuffer);              // Unsigned value

    output = (output * static_cast<double>(0.0005)) + static_cast<double>(-10); // (output * scale_factor) + bias

    if ((output > static_cast<double>(10)) || (output < static_cast<double>(-10))) // check for boundary
    {
        output = static_cast<double>(VIC_Dynamic_Error_CONSTS::ERROR_OUT_OF_BOUNDARY);
    }
    return output;
}

double VIC_Dynamic_Error::getL_VIC_station_error_m_A() const noexcept
{
    uint64_t bitBuffer{0};

    bitBuffer |= static_cast<uint64_t>(canPacket_.data[18]);       // Bytes containing signal
    bitBuffer |= static_cast<uint64_t>(canPacket_.data[19]) << 8;  // Bytes containing signal
    bitBuffer |= static_cast<uint64_t>(canPacket_.data[20]) << 16; // Bytes containing signal
    bitBuffer &= UINT64_C(0x7ff80);                                // Bitmask for signal data
    bitBuffer = bitBuffer >> 7;                                    // right shift to remove 0s

    double output = static_cast<double>(bitBuffer); // Unsigned value

    output = (output * static_cast<double>(0.005)) + static_cast<double>(-10); // (output * scale_factor) + bias

    if ((output > static_cast<double>(10)) || (output < static_cast<double>(-10))) // check for boundary
    {
        output = static_cast<double>(VIC_Dynamic_Error_CONSTS::ERROR_OUT_OF_BOUNDARY);
    }
    return output;
}

double VIC_Dynamic_Error::getL_VIC_station_error_m_B() const noexcept
{
    uint64_t bitBuffer{0};

    bitBuffer |= static_cast<uint64_t>(canPacket_.data[20]);      // Bytes containing signal
    bitBuffer |= static_cast<uint64_t>(canPacket_.data[21]) << 8; // Bytes containing signal
    bitBuffer &= UINT64_C(0x7ff8);                                // Bitmask for signal data
    bitBuffer = bitBuffer >> 3;                                   // right shift to remove 0s

    double output = static_cast<double>(bitBuffer); // Unsigned value

    output = (output * static_cast<double>(0.005)) + static_cast<double>(-10); // (output * scale_factor) + bias

    if ((output > static_cast<double>(10)) || (output < static_cast<double>(-10))) // check for boundary
    {
        output = static_cast<double>(VIC_Dynamic_Error_CONSTS::ERROR_OUT_OF_BOUNDARY);
    }
    return output;
}

double VIC_Dynamic_Error::getP_BrakePressureTrackingError() const noexcept
{
    uint64_t bitBuffer{0};

    bitBuffer |= static_cast<uint64_t>(canPacket_.data[13]);       // Bytes containing signal
    bitBuffer |= static_cast<uint64_t>(canPacket_.data[14]) << 8;  // Bytes containing signal
    bitBuffer |= static_cast<uint64_t>(canPacket_.data[15]) << 16; // Bytes containing signal
    bitBuffer &= UINT64_C(0x1ffe0);                                // Bitmask for signal data
    bitBuffer = bitBuffer >> 5;                                    // right shift to remove 0s

    double output = static_cast<double>(bitBuffer); // Unsigned value

    output = (output * static_cast<double>(0.1)) + static_cast<double>(-200); // (output * scale_factor) + bias

    if ((output > static_cast<double>(200)) || (output < static_cast<double>(-200))) // check for boundary
    {
        output = static_cast<double>(VIC_Dynamic_Error_CONSTS::ERROR_OUT_OF_BOUNDARY);
    }
    return output;
}

double VIC_Dynamic_Error::getTq_WheelTorqueTrackingError() const noexcept
{
    uint64_t bitBuffer{0};

    bitBuffer |= static_cast<uint64_t>(canPacket_.data[9]);        // Bytes containing signal
    bitBuffer |= static_cast<uint64_t>(canPacket_.data[10]) << 8;  // Bytes containing signal
    bitBuffer |= static_cast<uint64_t>(canPacket_.data[11]) << 16; // Bytes containing signal
    bitBuffer &= UINT64_C(0x3fffc0);                               // Bitmask for signal data
    bitBuffer = bitBuffer >> 6;                                    // right shift to remove 0s

    double output = static_cast<double>(bitBuffer); // Unsigned value

    output = output + static_cast<double>(-32768); // output + bias

    return output;
}

double VIC_Dynamic_Error::geta_AccelTrackingError() const noexcept
{
    uint64_t bitBuffer{0};

    bitBuffer |= static_cast<uint64_t>(canPacket_.data[8]);      // Bytes containing signal
    bitBuffer |= static_cast<uint64_t>(canPacket_.data[9]) << 8; // Bytes containing signal
    bitBuffer &= UINT64_C(0x3ffe);                               // Bitmask for signal data
    bitBuffer = bitBuffer >> 1;                                  // right shift to remove 0s

    double output = static_cast<double>(bitBuffer); // Unsigned value

    output = (output * static_cast<double>(0.01)) + static_cast<double>(-40); // (output * scale_factor) + bias

    if ((output > static_cast<double>(40)) || (output < static_cast<double>(-40))) // check for boundary
    {
        output = static_cast<double>(VIC_Dynamic_Error_CONSTS::ERROR_OUT_OF_BOUNDARY);
    }
    return output;
}

double VIC_Dynamic_Error::getper_AccelPedalTrackingError() const noexcept
{
    uint64_t bitBuffer{0};

    bitBuffer |= static_cast<uint64_t>(canPacket_.data[11]);       // Bytes containing signal
    bitBuffer |= static_cast<uint64_t>(canPacket_.data[12]) << 8;  // Bytes containing signal
    bitBuffer |= static_cast<uint64_t>(canPacket_.data[13]) << 16; // Bytes containing signal
    bitBuffer &= UINT64_C(0x1fffc0);                               // Bitmask for signal data
    bitBuffer = bitBuffer >> 6;                                    // right shift to remove 0s

    double output = static_cast<double>(bitBuffer); // Unsigned value

    output = (output * static_cast<double>(0.01)) + static_cast<double>(-100); // (output * scale_factor) + bias

    if ((output > static_cast<double>(100)) || (output < static_cast<double>(-100))) // check for boundary
    {
        output = static_cast<double>(VIC_Dynamic_Error_CONSTS::ERROR_OUT_OF_BOUNDARY);
    }
    return output;
}

double VIC_Dynamic_Error::getper_BrakePedalTrackingError() const noexcept
{
    uint64_t bitBuffer{0};

    bitBuffer |= static_cast<uint64_t>(canPacket_.data[15]);      // Bytes containing signal
    bitBuffer |= static_cast<uint64_t>(canPacket_.data[16]) << 8; // Bytes containing signal
    bitBuffer &= UINT64_C(0xfffe);                                // Bitmask for signal data
    bitBuffer = bitBuffer >> 1;                                   // right shift to remove 0s

    double output = static_cast<double>(bitBuffer); // Unsigned value

    output = (output * static_cast<double>(0.01)) + static_cast<double>(-100); // (output * scale_factor) + bias

    if ((output > static_cast<double>(100)) || (output < static_cast<double>(-100))) // check for boundary
    {
        output = static_cast<double>(VIC_Dynamic_Error_CONSTS::ERROR_OUT_OF_BOUNDARY);
    }
    return output;
}

double VIC_Dynamic_Error::getu_HeadingError_10ms() const noexcept
{
    uint64_t bitBuffer{0};

    bitBuffer |= static_cast<uint64_t>(canPacket_.data[5]);      // Bytes containing signal
    bitBuffer |= static_cast<uint64_t>(canPacket_.data[6]) << 8; // Bytes containing signal
    bitBuffer &= UINT64_C(0xfff);                                // Bitmask for signal data
    double output = static_cast<double>(bitBuffer);              // Unsigned value

    output = (output * static_cast<double>(0.001)) + static_cast<double>(-1.57); // (output * scale_factor) + bias

    if ((output > static_cast<double>(1.57)) || (output < static_cast<double>(-1.57))) // check for boundary
    {
        output = static_cast<double>(VIC_Dynamic_Error_CONSTS::ERROR_OUT_OF_BOUNDARY);
    }
    return output;
}

double VIC_Dynamic_Error::getu_SteeringWheelTrackingError() const noexcept
{
    uint64_t bitBuffer{0};

    bitBuffer |= static_cast<uint64_t>(canPacket_.data[17]);      // Bytes containing signal
    bitBuffer |= static_cast<uint64_t>(canPacket_.data[18]) << 8; // Bytes containing signal
    bitBuffer &= UINT64_C(0x7fff);                                // Bitmask for signal data
    double output = static_cast<double>(bitBuffer);               // Unsigned value

    output = (output * static_cast<double>(0.1)) + static_cast<double>(-1000); // (output * scale_factor) + bias

    if ((output > static_cast<double>(1000)) || (output < static_cast<double>(-1000))) // check for boundary
    {
        output = static_cast<double>(VIC_Dynamic_Error_CONSTS::ERROR_OUT_OF_BOUNDARY);
    }
    return output;
}

double VIC_Dynamic_Error::getv_SpeedTrackingError() const noexcept
{
    uint64_t bitBuffer{0};

    bitBuffer |= static_cast<uint64_t>(canPacket_.data[6]);       // Bytes containing signal
    bitBuffer |= static_cast<uint64_t>(canPacket_.data[7]) << 8;  // Bytes containing signal
    bitBuffer |= static_cast<uint64_t>(canPacket_.data[8]) << 16; // Bytes containing signal
    bitBuffer &= UINT64_C(0x1fff0);                               // Bitmask for signal data
    bitBuffer = bitBuffer >> 4;                                   // right shift to remove 0s

    double output = static_cast<double>(bitBuffer); // Unsigned value

    output = (output * static_cast<double>(0.01)) + static_cast<double>(-40); // (output * scale_factor) + bias

    if ((output > static_cast<double>(40)) || (output < static_cast<double>(-40))) // check for boundary
    {
        output = static_cast<double>(VIC_Dynamic_Error_CONSTS::ERROR_OUT_OF_BOUNDARY);
    }
    return output;
}

int64_t VIC_Dynamic_Error::setALIV_Dynamic_Error(const double value) noexcept
{
    if ((value > static_cast<double>(255)) ||
        (value < static_cast<double>(0))) // error in setting value due to boundary overflow/underflow
    {
        return static_cast<int64_t>(VIC_Dynamic_Error_CONSTS::ERROR_OUT_OF_BOUNDARY);
    }
    const int64_t rawValue = static_cast<int64_t>(value);

    const uint64_t bitBuffer = static_cast<uint64_t>(rawValue);

    canPacket_.data[2] &= static_cast<uint8_t>(~0xff);                // clear the bits for new data
    canPacket_.data[2] |= static_cast<uint8_t>((bitBuffer) & (0xff)); // push in new data
    return 0;
}

int64_t VIC_Dynamic_Error::setCRC_Dynamic_Error(const double value) noexcept
{
    if ((value > static_cast<double>(65535)) ||
        (value < static_cast<double>(0))) // error in setting value due to boundary overflow/underflow
    {
        return static_cast<int64_t>(VIC_Dynamic_Error_CONSTS::ERROR_OUT_OF_BOUNDARY);
    }
    const int64_t rawValue = static_cast<int64_t>(value);

    const uint64_t bitBuffer = static_cast<uint64_t>(rawValue);

    canPacket_.data[0] &= static_cast<uint8_t>(~0xff);                     // clear the bits for new data
    canPacket_.data[0] |= static_cast<uint8_t>((bitBuffer) & (0xff));      // push in new data
    canPacket_.data[1] &= static_cast<uint8_t>(~0xff);                     // clear the bits for new data
    canPacket_.data[1] |= static_cast<uint8_t>((bitBuffer >> 8) & (0xff)); // push in new data
    return 0;
}

int64_t VIC_Dynamic_Error::setL_CrossTrackError_10ms(const double value) noexcept
{
    if ((value > static_cast<double>(10)) ||
        (value < static_cast<double>(-10))) // error in setting value due to boundary overflow/underflow
    {
        return static_cast<int64_t>(VIC_Dynamic_Error_CONSTS::ERROR_OUT_OF_BOUNDARY);
    }
    const int64_t rawValue = static_cast<int64_t>((value - static_cast<double>(-10)) /
                                                  static_cast<double>(0.0005)); // (value - bias) / scale_factor

    const uint64_t bitBuffer = static_cast<uint64_t>(rawValue);

    canPacket_.data[3] &= static_cast<uint8_t>(~0xff);                     // clear the bits for new data
    canPacket_.data[3] |= static_cast<uint8_t>((bitBuffer) & (0xff));      // push in new data
    canPacket_.data[4] &= static_cast<uint8_t>(~0xff);                     // clear the bits for new data
    canPacket_.data[4] |= static_cast<uint8_t>((bitBuffer >> 8) & (0xff)); // push in new data
    return 0;
}

int64_t VIC_Dynamic_Error::setL_VIC_station_error_m_A(const double value) noexcept
{
    if ((value > static_cast<double>(10)) ||
        (value < static_cast<double>(-10))) // error in setting value due to boundary overflow/underflow
    {
        return static_cast<int64_t>(VIC_Dynamic_Error_CONSTS::ERROR_OUT_OF_BOUNDARY);
    }
    const int64_t rawValue = static_cast<int64_t>((value - static_cast<double>(-10)) /
                                                  static_cast<double>(0.005)); // (value - bias) / scale_factor

    const uint64_t bitBuffer = static_cast<uint64_t>(rawValue);

    canPacket_.data[18] &= static_cast<uint8_t>(~0x80);                     // clear the bits for new data
    canPacket_.data[18] |= static_cast<uint8_t>((bitBuffer << 7) & (0x80)); // push in new data
    canPacket_.data[19] &= static_cast<uint8_t>(~0xff);                     // clear the bits for new data
    canPacket_.data[19] |= static_cast<uint8_t>((bitBuffer >> 1) & (0xff)); // push in new data
    canPacket_.data[20] &= static_cast<uint8_t>(~0x7);                      // clear the bits for new data
    canPacket_.data[20] |= static_cast<uint8_t>((bitBuffer >> 9) & (0x7));  // push in new data
    return 0;
}

int64_t VIC_Dynamic_Error::setL_VIC_station_error_m_B(const double value) noexcept
{
    if ((value > static_cast<double>(10)) ||
        (value < static_cast<double>(-10))) // error in setting value due to boundary overflow/underflow
    {
        return static_cast<int64_t>(VIC_Dynamic_Error_CONSTS::ERROR_OUT_OF_BOUNDARY);
    }
    const int64_t rawValue = static_cast<int64_t>((value - static_cast<double>(-10)) /
                                                  static_cast<double>(0.005)); // (value - bias) / scale_factor

    const uint64_t bitBuffer = static_cast<uint64_t>(rawValue);

    canPacket_.data[20] &= static_cast<uint8_t>(~0xf8);                     // clear the bits for new data
    canPacket_.data[20] |= static_cast<uint8_t>((bitBuffer << 3) & (0xf8)); // push in new data
    canPacket_.data[21] &= static_cast<uint8_t>(~0x7f);                     // clear the bits for new data
    canPacket_.data[21] |= static_cast<uint8_t>((bitBuffer >> 5) & (0x7f)); // push in new data
    return 0;
}

int64_t VIC_Dynamic_Error::setP_BrakePressureTrackingError(const double value) noexcept
{
    if ((value > static_cast<double>(200)) ||
        (value < static_cast<double>(-200))) // error in setting value due to boundary overflow/underflow
    {
        return static_cast<int64_t>(VIC_Dynamic_Error_CONSTS::ERROR_OUT_OF_BOUNDARY);
    }
    const int64_t rawValue = static_cast<int64_t>((value - static_cast<double>(-200)) /
                                                  static_cast<double>(0.1)); // (value - bias) / scale_factor

    const uint64_t bitBuffer = static_cast<uint64_t>(rawValue);

    canPacket_.data[13] &= static_cast<uint8_t>(~0xe0);                     // clear the bits for new data
    canPacket_.data[13] |= static_cast<uint8_t>((bitBuffer << 5) & (0xe0)); // push in new data
    canPacket_.data[14] &= static_cast<uint8_t>(~0xff);                     // clear the bits for new data
    canPacket_.data[14] |= static_cast<uint8_t>((bitBuffer >> 3) & (0xff)); // push in new data
    canPacket_.data[15] &= static_cast<uint8_t>(~0x1);                      // clear the bits for new data
    canPacket_.data[15] |= static_cast<uint8_t>((bitBuffer >> 11) & (0x1)); // push in new data
    return 0;
}

int64_t VIC_Dynamic_Error::setTq_WheelTorqueTrackingError(const double value) noexcept
{
    if ((value > static_cast<double>(32767)) ||
        (value < static_cast<double>(-32768))) // error in setting value due to boundary overflow/underflow
    {
        return static_cast<int64_t>(VIC_Dynamic_Error_CONSTS::ERROR_OUT_OF_BOUNDARY);
    }
    const int64_t rawValue = static_cast<int64_t>(value) - static_cast<int64_t>(-32768); // value - bias

    const uint64_t bitBuffer = static_cast<uint64_t>(rawValue);

    canPacket_.data[9] &= static_cast<uint8_t>(~0xc0);                       // clear the bits for new data
    canPacket_.data[9] |= static_cast<uint8_t>((bitBuffer << 6) & (0xc0));   // push in new data
    canPacket_.data[10] &= static_cast<uint8_t>(~0xff);                      // clear the bits for new data
    canPacket_.data[10] |= static_cast<uint8_t>((bitBuffer >> 2) & (0xff));  // push in new data
    canPacket_.data[11] &= static_cast<uint8_t>(~0x3f);                      // clear the bits for new data
    canPacket_.data[11] |= static_cast<uint8_t>((bitBuffer >> 10) & (0x3f)); // push in new data
    return 0;
}

int64_t VIC_Dynamic_Error::seta_AccelTrackingError(const double value) noexcept
{
    if ((value > static_cast<double>(40)) ||
        (value < static_cast<double>(-40))) // error in setting value due to boundary overflow/underflow
    {
        return static_cast<int64_t>(VIC_Dynamic_Error_CONSTS::ERROR_OUT_OF_BOUNDARY);
    }
    const int64_t rawValue = static_cast<int64_t>((value - static_cast<double>(-40)) /
                                                  static_cast<double>(0.01)); // (value - bias) / scale_factor

    const uint64_t bitBuffer = static_cast<uint64_t>(rawValue);

    canPacket_.data[8] &= static_cast<uint8_t>(~0xfe);                     // clear the bits for new data
    canPacket_.data[8] |= static_cast<uint8_t>((bitBuffer << 1) & (0xfe)); // push in new data
    canPacket_.data[9] &= static_cast<uint8_t>(~0x3f);                     // clear the bits for new data
    canPacket_.data[9] |= static_cast<uint8_t>((bitBuffer >> 7) & (0x3f)); // push in new data
    return 0;
}

int64_t VIC_Dynamic_Error::setper_AccelPedalTrackingError(const double value) noexcept
{
    if ((value > static_cast<double>(100)) ||
        (value < static_cast<double>(-100))) // error in setting value due to boundary overflow/underflow
    {
        return static_cast<int64_t>(VIC_Dynamic_Error_CONSTS::ERROR_OUT_OF_BOUNDARY);
    }
    const int64_t rawValue = static_cast<int64_t>((value - static_cast<double>(-100)) /
                                                  static_cast<double>(0.01)); // (value - bias) / scale_factor

    const uint64_t bitBuffer = static_cast<uint64_t>(rawValue);

    canPacket_.data[11] &= static_cast<uint8_t>(~0xc0);                      // clear the bits for new data
    canPacket_.data[11] |= static_cast<uint8_t>((bitBuffer << 6) & (0xc0));  // push in new data
    canPacket_.data[12] &= static_cast<uint8_t>(~0xff);                      // clear the bits for new data
    canPacket_.data[12] |= static_cast<uint8_t>((bitBuffer >> 2) & (0xff));  // push in new data
    canPacket_.data[13] &= static_cast<uint8_t>(~0x1f);                      // clear the bits for new data
    canPacket_.data[13] |= static_cast<uint8_t>((bitBuffer >> 10) & (0x1f)); // push in new data
    return 0;
}

int64_t VIC_Dynamic_Error::setper_BrakePedalTrackingError(const double value) noexcept
{
    if ((value > static_cast<double>(100)) ||
        (value < static_cast<double>(-100))) // error in setting value due to boundary overflow/underflow
    {
        return static_cast<int64_t>(VIC_Dynamic_Error_CONSTS::ERROR_OUT_OF_BOUNDARY);
    }
    const int64_t rawValue = static_cast<int64_t>((value - static_cast<double>(-100)) /
                                                  static_cast<double>(0.01)); // (value - bias) / scale_factor

    const uint64_t bitBuffer = static_cast<uint64_t>(rawValue);

    canPacket_.data[15] &= static_cast<uint8_t>(~0xfe);                     // clear the bits for new data
    canPacket_.data[15] |= static_cast<uint8_t>((bitBuffer << 1) & (0xfe)); // push in new data
    canPacket_.data[16] &= static_cast<uint8_t>(~0xff);                     // clear the bits for new data
    canPacket_.data[16] |= static_cast<uint8_t>((bitBuffer >> 7) & (0xff)); // push in new data
    return 0;
}

int64_t VIC_Dynamic_Error::setu_HeadingError_10ms(const double value) noexcept
{
    if ((value > static_cast<double>(1.57)) ||
        (value < static_cast<double>(-1.57))) // error in setting value due to boundary overflow/underflow
    {
        return static_cast<int64_t>(VIC_Dynamic_Error_CONSTS::ERROR_OUT_OF_BOUNDARY);
    }
    const int64_t rawValue = static_cast<int64_t>((value - static_cast<double>(-1.57)) /
                                                  static_cast<double>(0.001)); // (value - bias) / scale_factor

    const uint64_t bitBuffer = static_cast<uint64_t>(rawValue);

    canPacket_.data[5] &= static_cast<uint8_t>(~0xff);                    // clear the bits for new data
    canPacket_.data[5] |= static_cast<uint8_t>((bitBuffer) & (0xff));     // push in new data
    canPacket_.data[6] &= static_cast<uint8_t>(~0xf);                     // clear the bits for new data
    canPacket_.data[6] |= static_cast<uint8_t>((bitBuffer >> 8) & (0xf)); // push in new data
    return 0;
}

int64_t VIC_Dynamic_Error::setu_SteeringWheelTrackingError(const double value) noexcept
{
    if ((value > static_cast<double>(1000)) ||
        (value < static_cast<double>(-1000))) // error in setting value due to boundary overflow/underflow
    {
        return static_cast<int64_t>(VIC_Dynamic_Error_CONSTS::ERROR_OUT_OF_BOUNDARY);
    }
    const int64_t rawValue = static_cast<int64_t>((value - static_cast<double>(-1000)) /
                                                  static_cast<double>(0.1)); // (value - bias) / scale_factor

    const uint64_t bitBuffer = static_cast<uint64_t>(rawValue);

    canPacket_.data[17] &= static_cast<uint8_t>(~0xff);                     // clear the bits for new data
    canPacket_.data[17] |= static_cast<uint8_t>((bitBuffer) & (0xff));      // push in new data
    canPacket_.data[18] &= static_cast<uint8_t>(~0x7f);                     // clear the bits for new data
    canPacket_.data[18] |= static_cast<uint8_t>((bitBuffer >> 8) & (0x7f)); // push in new data
    return 0;
}

int64_t VIC_Dynamic_Error::setv_SpeedTrackingError(const double value) noexcept
{
    if ((value > static_cast<double>(40)) ||
        (value < static_cast<double>(-40))) // error in setting value due to boundary overflow/underflow
    {
        return static_cast<int64_t>(VIC_Dynamic_Error_CONSTS::ERROR_OUT_OF_BOUNDARY);
    }
    const int64_t rawValue = static_cast<int64_t>((value - static_cast<double>(-40)) /
                                                  static_cast<double>(0.01)); // (value - bias) / scale_factor

    const uint64_t bitBuffer = static_cast<uint64_t>(rawValue);

    canPacket_.data[6] &= static_cast<uint8_t>(~0xf0);                     // clear the bits for new data
    canPacket_.data[6] |= static_cast<uint8_t>((bitBuffer << 4) & (0xf0)); // push in new data
    canPacket_.data[7] &= static_cast<uint8_t>(~0xff);                     // clear the bits for new data
    canPacket_.data[7] |= static_cast<uint8_t>((bitBuffer >> 4) & (0xff)); // push in new data
    canPacket_.data[8] &= static_cast<uint8_t>(~0x1);                      // clear the bits for new data
    canPacket_.data[8] |= static_cast<uint8_t>((bitBuffer >> 12) & (0x1)); // push in new data
    return 0;
}
