/*!
 ***********************************************************************************************************************
 * APTIV PRIVILEGED AND CONFIDENTIAL
 *
 * @file InterfaceParser.h
 * @product config
 * @version
 * @released
 * @updated 02/05/2019
 * @brief Interface configuration parser
 *
 * @section License
 *
 * Copyright (c) 2019- Aptiv. All rights reserved.
 * NOTE: Removing this heading is prohibited.
 *
 * @section Open Source License List
 *
 ***********************************************************************************************************************
 */

#include <config/InterfaceParser.h>

const std::string InterfaceParser::READ_ONLY_INTERFACES{"ReadOnlyInterfaces"};
const std::string InterfaceParser::WRITE_ONLY_INTERFACES{"WriteOnlyInterfaces"};
const std::string InterfaceParser::READ_WRITE_INTERFACES{"ReadWriteInterfaces"};

bool InterfaceParser::hasReadOnlyInterfaces()
{
    return hasSection(READ_ONLY_INTERFACES);
}

bool InterfaceParser::hasReadOnlyInterface(const std::string& interfaceName)
{
    return hasInterface(READ_ONLY_INTERFACES, interfaceName, getAllReadOnlyInterfaces());
}

bool InterfaceParser::hasWriteOnlyInterfaces()
{
    return hasSection(WRITE_ONLY_INTERFACES);
}

bool InterfaceParser::hasWriteOnlyInterface(const std::string& interfaceName)
{
    return hasInterface(WRITE_ONLY_INTERFACES, interfaceName, getAllWriteOnlyInterfaces());
}

bool InterfaceParser::hasReadWriteInterfaces()
{
    return hasSection(READ_WRITE_INTERFACES);
}

bool InterfaceParser::hasReadWriteInterface(const std::string& interfaceName)
{
    return hasInterface(READ_WRITE_INTERFACES, interfaceName, getAllReadWriteInterfaces());
}

bool InterfaceParser::getReadOnlyInterfacesSection(ConfigSection& roIntf)
{
    return this->getSection(READ_ONLY_INTERFACES, roIntf);
}

bool InterfaceParser::getWriteOnlyInterfacesSection(ConfigSection& woIntf)
{
    return this->getSection(WRITE_ONLY_INTERFACES, woIntf);
}

bool InterfaceParser::getReadWriteInterfacesSection(ConfigSection& rwIntf)
{
    return this->getSection(READ_WRITE_INTERFACES, rwIntf);
}

std::list<ConfigSection> InterfaceParser::getAllReadOnlyInterfaces()
{
    ConfigSection section;
    std::list<ConfigSection> toReturn;
    if (getReadOnlyInterfacesSection(section)) {
        getAllSectionsWithParamName(toReturn, section, "implementation", -1);
    }
    return toReturn;
}

ConfigSection InterfaceParser::getReadOnlyInterface(const std::string& interfaceName)
{
    auto interfaces = getAllReadOnlyInterfaces();
    const auto it = std::find_if(interfaces.begin(), interfaces.end(), [interfaceName](const ConfigSection& c) -> bool {
        return ((READ_ONLY_INTERFACES + "." + interfaceName) == c.getName());
    });
    return (it != interfaces.end()) ? *it : ConfigSection();
}

std::list<ConfigSection> InterfaceParser::getAllWriteOnlyInterfaces()
{
    ConfigSection section;
    std::list<ConfigSection> toReturn;
    if (getWriteOnlyInterfacesSection(section)) {
        getAllSectionsWithParamName(toReturn, section, "implementation", -1);
    }
    return toReturn;
}

ConfigSection InterfaceParser::getWriteOnlyInterface(const std::string& interfaceName)
{
    auto interfaces = getAllWriteOnlyInterfaces();
    const auto it = std::find_if(interfaces.begin(), interfaces.end(), [interfaceName](const ConfigSection& c) -> bool {
        return ((WRITE_ONLY_INTERFACES + "." + interfaceName) == c.getName());
    });
    return (it != interfaces.end()) ? *it : ConfigSection();
}

std::list<ConfigSection> InterfaceParser::getAllReadWriteInterfaces()
{
    ConfigSection section;
    std::list<ConfigSection> toReturn;
    if (getReadWriteInterfacesSection(section)) {
        getAllSectionsWithParamName(toReturn, section, "implementation", -1);
    }
    return toReturn;
}

ConfigSection InterfaceParser::getReadWriteInterface(const std::string& interfaceName)
{
    auto interfaces = getAllReadWriteInterfaces();
    const auto it = std::find_if(interfaces.begin(), interfaces.end(), [interfaceName](const ConfigSection& c) -> bool {
        return ((READ_WRITE_INTERFACES + "." + interfaceName) == c.getName());
    });
    return (it != interfaces.end()) ? *it : ConfigSection();
}
