/*!
 ***********************************************************
 * @file TaskParser.cc
 *
 * @brief file implementing TaskParser class
 *
 * @section License
 *
 * Copyright (c) 2015- Delphi Automotive. All rights
 * reserved.
 *
 * @section changeLogs Change Logs
 *
 * @author Vasudeva Pai (vasudeva.pai@delphi.com)
 * @date 09/19/2017
 * @remark replacing ruby functions with libconfig++.
 *
 ***********************************************************
 */

/**********************************************************
  Copyright (c) 2014-2015 Ottomatika, Inc.
 **********************************************************/

/**********************************************************
  Copyright (c) 2010-2013 Carnegie Mellon University,
  All rights reserved.

  This source code was developed in part with support from
  sponsors including General Motors, the National Science
  Foundation and the US Department of Transportation.

  Use of this software is strictly governed by agreements
  between Carnegie Mellon University and various sponsors.

  Users of this software must also be fully aware and agree
  that Carnegie Mellon does not guarantee the correct
  functioning of this software in any system. Carnegie
  Mellon or any of its affiliates will not be liable for any
  damage and/or penalties resulting from the use of this
  software. Any user of this software takes complete
  responsibility for the use of any software or design.
 **********************************************************/

/*
 * TaskParser.cc
 * Chris Urmson (curmson@ri.cmu.edu)
 * August 23, 2006
 *
 * Copyright (c) 2006
 * Carnegie Mellon University
 * All rights reserved.
 */
#ifndef TASKPARSER_CC
#define TASKPARSER_CC

#include <config/TaskParser.h>
#include <utility/utility.h>

#include "lib/avlog/avlog.hpp"

TaskParser::TaskParser()
    : ConfigParser()
{}

void TaskParser::setTaskParamDirectory(const std::string& dir)
{
    taskParamDir_ = dir + "/";
}

bool TaskParser::parseTask(const std::string& taskName)
{
    taskName_ = taskName;

    bool retStatus(false);

    if (taskParamDir_.empty() == false) {
        bool parsed(false);

        if (taskName_ == "SaferDaemon") {
            // Load configs from SaferDaemon.py file
            // (Safer still needs to load its configs from a python file because the config-generator
            // command is only run after safer is launched)
            std::string saferConfigPath(taskParamDir_ + "common/SaferDaemon.py");
            AVLOG_INFO("Loading Python config for Safer: {}", saferConfigPath);
            parsed = parseFile(saferConfigPath);
        } else {
            // Load configs from libconfig (.cfg) file generated by config-generator
            const std::string cfgPath(taskParamDir_ + taskName + ".cfg");
            AVLOG_INFO("Loading libconfig config for {}: {}", taskName_, cfgPath);
            parsed = parseFile(cfgPath);
        }

        if (parsed) {
            interfaceParser_ = InterfaceParser{inputConfig_};
        }

        retStatus = parsed;
    } else {
        AVLOG_WARN("taskParam directory unset!");
    }
    return retStatus;
}

#endif // ifndef TASKPARSER_CC
