/// @copyright Copyright (c) 2021- Motional. All rights reserved.
///
/// NOTICE: Removing this heading is prohibited.
///
/// All information contained herein is, and remains the property of Motional,
/// its affiliates, and its suppliers (if any). The intellectual and technical
/// concepts contained herein are proprietary to Motional, its affiliates, and
/// its suppliers (if any) and may be covered by U.S. and Foreign Patents,
/// patents in process, and are protected by trade secrets or copyright law.
/// Dissemination of this information or reproduction of this material is
/// strictly forbidden unless prior written permission is obtained from Motional.
///
/// @file
/// @brief Unit tests for @ref ConfigParser

#include "ConfigFixture.hpp"
#include "gtest/gtest.h"
#include <config/ConfigParser.h>

const std::string TESTDIR = std::getenv("TESTDIR");

class SimpleConfigTest : public ConfigFixture
{
public:
    ConfigParser parser_;
    std::string fileName_;
    std::string sectionName_;

    virtual void SetUp() override
    {
        fileName_ = TESTDIR + "/ConfigParserTest1.cfg";
        sectionName_ = "Section1";
    }
};

TEST_F(SimpleConfigTest, parseFile)
{
    ASSERT_TRUE(parser_.parseFile(fileName_)) << "Failed to parse file: " << fileName_;
}

TEST_F(SimpleConfigTest, parseBadFile)
{
    ASSERT_FALSE(parser_.parseFile("CMakeLists.txt")) << "Failed to parse file: " << fileName_;
}

TEST_F(SimpleConfigTest, DISABLED_parseEmptyFile) // reading empty file causes core dump.  And that's OK.
{
    ASSERT_FALSE(parser_.parseFile(TESTDIR + "/emptyFile.cfg")) << "Failed to parse file: " << fileName_;
}

TEST_F(SimpleConfigTest, hasSection)
{
    ASSERT_TRUE(parser_.parseFile(fileName_)) << "Failed to parse file: " << fileName_;
    ASSERT_TRUE(parser_.hasSection(sectionName_)) << sectionName_ << " NOT found!";
}

TEST_F(SimpleConfigTest, getSection)
{
    ASSERT_TRUE(parser_.parseFile(fileName_)) << "Failed to parse file: " << fileName_;
    ConfigSection section;
    ASSERT_TRUE(parser_.getSection(sectionName_, section)) << "Cannot parse section: " << sectionName_;
}

TEST_F(SimpleConfigTest, getNewSection)
{
    ASSERT_TRUE(parser_.parseFile(fileName_)) << "Failed to parse file: " << fileName_;
    sectionName_ = "Section2";
    ConfigSection section;
    ASSERT_TRUE(parser_.getSection(sectionName_, section));
}

TEST_F(SimpleConfigTest, getNewSectionBool)
{
    ASSERT_TRUE(parser_.parseFile(fileName_)) << "Failed to parse file: " << fileName_;
    sectionName_ = "Section2";
    ASSERT_TRUE(parser_.hasSection(sectionName_)) << sectionName_ << " NOT found!";
    ConfigSection section;
    ASSERT_TRUE(parser_.getSection(sectionName_, section)) << "Failed to get section :" << sectionName_;
    ASSERT_TRUE(section.hasParameter("Bool_Parameter")) << "Failed to get parameter :"
                                                        << "Bool_Parameter";
    bool param;
    ASSERT_EQ(ConfigSection::GetStatus::GS_OK, section.get("Bool_Parameter", param)) << "Failed to get parameter :"
                                                                                     << "Bool_Parameter";
    ASSERT_TRUE(param);
}
