/*!
 ***********************************************************************************************************************
 * APTIV PRIVILEGED AND CONFIDENTIAL
 *
 * @file configIo.h
 *
 * @brief utilities for handling config files
 * @author Yimu Wang (yimu.wang@aptiv.com)
 * @date 01/12/2018
 * @remark
 * Initial implementation.
 *
 * @section License
 *
 * Copyright (c) 2018- Aptiv. All rights reserved.
 * NOTE: Removing this heading is prohibited.
 *
 * @section Open Source License List
 *
 ***********************************************************************************************************************
 */

#ifndef CONFIGUTILS_CONFIGIO_H
#define CONFIGUTILS_CONFIGIO_H

#include "inputConfig.h"
#include "outputConfig.h"

namespace config {

#define ARCHIVE_NAME ar

#define CONFIG(T) ARCHIVE_NAME(#T, T)

#define CONFIG_WITH_COMMENT(T, COMMENT) ARCHIVE_NAME(#T, T, COMMENT)

#define CONFIGURABLE(YOUR_VARS)                                                                                        \
    template<typename Archive>                                                                                         \
    void serializeConfig(Archive& ARCHIVE_NAME)                                                                        \
    {                                                                                                                  \
        ARCHIVE_NAME.start();                                                                                          \
        YOUR_VARS;                                                                                                     \
        ARCHIVE_NAME.end();                                                                                            \
    }

template<typename T>
inline std::string toInnerPythonDict(T t, const std::string& name = "config")
{
    config::OutputConfig outputConfig;
    outputConfig(name, t);

    return outputConfig.str();
}

template<typename T>
inline std::string toPythonDict(T t, const std::string& name = "config")
{
    std::ostringstream os;

    os << "root = { " << std::endl << toInnerPythonDict(t, name) << std::endl << "}";

    return os.str();
}

} // namespace config

#endif
