/*!
 ***********************************************************************************************************************
 * APTIV PRIVILEGED AND CONFIDENTIAL
 *
 * @file inputConfig.h
 *
 * @brief utilities for handling config files
 * @author Yimu Wang (yimu.wang@aptiv.com)
 * @date 01/12/2018
 * @remark
 * Initial implementation.
 *
 * @section License
 *
 * Copyright (c) 2018- Aptiv. All rights reserved.
 * NOTE: Removing this heading is prohibited.
 *
 * @section Open Source License List
 *
 ***********************************************************************************************************************
 */

#ifndef CONFIGUTILS_INPUTCONFIG_H
#define CONFIGUTILS_INPUTCONFIG_H

#include <exception>

#include <config/ConfigSection.h>

namespace config {

class InputConfig
{
public:
    InputConfig() {}

    template<typename T>
    void operator()(const std::string& name, T& t, const std::string& = "")
    {
        checkConfigExist(name);
        operatorImpl(name, t);
    }

    void checkConfigExist(const std::string& name)
    {
        if (!section_.hasParameter(name)) {
            std::stringstream ss;
            ss << "[inputConfig.h] config does not exist! name: " << name;
            throw std::runtime_error(ss.str());
        }
    }

    template<typename T>
    typename std::enable_if<!std::is_fundamental<T>::value, void>::type operatorImpl(const std::string& name, T& t)
    {
        InputConfig ar;

        ConfigSection innerSection;
        section_.get(name, innerSection);
        ar.getConfigSection() = innerSection;

        t.serializeConfig(ar);
    }

    template<typename T>
    typename std::enable_if<std::is_fundamental<T>::value, void>::type operatorImpl(const std::string& name, T& t)
    {
        section_.get(name, t);
    }

    // Overload for std::string
    void operatorImpl(const std::string& name, std::string& s) { section_.get(name, s); }

    // Overload for vector
    template<typename T>
    void operatorImpl(const std::string& name, std::vector<T>& vec)
    {
        section_.getArray(name, vec);
    }

    ConfigSection& getConfigSection() { return section_; }

    // TODO: not used, might be safe to remove
    void setFirstIter() { isFirstIter_ = true; }

    // used in serializeConfig in configIo.h
    void start() {}

    // used in serializeConfig in configIo.h
    void end() {}

private:
    ConfigSection section_;

    // TODO: not used, might be safe to remove
    bool verbose_ = true;

    // TODO: not used, might be safe to remove
    bool isFirstIter_ = false;
};

} // namespace config

#endif
