/*!
 ***********************************************************************************************************************
 * APTIV PRIVILEGED AND CONFIDENTIAL
 *
 * @file ConfigUtilsUnitTest
 * @product Tests
 * @version 1 <Release Number>
 * @released TODO <date>
 * @updated 2019-04-02 <date>
 * @brief This file implements a unit-test suite for Configuration Utilities
 *
 * Copyright (c) 2019 - Aptiv. All rights reserved.
 * NOTE: Removing this heading is prohibited.
 ***********************************************************************************************************************
 */
#ifndef CONFIG_UTILS_TEST_STRUCT_H
#define CONFIG_UTILS_TEST_STRUCT_H

#include <configUtils/configIo.h>

/*!
 * @struct OutputConfigSimpleTestParams
 *
 * @brief structure to hold a single configurable test param
 *
 */
struct OutputConfigSimpleTestParams
{
    bool testParam1 = false; // to test bools

    CONFIGURABLE(CONFIG(testParam1);)
};

/*!
 * @class TestClass
 *
 * @brief A configuration CLASS that is used to test classy configs
 *
 */
class TestClass
{
public:
    TestClass() = default;

    std::vector<int> vec = {1, 2, 3, 4, -1, -2};
    int aaa = 2;
    double bbbb = 23333.33;
    float ccccc = 4;
    bool d = true;
    std::string s = "test";

    CONFIGURABLE(CONFIG(aaa); CONFIG(bbbb); CONFIG(ccccc); CONFIG(d); CONFIG(s); CONFIG_WITH_COMMENT(vec, "A vector"))
};

/*!
 * @struct TestStruct
 *
 * @brief Used to test the usage of configuration structs.
 *
 */
struct TestStruct
{
    uint a = 101;
    int32_t b = 2;
    uint16_t u16 = 1;

    std::vector<std::string> strings = {"s1", "s2", "s23333"};

    TestClass testClass;

    CONFIGURABLE(CONFIG(a); CONFIG(b); CONFIG(u16); CONFIG(strings);
                 CONFIG_WITH_COMMENT(testClass, "A class which contains lots of parameters"))
};

/*!
 * @class TestParams
 *
 * @brief a class that contains sub-structures and classes with configuration
 *
 */
class TestParams
{
public:
    TestParams() = default;

    TestClass testClass;
    TestStruct testStruct;
    int aaa = 2;
    size_t c = 23;
    std::string s = "test";

    CONFIGURABLE(CONFIG(aaa); CONFIG(c); CONFIG(s);
                 CONFIG_WITH_COMMENT(testClass, "A class which contain lots of parameters");
                 CONFIG_WITH_COMMENT(testStruct, "A struct which contains lots of parameters"))
};

#endif // CONFIG_UTILS_TEST_STRUCT_H
