/*!
 ***********************************************************************************************************************
 * APTIV PRIVILEGED AND CONFIDENTIAL
 *
 * @file scs/broadcaster.cpp
 * @product coordinate-transform
 * @version 0.1
 * @released 05-Nov-2018
 * @updated 17-Nov-2018
 * @brief Implementation for ScsTransformBroadcaster.
 *
 * @section License
 *
 * Copyright (c) 2018- Aptiv. All rights reserved.
 * NOTE: Removing this heading is prohibited.
 ***********************************************************************************************************************
 */

#include <coordinate-transform/broadcaster.hpp>
#include <interfaces/TransformStampedList/TransformStampedList.h>

#include "lib/avlog/avlog.hpp"

#include <thread>
namespace aptv {
namespace ct {

bool ScsTransformBroadcaster::connect(const std::shared_ptr<GenericOutput<av::msg::TransformStampedList>>& interface)
{
    if (interface == nullptr) {
        return false;
    }
    interface_ = interface;
    return true;
}

bool ScsTransformBroadcaster::sendTransform(const av::msg::TransformStampedList& msg) const
{
    if (interface_ != nullptr) {
        interface_->publish(msg);
    } else {
        AVLOG_CRITICAL("ScsTransformBroadcaster::sendTransform failed because the output interface is NULL.");
        return false;
    }
    return true;
}

bool ScsTransformBroadcaster::sendTransform(const av::msg::TransformStampedList& msg,
                                            av::latency::Pred predecessor) const
{
    if (interface_ != nullptr) {
        interface_->publish(msg, predecessor);
    } else {
        AVLOG_CRITICAL("ScsTransformBroadcaster::sendTransform failed because the output interface is NULL.");
        return false;
    }
    return true;
}
} // namespace ct
} // namespace aptv
