/*!
 ***********************************************************************************************************************
 * APTIV PRIVILEGED AND CONFIDENTIAL
 *
 * @file scs/broadcaster.hpp
 * @product coordinate-transform
 * @version 0.1
 * @released 05-Nov-2018
 * @updated 17-Nov-2018
 * @brief Header for ScsTransformBroadcaster.
 *
 * @section License
 *
 * Copyright (c) 2018- Aptiv. All rights reserved.
 * NOTE: Removing this heading is prohibited.
 ***********************************************************************************************************************
 */

#ifndef __CT_SCS_BROADCASTER_H_
#define __CT_SCS_BROADCASTER_H_

#include "conversions.hpp"
// SimpleComms
#include "platform/middleware/libs/LatencyLogger/include/LatencyLogger/LatencyLogger.h"

#include <interfaces/Generic/Output/Abstract.h>
#include <interfaces/TransformStampedList/TransformStampedList.h>

namespace aptv {
namespace ct {

/*!
   \brief This class provides an easy way to publish coordinate frame transform information through \b SimpleComms.

   It will handle all the messaging and stuffing of messages. And the function prototypes lay out all the
   necessary data needed for each message.
*/
class ScsTransformBroadcaster
{
public:
    /*!
       \brief default Constructor
    */
    ScsTransformBroadcaster() = default;

    /*!
       \brief Connect to SimpleComms and prepare channel where the transforms will be published
       \param channelName name of the channel we want to publish to
       \return true if the connection is successful, false otherwise.
    */
    bool connect(const std::shared_ptr<GenericOutput<av::msg::TransformStampedList>>& interface);

    /*!
       \brief Send a TransformStamped
       The TransformStamped data structure includes stamp, parentFrame and childFrame
       \param msg The TransformStamped message to be sent
       \return true if the operation is successful, false otherwise.
    */
    bool sendTransform(const av::msg::TransformStampedList& msg) const;

    /*!
       \brief Send a TransformStamped with latency event predecessor
       The TransformStamped data structure includes stamp, parentFrame and childFrame
       \param msg The TransformStamped message to be sent
       \param predecessor LatencyLogger event which is a predecessor to this function
       \return true if the operation is successful, false otherwise.
    */
    bool sendTransform(const av::msg::TransformStampedList& msg, av::latency::Pred predecessor) const;

private:
    // SimpleComms
    std::shared_ptr<GenericOutput<av::msg::TransformStampedList>> interface_{
        nullptr}; /*!< The output interface to send TransformStamped messages */
};
using ScsTransformBroadcasterPtr = std::shared_ptr<ScsTransformBroadcaster>;

} // namespace ct
} // namespace aptv

#endif
