/*!
 ***********************************************************************************************************************
 * APTIV PRIVILEGED AND CONFIDENTIAL
 *
 * @file scs/conversions.hpp
 * @product coordinate-transform
 * @version 0.1
 * @released 05-Nov-2018
 * @updated 17-Nov-2018
 * @brief Conversions for different transforms backends to be used alongside SimpleComms.
 *
 * @section License
 *
 * Copyright (c) 2018- Aptiv. All rights reserved.
 * NOTE: Removing this heading is prohibited.
 ***********************************************************************************************************************
 */

#ifndef __CT_SCS_CONVERSIONS_H_
#define __CT_SCS_CONVERSIONS_H_

#include "ct/cache.hpp"
#include <TimeStamp/TimeStamp.h>
#include <interfaces/SmoothedLocalizationPoseList/SmoothedLocalizationPoseList.h>
#include <interfaces/TransformStampedList/TransformStampedList.h>
#include <interfaces/rec_geometry_avt/rec_geometry_avt.h>
#include <recgeometry/recGeometry.h>

namespace aptv {
namespace ct {
namespace scs {
namespace bullet {

/*!
   \brief converts a TransformStamped message/interface into a bullet btTransform
   \param msg the imput TransformStamped
   \return The resulting bullet btTransform
   \note The frame names and the timestamp won't be present in the returned btTransform.
*/
btTransform fromMsg(const av::msg::TransformStamped& msg);

/*!
   \brief converts a bullet btTransform into a TransformStamped message/interface
   \param bt the input bullet btTransform.
   \param stamp timestamp, in microseconds, to be assigned to the message.
   \param parentFrame Parent frame name
   \param childFrame Child frame name
   \return The resulting TransformStamped msg
*/
av::msg::TransformStamped toMsg(const btTransform& bt,
                                const ottotime::otimePoint& stamp,
                                const std::string& parentFrame,
                                const std::string& childFrame);

} // namespace bullet

namespace storage {

/*!
   \brief converts a TransformStamped message/interface into a TransformStorage object
   \param msg the imput TransformStamped
   \return The resulting TransformStorage object
*/
TransformStorage fromMsg(const av::msg::TransformStamped& msg);

/*!
   \brief converts a TransformStorage object into a TransformStamped message/interface
   \param tf the input TransformStorage object
   \return The resulting TransformStamped msg
*/
av::msg::TransformStamped toMsg(const TransformStorage& ts);

} // namespace storage

/**
 * \brief convert a SmoothedLocalizationPose object into a transformStampedList object
 * \param smoothedPose the input SmoothedLocalizationPose object
 * \return TransformStampedList The resulting transformStampedList object. Notice these are all dynamic transforms.
 */
av::msg::TransformStampedList toTransformList(const av::msg::SmoothedLocalizationPose& smoothedPose);

RecPose3D getPoseInENU(const RecPose3D& transform);

av::geometry::RecPose3D getPoseInENU(const av::geometry::RecPose3D& transformMsg);

RecPose3D getSTDInENU(const RecPose3D& transform);

av::geometry::RecPose3D getSTDInENU(const av::geometry::RecPose3D& transformMsg);

RecPoint3D getAccelerationInENU(const RecPoint3D& acc);

av::geometry::RecPoint3D getAccelerationInENU(const av::geometry::RecPoint3D& accMsg);

RecDifferentialPose3D getVelocityInENU(const RecDifferentialPose3D& velocity);

av::geometry::RecDifferentialPose3D getVelocityInENU(const av::geometry::RecDifferentialPose3D& velocityMsg);

RecDifferentialPose3D getVelocitySTDInENU(const RecDifferentialPose3D& velocitySTD);

av::geometry::RecDifferentialPose3D getVelocitySTDInENU(const av::geometry::RecDifferentialPose3D& velocitySTDMsg);

void fillInTransAndRot(const RecPose3D& pENU, av::msg::TransformStamped& tfMsg);
} // namespace scs

RecPose3D getPoseFromTransform(const TransformStorage& tf);

} // namespace ct
} // namespace aptv

#endif
