/**********************************************************
  Copyright (c) 2014-    Ottomatika, Inc.
 **********************************************************/

/**********************************************************
  Copyright (c) 2010-2013 Carnegie Mellon University,
  All rights reserved.

  This source code was developed in part with support from
  sponsors including General Motors, the National Science
  Foundation and the US Department of Transportation.

  Use of this software is strictly governed by agreements
  between Carnegie Mellon University and various sponsors.

  Users of this software must also be fully aware and agree
  that Carnegie Mellon does not guarantee the correct
  functioning of this software in any system. Carnegie
  Mellon or any of its affiliates will not be liable for any
  damage and/or penalties resulting from the use of this
  software. Any user of this software takes complete
  responsibility for the use of any software or design.
 **********************************************************/

/**
 * @file    CanPort/Type.h
 * @author  Tugrul Galatali
 * @date    10/10/2006
 *
 * @attention Copyright (c) 2006
 * @attention Carnegie Mellon University
 * @attention All rights reserved.
 */
#ifndef _CAN_FD_PORT_TYPE_H_
#define _CAN_FD_PORT_TYPE_H_

#include <stdint.h>

#include <cereal/types/chrono.hpp>
#include <chrono>

#include <ODatum/ODatum.h>
#include <TimeStamp/otime.h>

struct CanFdPacket : public aptv::ODatum
{
    ottotime::otimePoint timeStamp;
    uint32_t id;
    uint8_t flags; // additional flags for CAN FD
    bool extended;
    uint8_t length;
    uint8_t data[64];

    CanFdPacket()
        : id(0)
        , flags(0)
        , extended(false)
        , length(0)
        , data{}
    {}

    template<class Archive>
    void serialize(Archive& ar, [[maybe_unused]] const unsigned int version)
    {
        ar& CEREAL_NVP(timeStamp) & CEREAL_NVP(id) & CEREAL_NVP(flags) & CEREAL_NVP(extended) & CEREAL_NVP(length) &
            CEREAL_NVP(data);
    }
};

CEREAL_CLASS_VERSION(CanFdPacket, 1)
CEREAL_REGISTER_TYPE(CanFdPacket)

#endif
