/**********************************************************
  Copyright (c) 2014-    Ottomatika, Inc.
 **********************************************************/

/**********************************************************
  Copyright (c) 2010-2013 Carnegie Mellon University,
  All rights reserved.

  This source code was developed in part with support from
  sponsors including General Motors, the National Science
  Foundation and the US Department of Transportation.

  Use of this software is strictly governed by agreements
  between Carnegie Mellon University and various sponsors.

  Users of this software must also be fully aware and agree
  that Carnegie Mellon does not guarantee the correct
  functioning of this software in any system. Carnegie
  Mellon or any of its affiliates will not be liable for any
  damage and/or penalties resulting from the use of this
  software. Any user of this software takes complete
  responsibility for the use of any software or design.
 **********************************************************/

#include <interfaces/CanPort/CanTest/Impl.h>

using namespace std;

CanPortTest::CanPortTest(ConfigSection& cs, const task::Access& acc)
    : CanPort(cs, acc)
    , connected_(false)
    , timesReq_(0)
{}

CanPortTest::~CanPortTest()
{
    cleanup();
}

bool CanPortTest::initialize()
{
    return connect();
}

void CanPortTest::cleanup()
{
    if (isConnected()) {
        disconnect();
    }
}

bool CanPortTest::connect()
{
    printf("CanTest Connected\n");
    connected_ = true;
    timesReq_ = 0;

    return true;
}

bool CanPortTest::disconnect()
{
    printf("CanTest Disconnected\n");
    connected_ = false;
    return true;
}

bool CanPortTest::isConnected()
{
    return connected_;
}

bool CanPortTest::read_packet(CanPacket& pkt, bool /*blocking*/)
{
    pkt.data[0] = 123;
    pkt.data[1] = timesReq_;
    pkt.data[2] = 228;
    pkt.data[3] = 45;
    pkt.data[4] = 67;
    pkt.data[5] = 89;
    pkt.data[6] = 32;
    pkt.data[7] = 87;
    pkt.length = 8;
    pkt.id = 0x0c9;

    // return !((timesReq_ = timesReq_++ % 30)%10); // Undefined behaviour, depending on compiler!

    // Not sure of original desired behavior, but lets return true for every 10th count:
    //
    // 0 --> bool = false, therefore !0 = true
    // 1 to 9 --> bool = true, therefore !(1 to 9) = false
    timesReq_++;
    return !(timesReq_ % 10);
}

bool CanPortTest::write_packet(CanPacket /*pkt*/)
{
    return false;
}

bool CanPortTest::write_packet(const std::vector<CanPacket>& /*pktArray*/)
{
    return false;
}

bool CanPortTest::set_filter(std::vector<uint32_t> /*ids*/)
{
    return false;
}

bool CanPortTest::add_filter(std::vector<uint32_t> /*ids*/)
{
    return false;
}

void CanPortTest::unblock() {}

void CanPortTest::clear() {}
