/*!
 ***********************************************************
 * @file interfaces/CanPort/Abstract.h
 * @brief A file defining the CanPort class
 *
 * @section License
 *
 * Copyright (c) 2015- Delphi Automotive. All rights
 * reserved.
 *
 * @section changeLogs Change Logs
 *
 * @author Junsung Kim (junsung.kim@delphi.com)
 * @date 11/03/2017
 * @remark
 * handling the access mode of Standby to support a backup
 * task properly
 * @n
 *
 * @author Junsung Kim (junsung.kim@delphi.com)
 * @date 06/09/2016
 * @remark
 * added Header info for Interface
 * @n
 *
 ***********************************************************
 */

/**********************************************************
  Copyright (c) 2014-2015 Ottomatika, Inc.
 **********************************************************/

/**********************************************************
  Copyright (c) 2010-2013 Carnegie Mellon University,
  All rights reserved.

  This source code was developed in part with support from
  sponsors including General Motors, the National Science
  Foundation and the US Department of Transportation.

  Use of this software is strictly governed by agreements
  between Carnegie Mellon University and various sponsors.

  Users of this software must also be fully aware and agree
  that Carnegie Mellon does not guarantee the correct
  functioning of this software in any system. Carnegie
  Mellon or any of its affiliates will not be liable for any
  damage and/or penalties resulting from the use of this
  software. Any user of this software takes complete
  responsibility for the use of any software or design.
 **********************************************************/

/*
 * CanPort/Abstract.h
 * Tugrul Galatali
 * 10/10/2006
 *
 * Copyright (c) 2006
 * Carnegie Mellon University
 * All rights reserved.
 */
#ifndef _CANPORT_ABSTRACT_H_
#define _CANPORT_ABSTRACT_H_

#include <Comms/Interface.h>
#include <interfaces/baseTypes/Header.h>
#include <interfaces/baseTypes/Input.h>
#include <interfaces/baseTypes/Output.h>

#include <interfaces/CanPort/Type.h>

class CanPort
    : public task::Interface<CanPacket>
    , public interfaces::baseTypes::Input<CanPacket>
    , public interfaces::baseTypes::Output<CanPacket>
{
public:
    CanPort(ConfigSection& cs, const task::Access& acc)
        : task::Interface<CanPacket>(cs, acc)
    {
        mode_ = acc;
    }
    virtual ~CanPort(){};

    virtual bool connect() = 0;
    virtual bool disconnect() = 0;

    virtual bool isConnected() = 0;

    virtual void chmod(const task::Access& acc) { mode_ = acc; }

    virtual bool read_packet(CanPacket& pkt, bool blocking = false) = 0;
    virtual bool write_packet(CanPacket pkt) = 0;
    virtual bool write_packet(const std::vector<CanPacket>& pktArray) = 0;
    virtual bool set_filter(std::vector<uint32_t> ids) = 0;
    virtual bool add_filter(std::vector<uint32_t> ids) = 0;
    virtual void unblock() override = 0;

    virtual bool get(CanPacket& val, bool blocking = false) { return read_packet(val, blocking); }

    template<class... Preds>
    bool publish(const CanPacket& val, Preds... predecessors)
    {
        std::vector<av::latency::Pred> p{predecessors...};
        return publishV(val, std::move(p));
    }

    virtual bool publishV(const CanPacket& val, std::vector<av::latency::Pred>&& /*predecessors*/)
    {
        if ((mode_ == task::Write) || (mode_ == task::ReadWrite)) {
            return write_packet(val);
        } else {
            return true;
        }
    }

    template<class... Preds>
    bool publish(const std::vector<CanPacket>& val, Preds... predecessors)
    {
        std::vector<av::latency::Pred> p{predecessors...};
        return publishV(val, std::move(p));
    }

    virtual bool publishV(const std::vector<CanPacket>& val, std::vector<av::latency::Pred>&& /*predecessors*/)
    {
        if ((mode_ == task::Write) || (mode_ == task::ReadWrite)) {
            return write_packet(val);
        } else {
            return true;
        }
    }

private:
    task::Access mode_;
};

#endif
