/*!
 **********************************************************
 * @file      interfaces/CanPort/Channel/Impl.h
 * @author    Junsung Kim
 * @date      06/09/2016
 * @attention using a timestamp marked when the data is
 * @attention sent for the log playback support
 *
 * @attention Copyright (c) 2016- Delphi Automotive.
 * @attention All rights reserved.
 **********************************************************
 */

/**********************************************************
  Copyright (c) 2014-2015 Ottomatika, Inc.
 **********************************************************/

/**********************************************************
  Copyright (c) 2010-2013 Carnegie Mellon University,
  All rights reserved.

  This source code was developed in part with support from
  sponsors including General Motors, the National Science
  Foundation and the US Department of Transportation.

  Use of this software is strictly governed by agreements
  between Carnegie Mellon University and various sponsors.

  Users of this software must also be fully aware and agree
  that Carnegie Mellon does not guarantee the correct
  functioning of this software in any system. Carnegie
  Mellon or any of its affiliates will not be liable for any
  damage and/or penalties resulting from the use of this
  software. Any user of this software takes complete
  responsibility for the use of any software or design.
 **********************************************************/

/**
 * @file    CanPort/Channel/Impl.h
 * @author  Tugrul Galatali
 * @date    10/14/2006
 *
 * @attention Copyright (c) 2006
 * @attention Carnegie Mellon University
 * @attention All rights reserved.
 */
#ifndef _CANPORT_CHANNEL_IMPL_H_
#define _CANPORT_CHANNEL_IMPL_H_

#include <interfaces/CanPort/Abstract.h>

class CanPortChannel : public CanPort
{
public:
    CanPortChannel(ConfigSection& cs, const task::Access& acc);
    virtual ~CanPortChannel();

    virtual bool initialize();
    virtual void cleanup();

    virtual bool connect();
    virtual bool disconnect();

    virtual bool isConnected();

    virtual bool read_packet(CanPacket& pkt, bool blocking = false);
    virtual bool write_packet(CanPacket pkt);
    virtual bool write_packet(const std::vector<CanPacket>& pktArray);
    virtual bool set_filter(std::vector<uint32_t> ids);
    virtual bool add_filter(std::vector<uint32_t> ids);
    virtual void unblock() override;
    virtual void clear() override;

private:
    bool connected_;
    uint32_t headerVersion_;
};

#endif
