/**********************************************************
  Copyright (c) 2014-    Ottomatika, Inc.
 **********************************************************/

/**********************************************************
  Copyright (c) 2010-2013 Carnegie Mellon University,
  All rights reserved.

  This source code was developed in part with support from
  sponsors including General Motors, the National Science
  Foundation and the US Department of Transportation.

  Use of this software is strictly governed by agreements
  between Carnegie Mellon University and various sponsors.

  Users of this software must also be fully aware and agree
  that Carnegie Mellon does not guarantee the correct
  functioning of this software in any system. Carnegie
  Mellon or any of its affiliates will not be liable for any
  damage and/or penalties resulting from the use of this
  software. Any user of this software takes complete
  responsibility for the use of any software or design.
 **********************************************************/

/**
 * @file    CanPort/PCAN/Impl.h
 * @author  Junsung Kim
 * @date    10/15/2011
 *
 * @attention Copyright (c) 2011
 * @attention Carnegie Mellon University
 * @attention All rights reserved.
 */

#ifndef _CANPORT_PCAN_IMPL_H_
#define _CANPORT_PCAN_IMPL_H_

#include <interfaces/CanPort/Abstract.h>

#include <pthread.h>

#include <string>

#include <RingBuffer/RingBuffer.hpp>
#include <algorithm>
#include <cstdlib>
#include <cstring>
#include <errno.h>
#include <fcntl.h>
#include <linux/can.h>
#include <linux/can/raw.h>
#include <linux/sockios.h>
#include <net/if.h>
#include <stdio.h>
#include <sys/ioctl.h>
#include <sys/socket.h>
#include <termios.h>
#include <unistd.h>

class CanPortPCAN : public CanPort
{
public:
    CanPortPCAN(ConfigSection& cs, const task::Access& acc);
    virtual ~CanPortPCAN();

    virtual bool initialize();
    virtual void cleanup();

    virtual bool connect();
    virtual bool disconnect();

    virtual bool isConnected();

    virtual bool read_packet(CanPacket& pkt, bool blocking = false);
    virtual bool write_packet(CanPacket pkt);
    virtual bool write_packet(const std::vector<CanPacket>& pktArray);
    virtual bool set_filter(std::vector<uint32_t> ids);
    virtual bool add_filter(std::vector<uint32_t> ids);
    virtual void unblock() override;
    virtual void clear() override;

protected:
    std::string _device;
    struct ifreq _ifr;
    struct sockaddr_can _addr;

    std::vector<uint32_t> ids_;

    int _fd;

    static constexpr size_t inboundMessageQueueCapacity{1200};
    av::FCRingBuffer<CanPacket, inboundMessageQueueCapacity> _in_queue;

    pthread_t _io_thread;
    pthread_mutex_t _in_queue_mutex;
    pthread_cond_t _packet_available_cond, _write_complete_cond;

    static void* start_io_thread(void* arg) { return static_cast<CanPortPCAN*>(arg)->io_thread(); }

    volatile bool _connected;
    virtual void* io_thread();
};

#endif
