/*!
 **********************************************************
 * @file      Clock/Playback/Client/Impl.cc
 * @author    Junsung Kim
 * @date      06/09/2016
 * @attention using a timestamp marked when the data is
 * @attention sent for the log playback support
 *
 * @author    Junsung Kim
 * @date      02/18/2016
 * @attention The clock format is otime instead of boost
 *
 * @attention Copyright (c) 2016- Delphi Automotive.
 * @attention All rights reserved.
 **********************************************************
 */

/**********************************************************
  Copyright (c) 2014-2015 Ottomatika, Inc.
 **********************************************************/

/**********************************************************
  Copyright (c) 2010-2013 Carnegie Mellon University,
  All rights reserved.

  This source code was developed in part with support from
  sponsors including General Motors, the National Science
  Foundation and the US Department of Transportation.

  Use of this software is strictly governed by agreements
  between Carnegie Mellon University and various sponsors.

  Users of this software must also be fully aware and agree
  that Carnegie Mellon does not guarantee the correct
  functioning of this software in any system. Carnegie
  Mellon or any of its affiliates will not be liable for any
  damage and/or penalties resulting from the use of this
  software. Any user of this software takes complete
  responsibility for the use of any software or design.
 **********************************************************/

/**
 * @file    Clock/Playback/Client/Impl.cc
 * @author  Tugrul Galatali
 * @date    11/06/2006
 *
 * @attention Copyright (c) 2006
 * @attention Carnegie Mellon University
 * @attention All rights reserved.
 */
#include <interfaces/Clock/Playback/Client/Impl.h>

using namespace ottotime;
using namespace std::chrono;

ClockPlaybackClient::ClockPlaybackClient(ConfigSection& cs, const task::Access& acc)
    : PlaybackClockParamsInput(cs, acc)
    , receivedParams_(false)
{
    headerVersion_ = OTTO_INTERFACE_HEADER_VERSION;
}

ClockPlaybackClient::~ClockPlaybackClient() {}

bool ClockPlaybackClient::initialize()
{
    return (task::Interface<PlaybackClockParams>::channel_->connect() == task::CS_Success);
}

void ClockPlaybackClient::cleanup()
{
    task::Interface<PlaybackClockParams>::channel_->disconnect();
}

otimePoint ClockPlaybackClient::getCurrentTime()
{
    get(latestParams_);
    return latestParams_.apply(otimePoint::buildCurrentSteadyTime());
}

bool ClockPlaybackClient::get(PlaybackClockParams& val, bool blocking)
{
    {
        PlaybackClockParams newParams;
        interfaces::baseTypes::InterfaceHeader hdr;
        std::size_t hdrSize = interfaces::baseTypes::getInterfaceHeaderSize(headerVersion_);
        auto hdr_ptr = reinterpret_cast<char*>(&hdr); // NOLINT(cppcoreguidelines-pro-type-reinterpret-cast)
        while (channel_->read(hdr_ptr, hdrSize, newParams, blocking) == task::CS_NewData) {
            uint32_t hdrVer;
            getHeaderVersion(hdr, hdrVer);
            if (hdrVer != headerVersion_) {
                headerVersion_ = hdrVer;
                hdrSize = interfaces::baseTypes::getInterfaceHeaderSize(headerVersion_);
                continue;
            }
            latestParams_ = newParams;
            receivedParams_ = true;
        }
    }

    if (receivedParams_) {
        val = latestParams_;
    }

    return receivedParams_;
}

void ClockPlaybackClient::unblock(void)
{
    if (this->channel_) {
        this->channel_->unblock();
    }
}

void ClockPlaybackClient::clear()
{
    if (this->channel_) {
        this->channel_->clear();
    }
}
