/**
 * \copyright
 * Copyright (c) 2019- Aptiv. All rights reserved.
 * NOTICE: Removing this heading is prohibited.
 * All information contained herein is, and remains the property of Aptiv and its suppliers, if any.
 * The intellectual and technical concepts contained herein are proprietary to Aptiv and its suppliers and may be
 * covered by U.S. and Foreign Patents, patents in process, and are protected by trade secret or copyright law.
 * Dissemination of this information or reproduction of this material is strictly forbidden unless prior written
 * permission is obtained from Aptiv.
 *
 * \brief This interface sends all the diagnostics information from different modules to the Collector.
 */

#ifndef _DIAGMONITORSTATUS_H_
#define _DIAGMONITORSTATUS_H_

#include <ODatum/ODatum.h>
#include <TimeStamp/otime.h>
#include <interfaces/traits/csvable.h>
#include <log/NamedVars.h>

#include <cereal/types/utility.hpp>
#include <utility>

/**
 * @brief The Severity enum for defining the severity of diagnostics messages.
 */
enum class Severity : uint8_t
{
    INFO = 0,
    WARN,
    FAIL
};

DEFINE_ENUM_OSTREAM(Severity::INFO, Severity::WARN, Severity::FAIL)

struct DiagMonitorStatus
    : public csvable
    , public aptv::ODatum
{
public:
    DiagMonitorStatus() = default;

    ottotime::otimePoint timestamp;          ///< timestamp of the diagnostic message.
    std::string moduleName;                  ///< name of the module sending the diagnostic message.
    std::pair<Severity, std::string> status; ///< diagnostic message.

    virtual void toCsv(std::ostream& out) const
    {
        out.precision(16);
        out << timestamp << ", ";
        out << moduleName << ", ";
        out << aptv::toString(status.first);
        out << " - " << status.second;
    }

    template<class Archive>
    void serialize(Archive& ar, unsigned int version)
    {
        if (version >= 1) {
            ar& CEREAL_NVP(timestamp);
            ar& CEREAL_NVP(moduleName);
            ar& CEREAL_NVP(status);
        }
    }
};

CEREAL_CLASS_VERSION(DiagMonitorStatus, 1)
CEREAL_REGISTER_TYPE(DiagMonitorStatus)

#endif // _DIAGMONITORSTATUS_H_
