/*!
 **********************************************************
 * @file      Generic/Input/Abstract.h
 * @author    Junsung Kim
 * @date      06/09/2016
 * @attention added Header to Interface
 *
 * @attention Copyright (c) 2016- Delphi Automotive.
 * @attention All rights reserved.
 **********************************************************
 */

/**********************************************************
  Copyright (c) 2014-2015 Ottomatika, Inc.
 **********************************************************/

/**********************************************************
  Copyright (c) 2010-2013 Carnegie Mellon University,
  All rights reserved.

  This source code was developed in part with support from
  sponsors including General Motors, the National Science
  Foundation and the US Department of Transportation.

  Use of this software is strictly governed by agreements
  between Carnegie Mellon University and various sponsors.

  Users of this software must also be fully aware and agree
  that Carnegie Mellon does not guarantee the correct
  functioning of this software in any system. Carnegie
  Mellon or any of its affiliates will not be liable for any
  damage and/or penalties resulting from the use of this
  software. Any user of this software takes complete
  responsibility for the use of any software or design.
 **********************************************************/

/**
 * @file    Generic/Input/Abstract.h
 * @author  Tugrul Galatali
 * @date    09/13/2006
 *
 * @author  Junsung Kim
 * @date    12/16/2012
 * @attention A function chmod() is added to support fault-tolerance.
 *
 * @attention Copyright (c) 2006
 * @attention Carnegie Mellon University
 * @attention All rights reserved.
 */
#ifndef _GENERIC_INPUT_ABSTRACT_H_
#define _GENERIC_INPUT_ABSTRACT_H_

#include <Comms/Interface.h>
#include <interfaces/baseTypes/Header.h>
#include <interfaces/baseTypes/Input.h>

/**
 * \brief Interface Abstract Base Class for a GenericInput
 */
template<typename T>
class GenericInputIntf
    : virtual public task::IDbObject
    , public interfaces::baseTypes::InterfaceHeader
    , public interfaces::baseTypes::Input<T>
{
public:
    ~GenericInputIntf() override = default;
};

template<typename T>
class GenericInput
    : public GenericInputIntf<T>
    , public task::Interface<T>
{
public:
    GenericInput(ConfigSection& cs, const task::Access& acc)
        : task::Interface<T>(cs, acc)
    {}
    ~GenericInput() override = default;
    void chmod([[maybe_unused]] const task::Access& acc) override {}
};

#endif
