/*!
 **********************************************************
 * @file      Generic/Input/Channel/Impl.h
 * @author    Junsung Kim
 * @date      06/09/2016
 * @attention get() is updated to use a timestamp marked
 * @attention when the data is sent for the log playback
 *
 * @attention Copyright (c) 2016- Delphi Automotive.
 * @attention All rights reserved.
 *
 * @updated   03/12/2018 Added latency information calculation
 * using the LatencyStats class
 **********************************************************
 */

/**********************************************************
  Copyright (c) 2014-2015 Ottomatika, Inc.
 **********************************************************/

/**********************************************************
  Copyright (c) 2010-2013 Carnegie Mellon University,
  All rights reserved.

  This source code was developed in part with support from
  sponsors including General Motors, the National Science
  Foundation and the US Department of Transportation.

  Use of this software is strictly governed by agreements
  between Carnegie Mellon University and various sponsors.

  Users of this software must also be fully aware and agree
  that Carnegie Mellon does not guarantee the correct
  functioning of this software in any system. Carnegie
  Mellon or any of its affiliates will not be liable for any
  damage and/or penalties resulting from the use of this
  software. Any user of this software takes complete
  responsibility for the use of any software or design.
 **********************************************************/

/**
 * @file    Generic/Input/Channel/Impl.h
 * @author  Tugrul Galatali
 * @date    09/13/2006
 *
 * @author  Junsung Kim
 * @date    12/16/2012
 * @attention A function chmod() is added to support fault-tolerance.
 *
 * @attention Copyright (c) 2006
 * @attention Carnegie Mellon University
 * @attention All rights reserved.
 */
#ifndef _GENERIC_INPUT_CHANNEL_IMPL_H_
#define _GENERIC_INPUT_CHANNEL_IMPL_H_

#include <interfaces/Generic/Input/Abstract.h>

template<typename T>
class GenericInputChannel : public GenericInput<T>
{
public:
    GenericInputChannel(ConfigSection& cs, const task::Access& acc);
    ~GenericInputChannel() override;

    bool get(T& val, bool blocking = false) override;

    void unblock() override;
    bool initialize() override;
    void cleanup() override;
    void chmod(const task::Access& acc) override;
    void clear() override;

private:
    uint32_t headerVersion_;
};

#include "interfaces/Generic/Input/Channel/Impl.def.h"

#endif
