/**********************************************************
  Copyright (c) 2014-    Ottomatika, Inc.
 **********************************************************/

/**********************************************************
  Copyright (c) 2010-2013 Carnegie Mellon University,
  All rights reserved.

  This source code was developed in part with support from
  sponsors including General Motors, the National Science
  Foundation and the US Department of Transportation.

  Use of this software is strictly governed by agreements
  between Carnegie Mellon University and various sponsors.

  Users of this software must also be fully aware and agree
  that Carnegie Mellon does not guarantee the correct
  functioning of this software in any system. Carnegie
  Mellon or any of its affiliates will not be liable for any
  damage and/or penalties resulting from the use of this
  software. Any user of this software takes complete
  responsibility for the use of any software or design.
 **********************************************************/

/**
 * @file    Generic/Output/Channel/Impl.h
 * @author  Tugrul Galatali
 * @date    09/13/2006
 *
 * @author  Junsung Kim
 * @date    12/16/2012
 * @attention A function chmod() is added to support fault-tolerance.
 *
 * @attention Copyright (c) 2006
 * @attention Carnegie Mellon University
 * @attention All rights reserved.
 */
#ifndef _GENERIC_OUTPUT_CHANNEL_IMPL_H_
#define _GENERIC_OUTPUT_CHANNEL_IMPL_H_

#include <Comms/Access.h>
#include <cstdint>
#include <interfaces/Generic/Output/Abstract.h>

template<typename T>
class GenericOutputChannel : public GenericOutput<T>
{
public:
    GenericOutputChannel(ConfigSection& cs, const task::Access& acc);
    ~GenericOutputChannel() override;

    bool publishV(const T& val, std::vector<av::latency::Pred>&& predecessors) override;

    bool initialize() override;
    void cleanup() override;
    void chmod(const task::Access& acc) override;

private:
    static uint32_t generatePublisherId(const std::string& channelName);

private:
    uint32_t publisherId_{0};
    uint32_t messageCounter_{0};
};

#include "interfaces/Generic/Output/Channel/Impl.def.h"

#endif
