/*!
 ***********************************************************
 * @file AVLogWriter.cc
 *
 * @brief file implementing AVLogWriter class
 *
 * This class will output LogEvent objects to avlog.
 *
 * @section License
 *
 * Copyright (c) 2021- Motional, Inc. All rights reserved.
 *
 * @section changeLogs Change Logs
 *
 ***********************************************************
 */

#include <log/AVLogWriter.h>

using namespace logger;

AVLogWriter::AVLogWriter(Priority::PriorityLevel threshold)
    : LogWriter(threshold)
{}

void AVLogWriter::write_(const LogEvent& event)
{
    // In order to get the text of the message to be included in every entry in the structured log,
    // it needs to be treated as an argument rather than as the format string itself.
    const avlog::LogLevel level = translateLogPriorityLevel(event.getPriorityLevel());
    avlog::details::log(level, event.getFile(), event.getLine(), "", "{}", event.message());
}

avlog::LogLevel AVLogWriter::translateLogPriorityLevel(Priority::PriorityLevel inLevel)
{
    switch (inLevel) {
        case Priority::PriorityLevel::emerg:
            return avlog::LogLevel::critical;
            break;
        case Priority::PriorityLevel::fatal:
            return avlog::LogLevel::critical;
            break;
        case Priority::PriorityLevel::alert:
            return avlog::LogLevel::critical;
            break;
        case Priority::PriorityLevel::crit:
            return avlog::LogLevel::critical;
            break;
        case Priority::PriorityLevel::error:
            return avlog::LogLevel::err;
            break;
        case Priority::PriorityLevel::warn:
            return avlog::LogLevel::warn;
            break;
        case Priority::PriorityLevel::notice:
            return avlog::LogLevel::info;
            break;
        case Priority::PriorityLevel::info:
            return avlog::LogLevel::info;
            break;
        case Priority::PriorityLevel::debug:
            return avlog::LogLevel::debug;
            break;
        case Priority::PriorityLevel::none:
        default:
            return avlog::LogLevel::off;
            break;
    }

    return avlog::LogLevel::off;
}
