/*!
 ***********************************************************
 * @file FileWriter.h
 *
 * @brief file defining FileWriter class (Incomplete/Obsolete)
 *
 * A writer that outputs log events to file.
 *
 * This class writes LogEvent objects to file.  If the file exists, it is
 * overwritten.  Each LogEvent is written to a separate line in the file.
 *
 *
 * @section License
 *
 * Copyright (c) 2015- Delphi Automotive. All rights
 * reserved.
 *
 * @section changeLogs Change Logs
 *
 ***********************************************************
 */

/**********************************************************
  Copyright (c) 2014-2015  Ottomatika, Inc.
 **********************************************************/

/**********************************************************
  Copyright (c) 2010-2013 Carnegie Mellon University,
  All rights reserved.

  This source code was developed in part with support from
  sponsors including General Motors, the National Science
  Foundation and the US Department of Transportation.

  Use of this software is strictly governed by agreements
  between Carnegie Mellon University and various sponsors.

  Users of this software must also be fully aware and agree
  that Carnegie Mellon does not guarantee the correct
  functioning of this software in any system. Carnegie
  Mellon or any of its affiliates will not be liable for any
  damage and/or penalties resulting from the use of this
  software. Any user of this software takes complete
  responsibility for the use of any software or design.
 **********************************************************/

/*
 ***********************************************************
 * FileWriter.h
 * Bryan Salesky
 * 02/28/2006
 *
 * Copyright (C) 2006 CMU. All rights reserved.
 ***********************************************************
 */
#ifndef _FILE_WRITER_H_
#define _FILE_WRITER_H_

#include "LogWriter.h"

/*
 * Note: Disk space is never checked as log data written.  A natural
 * addition to this class would be to:
 *   a) log data up to some maximum size, then start a new file with a
 *      different name (such that previous files are saved).
 *   b) log data up to some maximum size, then recycle the file and start
 *      overwriting data at the beginnning of the file.
 *   c) log data up to some maximum size, then start a new file with the
 *      same name and overwrite the previous data.
 *   d) log data up to some maximum, then quit logging.
 *   e) log data up to some maximum size, the start writing to a new file
 *      until some total disk usage is consumed by all the log files.
 */

namespace logger {

class FileWriter : public LogWriter
{
public:
    /*!
     * \brief The default constructor
     */
    FileWriter();

    /*!
     * \brief The destructor
     */
    virtual ~FileWriter();

    /*!
     * \brief This method writes the event to a file.
     * \param event The event to write.
     */
    virtual void write_(const LogEvent& event);

protected:
private:
    /*!
     * \brief The copy constructor
     */
    FileWriter(const FileWriter&);

    /*!
     * \brief The assignment operator
     */
    void operator=(const FileWriter&);
};

} // namespace logger

#endif
