/*!
 ***********************************************************
 * @file LogWriter.cc
 *
 * @brief file implementing LogWriter class
 *
 * A base class LogWriter.
 *
 * This class is inherited by all derived LogWriter objects.  It is an
 * abstract class that required derived LogWriter's to implement the
 * write_() method.
 *
 * @section License
 *
 * Copyright (c) 2015- Delphi Automotive. All rights
 * reserved.
 *
 * @section changeLogs Change Logs
 *
 ***********************************************************
 */

/**********************************************************
  Copyright (c) 2014-2015 Ottomatika, Inc.
 **********************************************************/

/**********************************************************
  Copyright (c) 2010-2013 Carnegie Mellon University,
  All rights reserved.

  This source code was developed in part with support from
  sponsors including General Motors, the National Science
  Foundation and the US Department of Transportation.

  Use of this software is strictly governed by agreements
  between Carnegie Mellon University and various sponsors.

  Users of this software must also be fully aware and agree
  that Carnegie Mellon does not guarantee the correct
  functioning of this software in any system. Carnegie
  Mellon or any of its affiliates will not be liable for any
  damage and/or penalties resulting from the use of this
  software. Any user of this software takes complete
  responsibility for the use of any software or design.
 **********************************************************/

/*
 ***********************************************************
 * LogWriter.cc
 * Bryan Salesky
 * 02/28/2006
 *
 * Copyright (C) 2006 CMU. All rights reserved.
 ***********************************************************
 */

#include <iostream>
#include <log/LogWriter.h>

using namespace std;
using namespace logger;

LogWriter::LogWriter()
    : threshold_(Priority::debug)
{}

LogWriter::LogWriter(Priority::PriorityLevel threshold)
    : threshold_(threshold)
{}

LogWriter::~LogWriter() {}

void LogWriter::write(const LogEvent& event)
{
    if (event.getPriorityLevel() <= threshold_) {
        write_(event);
    }
}
