/*!
 ***********************************************************
 * @file OStreamWriter.cc
 *
 * @brief file implementing OStreamWriter class
 *
 * This class will output LogEvent objects to the supplied ostream.
 *
 * @section License
 *
 * Copyright (c) 2015- Delphi Automotive. All rights
 * reserved.
 *
 * @section changeLogs Change Logs
 *
 ***********************************************************
 */

/**********************************************************
  Copyright (c) 2014-2015 Ottomatika, Inc.
 **********************************************************/

/**********************************************************
  Copyright (c) 2010-2013 Carnegie Mellon University,
  All rights reserved.

  This source code was developed in part with support from
  sponsors including General Motors, the National Science
  Foundation and the US Department of Transportation.

  Use of this software is strictly governed by agreements
  between Carnegie Mellon University and various sponsors.

  Users of this software must also be fully aware and agree
  that Carnegie Mellon does not guarantee the correct
  functioning of this software in any system. Carnegie
  Mellon or any of its affiliates will not be liable for any
  damage and/or penalties resulting from the use of this
  software. Any user of this software takes complete
  responsibility for the use of any software or design.
 **********************************************************/

/*
 ***********************************************************
 * @file      OStreamWriter.cc
 * @author    Bryan Salesky
 * @date      02/28/2006
 *
 * Copyright (C) 2006 CMU. All rights reserved.
 ***********************************************************
 */
#include <log/OStreamWriter.h>

using namespace std;
using namespace logger;

OStreamWriter::OStreamWriter(ostream& out)
    : LogWriter()
    , ostr_(&out)
{}

OStreamWriter::OStreamWriter(ostream& out, Priority::PriorityLevel threshold)
    : LogWriter(threshold)
    , ostr_(&out)
{}

OStreamWriter::~OStreamWriter() {}

void OStreamWriter::write_(const LogEvent& event)
{
    if (ostr_) {
        event.toStream(*ostr_);
    }
}
