/*!
 ***********************************************************
 * @file Priority.cc
 *
 * @brief file implementing Priority class
 *
 * This class defines the allowable priorities.
 *
 * Each event has a particular PriorityLevel associated with it.  This
 * class defines these priorities, and also contains mask that helps to
 * keep track of which priorities are 'enabled' or 'disabled'.
 *
 * @section License
 *
 * Copyright (c) 2015- Delphi Automotive. All rights
 * reserved.
 *
 * @section changeLogs Change Logs
 *
 * @author Yunpeng Xu (yunpeng.xu@aptiv.com)
 * @date 03/16/2018
 * @remark port log part to QNX.
 *         - remove boost dependency
 *
 ***********************************************************
 */

/**********************************************************
  Copyright (c) 2014-2015 Ottomatika, Inc.
 **********************************************************/

/**********************************************************
  Copyright (c) 2010-2013 Carnegie Mellon University,
  All rights reserved.

  This source code was developed in part with support from
  sponsors including General Motors, the National Science
  Foundation and the US Department of Transportation.

  Use of this software is strictly governed by agreements
  between Carnegie Mellon University and various sponsors.

  Users of this software must also be fully aware and agree
  that Carnegie Mellon does not guarantee the correct
  functioning of this software in any system. Carnegie
  Mellon or any of its affiliates will not be liable for any
  damage and/or penalties resulting from the use of this
  software. Any user of this software takes complete
  responsibility for the use of any software or design.
 **********************************************************/

/*
 ***********************************************************
 * Priority.cc
 * Bryan Salesky
 * 02/28/2006
 *
 * Copyright (C) 2006 CMU. All rights reserved.
 ***********************************************************
 */

#include <algorithm>
#include <iostream>
#include <string>

#include <log/Priority.h>

using namespace std;
using namespace logger;

Priority::Priority()
    : pri_mask_(Priority::emerg)
{}

Priority::Priority(const Priority& copy)
    : pri_mask_(copy.pri_mask_)
{}

Priority& Priority::operator=(const Priority& copy)
{
    if (this != &copy) {
        pri_mask_ = copy.pri_mask_;
    }
    return *this;
}

Priority::~Priority() {}

const char* Priority::priorityLevelAsString(const PriorityLevel& level)
{
    switch (level) { // embedded returns follow ...
        // keep to 6 characters or less
        case emerg:
            return "EMERG";
            break;
        case fatal:
            return "FATAL";
            break;
        case alert:
            return "ALERT";
            break;
        case crit:
            return "CRIT";
            break;
        case error:
            return "ERROR";
            break;
        case warn:
            return "WARN";
            break;
        case notice:
            return "NOTICE";
            break;
        case info:
            return "INFO";
            break;
        case debug:
            return "DEBUG";
            break;
        case none:
            return "NONE";
            break;
        default:
            break;
    }
    return "UNKNOWN";
}

Priority::PriorityLevel Priority::stringAsPriorityLevel(const string& typeString)
{
    string typeStringU;
    transform(typeString.begin(), typeString.end(), back_inserter(typeStringU), ::toupper);

    if (typeStringU == "EMERG") {
        return emerg;
    }
    if (typeStringU == "FATAL") {
        return fatal;
    }
    if (typeStringU == "ALERT") {
        return alert;
    }
    if (typeStringU == "CRIT") {
        return crit;
    }
    if (typeStringU == "ERROR") {
        return error;
    }
    if (typeStringU == "WARN") {
        return warn;
    }
    if (typeStringU == "NOTICE") {
        return notice;
    }
    if (typeStringU == "INFO") {
        return info;
    }
    if (typeStringU == "DEBUG") {
        return debug;
    }
    if (typeStringU == "NONE") {
        return none;
    }

    return none;
}
