/*!
 ***********************************************************
 * @file AVLogWriter.h
 *
 * @brief file defining AVLogWriter class
 *
 * This class will output LogEvent objects to the avlog library
 * from avutils.
 *
 * @section License
 *
 * Copyright (c) 2021- Motional, Inc. All rights reserved.
 *
 * @section changeLogs Change Logs
 *
 ***********************************************************
 */

#ifndef _AVLOG_WRITER_H_
#define _AVLOG_WRITER_H_

#include "LogWriter.h"

#include "lib/avlog/avlog.hpp"

namespace logger {

/// A LogWriter that outputs its messages to the avlog system.
class AVLogWriter : public LogWriter
{
public:
    /// Constructor that takes a priority level.
    /// @param threshold The priority level to log (defaulting to debug level).
    AVLogWriter(Priority::PriorityLevel threshold = Priority::PriorityLevel::debug);

    AVLogWriter(const AVLogWriter&) = delete;
    AVLogWriter(AVLogWriter&&) = delete;

    ~AVLogWriter() override = default;

    void operator=(const AVLogWriter&) = delete;
    void operator=(AVLogWriter&&) = delete;

    /// Helper function to translate logger priority levels.
    static avlog::LogLevel translateLogPriorityLevel(Priority::PriorityLevel inLevel);

protected:
    //// The write method that outputs a LogEvent to an ostream.
    /// @param event  The event to write to an ostream.
    void write_(const LogEvent& event) override;
};

} // namespace logger

#endif
