/*!
 ***********************************************************
 * @file OStreamWriter.h
 *
 * @brief file defining OStreamWriter class
 *
 * This class will output LogEvent objects to the supplied
 * ostream.
 *
 * @section License
 *
 * Copyright (c) 2015- Delphi Automotive. All rights
 * reserved.
 *
 * @section changeLogs Change Logs
 *
 ***********************************************************
 */

/**********************************************************
  Copyright (c) 2014-2015 Ottomatika, Inc.
 **********************************************************/

/**********************************************************
  Copyright (c) 2010-2013 Carnegie Mellon University,
  All rights reserved.

  This source code was developed in part with support from
  sponsors including General Motors, the National Science
  Foundation and the US Department of Transportation.

  Use of this software is strictly governed by agreements
  between Carnegie Mellon University and various sponsors.

  Users of this software must also be fully aware and agree
  that Carnegie Mellon does not guarantee the correct
  functioning of this software in any system. Carnegie
  Mellon or any of its affiliates will not be liable for any
  damage and/or penalties resulting from the use of this
  software. Any user of this software takes complete
  responsibility for the use of any software or design.
 **********************************************************/

/*
 ***********************************************************
 * OStreamWriter.h
 * Bryan Salesky
 * 02/28/2006
 *
 * Copyright (C) 2006 CMU. All rights reserved.
 ***********************************************************
 */
#ifndef _OSTREAM_WRITER_H_
#define _OSTREAM_WRITER_H_

#include "LogWriter.h"
#include <iostream>

namespace logger {

/*!
 * @class OStreamWriter
 * @brief An output stream writer.
 *
 * Note that flush is *not* called each time an object is written to the
 * ostream.
 */
class OStreamWriter : public LogWriter
{
public:
    /*!
     * @brief The constructor that takes an output stream.
     *
     * @param out The output stream to write to.
     */
    OStreamWriter(std::ostream& out);

    /*!
     * @brief The constructor that takes an output stream, and a PriorityLevel.
     *
     * @param out The output stream to write to.
     */
    OStreamWriter(std::ostream& out, Priority::PriorityLevel threshold);

    /**
     * @brief The virtual destructor
     */
    virtual ~OStreamWriter();

protected:
    /*!
     * @brief The write method that outputs a LogEvent to an ostream.
     *
     * @param event  The event to write to an ostream.
     */
    virtual void write_(const LogEvent& event);

private:
    std::ostream* ostr_; ///< A pointer to Output stream

    /**
     * @brief The Copy constructor
     */
    OStreamWriter(const OStreamWriter&);

    /**
     * @brief The assignment operator
     */
    void operator=(const OStreamWriter&);
};

} // namespace logger

#endif
