/**
 * \copyright
 * Copyright (c) 2020- Motional. All rights reserved.
 * NOTICE: Removing this heading is prohibited.
 * All informatTimeion contained herein is, and remains the property of Motional, its affiliates, and its suppliers (if
 * any). The intellectual and technical concepts contained herein are proprietary to Motional, its affiliates, and its
 * suppliers (if any) and may be covered by U.S. and Foreign Patents, patents in process, and are protected by trade
 * secrets or copyright law. Dissemination of this informatTimeion or reproduction of this material is strictly
 * forbidden unless prior written permission is obtained from Motional.
 *
 * \brief Verfies LogEvent functionality
 */

#include <gtest/gtest.h>
#include <iomanip>
#include <iostream>
#include <log/LogEvent.h>
#include <sstream>

using namespace logger;
using namespace std;
using namespace chrono;

TEST(LogEvent, formatTimeAndParseTime_handlesDefaultTimePoint)
{
    LogEvent::ChronoTimePoint timePoint;
    EXPECT_EQ(LogEvent::parseTime((LogEvent::formatTime(timePoint))), timePoint);
}

TEST(LogEvent, formatTimeAndParse_handlesAnyTimePoint)
{
    LogEvent::ChronoTimePoint timePoint = time_point_cast<milliseconds>(std::chrono::system_clock::now());
    EXPECT_EQ(LogEvent::parseTime((LogEvent::formatTime(timePoint))), timePoint);
}

TEST(LogEvent, parseTime_returnsDefaultTimePointForInvalidFormats)
{
    LogEvent::ChronoTimePoint timePoint;
    EXPECT_EQ(LogEvent::parseTime("invalid"), timePoint);
    EXPECT_EQ(LogEvent::parseTime("2000/01/01 00:00:00.000"), timePoint);
    EXPECT_EQ(LogEvent::parseTime("2000-01-01 24:00:00.000"), timePoint);
    EXPECT_EQ(LogEvent::parseTime("2000-01-01 23:60:00.000"), timePoint);
    EXPECT_EQ(LogEvent::parseTime("2000-13-01 00:00:00.000"), timePoint);
    EXPECT_EQ(LogEvent::parseTime("2000-01-32 00:00:00.000"), timePoint);
}

TEST(LogEvent, parseTime_handlesFormatsWithoutMillisecondPrecision)
{
    time_t currentTime = std::time(nullptr);
    LogEvent::ChronoTimePoint timePoint = system_clock::from_time_t(currentTime);
    std::ostringstream dateTime;
    dateTime << put_time(gmtime(&currentTime), "%F %T");

    EXPECT_EQ(LogEvent::parseTime(dateTime.str()), timePoint);
}

TEST(LogEvent, toStreamOutputsCorrectlyFormattedDateTime)
{
    // Verify log message timestamp is parsable.
    std::ostringstream logOutput;
    LogEvent event;
    event.toStream(logOutput);
    EXPECT_NE(LogEvent::parseTime(logOutput.str()), LogEvent::ChronoTimePoint{});
}

int main(int argc, char** argv)
{
    ::testing::InitGoogleTest(&argc, argv);
    return RUN_ALL_TESTS();
}
