/*!
 ***********************************************************
 * @file va_str.cc
 *
 * @brief  An abstraction around vsnprintf and its variants.
 *
 * @section License
 *
 * Copyright (c) 2015- Delphi Automotive. All rights
 * reserved.
 *
 * @section changeLogs Change Logs
 *
 ***********************************************************
 */

/**********************************************************
  Copyright (c) 2014-2015 Ottomatika, Inc.
 **********************************************************/

/**********************************************************
  Copyright (c) 2010-2013 Carnegie Mellon University,
  All rights reserved.

  This source code was developed in part with support from
  sponsors including General Motors, the National Science
  Foundation and the US Department of Transportation.

  Use of this software is strictly governed by agreements
  between Carnegie Mellon University and various sponsors.

  Users of this software must also be fully aware and agree
  that Carnegie Mellon does not guarantee the correct
  functioning of this software in any system. Carnegie
  Mellon or any of its affiliates will not be liable for any
  damage and/or penalties resulting from the use of this
  software. Any user of this software takes complete
  responsibility for the use of any software or design.
 **********************************************************/

/*
 ***********************************************************
 * va_str.cc
 * Bryan Salesky
 * 02/28/2006
 *
 * Copyright (C) 2006 CMU. All rights reserved.
 ***********************************************************
 */

#include "va_str.h"

#include <stdio.h>
#include <stdlib.h>

/*!
 *****************************************************************************
 * @brief An abstraction around vsnprintf and its variants.
 *
 * @param format The format string.
 * @param args   The variable args list.
 *
 * @return A string that has been formatted using the variable args list.
 *****************************************************************************
 */

std::string va_str(const char* format, va_list args)
{
    va_list argsCopy;
    va_copy(argsCopy, args);
    const int length = vsnprintf(nullptr, 0, format, args);
    if (length < 0) {
        va_end(argsCopy);
        return {};
    }
    std::string buf(static_cast<size_t>(length + 1), 0);
    vsnprintf(&buf.front(), buf.size(), format, argsCopy);
    buf.pop_back();
    va_end(argsCopy);
    return buf;
}
