
/*!
 ***********************************************************************************************************************
 * APTIV PRIVILEGED AND CONFIDENTIAL
 *
 * @file TestRecClipperRoadLane.h
 * @product Common / RecGeometry
 * @version TODO <Release Number>
 * @released TODO <date>
 * @updated May 28, 2018
 * @brief Unit Tests for RecClipper with polygons created from Roadlane objects.
 *
 * @section License
 *
 * Copyright (c) 2018- Aptiv. All rights reserved.
 * NOTE: Removing this heading is prohibited.
 *
 ***********************************************************************************************************************
 */

#include "RoadLaneTestUtil.h"
#include "test_helper.h"

#include <iostream> // cout
#include <vector>

#include <gtest/gtest.h>

#include <RoadWorldModel/WorldId.h> // WorldIdVector
#include <recgeometry/recClipper.h>
#include <recgeometry/recGeometry.h>

class RecClipperRoadLaneTest : public testing::Test
{
protected:
    auto rwm_ = roadWorldModel::RoadWorldModel::getInstance();
    RoadLaneTestUtil util_;

    virtual void SetUp() { rwm_.loadImpl(std::move(roadWorldModel::RoadWorldModelImpl::create(&util_))); }
};

// #1
// Create a BoundingPolygon for a RoadLane and
// check that it has CW winding order.
TEST_F(RecClipperRoadLaneTest, GetBoundingPolygonAlong_WindingOrder1)
{
    // RoadLane north-to-south
    const roadWorldModel::WorldId segWId1(1, roadWorldModel::WorldId::RoadSegment);
    const RNDFId rlaneid1(1, 1);
    const roadWorldModel::WorldId laneWId1(rlaneid1, roadWorldModel::WorldId::RoadLane);
    std::vector<roadWorldModel::WorldId> waypointIds1;
    {
        const roadWorldModel::Waypoint* waypt1 =
            util_.createTestWaypoint(rwm_, segWId1, laneWId1, 1, 1, 1, 10.0, 30.0); // Waypoint ID and (x,y)
        const roadWorldModel::Waypoint* waypt2 = util_.createTestWaypoint(rwm_, segWId1, laneWId1, 1, 1, 5, 10.0, 26.0);
        const roadWorldModel::Waypoint* waypt3 =
            util_.createTestWaypoint(rwm_, segWId1, laneWId1, 1, 1, 10, 10.0, 22.0);
        const roadWorldModel::Waypoint* waypt4 =
            util_.createTestWaypoint(rwm_, segWId1, laneWId1, 1, 1, 15, 12.0, 18.0);
        const roadWorldModel::Waypoint* waypt5 =
            util_.createTestWaypoint(rwm_, segWId1, laneWId1, 1, 1, 20, 12.0, 14.0);
        const roadWorldModel::Waypoint* waypt6 =
            util_.createTestWaypoint(rwm_, segWId1, laneWId1, 1, 1, 25, 12.0, 10.0);
        waypointIds1 = {waypt1->getWorldId(),
                        waypt2->getWorldId(),
                        waypt3->getWorldId(),
                        waypt4->getWorldId(),
                        waypt5->getWorldId(),
                        waypt6->getWorldId()};
    }
    roadWorldModel::RoadLane* roadLane1 = util_.createTestRoadLane(rwm_, segWId1, laneWId1, waypointIds1, 1);

    RecPolygon2D road_lane_polygon1;
    const bool res1 = roadLane1->getBoundingPolygonAlong(0.0,
                                                         roadLane1->getLength(),
                                                         road_lane_polygon1,
                                                         0.0); // padding
    ASSERT_EQ(res1, true);

    ASSERT_EQ(RecClipper::polygonCW(road_lane_polygon1), true);
}

// #2
// Create a BoundingPolygon for a RoadLane and
// check that it has CW winding order.
TEST_F(RecClipperRoadLaneTest, GetBoundingPolygonAlong_WindingOrder2)
{
    // RoadLane south-to-north
    const roadWorldModel::WorldId segWId1(1, roadWorldModel::WorldId::RoadSegment);
    const RNDFId rlaneid1(1, 1);
    const roadWorldModel::WorldId laneWId1(rlaneid1, roadWorldModel::WorldId::RoadLane);
    std::vector<roadWorldModel::WorldId> waypointIds1;
    {
        const roadWorldModel::Waypoint* waypt1 =
            util_.createTestWaypoint(rwm_, segWId1, laneWId1, 1, 1, 1, 10.0, 10.0); // Waypoint ID and (x,y)
        const roadWorldModel::Waypoint* waypt2 = util_.createTestWaypoint(rwm_, segWId1, laneWId1, 1, 1, 5, 10.0, 14.0);
        const roadWorldModel::Waypoint* waypt3 =
            util_.createTestWaypoint(rwm_, segWId1, laneWId1, 1, 1, 10, 10.0, 18.0);
        const roadWorldModel::Waypoint* waypt4 =
            util_.createTestWaypoint(rwm_, segWId1, laneWId1, 1, 1, 15, 12.0, 20.0);
        const roadWorldModel::Waypoint* waypt5 =
            util_.createTestWaypoint(rwm_, segWId1, laneWId1, 1, 1, 20, 12.0, 24.0);
        const roadWorldModel::Waypoint* waypt6 =
            util_.createTestWaypoint(rwm_, segWId1, laneWId1, 1, 1, 25, 12.0, 28.0);
        waypointIds1 = {waypt1->getWorldId(),
                        waypt2->getWorldId(),
                        waypt3->getWorldId(),
                        waypt4->getWorldId(),
                        waypt5->getWorldId(),
                        waypt6->getWorldId()};
    }
    roadWorldModel::RoadLane* roadLane1 = util_.createTestRoadLane(rwm_, segWId1, laneWId1, waypointIds1, 1);

    RecPolygon2D road_lane_polygon1;
    const bool res1 = roadLane1->getBoundingPolygonAlong(0.0,
                                                         roadLane1->getLength(),
                                                         road_lane_polygon1,
                                                         0.0); // padding
    ASSERT_EQ(res1, true);

    ASSERT_EQ(RecClipper::polygonCW(road_lane_polygon1), true);
}

// #3
// Create a BoundingPolygon for a RoadLane and
// check that it has CW winding order.
TEST_F(RecClipperRoadLaneTest, GetBoundingPolygonAlong_WindingOrder3)
{
    // RoadLane west-to-east
    const roadWorldModel::WorldId segWId1(1, roadWorldModel::WorldId::RoadSegment);
    const RNDFId rlaneid1(1, 1);
    const roadWorldModel::WorldId laneWId1(rlaneid1, roadWorldModel::WorldId::RoadLane);
    std::vector<roadWorldModel::WorldId> waypointIds1;
    {
        const roadWorldModel::Waypoint* waypt1 =
            util_.createTestWaypoint(rwm_, segWId1, laneWId1, 1, 1, 1, 10.0, 10.0); // Waypoint ID and (x,y)
        const roadWorldModel::Waypoint* waypt2 = util_.createTestWaypoint(rwm_, segWId1, laneWId1, 1, 1, 5, 14.0, 10.0);
        const roadWorldModel::Waypoint* waypt3 =
            util_.createTestWaypoint(rwm_, segWId1, laneWId1, 1, 1, 10, 18.0, 10.0);
        const roadWorldModel::Waypoint* waypt4 =
            util_.createTestWaypoint(rwm_, segWId1, laneWId1, 1, 1, 15, 22.0, 14.0);
        const roadWorldModel::Waypoint* waypt5 =
            util_.createTestWaypoint(rwm_, segWId1, laneWId1, 1, 1, 20, 26.0, 14.0);
        const roadWorldModel::Waypoint* waypt6 =
            util_.createTestWaypoint(rwm_, segWId1, laneWId1, 1, 1, 25, 30.0, 14.0);
        waypointIds1 = {waypt1->getWorldId(),
                        waypt2->getWorldId(),
                        waypt3->getWorldId(),
                        waypt4->getWorldId(),
                        waypt5->getWorldId(),
                        waypt6->getWorldId()};
    }
    roadWorldModel::RoadLane* roadLane1 = util_.createTestRoadLane(rwm_, segWId1, laneWId1, waypointIds1, 1);

    RecPolygon2D road_lane_polygon1;
    const bool res1 = roadLane1->getBoundingPolygonAlong(0.0,
                                                         roadLane1->getLength(),
                                                         road_lane_polygon1,
                                                         0.0); // padding
    ASSERT_EQ(res1, true);

    ASSERT_EQ(RecClipper::polygonCW(road_lane_polygon1), true);
}

// #4
// Create a BoundingPolygon for a RoadLane and
// check that it has CW winding order.
TEST_F(RecClipperRoadLaneTest, GetBoundingPolygonAlong_WindingOrder4)
{
    // RoadLane east-to-west
    const roadWorldModel::WorldId segWId1(1, roadWorldModel::WorldId::RoadSegment);
    const RNDFId rlaneid1(1, 1);
    const roadWorldModel::WorldId laneWId1(rlaneid1, roadWorldModel::WorldId::RoadLane);
    std::vector<roadWorldModel::WorldId> waypointIds1;
    {
        const roadWorldModel::Waypoint* waypt1 =
            util_.createTestWaypoint(rwm_, segWId1, laneWId1, 1, 1, 1, 30.0, 10.0); // Waypoint ID and (x,y)
        const roadWorldModel::Waypoint* waypt2 = util_.createTestWaypoint(rwm_, segWId1, laneWId1, 1, 1, 5, 26.0, 10.0);
        const roadWorldModel::Waypoint* waypt3 =
            util_.createTestWaypoint(rwm_, segWId1, laneWId1, 1, 1, 10, 22.0, 10.0);
        const roadWorldModel::Waypoint* waypt4 =
            util_.createTestWaypoint(rwm_, segWId1, laneWId1, 1, 1, 15, 18.0, 14.0);
        const roadWorldModel::Waypoint* waypt5 =
            util_.createTestWaypoint(rwm_, segWId1, laneWId1, 1, 1, 20, 14.0, 14.0);
        const roadWorldModel::Waypoint* waypt6 =
            util_.createTestWaypoint(rwm_, segWId1, laneWId1, 1, 1, 25, 10.0, 14.0);
        waypointIds1 = {waypt1->getWorldId(),
                        waypt2->getWorldId(),
                        waypt3->getWorldId(),
                        waypt4->getWorldId(),
                        waypt5->getWorldId(),
                        waypt6->getWorldId()};
    }
    roadWorldModel::RoadLane* roadLane1 = util_.createTestRoadLane(rwm_, segWId1, laneWId1, waypointIds1, 1);

    RecPolygon2D road_lane_polygon1;
    const bool res1 = roadLane1->getBoundingPolygonAlong(0.0,
                                                         roadLane1->getLength(),
                                                         road_lane_polygon1,
                                                         0.0); // padding
    ASSERT_EQ(res1, true);

    ASSERT_EQ(RecClipper::polygonCW(road_lane_polygon1), true);
}

// #5
// Compute the binary union of 2 overlapping RoadLane polygons
TEST_F(RecClipperRoadLaneTest, BinaryUnion1_TwoOverlappingRoadLanes)
{
    // RoadLane south-to-north
    const roadWorldModel::WorldId segWId1(1, roadWorldModel::WorldId::RoadSegment);
    const RNDFId rlaneid1(1, 1);
    const roadWorldModel::WorldId laneWId1(rlaneid1, roadWorldModel::WorldId::RoadLane);
    std::vector<roadWorldModel::WorldId> waypointIds1;
    {
        const roadWorldModel::Waypoint* waypt1 =
            util_.createTestWaypoint(rwm_, segWId1, laneWId1, 1, 1, 1, 10.0, 10.0); // Waypoint ID and (x,y)
        const roadWorldModel::Waypoint* waypt2 = util_.createTestWaypoint(rwm_, segWId1, laneWId1, 1, 1, 5, 10.0, 14.0);
        const roadWorldModel::Waypoint* waypt3 =
            util_.createTestWaypoint(rwm_, segWId1, laneWId1, 1, 1, 10, 10.0, 18.0);
        const roadWorldModel::Waypoint* waypt4 =
            util_.createTestWaypoint(rwm_, segWId1, laneWId1, 1, 1, 15, 12.0, 20.0);
        const roadWorldModel::Waypoint* waypt5 =
            util_.createTestWaypoint(rwm_, segWId1, laneWId1, 1, 1, 20, 12.0, 24.0);
        const roadWorldModel::Waypoint* waypt6 =
            util_.createTestWaypoint(rwm_, segWId1, laneWId1, 1, 1, 25, 12.0, 28.0);
        waypointIds1 = {waypt1->getWorldId(),
                        waypt2->getWorldId(),
                        waypt3->getWorldId(),
                        waypt4->getWorldId(),
                        waypt5->getWorldId(),
                        waypt6->getWorldId()};
    }
    roadWorldModel::RoadLane* roadLane1 = util_.createTestRoadLane(rwm_, segWId1, laneWId1, waypointIds1, 1);

    // RoadLane at top, west-to-east
    const roadWorldModel::WorldId segWId2(2, roadWorldModel::WorldId::RoadSegment);
    const RNDFId rlaneid2(2, 1);
    const roadWorldModel::WorldId laneWId2(rlaneid2, roadWorldModel::WorldId::RoadLane);
    std::vector<roadWorldModel::WorldId> waypointIds2;
    {
        const roadWorldModel::Waypoint* waypt1 = util_.createTestWaypoint(rwm_, segWId2, laneWId2, 2, 1, 1, 2.0, 25.0);
        const roadWorldModel::Waypoint* waypt2 = util_.createTestWaypoint(rwm_, segWId2, laneWId2, 2, 1, 5, 16.0, 25.0);

        waypointIds2 = {waypt1->getWorldId(), waypt2->getWorldId()};
    }
    roadWorldModel::RoadLane* roadLane2 = util_.createTestRoadLane(rwm_, segWId2, laneWId2, waypointIds2, 1);

    // std::vector<roadWorldModel::WorldId> lanes;
    // lanes.push_back(roadLane1->getWorldId());
    // roadWorldModel::RoadSegment* roadSegment = util_.createTestRoadSegment(rwm_, segWId, lanes, waypointIds1);

    RecPolygon2D road_lane_polygon1;
    const bool res1 = roadLane1->getBoundingPolygonAlong(0.0,
                                                         roadLane1->getLength(),
                                                         road_lane_polygon1,
                                                         0.0); // padding
    ASSERT_EQ(res1, true);
    // std::cout << "Road Lane 1: " << std::endl; printVertices(road_lane_polygon1);
    // printPolygonOrientation(road_lane_polygon1);

    RecPolygon2D road_lane_polygon2;
    const bool res2 = roadLane2->getBoundingPolygonAlong(0.0,
                                                         roadLane2->getLength(),
                                                         road_lane_polygon2,
                                                         0.0); // padding
    ASSERT_EQ(res2, true);
    // std::cout << "Road Lane 2: " << std::endl; printVertices(road_lane_polygon2);
    // printPolygonOrientation(road_lane_polygon2);

    // Check that getBoundingPolygonAlong returns polygons with CW winding order
    ASSERT_EQ(RecClipper::polygonCW(road_lane_polygon1), true);
    ASSERT_EQ(RecClipper::polygonCW(road_lane_polygon2), true);

    std::vector<RecPolygon2D> input_polygons = {road_lane_polygon1, road_lane_polygon2};

    std::vector<RecPolygon2D> output_polygons;
    RecClipper::computeUnion(input_polygons, // Vector of 'subject' polygons
                             output_polygons);

    ASSERT_EQ(output_polygons.size(), 1);
    // std::cout << "output_polygons.size() = " << output_polygons.size() << std::endl;
    // std::cout << "output: " << std::endl; printVertices(output_polygons.at(0));

    RecPolygon2D output_poly1 = output_polygons.at(0);
    ASSERT_EQ(RecClipper::polygonCW(output_poly1), true);

    std::vector<RecPoint2D> poly1_expected_vertices = {RecPoint2D(13.499219350000001, 27.951612610000002),
                                                       RecPoint2D(13.45236847, 26.5),
                                                       RecPoint2D(16, 26.5),
                                                       RecPoint2D(16, 23.5),
                                                       RecPoint2D(13.58964879, 23.5),
                                                       RecPoint2D(13.79719542, 22.08850142),
                                                       RecPoint2D(13.79255869, 21.622582749999999),
                                                       RecPoint2D(13.47940601, 19.752294840000001),
                                                       RecPoint2D(13.06066017, 18.939339820000001),
                                                       RecPoint2D(11.40012731, 17.278806960000001),
                                                       RecPoint2D(11.20519797, 16.114601759999999),
                                                       RecPoint2D(11.48404274, 14.21821351),
                                                       RecPoint2D(11.499259439999999, 14.04712857),
                                                       RecPoint2D(11.56295798, 12.020743769999999),
                                                       RecPoint2D(11.562917880000001, 11.925227810000001),
                                                       RecPoint2D(11.499219350000001, 9.9516126099999997),
                                                       RecPoint2D(8.5007806400000003, 10.048387379999999),
                                                       RecPoint2D(8.5629378099999993, 11.97424524),
                                                       RecPoint2D(8.5034432500000001, 13.866892630000001),
                                                       RecPoint2D(8.2028045699999996, 15.911498569999999),
                                                       RecPoint2D(8.2074412999999993, 16.37741724),
                                                       RecPoint2D(8.5205939799999992, 18.247705150000002),
                                                       RecPoint2D(8.9393398200000007, 19.060660169999998),
                                                       RecPoint2D(10.599872680000001, 20.721193029999998),
                                                       RecPoint2D(10.794802020000001, 21.88539823),
                                                       RecPoint2D(10.557391109999999, 23.5),
                                                       RecPoint2D(2, 23.5),
                                                       RecPoint2D(2, 26.5),
                                                       RecPoint2D(10.45080636, 26.5),
                                                       RecPoint2D(10.50078064, 28.048387380000001),
                                                       RecPoint2D(13.499219350000001, 27.951612610000002)};
    RecPolygon2D expected_polygon1(poly1_expected_vertices);
    ASSERT_CIRCULAR_EQ(output_poly1, expected_polygon1);
}

int main(int argc, char** argv)
{
    ::testing::InitGoogleTest(&argc, argv);
    return RUN_ALL_TESTS();
}
