/**
 * \copyright
 * Copyright (c) 2021- Motional. All rights reserved.
 * NOTICE: Removing this heading is prohibited.
 * All information contained herein is, and remains the property of Motional and its suppliers, if any.
 * The intellectual and technical concepts contained herein are proprietary to Motional and its suppliers and may be
 * covered by U.S. and Foreign Patents, patents in process, and are protected by trade secret or copyright law.
 * Dissemination of this information or reproduction of this material is strictly forbidden unless prior written
 * permission is obtained from Motional.
 *
 */

#include <fstream>

#include <sms/SmsLogger.hpp>
#include <sms/TraceClient.hpp>

namespace av {
namespace sms {

std::unique_ptr<SmsLogger> TraceClient::s_defaultPublisher{};

void TraceClient::initialize(std::unique_ptr<SmsLogger> publisher, const std::string& name, uint32_t limit)
{
    s_defaultPublisher = std::move(publisher);
    if (s_defaultPublisher) {
        s_defaultPublisher->setProcessName(name);
        s_defaultPublisher->setRateLimit(limit);
    }
}

TraceClient::TraceClient(av::sms::TraceEvent& message)
    : TraceClient(message, s_defaultPublisher.get())
{}

TraceClient::TraceClient(av::sms::TraceEvent& message, SmsLogger* publisher)
    : message_{message}
    , logger_(publisher)
{
    initializeMessage();
}

void TraceClient::initializeMessage()
{
    static int32_t processID = getpid();
    thread_local auto threadID = static_cast<int>(syscall(SYS_gettid)); // NOLINT(cppcoreguidelines-pro-type-vararg)

    message_.processId = processID;
    message_.threadId = threadID;
    message_.startTime = ::av::AvClock::now();
}

uint32_t TraceClient::rateLimit()
{
    return s_defaultPublisher ? s_defaultPublisher->getRateLimit() : 0;
}

TraceClient::~TraceClient()
{
    end();
}

void TraceClient::end(bool trackEndTime)
{
    if (!ended_ && logger_ != nullptr) {
        if (!logger_->checkRateLimit(message_.startTime)) {
            return;
        }
        message_.processName = logger_->getProcessName();
        if (trackEndTime) {
            message_.endTime = ::av::AvClock::now();
        } else {
            message_.endTime = message_.startTime;
        }
        logger_->log(message_);
    }
    ended_ = true;
}

} // namespace sms
} // namespace av
