/**
 * @copyright
 * Copyright (c) 2021- Motional. All rights reserved.
 * NOTICE: Removing this heading is prohibited.
 * All information contained herein is, and remains the property of Motional and its suppliers, if any.
 * The intellectual and technical concepts contained herein are proprietary to Motional and its suppliers and may be
 * covered by U.S. and Foreign Patents, patents in process, and are protected by trade secret or copyright law.
 * Dissemination of this information or reproduction of this material is strictly forbidden unless prior written
 * permission is obtained from Motional.
 *
 */

#ifndef TRACE_EVENT_VALUE_HPP
#define TRACE_EVENT_VALUE_HPP

#include "sms/TraceEvent.hpp"

namespace av {
namespace sms {

/// @brief TraceEventValue struct Convenience structure for retrieving the correct value from \ref av::sms::TraceEvent.
/// TODO: Maybe overkill, but this approach generates a compile error when attempting to retrieve an unsupported
/// type.
template<typename T>
struct TraceEventValue
{
    static void set(av::sms::TraceEvent& event, const T value);
};

template<>
inline void TraceEventValue<std::int64_t>::set(av::sms::TraceEvent& event, const std::int64_t value)
{
    event.valueType = av::sms::ValueType::Int64;
    event.intValue = value;
}
template<>
inline void TraceEventValue<std::uint64_t>::set(av::sms::TraceEvent& event, const std::uint64_t value)
{
    event.valueType = av::sms::ValueType::Uint64;
    event.uIntValue = value;
}
template<>
inline void TraceEventValue<float>::set(av::sms::TraceEvent& event, const float value)
{
    event.valueType = av::sms::ValueType::Float32;
    event.floatValue = value;
}
template<>
inline void TraceEventValue<double>::set(av::sms::TraceEvent& event, const double value)
{
    event.valueType = av::sms::ValueType::Float64;
    event.doubleValue = value;
}
template<>
inline void TraceEventValue<std::string>::set(av::sms::TraceEvent& event, const std::string value)
{
    event.valueType = av::sms::ValueType::String;
    event.stringValue = value;
}
template<>
inline void TraceEventValue<bool>::set(av::sms::TraceEvent& event, const bool value)
{
    event.valueType = av::sms::ValueType::Boolean;
    event.boolValue = value;
}

} // namespace sms
} // namespace av

#endif // TRACE_EVENT_VALUE_HPP
