/**
 * \copyright
 * Copyright (c) 2021- Motional. All rights reserved.
 * NOTICE: Removing this heading is prohibited.
 * All information contained herein is, and remains the property of Motional and its suppliers, if any.
 * The intellectual and technical concepts contained herein are proprietary to Motional and its suppliers and may be
 * covered by U.S. and Foreign Patents, patents in process, and are protected by trade secret or copyright law.
 * Dissemination of this information or reproduction of this material is strictly forbidden unless prior written
 * permission is obtained from Motional.
 *
 */

#include "TraceEventFaker.hpp"
#include "lib/base/enum/Trait.hpp"

#include <sms/Sms.hpp>
#include <sms/SmsLogger.hpp>

#include "lib/avlog/avlog.hpp"

enum class CustomEnumLabels
{
    Foo,
    Bar,
    Baz
};

template<>
struct base::EnumTrait<CustomEnumLabels>
{
    static constexpr std::array namedEnumerators = {
        base::Enumerator{CustomEnumLabels::Foo, "Foo"},
        base::Enumerator{CustomEnumLabels::Bar, "Bar"},
        base::Enumerator{CustomEnumLabels::Baz, "Baz"},
    };
};

task::Task* task::Task::get()
{
    static TraceEventFaker task(std::string("TraceEventFaker"));
    return &task;
}

TraceEventFaker::TraceEventFaker(const std::string& taskName)
    : Task(taskName)
{
    av::sms::TraceClient::initialize(std::make_unique<av::sms::AvlogSmsLogger>(taskName), taskName, 0);
}

bool TraceEventFaker::initialize()
{
    AV_TRACE_LOG_INSTANT(Event1, CustomEnumLabels::Foo, "MyLabel", static_cast<int64_t>(42));
    return true;
}

bool TraceEventFaker::executive()
{
    if (++counter_ % 10 == 0) {
        AVLOG_INFO("TraceEventFaker::executive");
    }

    AV_TRACE_SCOPED_PERF(Faker, CustomEnumLabels::Bar, "MyLabel");

    return true;
}

void TraceEventFaker::cleanup() {}
