/**
 * @copyright
 * Copyright (c) 2021- Motional. All rights reserved.
 * NOTICE: Removing this heading is prohibited.
 * All information contained herein is, and remains the property of Motional and its suppliers, if any.
 * The intellectual and technical concepts contained herein are proprietary to Motional and its suppliers and may be
 * covered by U.S. and Foreign Patents, patents in process, and are protected by trade secret or copyright law.
 * Dissemination of this information or reproduction of this material is strictly forbidden unless prior written
 * permission is obtained from Motional.
 *
 */

#ifndef TRACE_EVENT_HPP
#define TRACE_EVENT_HPP

#include <avtime/AvClock.hpp>

namespace av {
namespace sms {

enum class ValueType
{
    Timing = 0, // special case for performance timing
    Int64 = 1,
    Uint64 = 2,
    Float32 = 3,
    Float64 = 4,
    String = 5,
    Boolean = 6
};

struct TraceEvent
{
    av::TimePoint startTime{};  // start time of a performance metric
    av::TimePoint endTime{};    // end time of a performance metric
    std::string processName{};  // process / av stack task name?
    int32_t processId{};        // OS process ID
    int32_t threadId{};         // OS thread ID
    std::string fileName{};     // file name trace statement was called in
    std::string functionName{}; // function name trace statement was called in
    int32_t lineNumber{};       // line number of trace statement

    std::string category{}; // category (Enum name::value, required)
    std::string label{};    // label (user defined, required)
    int64_t cycleCount{};   // cycle counter (user defined, optional)

    ValueType valueType{ValueType::Timing}; // Type of `value` (used for identifying which `value` to parse below)

    std::string name{}; // name of user defined value

    // One of the following provided by the user
    // If this gets to cumbersome, we can replace with an byte array and custom parsing code
    // Opted to go this route as its smaller overall than a single string + bit for bit matches the values vs string
    // parsing + speed
    int64_t intValue{};
    uint64_t uIntValue{};
    float floatValue{};
    double doubleValue{};
    std::string stringValue{};
    bool boolValue{};
};

} // namespace sms
} // namespace av

#endif // TRACE_EVENT_HPP
