#include "SmsIngestionReader.hpp"
#include <stdexcept>

namespace av::sms {

void SmsIngestionReader::addInternString(InternString str)
{
    if (internTable_.find(str.id) != internTable_.end()) {
        throw std::invalid_argument("Attempted to overwrite existing ID in intern table");
    }

    internTable_[str.id] = std::move(str.value);
}

nlohmann::json SmsIngestionReader::read()
{
    bool encounteredUniformMetadata{false};
    nlohmann::json events = nlohmann::json::array();
    UniformMetadata metadata;

    // Parse through entirety of msgpack messages
    while (true) {
        nlohmann::json jdata = reader_.read();
        if (jdata.is_discarded()) {
            break;
        }

        // Array object is either an intern'd string or trace event
        if (jdata.is_array()) {
            if (jdata.size() == 2) {
                addInternString(extractInternEvent(jdata));
            } else {
                events.emplace_back(std::move(jdata));
            }
        } else if (jdata.is_object()) {
            if (jdata.contains("metadata")) {
                // `UniformMetadata` message
                if (encounteredUniformMetadata) {
                    throw std::invalid_argument("Read more than one static metadata message");
                }

                metadata = extractUniformMetadataEvent(jdata);
                encounteredUniformMetadata = true;
            } else if (jdata.contains("message")) {
                // A general message from the logger
                if (messageOutput_) {
                    events.emplace_back(std::move(jdata));
                }
            }
        } else {
            throw std::invalid_argument("Unknown message encountered");
        }
    }

    for (nlohmann::json& message : events) {
        if (message.is_array()) {
            // `extractTraceEvent` handles size checking of the JSON entry
            nlohmann::json traceEvent;
            extractTraceEvent(traceEvent, message, internTable_, metadata);
            message = traceEvent;
        }
    }

    return events;
}

} // namespace av::sms
