/**
 * \copyright
 * Copyright (c) 2022- Motional. All rights reserved.
 * NOTICE: Removing this heading is prohibited.
 * All information contained herein is, and remains the property of Motional and its suppliers, if any.
 * The intellectual and technical concepts contained herein are proprietary to Motional and its suppliers and may be
 * covered by U.S. and Foreign Patents, patents in process, and are protected by trade secret or copyright law.
 * Dissemination of this information or reproduction of this material is strictly forbidden unless prior written
 * permission is obtained from Motional.
 *
 */

#include <sms/Sms.hpp>
#include <sms/SmsLogger.hpp>
#include <sms/sms-ingest/SmsIngestionReader.hpp>

#include "lib/avutils/process/Process.hpp"

#include <gtest/gtest.h>

#include <algorithm>
#include <experimental/filesystem>
#include <iostream>

namespace fs = std::experimental::filesystem;

namespace {
namespace file {
static const fs::path base{fs::path(getenv("TEST_SRCDIR")) / "av_stack/platform/middleware/libs/sms"};
static const fs::path path_binary{base / "sms-ingest" / "sms-ingest"};
} // namespace file
} // anonymous namespace

TEST(SmsIngest, FileOutput)
{
    const std::string inFile = std::string(getenv("TESTDIR")) + "/smsIngest-test1.sms.msgpack";
    const std::string outFile = "test1.json";
    av::Process process(file::path_binary, "-i", inFile, "-o", outFile);
    process.wait();
    if (!fs::exists(outFile) or (fs::file_size(outFile) == 0)) {
        throw std::runtime_error("File not found: " + outFile);
    }
}

TEST(SmsIngest, FileOutputPretty)
{
    const std::string inFile = std::string(getenv("TESTDIR")) + "/smsIngest-test1.sms.msgpack";
    const std::string outFile = "test1.json";
    av::Process process(file::path_binary, "-i", inFile, "-p", "-o", outFile);
    process.wait();
    if (!fs::exists(outFile) or (fs::file_size(outFile) == 0)) {
        throw std::runtime_error("File not found: " + outFile);
    }
}

TEST(SmsIngest, PrintHelp)
{
    av::Process process(file::path_binary, "-h");
    process.wait();
    EXPECT_EQ(process.exitCode(), 1);
}
