/**
 * \copyright
 * Copyright (c) 2021- Motional. All rights reserved.
 * NOTICE: Removing this heading is prohibited.
 * All information contained herein is, and remains the property of Motional, its affiliates, and its suppliers (if
 * any). The intellectual and technical concepts contained herein are proprietary to Motional, its affiliates, and its
 * suppliers (if any) and may be covered by U.S. and Foreign Patents, patents in process, and are protected by trade
 * secrets or copyright law. Dissemination of this information or reproduction of this material is strictly forbidden
 * unless prior written permission is obtained from Motional.
 *
 */

#include "task/MonitoringStatusProvider.hpp"

av::msg::AvTaskStatus MonitoringStatusProvider::getStatus()
{
    const std::lock_guard<std::mutex> getStatusLock{mutex_};
    return avTaskStatus_;
}

void MonitoringStatusProvider::setStatus(av::msg::AvTaskStatus status)
{
    const std::lock_guard<std::mutex> setStatusLock{mutex_};
    avTaskStatus_ = std::move(status);
}

void MonitoringStatusProvider::setStatus(int64_t timestamp,
                                         av::msg::TaskState state,
                                         av::msg::FaultSource source,
                                         const std::string& message,
                                         av::msg::ParameterType type,
                                         const std::string& parameter_name,
                                         const std::string& measured_value,
                                         const std::string& threshold_value)
{
    const std::lock_guard<std::mutex> setStatusLock{mutex_};
    avTaskStatus_.timestamp = timestamp;
    avTaskStatus_.state = state;
    avTaskStatus_.source = source;
    avTaskStatus_.message = message;
    avTaskStatus_.type = type;
    avTaskStatus_.parameter_name = parameter_name;
    avTaskStatus_.measured_value = measured_value;
    avTaskStatus_.threshold_value = threshold_value;
}

bool MonitoringStatusProvider::isCurrentStateMoreSevereThan(av::msg::TaskState intendedState)
{
    const std::lock_guard<std::mutex> isCurrentStateMoreSevereThanLock{mutex_};

    return std::greater<av::msg::TaskState>{}(intendedState, avTaskStatus_.state);
}

bool MonitoringStatusProvider::updateStatus(int64_t timestamp,
                                            av::msg::TaskState intendedState,
                                            av::msg::FaultSource source,
                                            const std::string& message,
                                            av::msg::ParameterType type,
                                            const std::string& parameter_name,
                                            const std::string& measured_value,
                                            const std::string& threshold_value)
{
    if (isCurrentStateMoreSevereThan(intendedState)) {
        setStatus(timestamp, intendedState, source, message, type, parameter_name, measured_value, threshold_value);
        return true;
    }
    return false;
}
