/**
 * \copyright
 * Copyright (c) 2021- Motional. All rights reserved.
 * NOTICE: Removing this heading is prohibited.
 * All information contained herein is, and remains the property of Motional, its affiliates, and its suppliers (if
 * any). The intellectual and technical concepts contained herein are proprietary to Motional, its affiliates, and its
 * suppliers (if any) and may be covered by U.S. and Foreign Patents, patents in process, and are protected by trade
 * secrets or copyright law. Dissemination of this information or reproduction of this material is strictly forbidden
 * unless prior written permission is obtained from Motional.
 *
 * \file
 * \brief TaskTimeProvider member function definitions.
 */
#include <Clock/SystemClock.h>
#include <avtime/ConversionsTimespec.hpp>
#include <task/TaskTime.hpp>

namespace task::time {

ottotime::otimePoint detail::SystemClockProvider::operator()()
{
    return SystemClock::getSystemTime();
}

ottotime::otimePoint detail::AvTimeProvider::operator()()
{
    return av::timespecFromTimePoint(av::AvClock::now());
}

TaskTimeProvider& TaskTimeProvider::get()
{
    static TaskTimeProvider instance;
    return instance;
}

void TaskTimeProvider::setTimeSource_(OTimeProvider otimeProvider)
{
    oTimeProvider_ = std::move(otimeProvider);
}

ottotime::otimePoint TaskTimeProvider::getSystemOTime_() const
{
    return oTimeProvider_();
}

void TaskTimeProvider::setTimeSource(const OTimeProvider& otimeProvider)
{
    get().setTimeSource_(otimeProvider);
}

ottotime::otimePoint TaskTimeProvider::getSystemOTime()
{
    return get().getSystemOTime_();
}

av::Duration TaskTimeProvider::getSystemTime()
{
    return av::durationFromTimespec(get().getSystemOTime_().to_otimeSpec());
}

} // namespace task::time