/**
 * \copyright
 * Copyright (c) 2020- Motional. All rights reserved.
 * NOTICE: Removing this heading is prohibited.
 * All information contained herein is, and remains the property of Motional and its suppliers, if any.
 * The intellectual and technical concepts contained herein are proprietary to Motional and its suppliers and may be
 * covered by U.S. and Foreign Patents, patents in process, and are protected by trade secret or copyright law.
 * Dissemination of this information or reproduction of this material is strictly forbidden unless prior written
 * permission is obtained from Motional.
 *
 * \brief Defines an implementation of the interface for handling closing signals
 */

#ifndef CLOSING_SIGNAL_HANDLER_H
#define CLOSING_SIGNAL_HANDLER_H

#include <task/ClosingSignalHandlerInterface.h>

#include <signal.h>

#include <atomic>
#include <condition_variable>
#include <mutex>
#include <optional>
#include <thread>

namespace task {

/// Implementation of the interface for handling closing signals.
class ClosingSignalHandler : public ClosingSignalHandlerInterface
{
public:
    /// Default constructor.
    ClosingSignalHandler();
    /// Destructor.
    ~ClosingSignalHandler() override;

    bool shouldExit() const override;
    void wait(std::optional<uint64_t> timeoutMS = std::nullopt) const override;
    void quit() override;

    bool setSignals(const std::vector<int>& closingSignals) override;
    bool handleClosingSignals() override;

private:
    // Entry point for the thread that listens for closing signals.
    void threadEntry();

    // Handle a signal that just arrived. Returns false if handling fails.
    bool handleArrivingSignal();

private:
    std::thread handlingThread_{};                  ///< Thread to handle closing signals
    sigset_t signalSet_{};                          ///< The set of closing signals
    int numberSignalsCaught_{0};                    ///< The number of signals number that have been caught
    std::atomic<bool> exitFlag_{false};             ///< ! if the exit flag is set
    std::atomic<bool> joinFlag_{false};             ///< flag indicating that the signal handling thread should join.
    mutable std::mutex thrd_mtx_{};                 ///< mutex for interrupting the thread and waiting.
    mutable std::condition_variable exitCondition_; ///< condition variable for signaling when the exit flag is set
};

} // namespace task

#endif // CLOSING_SIGNAL_HANDLER_H
