/**
 * \copyright
 * Copyright (c) 2020- Motional. All rights reserved.
 * NOTICE: Removing this heading is prohibited.
 * All information contained herein is, and remains the property of Motional and its suppliers, if any.
 * The intellectual and technical concepts contained herein are proprietary to Motional and its suppliers and may be
 * covered by U.S. and Foreign Patents, patents in process, and are protected by trade secret or copyright law.
 * Dissemination of this information or reproduction of this material is strictly forbidden unless prior written
 * permission is obtained from Motional.
 *
 * \brief Defines an interface for handling closing signals
 */

#ifndef CLOSING_SIGNAL_HANDLER_INTERFACE_H
#define CLOSING_SIGNAL_HANDLER_INTERFACE_H

#include <cstdint>
#include <optional>
#include <vector>

namespace task {

/// Interface for handling closing signals.
class ClosingSignalHandlerInterface
{
public:
    /// Default constructor.
    ClosingSignalHandlerInterface() = default;
    /// Default destructor.
    virtual ~ClosingSignalHandlerInterface() = default;

    /// Check if the exit flag is set.
    /// @return True if the flag is set.
    virtual bool shouldExit() const = 0;
    /// Wait until shouldExit() is true.
    virtual void wait(std::optional<uint64_t> timeoutMS = std::nullopt) const = 0;
    /// Quit the task, e.g. because the executive returned false.
    virtual void quit() = 0;

    /// Set the signal mask.
    /// @param closingSignals The signals to listen to. This must contain at least one signal, and no invalid signals.
    /// @return False if setting the signal mask fails.
    /// @note If this is called multiple times, the signal masks are combined.
    virtual bool setSignals(const std::vector<int>& closingSignals) = 0;
    /// Start up the thread that listens for closing signals.
    /// @return False if starting the thread fails.
    virtual bool handleClosingSignals() = 0;
};

} // namespace task

#endif // CLOSING_SIGNAL_HANDLER_INTERFACE_H
