/**
 * \copyright
 * Copyright (c) 2021- Motional. All rights reserved.
 * NOTICE: Removing this heading is prohibited.
 * All information contained herein is, and remains the property of Motional and its suppliers, if any.
 * The intellectual and technical concepts contained herein are proprietary to Motional and its suppliers and may be
 * covered by U.S. and Foreign Patents, patents in process, and are protected by trade secret or copyright law.
 * Dissemination of this information or reproduction of this material is strictly forbidden unless prior written
 * permission is obtained from Motional.
 *
 * \brief Defines a generic message publisher
 */

#ifndef GENERIC_MESSAGE_PUBLISHER_H
#error Should only be included from GenericMessagePublisher.h
#endif

namespace task {

template<class OutputInterfaceType, class MessageType>
GenericMessagePublisher<OutputInterfaceType, MessageType>::GenericMessagePublisher(
    std::shared_ptr<OutputInterfaceType> outputInterface,
    const logger::Logger& logger)
    : m_outputInterface(std::move(outputInterface))
    , m_logger(logger)
{}

template<class OutputInterfaceType, class MessageType>
bool GenericMessagePublisher<OutputInterfaceType, MessageType>::publishMessage(const MessageType& message)
{
    bool result{false};
    if (m_outputInterface != nullptr) {
        result = m_outputInterface->publish(message);
    } else {
        // TODO: MW-424 Use AVLOG after GCC 10
        m_logger.log_error("Message output interface is null");
    }

    return result;
}

} // namespace task
