/**
 * \copyright
 * Copyright (c) 2021- Motional. All rights reserved.
 * NOTICE: Removing this heading is prohibited.
 * All information contained herein is, and remains the property of Motional and its suppliers, if any.
 * The intellectual and technical concepts contained herein are proprietary to Motional and its suppliers and may be
 * covered by U.S. and Foreign Patents, patents in process, and are protected by trade secret or copyright law.
 * Dissemination of this information or reproduction of this material is strictly forbidden unless prior written
 * permission is obtained from Motional.
 *
 * \brief Defines a generic message subscriber
 */

#ifndef GENERIC_MESSAGE_SUBSCRIBER_H
#error Should only be included from GenericMessageSubscriber.h
#endif

namespace task {

template<class InputInterfaceType, class MessageType>
GenericMessageSubscriber<InputInterfaceType, MessageType>::GenericMessageSubscriber(
    std::shared_ptr<InputInterfaceType> inputInterface,
    const logger::Logger& logger)
    : m_inputInterface(std::move(inputInterface))
    , m_logger(logger)
{}

template<class InputInterfaceType, class MessageType>
bool GenericMessageSubscriber<InputInterfaceType, MessageType>::receiveMessage(MessageType& message,
                                                                               const bool blocking)
{
    bool result{false};
    if (m_inputInterface != nullptr) {
        result = m_inputInterface->get(message, blocking);
    } else {
        // TODO: MW-424 Use AVLOG after GCC 10
        m_logger.log_error("Message input interface is null");
    }

    return result;
}

} // namespace task
