/**
 * \copyright
 * Copyright (c) 2020- Motional. All rights reserved.
 * NOTICE: Removing this heading is prohibited.
 * All information contained herein is, and remains the property of Motional and its suppliers, if any.
 * The intellectual and technical concepts contained herein are proprietary to Motional and its suppliers and may be
 * covered by U.S. and Foreign Patents, patents in process, and are protected by trade secret or copyright law.
 * Dissemination of this information or reproduction of this material is strictly forbidden unless prior written
 * permission is obtained from Motional.
 *
 * \brief Defines an interface for sending heartbeats at a frequency defined in the task's configuration file
 */

#ifndef HEARTBEAT_SENDER_INTERFACE_H
#define HEARTBEAT_SENDER_INTERFACE_H

#include <interfaces/ProcessHealthMessage/ProcessHealthMessage.h>

#include <string>

namespace task {

/**
 * \brief Defines an interface for sending heartbeats at a frequency defined in the task's configuration file
 */
class HeartbeatSenderInterface
{
public:
    /**
     * \brief Default constructor
     */
    HeartbeatSenderInterface() = default;

    /**
     * \brief Default destructor
     */
    virtual ~HeartbeatSenderInterface() = default;

    /**
     * \brief Set the heartbeat with the given message
     *
     * \param hbMessage The message to be set
     *
     * \return Void
     */
    virtual void setHeartbeatMessage(const av::msg::ProcessHealthMessage& hbMessage) = 0;

    /**
     * \brief Get the current heartbeat message
     *
     * \param Void
     *
     * \return The current heartbeat content
     */
    virtual const av::msg::ProcessHealthMessage& getHeartbeatMessage() const = 0;

    /**
     * \brief Send a heartbeat with current message including the actual AV task process frequency calculated by the
     * heartbeat sender
     *
     * \param terminating true if this is to be the last heartbeat sent.
     *
     * \return True if the heartbeat was successfully sent
     */
    virtual bool sendHeartbeat(bool terminating = false) = 0;

    /**
     * \brief Check if a heartbeat needs to be sent, for example, the heartbeat sending cycle is expired. By doing that,
     * it also calculates the actual frequency of the AV task process every cycle time of the heartbeat sender in
     * seconds
     *
     * \param Void
     *
     * \return True if the heartbeat needs to be sent
     */
    virtual bool needToSendHeartbeat() = 0;
};

} // namespace task

#endif // HEARTBEAT_SENDER_INTERFACE_H
