/**
 * \copyright
 * Copyright (c) 2021- Motional. All rights reserved.
 * NOTICE: Removing this heading is prohibited.
 * All information contained herein is, and remains the property of Motional and its suppliers, if any.
 * The intellectual and technical concepts contained herein are proprietary to Motional and its suppliers and may be
 * covered by U.S. and Foreign Patents, patents in process, and are protected by trade secret or copyright law.
 * Dissemination of this information or reproduction of this material is strictly forbidden unless prior written
 * permission is obtained from Motional.
 *
 * \brief Defines an interface for getting AV tasks internal status to send to the monitoring system
 */

#ifndef MONITORING_STATUS_PROVIDER_INTERFACE_H
#define MONITORING_STATUS_PROVIDER_INTERFACE_H

#include <memory>

// Forward declarations
namespace av {
namespace msg {
class AvTaskStatus;
} // namespace msg
} // namespace av

namespace task {

/**
 * \brief Defines an interface for getting AV tasks internal status to send to the monitoring system
 */
class MonitoringStatusProviderInterface
{
public:
    /**
     * \brief Default constructor
     */
    MonitoringStatusProviderInterface() = default;

    /**
     * \brief Default destructor
     */
    virtual ~MonitoringStatusProviderInterface() = default;

    /**
     * \brief Get the current internal status
     *
     * \param Void
     *
     * \return The current status
     */
    virtual av::msg::AvTaskStatus getStatus() = 0;
};

} // namespace task

#endif // MONITORING_STATUS_PROVIDER_INTERFACE_H
