/**
 * \copyright
 * Copyright (c) 2021- Motional. All rights reserved.
 * NOTICE: Removing this heading is prohibited.
 * All information contained herein is, and remains the property of Motional and its suppliers, if any.
 * The intellectual and technical concepts contained herein are proprietary to Motional and its suppliers and may be
 * covered by U.S. and Foreign Patents, patents in process, and are protected by trade secret or copyright law.
 * Dissemination of this information or reproduction of this material is strictly forbidden unless prior written
 * permission is obtained from Motional.
 *
 * \brief Defines an interface for sending AV tasks internal status to the monitoring system
 */

#ifndef MONITORING_STATUS_SENDER_INTERFACE_H
#define MONITORING_STATUS_SENDER_INTERFACE_H

// Forward declarations
class ConfigSection;

namespace av {
namespace msg {
class AvTaskStatus;
} // namespace msg
} // namespace av

namespace task {

/**
 * \brief Defines an interface for sending AV tasks internal status to the monitoring system
 */
class MonitoringStatusSenderInterface
{
public:
    /**
     * \brief Default constructor
     */
    MonitoringStatusSenderInterface() = default;

    /**
     * \brief Default destructor
     */
    virtual ~MonitoringStatusSenderInterface() = default;

    /**
     * \brief Initialize the sender
     *
     * \param taskParams The task parameters
     *
     * \return True if the initialization is fine, false otherwise
     */
    virtual bool initialize(ConfigSection& taskParams) = 0;

    /**
     * \brief Send the given AvTaskStatus status
     *
     * \param status The status to be sent
     *
     * \return True if the sending is fine, false otherwise
     */
    virtual bool sendStatus(const av::msg::AvTaskStatus& status) = 0;
};

} // namespace task

#endif // MONITORING_STATUS_SENDER_INTERFACE_H
