/**
 * \copyright
 * Copyright (c) 2021- Motional. All rights reserved.
 * NOTICE: Removing this heading is prohibited.
 * All information contained herein is, and remains the property of Motional, its affiliates, and its suppliers (if
 * any). The intellectual and technical concepts contained herein are proprietary to Motional, its affiliates, and its
 * suppliers (if any) and may be covered by U.S. and Foreign Patents, patents in process, and are protected by trade
 * secrets or copyright law. Dissemination of this information or reproduction of this material is strictly forbidden
 * unless prior written permission is obtained from Motional.
 *
 * \file
 * \brief Definition of the MockCallbackSet class, for testing message outputs.
 */
#pragma once

#include <ODatum/ODatum.h>
#include <SimpleComms/test/MockClient.h>
#include <gmock/gmock.h>
#include <interfaces/baseTypes/Header.h>
#include <repro/EventFramework.hpp>
#include <task/CallbackSet.h>

namespace task {

/// \brief A CallbackSet with a mock client interface, for testing MessageOutputs.
class MockCallbackSet : public CallbackSet
{
public:
    MockCallbackSet()
        : CallbackSet(framework_)
    {
        client_ = std::make_shared<testing::NiceMock<SimpleComms::MockClient>>();
        framework_.setClient(client_);
    }

    /// \brief Expect that sendMsg() will not be called at all in this test.
    void expectNoSendCalls()
    {
        EXPECT_CALL(*client_, sendMsgV(testing::_, testing::_, testing::_)).Times(testing::Exactly(0));
    }

    /// \brief Expect sendMsg() to be called; getSentData() can be used to check the sent value.
    void expectSendCall()
    {
        EXPECT_CALL(*client_, sendMsgV(testing::_, testing::_, testing::_))
            .WillOnce(testing::DoAll(testing::SaveArg<1>(&data_), testing::Return(SimpleComms::Status::Ok)));
    }

    /// \brief Get the data that was sent most recently, deserialized into a given data type.
    template<class Type>
    Type getSentData() const
    {
        Type sentData;
        std::string body = data_.substr(sizeof(interfaces::baseTypes::InterfaceHeader));
        aptv::deserializeODatum(sentData, body);
        return sentData;
    }

    std::shared_ptr<SimpleComms::MockClient> client() { return client_; }

private:
    av::repro::EventFramework framework_;
    std::shared_ptr<testing::NiceMock<SimpleComms::MockClient>> client_;
    std::string data_;
};

} // namespace task
