/// @copyright Copyright (c) 2021- Motional. All rights reserved.
///
/// NOTICE: Removing this heading is prohibited.
///
/// All information contained herein is, and remains the property of Motional,
/// its affiliates, and its suppliers (if any). The intellectual and technical
/// concepts contained herein are proprietary to Motional, its affiliates, and
/// its suppliers (if any) and may be covered by U.S. and Foreign Patents,
/// patents in process, and are protected by trade secrets or copyright law.
/// Dissemination of this information or reproduction of this material is
/// strictly forbidden unless prior written permission is obtained from Motional.
///
/// @file
/// @brief Definition of the EventInfo class.
#pragma once

#include <repro/EventInfo.hpp>

#include <cstdint>
#include <functional>

namespace av::repro {

/// Class representing an event: a handler function plus associated metadata.
class Event : public EventInfo
{
public:
    Event() noexcept = default;
    /// Constructor.
    Event(const EventInfo& info, std::function<uint32_t(uint32_t)>&& function);
    Event(const Event&) = default;
    Event(Event&&) noexcept = default;
    ~Event() = default;

    Event& operator=(const Event&) = default;
    Event& operator=(Event&&) noexcept = default;

    /// Run the handler.
    /// @param lastID: (Replay mode): the ID of the last message in this batch.
    /// @return (Non-replay mode): The ID of the last message that was handled in the batch.
    uint32_t operator()(uint32_t lastID = 0) const;

private:
    std::function<uint32_t(uint32_t)> function_;
};

// Inline the constructor.
inline Event::Event(const EventInfo& info, std::function<uint32_t(uint32_t)>&& function)
    : EventInfo(info)
    , function_(std::move(function))
{}

inline uint32_t Event::operator()(uint32_t lastID) const
{
    return function_(lastID);
}

} // namespace av::repro
