/// @copyright Copyright (c) 2021- Motional. All rights reserved.
///
/// NOTICE: Removing this heading is prohibited.
///
/// All information contained herein is, and remains the property of Motional,
/// its affiliates, and its suppliers (if any). The intellectual and technical
/// concepts contained herein are proprietary to Motional, its affiliates, and
/// its suppliers (if any) and may be covered by U.S. and Foreign Patents,
/// patents in process, and are protected by trade secrets or copyright law.
/// Dissemination of this information or reproduction of this material is
/// strictly forbidden unless prior written permission is obtained from Motional.
///
/// @file
/// @brief Definition of the EventInfo class.
#pragma once

#include <avtime/AvClock.hpp>

#include <string_view>

namespace av::repro {

/// A lightweight class storing metadata about an event (a message or timeout).
/// This is the information that must be logged for reproducibility when the event runs.
class EventInfo
{
public:
    EventInfo() noexcept = default;
    /// Constructor.
    EventInfo(Duration time, std::string_view topic, std::string_view origin, uint32_t id) noexcept;
    EventInfo(const EventInfo&) noexcept = default;
    EventInfo(EventInfo&&) noexcept = default;
    ~EventInfo() noexcept = default;

    EventInfo& operator=(const EventInfo&) noexcept = default;
    EventInfo& operator=(EventInfo&&) noexcept = default;

    /// Get the "sent timestamp" from the message, in duration since the epoch.
    /// @note This does not use TimePoint because we use different clocks in different modes.
    Duration time() const noexcept;
    /// Get the topic this message was received from. Timeout topics begin with "!".
    const std::string_view& topic() const noexcept;
    /// Get the sender of this messages.
    /// @note This will be an empty string if this is a timeout event.
    const std::string_view& origin() const noexcept;
    /// Get the rolling message ID; combined with the origin this uniquely identifies a message.
    uint32_t id() const noexcept;

private:
    Duration time_{0};
    std::string_view topic_;
    std::string_view origin_;
    uint32_t id_{0};
};

} // namespace av::repro
