/// @copyright Copyright (c) 2021- Motional. All rights reserved.
///
/// NOTICE: Removing this heading is prohibited.
///
/// All information contained herein is, and remains the property of Motional,
/// its affiliates, and its suppliers (if any). The intellectual and technical
/// concepts contained herein are proprietary to Motional, its affiliates, and
/// its suppliers (if any) and may be covered by U.S. and Foreign Patents,
/// patents in process, and are protected by trade secrets or copyright law.
/// Dissemination of this information or reproduction of this material is
/// strictly forbidden unless prior written permission is obtained from Motional.
///
/// @file
/// @brief Definition of the EventInfo class.
#pragma once

#include <repro/EventInfo.hpp>

#include <memory>

namespace av::repro {

/// Class representing a message plus associated metadata.
template<typename Type>
class Message
    : public EventInfo
    , public std::shared_ptr<Type>
{
public:
    Message() noexcept = default;
    ~Message() noexcept = default;
    /// Constructor.
    /// @note Only Deserializer should construct new messages.
    Message(const EventInfo& info, std::shared_ptr<Type>&& data) noexcept;
    Message(const Message&) noexcept = default;
    Message(Message&&) noexcept = default;

    Message& operator=(const Message&) noexcept = default;
    Message& operator=(Message&&) noexcept = default;

    using std::shared_ptr<Type>::operator*;
    using std::shared_ptr<Type>::operator->;
};

// Inline the constructor.
template<typename Type>
Message<Type>::Message(const EventInfo& info, std::shared_ptr<Type>&& data) noexcept
    : EventInfo(info)
    , std::shared_ptr<Type>(std::move(data))
{}

} // namespace av::repro
