/// @copyright Copyright (c) 2021- Motional. All rights reserved.
///
/// NOTICE: Removing this heading is prohibited.
///
/// All information contained herein is, and remains the property of Motional,
/// its affiliates, and its suppliers (if any). The intellectual and technical
/// concepts contained herein are proprietary to Motional, its affiliates, and
/// its suppliers (if any) and may be covered by U.S. and Foreign Patents,
/// patents in process, and are protected by trade secrets or copyright law.
/// Dissemination of this information or reproduction of this material is
/// strictly forbidden unless prior written permission is obtained from Motional.
///
/// @file
/// @brief Declaration of functions for operating on serialized blobs with headers.
#include <repro/blobUtils.hpp>

#include <ODatum/ODatum.h>
#include <avtime/ConversionsTimespec.hpp>
#include <interfaces/Clock/Playback/PlaybackClockParams.h>
#include <interfaces/baseTypes/Header.h>

namespace av::repro {

Duration parseTimestamp(const std::string& blob)
{
    // NOLINTNEXTLINE(cppcoreguidelines-pro-type-reinterpret-cast)
    const auto& header = *reinterpret_cast<const interfaces::baseTypes::InterfaceHeader*>(blob.data());
    otimeSpec time;
    interfaces::baseTypes::getSendTimeStamp(header, time);
    return durationFromTimespec(time);
}

Duration parsePlaybackClock(const std::string& blob)
{
    std::string body = blob.substr(sizeof(interfaces::baseTypes::InterfaceHeader));
    PlaybackClockParams params;
    aptv::deserializeODatum(params, body);
    return std::chrono::nanoseconds(params.baseTime.toNanosecondsSinceEpoch());
}

uint32_t parseID(const std::string& blob)
{
    // NOLINTNEXTLINE(cppcoreguidelines-pro-type-reinterpret-cast)
    const auto& header = *reinterpret_cast<const interfaces::baseTypes::InterfaceHeader*>(blob.data());
    uint32_t id{0};
    interfaces::baseTypes::getHeaderMessageCounter(header, id);
    return id;
}

} // namespace av::repro
