/// @copyright Copyright (c) 2022- Motional. All rights reserved.
///
/// NOTICE: Removing this heading is prohibited.
///
/// All information contained herein is, and remains the property of Motional,
/// its affiliates, and its suppliers (if any). The intellectual and technical
/// concepts contained herein are proprietary to Motional, its affiliates, and
/// its suppliers (if any) and may be covered by U.S. and Foreign Patents,
/// patents in process, and are protected by trade secrets or copyright law.
/// Dissemination of this information or reproduction of this material is
/// strictly forbidden unless prior written permission is obtained from Motional.
///
/// @file
/// @brief Function declarations for a lightweight comms framework.
#pragma once

#include <repro/EventFramework.hpp>

#include <string>

namespace av::comms {

/// Attempt to initialize comms.
/// @param taskName: The name of this task.
/// @param commsSystem: The comms system to use; must be "SCS" or "DDS".
/// @return True if successful.
bool init(const std::string& taskName, const std::string& commsSystem = "SCS");

/// Access the EventFramework, which can be used for registering inputs or outputs. For example:
/// comms::framework().addMessageHandler("MyInput", &myClass, &MyClass::myInputHandler);
/// comms::framework().setTimeoutHandler("!periodic", &myClass, &MyClass::executive, std::chrono::milliseconds(50));
/// MessageOutput<MyType> output("MyOutput", comms::framework());
repro::EventFramework& framework();

/// Shut down the connection to the comms.
void shutdown();

} // namespace av::comms
