/**
 * \copyright
 * Copyright (c) 2020- Motional. All rights reserved.
 * NOTICE: Removing this heading is prohibited.
 * All information contained herein is, and remains the property of Motional and its suppliers, if any.
 * The intellectual and technical concepts contained herein are proprietary to Motional and its suppliers and may be
 * covered by U.S. and Foreign Patents, patents in process, and are protected by trade secret or copyright law.
 * Dissemination of this information or reproduction of this material is strictly forbidden unless prior written
 * permission is obtained from Motional.
 *
 * \brief Defines unit tests for GenericMessagePublisher
 */

#include "MockMessageOutputInterface.h"

#include <log/Logger.h>
#include <task/GenericMessagePublisher.h>

#include <gmock/gmock.h>
#include <gtest/gtest.h>
#include <memory>
#include <string>

using ::testing::Exactly;
using ::testing::Return;
using namespace task;

/**
 * \brief This class defines unit tests for GenericMessagePublisher class
 */
class GenericMessagePublisherTest : public ::testing::Test
{
public:
    /**
     * \brief Set up the tests
     */
    void SetUp() override {}

    /**
     * \brief Clean up the tests
     */
    void TearDown() override {}

    /**
     * \brief Test publishMessage()
     */
    template<bool publishResult>
    bool testPublishMessage()
    {
        auto mockMsgOutput = std::make_shared<MockMessageOutputInterface<publishResult>>();
        GenericMessagePublisher<MockMessageOutputInterface<publishResult>, std::string> testPublisher{mockMsgOutput,
                                                                                                      m_logger};
        return testPublisher.publishMessage(m_message);
    }

protected:
    std::string m_message{"Default message"}; ///< The message to be sent
    logger::Logger m_logger{};                ///< The logger with defautl level
};

TEST_F(GenericMessagePublisherTest, nullInterface)
{
    GenericMessagePublisher<MockMessageOutputInterface<true>, std::string> testPublisher{nullptr, m_logger};

    ASSERT_FALSE(testPublisher.publishMessage(m_message));
}

TEST_F(GenericMessagePublisherTest, successfulSending)
{
    auto result = testPublishMessage<true>();
    ASSERT_TRUE(result);
}

TEST_F(GenericMessagePublisherTest, failedSending)
{
    auto result = testPublishMessage<false>();

    ASSERT_FALSE(result);
}
