/**
 * \copyright
 * Copyright (c) 2021- Motional. All rights reserved.
 * NOTICE: Removing this heading is prohibited.
 * All information contained herein is, and remains the property of Motional and its suppliers, if any.
 * The intellectual and technical concepts contained herein are proprietary to Motional and its suppliers and may be
 * covered by U.S. and Foreign Patents, patents in process, and are protected by trade secret or copyright law.
 * Dissemination of this information or reproduction of this material is strictly forbidden unless prior written
 * permission is obtained from Motional.
 *
 * \brief Defines a mocked version of the message output interface
 */

#ifndef MOCK_MESSAGE_OUTPUT_INTERFACE_H
#define MOCK_MESSAGE_OUTPUT_INTERFACE_H

#include <gmock/gmock.h>
#include <gtest/gtest.h>
#include <string>

/**
 * \brief This class defines a mock version of generic message output interface
 */
template<bool publishResult>
class MockMessageOutputInterface
{
public:
    /**
     * \brief The default constructor
     */
    MockMessageOutputInterface() = default;

    /**
     * \brief The default destructor
     */
    ~MockMessageOutputInterface() = default;

    /**
     * \brief Mock the publish method
     *
     * \param message The message to be sent
     *
     * \return True if the sending is fine
     */
    bool publish(const std::string& message) const
    {
        static_cast<void>(message);
        return publishResult;
    }
};

#endif // MOCK_MESSAGE_OUTPUT_INTERFACE_H
