/**
 * \copyright
 * Copyright (c) 2020- Motional. All rights reserved.
 * NOTICE: Removing this heading is prohibited.
 * All information contained herein is, and remains the property of Motional and its suppliers, if any.
 * The intellectual and technical concepts contained herein are proprietary to Motional and its suppliers and may be
 * covered by U.S. and Foreign Patents, patents in process, and are protected by trade secret or copyright law.
 * Dissemination of this information or reproduction of this material is strictly forbidden unless prior written
 * permission is obtained from Motional.
 *
 * \brief Defines unit tests for GenericMessageSubscriber
 */

#include "MockMessageInputInterface.h"

#include <log/Logger.h>
#include <task/GenericMessageSubscriber.h>

#include <gmock/gmock.h>
#include <gtest/gtest.h>
#include <memory>
#include <string>

using ::testing::Exactly;
using ::testing::Return;
using namespace task;

/**
 * \brief This class defines unit tests for GenericMessageSubscriber class
 */
class GenericMessageSubscriberTest : public ::testing::Test
{
public:
    /**
     * \brief Set up the tests
     */
    void SetUp() override {}

    /**
     * \brief Clean up the tests
     */
    void TearDown() override {}

    /**
     * \brief Test publishMessage()
     */
    template<bool receiveResult>
    bool testReceiveMessage()
    {
        auto mockMsgInput = std::make_shared<MockMessageInputInterface<receiveResult>>();
        GenericMessageSubscriber<MockMessageInputInterface<receiveResult>, std::string> testSubscriber{mockMsgInput,
                                                                                                       m_logger};
        return testSubscriber.receiveMessage(m_message);
    }

protected:
    std::string m_message{""}; ///< The recived message
    logger::Logger m_logger{}; ///< The logger with defautl level
};

TEST_F(GenericMessageSubscriberTest, nullInterface)
{
    GenericMessageSubscriber<MockMessageInputInterface<true>, std::string> testSubscriber{nullptr, m_logger};

    ASSERT_FALSE(testSubscriber.receiveMessage(m_message));
}

TEST_F(GenericMessageSubscriberTest, successfulReceiving)
{
    auto result = testReceiveMessage<true>();
    ASSERT_TRUE(result);
}

TEST_F(GenericMessageSubscriberTest, failedReceiving)
{
    auto result = testReceiveMessage<false>();

    ASSERT_FALSE(result);
}
