/**
 * \copyright
 * Copyright (c) 2021- Motional. All rights reserved.
 * NOTICE: Removing this heading is prohibited.
 * All information contained herein is, and remains the property of Motional and its suppliers, if any.
 * The intellectual and technical concepts contained herein are proprietary to Motional and its suppliers and may be
 * covered by U.S. and Foreign Patents, patents in process, and are protected by trade secret or copyright law.
 * Dissemination of this information or reproduction of this material is strictly forbidden unless prior written
 * permission is obtained from Motional.
 *
 * \brief Defines a mocked version of the message input interface
 */

#ifndef MOCK_MESSAGE_INPUT_INTERFACE_H
#define MOCK_MESSAGE_INPUT_INTERFACE_H

#include <gmock/gmock.h>
#include <gtest/gtest.h>
#include <string>

/**
 * \brief This class defines a mock version of generic message input interface
 */
template<bool receiveResult>
class MockMessageInputInterface
{
public:
    /**
     * \brief The default constructor
     */
    MockMessageInputInterface() = default;

    /**
     * \brief The default destructor
     */
    ~MockMessageInputInterface() = default;

    /**
     * \brief Mock the receive method
     *
     * \param signal The signal to be sent
     *
     * \return True if the sending is fine
     */
    bool get(std::string& message, bool blocking = false) const
    {
        static_cast<void>(message);
        static_cast<void>(blocking);
        return receiveResult;
    }
};

#endif // MOCK_MESSAGE_INPUT_INTERFACE_H