/**
 * \copyright
 * Copyright (c) 2020- Motional. All rights reserved.
 * NOTICE: Removing this heading is prohibited.
 * All information contained herein is, and remains the property of Motional and its suppliers, if any.
 * The intellectual and technical concepts contained herein are proprietary to Motional and its suppliers and may be
 * covered by U.S. and Foreign Patents, patents in process, and are protected by trade secret or copyright law.
 * Dissemination of this information or reproduction of this material is strictly forbidden unless prior written
 * permission is obtained from Motional.
 *
 * \brief Defines a mocked version of the heartbeat message publisher
 */

#ifndef MOCK_HEARTBEAT_MSG_PUBLISHER_H
#define MOCK_HEARTBEAT_MSG_PUBLISHER_H

#include <interfaces/Generic/Output/Abstract.h>
#include <interfaces/ProcessHealthMessage/ProcessHealthMessage.h>
#include <log/Logger.h>
#include <task/GenericMessagePublisher.h>

#include <gmock/gmock.h>
#include <gtest/gtest.h>

/**
 * \brief This class defines a mock version of GenericMessagePublisher
 */
class MockHeartbeatMsgPublisher
    : public task::GenericMessagePublisher<GenericOutput<av::msg::ProcessHealthMessage>, av::msg::ProcessHealthMessage>
{
public:
    /**
     * \brief The user-defined constructor
     *
     * \param heartbeatMsgOut The interface for sending heartbeat messages
     */
    MockHeartbeatMsgPublisher(const std::shared_ptr<GenericOutput<av::msg::ProcessHealthMessage>>& heartbeatMsgOut)
        : task::GenericMessagePublisher<GenericOutput<av::msg::ProcessHealthMessage>, av::msg::ProcessHealthMessage>(
              heartbeatMsgOut,
              logger::Logger())
    {}

    /**
     * \brief Mock the sendHeartbeatMsg method
     *
     * \param  heartbeatMsg Message to be sent
     *
     * \return True if the message is sent
     */
    MOCK_METHOD1(publishMessage, bool(const av::msg::ProcessHealthMessage& heartbeatMsg));
};

#endif // MOCK_HEARTBEAT_MSG_PUBLISHER_H
