#include "lib/avutils/process/Process.hpp"
#include "platform/middleware/libs/task/test/ResimTest/resimHelpers.hpp"

#include <gtest/gtest.h>

#include <experimental/filesystem>
#include <fstream>
#include <string>

namespace fs = std::experimental::filesystem;

namespace {
const fs::path tmpDir = getenv("TEST_TMPDIR");
const fs::path srcDir = getenv("TEST_SRCDIR");
const fs::path taskBin = srcDir / "av_stack/platform/middleware/libs/task/test/LongInitTest/LongInitTask";
const fs::path scsBin = srcDir / "av_stack/platform/middleware/tasks/SimpleComms/SimpleComms/scs";
} // namespace

void writeConfig(int initTimeout)
{
    std::ofstream out("LongInitTask.cfg");
    out << "Parameters =\n{\n";
    out << "    initTimeout = " << initTimeout << ";\n";
    out << "    loggerThreshold = \"info\";\n";
    out << "    clockType = \"central\";\n";
    out << "    cycleRate_hz = 1.0;\n};\n";
    out << "Execution =\n{\n    executableName = \"LongInitTask\";\n};\n";
}

bool runTest(int timeout, const std::string& expectedMessage)
{
    const std::string scsPath = resim::setSCSPath();
    EXPECT_EQ(chdir(tmpDir.c_str()), 0);
    writeConfig(timeout);

    av::Process scs(scsBin, "--maxExitSignalsToCatch", "1", "--unixPath", scsPath);
    av::Process task(taskBin);

    bool matched = false;
    std::string line;
    while (task.out().getLine(line)) {
        matched |= (line.find(expectedMessage) != std::string::npos);
    }

    // SCS will not exit unless we interrupt it.
    scs.kill();

    // Return true if we saw the expected output string.
    return matched;
}

TEST(LongInitTest, LongInitTest_FailedInit)
{
    // path to Sanitizers suppression file is relative to file::workDir.
    // we should copy suppression file to the file::tmp , where other processess will be executed
    if (getenv("SANITIZER_SUPPRESSION_FILE") != NULL) {
        fs::copy(fs::current_path() / fs::path(getenv("SANITIZER_SUPPRESSION_FILE")),
                 tmpDir / fs::path(getenv("SANITIZER_SUPPRESSION_FILE")));
    }

    EXPECT_TRUE(runTest(2, "Task::initialize() timeout after 2 seconds. Terminating..."));
}

TEST(LongInitTest, LongInitTest_SuccessfulInit)
{
    EXPECT_TRUE(runTest(7, "LongInitTask::initialize end"));
}
