/**
 * \copyright
 * Copyright (c) 2020- Motional. All rights reserved.
 * NOTICE: Removing this heading is prohibited.
 * All information contained herein is, and remains the property of Motional and its suppliers, if any.
 * The intellectual and technical concepts contained herein are proprietary to Motional and its suppliers and may be
 * covered by U.S. and Foreign Patents, patents in process, and are protected by trade secret or copyright law.
 * Dissemination of this information or reproduction of this material is strictly forbidden unless prior written
 * permission is obtained from Motional.
 *
 * \brief Defines a task whose execution is infinite
 */

#ifndef INFINITE_EXE_TASK_TEST_H
#define INFINITE_EXE_TASK_TEST_H

#include <task/Task.h>

#include <string>

/**
 * \brief This implements a testing class for Task
 */
class InfiniteExeTaskTest : public task::Task
{
public:
    /**
     * \brief Default constructor, copy and assignment constructors are deleted
     */
    InfiniteExeTaskTest() = delete;
    InfiniteExeTaskTest(const InfiniteExeTaskTest&) = delete;
    InfiniteExeTaskTest& operator=(const InfiniteExeTaskTest&) = delete;
    InfiniteExeTaskTest(const InfiniteExeTaskTest&&) = delete;
    InfiniteExeTaskTest& operator=(const InfiniteExeTaskTest&&) = delete;

    /**
     * \brief User-defined constructor
     *
     * @param taskName The task name
     */
    explicit InfiniteExeTaskTest(const std::string& taskName);

    /**
     * @brief Default destructor
     */
    virtual ~InfiniteExeTaskTest() = default;

    /**
     * \brief Initializes the task
     *
     * \param Void
     *
     * \return Void
     */
    bool initialize() override;

    /**
     * \brief Execute a task cycle
     *
     * \param Void
     *
     * \return True if the execution is fine, false, otherwise
     */
    bool executive() override;

    /**
     * \brief Cleans up when task stops
     *
     * \param Void
     *
     * \return Void
     */
    void cleanup() override;

    /**
     * \brief The initialize method called by main
     *
     * \return True if initialization should continue, false to exit
     */
    bool _initialize() override;

    /**
     * \brief The executive loop called by main
     *
     * \return True if the application should continue executing, false to exit
     */
    bool _executive() override;

    /**
     * \brief The cleanup method called by main
     */
    void _cleanup() override;
};

#endif // INFINITE_EXE_TASK_TEST_H
