/**
 * \copyright
 * Copyright (c) 2020- Motional. All rights reserved.
 * NOTICE: Removing this heading is prohibited.
 * All information contained herein is, and remains the property of Motional and its suppliers, if any.
 * The intellectual and technical concepts contained herein are proprietary to Motional and its suppliers and may be
 * covered by U.S. and Foreign Patents, patents in process, and are protected by trade secret or copyright law.
 * Dissemination of this information or reproduction of this material is strictly forbidden unless prior written
 * permission is obtained from Motional.
 *
 * \brief Defines a mocked object for TaskApplication class
 */

#ifndef MOCK_TASK_APPLICATION_H
#define MOCK_TASK_APPLICATION_H

#include <task/ClosingSignalHandlerInterface.h>
#include <task/CmdLineOptProcessorInterface.h>
#include <task/Task.h>
#include <task/TaskApplicationInterface.h>
#include <task/VersionPrinterInterface.h>

#include <memory>

#include <gmock/gmock.h>
#include <gtest/gtest.h>

namespace task {

/**
 * \brief This class defines a mock version of TaskApplication
 */
template<bool InitTaskResult, bool RunTaskResult>
class MockTaskApplication : public TaskApplicationInterface
{
public:
    /**
     * \brief User-defined constructor
     *
     * \param closingSignalHandler The closing signal handler
     * \param cmdLineOptProcessor The command line options processor
     * \param versionPrinter The version printer
     * \param task The task implementation
     */
    explicit MockTaskApplication(std::unique_ptr<ClosingSignalHandlerInterface> /*closingSignalHandler*/,
                                 std::unique_ptr<CmdLineOptProcessorInterface> /*cmdLineOptProcessor*/,
                                 std::shared_ptr<VersionPrinterInterface> /*versionPrinter*/,
                                 Task* /*task*/)
        : TaskApplicationInterface()
    {}

    /**
     * \brief Initialize the task application
     *
     * \param argc The number of arguments
     * \param argv The list of arguments
     *
     * \return True if the creation is fine, false, otherwise
     */
    inline bool initTask(int /*argc*/, char** /*argv*/) override { return InitTaskResult; }

    /**
     * \brief Run the task application
     *
     * \param Void
     *
     * \return True if the running is fine, false, otherwise
     */
    inline bool runTask() override { return RunTaskResult; }
};

} // namespace task

#endif // MOCK_TASK_APPLICATION_H
