/**
 * \copyright
 * Copyright (c) 2021- Motional. All rights reserved.
 * NOTICE: Removing this heading is prohibited.
 * All information contained herein is, and remains the property of Motional and its suppliers, if any.
 * The intellectual and technical concepts contained herein are proprietary to Motional and its suppliers and may be
 * covered by U.S. and Foreign Patents, patents in process, and are protected by trade secret or copyright law.
 * Dissemination of this information or reproduction of this material is strictly forbidden unless prior written
 * permission is obtained from Motional.
 *
 * \brief Defines a mocked version of the monitoring status publisher
 */

#ifndef MOCK_AVTASK_STATUS_PUBLISHER_H
#define MOCK_AVTASK_STATUS_PUBLISHER_H

#include <interfaces/AvTaskStatus/AvTaskStatus.h>
#include <interfaces/Generic/Output/Abstract.h>
#include <log/Logger.h>
#include <task/GenericMessagePublisher.h>

#include <gmock/gmock.h>
#include <gtest/gtest.h>

/**
 * \brief This class defines a mock version of GenericMessagePublisher
 */
class MockAvTaskStatusPublisher
    : public task::GenericMessagePublisher<GenericOutput<av::msg::AvTaskStatus>, av::msg::AvTaskStatus>
{
public:
    /**
     * \brief The user-defined constructor
     *
     * \param avTaskStatusOut The interface for sending status
     */
    MockAvTaskStatusPublisher(const std::shared_ptr<GenericOutput<av::msg::AvTaskStatus>>& avTaskStatusOut)
        : task::GenericMessagePublisher<GenericOutput<av::msg::AvTaskStatus>, av::msg::AvTaskStatus>(avTaskStatusOut,
                                                                                                     logger::Logger())
    {}

    /**
     * \brief Mock the publishMessage method
     *
     * \param  message Message to be sent
     *
     * \return True if the message is sent
     */
    MOCK_METHOD1(publishMessage, bool(const av::msg::AvTaskStatus& message));
};

#endif // MOCK_AVTASK_STATUS_PUBLISHER_H
