/**
 * \copyright
 * Copyright (c) 2021- Motional. All rights reserved.
 * NOTICE: Removing this heading is prohibited.
 * All information contained herein is, and remains the property of Motional and its suppliers, if any.
 * The intellectual and technical concepts contained herein are proprietary to Motional and its suppliers and may be
 * covered by U.S. and Foreign Patents, patents in process, and are protected by trade secret or copyright law.
 * Dissemination of this information or reproduction of this material is strictly forbidden unless prior written
 * permission is obtained from Motional.
 *
 * \brief Defines a unit test that runs SampleSender and BatchReceiver in client-side mode to test reproducible resim.
 */
#include "resimHelpers.hpp"

#include "lib/avutils/process/Process.hpp"

#include <gtest/gtest.h>

#include <unistd.h>

#include <experimental/filesystem>
#include <iostream>
#include <map>
#include <set>
#include <string>
#include <vector>

namespace fs = std::experimental::filesystem;

namespace file {
const fs::path base = fs::path(getenv("TEST_SRCDIR")) / "av_stack/platform/middleware";
const fs::path sampleSender = base / "tasks/task/exampleTask/SampleSender";
const fs::path sampleReceiver = base / "tasks/task/exampleTask/BatchReceiver";
} // namespace file

TEST(BatchResimTest, LogAndReplay)
{
    // Set up the config files and environment variables.
    const std::string scsPath = resim::setup("system", "BatchReceiver");

    // Launch the batched executables in logging mode.
    std::vector<std::string> original = resim::runOriginal(file::sampleSender, file::sampleReceiver);
    // Launch the executables in replay mode.
    std::vector<std::string> replay = resim::runReplay(file::sampleSender, file::sampleReceiver);
    resim::compareOutputs(original, replay);
}
