/**
 * \copyright
 * Copyright (c) 2021- Motional. All rights reserved.
 * NOTICE: Removing this heading is prohibited.
 * All information contained herein is, and remains the property of Motional and its suppliers, if any.
 * The intellectual and technical concepts contained herein are proprietary to Motional and its suppliers and may be
 * covered by U.S. and Foreign Patents, patents in process, and are protected by trade secret or copyright law.
 * Dissemination of this information or reproduction of this material is strictly forbidden unless prior written
 * permission is obtained from Motional.
 *
 * \brief Defines a unit test that runs LiteSender and LiteReceiver to test the commsLite framework.
 */
#include "resimHelpers.hpp"

#include "lib/avutils/process/Process.hpp"

#include <gtest/gtest.h>

#include <unistd.h>

#include <experimental/filesystem>
#include <iostream>
#include <map>
#include <set>
#include <string>
#include <vector>

namespace fs = std::experimental::filesystem;

namespace file {
const fs::path workDir = fs::current_path();
const fs::path tmp = fs::path(getenv("TEST_TMPDIR"));
const fs::path base = fs::path(getenv("TEST_SRCDIR")) / "av_stack/platform/middleware";
const fs::path scs = base / "tasks/SimpleComms/SimpleComms/scs";
const fs::path sampleSender = base / "tasks/task/exampleTask/LiteSender";
const fs::path sampleReceiver = base / "tasks/task/exampleTask/LiteReceiver";
} // namespace file

TEST(LiteResimTest, run)
{
    // Set up the config files and environment variables.
    const std::string scsPath = resim::setSCSPath();

    // Launch SCS and the receiver.
    av::Process scs(file::scs, "--maxExitSignalsToCatch", "1", "--unixPath", scsPath);
    av::Process receiver(file::sampleReceiver);

    // Wait for the receiver to be ready.
    bool initialized = false;
    std::string line;
    while (!initialized && receiver.out().getLine(line)) {
        initialized |= (line.find("Waiting for input.") != std::string::npos);
    }
    ASSERT_TRUE(initialized);

    // Launch the sender.
    av::Process sender(file::sampleSender);

    // Read the "received" messages, and make sure they're all there.
    int count = 0;
    while (receiver.out().getLine(line)) {
        static const std::string prefix = "Receiver received ";
        const std::string expected = prefix + std::to_string(count);
        if (line.find(expected) != std::string::npos) {
            ++count;
        }
    }
    EXPECT_EQ(count, 100);

    // SCS will not exit until it is interrupted.
    scs.kill();
}
