/**
 * \copyright
 * Copyright (c) 2021- Motional. All rights reserved.
 * NOTICE: Removing this heading is prohibited.
 * All information contained herein is, and remains the property of Motional and its suppliers, if any.
 * The intellectual and technical concepts contained herein are proprietary to Motional and its suppliers and may be
 * covered by U.S. and Foreign Patents, patents in process, and are protected by trade secret or copyright law.
 * Dissemination of this information or reproduction of this material is strictly forbidden unless prior written
 * permission is obtained from Motional.
 *
 * \brief Defines a unit test that runs SCS, SampleSender, SampleReceiver, and Replayer to test reproducible resim.
 */
#include "resimHelpers.hpp"

#include "lib/avutils/process/Process.hpp"

#include <gtest/gtest.h>

#include <experimental/filesystem>
#include <iostream>
#include <map>
#include <set>
#include <string>
#include <vector>

namespace fs = std::experimental::filesystem;

namespace file {
const fs::path base = fs::path(getenv("TEST_SRCDIR")) / "av_stack/platform/middleware";
const fs::path scs = base / "tasks/SimpleComms/SimpleComms/scs";
const fs::path sampleSender = base / "tasks/task/exampleTask/SampleSender";
const fs::path sampleReceiver = base / "tasks/task/exampleTask/SampleReceiver";
const fs::path replayer = base / "tasks/task/exampleTask/Replayer";
} // namespace file

TEST(ResimTest, LogAndReplay)
{
    // Path to the messages.db log:
    const std::string LOG = "messages.db";

    // Set up the config files and environment variables.
    const std::string scsPath = resim::setup("central");

    // Launch the non-batched executables in logging mode.
    std::vector<std::string> original = resim::runOriginal(file::sampleSender, file::sampleReceiver, LOG);
    // Launch the executables in client-side replay mode.
    std::vector<std::string> replay = resim::runReplay(file::sampleSender, file::sampleReceiver);
    resim::compareOutputs(original, replay);

    // Launch the executables in server-side replay mode.
    replay.clear();
    {
        av::Process scs(file::scs, "--maxExitSignalsToCatch", "1", "--unixPath", scsPath, "--replay", LOG);
        // In server-side mode it's safe to launch all the executables simultaneously.
        av::Process sender(file::sampleSender);
        av::Process receiver(file::sampleReceiver);
        av::Process replayer(file::replayer, LOG);

        // Store all the lines that report received messages.
        std::string line;
        while (receiver.out().getLine(line)) {
            if (line.find("Receiver received") != std::string::npos) {
                replay.push_back(line);
            }
        }
        // The sender and SCS must be manually killed.
        sender.kill();
        scs.kill();
    }
    resim::compareOutputs(original, replay);
}
