/// @copyright Copyright (c) 2021- Motional. All rights reserved.
///
/// NOTICE: Removing this heading is prohibited.
///
/// All information contained herein is, and remains the property of Motional,
/// its affiliates, and its suppliers (if any). The intellectual and technical
/// concepts contained herein are proprietary to Motional, its affiliates, and
/// its suppliers (if any) and may be covered by U.S. and Foreign Patents,
/// patents in process, and are protected by trade secrets or copyright law.
/// Dissemination of this information or reproduction of this material is
/// strictly forbidden unless prior written permission is obtained from Motional.
///
/// @file
/// @brief Declarations of resim test helper functions.
#pragma once

#include <sys/types.h>

#include <map>
#include <set>
#include <string>
#include <vector>

namespace resim {

/// Set up the environment variables and config files, and return the SCS path.
std::string setup(const std::string& clock, const std::string& receiver = "SampleReceiver");
/// Set a random SCS path, and return the path.
std::string setSCSPath();

/// Launch the original test run for the given sender and receiver, and get the receiver's output.
std::vector<std::string> runOriginal(const std::string& sampleSender,
                                     const std::string& sampleReceiver,
                                     const std::string& messageLog = "");
/// Launch a replay test run for the given sender and receiver, and get the receiver's output.
std::vector<std::string> runReplay(const std::string& sampleSender, const std::string& sampleReceiver);
/// Compare the two given sets of output lines.
void compareOutputs(const std::vector<std::string>& original, const std::vector<std::string>& replay);

} // namespace resim
