#include <gtest/gtest-param-test.h>
#include <gtest/gtest.h>

#include <interfaces/Point/Point.h>
#include <interfaces/SampleMessage/SampleMessage.h>
#include <interfaces/baseTypes/Header.h>
#include <task/MessageCallback.h>

#include "MockReceiver.h"

#include <exception>
#include <memory>

TEST(MessageCallbackTest, testCallbackOnInputNoThrow)
{
    Receiver<av::msg::SampleMessage> receiver;
    auto callbackPtr = std::make_unique<task::MessageCallbackT<Receiver<av::msg::SampleMessage>, ReceiverFunction>>(
        &receiver, &Receiver<av::msg::SampleMessage>::receive);

    // Prepare the data
    std::string blob;
    av::msg::SampleMessage payload;
    payload.value = 12345;
    interfaces::baseTypes::InterfaceHeader header;
    header.headerVersion = htonl(OTTO_INTERFACE_HEADER_VERSION);
    aptv::serializeODatumMessage(std::string(reinterpret_cast<char*>(&header), sizeof(header)), blob, payload);

    ASSERT_NO_THROW((*callbackPtr)(blob));
    EXPECT_EQ(receiver.getNumReceivedData(), 1);
    EXPECT_EQ(receiver.received[0].value, payload.value);
}

TEST(MessageCallbackTest, testCallbackOnNoInputNoThrow)
{
    ReceiverNoInput<av::msg::SampleMessage> receiver;
    auto callbackPtr =
        std::make_unique<task::MessageCallbackT<ReceiverNoInput<av::msg::SampleMessage>, ReceiverNoInputFunction>>(
            &receiver, &ReceiverNoInput<av::msg::SampleMessage>::receive);

    ASSERT_NO_THROW((*callbackPtr)(""));
    EXPECT_EQ(receiver.m_received, 1);
}

TEST(MessageCallbackTest, testCallbackOnBadDataThrows)
{
    Receiver<av::msg::SampleMessage> receiver;
    auto callbackPtr = std::make_unique<task::MessageCallbackT<Receiver<av::msg::SampleMessage>, ReceiverFunction>>(
        &receiver, &Receiver<av::msg::SampleMessage>::receive);

    const std::string blob{"blob"};
    EXPECT_THROW((*callbackPtr)(blob), std::exception);

    // Send a blob that contains data of different type
    // Prepare the data
    std::string blobAux;
    av::Point in;
    std::string header = "header";
    aptv::serializeODatumMessage(header, blobAux, in);
    EXPECT_THROW((*callbackPtr)(blobAux), std::exception);
}
