/**
 * \copyright
 * Copyright (c) 2020- Motional. All rights reserved.
 * NOTICE: Removing this heading is prohibited.
 * All information contained herein is, and remains the property of Motional and its suppliers, if any.
 * The intellectual and technical concepts contained herein are proprietary to Motional and its suppliers and may be
 * covered by U.S. and Foreign Patents, patents in process, and are protected by trade secret or copyright law.
 * Dissemination of this information or reproduction of this material is strictly forbidden unless prior written
 * permission is obtained from Motional.
 *
 * \brief Defines a mocked object for backup recovery handler class
 */

#ifndef MOCK_BACKUP_RECOVERY_HANDLER_H
#define MOCK_BACKUP_RECOVERY_HANDLER_H

#include <task/BackupRecoveryHandlerInterface.h>

#include <gmock/gmock.h>
#include <gtest/gtest.h>

namespace task {

/**
 * \brief This class defines a mock version of BackupRecoveryHandlerInterface
 */
class MockBackupHandler : public BackupRecoveryHandlerInterface
{
public:
    /**
     * \brief Initialize the handler such as the interfaces initialization
     *
     *
     * \return True if the initialization is fine, false, otherwise
     */
    MOCK_METHOD0(initialize, bool());

    /**
     * \brief Handler the backup recovery requested from Safer
     *
     * \param[in] writeOnlyInterfaces The task's list of write only interface configs
     * \param[in] readWriteInterfaces The task's list of read/write interface configs
     * \param[in,out] handlingParams The recovery handling parameters
     * \param[in,out] rawPtrInterfaces Raw pointer interfaces map
     * \param[in,out] sharedPtrInterfaces Shared pointer interfaces map
     *
     * \return True if the handling is fine, false, otherwise
     */
    MOCK_METHOD5(handleBackupRecovery,
                 bool(ConfigSectionList& writeOnlyInterfaces,
                      ConfigSectionList& readWriteInterfaces,
                      BackupRecoveryParameter& handlingParams,
                      std::map<std::string, IDbObject*>& rawPtrInterfaces,
                      std::map<std::string, std::shared_ptr<IDbObject>>& sharedPtrInterfaces));

    /**
     * \brief Sends the given message to the Safer daemon
     *
     * \param reportMsg The message to be sent
     *
     * \return True if the message was sent, false, otherwise
     */
    MOCK_METHOD1(sendSaferResponse, bool(const std::string& reportMsg));
};

} // namespace task

#endif // MOCK_BACKUP_RECOVERY_HANDLER_H
