/**
 * \copyright
 * Copyright (c) 2020- Motional. All rights reserved.
 * NOTICE: Removing this heading is prohibited.
 * All information contained herein is, and remains the property of Motional and its suppliers, if any.
 * The intellectual and technical concepts contained herein are proprietary to Motional and its suppliers and may be
 * covered by U.S. and Foreign Patents, patents in process, and are protected by trade secret or copyright law.
 * Dissemination of this information or reproduction of this material is strictly forbidden unless prior written
 * permission is obtained from Motional.
 *
 * \brief Defines a mocked object for heartbeat sender class
 */

#ifndef MOCK_HEARTBEAT_SENDER_H
#define MOCK_HEARTBEAT_SENDER_H

#include <task/HeartbeatSenderInterface.h>

#include <gmock/gmock.h>
#include <gtest/gtest.h>

namespace task {

/**
 * \brief This class defines a mock version of HeartbeatSenderInterface
 */
class MockHeartbeatSender : public HeartbeatSenderInterface
{
public:
    /**
     * \brief Set the heartbeat with the given message
     *
     * \param hbMessage The message to be set
     *
     * \return Void
     */
    MOCK_METHOD1(setHeartbeatMessage, void(const ProcessHealthMessage& hbMessage));

    /**
     * \brief Get the current heartbeat message
     *
     * \param Void
     *
     * \return The current heartbeat content
     */
    MOCK_CONST_METHOD0(getHeartbeatMessage, const ProcessHealthMessage&());

    /**
     * \brief Send a heartbeat with current message including the actual AV task process frequency calculated by the
     * heartbeat sender
     *
     * \param Void
     *
     * \return True if the heartbeat was successfully sent
     */
    MOCK_METHOD0(sendHeartbeat, bool());

    /**
     * \brief Check if a heartbeat needs to be sent, for example, the heartbeat sending cycle is expired. By doing that,
     * it also calculates the actual frequency of the AV task process every cycle time of the heartbeat sender in
     * seconds
     *
     * \param Void
     *
     * \return True if the heartbeat needs to be sent
     */
    MOCK_METHOD0(needToSendHeartbeat, bool());
};

} // namespace task

#endif // MOCK_HEARTBEAT_SENDER_H
