/**
 * \copyright
 * Copyright (c) 2021- Motional. All rights reserved.
 * NOTICE: Removing this heading is prohibited.
 * All information contained herein is, and remains the property of Motional, its affiliates, and its suppliers (if
 * any). The intellectual and technical concepts contained herein are proprietary to Motional, its affiliates, and its
 * suppliers (if any) and may be covered by U.S. and Foreign Patents, patents in process, and are protected by trade
 * secrets or copyright law. Dissemination of this information or reproduction of this material is strictly forbidden
 * unless prior written permission is obtained from Motional.
 *
 * \file
 * \brief Definition of a Mock receiver class.
 */
#pragma once

#include <interfaces/SampleMessage/SampleMessage.h>

#include <condition_variable>
#include <cstddef>
#include <mutex>
#include <vector>

template<class Type>
struct Receiver
{
    std::vector<Type> received;
    std::mutex m_mutex;
    std::condition_variable m_condition;
    int m_numExpectedMsgs{1};

    void receive(const Type& data)
    {
        {
            std::unique_lock<std::mutex> lock(m_mutex);
            received.push_back(data);
        }
        m_condition.notify_all();
    }

    int getNumReceivedData() const noexcept { return static_cast<int>(received.size()); }
    void setExpectMsgs(int numMsgs) { m_numExpectedMsgs = numMsgs; }

    void wait()
    {
        std::unique_lock<std::mutex> lock(m_mutex);
        m_condition.wait(lock, [this] { return static_cast<int>(received.size()) >= m_numExpectedMsgs; });
    }
};

template<class Type>
struct ReceiverNoInput
{
    int m_received{0};
    std::mutex m_mutex;
    std::condition_variable m_condition;
    int m_numExpectedMsgs{1};

    void receive()
    {
        {
            std::unique_lock<std::mutex> lock(m_mutex);
            ++m_received;
        }
        m_condition.notify_all();
    }
    void setExpectMsgs(int numMsgs) { m_numExpectedMsgs = numMsgs; }

    void wait()
    {
        std::unique_lock<std::mutex> lock(m_mutex);
        m_condition.wait(lock, [this] { return m_received >= m_numExpectedMsgs; });
    }
};

using ReceiverFunction = void (Receiver<av::msg::SampleMessage>::*)(const av::msg::SampleMessage&);
using ReceiverNoInputFunction = void (ReceiverNoInput<av::msg::SampleMessage>::*)();
