/// @copyright Copyright (c) 2021- Motional. All rights reserved.
///
/// NOTICE: Removing this heading is prohibited.
///
/// All information contained herein is, and remains the property of Motional,
/// its affiliates, and its suppliers (if any). The intellectual and technical
/// concepts contained herein are proprietary to Motional, its affiliates, and
/// its suppliers (if any) and may be covered by U.S. and Foreign Patents,
/// patents in process, and are protected by trade secrets or copyright law.
/// Dissemination of this information or reproduction of this material is
/// strictly forbidden unless prior written permission is obtained from Motional.
///
/// @file
/// @brief Testing functionality for IDbObject

#pragma once

#include <Comms/Access.h>
#include <Comms/InterfaceDb.h>

#include <gmock/gmock.h>

namespace task {
namespace TestUtils {

/**
 * \brief This class defines a fake version of IDbObject which keeps some internal state.
 */
class FakeIDbObject : public task::IDbObject
{
public:
    /**
     * \brief The default constructor
     */
    FakeIDbObject() = default;

    /**
     * \brief The user constructor
     */
    FakeIDbObject(const task::Access& acc)
        : accessMode_(acc){};

    /**
     * \brief The default destructor
     */
    virtual ~FakeIDbObject() = default;

    /**
     * \brief Change the access mode of the interface to the given mode
     * \param acc The new mode to be changed
     * \return Void
     */
    inline void chmod(const task::Access& acc) override { accessMode_ = acc; }

    /**
     * \brief Get the access mode of the interface
     * \return the current access mode
     */
    inline task::Access getAccessMode() { return accessMode_; }

    MOCK_METHOD0(initialize, bool());
    MOCK_METHOD0(cleanup, void());

private:
    task::Access accessMode_{task::Access::Read}; ///< The interface access mode
};

} // namespace TestUtils
} // namespace task