/**
 * \copyright
 * Copyright (c) 2020- Motional. All rights reserved.
 * NOTICE: Removing this heading is prohibited.
 * All information contained herein is, and remains the property of Motional and its suppliers, if any.
 * The intellectual and technical concepts contained herein are proprietary to Motional and its suppliers and may be
 * covered by U.S. and Foreign Patents, patents in process, and are protected by trade secret or copyright law.
 * Dissemination of this information or reproduction of this material is strictly forbidden unless prior written
 * permission is obtained from Motional.
 *
 * \brief Defines some mock classes for writing unit tests for the
 * task framework.
 */

#ifndef TASK_FRAMEWORK_TESTUTILS_MOCKOBJECTS_H
#define TASK_FRAMEWORK_TESTUTILS_MOCKOBJECTS_H

#include <task/ClosingSignalHandlerInterface.h>
#include <task/CmdLineOptProcessor.h>
#include <task/Task.h>
#include <task/VersionPrinterInterface.h>

#include <gmock/gmock.h>

namespace task {
namespace TestUtils {
class MockVersionPrinter : public VersionPrinterInterface
{
public:
    MOCK_METHOD2(getVehicleName, std::string(const std::string& configFile, const bool logMode));
    MOCK_METHOD3(getAVStackBuildNumber,
                 AVStackBuildNumber(const std::string& mode, const std::string& buildTagFile, const bool logMode));
};

class MockClosingSignalHandler : public ClosingSignalHandlerInterface
{
public:
    MOCK_CONST_METHOD0(shouldExit, bool());
    MOCK_CONST_METHOD1(wait, void(std::optional<uint64_t>));
    MOCK_METHOD0(quit, void());
    MOCK_METHOD1(setSignals, bool(const std::vector<int>&));
    MOCK_METHOD0(handleClosingSignals, bool());
};

class MockCmdLineOptProcessor : public CmdLineOptProcessorInterface
{
public:
    MOCK_METHOD2(parseCmdLine, CmdLineOptMap(int, char**));
    MOCK_METHOD2(addOption, bool(const option&, const std::string&));
    MOCK_METHOD0(addDefaultOptions, bool());
    MOCK_METHOD0(clearOptions, void());
};

class MockTask : public Task
{
public:
    MockTask()
        : Task("mock")
    {}

    bool _initialize();

    MOCK_METHOD0(initialize, bool());
    MOCK_METHOD0(executive, bool());
    MOCK_METHOD0(cleanup, void());
};
} // namespace TestUtils
} // namespace task
#endif // TASK_FRAMEWORK_TESTUTILS_MOCKOBJECTS_H
