/**
 * \copyright
 * Copyright (c) 2022- Motional. All rights reserved.
 * NOTICE: Removing this heading is prohibited.
 * All information contained herein is, and remains the property of Motional and its suppliers, if any.
 * The intellectual and technical concepts contained herein are proprietary to Motional and its suppliers and may be
 * covered by U.S. and Foreign Patents, patents in process, and are protected by trade secret or copyright law.
 * Dissemination of this information or reproduction of this material is strictly forbidden unless prior written
 * permission is obtained from Motional.
 *
 * \brief unit tests for TaskTimeProvider
 */

#include "platform/middleware/libs/task/include/task/TaskTime.hpp"
#include <avtime/ConversionsTimespec.hpp>
#include <avtime/Time.hpp>
#include <gtest/gtest.h>
#include <thread>

namespace {

using namespace task::time;

ottotime::otimePoint makeOtimePoint(const timespec& ts)
{
    return ottotime::otimePoint{ts};
}

TEST(TaskTime, TaskTimetest)
{
    EXPECT_NO_THROW(static_cast<void>(TaskTimeProvider::getSystemOTime()));
}

void testTime(const TaskTimeProvider::OTimeProvider& provider)
{
    auto p1 = provider();
    std::this_thread::sleep_for(std::chrono::microseconds(10));
    auto p2 = av::timespecFromTimePoint(av::AvClock::now());
    ASSERT_LT(p1, p2);
}

TEST(TaskTime, AvTimeProviderFunctorTest)
{
    detail::AvTimeProvider provider{};
    testTime(provider);
}

TEST(TaskTime, SystemClockProviderFunctorTest)
{
    detail::SystemClockProvider provider{};
    testTime(provider);
}

TEST(TaskTime, SetGetStatictime)
{
    TaskTimeProvider::setTimeSource([]() -> ottotime::otimePoint { return makeOtimePoint({100, 200}); });

    ASSERT_TRUE(makeOtimePoint({100, 200}) == TaskTimeProvider::getSystemOTime());
}

TEST(TaskTime, SetGetStaticAvtime)
{
    TaskTimeProvider::setTimeSource([]() -> ottotime::otimePoint { return makeOtimePoint({4, 0}); });

    ASSERT_TRUE(av::Duration(std::chrono::seconds(4)) == TaskTimeProvider::getSystemTime());
}

TEST(TaskTime, SetGetIncrementingTime)
{
    TaskTimeProvider::setTimeSource(
        []() -> ottotime::otimePoint { return av::timespecFromTimePoint(av::AvClock::now()); });

    auto p1 = TaskTimeProvider::getSystemOTime();
    std::this_thread::sleep_for(std::chrono::microseconds(10));
    auto p2 = TaskTimeProvider::getSystemOTime();
    std::this_thread::sleep_for(std::chrono::microseconds(10));
    auto p3 = TaskTimeProvider::getSystemOTime();
    ASSERT_LE(p1, p2);
    ASSERT_LE(p2, p3);
}

TEST(TaskTime, RepeatSetSource)
{
    for (uint32_t i = 1; i < 50; ++i) {
        timespec ts{100 * i, 200 * i};
        TaskTimeProvider::setTimeSource([&ts]() -> ottotime::otimePoint { return makeOtimePoint(ts); });
        ASSERT_TRUE(makeOtimePoint(ts) == TaskTimeProvider::getSystemOTime());
        ASSERT_TRUE(makeOtimePoint(ts) == TaskTimeProvider::getSystemOTime());
    }
}

} // namespace