/**
 * \copyright
 * Copyright (c) 2020- Motional. All rights reserved.
 * NOTICE: Removing this heading is prohibited.
 * All information contained herein is, and remains the property of Motional and its suppliers, if any.
 * The intellectual and technical concepts contained herein are proprietary to Motional and its suppliers and may be
 * covered by U.S. and Foreign Patents, patents in process, and are protected by trade secret or copyright law.
 * Dissemination of this information or reproduction of this material is strictly forbidden unless prior written
 * permission is obtained from Motional.
 *
 * \brief Defines a mock version of IDbObject
 */

#ifndef MOCK_IDB_OBJECT_H
#define MOCK_IDB_OBJECT_H

#include <Comms/Access.h>
#include <Comms/InterfaceDb.h>

/**
 * \brief This class defines a mock version of IDbObject
 */
class MockIDbObject : public task::IDbObject
{
public:
    /**
     * \brief The default constructor
     */
    MockIDbObject() = default;

    /**
     * \brief The default destructor
     */
    virtual ~MockIDbObject() = default;

    /**
     * \brief Initialize the interface
     *
     * \param Void
     *
     * \return True if the initialize is fine, otherwise, false
     */
    inline bool initialize() override { return true; }

    /**
     * \brief Cleanup the interface
     *
     * \param Void
     *
     * \return Void
     */
    inline void cleanup() override {}

    /**
     * \brief Change the access mode of the interface to the given mode
     *
     * \param acc The new mode to be changed
     *
     * \return Void
     */
    inline void chmod(const task::Access& acc) override { accessMode_ = acc; }

private:
    task::Access accessMode_{task::Access::Read}; ///< The interface access mode
};

#endif // MOCK_IDB_OBJECT_H
