/*!
 ***********************************************************************************************************************
 * APTIV PRIVILEGED AND CONFIDENTIAL
 *
 * @file CalibrationConversion.cc
 * @product common
 * @version 1.0
 * @released TODO
 * @updated 2019-04-01
 * @brief Provides functions for converting between ottoware and nutonomy calibration frames
 *
 * @section License
 *
 * Copyright (c) 2019- Aptiv. All rights reserved.
 * NOTICE: Removing this heading is prohibited.
 * All information contained herein is, and remains the property of Aptiv and its suppliers, if any.
 * The intellectual and technical concepts contained herein are proprietary to Aptiv and its suppliers and may be
 * covered by U.S. and Foreign Patents, patents in process, and are protected by trade secret or copyright law.
 * Dissemination of this information or reproduction of this material is strictly forbidden unless prior written
 * permission is obtained from Aptiv.
 *
 * @section Open Source License List
 *
 * This is a list of any open-source project used by this file.
 ***********************************************************************************************************************
 */

#ifndef CALIBRATIONCONVERSION_H
#define CALIBRATIONCONVERSION_H

#include <Eigen/Geometry>
#include <array>

namespace utils {

struct XyzRpyInDeg
{
    double x;
    double y;
    double z;
    double roll;
    double pitch;
    double yaw;
};

Eigen::Matrix3d getMat3FromYPRInDeg(double yawInDeg, double pitchInDeg, double rollInDeg);

Eigen::Matrix4d getMat4FromYPRXYZ(const Eigen::Matrix3d& ypr, const Eigen::Vector3d& xyz);

const XyzRpyInDeg convertCalibrationBetweenNutOtto(const XyzRpyInDeg& xyzRpyInDeg);

static const Eigen::Matrix4d nutOriginOnOttoOrigin{
    getMat4FromYPRXYZ(getMat3FromYPRInDeg(0.0, 0.0, 180.0), Eigen::Vector3d{0.0, 0.0, 0.37})};
static const Eigen::Matrix4d ottoLidarOnNutLidar{
    getMat4FromYPRXYZ(getMat3FromYPRInDeg(-90.0, 0.0, 180.0), Eigen::Vector3d{0.0, 0.0, 0.0})};

} // namespace utils

#endif // CALIBRATIONCONVERSION_H
