#include <utility/MessageCallback.hpp>

namespace task {
namespace stats {
std::optional<MessageCallbackID> MessageCallbackRegistry::registerCallback(MessageCallback callback)
{
    std::unique_lock<std::mutex> lock(mutex_);
    for (MessageCallbackID id = 0; id < s_maxCallbacks; ++id) {
        if (!callbackIdsInUse_[id]) {
            callbackStorage_[id] = callback;
            callbacks_[id] = &(callbackStorage_[id]);
            callbackIdsInUse_[id] = true;
            return id;
        }
    }

    return std::nullopt;
}

void MessageCallbackRegistry::unregisterCallback(MessageCallbackID id)
{
    std::unique_lock<std::mutex> lock(mutex_);
    callbackIdsInUse_[id] = false;
    callbackStorage_[id] = [](const MessageReport&) {};
    callbacks_[id] = nullptr;
}

void MessageCallbackRegistry::invokeCallbacks(const MessageReport& report)
{
    for (const auto& atom : callbacks_) {
        const auto callbackOpt = atom.load();
        if (callbackOpt) {
            (*callbackOpt)(report);
        }
    }
}

} // namespace stats
} // namespace task
