/*!
***********************************************************************************************************************
* APTIV PRIVILEGED AND CONFIDENTIAL
*
* @file hash.h
* @product Core
* @version
* @released
* @updated
* @brief hashing helpers
*
* This file contains helper functions for hashing
*
* @section License
*
* Copyright (c) 2020- Aptiv. All rights reserved.
* NOTE: Removing this heading is prohibited.
***********************************************************************************************************************
*/

#ifndef UTILITY_HASH_H_
#define UTILITY_HASH_H_

#include <cstdint>

namespace utility {
namespace hash {

// boost::hash_combine
template<class T>
inline void HashCombine(std::size_t& seed, const T& v)
{
    std::hash<T> hasher;
    seed ^= hasher(v) + 0x9e3779b9 + (seed << 6) + (seed >> 2);
}

// Thomas Wang 64 bit to 32 bit Hash Functions
// http://web.archive.org/web/20071223173210/http://www.concentric.net/~Ttwang/tech/inthash.htm
inline uint32_t Hash32From64(uint64_t key)
{
    key = (~key) + (key << 18);
    key = key ^ (key >> 31);
    key = key * 21;
    key = key ^ (key >> 11);
    key = key + (key << 6);
    key = key ^ (key >> 22);

    return static_cast<uint32_t>(key);
}

} // namespace hash
} // namespace utility

#endif // UTILITY_HASH_H_