#include "../CalibrationConversion.h"
#include <gmock/gmock.h>
#include <gtest/gtest.h>

namespace utils {
// Calibration values taken from mvp-8
constexpr XyzRpyInDeg NutLidarCARF{1.55282, 0, 2.0475, 0.189272, 0.151934, -90.8246};
constexpr XyzRpyInDeg NutLidarLAML{2.517, 0.860, 1.442, 0.978, -0.245, -47.397};
constexpr XyzRpyInDeg NutLidarRAMR{2.468, -0.825, 1.421, 0.822, 0.968, -137.723};
constexpr XyzRpyInDeg NutLidarCFBF{4.05, 0.019, 0.386, -5.549, -0.044, -91.632};
constexpr XyzRpyInDeg NutLidarCRMB{-1.054, -0.237, 0.918, -2.224, -0.477, 87.152};

constexpr XyzRpyInDeg OttoLidarCARF{1.552820, -0.000000, -1.677500, -0.151935, -0.189271, -179.174898};
constexpr XyzRpyInDeg OttoLidarLAML{2.517000, -0.860000, -1.072000, 0.245036, -0.977991, 137.392818};
constexpr XyzRpyInDeg OttoLidarRAMR{2.468000, 0.825000, -1.051000, -0.968100, -0.821883, -132.263112};
constexpr XyzRpyInDeg OttoLidarCFBF{4.050000, -0.019000, -0.016000, 0.044207, 5.548998, -178.363725};
constexpr XyzRpyInDeg OttoLidarCRMB{-1.054000, 0.237000, -0.548000, 0.477360, 2.223923, 2.866524};

void validateCalibrationValues(const XyzRpyInDeg& a, const XyzRpyInDeg& b)
{
    ASSERT_NEAR(a.x, b.x, 1e-6);
    ASSERT_NEAR(a.y, b.y, 1e-6);
    ASSERT_NEAR(a.z, b.z, 1e-6);
    ASSERT_NEAR(a.roll, b.roll, 1e-6);
    ASSERT_NEAR(a.pitch, b.pitch, 1e-6);
    ASSERT_NEAR(a.yaw, b.yaw, 1e-6);
}

void validateRoundTrip(const XyzRpyInDeg& a)
{
    const auto bb = utils::convertCalibrationBetweenNutOtto(a);
    const auto aa = utils::convertCalibrationBetweenNutOtto(bb);
    validateCalibrationValues(a, aa);
}

TEST(NutToOttoTest, NutToOttoForward)
{
    validateCalibrationValues(utils::convertCalibrationBetweenNutOtto(NutLidarCARF), OttoLidarCARF);
    validateCalibrationValues(utils::convertCalibrationBetweenNutOtto(NutLidarLAML), OttoLidarLAML);
    validateCalibrationValues(utils::convertCalibrationBetweenNutOtto(NutLidarRAMR), OttoLidarRAMR);
    validateCalibrationValues(utils::convertCalibrationBetweenNutOtto(NutLidarCFBF), OttoLidarCFBF);
    validateCalibrationValues(utils::convertCalibrationBetweenNutOtto(NutLidarCRMB), OttoLidarCRMB);
}

TEST(NutToOttoTest, NutToOttoBack)
{
    validateCalibrationValues(utils::convertCalibrationBetweenNutOtto(OttoLidarCARF), NutLidarCARF);
    validateCalibrationValues(utils::convertCalibrationBetweenNutOtto(OttoLidarLAML), NutLidarLAML);
    validateCalibrationValues(utils::convertCalibrationBetweenNutOtto(OttoLidarRAMR), NutLidarRAMR);
    validateCalibrationValues(utils::convertCalibrationBetweenNutOtto(OttoLidarCFBF), NutLidarCFBF);
    validateCalibrationValues(utils::convertCalibrationBetweenNutOtto(OttoLidarCRMB), NutLidarCRMB);
}

TEST(NutToOttoTest, NutRoundTrip)
{
    validateRoundTrip(NutLidarCARF);
    validateRoundTrip(NutLidarLAML);
    validateRoundTrip(NutLidarRAMR);
    validateRoundTrip(NutLidarCFBF);
    validateRoundTrip(NutLidarCRMB);
}

TEST(NutToOttoTest, OttoRoundTrip)
{
    validateRoundTrip(OttoLidarCARF);
    validateRoundTrip(OttoLidarLAML);
    validateRoundTrip(OttoLidarRAMR);
    validateRoundTrip(OttoLidarCFBF);
    validateRoundTrip(OttoLidarCRMB);
}
} // namespace utils
