/// @copyright Copyright (c) 2021- Motional. All rights reserved.
///
/// NOTICE: Removing this heading is prohibited.
///
/// All information contained herein is, and remains the property of Motional,
/// its affiliates, and its suppliers (if any). The intellectual and technical
/// concepts contained herein are proprietary to Motional, its affiliates, and
/// its suppliers (if any) and may be covered by U.S. and Foreign Patents,
/// patents in process, and are protected by trade secrets or copyright law.
/// Dissemination of this information or reproduction of this material is
/// strictly forbidden unless prior written permission is obtained from Motional.
///
/// @file
/// @brief Test addition (concatenation) between string_view and string.

#include "platform/middleware/libs/utility/StringViewUtils.hpp"

#include <gtest/gtest.h>
#include <string>
#include <string_view>

TEST(StringViewAddition, NormalStringViewLHS)
{
    std::string lhsString{"This is a "};
    std::string_view lhs{lhsString.data(), 5};
    std::string rhs{"test"};

    EXPECT_EQ(lhs + rhs, "This test");
}

TEST(StringViewAddition, EmptyStringStringViewLHS)
{
    std::string lhsString{"This is a "};
    std::string_view lhs{lhsString.data(), 5};
    std::string rhs{};

    EXPECT_EQ(lhs + rhs, "This ");
}

TEST(StringViewAddition, EmptyViewStringViewLHS)
{
    std::string_view lhs{};
    std::string rhs{"test"};

    EXPECT_EQ(lhs + rhs, "test");
}

TEST(StringViewAddition, BothEmptyStringViewLHS)
{
    std::string_view lhs{};
    std::string rhs{};

    EXPECT_EQ(lhs + rhs, "");
}

TEST(StringViewAddition, NormalStringViewRHS)
{
    std::string rhsString{"This is a "};
    std::string_view rhs{rhsString.data(), 5};
    std::string lhs{"test"};

    EXPECT_EQ(lhs + rhs, "testThis ");
}

TEST(StringViewAddition, EmptyStringStringViewRHS)
{
    std::string rhsString{"This is a "};
    std::string_view rhs{rhsString.data(), 5};
    std::string lhs{};

    EXPECT_EQ(lhs + rhs, "This ");
}

TEST(StringViewAddition, EmptyViewStringViewRHS)
{
    std::string_view rhs{};
    std::string lhs{"test"};

    EXPECT_EQ(lhs + rhs, "test");
}

TEST(StringViewAddition, BothEmptyStringViewRHS)
{
    std::string_view rhs{};
    std::string lhs{};

    EXPECT_EQ(lhs + rhs, "");
}