/*!
 ***********************************************************
 * @file utility.cc.
 * @brief A file implementing all the helper functions to replace C++ Boost
          library in Ottoware
 *
 * @section License
 *
 * Copyright (c) 2015- Delphi Automotive. All rights
 * reserved.
 *
 * @section changeLogs Change Logs
 *
 * @author Yunpeng Xu (yunpeng.xu@aptiv.com)
 * @date 04/04/2018
 * @remark add initial source code for utility
 *
 ***********************************************************
 */

#include <unistd.h>
#include <utility/utility.h>

using namespace std;

vector<string> split(const string& str, const char delim)
{
    vector<string> elems;
    split(str, delim, back_inserter(elems));
    return elems;
}

std::string escapeForCSV(const std::string& str)
{
    const size_t len = str.size();
    size_t numDoubleQuotes = 0;
    bool needsToBeEscaped = false;
    for (size_t pos = 0; pos < len; ++pos) {
        const char ch = str[pos];
        needsToBeEscaped = needsToBeEscaped || (ch == ',') || (ch == '"') || (ch == '\n');
        if (ch == '"') {
            ++numDoubleQuotes;
        }
    }

    std::string res{""};

    if (needsToBeEscaped) {
        res.reserve(len + 2 + numDoubleQuotes);
        res.push_back('"');
        for (char ch : str) {
            res.push_back(ch);
            if (ch == '"') {
                // Double the double-quotes.
                res.push_back('"');
            }
        }
        res.push_back('"');
    } else {
        res = str;
    }

    return res;
}

bool startsWith(const string& str, const string& prefix)
{
    return (str.size() >= prefix.size() && str.compare(0, prefix.size(), prefix) == 0);
}

bool endsWith(const string& str, const string& suffix)
{
    return (str.size() >= suffix.size() && str.compare(str.size() - suffix.size(), suffix.size(), suffix) == 0);
}

bool fileExists(const string& name)
{
    return (access(name.c_str(), F_OK) == 0);
}

bool isDirectory(const string& path)
{
    struct stat buffer;
    return (!path.empty() && stat(path.c_str(), &buffer) == 0 && (buffer.st_mode & S_IFDIR));
}
