// Copyright (c) 2021- Motional. All rights reserved.
//
// NOTICE: Removing this heading is prohibited.
//
// All information contained herein is, and remains the property of Motional, its affiliates, and
// its suppliers (if any). The intellectual and technical concepts contained herein are proprietary
// to Motional, its affiliates, and its suppliers (if any) and may be covered by U.S. and Foreign
// Patents, patents in process, and are protected by trade secrets or copyright law. Dissemination
// of this information or reproduction of this material is strictly forbidden unless prior written
// permission is obtained from Motional.

#pragma once

#include <chrono>
#include <ctime>

namespace av {

// A timespec type is anything that has assignable member variables `tv_sec` and
// `tv_nsec`. This will cover at least `otimeSpec` and `timespec`. Anything else
// will need another conversion.

template<typename TimespecType = timespec>
TimespecType timespecFromDuration(const std::chrono::nanoseconds duration)
{
    constexpr int64_t NSEC_PER_SEC = 1000000000;
    TimespecType ts;
    ts.tv_sec = duration.count() / NSEC_PER_SEC;
    ts.tv_nsec = duration.count() % NSEC_PER_SEC;
    return ts;
}

template<typename TimespecType = timespec, typename Clock>
TimespecType timespecFromTimePoint(const std::chrono::time_point<Clock>& timePoint)
{
    return timespecFromDuration<TimespecType>(timePoint.time_since_epoch());
}

template<typename TimespecType>
std::chrono::nanoseconds durationFromTimespec(const TimespecType& ts)
{
    constexpr int64_t NSEC_PER_SEC = 1000000000;
    return std::chrono::nanoseconds{NSEC_PER_SEC * ts.tv_sec + ts.tv_nsec};
}

template<typename Clock, typename TimespecType>
std::chrono::time_point<Clock> timePointFromTimespec(const TimespecType& ts)
{
    return std::chrono::time_point<Clock>{durationFromTimespec(ts)};
}

} // namespace av
