// Copyright (c) 2021- Motional. All rights reserved.
//
// NOTICE: Removing this heading is prohibited.
//
// All information contained herein is, and remains the property of Motional, its affiliates, and
// its suppliers (if any). The intellectual and technical concepts contained herein are proprietary
// to Motional, its affiliates, and its suppliers (if any) and may be covered by U.S. and Foreign
// Patents, patents in process, and are protected by trade secrets or copyright law. Dissemination
// of this information or reproduction of this material is strictly forbidden unless prior written
// permission is obtained from Motional.

#include "lib/avtime/AvClock.hpp"
#include <gtest/gtest.h>

#include <chrono>
#include <unordered_set>

using namespace std::chrono_literals;

namespace av {

TEST(AvClock, Now)
{
    // Just make sure this doesn't explode
    const TimePoint t = AvClock::now();
    static_cast<void>(t);
}

TEST(AvClock, HashTimePoint)
{
    EXPECT_EQ(std::hash<TimePoint>{}(TimePoint{}), 0);
    EXPECT_EQ(std::hash<TimePoint>{}(TimePoint{1ns}), 1);
    EXPECT_EQ(std::hash<TimePoint>{}(TimePoint{1us}), 1'000);
    EXPECT_EQ(std::hash<TimePoint>{}(TimePoint{1ms}), 1'000'000);
    EXPECT_EQ(std::hash<TimePoint>{}(TimePoint{1s}), 1'000'000'000);

    // Technically negative TimePoint hashing is also supported.
    EXPECT_EQ(std::hash<TimePoint>{}(TimePoint{-1ns}), 18'446'744'073'709'551'615);

    // std::hash<TimePoint> should work for hashtable-based containers like set/map.
    std::unordered_set<TimePoint> tpSet;
    tpSet.insert(TimePoint{123ns});
    tpSet.insert(TimePoint{456ms});
    tpSet.insert(TimePoint{789s});
    ASSERT_EQ(tpSet.size(), 3);
    EXPECT_EQ(tpSet.count(TimePoint{123ns}), 1);
    EXPECT_EQ(tpSet.count(TimePoint{456ms}), 1);
    EXPECT_EQ(tpSet.count(TimePoint{789s}), 1);
    // Non-existing elements.
    EXPECT_EQ(tpSet.count(TimePoint{321ns}), 0);
    EXPECT_EQ(tpSet.count(TimePoint{654ms}), 0);
    EXPECT_EQ(tpSet.count(TimePoint{987s}), 0);
}

} // namespace av
