// Copyright (c) 2021- Motional. All rights reserved.
//
// NOTICE: Removing this heading is prohibited.
//
// All information contained herein is, and remains the property of Motional, its affiliates, and
// its suppliers (if any). The intellectual and technical concepts contained herein are proprietary
// to Motional, its affiliates, and its suppliers (if any) and may be covered by U.S. and Foreign
// Patents, patents in process, and are protected by trade secrets or copyright law. Dissemination
// of this information or reproduction of this material is strictly forbidden unless prior written
// permission is obtained from Motional.

#include "lib/avtime/ConversionsTimespec.hpp"
#include "lib/avtime/AvClock.hpp"
#include <gtest/gtest.h>

bool operator==(const timespec& ts1, const timespec& ts2)
{
    return ts1.tv_sec == ts2.tv_sec && ts1.tv_nsec == ts2.tv_nsec;
}

namespace av {

constexpr uint64_t TEST_NSEC = 789000123456u;
constexpr uint64_t TEST_TV_SEC = 789u;
constexpr uint64_t TEST_TV_NSEC = 123456u;

constexpr uint64_t EXPECTED_NSEC = TEST_NSEC;
constexpr uint64_t EXPECTED_TV_SEC = TEST_TV_SEC;
constexpr uint64_t EXPECTED_TV_NSEC = TEST_TV_NSEC;

TEST(ConversionsTimespec, TimespecFromDuration)
{
    const std::chrono::nanoseconds dur1{TEST_NSEC};
    EXPECT_EQ(timespecFromDuration(dur1), (timespec{EXPECTED_TV_SEC, EXPECTED_TV_NSEC}));
}

TEST(ConversionsTimespec, TimespecFromTimePoint)
{
    const av::TimePoint tp1{std::chrono::nanoseconds{TEST_NSEC}};
    EXPECT_EQ(timespecFromTimePoint(tp1), (timespec{EXPECTED_TV_SEC, EXPECTED_TV_NSEC}));
}

TEST(ConversionsTimespec, DurationFromTimespec)
{
    const timespec ts1{EXPECTED_TV_SEC, EXPECTED_TV_NSEC};
    EXPECT_EQ(durationFromTimespec(ts1), std::chrono::nanoseconds{EXPECTED_NSEC});
}

TEST(ConversionsTimespec, TimePointFromTimespec)
{
    const timespec ts1{EXPECTED_TV_SEC, EXPECTED_TV_NSEC};
    EXPECT_EQ(timePointFromTimespec<AvClock>(ts1), av::TimePoint{std::chrono::nanoseconds{EXPECTED_NSEC}});
}

} // namespace av
