// Copyright (c) 2021- Motional. All rights reserved.
//
// NOTICE: Removing this heading is prohibited.
//
// All information contained herein is, and remains the property of Motional, its affiliates, and
// its suppliers (if any). The intellectual and technical concepts contained herein are proprietary
// to Motional, its affiliates, and its suppliers (if any) and may be covered by U.S. and Foreign
// Patents, patents in process, and are protected by trade secrets or copyright law. Dissemination
// of this information or reproduction of this material is strictly forbidden unless prior written
// permission is obtained from Motional.

#include "lib/avtime/ConversionsUsec.hpp"
#include "lib/avtime/AvClock.hpp"
#include <gtest/gtest.h>

namespace av {

constexpr uint64_t TEST_NSEC = 123456u;
constexpr uint64_t TEST_USEC = 123u;

constexpr uint64_t EXPECTED_NSEC = 123000u;
constexpr uint64_t EXPECTED_USEC = 123u;

TEST(ConversionsUsec, UsecFromDuration)
{
    const std::chrono::nanoseconds dur1{TEST_NSEC};
    EXPECT_EQ(usecFromDuration(dur1), EXPECTED_USEC);
    const std::chrono::microseconds dur2{TEST_USEC};
    EXPECT_EQ(usecFromDuration(dur2), EXPECTED_USEC);
}

TEST(ConversionsUsec, UsecFromTimePoint)
{
    const av::TimePoint tp1{std::chrono::nanoseconds{TEST_NSEC}};
    EXPECT_EQ(usecFromTimePoint(tp1), EXPECTED_USEC);
}

TEST(ConversionsUsec, DurationFromUsec)
{
    EXPECT_EQ(durationFromUsec(TEST_USEC), std::chrono::nanoseconds{EXPECTED_NSEC});
}

TEST(ConversionsUsec, TimePointFromUsec)
{
    EXPECT_EQ(timePointFromUsec<AvClock>(TEST_USEC), av::TimePoint{std::chrono::nanoseconds{EXPECTED_NSEC}});
}

} // namespace av
