#include <interfaces/AccStatusMessage/AccStatusMessage.h>

#include <iterator>
#include <type_traits>

#include <cereal/cereal.hpp>
#include <cereal/archives/json.hpp>
#include <cereal/archives/binary.hpp>

#include <cereal/types/map.hpp>
#include <cereal/types/vector.hpp>
#include <cereal/types/array.hpp>
#include <cereal/types/string.hpp>
#include <cereal/types/complex.hpp>

#include <ODatum/IntrospectionArchive.h>
#include <ODatum/Hash.h>





 namespace av {
 namespace msg {


AccStatusMessage::AccStatusMessage() = default;

AccStatusMessage::AccStatusMessage(
    TimePoint timestamp_arg ,
    CrsSwSta crsSwSta_arg 
    ):
    timestamp { std::move(timestamp_arg) } // NOLINT
      ,crsSwSta { std::move(crsSwSta_arg) } // NOLINT
      {
}

AccStatusMessage::AccStatusMessage(const AccStatusMessage&) = default;

// NOLINTNEXTLINE
AccStatusMessage::AccStatusMessage(AccStatusMessage&&) = default;

AccStatusMessage& AccStatusMessage::operator=(const AccStatusMessage&) = default;

// NOLINTNEXTLINE
AccStatusMessage& AccStatusMessage::operator=(AccStatusMessage&&) = default;

void AccStatusMessage::toCsv(std::ostream& out) const
{
    out.precision(16);
    out << std::boolalpha;
    out << timestamp;out << ",";
    out << static_cast<uint32_t>(crsSwSta);
    
}

std::stringstream AccStatusMessage::toStringstream() const
{
    std::stringstream out;
    toCsv(out);
    return out;
}

template<class Archive>
    void AccStatusMessage::serialize(Archive& ar, [[maybe_unused]] unsigned int version)
    {

        ar& CEREAL_NVP(timestamp);
        ar& CEREAL_NVP(crsSwSta);
    }


template void AccStatusMessage::serialize(cereal::BinaryInputArchive& ar, unsigned int version);
template void AccStatusMessage::serialize(cereal::BinaryOutputArchive& ar, unsigned int version);
template void AccStatusMessage::serialize(cereal::IntrospectionArchive& ar, unsigned int version);
template void AccStatusMessage::serialize(cereal::JSONInputArchive& ar, unsigned int version);
template void AccStatusMessage::serialize(cereal::JSONOutputArchive& ar, unsigned int version);
template void AccStatusMessage::serialize(cereal::PortableBinaryInputArchive& ar, unsigned int version);
template void AccStatusMessage::serialize(cereal::PortableBinaryOutputArchive& ar, unsigned int version);

std::ostream& operator<< (std::ostream& os, const AccStatusMessage& accstatusmessage)
{
    accstatusmessage.toCsv(os);
    return os;
}

bool operator== (const AccStatusMessage &lhs, const AccStatusMessage &rhs)
{
    return (  lhs.timestamp== rhs.timestamp &&
     lhs.crsSwSta== rhs.crsSwSta 
    );
}

std::size_t AccStatusMessage::hash() const noexcept
{
    std::size_t result = 17;
    result = result * 31 + 0;
    result = result * 31 + std::hash<CrsSwSta>()(crsSwSta);
    return result;
}

 } // namespace av
 } // namespace msg


CEREAL_BIND_TO_ARCHIVES(av::msg::AccStatusMessage)

