#include <interfaces/AggregatedStatusMessage/AggregatedStatusMessage.h>

#include <iterator>
#include <type_traits>

#include <cereal/cereal.hpp>
#include <cereal/archives/json.hpp>
#include <cereal/archives/binary.hpp>

#include <cereal/types/map.hpp>
#include <cereal/types/vector.hpp>
#include <cereal/types/array.hpp>
#include <cereal/types/string.hpp>
#include <cereal/types/complex.hpp>

#include <ODatum/IntrospectionArchive.h>
#include <ODatum/Hash.h>





 namespace av {
 namespace msg {


AggregatedStatusMessage::AggregatedStatusMessage() = default;

AggregatedStatusMessage::AggregatedStatusMessage(
    int64_t timestamp_arg ,
    std::vector<Parameter> parameters_arg 
    ):
    timestamp { std::move(timestamp_arg) } // NOLINT
      ,parameters { std::move(parameters_arg) } // NOLINT
      {
}

AggregatedStatusMessage::AggregatedStatusMessage(const AggregatedStatusMessage&) = default;

// NOLINTNEXTLINE
AggregatedStatusMessage::AggregatedStatusMessage(AggregatedStatusMessage&&) = default;

AggregatedStatusMessage& AggregatedStatusMessage::operator=(const AggregatedStatusMessage&) = default;

// NOLINTNEXTLINE
AggregatedStatusMessage& AggregatedStatusMessage::operator=(AggregatedStatusMessage&&) = default;

void AggregatedStatusMessage::toCsv(std::ostream& out) const
{
    out.precision(16);
    out << std::boolalpha;
    out << timestamp;out << ",";
    // Enclose sequence in double quotations and curly brackets to keep CSV line length uniform
        
        out << "\"{{";
        
        if (!parameters.empty())
        {
            std::copy(parameters.begin(), parameters.end() - 1, std::ostream_iterator<Parameter>(out, "},{"));
            std::copy(parameters.end()-1, parameters.end(), std::ostream_iterator<Parameter>(out, ""));
        }
        out << "}}\"";
    
}

std::stringstream AggregatedStatusMessage::toStringstream() const
{
    std::stringstream out;
    toCsv(out);
    return out;
}

template<class Archive>
    void AggregatedStatusMessage::serialize(Archive& ar, unsigned int version)
{
    if (version == 1)
    {
        uint64_t uTimestamp{0};
        ar(cereal::make_nvp("timestamp", uTimestamp));
        timestamp = static_cast<int64_t>(uTimestamp);
    }
    else{
        ar(CEREAL_NVP(timestamp));
    }
    ar(CEREAL_NVP(parameters));
}



template void AggregatedStatusMessage::serialize(cereal::BinaryInputArchive& ar, unsigned int version);
template void AggregatedStatusMessage::serialize(cereal::BinaryOutputArchive& ar, unsigned int version);
template void AggregatedStatusMessage::serialize(cereal::IntrospectionArchive& ar, unsigned int version);
template void AggregatedStatusMessage::serialize(cereal::JSONInputArchive& ar, unsigned int version);
template void AggregatedStatusMessage::serialize(cereal::JSONOutputArchive& ar, unsigned int version);
template void AggregatedStatusMessage::serialize(cereal::PortableBinaryInputArchive& ar, unsigned int version);
template void AggregatedStatusMessage::serialize(cereal::PortableBinaryOutputArchive& ar, unsigned int version);

std::ostream& operator<< (std::ostream& os, const AggregatedStatusMessage& aggregatedstatusmessage)
{
    aggregatedstatusmessage.toCsv(os);
    return os;
}

bool operator== (const AggregatedStatusMessage &lhs, const AggregatedStatusMessage &rhs)
{
    return (  lhs.timestamp== rhs.timestamp &&
     lhs.parameters== rhs.parameters 
    );
}

std::size_t AggregatedStatusMessage::hash() const noexcept
{
    std::size_t result = 17;
    result = result * 31 + std::hash<int64_t>()(timestamp);
    result = result * 31 + hashVector(parameters);
    return result;
}

 } // namespace av
 } // namespace msg


CEREAL_BIND_TO_ARCHIVES(av::msg::AggregatedStatusMessage)

