#include <interfaces/AvBodyCommandMessage/AvBodyCommandMessage.h>

#include <iterator>
#include <type_traits>

#include <cereal/cereal.hpp>
#include <cereal/archives/json.hpp>
#include <cereal/archives/binary.hpp>

#include <cereal/types/map.hpp>
#include <cereal/types/vector.hpp>
#include <cereal/types/array.hpp>
#include <cereal/types/string.hpp>
#include <cereal/types/complex.hpp>

#include <ODatum/IntrospectionArchive.h>
#include <ODatum/Hash.h>


 namespace av {
 namespace msg {


DoorCommands::DoorCommands() = default;

DoorCommands::DoorCommands(
    DoorCommand rearRightDoorCommand_arg ,
    DoorCommand rearLeftDoorCommand_arg ,
    DoorCommand frontPassengerDoorCommand_arg ,
    DoorCommand driverDoorCommand_arg 
    ):
    rearRightDoorCommand { std::move(rearRightDoorCommand_arg) } // NOLINT
      ,
    rearLeftDoorCommand { std::move(rearLeftDoorCommand_arg) } // NOLINT
      ,
    frontPassengerDoorCommand { std::move(frontPassengerDoorCommand_arg) } // NOLINT
      ,
    driverDoorCommand { std::move(driverDoorCommand_arg) } // NOLINT
      
    {
}

DoorCommands::DoorCommands(const DoorCommands&) = default;

// NOLINTNEXTLINE
DoorCommands::DoorCommands(DoorCommands&&) = default;

DoorCommands& DoorCommands::operator=(const DoorCommands&) = default;

// NOLINTNEXTLINE
DoorCommands& DoorCommands::operator=(DoorCommands&&) = default;

void DoorCommands::toCsv(std::ostream& out) const
{
    out.precision(16);
    out << std::boolalpha;
    out << static_cast<uint32_t>(rearRightDoorCommand);out << ",";
    out << static_cast<uint32_t>(rearLeftDoorCommand);out << ",";
    out << static_cast<uint32_t>(frontPassengerDoorCommand);out << ",";
    out << static_cast<uint32_t>(driverDoorCommand);
    
}

std::stringstream DoorCommands::toStringstream() const
{
    std::stringstream out;
    toCsv(out);
    return out;
}

template<class Archive>
    void DoorCommands::serialize(Archive& ar, [[maybe_unused]] unsigned int version)
    {

        ar& CEREAL_NVP(rearRightDoorCommand);
        ar& CEREAL_NVP(rearLeftDoorCommand);
        ar& CEREAL_NVP(frontPassengerDoorCommand);
        ar& CEREAL_NVP(driverDoorCommand);
    }


template void DoorCommands::serialize(cereal::BinaryInputArchive& ar, unsigned int version);
template void DoorCommands::serialize(cereal::BinaryOutputArchive& ar, unsigned int version);
template void DoorCommands::serialize(cereal::IntrospectionArchive& ar, unsigned int version);
template void DoorCommands::serialize(cereal::JSONInputArchive& ar, unsigned int version);
template void DoorCommands::serialize(cereal::JSONOutputArchive& ar, unsigned int version);
template void DoorCommands::serialize(cereal::PortableBinaryInputArchive& ar, unsigned int version);
template void DoorCommands::serialize(cereal::PortableBinaryOutputArchive& ar, unsigned int version);

std::ostream& operator<< (std::ostream& os, const DoorCommands& doorcommands)
{
    doorcommands.toCsv(os);
    return os;
}

bool operator== (const DoorCommands &lhs, const DoorCommands &rhs)
{
    return (  lhs.rearRightDoorCommand== rhs.rearRightDoorCommand &&
     lhs.rearLeftDoorCommand== rhs.rearLeftDoorCommand &&
     lhs.frontPassengerDoorCommand== rhs.frontPassengerDoorCommand &&
     lhs.driverDoorCommand== rhs.driverDoorCommand 
    );
}

std::size_t DoorCommands::hash() const noexcept
{
    std::size_t result = 17;
    result = result * 31 + std::hash<DoorCommand>()(rearRightDoorCommand);
    result = result * 31 + std::hash<DoorCommand>()(rearLeftDoorCommand);
    result = result * 31 + std::hash<DoorCommand>()(frontPassengerDoorCommand);
    result = result * 31 + std::hash<DoorCommand>()(driverDoorCommand);
    return result;
}

 } // namespace av
 } // namespace msg


CEREAL_BIND_TO_ARCHIVES(av::msg::DoorCommands)

 namespace av {
 namespace msg {


WindowCommands::WindowCommands() = default;

WindowCommands::WindowCommands(
    WindowCommand rearRightWindowCommand_arg ,
    WindowCommand rearLeftWindowCommand_arg ,
    WindowCommand frontPassengerWindowCommand_arg ,
    WindowCommand driverWindowCommand_arg 
    ):
    rearRightWindowCommand { std::move(rearRightWindowCommand_arg) } // NOLINT
      ,
    rearLeftWindowCommand { std::move(rearLeftWindowCommand_arg) } // NOLINT
      ,
    frontPassengerWindowCommand { std::move(frontPassengerWindowCommand_arg) } // NOLINT
      ,
    driverWindowCommand { std::move(driverWindowCommand_arg) } // NOLINT
      
    {
}

WindowCommands::WindowCommands(const WindowCommands&) = default;

// NOLINTNEXTLINE
WindowCommands::WindowCommands(WindowCommands&&) = default;

WindowCommands& WindowCommands::operator=(const WindowCommands&) = default;

// NOLINTNEXTLINE
WindowCommands& WindowCommands::operator=(WindowCommands&&) = default;

void WindowCommands::toCsv(std::ostream& out) const
{
    out.precision(16);
    out << std::boolalpha;
    out << static_cast<uint32_t>(rearRightWindowCommand);out << ",";
    out << static_cast<uint32_t>(rearLeftWindowCommand);out << ",";
    out << static_cast<uint32_t>(frontPassengerWindowCommand);out << ",";
    out << static_cast<uint32_t>(driverWindowCommand);
    
}

std::stringstream WindowCommands::toStringstream() const
{
    std::stringstream out;
    toCsv(out);
    return out;
}

template<class Archive>
    void WindowCommands::serialize(Archive& ar, [[maybe_unused]] unsigned int version)
    {

        ar& CEREAL_NVP(rearRightWindowCommand);
        ar& CEREAL_NVP(rearLeftWindowCommand);
        ar& CEREAL_NVP(frontPassengerWindowCommand);
        ar& CEREAL_NVP(driverWindowCommand);
    }


template void WindowCommands::serialize(cereal::BinaryInputArchive& ar, unsigned int version);
template void WindowCommands::serialize(cereal::BinaryOutputArchive& ar, unsigned int version);
template void WindowCommands::serialize(cereal::IntrospectionArchive& ar, unsigned int version);
template void WindowCommands::serialize(cereal::JSONInputArchive& ar, unsigned int version);
template void WindowCommands::serialize(cereal::JSONOutputArchive& ar, unsigned int version);
template void WindowCommands::serialize(cereal::PortableBinaryInputArchive& ar, unsigned int version);
template void WindowCommands::serialize(cereal::PortableBinaryOutputArchive& ar, unsigned int version);

std::ostream& operator<< (std::ostream& os, const WindowCommands& windowcommands)
{
    windowcommands.toCsv(os);
    return os;
}

bool operator== (const WindowCommands &lhs, const WindowCommands &rhs)
{
    return (  lhs.rearRightWindowCommand== rhs.rearRightWindowCommand &&
     lhs.rearLeftWindowCommand== rhs.rearLeftWindowCommand &&
     lhs.frontPassengerWindowCommand== rhs.frontPassengerWindowCommand &&
     lhs.driverWindowCommand== rhs.driverWindowCommand 
    );
}

std::size_t WindowCommands::hash() const noexcept
{
    std::size_t result = 17;
    result = result * 31 + std::hash<WindowCommand>()(rearRightWindowCommand);
    result = result * 31 + std::hash<WindowCommand>()(rearLeftWindowCommand);
    result = result * 31 + std::hash<WindowCommand>()(frontPassengerWindowCommand);
    result = result * 31 + std::hash<WindowCommand>()(driverWindowCommand);
    return result;
}

 } // namespace av
 } // namespace msg


CEREAL_BIND_TO_ARCHIVES(av::msg::WindowCommands)

 namespace av {
 namespace msg {


CeilingLampCommands::CeilingLampCommands() = default;

CeilingLampCommands::CeilingLampCommands(
    LampCommand frontRowCeilingSpotLampCommand_arg ,
    LampCommand rearLeftCeilingSpotLampCommand_arg ,
    LampCommand rearRightCeilingSpotLampCommand_arg ,
    LampCommand CeilingMoodLampCommand_arg 
    ):
    frontRowCeilingSpotLampCommand { std::move(frontRowCeilingSpotLampCommand_arg) } // NOLINT
      ,
    rearLeftCeilingSpotLampCommand { std::move(rearLeftCeilingSpotLampCommand_arg) } // NOLINT
      ,
    rearRightCeilingSpotLampCommand { std::move(rearRightCeilingSpotLampCommand_arg) } // NOLINT
      ,
    CeilingMoodLampCommand { std::move(CeilingMoodLampCommand_arg) } // NOLINT
      
    {
}

CeilingLampCommands::CeilingLampCommands(const CeilingLampCommands&) = default;

// NOLINTNEXTLINE
CeilingLampCommands::CeilingLampCommands(CeilingLampCommands&&) = default;

CeilingLampCommands& CeilingLampCommands::operator=(const CeilingLampCommands&) = default;

// NOLINTNEXTLINE
CeilingLampCommands& CeilingLampCommands::operator=(CeilingLampCommands&&) = default;

void CeilingLampCommands::toCsv(std::ostream& out) const
{
    out.precision(16);
    out << std::boolalpha;
    out << static_cast<uint32_t>(frontRowCeilingSpotLampCommand);out << ",";
    out << static_cast<uint32_t>(rearLeftCeilingSpotLampCommand);out << ",";
    out << static_cast<uint32_t>(rearRightCeilingSpotLampCommand);out << ",";
    out << static_cast<uint32_t>(CeilingMoodLampCommand);
    
}

std::stringstream CeilingLampCommands::toStringstream() const
{
    std::stringstream out;
    toCsv(out);
    return out;
}

template<class Archive>
    void CeilingLampCommands::serialize(Archive& ar, [[maybe_unused]] unsigned int version)
    {

        ar& CEREAL_NVP(frontRowCeilingSpotLampCommand);
        ar& CEREAL_NVP(rearLeftCeilingSpotLampCommand);
        ar& CEREAL_NVP(rearRightCeilingSpotLampCommand);
        ar& CEREAL_NVP(CeilingMoodLampCommand);
    }


template void CeilingLampCommands::serialize(cereal::BinaryInputArchive& ar, unsigned int version);
template void CeilingLampCommands::serialize(cereal::BinaryOutputArchive& ar, unsigned int version);
template void CeilingLampCommands::serialize(cereal::IntrospectionArchive& ar, unsigned int version);
template void CeilingLampCommands::serialize(cereal::JSONInputArchive& ar, unsigned int version);
template void CeilingLampCommands::serialize(cereal::JSONOutputArchive& ar, unsigned int version);
template void CeilingLampCommands::serialize(cereal::PortableBinaryInputArchive& ar, unsigned int version);
template void CeilingLampCommands::serialize(cereal::PortableBinaryOutputArchive& ar, unsigned int version);

std::ostream& operator<< (std::ostream& os, const CeilingLampCommands& ceilinglampcommands)
{
    ceilinglampcommands.toCsv(os);
    return os;
}

bool operator== (const CeilingLampCommands &lhs, const CeilingLampCommands &rhs)
{
    return (  lhs.frontRowCeilingSpotLampCommand== rhs.frontRowCeilingSpotLampCommand &&
     lhs.rearLeftCeilingSpotLampCommand== rhs.rearLeftCeilingSpotLampCommand &&
     lhs.rearRightCeilingSpotLampCommand== rhs.rearRightCeilingSpotLampCommand &&
     lhs.CeilingMoodLampCommand== rhs.CeilingMoodLampCommand 
    );
}

std::size_t CeilingLampCommands::hash() const noexcept
{
    std::size_t result = 17;
    result = result * 31 + std::hash<LampCommand>()(frontRowCeilingSpotLampCommand);
    result = result * 31 + std::hash<LampCommand>()(rearLeftCeilingSpotLampCommand);
    result = result * 31 + std::hash<LampCommand>()(rearRightCeilingSpotLampCommand);
    result = result * 31 + std::hash<LampCommand>()(CeilingMoodLampCommand);
    return result;
}

 } // namespace av
 } // namespace msg


CEREAL_BIND_TO_ARCHIVES(av::msg::CeilingLampCommands)

 namespace av {
 namespace msg {


HvacCommands::HvacCommands() = default;

HvacCommands::HvacCommands(
    int32_t hvacTemperatureDiverSide_arg ,
    int32_t hvacTemperaturePassengerSide_arg ,
    RecircCommand hvacCirculationCommand_arg ,
    AirflowDirectionCommand airflowDirectionCommand_arg ,
    AirflowSpeedCommand airflowSpeedCommand_arg 
    ):
    hvacTemperatureDiverSide { std::move(hvacTemperatureDiverSide_arg) } // NOLINT
      ,
    hvacTemperaturePassengerSide { std::move(hvacTemperaturePassengerSide_arg) } // NOLINT
      ,
    hvacCirculationCommand { std::move(hvacCirculationCommand_arg) } // NOLINT
      ,
    airflowDirectionCommand { std::move(airflowDirectionCommand_arg) } // NOLINT
      ,
    airflowSpeedCommand { std::move(airflowSpeedCommand_arg) } // NOLINT
      
    {
}

HvacCommands::HvacCommands(const HvacCommands&) = default;

// NOLINTNEXTLINE
HvacCommands::HvacCommands(HvacCommands&&) = default;

HvacCommands& HvacCommands::operator=(const HvacCommands&) = default;

// NOLINTNEXTLINE
HvacCommands& HvacCommands::operator=(HvacCommands&&) = default;

void HvacCommands::toCsv(std::ostream& out) const
{
    out.precision(16);
    out << std::boolalpha;
    out << hvacTemperatureDiverSide;out << ",";
    out << hvacTemperaturePassengerSide;out << ",";
    out << static_cast<uint32_t>(hvacCirculationCommand);out << ",";
    out << static_cast<uint32_t>(airflowDirectionCommand);out << ",";
    out << static_cast<uint32_t>(airflowSpeedCommand);
    
}

std::stringstream HvacCommands::toStringstream() const
{
    std::stringstream out;
    toCsv(out);
    return out;
}

template<class Archive>
    void HvacCommands::serialize(Archive& ar, [[maybe_unused]] unsigned int version)
    {

        ar& CEREAL_NVP(hvacTemperatureDiverSide);
        ar& CEREAL_NVP(hvacTemperaturePassengerSide);
        ar& CEREAL_NVP(hvacCirculationCommand);
        ar& CEREAL_NVP(airflowDirectionCommand);
        ar& CEREAL_NVP(airflowSpeedCommand);
    }


template void HvacCommands::serialize(cereal::BinaryInputArchive& ar, unsigned int version);
template void HvacCommands::serialize(cereal::BinaryOutputArchive& ar, unsigned int version);
template void HvacCommands::serialize(cereal::IntrospectionArchive& ar, unsigned int version);
template void HvacCommands::serialize(cereal::JSONInputArchive& ar, unsigned int version);
template void HvacCommands::serialize(cereal::JSONOutputArchive& ar, unsigned int version);
template void HvacCommands::serialize(cereal::PortableBinaryInputArchive& ar, unsigned int version);
template void HvacCommands::serialize(cereal::PortableBinaryOutputArchive& ar, unsigned int version);

std::ostream& operator<< (std::ostream& os, const HvacCommands& hvaccommands)
{
    hvaccommands.toCsv(os);
    return os;
}

bool operator== (const HvacCommands &lhs, const HvacCommands &rhs)
{
    return (  lhs.hvacTemperatureDiverSide== rhs.hvacTemperatureDiverSide &&
     lhs.hvacTemperaturePassengerSide== rhs.hvacTemperaturePassengerSide &&
     lhs.hvacCirculationCommand== rhs.hvacCirculationCommand &&
     lhs.airflowDirectionCommand== rhs.airflowDirectionCommand &&
     lhs.airflowSpeedCommand== rhs.airflowSpeedCommand 
    );
}

std::size_t HvacCommands::hash() const noexcept
{
    std::size_t result = 17;
    result = result * 31 + std::hash<int32_t>()(hvacTemperatureDiverSide);
    result = result * 31 + std::hash<int32_t>()(hvacTemperaturePassengerSide);
    result = result * 31 + std::hash<RecircCommand>()(hvacCirculationCommand);
    result = result * 31 + std::hash<AirflowDirectionCommand>()(airflowDirectionCommand);
    result = result * 31 + std::hash<AirflowSpeedCommand>()(airflowSpeedCommand);
    return result;
}

 } // namespace av
 } // namespace msg


CEREAL_BIND_TO_ARCHIVES(av::msg::HvacCommands)




 namespace av {
 namespace msg {


AvBodyCommandMessage::AvBodyCommandMessage() = default;

AvBodyCommandMessage::AvBodyCommandMessage(
    LockCommand lockCommand_arg ,
    DoorCommands doorCommands_arg ,
    HvacCommands hvacCommands_arg ,
    WindowCommands windowCommands_arg ,
    WindshieldDefogCommand windshieldDefogCommand_arg ,
    WindshieldDefrostCommand windshieldDefrostCommand_arg ,
    CeilingLampCommands ceilingLampCommands_arg ,
    LampCommand frontFootwellMoodLamp_arg ,
    LampCommand rearFootwellMoodLamp_arg 
    ):
    lockCommand { std::move(lockCommand_arg) } // NOLINT
      ,doorCommands { std::move(doorCommands_arg) } // NOLINT
      ,hvacCommands { std::move(hvacCommands_arg) } // NOLINT
      ,windowCommands { std::move(windowCommands_arg) } // NOLINT
      ,windshieldDefogCommand { std::move(windshieldDefogCommand_arg) } // NOLINT
      ,windshieldDefrostCommand { std::move(windshieldDefrostCommand_arg) } // NOLINT
      ,ceilingLampCommands { std::move(ceilingLampCommands_arg) } // NOLINT
      ,frontFootwellMoodLamp { std::move(frontFootwellMoodLamp_arg) } // NOLINT
      ,rearFootwellMoodLamp { std::move(rearFootwellMoodLamp_arg) } // NOLINT
      {
}

AvBodyCommandMessage::AvBodyCommandMessage(const AvBodyCommandMessage&) = default;

// NOLINTNEXTLINE
AvBodyCommandMessage::AvBodyCommandMessage(AvBodyCommandMessage&&) = default;

AvBodyCommandMessage& AvBodyCommandMessage::operator=(const AvBodyCommandMessage&) = default;

// NOLINTNEXTLINE
AvBodyCommandMessage& AvBodyCommandMessage::operator=(AvBodyCommandMessage&&) = default;

void AvBodyCommandMessage::toCsv(std::ostream& out) const
{
    out.precision(16);
    out << std::boolalpha;
    out << static_cast<uint32_t>(lockCommand);out << ",";
    out << doorCommands;out << ",";
    out << hvacCommands;out << ",";
    out << windowCommands;out << ",";
    out << static_cast<uint32_t>(windshieldDefogCommand);out << ",";
    out << static_cast<uint32_t>(windshieldDefrostCommand);out << ",";
    out << ceilingLampCommands;out << ",";
    out << static_cast<uint32_t>(frontFootwellMoodLamp);out << ",";
    out << static_cast<uint32_t>(rearFootwellMoodLamp);
    
}

std::stringstream AvBodyCommandMessage::toStringstream() const
{
    std::stringstream out;
    toCsv(out);
    return out;
}

template<class Archive>
    void AvBodyCommandMessage::serialize(Archive& ar, [[maybe_unused]] unsigned int version)
    {

        ar& CEREAL_NVP(lockCommand);
        ar& CEREAL_NVP(doorCommands);
        ar& CEREAL_NVP(hvacCommands);
        ar& CEREAL_NVP(windowCommands);
        ar& CEREAL_NVP(windshieldDefogCommand);
        ar& CEREAL_NVP(windshieldDefrostCommand);
        ar& CEREAL_NVP(ceilingLampCommands);
        ar& CEREAL_NVP(frontFootwellMoodLamp);
        ar& CEREAL_NVP(rearFootwellMoodLamp);
    }


template void AvBodyCommandMessage::serialize(cereal::BinaryInputArchive& ar, unsigned int version);
template void AvBodyCommandMessage::serialize(cereal::BinaryOutputArchive& ar, unsigned int version);
template void AvBodyCommandMessage::serialize(cereal::IntrospectionArchive& ar, unsigned int version);
template void AvBodyCommandMessage::serialize(cereal::JSONInputArchive& ar, unsigned int version);
template void AvBodyCommandMessage::serialize(cereal::JSONOutputArchive& ar, unsigned int version);
template void AvBodyCommandMessage::serialize(cereal::PortableBinaryInputArchive& ar, unsigned int version);
template void AvBodyCommandMessage::serialize(cereal::PortableBinaryOutputArchive& ar, unsigned int version);

std::ostream& operator<< (std::ostream& os, const AvBodyCommandMessage& avbodycommandmessage)
{
    avbodycommandmessage.toCsv(os);
    return os;
}

bool operator== (const AvBodyCommandMessage &lhs, const AvBodyCommandMessage &rhs)
{
    return (  lhs.lockCommand== rhs.lockCommand &&
     lhs.doorCommands== rhs.doorCommands &&
     lhs.hvacCommands== rhs.hvacCommands &&
     lhs.windowCommands== rhs.windowCommands &&
     lhs.windshieldDefogCommand== rhs.windshieldDefogCommand &&
     lhs.windshieldDefrostCommand== rhs.windshieldDefrostCommand &&
     lhs.ceilingLampCommands== rhs.ceilingLampCommands &&
     lhs.frontFootwellMoodLamp== rhs.frontFootwellMoodLamp &&
     lhs.rearFootwellMoodLamp== rhs.rearFootwellMoodLamp 
    );
}

std::size_t AvBodyCommandMessage::hash() const noexcept
{
    std::size_t result = 17;
    result = result * 31 + std::hash<LockCommand>()(lockCommand);
    result = result * 31 + std::hash<DoorCommands>()(doorCommands);
    result = result * 31 + std::hash<HvacCommands>()(hvacCommands);
    result = result * 31 + std::hash<WindowCommands>()(windowCommands);
    result = result * 31 + std::hash<WindshieldDefogCommand>()(windshieldDefogCommand);
    result = result * 31 + std::hash<WindshieldDefrostCommand>()(windshieldDefrostCommand);
    result = result * 31 + std::hash<CeilingLampCommands>()(ceilingLampCommands);
    result = result * 31 + std::hash<LampCommand>()(frontFootwellMoodLamp);
    result = result * 31 + std::hash<LampCommand>()(rearFootwellMoodLamp);
    return result;
}

 } // namespace av
 } // namespace msg


CEREAL_BIND_TO_ARCHIVES(av::msg::AvBodyCommandMessage)

