#include <interfaces/AvSystemStatus/AvSystemStatus.h>

#include <iterator>
#include <type_traits>

#include <cereal/cereal.hpp>
#include <cereal/archives/json.hpp>
#include <cereal/archives/binary.hpp>

#include <cereal/types/map.hpp>
#include <cereal/types/vector.hpp>
#include <cereal/types/array.hpp>
#include <cereal/types/string.hpp>
#include <cereal/types/complex.hpp>

#include <ODatum/IntrospectionArchive.h>
#include <ODatum/Hash.h>


 namespace av {
 namespace msg {


StateChangeReason::StateChangeReason() = default;

StateChangeReason::StateChangeReason(
    uint8_t first_arg ,
    std::string second_arg 
    ):
    first { std::move(first_arg) } // NOLINT
      ,
    second { std::move(second_arg) } // NOLINT
      
    {
}

StateChangeReason::StateChangeReason(const StateChangeReason&) = default;

// NOLINTNEXTLINE
StateChangeReason::StateChangeReason(StateChangeReason&&) = default;

StateChangeReason& StateChangeReason::operator=(const StateChangeReason&) = default;

// NOLINTNEXTLINE
StateChangeReason& StateChangeReason::operator=(StateChangeReason&&) = default;

void StateChangeReason::toCsv(std::ostream& out) const
{
    out.precision(16);
    out << std::boolalpha;
    // Cast to uint16_t to print as integer instead of char
        out << static_cast<uint16_t>(first);out << ",";
    out << "\"";
        out << second;
        out << "\"";
    
}

std::stringstream StateChangeReason::toStringstream() const
{
    std::stringstream out;
    toCsv(out);
    return out;
}

template<class Archive>
    void StateChangeReason::serialize(Archive& ar, [[maybe_unused]] unsigned int version)
    {

        ar& CEREAL_NVP(first);
        ar& CEREAL_NVP(second);
    }


template void StateChangeReason::serialize(cereal::BinaryInputArchive& ar, unsigned int version);
template void StateChangeReason::serialize(cereal::BinaryOutputArchive& ar, unsigned int version);
template void StateChangeReason::serialize(cereal::IntrospectionArchive& ar, unsigned int version);
template void StateChangeReason::serialize(cereal::JSONInputArchive& ar, unsigned int version);
template void StateChangeReason::serialize(cereal::JSONOutputArchive& ar, unsigned int version);
template void StateChangeReason::serialize(cereal::PortableBinaryInputArchive& ar, unsigned int version);
template void StateChangeReason::serialize(cereal::PortableBinaryOutputArchive& ar, unsigned int version);

std::ostream& operator<< (std::ostream& os, const StateChangeReason& statechangereason)
{
    statechangereason.toCsv(os);
    return os;
}

bool operator== (const StateChangeReason &lhs, const StateChangeReason &rhs)
{
    return (  lhs.first== rhs.first &&
     lhs.second== rhs.second 
    );
}

std::size_t StateChangeReason::hash() const noexcept
{
    std::size_t result = 17;
    result = result * 31 + std::hash<uint8_t>()(first);
    result = result * 31 + std::hash<std::string>()(second);
    return result;
}

 } // namespace av
 } // namespace msg


CEREAL_BIND_TO_ARCHIVES(av::msg::StateChangeReason)

 namespace av {
 namespace msg {


StateChangeInfo::StateChangeInfo() = default;

StateChangeInfo::StateChangeInfo(
    std::string first_arg ,
    StateChangeReason second_arg 
    ):
    first { std::move(first_arg) } // NOLINT
      ,
    second { std::move(second_arg) } // NOLINT
      
    {
}

StateChangeInfo::StateChangeInfo(const StateChangeInfo&) = default;

// NOLINTNEXTLINE
StateChangeInfo::StateChangeInfo(StateChangeInfo&&) = default;

StateChangeInfo& StateChangeInfo::operator=(const StateChangeInfo&) = default;

// NOLINTNEXTLINE
StateChangeInfo& StateChangeInfo::operator=(StateChangeInfo&&) = default;

void StateChangeInfo::toCsv(std::ostream& out) const
{
    out.precision(16);
    out << std::boolalpha;
    out << "\"";
        out << first;
        out << "\"";out << ",";
    out << second;
    
}

std::stringstream StateChangeInfo::toStringstream() const
{
    std::stringstream out;
    toCsv(out);
    return out;
}

template<class Archive>
    void StateChangeInfo::serialize(Archive& ar, [[maybe_unused]] unsigned int version)
    {

        ar& CEREAL_NVP(first);
        ar& CEREAL_NVP(second);
    }


template void StateChangeInfo::serialize(cereal::BinaryInputArchive& ar, unsigned int version);
template void StateChangeInfo::serialize(cereal::BinaryOutputArchive& ar, unsigned int version);
template void StateChangeInfo::serialize(cereal::IntrospectionArchive& ar, unsigned int version);
template void StateChangeInfo::serialize(cereal::JSONInputArchive& ar, unsigned int version);
template void StateChangeInfo::serialize(cereal::JSONOutputArchive& ar, unsigned int version);
template void StateChangeInfo::serialize(cereal::PortableBinaryInputArchive& ar, unsigned int version);
template void StateChangeInfo::serialize(cereal::PortableBinaryOutputArchive& ar, unsigned int version);

std::ostream& operator<< (std::ostream& os, const StateChangeInfo& statechangeinfo)
{
    statechangeinfo.toCsv(os);
    return os;
}

bool operator== (const StateChangeInfo &lhs, const StateChangeInfo &rhs)
{
    return (  lhs.first== rhs.first &&
     lhs.second== rhs.second 
    );
}

std::size_t StateChangeInfo::hash() const noexcept
{
    std::size_t result = 17;
    result = result * 31 + std::hash<std::string>()(first);
    result = result * 31 + std::hash<StateChangeReason>()(second);
    return result;
}

 } // namespace av
 } // namespace msg


CEREAL_BIND_TO_ARCHIVES(av::msg::StateChangeInfo)




 namespace av {
 namespace msg {


AvSystemStatus::AvSystemStatus() = default;

AvSystemStatus::AvSystemStatus(
    uint8_t m_CRC_arg ,
    uint8_t m_Counter_arg ,
    uint8_t m_AvState_arg ,
    uint8_t m_HealthState_arg ,
    StateChangeInfo m_StateChangeInfo_arg 
    ):
    m_CRC { std::move(m_CRC_arg) } // NOLINT
      ,m_Counter { std::move(m_Counter_arg) } // NOLINT
      ,m_AvState { std::move(m_AvState_arg) } // NOLINT
      ,m_HealthState { std::move(m_HealthState_arg) } // NOLINT
      ,m_StateChangeInfo { std::move(m_StateChangeInfo_arg) } // NOLINT
      {
}

AvSystemStatus::AvSystemStatus(const AvSystemStatus&) = default;

// NOLINTNEXTLINE
AvSystemStatus::AvSystemStatus(AvSystemStatus&&) = default;

AvSystemStatus& AvSystemStatus::operator=(const AvSystemStatus&) = default;

// NOLINTNEXTLINE
AvSystemStatus& AvSystemStatus::operator=(AvSystemStatus&&) = default;

void AvSystemStatus::toCsv(std::ostream& out) const
{
    out.precision(16);
    out << std::boolalpha;
    // Cast to uint16_t to print as integer instead of char
        out << static_cast<uint16_t>(m_CRC);out << ",";
    // Cast to uint16_t to print as integer instead of char
        out << static_cast<uint16_t>(m_Counter);out << ",";
    // Cast to uint16_t to print as integer instead of char
        out << static_cast<uint16_t>(m_AvState);out << ",";
    // Cast to uint16_t to print as integer instead of char
        out << static_cast<uint16_t>(m_HealthState);out << ",";
    out << m_StateChangeInfo;
    
}

std::stringstream AvSystemStatus::toStringstream() const
{
    std::stringstream out;
    toCsv(out);
    return out;
}

template<class Archive>
    void AvSystemStatus::serialize(Archive& ar, const unsigned int version)
{
    if (version >= 1)
    {
        ar& CEREAL_NVP(m_CRC);
        ar& CEREAL_NVP(m_Counter);
        ar& CEREAL_NVP(m_AvState);
        ar& CEREAL_NVP(m_HealthState);
    }

    if (version >= 2)
    {
        ar& CEREAL_NVP(m_StateChangeInfo);
    }
}

}} // to close av::msg namespace temporary

struct AvSystemStatus : public av::msg::AvSystemStatus {};
CEREAL_CLASS_VERSION(AvSystemStatus, 3)
CEREAL_REGISTER_TYPE(AvSystemStatus)

namespace av {
namespace msg {



template void AvSystemStatus::serialize(cereal::BinaryInputArchive& ar, unsigned int version);
template void AvSystemStatus::serialize(cereal::BinaryOutputArchive& ar, unsigned int version);
template void AvSystemStatus::serialize(cereal::IntrospectionArchive& ar, unsigned int version);
template void AvSystemStatus::serialize(cereal::JSONInputArchive& ar, unsigned int version);
template void AvSystemStatus::serialize(cereal::JSONOutputArchive& ar, unsigned int version);
template void AvSystemStatus::serialize(cereal::PortableBinaryInputArchive& ar, unsigned int version);
template void AvSystemStatus::serialize(cereal::PortableBinaryOutputArchive& ar, unsigned int version);

std::ostream& operator<< (std::ostream& os, const AvSystemStatus& avsystemstatus)
{
    avsystemstatus.toCsv(os);
    return os;
}

bool operator== (const AvSystemStatus &lhs, const AvSystemStatus &rhs)
{
    return (  lhs.m_CRC== rhs.m_CRC &&
     lhs.m_Counter== rhs.m_Counter &&
     lhs.m_AvState== rhs.m_AvState &&
     lhs.m_HealthState== rhs.m_HealthState &&
     lhs.m_StateChangeInfo== rhs.m_StateChangeInfo 
    );
}

std::size_t AvSystemStatus::hash() const noexcept
{
    std::size_t result = 17;
    result = result * 31 + std::hash<uint8_t>()(m_CRC);
    result = result * 31 + std::hash<uint8_t>()(m_Counter);
    result = result * 31 + std::hash<uint8_t>()(m_AvState);
    result = result * 31 + std::hash<uint8_t>()(m_HealthState);
    result = result * 31 + std::hash<StateChangeInfo>()(m_StateChangeInfo);
    return result;
}

 } // namespace av
 } // namespace msg


CEREAL_BIND_TO_ARCHIVES(av::msg::AvSystemStatus)

