#include <interfaces/AvTaskStatus/AvTaskStatus.h>

#include <iterator>
#include <type_traits>

#include <cereal/cereal.hpp>
#include <cereal/archives/json.hpp>
#include <cereal/archives/binary.hpp>

#include <cereal/types/map.hpp>
#include <cereal/types/vector.hpp>
#include <cereal/types/array.hpp>
#include <cereal/types/string.hpp>
#include <cereal/types/complex.hpp>

#include <ODatum/IntrospectionArchive.h>
#include <ODatum/Hash.h>





 namespace av {
 namespace msg {


AvTaskStatus::AvTaskStatus() = default;

AvTaskStatus::AvTaskStatus(
    int64_t timestamp_arg ,
    std::string taskName_arg ,
    TaskState state_arg ,
    FaultSource source_arg ,
    std::string message_arg ,
    ParameterType type_arg ,
    std::string parameter_name_arg ,
    std::string measured_value_arg ,
    std::string threshold_value_arg 
    ):
    timestamp { std::move(timestamp_arg) } // NOLINT
      ,taskName { std::move(taskName_arg) } // NOLINT
      ,state { std::move(state_arg) } // NOLINT
      ,source { std::move(source_arg) } // NOLINT
      ,message { std::move(message_arg) } // NOLINT
      ,type { std::move(type_arg) } // NOLINT
      ,parameter_name { std::move(parameter_name_arg) } // NOLINT
      ,measured_value { std::move(measured_value_arg) } // NOLINT
      ,threshold_value { std::move(threshold_value_arg) } // NOLINT
      {
}

AvTaskStatus::AvTaskStatus(const AvTaskStatus&) = default;

// NOLINTNEXTLINE
AvTaskStatus::AvTaskStatus(AvTaskStatus&&) = default;

AvTaskStatus& AvTaskStatus::operator=(const AvTaskStatus&) = default;

// NOLINTNEXTLINE
AvTaskStatus& AvTaskStatus::operator=(AvTaskStatus&&) = default;

void AvTaskStatus::toCsv(std::ostream& out) const
{
    out.precision(16);
    out << std::boolalpha;
    out << timestamp;out << ",";
    out << "\"";
        out << taskName;
        out << "\"";out << ",";
    out << static_cast<uint32_t>(state);out << ",";
    out << static_cast<uint32_t>(source);out << ",";
    out << "\"";
        out << message;
        out << "\"";out << ",";
    out << static_cast<uint32_t>(type);out << ",";
    out << "\"";
        out << parameter_name;
        out << "\"";out << ",";
    out << "\"";
        out << measured_value;
        out << "\"";out << ",";
    out << "\"";
        out << threshold_value;
        out << "\"";
    
}

std::stringstream AvTaskStatus::toStringstream() const
{
    std::stringstream out;
    toCsv(out);
    return out;
}

template<class Archive>
    void AvTaskStatus::serialize(Archive& ar, [[maybe_unused]] unsigned int version)
    {

        ar& CEREAL_NVP(timestamp);
        ar& CEREAL_NVP(taskName);
        ar& CEREAL_NVP(state);
        ar& CEREAL_NVP(source);
        ar& CEREAL_NVP(message);
        ar& CEREAL_NVP(type);
        ar& CEREAL_NVP(parameter_name);
        ar& CEREAL_NVP(measured_value);
        ar& CEREAL_NVP(threshold_value);
    }


template void AvTaskStatus::serialize(cereal::BinaryInputArchive& ar, unsigned int version);
template void AvTaskStatus::serialize(cereal::BinaryOutputArchive& ar, unsigned int version);
template void AvTaskStatus::serialize(cereal::IntrospectionArchive& ar, unsigned int version);
template void AvTaskStatus::serialize(cereal::JSONInputArchive& ar, unsigned int version);
template void AvTaskStatus::serialize(cereal::JSONOutputArchive& ar, unsigned int version);
template void AvTaskStatus::serialize(cereal::PortableBinaryInputArchive& ar, unsigned int version);
template void AvTaskStatus::serialize(cereal::PortableBinaryOutputArchive& ar, unsigned int version);

std::ostream& operator<< (std::ostream& os, const AvTaskStatus& avtaskstatus)
{
    avtaskstatus.toCsv(os);
    return os;
}

bool operator== (const AvTaskStatus &lhs, const AvTaskStatus &rhs)
{
    return (  lhs.timestamp== rhs.timestamp &&
     lhs.taskName== rhs.taskName &&
     lhs.state== rhs.state &&
     lhs.source== rhs.source &&
     lhs.message== rhs.message &&
     lhs.type== rhs.type &&
     lhs.parameter_name== rhs.parameter_name &&
     lhs.measured_value== rhs.measured_value &&
     lhs.threshold_value== rhs.threshold_value 
    );
}

std::size_t AvTaskStatus::hash() const noexcept
{
    std::size_t result = 17;
    result = result * 31 + std::hash<int64_t>()(timestamp);
    result = result * 31 + std::hash<std::string>()(taskName);
    result = result * 31 + std::hash<TaskState>()(state);
    result = result * 31 + std::hash<FaultSource>()(source);
    result = result * 31 + std::hash<std::string>()(message);
    result = result * 31 + std::hash<ParameterType>()(type);
    result = result * 31 + std::hash<std::string>()(parameter_name);
    result = result * 31 + std::hash<std::string>()(measured_value);
    result = result * 31 + std::hash<std::string>()(threshold_value);
    return result;
}

 } // namespace av
 } // namespace msg


CEREAL_BIND_TO_ARCHIVES(av::msg::AvTaskStatus)

