#include <interfaces/AvssmState/AvssmState.h>

#include <iterator>
#include <type_traits>

#include <cereal/cereal.hpp>
#include <cereal/archives/json.hpp>
#include <cereal/archives/binary.hpp>

#include <cereal/types/map.hpp>
#include <cereal/types/vector.hpp>
#include <cereal/types/array.hpp>
#include <cereal/types/string.hpp>
#include <cereal/types/complex.hpp>

#include <ODatum/IntrospectionArchive.h>
#include <ODatum/Hash.h>





 namespace av {
 namespace msg {


AvssmState::AvssmState() = default;

AvssmState::AvssmState(
    TimePoint timestamp_arg ,
    AvssmStateName currentState_arg 
    ):
    timestamp { std::move(timestamp_arg) } // NOLINT
      ,currentState { std::move(currentState_arg) } // NOLINT
      {
}

AvssmState::AvssmState(const AvssmState&) = default;

// NOLINTNEXTLINE
AvssmState::AvssmState(AvssmState&&) = default;

AvssmState& AvssmState::operator=(const AvssmState&) = default;

// NOLINTNEXTLINE
AvssmState& AvssmState::operator=(AvssmState&&) = default;

void AvssmState::toCsv(std::ostream& out) const
{
    out.precision(16);
    out << std::boolalpha;
    out << timestamp;out << ",";
    out << static_cast<uint32_t>(currentState);
    
}

std::stringstream AvssmState::toStringstream() const
{
    std::stringstream out;
    toCsv(out);
    return out;
}

template<class Archive>
    void AvssmState::serialize(Archive& ar, [[maybe_unused]] unsigned int version)
    {

        ar& CEREAL_NVP(timestamp);
        ar& CEREAL_NVP(currentState);
    }


template void AvssmState::serialize(cereal::BinaryInputArchive& ar, unsigned int version);
template void AvssmState::serialize(cereal::BinaryOutputArchive& ar, unsigned int version);
template void AvssmState::serialize(cereal::IntrospectionArchive& ar, unsigned int version);
template void AvssmState::serialize(cereal::JSONInputArchive& ar, unsigned int version);
template void AvssmState::serialize(cereal::JSONOutputArchive& ar, unsigned int version);
template void AvssmState::serialize(cereal::PortableBinaryInputArchive& ar, unsigned int version);
template void AvssmState::serialize(cereal::PortableBinaryOutputArchive& ar, unsigned int version);

std::ostream& operator<< (std::ostream& os, const AvssmState& avssmstate)
{
    avssmstate.toCsv(os);
    return os;
}

bool operator== (const AvssmState &lhs, const AvssmState &rhs)
{
    return (  lhs.timestamp== rhs.timestamp &&
     lhs.currentState== rhs.currentState 
    );
}

std::size_t AvssmState::hash() const noexcept
{
    std::size_t result = 17;
    result = result * 31 + 0;
    result = result * 31 + std::hash<AvssmStateName>()(currentState);
    return result;
}

 } // namespace av
 } // namespace msg


CEREAL_BIND_TO_ARCHIVES(av::msg::AvssmState)

