#include <interfaces/BlockageStatus/BlockageStatus.h>

#include <iterator>
#include <type_traits>

#include <cereal/cereal.hpp>
#include <cereal/archives/json.hpp>
#include <cereal/archives/binary.hpp>

#include <cereal/types/map.hpp>
#include <cereal/types/vector.hpp>
#include <cereal/types/array.hpp>
#include <cereal/types/string.hpp>
#include <cereal/types/complex.hpp>

#include <ODatum/IntrospectionArchive.h>
#include <ODatum/Hash.h>


 namespace av {
 namespace msg {


BlockageStatus::BlockageStatus() = default;

BlockageStatus::BlockageStatus(
    av::msg::TimePoint timeStamp_arg ,
    av::msg::TimePoint checkTimeStamp_arg ,
    std::string sensorId_arg ,
    double confidence_arg ,
    bool blocked_arg 
    ):
    timeStamp { std::move(timeStamp_arg) } // NOLINT
      ,
    checkTimeStamp { std::move(checkTimeStamp_arg) } // NOLINT
      ,
    sensorId { std::move(sensorId_arg) } // NOLINT
      ,
    confidence { std::move(confidence_arg) } // NOLINT
      ,
    blocked { std::move(blocked_arg) } // NOLINT
      
    {
}

BlockageStatus::BlockageStatus(const BlockageStatus&) = default;

// NOLINTNEXTLINE
BlockageStatus::BlockageStatus(BlockageStatus&&) = default;

BlockageStatus& BlockageStatus::operator=(const BlockageStatus&) = default;

// NOLINTNEXTLINE
BlockageStatus& BlockageStatus::operator=(BlockageStatus&&) = default;

void BlockageStatus::toCsv(std::ostream& out) const
{
    out.precision(16);
    out << std::boolalpha;
    out << timeStamp;out << ",";
    out << checkTimeStamp;out << ",";
    out << "\"";
        out << sensorId;
        out << "\"";out << ",";
    out << confidence;out << ",";
    out << blocked;
    
}

std::stringstream BlockageStatus::toStringstream() const
{
    std::stringstream out;
    toCsv(out);
    return out;
}

template<class Archive>
    void BlockageStatus::serialize(Archive& ar, [[maybe_unused]] unsigned int version)
    {

        ar& CEREAL_NVP(timeStamp);
        ar& CEREAL_NVP(checkTimeStamp);
        ar& CEREAL_NVP(sensorId);
        ar& CEREAL_NVP(confidence);
        ar& CEREAL_NVP(blocked);
    }


template void BlockageStatus::serialize(cereal::BinaryInputArchive& ar, unsigned int version);
template void BlockageStatus::serialize(cereal::BinaryOutputArchive& ar, unsigned int version);
template void BlockageStatus::serialize(cereal::IntrospectionArchive& ar, unsigned int version);
template void BlockageStatus::serialize(cereal::JSONInputArchive& ar, unsigned int version);
template void BlockageStatus::serialize(cereal::JSONOutputArchive& ar, unsigned int version);
template void BlockageStatus::serialize(cereal::PortableBinaryInputArchive& ar, unsigned int version);
template void BlockageStatus::serialize(cereal::PortableBinaryOutputArchive& ar, unsigned int version);

std::ostream& operator<< (std::ostream& os, const BlockageStatus& blockagestatus)
{
    blockagestatus.toCsv(os);
    return os;
}

bool operator== (const BlockageStatus &lhs, const BlockageStatus &rhs)
{
    return (  lhs.timeStamp== rhs.timeStamp &&
     lhs.checkTimeStamp== rhs.checkTimeStamp &&
     lhs.sensorId== rhs.sensorId &&
     lhs.confidence== rhs.confidence &&
     lhs.blocked== rhs.blocked 
    );
}

std::size_t BlockageStatus::hash() const noexcept
{
    std::size_t result = 17;
    result = result * 31 + std::hash<av::msg::TimePoint>()(timeStamp);
    result = result * 31 + std::hash<av::msg::TimePoint>()(checkTimeStamp);
    result = result * 31 + std::hash<std::string>()(sensorId);
    result = result * 31 + std::hash<double>()(confidence);
    result = result * 31 + std::hash<bool>()(blocked);
    return result;
}

 } // namespace av
 } // namespace msg


CEREAL_BIND_TO_ARCHIVES(av::msg::BlockageStatus)


