#include <interfaces/BoundingBox2D/BoundingBox2D.h>

#include <iterator>
#include <type_traits>

#include <cereal/cereal.hpp>
#include <cereal/archives/json.hpp>
#include <cereal/archives/binary.hpp>

#include <cereal/types/map.hpp>
#include <cereal/types/vector.hpp>
#include <cereal/types/array.hpp>
#include <cereal/types/string.hpp>
#include <cereal/types/complex.hpp>

#include <ODatum/IntrospectionArchive.h>
#include <ODatum/Hash.h>


 namespace av {
 namespace horizon {


BoundingBox2D::BoundingBox2D() = default;

BoundingBox2D::BoundingBox2D(
    int32_t topleftX_pixel_arg ,
    int32_t topleftY_pixel_arg ,
    uint32_t width_arg ,
    uint32_t height_arg 
    ):
    topleftX_pixel { std::move(topleftX_pixel_arg) } // NOLINT
      ,
    topleftY_pixel { std::move(topleftY_pixel_arg) } // NOLINT
      ,
    width { std::move(width_arg) } // NOLINT
      ,
    height { std::move(height_arg) } // NOLINT
      
    {
}

BoundingBox2D::BoundingBox2D(const BoundingBox2D&) = default;

// NOLINTNEXTLINE
BoundingBox2D::BoundingBox2D(BoundingBox2D&&) = default;

BoundingBox2D& BoundingBox2D::operator=(const BoundingBox2D&) = default;

// NOLINTNEXTLINE
BoundingBox2D& BoundingBox2D::operator=(BoundingBox2D&&) = default;

void BoundingBox2D::toCsv(std::ostream& out) const
{
    out.precision(16);
    out << std::boolalpha;
    out << topleftX_pixel;out << ",";
    out << topleftY_pixel;out << ",";
    out << width;out << ",";
    out << height;
    
}

std::stringstream BoundingBox2D::toStringstream() const
{
    std::stringstream out;
    toCsv(out);
    return out;
}

template<class Archive>
    void BoundingBox2D::serialize(Archive& ar, [[maybe_unused]] unsigned int version)
    {

        ar& CEREAL_NVP(topleftX_pixel);
        ar& CEREAL_NVP(topleftY_pixel);
        ar& CEREAL_NVP(width);
        ar& CEREAL_NVP(height);
    }


template void BoundingBox2D::serialize(cereal::BinaryInputArchive& ar, unsigned int version);
template void BoundingBox2D::serialize(cereal::BinaryOutputArchive& ar, unsigned int version);
template void BoundingBox2D::serialize(cereal::IntrospectionArchive& ar, unsigned int version);
template void BoundingBox2D::serialize(cereal::JSONInputArchive& ar, unsigned int version);
template void BoundingBox2D::serialize(cereal::JSONOutputArchive& ar, unsigned int version);
template void BoundingBox2D::serialize(cereal::PortableBinaryInputArchive& ar, unsigned int version);
template void BoundingBox2D::serialize(cereal::PortableBinaryOutputArchive& ar, unsigned int version);

std::ostream& operator<< (std::ostream& os, const BoundingBox2D& boundingbox2d)
{
    boundingbox2d.toCsv(os);
    return os;
}

bool operator== (const BoundingBox2D &lhs, const BoundingBox2D &rhs)
{
    return (  lhs.topleftX_pixel== rhs.topleftX_pixel &&
     lhs.topleftY_pixel== rhs.topleftY_pixel &&
     lhs.width== rhs.width &&
     lhs.height== rhs.height 
    );
}

std::size_t BoundingBox2D::hash() const noexcept
{
    std::size_t result = 17;
    result = result * 31 + std::hash<int32_t>()(topleftX_pixel);
    result = result * 31 + std::hash<int32_t>()(topleftY_pixel);
    result = result * 31 + std::hash<uint32_t>()(width);
    result = result * 31 + std::hash<uint32_t>()(height);
    return result;
}

 } // namespace av
 } // namespace horizon


CEREAL_BIND_TO_ARCHIVES(av::horizon::BoundingBox2D)


