#include <interfaces/BoundingBox3D/BoundingBox3D.h>

#include <iterator>
#include <type_traits>

#include <cereal/cereal.hpp>
#include <cereal/archives/json.hpp>
#include <cereal/archives/binary.hpp>

#include <cereal/types/map.hpp>
#include <cereal/types/vector.hpp>
#include <cereal/types/array.hpp>
#include <cereal/types/string.hpp>
#include <cereal/types/complex.hpp>

#include <ODatum/IntrospectionArchive.h>
#include <ODatum/Hash.h>


 namespace av {
 namespace horizon {


BoundingBox3D::BoundingBox3D() = default;

BoundingBox3D::BoundingBox3D(
    Point3D lowerLt_arg ,
    Point3D lowerLb_arg ,
    Point3D lowerRb_arg ,
    Point3D lowerRt_arg ,
    Point3D upperLt_arg ,
    Point3D upperLb_arg ,
    Point3D upperRb_arg ,
    Point3D upperRt_arg ,
    double confidence_arg 
    ):
    lowerLt { std::move(lowerLt_arg) } // NOLINT
      ,
    lowerLb { std::move(lowerLb_arg) } // NOLINT
      ,
    lowerRb { std::move(lowerRb_arg) } // NOLINT
      ,
    lowerRt { std::move(lowerRt_arg) } // NOLINT
      ,
    upperLt { std::move(upperLt_arg) } // NOLINT
      ,
    upperLb { std::move(upperLb_arg) } // NOLINT
      ,
    upperRb { std::move(upperRb_arg) } // NOLINT
      ,
    upperRt { std::move(upperRt_arg) } // NOLINT
      ,
    confidence { std::move(confidence_arg) } // NOLINT
      
    {
}

BoundingBox3D::BoundingBox3D(const BoundingBox3D&) = default;

// NOLINTNEXTLINE
BoundingBox3D::BoundingBox3D(BoundingBox3D&&) = default;

BoundingBox3D& BoundingBox3D::operator=(const BoundingBox3D&) = default;

// NOLINTNEXTLINE
BoundingBox3D& BoundingBox3D::operator=(BoundingBox3D&&) = default;

void BoundingBox3D::toCsv(std::ostream& out) const
{
    out.precision(16);
    out << std::boolalpha;
    out << lowerLt;out << ",";
    out << lowerLb;out << ",";
    out << lowerRb;out << ",";
    out << lowerRt;out << ",";
    out << upperLt;out << ",";
    out << upperLb;out << ",";
    out << upperRb;out << ",";
    out << upperRt;out << ",";
    out << confidence;
    
}

std::stringstream BoundingBox3D::toStringstream() const
{
    std::stringstream out;
    toCsv(out);
    return out;
}

template<class Archive>
    void BoundingBox3D::serialize(Archive& ar, [[maybe_unused]] unsigned int version)
    {

        ar& CEREAL_NVP(lowerLt);
        ar& CEREAL_NVP(lowerLb);
        ar& CEREAL_NVP(lowerRb);
        ar& CEREAL_NVP(lowerRt);
        ar& CEREAL_NVP(upperLt);
        ar& CEREAL_NVP(upperLb);
        ar& CEREAL_NVP(upperRb);
        ar& CEREAL_NVP(upperRt);
        ar& CEREAL_NVP(confidence);
    }


template void BoundingBox3D::serialize(cereal::BinaryInputArchive& ar, unsigned int version);
template void BoundingBox3D::serialize(cereal::BinaryOutputArchive& ar, unsigned int version);
template void BoundingBox3D::serialize(cereal::IntrospectionArchive& ar, unsigned int version);
template void BoundingBox3D::serialize(cereal::JSONInputArchive& ar, unsigned int version);
template void BoundingBox3D::serialize(cereal::JSONOutputArchive& ar, unsigned int version);
template void BoundingBox3D::serialize(cereal::PortableBinaryInputArchive& ar, unsigned int version);
template void BoundingBox3D::serialize(cereal::PortableBinaryOutputArchive& ar, unsigned int version);

std::ostream& operator<< (std::ostream& os, const BoundingBox3D& boundingbox3d)
{
    boundingbox3d.toCsv(os);
    return os;
}

bool operator== (const BoundingBox3D &lhs, const BoundingBox3D &rhs)
{
    return (  lhs.lowerLt== rhs.lowerLt &&
     lhs.lowerLb== rhs.lowerLb &&
     lhs.lowerRb== rhs.lowerRb &&
     lhs.lowerRt== rhs.lowerRt &&
     lhs.upperLt== rhs.upperLt &&
     lhs.upperLb== rhs.upperLb &&
     lhs.upperRb== rhs.upperRb &&
     lhs.upperRt== rhs.upperRt &&
     lhs.confidence== rhs.confidence 
    );
}

std::size_t BoundingBox3D::hash() const noexcept
{
    std::size_t result = 17;
    result = result * 31 + std::hash<Point3D>()(lowerLt);
    result = result * 31 + std::hash<Point3D>()(lowerLb);
    result = result * 31 + std::hash<Point3D>()(lowerRb);
    result = result * 31 + std::hash<Point3D>()(lowerRt);
    result = result * 31 + std::hash<Point3D>()(upperLt);
    result = result * 31 + std::hash<Point3D>()(upperLb);
    result = result * 31 + std::hash<Point3D>()(upperRb);
    result = result * 31 + std::hash<Point3D>()(upperRt);
    result = result * 31 + std::hash<double>()(confidence);
    return result;
}

 } // namespace av
 } // namespace horizon


CEREAL_BIND_TO_ARCHIVES(av::horizon::BoundingBox3D)


