#include <interfaces/ChannelStatusMessage/ChannelStatusMessage.h>

#include <iterator>
#include <type_traits>

#include <cereal/cereal.hpp>
#include <cereal/archives/json.hpp>
#include <cereal/archives/binary.hpp>

#include <cereal/types/map.hpp>
#include <cereal/types/vector.hpp>
#include <cereal/types/array.hpp>
#include <cereal/types/string.hpp>
#include <cereal/types/complex.hpp>

#include <ODatum/IntrospectionArchive.h>
#include <ODatum/Hash.h>





 namespace av {
 namespace msg {


ChannelStatusMessage::ChannelStatusMessage() = default;

ChannelStatusMessage::ChannelStatusMessage(
    std::string channelName_arg ,
    ChannelStatus status_arg ,
    TimePoint timestamp_arg 
    ):
    channelName { std::move(channelName_arg) } // NOLINT
      ,status { std::move(status_arg) } // NOLINT
      ,timestamp { std::move(timestamp_arg) } // NOLINT
      {
}

ChannelStatusMessage::ChannelStatusMessage(const ChannelStatusMessage&) = default;

// NOLINTNEXTLINE
ChannelStatusMessage::ChannelStatusMessage(ChannelStatusMessage&&) = default;

ChannelStatusMessage& ChannelStatusMessage::operator=(const ChannelStatusMessage&) = default;

// NOLINTNEXTLINE
ChannelStatusMessage& ChannelStatusMessage::operator=(ChannelStatusMessage&&) = default;

void ChannelStatusMessage::toCsv(std::ostream& out) const
{
    out.precision(16);
    out << std::boolalpha;
    out << "\"";
        out << channelName;
        out << "\"";out << ",";
    out << static_cast<uint32_t>(status);out << ",";
    out << timestamp;
    
}

std::stringstream ChannelStatusMessage::toStringstream() const
{
    std::stringstream out;
    toCsv(out);
    return out;
}

template<class Archive>
    void ChannelStatusMessage::serialize(Archive& ar, [[maybe_unused]] unsigned int version)
    {

        ar& CEREAL_NVP(channelName);
        ar& CEREAL_NVP(status);
        ar& CEREAL_NVP(timestamp);
    }


template void ChannelStatusMessage::serialize(cereal::BinaryInputArchive& ar, unsigned int version);
template void ChannelStatusMessage::serialize(cereal::BinaryOutputArchive& ar, unsigned int version);
template void ChannelStatusMessage::serialize(cereal::IntrospectionArchive& ar, unsigned int version);
template void ChannelStatusMessage::serialize(cereal::JSONInputArchive& ar, unsigned int version);
template void ChannelStatusMessage::serialize(cereal::JSONOutputArchive& ar, unsigned int version);
template void ChannelStatusMessage::serialize(cereal::PortableBinaryInputArchive& ar, unsigned int version);
template void ChannelStatusMessage::serialize(cereal::PortableBinaryOutputArchive& ar, unsigned int version);

std::ostream& operator<< (std::ostream& os, const ChannelStatusMessage& channelstatusmessage)
{
    channelstatusmessage.toCsv(os);
    return os;
}

bool operator== (const ChannelStatusMessage &lhs, const ChannelStatusMessage &rhs)
{
    return (  lhs.channelName== rhs.channelName &&
     lhs.status== rhs.status &&
     lhs.timestamp== rhs.timestamp 
    );
}

std::size_t ChannelStatusMessage::hash() const noexcept
{
    std::size_t result = 17;
    result = result * 31 + std::hash<std::string>()(channelName);
    result = result * 31 + std::hash<ChannelStatus>()(status);
    result = result * 31 + 0;
    return result;
}

 } // namespace av
 } // namespace msg


CEREAL_BIND_TO_ARCHIVES(av::msg::ChannelStatusMessage)

