#include <interfaces/CleaningRequest/CleaningRequest.h>

#include <iterator>
#include <type_traits>

#include <cereal/cereal.hpp>
#include <cereal/archives/json.hpp>
#include <cereal/archives/binary.hpp>

#include <cereal/types/map.hpp>
#include <cereal/types/vector.hpp>
#include <cereal/types/array.hpp>
#include <cereal/types/string.hpp>
#include <cereal/types/complex.hpp>

#include <ODatum/IntrospectionArchive.h>
#include <ODatum/Hash.h>


 namespace av {
 namespace msg {
 namespace rva {
 namespace sensor_cleaning {


CleaningInfo::CleaningInfo() = default;

CleaningInfo::CleaningInfo(
    std::string sensorId_arg ,
    CleaningType cleaningType_arg 
    ):
    sensorId { std::move(sensorId_arg) } // NOLINT
      ,
    cleaningType { std::move(cleaningType_arg) } // NOLINT
      
    {
}

CleaningInfo::CleaningInfo(const CleaningInfo&) = default;

// NOLINTNEXTLINE
CleaningInfo::CleaningInfo(CleaningInfo&&) = default;

CleaningInfo& CleaningInfo::operator=(const CleaningInfo&) = default;

// NOLINTNEXTLINE
CleaningInfo& CleaningInfo::operator=(CleaningInfo&&) = default;

void CleaningInfo::toCsv(std::ostream& out) const
{
    out.precision(16);
    out << std::boolalpha;
    out << "\"";
        out << sensorId;
        out << "\"";out << ",";
    out << static_cast<uint32_t>(cleaningType);
    
}

std::stringstream CleaningInfo::toStringstream() const
{
    std::stringstream out;
    toCsv(out);
    return out;
}

template<class Archive>
    void CleaningInfo::serialize(Archive& ar, [[maybe_unused]] unsigned int version)
    {

        ar& CEREAL_NVP(sensorId);
        ar& CEREAL_NVP(cleaningType);
    }


template void CleaningInfo::serialize(cereal::BinaryInputArchive& ar, unsigned int version);
template void CleaningInfo::serialize(cereal::BinaryOutputArchive& ar, unsigned int version);
template void CleaningInfo::serialize(cereal::IntrospectionArchive& ar, unsigned int version);
template void CleaningInfo::serialize(cereal::JSONInputArchive& ar, unsigned int version);
template void CleaningInfo::serialize(cereal::JSONOutputArchive& ar, unsigned int version);
template void CleaningInfo::serialize(cereal::PortableBinaryInputArchive& ar, unsigned int version);
template void CleaningInfo::serialize(cereal::PortableBinaryOutputArchive& ar, unsigned int version);

std::ostream& operator<< (std::ostream& os, const CleaningInfo& cleaninginfo)
{
    cleaninginfo.toCsv(os);
    return os;
}

bool operator== (const CleaningInfo &lhs, const CleaningInfo &rhs)
{
    return (  lhs.sensorId== rhs.sensorId &&
     lhs.cleaningType== rhs.cleaningType 
    );
}

std::size_t CleaningInfo::hash() const noexcept
{
    std::size_t result = 17;
    result = result * 31 + std::hash<std::string>()(sensorId);
    result = result * 31 + std::hash<CleaningType>()(cleaningType);
    return result;
}

 } // namespace av
 } // namespace msg
 } // namespace rva
 } // namespace sensor_cleaning


CEREAL_BIND_TO_ARCHIVES(av::msg::rva::sensor_cleaning::CleaningInfo)




 namespace av {
 namespace msg {
 namespace rva {
 namespace sensor_cleaning {


CleaningRequest::CleaningRequest() = default;

CleaningRequest::CleaningRequest(
    std::string id_arg ,
    TimePoint timestamp_arg ,
    std::vector<CleaningInfo> payloads_arg 
    ):
    id { std::move(id_arg) } // NOLINT
      ,timestamp { std::move(timestamp_arg) } // NOLINT
      ,payloads { std::move(payloads_arg) } // NOLINT
      {
}

CleaningRequest::CleaningRequest(const CleaningRequest&) = default;

// NOLINTNEXTLINE
CleaningRequest::CleaningRequest(CleaningRequest&&) = default;

CleaningRequest& CleaningRequest::operator=(const CleaningRequest&) = default;

// NOLINTNEXTLINE
CleaningRequest& CleaningRequest::operator=(CleaningRequest&&) = default;

void CleaningRequest::toCsv(std::ostream& out) const
{
    out.precision(16);
    out << std::boolalpha;
    out << "\"";
        out << id;
        out << "\"";out << ",";
    out << timestamp;out << ",";
    // Enclose sequence in double quotations and curly brackets to keep CSV line length uniform
        
        out << "\"{{";
        
        if (!payloads.empty())
        {
            std::copy(payloads.begin(), payloads.end() - 1, std::ostream_iterator<CleaningInfo>(out, "},{"));
            std::copy(payloads.end()-1, payloads.end(), std::ostream_iterator<CleaningInfo>(out, ""));
        }
        out << "}}\"";
    
}

std::stringstream CleaningRequest::toStringstream() const
{
    std::stringstream out;
    toCsv(out);
    return out;
}

template<class Archive>
    void CleaningRequest::serialize(Archive& ar, [[maybe_unused]] unsigned int version)
    {

        ar& CEREAL_NVP(id);
        ar& CEREAL_NVP(timestamp);
        ar& CEREAL_NVP(payloads);
    }


template void CleaningRequest::serialize(cereal::BinaryInputArchive& ar, unsigned int version);
template void CleaningRequest::serialize(cereal::BinaryOutputArchive& ar, unsigned int version);
template void CleaningRequest::serialize(cereal::IntrospectionArchive& ar, unsigned int version);
template void CleaningRequest::serialize(cereal::JSONInputArchive& ar, unsigned int version);
template void CleaningRequest::serialize(cereal::JSONOutputArchive& ar, unsigned int version);
template void CleaningRequest::serialize(cereal::PortableBinaryInputArchive& ar, unsigned int version);
template void CleaningRequest::serialize(cereal::PortableBinaryOutputArchive& ar, unsigned int version);

std::ostream& operator<< (std::ostream& os, const CleaningRequest& cleaningrequest)
{
    cleaningrequest.toCsv(os);
    return os;
}

bool operator== (const CleaningRequest &lhs, const CleaningRequest &rhs)
{
    return (  lhs.id== rhs.id &&
     lhs.timestamp== rhs.timestamp &&
     lhs.payloads== rhs.payloads 
    );
}

std::size_t CleaningRequest::hash() const noexcept
{
    std::size_t result = 17;
    result = result * 31 + std::hash<std::string>()(id);
    result = result * 31 + 0;
    result = result * 31 + hashVector(payloads);
    return result;
}

 } // namespace av
 } // namespace msg
 } // namespace rva
 } // namespace sensor_cleaning


CEREAL_BIND_TO_ARCHIVES(av::msg::rva::sensor_cleaning::CleaningRequest)

