#include <interfaces/CleaningStatus/CleaningStatus.h>

#include <iterator>
#include <type_traits>

#include <cereal/cereal.hpp>
#include <cereal/archives/json.hpp>
#include <cereal/archives/binary.hpp>

#include <cereal/types/map.hpp>
#include <cereal/types/vector.hpp>
#include <cereal/types/array.hpp>
#include <cereal/types/string.hpp>
#include <cereal/types/complex.hpp>

#include <ODatum/IntrospectionArchive.h>
#include <ODatum/Hash.h>


 namespace av {
 namespace msg {
 namespace rva {
 namespace sensor_cleaning {


CleaningStatusInfo::CleaningStatusInfo() = default;

CleaningStatusInfo::CleaningStatusInfo(
    CleaningState status_arg ,
    std::string sensor_id_arg ,
    std::string info_arg 
    ):
    status { std::move(status_arg) } // NOLINT
      ,
    sensor_id { std::move(sensor_id_arg) } // NOLINT
      ,
    info { std::move(info_arg) } // NOLINT
      
    {
}

CleaningStatusInfo::CleaningStatusInfo(const CleaningStatusInfo&) = default;

// NOLINTNEXTLINE
CleaningStatusInfo::CleaningStatusInfo(CleaningStatusInfo&&) = default;

CleaningStatusInfo& CleaningStatusInfo::operator=(const CleaningStatusInfo&) = default;

// NOLINTNEXTLINE
CleaningStatusInfo& CleaningStatusInfo::operator=(CleaningStatusInfo&&) = default;

void CleaningStatusInfo::toCsv(std::ostream& out) const
{
    out.precision(16);
    out << std::boolalpha;
    out << static_cast<uint32_t>(status);out << ",";
    out << "\"";
        out << sensor_id;
        out << "\"";out << ",";
    out << "\"";
        out << info;
        out << "\"";
    
}

std::stringstream CleaningStatusInfo::toStringstream() const
{
    std::stringstream out;
    toCsv(out);
    return out;
}

template<class Archive>
    void CleaningStatusInfo::serialize(Archive& ar, [[maybe_unused]] unsigned int version)
    {

        ar& CEREAL_NVP(status);
        ar& CEREAL_NVP(sensor_id);
        ar& CEREAL_NVP(info);
    }


template void CleaningStatusInfo::serialize(cereal::BinaryInputArchive& ar, unsigned int version);
template void CleaningStatusInfo::serialize(cereal::BinaryOutputArchive& ar, unsigned int version);
template void CleaningStatusInfo::serialize(cereal::IntrospectionArchive& ar, unsigned int version);
template void CleaningStatusInfo::serialize(cereal::JSONInputArchive& ar, unsigned int version);
template void CleaningStatusInfo::serialize(cereal::JSONOutputArchive& ar, unsigned int version);
template void CleaningStatusInfo::serialize(cereal::PortableBinaryInputArchive& ar, unsigned int version);
template void CleaningStatusInfo::serialize(cereal::PortableBinaryOutputArchive& ar, unsigned int version);

std::ostream& operator<< (std::ostream& os, const CleaningStatusInfo& cleaningstatusinfo)
{
    cleaningstatusinfo.toCsv(os);
    return os;
}

bool operator== (const CleaningStatusInfo &lhs, const CleaningStatusInfo &rhs)
{
    return (  lhs.status== rhs.status &&
     lhs.sensor_id== rhs.sensor_id &&
     lhs.info== rhs.info 
    );
}

std::size_t CleaningStatusInfo::hash() const noexcept
{
    std::size_t result = 17;
    result = result * 31 + std::hash<CleaningState>()(status);
    result = result * 31 + std::hash<std::string>()(sensor_id);
    result = result * 31 + std::hash<std::string>()(info);
    return result;
}

 } // namespace av
 } // namespace msg
 } // namespace rva
 } // namespace sensor_cleaning


CEREAL_BIND_TO_ARCHIVES(av::msg::rva::sensor_cleaning::CleaningStatusInfo)




 namespace av {
 namespace msg {
 namespace rva {
 namespace sensor_cleaning {


CleaningStatus::CleaningStatus() = default;

CleaningStatus::CleaningStatus(
    std::string id_arg ,
    TimePoint timestamp_arg ,
    std::vector<CleaningStatusInfo> payloads_arg 
    ):
    id { std::move(id_arg) } // NOLINT
      ,timestamp { std::move(timestamp_arg) } // NOLINT
      ,payloads { std::move(payloads_arg) } // NOLINT
      {
}

CleaningStatus::CleaningStatus(const CleaningStatus&) = default;

// NOLINTNEXTLINE
CleaningStatus::CleaningStatus(CleaningStatus&&) = default;

CleaningStatus& CleaningStatus::operator=(const CleaningStatus&) = default;

// NOLINTNEXTLINE
CleaningStatus& CleaningStatus::operator=(CleaningStatus&&) = default;

void CleaningStatus::toCsv(std::ostream& out) const
{
    out.precision(16);
    out << std::boolalpha;
    out << "\"";
        out << id;
        out << "\"";out << ",";
    out << timestamp;out << ",";
    // Enclose sequence in double quotations and curly brackets to keep CSV line length uniform
        
        out << "\"{{";
        
        if (!payloads.empty())
        {
            std::copy(payloads.begin(), payloads.end() - 1, std::ostream_iterator<CleaningStatusInfo>(out, "},{"));
            std::copy(payloads.end()-1, payloads.end(), std::ostream_iterator<CleaningStatusInfo>(out, ""));
        }
        out << "}}\"";
    
}

std::stringstream CleaningStatus::toStringstream() const
{
    std::stringstream out;
    toCsv(out);
    return out;
}

template<class Archive>
    void CleaningStatus::serialize(Archive& ar, [[maybe_unused]] unsigned int version)
    {

        ar& CEREAL_NVP(id);
        ar& CEREAL_NVP(timestamp);
        ar& CEREAL_NVP(payloads);
    }


template void CleaningStatus::serialize(cereal::BinaryInputArchive& ar, unsigned int version);
template void CleaningStatus::serialize(cereal::BinaryOutputArchive& ar, unsigned int version);
template void CleaningStatus::serialize(cereal::IntrospectionArchive& ar, unsigned int version);
template void CleaningStatus::serialize(cereal::JSONInputArchive& ar, unsigned int version);
template void CleaningStatus::serialize(cereal::JSONOutputArchive& ar, unsigned int version);
template void CleaningStatus::serialize(cereal::PortableBinaryInputArchive& ar, unsigned int version);
template void CleaningStatus::serialize(cereal::PortableBinaryOutputArchive& ar, unsigned int version);

std::ostream& operator<< (std::ostream& os, const CleaningStatus& cleaningstatus)
{
    cleaningstatus.toCsv(os);
    return os;
}

bool operator== (const CleaningStatus &lhs, const CleaningStatus &rhs)
{
    return (  lhs.id== rhs.id &&
     lhs.timestamp== rhs.timestamp &&
     lhs.payloads== rhs.payloads 
    );
}

std::size_t CleaningStatus::hash() const noexcept
{
    std::size_t result = 17;
    result = result * 31 + std::hash<std::string>()(id);
    result = result * 31 + 0;
    result = result * 31 + hashVector(payloads);
    return result;
}

 } // namespace av
 } // namespace msg
 } // namespace rva
 } // namespace sensor_cleaning


CEREAL_BIND_TO_ARCHIVES(av::msg::rva::sensor_cleaning::CleaningStatus)

