#include <interfaces/CleaningType/CleaningType.h>

#include <iterator>
#include <type_traits>

#include <cereal/cereal.hpp>
#include <cereal/archives/json.hpp>
#include <cereal/archives/binary.hpp>

#include <cereal/types/map.hpp>
#include <cereal/types/vector.hpp>
#include <cereal/types/array.hpp>
#include <cereal/types/string.hpp>
#include <cereal/types/complex.hpp>

#include <ODatum/IntrospectionArchive.h>
#include <ODatum/Hash.h>


 namespace av {
 namespace msg {
 namespace scm {


CleaningType::CleaningType() = default;

CleaningType::CleaningType(
    CleaningType_t privateData_arg 
    ):
    privateData { std::move(privateData_arg) } // NOLINT
      
    {
}

CleaningType::CleaningType(const CleaningType&) = default;

// NOLINTNEXTLINE
CleaningType::CleaningType(CleaningType&&) = default;

CleaningType& CleaningType::operator=(const CleaningType&) = default;

// NOLINTNEXTLINE
CleaningType& CleaningType::operator=(CleaningType&&) = default;

void CleaningType::toCsv(std::ostream& out) const
{
    out.precision(16);
    out << std::boolalpha;
    out << static_cast<uint32_t>(privateData);
    
}

std::stringstream CleaningType::toStringstream() const
{
    std::stringstream out;
    toCsv(out);
    return out;
}

template<class Archive>
    void CleaningType::serialize(Archive& ar, [[maybe_unused]] unsigned int version)
    {

        ar& CEREAL_NVP(privateData);
    }


template void CleaningType::serialize(cereal::BinaryInputArchive& ar, unsigned int version);
template void CleaningType::serialize(cereal::BinaryOutputArchive& ar, unsigned int version);
template void CleaningType::serialize(cereal::IntrospectionArchive& ar, unsigned int version);
template void CleaningType::serialize(cereal::JSONInputArchive& ar, unsigned int version);
template void CleaningType::serialize(cereal::JSONOutputArchive& ar, unsigned int version);
template void CleaningType::serialize(cereal::PortableBinaryInputArchive& ar, unsigned int version);
template void CleaningType::serialize(cereal::PortableBinaryOutputArchive& ar, unsigned int version);

std::ostream& operator<< (std::ostream& os, const CleaningType& cleaningtype)
{
    cleaningtype.toCsv(os);
    return os;
}

bool operator== (const CleaningType &lhs, const CleaningType &rhs)
{
    return (  lhs.privateData== rhs.privateData 
    );
}

std::size_t CleaningType::hash() const noexcept
{
    std::size_t result = 17;
    result = result * 31 + std::hash<CleaningType_t>()(privateData);
    return result;
}

 } // namespace av
 } // namespace msg
 } // namespace scm


CEREAL_BIND_TO_ARCHIVES(av::msg::scm::CleaningType)


