#include <interfaces/ClusterInfo/ClusterInfo.h>

#include <iterator>
#include <type_traits>

#include <cereal/cereal.hpp>
#include <cereal/archives/json.hpp>
#include <cereal/archives/binary.hpp>

#include <cereal/types/map.hpp>
#include <cereal/types/vector.hpp>
#include <cereal/types/array.hpp>
#include <cereal/types/string.hpp>
#include <cereal/types/complex.hpp>

#include <ODatum/IntrospectionArchive.h>
#include <ODatum/Hash.h>





 namespace av {
 namespace msg {
 namespace plan_ctrl {


ClusterInfo::ClusterInfo() = default;

ClusterInfo::ClusterInfo(
    uint64_t clusterID_arg ,
    std::vector<uint64_t> trackIDs_arg ,
    double centerX_arg ,
    double centerY_arg 
    ):
    clusterID { std::move(clusterID_arg) } // NOLINT
      ,trackIDs { std::move(trackIDs_arg) } // NOLINT
      ,centerX { std::move(centerX_arg) } // NOLINT
      ,centerY { std::move(centerY_arg) } // NOLINT
      {
}

ClusterInfo::ClusterInfo(const ClusterInfo&) = default;

// NOLINTNEXTLINE
ClusterInfo::ClusterInfo(ClusterInfo&&) = default;

ClusterInfo& ClusterInfo::operator=(const ClusterInfo&) = default;

// NOLINTNEXTLINE
ClusterInfo& ClusterInfo::operator=(ClusterInfo&&) = default;

void ClusterInfo::toCsv(std::ostream& out) const
{
    out.precision(16);
    out << std::boolalpha;
    out << clusterID;out << ",";
    // Enclose sequence in double quotations and curly brackets to keep CSV line length uniform
        
        out << "\"{{";
        
        if (!trackIDs.empty())
        {
            std::copy(trackIDs.begin(), trackIDs.end() - 1, std::ostream_iterator<uint64_t>(out, "},{"));
            std::copy(trackIDs.end()-1, trackIDs.end(), std::ostream_iterator<uint64_t>(out, ""));
        }
        out << "}}\"";out << ",";
    out << centerX;out << ",";
    out << centerY;
    
}

std::stringstream ClusterInfo::toStringstream() const
{
    std::stringstream out;
    toCsv(out);
    return out;
}

template<class Archive>
    void ClusterInfo::serialize(Archive& ar, [[maybe_unused]] unsigned int version)
    {

        ar& CEREAL_NVP(clusterID);
        ar& CEREAL_NVP(trackIDs);
        ar& CEREAL_NVP(centerX);
        ar& CEREAL_NVP(centerY);
    }


template void ClusterInfo::serialize(cereal::BinaryInputArchive& ar, unsigned int version);
template void ClusterInfo::serialize(cereal::BinaryOutputArchive& ar, unsigned int version);
template void ClusterInfo::serialize(cereal::IntrospectionArchive& ar, unsigned int version);
template void ClusterInfo::serialize(cereal::JSONInputArchive& ar, unsigned int version);
template void ClusterInfo::serialize(cereal::JSONOutputArchive& ar, unsigned int version);
template void ClusterInfo::serialize(cereal::PortableBinaryInputArchive& ar, unsigned int version);
template void ClusterInfo::serialize(cereal::PortableBinaryOutputArchive& ar, unsigned int version);

std::ostream& operator<< (std::ostream& os, const ClusterInfo& clusterinfo)
{
    clusterinfo.toCsv(os);
    return os;
}

bool operator== (const ClusterInfo &lhs, const ClusterInfo &rhs)
{
    return (  lhs.clusterID== rhs.clusterID &&
     lhs.trackIDs== rhs.trackIDs &&
     lhs.centerX== rhs.centerX &&
     lhs.centerY== rhs.centerY 
    );
}

std::size_t ClusterInfo::hash() const noexcept
{
    std::size_t result = 17;
    result = result * 31 + std::hash<uint64_t>()(clusterID);
    result = result * 31 + hashVector(trackIDs);
    result = result * 31 + std::hash<double>()(centerX);
    result = result * 31 + std::hash<double>()(centerY);
    return result;
}

 } // namespace av
 } // namespace msg
 } // namespace plan_ctrl


CEREAL_BIND_TO_ARCHIVES(av::msg::plan_ctrl::ClusterInfo)

