#include <interfaces/CompressedImageMessage/CompressedImageMessage.h>

#include <iterator>
#include <type_traits>

#include <cereal/cereal.hpp>
#include <cereal/archives/json.hpp>
#include <cereal/archives/binary.hpp>

#include <cereal/types/map.hpp>
#include <cereal/types/vector.hpp>
#include <cereal/types/array.hpp>
#include <cereal/types/string.hpp>
#include <cereal/types/complex.hpp>

#include <ODatum/IntrospectionArchive.h>
#include <ODatum/Hash.h>







CompressedImageMessage::CompressedImageMessage() = default;

CompressedImageMessage::CompressedImageMessage(
    int64_t timestamp__arg ,
    std::string frameId__arg ,
    int32_t height__arg ,
    int32_t width__arg ,
    std::vector<uint8_t> data__arg ,
    commonImageTypes::IMAGE_TYPE imageType__arg ,
    commonImageTypes::COMPRESSION_TYPE compressionType__arg 
    ):
    timestamp_ { std::move(timestamp__arg) } // NOLINT
      ,frameId_ { std::move(frameId__arg) } // NOLINT
      ,height_ { std::move(height__arg) } // NOLINT
      ,width_ { std::move(width__arg) } // NOLINT
      ,data_ { std::move(data__arg) } // NOLINT
      ,imageType_ { std::move(imageType__arg) } // NOLINT
      ,compressionType_ { std::move(compressionType__arg) } // NOLINT
      {
}

CompressedImageMessage::CompressedImageMessage(const CompressedImageMessage&) = default;

// NOLINTNEXTLINE
CompressedImageMessage::CompressedImageMessage(CompressedImageMessage&&) = default;

CompressedImageMessage& CompressedImageMessage::operator=(const CompressedImageMessage&) = default;

// NOLINTNEXTLINE
CompressedImageMessage& CompressedImageMessage::operator=(CompressedImageMessage&&) = default;

void CompressedImageMessage::toCsv(std::ostream& out) const
{
    out.precision(16);
    out << std::boolalpha;
    out << timestamp_;out << ",";
    out << "\"";
        out << frameId_;
        out << "\"";out << ",";
    out << height_;out << ",";
    out << width_;out << ",";
    // Enclose sequence in double quotations and curly brackets to keep CSV line length uniform
        
        out << "\"{{";
        
        if (!data_.empty())
        {
            std::copy(data_.begin(), data_.end() - 1, std::ostream_iterator<uint8_t>(out, "},{"));
            std::copy(data_.end()-1, data_.end(), std::ostream_iterator<uint8_t>(out, ""));
        }
        out << "}}\"";out << ",";
    out << static_cast<uint32_t>(imageType_);out << ",";
    out << static_cast<uint32_t>(compressionType_);
    
}

std::stringstream CompressedImageMessage::toStringstream() const
{
    std::stringstream out;
    toCsv(out);
    return out;
}

template<class Archive>
    void CompressedImageMessage::serialize(Archive& ar, unsigned int /*version*/)
{

    ar& CEREAL_NVP(timestamp_);
    ar& CEREAL_NVP(frameId_);
    ar& CEREAL_NVP(height_);
    ar& CEREAL_NVP(width_);
    ar& CEREAL_NVP(data_);
    ar& CEREAL_NVP(imageType_);
    ar& CEREAL_NVP(compressionType_);
}



template void CompressedImageMessage::serialize(cereal::BinaryInputArchive& ar, unsigned int version);
template void CompressedImageMessage::serialize(cereal::BinaryOutputArchive& ar, unsigned int version);
template void CompressedImageMessage::serialize(cereal::IntrospectionArchive& ar, unsigned int version);
template void CompressedImageMessage::serialize(cereal::JSONInputArchive& ar, unsigned int version);
template void CompressedImageMessage::serialize(cereal::JSONOutputArchive& ar, unsigned int version);
template void CompressedImageMessage::serialize(cereal::PortableBinaryInputArchive& ar, unsigned int version);
template void CompressedImageMessage::serialize(cereal::PortableBinaryOutputArchive& ar, unsigned int version);

std::ostream& operator<< (std::ostream& os, const CompressedImageMessage& compressedimagemessage)
{
    compressedimagemessage.toCsv(os);
    return os;
}

bool operator== (const CompressedImageMessage &lhs, const CompressedImageMessage &rhs)
{
    return (  lhs.timestamp_== rhs.timestamp_ &&
     lhs.frameId_== rhs.frameId_ &&
     lhs.height_== rhs.height_ &&
     lhs.width_== rhs.width_ &&
     lhs.data_== rhs.data_ &&
     lhs.imageType_== rhs.imageType_ &&
     lhs.compressionType_== rhs.compressionType_ 
    );
}

std::size_t CompressedImageMessage::hash() const noexcept
{
    std::size_t result = 17;
    result = result * 31 + std::hash<int64_t>()(timestamp_);
    result = result * 31 + std::hash<std::string>()(frameId_);
    result = result * 31 + std::hash<int32_t>()(height_);
    result = result * 31 + std::hash<int32_t>()(width_);
    result = result * 31 + hashVector(data_);
    result = result * 31 + std::hash<commonImageTypes::IMAGE_TYPE>()(imageType_);
    result = result * 31 + std::hash<commonImageTypes::COMPRESSION_TYPE>()(compressionType_);
    return result;
}



CEREAL_BIND_TO_ARCHIVES(CompressedImageMessage)

