#include <interfaces/ControlStep/ControlStep.h>

#include <iterator>
#include <type_traits>

#include <cereal/cereal.hpp>
#include <cereal/archives/json.hpp>
#include <cereal/archives/binary.hpp>

#include <cereal/types/map.hpp>
#include <cereal/types/vector.hpp>
#include <cereal/types/array.hpp>
#include <cereal/types/string.hpp>
#include <cereal/types/complex.hpp>

#include <ODatum/IntrospectionArchive.h>
#include <ODatum/Hash.h>


 namespace av {
 namespace msg {


ControlStep::ControlStep() = default;

ControlStep::ControlStep(
    int64_t time_us_arg ,
    double ref_arg ,
    double dotRef_arg ,
    double dotdotRef_arg 
    ):
    time_us { std::move(time_us_arg) } // NOLINT
      ,
    ref { std::move(ref_arg) } // NOLINT
      ,
    dotRef { std::move(dotRef_arg) } // NOLINT
      ,
    dotdotRef { std::move(dotdotRef_arg) } // NOLINT
      
    {
}

ControlStep::ControlStep(const ControlStep&) = default;

// NOLINTNEXTLINE
ControlStep::ControlStep(ControlStep&&) = default;

ControlStep& ControlStep::operator=(const ControlStep&) = default;

// NOLINTNEXTLINE
ControlStep& ControlStep::operator=(ControlStep&&) = default;

void ControlStep::toCsv(std::ostream& out) const
{
    out.precision(16);
    out << std::boolalpha;
    out << time_us;out << ",";
    out << ref;out << ",";
    out << dotRef;out << ",";
    out << dotdotRef;
    
}

std::stringstream ControlStep::toStringstream() const
{
    std::stringstream out;
    toCsv(out);
    return out;
}

template<class Archive>
    void ControlStep::serialize(Archive& ar, [[maybe_unused]] unsigned int version)
    {

        ar& CEREAL_NVP(time_us);
        ar& CEREAL_NVP(ref);
        ar& CEREAL_NVP(dotRef);
        ar& CEREAL_NVP(dotdotRef);
    }


template void ControlStep::serialize(cereal::BinaryInputArchive& ar, unsigned int version);
template void ControlStep::serialize(cereal::BinaryOutputArchive& ar, unsigned int version);
template void ControlStep::serialize(cereal::IntrospectionArchive& ar, unsigned int version);
template void ControlStep::serialize(cereal::JSONInputArchive& ar, unsigned int version);
template void ControlStep::serialize(cereal::JSONOutputArchive& ar, unsigned int version);
template void ControlStep::serialize(cereal::PortableBinaryInputArchive& ar, unsigned int version);
template void ControlStep::serialize(cereal::PortableBinaryOutputArchive& ar, unsigned int version);

std::ostream& operator<< (std::ostream& os, const ControlStep& controlstep)
{
    controlstep.toCsv(os);
    return os;
}

bool operator== (const ControlStep &lhs, const ControlStep &rhs)
{
    return (  lhs.time_us== rhs.time_us &&
     lhs.ref== rhs.ref &&
     lhs.dotRef== rhs.dotRef &&
     lhs.dotdotRef== rhs.dotdotRef 
    );
}

std::size_t ControlStep::hash() const noexcept
{
    std::size_t result = 17;
    result = result * 31 + std::hash<int64_t>()(time_us);
    result = result * 31 + std::hash<double>()(ref);
    result = result * 31 + std::hash<double>()(dotRef);
    result = result * 31 + std::hash<double>()(dotdotRef);
    return result;
}

 } // namespace av
 } // namespace msg


CEREAL_BIND_TO_ARCHIVES(av::msg::ControlStep)


