#include <interfaces/CustomDrawing/CustomDrawing.h>

#include <iterator>
#include <type_traits>

#include <cereal/cereal.hpp>
#include <cereal/archives/json.hpp>
#include <cereal/archives/binary.hpp>

#include <cereal/types/map.hpp>
#include <cereal/types/vector.hpp>
#include <cereal/types/array.hpp>
#include <cereal/types/string.hpp>
#include <cereal/types/complex.hpp>

#include <ODatum/IntrospectionArchive.h>
#include <ODatum/Hash.h>


 namespace av {
 namespace msg {
 namespace planner {


Color::Color() = default;

Color::Color(
    float r_arg ,
    float g_arg ,
    float b_arg ,
    float a_arg 
    ):
    r { std::move(r_arg) } // NOLINT
      ,
    g { std::move(g_arg) } // NOLINT
      ,
    b { std::move(b_arg) } // NOLINT
      ,
    a { std::move(a_arg) } // NOLINT
      
    {
}

Color::Color(const Color&) = default;

// NOLINTNEXTLINE
Color::Color(Color&&) = default;

Color& Color::operator=(const Color&) = default;

// NOLINTNEXTLINE
Color& Color::operator=(Color&&) = default;

void Color::toCsv(std::ostream& out) const
{
    out.precision(16);
    out << std::boolalpha;
    out << r;out << ",";
    out << g;out << ",";
    out << b;out << ",";
    out << a;
    
}

std::stringstream Color::toStringstream() const
{
    std::stringstream out;
    toCsv(out);
    return out;
}

template<class Archive>
    void Color::serialize(Archive& ar, [[maybe_unused]] unsigned int version)
    {

        ar& CEREAL_NVP(r);
        ar& CEREAL_NVP(g);
        ar& CEREAL_NVP(b);
        ar& CEREAL_NVP(a);
    }


template void Color::serialize(cereal::BinaryInputArchive& ar, unsigned int version);
template void Color::serialize(cereal::BinaryOutputArchive& ar, unsigned int version);
template void Color::serialize(cereal::IntrospectionArchive& ar, unsigned int version);
template void Color::serialize(cereal::JSONInputArchive& ar, unsigned int version);
template void Color::serialize(cereal::JSONOutputArchive& ar, unsigned int version);
template void Color::serialize(cereal::PortableBinaryInputArchive& ar, unsigned int version);
template void Color::serialize(cereal::PortableBinaryOutputArchive& ar, unsigned int version);

std::ostream& operator<< (std::ostream& os, const Color& color)
{
    color.toCsv(os);
    return os;
}

bool operator== (const Color &lhs, const Color &rhs)
{
    return (  lhs.r== rhs.r &&
     lhs.g== rhs.g &&
     lhs.b== rhs.b &&
     lhs.a== rhs.a 
    );
}

std::size_t Color::hash() const noexcept
{
    std::size_t result = 17;
    result = result * 31 + std::hash<float>()(r);
    result = result * 31 + std::hash<float>()(g);
    result = result * 31 + std::hash<float>()(b);
    result = result * 31 + std::hash<float>()(a);
    return result;
}

 } // namespace av
 } // namespace msg
 } // namespace planner


CEREAL_BIND_TO_ARCHIVES(av::msg::planner::Color)

 namespace av {
 namespace msg {
 namespace planner {


Point::Point() = default;

Point::Point(
    double x_arg ,
    double y_arg ,
    double z_arg 
    ):
    x { std::move(x_arg) } // NOLINT
      ,
    y { std::move(y_arg) } // NOLINT
      ,
    z { std::move(z_arg) } // NOLINT
      
    {
}

Point::Point(const Point&) = default;

// NOLINTNEXTLINE
Point::Point(Point&&) = default;

Point& Point::operator=(const Point&) = default;

// NOLINTNEXTLINE
Point& Point::operator=(Point&&) = default;

void Point::toCsv(std::ostream& out) const
{
    out.precision(16);
    out << std::boolalpha;
    out << x;out << ",";
    out << y;out << ",";
    out << z;
    
}

std::stringstream Point::toStringstream() const
{
    std::stringstream out;
    toCsv(out);
    return out;
}

template<class Archive>
    void Point::serialize(Archive& ar, [[maybe_unused]] unsigned int version)
    {

        ar& CEREAL_NVP(x);
        ar& CEREAL_NVP(y);
        ar& CEREAL_NVP(z);
    }


template void Point::serialize(cereal::BinaryInputArchive& ar, unsigned int version);
template void Point::serialize(cereal::BinaryOutputArchive& ar, unsigned int version);
template void Point::serialize(cereal::IntrospectionArchive& ar, unsigned int version);
template void Point::serialize(cereal::JSONInputArchive& ar, unsigned int version);
template void Point::serialize(cereal::JSONOutputArchive& ar, unsigned int version);
template void Point::serialize(cereal::PortableBinaryInputArchive& ar, unsigned int version);
template void Point::serialize(cereal::PortableBinaryOutputArchive& ar, unsigned int version);

std::ostream& operator<< (std::ostream& os, const Point& point)
{
    point.toCsv(os);
    return os;
}

bool operator== (const Point &lhs, const Point &rhs)
{
    return (  lhs.x== rhs.x &&
     lhs.y== rhs.y &&
     lhs.z== rhs.z 
    );
}

std::size_t Point::hash() const noexcept
{
    std::size_t result = 17;
    result = result * 31 + std::hash<double>()(x);
    result = result * 31 + std::hash<double>()(y);
    result = result * 31 + std::hash<double>()(z);
    return result;
}

 } // namespace av
 } // namespace msg
 } // namespace planner


CEREAL_BIND_TO_ARCHIVES(av::msg::planner::Point)

 namespace av {
 namespace msg {
 namespace planner {


DrawingPoint::DrawingPoint() = default;

DrawingPoint::DrawingPoint(
    Point vertex_arg ,
    Color color_arg ,
    float size_arg 
    ):
    vertex { std::move(vertex_arg) } // NOLINT
      ,
    color { std::move(color_arg) } // NOLINT
      ,
    size { std::move(size_arg) } // NOLINT
      
    {
}

DrawingPoint::DrawingPoint(const DrawingPoint&) = default;

// NOLINTNEXTLINE
DrawingPoint::DrawingPoint(DrawingPoint&&) = default;

DrawingPoint& DrawingPoint::operator=(const DrawingPoint&) = default;

// NOLINTNEXTLINE
DrawingPoint& DrawingPoint::operator=(DrawingPoint&&) = default;

void DrawingPoint::toCsv(std::ostream& out) const
{
    out.precision(16);
    out << std::boolalpha;
    out << vertex;out << ",";
    out << color;out << ",";
    out << size;
    
}

std::stringstream DrawingPoint::toStringstream() const
{
    std::stringstream out;
    toCsv(out);
    return out;
}

template<class Archive>
    void DrawingPoint::serialize(Archive& ar, [[maybe_unused]] unsigned int version)
    {

        ar& CEREAL_NVP(vertex);
        ar& CEREAL_NVP(color);
        ar& CEREAL_NVP(size);
    }


template void DrawingPoint::serialize(cereal::BinaryInputArchive& ar, unsigned int version);
template void DrawingPoint::serialize(cereal::BinaryOutputArchive& ar, unsigned int version);
template void DrawingPoint::serialize(cereal::IntrospectionArchive& ar, unsigned int version);
template void DrawingPoint::serialize(cereal::JSONInputArchive& ar, unsigned int version);
template void DrawingPoint::serialize(cereal::JSONOutputArchive& ar, unsigned int version);
template void DrawingPoint::serialize(cereal::PortableBinaryInputArchive& ar, unsigned int version);
template void DrawingPoint::serialize(cereal::PortableBinaryOutputArchive& ar, unsigned int version);

std::ostream& operator<< (std::ostream& os, const DrawingPoint& drawingpoint)
{
    drawingpoint.toCsv(os);
    return os;
}

bool operator== (const DrawingPoint &lhs, const DrawingPoint &rhs)
{
    return (  lhs.vertex== rhs.vertex &&
     lhs.color== rhs.color &&
     lhs.size== rhs.size 
    );
}

std::size_t DrawingPoint::hash() const noexcept
{
    std::size_t result = 17;
    result = result * 31 + std::hash<Point>()(vertex);
    result = result * 31 + std::hash<Color>()(color);
    result = result * 31 + std::hash<float>()(size);
    return result;
}

 } // namespace av
 } // namespace msg
 } // namespace planner


CEREAL_BIND_TO_ARCHIVES(av::msg::planner::DrawingPoint)

 namespace av {
 namespace msg {
 namespace planner {


DrawingLine::DrawingLine() = default;

DrawingLine::DrawingLine(
    Point start_arg ,
    Point end_arg ,
    Color color_arg ,
    float width_arg 
    ):
    start { std::move(start_arg) } // NOLINT
      ,
    end { std::move(end_arg) } // NOLINT
      ,
    color { std::move(color_arg) } // NOLINT
      ,
    width { std::move(width_arg) } // NOLINT
      
    {
}

DrawingLine::DrawingLine(const DrawingLine&) = default;

// NOLINTNEXTLINE
DrawingLine::DrawingLine(DrawingLine&&) = default;

DrawingLine& DrawingLine::operator=(const DrawingLine&) = default;

// NOLINTNEXTLINE
DrawingLine& DrawingLine::operator=(DrawingLine&&) = default;

void DrawingLine::toCsv(std::ostream& out) const
{
    out.precision(16);
    out << std::boolalpha;
    out << start;out << ",";
    out << end;out << ",";
    out << color;out << ",";
    out << width;
    
}

std::stringstream DrawingLine::toStringstream() const
{
    std::stringstream out;
    toCsv(out);
    return out;
}

template<class Archive>
    void DrawingLine::serialize(Archive& ar, [[maybe_unused]] unsigned int version)
    {

        ar& CEREAL_NVP(start);
        ar& CEREAL_NVP(end);
        ar& CEREAL_NVP(color);
        ar& CEREAL_NVP(width);
    }


template void DrawingLine::serialize(cereal::BinaryInputArchive& ar, unsigned int version);
template void DrawingLine::serialize(cereal::BinaryOutputArchive& ar, unsigned int version);
template void DrawingLine::serialize(cereal::IntrospectionArchive& ar, unsigned int version);
template void DrawingLine::serialize(cereal::JSONInputArchive& ar, unsigned int version);
template void DrawingLine::serialize(cereal::JSONOutputArchive& ar, unsigned int version);
template void DrawingLine::serialize(cereal::PortableBinaryInputArchive& ar, unsigned int version);
template void DrawingLine::serialize(cereal::PortableBinaryOutputArchive& ar, unsigned int version);

std::ostream& operator<< (std::ostream& os, const DrawingLine& drawingline)
{
    drawingline.toCsv(os);
    return os;
}

bool operator== (const DrawingLine &lhs, const DrawingLine &rhs)
{
    return (  lhs.start== rhs.start &&
     lhs.end== rhs.end &&
     lhs.color== rhs.color &&
     lhs.width== rhs.width 
    );
}

std::size_t DrawingLine::hash() const noexcept
{
    std::size_t result = 17;
    result = result * 31 + std::hash<Point>()(start);
    result = result * 31 + std::hash<Point>()(end);
    result = result * 31 + std::hash<Color>()(color);
    result = result * 31 + std::hash<float>()(width);
    return result;
}

 } // namespace av
 } // namespace msg
 } // namespace planner


CEREAL_BIND_TO_ARCHIVES(av::msg::planner::DrawingLine)




 namespace av {
 namespace msg {
 namespace planner {


CustomDrawing::CustomDrawing() = default;

CustomDrawing::CustomDrawing(
    std::vector<DrawingPoint> points_arg ,
    std::vector<DrawingLine> lines_arg 
    ):
    points { std::move(points_arg) } // NOLINT
      ,lines { std::move(lines_arg) } // NOLINT
      {
}

CustomDrawing::CustomDrawing(const CustomDrawing&) = default;

// NOLINTNEXTLINE
CustomDrawing::CustomDrawing(CustomDrawing&&) = default;

CustomDrawing& CustomDrawing::operator=(const CustomDrawing&) = default;

// NOLINTNEXTLINE
CustomDrawing& CustomDrawing::operator=(CustomDrawing&&) = default;

void CustomDrawing::toCsv(std::ostream& out) const
{
    out.precision(16);
    out << std::boolalpha;
    // Enclose sequence in double quotations and curly brackets to keep CSV line length uniform
        
        out << "\"{{";
        
        if (!points.empty())
        {
            std::copy(points.begin(), points.end() - 1, std::ostream_iterator<DrawingPoint>(out, "},{"));
            std::copy(points.end()-1, points.end(), std::ostream_iterator<DrawingPoint>(out, ""));
        }
        out << "}}\"";out << ",";
    // Enclose sequence in double quotations and curly brackets to keep CSV line length uniform
        
        out << "\"{{";
        
        if (!lines.empty())
        {
            std::copy(lines.begin(), lines.end() - 1, std::ostream_iterator<DrawingLine>(out, "},{"));
            std::copy(lines.end()-1, lines.end(), std::ostream_iterator<DrawingLine>(out, ""));
        }
        out << "}}\"";
    
}

std::stringstream CustomDrawing::toStringstream() const
{
    std::stringstream out;
    toCsv(out);
    return out;
}

template<class Archive>
    void CustomDrawing::serialize(Archive& ar, [[maybe_unused]] unsigned int version)
    {

        ar& CEREAL_NVP(points);
        ar& CEREAL_NVP(lines);
    }


template void CustomDrawing::serialize(cereal::BinaryInputArchive& ar, unsigned int version);
template void CustomDrawing::serialize(cereal::BinaryOutputArchive& ar, unsigned int version);
template void CustomDrawing::serialize(cereal::IntrospectionArchive& ar, unsigned int version);
template void CustomDrawing::serialize(cereal::JSONInputArchive& ar, unsigned int version);
template void CustomDrawing::serialize(cereal::JSONOutputArchive& ar, unsigned int version);
template void CustomDrawing::serialize(cereal::PortableBinaryInputArchive& ar, unsigned int version);
template void CustomDrawing::serialize(cereal::PortableBinaryOutputArchive& ar, unsigned int version);

std::ostream& operator<< (std::ostream& os, const CustomDrawing& customdrawing)
{
    customdrawing.toCsv(os);
    return os;
}

bool operator== (const CustomDrawing &lhs, const CustomDrawing &rhs)
{
    return (  lhs.points== rhs.points &&
     lhs.lines== rhs.lines 
    );
}

std::size_t CustomDrawing::hash() const noexcept
{
    std::size_t result = 17;
    result = result * 31 + hashVector(points);
    result = result * 31 + hashVector(lines);
    return result;
}

 } // namespace av
 } // namespace msg
 } // namespace planner


CEREAL_BIND_TO_ARCHIVES(av::msg::planner::CustomDrawing)

