#include <interfaces/D3Image/D3Image.h>

#include <iterator>
#include <type_traits>

#include <cereal/cereal.hpp>
#include <cereal/archives/json.hpp>
#include <cereal/archives/binary.hpp>

#include <cereal/types/map.hpp>
#include <cereal/types/vector.hpp>
#include <cereal/types/array.hpp>
#include <cereal/types/string.hpp>
#include <cereal/types/complex.hpp>

#include <ODatum/IntrospectionArchive.h>
#include <ODatum/Hash.h>







D3Image::D3Image() = default;

D3Image::D3Image(
    std::vector<uint8_t> data_arg ,
    uint32_t width_arg ,
    uint32_t height_arg ,
    d3::D3_IMAGE_TYPE imageType_arg ,
    d3::D3_TIME_SYNC_STATUS timeSyncStatus_arg ,
    int64_t captureTime_ns_arg ,
    int64_t ingestTime_ns_arg ,
    int64_t readOutTime_ns_arg ,
    float digitalGain_arg ,
    float analogGain1_arg ,
    float analogGain2_arg ,
    float whiteBalanceCF00_arg ,
    float whiteBalanceCF01_arg ,
    float whiteBalanceCF10_arg ,
    float whiteBalanceCF11_arg ,
    uint32_t exposureHigh_us_arg ,
    uint32_t exposureLow_us_arg ,
    int64_t expectedTriggerTime_ns_arg ,
    d3::D3ImageDebugInfo debugInfo_arg 
    ):
    data { std::move(data_arg) } // NOLINT
      ,width { std::move(width_arg) } // NOLINT
      ,height { std::move(height_arg) } // NOLINT
      ,imageType { std::move(imageType_arg) } // NOLINT
      ,timeSyncStatus { std::move(timeSyncStatus_arg) } // NOLINT
      ,captureTime_ns { std::move(captureTime_ns_arg) } // NOLINT
      ,ingestTime_ns { std::move(ingestTime_ns_arg) } // NOLINT
      ,readOutTime_ns { std::move(readOutTime_ns_arg) } // NOLINT
      ,digitalGain { std::move(digitalGain_arg) } // NOLINT
      ,analogGain1 { std::move(analogGain1_arg) } // NOLINT
      ,analogGain2 { std::move(analogGain2_arg) } // NOLINT
      ,whiteBalanceCF00 { std::move(whiteBalanceCF00_arg) } // NOLINT
      ,whiteBalanceCF01 { std::move(whiteBalanceCF01_arg) } // NOLINT
      ,whiteBalanceCF10 { std::move(whiteBalanceCF10_arg) } // NOLINT
      ,whiteBalanceCF11 { std::move(whiteBalanceCF11_arg) } // NOLINT
      ,exposureHigh_us { std::move(exposureHigh_us_arg) } // NOLINT
      ,exposureLow_us { std::move(exposureLow_us_arg) } // NOLINT
      ,expectedTriggerTime_ns { std::move(expectedTriggerTime_ns_arg) } // NOLINT
      ,debugInfo { std::move(debugInfo_arg) } // NOLINT
      {
}

D3Image::D3Image(const D3Image&) = default;

// NOLINTNEXTLINE
D3Image::D3Image(D3Image&&) = default;

D3Image& D3Image::operator=(const D3Image&) = default;

// NOLINTNEXTLINE
D3Image& D3Image::operator=(D3Image&&) = default;

void D3Image::toCsv(std::ostream& out) const
{
    out.precision(16);
    out << std::boolalpha;
    // Enclose sequence in double quotations and curly brackets to keep CSV line length uniform
        
        out << "\"{{";
        
        if (!data.empty())
        {
            std::copy(data.begin(), data.end() - 1, std::ostream_iterator<uint8_t>(out, "},{"));
            std::copy(data.end()-1, data.end(), std::ostream_iterator<uint8_t>(out, ""));
        }
        out << "}}\"";out << ",";
    out << width;out << ",";
    out << height;out << ",";
    out << static_cast<uint32_t>(imageType);out << ",";
    out << static_cast<uint32_t>(timeSyncStatus);out << ",";
    out << captureTime_ns;out << ",";
    out << ingestTime_ns;out << ",";
    out << readOutTime_ns;out << ",";
    out << digitalGain;out << ",";
    out << analogGain1;out << ",";
    out << analogGain2;out << ",";
    out << whiteBalanceCF00;out << ",";
    out << whiteBalanceCF01;out << ",";
    out << whiteBalanceCF10;out << ",";
    out << whiteBalanceCF11;out << ",";
    out << exposureHigh_us;out << ",";
    out << exposureLow_us;out << ",";
    out << expectedTriggerTime_ns;out << ",";
    out << debugInfo;
    
}

std::stringstream D3Image::toStringstream() const
{
    std::stringstream out;
    toCsv(out);
    return out;
}

template<class Archive>
    void D3Image::serialize(Archive& ar, [[maybe_unused]] unsigned int version)
    {

        ar& CEREAL_NVP(data);
        ar& CEREAL_NVP(width);
        ar& CEREAL_NVP(height);
        ar& CEREAL_NVP(imageType);
        ar& CEREAL_NVP(timeSyncStatus);
        ar& CEREAL_NVP(captureTime_ns);
        ar& CEREAL_NVP(ingestTime_ns);
        ar& CEREAL_NVP(readOutTime_ns);
        ar& CEREAL_NVP(digitalGain);
        ar& CEREAL_NVP(analogGain1);
        ar& CEREAL_NVP(analogGain2);
        ar& CEREAL_NVP(whiteBalanceCF00);
        ar& CEREAL_NVP(whiteBalanceCF01);
        ar& CEREAL_NVP(whiteBalanceCF10);
        ar& CEREAL_NVP(whiteBalanceCF11);
        ar& CEREAL_NVP(exposureHigh_us);
        ar& CEREAL_NVP(exposureLow_us);
        ar& CEREAL_NVP(expectedTriggerTime_ns);
        ar& CEREAL_NVP(debugInfo);
    }


template void D3Image::serialize(cereal::BinaryInputArchive& ar, unsigned int version);
template void D3Image::serialize(cereal::BinaryOutputArchive& ar, unsigned int version);
template void D3Image::serialize(cereal::IntrospectionArchive& ar, unsigned int version);
template void D3Image::serialize(cereal::JSONInputArchive& ar, unsigned int version);
template void D3Image::serialize(cereal::JSONOutputArchive& ar, unsigned int version);
template void D3Image::serialize(cereal::PortableBinaryInputArchive& ar, unsigned int version);
template void D3Image::serialize(cereal::PortableBinaryOutputArchive& ar, unsigned int version);

std::ostream& operator<< (std::ostream& os, const D3Image& d3image)
{
    d3image.toCsv(os);
    return os;
}

bool operator== (const D3Image &lhs, const D3Image &rhs)
{
    return (  lhs.data== rhs.data &&
     lhs.width== rhs.width &&
     lhs.height== rhs.height &&
     lhs.imageType== rhs.imageType &&
     lhs.timeSyncStatus== rhs.timeSyncStatus &&
     lhs.captureTime_ns== rhs.captureTime_ns &&
     lhs.ingestTime_ns== rhs.ingestTime_ns &&
     lhs.readOutTime_ns== rhs.readOutTime_ns &&
     lhs.digitalGain== rhs.digitalGain &&
     lhs.analogGain1== rhs.analogGain1 &&
     lhs.analogGain2== rhs.analogGain2 &&
     lhs.whiteBalanceCF00== rhs.whiteBalanceCF00 &&
     lhs.whiteBalanceCF01== rhs.whiteBalanceCF01 &&
     lhs.whiteBalanceCF10== rhs.whiteBalanceCF10 &&
     lhs.whiteBalanceCF11== rhs.whiteBalanceCF11 &&
     lhs.exposureHigh_us== rhs.exposureHigh_us &&
     lhs.exposureLow_us== rhs.exposureLow_us &&
     lhs.expectedTriggerTime_ns== rhs.expectedTriggerTime_ns &&
     lhs.debugInfo== rhs.debugInfo 
    );
}

std::size_t D3Image::hash() const noexcept
{
    std::size_t result = 17;
    result = result * 31 + hashVector(data);
    result = result * 31 + std::hash<uint32_t>()(width);
    result = result * 31 + std::hash<uint32_t>()(height);
    result = result * 31 + std::hash<d3::D3_IMAGE_TYPE>()(imageType);
    result = result * 31 + std::hash<d3::D3_TIME_SYNC_STATUS>()(timeSyncStatus);
    result = result * 31 + std::hash<int64_t>()(captureTime_ns);
    result = result * 31 + std::hash<int64_t>()(ingestTime_ns);
    result = result * 31 + std::hash<int64_t>()(readOutTime_ns);
    result = result * 31 + std::hash<float>()(digitalGain);
    result = result * 31 + std::hash<float>()(analogGain1);
    result = result * 31 + std::hash<float>()(analogGain2);
    result = result * 31 + std::hash<float>()(whiteBalanceCF00);
    result = result * 31 + std::hash<float>()(whiteBalanceCF01);
    result = result * 31 + std::hash<float>()(whiteBalanceCF10);
    result = result * 31 + std::hash<float>()(whiteBalanceCF11);
    result = result * 31 + std::hash<uint32_t>()(exposureHigh_us);
    result = result * 31 + std::hash<uint32_t>()(exposureLow_us);
    result = result * 31 + std::hash<int64_t>()(expectedTriggerTime_ns);
    result = result * 31 + std::hash<d3::D3ImageDebugInfo>()(debugInfo);
    return result;
}



CEREAL_BIND_TO_ARCHIVES(D3Image)

