#include <interfaces/D3ImageDebugInfo/D3ImageDebugInfo.h>

#include <iterator>
#include <type_traits>

#include <cereal/cereal.hpp>
#include <cereal/archives/json.hpp>
#include <cereal/archives/binary.hpp>

#include <cereal/types/map.hpp>
#include <cereal/types/vector.hpp>
#include <cereal/types/array.hpp>
#include <cereal/types/string.hpp>
#include <cereal/types/complex.hpp>

#include <ODatum/IntrospectionArchive.h>
#include <ODatum/Hash.h>





 namespace d3 {


D3ImageDebugInfo::D3ImageDebugInfo() = default;

D3ImageDebugInfo::D3ImageDebugInfo(
    std::string serialNumber_arg ,
    std::vector<uint8_t> rawFrontData_arg ,
    std::vector<uint8_t> rawRearData_arg ,
    FRAME_ERROR frameErrorCode_arg 
    ):
    serialNumber { std::move(serialNumber_arg) } // NOLINT
      ,rawFrontData { std::move(rawFrontData_arg) } // NOLINT
      ,rawRearData { std::move(rawRearData_arg) } // NOLINT
      ,frameErrorCode { std::move(frameErrorCode_arg) } // NOLINT
      {
}

D3ImageDebugInfo::D3ImageDebugInfo(const D3ImageDebugInfo&) = default;

// NOLINTNEXTLINE
D3ImageDebugInfo::D3ImageDebugInfo(D3ImageDebugInfo&&) = default;

D3ImageDebugInfo& D3ImageDebugInfo::operator=(const D3ImageDebugInfo&) = default;

// NOLINTNEXTLINE
D3ImageDebugInfo& D3ImageDebugInfo::operator=(D3ImageDebugInfo&&) = default;

void D3ImageDebugInfo::toCsv(std::ostream& out) const
{
    out.precision(16);
    out << std::boolalpha;
    out << "\"";
        out << serialNumber;
        out << "\"";out << ",";
    // Enclose sequence in double quotations and curly brackets to keep CSV line length uniform
        
        out << "\"{{";
        
        if (!rawFrontData.empty())
        {
            std::copy(rawFrontData.begin(), rawFrontData.end() - 1, std::ostream_iterator<uint8_t>(out, "},{"));
            std::copy(rawFrontData.end()-1, rawFrontData.end(), std::ostream_iterator<uint8_t>(out, ""));
        }
        out << "}}\"";out << ",";
    // Enclose sequence in double quotations and curly brackets to keep CSV line length uniform
        
        out << "\"{{";
        
        if (!rawRearData.empty())
        {
            std::copy(rawRearData.begin(), rawRearData.end() - 1, std::ostream_iterator<uint8_t>(out, "},{"));
            std::copy(rawRearData.end()-1, rawRearData.end(), std::ostream_iterator<uint8_t>(out, ""));
        }
        out << "}}\"";out << ",";
    out << static_cast<uint32_t>(frameErrorCode);
    
}

std::stringstream D3ImageDebugInfo::toStringstream() const
{
    std::stringstream out;
    toCsv(out);
    return out;
}

template<class Archive>
    void D3ImageDebugInfo::serialize(Archive& ar, unsigned int version)
{

    ar& CEREAL_NVP(serialNumber);
    ar& CEREAL_NVP(rawFrontData);
    ar& CEREAL_NVP(rawRearData);
    if(version >= 2)
    {
        ar& CEREAL_NVP(frameErrorCode);
    }
}



template void D3ImageDebugInfo::serialize(cereal::BinaryInputArchive& ar, unsigned int version);
template void D3ImageDebugInfo::serialize(cereal::BinaryOutputArchive& ar, unsigned int version);
template void D3ImageDebugInfo::serialize(cereal::IntrospectionArchive& ar, unsigned int version);
template void D3ImageDebugInfo::serialize(cereal::JSONInputArchive& ar, unsigned int version);
template void D3ImageDebugInfo::serialize(cereal::JSONOutputArchive& ar, unsigned int version);
template void D3ImageDebugInfo::serialize(cereal::PortableBinaryInputArchive& ar, unsigned int version);
template void D3ImageDebugInfo::serialize(cereal::PortableBinaryOutputArchive& ar, unsigned int version);

std::ostream& operator<< (std::ostream& os, const D3ImageDebugInfo& d3imagedebuginfo)
{
    d3imagedebuginfo.toCsv(os);
    return os;
}

bool operator== (const D3ImageDebugInfo &lhs, const D3ImageDebugInfo &rhs)
{
    return (  lhs.serialNumber== rhs.serialNumber &&
     lhs.rawFrontData== rhs.rawFrontData &&
     lhs.rawRearData== rhs.rawRearData &&
     lhs.frameErrorCode== rhs.frameErrorCode 
    );
}

std::size_t D3ImageDebugInfo::hash() const noexcept
{
    std::size_t result = 17;
    result = result * 31 + std::hash<std::string>()(serialNumber);
    result = result * 31 + hashVector(rawFrontData);
    result = result * 31 + hashVector(rawRearData);
    result = result * 31 + std::hash<FRAME_ERROR>()(frameErrorCode);
    return result;
}

 } // namespace d3


CEREAL_BIND_TO_ARCHIVES(d3::D3ImageDebugInfo)

