#include <interfaces/DCTStatus/DCTStatus.h>

#include <iterator>
#include <type_traits>

#include <cereal/cereal.hpp>
#include <cereal/archives/json.hpp>
#include <cereal/archives/binary.hpp>

#include <cereal/types/map.hpp>
#include <cereal/types/vector.hpp>
#include <cereal/types/array.hpp>
#include <cereal/types/string.hpp>
#include <cereal/types/complex.hpp>

#include <ODatum/IntrospectionArchive.h>
#include <ODatum/Hash.h>







DCTStatus::DCTStatus() = default;

DCTStatus::DCTStatus(
    bool dspaceControllerTaskStatus_arg ,
    Timestamp128 timeStamp_arg ,
    bool dspaceControllerMode_arg 
    ):
    dspaceControllerTaskStatus { std::move(dspaceControllerTaskStatus_arg) } // NOLINT
      ,timeStamp { std::move(timeStamp_arg) } // NOLINT
      ,dspaceControllerMode { std::move(dspaceControllerMode_arg) } // NOLINT
      {
}

DCTStatus::DCTStatus(const DCTStatus&) = default;

// NOLINTNEXTLINE
DCTStatus::DCTStatus(DCTStatus&&) = default;

DCTStatus& DCTStatus::operator=(const DCTStatus&) = default;

// NOLINTNEXTLINE
DCTStatus& DCTStatus::operator=(DCTStatus&&) = default;

void DCTStatus::toCsv(std::ostream& out) const
{
    out.precision(16);
    out << std::boolalpha;
    out << dspaceControllerTaskStatus;out << ",";
    out << timeStamp;out << ",";
    out << dspaceControllerMode;
    
}

std::stringstream DCTStatus::toStringstream() const
{
    std::stringstream out;
    toCsv(out);
    return out;
}

template<class Archive>
    void DCTStatus::serialize(Archive& ar, unsigned int version)
{
	if (version >= 1)
	{
		ar& CEREAL_NVP(dspaceControllerTaskStatus);
	}
	if (version >= 2)
	{
		ar& CEREAL_NVP(timeStamp);
		ar& CEREAL_NVP(dspaceControllerMode);
	}
}



template void DCTStatus::serialize(cereal::BinaryInputArchive& ar, unsigned int version);
template void DCTStatus::serialize(cereal::BinaryOutputArchive& ar, unsigned int version);
template void DCTStatus::serialize(cereal::IntrospectionArchive& ar, unsigned int version);
template void DCTStatus::serialize(cereal::JSONInputArchive& ar, unsigned int version);
template void DCTStatus::serialize(cereal::JSONOutputArchive& ar, unsigned int version);
template void DCTStatus::serialize(cereal::PortableBinaryInputArchive& ar, unsigned int version);
template void DCTStatus::serialize(cereal::PortableBinaryOutputArchive& ar, unsigned int version);

std::ostream& operator<< (std::ostream& os, const DCTStatus& dctstatus)
{
    dctstatus.toCsv(os);
    return os;
}

bool operator== (const DCTStatus &lhs, const DCTStatus &rhs)
{
    return (  lhs.dspaceControllerTaskStatus== rhs.dspaceControllerTaskStatus &&
     lhs.timeStamp== rhs.timeStamp &&
     lhs.dspaceControllerMode== rhs.dspaceControllerMode 
    );
}

std::size_t DCTStatus::hash() const noexcept
{
    std::size_t result = 17;
    result = result * 31 + std::hash<bool>()(dspaceControllerTaskStatus);
    result = result * 31 + std::hash<Timestamp128>()(timeStamp);
    result = result * 31 + std::hash<bool>()(dspaceControllerMode);
    return result;
}



CEREAL_BIND_TO_ARCHIVES(DCTStatus)

