#include <interfaces/DrivingMetrics/DrivingMetrics.h>

#include <iterator>
#include <type_traits>

#include <cereal/cereal.hpp>
#include <cereal/archives/json.hpp>
#include <cereal/archives/binary.hpp>

#include <cereal/types/map.hpp>
#include <cereal/types/vector.hpp>
#include <cereal/types/array.hpp>
#include <cereal/types/string.hpp>
#include <cereal/types/complex.hpp>

#include <ODatum/IntrospectionArchive.h>
#include <ODatum/Hash.h>





 namespace av {
 namespace msg {


DrivingMetrics::DrivingMetrics() = default;

DrivingMetrics::DrivingMetrics(
    TimePoint timeStamp_arg ,
    uint32_t numIntersection_arg ,
    uint32_t numHardStop_arg 
    ):
    timeStamp { std::move(timeStamp_arg) } // NOLINT
      ,numIntersection { std::move(numIntersection_arg) } // NOLINT
      ,numHardStop { std::move(numHardStop_arg) } // NOLINT
      {
}

DrivingMetrics::DrivingMetrics(const DrivingMetrics&) = default;

// NOLINTNEXTLINE
DrivingMetrics::DrivingMetrics(DrivingMetrics&&) = default;

DrivingMetrics& DrivingMetrics::operator=(const DrivingMetrics&) = default;

// NOLINTNEXTLINE
DrivingMetrics& DrivingMetrics::operator=(DrivingMetrics&&) = default;

void DrivingMetrics::toCsv(std::ostream& out) const
{
    out.precision(16);
    out << std::boolalpha;
    out << timeStamp;out << ",";
    out << numIntersection;out << ",";
    out << numHardStop;
    
}

std::stringstream DrivingMetrics::toStringstream() const
{
    std::stringstream out;
    toCsv(out);
    return out;
}

template<class Archive>
    void DrivingMetrics::serialize(Archive& ar, unsigned int version)
{
    if (version < 2)
    {
        otimePoint otp;
        ar& cereal::make_nvp("timeStamp", otp);
        timeStamp = av::TimePoint(std::chrono::seconds(otp.tv_sec) + std::chrono::nanoseconds(otp.tv_nsec));
    }
    else
    {
        ar& CEREAL_NVP(timeStamp);
    }

    ar& CEREAL_NVP(numIntersection);
    ar& CEREAL_NVP(numHardStop);
}

}} // to close av::msg namespace temporary

struct DrivingMetrics : public av::msg::DrivingMetrics{};
CEREAL_CLASS_VERSION(DrivingMetrics, 2)
CEREAL_REGISTER_TYPE(DrivingMetrics)

namespace av {
namespace msg {



template void DrivingMetrics::serialize(cereal::BinaryInputArchive& ar, unsigned int version);
template void DrivingMetrics::serialize(cereal::BinaryOutputArchive& ar, unsigned int version);
template void DrivingMetrics::serialize(cereal::IntrospectionArchive& ar, unsigned int version);
template void DrivingMetrics::serialize(cereal::JSONInputArchive& ar, unsigned int version);
template void DrivingMetrics::serialize(cereal::JSONOutputArchive& ar, unsigned int version);
template void DrivingMetrics::serialize(cereal::PortableBinaryInputArchive& ar, unsigned int version);
template void DrivingMetrics::serialize(cereal::PortableBinaryOutputArchive& ar, unsigned int version);

std::ostream& operator<< (std::ostream& os, const DrivingMetrics& drivingmetrics)
{
    drivingmetrics.toCsv(os);
    return os;
}

bool operator== (const DrivingMetrics &lhs, const DrivingMetrics &rhs)
{
    return (  lhs.timeStamp== rhs.timeStamp &&
     lhs.numIntersection== rhs.numIntersection &&
     lhs.numHardStop== rhs.numHardStop 
    );
}

std::size_t DrivingMetrics::hash() const noexcept
{
    std::size_t result = 17;
    result = result * 31 + 0;
    result = result * 31 + std::hash<uint32_t>()(numIntersection);
    result = result * 31 + std::hash<uint32_t>()(numHardStop);
    return result;
}

 } // namespace av
 } // namespace msg


CEREAL_BIND_TO_ARCHIVES(av::msg::DrivingMetrics)

