#include <interfaces/DrivingScenario/DrivingScenario.h>

#include <iterator>
#include <type_traits>

#include <cereal/cereal.hpp>
#include <cereal/archives/json.hpp>
#include <cereal/archives/binary.hpp>

#include <cereal/types/map.hpp>
#include <cereal/types/vector.hpp>
#include <cereal/types/array.hpp>
#include <cereal/types/string.hpp>
#include <cereal/types/complex.hpp>

#include <ODatum/IntrospectionArchive.h>
#include <ODatum/Hash.h>





 namespace av {
 namespace msg {
 namespace plan_ctrl {


DrivingScenario::DrivingScenario() = default;

DrivingScenario::DrivingScenario(
    int64_t timestamp_us_arg ,
    int64_t counter_arg ,
    DRIVING_SCENARIO drivingScenario_arg 
    ):
    timestamp_us { std::move(timestamp_us_arg) } // NOLINT
      ,counter { std::move(counter_arg) } // NOLINT
      ,drivingScenario { std::move(drivingScenario_arg) } // NOLINT
      {
}

DrivingScenario::DrivingScenario(const DrivingScenario&) = default;

// NOLINTNEXTLINE
DrivingScenario::DrivingScenario(DrivingScenario&&) = default;

DrivingScenario& DrivingScenario::operator=(const DrivingScenario&) = default;

// NOLINTNEXTLINE
DrivingScenario& DrivingScenario::operator=(DrivingScenario&&) = default;

void DrivingScenario::toCsv(std::ostream& out) const
{
    out.precision(16);
    out << std::boolalpha;
    out << timestamp_us;out << ",";
    out << counter;out << ",";
    out << static_cast<uint32_t>(drivingScenario);
    
}

std::stringstream DrivingScenario::toStringstream() const
{
    std::stringstream out;
    toCsv(out);
    return out;
}

template<class Archive>
    void DrivingScenario::serialize(Archive& ar, [[maybe_unused]] unsigned int version)
    {

        ar& CEREAL_NVP(timestamp_us);
        ar& CEREAL_NVP(counter);
        ar& CEREAL_NVP(drivingScenario);
    }


template void DrivingScenario::serialize(cereal::BinaryInputArchive& ar, unsigned int version);
template void DrivingScenario::serialize(cereal::BinaryOutputArchive& ar, unsigned int version);
template void DrivingScenario::serialize(cereal::IntrospectionArchive& ar, unsigned int version);
template void DrivingScenario::serialize(cereal::JSONInputArchive& ar, unsigned int version);
template void DrivingScenario::serialize(cereal::JSONOutputArchive& ar, unsigned int version);
template void DrivingScenario::serialize(cereal::PortableBinaryInputArchive& ar, unsigned int version);
template void DrivingScenario::serialize(cereal::PortableBinaryOutputArchive& ar, unsigned int version);

std::ostream& operator<< (std::ostream& os, const DrivingScenario& drivingscenario)
{
    drivingscenario.toCsv(os);
    return os;
}

bool operator== (const DrivingScenario &lhs, const DrivingScenario &rhs)
{
    return (  lhs.timestamp_us== rhs.timestamp_us &&
     lhs.counter== rhs.counter &&
     lhs.drivingScenario== rhs.drivingScenario 
    );
}

std::size_t DrivingScenario::hash() const noexcept
{
    std::size_t result = 17;
    result = result * 31 + std::hash<int64_t>()(timestamp_us);
    result = result * 31 + std::hash<int64_t>()(counter);
    result = result * 31 + std::hash<DRIVING_SCENARIO>()(drivingScenario);
    return result;
}

 } // namespace av
 } // namespace msg
 } // namespace plan_ctrl


CEREAL_BIND_TO_ARCHIVES(av::msg::plan_ctrl::DrivingScenario)

