#include <interfaces/DspaceDiagnosticStatus/DspaceDiagnosticStatus.h>

#include <iterator>
#include <type_traits>

#include <cereal/cereal.hpp>
#include <cereal/archives/json.hpp>
#include <cereal/archives/binary.hpp>

#include <cereal/types/map.hpp>
#include <cereal/types/vector.hpp>
#include <cereal/types/array.hpp>
#include <cereal/types/string.hpp>
#include <cereal/types/complex.hpp>

#include <ODatum/IntrospectionArchive.h>
#include <ODatum/Hash.h>







DspaceDiagnosticStatus::DspaceDiagnosticStatus() = default;

DspaceDiagnosticStatus::DspaceDiagnosticStatus(
    Timestamp128 timeStamp_arg ,
    std::string dSpaceVersion_arg ,
    uint16_t dSpaceSystemState_arg ,
    double rxMotionCtrlRoadWheelAngleDeg_arg ,
    double rxMotionCtrlLongAccelAcctTilt_arg ,
    double latCtrlDesRoadWheelAngle_arg ,
    double inputACC_arg ,
    bool autoManualLockPeriod_arg ,
    bool rxAuxCtrlAutoManualButtonPressed_arg ,
    double steeringWheelAngle_arg ,
    double yawRate_arg ,
    bool throttleOverride_arg ,
    bool brakeOverride_arg ,
    bool steeringOverride_arg ,
    bool ErrorBrakesNotFollowAccelCmd_arg ,
    bool ErrorCCAN_CCAN3Mismatch_arg ,
    bool ErrorEBCMBrakeState_arg ,
    bool ErrorESCBrakeState_arg ,
    bool ErrorInAutoSteerInterfaceNotOpen_arg ,
    bool ErrorLimpHomeMode_arg 
    ):
    timeStamp { std::move(timeStamp_arg) } // NOLINT
      ,dSpaceVersion { std::move(dSpaceVersion_arg) } // NOLINT
      ,dSpaceSystemState { std::move(dSpaceSystemState_arg) } // NOLINT
      ,rxMotionCtrlRoadWheelAngleDeg { std::move(rxMotionCtrlRoadWheelAngleDeg_arg) } // NOLINT
      ,rxMotionCtrlLongAccelAcctTilt { std::move(rxMotionCtrlLongAccelAcctTilt_arg) } // NOLINT
      ,latCtrlDesRoadWheelAngle { std::move(latCtrlDesRoadWheelAngle_arg) } // NOLINT
      ,inputACC { std::move(inputACC_arg) } // NOLINT
      ,autoManualLockPeriod { std::move(autoManualLockPeriod_arg) } // NOLINT
      ,rxAuxCtrlAutoManualButtonPressed { std::move(rxAuxCtrlAutoManualButtonPressed_arg) } // NOLINT
      ,steeringWheelAngle { std::move(steeringWheelAngle_arg) } // NOLINT
      ,yawRate { std::move(yawRate_arg) } // NOLINT
      ,throttleOverride { std::move(throttleOverride_arg) } // NOLINT
      ,brakeOverride { std::move(brakeOverride_arg) } // NOLINT
      ,steeringOverride { std::move(steeringOverride_arg) } // NOLINT
      ,ErrorBrakesNotFollowAccelCmd { std::move(ErrorBrakesNotFollowAccelCmd_arg) } // NOLINT
      ,ErrorCCAN_CCAN3Mismatch { std::move(ErrorCCAN_CCAN3Mismatch_arg) } // NOLINT
      ,ErrorEBCMBrakeState { std::move(ErrorEBCMBrakeState_arg) } // NOLINT
      ,ErrorESCBrakeState { std::move(ErrorESCBrakeState_arg) } // NOLINT
      ,ErrorInAutoSteerInterfaceNotOpen { std::move(ErrorInAutoSteerInterfaceNotOpen_arg) } // NOLINT
      ,ErrorLimpHomeMode { std::move(ErrorLimpHomeMode_arg) } // NOLINT
      {
}

DspaceDiagnosticStatus::DspaceDiagnosticStatus(const DspaceDiagnosticStatus&) = default;

// NOLINTNEXTLINE
DspaceDiagnosticStatus::DspaceDiagnosticStatus(DspaceDiagnosticStatus&&) = default;

DspaceDiagnosticStatus& DspaceDiagnosticStatus::operator=(const DspaceDiagnosticStatus&) = default;

// NOLINTNEXTLINE
DspaceDiagnosticStatus& DspaceDiagnosticStatus::operator=(DspaceDiagnosticStatus&&) = default;

void DspaceDiagnosticStatus::toCsv(std::ostream& out) const
{
    out.precision(16);
    out << std::boolalpha;
    out << timeStamp;out << ",";
    out << "\"";
        out << dSpaceVersion;
        out << "\"";out << ",";
    out << dSpaceSystemState;out << ",";
    out << rxMotionCtrlRoadWheelAngleDeg;out << ",";
    out << rxMotionCtrlLongAccelAcctTilt;out << ",";
    out << latCtrlDesRoadWheelAngle;out << ",";
    out << inputACC;out << ",";
    out << autoManualLockPeriod;out << ",";
    out << rxAuxCtrlAutoManualButtonPressed;out << ",";
    out << steeringWheelAngle;out << ",";
    out << yawRate;out << ",";
    out << throttleOverride;out << ",";
    out << brakeOverride;out << ",";
    out << steeringOverride;out << ",";
    out << ErrorBrakesNotFollowAccelCmd;out << ",";
    out << ErrorCCAN_CCAN3Mismatch;out << ",";
    out << ErrorEBCMBrakeState;out << ",";
    out << ErrorESCBrakeState;out << ",";
    out << ErrorInAutoSteerInterfaceNotOpen;out << ",";
    out << ErrorLimpHomeMode;
    
}

std::stringstream DspaceDiagnosticStatus::toStringstream() const
{
    std::stringstream out;
    toCsv(out);
    return out;
}

template<class Archive>
    void DspaceDiagnosticStatus::serialize(Archive& ar, unsigned int version)
{
	if (version >= 1)
	{
		ar& CEREAL_NVP(timeStamp) & CEREAL_NVP(dSpaceVersion);
		ar& CEREAL_NVP(dSpaceSystemState) & CEREAL_NVP(rxMotionCtrlRoadWheelAngleDeg);
		ar& CEREAL_NVP(rxMotionCtrlLongAccelAcctTilt) & CEREAL_NVP(latCtrlDesRoadWheelAngle);
		ar& CEREAL_NVP(inputACC) & CEREAL_NVP(autoManualLockPeriod) & CEREAL_NVP(rxAuxCtrlAutoManualButtonPressed);
		ar& CEREAL_NVP(steeringWheelAngle) & CEREAL_NVP(yawRate);
		ar& CEREAL_NVP(throttleOverride) & CEREAL_NVP(brakeOverride) & CEREAL_NVP(steeringOverride);
	}

	if (version >= 2)
	{
		ar& CEREAL_NVP(ErrorBrakesNotFollowAccelCmd) & CEREAL_NVP(ErrorCCAN_CCAN3Mismatch);
		ar& CEREAL_NVP(ErrorEBCMBrakeState) & CEREAL_NVP(ErrorESCBrakeState);
		ar& CEREAL_NVP(ErrorInAutoSteerInterfaceNotOpen) & CEREAL_NVP(ErrorLimpHomeMode);
	}
}




template void DspaceDiagnosticStatus::serialize(cereal::BinaryInputArchive& ar, unsigned int version);
template void DspaceDiagnosticStatus::serialize(cereal::BinaryOutputArchive& ar, unsigned int version);
template void DspaceDiagnosticStatus::serialize(cereal::IntrospectionArchive& ar, unsigned int version);
template void DspaceDiagnosticStatus::serialize(cereal::JSONInputArchive& ar, unsigned int version);
template void DspaceDiagnosticStatus::serialize(cereal::JSONOutputArchive& ar, unsigned int version);
template void DspaceDiagnosticStatus::serialize(cereal::PortableBinaryInputArchive& ar, unsigned int version);
template void DspaceDiagnosticStatus::serialize(cereal::PortableBinaryOutputArchive& ar, unsigned int version);

std::ostream& operator<< (std::ostream& os, const DspaceDiagnosticStatus& dspacediagnosticstatus)
{
    dspacediagnosticstatus.toCsv(os);
    return os;
}

bool operator== (const DspaceDiagnosticStatus &lhs, const DspaceDiagnosticStatus &rhs)
{
    return (  lhs.timeStamp== rhs.timeStamp &&
     lhs.dSpaceVersion== rhs.dSpaceVersion &&
     lhs.dSpaceSystemState== rhs.dSpaceSystemState &&
     lhs.rxMotionCtrlRoadWheelAngleDeg== rhs.rxMotionCtrlRoadWheelAngleDeg &&
     lhs.rxMotionCtrlLongAccelAcctTilt== rhs.rxMotionCtrlLongAccelAcctTilt &&
     lhs.latCtrlDesRoadWheelAngle== rhs.latCtrlDesRoadWheelAngle &&
     lhs.inputACC== rhs.inputACC &&
     lhs.autoManualLockPeriod== rhs.autoManualLockPeriod &&
     lhs.rxAuxCtrlAutoManualButtonPressed== rhs.rxAuxCtrlAutoManualButtonPressed &&
     lhs.steeringWheelAngle== rhs.steeringWheelAngle &&
     lhs.yawRate== rhs.yawRate &&
     lhs.throttleOverride== rhs.throttleOverride &&
     lhs.brakeOverride== rhs.brakeOverride &&
     lhs.steeringOverride== rhs.steeringOverride &&
     lhs.ErrorBrakesNotFollowAccelCmd== rhs.ErrorBrakesNotFollowAccelCmd &&
     lhs.ErrorCCAN_CCAN3Mismatch== rhs.ErrorCCAN_CCAN3Mismatch &&
     lhs.ErrorEBCMBrakeState== rhs.ErrorEBCMBrakeState &&
     lhs.ErrorESCBrakeState== rhs.ErrorESCBrakeState &&
     lhs.ErrorInAutoSteerInterfaceNotOpen== rhs.ErrorInAutoSteerInterfaceNotOpen &&
     lhs.ErrorLimpHomeMode== rhs.ErrorLimpHomeMode 
    );
}

std::size_t DspaceDiagnosticStatus::hash() const noexcept
{
    std::size_t result = 17;
    result = result * 31 + std::hash<Timestamp128>()(timeStamp);
    result = result * 31 + std::hash<std::string>()(dSpaceVersion);
    result = result * 31 + std::hash<uint16_t>()(dSpaceSystemState);
    result = result * 31 + std::hash<double>()(rxMotionCtrlRoadWheelAngleDeg);
    result = result * 31 + std::hash<double>()(rxMotionCtrlLongAccelAcctTilt);
    result = result * 31 + std::hash<double>()(latCtrlDesRoadWheelAngle);
    result = result * 31 + std::hash<double>()(inputACC);
    result = result * 31 + std::hash<bool>()(autoManualLockPeriod);
    result = result * 31 + std::hash<bool>()(rxAuxCtrlAutoManualButtonPressed);
    result = result * 31 + std::hash<double>()(steeringWheelAngle);
    result = result * 31 + std::hash<double>()(yawRate);
    result = result * 31 + std::hash<bool>()(throttleOverride);
    result = result * 31 + std::hash<bool>()(brakeOverride);
    result = result * 31 + std::hash<bool>()(steeringOverride);
    result = result * 31 + std::hash<bool>()(ErrorBrakesNotFollowAccelCmd);
    result = result * 31 + std::hash<bool>()(ErrorCCAN_CCAN3Mismatch);
    result = result * 31 + std::hash<bool>()(ErrorEBCMBrakeState);
    result = result * 31 + std::hash<bool>()(ErrorESCBrakeState);
    result = result * 31 + std::hash<bool>()(ErrorInAutoSteerInterfaceNotOpen);
    result = result * 31 + std::hash<bool>()(ErrorLimpHomeMode);
    return result;
}



CEREAL_BIND_TO_ARCHIVES(DspaceDiagnosticStatus)

