#include <interfaces/Duration/Duration.h>

#include <iterator>
#include <type_traits>

#include <cereal/cereal.hpp>
#include <cereal/archives/json.hpp>
#include <cereal/archives/binary.hpp>

#include <cereal/types/map.hpp>
#include <cereal/types/vector.hpp>
#include <cereal/types/array.hpp>
#include <cereal/types/string.hpp>
#include <cereal/types/complex.hpp>

#include <ODatum/IntrospectionArchive.h>
#include <ODatum/Hash.h>





 namespace av {
 namespace msg {


Duration::Duration() = default;

Duration::Duration(
    int64_t privateData_arg 
    ):
    privateData { std::move(privateData_arg) } // NOLINT
      {
}

Duration::Duration(const Duration&) = default;

// NOLINTNEXTLINE
Duration::Duration(Duration&&) = default;

Duration& Duration::operator=(const Duration&) = default;

// NOLINTNEXTLINE
Duration& Duration::operator=(Duration&&) = default;

void Duration::toCsv(std::ostream& out) const
{
    out.precision(16);
    out << std::boolalpha;
    out << privateData;
    
}

std::stringstream Duration::toStringstream() const
{
    std::stringstream out;
    toCsv(out);
    return out;
}

template<class Archive>
    void Duration::serialize(Archive& ar, [[maybe_unused]] unsigned int version)
{
    ar& CEREAL_NVP(privateData);
}

// Custom logic to allow implicit conversion between av::Duration and av::msg::Duration

// Constructor from av::Duration to av::msg::Duration
Duration::Duration(const av::Duration& duration)
    : privateData{duration.count()}
{
}

// Assignment operator from av::Duration to av::msg::Duration
Duration& Duration::operator=(const av::Duration& duration)
{
    privateData = duration.count();
    return *this;
}

// Implicit conversion function from av::msg::Duration to av::Duration
Duration::operator av::Duration() const
{
    return av::Duration(privateData);
}



template void Duration::serialize(cereal::BinaryInputArchive& ar, unsigned int version);
template void Duration::serialize(cereal::BinaryOutputArchive& ar, unsigned int version);
template void Duration::serialize(cereal::IntrospectionArchive& ar, unsigned int version);
template void Duration::serialize(cereal::JSONInputArchive& ar, unsigned int version);
template void Duration::serialize(cereal::JSONOutputArchive& ar, unsigned int version);
template void Duration::serialize(cereal::PortableBinaryInputArchive& ar, unsigned int version);
template void Duration::serialize(cereal::PortableBinaryOutputArchive& ar, unsigned int version);

std::ostream& operator<< (std::ostream& os, const Duration& duration)
{
    duration.toCsv(os);
    return os;
}

bool operator== (const Duration &lhs, const Duration &rhs)
{
    return (  lhs.privateData== rhs.privateData 
    );
}

std::size_t Duration::hash() const noexcept
{
    std::size_t result = 17;
    result = result * 31 + std::hash<int64_t>()(privateData);
    return result;
}

 } // namespace av
 } // namespace msg


CEREAL_BIND_TO_ARCHIVES(av::msg::Duration)

