#include <interfaces/EStopStatus/EStopStatus.h>

#include <iterator>
#include <type_traits>

#include <cereal/cereal.hpp>
#include <cereal/archives/json.hpp>
#include <cereal/archives/binary.hpp>

#include <cereal/types/map.hpp>
#include <cereal/types/vector.hpp>
#include <cereal/types/array.hpp>
#include <cereal/types/string.hpp>
#include <cereal/types/complex.hpp>

#include <ODatum/IntrospectionArchive.h>
#include <ODatum/Hash.h>





 namespace av {
 namespace msg {


EStopStatus::EStopStatus() = default;

EStopStatus::EStopStatus(
    bool manualOverride_arg ,
    bool enabled_arg ,
    bool run_arg ,
    bool prerun_arg ,
    bool brakeOverride_arg ,
    bool steerOverride_arg ,
    bool throttleOverride_arg 
    ):
    manualOverride { std::move(manualOverride_arg) } // NOLINT
      ,enabled { std::move(enabled_arg) } // NOLINT
      ,run { std::move(run_arg) } // NOLINT
      ,prerun { std::move(prerun_arg) } // NOLINT
      ,brakeOverride { std::move(brakeOverride_arg) } // NOLINT
      ,steerOverride { std::move(steerOverride_arg) } // NOLINT
      ,throttleOverride { std::move(throttleOverride_arg) } // NOLINT
      {
}

EStopStatus::EStopStatus(const EStopStatus&) = default;

// NOLINTNEXTLINE
EStopStatus::EStopStatus(EStopStatus&&) = default;

EStopStatus& EStopStatus::operator=(const EStopStatus&) = default;

// NOLINTNEXTLINE
EStopStatus& EStopStatus::operator=(EStopStatus&&) = default;

void EStopStatus::toCsv(std::ostream& out) const
{
    out.precision(16);
    out << std::boolalpha;
    out << manualOverride;out << ",";
    out << enabled;out << ",";
    out << run;out << ",";
    out << prerun;out << ",";
    out << brakeOverride;out << ",";
    out << steerOverride;out << ",";
    out << throttleOverride;
    
}

std::stringstream EStopStatus::toStringstream() const
{
    std::stringstream out;
    toCsv(out);
    return out;
}

template<class Archive>
    void EStopStatus::serialize(Archive& ar, unsigned int version)
{
    if(version < 3) { // timeStamp removed in version 3
        int64_t tv_sec, tv_nsec;
        ar& cereal::make_nvp("time_.tv_sec", tv_sec) & cereal::make_nvp("time_.tv_nsec", tv_nsec);
    }
    ar& CEREAL_NVP(manualOverride);
    ar& CEREAL_NVP(enabled);
    ar& CEREAL_NVP(run);
    if (version >= 1) {
        ar& CEREAL_NVP(prerun);
    }
    if (version >= 2) {
        ar& CEREAL_NVP(brakeOverride);
        ar& CEREAL_NVP(steerOverride);
        ar& CEREAL_NVP(throttleOverride);
    }
}



template void EStopStatus::serialize(cereal::BinaryInputArchive& ar, unsigned int version);
template void EStopStatus::serialize(cereal::BinaryOutputArchive& ar, unsigned int version);
template void EStopStatus::serialize(cereal::IntrospectionArchive& ar, unsigned int version);
template void EStopStatus::serialize(cereal::JSONInputArchive& ar, unsigned int version);
template void EStopStatus::serialize(cereal::JSONOutputArchive& ar, unsigned int version);
template void EStopStatus::serialize(cereal::PortableBinaryInputArchive& ar, unsigned int version);
template void EStopStatus::serialize(cereal::PortableBinaryOutputArchive& ar, unsigned int version);

std::ostream& operator<< (std::ostream& os, const EStopStatus& estopstatus)
{
    estopstatus.toCsv(os);
    return os;
}

bool operator== (const EStopStatus &lhs, const EStopStatus &rhs)
{
    return (  lhs.manualOverride== rhs.manualOverride &&
     lhs.enabled== rhs.enabled &&
     lhs.run== rhs.run &&
     lhs.prerun== rhs.prerun &&
     lhs.brakeOverride== rhs.brakeOverride &&
     lhs.steerOverride== rhs.steerOverride &&
     lhs.throttleOverride== rhs.throttleOverride 
    );
}

std::size_t EStopStatus::hash() const noexcept
{
    std::size_t result = 17;
    result = result * 31 + std::hash<bool>()(manualOverride);
    result = result * 31 + std::hash<bool>()(enabled);
    result = result * 31 + std::hash<bool>()(run);
    result = result * 31 + std::hash<bool>()(prerun);
    result = result * 31 + std::hash<bool>()(brakeOverride);
    result = result * 31 + std::hash<bool>()(steerOverride);
    result = result * 31 + std::hash<bool>()(throttleOverride);
    return result;
}

 } // namespace av
 } // namespace msg


CEREAL_BIND_TO_ARCHIVES(av::msg::EStopStatus)

