#include <interfaces/Event/Event.h>

#include <iterator>
#include <type_traits>

#include <cereal/cereal.hpp>
#include <cereal/archives/json.hpp>
#include <cereal/archives/binary.hpp>

#include <cereal/types/map.hpp>
#include <cereal/types/vector.hpp>
#include <cereal/types/array.hpp>
#include <cereal/types/string.hpp>
#include <cereal/types/complex.hpp>

#include <ODatum/IntrospectionArchive.h>
#include <ODatum/Hash.h>





 namespace timingframework {


Event::Event() = default;

Event::Event(
    std::string id_arg ,
    std::string name_arg ,
    double timestamp_arg 
    ):
    id { std::move(id_arg) } // NOLINT
      ,name { std::move(name_arg) } // NOLINT
      ,timestamp { std::move(timestamp_arg) } // NOLINT
      {
}

Event::Event(const Event&) = default;

// NOLINTNEXTLINE
Event::Event(Event&&) = default;

Event& Event::operator=(const Event&) = default;

// NOLINTNEXTLINE
Event& Event::operator=(Event&&) = default;

void Event::toCsv(std::ostream& out) const
{
    out.precision(16);
    out << std::boolalpha;
    out << "\"";
        out << id;
        out << "\"";out << ",";
    out << "\"";
        out << name;
        out << "\"";out << ",";
    out << timestamp;
    
}

std::stringstream Event::toStringstream() const
{
    std::stringstream out;
    toCsv(out);
    return out;
}

template<class Archive>
    void Event::serialize(Archive& ar, [[maybe_unused]] unsigned int version)
    {

        ar& CEREAL_NVP(id);
        ar& CEREAL_NVP(name);
        ar& CEREAL_NVP(timestamp);
    }


template void Event::serialize(cereal::BinaryInputArchive& ar, unsigned int version);
template void Event::serialize(cereal::BinaryOutputArchive& ar, unsigned int version);
template void Event::serialize(cereal::IntrospectionArchive& ar, unsigned int version);
template void Event::serialize(cereal::JSONInputArchive& ar, unsigned int version);
template void Event::serialize(cereal::JSONOutputArchive& ar, unsigned int version);
template void Event::serialize(cereal::PortableBinaryInputArchive& ar, unsigned int version);
template void Event::serialize(cereal::PortableBinaryOutputArchive& ar, unsigned int version);

std::ostream& operator<< (std::ostream& os, const Event& event)
{
    event.toCsv(os);
    return os;
}

bool operator== (const Event &lhs, const Event &rhs)
{
    return (  lhs.id== rhs.id &&
     lhs.name== rhs.name &&
     lhs.timestamp== rhs.timestamp 
    );
}

std::size_t Event::hash() const noexcept
{
    std::size_t result = 17;
    result = result * 31 + std::hash<std::string>()(id);
    result = result * 31 + std::hash<std::string>()(name);
    result = result * 31 + std::hash<double>()(timestamp);
    return result;
}

 } // namespace timingframework


CEREAL_BIND_TO_ARCHIVES(timingframework::Event)

