#include <interfaces/EventAssociation/EventAssociation.h>

#include <iterator>
#include <type_traits>

#include <cereal/cereal.hpp>
#include <cereal/archives/json.hpp>
#include <cereal/archives/binary.hpp>

#include <cereal/types/map.hpp>
#include <cereal/types/vector.hpp>
#include <cereal/types/array.hpp>
#include <cereal/types/string.hpp>
#include <cereal/types/complex.hpp>

#include <ODatum/IntrospectionArchive.h>
#include <ODatum/Hash.h>





 namespace timingframework {


EventAssociation::EventAssociation() = default;

EventAssociation::EventAssociation(
    std::string eventid1_arg ,
    std::string eventid2_arg 
    ):
    eventid1 { std::move(eventid1_arg) } // NOLINT
      ,eventid2 { std::move(eventid2_arg) } // NOLINT
      {
}

EventAssociation::EventAssociation(const EventAssociation&) = default;

// NOLINTNEXTLINE
EventAssociation::EventAssociation(EventAssociation&&) = default;

EventAssociation& EventAssociation::operator=(const EventAssociation&) = default;

// NOLINTNEXTLINE
EventAssociation& EventAssociation::operator=(EventAssociation&&) = default;

void EventAssociation::toCsv(std::ostream& out) const
{
    out.precision(16);
    out << std::boolalpha;
    out << "\"";
        out << eventid1;
        out << "\"";out << ",";
    out << "\"";
        out << eventid2;
        out << "\"";
    
}

std::stringstream EventAssociation::toStringstream() const
{
    std::stringstream out;
    toCsv(out);
    return out;
}

template<class Archive>
    void EventAssociation::serialize(Archive& ar, [[maybe_unused]] unsigned int version)
    {

        ar& CEREAL_NVP(eventid1);
        ar& CEREAL_NVP(eventid2);
    }


template void EventAssociation::serialize(cereal::BinaryInputArchive& ar, unsigned int version);
template void EventAssociation::serialize(cereal::BinaryOutputArchive& ar, unsigned int version);
template void EventAssociation::serialize(cereal::IntrospectionArchive& ar, unsigned int version);
template void EventAssociation::serialize(cereal::JSONInputArchive& ar, unsigned int version);
template void EventAssociation::serialize(cereal::JSONOutputArchive& ar, unsigned int version);
template void EventAssociation::serialize(cereal::PortableBinaryInputArchive& ar, unsigned int version);
template void EventAssociation::serialize(cereal::PortableBinaryOutputArchive& ar, unsigned int version);

std::ostream& operator<< (std::ostream& os, const EventAssociation& eventassociation)
{
    eventassociation.toCsv(os);
    return os;
}

bool operator== (const EventAssociation &lhs, const EventAssociation &rhs)
{
    return (  lhs.eventid1== rhs.eventid1 &&
     lhs.eventid2== rhs.eventid2 
    );
}

std::size_t EventAssociation::hash() const noexcept
{
    std::size_t result = 17;
    result = result * 31 + std::hash<std::string>()(eventid1);
    result = result * 31 + std::hash<std::string>()(eventid2);
    return result;
}

 } // namespace timingframework


CEREAL_BIND_TO_ARCHIVES(timingframework::EventAssociation)

