#include <interfaces/EventGenerator/EventGenerator.h>

#include <iterator>
#include <type_traits>

#include <cereal/cereal.hpp>
#include <cereal/archives/json.hpp>
#include <cereal/archives/binary.hpp>

#include <cereal/types/map.hpp>
#include <cereal/types/vector.hpp>
#include <cereal/types/array.hpp>
#include <cereal/types/string.hpp>
#include <cereal/types/complex.hpp>

#include <ODatum/IntrospectionArchive.h>
#include <ODatum/Hash.h>





 namespace av {
 namespace msg {


EventGenerator::EventGenerator() = default;

EventGenerator::EventGenerator(
    TimePoint timeStamp_arg ,
    bool generateEvent_arg ,
    std::string sender_arg ,
    std::string featureHashTag_arg ,
    std::string comments_arg ,
    std::string drivername_arg ,
    std::string passengername_arg ,
    std::string testType_arg ,
    std::string roadType_arg ,
    bool resendEvents_arg ,
    std::string scenarioName_arg ,
    uint64_t eventCount_arg ,
    std::string ticketId_arg ,
    std::string weather_arg ,
    std::string trackAndLayout_arg ,
    std::string route_arg ,
    std::string region_arg ,
    std::string testSubType_arg 
    ):
    timeStamp { std::move(timeStamp_arg) } // NOLINT
      ,generateEvent { std::move(generateEvent_arg) } // NOLINT
      ,sender { std::move(sender_arg) } // NOLINT
      ,featureHashTag { std::move(featureHashTag_arg) } // NOLINT
      ,comments { std::move(comments_arg) } // NOLINT
      ,drivername { std::move(drivername_arg) } // NOLINT
      ,passengername { std::move(passengername_arg) } // NOLINT
      ,testType { std::move(testType_arg) } // NOLINT
      ,roadType { std::move(roadType_arg) } // NOLINT
      ,resendEvents { std::move(resendEvents_arg) } // NOLINT
      ,scenarioName { std::move(scenarioName_arg) } // NOLINT
      ,eventCount { std::move(eventCount_arg) } // NOLINT
      ,ticketId { std::move(ticketId_arg) } // NOLINT
      ,weather { std::move(weather_arg) } // NOLINT
      ,trackAndLayout { std::move(trackAndLayout_arg) } // NOLINT
      ,route { std::move(route_arg) } // NOLINT
      ,region { std::move(region_arg) } // NOLINT
      ,testSubType { std::move(testSubType_arg) } // NOLINT
      {
}

EventGenerator::EventGenerator(const EventGenerator&) = default;

// NOLINTNEXTLINE
EventGenerator::EventGenerator(EventGenerator&&) = default;

EventGenerator& EventGenerator::operator=(const EventGenerator&) = default;

// NOLINTNEXTLINE
EventGenerator& EventGenerator::operator=(EventGenerator&&) = default;

void EventGenerator::toCsv(std::ostream& out) const
{
    out.precision(16);
    out << std::boolalpha;
    out << timeStamp;out << ",";
    out << generateEvent;out << ",";
    out << "\"";
        out << sender;
        out << "\"";out << ",";
    out << "\"";
        out << featureHashTag;
        out << "\"";out << ",";
    out << "\"";
        out << comments;
        out << "\"";out << ",";
    out << "\"";
        out << drivername;
        out << "\"";out << ",";
    out << "\"";
        out << passengername;
        out << "\"";out << ",";
    out << "\"";
        out << testType;
        out << "\"";out << ",";
    out << "\"";
        out << roadType;
        out << "\"";out << ",";
    out << resendEvents;out << ",";
    out << "\"";
        out << scenarioName;
        out << "\"";out << ",";
    out << eventCount;out << ",";
    out << "\"";
        out << ticketId;
        out << "\"";out << ",";
    out << "\"";
        out << weather;
        out << "\"";out << ",";
    out << "\"";
        out << trackAndLayout;
        out << "\"";out << ",";
    out << "\"";
        out << route;
        out << "\"";out << ",";
    out << "\"";
        out << region;
        out << "\"";out << ",";
    out << "\"";
        out << testSubType;
        out << "\"";
    
}

std::stringstream EventGenerator::toStringstream() const
{
    std::stringstream out;
    toCsv(out);
    return out;
}

template<class Archive>
    void EventGenerator::serialize(Archive& ar, unsigned int version)
{
    if (version < 10)
    {
        otimePoint otp;
        ar& cereal::make_nvp("timeStamp", otp);
        timeStamp = av::TimePoint(std::chrono::seconds(otp.tv_sec) + std::chrono::nanoseconds(otp.tv_nsec));
    }
    else{
        ar& CEREAL_NVP(timeStamp);
    }
    ar& CEREAL_NVP(generateEvent);
    ar& CEREAL_NVP(sender);
    ar& CEREAL_NVP(featureHashTag);
    ar& CEREAL_NVP(comments);
    if (version > 1)
    {
        ar& CEREAL_NVP(drivername) & CEREAL_NVP(passengername) & CEREAL_NVP(resendEvents);
    }
    if (version > 2)
    {
        ar& CEREAL_NVP(scenarioName);
    }
    if (version > 3)
    {
        ar& CEREAL_NVP(eventCount);
    }
    if (version > 4)
    {
        ar& CEREAL_NVP(testType) & CEREAL_NVP(roadType);
    }
    if (version > 5)
    {
        ar& CEREAL_NVP(ticketId);
    }
    if (version > 6)
    {
        ar& CEREAL_NVP(weather) & CEREAL_NVP(trackAndLayout);
    }
    if (version > 7)
    {
        ar& CEREAL_NVP(route);
    }
    if (version > 8)
    {
        ar& CEREAL_NVP(region);
    }
    if (version > 10)
    {
        ar& CEREAL_NVP(testSubType);
    }
}

}} // to close av::msg namespace temporary

struct EventGenerator : public av::msg::EventGenerator{};
CEREAL_CLASS_VERSION(EventGenerator, 11)
CEREAL_REGISTER_TYPE(EventGenerator)

namespace av {
namespace msg {



template void EventGenerator::serialize(cereal::BinaryInputArchive& ar, unsigned int version);
template void EventGenerator::serialize(cereal::BinaryOutputArchive& ar, unsigned int version);
template void EventGenerator::serialize(cereal::IntrospectionArchive& ar, unsigned int version);
template void EventGenerator::serialize(cereal::JSONInputArchive& ar, unsigned int version);
template void EventGenerator::serialize(cereal::JSONOutputArchive& ar, unsigned int version);
template void EventGenerator::serialize(cereal::PortableBinaryInputArchive& ar, unsigned int version);
template void EventGenerator::serialize(cereal::PortableBinaryOutputArchive& ar, unsigned int version);

std::ostream& operator<< (std::ostream& os, const EventGenerator& eventgenerator)
{
    eventgenerator.toCsv(os);
    return os;
}

bool operator== (const EventGenerator &lhs, const EventGenerator &rhs)
{
    return (  lhs.timeStamp== rhs.timeStamp &&
     lhs.generateEvent== rhs.generateEvent &&
     lhs.sender== rhs.sender &&
     lhs.featureHashTag== rhs.featureHashTag &&
     lhs.comments== rhs.comments &&
     lhs.drivername== rhs.drivername &&
     lhs.passengername== rhs.passengername &&
     lhs.testType== rhs.testType &&
     lhs.roadType== rhs.roadType &&
     lhs.resendEvents== rhs.resendEvents &&
     lhs.scenarioName== rhs.scenarioName &&
     lhs.eventCount== rhs.eventCount &&
     lhs.ticketId== rhs.ticketId &&
     lhs.weather== rhs.weather &&
     lhs.trackAndLayout== rhs.trackAndLayout &&
     lhs.route== rhs.route &&
     lhs.region== rhs.region &&
     lhs.testSubType== rhs.testSubType 
    );
}

std::size_t EventGenerator::hash() const noexcept
{
    std::size_t result = 17;
    result = result * 31 + 0;
    result = result * 31 + std::hash<bool>()(generateEvent);
    result = result * 31 + std::hash<std::string>()(sender);
    result = result * 31 + std::hash<std::string>()(featureHashTag);
    result = result * 31 + std::hash<std::string>()(comments);
    result = result * 31 + std::hash<std::string>()(drivername);
    result = result * 31 + std::hash<std::string>()(passengername);
    result = result * 31 + std::hash<std::string>()(testType);
    result = result * 31 + std::hash<std::string>()(roadType);
    result = result * 31 + std::hash<bool>()(resendEvents);
    result = result * 31 + std::hash<std::string>()(scenarioName);
    result = result * 31 + std::hash<uint64_t>()(eventCount);
    result = result * 31 + std::hash<std::string>()(ticketId);
    result = result * 31 + std::hash<std::string>()(weather);
    result = result * 31 + std::hash<std::string>()(trackAndLayout);
    result = result * 31 + std::hash<std::string>()(route);
    result = result * 31 + std::hash<std::string>()(region);
    result = result * 31 + std::hash<std::string>()(testSubType);
    return result;
}

 } // namespace av
 } // namespace msg


CEREAL_BIND_TO_ARCHIVES(av::msg::EventGenerator)

