#include <interfaces/EventLoggerMessageList/EventLoggerMessageList.h>

#include <iterator>
#include <type_traits>

#include <cereal/cereal.hpp>
#include <cereal/archives/json.hpp>
#include <cereal/archives/binary.hpp>

#include <cereal/types/map.hpp>
#include <cereal/types/vector.hpp>
#include <cereal/types/array.hpp>
#include <cereal/types/string.hpp>
#include <cereal/types/complex.hpp>

#include <ODatum/IntrospectionArchive.h>
#include <ODatum/Hash.h>


 namespace av {
 namespace msg {


EventLoggerMessage::EventLoggerMessage() = default;

EventLoggerMessage::EventLoggerMessage(
    std::string timeStamp_arg ,
    TimePoint otimeStamp_arg ,
    EventType type_arg ,
    av::geometry::RecPose3D location_arg ,
    int32_t utmPlane_arg ,
    double vehSpeed_arg ,
    std::string timeSinceStart_arg ,
    std::string feature_arg ,
    std::string passOrFail_arg ,
    std::string passOrFailMode_arg ,
    std::string comments_arg ,
    std::string vehSpeedUnit_arg ,
    std::string locationUnit_arg ,
    bool resend_arg ,
    TimePoint startTime_arg ,
    std::string drivername_arg ,
    std::string passengername_arg ,
    uint32_t eventCount_arg ,
    EventCause eventCause_arg ,
    std::string resultingAutoMode_arg 
    ):
    timeStamp { std::move(timeStamp_arg) } // NOLINT
      ,
    otimeStamp { std::move(otimeStamp_arg) } // NOLINT
      ,
    type { std::move(type_arg) } // NOLINT
      ,
    location { std::move(location_arg) } // NOLINT
      ,
    utmPlane { std::move(utmPlane_arg) } // NOLINT
      ,
    vehSpeed { std::move(vehSpeed_arg) } // NOLINT
      ,
    timeSinceStart { std::move(timeSinceStart_arg) } // NOLINT
      ,
    feature { std::move(feature_arg) } // NOLINT
      ,
    passOrFail { std::move(passOrFail_arg) } // NOLINT
      ,
    passOrFailMode { std::move(passOrFailMode_arg) } // NOLINT
      ,
    comments { std::move(comments_arg) } // NOLINT
      ,
    vehSpeedUnit { std::move(vehSpeedUnit_arg) } // NOLINT
      ,
    locationUnit { std::move(locationUnit_arg) } // NOLINT
      ,
    resend { std::move(resend_arg) } // NOLINT
      ,
    startTime { std::move(startTime_arg) } // NOLINT
      ,
    drivername { std::move(drivername_arg) } // NOLINT
      ,
    passengername { std::move(passengername_arg) } // NOLINT
      ,
    eventCount { std::move(eventCount_arg) } // NOLINT
      ,
    eventCause { std::move(eventCause_arg) } // NOLINT
      ,
    resultingAutoMode { std::move(resultingAutoMode_arg) } // NOLINT
      
    {
}

EventLoggerMessage::EventLoggerMessage(const EventLoggerMessage&) = default;

// NOLINTNEXTLINE
EventLoggerMessage::EventLoggerMessage(EventLoggerMessage&&) = default;

EventLoggerMessage& EventLoggerMessage::operator=(const EventLoggerMessage&) = default;

// NOLINTNEXTLINE
EventLoggerMessage& EventLoggerMessage::operator=(EventLoggerMessage&&) = default;

void EventLoggerMessage::toCsv(std::ostream& out) const
{
    out.precision(16);
    out << std::boolalpha;
    out << "\"";
        out << timeStamp;
        out << "\"";out << ",";
    out << otimeStamp;out << ",";
    out << static_cast<uint32_t>(type);out << ",";
    out << location;out << ",";
    out << utmPlane;out << ",";
    out << vehSpeed;out << ",";
    out << "\"";
        out << timeSinceStart;
        out << "\"";out << ",";
    out << "\"";
        out << feature;
        out << "\"";out << ",";
    out << "\"";
        out << passOrFail;
        out << "\"";out << ",";
    out << "\"";
        out << passOrFailMode;
        out << "\"";out << ",";
    out << "\"";
        out << comments;
        out << "\"";out << ",";
    out << "\"";
        out << vehSpeedUnit;
        out << "\"";out << ",";
    out << "\"";
        out << locationUnit;
        out << "\"";out << ",";
    out << resend;out << ",";
    out << startTime;out << ",";
    out << "\"";
        out << drivername;
        out << "\"";out << ",";
    out << "\"";
        out << passengername;
        out << "\"";out << ",";
    out << eventCount;out << ",";
    out << static_cast<uint32_t>(eventCause);out << ",";
    out << "\"";
        out << resultingAutoMode;
        out << "\"";
    
}

std::stringstream EventLoggerMessage::toStringstream() const
{
    std::stringstream out;
    toCsv(out);
    return out;
}

template<class Archive>
    void EventLoggerMessage::serialize(Archive& ar, unsigned int version)
{
    if (version >= 1) {
        ar& CEREAL_NVP(timeStamp);
        ar& CEREAL_NVP(type);
        ar& CEREAL_NVP(location);
        ar& CEREAL_NVP(utmPlane);
        ar& CEREAL_NVP(vehSpeed);
        ar& CEREAL_NVP(timeSinceStart);
    }
    if (version >= 2) {
        if( version < 5 ) {
            otimePoint otp;
            ar& cereal::make_nvp("otimeStamp", otp);
            otimeStamp = av::TimePoint(std::chrono::seconds(otp.tv_sec) + std::chrono::nanoseconds(otp.tv_nsec));
        } else {
            ar& CEREAL_NVP(otimeStamp);
        }
        ar& CEREAL_NVP(feature);
        ar& CEREAL_NVP(passOrFail);
        ar& CEREAL_NVP(passOrFailMode);
        ar& CEREAL_NVP(comments);
        ar& CEREAL_NVP(vehSpeedUnit);
        ar& CEREAL_NVP(locationUnit);
        ar& CEREAL_NVP(resend);
        if( version < 5 ) {
            otimePoint otp;
            ar& cereal::make_nvp("startTime", otp);
            startTime = av::TimePoint(std::chrono::seconds(otp.tv_sec) + std::chrono::nanoseconds(otp.tv_nsec));
        } else {
            ar& CEREAL_NVP(startTime);
        }
        ar& CEREAL_NVP(drivername);
        ar& CEREAL_NVP(passengername);
    }
    if (version >= 3) {
        ar& CEREAL_NVP(eventCount);
    }
    if (version >= 4) {
        ar& CEREAL_NVP(eventCause);
        ar& CEREAL_NVP(resultingAutoMode);
    }
}

}} // to close av::msg namespace temporary

struct EventLoggerMessage : public av::msg::EventLoggerMessage {};
CEREAL_CLASS_VERSION(EventLoggerMessage, 4)
CEREAL_REGISTER_TYPE(EventLoggerMessage)

namespace av {
namespace msg {



template void EventLoggerMessage::serialize(cereal::BinaryInputArchive& ar, unsigned int version);
template void EventLoggerMessage::serialize(cereal::BinaryOutputArchive& ar, unsigned int version);
template void EventLoggerMessage::serialize(cereal::IntrospectionArchive& ar, unsigned int version);
template void EventLoggerMessage::serialize(cereal::JSONInputArchive& ar, unsigned int version);
template void EventLoggerMessage::serialize(cereal::JSONOutputArchive& ar, unsigned int version);
template void EventLoggerMessage::serialize(cereal::PortableBinaryInputArchive& ar, unsigned int version);
template void EventLoggerMessage::serialize(cereal::PortableBinaryOutputArchive& ar, unsigned int version);

std::ostream& operator<< (std::ostream& os, const EventLoggerMessage& eventloggermessage)
{
    eventloggermessage.toCsv(os);
    return os;
}

bool operator== (const EventLoggerMessage &lhs, const EventLoggerMessage &rhs)
{
    return (  lhs.timeStamp== rhs.timeStamp &&
     lhs.otimeStamp== rhs.otimeStamp &&
     lhs.type== rhs.type &&
     lhs.location== rhs.location &&
     lhs.utmPlane== rhs.utmPlane &&
     lhs.vehSpeed== rhs.vehSpeed &&
     lhs.timeSinceStart== rhs.timeSinceStart &&
     lhs.feature== rhs.feature &&
     lhs.passOrFail== rhs.passOrFail &&
     lhs.passOrFailMode== rhs.passOrFailMode &&
     lhs.comments== rhs.comments &&
     lhs.vehSpeedUnit== rhs.vehSpeedUnit &&
     lhs.locationUnit== rhs.locationUnit &&
     lhs.resend== rhs.resend &&
     lhs.startTime== rhs.startTime &&
     lhs.drivername== rhs.drivername &&
     lhs.passengername== rhs.passengername &&
     lhs.eventCount== rhs.eventCount &&
     lhs.eventCause== rhs.eventCause &&
     lhs.resultingAutoMode== rhs.resultingAutoMode 
    );
}

std::size_t EventLoggerMessage::hash() const noexcept
{
    std::size_t result = 17;
    result = result * 31 + std::hash<std::string>()(timeStamp);
    result = result * 31 + 0;
    result = result * 31 + std::hash<EventType>()(type);
    result = result * 31 + std::hash<av::geometry::RecPose3D>()(location);
    result = result * 31 + std::hash<int32_t>()(utmPlane);
    result = result * 31 + std::hash<double>()(vehSpeed);
    result = result * 31 + std::hash<std::string>()(timeSinceStart);
    result = result * 31 + std::hash<std::string>()(feature);
    result = result * 31 + std::hash<std::string>()(passOrFail);
    result = result * 31 + std::hash<std::string>()(passOrFailMode);
    result = result * 31 + std::hash<std::string>()(comments);
    result = result * 31 + std::hash<std::string>()(vehSpeedUnit);
    result = result * 31 + std::hash<std::string>()(locationUnit);
    result = result * 31 + std::hash<bool>()(resend);
    result = result * 31 + 0;
    result = result * 31 + std::hash<std::string>()(drivername);
    result = result * 31 + std::hash<std::string>()(passengername);
    result = result * 31 + std::hash<uint32_t>()(eventCount);
    result = result * 31 + std::hash<EventCause>()(eventCause);
    result = result * 31 + std::hash<std::string>()(resultingAutoMode);
    return result;
}

 } // namespace av
 } // namespace msg


CEREAL_BIND_TO_ARCHIVES(av::msg::EventLoggerMessage)




 namespace av {
 namespace msg {


EventLoggerMessageList::EventLoggerMessageList() = default;

EventLoggerMessageList::EventLoggerMessageList(
    std::vector<EventLoggerMessage> messageList_arg 
    ):
    messageList { std::move(messageList_arg) } // NOLINT
      {
}

EventLoggerMessageList::EventLoggerMessageList(const EventLoggerMessageList&) = default;

// NOLINTNEXTLINE
EventLoggerMessageList::EventLoggerMessageList(EventLoggerMessageList&&) = default;

EventLoggerMessageList& EventLoggerMessageList::operator=(const EventLoggerMessageList&) = default;

// NOLINTNEXTLINE
EventLoggerMessageList& EventLoggerMessageList::operator=(EventLoggerMessageList&&) = default;

void EventLoggerMessageList::toCsv(std::ostream& out) const
{
    out.precision(16);
    out << std::boolalpha;
    // Enclose sequence in double quotations and curly brackets to keep CSV line length uniform
        
        out << "\"{{";
        
        if (!messageList.empty())
        {
            std::copy(messageList.begin(), messageList.end() - 1, std::ostream_iterator<EventLoggerMessage>(out, "},{"));
            std::copy(messageList.end()-1, messageList.end(), std::ostream_iterator<EventLoggerMessage>(out, ""));
        }
        out << "}}\"";
    
}

std::stringstream EventLoggerMessageList::toStringstream() const
{
    std::stringstream out;
    toCsv(out);
    return out;
}

template<class Archive>
    void EventLoggerMessageList::serialize(Archive& ar, unsigned int version)
{
    ar& CEREAL_NVP(messageList);
}

}} // to close av::msg namespace temporary

struct EventLoggerMessageList : public av::msg::EventLoggerMessageList {};
CEREAL_CLASS_VERSION(EventLoggerMessageList, 1)
CEREAL_REGISTER_TYPE(EventLoggerMessageList)

namespace av {
namespace msg {



template void EventLoggerMessageList::serialize(cereal::BinaryInputArchive& ar, unsigned int version);
template void EventLoggerMessageList::serialize(cereal::BinaryOutputArchive& ar, unsigned int version);
template void EventLoggerMessageList::serialize(cereal::IntrospectionArchive& ar, unsigned int version);
template void EventLoggerMessageList::serialize(cereal::JSONInputArchive& ar, unsigned int version);
template void EventLoggerMessageList::serialize(cereal::JSONOutputArchive& ar, unsigned int version);
template void EventLoggerMessageList::serialize(cereal::PortableBinaryInputArchive& ar, unsigned int version);
template void EventLoggerMessageList::serialize(cereal::PortableBinaryOutputArchive& ar, unsigned int version);

std::ostream& operator<< (std::ostream& os, const EventLoggerMessageList& eventloggermessagelist)
{
    eventloggermessagelist.toCsv(os);
    return os;
}

bool operator== (const EventLoggerMessageList &lhs, const EventLoggerMessageList &rhs)
{
    return (  lhs.messageList== rhs.messageList 
    );
}

std::size_t EventLoggerMessageList::hash() const noexcept
{
    std::size_t result = 17;
    result = result * 31 + hashVector(messageList);
    return result;
}

 } // namespace av
 } // namespace msg


CEREAL_BIND_TO_ARCHIVES(av::msg::EventLoggerMessageList)

