#include <interfaces/FaultInjectionMessage/FaultInjectionMessage.h>

#include <iterator>
#include <type_traits>

#include <cereal/cereal.hpp>
#include <cereal/archives/json.hpp>
#include <cereal/archives/binary.hpp>

#include <cereal/types/map.hpp>
#include <cereal/types/vector.hpp>
#include <cereal/types/array.hpp>
#include <cereal/types/string.hpp>
#include <cereal/types/complex.hpp>

#include <ODatum/IntrospectionArchive.h>
#include <ODatum/Hash.h>





 namespace av {
 namespace msg {


FaultInjectionMessage::FaultInjectionMessage() = default;

FaultInjectionMessage::FaultInjectionMessage(
    double tireAngleRate_radps_arg ,
    double tireAngleOffset_rad_arg ,
    double crossTrackErrorRate_mps_arg ,
    double crossTrackErrorOffset_m_arg ,
    double accel_mps2_arg ,
    double speedOffset_mps_arg ,
    double tireAngleOscAmplitude_rad_arg ,
    double tireAngleOscFrequency_hz_arg ,
    bool stopFault_arg ,
    bool skipPublish_arg 
    ):
    tireAngleRate_radps { std::move(tireAngleRate_radps_arg) } // NOLINT
      ,tireAngleOffset_rad { std::move(tireAngleOffset_rad_arg) } // NOLINT
      ,crossTrackErrorRate_mps { std::move(crossTrackErrorRate_mps_arg) } // NOLINT
      ,crossTrackErrorOffset_m { std::move(crossTrackErrorOffset_m_arg) } // NOLINT
      ,accel_mps2 { std::move(accel_mps2_arg) } // NOLINT
      ,speedOffset_mps { std::move(speedOffset_mps_arg) } // NOLINT
      ,tireAngleOscAmplitude_rad { std::move(tireAngleOscAmplitude_rad_arg) } // NOLINT
      ,tireAngleOscFrequency_hz { std::move(tireAngleOscFrequency_hz_arg) } // NOLINT
      ,stopFault { std::move(stopFault_arg) } // NOLINT
      ,skipPublish { std::move(skipPublish_arg) } // NOLINT
      {
}

FaultInjectionMessage::FaultInjectionMessage(const FaultInjectionMessage&) = default;

// NOLINTNEXTLINE
FaultInjectionMessage::FaultInjectionMessage(FaultInjectionMessage&&) = default;

FaultInjectionMessage& FaultInjectionMessage::operator=(const FaultInjectionMessage&) = default;

// NOLINTNEXTLINE
FaultInjectionMessage& FaultInjectionMessage::operator=(FaultInjectionMessage&&) = default;

void FaultInjectionMessage::toCsv(std::ostream& out) const
{
    out.precision(16);
    out << std::boolalpha;
    out << tireAngleRate_radps;out << ",";
    out << tireAngleOffset_rad;out << ",";
    out << crossTrackErrorRate_mps;out << ",";
    out << crossTrackErrorOffset_m;out << ",";
    out << accel_mps2;out << ",";
    out << speedOffset_mps;out << ",";
    out << tireAngleOscAmplitude_rad;out << ",";
    out << tireAngleOscFrequency_hz;out << ",";
    out << stopFault;out << ",";
    out << skipPublish;
    
}

std::stringstream FaultInjectionMessage::toStringstream() const
{
    std::stringstream out;
    toCsv(out);
    return out;
}

template<class Archive>
    void FaultInjectionMessage::serialize(Archive& ar, unsigned int version)
{
    if (version >= 1) {
        ar& CEREAL_NVP(tireAngleRate_radps);
        ar& CEREAL_NVP(tireAngleOffset_rad);
        ar& CEREAL_NVP(accel_mps2);
        ar& CEREAL_NVP(speedOffset_mps);
        ar& CEREAL_NVP(stopFault);
    }
    if (version >= 2) {
        ar& CEREAL_NVP(crossTrackErrorRate_mps);
        ar& CEREAL_NVP(crossTrackErrorOffset_m);
    }
    if (version >= 3) {
        ar& CEREAL_NVP(skipPublish);
    }
    if (version >= 4) {
        ar& CEREAL_NVP(tireAngleOscAmplitude_rad);
        ar& CEREAL_NVP(tireAngleOscFrequency_hz);
    }
}



template void FaultInjectionMessage::serialize(cereal::BinaryInputArchive& ar, unsigned int version);
template void FaultInjectionMessage::serialize(cereal::BinaryOutputArchive& ar, unsigned int version);
template void FaultInjectionMessage::serialize(cereal::IntrospectionArchive& ar, unsigned int version);
template void FaultInjectionMessage::serialize(cereal::JSONInputArchive& ar, unsigned int version);
template void FaultInjectionMessage::serialize(cereal::JSONOutputArchive& ar, unsigned int version);
template void FaultInjectionMessage::serialize(cereal::PortableBinaryInputArchive& ar, unsigned int version);
template void FaultInjectionMessage::serialize(cereal::PortableBinaryOutputArchive& ar, unsigned int version);

std::ostream& operator<< (std::ostream& os, const FaultInjectionMessage& faultinjectionmessage)
{
    faultinjectionmessage.toCsv(os);
    return os;
}

bool operator== (const FaultInjectionMessage &lhs, const FaultInjectionMessage &rhs)
{
    return (  lhs.tireAngleRate_radps== rhs.tireAngleRate_radps &&
     lhs.tireAngleOffset_rad== rhs.tireAngleOffset_rad &&
     lhs.crossTrackErrorRate_mps== rhs.crossTrackErrorRate_mps &&
     lhs.crossTrackErrorOffset_m== rhs.crossTrackErrorOffset_m &&
     lhs.accel_mps2== rhs.accel_mps2 &&
     lhs.speedOffset_mps== rhs.speedOffset_mps &&
     lhs.tireAngleOscAmplitude_rad== rhs.tireAngleOscAmplitude_rad &&
     lhs.tireAngleOscFrequency_hz== rhs.tireAngleOscFrequency_hz &&
     lhs.stopFault== rhs.stopFault &&
     lhs.skipPublish== rhs.skipPublish 
    );
}

std::size_t FaultInjectionMessage::hash() const noexcept
{
    std::size_t result = 17;
    result = result * 31 + std::hash<double>()(tireAngleRate_radps);
    result = result * 31 + std::hash<double>()(tireAngleOffset_rad);
    result = result * 31 + std::hash<double>()(crossTrackErrorRate_mps);
    result = result * 31 + std::hash<double>()(crossTrackErrorOffset_m);
    result = result * 31 + std::hash<double>()(accel_mps2);
    result = result * 31 + std::hash<double>()(speedOffset_mps);
    result = result * 31 + std::hash<double>()(tireAngleOscAmplitude_rad);
    result = result * 31 + std::hash<double>()(tireAngleOscFrequency_hz);
    result = result * 31 + std::hash<bool>()(stopFault);
    result = result * 31 + std::hash<bool>()(skipPublish);
    return result;
}

 } // namespace av
 } // namespace msg


CEREAL_BIND_TO_ARCHIVES(av::msg::FaultInjectionMessage)

