#include <interfaces/FlrA1RadarData/FlrA1RadarData.h>

#include <iterator>
#include <type_traits>

#include <cereal/cereal.hpp>
#include <cereal/archives/json.hpp>
#include <cereal/archives/binary.hpp>

#include <cereal/types/map.hpp>
#include <cereal/types/vector.hpp>
#include <cereal/types/array.hpp>
#include <cereal/types/string.hpp>
#include <cereal/types/complex.hpp>

#include <ODatum/IntrospectionArchive.h>
#include <ODatum/Hash.h>


 namespace av {
 namespace msg {


RadarDetection::RadarDetection() = default;

RadarDetection::RadarDetection(
    uint32_t canId_arg ,
    double range_arg ,
    double rangeError_arg ,
    double rangeVelocity_arg ,
    double rangeVelocityError_arg ,
    double unfoldedRangeVelocity_arg ,
    double snr_arg ,
    double crossSection_arg ,
    double elevation_arg ,
    double elevationError_arg ,
    CONFIDENCE_LEVEL elevationConfidence_arg ,
    double azimuth_arg ,
    double azimuthError_arg ,
    CONFIDENCE_LEVEL azimuthConfidence_arg ,
    double azimuthAmbiguityProbability_arg ,
    uint8_t azimuthAmbiguityId_arg ,
    bool vehicleClutter_arg ,
    bool superResTarget_arg ,
    double pointTargetProbability_arg ,
    double existenceProbability_arg ,
    bool outsideSector_arg ,
    std::vector<double> meta_arg ,
    av::geometry::Vector3d positionWrtSensor_arg ,
    av::geometry::Vector3d positionWrtVehicle_arg ,
    av::geometry::Vector3d positionWrtGlobal_arg 
    ):
    canId { std::move(canId_arg) } // NOLINT
      ,
    range { std::move(range_arg) } // NOLINT
      ,
    rangeError { std::move(rangeError_arg) } // NOLINT
      ,
    rangeVelocity { std::move(rangeVelocity_arg) } // NOLINT
      ,
    rangeVelocityError { std::move(rangeVelocityError_arg) } // NOLINT
      ,
    unfoldedRangeVelocity { std::move(unfoldedRangeVelocity_arg) } // NOLINT
      ,
    snr { std::move(snr_arg) } // NOLINT
      ,
    crossSection { std::move(crossSection_arg) } // NOLINT
      ,
    elevation { std::move(elevation_arg) } // NOLINT
      ,
    elevationError { std::move(elevationError_arg) } // NOLINT
      ,
    elevationConfidence { std::move(elevationConfidence_arg) } // NOLINT
      ,
    azimuth { std::move(azimuth_arg) } // NOLINT
      ,
    azimuthError { std::move(azimuthError_arg) } // NOLINT
      ,
    azimuthConfidence { std::move(azimuthConfidence_arg) } // NOLINT
      ,
    azimuthAmbiguityProbability { std::move(azimuthAmbiguityProbability_arg) } // NOLINT
      ,
    azimuthAmbiguityId { std::move(azimuthAmbiguityId_arg) } // NOLINT
      ,
    vehicleClutter { std::move(vehicleClutter_arg) } // NOLINT
      ,
    superResTarget { std::move(superResTarget_arg) } // NOLINT
      ,
    pointTargetProbability { std::move(pointTargetProbability_arg) } // NOLINT
      ,
    existenceProbability { std::move(existenceProbability_arg) } // NOLINT
      ,
    outsideSector { std::move(outsideSector_arg) } // NOLINT
      ,
    meta { std::move(meta_arg) } // NOLINT
      ,
    positionWrtSensor { std::move(positionWrtSensor_arg) } // NOLINT
      ,
    positionWrtVehicle { std::move(positionWrtVehicle_arg) } // NOLINT
      ,
    positionWrtGlobal { std::move(positionWrtGlobal_arg) } // NOLINT
      
    {
}

RadarDetection::RadarDetection(const RadarDetection&) = default;

// NOLINTNEXTLINE
RadarDetection::RadarDetection(RadarDetection&&) = default;

RadarDetection& RadarDetection::operator=(const RadarDetection&) = default;

// NOLINTNEXTLINE
RadarDetection& RadarDetection::operator=(RadarDetection&&) = default;

void RadarDetection::toCsv(std::ostream& out) const
{
    out.precision(16);
    out << std::boolalpha;
    out << canId;out << ",";
    out << range;out << ",";
    out << rangeError;out << ",";
    out << rangeVelocity;out << ",";
    out << rangeVelocityError;out << ",";
    out << unfoldedRangeVelocity;out << ",";
    out << snr;out << ",";
    out << crossSection;out << ",";
    out << elevation;out << ",";
    out << elevationError;out << ",";
    out << static_cast<uint32_t>(elevationConfidence);out << ",";
    out << azimuth;out << ",";
    out << azimuthError;out << ",";
    out << static_cast<uint32_t>(azimuthConfidence);out << ",";
    out << azimuthAmbiguityProbability;out << ",";
    // Cast to uint16_t to print as integer instead of char
        out << static_cast<uint16_t>(azimuthAmbiguityId);out << ",";
    out << vehicleClutter;out << ",";
    out << superResTarget;out << ",";
    out << pointTargetProbability;out << ",";
    out << existenceProbability;out << ",";
    out << outsideSector;out << ",";
    // Enclose sequence in double quotations and curly brackets to keep CSV line length uniform
        
        out << "\"{{";
        
        if (!meta.empty())
        {
            std::copy(meta.begin(), meta.end() - 1, std::ostream_iterator<double>(out, "},{"));
            std::copy(meta.end()-1, meta.end(), std::ostream_iterator<double>(out, ""));
        }
        out << "}}\"";out << ",";
    out << positionWrtSensor;out << ",";
    out << positionWrtVehicle;out << ",";
    out << positionWrtGlobal;
    
}

std::stringstream RadarDetection::toStringstream() const
{
    std::stringstream out;
    toCsv(out);
    return out;
}

template<class Archive>
    void RadarDetection::serialize(Archive& ar, [[maybe_unused]] unsigned int version)
    {

        ar& CEREAL_NVP(canId);
        ar& CEREAL_NVP(range);
        ar& CEREAL_NVP(rangeError);
        ar& CEREAL_NVP(rangeVelocity);
        ar& CEREAL_NVP(rangeVelocityError);
        ar& CEREAL_NVP(unfoldedRangeVelocity);
        ar& CEREAL_NVP(snr);
        ar& CEREAL_NVP(crossSection);
        ar& CEREAL_NVP(elevation);
        ar& CEREAL_NVP(elevationError);
        ar& CEREAL_NVP(elevationConfidence);
        ar& CEREAL_NVP(azimuth);
        ar& CEREAL_NVP(azimuthError);
        ar& CEREAL_NVP(azimuthConfidence);
        ar& CEREAL_NVP(azimuthAmbiguityProbability);
        ar& CEREAL_NVP(azimuthAmbiguityId);
        ar& CEREAL_NVP(vehicleClutter);
        ar& CEREAL_NVP(superResTarget);
        ar& CEREAL_NVP(pointTargetProbability);
        ar& CEREAL_NVP(existenceProbability);
        ar& CEREAL_NVP(outsideSector);
        ar& CEREAL_NVP(meta);
        ar& CEREAL_NVP(positionWrtSensor);
        ar& CEREAL_NVP(positionWrtVehicle);
        ar& CEREAL_NVP(positionWrtGlobal);
    }


template void RadarDetection::serialize(cereal::BinaryInputArchive& ar, unsigned int version);
template void RadarDetection::serialize(cereal::BinaryOutputArchive& ar, unsigned int version);
template void RadarDetection::serialize(cereal::IntrospectionArchive& ar, unsigned int version);
template void RadarDetection::serialize(cereal::JSONInputArchive& ar, unsigned int version);
template void RadarDetection::serialize(cereal::JSONOutputArchive& ar, unsigned int version);
template void RadarDetection::serialize(cereal::PortableBinaryInputArchive& ar, unsigned int version);
template void RadarDetection::serialize(cereal::PortableBinaryOutputArchive& ar, unsigned int version);

std::ostream& operator<< (std::ostream& os, const RadarDetection& radardetection)
{
    radardetection.toCsv(os);
    return os;
}

bool operator== (const RadarDetection &lhs, const RadarDetection &rhs)
{
    return (  lhs.canId== rhs.canId &&
     lhs.range== rhs.range &&
     lhs.rangeError== rhs.rangeError &&
     lhs.rangeVelocity== rhs.rangeVelocity &&
     lhs.rangeVelocityError== rhs.rangeVelocityError &&
     lhs.unfoldedRangeVelocity== rhs.unfoldedRangeVelocity &&
     lhs.snr== rhs.snr &&
     lhs.crossSection== rhs.crossSection &&
     lhs.elevation== rhs.elevation &&
     lhs.elevationError== rhs.elevationError &&
     lhs.elevationConfidence== rhs.elevationConfidence &&
     lhs.azimuth== rhs.azimuth &&
     lhs.azimuthError== rhs.azimuthError &&
     lhs.azimuthConfidence== rhs.azimuthConfidence &&
     lhs.azimuthAmbiguityProbability== rhs.azimuthAmbiguityProbability &&
     lhs.azimuthAmbiguityId== rhs.azimuthAmbiguityId &&
     lhs.vehicleClutter== rhs.vehicleClutter &&
     lhs.superResTarget== rhs.superResTarget &&
     lhs.pointTargetProbability== rhs.pointTargetProbability &&
     lhs.existenceProbability== rhs.existenceProbability &&
     lhs.outsideSector== rhs.outsideSector &&
     lhs.meta== rhs.meta &&
     lhs.positionWrtSensor== rhs.positionWrtSensor &&
     lhs.positionWrtVehicle== rhs.positionWrtVehicle &&
     lhs.positionWrtGlobal== rhs.positionWrtGlobal 
    );
}

std::size_t RadarDetection::hash() const noexcept
{
    std::size_t result = 17;
    result = result * 31 + std::hash<uint32_t>()(canId);
    result = result * 31 + std::hash<double>()(range);
    result = result * 31 + std::hash<double>()(rangeError);
    result = result * 31 + std::hash<double>()(rangeVelocity);
    result = result * 31 + std::hash<double>()(rangeVelocityError);
    result = result * 31 + std::hash<double>()(unfoldedRangeVelocity);
    result = result * 31 + std::hash<double>()(snr);
    result = result * 31 + std::hash<double>()(crossSection);
    result = result * 31 + std::hash<double>()(elevation);
    result = result * 31 + std::hash<double>()(elevationError);
    result = result * 31 + std::hash<CONFIDENCE_LEVEL>()(elevationConfidence);
    result = result * 31 + std::hash<double>()(azimuth);
    result = result * 31 + std::hash<double>()(azimuthError);
    result = result * 31 + std::hash<CONFIDENCE_LEVEL>()(azimuthConfidence);
    result = result * 31 + std::hash<double>()(azimuthAmbiguityProbability);
    result = result * 31 + std::hash<uint8_t>()(azimuthAmbiguityId);
    result = result * 31 + std::hash<bool>()(vehicleClutter);
    result = result * 31 + std::hash<bool>()(superResTarget);
    result = result * 31 + std::hash<double>()(pointTargetProbability);
    result = result * 31 + std::hash<double>()(existenceProbability);
    result = result * 31 + std::hash<bool>()(outsideSector);
    result = result * 31 + hashVector(meta);
    result = result * 31 + std::hash<av::geometry::Vector3d>()(positionWrtSensor);
    result = result * 31 + std::hash<av::geometry::Vector3d>()(positionWrtVehicle);
    result = result * 31 + std::hash<av::geometry::Vector3d>()(positionWrtGlobal);
    return result;
}

 } // namespace av
 } // namespace msg


CEREAL_BIND_TO_ARCHIVES(av::msg::RadarDetection)




 namespace av {
 namespace msg {


FlrA1RadarData::FlrA1RadarData() = default;

FlrA1RadarData::FlrA1RadarData(
    std::string radarName_arg ,
    av::msg::TimePoint sensorTimeStamp_arg ,
    av::msg::TimePoint messageTimeStamp_arg ,
    uint16_t scanIndex_arg ,
    double dopplerCoverage_arg ,
    uint16_t rangeCoverage_arg ,
    double fovHorizontal_arg ,
    double fovVertical_arg ,
    std::vector<RadarDetection> detections_arg ,
    std::vector<double> metaBurst_arg 
    ):
    radarName { std::move(radarName_arg) } // NOLINT
      ,sensorTimeStamp { std::move(sensorTimeStamp_arg) } // NOLINT
      ,messageTimeStamp { std::move(messageTimeStamp_arg) } // NOLINT
      ,scanIndex { std::move(scanIndex_arg) } // NOLINT
      ,dopplerCoverage { std::move(dopplerCoverage_arg) } // NOLINT
      ,rangeCoverage { std::move(rangeCoverage_arg) } // NOLINT
      ,fovHorizontal { std::move(fovHorizontal_arg) } // NOLINT
      ,fovVertical { std::move(fovVertical_arg) } // NOLINT
      ,detections { std::move(detections_arg) } // NOLINT
      ,metaBurst { std::move(metaBurst_arg) } // NOLINT
      {
}

FlrA1RadarData::FlrA1RadarData(const FlrA1RadarData&) = default;

// NOLINTNEXTLINE
FlrA1RadarData::FlrA1RadarData(FlrA1RadarData&&) = default;

FlrA1RadarData& FlrA1RadarData::operator=(const FlrA1RadarData&) = default;

// NOLINTNEXTLINE
FlrA1RadarData& FlrA1RadarData::operator=(FlrA1RadarData&&) = default;

void FlrA1RadarData::toCsv(std::ostream& out) const
{
    out.precision(16);
    out << std::boolalpha;
    out << "\"";
        out << radarName;
        out << "\"";out << ",";
    out << sensorTimeStamp;out << ",";
    out << messageTimeStamp;out << ",";
    out << scanIndex;out << ",";
    out << dopplerCoverage;out << ",";
    out << rangeCoverage;out << ",";
    out << fovHorizontal;out << ",";
    out << fovVertical;out << ",";
    // Enclose sequence in double quotations and curly brackets to keep CSV line length uniform
        
        out << "\"{{";
        
        if (!detections.empty())
        {
            std::copy(detections.begin(), detections.end() - 1, std::ostream_iterator<RadarDetection>(out, "},{"));
            std::copy(detections.end()-1, detections.end(), std::ostream_iterator<RadarDetection>(out, ""));
        }
        out << "}}\"";out << ",";
    // Enclose sequence in double quotations and curly brackets to keep CSV line length uniform
        
        out << "\"{{";
        
        if (!metaBurst.empty())
        {
            std::copy(metaBurst.begin(), metaBurst.end() - 1, std::ostream_iterator<double>(out, "},{"));
            std::copy(metaBurst.end()-1, metaBurst.end(), std::ostream_iterator<double>(out, ""));
        }
        out << "}}\"";
    
}

std::stringstream FlrA1RadarData::toStringstream() const
{
    std::stringstream out;
    toCsv(out);
    return out;
}

template<class Archive>
    void FlrA1RadarData::serialize(Archive& ar, unsigned int version)
{
    if (version >= 1) {
        ar& CEREAL_NVP(radarName);
        ar& CEREAL_NVP(sensorTimeStamp);
        ar& CEREAL_NVP(messageTimeStamp);
        ar& CEREAL_NVP(scanIndex);
        ar& CEREAL_NVP(dopplerCoverage);
        ar& CEREAL_NVP(rangeCoverage);
        ar& CEREAL_NVP(fovHorizontal);
        ar& CEREAL_NVP(fovVertical);
        ar& CEREAL_NVP(detections);
    }
    if (version >= 2) {
        ar& CEREAL_NVP(metaBurst);
    }
}



template void FlrA1RadarData::serialize(cereal::BinaryInputArchive& ar, unsigned int version);
template void FlrA1RadarData::serialize(cereal::BinaryOutputArchive& ar, unsigned int version);
template void FlrA1RadarData::serialize(cereal::IntrospectionArchive& ar, unsigned int version);
template void FlrA1RadarData::serialize(cereal::JSONInputArchive& ar, unsigned int version);
template void FlrA1RadarData::serialize(cereal::JSONOutputArchive& ar, unsigned int version);
template void FlrA1RadarData::serialize(cereal::PortableBinaryInputArchive& ar, unsigned int version);
template void FlrA1RadarData::serialize(cereal::PortableBinaryOutputArchive& ar, unsigned int version);

std::ostream& operator<< (std::ostream& os, const FlrA1RadarData& flra1radardata)
{
    flra1radardata.toCsv(os);
    return os;
}

bool operator== (const FlrA1RadarData &lhs, const FlrA1RadarData &rhs)
{
    return (  lhs.radarName== rhs.radarName &&
     lhs.sensorTimeStamp== rhs.sensorTimeStamp &&
     lhs.messageTimeStamp== rhs.messageTimeStamp &&
     lhs.scanIndex== rhs.scanIndex &&
     lhs.dopplerCoverage== rhs.dopplerCoverage &&
     lhs.rangeCoverage== rhs.rangeCoverage &&
     lhs.fovHorizontal== rhs.fovHorizontal &&
     lhs.fovVertical== rhs.fovVertical &&
     lhs.detections== rhs.detections &&
     lhs.metaBurst== rhs.metaBurst 
    );
}

std::size_t FlrA1RadarData::hash() const noexcept
{
    std::size_t result = 17;
    result = result * 31 + std::hash<std::string>()(radarName);
    result = result * 31 + std::hash<av::msg::TimePoint>()(sensorTimeStamp);
    result = result * 31 + std::hash<av::msg::TimePoint>()(messageTimeStamp);
    result = result * 31 + std::hash<uint16_t>()(scanIndex);
    result = result * 31 + std::hash<double>()(dopplerCoverage);
    result = result * 31 + std::hash<uint16_t>()(rangeCoverage);
    result = result * 31 + std::hash<double>()(fovHorizontal);
    result = result * 31 + std::hash<double>()(fovVertical);
    result = result * 31 + hashVector(detections);
    result = result * 31 + hashVector(metaBurst);
    return result;
}

 } // namespace av
 } // namespace msg


CEREAL_BIND_TO_ARCHIVES(av::msg::FlrA1RadarData)

