#include <interfaces/FusedOdometry/FusedOdometry.h>

#include <iterator>
#include <type_traits>

#include <cereal/cereal.hpp>
#include <cereal/archives/json.hpp>
#include <cereal/archives/binary.hpp>

#include <cereal/types/map.hpp>
#include <cereal/types/vector.hpp>
#include <cereal/types/array.hpp>
#include <cereal/types/string.hpp>
#include <cereal/types/complex.hpp>

#include <ODatum/IntrospectionArchive.h>
#include <ODatum/Hash.h>





 namespace av {
 namespace msg {


FusedOdometry::FusedOdometry() = default;

FusedOdometry::FusedOdometry(
    TimePoint timeStamp_arg ,
    double yawRateRad_arg ,
    double speed_mps_arg 
    ):
    timeStamp { std::move(timeStamp_arg) } // NOLINT
      ,yawRateRad { std::move(yawRateRad_arg) } // NOLINT
      ,speed_mps { std::move(speed_mps_arg) } // NOLINT
      {
}

FusedOdometry::FusedOdometry(const FusedOdometry&) = default;

// NOLINTNEXTLINE
FusedOdometry::FusedOdometry(FusedOdometry&&) = default;

FusedOdometry& FusedOdometry::operator=(const FusedOdometry&) = default;

// NOLINTNEXTLINE
FusedOdometry& FusedOdometry::operator=(FusedOdometry&&) = default;

void FusedOdometry::toCsv(std::ostream& out) const
{
    out.precision(16);
    out << std::boolalpha;
    out << timeStamp;out << ",";
    out << yawRateRad;out << ",";
    out << speed_mps;
    
}

std::stringstream FusedOdometry::toStringstream() const
{
    std::stringstream out;
    toCsv(out);
    return out;
}

template<class Archive>
    
void FusedOdometry::serialize(Archive& ar, unsigned int version)
{
    if (version < 2)
    {
        otimePoint otp;
        ar& cereal::make_nvp("timeStamp", otp);
        timeStamp = av::TimePoint(std::chrono::seconds(otp.tv_sec) + std::chrono::nanoseconds(otp.tv_nsec));
    }
    else
    {
        ar& CEREAL_NVP(timeStamp);
    }
    
    ar& CEREAL_NVP(yawRateRad);
    ar& CEREAL_NVP(speed_mps);
}

}} // to close av::msg namespace temporary

struct FusedOdometryMsg : public av::msg::FusedOdometry {};
CEREAL_CLASS_VERSION(FusedOdometryMsg, 2)
CEREAL_REGISTER_TYPE(FusedOdometryMsg)

namespace av {
namespace msg {


template void FusedOdometry::serialize(cereal::BinaryInputArchive& ar, unsigned int version);
template void FusedOdometry::serialize(cereal::BinaryOutputArchive& ar, unsigned int version);
template void FusedOdometry::serialize(cereal::IntrospectionArchive& ar, unsigned int version);
template void FusedOdometry::serialize(cereal::JSONInputArchive& ar, unsigned int version);
template void FusedOdometry::serialize(cereal::JSONOutputArchive& ar, unsigned int version);
template void FusedOdometry::serialize(cereal::PortableBinaryInputArchive& ar, unsigned int version);
template void FusedOdometry::serialize(cereal::PortableBinaryOutputArchive& ar, unsigned int version);

std::ostream& operator<< (std::ostream& os, const FusedOdometry& fusedodometry)
{
    fusedodometry.toCsv(os);
    return os;
}

bool operator== (const FusedOdometry &lhs, const FusedOdometry &rhs)
{
    return (  lhs.timeStamp== rhs.timeStamp &&
     lhs.yawRateRad== rhs.yawRateRad &&
     lhs.speed_mps== rhs.speed_mps 
    );
}

std::size_t FusedOdometry::hash() const noexcept
{
    std::size_t result = 17;
    result = result * 31 + 0;
    result = result * 31 + std::hash<double>()(yawRateRad);
    result = result * 31 + std::hash<double>()(speed_mps);
    return result;
}

 } // namespace av
 } // namespace msg


CEREAL_BIND_TO_ARCHIVES(av::msg::FusedOdometry)

