#include <interfaces/GUID/GUID.h>

#include <iterator>
#include <type_traits>

#include <cereal/cereal.hpp>
#include <cereal/archives/json.hpp>
#include <cereal/archives/binary.hpp>

#include <cereal/types/map.hpp>
#include <cereal/types/vector.hpp>
#include <cereal/types/array.hpp>
#include <cereal/types/string.hpp>
#include <cereal/types/complex.hpp>

#include <ODatum/IntrospectionArchive.h>
#include <ODatum/Hash.h>


 namespace av {
 namespace msg {


GUID::GUID() = default;

GUID::GUID(
    std::string id_arg 
    ):
    id { std::move(id_arg) } // NOLINT
      
    {
}

GUID::GUID(const GUID&) = default;

// NOLINTNEXTLINE
GUID::GUID(GUID&&) = default;

GUID& GUID::operator=(const GUID&) = default;

// NOLINTNEXTLINE
GUID& GUID::operator=(GUID&&) = default;

void GUID::toCsv(std::ostream& out) const
{
    out.precision(16);
    out << std::boolalpha;
    out << "\"";
        out << id;
        out << "\"";
    
}

std::stringstream GUID::toStringstream() const
{
    std::stringstream out;
    toCsv(out);
    return out;
}

template<class Archive>
    void GUID::serialize(Archive& ar, [[maybe_unused]] unsigned int version)
    {

        ar& CEREAL_NVP(id);
    }


template void GUID::serialize(cereal::BinaryInputArchive& ar, unsigned int version);
template void GUID::serialize(cereal::BinaryOutputArchive& ar, unsigned int version);
template void GUID::serialize(cereal::IntrospectionArchive& ar, unsigned int version);
template void GUID::serialize(cereal::JSONInputArchive& ar, unsigned int version);
template void GUID::serialize(cereal::JSONOutputArchive& ar, unsigned int version);
template void GUID::serialize(cereal::PortableBinaryInputArchive& ar, unsigned int version);
template void GUID::serialize(cereal::PortableBinaryOutputArchive& ar, unsigned int version);

std::ostream& operator<< (std::ostream& os, const GUID& guid)
{
    guid.toCsv(os);
    return os;
}

bool operator== (const GUID &lhs, const GUID &rhs)
{
    return (  lhs.id== rhs.id 
    );
}

std::size_t GUID::hash() const noexcept
{
    std::size_t result = 17;
    result = result * 31 + std::hash<std::string>()(id);
    return result;
}

 } // namespace av
 } // namespace msg


CEREAL_BIND_TO_ARCHIVES(av::msg::GUID)


