#include <interfaces/Grid/Grid.h>

#include <iterator>
#include <type_traits>

#include <cereal/cereal.hpp>
#include <cereal/archives/json.hpp>
#include <cereal/archives/binary.hpp>

#include <cereal/types/map.hpp>
#include <cereal/types/vector.hpp>
#include <cereal/types/array.hpp>
#include <cereal/types/string.hpp>
#include <cereal/types/complex.hpp>

#include <ODatum/IntrospectionArchive.h>
#include <ODatum/Hash.h>


 namespace av {
 namespace msg {
 namespace plan_ctrl {


GridTransform::GridTransform() = default;

GridTransform::GridTransform(
    double x_m_arg ,
    double y_m_arg ,
    double heading_rad_arg 
    ):
    x_m { std::move(x_m_arg) } // NOLINT
      ,
    y_m { std::move(y_m_arg) } // NOLINT
      ,
    heading_rad { std::move(heading_rad_arg) } // NOLINT
      
    {
}

GridTransform::GridTransform(const GridTransform&) = default;

// NOLINTNEXTLINE
GridTransform::GridTransform(GridTransform&&) = default;

GridTransform& GridTransform::operator=(const GridTransform&) = default;

// NOLINTNEXTLINE
GridTransform& GridTransform::operator=(GridTransform&&) = default;

void GridTransform::toCsv(std::ostream& out) const
{
    out.precision(16);
    out << std::boolalpha;
    out << x_m;out << ",";
    out << y_m;out << ",";
    out << heading_rad;
    
}

std::stringstream GridTransform::toStringstream() const
{
    std::stringstream out;
    toCsv(out);
    return out;
}

template<class Archive>
    void GridTransform::serialize(Archive& ar, [[maybe_unused]] unsigned int version)
    {

        ar& CEREAL_NVP(x_m);
        ar& CEREAL_NVP(y_m);
        ar& CEREAL_NVP(heading_rad);
    }


template void GridTransform::serialize(cereal::BinaryInputArchive& ar, unsigned int version);
template void GridTransform::serialize(cereal::BinaryOutputArchive& ar, unsigned int version);
template void GridTransform::serialize(cereal::IntrospectionArchive& ar, unsigned int version);
template void GridTransform::serialize(cereal::JSONInputArchive& ar, unsigned int version);
template void GridTransform::serialize(cereal::JSONOutputArchive& ar, unsigned int version);
template void GridTransform::serialize(cereal::PortableBinaryInputArchive& ar, unsigned int version);
template void GridTransform::serialize(cereal::PortableBinaryOutputArchive& ar, unsigned int version);

std::ostream& operator<< (std::ostream& os, const GridTransform& gridtransform)
{
    gridtransform.toCsv(os);
    return os;
}

bool operator== (const GridTransform &lhs, const GridTransform &rhs)
{
    return (  lhs.x_m== rhs.x_m &&
     lhs.y_m== rhs.y_m &&
     lhs.heading_rad== rhs.heading_rad 
    );
}

std::size_t GridTransform::hash() const noexcept
{
    std::size_t result = 17;
    result = result * 31 + std::hash<double>()(x_m);
    result = result * 31 + std::hash<double>()(y_m);
    result = result * 31 + std::hash<double>()(heading_rad);
    return result;
}

 } // namespace av
 } // namespace msg
 } // namespace plan_ctrl


CEREAL_BIND_TO_ARCHIVES(av::msg::plan_ctrl::GridTransform)




 namespace av {
 namespace msg {
 namespace plan_ctrl {


Grid::Grid() = default;

Grid::Grid(
    std::vector<float> data_arg ,
    uint32_t height_arg ,
    uint32_t width_arg ,
    GridTransform transform_arg ,
    int64_t timestamp_us_arg ,
    int64_t counter_arg 
    ):
    data { std::move(data_arg) } // NOLINT
      ,height { std::move(height_arg) } // NOLINT
      ,width { std::move(width_arg) } // NOLINT
      ,transform { std::move(transform_arg) } // NOLINT
      ,timestamp_us { std::move(timestamp_us_arg) } // NOLINT
      ,counter { std::move(counter_arg) } // NOLINT
      {
}

Grid::Grid(const Grid&) = default;

// NOLINTNEXTLINE
Grid::Grid(Grid&&) = default;

Grid& Grid::operator=(const Grid&) = default;

// NOLINTNEXTLINE
Grid& Grid::operator=(Grid&&) = default;

void Grid::toCsv(std::ostream& out) const
{
    out.precision(16);
    out << std::boolalpha;
    // Enclose sequence in double quotations and curly brackets to keep CSV line length uniform
        
        out << "\"{{";
        
        if (!data.empty())
        {
            std::copy(data.begin(), data.end() - 1, std::ostream_iterator<float>(out, "},{"));
            std::copy(data.end()-1, data.end(), std::ostream_iterator<float>(out, ""));
        }
        out << "}}\"";out << ",";
    out << height;out << ",";
    out << width;out << ",";
    out << transform;out << ",";
    out << timestamp_us;out << ",";
    out << counter;
    
}

std::stringstream Grid::toStringstream() const
{
    std::stringstream out;
    toCsv(out);
    return out;
}

template<class Archive>
    void Grid::serialize(Archive& ar, [[maybe_unused]] unsigned int version)
    {

        ar& CEREAL_NVP(data);
        ar& CEREAL_NVP(height);
        ar& CEREAL_NVP(width);
        ar& CEREAL_NVP(transform);
        ar& CEREAL_NVP(timestamp_us);
        ar& CEREAL_NVP(counter);
    }


template void Grid::serialize(cereal::BinaryInputArchive& ar, unsigned int version);
template void Grid::serialize(cereal::BinaryOutputArchive& ar, unsigned int version);
template void Grid::serialize(cereal::IntrospectionArchive& ar, unsigned int version);
template void Grid::serialize(cereal::JSONInputArchive& ar, unsigned int version);
template void Grid::serialize(cereal::JSONOutputArchive& ar, unsigned int version);
template void Grid::serialize(cereal::PortableBinaryInputArchive& ar, unsigned int version);
template void Grid::serialize(cereal::PortableBinaryOutputArchive& ar, unsigned int version);

std::ostream& operator<< (std::ostream& os, const Grid& grid)
{
    grid.toCsv(os);
    return os;
}

bool operator== (const Grid &lhs, const Grid &rhs)
{
    return (  lhs.data== rhs.data &&
     lhs.height== rhs.height &&
     lhs.width== rhs.width &&
     lhs.transform== rhs.transform &&
     lhs.timestamp_us== rhs.timestamp_us &&
     lhs.counter== rhs.counter 
    );
}

std::size_t Grid::hash() const noexcept
{
    std::size_t result = 17;
    result = result * 31 + hashVector(data);
    result = result * 31 + std::hash<uint32_t>()(height);
    result = result * 31 + std::hash<uint32_t>()(width);
    result = result * 31 + std::hash<GridTransform>()(transform);
    result = result * 31 + std::hash<int64_t>()(timestamp_us);
    result = result * 31 + std::hash<int64_t>()(counter);
    return result;
}

 } // namespace av
 } // namespace msg
 } // namespace plan_ctrl


CEREAL_BIND_TO_ARCHIVES(av::msg::plan_ctrl::Grid)

