#include <interfaces/GvpmGnss/GvpmGnss.h>

#include <iterator>
#include <type_traits>

#include <cereal/cereal.hpp>
#include <cereal/archives/json.hpp>
#include <cereal/archives/binary.hpp>

#include <cereal/types/map.hpp>
#include <cereal/types/vector.hpp>
#include <cereal/types/array.hpp>
#include <cereal/types/string.hpp>
#include <cereal/types/complex.hpp>

#include <ODatum/IntrospectionArchive.h>
#include <ODatum/Hash.h>





 namespace av {
 namespace msg {


GvpmGnss::GvpmGnss() = default;

GvpmGnss::GvpmGnss(
    av::msg::TimePoint utcTime_arg ,
    uint32_t timeOfWeek_arg ,
    double latitude_arg ,
    double longitude_arg ,
    double altitude_arg ,
    double speed_arg ,
    double heading_arg ,
    uint8_t numberOfSatellites_arg ,
    gvpm_gnss::FixType fixType_arg ,
    gvpm_gnss::PowerSaveModeState powerSaveModeState_arg ,
    gvpm_gnss::CarrierPhaseStatus carrierPhaseStatus_arg ,
    bool differentialCorrectionsApplied_arg ,
    bool fixValid_arg ,
    bool headingValid_arg ,
    bool magneticDeclinationValidity_arg ,
    bool dateValidity_arg ,
    bool timeValidity_arg ,
    bool utcFullyResolved_arg ,
    bool confirmedDate_arg ,
    bool confirmedTime_arg ,
    double timingAccuracy_arg ,
    double horizontalAccuracy_arg ,
    double verticalAccuracy_arg ,
    double headingAccuracy_arg ,
    double velocityAccuracy_arg ,
    float timingDop_arg ,
    float horizontalDop_arg ,
    float verticalDop_arg ,
    float geometricDop_arg ,
    float positionDop_arg 
    ):
    utcTime { std::move(utcTime_arg) } // NOLINT
      ,timeOfWeek { std::move(timeOfWeek_arg) } // NOLINT
      ,latitude { std::move(latitude_arg) } // NOLINT
      ,longitude { std::move(longitude_arg) } // NOLINT
      ,altitude { std::move(altitude_arg) } // NOLINT
      ,speed { std::move(speed_arg) } // NOLINT
      ,heading { std::move(heading_arg) } // NOLINT
      ,numberOfSatellites { std::move(numberOfSatellites_arg) } // NOLINT
      ,fixType { std::move(fixType_arg) } // NOLINT
      ,powerSaveModeState { std::move(powerSaveModeState_arg) } // NOLINT
      ,carrierPhaseStatus { std::move(carrierPhaseStatus_arg) } // NOLINT
      ,differentialCorrectionsApplied { std::move(differentialCorrectionsApplied_arg) } // NOLINT
      ,fixValid { std::move(fixValid_arg) } // NOLINT
      ,headingValid { std::move(headingValid_arg) } // NOLINT
      ,magneticDeclinationValidity { std::move(magneticDeclinationValidity_arg) } // NOLINT
      ,dateValidity { std::move(dateValidity_arg) } // NOLINT
      ,timeValidity { std::move(timeValidity_arg) } // NOLINT
      ,utcFullyResolved { std::move(utcFullyResolved_arg) } // NOLINT
      ,confirmedDate { std::move(confirmedDate_arg) } // NOLINT
      ,confirmedTime { std::move(confirmedTime_arg) } // NOLINT
      ,timingAccuracy { std::move(timingAccuracy_arg) } // NOLINT
      ,horizontalAccuracy { std::move(horizontalAccuracy_arg) } // NOLINT
      ,verticalAccuracy { std::move(verticalAccuracy_arg) } // NOLINT
      ,headingAccuracy { std::move(headingAccuracy_arg) } // NOLINT
      ,velocityAccuracy { std::move(velocityAccuracy_arg) } // NOLINT
      ,timingDop { std::move(timingDop_arg) } // NOLINT
      ,horizontalDop { std::move(horizontalDop_arg) } // NOLINT
      ,verticalDop { std::move(verticalDop_arg) } // NOLINT
      ,geometricDop { std::move(geometricDop_arg) } // NOLINT
      ,positionDop { std::move(positionDop_arg) } // NOLINT
      {
}

GvpmGnss::GvpmGnss(const GvpmGnss&) = default;

// NOLINTNEXTLINE
GvpmGnss::GvpmGnss(GvpmGnss&&) = default;

GvpmGnss& GvpmGnss::operator=(const GvpmGnss&) = default;

// NOLINTNEXTLINE
GvpmGnss& GvpmGnss::operator=(GvpmGnss&&) = default;

void GvpmGnss::toCsv(std::ostream& out) const
{
    out.precision(16);
    out << std::boolalpha;
    out << utcTime;out << ",";
    out << timeOfWeek;out << ",";
    out << latitude;out << ",";
    out << longitude;out << ",";
    out << altitude;out << ",";
    out << speed;out << ",";
    out << heading;out << ",";
    // Cast to uint16_t to print as integer instead of char
        out << static_cast<uint16_t>(numberOfSatellites);out << ",";
    out << static_cast<uint32_t>(fixType);out << ",";
    out << static_cast<uint32_t>(powerSaveModeState);out << ",";
    out << static_cast<uint32_t>(carrierPhaseStatus);out << ",";
    out << differentialCorrectionsApplied;out << ",";
    out << fixValid;out << ",";
    out << headingValid;out << ",";
    out << magneticDeclinationValidity;out << ",";
    out << dateValidity;out << ",";
    out << timeValidity;out << ",";
    out << utcFullyResolved;out << ",";
    out << confirmedDate;out << ",";
    out << confirmedTime;out << ",";
    out << timingAccuracy;out << ",";
    out << horizontalAccuracy;out << ",";
    out << verticalAccuracy;out << ",";
    out << headingAccuracy;out << ",";
    out << velocityAccuracy;out << ",";
    out << timingDop;out << ",";
    out << horizontalDop;out << ",";
    out << verticalDop;out << ",";
    out << geometricDop;out << ",";
    out << positionDop;
    
}

std::stringstream GvpmGnss::toStringstream() const
{
    std::stringstream out;
    toCsv(out);
    return out;
}

template<class Archive>
    void GvpmGnss::serialize(Archive& ar, [[maybe_unused]] unsigned int version)
    {

        ar& CEREAL_NVP(utcTime);
        ar& CEREAL_NVP(timeOfWeek);
        ar& CEREAL_NVP(latitude);
        ar& CEREAL_NVP(longitude);
        ar& CEREAL_NVP(altitude);
        ar& CEREAL_NVP(speed);
        ar& CEREAL_NVP(heading);
        ar& CEREAL_NVP(numberOfSatellites);
        ar& CEREAL_NVP(fixType);
        ar& CEREAL_NVP(powerSaveModeState);
        ar& CEREAL_NVP(carrierPhaseStatus);
        ar& CEREAL_NVP(differentialCorrectionsApplied);
        ar& CEREAL_NVP(fixValid);
        ar& CEREAL_NVP(headingValid);
        ar& CEREAL_NVP(magneticDeclinationValidity);
        ar& CEREAL_NVP(dateValidity);
        ar& CEREAL_NVP(timeValidity);
        ar& CEREAL_NVP(utcFullyResolved);
        ar& CEREAL_NVP(confirmedDate);
        ar& CEREAL_NVP(confirmedTime);
        ar& CEREAL_NVP(timingAccuracy);
        ar& CEREAL_NVP(horizontalAccuracy);
        ar& CEREAL_NVP(verticalAccuracy);
        ar& CEREAL_NVP(headingAccuracy);
        ar& CEREAL_NVP(velocityAccuracy);
        ar& CEREAL_NVP(timingDop);
        ar& CEREAL_NVP(horizontalDop);
        ar& CEREAL_NVP(verticalDop);
        ar& CEREAL_NVP(geometricDop);
        ar& CEREAL_NVP(positionDop);
    }


template void GvpmGnss::serialize(cereal::BinaryInputArchive& ar, unsigned int version);
template void GvpmGnss::serialize(cereal::BinaryOutputArchive& ar, unsigned int version);
template void GvpmGnss::serialize(cereal::IntrospectionArchive& ar, unsigned int version);
template void GvpmGnss::serialize(cereal::JSONInputArchive& ar, unsigned int version);
template void GvpmGnss::serialize(cereal::JSONOutputArchive& ar, unsigned int version);
template void GvpmGnss::serialize(cereal::PortableBinaryInputArchive& ar, unsigned int version);
template void GvpmGnss::serialize(cereal::PortableBinaryOutputArchive& ar, unsigned int version);

std::ostream& operator<< (std::ostream& os, const GvpmGnss& gvpmgnss)
{
    gvpmgnss.toCsv(os);
    return os;
}

bool operator== (const GvpmGnss &lhs, const GvpmGnss &rhs)
{
    return (  lhs.utcTime== rhs.utcTime &&
     lhs.timeOfWeek== rhs.timeOfWeek &&
     lhs.latitude== rhs.latitude &&
     lhs.longitude== rhs.longitude &&
     lhs.altitude== rhs.altitude &&
     lhs.speed== rhs.speed &&
     lhs.heading== rhs.heading &&
     lhs.numberOfSatellites== rhs.numberOfSatellites &&
     lhs.fixType== rhs.fixType &&
     lhs.powerSaveModeState== rhs.powerSaveModeState &&
     lhs.carrierPhaseStatus== rhs.carrierPhaseStatus &&
     lhs.differentialCorrectionsApplied== rhs.differentialCorrectionsApplied &&
     lhs.fixValid== rhs.fixValid &&
     lhs.headingValid== rhs.headingValid &&
     lhs.magneticDeclinationValidity== rhs.magneticDeclinationValidity &&
     lhs.dateValidity== rhs.dateValidity &&
     lhs.timeValidity== rhs.timeValidity &&
     lhs.utcFullyResolved== rhs.utcFullyResolved &&
     lhs.confirmedDate== rhs.confirmedDate &&
     lhs.confirmedTime== rhs.confirmedTime &&
     lhs.timingAccuracy== rhs.timingAccuracy &&
     lhs.horizontalAccuracy== rhs.horizontalAccuracy &&
     lhs.verticalAccuracy== rhs.verticalAccuracy &&
     lhs.headingAccuracy== rhs.headingAccuracy &&
     lhs.velocityAccuracy== rhs.velocityAccuracy &&
     lhs.timingDop== rhs.timingDop &&
     lhs.horizontalDop== rhs.horizontalDop &&
     lhs.verticalDop== rhs.verticalDop &&
     lhs.geometricDop== rhs.geometricDop &&
     lhs.positionDop== rhs.positionDop 
    );
}

std::size_t GvpmGnss::hash() const noexcept
{
    std::size_t result = 17;
    result = result * 31 + std::hash<av::msg::TimePoint>()(utcTime);
    result = result * 31 + std::hash<uint32_t>()(timeOfWeek);
    result = result * 31 + std::hash<double>()(latitude);
    result = result * 31 + std::hash<double>()(longitude);
    result = result * 31 + std::hash<double>()(altitude);
    result = result * 31 + std::hash<double>()(speed);
    result = result * 31 + std::hash<double>()(heading);
    result = result * 31 + std::hash<uint8_t>()(numberOfSatellites);
    result = result * 31 + std::hash<gvpm_gnss::FixType>()(fixType);
    result = result * 31 + std::hash<gvpm_gnss::PowerSaveModeState>()(powerSaveModeState);
    result = result * 31 + std::hash<gvpm_gnss::CarrierPhaseStatus>()(carrierPhaseStatus);
    result = result * 31 + std::hash<bool>()(differentialCorrectionsApplied);
    result = result * 31 + std::hash<bool>()(fixValid);
    result = result * 31 + std::hash<bool>()(headingValid);
    result = result * 31 + std::hash<bool>()(magneticDeclinationValidity);
    result = result * 31 + std::hash<bool>()(dateValidity);
    result = result * 31 + std::hash<bool>()(timeValidity);
    result = result * 31 + std::hash<bool>()(utcFullyResolved);
    result = result * 31 + std::hash<bool>()(confirmedDate);
    result = result * 31 + std::hash<bool>()(confirmedTime);
    result = result * 31 + std::hash<double>()(timingAccuracy);
    result = result * 31 + std::hash<double>()(horizontalAccuracy);
    result = result * 31 + std::hash<double>()(verticalAccuracy);
    result = result * 31 + std::hash<double>()(headingAccuracy);
    result = result * 31 + std::hash<double>()(velocityAccuracy);
    result = result * 31 + std::hash<float>()(timingDop);
    result = result * 31 + std::hash<float>()(horizontalDop);
    result = result * 31 + std::hash<float>()(verticalDop);
    result = result * 31 + std::hash<float>()(geometricDop);
    result = result * 31 + std::hash<float>()(positionDop);
    return result;
}

 } // namespace av
 } // namespace msg


CEREAL_BIND_TO_ARCHIVES(av::msg::GvpmGnss)

