#include <interfaces/H264FrameMeta/H264FrameMeta.h>

#include <iterator>
#include <type_traits>

#include <cereal/cereal.hpp>
#include <cereal/archives/json.hpp>
#include <cereal/archives/binary.hpp>

#include <cereal/types/map.hpp>
#include <cereal/types/vector.hpp>
#include <cereal/types/array.hpp>
#include <cereal/types/string.hpp>
#include <cereal/types/complex.hpp>

#include <ODatum/IntrospectionArchive.h>
#include <ODatum/Hash.h>


 namespace av {
 namespace camera {


WhiteBalance::WhiteBalance() = default;

WhiteBalance::WhiteBalance(
    uint32_t red_arg ,
    uint32_t green_arg ,
    uint32_t blue_arg 
    ):
    red { std::move(red_arg) } // NOLINT
      ,
    green { std::move(green_arg) } // NOLINT
      ,
    blue { std::move(blue_arg) } // NOLINT
      
    {
}

WhiteBalance::WhiteBalance(const WhiteBalance&) = default;

// NOLINTNEXTLINE
WhiteBalance::WhiteBalance(WhiteBalance&&) = default;

WhiteBalance& WhiteBalance::operator=(const WhiteBalance&) = default;

// NOLINTNEXTLINE
WhiteBalance& WhiteBalance::operator=(WhiteBalance&&) = default;

void WhiteBalance::toCsv(std::ostream& out) const
{
    out.precision(16);
    out << std::boolalpha;
    out << red;out << ",";
    out << green;out << ",";
    out << blue;
    
}

std::stringstream WhiteBalance::toStringstream() const
{
    std::stringstream out;
    toCsv(out);
    return out;
}

template<class Archive>
    void WhiteBalance::serialize(Archive& ar, [[maybe_unused]] unsigned int version)
    {

        ar& CEREAL_NVP(red);
        ar& CEREAL_NVP(green);
        ar& CEREAL_NVP(blue);
    }


template void WhiteBalance::serialize(cereal::BinaryInputArchive& ar, unsigned int version);
template void WhiteBalance::serialize(cereal::BinaryOutputArchive& ar, unsigned int version);
template void WhiteBalance::serialize(cereal::IntrospectionArchive& ar, unsigned int version);
template void WhiteBalance::serialize(cereal::JSONInputArchive& ar, unsigned int version);
template void WhiteBalance::serialize(cereal::JSONOutputArchive& ar, unsigned int version);
template void WhiteBalance::serialize(cereal::PortableBinaryInputArchive& ar, unsigned int version);
template void WhiteBalance::serialize(cereal::PortableBinaryOutputArchive& ar, unsigned int version);

std::ostream& operator<< (std::ostream& os, const WhiteBalance& whitebalance)
{
    whitebalance.toCsv(os);
    return os;
}

bool operator== (const WhiteBalance &lhs, const WhiteBalance &rhs)
{
    return (  lhs.red== rhs.red &&
     lhs.green== rhs.green &&
     lhs.blue== rhs.blue 
    );
}

std::size_t WhiteBalance::hash() const noexcept
{
    std::size_t result = 17;
    result = result * 31 + std::hash<uint32_t>()(red);
    result = result * 31 + std::hash<uint32_t>()(green);
    result = result * 31 + std::hash<uint32_t>()(blue);
    return result;
}

 } // namespace av
 } // namespace camera


CEREAL_BIND_TO_ARCHIVES(av::camera::WhiteBalance)

 namespace av {
 namespace camera {


EmbeddedData::EmbeddedData() = default;

EmbeddedData::EmbeddedData(
    std::array<char,2986> frontEmbeddedDataLine_arg ,
    std::array<char,2986> rearEmbeddedDataLine1_arg ,
    std::array<char,2986> rearEmbeddedDataLine2_arg ,
    std::array<char,2986> rearEmbeddedDataLine3_arg ,
    std::array<char,2986> rearEmbeddedDataLine4_arg ,
    std::array<char,2986> rearEmbeddedDataLine5_arg ,
    std::array<char,2986> rearEmbeddedDataLine6_arg 
    ):
    frontEmbeddedDataLine { std::move(frontEmbeddedDataLine_arg) } // NOLINT
      ,
    rearEmbeddedDataLine1 { std::move(rearEmbeddedDataLine1_arg) } // NOLINT
      ,
    rearEmbeddedDataLine2 { std::move(rearEmbeddedDataLine2_arg) } // NOLINT
      ,
    rearEmbeddedDataLine3 { std::move(rearEmbeddedDataLine3_arg) } // NOLINT
      ,
    rearEmbeddedDataLine4 { std::move(rearEmbeddedDataLine4_arg) } // NOLINT
      ,
    rearEmbeddedDataLine5 { std::move(rearEmbeddedDataLine5_arg) } // NOLINT
      ,
    rearEmbeddedDataLine6 { std::move(rearEmbeddedDataLine6_arg) } // NOLINT
      
    {
}

EmbeddedData::EmbeddedData(const EmbeddedData&) = default;

// NOLINTNEXTLINE
EmbeddedData::EmbeddedData(EmbeddedData&&) = default;

EmbeddedData& EmbeddedData::operator=(const EmbeddedData&) = default;

// NOLINTNEXTLINE
EmbeddedData& EmbeddedData::operator=(EmbeddedData&&) = default;

void EmbeddedData::toCsv(std::ostream& out) const
{
    out.precision(16);
    out << std::boolalpha;
    // Enclose array in double quotations and curly brackets to keep CSV line length uniform
        
        out << "\"{{";
        
        if (!frontEmbeddedDataLine.empty())
        {
            std::copy(frontEmbeddedDataLine.begin(), frontEmbeddedDataLine.end() - 1, std::ostream_iterator<char>(out, "},{"));
            out << frontEmbeddedDataLine.back();
        }
        out << "}}\"";out << ",";
    // Enclose array in double quotations and curly brackets to keep CSV line length uniform
        
        out << "\"{{";
        
        if (!rearEmbeddedDataLine1.empty())
        {
            std::copy(rearEmbeddedDataLine1.begin(), rearEmbeddedDataLine1.end() - 1, std::ostream_iterator<char>(out, "},{"));
            out << rearEmbeddedDataLine1.back();
        }
        out << "}}\"";out << ",";
    // Enclose array in double quotations and curly brackets to keep CSV line length uniform
        
        out << "\"{{";
        
        if (!rearEmbeddedDataLine2.empty())
        {
            std::copy(rearEmbeddedDataLine2.begin(), rearEmbeddedDataLine2.end() - 1, std::ostream_iterator<char>(out, "},{"));
            out << rearEmbeddedDataLine2.back();
        }
        out << "}}\"";out << ",";
    // Enclose array in double quotations and curly brackets to keep CSV line length uniform
        
        out << "\"{{";
        
        if (!rearEmbeddedDataLine3.empty())
        {
            std::copy(rearEmbeddedDataLine3.begin(), rearEmbeddedDataLine3.end() - 1, std::ostream_iterator<char>(out, "},{"));
            out << rearEmbeddedDataLine3.back();
        }
        out << "}}\"";out << ",";
    // Enclose array in double quotations and curly brackets to keep CSV line length uniform
        
        out << "\"{{";
        
        if (!rearEmbeddedDataLine4.empty())
        {
            std::copy(rearEmbeddedDataLine4.begin(), rearEmbeddedDataLine4.end() - 1, std::ostream_iterator<char>(out, "},{"));
            out << rearEmbeddedDataLine4.back();
        }
        out << "}}\"";out << ",";
    // Enclose array in double quotations and curly brackets to keep CSV line length uniform
        
        out << "\"{{";
        
        if (!rearEmbeddedDataLine5.empty())
        {
            std::copy(rearEmbeddedDataLine5.begin(), rearEmbeddedDataLine5.end() - 1, std::ostream_iterator<char>(out, "},{"));
            out << rearEmbeddedDataLine5.back();
        }
        out << "}}\"";out << ",";
    // Enclose array in double quotations and curly brackets to keep CSV line length uniform
        
        out << "\"{{";
        
        if (!rearEmbeddedDataLine6.empty())
        {
            std::copy(rearEmbeddedDataLine6.begin(), rearEmbeddedDataLine6.end() - 1, std::ostream_iterator<char>(out, "},{"));
            out << rearEmbeddedDataLine6.back();
        }
        out << "}}\"";
    
}

std::stringstream EmbeddedData::toStringstream() const
{
    std::stringstream out;
    toCsv(out);
    return out;
}

template<class Archive>
    void EmbeddedData::serialize(Archive& ar, [[maybe_unused]] unsigned int version)
    {

        ar& CEREAL_NVP(frontEmbeddedDataLine);
        ar& CEREAL_NVP(rearEmbeddedDataLine1);
        ar& CEREAL_NVP(rearEmbeddedDataLine2);
        ar& CEREAL_NVP(rearEmbeddedDataLine3);
        ar& CEREAL_NVP(rearEmbeddedDataLine4);
        ar& CEREAL_NVP(rearEmbeddedDataLine5);
        ar& CEREAL_NVP(rearEmbeddedDataLine6);
    }


template void EmbeddedData::serialize(cereal::BinaryInputArchive& ar, unsigned int version);
template void EmbeddedData::serialize(cereal::BinaryOutputArchive& ar, unsigned int version);
template void EmbeddedData::serialize(cereal::IntrospectionArchive& ar, unsigned int version);
template void EmbeddedData::serialize(cereal::JSONInputArchive& ar, unsigned int version);
template void EmbeddedData::serialize(cereal::JSONOutputArchive& ar, unsigned int version);
template void EmbeddedData::serialize(cereal::PortableBinaryInputArchive& ar, unsigned int version);
template void EmbeddedData::serialize(cereal::PortableBinaryOutputArchive& ar, unsigned int version);

std::ostream& operator<< (std::ostream& os, const EmbeddedData& embeddeddata)
{
    embeddeddata.toCsv(os);
    return os;
}

bool operator== (const EmbeddedData &lhs, const EmbeddedData &rhs)
{
    return (  lhs.frontEmbeddedDataLine== rhs.frontEmbeddedDataLine &&
     lhs.rearEmbeddedDataLine1== rhs.rearEmbeddedDataLine1 &&
     lhs.rearEmbeddedDataLine2== rhs.rearEmbeddedDataLine2 &&
     lhs.rearEmbeddedDataLine3== rhs.rearEmbeddedDataLine3 &&
     lhs.rearEmbeddedDataLine4== rhs.rearEmbeddedDataLine4 &&
     lhs.rearEmbeddedDataLine5== rhs.rearEmbeddedDataLine5 &&
     lhs.rearEmbeddedDataLine6== rhs.rearEmbeddedDataLine6 
    );
}

std::size_t EmbeddedData::hash() const noexcept
{
    std::size_t result = 17;
    result = result * 31 + hashVector(frontEmbeddedDataLine);
    result = result * 31 + hashVector(rearEmbeddedDataLine1);
    result = result * 31 + hashVector(rearEmbeddedDataLine2);
    result = result * 31 + hashVector(rearEmbeddedDataLine3);
    result = result * 31 + hashVector(rearEmbeddedDataLine4);
    result = result * 31 + hashVector(rearEmbeddedDataLine5);
    result = result * 31 + hashVector(rearEmbeddedDataLine6);
    return result;
}

 } // namespace av
 } // namespace camera


CEREAL_BIND_TO_ARCHIVES(av::camera::EmbeddedData)




 namespace av {
 namespace camera {


H264FrameMeta::H264FrameMeta() = default;

H264FrameMeta::H264FrameMeta(
    std::string cameraId_arg ,
    uint16_t width_arg ,
    uint16_t height_arg ,
    PIXEL_FORMAT pixelFormat_arg ,
    TIME_SYNC_STATUS timeSyncStatus_arg ,
    int64_t syncTriggerOffset_us_arg ,
    int64_t captureTime_us_arg ,
    int64_t receiveTime_us_arg ,
    int64_t ingestTime_us_arg ,
    std::array<uint32_t,4> analogGain_arg ,
    std::array<uint32_t,4> digitalGain_arg ,
    WhiteBalance whiteBalance_arg ,
    std::array<uint32_t,4> exposureTime_us_arg ,
    int64_t startOfFrame_us_arg ,
    uint32_t capPts_arg ,
    EmbeddedData embeddedDataLines_arg ,
    std::array<char,40> firmwareHash_arg 
    ):
    cameraId { std::move(cameraId_arg) } // NOLINT
      ,width { std::move(width_arg) } // NOLINT
      ,height { std::move(height_arg) } // NOLINT
      ,pixelFormat { std::move(pixelFormat_arg) } // NOLINT
      ,timeSyncStatus { std::move(timeSyncStatus_arg) } // NOLINT
      ,syncTriggerOffset_us { std::move(syncTriggerOffset_us_arg) } // NOLINT
      ,captureTime_us { std::move(captureTime_us_arg) } // NOLINT
      ,receiveTime_us { std::move(receiveTime_us_arg) } // NOLINT
      ,ingestTime_us { std::move(ingestTime_us_arg) } // NOLINT
      ,analogGain { std::move(analogGain_arg) } // NOLINT
      ,digitalGain { std::move(digitalGain_arg) } // NOLINT
      ,whiteBalance { std::move(whiteBalance_arg) } // NOLINT
      ,exposureTime_us { std::move(exposureTime_us_arg) } // NOLINT
      ,startOfFrame_us { std::move(startOfFrame_us_arg) } // NOLINT
      ,capPts { std::move(capPts_arg) } // NOLINT
      ,embeddedDataLines { std::move(embeddedDataLines_arg) } // NOLINT
      ,firmwareHash { std::move(firmwareHash_arg) } // NOLINT
      {
}

H264FrameMeta::H264FrameMeta(const H264FrameMeta&) = default;

// NOLINTNEXTLINE
H264FrameMeta::H264FrameMeta(H264FrameMeta&&) = default;

H264FrameMeta& H264FrameMeta::operator=(const H264FrameMeta&) = default;

// NOLINTNEXTLINE
H264FrameMeta& H264FrameMeta::operator=(H264FrameMeta&&) = default;

void H264FrameMeta::toCsv(std::ostream& out) const
{
    out.precision(16);
    out << std::boolalpha;
    out << "\"";
        out << cameraId;
        out << "\"";out << ",";
    out << width;out << ",";
    out << height;out << ",";
    out << static_cast<uint32_t>(pixelFormat);out << ",";
    out << static_cast<uint32_t>(timeSyncStatus);out << ",";
    out << syncTriggerOffset_us;out << ",";
    out << captureTime_us;out << ",";
    out << receiveTime_us;out << ",";
    out << ingestTime_us;out << ",";
    // Enclose array in double quotations and curly brackets to keep CSV line length uniform
        
        out << "\"{{";
        
        if (!analogGain.empty())
        {
            std::copy(analogGain.begin(), analogGain.end() - 1, std::ostream_iterator<uint32_t>(out, "},{"));
            out << analogGain.back();
        }
        out << "}}\"";out << ",";
    // Enclose array in double quotations and curly brackets to keep CSV line length uniform
        
        out << "\"{{";
        
        if (!digitalGain.empty())
        {
            std::copy(digitalGain.begin(), digitalGain.end() - 1, std::ostream_iterator<uint32_t>(out, "},{"));
            out << digitalGain.back();
        }
        out << "}}\"";out << ",";
    out << whiteBalance;out << ",";
    // Enclose array in double quotations and curly brackets to keep CSV line length uniform
        
        out << "\"{{";
        
        if (!exposureTime_us.empty())
        {
            std::copy(exposureTime_us.begin(), exposureTime_us.end() - 1, std::ostream_iterator<uint32_t>(out, "},{"));
            out << exposureTime_us.back();
        }
        out << "}}\"";out << ",";
    out << startOfFrame_us;out << ",";
    out << capPts;out << ",";
    out << embeddedDataLines;out << ",";
    // Enclose array in double quotations and curly brackets to keep CSV line length uniform
        
        out << "\"{{";
        
        if (!firmwareHash.empty())
        {
            std::copy(firmwareHash.begin(), firmwareHash.end() - 1, std::ostream_iterator<char>(out, "},{"));
            out << firmwareHash.back();
        }
        out << "}}\"";
    
}

std::stringstream H264FrameMeta::toStringstream() const
{
    std::stringstream out;
    toCsv(out);
    return out;
}

template<class Archive>
    void H264FrameMeta::serialize(Archive& ar, unsigned int version)
{
    ar& CEREAL_NVP(cameraId);
    ar& CEREAL_NVP(width);
    ar& CEREAL_NVP(height);
    ar& CEREAL_NVP(pixelFormat);
    ar& CEREAL_NVP(timeSyncStatus);
    ar& CEREAL_NVP(syncTriggerOffset_us);
    ar& CEREAL_NVP(captureTime_us);
    ar& CEREAL_NVP(receiveTime_us);
    ar& CEREAL_NVP(ingestTime_us);

    if (version >= 2) {
        ar& CEREAL_NVP(analogGain);
        ar& CEREAL_NVP(digitalGain);
        ar& CEREAL_NVP(whiteBalance);
        ar& CEREAL_NVP(exposureTime_us);
        ar& CEREAL_NVP(startOfFrame_us);
        if (version >= 3) {
            ar& CEREAL_NVP(capPts);
        }
        else {
            uint64_t origCapPts = capPts;
            ar& cereal::make_nvp("capPts", origCapPts);
            capPts = static_cast<uint32_t>(origCapPts);
        }
        ar& CEREAL_NVP(embeddedDataLines);
        ar& CEREAL_NVP(firmwareHash);
    } else {
        ar& cereal::make_nvp("analogGain", analogGain[0]);
        ar& cereal::make_nvp("digitalGain", digitalGain[0]);
        ar& cereal::make_nvp("whiteBalance", whiteBalance.red);
        ar& cereal::make_nvp("exposure", exposureTime_us);
    }
}


template void H264FrameMeta::serialize(cereal::BinaryInputArchive& ar, unsigned int version);
template void H264FrameMeta::serialize(cereal::BinaryOutputArchive& ar, unsigned int version);
template void H264FrameMeta::serialize(cereal::IntrospectionArchive& ar, unsigned int version);
template void H264FrameMeta::serialize(cereal::JSONInputArchive& ar, unsigned int version);
template void H264FrameMeta::serialize(cereal::JSONOutputArchive& ar, unsigned int version);
template void H264FrameMeta::serialize(cereal::PortableBinaryInputArchive& ar, unsigned int version);
template void H264FrameMeta::serialize(cereal::PortableBinaryOutputArchive& ar, unsigned int version);

std::ostream& operator<< (std::ostream& os, const H264FrameMeta& h264framemeta)
{
    h264framemeta.toCsv(os);
    return os;
}

bool operator== (const H264FrameMeta &lhs, const H264FrameMeta &rhs)
{
    return (  lhs.cameraId== rhs.cameraId &&
     lhs.width== rhs.width &&
     lhs.height== rhs.height &&
     lhs.pixelFormat== rhs.pixelFormat &&
     lhs.timeSyncStatus== rhs.timeSyncStatus &&
     lhs.syncTriggerOffset_us== rhs.syncTriggerOffset_us &&
     lhs.captureTime_us== rhs.captureTime_us &&
     lhs.receiveTime_us== rhs.receiveTime_us &&
     lhs.ingestTime_us== rhs.ingestTime_us &&
     lhs.analogGain== rhs.analogGain &&
     lhs.digitalGain== rhs.digitalGain &&
     lhs.whiteBalance== rhs.whiteBalance &&
     lhs.exposureTime_us== rhs.exposureTime_us &&
     lhs.startOfFrame_us== rhs.startOfFrame_us &&
     lhs.capPts== rhs.capPts &&
     lhs.embeddedDataLines== rhs.embeddedDataLines &&
     lhs.firmwareHash== rhs.firmwareHash 
    );
}

std::size_t H264FrameMeta::hash() const noexcept
{
    std::size_t result = 17;
    result = result * 31 + std::hash<std::string>()(cameraId);
    result = result * 31 + std::hash<uint16_t>()(width);
    result = result * 31 + std::hash<uint16_t>()(height);
    result = result * 31 + std::hash<PIXEL_FORMAT>()(pixelFormat);
    result = result * 31 + std::hash<TIME_SYNC_STATUS>()(timeSyncStatus);
    result = result * 31 + std::hash<int64_t>()(syncTriggerOffset_us);
    result = result * 31 + std::hash<int64_t>()(captureTime_us);
    result = result * 31 + std::hash<int64_t>()(receiveTime_us);
    result = result * 31 + std::hash<int64_t>()(ingestTime_us);
    result = result * 31 + hashVector(analogGain);
    result = result * 31 + hashVector(digitalGain);
    result = result * 31 + std::hash<WhiteBalance>()(whiteBalance);
    result = result * 31 + hashVector(exposureTime_us);
    result = result * 31 + std::hash<int64_t>()(startOfFrame_us);
    result = result * 31 + std::hash<uint32_t>()(capPts);
    result = result * 31 + std::hash<EmbeddedData>()(embeddedDataLines);
    result = result * 31 + hashVector(firmwareHash);
    return result;
}

 } // namespace av
 } // namespace camera


CEREAL_BIND_TO_ARCHIVES(av::camera::H264FrameMeta)

