#include <interfaces/HMIStatus/HMIStatus.h>

#include <iterator>
#include <type_traits>

#include <cereal/cereal.hpp>
#include <cereal/archives/json.hpp>
#include <cereal/archives/binary.hpp>

#include <cereal/types/map.hpp>
#include <cereal/types/vector.hpp>
#include <cereal/types/array.hpp>
#include <cereal/types/string.hpp>
#include <cereal/types/complex.hpp>

#include <ODatum/IntrospectionArchive.h>
#include <ODatum/Hash.h>





 namespace av {
 namespace msg {


HMIStatus::HMIStatus() = default;

HMIStatus::HMIStatus(
    TimePoint timeStamp_arg ,
    std::string source_arg ,
    double setSpeed_arg ,
    bool laneChangeConfig_arg ,
    int32_t speedControlConfig_arg ,
    int32_t distanceKeepingConfig_arg ,
    std::string dSpaceVersion_arg ,
    uint16_t dSpaceSystemState_arg 
    ):
    timeStamp { std::move(timeStamp_arg) } // NOLINT
      ,source { std::move(source_arg) } // NOLINT
      ,setSpeed { std::move(setSpeed_arg) } // NOLINT
      ,laneChangeConfig { std::move(laneChangeConfig_arg) } // NOLINT
      ,speedControlConfig { std::move(speedControlConfig_arg) } // NOLINT
      ,distanceKeepingConfig { std::move(distanceKeepingConfig_arg) } // NOLINT
      ,dSpaceVersion { std::move(dSpaceVersion_arg) } // NOLINT
      ,dSpaceSystemState { std::move(dSpaceSystemState_arg) } // NOLINT
      {
}

HMIStatus::HMIStatus(const HMIStatus&) = default;

// NOLINTNEXTLINE
HMIStatus::HMIStatus(HMIStatus&&) = default;

HMIStatus& HMIStatus::operator=(const HMIStatus&) = default;

// NOLINTNEXTLINE
HMIStatus& HMIStatus::operator=(HMIStatus&&) = default;

void HMIStatus::toCsv(std::ostream& out) const
{
    out.precision(16);
    out << std::boolalpha;
    out << timeStamp;out << ",";
    out << "\"";
        out << source;
        out << "\"";out << ",";
    out << setSpeed;out << ",";
    out << laneChangeConfig;out << ",";
    out << speedControlConfig;out << ",";
    out << distanceKeepingConfig;out << ",";
    out << "\"";
        out << dSpaceVersion;
        out << "\"";out << ",";
    out << dSpaceSystemState;
    
}

std::stringstream HMIStatus::toStringstream() const
{
    std::stringstream out;
    toCsv(out);
    return out;
}

template<class Archive>
    void HMIStatus::serialize(Archive& ar, unsigned int version)
{
    if (version < 3) {
        otimePoint otp;
        ar& cereal::make_nvp("timeStamp", otp);
        timeStamp = av::TimePoint(std::chrono::seconds(otp.tv_sec) + std::chrono::nanoseconds(otp.tv_nsec));
    } else {
        ar& CEREAL_NVP(timeStamp);
    }

    ar& CEREAL_NVP(source);
    ar& CEREAL_NVP(setSpeed);
    ar& CEREAL_NVP(laneChangeConfig);
    ar& CEREAL_NVP(speedControlConfig);
    ar& CEREAL_NVP(distanceKeepingConfig);
    ar& CEREAL_NVP(dSpaceVersion);

    if (version >= 2) {
        ar& CEREAL_NVP(dSpaceSystemState);
    }
}

}} // to close av::msg namespace temporary

struct HMIStatusMessage : public av::msg::HMIStatus {};
CEREAL_CLASS_VERSION(HMIStatusMessage, 3)
CEREAL_REGISTER_TYPE(HMIStatusMessage)

namespace av {
namespace msg {



template void HMIStatus::serialize(cereal::BinaryInputArchive& ar, unsigned int version);
template void HMIStatus::serialize(cereal::BinaryOutputArchive& ar, unsigned int version);
template void HMIStatus::serialize(cereal::IntrospectionArchive& ar, unsigned int version);
template void HMIStatus::serialize(cereal::JSONInputArchive& ar, unsigned int version);
template void HMIStatus::serialize(cereal::JSONOutputArchive& ar, unsigned int version);
template void HMIStatus::serialize(cereal::PortableBinaryInputArchive& ar, unsigned int version);
template void HMIStatus::serialize(cereal::PortableBinaryOutputArchive& ar, unsigned int version);

std::ostream& operator<< (std::ostream& os, const HMIStatus& hmistatus)
{
    hmistatus.toCsv(os);
    return os;
}

bool operator== (const HMIStatus &lhs, const HMIStatus &rhs)
{
    return (  lhs.timeStamp== rhs.timeStamp &&
     lhs.source== rhs.source &&
     lhs.setSpeed== rhs.setSpeed &&
     lhs.laneChangeConfig== rhs.laneChangeConfig &&
     lhs.speedControlConfig== rhs.speedControlConfig &&
     lhs.distanceKeepingConfig== rhs.distanceKeepingConfig &&
     lhs.dSpaceVersion== rhs.dSpaceVersion &&
     lhs.dSpaceSystemState== rhs.dSpaceSystemState 
    );
}

std::size_t HMIStatus::hash() const noexcept
{
    std::size_t result = 17;
    result = result * 31 + 0;
    result = result * 31 + std::hash<std::string>()(source);
    result = result * 31 + std::hash<double>()(setSpeed);
    result = result * 31 + std::hash<bool>()(laneChangeConfig);
    result = result * 31 + std::hash<int32_t>()(speedControlConfig);
    result = result * 31 + std::hash<int32_t>()(distanceKeepingConfig);
    result = result * 31 + std::hash<std::string>()(dSpaceVersion);
    result = result * 31 + std::hash<uint16_t>()(dSpaceSystemState);
    return result;
}

 } // namespace av
 } // namespace msg


CEREAL_BIND_TO_ARCHIVES(av::msg::HMIStatus)

