#include <interfaces/HealthDisplay/HealthDisplay.h>

#include <iterator>
#include <type_traits>

#include <cereal/cereal.hpp>
#include <cereal/archives/json.hpp>
#include <cereal/archives/binary.hpp>

#include <cereal/types/map.hpp>
#include <cereal/types/vector.hpp>
#include <cereal/types/array.hpp>
#include <cereal/types/string.hpp>
#include <cereal/types/complex.hpp>

#include <ODatum/IntrospectionArchive.h>
#include <ODatum/Hash.h>


 namespace av {
 namespace msg {


processIndex::processIndex() = default;

processIndex::processIndex(
    std::string processName_arg ,
    uint32_t index_arg 
    ):
    processName { std::move(processName_arg) } // NOLINT
      ,
    index { std::move(index_arg) } // NOLINT
      
    {
}

processIndex::processIndex(const processIndex&) = default;

// NOLINTNEXTLINE
processIndex::processIndex(processIndex&&) = default;

processIndex& processIndex::operator=(const processIndex&) = default;

// NOLINTNEXTLINE
processIndex& processIndex::operator=(processIndex&&) = default;

void processIndex::toCsv(std::ostream& out) const
{
    out.precision(16);
    out << std::boolalpha;
    out << "\"";
        out << processName;
        out << "\"";out << ",";
    out << index;
    
}

std::stringstream processIndex::toStringstream() const
{
    std::stringstream out;
    toCsv(out);
    return out;
}

template<class Archive>
    void processIndex::serialize(Archive& ar, [[maybe_unused]] unsigned int version)
    {

        ar& CEREAL_NVP(processName);
        ar& CEREAL_NVP(index);
    }


template void processIndex::serialize(cereal::BinaryInputArchive& ar, unsigned int version);
template void processIndex::serialize(cereal::BinaryOutputArchive& ar, unsigned int version);
template void processIndex::serialize(cereal::IntrospectionArchive& ar, unsigned int version);
template void processIndex::serialize(cereal::JSONInputArchive& ar, unsigned int version);
template void processIndex::serialize(cereal::JSONOutputArchive& ar, unsigned int version);
template void processIndex::serialize(cereal::PortableBinaryInputArchive& ar, unsigned int version);
template void processIndex::serialize(cereal::PortableBinaryOutputArchive& ar, unsigned int version);

std::ostream& operator<< (std::ostream& os, const processIndex& processindex)
{
    processindex.toCsv(os);
    return os;
}

bool operator== (const processIndex &lhs, const processIndex &rhs)
{
    return (  lhs.processName== rhs.processName &&
     lhs.index== rhs.index 
    );
}

std::size_t processIndex::hash() const noexcept
{
    std::size_t result = 17;
    result = result * 31 + std::hash<std::string>()(processName);
    result = result * 31 + std::hash<uint32_t>()(index);
    return result;
}

 } // namespace av
 } // namespace msg


CEREAL_BIND_TO_ARCHIVES(av::msg::processIndex)

 namespace av {
 namespace msg {


processValue::processValue() = default;

processValue::processValue(
    std::string processName_arg ,
    bool boolValue_arg 
    ):
    processName { std::move(processName_arg) } // NOLINT
      ,
    boolValue { std::move(boolValue_arg) } // NOLINT
      
    {
}

processValue::processValue(const processValue&) = default;

// NOLINTNEXTLINE
processValue::processValue(processValue&&) = default;

processValue& processValue::operator=(const processValue&) = default;

// NOLINTNEXTLINE
processValue& processValue::operator=(processValue&&) = default;

void processValue::toCsv(std::ostream& out) const
{
    out.precision(16);
    out << std::boolalpha;
    out << "\"";
        out << processName;
        out << "\"";out << ",";
    out << boolValue;
    
}

std::stringstream processValue::toStringstream() const
{
    std::stringstream out;
    toCsv(out);
    return out;
}

template<class Archive>
    void processValue::serialize(Archive& ar, [[maybe_unused]] unsigned int version)
    {

        ar& CEREAL_NVP(processName);
        ar& CEREAL_NVP(boolValue);
    }


template void processValue::serialize(cereal::BinaryInputArchive& ar, unsigned int version);
template void processValue::serialize(cereal::BinaryOutputArchive& ar, unsigned int version);
template void processValue::serialize(cereal::IntrospectionArchive& ar, unsigned int version);
template void processValue::serialize(cereal::JSONInputArchive& ar, unsigned int version);
template void processValue::serialize(cereal::JSONOutputArchive& ar, unsigned int version);
template void processValue::serialize(cereal::PortableBinaryInputArchive& ar, unsigned int version);
template void processValue::serialize(cereal::PortableBinaryOutputArchive& ar, unsigned int version);

std::ostream& operator<< (std::ostream& os, const processValue& processvalue)
{
    processvalue.toCsv(os);
    return os;
}

bool operator== (const processValue &lhs, const processValue &rhs)
{
    return (  lhs.processName== rhs.processName &&
     lhs.boolValue== rhs.boolValue 
    );
}

std::size_t processValue::hash() const noexcept
{
    std::size_t result = 17;
    result = result * 31 + std::hash<std::string>()(processName);
    result = result * 31 + std::hash<bool>()(boolValue);
    return result;
}

 } // namespace av
 } // namespace msg


CEREAL_BIND_TO_ARCHIVES(av::msg::processValue)

 namespace av {
 namespace msg {


processValues::processValues() = default;

processValues::processValues(
    std::vector<processValue> values_arg 
    ):
    values { std::move(values_arg) } // NOLINT
      
    {
}

processValues::processValues(const processValues&) = default;

// NOLINTNEXTLINE
processValues::processValues(processValues&&) = default;

processValues& processValues::operator=(const processValues&) = default;

// NOLINTNEXTLINE
processValues& processValues::operator=(processValues&&) = default;

void processValues::toCsv(std::ostream& out) const
{
    out.precision(16);
    out << std::boolalpha;
    // Enclose sequence in double quotations and curly brackets to keep CSV line length uniform
        
        out << "\"{{";
        
        if (!values.empty())
        {
            std::copy(values.begin(), values.end() - 1, std::ostream_iterator<processValue>(out, "},{"));
            std::copy(values.end()-1, values.end(), std::ostream_iterator<processValue>(out, ""));
        }
        out << "}}\"";
    
}

std::stringstream processValues::toStringstream() const
{
    std::stringstream out;
    toCsv(out);
    return out;
}

template<class Archive>
    void processValues::serialize(Archive& ar, [[maybe_unused]] unsigned int version)
    {

        ar& CEREAL_NVP(values);
    }


template void processValues::serialize(cereal::BinaryInputArchive& ar, unsigned int version);
template void processValues::serialize(cereal::BinaryOutputArchive& ar, unsigned int version);
template void processValues::serialize(cereal::IntrospectionArchive& ar, unsigned int version);
template void processValues::serialize(cereal::JSONInputArchive& ar, unsigned int version);
template void processValues::serialize(cereal::JSONOutputArchive& ar, unsigned int version);
template void processValues::serialize(cereal::PortableBinaryInputArchive& ar, unsigned int version);
template void processValues::serialize(cereal::PortableBinaryOutputArchive& ar, unsigned int version);

std::ostream& operator<< (std::ostream& os, const processValues& processvalues)
{
    processvalues.toCsv(os);
    return os;
}

bool operator== (const processValues &lhs, const processValues &rhs)
{
    return (  lhs.values== rhs.values 
    );
}

std::size_t processValues::hash() const noexcept
{
    std::size_t result = 17;
    result = result * 31 + hashVector(values);
    return result;
}

 } // namespace av
 } // namespace msg


CEREAL_BIND_TO_ARCHIVES(av::msg::processValues)




 namespace av {
 namespace msg {


HealthDisplay::HealthDisplay() = default;

HealthDisplay::HealthDisplay(
    std::vector<processIndex> indices_arg ,
    std::vector<ProcessOpinion> processOpinions_arg ,
    std::vector<std::string> processStatus_arg ,
    std::vector<double> processRates_hz_arg ,
    std::vector<double> intendedRates_hz_arg ,
    std::vector<int> processIDs_arg ,
    std::vector<processValues> boolDatas_arg 
    ):
    indices { std::move(indices_arg) } // NOLINT
      ,processOpinions { std::move(processOpinions_arg) } // NOLINT
      ,processStatus { std::move(processStatus_arg) } // NOLINT
      ,processRates_hz { std::move(processRates_hz_arg) } // NOLINT
      ,intendedRates_hz { std::move(intendedRates_hz_arg) } // NOLINT
      ,processIDs { std::move(processIDs_arg) } // NOLINT
      ,boolDatas { std::move(boolDatas_arg) } // NOLINT
      {
}

HealthDisplay::HealthDisplay(const HealthDisplay&) = default;

// NOLINTNEXTLINE
HealthDisplay::HealthDisplay(HealthDisplay&&) = default;

HealthDisplay& HealthDisplay::operator=(const HealthDisplay&) = default;

// NOLINTNEXTLINE
HealthDisplay& HealthDisplay::operator=(HealthDisplay&&) = default;

void HealthDisplay::toCsv(std::ostream& out) const
{
    out.precision(16);
    out << std::boolalpha;
    // Enclose sequence in double quotations and curly brackets to keep CSV line length uniform
        
        out << "\"{{";
        
        if (!indices.empty())
        {
            std::copy(indices.begin(), indices.end() - 1, std::ostream_iterator<processIndex>(out, "},{"));
            std::copy(indices.end()-1, indices.end(), std::ostream_iterator<processIndex>(out, ""));
        }
        out << "}}\"";out << ",";
    // Enclose sequence in double quotations and curly brackets to keep CSV line length uniform
        
        out << "\"{{";
        
        if (!processOpinions.empty())
        {
            std::copy(processOpinions.begin(), processOpinions.end() - 1, std::ostream_iterator<ProcessOpinion>(out, "},{"));
            std::copy(processOpinions.end()-1, processOpinions.end(), std::ostream_iterator<ProcessOpinion>(out, ""));
        }
        out << "}}\"";out << ",";
    // Enclose sequence in double quotations and curly brackets to keep CSV line length uniform
        
        out << "\"{{";
        
        if (!processStatus.empty())
        {
            std::copy(processStatus.begin(), processStatus.end() - 1, std::ostream_iterator<std::string>(out, "},{"));
            std::copy(processStatus.end()-1, processStatus.end(), std::ostream_iterator<std::string>(out, ""));
        }
        out << "}}\"";out << ",";
    // Enclose sequence in double quotations and curly brackets to keep CSV line length uniform
        
        out << "\"{{";
        
        if (!processRates_hz.empty())
        {
            std::copy(processRates_hz.begin(), processRates_hz.end() - 1, std::ostream_iterator<double>(out, "},{"));
            std::copy(processRates_hz.end()-1, processRates_hz.end(), std::ostream_iterator<double>(out, ""));
        }
        out << "}}\"";out << ",";
    // Enclose sequence in double quotations and curly brackets to keep CSV line length uniform
        
        out << "\"{{";
        
        if (!intendedRates_hz.empty())
        {
            std::copy(intendedRates_hz.begin(), intendedRates_hz.end() - 1, std::ostream_iterator<double>(out, "},{"));
            std::copy(intendedRates_hz.end()-1, intendedRates_hz.end(), std::ostream_iterator<double>(out, ""));
        }
        out << "}}\"";out << ",";
    // Enclose sequence in double quotations and curly brackets to keep CSV line length uniform
        
        out << "\"{{";
        
        if (!processIDs.empty())
        {
            std::copy(processIDs.begin(), processIDs.end() - 1, std::ostream_iterator<int>(out, "},{"));
            std::copy(processIDs.end()-1, processIDs.end(), std::ostream_iterator<int>(out, ""));
        }
        out << "}}\"";out << ",";
    // Enclose sequence in double quotations and curly brackets to keep CSV line length uniform
        
        out << "\"{{";
        
        if (!boolDatas.empty())
        {
            std::copy(boolDatas.begin(), boolDatas.end() - 1, std::ostream_iterator<processValues>(out, "},{"));
            std::copy(boolDatas.end()-1, boolDatas.end(), std::ostream_iterator<processValues>(out, ""));
        }
        out << "}}\"";
    
}

std::stringstream HealthDisplay::toStringstream() const
{
    std::stringstream out;
    toCsv(out);
    return out;
}

template<class Archive>
    #include <iostream>

void HealthDisplay::serialize(Archive& ar, unsigned int version)
{
	if(version == 1)
	{
		std::map<std::string, unsigned int> indicesProxy;
		ar& CEREAL_NVP(indicesProxy);
		for(auto index : indicesProxy)
		{
			indices.emplace_back(processIndex{index.first, index.second});
		}

		std::vector<int> processOpinionsProxy;
		ar& CEREAL_NVP(processOpinionsProxy);
		for(auto po : processOpinionsProxy)
		{
			processOpinions.emplace_back(static_cast<ProcessOpinion>(po));
		}
	}
	else
	{
		ar& CEREAL_NVP(indices);
		ar& CEREAL_NVP(processOpinions);
	}
        ar& CEREAL_NVP(processStatus);
        ar& CEREAL_NVP(processRates_hz);
        ar& CEREAL_NVP(intendedRates_hz);
        ar& CEREAL_NVP(processIDs);

	if(version == 1)
	{
		std::vector<std::map<std::string, bool>> dataProxy;
		ar& CEREAL_NVP(dataProxy);
		for(auto mapData : dataProxy)
		{
			processValues pValues;
			for(auto data : mapData)
			{
				pValues.values.emplace_back(processValue{data.first, data.second});
			}
			boolDatas.emplace_back(pValues);
		}
	}
	else
	{
        ar& CEREAL_NVP(boolDatas);
	}
}

std::ostream& operator<<(std::ostream& os, const av::msg::ProcessOpinion& po)
{
    os << static_cast<uint32_t>(po);
    return os;
}

}} // to close av::msg namespace temporary

struct HealthDisplayMessage : public av::msg::HealthDisplay{};
CEREAL_CLASS_VERSION(HealthDisplayMessage, 2)
CEREAL_REGISTER_TYPE(HealthDisplayMessage)

namespace av {
namespace msg {



template void HealthDisplay::serialize(cereal::BinaryInputArchive& ar, unsigned int version);
template void HealthDisplay::serialize(cereal::BinaryOutputArchive& ar, unsigned int version);
template void HealthDisplay::serialize(cereal::IntrospectionArchive& ar, unsigned int version);
template void HealthDisplay::serialize(cereal::JSONInputArchive& ar, unsigned int version);
template void HealthDisplay::serialize(cereal::JSONOutputArchive& ar, unsigned int version);
template void HealthDisplay::serialize(cereal::PortableBinaryInputArchive& ar, unsigned int version);
template void HealthDisplay::serialize(cereal::PortableBinaryOutputArchive& ar, unsigned int version);

std::ostream& operator<< (std::ostream& os, const HealthDisplay& healthdisplay)
{
    healthdisplay.toCsv(os);
    return os;
}

bool operator== (const HealthDisplay &lhs, const HealthDisplay &rhs)
{
    return (  lhs.indices== rhs.indices &&
     lhs.processOpinions== rhs.processOpinions &&
     lhs.processStatus== rhs.processStatus &&
     lhs.processRates_hz== rhs.processRates_hz &&
     lhs.intendedRates_hz== rhs.intendedRates_hz &&
     lhs.processIDs== rhs.processIDs &&
     lhs.boolDatas== rhs.boolDatas 
    );
}

std::size_t HealthDisplay::hash() const noexcept
{
    std::size_t result = 17;
    result = result * 31 + hashVector(indices);
    result = result * 31 + hashVector(processOpinions);
    result = result * 31 + hashVector(processStatus);
    result = result * 31 + hashVector(processRates_hz);
    result = result * 31 + hashVector(intendedRates_hz);
    result = result * 31 + hashVector(processIDs);
    result = result * 31 + hashVector(boolDatas);
    return result;
}

 } // namespace av
 } // namespace msg


CEREAL_BIND_TO_ARCHIVES(av::msg::HealthDisplay)

