#include <interfaces/HomotopyExtractorDebug/HomotopyExtractorDebug.h>

#include <iterator>
#include <type_traits>

#include <cereal/cereal.hpp>
#include <cereal/archives/json.hpp>
#include <cereal/archives/binary.hpp>

#include <cereal/types/map.hpp>
#include <cereal/types/vector.hpp>
#include <cereal/types/array.hpp>
#include <cereal/types/string.hpp>
#include <cereal/types/complex.hpp>

#include <ODatum/IntrospectionArchive.h>
#include <ODatum/Hash.h>


 namespace av {
 namespace msg {
 namespace plan_ctrl {


MapParams::MapParams() = default;

MapParams::MapParams(
    std::string mapFilePath_arg ,
    std::string mapBuilder_arg ,
    double minRadius_m_arg ,
    double baseLineResolution_m_arg ,
    double crosswalkExtentX_m_arg ,
    double crosswalkExtentY_m_arg 
    ):
    mapFilePath { std::move(mapFilePath_arg) } // NOLINT
      ,
    mapBuilder { std::move(mapBuilder_arg) } // NOLINT
      ,
    minRadius_m { std::move(minRadius_m_arg) } // NOLINT
      ,
    baseLineResolution_m { std::move(baseLineResolution_m_arg) } // NOLINT
      ,
    crosswalkExtentX_m { std::move(crosswalkExtentX_m_arg) } // NOLINT
      ,
    crosswalkExtentY_m { std::move(crosswalkExtentY_m_arg) } // NOLINT
      
    {
}

MapParams::MapParams(const MapParams&) = default;

// NOLINTNEXTLINE
MapParams::MapParams(MapParams&&) = default;

MapParams& MapParams::operator=(const MapParams&) = default;

// NOLINTNEXTLINE
MapParams& MapParams::operator=(MapParams&&) = default;

void MapParams::toCsv(std::ostream& out) const
{
    out.precision(16);
    out << std::boolalpha;
    out << "\"";
        out << mapFilePath;
        out << "\"";out << ",";
    out << "\"";
        out << mapBuilder;
        out << "\"";out << ",";
    out << minRadius_m;out << ",";
    out << baseLineResolution_m;out << ",";
    out << crosswalkExtentX_m;out << ",";
    out << crosswalkExtentY_m;
    
}

std::stringstream MapParams::toStringstream() const
{
    std::stringstream out;
    toCsv(out);
    return out;
}

template<class Archive>
    void MapParams::serialize(Archive& ar, [[maybe_unused]] unsigned int version)
    {

        ar& CEREAL_NVP(mapFilePath);
        ar& CEREAL_NVP(mapBuilder);
        ar& CEREAL_NVP(minRadius_m);
        ar& CEREAL_NVP(baseLineResolution_m);
        ar& CEREAL_NVP(crosswalkExtentX_m);
        ar& CEREAL_NVP(crosswalkExtentY_m);
    }


template void MapParams::serialize(cereal::BinaryInputArchive& ar, unsigned int version);
template void MapParams::serialize(cereal::BinaryOutputArchive& ar, unsigned int version);
template void MapParams::serialize(cereal::IntrospectionArchive& ar, unsigned int version);
template void MapParams::serialize(cereal::JSONInputArchive& ar, unsigned int version);
template void MapParams::serialize(cereal::JSONOutputArchive& ar, unsigned int version);
template void MapParams::serialize(cereal::PortableBinaryInputArchive& ar, unsigned int version);
template void MapParams::serialize(cereal::PortableBinaryOutputArchive& ar, unsigned int version);

std::ostream& operator<< (std::ostream& os, const MapParams& mapparams)
{
    mapparams.toCsv(os);
    return os;
}

bool operator== (const MapParams &lhs, const MapParams &rhs)
{
    return (  lhs.mapFilePath== rhs.mapFilePath &&
     lhs.mapBuilder== rhs.mapBuilder &&
     lhs.minRadius_m== rhs.minRadius_m &&
     lhs.baseLineResolution_m== rhs.baseLineResolution_m &&
     lhs.crosswalkExtentX_m== rhs.crosswalkExtentX_m &&
     lhs.crosswalkExtentY_m== rhs.crosswalkExtentY_m 
    );
}

std::size_t MapParams::hash() const noexcept
{
    std::size_t result = 17;
    result = result * 31 + std::hash<std::string>()(mapFilePath);
    result = result * 31 + std::hash<std::string>()(mapBuilder);
    result = result * 31 + std::hash<double>()(minRadius_m);
    result = result * 31 + std::hash<double>()(baseLineResolution_m);
    result = result * 31 + std::hash<double>()(crosswalkExtentX_m);
    result = result * 31 + std::hash<double>()(crosswalkExtentY_m);
    return result;
}

 } // namespace av
 } // namespace msg
 } // namespace plan_ctrl


CEREAL_BIND_TO_ARCHIVES(av::msg::plan_ctrl::MapParams)

 namespace av {
 namespace msg {
 namespace plan_ctrl {


AvtestTrack::AvtestTrack() = default;

AvtestTrack::AvtestTrack(
    bool active_arg ,
    double x_arg ,
    double y_arg ,
    double heading_arg ,
    double width_arg ,
    double length_arg ,
    uint64_t id_arg ,
    double vx_arg ,
    double vy_arg ,
    double acceleration_arg ,
    bool real_arg ,
    double radius_arg 
    ):
    active { std::move(active_arg) } // NOLINT
      ,
    x { std::move(x_arg) } // NOLINT
      ,
    y { std::move(y_arg) } // NOLINT
      ,
    heading { std::move(heading_arg) } // NOLINT
      ,
    width { std::move(width_arg) } // NOLINT
      ,
    length { std::move(length_arg) } // NOLINT
      ,
    id { std::move(id_arg) } // NOLINT
      ,
    vx { std::move(vx_arg) } // NOLINT
      ,
    vy { std::move(vy_arg) } // NOLINT
      ,
    acceleration { std::move(acceleration_arg) } // NOLINT
      ,
    real { std::move(real_arg) } // NOLINT
      ,
    radius { std::move(radius_arg) } // NOLINT
      
    {
}

AvtestTrack::AvtestTrack(const AvtestTrack&) = default;

// NOLINTNEXTLINE
AvtestTrack::AvtestTrack(AvtestTrack&&) = default;

AvtestTrack& AvtestTrack::operator=(const AvtestTrack&) = default;

// NOLINTNEXTLINE
AvtestTrack& AvtestTrack::operator=(AvtestTrack&&) = default;

void AvtestTrack::toCsv(std::ostream& out) const
{
    out.precision(16);
    out << std::boolalpha;
    out << active;out << ",";
    out << x;out << ",";
    out << y;out << ",";
    out << heading;out << ",";
    out << width;out << ",";
    out << length;out << ",";
    out << id;out << ",";
    out << vx;out << ",";
    out << vy;out << ",";
    out << acceleration;out << ",";
    out << real;out << ",";
    out << radius;
    
}

std::stringstream AvtestTrack::toStringstream() const
{
    std::stringstream out;
    toCsv(out);
    return out;
}

template<class Archive>
    void AvtestTrack::serialize(Archive& ar, [[maybe_unused]] unsigned int version)
    {

        ar& CEREAL_NVP(active);
        ar& CEREAL_NVP(x);
        ar& CEREAL_NVP(y);
        ar& CEREAL_NVP(heading);
        ar& CEREAL_NVP(width);
        ar& CEREAL_NVP(length);
        ar& CEREAL_NVP(id);
        ar& CEREAL_NVP(vx);
        ar& CEREAL_NVP(vy);
        ar& CEREAL_NVP(acceleration);
        ar& CEREAL_NVP(real);
        ar& CEREAL_NVP(radius);
    }


template void AvtestTrack::serialize(cereal::BinaryInputArchive& ar, unsigned int version);
template void AvtestTrack::serialize(cereal::BinaryOutputArchive& ar, unsigned int version);
template void AvtestTrack::serialize(cereal::IntrospectionArchive& ar, unsigned int version);
template void AvtestTrack::serialize(cereal::JSONInputArchive& ar, unsigned int version);
template void AvtestTrack::serialize(cereal::JSONOutputArchive& ar, unsigned int version);
template void AvtestTrack::serialize(cereal::PortableBinaryInputArchive& ar, unsigned int version);
template void AvtestTrack::serialize(cereal::PortableBinaryOutputArchive& ar, unsigned int version);

std::ostream& operator<< (std::ostream& os, const AvtestTrack& avtesttrack)
{
    avtesttrack.toCsv(os);
    return os;
}

bool operator== (const AvtestTrack &lhs, const AvtestTrack &rhs)
{
    return (  lhs.active== rhs.active &&
     lhs.x== rhs.x &&
     lhs.y== rhs.y &&
     lhs.heading== rhs.heading &&
     lhs.width== rhs.width &&
     lhs.length== rhs.length &&
     lhs.id== rhs.id &&
     lhs.vx== rhs.vx &&
     lhs.vy== rhs.vy &&
     lhs.acceleration== rhs.acceleration &&
     lhs.real== rhs.real &&
     lhs.radius== rhs.radius 
    );
}

std::size_t AvtestTrack::hash() const noexcept
{
    std::size_t result = 17;
    result = result * 31 + std::hash<bool>()(active);
    result = result * 31 + std::hash<double>()(x);
    result = result * 31 + std::hash<double>()(y);
    result = result * 31 + std::hash<double>()(heading);
    result = result * 31 + std::hash<double>()(width);
    result = result * 31 + std::hash<double>()(length);
    result = result * 31 + std::hash<uint64_t>()(id);
    result = result * 31 + std::hash<double>()(vx);
    result = result * 31 + std::hash<double>()(vy);
    result = result * 31 + std::hash<double>()(acceleration);
    result = result * 31 + std::hash<bool>()(real);
    result = result * 31 + std::hash<double>()(radius);
    return result;
}

 } // namespace av
 } // namespace msg
 } // namespace plan_ctrl


CEREAL_BIND_TO_ARCHIVES(av::msg::plan_ctrl::AvtestTrack)

 namespace av {
 namespace msg {
 namespace plan_ctrl {


AvtestConvexHull::AvtestConvexHull() = default;

AvtestConvexHull::AvtestConvexHull(
    std::vector<av::geometry::Vector3d> hull_arg ,
    av::geometry::Vector3d center_arg ,
    double radius_arg ,
    uint64_t id_arg ,
    double vx_arg ,
    double vy_arg 
    ):
    hull { std::move(hull_arg) } // NOLINT
      ,
    center { std::move(center_arg) } // NOLINT
      ,
    radius { std::move(radius_arg) } // NOLINT
      ,
    id { std::move(id_arg) } // NOLINT
      ,
    vx { std::move(vx_arg) } // NOLINT
      ,
    vy { std::move(vy_arg) } // NOLINT
      
    {
}

AvtestConvexHull::AvtestConvexHull(const AvtestConvexHull&) = default;

// NOLINTNEXTLINE
AvtestConvexHull::AvtestConvexHull(AvtestConvexHull&&) = default;

AvtestConvexHull& AvtestConvexHull::operator=(const AvtestConvexHull&) = default;

// NOLINTNEXTLINE
AvtestConvexHull& AvtestConvexHull::operator=(AvtestConvexHull&&) = default;

void AvtestConvexHull::toCsv(std::ostream& out) const
{
    out.precision(16);
    out << std::boolalpha;
    // Enclose sequence in double quotations and curly brackets to keep CSV line length uniform
        
        out << "\"{{";
        
        if (!hull.empty())
        {
            std::copy(hull.begin(), hull.end() - 1, std::ostream_iterator<av::geometry::Vector3d>(out, "},{"));
            std::copy(hull.end()-1, hull.end(), std::ostream_iterator<av::geometry::Vector3d>(out, ""));
        }
        out << "}}\"";out << ",";
    out << center;out << ",";
    out << radius;out << ",";
    out << id;out << ",";
    out << vx;out << ",";
    out << vy;
    
}

std::stringstream AvtestConvexHull::toStringstream() const
{
    std::stringstream out;
    toCsv(out);
    return out;
}

template<class Archive>
    void AvtestConvexHull::serialize(Archive& ar, [[maybe_unused]] unsigned int version)
    {

        ar& CEREAL_NVP(hull);
        ar& CEREAL_NVP(center);
        ar& CEREAL_NVP(radius);
        ar& CEREAL_NVP(id);
        ar& CEREAL_NVP(vx);
        ar& CEREAL_NVP(vy);
    }


template void AvtestConvexHull::serialize(cereal::BinaryInputArchive& ar, unsigned int version);
template void AvtestConvexHull::serialize(cereal::BinaryOutputArchive& ar, unsigned int version);
template void AvtestConvexHull::serialize(cereal::IntrospectionArchive& ar, unsigned int version);
template void AvtestConvexHull::serialize(cereal::JSONInputArchive& ar, unsigned int version);
template void AvtestConvexHull::serialize(cereal::JSONOutputArchive& ar, unsigned int version);
template void AvtestConvexHull::serialize(cereal::PortableBinaryInputArchive& ar, unsigned int version);
template void AvtestConvexHull::serialize(cereal::PortableBinaryOutputArchive& ar, unsigned int version);

std::ostream& operator<< (std::ostream& os, const AvtestConvexHull& avtestconvexhull)
{
    avtestconvexhull.toCsv(os);
    return os;
}

bool operator== (const AvtestConvexHull &lhs, const AvtestConvexHull &rhs)
{
    return (  lhs.hull== rhs.hull &&
     lhs.center== rhs.center &&
     lhs.radius== rhs.radius &&
     lhs.id== rhs.id &&
     lhs.vx== rhs.vx &&
     lhs.vy== rhs.vy 
    );
}

std::size_t AvtestConvexHull::hash() const noexcept
{
    std::size_t result = 17;
    result = result * 31 + hashVector(hull);
    result = result * 31 + std::hash<av::geometry::Vector3d>()(center);
    result = result * 31 + std::hash<double>()(radius);
    result = result * 31 + std::hash<uint64_t>()(id);
    result = result * 31 + std::hash<double>()(vx);
    result = result * 31 + std::hash<double>()(vy);
    return result;
}

 } // namespace av
 } // namespace msg
 } // namespace plan_ctrl


CEREAL_BIND_TO_ARCHIVES(av::msg::plan_ctrl::AvtestConvexHull)

 namespace av {
 namespace msg {
 namespace plan_ctrl {


AvtestWorld::AvtestWorld() = default;

AvtestWorld::AvtestWorld(
    std::vector<AvtestTrack> vehicles_arg ,
    std::vector<AvtestTrack> pedestrians_arg ,
    std::vector<AvtestTrack> bicyclists_arg ,
    std::vector<AvtestConvexHull> genericObjects_arg ,
    std::vector<AvtestConvexHull> constructionZones_arg 
    ):
    vehicles { std::move(vehicles_arg) } // NOLINT
      ,
    pedestrians { std::move(pedestrians_arg) } // NOLINT
      ,
    bicyclists { std::move(bicyclists_arg) } // NOLINT
      ,
    genericObjects { std::move(genericObjects_arg) } // NOLINT
      ,
    constructionZones { std::move(constructionZones_arg) } // NOLINT
      
    {
}

AvtestWorld::AvtestWorld(const AvtestWorld&) = default;

// NOLINTNEXTLINE
AvtestWorld::AvtestWorld(AvtestWorld&&) = default;

AvtestWorld& AvtestWorld::operator=(const AvtestWorld&) = default;

// NOLINTNEXTLINE
AvtestWorld& AvtestWorld::operator=(AvtestWorld&&) = default;

void AvtestWorld::toCsv(std::ostream& out) const
{
    out.precision(16);
    out << std::boolalpha;
    // Enclose sequence in double quotations and curly brackets to keep CSV line length uniform
        
        out << "\"{{";
        
        if (!vehicles.empty())
        {
            std::copy(vehicles.begin(), vehicles.end() - 1, std::ostream_iterator<AvtestTrack>(out, "},{"));
            std::copy(vehicles.end()-1, vehicles.end(), std::ostream_iterator<AvtestTrack>(out, ""));
        }
        out << "}}\"";out << ",";
    // Enclose sequence in double quotations and curly brackets to keep CSV line length uniform
        
        out << "\"{{";
        
        if (!pedestrians.empty())
        {
            std::copy(pedestrians.begin(), pedestrians.end() - 1, std::ostream_iterator<AvtestTrack>(out, "},{"));
            std::copy(pedestrians.end()-1, pedestrians.end(), std::ostream_iterator<AvtestTrack>(out, ""));
        }
        out << "}}\"";out << ",";
    // Enclose sequence in double quotations and curly brackets to keep CSV line length uniform
        
        out << "\"{{";
        
        if (!bicyclists.empty())
        {
            std::copy(bicyclists.begin(), bicyclists.end() - 1, std::ostream_iterator<AvtestTrack>(out, "},{"));
            std::copy(bicyclists.end()-1, bicyclists.end(), std::ostream_iterator<AvtestTrack>(out, ""));
        }
        out << "}}\"";out << ",";
    // Enclose sequence in double quotations and curly brackets to keep CSV line length uniform
        
        out << "\"{{";
        
        if (!genericObjects.empty())
        {
            std::copy(genericObjects.begin(), genericObjects.end() - 1, std::ostream_iterator<AvtestConvexHull>(out, "},{"));
            std::copy(genericObjects.end()-1, genericObjects.end(), std::ostream_iterator<AvtestConvexHull>(out, ""));
        }
        out << "}}\"";out << ",";
    // Enclose sequence in double quotations and curly brackets to keep CSV line length uniform
        
        out << "\"{{";
        
        if (!constructionZones.empty())
        {
            std::copy(constructionZones.begin(), constructionZones.end() - 1, std::ostream_iterator<AvtestConvexHull>(out, "},{"));
            std::copy(constructionZones.end()-1, constructionZones.end(), std::ostream_iterator<AvtestConvexHull>(out, ""));
        }
        out << "}}\"";
    
}

std::stringstream AvtestWorld::toStringstream() const
{
    std::stringstream out;
    toCsv(out);
    return out;
}

template<class Archive>
    void AvtestWorld::serialize(Archive& ar, [[maybe_unused]] unsigned int version)
    {

        ar& CEREAL_NVP(vehicles);
        ar& CEREAL_NVP(pedestrians);
        ar& CEREAL_NVP(bicyclists);
        ar& CEREAL_NVP(genericObjects);
        ar& CEREAL_NVP(constructionZones);
    }


template void AvtestWorld::serialize(cereal::BinaryInputArchive& ar, unsigned int version);
template void AvtestWorld::serialize(cereal::BinaryOutputArchive& ar, unsigned int version);
template void AvtestWorld::serialize(cereal::IntrospectionArchive& ar, unsigned int version);
template void AvtestWorld::serialize(cereal::JSONInputArchive& ar, unsigned int version);
template void AvtestWorld::serialize(cereal::JSONOutputArchive& ar, unsigned int version);
template void AvtestWorld::serialize(cereal::PortableBinaryInputArchive& ar, unsigned int version);
template void AvtestWorld::serialize(cereal::PortableBinaryOutputArchive& ar, unsigned int version);

std::ostream& operator<< (std::ostream& os, const AvtestWorld& avtestworld)
{
    avtestworld.toCsv(os);
    return os;
}

bool operator== (const AvtestWorld &lhs, const AvtestWorld &rhs)
{
    return (  lhs.vehicles== rhs.vehicles &&
     lhs.pedestrians== rhs.pedestrians &&
     lhs.bicyclists== rhs.bicyclists &&
     lhs.genericObjects== rhs.genericObjects &&
     lhs.constructionZones== rhs.constructionZones 
    );
}

std::size_t AvtestWorld::hash() const noexcept
{
    std::size_t result = 17;
    result = result * 31 + hashVector(vehicles);
    result = result * 31 + hashVector(pedestrians);
    result = result * 31 + hashVector(bicyclists);
    result = result * 31 + hashVector(genericObjects);
    result = result * 31 + hashVector(constructionZones);
    return result;
}

 } // namespace av
 } // namespace msg
 } // namespace plan_ctrl


CEREAL_BIND_TO_ARCHIVES(av::msg::plan_ctrl::AvtestWorld)

 namespace av {
 namespace msg {
 namespace plan_ctrl {


AvtestDynamicWorld::AvtestDynamicWorld() = default;

AvtestDynamicWorld::AvtestDynamicWorld(
    std::vector<AvtestWorld> world_arg ,
    std::vector<int64_t> time_us_arg 
    ):
    world { std::move(world_arg) } // NOLINT
      ,
    time_us { std::move(time_us_arg) } // NOLINT
      
    {
}

AvtestDynamicWorld::AvtestDynamicWorld(const AvtestDynamicWorld&) = default;

// NOLINTNEXTLINE
AvtestDynamicWorld::AvtestDynamicWorld(AvtestDynamicWorld&&) = default;

AvtestDynamicWorld& AvtestDynamicWorld::operator=(const AvtestDynamicWorld&) = default;

// NOLINTNEXTLINE
AvtestDynamicWorld& AvtestDynamicWorld::operator=(AvtestDynamicWorld&&) = default;

void AvtestDynamicWorld::toCsv(std::ostream& out) const
{
    out.precision(16);
    out << std::boolalpha;
    // Enclose sequence in double quotations and curly brackets to keep CSV line length uniform
        
        out << "\"{{";
        
        if (!world.empty())
        {
            std::copy(world.begin(), world.end() - 1, std::ostream_iterator<AvtestWorld>(out, "},{"));
            std::copy(world.end()-1, world.end(), std::ostream_iterator<AvtestWorld>(out, ""));
        }
        out << "}}\"";out << ",";
    // Enclose sequence in double quotations and curly brackets to keep CSV line length uniform
        
        out << "\"{{";
        
        if (!time_us.empty())
        {
            std::copy(time_us.begin(), time_us.end() - 1, std::ostream_iterator<int64_t>(out, "},{"));
            std::copy(time_us.end()-1, time_us.end(), std::ostream_iterator<int64_t>(out, ""));
        }
        out << "}}\"";
    
}

std::stringstream AvtestDynamicWorld::toStringstream() const
{
    std::stringstream out;
    toCsv(out);
    return out;
}

template<class Archive>
    void AvtestDynamicWorld::serialize(Archive& ar, [[maybe_unused]] unsigned int version)
    {

        ar& CEREAL_NVP(world);
        ar& CEREAL_NVP(time_us);
    }


template void AvtestDynamicWorld::serialize(cereal::BinaryInputArchive& ar, unsigned int version);
template void AvtestDynamicWorld::serialize(cereal::BinaryOutputArchive& ar, unsigned int version);
template void AvtestDynamicWorld::serialize(cereal::IntrospectionArchive& ar, unsigned int version);
template void AvtestDynamicWorld::serialize(cereal::JSONInputArchive& ar, unsigned int version);
template void AvtestDynamicWorld::serialize(cereal::JSONOutputArchive& ar, unsigned int version);
template void AvtestDynamicWorld::serialize(cereal::PortableBinaryInputArchive& ar, unsigned int version);
template void AvtestDynamicWorld::serialize(cereal::PortableBinaryOutputArchive& ar, unsigned int version);

std::ostream& operator<< (std::ostream& os, const AvtestDynamicWorld& avtestdynamicworld)
{
    avtestdynamicworld.toCsv(os);
    return os;
}

bool operator== (const AvtestDynamicWorld &lhs, const AvtestDynamicWorld &rhs)
{
    return (  lhs.world== rhs.world &&
     lhs.time_us== rhs.time_us 
    );
}

std::size_t AvtestDynamicWorld::hash() const noexcept
{
    std::size_t result = 17;
    result = result * 31 + hashVector(world);
    result = result * 31 + hashVector(time_us);
    return result;
}

 } // namespace av
 } // namespace msg
 } // namespace plan_ctrl


CEREAL_BIND_TO_ARCHIVES(av::msg::plan_ctrl::AvtestDynamicWorld)

 namespace av {
 namespace msg {
 namespace plan_ctrl {


EgoStateExtended::EgoStateExtended() = default;

EgoStateExtended::EgoStateExtended(
    double rear_axle_x_arg ,
    double rear_axle_y_arg ,
    double rear_axle_heading_arg ,
    double vx_arg ,
    double vy_arg ,
    double ax_arg ,
    double ay_arg ,
    double roadWheelAngle_arg ,
    double angularVelocity_arg ,
    double angularAcceleration_arg ,
    bool isAuto_arg ,
    int64_t time_stamp_arg 
    ):
    rear_axle_x { std::move(rear_axle_x_arg) } // NOLINT
      ,
    rear_axle_y { std::move(rear_axle_y_arg) } // NOLINT
      ,
    rear_axle_heading { std::move(rear_axle_heading_arg) } // NOLINT
      ,
    vx { std::move(vx_arg) } // NOLINT
      ,
    vy { std::move(vy_arg) } // NOLINT
      ,
    ax { std::move(ax_arg) } // NOLINT
      ,
    ay { std::move(ay_arg) } // NOLINT
      ,
    roadWheelAngle { std::move(roadWheelAngle_arg) } // NOLINT
      ,
    angularVelocity { std::move(angularVelocity_arg) } // NOLINT
      ,
    angularAcceleration { std::move(angularAcceleration_arg) } // NOLINT
      ,
    isAuto { std::move(isAuto_arg) } // NOLINT
      ,
    time_stamp { std::move(time_stamp_arg) } // NOLINT
      
    {
}

EgoStateExtended::EgoStateExtended(const EgoStateExtended&) = default;

// NOLINTNEXTLINE
EgoStateExtended::EgoStateExtended(EgoStateExtended&&) = default;

EgoStateExtended& EgoStateExtended::operator=(const EgoStateExtended&) = default;

// NOLINTNEXTLINE
EgoStateExtended& EgoStateExtended::operator=(EgoStateExtended&&) = default;

void EgoStateExtended::toCsv(std::ostream& out) const
{
    out.precision(16);
    out << std::boolalpha;
    out << rear_axle_x;out << ",";
    out << rear_axle_y;out << ",";
    out << rear_axle_heading;out << ",";
    out << vx;out << ",";
    out << vy;out << ",";
    out << ax;out << ",";
    out << ay;out << ",";
    out << roadWheelAngle;out << ",";
    out << angularVelocity;out << ",";
    out << angularAcceleration;out << ",";
    out << isAuto;out << ",";
    out << time_stamp;
    
}

std::stringstream EgoStateExtended::toStringstream() const
{
    std::stringstream out;
    toCsv(out);
    return out;
}

template<class Archive>
    void EgoStateExtended::serialize(Archive& ar, [[maybe_unused]] unsigned int version)
    {

        ar& CEREAL_NVP(rear_axle_x);
        ar& CEREAL_NVP(rear_axle_y);
        ar& CEREAL_NVP(rear_axle_heading);
        ar& CEREAL_NVP(vx);
        ar& CEREAL_NVP(vy);
        ar& CEREAL_NVP(ax);
        ar& CEREAL_NVP(ay);
        ar& CEREAL_NVP(roadWheelAngle);
        ar& CEREAL_NVP(angularVelocity);
        ar& CEREAL_NVP(angularAcceleration);
        ar& CEREAL_NVP(isAuto);
        ar& CEREAL_NVP(time_stamp);
    }


template void EgoStateExtended::serialize(cereal::BinaryInputArchive& ar, unsigned int version);
template void EgoStateExtended::serialize(cereal::BinaryOutputArchive& ar, unsigned int version);
template void EgoStateExtended::serialize(cereal::IntrospectionArchive& ar, unsigned int version);
template void EgoStateExtended::serialize(cereal::JSONInputArchive& ar, unsigned int version);
template void EgoStateExtended::serialize(cereal::JSONOutputArchive& ar, unsigned int version);
template void EgoStateExtended::serialize(cereal::PortableBinaryInputArchive& ar, unsigned int version);
template void EgoStateExtended::serialize(cereal::PortableBinaryOutputArchive& ar, unsigned int version);

std::ostream& operator<< (std::ostream& os, const EgoStateExtended& egostateextended)
{
    egostateextended.toCsv(os);
    return os;
}

bool operator== (const EgoStateExtended &lhs, const EgoStateExtended &rhs)
{
    return (  lhs.rear_axle_x== rhs.rear_axle_x &&
     lhs.rear_axle_y== rhs.rear_axle_y &&
     lhs.rear_axle_heading== rhs.rear_axle_heading &&
     lhs.vx== rhs.vx &&
     lhs.vy== rhs.vy &&
     lhs.ax== rhs.ax &&
     lhs.ay== rhs.ay &&
     lhs.roadWheelAngle== rhs.roadWheelAngle &&
     lhs.angularVelocity== rhs.angularVelocity &&
     lhs.angularAcceleration== rhs.angularAcceleration &&
     lhs.isAuto== rhs.isAuto &&
     lhs.time_stamp== rhs.time_stamp 
    );
}

std::size_t EgoStateExtended::hash() const noexcept
{
    std::size_t result = 17;
    result = result * 31 + std::hash<double>()(rear_axle_x);
    result = result * 31 + std::hash<double>()(rear_axle_y);
    result = result * 31 + std::hash<double>()(rear_axle_heading);
    result = result * 31 + std::hash<double>()(vx);
    result = result * 31 + std::hash<double>()(vy);
    result = result * 31 + std::hash<double>()(ax);
    result = result * 31 + std::hash<double>()(ay);
    result = result * 31 + std::hash<double>()(roadWheelAngle);
    result = result * 31 + std::hash<double>()(angularVelocity);
    result = result * 31 + std::hash<double>()(angularAcceleration);
    result = result * 31 + std::hash<bool>()(isAuto);
    result = result * 31 + std::hash<int64_t>()(time_stamp);
    return result;
}

 } // namespace av
 } // namespace msg
 } // namespace plan_ctrl


CEREAL_BIND_TO_ARCHIVES(av::msg::plan_ctrl::EgoStateExtended)




 namespace av {
 namespace msg {
 namespace plan_ctrl {


HomotopyExtractorDebug::HomotopyExtractorDebug() = default;

HomotopyExtractorDebug::HomotopyExtractorDebug(
    MapParams mapParams_arg ,
    EgoStateExtended x0_arg ,
    RoutePlanWithStaticConstraints routePlan_arg ,
    AvtestDynamicWorld dynamicWorld_arg ,
    std::vector<NamedVectorOfDouble> debugVariables_arg ,
    std::vector<WorldTrackDecisions> foundOptions_arg ,
    route_obstacles obstacles_arg 
    ):
    mapParams { std::move(mapParams_arg) } // NOLINT
      ,x0 { std::move(x0_arg) } // NOLINT
      ,routePlan { std::move(routePlan_arg) } // NOLINT
      ,dynamicWorld { std::move(dynamicWorld_arg) } // NOLINT
      ,debugVariables { std::move(debugVariables_arg) } // NOLINT
      ,foundOptions { std::move(foundOptions_arg) } // NOLINT
      ,obstacles { std::move(obstacles_arg) } // NOLINT
      {
}

HomotopyExtractorDebug::HomotopyExtractorDebug(const HomotopyExtractorDebug&) = default;

// NOLINTNEXTLINE
HomotopyExtractorDebug::HomotopyExtractorDebug(HomotopyExtractorDebug&&) = default;

HomotopyExtractorDebug& HomotopyExtractorDebug::operator=(const HomotopyExtractorDebug&) = default;

// NOLINTNEXTLINE
HomotopyExtractorDebug& HomotopyExtractorDebug::operator=(HomotopyExtractorDebug&&) = default;

void HomotopyExtractorDebug::toCsv(std::ostream& out) const
{
    out.precision(16);
    out << std::boolalpha;
    out << mapParams;out << ",";
    out << x0;out << ",";
    out << routePlan;out << ",";
    out << dynamicWorld;out << ",";
    // Enclose sequence in double quotations and curly brackets to keep CSV line length uniform
        
        out << "\"{{";
        
        if (!debugVariables.empty())
        {
            std::copy(debugVariables.begin(), debugVariables.end() - 1, std::ostream_iterator<NamedVectorOfDouble>(out, "},{"));
            std::copy(debugVariables.end()-1, debugVariables.end(), std::ostream_iterator<NamedVectorOfDouble>(out, ""));
        }
        out << "}}\"";out << ",";
    // Enclose sequence in double quotations and curly brackets to keep CSV line length uniform
        
        out << "\"{{";
        
        if (!foundOptions.empty())
        {
            std::copy(foundOptions.begin(), foundOptions.end() - 1, std::ostream_iterator<WorldTrackDecisions>(out, "},{"));
            std::copy(foundOptions.end()-1, foundOptions.end(), std::ostream_iterator<WorldTrackDecisions>(out, ""));
        }
        out << "}}\"";out << ",";
    out << obstacles;
    
}

std::stringstream HomotopyExtractorDebug::toStringstream() const
{
    std::stringstream out;
    toCsv(out);
    return out;
}

template<class Archive>
    void HomotopyExtractorDebug::serialize(Archive& ar, unsigned int version)
{
	if (version >= 1)
	{
		ar& CEREAL_NVP(mapParams);
		ar& CEREAL_NVP(x0);
		ar& CEREAL_NVP(routePlan);
		ar& CEREAL_NVP(dynamicWorld);
		ar& CEREAL_NVP(debugVariables);
		ar& CEREAL_NVP(foundOptions);
	}
	if (version >= 2)
	{
		ar& CEREAL_NVP(obstacles);
	}
}


template void HomotopyExtractorDebug::serialize(cereal::BinaryInputArchive& ar, unsigned int version);
template void HomotopyExtractorDebug::serialize(cereal::BinaryOutputArchive& ar, unsigned int version);
template void HomotopyExtractorDebug::serialize(cereal::IntrospectionArchive& ar, unsigned int version);
template void HomotopyExtractorDebug::serialize(cereal::JSONInputArchive& ar, unsigned int version);
template void HomotopyExtractorDebug::serialize(cereal::JSONOutputArchive& ar, unsigned int version);
template void HomotopyExtractorDebug::serialize(cereal::PortableBinaryInputArchive& ar, unsigned int version);
template void HomotopyExtractorDebug::serialize(cereal::PortableBinaryOutputArchive& ar, unsigned int version);

std::ostream& operator<< (std::ostream& os, const HomotopyExtractorDebug& homotopyextractordebug)
{
    homotopyextractordebug.toCsv(os);
    return os;
}

bool operator== (const HomotopyExtractorDebug &lhs, const HomotopyExtractorDebug &rhs)
{
    return (  lhs.mapParams== rhs.mapParams &&
     lhs.x0== rhs.x0 &&
     lhs.routePlan== rhs.routePlan &&
     lhs.dynamicWorld== rhs.dynamicWorld &&
     lhs.debugVariables== rhs.debugVariables &&
     lhs.foundOptions== rhs.foundOptions &&
     lhs.obstacles== rhs.obstacles 
    );
}

std::size_t HomotopyExtractorDebug::hash() const noexcept
{
    std::size_t result = 17;
    result = result * 31 + std::hash<MapParams>()(mapParams);
    result = result * 31 + std::hash<EgoStateExtended>()(x0);
    result = result * 31 + std::hash<RoutePlanWithStaticConstraints>()(routePlan);
    result = result * 31 + std::hash<AvtestDynamicWorld>()(dynamicWorld);
    result = result * 31 + hashVector(debugVariables);
    result = result * 31 + hashVector(foundOptions);
    result = result * 31 + std::hash<route_obstacles>()(obstacles);
    return result;
}

 } // namespace av
 } // namespace msg
 } // namespace plan_ctrl


CEREAL_BIND_TO_ARCHIVES(av::msg::plan_ctrl::HomotopyExtractorDebug)

