#include <interfaces/HomotopyOverPath/HomotopyOverPath.h>

#include <iterator>
#include <type_traits>

#include <cereal/cereal.hpp>
#include <cereal/archives/json.hpp>
#include <cereal/archives/binary.hpp>

#include <cereal/types/map.hpp>
#include <cereal/types/vector.hpp>
#include <cereal/types/array.hpp>
#include <cereal/types/string.hpp>
#include <cereal/types/complex.hpp>

#include <ODatum/IntrospectionArchive.h>
#include <ODatum/Hash.h>


 namespace av {
 namespace msg {
 namespace plan_ctrl {


DynamicHardAndSoftConstraints::DynamicHardAndSoftConstraints() = default;

DynamicHardAndSoftConstraints::DynamicHardAndSoftConstraints(
    std::vector<HardAndSoftLinearConstraints> constraints_arg ,
    std::vector<double> time_steps_arg 
    ):
    constraints { std::move(constraints_arg) } // NOLINT
      ,
    time_steps { std::move(time_steps_arg) } // NOLINT
      
    {
}

DynamicHardAndSoftConstraints::DynamicHardAndSoftConstraints(const DynamicHardAndSoftConstraints&) = default;

// NOLINTNEXTLINE
DynamicHardAndSoftConstraints::DynamicHardAndSoftConstraints(DynamicHardAndSoftConstraints&&) = default;

DynamicHardAndSoftConstraints& DynamicHardAndSoftConstraints::operator=(const DynamicHardAndSoftConstraints&) = default;

// NOLINTNEXTLINE
DynamicHardAndSoftConstraints& DynamicHardAndSoftConstraints::operator=(DynamicHardAndSoftConstraints&&) = default;

void DynamicHardAndSoftConstraints::toCsv(std::ostream& out) const
{
    out.precision(16);
    out << std::boolalpha;
    // Enclose sequence in double quotations and curly brackets to keep CSV line length uniform
        
        out << "\"{{";
        
        if (!constraints.empty())
        {
            std::copy(constraints.begin(), constraints.end() - 1, std::ostream_iterator<HardAndSoftLinearConstraints>(out, "},{"));
            std::copy(constraints.end()-1, constraints.end(), std::ostream_iterator<HardAndSoftLinearConstraints>(out, ""));
        }
        out << "}}\"";out << ",";
    // Enclose sequence in double quotations and curly brackets to keep CSV line length uniform
        
        out << "\"{{";
        
        if (!time_steps.empty())
        {
            std::copy(time_steps.begin(), time_steps.end() - 1, std::ostream_iterator<double>(out, "},{"));
            std::copy(time_steps.end()-1, time_steps.end(), std::ostream_iterator<double>(out, ""));
        }
        out << "}}\"";
    
}

std::stringstream DynamicHardAndSoftConstraints::toStringstream() const
{
    std::stringstream out;
    toCsv(out);
    return out;
}

template<class Archive>
    void DynamicHardAndSoftConstraints::serialize(Archive& ar, [[maybe_unused]] unsigned int version)
    {

        ar& CEREAL_NVP(constraints);
        ar& CEREAL_NVP(time_steps);
    }


template void DynamicHardAndSoftConstraints::serialize(cereal::BinaryInputArchive& ar, unsigned int version);
template void DynamicHardAndSoftConstraints::serialize(cereal::BinaryOutputArchive& ar, unsigned int version);
template void DynamicHardAndSoftConstraints::serialize(cereal::IntrospectionArchive& ar, unsigned int version);
template void DynamicHardAndSoftConstraints::serialize(cereal::JSONInputArchive& ar, unsigned int version);
template void DynamicHardAndSoftConstraints::serialize(cereal::JSONOutputArchive& ar, unsigned int version);
template void DynamicHardAndSoftConstraints::serialize(cereal::PortableBinaryInputArchive& ar, unsigned int version);
template void DynamicHardAndSoftConstraints::serialize(cereal::PortableBinaryOutputArchive& ar, unsigned int version);

std::ostream& operator<< (std::ostream& os, const DynamicHardAndSoftConstraints& dynamichardandsoftconstraints)
{
    dynamichardandsoftconstraints.toCsv(os);
    return os;
}

bool operator== (const DynamicHardAndSoftConstraints &lhs, const DynamicHardAndSoftConstraints &rhs)
{
    return (  lhs.constraints== rhs.constraints &&
     lhs.time_steps== rhs.time_steps 
    );
}

std::size_t DynamicHardAndSoftConstraints::hash() const noexcept
{
    std::size_t result = 17;
    result = result * 31 + hashVector(constraints);
    result = result * 31 + hashVector(time_steps);
    return result;
}

 } // namespace av
 } // namespace msg
 } // namespace plan_ctrl


CEREAL_BIND_TO_ARCHIVES(av::msg::plan_ctrl::DynamicHardAndSoftConstraints)




 namespace av {
 namespace msg {
 namespace plan_ctrl {


HomotopyOverPath::HomotopyOverPath() = default;

HomotopyOverPath::HomotopyOverPath(
    PlannerHeader header_arg ,
    Path pathWitness_arg ,
    std::vector<LinearContour> speedSquaredWitness_arg ,
    double currentS_m_arg ,
    HardAndSoftLinearConstraints spatialConstraints_arg ,
    HardAndSoftLinearConstraints speedSquaredConstraints_arg ,
    HardAndSoftQuadraticConstraints STConstraints_arg ,
    HardAndSoftLinearConstraints stationTimeConstraints_arg ,
    DynamicHardAndSoftConstraints dynamicSpatialConstraints_arg ,
    bool spatialConstraints_allowLaneChange_arg ,
    bool spatialConstraints_terminalReturnToOriginalLane_arg ,
    bool spatialConstraints_terminalStayInAdjacentLane_arg ,
    WorldTrackDecisions decisions_arg 
    ):
    header { std::move(header_arg) } // NOLINT
      ,pathWitness { std::move(pathWitness_arg) } // NOLINT
      ,speedSquaredWitness { std::move(speedSquaredWitness_arg) } // NOLINT
      ,currentS_m { std::move(currentS_m_arg) } // NOLINT
      ,spatialConstraints { std::move(spatialConstraints_arg) } // NOLINT
      ,speedSquaredConstraints { std::move(speedSquaredConstraints_arg) } // NOLINT
      ,STConstraints { std::move(STConstraints_arg) } // NOLINT
      ,stationTimeConstraints { std::move(stationTimeConstraints_arg) } // NOLINT
      ,dynamicSpatialConstraints { std::move(dynamicSpatialConstraints_arg) } // NOLINT
      ,spatialConstraints_allowLaneChange { std::move(spatialConstraints_allowLaneChange_arg) } // NOLINT
      ,spatialConstraints_terminalReturnToOriginalLane { std::move(spatialConstraints_terminalReturnToOriginalLane_arg) } // NOLINT
      ,spatialConstraints_terminalStayInAdjacentLane { std::move(spatialConstraints_terminalStayInAdjacentLane_arg) } // NOLINT
      ,decisions { std::move(decisions_arg) } // NOLINT
      {
}

HomotopyOverPath::HomotopyOverPath(const HomotopyOverPath&) = default;

// NOLINTNEXTLINE
HomotopyOverPath::HomotopyOverPath(HomotopyOverPath&&) = default;

HomotopyOverPath& HomotopyOverPath::operator=(const HomotopyOverPath&) = default;

// NOLINTNEXTLINE
HomotopyOverPath& HomotopyOverPath::operator=(HomotopyOverPath&&) = default;

void HomotopyOverPath::toCsv(std::ostream& out) const
{
    out.precision(16);
    out << std::boolalpha;
    out << header;out << ",";
    out << pathWitness;out << ",";
    // Enclose sequence in double quotations and curly brackets to keep CSV line length uniform
        
        out << "\"{{";
        
        if (!speedSquaredWitness.empty())
        {
            std::copy(speedSquaredWitness.begin(), speedSquaredWitness.end() - 1, std::ostream_iterator<LinearContour>(out, "},{"));
            std::copy(speedSquaredWitness.end()-1, speedSquaredWitness.end(), std::ostream_iterator<LinearContour>(out, ""));
        }
        out << "}}\"";out << ",";
    out << currentS_m;out << ",";
    out << spatialConstraints;out << ",";
    out << speedSquaredConstraints;out << ",";
    out << STConstraints;out << ",";
    out << stationTimeConstraints;out << ",";
    out << dynamicSpatialConstraints;out << ",";
    out << spatialConstraints_allowLaneChange;out << ",";
    out << spatialConstraints_terminalReturnToOriginalLane;out << ",";
    out << spatialConstraints_terminalStayInAdjacentLane;out << ",";
    out << decisions;
    
}

std::stringstream HomotopyOverPath::toStringstream() const
{
    std::stringstream out;
    toCsv(out);
    return out;
}

template<class Archive>
    void HomotopyOverPath::serialize(Archive& ar, [[maybe_unused]] unsigned int version)
    {

        ar& CEREAL_NVP(header);
        ar& CEREAL_NVP(pathWitness);
        ar& CEREAL_NVP(speedSquaredWitness);
        ar& CEREAL_NVP(currentS_m);
        ar& CEREAL_NVP(spatialConstraints);
        ar& CEREAL_NVP(speedSquaredConstraints);
        ar& CEREAL_NVP(STConstraints);
        ar& CEREAL_NVP(stationTimeConstraints);
        ar& CEREAL_NVP(dynamicSpatialConstraints);
        ar& CEREAL_NVP(spatialConstraints_allowLaneChange);
        ar& CEREAL_NVP(spatialConstraints_terminalReturnToOriginalLane);
        ar& CEREAL_NVP(spatialConstraints_terminalStayInAdjacentLane);
        ar& CEREAL_NVP(decisions);
    }


template void HomotopyOverPath::serialize(cereal::BinaryInputArchive& ar, unsigned int version);
template void HomotopyOverPath::serialize(cereal::BinaryOutputArchive& ar, unsigned int version);
template void HomotopyOverPath::serialize(cereal::IntrospectionArchive& ar, unsigned int version);
template void HomotopyOverPath::serialize(cereal::JSONInputArchive& ar, unsigned int version);
template void HomotopyOverPath::serialize(cereal::JSONOutputArchive& ar, unsigned int version);
template void HomotopyOverPath::serialize(cereal::PortableBinaryInputArchive& ar, unsigned int version);
template void HomotopyOverPath::serialize(cereal::PortableBinaryOutputArchive& ar, unsigned int version);

std::ostream& operator<< (std::ostream& os, const HomotopyOverPath& homotopyoverpath)
{
    homotopyoverpath.toCsv(os);
    return os;
}

bool operator== (const HomotopyOverPath &lhs, const HomotopyOverPath &rhs)
{
    return (  lhs.header== rhs.header &&
     lhs.pathWitness== rhs.pathWitness &&
     lhs.speedSquaredWitness== rhs.speedSquaredWitness &&
     lhs.currentS_m== rhs.currentS_m &&
     lhs.spatialConstraints== rhs.spatialConstraints &&
     lhs.speedSquaredConstraints== rhs.speedSquaredConstraints &&
     lhs.STConstraints== rhs.STConstraints &&
     lhs.stationTimeConstraints== rhs.stationTimeConstraints &&
     lhs.dynamicSpatialConstraints== rhs.dynamicSpatialConstraints &&
     lhs.spatialConstraints_allowLaneChange== rhs.spatialConstraints_allowLaneChange &&
     lhs.spatialConstraints_terminalReturnToOriginalLane== rhs.spatialConstraints_terminalReturnToOriginalLane &&
     lhs.spatialConstraints_terminalStayInAdjacentLane== rhs.spatialConstraints_terminalStayInAdjacentLane &&
     lhs.decisions== rhs.decisions 
    );
}

std::size_t HomotopyOverPath::hash() const noexcept
{
    std::size_t result = 17;
    result = result * 31 + std::hash<PlannerHeader>()(header);
    result = result * 31 + std::hash<Path>()(pathWitness);
    result = result * 31 + hashVector(speedSquaredWitness);
    result = result * 31 + std::hash<double>()(currentS_m);
    result = result * 31 + std::hash<HardAndSoftLinearConstraints>()(spatialConstraints);
    result = result * 31 + std::hash<HardAndSoftLinearConstraints>()(speedSquaredConstraints);
    result = result * 31 + std::hash<HardAndSoftQuadraticConstraints>()(STConstraints);
    result = result * 31 + std::hash<HardAndSoftLinearConstraints>()(stationTimeConstraints);
    result = result * 31 + std::hash<DynamicHardAndSoftConstraints>()(dynamicSpatialConstraints);
    result = result * 31 + std::hash<bool>()(spatialConstraints_allowLaneChange);
    result = result * 31 + std::hash<bool>()(spatialConstraints_terminalReturnToOriginalLane);
    result = result * 31 + std::hash<bool>()(spatialConstraints_terminalStayInAdjacentLane);
    result = result * 31 + std::hash<WorldTrackDecisions>()(decisions);
    return result;
}

 } // namespace av
 } // namespace msg
 } // namespace plan_ctrl


CEREAL_BIND_TO_ARCHIVES(av::msg::plan_ctrl::HomotopyOverPath)

