#include <interfaces/HomotopyOverPlan/HomotopyOverPlan.h>

#include <iterator>
#include <type_traits>

#include <cereal/cereal.hpp>
#include <cereal/archives/json.hpp>
#include <cereal/archives/binary.hpp>

#include <cereal/types/map.hpp>
#include <cereal/types/vector.hpp>
#include <cereal/types/array.hpp>
#include <cereal/types/string.hpp>
#include <cereal/types/complex.hpp>

#include <ODatum/IntrospectionArchive.h>
#include <ODatum/Hash.h>


 namespace av {
 namespace msg {
 namespace plan_ctrl {


HomotopyStationOverPlan::HomotopyStationOverPlan() = default;

HomotopyStationOverPlan::HomotopyStationOverPlan(
    HardAndSoftLinearConstraints anchorSTConstraint_arg ,
    HardAndSoftLinearConstraints combinedSTConstraint_arg ,
    std::vector<HardAndSoftLinearConstraints> adjacentPathsSTConstraints_arg 
    ):
    anchorSTConstraint { std::move(anchorSTConstraint_arg) } // NOLINT
      ,
    combinedSTConstraint { std::move(combinedSTConstraint_arg) } // NOLINT
      ,
    adjacentPathsSTConstraints { std::move(adjacentPathsSTConstraints_arg) } // NOLINT
      
    {
}

HomotopyStationOverPlan::HomotopyStationOverPlan(const HomotopyStationOverPlan&) = default;

// NOLINTNEXTLINE
HomotopyStationOverPlan::HomotopyStationOverPlan(HomotopyStationOverPlan&&) = default;

HomotopyStationOverPlan& HomotopyStationOverPlan::operator=(const HomotopyStationOverPlan&) = default;

// NOLINTNEXTLINE
HomotopyStationOverPlan& HomotopyStationOverPlan::operator=(HomotopyStationOverPlan&&) = default;

void HomotopyStationOverPlan::toCsv(std::ostream& out) const
{
    out.precision(16);
    out << std::boolalpha;
    out << anchorSTConstraint;out << ",";
    out << combinedSTConstraint;out << ",";
    // Enclose sequence in double quotations and curly brackets to keep CSV line length uniform
        
        out << "\"{{";
        
        if (!adjacentPathsSTConstraints.empty())
        {
            std::copy(adjacentPathsSTConstraints.begin(), adjacentPathsSTConstraints.end() - 1, std::ostream_iterator<HardAndSoftLinearConstraints>(out, "},{"));
            std::copy(adjacentPathsSTConstraints.end()-1, adjacentPathsSTConstraints.end(), std::ostream_iterator<HardAndSoftLinearConstraints>(out, ""));
        }
        out << "}}\"";
    
}

std::stringstream HomotopyStationOverPlan::toStringstream() const
{
    std::stringstream out;
    toCsv(out);
    return out;
}

template<class Archive>
    void HomotopyStationOverPlan::serialize(Archive& ar, [[maybe_unused]] unsigned int version)
    {

        ar& CEREAL_NVP(anchorSTConstraint);
        ar& CEREAL_NVP(combinedSTConstraint);
        ar& CEREAL_NVP(adjacentPathsSTConstraints);
    }


template void HomotopyStationOverPlan::serialize(cereal::BinaryInputArchive& ar, unsigned int version);
template void HomotopyStationOverPlan::serialize(cereal::BinaryOutputArchive& ar, unsigned int version);
template void HomotopyStationOverPlan::serialize(cereal::IntrospectionArchive& ar, unsigned int version);
template void HomotopyStationOverPlan::serialize(cereal::JSONInputArchive& ar, unsigned int version);
template void HomotopyStationOverPlan::serialize(cereal::JSONOutputArchive& ar, unsigned int version);
template void HomotopyStationOverPlan::serialize(cereal::PortableBinaryInputArchive& ar, unsigned int version);
template void HomotopyStationOverPlan::serialize(cereal::PortableBinaryOutputArchive& ar, unsigned int version);

std::ostream& operator<< (std::ostream& os, const HomotopyStationOverPlan& homotopystationoverplan)
{
    homotopystationoverplan.toCsv(os);
    return os;
}

bool operator== (const HomotopyStationOverPlan &lhs, const HomotopyStationOverPlan &rhs)
{
    return (  lhs.anchorSTConstraint== rhs.anchorSTConstraint &&
     lhs.combinedSTConstraint== rhs.combinedSTConstraint &&
     lhs.adjacentPathsSTConstraints== rhs.adjacentPathsSTConstraints 
    );
}

std::size_t HomotopyStationOverPlan::hash() const noexcept
{
    std::size_t result = 17;
    result = result * 31 + std::hash<HardAndSoftLinearConstraints>()(anchorSTConstraint);
    result = result * 31 + std::hash<HardAndSoftLinearConstraints>()(combinedSTConstraint);
    result = result * 31 + hashVector(adjacentPathsSTConstraints);
    return result;
}

 } // namespace av
 } // namespace msg
 } // namespace plan_ctrl


CEREAL_BIND_TO_ARCHIVES(av::msg::plan_ctrl::HomotopyStationOverPlan)




 namespace av {
 namespace msg {
 namespace plan_ctrl {


HomotopyOverPlan::HomotopyOverPlan() = default;

HomotopyOverPlan::HomotopyOverPlan(
    uint64_t homotopy_id_arg ,
    int64_t time_send_us_arg ,
    int64_t initial_time_us_arg ,
    RoutePlanWithStaticConstraints route_plan_arg ,
    HomotopyStationOverPlan dynamic_station_time_constraints_arg ,
    HomotopyStationOverPlan dynamic_terminal_station_time_constraints_arg ,
    DynamicHardAndSoftConstraints dynamic_spatial_constraints_arg ,
    bool spatialConstraints_allowLaneChange_arg ,
    bool spatialConstraints_terminalReturnToOriginalLane_arg ,
    bool spatialConstraints_terminalStayInAdjacentLane_arg ,
    WorldTrackDecisions decision_arg ,
    std::vector<NamedVectorOfDouble> debugVariables_arg 
    ):
    homotopy_id { std::move(homotopy_id_arg) } // NOLINT
      ,time_send_us { std::move(time_send_us_arg) } // NOLINT
      ,initial_time_us { std::move(initial_time_us_arg) } // NOLINT
      ,route_plan { std::move(route_plan_arg) } // NOLINT
      ,dynamic_station_time_constraints { std::move(dynamic_station_time_constraints_arg) } // NOLINT
      ,dynamic_terminal_station_time_constraints { std::move(dynamic_terminal_station_time_constraints_arg) } // NOLINT
      ,dynamic_spatial_constraints { std::move(dynamic_spatial_constraints_arg) } // NOLINT
      ,spatialConstraints_allowLaneChange { std::move(spatialConstraints_allowLaneChange_arg) } // NOLINT
      ,spatialConstraints_terminalReturnToOriginalLane { std::move(spatialConstraints_terminalReturnToOriginalLane_arg) } // NOLINT
      ,spatialConstraints_terminalStayInAdjacentLane { std::move(spatialConstraints_terminalStayInAdjacentLane_arg) } // NOLINT
      ,decision { std::move(decision_arg) } // NOLINT
      ,debugVariables { std::move(debugVariables_arg) } // NOLINT
      {
}

HomotopyOverPlan::HomotopyOverPlan(const HomotopyOverPlan&) = default;

// NOLINTNEXTLINE
HomotopyOverPlan::HomotopyOverPlan(HomotopyOverPlan&&) = default;

HomotopyOverPlan& HomotopyOverPlan::operator=(const HomotopyOverPlan&) = default;

// NOLINTNEXTLINE
HomotopyOverPlan& HomotopyOverPlan::operator=(HomotopyOverPlan&&) = default;

void HomotopyOverPlan::toCsv(std::ostream& out) const
{
    out.precision(16);
    out << std::boolalpha;
    out << homotopy_id;out << ",";
    out << time_send_us;out << ",";
    out << initial_time_us;out << ",";
    out << route_plan;out << ",";
    out << dynamic_station_time_constraints;out << ",";
    out << dynamic_terminal_station_time_constraints;out << ",";
    out << dynamic_spatial_constraints;out << ",";
    out << spatialConstraints_allowLaneChange;out << ",";
    out << spatialConstraints_terminalReturnToOriginalLane;out << ",";
    out << spatialConstraints_terminalStayInAdjacentLane;out << ",";
    out << decision;out << ",";
    // Enclose sequence in double quotations and curly brackets to keep CSV line length uniform
        
        out << "\"{{";
        
        if (!debugVariables.empty())
        {
            std::copy(debugVariables.begin(), debugVariables.end() - 1, std::ostream_iterator<NamedVectorOfDouble>(out, "},{"));
            std::copy(debugVariables.end()-1, debugVariables.end(), std::ostream_iterator<NamedVectorOfDouble>(out, ""));
        }
        out << "}}\"";
    
}

std::stringstream HomotopyOverPlan::toStringstream() const
{
    std::stringstream out;
    toCsv(out);
    return out;
}

template<class Archive>
    void HomotopyOverPlan::serialize(Archive& ar, [[maybe_unused]] unsigned int version)
    {

        ar& CEREAL_NVP(homotopy_id);
        ar& CEREAL_NVP(time_send_us);
        ar& CEREAL_NVP(initial_time_us);
        ar& CEREAL_NVP(route_plan);
        ar& CEREAL_NVP(dynamic_station_time_constraints);
        ar& CEREAL_NVP(dynamic_terminal_station_time_constraints);
        ar& CEREAL_NVP(dynamic_spatial_constraints);
        ar& CEREAL_NVP(spatialConstraints_allowLaneChange);
        ar& CEREAL_NVP(spatialConstraints_terminalReturnToOriginalLane);
        ar& CEREAL_NVP(spatialConstraints_terminalStayInAdjacentLane);
        ar& CEREAL_NVP(decision);
        ar& CEREAL_NVP(debugVariables);
    }


template void HomotopyOverPlan::serialize(cereal::BinaryInputArchive& ar, unsigned int version);
template void HomotopyOverPlan::serialize(cereal::BinaryOutputArchive& ar, unsigned int version);
template void HomotopyOverPlan::serialize(cereal::IntrospectionArchive& ar, unsigned int version);
template void HomotopyOverPlan::serialize(cereal::JSONInputArchive& ar, unsigned int version);
template void HomotopyOverPlan::serialize(cereal::JSONOutputArchive& ar, unsigned int version);
template void HomotopyOverPlan::serialize(cereal::PortableBinaryInputArchive& ar, unsigned int version);
template void HomotopyOverPlan::serialize(cereal::PortableBinaryOutputArchive& ar, unsigned int version);

std::ostream& operator<< (std::ostream& os, const HomotopyOverPlan& homotopyoverplan)
{
    homotopyoverplan.toCsv(os);
    return os;
}

bool operator== (const HomotopyOverPlan &lhs, const HomotopyOverPlan &rhs)
{
    return (  lhs.homotopy_id== rhs.homotopy_id &&
     lhs.time_send_us== rhs.time_send_us &&
     lhs.initial_time_us== rhs.initial_time_us &&
     lhs.route_plan== rhs.route_plan &&
     lhs.dynamic_station_time_constraints== rhs.dynamic_station_time_constraints &&
     lhs.dynamic_terminal_station_time_constraints== rhs.dynamic_terminal_station_time_constraints &&
     lhs.dynamic_spatial_constraints== rhs.dynamic_spatial_constraints &&
     lhs.spatialConstraints_allowLaneChange== rhs.spatialConstraints_allowLaneChange &&
     lhs.spatialConstraints_terminalReturnToOriginalLane== rhs.spatialConstraints_terminalReturnToOriginalLane &&
     lhs.spatialConstraints_terminalStayInAdjacentLane== rhs.spatialConstraints_terminalStayInAdjacentLane &&
     lhs.decision== rhs.decision &&
     lhs.debugVariables== rhs.debugVariables 
    );
}

std::size_t HomotopyOverPlan::hash() const noexcept
{
    std::size_t result = 17;
    result = result * 31 + std::hash<uint64_t>()(homotopy_id);
    result = result * 31 + std::hash<int64_t>()(time_send_us);
    result = result * 31 + std::hash<int64_t>()(initial_time_us);
    result = result * 31 + std::hash<RoutePlanWithStaticConstraints>()(route_plan);
    result = result * 31 + std::hash<HomotopyStationOverPlan>()(dynamic_station_time_constraints);
    result = result * 31 + std::hash<HomotopyStationOverPlan>()(dynamic_terminal_station_time_constraints);
    result = result * 31 + std::hash<DynamicHardAndSoftConstraints>()(dynamic_spatial_constraints);
    result = result * 31 + std::hash<bool>()(spatialConstraints_allowLaneChange);
    result = result * 31 + std::hash<bool>()(spatialConstraints_terminalReturnToOriginalLane);
    result = result * 31 + std::hash<bool>()(spatialConstraints_terminalStayInAdjacentLane);
    result = result * 31 + std::hash<WorldTrackDecisions>()(decision);
    result = result * 31 + hashVector(debugVariables);
    return result;
}

 } // namespace av
 } // namespace msg
 } // namespace plan_ctrl


CEREAL_BIND_TO_ARCHIVES(av::msg::plan_ctrl::HomotopyOverPlan)

