#include <interfaces/HomotopyOverPlanDebug/HomotopyOverPlanDebug.h>

#include <iterator>
#include <type_traits>

#include <cereal/cereal.hpp>
#include <cereal/archives/json.hpp>
#include <cereal/archives/binary.hpp>

#include <cereal/types/map.hpp>
#include <cereal/types/vector.hpp>
#include <cereal/types/array.hpp>
#include <cereal/types/string.hpp>
#include <cereal/types/complex.hpp>

#include <ODatum/IntrospectionArchive.h>
#include <ODatum/Hash.h>





 namespace av {
 namespace msg {
 namespace plan_ctrl {


HomotopyOverPlanDebug::HomotopyOverPlanDebug() = default;

HomotopyOverPlanDebug::HomotopyOverPlanDebug(
    MapParams mapParams_arg ,
    EgoStateExtended x0_arg ,
    HomotopyOverPlan homotopy_in_arg ,
    std::vector<NamedVectorOfDouble> debugVariables_arg 
    ):
    mapParams { std::move(mapParams_arg) } // NOLINT
      ,x0 { std::move(x0_arg) } // NOLINT
      ,homotopy_in { std::move(homotopy_in_arg) } // NOLINT
      ,debugVariables { std::move(debugVariables_arg) } // NOLINT
      {
}

HomotopyOverPlanDebug::HomotopyOverPlanDebug(const HomotopyOverPlanDebug&) = default;

// NOLINTNEXTLINE
HomotopyOverPlanDebug::HomotopyOverPlanDebug(HomotopyOverPlanDebug&&) = default;

HomotopyOverPlanDebug& HomotopyOverPlanDebug::operator=(const HomotopyOverPlanDebug&) = default;

// NOLINTNEXTLINE
HomotopyOverPlanDebug& HomotopyOverPlanDebug::operator=(HomotopyOverPlanDebug&&) = default;

void HomotopyOverPlanDebug::toCsv(std::ostream& out) const
{
    out.precision(16);
    out << std::boolalpha;
    out << mapParams;out << ",";
    out << x0;out << ",";
    out << homotopy_in;out << ",";
    // Enclose sequence in double quotations and curly brackets to keep CSV line length uniform
        
        out << "\"{{";
        
        if (!debugVariables.empty())
        {
            std::copy(debugVariables.begin(), debugVariables.end() - 1, std::ostream_iterator<NamedVectorOfDouble>(out, "},{"));
            std::copy(debugVariables.end()-1, debugVariables.end(), std::ostream_iterator<NamedVectorOfDouble>(out, ""));
        }
        out << "}}\"";
    
}

std::stringstream HomotopyOverPlanDebug::toStringstream() const
{
    std::stringstream out;
    toCsv(out);
    return out;
}

template<class Archive>
    void HomotopyOverPlanDebug::serialize(Archive& ar, [[maybe_unused]] unsigned int version)
    {

        ar& CEREAL_NVP(mapParams);
        ar& CEREAL_NVP(x0);
        ar& CEREAL_NVP(homotopy_in);
        ar& CEREAL_NVP(debugVariables);
    }


template void HomotopyOverPlanDebug::serialize(cereal::BinaryInputArchive& ar, unsigned int version);
template void HomotopyOverPlanDebug::serialize(cereal::BinaryOutputArchive& ar, unsigned int version);
template void HomotopyOverPlanDebug::serialize(cereal::IntrospectionArchive& ar, unsigned int version);
template void HomotopyOverPlanDebug::serialize(cereal::JSONInputArchive& ar, unsigned int version);
template void HomotopyOverPlanDebug::serialize(cereal::JSONOutputArchive& ar, unsigned int version);
template void HomotopyOverPlanDebug::serialize(cereal::PortableBinaryInputArchive& ar, unsigned int version);
template void HomotopyOverPlanDebug::serialize(cereal::PortableBinaryOutputArchive& ar, unsigned int version);

std::ostream& operator<< (std::ostream& os, const HomotopyOverPlanDebug& homotopyoverplandebug)
{
    homotopyoverplandebug.toCsv(os);
    return os;
}

bool operator== (const HomotopyOverPlanDebug &lhs, const HomotopyOverPlanDebug &rhs)
{
    return (  lhs.mapParams== rhs.mapParams &&
     lhs.x0== rhs.x0 &&
     lhs.homotopy_in== rhs.homotopy_in &&
     lhs.debugVariables== rhs.debugVariables 
    );
}

std::size_t HomotopyOverPlanDebug::hash() const noexcept
{
    std::size_t result = 17;
    result = result * 31 + std::hash<MapParams>()(mapParams);
    result = result * 31 + std::hash<EgoStateExtended>()(x0);
    result = result * 31 + std::hash<HomotopyOverPlan>()(homotopy_in);
    result = result * 31 + hashVector(debugVariables);
    return result;
}

 } // namespace av
 } // namespace msg
 } // namespace plan_ctrl


CEREAL_BIND_TO_ARCHIVES(av::msg::plan_ctrl::HomotopyOverPlanDebug)

