#include <interfaces/HomotopyRealization/HomotopyRealization.h>

#include <iterator>
#include <type_traits>

#include <cereal/cereal.hpp>
#include <cereal/archives/json.hpp>
#include <cereal/archives/binary.hpp>

#include <cereal/types/map.hpp>
#include <cereal/types/vector.hpp>
#include <cereal/types/array.hpp>
#include <cereal/types/string.hpp>
#include <cereal/types/complex.hpp>

#include <ODatum/IntrospectionArchive.h>
#include <ODatum/Hash.h>


 namespace av {
 namespace msg {
 namespace plan_ctrl {


ManeuverLabels::ManeuverLabels() = default;

ManeuverLabels::ManeuverLabels(
    bool isManeuverBlocked_arg 
    ):
    isManeuverBlocked { std::move(isManeuverBlocked_arg) } // NOLINT
      
    {
}

ManeuverLabels::ManeuverLabels(const ManeuverLabels&) = default;

// NOLINTNEXTLINE
ManeuverLabels::ManeuverLabels(ManeuverLabels&&) = default;

ManeuverLabels& ManeuverLabels::operator=(const ManeuverLabels&) = default;

// NOLINTNEXTLINE
ManeuverLabels& ManeuverLabels::operator=(ManeuverLabels&&) = default;

void ManeuverLabels::toCsv(std::ostream& out) const
{
    out.precision(16);
    out << std::boolalpha;
    out << isManeuverBlocked;
    
}

std::stringstream ManeuverLabels::toStringstream() const
{
    std::stringstream out;
    toCsv(out);
    return out;
}

template<class Archive>
    void ManeuverLabels::serialize(Archive& ar, [[maybe_unused]] unsigned int version)
    {

        ar& CEREAL_NVP(isManeuverBlocked);
    }


template void ManeuverLabels::serialize(cereal::BinaryInputArchive& ar, unsigned int version);
template void ManeuverLabels::serialize(cereal::BinaryOutputArchive& ar, unsigned int version);
template void ManeuverLabels::serialize(cereal::IntrospectionArchive& ar, unsigned int version);
template void ManeuverLabels::serialize(cereal::JSONInputArchive& ar, unsigned int version);
template void ManeuverLabels::serialize(cereal::JSONOutputArchive& ar, unsigned int version);
template void ManeuverLabels::serialize(cereal::PortableBinaryInputArchive& ar, unsigned int version);
template void ManeuverLabels::serialize(cereal::PortableBinaryOutputArchive& ar, unsigned int version);

std::ostream& operator<< (std::ostream& os, const ManeuverLabels& maneuverlabels)
{
    maneuverlabels.toCsv(os);
    return os;
}

bool operator== (const ManeuverLabels &lhs, const ManeuverLabels &rhs)
{
    return (  lhs.isManeuverBlocked== rhs.isManeuverBlocked 
    );
}

std::size_t ManeuverLabels::hash() const noexcept
{
    std::size_t result = 17;
    result = result * 31 + std::hash<bool>()(isManeuverBlocked);
    return result;
}

 } // namespace av
 } // namespace msg
 } // namespace plan_ctrl


CEREAL_BIND_TO_ARCHIVES(av::msg::plan_ctrl::ManeuverLabels)




 namespace av {
 namespace msg {
 namespace plan_ctrl {


HomotopyRealization::HomotopyRealization() = default;

HomotopyRealization::HomotopyRealization(
    uint64_t homotopy_id_arg ,
    PathPointMassModelTrajectory trajectory_arg ,
    WorldTrackDecisions decision_arg ,
    ManeuverLabels labeles_arg ,
    std::vector<NamedVectorOfDouble> debugVariables_arg 
    ):
    homotopy_id { std::move(homotopy_id_arg) } // NOLINT
      ,trajectory { std::move(trajectory_arg) } // NOLINT
      ,decision { std::move(decision_arg) } // NOLINT
      ,labeles { std::move(labeles_arg) } // NOLINT
      ,debugVariables { std::move(debugVariables_arg) } // NOLINT
      {
}

HomotopyRealization::HomotopyRealization(const HomotopyRealization&) = default;

// NOLINTNEXTLINE
HomotopyRealization::HomotopyRealization(HomotopyRealization&&) = default;

HomotopyRealization& HomotopyRealization::operator=(const HomotopyRealization&) = default;

// NOLINTNEXTLINE
HomotopyRealization& HomotopyRealization::operator=(HomotopyRealization&&) = default;

void HomotopyRealization::toCsv(std::ostream& out) const
{
    out.precision(16);
    out << std::boolalpha;
    out << homotopy_id;out << ",";
    out << trajectory;out << ",";
    out << decision;out << ",";
    out << labeles;out << ",";
    // Enclose sequence in double quotations and curly brackets to keep CSV line length uniform
        
        out << "\"{{";
        
        if (!debugVariables.empty())
        {
            std::copy(debugVariables.begin(), debugVariables.end() - 1, std::ostream_iterator<NamedVectorOfDouble>(out, "},{"));
            std::copy(debugVariables.end()-1, debugVariables.end(), std::ostream_iterator<NamedVectorOfDouble>(out, ""));
        }
        out << "}}\"";
    
}

std::stringstream HomotopyRealization::toStringstream() const
{
    std::stringstream out;
    toCsv(out);
    return out;
}

template<class Archive>
    void HomotopyRealization::serialize(Archive& ar, [[maybe_unused]] unsigned int version)
    {

        ar& CEREAL_NVP(homotopy_id);
        ar& CEREAL_NVP(trajectory);
        ar& CEREAL_NVP(decision);
        ar& CEREAL_NVP(labeles);
        ar& CEREAL_NVP(debugVariables);
    }


template void HomotopyRealization::serialize(cereal::BinaryInputArchive& ar, unsigned int version);
template void HomotopyRealization::serialize(cereal::BinaryOutputArchive& ar, unsigned int version);
template void HomotopyRealization::serialize(cereal::IntrospectionArchive& ar, unsigned int version);
template void HomotopyRealization::serialize(cereal::JSONInputArchive& ar, unsigned int version);
template void HomotopyRealization::serialize(cereal::JSONOutputArchive& ar, unsigned int version);
template void HomotopyRealization::serialize(cereal::PortableBinaryInputArchive& ar, unsigned int version);
template void HomotopyRealization::serialize(cereal::PortableBinaryOutputArchive& ar, unsigned int version);

std::ostream& operator<< (std::ostream& os, const HomotopyRealization& homotopyrealization)
{
    homotopyrealization.toCsv(os);
    return os;
}

bool operator== (const HomotopyRealization &lhs, const HomotopyRealization &rhs)
{
    return (  lhs.homotopy_id== rhs.homotopy_id &&
     lhs.trajectory== rhs.trajectory &&
     lhs.decision== rhs.decision &&
     lhs.labeles== rhs.labeles &&
     lhs.debugVariables== rhs.debugVariables 
    );
}

std::size_t HomotopyRealization::hash() const noexcept
{
    std::size_t result = 17;
    result = result * 31 + std::hash<uint64_t>()(homotopy_id);
    result = result * 31 + std::hash<PathPointMassModelTrajectory>()(trajectory);
    result = result * 31 + std::hash<WorldTrackDecisions>()(decision);
    result = result * 31 + std::hash<ManeuverLabels>()(labeles);
    result = result * 31 + hashVector(debugVariables);
    return result;
}

 } // namespace av
 } // namespace msg
 } // namespace plan_ctrl


CEREAL_BIND_TO_ARCHIVES(av::msg::plan_ctrl::HomotopyRealization)

