#include <interfaces/HomotopyRealizatorDebug/HomotopyRealizatorDebug.h>

#include <iterator>
#include <type_traits>

#include <cereal/cereal.hpp>
#include <cereal/archives/json.hpp>
#include <cereal/archives/binary.hpp>

#include <cereal/types/map.hpp>
#include <cereal/types/vector.hpp>
#include <cereal/types/array.hpp>
#include <cereal/types/string.hpp>
#include <cereal/types/complex.hpp>

#include <ODatum/IntrospectionArchive.h>
#include <ODatum/Hash.h>





 namespace av {
 namespace msg {
 namespace plan_ctrl {


HomotopyRealizatorDebug::HomotopyRealizatorDebug() = default;

HomotopyRealizatorDebug::HomotopyRealizatorDebug(
    MapParams mapParams_arg ,
    EgoStateExtended x0_arg ,
    HomotopyOverPlan routePlan_arg ,
    PathPointMassModelTrajectory committedTrajectory_arg ,
    PathPointMassModelTrajectory optimizedTrajectory_arg ,
    std::vector<NamedVectorOfDouble> debugVariables_arg ,
    int64_t timeStamp_us_arg ,
    std::vector<NamedVectorOfDouble> horizonDecisionVariables_arg ,
    std::vector<NamedVectorOfDouble> horizonSolverParameters_arg ,
    std::vector<NamedVectorOfDouble> horizonAdditionalParameters_arg 
    ):
    mapParams { std::move(mapParams_arg) } // NOLINT
      ,x0 { std::move(x0_arg) } // NOLINT
      ,routePlan { std::move(routePlan_arg) } // NOLINT
      ,committedTrajectory { std::move(committedTrajectory_arg) } // NOLINT
      ,optimizedTrajectory { std::move(optimizedTrajectory_arg) } // NOLINT
      ,debugVariables { std::move(debugVariables_arg) } // NOLINT
      ,timeStamp_us { std::move(timeStamp_us_arg) } // NOLINT
      ,horizonDecisionVariables { std::move(horizonDecisionVariables_arg) } // NOLINT
      ,horizonSolverParameters { std::move(horizonSolverParameters_arg) } // NOLINT
      ,horizonAdditionalParameters { std::move(horizonAdditionalParameters_arg) } // NOLINT
      {
}

HomotopyRealizatorDebug::HomotopyRealizatorDebug(const HomotopyRealizatorDebug&) = default;

// NOLINTNEXTLINE
HomotopyRealizatorDebug::HomotopyRealizatorDebug(HomotopyRealizatorDebug&&) = default;

HomotopyRealizatorDebug& HomotopyRealizatorDebug::operator=(const HomotopyRealizatorDebug&) = default;

// NOLINTNEXTLINE
HomotopyRealizatorDebug& HomotopyRealizatorDebug::operator=(HomotopyRealizatorDebug&&) = default;

void HomotopyRealizatorDebug::toCsv(std::ostream& out) const
{
    out.precision(16);
    out << std::boolalpha;
    out << mapParams;out << ",";
    out << x0;out << ",";
    out << routePlan;out << ",";
    out << committedTrajectory;out << ",";
    out << optimizedTrajectory;out << ",";
    // Enclose sequence in double quotations and curly brackets to keep CSV line length uniform
        
        out << "\"{{";
        
        if (!debugVariables.empty())
        {
            std::copy(debugVariables.begin(), debugVariables.end() - 1, std::ostream_iterator<NamedVectorOfDouble>(out, "},{"));
            std::copy(debugVariables.end()-1, debugVariables.end(), std::ostream_iterator<NamedVectorOfDouble>(out, ""));
        }
        out << "}}\"";out << ",";
    out << timeStamp_us;out << ",";
    // Enclose sequence in double quotations and curly brackets to keep CSV line length uniform
        
        out << "\"{{";
        
        if (!horizonDecisionVariables.empty())
        {
            std::copy(horizonDecisionVariables.begin(), horizonDecisionVariables.end() - 1, std::ostream_iterator<NamedVectorOfDouble>(out, "},{"));
            std::copy(horizonDecisionVariables.end()-1, horizonDecisionVariables.end(), std::ostream_iterator<NamedVectorOfDouble>(out, ""));
        }
        out << "}}\"";out << ",";
    // Enclose sequence in double quotations and curly brackets to keep CSV line length uniform
        
        out << "\"{{";
        
        if (!horizonSolverParameters.empty())
        {
            std::copy(horizonSolverParameters.begin(), horizonSolverParameters.end() - 1, std::ostream_iterator<NamedVectorOfDouble>(out, "},{"));
            std::copy(horizonSolverParameters.end()-1, horizonSolverParameters.end(), std::ostream_iterator<NamedVectorOfDouble>(out, ""));
        }
        out << "}}\"";out << ",";
    // Enclose sequence in double quotations and curly brackets to keep CSV line length uniform
        
        out << "\"{{";
        
        if (!horizonAdditionalParameters.empty())
        {
            std::copy(horizonAdditionalParameters.begin(), horizonAdditionalParameters.end() - 1, std::ostream_iterator<NamedVectorOfDouble>(out, "},{"));
            std::copy(horizonAdditionalParameters.end()-1, horizonAdditionalParameters.end(), std::ostream_iterator<NamedVectorOfDouble>(out, ""));
        }
        out << "}}\"";
    
}

std::stringstream HomotopyRealizatorDebug::toStringstream() const
{
    std::stringstream out;
    toCsv(out);
    return out;
}

template<class Archive>
    void HomotopyRealizatorDebug::serialize(Archive& ar, unsigned int version)
{
	if (version >= 1)
	{
		ar& CEREAL_NVP(mapParams);
		ar& CEREAL_NVP(x0);
		ar& CEREAL_NVP(routePlan);
		ar& CEREAL_NVP(committedTrajectory);
		ar& CEREAL_NVP(optimizedTrajectory);
		ar& CEREAL_NVP(debugVariables);
	}
	if (version >= 2)
	{
		ar& CEREAL_NVP(timeStamp_us);
		ar& CEREAL_NVP(horizonDecisionVariables);
		ar& CEREAL_NVP(horizonSolverParameters);
		ar& CEREAL_NVP(horizonAdditionalParameters);
	}
}



template void HomotopyRealizatorDebug::serialize(cereal::BinaryInputArchive& ar, unsigned int version);
template void HomotopyRealizatorDebug::serialize(cereal::BinaryOutputArchive& ar, unsigned int version);
template void HomotopyRealizatorDebug::serialize(cereal::IntrospectionArchive& ar, unsigned int version);
template void HomotopyRealizatorDebug::serialize(cereal::JSONInputArchive& ar, unsigned int version);
template void HomotopyRealizatorDebug::serialize(cereal::JSONOutputArchive& ar, unsigned int version);
template void HomotopyRealizatorDebug::serialize(cereal::PortableBinaryInputArchive& ar, unsigned int version);
template void HomotopyRealizatorDebug::serialize(cereal::PortableBinaryOutputArchive& ar, unsigned int version);

std::ostream& operator<< (std::ostream& os, const HomotopyRealizatorDebug& homotopyrealizatordebug)
{
    homotopyrealizatordebug.toCsv(os);
    return os;
}

bool operator== (const HomotopyRealizatorDebug &lhs, const HomotopyRealizatorDebug &rhs)
{
    return (  lhs.mapParams== rhs.mapParams &&
     lhs.x0== rhs.x0 &&
     lhs.routePlan== rhs.routePlan &&
     lhs.committedTrajectory== rhs.committedTrajectory &&
     lhs.optimizedTrajectory== rhs.optimizedTrajectory &&
     lhs.debugVariables== rhs.debugVariables &&
     lhs.timeStamp_us== rhs.timeStamp_us &&
     lhs.horizonDecisionVariables== rhs.horizonDecisionVariables &&
     lhs.horizonSolverParameters== rhs.horizonSolverParameters &&
     lhs.horizonAdditionalParameters== rhs.horizonAdditionalParameters 
    );
}

std::size_t HomotopyRealizatorDebug::hash() const noexcept
{
    std::size_t result = 17;
    result = result * 31 + std::hash<MapParams>()(mapParams);
    result = result * 31 + std::hash<EgoStateExtended>()(x0);
    result = result * 31 + std::hash<HomotopyOverPlan>()(routePlan);
    result = result * 31 + std::hash<PathPointMassModelTrajectory>()(committedTrajectory);
    result = result * 31 + std::hash<PathPointMassModelTrajectory>()(optimizedTrajectory);
    result = result * 31 + hashVector(debugVariables);
    result = result * 31 + std::hash<int64_t>()(timeStamp_us);
    result = result * 31 + hashVector(horizonDecisionVariables);
    result = result * 31 + hashVector(horizonSolverParameters);
    result = result * 31 + hashVector(horizonAdditionalParameters);
    return result;
}

 } // namespace av
 } // namespace msg
 } // namespace plan_ctrl


CEREAL_BIND_TO_ARCHIVES(av::msg::plan_ctrl::HomotopyRealizatorDebug)

