#include <interfaces/HorizonSpeedControlCommand/HorizonSpeedControlCommand.h>

#include <iterator>
#include <type_traits>

#include <cereal/cereal.hpp>
#include <cereal/archives/json.hpp>
#include <cereal/archives/binary.hpp>

#include <cereal/types/map.hpp>
#include <cereal/types/vector.hpp>
#include <cereal/types/array.hpp>
#include <cereal/types/string.hpp>
#include <cereal/types/complex.hpp>

#include <ODatum/IntrospectionArchive.h>
#include <ODatum/Hash.h>





 namespace av {
 namespace msg {


HorizonSpeedControlCommand::HorizonSpeedControlCommand() = default;

HorizonSpeedControlCommand::HorizonSpeedControlCommand(
    int64_t time_us_arg ,
    std::vector<av::msg::ControlStep> commandSteps_arg 
    ):
    time_us { std::move(time_us_arg) } // NOLINT
      ,commandSteps { std::move(commandSteps_arg) } // NOLINT
      {
}

HorizonSpeedControlCommand::HorizonSpeedControlCommand(const HorizonSpeedControlCommand&) = default;

// NOLINTNEXTLINE
HorizonSpeedControlCommand::HorizonSpeedControlCommand(HorizonSpeedControlCommand&&) = default;

HorizonSpeedControlCommand& HorizonSpeedControlCommand::operator=(const HorizonSpeedControlCommand&) = default;

// NOLINTNEXTLINE
HorizonSpeedControlCommand& HorizonSpeedControlCommand::operator=(HorizonSpeedControlCommand&&) = default;

void HorizonSpeedControlCommand::toCsv(std::ostream& out) const
{
    out.precision(16);
    out << std::boolalpha;
    out << time_us;out << ",";
    // Enclose sequence in double quotations and curly brackets to keep CSV line length uniform
        
        out << "\"{{";
        
        if (!commandSteps.empty())
        {
            std::copy(commandSteps.begin(), commandSteps.end() - 1, std::ostream_iterator<av::msg::ControlStep>(out, "},{"));
            std::copy(commandSteps.end()-1, commandSteps.end(), std::ostream_iterator<av::msg::ControlStep>(out, ""));
        }
        out << "}}\"";
    
}

std::stringstream HorizonSpeedControlCommand::toStringstream() const
{
    std::stringstream out;
    toCsv(out);
    return out;
}

template<class Archive>
    void HorizonSpeedControlCommand::serialize(Archive& ar, [[maybe_unused]] unsigned int version)
    {

        ar& CEREAL_NVP(time_us);
        ar& CEREAL_NVP(commandSteps);
    }


template void HorizonSpeedControlCommand::serialize(cereal::BinaryInputArchive& ar, unsigned int version);
template void HorizonSpeedControlCommand::serialize(cereal::BinaryOutputArchive& ar, unsigned int version);
template void HorizonSpeedControlCommand::serialize(cereal::IntrospectionArchive& ar, unsigned int version);
template void HorizonSpeedControlCommand::serialize(cereal::JSONInputArchive& ar, unsigned int version);
template void HorizonSpeedControlCommand::serialize(cereal::JSONOutputArchive& ar, unsigned int version);
template void HorizonSpeedControlCommand::serialize(cereal::PortableBinaryInputArchive& ar, unsigned int version);
template void HorizonSpeedControlCommand::serialize(cereal::PortableBinaryOutputArchive& ar, unsigned int version);

std::ostream& operator<< (std::ostream& os, const HorizonSpeedControlCommand& horizonspeedcontrolcommand)
{
    horizonspeedcontrolcommand.toCsv(os);
    return os;
}

bool operator== (const HorizonSpeedControlCommand &lhs, const HorizonSpeedControlCommand &rhs)
{
    return (  lhs.time_us== rhs.time_us &&
     lhs.commandSteps== rhs.commandSteps 
    );
}

std::size_t HorizonSpeedControlCommand::hash() const noexcept
{
    std::size_t result = 17;
    result = result * 31 + std::hash<int64_t>()(time_us);
    result = result * 31 + hashVector(commandSteps);
    return result;
}

 } // namespace av
 } // namespace msg


CEREAL_BIND_TO_ARCHIVES(av::msg::HorizonSpeedControlCommand)

