#include <interfaces/IMUState/IMUState.h>

#include <iterator>
#include <type_traits>

#include <cereal/cereal.hpp>
#include <cereal/archives/json.hpp>
#include <cereal/archives/binary.hpp>

#include <cereal/types/map.hpp>
#include <cereal/types/vector.hpp>
#include <cereal/types/array.hpp>
#include <cereal/types/string.hpp>
#include <cereal/types/complex.hpp>

#include <ODatum/IntrospectionArchive.h>
#include <ODatum/Hash.h>





 namespace av {
 namespace msg {


IMUState::IMUState() = default;

IMUState::IMUState(
    TimePoint timeStamp_arg ,
    geometry::RecPoint3D accel_arg ,
    geometry::RecPoint3D gyro_arg ,
    geometry::RecPoint3D mag_arg ,
    geometry::RecPoint3D deltaAccel_arg ,
    geometry::RecPoint3D deltaGyro_arg 
    ):
    timeStamp { std::move(timeStamp_arg) } // NOLINT
      ,accel { std::move(accel_arg) } // NOLINT
      ,gyro { std::move(gyro_arg) } // NOLINT
      ,mag { std::move(mag_arg) } // NOLINT
      ,deltaAccel { std::move(deltaAccel_arg) } // NOLINT
      ,deltaGyro { std::move(deltaGyro_arg) } // NOLINT
      {
}

IMUState::IMUState(const IMUState&) = default;

// NOLINTNEXTLINE
IMUState::IMUState(IMUState&&) = default;

IMUState& IMUState::operator=(const IMUState&) = default;

// NOLINTNEXTLINE
IMUState& IMUState::operator=(IMUState&&) = default;

void IMUState::toCsv(std::ostream& out) const
{
    out.precision(16);
    out << std::boolalpha;
    out << timeStamp;out << ",";
    out << accel;out << ",";
    out << gyro;out << ",";
    out << mag;out << ",";
    out << deltaAccel;out << ",";
    out << deltaGyro;
    
}

std::stringstream IMUState::toStringstream() const
{
    std::stringstream out;
    toCsv(out);
    return out;
}

template<class Archive>
    void IMUState::serialize(Archive& ar, [[maybe_unused]] unsigned int version)
    {

        ar& CEREAL_NVP(timeStamp);
        ar& CEREAL_NVP(accel);
        ar& CEREAL_NVP(gyro);
        ar& CEREAL_NVP(mag);
        ar& CEREAL_NVP(deltaAccel);
        ar& CEREAL_NVP(deltaGyro);
    }


template void IMUState::serialize(cereal::BinaryInputArchive& ar, unsigned int version);
template void IMUState::serialize(cereal::BinaryOutputArchive& ar, unsigned int version);
template void IMUState::serialize(cereal::IntrospectionArchive& ar, unsigned int version);
template void IMUState::serialize(cereal::JSONInputArchive& ar, unsigned int version);
template void IMUState::serialize(cereal::JSONOutputArchive& ar, unsigned int version);
template void IMUState::serialize(cereal::PortableBinaryInputArchive& ar, unsigned int version);
template void IMUState::serialize(cereal::PortableBinaryOutputArchive& ar, unsigned int version);

std::ostream& operator<< (std::ostream& os, const IMUState& imustate)
{
    imustate.toCsv(os);
    return os;
}

bool operator== (const IMUState &lhs, const IMUState &rhs)
{
    return (  lhs.timeStamp== rhs.timeStamp &&
     lhs.accel== rhs.accel &&
     lhs.gyro== rhs.gyro &&
     lhs.mag== rhs.mag &&
     lhs.deltaAccel== rhs.deltaAccel &&
     lhs.deltaGyro== rhs.deltaGyro 
    );
}

std::size_t IMUState::hash() const noexcept
{
    std::size_t result = 17;
    result = result * 31 + 0;
    result = result * 31 + std::hash<geometry::RecPoint3D>()(accel);
    result = result * 31 + std::hash<geometry::RecPoint3D>()(gyro);
    result = result * 31 + std::hash<geometry::RecPoint3D>()(mag);
    result = result * 31 + std::hash<geometry::RecPoint3D>()(deltaAccel);
    result = result * 31 + std::hash<geometry::RecPoint3D>()(deltaGyro);
    return result;
}

 } // namespace av
 } // namespace msg


CEREAL_BIND_TO_ARCHIVES(av::msg::IMUState)

