#include <interfaces/IMUStateList/IMUStateList.h>

#include <iterator>
#include <type_traits>

#include <cereal/cereal.hpp>
#include <cereal/archives/json.hpp>
#include <cereal/archives/binary.hpp>

#include <cereal/types/map.hpp>
#include <cereal/types/vector.hpp>
#include <cereal/types/array.hpp>
#include <cereal/types/string.hpp>
#include <cereal/types/complex.hpp>

#include <ODatum/IntrospectionArchive.h>
#include <ODatum/Hash.h>





 namespace av {
 namespace msg {


IMUStateList::IMUStateList() = default;

IMUStateList::IMUStateList(
    std::vector<av::msg::IMUState> imuList_arg 
    ):
    imuList { std::move(imuList_arg) } // NOLINT
      {
}

IMUStateList::IMUStateList(const IMUStateList&) = default;

// NOLINTNEXTLINE
IMUStateList::IMUStateList(IMUStateList&&) = default;

IMUStateList& IMUStateList::operator=(const IMUStateList&) = default;

// NOLINTNEXTLINE
IMUStateList& IMUStateList::operator=(IMUStateList&&) = default;

void IMUStateList::toCsv(std::ostream& out) const
{
    out.precision(16);
    out << std::boolalpha;
    // Enclose sequence in double quotations and curly brackets to keep CSV line length uniform
        
        out << "\"{{";
        
        if (!imuList.empty())
        {
            std::copy(imuList.begin(), imuList.end() - 1, std::ostream_iterator<av::msg::IMUState>(out, "},{"));
            std::copy(imuList.end()-1, imuList.end(), std::ostream_iterator<av::msg::IMUState>(out, ""));
        }
        out << "}}\"";
    
}

std::stringstream IMUStateList::toStringstream() const
{
    std::stringstream out;
    toCsv(out);
    return out;
}

template<class Archive>
    void IMUStateList::serialize(Archive& ar, [[maybe_unused]] unsigned int version)
    {

        ar& CEREAL_NVP(imuList);
    }


template void IMUStateList::serialize(cereal::BinaryInputArchive& ar, unsigned int version);
template void IMUStateList::serialize(cereal::BinaryOutputArchive& ar, unsigned int version);
template void IMUStateList::serialize(cereal::IntrospectionArchive& ar, unsigned int version);
template void IMUStateList::serialize(cereal::JSONInputArchive& ar, unsigned int version);
template void IMUStateList::serialize(cereal::JSONOutputArchive& ar, unsigned int version);
template void IMUStateList::serialize(cereal::PortableBinaryInputArchive& ar, unsigned int version);
template void IMUStateList::serialize(cereal::PortableBinaryOutputArchive& ar, unsigned int version);

std::ostream& operator<< (std::ostream& os, const IMUStateList& imustatelist)
{
    imustatelist.toCsv(os);
    return os;
}

bool operator== (const IMUStateList &lhs, const IMUStateList &rhs)
{
    return (  lhs.imuList== rhs.imuList 
    );
}

std::size_t IMUStateList::hash() const noexcept
{
    std::size_t result = 17;
    result = result * 31 + hashVector(imuList);
    return result;
}

 } // namespace av
 } // namespace msg


CEREAL_BIND_TO_ARCHIVES(av::msg::IMUStateList)

