#include <interfaces/IntersectionTLStatus/IntersectionTLStatus.h>

#include <iterator>
#include <type_traits>

#include <cereal/cereal.hpp>
#include <cereal/archives/json.hpp>
#include <cereal/archives/binary.hpp>

#include <cereal/types/map.hpp>
#include <cereal/types/vector.hpp>
#include <cereal/types/array.hpp>
#include <cereal/types/string.hpp>
#include <cereal/types/complex.hpp>

#include <ODatum/IntrospectionArchive.h>
#include <ODatum/Hash.h>


 namespace av {
 namespace msg {


StateScoreMapping::StateScoreMapping() = default;

StateScoreMapping::StateScoreMapping(
    EDGE_STATE state_arg ,
    double probability_arg 
    ):
    state { std::move(state_arg) } // NOLINT
      ,
    probability { std::move(probability_arg) } // NOLINT
      
    {
}

StateScoreMapping::StateScoreMapping(const StateScoreMapping&) = default;

// NOLINTNEXTLINE
StateScoreMapping::StateScoreMapping(StateScoreMapping&&) = default;

StateScoreMapping& StateScoreMapping::operator=(const StateScoreMapping&) = default;

// NOLINTNEXTLINE
StateScoreMapping& StateScoreMapping::operator=(StateScoreMapping&&) = default;

void StateScoreMapping::toCsv(std::ostream& out) const
{
    out.precision(16);
    out << std::boolalpha;
    out << static_cast<uint32_t>(state);out << ",";
    out << probability;
    
}

std::stringstream StateScoreMapping::toStringstream() const
{
    std::stringstream out;
    toCsv(out);
    return out;
}

template<class Archive>
    void StateScoreMapping::serialize(Archive& ar, [[maybe_unused]] unsigned int version)
    {

        ar& CEREAL_NVP(state);
        ar& CEREAL_NVP(probability);
    }


template void StateScoreMapping::serialize(cereal::BinaryInputArchive& ar, unsigned int version);
template void StateScoreMapping::serialize(cereal::BinaryOutputArchive& ar, unsigned int version);
template void StateScoreMapping::serialize(cereal::IntrospectionArchive& ar, unsigned int version);
template void StateScoreMapping::serialize(cereal::JSONInputArchive& ar, unsigned int version);
template void StateScoreMapping::serialize(cereal::JSONOutputArchive& ar, unsigned int version);
template void StateScoreMapping::serialize(cereal::PortableBinaryInputArchive& ar, unsigned int version);
template void StateScoreMapping::serialize(cereal::PortableBinaryOutputArchive& ar, unsigned int version);

std::ostream& operator<< (std::ostream& os, const StateScoreMapping& statescoremapping)
{
    statescoremapping.toCsv(os);
    return os;
}

bool operator== (const StateScoreMapping &lhs, const StateScoreMapping &rhs)
{
    return (  lhs.state== rhs.state &&
     lhs.probability== rhs.probability 
    );
}

std::size_t StateScoreMapping::hash() const noexcept
{
    std::size_t result = 17;
    result = result * 31 + std::hash<EDGE_STATE>()(state);
    result = result * 31 + std::hash<double>()(probability);
    return result;
}

 } // namespace av
 } // namespace msg


CEREAL_BIND_TO_ARCHIVES(av::msg::StateScoreMapping)

 namespace av {
 namespace msg {


EdgeTLStatus::EdgeTLStatus() = default;

EdgeTLStatus::EdgeTLStatus(
    FOV_STATE fov_state_arg ,
    LIGHT_COLOR color_arg ,
    LIGHT_SHAPE shape_arg ,
    LIGHT_STATE state_arg ,
    std::vector<StateScoreMapping> score_arg ,
    double remaining_time_arg ,
    TimePoint timestamp_arg 
    ):
    fov_state { std::move(fov_state_arg) } // NOLINT
      ,
    color { std::move(color_arg) } // NOLINT
      ,
    shape { std::move(shape_arg) } // NOLINT
      ,
    state { std::move(state_arg) } // NOLINT
      ,
    score { std::move(score_arg) } // NOLINT
      ,
    remaining_time { std::move(remaining_time_arg) } // NOLINT
      ,
    timestamp { std::move(timestamp_arg) } // NOLINT
      
    {
}

EdgeTLStatus::EdgeTLStatus(const EdgeTLStatus&) = default;

// NOLINTNEXTLINE
EdgeTLStatus::EdgeTLStatus(EdgeTLStatus&&) = default;

EdgeTLStatus& EdgeTLStatus::operator=(const EdgeTLStatus&) = default;

// NOLINTNEXTLINE
EdgeTLStatus& EdgeTLStatus::operator=(EdgeTLStatus&&) = default;

void EdgeTLStatus::toCsv(std::ostream& out) const
{
    out.precision(16);
    out << std::boolalpha;
    out << static_cast<uint32_t>(fov_state);out << ",";
    out << static_cast<uint32_t>(color);out << ",";
    out << static_cast<uint32_t>(shape);out << ",";
    out << static_cast<uint32_t>(state);out << ",";
    // Enclose sequence in double quotations and curly brackets to keep CSV line length uniform
        
        out << "\"{{";
        
        if (!score.empty())
        {
            std::copy(score.begin(), score.end() - 1, std::ostream_iterator<StateScoreMapping>(out, "},{"));
            std::copy(score.end()-1, score.end(), std::ostream_iterator<StateScoreMapping>(out, ""));
        }
        out << "}}\"";out << ",";
    out << remaining_time;out << ",";
    out << timestamp;
    
}

std::stringstream EdgeTLStatus::toStringstream() const
{
    std::stringstream out;
    toCsv(out);
    return out;
}

template<class Archive>
    void EdgeTLStatus::serialize(Archive& ar, const unsigned int version)
{
    ar& CEREAL_NVP(fov_state);
    ar& CEREAL_NVP(color);
    ar& CEREAL_NVP(shape);
    ar& CEREAL_NVP(state);
    ar& CEREAL_NVP(score);
    ar& CEREAL_NVP(remaining_time);

    if (version >= 2)
    {
        ar& CEREAL_NVP(timestamp);
    }
}


template void EdgeTLStatus::serialize(cereal::BinaryInputArchive& ar, unsigned int version);
template void EdgeTLStatus::serialize(cereal::BinaryOutputArchive& ar, unsigned int version);
template void EdgeTLStatus::serialize(cereal::IntrospectionArchive& ar, unsigned int version);
template void EdgeTLStatus::serialize(cereal::JSONInputArchive& ar, unsigned int version);
template void EdgeTLStatus::serialize(cereal::JSONOutputArchive& ar, unsigned int version);
template void EdgeTLStatus::serialize(cereal::PortableBinaryInputArchive& ar, unsigned int version);
template void EdgeTLStatus::serialize(cereal::PortableBinaryOutputArchive& ar, unsigned int version);

std::ostream& operator<< (std::ostream& os, const EdgeTLStatus& edgetlstatus)
{
    edgetlstatus.toCsv(os);
    return os;
}

bool operator== (const EdgeTLStatus &lhs, const EdgeTLStatus &rhs)
{
    return (  lhs.fov_state== rhs.fov_state &&
     lhs.color== rhs.color &&
     lhs.shape== rhs.shape &&
     lhs.state== rhs.state &&
     lhs.score== rhs.score &&
     lhs.remaining_time== rhs.remaining_time &&
     lhs.timestamp== rhs.timestamp 
    );
}

std::size_t EdgeTLStatus::hash() const noexcept
{
    std::size_t result = 17;
    result = result * 31 + std::hash<FOV_STATE>()(fov_state);
    result = result * 31 + std::hash<LIGHT_COLOR>()(color);
    result = result * 31 + std::hash<LIGHT_SHAPE>()(shape);
    result = result * 31 + std::hash<LIGHT_STATE>()(state);
    result = result * 31 + hashVector(score);
    result = result * 31 + std::hash<double>()(remaining_time);
    result = result * 31 + 0;
    return result;
}

 } // namespace av
 } // namespace msg


CEREAL_BIND_TO_ARCHIVES(av::msg::EdgeTLStatus)

 namespace av {
 namespace msg {


EdgeTLStatusMapping::EdgeTLStatusMapping() = default;

EdgeTLStatusMapping::EdgeTLStatusMapping(
    std::string to_roadblock_id_arg ,
    EdgeTLStatus tl_status_arg 
    ):
    to_roadblock_id { std::move(to_roadblock_id_arg) } // NOLINT
      ,
    tl_status { std::move(tl_status_arg) } // NOLINT
      
    {
}

EdgeTLStatusMapping::EdgeTLStatusMapping(const EdgeTLStatusMapping&) = default;

// NOLINTNEXTLINE
EdgeTLStatusMapping::EdgeTLStatusMapping(EdgeTLStatusMapping&&) = default;

EdgeTLStatusMapping& EdgeTLStatusMapping::operator=(const EdgeTLStatusMapping&) = default;

// NOLINTNEXTLINE
EdgeTLStatusMapping& EdgeTLStatusMapping::operator=(EdgeTLStatusMapping&&) = default;

void EdgeTLStatusMapping::toCsv(std::ostream& out) const
{
    out.precision(16);
    out << std::boolalpha;
    out << "\"";
        out << to_roadblock_id;
        out << "\"";out << ",";
    out << tl_status;
    
}

std::stringstream EdgeTLStatusMapping::toStringstream() const
{
    std::stringstream out;
    toCsv(out);
    return out;
}

template<class Archive>
    void EdgeTLStatusMapping::serialize(Archive& ar, [[maybe_unused]] unsigned int version)
    {

        ar& CEREAL_NVP(to_roadblock_id);
        ar& CEREAL_NVP(tl_status);
    }


template void EdgeTLStatusMapping::serialize(cereal::BinaryInputArchive& ar, unsigned int version);
template void EdgeTLStatusMapping::serialize(cereal::BinaryOutputArchive& ar, unsigned int version);
template void EdgeTLStatusMapping::serialize(cereal::IntrospectionArchive& ar, unsigned int version);
template void EdgeTLStatusMapping::serialize(cereal::JSONInputArchive& ar, unsigned int version);
template void EdgeTLStatusMapping::serialize(cereal::JSONOutputArchive& ar, unsigned int version);
template void EdgeTLStatusMapping::serialize(cereal::PortableBinaryInputArchive& ar, unsigned int version);
template void EdgeTLStatusMapping::serialize(cereal::PortableBinaryOutputArchive& ar, unsigned int version);

std::ostream& operator<< (std::ostream& os, const EdgeTLStatusMapping& edgetlstatusmapping)
{
    edgetlstatusmapping.toCsv(os);
    return os;
}

bool operator== (const EdgeTLStatusMapping &lhs, const EdgeTLStatusMapping &rhs)
{
    return (  lhs.to_roadblock_id== rhs.to_roadblock_id &&
     lhs.tl_status== rhs.tl_status 
    );
}

std::size_t EdgeTLStatusMapping::hash() const noexcept
{
    std::size_t result = 17;
    result = result * 31 + std::hash<std::string>()(to_roadblock_id);
    result = result * 31 + std::hash<EdgeTLStatus>()(tl_status);
    return result;
}

 } // namespace av
 } // namespace msg


CEREAL_BIND_TO_ARCHIVES(av::msg::EdgeTLStatusMapping)




 namespace av {
 namespace msg {


IntersectionTLStatus::IntersectionTLStatus() = default;

IntersectionTLStatus::IntersectionTLStatus(
    int64_t stamp_arg ,
    std::string intersection_id_arg ,
    std::string from_roadblock_id_arg ,
    std::vector<EdgeTLStatusMapping> topology_edges_arg 
    ):
    stamp { std::move(stamp_arg) } // NOLINT
      ,intersection_id { std::move(intersection_id_arg) } // NOLINT
      ,from_roadblock_id { std::move(from_roadblock_id_arg) } // NOLINT
      ,topology_edges { std::move(topology_edges_arg) } // NOLINT
      {
}

IntersectionTLStatus::IntersectionTLStatus(const IntersectionTLStatus&) = default;

// NOLINTNEXTLINE
IntersectionTLStatus::IntersectionTLStatus(IntersectionTLStatus&&) = default;

IntersectionTLStatus& IntersectionTLStatus::operator=(const IntersectionTLStatus&) = default;

// NOLINTNEXTLINE
IntersectionTLStatus& IntersectionTLStatus::operator=(IntersectionTLStatus&&) = default;

void IntersectionTLStatus::toCsv(std::ostream& out) const
{
    out.precision(16);
    out << std::boolalpha;
    out << stamp;out << ",";
    out << "\"";
        out << intersection_id;
        out << "\"";out << ",";
    out << "\"";
        out << from_roadblock_id;
        out << "\"";out << ",";
    // Enclose sequence in double quotations and curly brackets to keep CSV line length uniform
        
        out << "\"{{";
        
        if (!topology_edges.empty())
        {
            std::copy(topology_edges.begin(), topology_edges.end() - 1, std::ostream_iterator<EdgeTLStatusMapping>(out, "},{"));
            std::copy(topology_edges.end()-1, topology_edges.end(), std::ostream_iterator<EdgeTLStatusMapping>(out, ""));
        }
        out << "}}\"";
    
}

std::stringstream IntersectionTLStatus::toStringstream() const
{
    std::stringstream out;
    toCsv(out);
    return out;
}

template<class Archive>
    void IntersectionTLStatus::serialize(Archive& ar, [[maybe_unused]] unsigned int version)
    {

        ar& CEREAL_NVP(stamp);
        ar& CEREAL_NVP(intersection_id);
        ar& CEREAL_NVP(from_roadblock_id);
        ar& CEREAL_NVP(topology_edges);
    }


template void IntersectionTLStatus::serialize(cereal::BinaryInputArchive& ar, unsigned int version);
template void IntersectionTLStatus::serialize(cereal::BinaryOutputArchive& ar, unsigned int version);
template void IntersectionTLStatus::serialize(cereal::IntrospectionArchive& ar, unsigned int version);
template void IntersectionTLStatus::serialize(cereal::JSONInputArchive& ar, unsigned int version);
template void IntersectionTLStatus::serialize(cereal::JSONOutputArchive& ar, unsigned int version);
template void IntersectionTLStatus::serialize(cereal::PortableBinaryInputArchive& ar, unsigned int version);
template void IntersectionTLStatus::serialize(cereal::PortableBinaryOutputArchive& ar, unsigned int version);

std::ostream& operator<< (std::ostream& os, const IntersectionTLStatus& intersectiontlstatus)
{
    intersectiontlstatus.toCsv(os);
    return os;
}

bool operator== (const IntersectionTLStatus &lhs, const IntersectionTLStatus &rhs)
{
    return (  lhs.stamp== rhs.stamp &&
     lhs.intersection_id== rhs.intersection_id &&
     lhs.from_roadblock_id== rhs.from_roadblock_id &&
     lhs.topology_edges== rhs.topology_edges 
    );
}

std::size_t IntersectionTLStatus::hash() const noexcept
{
    std::size_t result = 17;
    result = result * 31 + std::hash<int64_t>()(stamp);
    result = result * 31 + std::hash<std::string>()(intersection_id);
    result = result * 31 + std::hash<std::string>()(from_roadblock_id);
    result = result * 31 + hashVector(topology_edges);
    return result;
}

 } // namespace av
 } // namespace msg


CEREAL_BIND_TO_ARCHIVES(av::msg::IntersectionTLStatus)

