#include <interfaces/InterventionStatusUpdate/InterventionStatusUpdate.h>

#include <iterator>
#include <type_traits>

#include <cereal/cereal.hpp>
#include <cereal/archives/json.hpp>
#include <cereal/archives/binary.hpp>

#include <cereal/types/map.hpp>
#include <cereal/types/vector.hpp>
#include <cereal/types/array.hpp>
#include <cereal/types/string.hpp>
#include <cereal/types/complex.hpp>

#include <ODatum/IntrospectionArchive.h>
#include <ODatum/Hash.h>





 namespace av {
 namespace msg {


InterventionStatusUpdate::InterventionStatusUpdate() = default;

InterventionStatusUpdate::InterventionStatusUpdate(
    av::msg::TimePoint timestamp_arg ,
    av::msg::InterventionStatusUpdateType status_arg ,
    std::string assistanceRequestId_arg 
    ):
    timestamp { std::move(timestamp_arg) } // NOLINT
      ,status { std::move(status_arg) } // NOLINT
      ,assistanceRequestId { std::move(assistanceRequestId_arg) } // NOLINT
      {
}

InterventionStatusUpdate::InterventionStatusUpdate(const InterventionStatusUpdate&) = default;

// NOLINTNEXTLINE
InterventionStatusUpdate::InterventionStatusUpdate(InterventionStatusUpdate&&) = default;

InterventionStatusUpdate& InterventionStatusUpdate::operator=(const InterventionStatusUpdate&) = default;

// NOLINTNEXTLINE
InterventionStatusUpdate& InterventionStatusUpdate::operator=(InterventionStatusUpdate&&) = default;

void InterventionStatusUpdate::toCsv(std::ostream& out) const
{
    out.precision(16);
    out << std::boolalpha;
    out << timestamp;out << ",";
    out << static_cast<uint32_t>(status);out << ",";
    out << "\"";
        out << assistanceRequestId;
        out << "\"";
    
}

std::stringstream InterventionStatusUpdate::toStringstream() const
{
    std::stringstream out;
    toCsv(out);
    return out;
}

template<class Archive>
    void InterventionStatusUpdate::serialize(Archive& ar, [[maybe_unused]] unsigned int version)
    {

        ar& CEREAL_NVP(timestamp);
        ar& CEREAL_NVP(status);
        ar& CEREAL_NVP(assistanceRequestId);
    }


template void InterventionStatusUpdate::serialize(cereal::BinaryInputArchive& ar, unsigned int version);
template void InterventionStatusUpdate::serialize(cereal::BinaryOutputArchive& ar, unsigned int version);
template void InterventionStatusUpdate::serialize(cereal::IntrospectionArchive& ar, unsigned int version);
template void InterventionStatusUpdate::serialize(cereal::JSONInputArchive& ar, unsigned int version);
template void InterventionStatusUpdate::serialize(cereal::JSONOutputArchive& ar, unsigned int version);
template void InterventionStatusUpdate::serialize(cereal::PortableBinaryInputArchive& ar, unsigned int version);
template void InterventionStatusUpdate::serialize(cereal::PortableBinaryOutputArchive& ar, unsigned int version);

std::ostream& operator<< (std::ostream& os, const InterventionStatusUpdate& interventionstatusupdate)
{
    interventionstatusupdate.toCsv(os);
    return os;
}

bool operator== (const InterventionStatusUpdate &lhs, const InterventionStatusUpdate &rhs)
{
    return (  lhs.timestamp== rhs.timestamp &&
     lhs.status== rhs.status &&
     lhs.assistanceRequestId== rhs.assistanceRequestId 
    );
}

std::size_t InterventionStatusUpdate::hash() const noexcept
{
    std::size_t result = 17;
    result = result * 31 + std::hash<av::msg::TimePoint>()(timestamp);
    result = result * 31 + std::hash<av::msg::InterventionStatusUpdateType>()(status);
    result = result * 31 + std::hash<std::string>()(assistanceRequestId);
    return result;
}

 } // namespace av
 } // namespace msg


CEREAL_BIND_TO_ARCHIVES(av::msg::InterventionStatusUpdate)

