#include <interfaces/KingResponse/KingResponse.h>

#include <iterator>
#include <type_traits>

#include <cereal/cereal.hpp>
#include <cereal/archives/json.hpp>
#include <cereal/archives/binary.hpp>

#include <cereal/types/map.hpp>
#include <cereal/types/vector.hpp>
#include <cereal/types/array.hpp>
#include <cereal/types/string.hpp>
#include <cereal/types/complex.hpp>

#include <ODatum/IntrospectionArchive.h>
#include <ODatum/Hash.h>


 namespace av {
 namespace msg {


ConfigDirectory::ConfigDirectory() = default;

ConfigDirectory::ConfigDirectory(
    std::string config__arg ,
    std::vector<std::string> scenarios__arg 
    ):
    config_ { std::move(config__arg) } // NOLINT
      ,
    scenarios_ { std::move(scenarios__arg) } // NOLINT
      
    {
}

ConfigDirectory::ConfigDirectory(const ConfigDirectory&) = default;

// NOLINTNEXTLINE
ConfigDirectory::ConfigDirectory(ConfigDirectory&&) = default;

ConfigDirectory& ConfigDirectory::operator=(const ConfigDirectory&) = default;

// NOLINTNEXTLINE
ConfigDirectory& ConfigDirectory::operator=(ConfigDirectory&&) = default;

void ConfigDirectory::toCsv(std::ostream& out) const
{
    out.precision(16);
    out << std::boolalpha;
    out << "\"";
        out << config_;
        out << "\"";out << ",";
    // Enclose sequence in double quotations and curly brackets to keep CSV line length uniform
        
        out << "\"{{";
        
        if (!scenarios_.empty())
        {
            std::copy(scenarios_.begin(), scenarios_.end() - 1, std::ostream_iterator<std::string>(out, "},{"));
            std::copy(scenarios_.end()-1, scenarios_.end(), std::ostream_iterator<std::string>(out, ""));
        }
        out << "}}\"";
    
}

std::stringstream ConfigDirectory::toStringstream() const
{
    std::stringstream out;
    toCsv(out);
    return out;
}

template<class Archive>
    void ConfigDirectory::serialize(Archive& ar, [[maybe_unused]] unsigned int version)
    {

        ar& CEREAL_NVP(config_);
        ar& CEREAL_NVP(scenarios_);
    }


template void ConfigDirectory::serialize(cereal::BinaryInputArchive& ar, unsigned int version);
template void ConfigDirectory::serialize(cereal::BinaryOutputArchive& ar, unsigned int version);
template void ConfigDirectory::serialize(cereal::IntrospectionArchive& ar, unsigned int version);
template void ConfigDirectory::serialize(cereal::JSONInputArchive& ar, unsigned int version);
template void ConfigDirectory::serialize(cereal::JSONOutputArchive& ar, unsigned int version);
template void ConfigDirectory::serialize(cereal::PortableBinaryInputArchive& ar, unsigned int version);
template void ConfigDirectory::serialize(cereal::PortableBinaryOutputArchive& ar, unsigned int version);

std::ostream& operator<< (std::ostream& os, const ConfigDirectory& configdirectory)
{
    configdirectory.toCsv(os);
    return os;
}

bool operator== (const ConfigDirectory &lhs, const ConfigDirectory &rhs)
{
    return (  lhs.config_== rhs.config_ &&
     lhs.scenarios_== rhs.scenarios_ 
    );
}

std::size_t ConfigDirectory::hash() const noexcept
{
    std::size_t result = 17;
    result = result * 31 + std::hash<std::string>()(config_);
    result = result * 31 + hashVector(scenarios_);
    return result;
}

 } // namespace av
 } // namespace msg


CEREAL_BIND_TO_ARCHIVES(av::msg::ConfigDirectory)




 namespace av {
 namespace msg {


KingResponse::KingResponse() = default;

KingResponse::KingResponse(
    MessageId messageType__arg ,
    std::string sender__arg ,
    std::string message__arg ,
    std::string recipient__arg ,
    std::vector<ConfigDirectory> configDir__arg 
    ):
    messageType_ { std::move(messageType__arg) } // NOLINT
      ,sender_ { std::move(sender__arg) } // NOLINT
      ,message_ { std::move(message__arg) } // NOLINT
      ,recipient_ { std::move(recipient__arg) } // NOLINT
      ,configDir_ { std::move(configDir__arg) } // NOLINT
      {
}

KingResponse::KingResponse(const KingResponse&) = default;

// NOLINTNEXTLINE
KingResponse::KingResponse(KingResponse&&) = default;

KingResponse& KingResponse::operator=(const KingResponse&) = default;

// NOLINTNEXTLINE
KingResponse& KingResponse::operator=(KingResponse&&) = default;

void KingResponse::toCsv(std::ostream& out) const
{
    out.precision(16);
    out << std::boolalpha;
    out << static_cast<uint32_t>(messageType_);out << ",";
    out << "\"";
        out << sender_;
        out << "\"";out << ",";
    out << "\"";
        out << message_;
        out << "\"";out << ",";
    out << "\"";
        out << recipient_;
        out << "\"";out << ",";
    // Enclose sequence in double quotations and curly brackets to keep CSV line length uniform
        
        out << "\"{{";
        
        if (!configDir_.empty())
        {
            std::copy(configDir_.begin(), configDir_.end() - 1, std::ostream_iterator<ConfigDirectory>(out, "},{"));
            std::copy(configDir_.end()-1, configDir_.end(), std::ostream_iterator<ConfigDirectory>(out, ""));
        }
        out << "}}\"";
    
}

std::stringstream KingResponse::toStringstream() const
{
    std::stringstream out;
    toCsv(out);
    return out;
}

template<class Archive>
    void KingResponse::serialize(Archive& ar, [[maybe_unused]] unsigned int version)
    {

        ar& CEREAL_NVP(messageType_);
        ar& CEREAL_NVP(sender_);
        ar& CEREAL_NVP(message_);
        ar& CEREAL_NVP(recipient_);
        ar& CEREAL_NVP(configDir_);
    }


template void KingResponse::serialize(cereal::BinaryInputArchive& ar, unsigned int version);
template void KingResponse::serialize(cereal::BinaryOutputArchive& ar, unsigned int version);
template void KingResponse::serialize(cereal::IntrospectionArchive& ar, unsigned int version);
template void KingResponse::serialize(cereal::JSONInputArchive& ar, unsigned int version);
template void KingResponse::serialize(cereal::JSONOutputArchive& ar, unsigned int version);
template void KingResponse::serialize(cereal::PortableBinaryInputArchive& ar, unsigned int version);
template void KingResponse::serialize(cereal::PortableBinaryOutputArchive& ar, unsigned int version);

std::ostream& operator<< (std::ostream& os, const KingResponse& kingresponse)
{
    kingresponse.toCsv(os);
    return os;
}

bool operator== (const KingResponse &lhs, const KingResponse &rhs)
{
    return (  lhs.messageType_== rhs.messageType_ &&
     lhs.sender_== rhs.sender_ &&
     lhs.message_== rhs.message_ &&
     lhs.recipient_== rhs.recipient_ &&
     lhs.configDir_== rhs.configDir_ 
    );
}

std::size_t KingResponse::hash() const noexcept
{
    std::size_t result = 17;
    result = result * 31 + std::hash<MessageId>()(messageType_);
    result = result * 31 + std::hash<std::string>()(sender_);
    result = result * 31 + std::hash<std::string>()(message_);
    result = result * 31 + std::hash<std::string>()(recipient_);
    result = result * 31 + hashVector(configDir_);
    return result;
}

 } // namespace av
 } // namespace msg


CEREAL_BIND_TO_ARCHIVES(av::msg::KingResponse)

