#include <interfaces/LaneChangeEarlyWarning/LaneChangeEarlyWarning.h>

#include <iterator>
#include <type_traits>

#include <cereal/cereal.hpp>
#include <cereal/archives/json.hpp>
#include <cereal/archives/binary.hpp>

#include <cereal/types/map.hpp>
#include <cereal/types/vector.hpp>
#include <cereal/types/array.hpp>
#include <cereal/types/string.hpp>
#include <cereal/types/complex.hpp>

#include <ODatum/IntrospectionArchive.h>
#include <ODatum/Hash.h>





 namespace av {
 namespace msg {


LaneChangeEarlyWarning::LaneChangeEarlyWarning() = default;

LaneChangeEarlyWarning::LaneChangeEarlyWarning(
    LaneChangeEarlyWarningMode mode_arg ,
    LaneChangeDirection direction_arg ,
    Duration timeBeforeActiveLaneChangeEnabled_arg ,
    TimePoint preparingLaneChangeFromTimestamp_arg ,
    TimePoint disableLaneChangeTimestamp_arg ,
    bool isAttemptingLaneChange_arg 
    ):
    mode { std::move(mode_arg) } // NOLINT
      ,direction { std::move(direction_arg) } // NOLINT
      ,timeBeforeActiveLaneChangeEnabled { std::move(timeBeforeActiveLaneChangeEnabled_arg) } // NOLINT
      ,preparingLaneChangeFromTimestamp { std::move(preparingLaneChangeFromTimestamp_arg) } // NOLINT
      ,disableLaneChangeTimestamp { std::move(disableLaneChangeTimestamp_arg) } // NOLINT
      ,isAttemptingLaneChange { std::move(isAttemptingLaneChange_arg) } // NOLINT
      {
}

LaneChangeEarlyWarning::LaneChangeEarlyWarning(const LaneChangeEarlyWarning&) = default;

// NOLINTNEXTLINE
LaneChangeEarlyWarning::LaneChangeEarlyWarning(LaneChangeEarlyWarning&&) = default;

LaneChangeEarlyWarning& LaneChangeEarlyWarning::operator=(const LaneChangeEarlyWarning&) = default;

// NOLINTNEXTLINE
LaneChangeEarlyWarning& LaneChangeEarlyWarning::operator=(LaneChangeEarlyWarning&&) = default;

void LaneChangeEarlyWarning::toCsv(std::ostream& out) const
{
    out.precision(16);
    out << std::boolalpha;
    out << static_cast<uint32_t>(mode);out << ",";
    out << static_cast<uint32_t>(direction);out << ",";
    out << timeBeforeActiveLaneChangeEnabled;out << ",";
    out << preparingLaneChangeFromTimestamp;out << ",";
    out << disableLaneChangeTimestamp;out << ",";
    out << isAttemptingLaneChange;
    
}

std::stringstream LaneChangeEarlyWarning::toStringstream() const
{
    std::stringstream out;
    toCsv(out);
    return out;
}

template<class Archive>
    void LaneChangeEarlyWarning::serialize(Archive& ar, unsigned int version)
{
	if (version >= 1)
	{
		ar& CEREAL_NVP(mode);
		ar& CEREAL_NVP(direction);
		ar& CEREAL_NVP(timeBeforeActiveLaneChangeEnabled);
		ar& CEREAL_NVP(preparingLaneChangeFromTimestamp);
		ar& CEREAL_NVP(disableLaneChangeTimestamp);
	}
	if (version >= 2)
	{
		ar& CEREAL_NVP(isAttemptingLaneChange);
	}
}



template void LaneChangeEarlyWarning::serialize(cereal::BinaryInputArchive& ar, unsigned int version);
template void LaneChangeEarlyWarning::serialize(cereal::BinaryOutputArchive& ar, unsigned int version);
template void LaneChangeEarlyWarning::serialize(cereal::IntrospectionArchive& ar, unsigned int version);
template void LaneChangeEarlyWarning::serialize(cereal::JSONInputArchive& ar, unsigned int version);
template void LaneChangeEarlyWarning::serialize(cereal::JSONOutputArchive& ar, unsigned int version);
template void LaneChangeEarlyWarning::serialize(cereal::PortableBinaryInputArchive& ar, unsigned int version);
template void LaneChangeEarlyWarning::serialize(cereal::PortableBinaryOutputArchive& ar, unsigned int version);

std::ostream& operator<< (std::ostream& os, const LaneChangeEarlyWarning& lanechangeearlywarning)
{
    lanechangeearlywarning.toCsv(os);
    return os;
}

bool operator== (const LaneChangeEarlyWarning &lhs, const LaneChangeEarlyWarning &rhs)
{
    return (  lhs.mode== rhs.mode &&
     lhs.direction== rhs.direction &&
     lhs.timeBeforeActiveLaneChangeEnabled== rhs.timeBeforeActiveLaneChangeEnabled &&
     lhs.preparingLaneChangeFromTimestamp== rhs.preparingLaneChangeFromTimestamp &&
     lhs.disableLaneChangeTimestamp== rhs.disableLaneChangeTimestamp &&
     lhs.isAttemptingLaneChange== rhs.isAttemptingLaneChange 
    );
}

std::size_t LaneChangeEarlyWarning::hash() const noexcept
{
    std::size_t result = 17;
    result = result * 31 + std::hash<LaneChangeEarlyWarningMode>()(mode);
    result = result * 31 + std::hash<LaneChangeDirection>()(direction);
    result = result * 31 + 0;
    result = result * 31 + 0;
    result = result * 31 + 0;
    result = result * 31 + std::hash<bool>()(isAttemptingLaneChange);
    return result;
}

 } // namespace av
 } // namespace msg


CEREAL_BIND_TO_ARCHIVES(av::msg::LaneChangeEarlyWarning)

