#include <interfaces/LaneRejectList/LaneRejectList.h>

#include <iterator>
#include <type_traits>

#include <cereal/cereal.hpp>
#include <cereal/archives/json.hpp>
#include <cereal/archives/binary.hpp>

#include <cereal/types/map.hpp>
#include <cereal/types/vector.hpp>
#include <cereal/types/array.hpp>
#include <cereal/types/string.hpp>
#include <cereal/types/complex.hpp>

#include <ODatum/IntrospectionArchive.h>
#include <ODatum/Hash.h>


 namespace av {
 namespace msg {


Bounds::Bounds() = default;

Bounds::Bounds(
    double upper_arg ,
    double lower_arg 
    ):
    upper { std::move(upper_arg) } // NOLINT
      ,
    lower { std::move(lower_arg) } // NOLINT
      
    {
}

Bounds::Bounds(const Bounds&) = default;

// NOLINTNEXTLINE
Bounds::Bounds(Bounds&&) = default;

Bounds& Bounds::operator=(const Bounds&) = default;

// NOLINTNEXTLINE
Bounds& Bounds::operator=(Bounds&&) = default;

void Bounds::toCsv(std::ostream& out) const
{
    out.precision(16);
    out << std::boolalpha;
    out << upper;out << ",";
    out << lower;
    
}

std::stringstream Bounds::toStringstream() const
{
    std::stringstream out;
    toCsv(out);
    return out;
}

template<class Archive>
    void Bounds::serialize(Archive& ar, [[maybe_unused]] unsigned int version)
    {

        ar& CEREAL_NVP(upper);
        ar& CEREAL_NVP(lower);
    }


template void Bounds::serialize(cereal::BinaryInputArchive& ar, unsigned int version);
template void Bounds::serialize(cereal::BinaryOutputArchive& ar, unsigned int version);
template void Bounds::serialize(cereal::IntrospectionArchive& ar, unsigned int version);
template void Bounds::serialize(cereal::JSONInputArchive& ar, unsigned int version);
template void Bounds::serialize(cereal::JSONOutputArchive& ar, unsigned int version);
template void Bounds::serialize(cereal::PortableBinaryInputArchive& ar, unsigned int version);
template void Bounds::serialize(cereal::PortableBinaryOutputArchive& ar, unsigned int version);

std::ostream& operator<< (std::ostream& os, const Bounds& bounds)
{
    bounds.toCsv(os);
    return os;
}

bool operator== (const Bounds &lhs, const Bounds &rhs)
{
    return (  lhs.upper== rhs.upper &&
     lhs.lower== rhs.lower 
    );
}

std::size_t Bounds::hash() const noexcept
{
    std::size_t result = 17;
    result = result * 31 + std::hash<double>()(upper);
    result = result * 31 + std::hash<double>()(lower);
    return result;
}

 } // namespace av
 } // namespace msg


CEREAL_BIND_TO_ARCHIVES(av::msg::Bounds)

 namespace av {
 namespace msg {


RejectObject::RejectObject() = default;

RejectObject::RejectObject(
    MapId rejectObjectId_arg ,
    std::vector<Bounds> rejectBounds_arg 
    ):
    rejectObjectId { std::move(rejectObjectId_arg) } // NOLINT
      ,
    rejectBounds { std::move(rejectBounds_arg) } // NOLINT
      
    {
}

RejectObject::RejectObject(const RejectObject&) = default;

// NOLINTNEXTLINE
RejectObject::RejectObject(RejectObject&&) = default;

RejectObject& RejectObject::operator=(const RejectObject&) = default;

// NOLINTNEXTLINE
RejectObject& RejectObject::operator=(RejectObject&&) = default;

void RejectObject::toCsv(std::ostream& out) const
{
    out.precision(16);
    out << std::boolalpha;
    out << rejectObjectId;out << ",";
    // Enclose sequence in double quotations and curly brackets to keep CSV line length uniform
        
        out << "\"{{";
        
        if (!rejectBounds.empty())
        {
            std::copy(rejectBounds.begin(), rejectBounds.end() - 1, std::ostream_iterator<Bounds>(out, "},{"));
            std::copy(rejectBounds.end()-1, rejectBounds.end(), std::ostream_iterator<Bounds>(out, ""));
        }
        out << "}}\"";
    
}

std::stringstream RejectObject::toStringstream() const
{
    std::stringstream out;
    toCsv(out);
    return out;
}

template<class Archive>
    void RejectObject::serialize(Archive& ar, [[maybe_unused]] unsigned int version)
    {

        ar& CEREAL_NVP(rejectObjectId);
        ar& CEREAL_NVP(rejectBounds);
    }


template void RejectObject::serialize(cereal::BinaryInputArchive& ar, unsigned int version);
template void RejectObject::serialize(cereal::BinaryOutputArchive& ar, unsigned int version);
template void RejectObject::serialize(cereal::IntrospectionArchive& ar, unsigned int version);
template void RejectObject::serialize(cereal::JSONInputArchive& ar, unsigned int version);
template void RejectObject::serialize(cereal::JSONOutputArchive& ar, unsigned int version);
template void RejectObject::serialize(cereal::PortableBinaryInputArchive& ar, unsigned int version);
template void RejectObject::serialize(cereal::PortableBinaryOutputArchive& ar, unsigned int version);

std::ostream& operator<< (std::ostream& os, const RejectObject& rejectobject)
{
    rejectobject.toCsv(os);
    return os;
}

bool operator== (const RejectObject &lhs, const RejectObject &rhs)
{
    return (  lhs.rejectObjectId== rhs.rejectObjectId &&
     lhs.rejectBounds== rhs.rejectBounds 
    );
}

std::size_t RejectObject::hash() const noexcept
{
    std::size_t result = 17;
    result = result * 31 + std::hash<MapId>()(rejectObjectId);
    result = result * 31 + hashVector(rejectBounds);
    return result;
}

 } // namespace av
 } // namespace msg


CEREAL_BIND_TO_ARCHIVES(av::msg::RejectObject)




 namespace av {
 namespace msg {


LaneRejectList::LaneRejectList() = default;

LaneRejectList::LaneRejectList(
    std::vector<RejectObject> rejectObjects_arg 
    ):
    rejectObjects { std::move(rejectObjects_arg) } // NOLINT
      {
}

LaneRejectList::LaneRejectList(const LaneRejectList&) = default;

// NOLINTNEXTLINE
LaneRejectList::LaneRejectList(LaneRejectList&&) = default;

LaneRejectList& LaneRejectList::operator=(const LaneRejectList&) = default;

// NOLINTNEXTLINE
LaneRejectList& LaneRejectList::operator=(LaneRejectList&&) = default;

void LaneRejectList::toCsv(std::ostream& out) const
{
    out.precision(16);
    out << std::boolalpha;
    // Enclose sequence in double quotations and curly brackets to keep CSV line length uniform
        
        out << "\"{{";
        
        if (!rejectObjects.empty())
        {
            std::copy(rejectObjects.begin(), rejectObjects.end() - 1, std::ostream_iterator<RejectObject>(out, "},{"));
            std::copy(rejectObjects.end()-1, rejectObjects.end(), std::ostream_iterator<RejectObject>(out, ""));
        }
        out << "}}\"";
    
}

std::stringstream LaneRejectList::toStringstream() const
{
    std::stringstream out;
    toCsv(out);
    return out;
}

template<class Archive>
    void LaneRejectList::serialize(Archive& ar, [[maybe_unused]] unsigned int version)
    {

        ar& CEREAL_NVP(rejectObjects);
    }


template void LaneRejectList::serialize(cereal::BinaryInputArchive& ar, unsigned int version);
template void LaneRejectList::serialize(cereal::BinaryOutputArchive& ar, unsigned int version);
template void LaneRejectList::serialize(cereal::IntrospectionArchive& ar, unsigned int version);
template void LaneRejectList::serialize(cereal::JSONInputArchive& ar, unsigned int version);
template void LaneRejectList::serialize(cereal::JSONOutputArchive& ar, unsigned int version);
template void LaneRejectList::serialize(cereal::PortableBinaryInputArchive& ar, unsigned int version);
template void LaneRejectList::serialize(cereal::PortableBinaryOutputArchive& ar, unsigned int version);

std::ostream& operator<< (std::ostream& os, const LaneRejectList& lanerejectlist)
{
    lanerejectlist.toCsv(os);
    return os;
}

bool operator== (const LaneRejectList &lhs, const LaneRejectList &rhs)
{
    return (  lhs.rejectObjects== rhs.rejectObjects 
    );
}

std::size_t LaneRejectList::hash() const noexcept
{
    std::size_t result = 17;
    result = result * 31 + hashVector(rejectObjects);
    return result;
}

 } // namespace av
 } // namespace msg


CEREAL_BIND_TO_ARCHIVES(av::msg::LaneRejectList)

