#include <interfaces/LaneRouterArtifacts/LaneRouterArtifacts.h>

#include <iterator>
#include <type_traits>

#include <cereal/cereal.hpp>
#include <cereal/archives/json.hpp>
#include <cereal/archives/binary.hpp>

#include <cereal/types/map.hpp>
#include <cereal/types/vector.hpp>
#include <cereal/types/array.hpp>
#include <cereal/types/string.hpp>
#include <cereal/types/complex.hpp>

#include <ODatum/IntrospectionArchive.h>
#include <ODatum/Hash.h>


 namespace av {
 namespace msg {


EgoVertex::EgoVertex() = default;

EgoVertex::EgoVertex(
    MapId id_arg 
    ):
    id { std::move(id_arg) } // NOLINT
      
    {
}

EgoVertex::EgoVertex(const EgoVertex&) = default;

// NOLINTNEXTLINE
EgoVertex::EgoVertex(EgoVertex&&) = default;

EgoVertex& EgoVertex::operator=(const EgoVertex&) = default;

// NOLINTNEXTLINE
EgoVertex& EgoVertex::operator=(EgoVertex&&) = default;

void EgoVertex::toCsv(std::ostream& out) const
{
    out.precision(16);
    out << std::boolalpha;
    out << id;
    
}

std::stringstream EgoVertex::toStringstream() const
{
    std::stringstream out;
    toCsv(out);
    return out;
}

template<class Archive>
    void EgoVertex::serialize(Archive& ar, [[maybe_unused]] unsigned int version)
    {

        ar& CEREAL_NVP(id);
    }


template void EgoVertex::serialize(cereal::BinaryInputArchive& ar, unsigned int version);
template void EgoVertex::serialize(cereal::BinaryOutputArchive& ar, unsigned int version);
template void EgoVertex::serialize(cereal::IntrospectionArchive& ar, unsigned int version);
template void EgoVertex::serialize(cereal::JSONInputArchive& ar, unsigned int version);
template void EgoVertex::serialize(cereal::JSONOutputArchive& ar, unsigned int version);
template void EgoVertex::serialize(cereal::PortableBinaryInputArchive& ar, unsigned int version);
template void EgoVertex::serialize(cereal::PortableBinaryOutputArchive& ar, unsigned int version);

std::ostream& operator<< (std::ostream& os, const EgoVertex& egovertex)
{
    egovertex.toCsv(os);
    return os;
}

bool operator== (const EgoVertex &lhs, const EgoVertex &rhs)
{
    return (  lhs.id== rhs.id 
    );
}

std::size_t EgoVertex::hash() const noexcept
{
    std::size_t result = 17;
    result = result * 31 + std::hash<MapId>()(id);
    return result;
}

 } // namespace av
 } // namespace msg


CEREAL_BIND_TO_ARCHIVES(av::msg::EgoVertex)

 namespace av {
 namespace msg {


GoalVertex::GoalVertex() = default;

GoalVertex::GoalVertex(
    MapId id_arg 
    ):
    id { std::move(id_arg) } // NOLINT
      
    {
}

GoalVertex::GoalVertex(const GoalVertex&) = default;

// NOLINTNEXTLINE
GoalVertex::GoalVertex(GoalVertex&&) = default;

GoalVertex& GoalVertex::operator=(const GoalVertex&) = default;

// NOLINTNEXTLINE
GoalVertex& GoalVertex::operator=(GoalVertex&&) = default;

void GoalVertex::toCsv(std::ostream& out) const
{
    out.precision(16);
    out << std::boolalpha;
    out << id;
    
}

std::stringstream GoalVertex::toStringstream() const
{
    std::stringstream out;
    toCsv(out);
    return out;
}

template<class Archive>
    void GoalVertex::serialize(Archive& ar, [[maybe_unused]] unsigned int version)
    {

        ar& CEREAL_NVP(id);
    }


template void GoalVertex::serialize(cereal::BinaryInputArchive& ar, unsigned int version);
template void GoalVertex::serialize(cereal::BinaryOutputArchive& ar, unsigned int version);
template void GoalVertex::serialize(cereal::IntrospectionArchive& ar, unsigned int version);
template void GoalVertex::serialize(cereal::JSONInputArchive& ar, unsigned int version);
template void GoalVertex::serialize(cereal::JSONOutputArchive& ar, unsigned int version);
template void GoalVertex::serialize(cereal::PortableBinaryInputArchive& ar, unsigned int version);
template void GoalVertex::serialize(cereal::PortableBinaryOutputArchive& ar, unsigned int version);

std::ostream& operator<< (std::ostream& os, const GoalVertex& goalvertex)
{
    goalvertex.toCsv(os);
    return os;
}

bool operator== (const GoalVertex &lhs, const GoalVertex &rhs)
{
    return (  lhs.id== rhs.id 
    );
}

std::size_t GoalVertex::hash() const noexcept
{
    std::size_t result = 17;
    result = result * 31 + std::hash<MapId>()(id);
    return result;
}

 } // namespace av
 } // namespace msg


CEREAL_BIND_TO_ARCHIVES(av::msg::GoalVertex)




 namespace av {
 namespace msg {


LaneRouterArtifacts::LaneRouterArtifacts() = default;

LaneRouterArtifacts::LaneRouterArtifacts(
    bool hasMissionRoute_arg ,
    MissionRoute missionRoute_arg ,
    bool hasMissionStatus_arg ,
    MissionRouteAcceptanceStatus missionStatus_arg ,
    bool hasEgoPose_arg ,
    SE2 egoPose_arg ,
    bool hasEgoVertex_arg ,
    EgoVertex egoVertex_arg ,
    bool hasGoalVertex_arg ,
    GoalVertex goalVertex_arg ,
    bool hasLaneGraph_arg ,
    LaneGraph laneGraph_arg ,
    bool hasVertexRoute_arg ,
    VertexRoute vertexRoute_arg ,
    bool hasRoutePlan_arg ,
    plan_ctrl::RoutePlan routePlan_arg 
    ):
    hasMissionRoute { std::move(hasMissionRoute_arg) } // NOLINT
      ,missionRoute { std::move(missionRoute_arg) } // NOLINT
      ,hasMissionStatus { std::move(hasMissionStatus_arg) } // NOLINT
      ,missionStatus { std::move(missionStatus_arg) } // NOLINT
      ,hasEgoPose { std::move(hasEgoPose_arg) } // NOLINT
      ,egoPose { std::move(egoPose_arg) } // NOLINT
      ,hasEgoVertex { std::move(hasEgoVertex_arg) } // NOLINT
      ,egoVertex { std::move(egoVertex_arg) } // NOLINT
      ,hasGoalVertex { std::move(hasGoalVertex_arg) } // NOLINT
      ,goalVertex { std::move(goalVertex_arg) } // NOLINT
      ,hasLaneGraph { std::move(hasLaneGraph_arg) } // NOLINT
      ,laneGraph { std::move(laneGraph_arg) } // NOLINT
      ,hasVertexRoute { std::move(hasVertexRoute_arg) } // NOLINT
      ,vertexRoute { std::move(vertexRoute_arg) } // NOLINT
      ,hasRoutePlan { std::move(hasRoutePlan_arg) } // NOLINT
      ,routePlan { std::move(routePlan_arg) } // NOLINT
      {
}

LaneRouterArtifacts::LaneRouterArtifacts(const LaneRouterArtifacts&) = default;

// NOLINTNEXTLINE
LaneRouterArtifacts::LaneRouterArtifacts(LaneRouterArtifacts&&) = default;

LaneRouterArtifacts& LaneRouterArtifacts::operator=(const LaneRouterArtifacts&) = default;

// NOLINTNEXTLINE
LaneRouterArtifacts& LaneRouterArtifacts::operator=(LaneRouterArtifacts&&) = default;

void LaneRouterArtifacts::toCsv(std::ostream& out) const
{
    out.precision(16);
    out << std::boolalpha;
    out << hasMissionRoute;out << ",";
    out << missionRoute;out << ",";
    out << hasMissionStatus;out << ",";
    out << missionStatus;out << ",";
    out << hasEgoPose;out << ",";
    out << egoPose;out << ",";
    out << hasEgoVertex;out << ",";
    out << egoVertex;out << ",";
    out << hasGoalVertex;out << ",";
    out << goalVertex;out << ",";
    out << hasLaneGraph;out << ",";
    out << laneGraph;out << ",";
    out << hasVertexRoute;out << ",";
    out << vertexRoute;out << ",";
    out << hasRoutePlan;out << ",";
    out << routePlan;
    
}

std::stringstream LaneRouterArtifacts::toStringstream() const
{
    std::stringstream out;
    toCsv(out);
    return out;
}

template<class Archive>
    void LaneRouterArtifacts::serialize(Archive& ar, [[maybe_unused]] unsigned int version)
    {

        ar& CEREAL_NVP(hasMissionRoute);
        ar& CEREAL_NVP(missionRoute);
        ar& CEREAL_NVP(hasMissionStatus);
        ar& CEREAL_NVP(missionStatus);
        ar& CEREAL_NVP(hasEgoPose);
        ar& CEREAL_NVP(egoPose);
        ar& CEREAL_NVP(hasEgoVertex);
        ar& CEREAL_NVP(egoVertex);
        ar& CEREAL_NVP(hasGoalVertex);
        ar& CEREAL_NVP(goalVertex);
        ar& CEREAL_NVP(hasLaneGraph);
        ar& CEREAL_NVP(laneGraph);
        ar& CEREAL_NVP(hasVertexRoute);
        ar& CEREAL_NVP(vertexRoute);
        ar& CEREAL_NVP(hasRoutePlan);
        ar& CEREAL_NVP(routePlan);
    }


template void LaneRouterArtifacts::serialize(cereal::BinaryInputArchive& ar, unsigned int version);
template void LaneRouterArtifacts::serialize(cereal::BinaryOutputArchive& ar, unsigned int version);
template void LaneRouterArtifacts::serialize(cereal::IntrospectionArchive& ar, unsigned int version);
template void LaneRouterArtifacts::serialize(cereal::JSONInputArchive& ar, unsigned int version);
template void LaneRouterArtifacts::serialize(cereal::JSONOutputArchive& ar, unsigned int version);
template void LaneRouterArtifacts::serialize(cereal::PortableBinaryInputArchive& ar, unsigned int version);
template void LaneRouterArtifacts::serialize(cereal::PortableBinaryOutputArchive& ar, unsigned int version);

std::ostream& operator<< (std::ostream& os, const LaneRouterArtifacts& lanerouterartifacts)
{
    lanerouterartifacts.toCsv(os);
    return os;
}

bool operator== (const LaneRouterArtifacts &lhs, const LaneRouterArtifacts &rhs)
{
    return (  lhs.hasMissionRoute== rhs.hasMissionRoute &&
     lhs.missionRoute== rhs.missionRoute &&
     lhs.hasMissionStatus== rhs.hasMissionStatus &&
     lhs.missionStatus== rhs.missionStatus &&
     lhs.hasEgoPose== rhs.hasEgoPose &&
     lhs.egoPose== rhs.egoPose &&
     lhs.hasEgoVertex== rhs.hasEgoVertex &&
     lhs.egoVertex== rhs.egoVertex &&
     lhs.hasGoalVertex== rhs.hasGoalVertex &&
     lhs.goalVertex== rhs.goalVertex &&
     lhs.hasLaneGraph== rhs.hasLaneGraph &&
     lhs.laneGraph== rhs.laneGraph &&
     lhs.hasVertexRoute== rhs.hasVertexRoute &&
     lhs.vertexRoute== rhs.vertexRoute &&
     lhs.hasRoutePlan== rhs.hasRoutePlan &&
     lhs.routePlan== rhs.routePlan 
    );
}

std::size_t LaneRouterArtifacts::hash() const noexcept
{
    std::size_t result = 17;
    result = result * 31 + std::hash<bool>()(hasMissionRoute);
    result = result * 31 + std::hash<MissionRoute>()(missionRoute);
    result = result * 31 + std::hash<bool>()(hasMissionStatus);
    result = result * 31 + std::hash<MissionRouteAcceptanceStatus>()(missionStatus);
    result = result * 31 + std::hash<bool>()(hasEgoPose);
    result = result * 31 + std::hash<SE2>()(egoPose);
    result = result * 31 + std::hash<bool>()(hasEgoVertex);
    result = result * 31 + std::hash<EgoVertex>()(egoVertex);
    result = result * 31 + std::hash<bool>()(hasGoalVertex);
    result = result * 31 + std::hash<GoalVertex>()(goalVertex);
    result = result * 31 + std::hash<bool>()(hasLaneGraph);
    result = result * 31 + std::hash<LaneGraph>()(laneGraph);
    result = result * 31 + std::hash<bool>()(hasVertexRoute);
    result = result * 31 + std::hash<VertexRoute>()(vertexRoute);
    result = result * 31 + std::hash<bool>()(hasRoutePlan);
    result = result * 31 + std::hash<plan_ctrl::RoutePlan>()(routePlan);
    return result;
}

 } // namespace av
 } // namespace msg


CEREAL_BIND_TO_ARCHIVES(av::msg::LaneRouterArtifacts)

