#include <interfaces/LatencyEventMessage/LatencyEventMessage.h>

#include <iterator>
#include <type_traits>

#include <cereal/cereal.hpp>
#include <cereal/archives/json.hpp>
#include <cereal/archives/binary.hpp>

#include <cereal/types/map.hpp>
#include <cereal/types/vector.hpp>
#include <cereal/types/array.hpp>
#include <cereal/types/string.hpp>
#include <cereal/types/complex.hpp>

#include <ODatum/IntrospectionArchive.h>
#include <ODatum/Hash.h>





 namespace av {
 namespace msg {


LatencyEventMessage::LatencyEventMessage() = default;

LatencyEventMessage::LatencyEventMessage(
    std::string loggingName_arg ,
    std::string checkpointName_arg ,
    int64_t eventTimestamp_arg ,
    uint64_t eventID_arg ,
    std::vector<uint64_t> predecessorIds_arg 
    ):
    loggingName { std::move(loggingName_arg) } // NOLINT
      ,checkpointName { std::move(checkpointName_arg) } // NOLINT
      ,eventTimestamp { std::move(eventTimestamp_arg) } // NOLINT
      ,eventID { std::move(eventID_arg) } // NOLINT
      ,predecessorIds { std::move(predecessorIds_arg) } // NOLINT
      {
}

LatencyEventMessage::LatencyEventMessage(const LatencyEventMessage&) = default;

// NOLINTNEXTLINE
LatencyEventMessage::LatencyEventMessage(LatencyEventMessage&&) = default;

LatencyEventMessage& LatencyEventMessage::operator=(const LatencyEventMessage&) = default;

// NOLINTNEXTLINE
LatencyEventMessage& LatencyEventMessage::operator=(LatencyEventMessage&&) = default;

void LatencyEventMessage::toCsv(std::ostream& out) const
{
    out.precision(16);
    out << std::boolalpha;
    out << "\"";
        out << loggingName;
        out << "\"";out << ",";
    out << "\"";
        out << checkpointName;
        out << "\"";out << ",";
    out << eventTimestamp;out << ",";
    out << eventID;out << ",";
    // Enclose sequence in double quotations and curly brackets to keep CSV line length uniform
        
        out << "\"{{";
        
        if (!predecessorIds.empty())
        {
            std::copy(predecessorIds.begin(), predecessorIds.end() - 1, std::ostream_iterator<uint64_t>(out, "},{"));
            std::copy(predecessorIds.end()-1, predecessorIds.end(), std::ostream_iterator<uint64_t>(out, ""));
        }
        out << "}}\"";
    
}

std::stringstream LatencyEventMessage::toStringstream() const
{
    std::stringstream out;
    toCsv(out);
    return out;
}

template<class Archive>
    void LatencyEventMessage::serialize(Archive& ar, unsigned int version)
{
    if (version == 1) {
        throw std::runtime_error("Conversion of LatencyEventMessage (v < 2) not supported");
    }
    ar & CEREAL_NVP(loggingName);
    ar & CEREAL_NVP(checkpointName);
    ar & CEREAL_NVP(eventTimestamp);
    ar & CEREAL_NVP(eventID);
    ar & CEREAL_NVP(predecessorIds);
}



template void LatencyEventMessage::serialize(cereal::BinaryInputArchive& ar, unsigned int version);
template void LatencyEventMessage::serialize(cereal::BinaryOutputArchive& ar, unsigned int version);
template void LatencyEventMessage::serialize(cereal::IntrospectionArchive& ar, unsigned int version);
template void LatencyEventMessage::serialize(cereal::JSONInputArchive& ar, unsigned int version);
template void LatencyEventMessage::serialize(cereal::JSONOutputArchive& ar, unsigned int version);
template void LatencyEventMessage::serialize(cereal::PortableBinaryInputArchive& ar, unsigned int version);
template void LatencyEventMessage::serialize(cereal::PortableBinaryOutputArchive& ar, unsigned int version);

std::ostream& operator<< (std::ostream& os, const LatencyEventMessage& latencyeventmessage)
{
    latencyeventmessage.toCsv(os);
    return os;
}

bool operator== (const LatencyEventMessage &lhs, const LatencyEventMessage &rhs)
{
    return (  lhs.loggingName== rhs.loggingName &&
     lhs.checkpointName== rhs.checkpointName &&
     lhs.eventTimestamp== rhs.eventTimestamp &&
     lhs.eventID== rhs.eventID &&
     lhs.predecessorIds== rhs.predecessorIds 
    );
}

std::size_t LatencyEventMessage::hash() const noexcept
{
    std::size_t result = 17;
    result = result * 31 + std::hash<std::string>()(loggingName);
    result = result * 31 + std::hash<std::string>()(checkpointName);
    result = result * 31 + std::hash<int64_t>()(eventTimestamp);
    result = result * 31 + std::hash<uint64_t>()(eventID);
    result = result * 31 + hashVector(predecessorIds);
    return result;
}

 } // namespace av
 } // namespace msg


CEREAL_BIND_TO_ARCHIVES(av::msg::LatencyEventMessage)

