#include <interfaces/LidarPointCloud/LidarPointCloud.h>

#include <iterator>
#include <type_traits>

#include <cereal/cereal.hpp>
#include <cereal/archives/json.hpp>
#include <cereal/archives/binary.hpp>

#include <cereal/types/map.hpp>
#include <cereal/types/vector.hpp>
#include <cereal/types/array.hpp>
#include <cereal/types/string.hpp>
#include <cereal/types/complex.hpp>

#include <ODatum/IntrospectionArchive.h>
#include <ODatum/Hash.h>





 namespace av {
 namespace msg {


LidarPointCloud::LidarPointCloud() = default;

LidarPointCloud::LidarPointCloud(
    std::string frame_id_arg ,
    uint64_t timestamp_arg ,
    uint32_t num_points_arg ,
    uint8_t num_lidars_arg ,
    std::vector<uint32_t> num_points_in_each_lidar_arg ,
    std::vector<uint16_t> sync_angle_azimuths_arg ,
    std::vector<uint64_t> sync_angle_timestamps_arg ,
    std::vector<uint64_t> camera_fov_timestamps_arg ,
    std::vector<uint8_t> lidars_with_confidence_arg ,
    std::vector<uint8_t> lidars_with_range_arg ,
    std::vector<uint8_t> lidars_with_dual_return_arg ,
    std::vector<float> x_arg ,
    std::vector<float> y_arg ,
    std::vector<float> z_arg ,
    std::vector<uint8_t> intensity_arg ,
    std::vector<uint8_t> ring_arg ,
    std::vector<uint8_t> lidar_info_arg ,
    std::vector<uint8_t> semantic_flag_arg ,
    std::vector<float> azimuth_arg ,
    std::vector<float> range_arg ,
    std::vector<uint8_t> drop_arg ,
    std::vector<uint8_t> retro_shadow_arg ,
    std::vector<uint8_t> range_limited_arg ,
    std::vector<uint8_t> retro_ghost_arg ,
    std::vector<uint8_t> interference_arg ,
    std::vector<uint8_t> sun_level_arg ,
    std::vector<uint8_t> confidence_arg ,
    std::vector<uint8_t> is_second_return_arg ,
    std::vector<std::string> lidars_frame_id_arg 
    ):
    frame_id { std::move(frame_id_arg) } // NOLINT
      ,timestamp { std::move(timestamp_arg) } // NOLINT
      ,num_points { std::move(num_points_arg) } // NOLINT
      ,num_lidars { std::move(num_lidars_arg) } // NOLINT
      ,num_points_in_each_lidar { std::move(num_points_in_each_lidar_arg) } // NOLINT
      ,sync_angle_azimuths { std::move(sync_angle_azimuths_arg) } // NOLINT
      ,sync_angle_timestamps { std::move(sync_angle_timestamps_arg) } // NOLINT
      ,camera_fov_timestamps { std::move(camera_fov_timestamps_arg) } // NOLINT
      ,lidars_with_confidence { std::move(lidars_with_confidence_arg) } // NOLINT
      ,lidars_with_range { std::move(lidars_with_range_arg) } // NOLINT
      ,lidars_with_dual_return { std::move(lidars_with_dual_return_arg) } // NOLINT
      ,x { std::move(x_arg) } // NOLINT
      ,y { std::move(y_arg) } // NOLINT
      ,z { std::move(z_arg) } // NOLINT
      ,intensity { std::move(intensity_arg) } // NOLINT
      ,ring { std::move(ring_arg) } // NOLINT
      ,lidar_info { std::move(lidar_info_arg) } // NOLINT
      ,semantic_flag { std::move(semantic_flag_arg) } // NOLINT
      ,azimuth { std::move(azimuth_arg) } // NOLINT
      ,range { std::move(range_arg) } // NOLINT
      ,drop { std::move(drop_arg) } // NOLINT
      ,retro_shadow { std::move(retro_shadow_arg) } // NOLINT
      ,range_limited { std::move(range_limited_arg) } // NOLINT
      ,retro_ghost { std::move(retro_ghost_arg) } // NOLINT
      ,interference { std::move(interference_arg) } // NOLINT
      ,sun_level { std::move(sun_level_arg) } // NOLINT
      ,confidence { std::move(confidence_arg) } // NOLINT
      ,is_second_return { std::move(is_second_return_arg) } // NOLINT
      ,lidars_frame_id { std::move(lidars_frame_id_arg) } // NOLINT
      {
}

LidarPointCloud::LidarPointCloud(const LidarPointCloud&) = default;

// NOLINTNEXTLINE
LidarPointCloud::LidarPointCloud(LidarPointCloud&&) = default;

LidarPointCloud& LidarPointCloud::operator=(const LidarPointCloud&) = default;

// NOLINTNEXTLINE
LidarPointCloud& LidarPointCloud::operator=(LidarPointCloud&&) = default;

void LidarPointCloud::toCsv(std::ostream& out) const
{
    out.precision(16);
    out << std::boolalpha;
    out << "\"";
        out << frame_id;
        out << "\"";out << ",";
    out << timestamp;out << ",";
    out << num_points;out << ",";
    // Cast to uint16_t to print as integer instead of char
        out << static_cast<uint16_t>(num_lidars);out << ",";
    // Enclose sequence in double quotations and curly brackets to keep CSV line length uniform
        
        out << "\"{{";
        
        if (!num_points_in_each_lidar.empty())
        {
            std::copy(num_points_in_each_lidar.begin(), num_points_in_each_lidar.end() - 1, std::ostream_iterator<uint32_t>(out, "},{"));
            std::copy(num_points_in_each_lidar.end()-1, num_points_in_each_lidar.end(), std::ostream_iterator<uint32_t>(out, ""));
        }
        out << "}}\"";out << ",";
    // Enclose sequence in double quotations and curly brackets to keep CSV line length uniform
        
        out << "\"{{";
        
        if (!sync_angle_azimuths.empty())
        {
            std::copy(sync_angle_azimuths.begin(), sync_angle_azimuths.end() - 1, std::ostream_iterator<uint16_t>(out, "},{"));
            std::copy(sync_angle_azimuths.end()-1, sync_angle_azimuths.end(), std::ostream_iterator<uint16_t>(out, ""));
        }
        out << "}}\"";out << ",";
    // Enclose sequence in double quotations and curly brackets to keep CSV line length uniform
        
        out << "\"{{";
        
        if (!sync_angle_timestamps.empty())
        {
            std::copy(sync_angle_timestamps.begin(), sync_angle_timestamps.end() - 1, std::ostream_iterator<uint64_t>(out, "},{"));
            std::copy(sync_angle_timestamps.end()-1, sync_angle_timestamps.end(), std::ostream_iterator<uint64_t>(out, ""));
        }
        out << "}}\"";out << ",";
    // Enclose sequence in double quotations and curly brackets to keep CSV line length uniform
        
        out << "\"{{";
        
        if (!camera_fov_timestamps.empty())
        {
            std::copy(camera_fov_timestamps.begin(), camera_fov_timestamps.end() - 1, std::ostream_iterator<uint64_t>(out, "},{"));
            std::copy(camera_fov_timestamps.end()-1, camera_fov_timestamps.end(), std::ostream_iterator<uint64_t>(out, ""));
        }
        out << "}}\"";out << ",";
    // Enclose sequence in double quotations and curly brackets to keep CSV line length uniform
        
        out << "\"{{";
        
        if (!lidars_with_confidence.empty())
        {
            std::copy(lidars_with_confidence.begin(), lidars_with_confidence.end() - 1, std::ostream_iterator<uint8_t>(out, "},{"));
            std::copy(lidars_with_confidence.end()-1, lidars_with_confidence.end(), std::ostream_iterator<uint8_t>(out, ""));
        }
        out << "}}\"";out << ",";
    // Enclose sequence in double quotations and curly brackets to keep CSV line length uniform
        
        out << "\"{{";
        
        if (!lidars_with_range.empty())
        {
            std::copy(lidars_with_range.begin(), lidars_with_range.end() - 1, std::ostream_iterator<uint8_t>(out, "},{"));
            std::copy(lidars_with_range.end()-1, lidars_with_range.end(), std::ostream_iterator<uint8_t>(out, ""));
        }
        out << "}}\"";out << ",";
    // Enclose sequence in double quotations and curly brackets to keep CSV line length uniform
        
        out << "\"{{";
        
        if (!lidars_with_dual_return.empty())
        {
            std::copy(lidars_with_dual_return.begin(), lidars_with_dual_return.end() - 1, std::ostream_iterator<uint8_t>(out, "},{"));
            std::copy(lidars_with_dual_return.end()-1, lidars_with_dual_return.end(), std::ostream_iterator<uint8_t>(out, ""));
        }
        out << "}}\"";out << ",";
    // Enclose sequence in double quotations and curly brackets to keep CSV line length uniform
        
        out << "\"{{";
        
        if (!x.empty())
        {
            std::copy(x.begin(), x.end() - 1, std::ostream_iterator<float>(out, "},{"));
            std::copy(x.end()-1, x.end(), std::ostream_iterator<float>(out, ""));
        }
        out << "}}\"";out << ",";
    // Enclose sequence in double quotations and curly brackets to keep CSV line length uniform
        
        out << "\"{{";
        
        if (!y.empty())
        {
            std::copy(y.begin(), y.end() - 1, std::ostream_iterator<float>(out, "},{"));
            std::copy(y.end()-1, y.end(), std::ostream_iterator<float>(out, ""));
        }
        out << "}}\"";out << ",";
    // Enclose sequence in double quotations and curly brackets to keep CSV line length uniform
        
        out << "\"{{";
        
        if (!z.empty())
        {
            std::copy(z.begin(), z.end() - 1, std::ostream_iterator<float>(out, "},{"));
            std::copy(z.end()-1, z.end(), std::ostream_iterator<float>(out, ""));
        }
        out << "}}\"";out << ",";
    // Enclose sequence in double quotations and curly brackets to keep CSV line length uniform
        
        out << "\"{{";
        
        if (!intensity.empty())
        {
            std::copy(intensity.begin(), intensity.end() - 1, std::ostream_iterator<uint8_t>(out, "},{"));
            std::copy(intensity.end()-1, intensity.end(), std::ostream_iterator<uint8_t>(out, ""));
        }
        out << "}}\"";out << ",";
    // Enclose sequence in double quotations and curly brackets to keep CSV line length uniform
        
        out << "\"{{";
        
        if (!ring.empty())
        {
            std::copy(ring.begin(), ring.end() - 1, std::ostream_iterator<uint8_t>(out, "},{"));
            std::copy(ring.end()-1, ring.end(), std::ostream_iterator<uint8_t>(out, ""));
        }
        out << "}}\"";out << ",";
    // Enclose sequence in double quotations and curly brackets to keep CSV line length uniform
        
        out << "\"{{";
        
        if (!lidar_info.empty())
        {
            std::copy(lidar_info.begin(), lidar_info.end() - 1, std::ostream_iterator<uint8_t>(out, "},{"));
            std::copy(lidar_info.end()-1, lidar_info.end(), std::ostream_iterator<uint8_t>(out, ""));
        }
        out << "}}\"";out << ",";
    // Enclose sequence in double quotations and curly brackets to keep CSV line length uniform
        
        out << "\"{{";
        
        if (!semantic_flag.empty())
        {
            std::copy(semantic_flag.begin(), semantic_flag.end() - 1, std::ostream_iterator<uint8_t>(out, "},{"));
            std::copy(semantic_flag.end()-1, semantic_flag.end(), std::ostream_iterator<uint8_t>(out, ""));
        }
        out << "}}\"";out << ",";
    // Enclose sequence in double quotations and curly brackets to keep CSV line length uniform
        
        out << "\"{{";
        
        if (!azimuth.empty())
        {
            std::copy(azimuth.begin(), azimuth.end() - 1, std::ostream_iterator<float>(out, "},{"));
            std::copy(azimuth.end()-1, azimuth.end(), std::ostream_iterator<float>(out, ""));
        }
        out << "}}\"";out << ",";
    // Enclose sequence in double quotations and curly brackets to keep CSV line length uniform
        
        out << "\"{{";
        
        if (!range.empty())
        {
            std::copy(range.begin(), range.end() - 1, std::ostream_iterator<float>(out, "},{"));
            std::copy(range.end()-1, range.end(), std::ostream_iterator<float>(out, ""));
        }
        out << "}}\"";out << ",";
    // Enclose sequence in double quotations and curly brackets to keep CSV line length uniform
        
        out << "\"{{";
        
        if (!drop.empty())
        {
            std::copy(drop.begin(), drop.end() - 1, std::ostream_iterator<uint8_t>(out, "},{"));
            std::copy(drop.end()-1, drop.end(), std::ostream_iterator<uint8_t>(out, ""));
        }
        out << "}}\"";out << ",";
    // Enclose sequence in double quotations and curly brackets to keep CSV line length uniform
        
        out << "\"{{";
        
        if (!retro_shadow.empty())
        {
            std::copy(retro_shadow.begin(), retro_shadow.end() - 1, std::ostream_iterator<uint8_t>(out, "},{"));
            std::copy(retro_shadow.end()-1, retro_shadow.end(), std::ostream_iterator<uint8_t>(out, ""));
        }
        out << "}}\"";out << ",";
    // Enclose sequence in double quotations and curly brackets to keep CSV line length uniform
        
        out << "\"{{";
        
        if (!range_limited.empty())
        {
            std::copy(range_limited.begin(), range_limited.end() - 1, std::ostream_iterator<uint8_t>(out, "},{"));
            std::copy(range_limited.end()-1, range_limited.end(), std::ostream_iterator<uint8_t>(out, ""));
        }
        out << "}}\"";out << ",";
    // Enclose sequence in double quotations and curly brackets to keep CSV line length uniform
        
        out << "\"{{";
        
        if (!retro_ghost.empty())
        {
            std::copy(retro_ghost.begin(), retro_ghost.end() - 1, std::ostream_iterator<uint8_t>(out, "},{"));
            std::copy(retro_ghost.end()-1, retro_ghost.end(), std::ostream_iterator<uint8_t>(out, ""));
        }
        out << "}}\"";out << ",";
    // Enclose sequence in double quotations and curly brackets to keep CSV line length uniform
        
        out << "\"{{";
        
        if (!interference.empty())
        {
            std::copy(interference.begin(), interference.end() - 1, std::ostream_iterator<uint8_t>(out, "},{"));
            std::copy(interference.end()-1, interference.end(), std::ostream_iterator<uint8_t>(out, ""));
        }
        out << "}}\"";out << ",";
    // Enclose sequence in double quotations and curly brackets to keep CSV line length uniform
        
        out << "\"{{";
        
        if (!sun_level.empty())
        {
            std::copy(sun_level.begin(), sun_level.end() - 1, std::ostream_iterator<uint8_t>(out, "},{"));
            std::copy(sun_level.end()-1, sun_level.end(), std::ostream_iterator<uint8_t>(out, ""));
        }
        out << "}}\"";out << ",";
    // Enclose sequence in double quotations and curly brackets to keep CSV line length uniform
        
        out << "\"{{";
        
        if (!confidence.empty())
        {
            std::copy(confidence.begin(), confidence.end() - 1, std::ostream_iterator<uint8_t>(out, "},{"));
            std::copy(confidence.end()-1, confidence.end(), std::ostream_iterator<uint8_t>(out, ""));
        }
        out << "}}\"";out << ",";
    // Enclose sequence in double quotations and curly brackets to keep CSV line length uniform
        
        out << "\"{{";
        
        if (!is_second_return.empty())
        {
            std::copy(is_second_return.begin(), is_second_return.end() - 1, std::ostream_iterator<uint8_t>(out, "},{"));
            std::copy(is_second_return.end()-1, is_second_return.end(), std::ostream_iterator<uint8_t>(out, ""));
        }
        out << "}}\"";out << ",";
    // Enclose sequence in double quotations and curly brackets to keep CSV line length uniform
        
        out << "\"{{";
        
        if (!lidars_frame_id.empty())
        {
            std::copy(lidars_frame_id.begin(), lidars_frame_id.end() - 1, std::ostream_iterator<std::string>(out, "},{"));
            std::copy(lidars_frame_id.end()-1, lidars_frame_id.end(), std::ostream_iterator<std::string>(out, ""));
        }
        out << "}}\"";
    
}

std::stringstream LidarPointCloud::toStringstream() const
{
    std::stringstream out;
    toCsv(out);
    return out;
}

template<class Archive>
    void LidarPointCloud::serialize(Archive& ar, const unsigned int version)
{
    ar(cereal::make_nvp("frame_id", frame_id),
        cereal::make_nvp("timestamp", timestamp),
        cereal::make_nvp("num_points", num_points),
        cereal::make_nvp("num_lidars", num_lidars),
        cereal::make_nvp("num_points_in_each_lidar", num_points_in_each_lidar),
        cereal::make_nvp("sync_angle_azimuths", sync_angle_azimuths),
        cereal::make_nvp("sync_angle_timestamps", sync_angle_timestamps),
        cereal::make_nvp("camera_fov_timestamps", camera_fov_timestamps),
        cereal::make_nvp("x", x),
        cereal::make_nvp("y", y),
        cereal::make_nvp("z", z),
        cereal::make_nvp("intensity", intensity),
        cereal::make_nvp("ring", ring),
        cereal::make_nvp("lidar_info", lidar_info),
        cereal::make_nvp("semantic_flag", semantic_flag));

    if (version <= 6) {
        ar(cereal::make_nvp("height_to_est_gnd", azimuth));
    } else {
        ar(cereal::make_nvp("azimuth", azimuth));
    }

	if (version >= 3) {
		ar(cereal::make_nvp("lidars_with_confidence", lidars_with_confidence),
			cereal::make_nvp("drop", drop),
			cereal::make_nvp("retro_shadow", retro_shadow),
			cereal::make_nvp("range_limited", range_limited),
			cereal::make_nvp("retro_ghost", retro_ghost),
			cereal::make_nvp("interference", interference),
			cereal::make_nvp("sun_level", sun_level),
			cereal::make_nvp("confidence", confidence));
	}
	if (version >= 4) {
		ar(cereal::make_nvp("lidars_with_range", lidars_with_range), cereal::make_nvp("distance", range));
	}
	if (version >= 5) {
		ar(cereal::make_nvp("lidars_with_dual_return", lidars_with_dual_return),
           cereal::make_nvp("is_second_return", is_second_return));
	}
    if (version >= 8) {
		ar(cereal::make_nvp("lidars_frame_id", lidars_frame_id));
    }
}

}} // to close av::msg namespace temporary

struct LidarPointCloud_t : public av::msg::LidarPointCloud {};
CEREAL_CLASS_VERSION(LidarPointCloud_t, 8)
CEREAL_REGISTER_TYPE(LidarPointCloud_t)

namespace av {
namespace msg {


template void LidarPointCloud::serialize(cereal::BinaryInputArchive& ar, unsigned int version);
template void LidarPointCloud::serialize(cereal::BinaryOutputArchive& ar, unsigned int version);
template void LidarPointCloud::serialize(cereal::IntrospectionArchive& ar, unsigned int version);
template void LidarPointCloud::serialize(cereal::JSONInputArchive& ar, unsigned int version);
template void LidarPointCloud::serialize(cereal::JSONOutputArchive& ar, unsigned int version);
template void LidarPointCloud::serialize(cereal::PortableBinaryInputArchive& ar, unsigned int version);
template void LidarPointCloud::serialize(cereal::PortableBinaryOutputArchive& ar, unsigned int version);

std::ostream& operator<< (std::ostream& os, const LidarPointCloud& lidarpointcloud)
{
    lidarpointcloud.toCsv(os);
    return os;
}

bool operator== (const LidarPointCloud &lhs, const LidarPointCloud &rhs)
{
    return (  lhs.frame_id== rhs.frame_id &&
     lhs.timestamp== rhs.timestamp &&
     lhs.num_points== rhs.num_points &&
     lhs.num_lidars== rhs.num_lidars &&
     lhs.num_points_in_each_lidar== rhs.num_points_in_each_lidar &&
     lhs.sync_angle_azimuths== rhs.sync_angle_azimuths &&
     lhs.sync_angle_timestamps== rhs.sync_angle_timestamps &&
     lhs.camera_fov_timestamps== rhs.camera_fov_timestamps &&
     lhs.lidars_with_confidence== rhs.lidars_with_confidence &&
     lhs.lidars_with_range== rhs.lidars_with_range &&
     lhs.lidars_with_dual_return== rhs.lidars_with_dual_return &&
     lhs.x== rhs.x &&
     lhs.y== rhs.y &&
     lhs.z== rhs.z &&
     lhs.intensity== rhs.intensity &&
     lhs.ring== rhs.ring &&
     lhs.lidar_info== rhs.lidar_info &&
     lhs.semantic_flag== rhs.semantic_flag &&
     lhs.azimuth== rhs.azimuth &&
     lhs.range== rhs.range &&
     lhs.drop== rhs.drop &&
     lhs.retro_shadow== rhs.retro_shadow &&
     lhs.range_limited== rhs.range_limited &&
     lhs.retro_ghost== rhs.retro_ghost &&
     lhs.interference== rhs.interference &&
     lhs.sun_level== rhs.sun_level &&
     lhs.confidence== rhs.confidence &&
     lhs.is_second_return== rhs.is_second_return &&
     lhs.lidars_frame_id== rhs.lidars_frame_id 
    );
}

std::size_t LidarPointCloud::hash() const noexcept
{
    std::size_t result = 17;
    result = result * 31 + std::hash<std::string>()(frame_id);
    result = result * 31 + std::hash<uint64_t>()(timestamp);
    result = result * 31 + std::hash<uint32_t>()(num_points);
    result = result * 31 + std::hash<uint8_t>()(num_lidars);
    result = result * 31 + hashVector(num_points_in_each_lidar);
    result = result * 31 + hashVector(sync_angle_azimuths);
    result = result * 31 + hashVector(sync_angle_timestamps);
    result = result * 31 + hashVector(camera_fov_timestamps);
    result = result * 31 + hashVector(lidars_with_confidence);
    result = result * 31 + hashVector(lidars_with_range);
    result = result * 31 + hashVector(lidars_with_dual_return);
    result = result * 31 + hashVector(x);
    result = result * 31 + hashVector(y);
    result = result * 31 + hashVector(z);
    result = result * 31 + hashVector(intensity);
    result = result * 31 + hashVector(ring);
    result = result * 31 + hashVector(lidar_info);
    result = result * 31 + hashVector(semantic_flag);
    result = result * 31 + hashVector(azimuth);
    result = result * 31 + hashVector(range);
    result = result * 31 + hashVector(drop);
    result = result * 31 + hashVector(retro_shadow);
    result = result * 31 + hashVector(range_limited);
    result = result * 31 + hashVector(retro_ghost);
    result = result * 31 + hashVector(interference);
    result = result * 31 + hashVector(sun_level);
    result = result * 31 + hashVector(confidence);
    result = result * 31 + hashVector(is_second_return);
    result = result * 31 + hashVector(lidars_frame_id);
    return result;
}

 } // namespace av
 } // namespace msg


CEREAL_BIND_TO_ARCHIVES(av::msg::LidarPointCloud)

