#include <interfaces/LidarUdpPacket/LidarUdpPacket.h>

#include <iterator>
#include <type_traits>

#include <cereal/cereal.hpp>
#include <cereal/archives/json.hpp>
#include <cereal/archives/binary.hpp>

#include <cereal/types/map.hpp>
#include <cereal/types/vector.hpp>
#include <cereal/types/array.hpp>
#include <cereal/types/string.hpp>
#include <cereal/types/complex.hpp>

#include <ODatum/IntrospectionArchive.h>
#include <ODatum/Hash.h>







LidarUdpPacket::LidarUdpPacket() = default;

LidarUdpPacket::LidarUdpPacket(
    av::msg::TimePoint timestamp_arg ,
    std::vector<uint8_t> data_arg ,
    av::msg::TimePoint arrivalTimestamp_arg 
    ):
    timestamp { std::move(timestamp_arg) } // NOLINT
      ,data { std::move(data_arg) } // NOLINT
      ,arrivalTimestamp { std::move(arrivalTimestamp_arg) } // NOLINT
      {
}

LidarUdpPacket::LidarUdpPacket(const LidarUdpPacket&) = default;

// NOLINTNEXTLINE
LidarUdpPacket::LidarUdpPacket(LidarUdpPacket&&) = default;

LidarUdpPacket& LidarUdpPacket::operator=(const LidarUdpPacket&) = default;

// NOLINTNEXTLINE
LidarUdpPacket& LidarUdpPacket::operator=(LidarUdpPacket&&) = default;

void LidarUdpPacket::toCsv(std::ostream& out) const
{
    out.precision(16);
    out << std::boolalpha;
    out << timestamp;out << ",";
    // Enclose sequence in double quotations and curly brackets to keep CSV line length uniform
        
        out << "\"{{";
        
        if (!data.empty())
        {
            std::copy(data.begin(), data.end() - 1, std::ostream_iterator<uint8_t>(out, "},{"));
            std::copy(data.end()-1, data.end(), std::ostream_iterator<uint8_t>(out, ""));
        }
        out << "}}\"";out << ",";
    out << arrivalTimestamp;
    
}

std::stringstream LidarUdpPacket::toStringstream() const
{
    std::stringstream out;
    toCsv(out);
    return out;
}

template<class Archive>
    
void LidarUdpPacket::serialize(Archive& ar, unsigned int version)
{
    if (version < 2)
    {
        int64_t timestampInt;   ///< microseconds
        ar& cereal::make_nvp("timestamp", timestampInt);
        timestamp = av::TimePoint(std::chrono::microseconds(timestampInt));
    }
    else
    {
        ar& CEREAL_NVP(timestamp);
    }
    if (version >= 3)
    {
        ar& CEREAL_NVP(arrivalTimestamp);
    }
    ar& CEREAL_NVP(data);
}



template void LidarUdpPacket::serialize(cereal::BinaryInputArchive& ar, unsigned int version);
template void LidarUdpPacket::serialize(cereal::BinaryOutputArchive& ar, unsigned int version);
template void LidarUdpPacket::serialize(cereal::IntrospectionArchive& ar, unsigned int version);
template void LidarUdpPacket::serialize(cereal::JSONInputArchive& ar, unsigned int version);
template void LidarUdpPacket::serialize(cereal::JSONOutputArchive& ar, unsigned int version);
template void LidarUdpPacket::serialize(cereal::PortableBinaryInputArchive& ar, unsigned int version);
template void LidarUdpPacket::serialize(cereal::PortableBinaryOutputArchive& ar, unsigned int version);

std::ostream& operator<< (std::ostream& os, const LidarUdpPacket& lidarudppacket)
{
    lidarudppacket.toCsv(os);
    return os;
}

bool operator== (const LidarUdpPacket &lhs, const LidarUdpPacket &rhs)
{
    return (  lhs.timestamp== rhs.timestamp &&
     lhs.data== rhs.data &&
     lhs.arrivalTimestamp== rhs.arrivalTimestamp 
    );
}

std::size_t LidarUdpPacket::hash() const noexcept
{
    std::size_t result = 17;
    result = result * 31 + std::hash<av::msg::TimePoint>()(timestamp);
    result = result * 31 + hashVector(data);
    result = result * 31 + std::hash<av::msg::TimePoint>()(arrivalTimestamp);
    return result;
}



CEREAL_BIND_TO_ARCHIVES(LidarUdpPacket)

