#include <interfaces/LidarUdpPacketList/LidarUdpPacketList.h>

#include <iterator>
#include <type_traits>

#include <cereal/cereal.hpp>
#include <cereal/archives/json.hpp>
#include <cereal/archives/binary.hpp>

#include <cereal/types/map.hpp>
#include <cereal/types/vector.hpp>
#include <cereal/types/array.hpp>
#include <cereal/types/string.hpp>
#include <cereal/types/complex.hpp>

#include <ODatum/IntrospectionArchive.h>
#include <ODatum/Hash.h>







LidarUdpPacketList::LidarUdpPacketList() = default;

LidarUdpPacketList::LidarUdpPacketList(
    std::vector<LidarUdpPacket> packet_list_arg 
    ):
    packet_list { std::move(packet_list_arg) } // NOLINT
      {
}

LidarUdpPacketList::LidarUdpPacketList(const LidarUdpPacketList&) = default;

// NOLINTNEXTLINE
LidarUdpPacketList::LidarUdpPacketList(LidarUdpPacketList&&) = default;

LidarUdpPacketList& LidarUdpPacketList::operator=(const LidarUdpPacketList&) = default;

// NOLINTNEXTLINE
LidarUdpPacketList& LidarUdpPacketList::operator=(LidarUdpPacketList&&) = default;

void LidarUdpPacketList::toCsv(std::ostream& out) const
{
    out.precision(16);
    out << std::boolalpha;
    // Enclose sequence in double quotations and curly brackets to keep CSV line length uniform
        
        out << "\"{{";
        
        if (!packet_list.empty())
        {
            std::copy(packet_list.begin(), packet_list.end() - 1, std::ostream_iterator<LidarUdpPacket>(out, "},{"));
            std::copy(packet_list.end()-1, packet_list.end(), std::ostream_iterator<LidarUdpPacket>(out, ""));
        }
        out << "}}\"";
    
}

std::stringstream LidarUdpPacketList::toStringstream() const
{
    std::stringstream out;
    toCsv(out);
    return out;
}

template<class Archive>
    void LidarUdpPacketList::serialize(Archive& ar, [[maybe_unused]] unsigned int version)
    {

        ar& CEREAL_NVP(packet_list);
    }


template void LidarUdpPacketList::serialize(cereal::BinaryInputArchive& ar, unsigned int version);
template void LidarUdpPacketList::serialize(cereal::BinaryOutputArchive& ar, unsigned int version);
template void LidarUdpPacketList::serialize(cereal::IntrospectionArchive& ar, unsigned int version);
template void LidarUdpPacketList::serialize(cereal::JSONInputArchive& ar, unsigned int version);
template void LidarUdpPacketList::serialize(cereal::JSONOutputArchive& ar, unsigned int version);
template void LidarUdpPacketList::serialize(cereal::PortableBinaryInputArchive& ar, unsigned int version);
template void LidarUdpPacketList::serialize(cereal::PortableBinaryOutputArchive& ar, unsigned int version);

std::ostream& operator<< (std::ostream& os, const LidarUdpPacketList& lidarudppacketlist)
{
    lidarudppacketlist.toCsv(os);
    return os;
}

bool operator== (const LidarUdpPacketList &lhs, const LidarUdpPacketList &rhs)
{
    return (  lhs.packet_list== rhs.packet_list 
    );
}

std::size_t LidarUdpPacketList::hash() const noexcept
{
    std::size_t result = 17;
    result = result * 31 + hashVector(packet_list);
    return result;
}



CEREAL_BIND_TO_ARCHIVES(LidarUdpPacketList)

