#include <interfaces/LocalizationHealth/LocalizationHealth.h>

#include <iterator>
#include <type_traits>

#include <cereal/cereal.hpp>
#include <cereal/archives/json.hpp>
#include <cereal/archives/binary.hpp>

#include <cereal/types/map.hpp>
#include <cereal/types/vector.hpp>
#include <cereal/types/array.hpp>
#include <cereal/types/string.hpp>
#include <cereal/types/complex.hpp>

#include <ODatum/IntrospectionArchive.h>
#include <ODatum/Hash.h>





 namespace localization {


LocalizationHealth::LocalizationHealth() = default;

LocalizationHealth::LocalizationHealth(
    bool overallHealth_arg ,
    double std_longitudinal_pose_arg ,
    double std_lateral_pose_arg ,
    double std_yaw_arg 
    ):
    overallHealth { std::move(overallHealth_arg) } // NOLINT
      ,std_longitudinal_pose { std::move(std_longitudinal_pose_arg) } // NOLINT
      ,std_lateral_pose { std::move(std_lateral_pose_arg) } // NOLINT
      ,std_yaw { std::move(std_yaw_arg) } // NOLINT
      {
}

LocalizationHealth::LocalizationHealth(const LocalizationHealth&) = default;

// NOLINTNEXTLINE
LocalizationHealth::LocalizationHealth(LocalizationHealth&&) = default;

LocalizationHealth& LocalizationHealth::operator=(const LocalizationHealth&) = default;

// NOLINTNEXTLINE
LocalizationHealth& LocalizationHealth::operator=(LocalizationHealth&&) = default;

void LocalizationHealth::toCsv(std::ostream& out) const
{
    out.precision(16);
    out << std::boolalpha;
    out << overallHealth;out << ",";
    out << std_longitudinal_pose;out << ",";
    out << std_lateral_pose;out << ",";
    out << std_yaw;
    
}

std::stringstream LocalizationHealth::toStringstream() const
{
    std::stringstream out;
    toCsv(out);
    return out;
}

template<class Archive>
    void LocalizationHealth::serialize(Archive& ar, [[maybe_unused]] unsigned int version)
    {

        ar& CEREAL_NVP(overallHealth);
        ar& CEREAL_NVP(std_longitudinal_pose);
        ar& CEREAL_NVP(std_lateral_pose);
        ar& CEREAL_NVP(std_yaw);
    }


template void LocalizationHealth::serialize(cereal::BinaryInputArchive& ar, unsigned int version);
template void LocalizationHealth::serialize(cereal::BinaryOutputArchive& ar, unsigned int version);
template void LocalizationHealth::serialize(cereal::IntrospectionArchive& ar, unsigned int version);
template void LocalizationHealth::serialize(cereal::JSONInputArchive& ar, unsigned int version);
template void LocalizationHealth::serialize(cereal::JSONOutputArchive& ar, unsigned int version);
template void LocalizationHealth::serialize(cereal::PortableBinaryInputArchive& ar, unsigned int version);
template void LocalizationHealth::serialize(cereal::PortableBinaryOutputArchive& ar, unsigned int version);

std::ostream& operator<< (std::ostream& os, const LocalizationHealth& localizationhealth)
{
    localizationhealth.toCsv(os);
    return os;
}

bool operator== (const LocalizationHealth &lhs, const LocalizationHealth &rhs)
{
    return (  lhs.overallHealth== rhs.overallHealth &&
     lhs.std_longitudinal_pose== rhs.std_longitudinal_pose &&
     lhs.std_lateral_pose== rhs.std_lateral_pose &&
     lhs.std_yaw== rhs.std_yaw 
    );
}

std::size_t LocalizationHealth::hash() const noexcept
{
    std::size_t result = 17;
    result = result * 31 + std::hash<bool>()(overallHealth);
    result = result * 31 + std::hash<double>()(std_longitudinal_pose);
    result = result * 31 + std::hash<double>()(std_lateral_pose);
    result = result * 31 + std::hash<double>()(std_yaw);
    return result;
}

 } // namespace localization


CEREAL_BIND_TO_ARCHIVES(localization::LocalizationHealth)

