#include <interfaces/LocalizationPose/LocalizationPose.h>

#include <iterator>
#include <type_traits>

#include <cereal/cereal.hpp>
#include <cereal/archives/json.hpp>
#include <cereal/archives/binary.hpp>

#include <cereal/types/map.hpp>
#include <cereal/types/vector.hpp>
#include <cereal/types/array.hpp>
#include <cereal/types/string.hpp>
#include <cereal/types/complex.hpp>

#include <ODatum/IntrospectionArchive.h>
#include <ODatum/Hash.h>







LocalizationPose::LocalizationPose() = default;

LocalizationPose::LocalizationPose(
    int64_t timestamp_arg ,
    av::geometry::Pose pose_arg ,
    av::geometry::Vector3d velocity_arg ,
    av::geometry::Vector3d angular_rate_arg ,
    av::geometry::Vector3d acceleration_arg ,
    int epsg_arg ,
    localization::LocalizationHealth health_arg 
    ):
    timestamp { std::move(timestamp_arg) } // NOLINT
      ,pose { std::move(pose_arg) } // NOLINT
      ,velocity { std::move(velocity_arg) } // NOLINT
      ,angular_rate { std::move(angular_rate_arg) } // NOLINT
      ,acceleration { std::move(acceleration_arg) } // NOLINT
      ,epsg { std::move(epsg_arg) } // NOLINT
      ,health { std::move(health_arg) } // NOLINT
      {
}

LocalizationPose::LocalizationPose(const LocalizationPose&) = default;

// NOLINTNEXTLINE
LocalizationPose::LocalizationPose(LocalizationPose&&) = default;

LocalizationPose& LocalizationPose::operator=(const LocalizationPose&) = default;

// NOLINTNEXTLINE
LocalizationPose& LocalizationPose::operator=(LocalizationPose&&) = default;

void LocalizationPose::toCsv(std::ostream& out) const
{
    out.precision(16);
    out << std::boolalpha;
    out << timestamp;out << ",";
    out << pose;out << ",";
    out << velocity;out << ",";
    out << angular_rate;out << ",";
    out << acceleration;out << ",";
    out << epsg;out << ",";
    out << health;
    
}

std::stringstream LocalizationPose::toStringstream() const
{
    std::stringstream out;
    toCsv(out);
    return out;
}

template<class Archive>
    void LocalizationPose::serialize(Archive& ar, [[maybe_unused]] unsigned int version)
    {

        ar& CEREAL_NVP(timestamp);
        ar& CEREAL_NVP(pose);
        ar& CEREAL_NVP(velocity);
        ar& CEREAL_NVP(angular_rate);
        ar& CEREAL_NVP(acceleration);
        ar& CEREAL_NVP(epsg);
        ar& CEREAL_NVP(health);
    }


template void LocalizationPose::serialize(cereal::BinaryInputArchive& ar, unsigned int version);
template void LocalizationPose::serialize(cereal::BinaryOutputArchive& ar, unsigned int version);
template void LocalizationPose::serialize(cereal::IntrospectionArchive& ar, unsigned int version);
template void LocalizationPose::serialize(cereal::JSONInputArchive& ar, unsigned int version);
template void LocalizationPose::serialize(cereal::JSONOutputArchive& ar, unsigned int version);
template void LocalizationPose::serialize(cereal::PortableBinaryInputArchive& ar, unsigned int version);
template void LocalizationPose::serialize(cereal::PortableBinaryOutputArchive& ar, unsigned int version);

std::ostream& operator<< (std::ostream& os, const LocalizationPose& localizationpose)
{
    localizationpose.toCsv(os);
    return os;
}

bool operator== (const LocalizationPose &lhs, const LocalizationPose &rhs)
{
    return (  lhs.timestamp== rhs.timestamp &&
     lhs.pose== rhs.pose &&
     lhs.velocity== rhs.velocity &&
     lhs.angular_rate== rhs.angular_rate &&
     lhs.acceleration== rhs.acceleration &&
     lhs.epsg== rhs.epsg &&
     lhs.health== rhs.health 
    );
}

std::size_t LocalizationPose::hash() const noexcept
{
    std::size_t result = 17;
    result = result * 31 + std::hash<int64_t>()(timestamp);
    result = result * 31 + std::hash<av::geometry::Pose>()(pose);
    result = result * 31 + std::hash<av::geometry::Vector3d>()(velocity);
    result = result * 31 + std::hash<av::geometry::Vector3d>()(angular_rate);
    result = result * 31 + std::hash<av::geometry::Vector3d>()(acceleration);
    result = result * 31 + std::hash<int>()(epsg);
    result = result * 31 + std::hash<localization::LocalizationHealth>()(health);
    return result;
}



CEREAL_BIND_TO_ARCHIVES(LocalizationPose)

