#include <interfaces/LocalizedPose/LocalizedPose.h>

#include <iterator>
#include <type_traits>

#include <cereal/cereal.hpp>
#include <cereal/archives/json.hpp>
#include <cereal/archives/binary.hpp>

#include <cereal/types/map.hpp>
#include <cereal/types/vector.hpp>
#include <cereal/types/array.hpp>
#include <cereal/types/string.hpp>
#include <cereal/types/complex.hpp>

#include <ODatum/IntrospectionArchive.h>
#include <ODatum/Hash.h>





 namespace av {
 namespace msg {
 namespace tc {


LocalizedPose::LocalizedPose() = default;

LocalizedPose::LocalizedPose(
    int64_t timeUs_arg ,
    av::geometry::Pose pose_arg ,
    av::geometry::Vector3d velocity_arg ,
    av::geometry::Vector3d angularRate_arg ,
    av::geometry::Vector3d acceleration_arg 
    ):
    timeUs { std::move(timeUs_arg) } // NOLINT
      ,pose { std::move(pose_arg) } // NOLINT
      ,velocity { std::move(velocity_arg) } // NOLINT
      ,angularRate { std::move(angularRate_arg) } // NOLINT
      ,acceleration { std::move(acceleration_arg) } // NOLINT
      {
}

LocalizedPose::LocalizedPose(const LocalizedPose&) = default;

// NOLINTNEXTLINE
LocalizedPose::LocalizedPose(LocalizedPose&&) = default;

LocalizedPose& LocalizedPose::operator=(const LocalizedPose&) = default;

// NOLINTNEXTLINE
LocalizedPose& LocalizedPose::operator=(LocalizedPose&&) = default;

void LocalizedPose::toCsv(std::ostream& out) const
{
    out.precision(16);
    out << std::boolalpha;
    out << timeUs;out << ",";
    out << pose;out << ",";
    out << velocity;out << ",";
    out << angularRate;out << ",";
    out << acceleration;
    
}

std::stringstream LocalizedPose::toStringstream() const
{
    std::stringstream out;
    toCsv(out);
    return out;
}

template<class Archive>
    void LocalizedPose::serialize(Archive& ar, [[maybe_unused]] unsigned int version)
    {

        ar& CEREAL_NVP(timeUs);
        ar& CEREAL_NVP(pose);
        ar& CEREAL_NVP(velocity);
        ar& CEREAL_NVP(angularRate);
        ar& CEREAL_NVP(acceleration);
    }


template void LocalizedPose::serialize(cereal::BinaryInputArchive& ar, unsigned int version);
template void LocalizedPose::serialize(cereal::BinaryOutputArchive& ar, unsigned int version);
template void LocalizedPose::serialize(cereal::IntrospectionArchive& ar, unsigned int version);
template void LocalizedPose::serialize(cereal::JSONInputArchive& ar, unsigned int version);
template void LocalizedPose::serialize(cereal::JSONOutputArchive& ar, unsigned int version);
template void LocalizedPose::serialize(cereal::PortableBinaryInputArchive& ar, unsigned int version);
template void LocalizedPose::serialize(cereal::PortableBinaryOutputArchive& ar, unsigned int version);

std::ostream& operator<< (std::ostream& os, const LocalizedPose& localizedpose)
{
    localizedpose.toCsv(os);
    return os;
}

bool operator== (const LocalizedPose &lhs, const LocalizedPose &rhs)
{
    return (  lhs.timeUs== rhs.timeUs &&
     lhs.pose== rhs.pose &&
     lhs.velocity== rhs.velocity &&
     lhs.angularRate== rhs.angularRate &&
     lhs.acceleration== rhs.acceleration 
    );
}

std::size_t LocalizedPose::hash() const noexcept
{
    std::size_t result = 17;
    result = result * 31 + std::hash<int64_t>()(timeUs);
    result = result * 31 + std::hash<av::geometry::Pose>()(pose);
    result = result * 31 + std::hash<av::geometry::Vector3d>()(velocity);
    result = result * 31 + std::hash<av::geometry::Vector3d>()(angularRate);
    result = result * 31 + std::hash<av::geometry::Vector3d>()(acceleration);
    return result;
}

 } // namespace av
 } // namespace msg
 } // namespace tc


CEREAL_BIND_TO_ARCHIVES(av::msg::tc::LocalizedPose)

