#include <interfaces/MLPlannerDebug/MLPlannerDebug.h>

#include <iterator>
#include <type_traits>

#include <cereal/cereal.hpp>
#include <cereal/archives/json.hpp>
#include <cereal/archives/binary.hpp>

#include <cereal/types/map.hpp>
#include <cereal/types/vector.hpp>
#include <cereal/types/array.hpp>
#include <cereal/types/string.hpp>
#include <cereal/types/complex.hpp>

#include <ODatum/IntrospectionArchive.h>
#include <ODatum/Hash.h>


 namespace av {
 namespace msg {
 namespace plan_ctrl {


CartesianState::CartesianState() = default;

CartesianState::CartesianState(
    double x_arg ,
    double y_arg ,
    double heading_arg ,
    double velocity_arg ,
    double acceleration_arg ,
    double delta_arg 
    ):
    x { std::move(x_arg) } // NOLINT
      ,
    y { std::move(y_arg) } // NOLINT
      ,
    heading { std::move(heading_arg) } // NOLINT
      ,
    velocity { std::move(velocity_arg) } // NOLINT
      ,
    acceleration { std::move(acceleration_arg) } // NOLINT
      ,
    delta { std::move(delta_arg) } // NOLINT
      
    {
}

CartesianState::CartesianState(const CartesianState&) = default;

// NOLINTNEXTLINE
CartesianState::CartesianState(CartesianState&&) = default;

CartesianState& CartesianState::operator=(const CartesianState&) = default;

// NOLINTNEXTLINE
CartesianState& CartesianState::operator=(CartesianState&&) = default;

void CartesianState::toCsv(std::ostream& out) const
{
    out.precision(16);
    out << std::boolalpha;
    out << x;out << ",";
    out << y;out << ",";
    out << heading;out << ",";
    out << velocity;out << ",";
    out << acceleration;out << ",";
    out << delta;
    
}

std::stringstream CartesianState::toStringstream() const
{
    std::stringstream out;
    toCsv(out);
    return out;
}

template<class Archive>
    void CartesianState::serialize(Archive& ar, [[maybe_unused]] unsigned int version)
    {

        ar& CEREAL_NVP(x);
        ar& CEREAL_NVP(y);
        ar& CEREAL_NVP(heading);
        ar& CEREAL_NVP(velocity);
        ar& CEREAL_NVP(acceleration);
        ar& CEREAL_NVP(delta);
    }


template void CartesianState::serialize(cereal::BinaryInputArchive& ar, unsigned int version);
template void CartesianState::serialize(cereal::BinaryOutputArchive& ar, unsigned int version);
template void CartesianState::serialize(cereal::IntrospectionArchive& ar, unsigned int version);
template void CartesianState::serialize(cereal::JSONInputArchive& ar, unsigned int version);
template void CartesianState::serialize(cereal::JSONOutputArchive& ar, unsigned int version);
template void CartesianState::serialize(cereal::PortableBinaryInputArchive& ar, unsigned int version);
template void CartesianState::serialize(cereal::PortableBinaryOutputArchive& ar, unsigned int version);

std::ostream& operator<< (std::ostream& os, const CartesianState& cartesianstate)
{
    cartesianstate.toCsv(os);
    return os;
}

bool operator== (const CartesianState &lhs, const CartesianState &rhs)
{
    return (  lhs.x== rhs.x &&
     lhs.y== rhs.y &&
     lhs.heading== rhs.heading &&
     lhs.velocity== rhs.velocity &&
     lhs.acceleration== rhs.acceleration &&
     lhs.delta== rhs.delta 
    );
}

std::size_t CartesianState::hash() const noexcept
{
    std::size_t result = 17;
    result = result * 31 + std::hash<double>()(x);
    result = result * 31 + std::hash<double>()(y);
    result = result * 31 + std::hash<double>()(heading);
    result = result * 31 + std::hash<double>()(velocity);
    result = result * 31 + std::hash<double>()(acceleration);
    result = result * 31 + std::hash<double>()(delta);
    return result;
}

 } // namespace av
 } // namespace msg
 } // namespace plan_ctrl


CEREAL_BIND_TO_ARCHIVES(av::msg::plan_ctrl::CartesianState)

 namespace av {
 namespace msg {
 namespace plan_ctrl {


CartesianStateTrajectory::CartesianStateTrajectory() = default;

CartesianStateTrajectory::CartesianStateTrajectory(
    std::vector<CartesianState> states_arg 
    ):
    states { std::move(states_arg) } // NOLINT
      
    {
}

CartesianStateTrajectory::CartesianStateTrajectory(const CartesianStateTrajectory&) = default;

// NOLINTNEXTLINE
CartesianStateTrajectory::CartesianStateTrajectory(CartesianStateTrajectory&&) = default;

CartesianStateTrajectory& CartesianStateTrajectory::operator=(const CartesianStateTrajectory&) = default;

// NOLINTNEXTLINE
CartesianStateTrajectory& CartesianStateTrajectory::operator=(CartesianStateTrajectory&&) = default;

void CartesianStateTrajectory::toCsv(std::ostream& out) const
{
    out.precision(16);
    out << std::boolalpha;
    // Enclose sequence in double quotations and curly brackets to keep CSV line length uniform
        
        out << "\"{{";
        
        if (!states.empty())
        {
            std::copy(states.begin(), states.end() - 1, std::ostream_iterator<CartesianState>(out, "},{"));
            std::copy(states.end()-1, states.end(), std::ostream_iterator<CartesianState>(out, ""));
        }
        out << "}}\"";
    
}

std::stringstream CartesianStateTrajectory::toStringstream() const
{
    std::stringstream out;
    toCsv(out);
    return out;
}

template<class Archive>
    void CartesianStateTrajectory::serialize(Archive& ar, [[maybe_unused]] unsigned int version)
    {

        ar& CEREAL_NVP(states);
    }


template void CartesianStateTrajectory::serialize(cereal::BinaryInputArchive& ar, unsigned int version);
template void CartesianStateTrajectory::serialize(cereal::BinaryOutputArchive& ar, unsigned int version);
template void CartesianStateTrajectory::serialize(cereal::IntrospectionArchive& ar, unsigned int version);
template void CartesianStateTrajectory::serialize(cereal::JSONInputArchive& ar, unsigned int version);
template void CartesianStateTrajectory::serialize(cereal::JSONOutputArchive& ar, unsigned int version);
template void CartesianStateTrajectory::serialize(cereal::PortableBinaryInputArchive& ar, unsigned int version);
template void CartesianStateTrajectory::serialize(cereal::PortableBinaryOutputArchive& ar, unsigned int version);

std::ostream& operator<< (std::ostream& os, const CartesianStateTrajectory& cartesianstatetrajectory)
{
    cartesianstatetrajectory.toCsv(os);
    return os;
}

bool operator== (const CartesianStateTrajectory &lhs, const CartesianStateTrajectory &rhs)
{
    return (  lhs.states== rhs.states 
    );
}

std::size_t CartesianStateTrajectory::hash() const noexcept
{
    std::size_t result = 17;
    result = result * 31 + hashVector(states);
    return result;
}

 } // namespace av
 } // namespace msg
 } // namespace plan_ctrl


CEREAL_BIND_TO_ARCHIVES(av::msg::plan_ctrl::CartesianStateTrajectory)

 namespace av {
 namespace msg {
 namespace plan_ctrl {


MLPlannerTrajectory::MLPlannerTrajectory() = default;

MLPlannerTrajectory::MLPlannerTrajectory(
    uint64_t index_arg ,
    std::vector<NamedVectorOfDouble> featureValues_arg ,
    double reward_arg ,
    CartesianStateTrajectory trajectory_arg 
    ):
    index { std::move(index_arg) } // NOLINT
      ,
    featureValues { std::move(featureValues_arg) } // NOLINT
      ,
    reward { std::move(reward_arg) } // NOLINT
      ,
    trajectory { std::move(trajectory_arg) } // NOLINT
      
    {
}

MLPlannerTrajectory::MLPlannerTrajectory(const MLPlannerTrajectory&) = default;

// NOLINTNEXTLINE
MLPlannerTrajectory::MLPlannerTrajectory(MLPlannerTrajectory&&) = default;

MLPlannerTrajectory& MLPlannerTrajectory::operator=(const MLPlannerTrajectory&) = default;

// NOLINTNEXTLINE
MLPlannerTrajectory& MLPlannerTrajectory::operator=(MLPlannerTrajectory&&) = default;

void MLPlannerTrajectory::toCsv(std::ostream& out) const
{
    out.precision(16);
    out << std::boolalpha;
    out << index;out << ",";
    // Enclose sequence in double quotations and curly brackets to keep CSV line length uniform
        
        out << "\"{{";
        
        if (!featureValues.empty())
        {
            std::copy(featureValues.begin(), featureValues.end() - 1, std::ostream_iterator<NamedVectorOfDouble>(out, "},{"));
            std::copy(featureValues.end()-1, featureValues.end(), std::ostream_iterator<NamedVectorOfDouble>(out, ""));
        }
        out << "}}\"";out << ",";
    out << reward;out << ",";
    out << trajectory;
    
}

std::stringstream MLPlannerTrajectory::toStringstream() const
{
    std::stringstream out;
    toCsv(out);
    return out;
}

template<class Archive>
    void MLPlannerTrajectory::serialize(Archive& ar, [[maybe_unused]] unsigned int version)
    {

        ar& CEREAL_NVP(index);
        ar& CEREAL_NVP(featureValues);
        ar& CEREAL_NVP(reward);
        ar& CEREAL_NVP(trajectory);
    }


template void MLPlannerTrajectory::serialize(cereal::BinaryInputArchive& ar, unsigned int version);
template void MLPlannerTrajectory::serialize(cereal::BinaryOutputArchive& ar, unsigned int version);
template void MLPlannerTrajectory::serialize(cereal::IntrospectionArchive& ar, unsigned int version);
template void MLPlannerTrajectory::serialize(cereal::JSONInputArchive& ar, unsigned int version);
template void MLPlannerTrajectory::serialize(cereal::JSONOutputArchive& ar, unsigned int version);
template void MLPlannerTrajectory::serialize(cereal::PortableBinaryInputArchive& ar, unsigned int version);
template void MLPlannerTrajectory::serialize(cereal::PortableBinaryOutputArchive& ar, unsigned int version);

std::ostream& operator<< (std::ostream& os, const MLPlannerTrajectory& mlplannertrajectory)
{
    mlplannertrajectory.toCsv(os);
    return os;
}

bool operator== (const MLPlannerTrajectory &lhs, const MLPlannerTrajectory &rhs)
{
    return (  lhs.index== rhs.index &&
     lhs.featureValues== rhs.featureValues &&
     lhs.reward== rhs.reward &&
     lhs.trajectory== rhs.trajectory 
    );
}

std::size_t MLPlannerTrajectory::hash() const noexcept
{
    std::size_t result = 17;
    result = result * 31 + std::hash<uint64_t>()(index);
    result = result * 31 + hashVector(featureValues);
    result = result * 31 + std::hash<double>()(reward);
    result = result * 31 + std::hash<CartesianStateTrajectory>()(trajectory);
    return result;
}

 } // namespace av
 } // namespace msg
 } // namespace plan_ctrl


CEREAL_BIND_TO_ARCHIVES(av::msg::plan_ctrl::MLPlannerTrajectory)




 namespace av {
 namespace msg {
 namespace plan_ctrl {


MLPlannerDebug::MLPlannerDebug() = default;

MLPlannerDebug::MLPlannerDebug(
    TimePoint timeStamp_arg ,
    std::string versionString_arg ,
    EgoStateExtended x0_arg ,
    std::vector<double> accelerationOptions_arg ,
    double trajectoryEpsilon_arg ,
    std::vector<MLPlannerTrajectory> trajectories_arg ,
    RoutePlan routePlan_arg ,
    PathPointMassModelTrajectory bookkeptTrajectory_arg 
    ):
    timeStamp { std::move(timeStamp_arg) } // NOLINT
      ,versionString { std::move(versionString_arg) } // NOLINT
      ,x0 { std::move(x0_arg) } // NOLINT
      ,accelerationOptions { std::move(accelerationOptions_arg) } // NOLINT
      ,trajectoryEpsilon { std::move(trajectoryEpsilon_arg) } // NOLINT
      ,trajectories { std::move(trajectories_arg) } // NOLINT
      ,routePlan { std::move(routePlan_arg) } // NOLINT
      ,bookkeptTrajectory { std::move(bookkeptTrajectory_arg) } // NOLINT
      {
}

MLPlannerDebug::MLPlannerDebug(const MLPlannerDebug&) = default;

// NOLINTNEXTLINE
MLPlannerDebug::MLPlannerDebug(MLPlannerDebug&&) = default;

MLPlannerDebug& MLPlannerDebug::operator=(const MLPlannerDebug&) = default;

// NOLINTNEXTLINE
MLPlannerDebug& MLPlannerDebug::operator=(MLPlannerDebug&&) = default;

void MLPlannerDebug::toCsv(std::ostream& out) const
{
    out.precision(16);
    out << std::boolalpha;
    out << timeStamp;out << ",";
    out << "\"";
        out << versionString;
        out << "\"";out << ",";
    out << x0;out << ",";
    // Enclose sequence in double quotations and curly brackets to keep CSV line length uniform
        
        out << "\"{{";
        
        if (!accelerationOptions.empty())
        {
            std::copy(accelerationOptions.begin(), accelerationOptions.end() - 1, std::ostream_iterator<double>(out, "},{"));
            std::copy(accelerationOptions.end()-1, accelerationOptions.end(), std::ostream_iterator<double>(out, ""));
        }
        out << "}}\"";out << ",";
    out << trajectoryEpsilon;out << ",";
    // Enclose sequence in double quotations and curly brackets to keep CSV line length uniform
        
        out << "\"{{";
        
        if (!trajectories.empty())
        {
            std::copy(trajectories.begin(), trajectories.end() - 1, std::ostream_iterator<MLPlannerTrajectory>(out, "},{"));
            std::copy(trajectories.end()-1, trajectories.end(), std::ostream_iterator<MLPlannerTrajectory>(out, ""));
        }
        out << "}}\"";out << ",";
    out << routePlan;out << ",";
    out << bookkeptTrajectory;
    
}

std::stringstream MLPlannerDebug::toStringstream() const
{
    std::stringstream out;
    toCsv(out);
    return out;
}

template<class Archive>
    void MLPlannerDebug::serialize(Archive& ar, unsigned int version)
{
    if (version >= 1)
	{
		ar& CEREAL_NVP(timeStamp);
        ar& CEREAL_NVP(versionString);
		ar& CEREAL_NVP(x0);
		ar& CEREAL_NVP(accelerationOptions);
		ar& CEREAL_NVP(trajectoryEpsilon);
		ar& CEREAL_NVP(trajectories);
	}
    if(version >= 2)
    {
        ar& CEREAL_NVP(routePlan);
    }
    if(version >= 3)
    {
        ar& CEREAL_NVP(bookkeptTrajectory);
    }
}



template void MLPlannerDebug::serialize(cereal::BinaryInputArchive& ar, unsigned int version);
template void MLPlannerDebug::serialize(cereal::BinaryOutputArchive& ar, unsigned int version);
template void MLPlannerDebug::serialize(cereal::IntrospectionArchive& ar, unsigned int version);
template void MLPlannerDebug::serialize(cereal::JSONInputArchive& ar, unsigned int version);
template void MLPlannerDebug::serialize(cereal::JSONOutputArchive& ar, unsigned int version);
template void MLPlannerDebug::serialize(cereal::PortableBinaryInputArchive& ar, unsigned int version);
template void MLPlannerDebug::serialize(cereal::PortableBinaryOutputArchive& ar, unsigned int version);

std::ostream& operator<< (std::ostream& os, const MLPlannerDebug& mlplannerdebug)
{
    mlplannerdebug.toCsv(os);
    return os;
}

bool operator== (const MLPlannerDebug &lhs, const MLPlannerDebug &rhs)
{
    return (  lhs.timeStamp== rhs.timeStamp &&
     lhs.versionString== rhs.versionString &&
     lhs.x0== rhs.x0 &&
     lhs.accelerationOptions== rhs.accelerationOptions &&
     lhs.trajectoryEpsilon== rhs.trajectoryEpsilon &&
     lhs.trajectories== rhs.trajectories &&
     lhs.routePlan== rhs.routePlan &&
     lhs.bookkeptTrajectory== rhs.bookkeptTrajectory 
    );
}

std::size_t MLPlannerDebug::hash() const noexcept
{
    std::size_t result = 17;
    result = result * 31 + 0;
    result = result * 31 + std::hash<std::string>()(versionString);
    result = result * 31 + std::hash<EgoStateExtended>()(x0);
    result = result * 31 + hashVector(accelerationOptions);
    result = result * 31 + std::hash<double>()(trajectoryEpsilon);
    result = result * 31 + hashVector(trajectories);
    result = result * 31 + std::hash<RoutePlan>()(routePlan);
    result = result * 31 + std::hash<PathPointMassModelTrajectory>()(bookkeptTrajectory);
    return result;
}

 } // namespace av
 } // namespace msg
 } // namespace plan_ctrl


CEREAL_BIND_TO_ARCHIVES(av::msg::plan_ctrl::MLPlannerDebug)

