#include <interfaces/MapId/MapId.h>

#include <iterator>
#include <type_traits>

#include <cereal/cereal.hpp>
#include <cereal/archives/json.hpp>
#include <cereal/archives/binary.hpp>

#include <cereal/types/map.hpp>
#include <cereal/types/vector.hpp>
#include <cereal/types/array.hpp>
#include <cereal/types/string.hpp>
#include <cereal/types/complex.hpp>

#include <ODatum/IntrospectionArchive.h>
#include <ODatum/Hash.h>





 namespace av {
 namespace msg {


MapId::MapId() = default;

MapId::MapId(
    std::string id_arg 
    ):
    id { std::move(id_arg) } // NOLINT
      {
}

MapId::MapId(const MapId&) = default;

// NOLINTNEXTLINE
MapId::MapId(MapId&&) = default;

MapId& MapId::operator=(const MapId&) = default;

// NOLINTNEXTLINE
MapId& MapId::operator=(MapId&&) = default;

void MapId::toCsv(std::ostream& out) const
{
    out.precision(16);
    out << std::boolalpha;
    out << "\"";
        out << id;
        out << "\"";
    
}

std::stringstream MapId::toStringstream() const
{
    std::stringstream out;
    toCsv(out);
    return out;
}

template<class Archive>
    void MapId::serialize(Archive& ar, [[maybe_unused]] unsigned int version)
    {

        ar& CEREAL_NVP(id);
    }


template void MapId::serialize(cereal::BinaryInputArchive& ar, unsigned int version);
template void MapId::serialize(cereal::BinaryOutputArchive& ar, unsigned int version);
template void MapId::serialize(cereal::IntrospectionArchive& ar, unsigned int version);
template void MapId::serialize(cereal::JSONInputArchive& ar, unsigned int version);
template void MapId::serialize(cereal::JSONOutputArchive& ar, unsigned int version);
template void MapId::serialize(cereal::PortableBinaryInputArchive& ar, unsigned int version);
template void MapId::serialize(cereal::PortableBinaryOutputArchive& ar, unsigned int version);

std::ostream& operator<< (std::ostream& os, const MapId& mapid)
{
    mapid.toCsv(os);
    return os;
}

bool operator== (const MapId &lhs, const MapId &rhs)
{
    return (  lhs.id== rhs.id 
    );
}

std::size_t MapId::hash() const noexcept
{
    std::size_t result = 17;
    result = result * 31 + std::hash<std::string>()(id);
    return result;
}

 } // namespace av
 } // namespace msg


CEREAL_BIND_TO_ARCHIVES(av::msg::MapId)

