#include <interfaces/MatrixImage/MatrixImage.h>

#include <iterator>
#include <type_traits>

#include <cereal/cereal.hpp>
#include <cereal/archives/json.hpp>
#include <cereal/archives/binary.hpp>

#include <cereal/types/map.hpp>
#include <cereal/types/vector.hpp>
#include <cereal/types/array.hpp>
#include <cereal/types/string.hpp>
#include <cereal/types/complex.hpp>

#include <ODatum/IntrospectionArchive.h>
#include <ODatum/Hash.h>





 namespace av {
 namespace horizon {


MatrixImage::MatrixImage() = default;

MatrixImage::MatrixImage(
    MatrixImageMeta imageMetaData_arg ,
    std::vector<uint8_t> imageData_arg 
    ):
    imageMetaData { std::move(imageMetaData_arg) } // NOLINT
      ,imageData { std::move(imageData_arg) } // NOLINT
      {
}

MatrixImage::MatrixImage(const MatrixImage&) = default;

// NOLINTNEXTLINE
MatrixImage::MatrixImage(MatrixImage&&) = default;

MatrixImage& MatrixImage::operator=(const MatrixImage&) = default;

// NOLINTNEXTLINE
MatrixImage& MatrixImage::operator=(MatrixImage&&) = default;

void MatrixImage::toCsv(std::ostream& out) const
{
    out.precision(16);
    out << std::boolalpha;
    out << imageMetaData;out << ",";
    // Enclose sequence in double quotations and curly brackets to keep CSV line length uniform
        
        out << "\"{{";
        
        if (!imageData.empty())
        {
            std::copy(imageData.begin(), imageData.end() - 1, std::ostream_iterator<uint8_t>(out, "},{"));
            std::copy(imageData.end()-1, imageData.end(), std::ostream_iterator<uint8_t>(out, ""));
        }
        out << "}}\"";
    
}

std::stringstream MatrixImage::toStringstream() const
{
    std::stringstream out;
    toCsv(out);
    return out;
}

template<class Archive>
    void MatrixImage::serialize(Archive& ar, [[maybe_unused]] unsigned int version)
    {

        ar& CEREAL_NVP(imageMetaData);
        ar& CEREAL_NVP(imageData);
    }


template void MatrixImage::serialize(cereal::BinaryInputArchive& ar, unsigned int version);
template void MatrixImage::serialize(cereal::BinaryOutputArchive& ar, unsigned int version);
template void MatrixImage::serialize(cereal::IntrospectionArchive& ar, unsigned int version);
template void MatrixImage::serialize(cereal::JSONInputArchive& ar, unsigned int version);
template void MatrixImage::serialize(cereal::JSONOutputArchive& ar, unsigned int version);
template void MatrixImage::serialize(cereal::PortableBinaryInputArchive& ar, unsigned int version);
template void MatrixImage::serialize(cereal::PortableBinaryOutputArchive& ar, unsigned int version);

std::ostream& operator<< (std::ostream& os, const MatrixImage& matriximage)
{
    matriximage.toCsv(os);
    return os;
}

bool operator== (const MatrixImage &lhs, const MatrixImage &rhs)
{
    return (  lhs.imageMetaData== rhs.imageMetaData &&
     lhs.imageData== rhs.imageData 
    );
}

std::size_t MatrixImage::hash() const noexcept
{
    std::size_t result = 17;
    result = result * 31 + std::hash<MatrixImageMeta>()(imageMetaData);
    result = result * 31 + hashVector(imageData);
    return result;
}

 } // namespace av
 } // namespace horizon


CEREAL_BIND_TO_ARCHIVES(av::horizon::MatrixImage)

