#include <interfaces/MatrixImageMeta/MatrixImageMeta.h>

#include <iterator>
#include <type_traits>

#include <cereal/cereal.hpp>
#include <cereal/archives/json.hpp>
#include <cereal/archives/binary.hpp>

#include <cereal/types/map.hpp>
#include <cereal/types/vector.hpp>
#include <cereal/types/array.hpp>
#include <cereal/types/string.hpp>
#include <cereal/types/complex.hpp>

#include <ODatum/IntrospectionArchive.h>
#include <ODatum/Hash.h>





 namespace av {
 namespace horizon {


MatrixImageMeta::MatrixImageMeta() = default;

MatrixImageMeta::MatrixImageMeta(
    int32_t width_pixel_arg ,
    int32_t height_pixel_arg ,
    int32_t channel_arg ,
    int64_t timeStamp_arg ,
    SEND_MODE sendMode_arg ,
    IMAGE_FORMAT format_arg ,
    int32_t index_arg ,
    int32_t count_arg ,
    int32_t depth_bit_arg ,
    int32_t align_arg ,
    int32_t imageCounter_arg 
    ):
    width_pixel { std::move(width_pixel_arg) } // NOLINT
      ,height_pixel { std::move(height_pixel_arg) } // NOLINT
      ,channel { std::move(channel_arg) } // NOLINT
      ,timeStamp { std::move(timeStamp_arg) } // NOLINT
      ,sendMode { std::move(sendMode_arg) } // NOLINT
      ,format { std::move(format_arg) } // NOLINT
      ,index { std::move(index_arg) } // NOLINT
      ,count { std::move(count_arg) } // NOLINT
      ,depth_bit { std::move(depth_bit_arg) } // NOLINT
      ,align { std::move(align_arg) } // NOLINT
      ,imageCounter { std::move(imageCounter_arg) } // NOLINT
      {
}

MatrixImageMeta::MatrixImageMeta(const MatrixImageMeta&) = default;

// NOLINTNEXTLINE
MatrixImageMeta::MatrixImageMeta(MatrixImageMeta&&) = default;

MatrixImageMeta& MatrixImageMeta::operator=(const MatrixImageMeta&) = default;

// NOLINTNEXTLINE
MatrixImageMeta& MatrixImageMeta::operator=(MatrixImageMeta&&) = default;

void MatrixImageMeta::toCsv(std::ostream& out) const
{
    out.precision(16);
    out << std::boolalpha;
    out << width_pixel;out << ",";
    out << height_pixel;out << ",";
    out << channel;out << ",";
    out << timeStamp;out << ",";
    out << static_cast<uint32_t>(sendMode);out << ",";
    out << static_cast<uint32_t>(format);out << ",";
    out << index;out << ",";
    out << count;out << ",";
    out << depth_bit;out << ",";
    out << align;out << ",";
    out << imageCounter;
    
}

std::stringstream MatrixImageMeta::toStringstream() const
{
    std::stringstream out;
    toCsv(out);
    return out;
}

template<class Archive>
    void MatrixImageMeta::serialize(Archive& ar, [[maybe_unused]] unsigned int version)
    {

        ar& CEREAL_NVP(width_pixel);
        ar& CEREAL_NVP(height_pixel);
        ar& CEREAL_NVP(channel);
        ar& CEREAL_NVP(timeStamp);
        ar& CEREAL_NVP(sendMode);
        ar& CEREAL_NVP(format);
        ar& CEREAL_NVP(index);
        ar& CEREAL_NVP(count);
        ar& CEREAL_NVP(depth_bit);
        ar& CEREAL_NVP(align);
        ar& CEREAL_NVP(imageCounter);
    }


template void MatrixImageMeta::serialize(cereal::BinaryInputArchive& ar, unsigned int version);
template void MatrixImageMeta::serialize(cereal::BinaryOutputArchive& ar, unsigned int version);
template void MatrixImageMeta::serialize(cereal::IntrospectionArchive& ar, unsigned int version);
template void MatrixImageMeta::serialize(cereal::JSONInputArchive& ar, unsigned int version);
template void MatrixImageMeta::serialize(cereal::JSONOutputArchive& ar, unsigned int version);
template void MatrixImageMeta::serialize(cereal::PortableBinaryInputArchive& ar, unsigned int version);
template void MatrixImageMeta::serialize(cereal::PortableBinaryOutputArchive& ar, unsigned int version);

std::ostream& operator<< (std::ostream& os, const MatrixImageMeta& matriximagemeta)
{
    matriximagemeta.toCsv(os);
    return os;
}

bool operator== (const MatrixImageMeta &lhs, const MatrixImageMeta &rhs)
{
    return (  lhs.width_pixel== rhs.width_pixel &&
     lhs.height_pixel== rhs.height_pixel &&
     lhs.channel== rhs.channel &&
     lhs.timeStamp== rhs.timeStamp &&
     lhs.sendMode== rhs.sendMode &&
     lhs.format== rhs.format &&
     lhs.index== rhs.index &&
     lhs.count== rhs.count &&
     lhs.depth_bit== rhs.depth_bit &&
     lhs.align== rhs.align &&
     lhs.imageCounter== rhs.imageCounter 
    );
}

std::size_t MatrixImageMeta::hash() const noexcept
{
    std::size_t result = 17;
    result = result * 31 + std::hash<int32_t>()(width_pixel);
    result = result * 31 + std::hash<int32_t>()(height_pixel);
    result = result * 31 + std::hash<int32_t>()(channel);
    result = result * 31 + std::hash<int64_t>()(timeStamp);
    result = result * 31 + std::hash<SEND_MODE>()(sendMode);
    result = result * 31 + std::hash<IMAGE_FORMAT>()(format);
    result = result * 31 + std::hash<int32_t>()(index);
    result = result * 31 + std::hash<int32_t>()(count);
    result = result * 31 + std::hash<int32_t>()(depth_bit);
    result = result * 31 + std::hash<int32_t>()(align);
    result = result * 31 + std::hash<int32_t>()(imageCounter);
    return result;
}

 } // namespace av
 } // namespace horizon


CEREAL_BIND_TO_ARCHIVES(av::horizon::MatrixImageMeta)

