#include <interfaces/MatrixRawObjectList/MatrixRawObjectList.h>

#include <iterator>
#include <type_traits>

#include <cereal/cereal.hpp>
#include <cereal/archives/json.hpp>
#include <cereal/archives/binary.hpp>

#include <cereal/types/map.hpp>
#include <cereal/types/vector.hpp>
#include <cereal/types/array.hpp>
#include <cereal/types/string.hpp>
#include <cereal/types/complex.hpp>

#include <ODatum/IntrospectionArchive.h>
#include <ODatum/Hash.h>





 namespace av {
 namespace horizon {


MatrixRawObjectList::MatrixRawObjectList() = default;

MatrixRawObjectList::MatrixRawObjectList(
    int64_t timeStamp_arg ,
    int32_t camId_arg ,
    uint32_t numObjects_arg ,
    std::vector<MatrixRawObject> objects_arg 
    ):
    timeStamp { std::move(timeStamp_arg) } // NOLINT
      ,camId { std::move(camId_arg) } // NOLINT
      ,numObjects { std::move(numObjects_arg) } // NOLINT
      ,objects { std::move(objects_arg) } // NOLINT
      {
}

MatrixRawObjectList::MatrixRawObjectList(const MatrixRawObjectList&) = default;

// NOLINTNEXTLINE
MatrixRawObjectList::MatrixRawObjectList(MatrixRawObjectList&&) = default;

MatrixRawObjectList& MatrixRawObjectList::operator=(const MatrixRawObjectList&) = default;

// NOLINTNEXTLINE
MatrixRawObjectList& MatrixRawObjectList::operator=(MatrixRawObjectList&&) = default;

void MatrixRawObjectList::toCsv(std::ostream& out) const
{
    out.precision(16);
    out << std::boolalpha;
    out << timeStamp;out << ",";
    out << camId;out << ",";
    out << numObjects;out << ",";
    // Enclose sequence in double quotations and curly brackets to keep CSV line length uniform
        
        out << "\"{{";
        
        if (!objects.empty())
        {
            std::copy(objects.begin(), objects.end() - 1, std::ostream_iterator<MatrixRawObject>(out, "},{"));
            std::copy(objects.end()-1, objects.end(), std::ostream_iterator<MatrixRawObject>(out, ""));
        }
        out << "}}\"";
    
}

std::stringstream MatrixRawObjectList::toStringstream() const
{
    std::stringstream out;
    toCsv(out);
    return out;
}

template<class Archive>
    void MatrixRawObjectList::serialize(Archive& ar, [[maybe_unused]] unsigned int version)
    {

        ar& CEREAL_NVP(timeStamp);
        ar& CEREAL_NVP(camId);
        ar& CEREAL_NVP(numObjects);
        ar& CEREAL_NVP(objects);
    }


template void MatrixRawObjectList::serialize(cereal::BinaryInputArchive& ar, unsigned int version);
template void MatrixRawObjectList::serialize(cereal::BinaryOutputArchive& ar, unsigned int version);
template void MatrixRawObjectList::serialize(cereal::IntrospectionArchive& ar, unsigned int version);
template void MatrixRawObjectList::serialize(cereal::JSONInputArchive& ar, unsigned int version);
template void MatrixRawObjectList::serialize(cereal::JSONOutputArchive& ar, unsigned int version);
template void MatrixRawObjectList::serialize(cereal::PortableBinaryInputArchive& ar, unsigned int version);
template void MatrixRawObjectList::serialize(cereal::PortableBinaryOutputArchive& ar, unsigned int version);

std::ostream& operator<< (std::ostream& os, const MatrixRawObjectList& matrixrawobjectlist)
{
    matrixrawobjectlist.toCsv(os);
    return os;
}

bool operator== (const MatrixRawObjectList &lhs, const MatrixRawObjectList &rhs)
{
    return (  lhs.timeStamp== rhs.timeStamp &&
     lhs.camId== rhs.camId &&
     lhs.numObjects== rhs.numObjects &&
     lhs.objects== rhs.objects 
    );
}

std::size_t MatrixRawObjectList::hash() const noexcept
{
    std::size_t result = 17;
    result = result * 31 + std::hash<int64_t>()(timeStamp);
    result = result * 31 + std::hash<int32_t>()(camId);
    result = result * 31 + std::hash<uint32_t>()(numObjects);
    result = result * 31 + hashVector(objects);
    return result;
}

 } // namespace av
 } // namespace horizon


CEREAL_BIND_TO_ARCHIVES(av::horizon::MatrixRawObjectList)

