#include <interfaces/MatrixTrackedObjectList/MatrixTrackedObjectList.h>

#include <iterator>
#include <type_traits>

#include <cereal/cereal.hpp>
#include <cereal/archives/json.hpp>
#include <cereal/archives/binary.hpp>

#include <cereal/types/map.hpp>
#include <cereal/types/vector.hpp>
#include <cereal/types/array.hpp>
#include <cereal/types/string.hpp>
#include <cereal/types/complex.hpp>

#include <ODatum/IntrospectionArchive.h>
#include <ODatum/Hash.h>





 namespace av {
 namespace horizon {


MatrixTrackedObjectList::MatrixTrackedObjectList() = default;

MatrixTrackedObjectList::MatrixTrackedObjectList(
    int64_t timeStamp_arg ,
    int32_t camId_arg ,
    uint32_t numTracks_arg ,
    std::vector<MatrixTrackedObject> tracks_arg 
    ):
    timeStamp { std::move(timeStamp_arg) } // NOLINT
      ,camId { std::move(camId_arg) } // NOLINT
      ,numTracks { std::move(numTracks_arg) } // NOLINT
      ,tracks { std::move(tracks_arg) } // NOLINT
      {
}

MatrixTrackedObjectList::MatrixTrackedObjectList(const MatrixTrackedObjectList&) = default;

// NOLINTNEXTLINE
MatrixTrackedObjectList::MatrixTrackedObjectList(MatrixTrackedObjectList&&) = default;

MatrixTrackedObjectList& MatrixTrackedObjectList::operator=(const MatrixTrackedObjectList&) = default;

// NOLINTNEXTLINE
MatrixTrackedObjectList& MatrixTrackedObjectList::operator=(MatrixTrackedObjectList&&) = default;

void MatrixTrackedObjectList::toCsv(std::ostream& out) const
{
    out.precision(16);
    out << std::boolalpha;
    out << timeStamp;out << ",";
    out << camId;out << ",";
    out << numTracks;out << ",";
    // Enclose sequence in double quotations and curly brackets to keep CSV line length uniform
        
        out << "\"{{";
        
        if (!tracks.empty())
        {
            std::copy(tracks.begin(), tracks.end() - 1, std::ostream_iterator<MatrixTrackedObject>(out, "},{"));
            std::copy(tracks.end()-1, tracks.end(), std::ostream_iterator<MatrixTrackedObject>(out, ""));
        }
        out << "}}\"";
    
}

std::stringstream MatrixTrackedObjectList::toStringstream() const
{
    std::stringstream out;
    toCsv(out);
    return out;
}

template<class Archive>
    void MatrixTrackedObjectList::serialize(Archive& ar, [[maybe_unused]] unsigned int version)
    {

        ar& CEREAL_NVP(timeStamp);
        ar& CEREAL_NVP(camId);
        ar& CEREAL_NVP(numTracks);
        ar& CEREAL_NVP(tracks);
    }


template void MatrixTrackedObjectList::serialize(cereal::BinaryInputArchive& ar, unsigned int version);
template void MatrixTrackedObjectList::serialize(cereal::BinaryOutputArchive& ar, unsigned int version);
template void MatrixTrackedObjectList::serialize(cereal::IntrospectionArchive& ar, unsigned int version);
template void MatrixTrackedObjectList::serialize(cereal::JSONInputArchive& ar, unsigned int version);
template void MatrixTrackedObjectList::serialize(cereal::JSONOutputArchive& ar, unsigned int version);
template void MatrixTrackedObjectList::serialize(cereal::PortableBinaryInputArchive& ar, unsigned int version);
template void MatrixTrackedObjectList::serialize(cereal::PortableBinaryOutputArchive& ar, unsigned int version);

std::ostream& operator<< (std::ostream& os, const MatrixTrackedObjectList& matrixtrackedobjectlist)
{
    matrixtrackedobjectlist.toCsv(os);
    return os;
}

bool operator== (const MatrixTrackedObjectList &lhs, const MatrixTrackedObjectList &rhs)
{
    return (  lhs.timeStamp== rhs.timeStamp &&
     lhs.camId== rhs.camId &&
     lhs.numTracks== rhs.numTracks &&
     lhs.tracks== rhs.tracks 
    );
}

std::size_t MatrixTrackedObjectList::hash() const noexcept
{
    std::size_t result = 17;
    result = result * 31 + std::hash<int64_t>()(timeStamp);
    result = result * 31 + std::hash<int32_t>()(camId);
    result = result * 31 + std::hash<uint32_t>()(numTracks);
    result = result * 31 + hashVector(tracks);
    return result;
}

 } // namespace av
 } // namespace horizon


CEREAL_BIND_TO_ARCHIVES(av::horizon::MatrixTrackedObjectList)

