#include <interfaces/MatrixVehicleCoordSystemInfo/MatrixVehicleCoordSystemInfo.h>

#include <iterator>
#include <type_traits>

#include <cereal/cereal.hpp>
#include <cereal/archives/json.hpp>
#include <cereal/archives/binary.hpp>

#include <cereal/types/map.hpp>
#include <cereal/types/vector.hpp>
#include <cereal/types/array.hpp>
#include <cereal/types/string.hpp>
#include <cereal/types/complex.hpp>

#include <ODatum/IntrospectionArchive.h>
#include <ODatum/Hash.h>





 namespace av {
 namespace horizon {


MatrixVehicleCoordSystemInfo::MatrixVehicleCoordSystemInfo() = default;

MatrixVehicleCoordSystemInfo::MatrixVehicleCoordSystemInfo(
    double yawRate_rad_arg ,
    msg::Velocity velocity_arg ,
    double length_m_arg ,
    double width_m_arg ,
    double height_m_arg ,
    std::vector<Point3D> poly_arg ,
    Point3D position_arg ,
    int32_t traversable_arg ,
    double hmw_arg ,
    double ttc_arg ,
    int32_t currentLane_arg ,
    Point3D positionObs_arg ,
    double sigmaYaw_arg ,
    std::vector<double> sigmaVelocity_arg ,
    double sigmaWidth_arg ,
    double sigmaHeight_arg ,
    std::vector<double> sigmaPosition_arg 
    ):
    yawRate_rad { std::move(yawRate_rad_arg) } // NOLINT
      ,velocity { std::move(velocity_arg) } // NOLINT
      ,length_m { std::move(length_m_arg) } // NOLINT
      ,width_m { std::move(width_m_arg) } // NOLINT
      ,height_m { std::move(height_m_arg) } // NOLINT
      ,poly { std::move(poly_arg) } // NOLINT
      ,position { std::move(position_arg) } // NOLINT
      ,traversable { std::move(traversable_arg) } // NOLINT
      ,hmw { std::move(hmw_arg) } // NOLINT
      ,ttc { std::move(ttc_arg) } // NOLINT
      ,currentLane { std::move(currentLane_arg) } // NOLINT
      ,positionObs { std::move(positionObs_arg) } // NOLINT
      ,sigmaYaw { std::move(sigmaYaw_arg) } // NOLINT
      ,sigmaVelocity { std::move(sigmaVelocity_arg) } // NOLINT
      ,sigmaWidth { std::move(sigmaWidth_arg) } // NOLINT
      ,sigmaHeight { std::move(sigmaHeight_arg) } // NOLINT
      ,sigmaPosition { std::move(sigmaPosition_arg) } // NOLINT
      {
}

MatrixVehicleCoordSystemInfo::MatrixVehicleCoordSystemInfo(const MatrixVehicleCoordSystemInfo&) = default;

// NOLINTNEXTLINE
MatrixVehicleCoordSystemInfo::MatrixVehicleCoordSystemInfo(MatrixVehicleCoordSystemInfo&&) = default;

MatrixVehicleCoordSystemInfo& MatrixVehicleCoordSystemInfo::operator=(const MatrixVehicleCoordSystemInfo&) = default;

// NOLINTNEXTLINE
MatrixVehicleCoordSystemInfo& MatrixVehicleCoordSystemInfo::operator=(MatrixVehicleCoordSystemInfo&&) = default;

void MatrixVehicleCoordSystemInfo::toCsv(std::ostream& out) const
{
    out.precision(16);
    out << std::boolalpha;
    out << yawRate_rad;out << ",";
    out << velocity;out << ",";
    out << length_m;out << ",";
    out << width_m;out << ",";
    out << height_m;out << ",";
    // Enclose sequence in double quotations and curly brackets to keep CSV line length uniform
        
        out << "\"{{";
        
        if (!poly.empty())
        {
            std::copy(poly.begin(), poly.end() - 1, std::ostream_iterator<Point3D>(out, "},{"));
            std::copy(poly.end()-1, poly.end(), std::ostream_iterator<Point3D>(out, ""));
        }
        out << "}}\"";out << ",";
    out << position;out << ",";
    out << traversable;out << ",";
    out << hmw;out << ",";
    out << ttc;out << ",";
    out << currentLane;out << ",";
    out << positionObs;out << ",";
    out << sigmaYaw;out << ",";
    // Enclose sequence in double quotations and curly brackets to keep CSV line length uniform
        
        out << "\"{{";
        
        if (!sigmaVelocity.empty())
        {
            std::copy(sigmaVelocity.begin(), sigmaVelocity.end() - 1, std::ostream_iterator<double>(out, "},{"));
            std::copy(sigmaVelocity.end()-1, sigmaVelocity.end(), std::ostream_iterator<double>(out, ""));
        }
        out << "}}\"";out << ",";
    out << sigmaWidth;out << ",";
    out << sigmaHeight;out << ",";
    // Enclose sequence in double quotations and curly brackets to keep CSV line length uniform
        
        out << "\"{{";
        
        if (!sigmaPosition.empty())
        {
            std::copy(sigmaPosition.begin(), sigmaPosition.end() - 1, std::ostream_iterator<double>(out, "},{"));
            std::copy(sigmaPosition.end()-1, sigmaPosition.end(), std::ostream_iterator<double>(out, ""));
        }
        out << "}}\"";
    
}

std::stringstream MatrixVehicleCoordSystemInfo::toStringstream() const
{
    std::stringstream out;
    toCsv(out);
    return out;
}

template<class Archive>
    void MatrixVehicleCoordSystemInfo::serialize(Archive& ar, [[maybe_unused]] unsigned int version)
    {

        ar& CEREAL_NVP(yawRate_rad);
        ar& CEREAL_NVP(velocity);
        ar& CEREAL_NVP(length_m);
        ar& CEREAL_NVP(width_m);
        ar& CEREAL_NVP(height_m);
        ar& CEREAL_NVP(poly);
        ar& CEREAL_NVP(position);
        ar& CEREAL_NVP(traversable);
        ar& CEREAL_NVP(hmw);
        ar& CEREAL_NVP(ttc);
        ar& CEREAL_NVP(currentLane);
        ar& CEREAL_NVP(positionObs);
        ar& CEREAL_NVP(sigmaYaw);
        ar& CEREAL_NVP(sigmaVelocity);
        ar& CEREAL_NVP(sigmaWidth);
        ar& CEREAL_NVP(sigmaHeight);
        ar& CEREAL_NVP(sigmaPosition);
    }


template void MatrixVehicleCoordSystemInfo::serialize(cereal::BinaryInputArchive& ar, unsigned int version);
template void MatrixVehicleCoordSystemInfo::serialize(cereal::BinaryOutputArchive& ar, unsigned int version);
template void MatrixVehicleCoordSystemInfo::serialize(cereal::IntrospectionArchive& ar, unsigned int version);
template void MatrixVehicleCoordSystemInfo::serialize(cereal::JSONInputArchive& ar, unsigned int version);
template void MatrixVehicleCoordSystemInfo::serialize(cereal::JSONOutputArchive& ar, unsigned int version);
template void MatrixVehicleCoordSystemInfo::serialize(cereal::PortableBinaryInputArchive& ar, unsigned int version);
template void MatrixVehicleCoordSystemInfo::serialize(cereal::PortableBinaryOutputArchive& ar, unsigned int version);

std::ostream& operator<< (std::ostream& os, const MatrixVehicleCoordSystemInfo& matrixvehiclecoordsysteminfo)
{
    matrixvehiclecoordsysteminfo.toCsv(os);
    return os;
}

bool operator== (const MatrixVehicleCoordSystemInfo &lhs, const MatrixVehicleCoordSystemInfo &rhs)
{
    return (  lhs.yawRate_rad== rhs.yawRate_rad &&
     lhs.velocity== rhs.velocity &&
     lhs.length_m== rhs.length_m &&
     lhs.width_m== rhs.width_m &&
     lhs.height_m== rhs.height_m &&
     lhs.poly== rhs.poly &&
     lhs.position== rhs.position &&
     lhs.traversable== rhs.traversable &&
     lhs.hmw== rhs.hmw &&
     lhs.ttc== rhs.ttc &&
     lhs.currentLane== rhs.currentLane &&
     lhs.positionObs== rhs.positionObs &&
     lhs.sigmaYaw== rhs.sigmaYaw &&
     lhs.sigmaVelocity== rhs.sigmaVelocity &&
     lhs.sigmaWidth== rhs.sigmaWidth &&
     lhs.sigmaHeight== rhs.sigmaHeight &&
     lhs.sigmaPosition== rhs.sigmaPosition 
    );
}

std::size_t MatrixVehicleCoordSystemInfo::hash() const noexcept
{
    std::size_t result = 17;
    result = result * 31 + std::hash<double>()(yawRate_rad);
    result = result * 31 + std::hash<msg::Velocity>()(velocity);
    result = result * 31 + std::hash<double>()(length_m);
    result = result * 31 + std::hash<double>()(width_m);
    result = result * 31 + std::hash<double>()(height_m);
    result = result * 31 + hashVector(poly);
    result = result * 31 + std::hash<Point3D>()(position);
    result = result * 31 + std::hash<int32_t>()(traversable);
    result = result * 31 + std::hash<double>()(hmw);
    result = result * 31 + std::hash<double>()(ttc);
    result = result * 31 + std::hash<int32_t>()(currentLane);
    result = result * 31 + std::hash<Point3D>()(positionObs);
    result = result * 31 + std::hash<double>()(sigmaYaw);
    result = result * 31 + hashVector(sigmaVelocity);
    result = result * 31 + std::hash<double>()(sigmaWidth);
    result = result * 31 + std::hash<double>()(sigmaHeight);
    result = result * 31 + hashVector(sigmaPosition);
    return result;
}

 } // namespace av
 } // namespace horizon


CEREAL_BIND_TO_ARCHIVES(av::horizon::MatrixVehicleCoordSystemInfo)

