#include <interfaces/MissionGoal/MissionGoal.h>

#include <iterator>
#include <type_traits>

#include <cereal/cereal.hpp>
#include <cereal/archives/json.hpp>
#include <cereal/archives/binary.hpp>

#include <cereal/types/map.hpp>
#include <cereal/types/vector.hpp>
#include <cereal/types/array.hpp>
#include <cereal/types/string.hpp>
#include <cereal/types/complex.hpp>

#include <ODatum/IntrospectionArchive.h>
#include <ODatum/Hash.h>


 namespace av {
 namespace msg {


Point2DSequence::Point2DSequence() = default;

Point2DSequence::Point2DSequence(
    std::vector<Point2D> pointSequence_arg 
    ):
    pointSequence { std::move(pointSequence_arg) } // NOLINT
      
    {
}

Point2DSequence::Point2DSequence(const Point2DSequence&) = default;

// NOLINTNEXTLINE
Point2DSequence::Point2DSequence(Point2DSequence&&) = default;

Point2DSequence& Point2DSequence::operator=(const Point2DSequence&) = default;

// NOLINTNEXTLINE
Point2DSequence& Point2DSequence::operator=(Point2DSequence&&) = default;

void Point2DSequence::toCsv(std::ostream& out) const
{
    out.precision(16);
    out << std::boolalpha;
    // Enclose sequence in double quotations and curly brackets to keep CSV line length uniform
        
        out << "\"{{";
        
        if (!pointSequence.empty())
        {
            std::copy(pointSequence.begin(), pointSequence.end() - 1, std::ostream_iterator<Point2D>(out, "},{"));
            std::copy(pointSequence.end()-1, pointSequence.end(), std::ostream_iterator<Point2D>(out, ""));
        }
        out << "}}\"";
    
}

std::stringstream Point2DSequence::toStringstream() const
{
    std::stringstream out;
    toCsv(out);
    return out;
}

template<class Archive>
    void Point2DSequence::serialize(Archive& ar, [[maybe_unused]] unsigned int version)
    {

        ar& CEREAL_NVP(pointSequence);
    }


template void Point2DSequence::serialize(cereal::BinaryInputArchive& ar, unsigned int version);
template void Point2DSequence::serialize(cereal::BinaryOutputArchive& ar, unsigned int version);
template void Point2DSequence::serialize(cereal::IntrospectionArchive& ar, unsigned int version);
template void Point2DSequence::serialize(cereal::JSONInputArchive& ar, unsigned int version);
template void Point2DSequence::serialize(cereal::JSONOutputArchive& ar, unsigned int version);
template void Point2DSequence::serialize(cereal::PortableBinaryInputArchive& ar, unsigned int version);
template void Point2DSequence::serialize(cereal::PortableBinaryOutputArchive& ar, unsigned int version);

std::ostream& operator<< (std::ostream& os, const Point2DSequence& point2dsequence)
{
    point2dsequence.toCsv(os);
    return os;
}

bool operator== (const Point2DSequence &lhs, const Point2DSequence &rhs)
{
    return (  lhs.pointSequence== rhs.pointSequence 
    );
}

std::size_t Point2DSequence::hash() const noexcept
{
    std::size_t result = 17;
    result = result * 31 + hashVector(pointSequence);
    return result;
}

 } // namespace av
 } // namespace msg


CEREAL_BIND_TO_ARCHIVES(av::msg::Point2DSequence)

 namespace av {
 namespace msg {


OptionalGoalPoseType::OptionalGoalPoseType() = default;

OptionalGoalPoseType::OptionalGoalPoseType(
    bool hasValue_arg ,
    av::msg::SE2 value_arg 
    ):
    hasValue { std::move(hasValue_arg) } // NOLINT
      ,
    value { std::move(value_arg) } // NOLINT
      
    {
}

OptionalGoalPoseType::OptionalGoalPoseType(const OptionalGoalPoseType&) = default;

// NOLINTNEXTLINE
OptionalGoalPoseType::OptionalGoalPoseType(OptionalGoalPoseType&&) = default;

OptionalGoalPoseType& OptionalGoalPoseType::operator=(const OptionalGoalPoseType&) = default;

// NOLINTNEXTLINE
OptionalGoalPoseType& OptionalGoalPoseType::operator=(OptionalGoalPoseType&&) = default;

void OptionalGoalPoseType::toCsv(std::ostream& out) const
{
    out.precision(16);
    out << std::boolalpha;
    out << hasValue;out << ",";
    out << value;
    
}

std::stringstream OptionalGoalPoseType::toStringstream() const
{
    std::stringstream out;
    toCsv(out);
    return out;
}

template<class Archive>
    void OptionalGoalPoseType::serialize(Archive& ar, [[maybe_unused]] unsigned int version)
    {

        ar& CEREAL_NVP(hasValue);
        ar& CEREAL_NVP(value);
    }


template void OptionalGoalPoseType::serialize(cereal::BinaryInputArchive& ar, unsigned int version);
template void OptionalGoalPoseType::serialize(cereal::BinaryOutputArchive& ar, unsigned int version);
template void OptionalGoalPoseType::serialize(cereal::IntrospectionArchive& ar, unsigned int version);
template void OptionalGoalPoseType::serialize(cereal::JSONInputArchive& ar, unsigned int version);
template void OptionalGoalPoseType::serialize(cereal::JSONOutputArchive& ar, unsigned int version);
template void OptionalGoalPoseType::serialize(cereal::PortableBinaryInputArchive& ar, unsigned int version);
template void OptionalGoalPoseType::serialize(cereal::PortableBinaryOutputArchive& ar, unsigned int version);

std::ostream& operator<< (std::ostream& os, const OptionalGoalPoseType& optionalgoalposetype)
{
    optionalgoalposetype.toCsv(os);
    return os;
}

bool operator== (const OptionalGoalPoseType &lhs, const OptionalGoalPoseType &rhs)
{
    return (  lhs.hasValue== rhs.hasValue &&
     lhs.value== rhs.value 
    );
}

std::size_t OptionalGoalPoseType::hash() const noexcept
{
    std::size_t result = 17;
    result = result * 31 + std::hash<bool>()(hasValue);
    result = result * 31 + std::hash<av::msg::SE2>()(value);
    return result;
}

 } // namespace av
 } // namespace msg


CEREAL_BIND_TO_ARCHIVES(av::msg::OptionalGoalPoseType)




 namespace av {
 namespace msg {


MissionGoal::MissionGoal() = default;

MissionGoal::MissionGoal(
    plan_ctrl::PlannerHeader header_arg ,
    msg::GUID goalId_arg ,
    Point2DSequence missionRoute_arg ,
    OptionalGoalPoseType optionalGoalPose_arg 
    ):
    header { std::move(header_arg) } // NOLINT
      ,goalId { std::move(goalId_arg) } // NOLINT
      ,missionRoute { std::move(missionRoute_arg) } // NOLINT
      ,optionalGoalPose { std::move(optionalGoalPose_arg) } // NOLINT
      {
}

MissionGoal::MissionGoal(const MissionGoal&) = default;

// NOLINTNEXTLINE
MissionGoal::MissionGoal(MissionGoal&&) = default;

MissionGoal& MissionGoal::operator=(const MissionGoal&) = default;

// NOLINTNEXTLINE
MissionGoal& MissionGoal::operator=(MissionGoal&&) = default;

void MissionGoal::toCsv(std::ostream& out) const
{
    out.precision(16);
    out << std::boolalpha;
    out << header;out << ",";
    out << goalId;out << ",";
    out << missionRoute;out << ",";
    out << optionalGoalPose;
    
}

std::stringstream MissionGoal::toStringstream() const
{
    std::stringstream out;
    toCsv(out);
    return out;
}

template<class Archive>
    void MissionGoal::serialize(Archive& ar, unsigned int version)
{
    ar& CEREAL_NVP(header);
    ar& CEREAL_NVP(missionRoute);
    if (version <= 2)
    {
        // Before version 3, optionalGoalPose was a non-versioned type (std::pair).
        // The right way to do the upgrade would be to serialize a std::pair and
        // then copy the values to/from optionalGoalPose. But that would require
        // #including <cereal/types/utility.hpp> in the struct.template file.
        // As a work-around, rely on the fact that in binary archives the name part
        // of an NVP is not actually saved, and just serialize the members of
        // optionalGoalPose directly:
        ar& cereal::make_nvp("first", optionalGoalPose.hasValue);
        ar& cereal::make_nvp("second", optionalGoalPose.value);
    }
    else
    {
        ar& CEREAL_NVP(optionalGoalPose);
    }
    if(version >= 2)
    {
        ar& CEREAL_NVP(goalId);
    }
}

}} // to close av::msg namespace temporary

struct MissionGoal : public av::msg::MissionGoal{};
CEREAL_CLASS_VERSION(MissionGoal, 3)
CEREAL_REGISTER_TYPE(MissionGoal)

namespace av {
namespace msg {



template void MissionGoal::serialize(cereal::BinaryInputArchive& ar, unsigned int version);
template void MissionGoal::serialize(cereal::BinaryOutputArchive& ar, unsigned int version);
template void MissionGoal::serialize(cereal::IntrospectionArchive& ar, unsigned int version);
template void MissionGoal::serialize(cereal::JSONInputArchive& ar, unsigned int version);
template void MissionGoal::serialize(cereal::JSONOutputArchive& ar, unsigned int version);
template void MissionGoal::serialize(cereal::PortableBinaryInputArchive& ar, unsigned int version);
template void MissionGoal::serialize(cereal::PortableBinaryOutputArchive& ar, unsigned int version);

std::ostream& operator<< (std::ostream& os, const MissionGoal& missiongoal)
{
    missiongoal.toCsv(os);
    return os;
}

bool operator== (const MissionGoal &lhs, const MissionGoal &rhs)
{
    return (  lhs.header== rhs.header &&
     lhs.goalId== rhs.goalId &&
     lhs.missionRoute== rhs.missionRoute &&
     lhs.optionalGoalPose== rhs.optionalGoalPose 
    );
}

std::size_t MissionGoal::hash() const noexcept
{
    std::size_t result = 17;
    result = result * 31 + std::hash<plan_ctrl::PlannerHeader>()(header);
    result = result * 31 + std::hash<msg::GUID>()(goalId);
    result = result * 31 + std::hash<Point2DSequence>()(missionRoute);
    result = result * 31 + std::hash<OptionalGoalPoseType>()(optionalGoalPose);
    return result;
}

 } // namespace av
 } // namespace msg


CEREAL_BIND_TO_ARCHIVES(av::msg::MissionGoal)

