#include <interfaces/MissionGoalStatus/MissionGoalStatus.h>

#include <iterator>
#include <type_traits>

#include <cereal/cereal.hpp>
#include <cereal/archives/json.hpp>
#include <cereal/archives/binary.hpp>

#include <cereal/types/map.hpp>
#include <cereal/types/vector.hpp>
#include <cereal/types/array.hpp>
#include <cereal/types/string.hpp>
#include <cereal/types/complex.hpp>

#include <ODatum/IntrospectionArchive.h>
#include <ODatum/Hash.h>





 namespace av {
 namespace msg {


MissionGoalStatus::MissionGoalStatus() = default;

MissionGoalStatus::MissionGoalStatus(
    plan_ctrl::PlannerHeader header_arg ,
    msg::GUID goalId_arg ,
    GoalState status_arg ,
    std::string details_arg 
    ):
    header { std::move(header_arg) } // NOLINT
      ,goalId { std::move(goalId_arg) } // NOLINT
      ,status { std::move(status_arg) } // NOLINT
      ,details { std::move(details_arg) } // NOLINT
      {
}

MissionGoalStatus::MissionGoalStatus(const MissionGoalStatus&) = default;

// NOLINTNEXTLINE
MissionGoalStatus::MissionGoalStatus(MissionGoalStatus&&) = default;

MissionGoalStatus& MissionGoalStatus::operator=(const MissionGoalStatus&) = default;

// NOLINTNEXTLINE
MissionGoalStatus& MissionGoalStatus::operator=(MissionGoalStatus&&) = default;

void MissionGoalStatus::toCsv(std::ostream& out) const
{
    out.precision(16);
    out << std::boolalpha;
    out << header;out << ",";
    out << goalId;out << ",";
    out << static_cast<uint32_t>(status);out << ",";
    out << "\"";
        out << details;
        out << "\"";
    
}

std::stringstream MissionGoalStatus::toStringstream() const
{
    std::stringstream out;
    toCsv(out);
    return out;
}

template<class Archive>
    void MissionGoalStatus::serialize(Archive& ar, [[maybe_unused]] unsigned int version)
    {

        ar& CEREAL_NVP(header);
        ar& CEREAL_NVP(goalId);
        ar& CEREAL_NVP(status);
        ar& CEREAL_NVP(details);
    }


template void MissionGoalStatus::serialize(cereal::BinaryInputArchive& ar, unsigned int version);
template void MissionGoalStatus::serialize(cereal::BinaryOutputArchive& ar, unsigned int version);
template void MissionGoalStatus::serialize(cereal::IntrospectionArchive& ar, unsigned int version);
template void MissionGoalStatus::serialize(cereal::JSONInputArchive& ar, unsigned int version);
template void MissionGoalStatus::serialize(cereal::JSONOutputArchive& ar, unsigned int version);
template void MissionGoalStatus::serialize(cereal::PortableBinaryInputArchive& ar, unsigned int version);
template void MissionGoalStatus::serialize(cereal::PortableBinaryOutputArchive& ar, unsigned int version);

std::ostream& operator<< (std::ostream& os, const MissionGoalStatus& missiongoalstatus)
{
    missiongoalstatus.toCsv(os);
    return os;
}

bool operator== (const MissionGoalStatus &lhs, const MissionGoalStatus &rhs)
{
    return (  lhs.header== rhs.header &&
     lhs.goalId== rhs.goalId &&
     lhs.status== rhs.status &&
     lhs.details== rhs.details 
    );
}

std::size_t MissionGoalStatus::hash() const noexcept
{
    std::size_t result = 17;
    result = result * 31 + std::hash<plan_ctrl::PlannerHeader>()(header);
    result = result * 31 + std::hash<msg::GUID>()(goalId);
    result = result * 31 + std::hash<GoalState>()(status);
    result = result * 31 + std::hash<std::string>()(details);
    return result;
}

 } // namespace av
 } // namespace msg


CEREAL_BIND_TO_ARCHIVES(av::msg::MissionGoalStatus)

