#include <interfaces/MissionRoute/MissionRoute.h>

#include <iterator>
#include <type_traits>

#include <cereal/cereal.hpp>
#include <cereal/archives/json.hpp>
#include <cereal/archives/binary.hpp>

#include <cereal/types/map.hpp>
#include <cereal/types/vector.hpp>
#include <cereal/types/array.hpp>
#include <cereal/types/string.hpp>
#include <cereal/types/complex.hpp>

#include <ODatum/IntrospectionArchive.h>
#include <ODatum/Hash.h>


 namespace av {
 namespace msg {


StopZone::StopZone() = default;

StopZone::StopZone(
    av::msg::SE2 finalPose_arg 
    ):
    finalPose { std::move(finalPose_arg) } // NOLINT
      
    {
}

StopZone::StopZone(const StopZone&) = default;

// NOLINTNEXTLINE
StopZone::StopZone(StopZone&&) = default;

StopZone& StopZone::operator=(const StopZone&) = default;

// NOLINTNEXTLINE
StopZone& StopZone::operator=(StopZone&&) = default;

void StopZone::toCsv(std::ostream& out) const
{
    out.precision(16);
    out << std::boolalpha;
    out << finalPose;
    
}

std::stringstream StopZone::toStringstream() const
{
    std::stringstream out;
    toCsv(out);
    return out;
}

template<class Archive>
    void StopZone::serialize(Archive& ar, [[maybe_unused]] unsigned int version)
    {

        ar& CEREAL_NVP(finalPose);
    }


template void StopZone::serialize(cereal::BinaryInputArchive& ar, unsigned int version);
template void StopZone::serialize(cereal::BinaryOutputArchive& ar, unsigned int version);
template void StopZone::serialize(cereal::IntrospectionArchive& ar, unsigned int version);
template void StopZone::serialize(cereal::JSONInputArchive& ar, unsigned int version);
template void StopZone::serialize(cereal::JSONOutputArchive& ar, unsigned int version);
template void StopZone::serialize(cereal::PortableBinaryInputArchive& ar, unsigned int version);
template void StopZone::serialize(cereal::PortableBinaryOutputArchive& ar, unsigned int version);

std::ostream& operator<< (std::ostream& os, const StopZone& stopzone)
{
    stopzone.toCsv(os);
    return os;
}

bool operator== (const StopZone &lhs, const StopZone &rhs)
{
    return (  lhs.finalPose== rhs.finalPose 
    );
}

std::size_t StopZone::hash() const noexcept
{
    std::size_t result = 17;
    result = result * 31 + std::hash<av::msg::SE2>()(finalPose);
    return result;
}

 } // namespace av
 } // namespace msg


CEREAL_BIND_TO_ARCHIVES(av::msg::StopZone)




 namespace av {
 namespace msg {


MissionRoute::MissionRoute() = default;

MissionRoute::MissionRoute(
    plan_ctrl::PlannerHeader header_arg ,
    msg::GUID routeId_arg ,
    std::vector<MapId> viaRoadBlocks_arg ,
    StopZone stopZone_arg ,
    LaneRejectList laneRejectList_arg 
    ):
    header { std::move(header_arg) } // NOLINT
      ,routeId { std::move(routeId_arg) } // NOLINT
      ,viaRoadBlocks { std::move(viaRoadBlocks_arg) } // NOLINT
      ,stopZone { std::move(stopZone_arg) } // NOLINT
      ,laneRejectList { std::move(laneRejectList_arg) } // NOLINT
      {
}

MissionRoute::MissionRoute(const MissionRoute&) = default;

// NOLINTNEXTLINE
MissionRoute::MissionRoute(MissionRoute&&) = default;

MissionRoute& MissionRoute::operator=(const MissionRoute&) = default;

// NOLINTNEXTLINE
MissionRoute& MissionRoute::operator=(MissionRoute&&) = default;

void MissionRoute::toCsv(std::ostream& out) const
{
    out.precision(16);
    out << std::boolalpha;
    out << header;out << ",";
    out << routeId;out << ",";
    // Enclose sequence in double quotations and curly brackets to keep CSV line length uniform
        
        out << "\"{{";
        
        if (!viaRoadBlocks.empty())
        {
            std::copy(viaRoadBlocks.begin(), viaRoadBlocks.end() - 1, std::ostream_iterator<MapId>(out, "},{"));
            std::copy(viaRoadBlocks.end()-1, viaRoadBlocks.end(), std::ostream_iterator<MapId>(out, ""));
        }
        out << "}}\"";out << ",";
    out << stopZone;out << ",";
    out << laneRejectList;
    
}

std::stringstream MissionRoute::toStringstream() const
{
    std::stringstream out;
    toCsv(out);
    return out;
}

template<class Archive>
    void MissionRoute::serialize(Archive& ar, unsigned int version)
{
    if(version >= 1)
	{
		ar& CEREAL_NVP(header);
        ar& CEREAL_NVP(routeId);
		ar& CEREAL_NVP(viaRoadBlocks);
		ar& CEREAL_NVP(stopZone);
	}
    if(version >= 2)
    {
        ar& CEREAL_NVP(laneRejectList);
    }
}


template void MissionRoute::serialize(cereal::BinaryInputArchive& ar, unsigned int version);
template void MissionRoute::serialize(cereal::BinaryOutputArchive& ar, unsigned int version);
template void MissionRoute::serialize(cereal::IntrospectionArchive& ar, unsigned int version);
template void MissionRoute::serialize(cereal::JSONInputArchive& ar, unsigned int version);
template void MissionRoute::serialize(cereal::JSONOutputArchive& ar, unsigned int version);
template void MissionRoute::serialize(cereal::PortableBinaryInputArchive& ar, unsigned int version);
template void MissionRoute::serialize(cereal::PortableBinaryOutputArchive& ar, unsigned int version);

std::ostream& operator<< (std::ostream& os, const MissionRoute& missionroute)
{
    missionroute.toCsv(os);
    return os;
}

bool operator== (const MissionRoute &lhs, const MissionRoute &rhs)
{
    return (  lhs.header== rhs.header &&
     lhs.routeId== rhs.routeId &&
     lhs.viaRoadBlocks== rhs.viaRoadBlocks &&
     lhs.stopZone== rhs.stopZone &&
     lhs.laneRejectList== rhs.laneRejectList 
    );
}

std::size_t MissionRoute::hash() const noexcept
{
    std::size_t result = 17;
    result = result * 31 + std::hash<plan_ctrl::PlannerHeader>()(header);
    result = result * 31 + std::hash<msg::GUID>()(routeId);
    result = result * 31 + hashVector(viaRoadBlocks);
    result = result * 31 + std::hash<StopZone>()(stopZone);
    result = result * 31 + std::hash<LaneRejectList>()(laneRejectList);
    return result;
}

 } // namespace av
 } // namespace msg


CEREAL_BIND_TO_ARCHIVES(av::msg::MissionRoute)

