#include <interfaces/MissionRouteAcceptanceStatus/MissionRouteAcceptanceStatus.h>

#include <iterator>
#include <type_traits>

#include <cereal/cereal.hpp>
#include <cereal/archives/json.hpp>
#include <cereal/archives/binary.hpp>

#include <cereal/types/map.hpp>
#include <cereal/types/vector.hpp>
#include <cereal/types/array.hpp>
#include <cereal/types/string.hpp>
#include <cereal/types/complex.hpp>

#include <ODatum/IntrospectionArchive.h>
#include <ODatum/Hash.h>





 namespace av {
 namespace msg {


MissionRouteAcceptanceStatus::MissionRouteAcceptanceStatus() = default;

MissionRouteAcceptanceStatus::MissionRouteAcceptanceStatus(
    plan_ctrl::PlannerHeader header_arg ,
    msg::GUID routeId_arg ,
    RouteAcceptanceStatus status_arg ,
    std::string rejectionInfo_arg 
    ):
    header { std::move(header_arg) } // NOLINT
      ,routeId { std::move(routeId_arg) } // NOLINT
      ,status { std::move(status_arg) } // NOLINT
      ,rejectionInfo { std::move(rejectionInfo_arg) } // NOLINT
      {
}

MissionRouteAcceptanceStatus::MissionRouteAcceptanceStatus(const MissionRouteAcceptanceStatus&) = default;

// NOLINTNEXTLINE
MissionRouteAcceptanceStatus::MissionRouteAcceptanceStatus(MissionRouteAcceptanceStatus&&) = default;

MissionRouteAcceptanceStatus& MissionRouteAcceptanceStatus::operator=(const MissionRouteAcceptanceStatus&) = default;

// NOLINTNEXTLINE
MissionRouteAcceptanceStatus& MissionRouteAcceptanceStatus::operator=(MissionRouteAcceptanceStatus&&) = default;

void MissionRouteAcceptanceStatus::toCsv(std::ostream& out) const
{
    out.precision(16);
    out << std::boolalpha;
    out << header;out << ",";
    out << routeId;out << ",";
    out << static_cast<uint32_t>(status);out << ",";
    out << "\"";
        out << rejectionInfo;
        out << "\"";
    
}

std::stringstream MissionRouteAcceptanceStatus::toStringstream() const
{
    std::stringstream out;
    toCsv(out);
    return out;
}

template<class Archive>
    void MissionRouteAcceptanceStatus::serialize(Archive& ar, [[maybe_unused]] unsigned int version)
    {

        ar& CEREAL_NVP(header);
        ar& CEREAL_NVP(routeId);
        ar& CEREAL_NVP(status);
        ar& CEREAL_NVP(rejectionInfo);
    }


template void MissionRouteAcceptanceStatus::serialize(cereal::BinaryInputArchive& ar, unsigned int version);
template void MissionRouteAcceptanceStatus::serialize(cereal::BinaryOutputArchive& ar, unsigned int version);
template void MissionRouteAcceptanceStatus::serialize(cereal::IntrospectionArchive& ar, unsigned int version);
template void MissionRouteAcceptanceStatus::serialize(cereal::JSONInputArchive& ar, unsigned int version);
template void MissionRouteAcceptanceStatus::serialize(cereal::JSONOutputArchive& ar, unsigned int version);
template void MissionRouteAcceptanceStatus::serialize(cereal::PortableBinaryInputArchive& ar, unsigned int version);
template void MissionRouteAcceptanceStatus::serialize(cereal::PortableBinaryOutputArchive& ar, unsigned int version);

std::ostream& operator<< (std::ostream& os, const MissionRouteAcceptanceStatus& missionrouteacceptancestatus)
{
    missionrouteacceptancestatus.toCsv(os);
    return os;
}

bool operator== (const MissionRouteAcceptanceStatus &lhs, const MissionRouteAcceptanceStatus &rhs)
{
    return (  lhs.header== rhs.header &&
     lhs.routeId== rhs.routeId &&
     lhs.status== rhs.status &&
     lhs.rejectionInfo== rhs.rejectionInfo 
    );
}

std::size_t MissionRouteAcceptanceStatus::hash() const noexcept
{
    std::size_t result = 17;
    result = result * 31 + std::hash<plan_ctrl::PlannerHeader>()(header);
    result = result * 31 + std::hash<msg::GUID>()(routeId);
    result = result * 31 + std::hash<RouteAcceptanceStatus>()(status);
    result = result * 31 + std::hash<std::string>()(rejectionInfo);
    return result;
}

 } // namespace av
 } // namespace msg


CEREAL_BIND_TO_ARCHIVES(av::msg::MissionRouteAcceptanceStatus)

