#include <interfaces/MonitorMessageAurix/MonitorMessageAurix.h>

#include <iterator>
#include <type_traits>

#include <cereal/cereal.hpp>
#include <cereal/archives/json.hpp>
#include <cereal/archives/binary.hpp>

#include <cereal/types/map.hpp>
#include <cereal/types/vector.hpp>
#include <cereal/types/array.hpp>
#include <cereal/types/string.hpp>
#include <cereal/types/complex.hpp>

#include <ODatum/IntrospectionArchive.h>
#include <ODatum/Hash.h>


 namespace av {
 namespace msg {
 namespace sysmonaurix {


ParameterCompact::ParameterCompact() = default;

ParameterCompact::ParameterCompact(
    uint32_t faultId_arg ,
    ParameterStatus status_arg ,
    ParameterType type_arg ,
    std::array<char,32> faultName_arg ,
    std::array<char,32> measuredValue_arg ,
    std::array<char,32> thresholdValue_arg 
    ):
    faultId { std::move(faultId_arg) } // NOLINT
      ,
    status { std::move(status_arg) } // NOLINT
      ,
    type { std::move(type_arg) } // NOLINT
      ,
    faultName { std::move(faultName_arg) } // NOLINT
      ,
    measuredValue { std::move(measuredValue_arg) } // NOLINT
      ,
    thresholdValue { std::move(thresholdValue_arg) } // NOLINT
      
    {
}

ParameterCompact::ParameterCompact(const ParameterCompact&) = default;

// NOLINTNEXTLINE
ParameterCompact::ParameterCompact(ParameterCompact&&) = default;

ParameterCompact& ParameterCompact::operator=(const ParameterCompact&) = default;

// NOLINTNEXTLINE
ParameterCompact& ParameterCompact::operator=(ParameterCompact&&) = default;

void ParameterCompact::toCsv(std::ostream& out) const
{
    out.precision(16);
    out << std::boolalpha;
    out << faultId;out << ",";
    out << static_cast<uint32_t>(status);out << ",";
    out << static_cast<uint32_t>(type);out << ",";
    // Enclose array in double quotations and curly brackets to keep CSV line length uniform
        
        out << "\"{{";
        
        if (!faultName.empty())
        {
            std::copy(faultName.begin(), faultName.end() - 1, std::ostream_iterator<char>(out, "},{"));
            out << faultName.back();
        }
        out << "}}\"";out << ",";
    // Enclose array in double quotations and curly brackets to keep CSV line length uniform
        
        out << "\"{{";
        
        if (!measuredValue.empty())
        {
            std::copy(measuredValue.begin(), measuredValue.end() - 1, std::ostream_iterator<char>(out, "},{"));
            out << measuredValue.back();
        }
        out << "}}\"";out << ",";
    // Enclose array in double quotations and curly brackets to keep CSV line length uniform
        
        out << "\"{{";
        
        if (!thresholdValue.empty())
        {
            std::copy(thresholdValue.begin(), thresholdValue.end() - 1, std::ostream_iterator<char>(out, "},{"));
            out << thresholdValue.back();
        }
        out << "}}\"";
    
}

std::stringstream ParameterCompact::toStringstream() const
{
    std::stringstream out;
    toCsv(out);
    return out;
}

template<class Archive>
    void ParameterCompact::serialize(Archive& ar, [[maybe_unused]] unsigned int version)
    {

        ar& CEREAL_NVP(faultId);
        ar& CEREAL_NVP(status);
        ar& CEREAL_NVP(type);
        ar& CEREAL_NVP(faultName);
        ar& CEREAL_NVP(measuredValue);
        ar& CEREAL_NVP(thresholdValue);
    }


template void ParameterCompact::serialize(cereal::BinaryInputArchive& ar, unsigned int version);
template void ParameterCompact::serialize(cereal::BinaryOutputArchive& ar, unsigned int version);
template void ParameterCompact::serialize(cereal::IntrospectionArchive& ar, unsigned int version);
template void ParameterCompact::serialize(cereal::JSONInputArchive& ar, unsigned int version);
template void ParameterCompact::serialize(cereal::JSONOutputArchive& ar, unsigned int version);
template void ParameterCompact::serialize(cereal::PortableBinaryInputArchive& ar, unsigned int version);
template void ParameterCompact::serialize(cereal::PortableBinaryOutputArchive& ar, unsigned int version);

std::ostream& operator<< (std::ostream& os, const ParameterCompact& parametercompact)
{
    parametercompact.toCsv(os);
    return os;
}

bool operator== (const ParameterCompact &lhs, const ParameterCompact &rhs)
{
    return (  lhs.faultId== rhs.faultId &&
     lhs.status== rhs.status &&
     lhs.type== rhs.type &&
     lhs.faultName== rhs.faultName &&
     lhs.measuredValue== rhs.measuredValue &&
     lhs.thresholdValue== rhs.thresholdValue 
    );
}

std::size_t ParameterCompact::hash() const noexcept
{
    std::size_t result = 17;
    result = result * 31 + std::hash<uint32_t>()(faultId);
    result = result * 31 + std::hash<ParameterStatus>()(status);
    result = result * 31 + std::hash<ParameterType>()(type);
    result = result * 31 + hashVector(faultName);
    result = result * 31 + hashVector(measuredValue);
    result = result * 31 + hashVector(thresholdValue);
    return result;
}

 } // namespace av
 } // namespace msg
 } // namespace sysmonaurix


CEREAL_BIND_TO_ARCHIVES(av::msg::sysmonaurix::ParameterCompact)




 namespace av {
 namespace msg {
 namespace sysmonaurix {


MonitorMessageAurix::MonitorMessageAurix() = default;

MonitorMessageAurix::MonitorMessageAurix(
    TimePoint timeStamp_arg ,
    std::array<char,32> moduleName_arg ,
    std::array<ParameterCompact,10> parameterList_arg 
    ):
    timeStamp { std::move(timeStamp_arg) } // NOLINT
      ,moduleName { std::move(moduleName_arg) } // NOLINT
      ,parameterList { std::move(parameterList_arg) } // NOLINT
      {
}

MonitorMessageAurix::MonitorMessageAurix(const MonitorMessageAurix&) = default;

// NOLINTNEXTLINE
MonitorMessageAurix::MonitorMessageAurix(MonitorMessageAurix&&) = default;

MonitorMessageAurix& MonitorMessageAurix::operator=(const MonitorMessageAurix&) = default;

// NOLINTNEXTLINE
MonitorMessageAurix& MonitorMessageAurix::operator=(MonitorMessageAurix&&) = default;

void MonitorMessageAurix::toCsv(std::ostream& out) const
{
    out.precision(16);
    out << std::boolalpha;
    out << timeStamp;out << ",";
    // Enclose array in double quotations and curly brackets to keep CSV line length uniform
        
        out << "\"{{";
        
        if (!moduleName.empty())
        {
            std::copy(moduleName.begin(), moduleName.end() - 1, std::ostream_iterator<char>(out, "},{"));
            out << moduleName.back();
        }
        out << "}}\"";out << ",";
    // Enclose array in double quotations and curly brackets to keep CSV line length uniform
        
        out << "\"{{";
        
        if (!parameterList.empty())
        {
            std::copy(parameterList.begin(), parameterList.end() - 1, std::ostream_iterator<ParameterCompact>(out, "},{"));
            out << parameterList.back();
        }
        out << "}}\"";
    
}

std::stringstream MonitorMessageAurix::toStringstream() const
{
    std::stringstream out;
    toCsv(out);
    return out;
}

template<class Archive>
    void MonitorMessageAurix::serialize(Archive& ar, [[maybe_unused]] unsigned int version)
    {

        ar& CEREAL_NVP(timeStamp);
        ar& CEREAL_NVP(moduleName);
        ar& CEREAL_NVP(parameterList);
    }


template void MonitorMessageAurix::serialize(cereal::BinaryInputArchive& ar, unsigned int version);
template void MonitorMessageAurix::serialize(cereal::BinaryOutputArchive& ar, unsigned int version);
template void MonitorMessageAurix::serialize(cereal::IntrospectionArchive& ar, unsigned int version);
template void MonitorMessageAurix::serialize(cereal::JSONInputArchive& ar, unsigned int version);
template void MonitorMessageAurix::serialize(cereal::JSONOutputArchive& ar, unsigned int version);
template void MonitorMessageAurix::serialize(cereal::PortableBinaryInputArchive& ar, unsigned int version);
template void MonitorMessageAurix::serialize(cereal::PortableBinaryOutputArchive& ar, unsigned int version);

std::ostream& operator<< (std::ostream& os, const MonitorMessageAurix& monitormessageaurix)
{
    monitormessageaurix.toCsv(os);
    return os;
}

bool operator== (const MonitorMessageAurix &lhs, const MonitorMessageAurix &rhs)
{
    return (  lhs.timeStamp== rhs.timeStamp &&
     lhs.moduleName== rhs.moduleName &&
     lhs.parameterList== rhs.parameterList 
    );
}

std::size_t MonitorMessageAurix::hash() const noexcept
{
    std::size_t result = 17;
    result = result * 31 + 0;
    result = result * 31 + hashVector(moduleName);
    result = result * 31 + hashVector(parameterList);
    return result;
}

 } // namespace av
 } // namespace msg
 } // namespace sysmonaurix


CEREAL_BIND_TO_ARCHIVES(av::msg::sysmonaurix::MonitorMessageAurix)

