#include <interfaces/MonteCarloLocalizationPose/MonteCarloLocalizationPose.h>

#include <iterator>
#include <type_traits>

#include <cereal/cereal.hpp>
#include <cereal/archives/json.hpp>
#include <cereal/archives/binary.hpp>

#include <cereal/types/map.hpp>
#include <cereal/types/vector.hpp>
#include <cereal/types/array.hpp>
#include <cereal/types/string.hpp>
#include <cereal/types/complex.hpp>

#include <ODatum/IntrospectionArchive.h>
#include <ODatum/Hash.h>





 namespace av {
 namespace msg {


MonteCarloLocalizationPose::MonteCarloLocalizationPose() = default;

MonteCarloLocalizationPose::MonteCarloLocalizationPose(
    TimePoint timeStamp_arg ,
    av::geometry::RecPose3D pose_arg ,
    av::geometry::RecPose3D poseSTD_arg ,
    av::geometry::RecDifferentialPose3D velocity_arg ,
    double confidence_arg ,
    bool health_arg ,
    int epsg_arg 
    ):
    timeStamp { std::move(timeStamp_arg) } // NOLINT
      ,pose { std::move(pose_arg) } // NOLINT
      ,poseSTD { std::move(poseSTD_arg) } // NOLINT
      ,velocity { std::move(velocity_arg) } // NOLINT
      ,confidence { std::move(confidence_arg) } // NOLINT
      ,health { std::move(health_arg) } // NOLINT
      ,epsg { std::move(epsg_arg) } // NOLINT
      {
}

MonteCarloLocalizationPose::MonteCarloLocalizationPose(const MonteCarloLocalizationPose&) = default;

// NOLINTNEXTLINE
MonteCarloLocalizationPose::MonteCarloLocalizationPose(MonteCarloLocalizationPose&&) = default;

MonteCarloLocalizationPose& MonteCarloLocalizationPose::operator=(const MonteCarloLocalizationPose&) = default;

// NOLINTNEXTLINE
MonteCarloLocalizationPose& MonteCarloLocalizationPose::operator=(MonteCarloLocalizationPose&&) = default;

void MonteCarloLocalizationPose::toCsv(std::ostream& out) const
{
    out.precision(16);
    out << std::boolalpha;
    out << timeStamp;out << ",";
    out << pose;out << ",";
    out << poseSTD;out << ",";
    out << velocity;out << ",";
    out << confidence;out << ",";
    out << health;out << ",";
    out << epsg;
    
}

std::stringstream MonteCarloLocalizationPose::toStringstream() const
{
    std::stringstream out;
    toCsv(out);
    return out;
}

template<class Archive>
    void MonteCarloLocalizationPose::serialize(Archive& ar, unsigned int version)
{    
    if (version < 5)
    {
        otimePoint otp;
        ar& cereal::make_nvp("timeStamp", otp);
        timeStamp = av::TimePoint(std::chrono::seconds(otp.tv_sec) + std::chrono::nanoseconds(otp.tv_nsec));
    }
    else
    {
        ar& CEREAL_NVP(timeStamp);
    }

    if (version >= 1) {
        ar& CEREAL_NVP(pose);
        ar& CEREAL_NVP(poseSTD);
        ar& CEREAL_NVP(velocity);
    }

    if (version >= 2) {
        ar& CEREAL_NVP(confidence);
    }

    if (version >= 3) {
        ar& CEREAL_NVP(health);
    }

    if (version >= 4) {
        ar& CEREAL_NVP(epsg);
    }
}

}} // to close av::msg namespace temporary

struct MonteCarloLocalizationPoseMsg : public av::msg::MonteCarloLocalizationPose {};
CEREAL_CLASS_VERSION(MonteCarloLocalizationPoseMsg, 5)
CEREAL_REGISTER_TYPE(MonteCarloLocalizationPoseMsg)

namespace av {
namespace msg {



template void MonteCarloLocalizationPose::serialize(cereal::BinaryInputArchive& ar, unsigned int version);
template void MonteCarloLocalizationPose::serialize(cereal::BinaryOutputArchive& ar, unsigned int version);
template void MonteCarloLocalizationPose::serialize(cereal::IntrospectionArchive& ar, unsigned int version);
template void MonteCarloLocalizationPose::serialize(cereal::JSONInputArchive& ar, unsigned int version);
template void MonteCarloLocalizationPose::serialize(cereal::JSONOutputArchive& ar, unsigned int version);
template void MonteCarloLocalizationPose::serialize(cereal::PortableBinaryInputArchive& ar, unsigned int version);
template void MonteCarloLocalizationPose::serialize(cereal::PortableBinaryOutputArchive& ar, unsigned int version);

std::ostream& operator<< (std::ostream& os, const MonteCarloLocalizationPose& montecarlolocalizationpose)
{
    montecarlolocalizationpose.toCsv(os);
    return os;
}

bool operator== (const MonteCarloLocalizationPose &lhs, const MonteCarloLocalizationPose &rhs)
{
    return (  lhs.timeStamp== rhs.timeStamp &&
     lhs.pose== rhs.pose &&
     lhs.poseSTD== rhs.poseSTD &&
     lhs.velocity== rhs.velocity &&
     lhs.confidence== rhs.confidence &&
     lhs.health== rhs.health &&
     lhs.epsg== rhs.epsg 
    );
}

std::size_t MonteCarloLocalizationPose::hash() const noexcept
{
    std::size_t result = 17;
    result = result * 31 + 0;
    result = result * 31 + std::hash<av::geometry::RecPose3D>()(pose);
    result = result * 31 + std::hash<av::geometry::RecPose3D>()(poseSTD);
    result = result * 31 + std::hash<av::geometry::RecDifferentialPose3D>()(velocity);
    result = result * 31 + std::hash<double>()(confidence);
    result = result * 31 + std::hash<bool>()(health);
    result = result * 31 + std::hash<int>()(epsg);
    return result;
}

 } // namespace av
 } // namespace msg


CEREAL_BIND_TO_ARCHIVES(av::msg::MonteCarloLocalizationPose)

