#include <interfaces/MotionPlannerMsgs/MotionPlannerMsgs.h>

#include <iterator>
#include <type_traits>

#include <cereal/cereal.hpp>
#include <cereal/archives/json.hpp>
#include <cereal/archives/binary.hpp>

#include <cereal/types/map.hpp>
#include <cereal/types/vector.hpp>
#include <cereal/types/array.hpp>
#include <cereal/types/string.hpp>
#include <cereal/types/complex.hpp>

#include <ODatum/IntrospectionArchive.h>
#include <ODatum/Hash.h>


 namespace av {
 namespace msg {


SE2Vertex::SE2Vertex() = default;

SE2Vertex::SE2Vertex(
    int32_t id_arg ,
    msg::SE2 state_arg 
    ):
    id { std::move(id_arg) } // NOLINT
      ,
    state { std::move(state_arg) } // NOLINT
      
    {
}

SE2Vertex::SE2Vertex(const SE2Vertex&) = default;

// NOLINTNEXTLINE
SE2Vertex::SE2Vertex(SE2Vertex&&) = default;

SE2Vertex& SE2Vertex::operator=(const SE2Vertex&) = default;

// NOLINTNEXTLINE
SE2Vertex& SE2Vertex::operator=(SE2Vertex&&) = default;

void SE2Vertex::toCsv(std::ostream& out) const
{
    out.precision(16);
    out << std::boolalpha;
    out << id;out << ",";
    out << state;
    
}

std::stringstream SE2Vertex::toStringstream() const
{
    std::stringstream out;
    toCsv(out);
    return out;
}

template<class Archive>
    void SE2Vertex::serialize(Archive& ar, [[maybe_unused]] unsigned int version)
    {

        ar& CEREAL_NVP(id);
        ar& CEREAL_NVP(state);
    }


template void SE2Vertex::serialize(cereal::BinaryInputArchive& ar, unsigned int version);
template void SE2Vertex::serialize(cereal::BinaryOutputArchive& ar, unsigned int version);
template void SE2Vertex::serialize(cereal::IntrospectionArchive& ar, unsigned int version);
template void SE2Vertex::serialize(cereal::JSONInputArchive& ar, unsigned int version);
template void SE2Vertex::serialize(cereal::JSONOutputArchive& ar, unsigned int version);
template void SE2Vertex::serialize(cereal::PortableBinaryInputArchive& ar, unsigned int version);
template void SE2Vertex::serialize(cereal::PortableBinaryOutputArchive& ar, unsigned int version);

std::ostream& operator<< (std::ostream& os, const SE2Vertex& se2vertex)
{
    se2vertex.toCsv(os);
    return os;
}

bool operator== (const SE2Vertex &lhs, const SE2Vertex &rhs)
{
    return (  lhs.id== rhs.id &&
     lhs.state== rhs.state 
    );
}

std::size_t SE2Vertex::hash() const noexcept
{
    std::size_t result = 17;
    result = result * 31 + std::hash<int32_t>()(id);
    result = result * 31 + std::hash<msg::SE2>()(state);
    return result;
}

 } // namespace av
 } // namespace msg


CEREAL_BIND_TO_ARCHIVES(av::msg::SE2Vertex)

 namespace av {
 namespace msg {


DubinsEdge::DubinsEdge() = default;

DubinsEdge::DubinsEdge(
    int32_t parentId_arg ,
    int32_t childId_arg ,
    DubinsTransition transition_arg 
    ):
    parentId { std::move(parentId_arg) } // NOLINT
      ,
    childId { std::move(childId_arg) } // NOLINT
      ,
    transition { std::move(transition_arg) } // NOLINT
      
    {
}

DubinsEdge::DubinsEdge(const DubinsEdge&) = default;

// NOLINTNEXTLINE
DubinsEdge::DubinsEdge(DubinsEdge&&) = default;

DubinsEdge& DubinsEdge::operator=(const DubinsEdge&) = default;

// NOLINTNEXTLINE
DubinsEdge& DubinsEdge::operator=(DubinsEdge&&) = default;

void DubinsEdge::toCsv(std::ostream& out) const
{
    out.precision(16);
    out << std::boolalpha;
    out << parentId;out << ",";
    out << childId;out << ",";
    out << transition;
    
}

std::stringstream DubinsEdge::toStringstream() const
{
    std::stringstream out;
    toCsv(out);
    return out;
}

template<class Archive>
    void DubinsEdge::serialize(Archive& ar, [[maybe_unused]] unsigned int version)
    {

        ar& CEREAL_NVP(parentId);
        ar& CEREAL_NVP(childId);
        ar& CEREAL_NVP(transition);
    }


template void DubinsEdge::serialize(cereal::BinaryInputArchive& ar, unsigned int version);
template void DubinsEdge::serialize(cereal::BinaryOutputArchive& ar, unsigned int version);
template void DubinsEdge::serialize(cereal::IntrospectionArchive& ar, unsigned int version);
template void DubinsEdge::serialize(cereal::JSONInputArchive& ar, unsigned int version);
template void DubinsEdge::serialize(cereal::JSONOutputArchive& ar, unsigned int version);
template void DubinsEdge::serialize(cereal::PortableBinaryInputArchive& ar, unsigned int version);
template void DubinsEdge::serialize(cereal::PortableBinaryOutputArchive& ar, unsigned int version);

std::ostream& operator<< (std::ostream& os, const DubinsEdge& dubinsedge)
{
    dubinsedge.toCsv(os);
    return os;
}

bool operator== (const DubinsEdge &lhs, const DubinsEdge &rhs)
{
    return (  lhs.parentId== rhs.parentId &&
     lhs.childId== rhs.childId &&
     lhs.transition== rhs.transition 
    );
}

std::size_t DubinsEdge::hash() const noexcept
{
    std::size_t result = 17;
    result = result * 31 + std::hash<int32_t>()(parentId);
    result = result * 31 + std::hash<int32_t>()(childId);
    result = result * 31 + std::hash<DubinsTransition>()(transition);
    return result;
}

 } // namespace av
 } // namespace msg


CEREAL_BIND_TO_ARCHIVES(av::msg::DubinsEdge)

 namespace av {
 namespace msg {


EdgeInfo::EdgeInfo() = default;

EdgeInfo::EdgeInfo(
    bool inCollision_arg ,
    bool isBaseline_arg ,
    bool isCommitted_arg 
    ):
    inCollision { std::move(inCollision_arg) } // NOLINT
      ,
    isBaseline { std::move(isBaseline_arg) } // NOLINT
      ,
    isCommitted { std::move(isCommitted_arg) } // NOLINT
      
    {
}

EdgeInfo::EdgeInfo(const EdgeInfo&) = default;

// NOLINTNEXTLINE
EdgeInfo::EdgeInfo(EdgeInfo&&) = default;

EdgeInfo& EdgeInfo::operator=(const EdgeInfo&) = default;

// NOLINTNEXTLINE
EdgeInfo& EdgeInfo::operator=(EdgeInfo&&) = default;

void EdgeInfo::toCsv(std::ostream& out) const
{
    out.precision(16);
    out << std::boolalpha;
    out << inCollision;out << ",";
    out << isBaseline;out << ",";
    out << isCommitted;
    
}

std::stringstream EdgeInfo::toStringstream() const
{
    std::stringstream out;
    toCsv(out);
    return out;
}

template<class Archive>
    void EdgeInfo::serialize(Archive& ar, [[maybe_unused]] unsigned int version)
    {

        ar& CEREAL_NVP(inCollision);
        ar& CEREAL_NVP(isBaseline);
        ar& CEREAL_NVP(isCommitted);
    }


template void EdgeInfo::serialize(cereal::BinaryInputArchive& ar, unsigned int version);
template void EdgeInfo::serialize(cereal::BinaryOutputArchive& ar, unsigned int version);
template void EdgeInfo::serialize(cereal::IntrospectionArchive& ar, unsigned int version);
template void EdgeInfo::serialize(cereal::JSONInputArchive& ar, unsigned int version);
template void EdgeInfo::serialize(cereal::JSONOutputArchive& ar, unsigned int version);
template void EdgeInfo::serialize(cereal::PortableBinaryInputArchive& ar, unsigned int version);
template void EdgeInfo::serialize(cereal::PortableBinaryOutputArchive& ar, unsigned int version);

std::ostream& operator<< (std::ostream& os, const EdgeInfo& edgeinfo)
{
    edgeinfo.toCsv(os);
    return os;
}

bool operator== (const EdgeInfo &lhs, const EdgeInfo &rhs)
{
    return (  lhs.inCollision== rhs.inCollision &&
     lhs.isBaseline== rhs.isBaseline &&
     lhs.isCommitted== rhs.isCommitted 
    );
}

std::size_t EdgeInfo::hash() const noexcept
{
    std::size_t result = 17;
    result = result * 31 + std::hash<bool>()(inCollision);
    result = result * 31 + std::hash<bool>()(isBaseline);
    result = result * 31 + std::hash<bool>()(isCommitted);
    return result;
}

 } // namespace av
 } // namespace msg


CEREAL_BIND_TO_ARCHIVES(av::msg::EdgeInfo)

 namespace av {
 namespace msg {


VertexInfo::VertexInfo() = default;

VertexInfo::VertexInfo(
    bool placeholder_arg 
    ):
    placeholder { std::move(placeholder_arg) } // NOLINT
      
    {
}

VertexInfo::VertexInfo(const VertexInfo&) = default;

// NOLINTNEXTLINE
VertexInfo::VertexInfo(VertexInfo&&) = default;

VertexInfo& VertexInfo::operator=(const VertexInfo&) = default;

// NOLINTNEXTLINE
VertexInfo& VertexInfo::operator=(VertexInfo&&) = default;

void VertexInfo::toCsv(std::ostream& out) const
{
    out.precision(16);
    out << std::boolalpha;
    out << placeholder;
    
}

std::stringstream VertexInfo::toStringstream() const
{
    std::stringstream out;
    toCsv(out);
    return out;
}

template<class Archive>
    void VertexInfo::serialize(Archive& ar, unsigned int version)
{
    if(version >= 2)
    {
        ar& CEREAL_NVP(placeholder);
    }
}


template void VertexInfo::serialize(cereal::BinaryInputArchive& ar, unsigned int version);
template void VertexInfo::serialize(cereal::BinaryOutputArchive& ar, unsigned int version);
template void VertexInfo::serialize(cereal::IntrospectionArchive& ar, unsigned int version);
template void VertexInfo::serialize(cereal::JSONInputArchive& ar, unsigned int version);
template void VertexInfo::serialize(cereal::JSONOutputArchive& ar, unsigned int version);
template void VertexInfo::serialize(cereal::PortableBinaryInputArchive& ar, unsigned int version);
template void VertexInfo::serialize(cereal::PortableBinaryOutputArchive& ar, unsigned int version);

std::ostream& operator<< (std::ostream& os, const VertexInfo& vertexinfo)
{
    vertexinfo.toCsv(os);
    return os;
}

bool operator== (const VertexInfo &lhs, const VertexInfo &rhs)
{
    return (  lhs.placeholder== rhs.placeholder 
    );
}

std::size_t VertexInfo::hash() const noexcept
{
    std::size_t result = 17;
    result = result * 31 + std::hash<bool>()(placeholder);
    return result;
}

 } // namespace av
 } // namespace msg


CEREAL_BIND_TO_ARCHIVES(av::msg::VertexInfo)

 namespace av {
 namespace msg {


PathInfo::PathInfo() = default;

PathInfo::PathInfo(
    std::vector<DubinsEdge> edges_arg ,
    std::vector<EdgeInfo> edgesInfo_arg ,
    std::vector<SE2Vertex> vertices_arg ,
    std::vector<VertexInfo> verticesInfo_arg 
    ):
    edges { std::move(edges_arg) } // NOLINT
      ,
    edgesInfo { std::move(edgesInfo_arg) } // NOLINT
      ,
    vertices { std::move(vertices_arg) } // NOLINT
      ,
    verticesInfo { std::move(verticesInfo_arg) } // NOLINT
      
    {
}

PathInfo::PathInfo(const PathInfo&) = default;

// NOLINTNEXTLINE
PathInfo::PathInfo(PathInfo&&) = default;

PathInfo& PathInfo::operator=(const PathInfo&) = default;

// NOLINTNEXTLINE
PathInfo& PathInfo::operator=(PathInfo&&) = default;

void PathInfo::toCsv(std::ostream& out) const
{
    out.precision(16);
    out << std::boolalpha;
    // Enclose sequence in double quotations and curly brackets to keep CSV line length uniform
        
        out << "\"{{";
        
        if (!edges.empty())
        {
            std::copy(edges.begin(), edges.end() - 1, std::ostream_iterator<DubinsEdge>(out, "},{"));
            std::copy(edges.end()-1, edges.end(), std::ostream_iterator<DubinsEdge>(out, ""));
        }
        out << "}}\"";out << ",";
    // Enclose sequence in double quotations and curly brackets to keep CSV line length uniform
        
        out << "\"{{";
        
        if (!edgesInfo.empty())
        {
            std::copy(edgesInfo.begin(), edgesInfo.end() - 1, std::ostream_iterator<EdgeInfo>(out, "},{"));
            std::copy(edgesInfo.end()-1, edgesInfo.end(), std::ostream_iterator<EdgeInfo>(out, ""));
        }
        out << "}}\"";out << ",";
    // Enclose sequence in double quotations and curly brackets to keep CSV line length uniform
        
        out << "\"{{";
        
        if (!vertices.empty())
        {
            std::copy(vertices.begin(), vertices.end() - 1, std::ostream_iterator<SE2Vertex>(out, "},{"));
            std::copy(vertices.end()-1, vertices.end(), std::ostream_iterator<SE2Vertex>(out, ""));
        }
        out << "}}\"";out << ",";
    // Enclose sequence in double quotations and curly brackets to keep CSV line length uniform
        
        out << "\"{{";
        
        if (!verticesInfo.empty())
        {
            std::copy(verticesInfo.begin(), verticesInfo.end() - 1, std::ostream_iterator<VertexInfo>(out, "},{"));
            std::copy(verticesInfo.end()-1, verticesInfo.end(), std::ostream_iterator<VertexInfo>(out, ""));
        }
        out << "}}\"";
    
}

std::stringstream PathInfo::toStringstream() const
{
    std::stringstream out;
    toCsv(out);
    return out;
}

template<class Archive>
    void PathInfo::serialize(Archive& ar, unsigned int /*version*/)
{
    ar& CEREAL_NVP(edges);
    ar& CEREAL_NVP(edgesInfo);
    ar& CEREAL_NVP(vertices);
    ar& CEREAL_NVP(verticesInfo);
}


template void PathInfo::serialize(cereal::BinaryInputArchive& ar, unsigned int version);
template void PathInfo::serialize(cereal::BinaryOutputArchive& ar, unsigned int version);
template void PathInfo::serialize(cereal::IntrospectionArchive& ar, unsigned int version);
template void PathInfo::serialize(cereal::JSONInputArchive& ar, unsigned int version);
template void PathInfo::serialize(cereal::JSONOutputArchive& ar, unsigned int version);
template void PathInfo::serialize(cereal::PortableBinaryInputArchive& ar, unsigned int version);
template void PathInfo::serialize(cereal::PortableBinaryOutputArchive& ar, unsigned int version);

std::ostream& operator<< (std::ostream& os, const PathInfo& pathinfo)
{
    pathinfo.toCsv(os);
    return os;
}

bool operator== (const PathInfo &lhs, const PathInfo &rhs)
{
    return (  lhs.edges== rhs.edges &&
     lhs.edgesInfo== rhs.edgesInfo &&
     lhs.vertices== rhs.vertices &&
     lhs.verticesInfo== rhs.verticesInfo 
    );
}

std::size_t PathInfo::hash() const noexcept
{
    std::size_t result = 17;
    result = result * 31 + hashVector(edges);
    result = result * 31 + hashVector(edgesInfo);
    result = result * 31 + hashVector(vertices);
    result = result * 31 + hashVector(verticesInfo);
    return result;
}

 } // namespace av
 } // namespace msg


CEREAL_BIND_TO_ARCHIVES(av::msg::PathInfo)

 namespace av {
 namespace msg {


GraphInfo::GraphInfo() = default;

GraphInfo::GraphInfo(
    int32_t rootId_arg ,
    int32_t goalId_arg ,
    std::vector<SE2Vertex> vertices_arg ,
    std::vector<DubinsEdge> edges_arg ,
    std::vector<EdgeInfo> edgesInfo_arg ,
    std::vector<VertexInfo> verticesInfo_arg 
    ):
    rootId { std::move(rootId_arg) } // NOLINT
      ,
    goalId { std::move(goalId_arg) } // NOLINT
      ,
    vertices { std::move(vertices_arg) } // NOLINT
      ,
    edges { std::move(edges_arg) } // NOLINT
      ,
    edgesInfo { std::move(edgesInfo_arg) } // NOLINT
      ,
    verticesInfo { std::move(verticesInfo_arg) } // NOLINT
      
    {
}

GraphInfo::GraphInfo(const GraphInfo&) = default;

// NOLINTNEXTLINE
GraphInfo::GraphInfo(GraphInfo&&) = default;

GraphInfo& GraphInfo::operator=(const GraphInfo&) = default;

// NOLINTNEXTLINE
GraphInfo& GraphInfo::operator=(GraphInfo&&) = default;

void GraphInfo::toCsv(std::ostream& out) const
{
    out.precision(16);
    out << std::boolalpha;
    out << rootId;out << ",";
    out << goalId;out << ",";
    // Enclose sequence in double quotations and curly brackets to keep CSV line length uniform
        
        out << "\"{{";
        
        if (!vertices.empty())
        {
            std::copy(vertices.begin(), vertices.end() - 1, std::ostream_iterator<SE2Vertex>(out, "},{"));
            std::copy(vertices.end()-1, vertices.end(), std::ostream_iterator<SE2Vertex>(out, ""));
        }
        out << "}}\"";out << ",";
    // Enclose sequence in double quotations and curly brackets to keep CSV line length uniform
        
        out << "\"{{";
        
        if (!edges.empty())
        {
            std::copy(edges.begin(), edges.end() - 1, std::ostream_iterator<DubinsEdge>(out, "},{"));
            std::copy(edges.end()-1, edges.end(), std::ostream_iterator<DubinsEdge>(out, ""));
        }
        out << "}}\"";out << ",";
    // Enclose sequence in double quotations and curly brackets to keep CSV line length uniform
        
        out << "\"{{";
        
        if (!edgesInfo.empty())
        {
            std::copy(edgesInfo.begin(), edgesInfo.end() - 1, std::ostream_iterator<EdgeInfo>(out, "},{"));
            std::copy(edgesInfo.end()-1, edgesInfo.end(), std::ostream_iterator<EdgeInfo>(out, ""));
        }
        out << "}}\"";out << ",";
    // Enclose sequence in double quotations and curly brackets to keep CSV line length uniform
        
        out << "\"{{";
        
        if (!verticesInfo.empty())
        {
            std::copy(verticesInfo.begin(), verticesInfo.end() - 1, std::ostream_iterator<VertexInfo>(out, "},{"));
            std::copy(verticesInfo.end()-1, verticesInfo.end(), std::ostream_iterator<VertexInfo>(out, ""));
        }
        out << "}}\"";
    
}

std::stringstream GraphInfo::toStringstream() const
{
    std::stringstream out;
    toCsv(out);
    return out;
}

template<class Archive>
    void GraphInfo::serialize(Archive& ar, unsigned int /*version*/)
{
    ar& CEREAL_NVP(rootId);
    ar& CEREAL_NVP(goalId);
    ar& CEREAL_NVP(vertices);
    ar& CEREAL_NVP(edges);
    ar& CEREAL_NVP(edgesInfo);
    ar& CEREAL_NVP(verticesInfo);
}


template void GraphInfo::serialize(cereal::BinaryInputArchive& ar, unsigned int version);
template void GraphInfo::serialize(cereal::BinaryOutputArchive& ar, unsigned int version);
template void GraphInfo::serialize(cereal::IntrospectionArchive& ar, unsigned int version);
template void GraphInfo::serialize(cereal::JSONInputArchive& ar, unsigned int version);
template void GraphInfo::serialize(cereal::JSONOutputArchive& ar, unsigned int version);
template void GraphInfo::serialize(cereal::PortableBinaryInputArchive& ar, unsigned int version);
template void GraphInfo::serialize(cereal::PortableBinaryOutputArchive& ar, unsigned int version);

std::ostream& operator<< (std::ostream& os, const GraphInfo& graphinfo)
{
    graphinfo.toCsv(os);
    return os;
}

bool operator== (const GraphInfo &lhs, const GraphInfo &rhs)
{
    return (  lhs.rootId== rhs.rootId &&
     lhs.goalId== rhs.goalId &&
     lhs.vertices== rhs.vertices &&
     lhs.edges== rhs.edges &&
     lhs.edgesInfo== rhs.edgesInfo &&
     lhs.verticesInfo== rhs.verticesInfo 
    );
}

std::size_t GraphInfo::hash() const noexcept
{
    std::size_t result = 17;
    result = result * 31 + std::hash<int32_t>()(rootId);
    result = result * 31 + std::hash<int32_t>()(goalId);
    result = result * 31 + hashVector(vertices);
    result = result * 31 + hashVector(edges);
    result = result * 31 + hashVector(edgesInfo);
    result = result * 31 + hashVector(verticesInfo);
    return result;
}

 } // namespace av
 } // namespace msg


CEREAL_BIND_TO_ARCHIVES(av::msg::GraphInfo)

 namespace av {
 namespace msg {


MotionPlannerDebugMsgs::MotionPlannerDebugMsgs() = default;

MotionPlannerDebugMsgs::MotionPlannerDebugMsgs(
    PathInfo pathInfo_arg ,
    GraphInfo graphInfo_arg 
    ):
    pathInfo { std::move(pathInfo_arg) } // NOLINT
      ,
    graphInfo { std::move(graphInfo_arg) } // NOLINT
      
    {
}

MotionPlannerDebugMsgs::MotionPlannerDebugMsgs(const MotionPlannerDebugMsgs&) = default;

// NOLINTNEXTLINE
MotionPlannerDebugMsgs::MotionPlannerDebugMsgs(MotionPlannerDebugMsgs&&) = default;

MotionPlannerDebugMsgs& MotionPlannerDebugMsgs::operator=(const MotionPlannerDebugMsgs&) = default;

// NOLINTNEXTLINE
MotionPlannerDebugMsgs& MotionPlannerDebugMsgs::operator=(MotionPlannerDebugMsgs&&) = default;

void MotionPlannerDebugMsgs::toCsv(std::ostream& out) const
{
    out.precision(16);
    out << std::boolalpha;
    out << pathInfo;out << ",";
    out << graphInfo;
    
}

std::stringstream MotionPlannerDebugMsgs::toStringstream() const
{
    std::stringstream out;
    toCsv(out);
    return out;
}

template<class Archive>
    void MotionPlannerDebugMsgs::serialize(Archive& ar, unsigned int /*version*/)
{
    ar& CEREAL_NVP(pathInfo);
    ar& CEREAL_NVP(graphInfo);
}


template void MotionPlannerDebugMsgs::serialize(cereal::BinaryInputArchive& ar, unsigned int version);
template void MotionPlannerDebugMsgs::serialize(cereal::BinaryOutputArchive& ar, unsigned int version);
template void MotionPlannerDebugMsgs::serialize(cereal::IntrospectionArchive& ar, unsigned int version);
template void MotionPlannerDebugMsgs::serialize(cereal::JSONInputArchive& ar, unsigned int version);
template void MotionPlannerDebugMsgs::serialize(cereal::JSONOutputArchive& ar, unsigned int version);
template void MotionPlannerDebugMsgs::serialize(cereal::PortableBinaryInputArchive& ar, unsigned int version);
template void MotionPlannerDebugMsgs::serialize(cereal::PortableBinaryOutputArchive& ar, unsigned int version);

std::ostream& operator<< (std::ostream& os, const MotionPlannerDebugMsgs& motionplannerdebugmsgs)
{
    motionplannerdebugmsgs.toCsv(os);
    return os;
}

bool operator== (const MotionPlannerDebugMsgs &lhs, const MotionPlannerDebugMsgs &rhs)
{
    return (  lhs.pathInfo== rhs.pathInfo &&
     lhs.graphInfo== rhs.graphInfo 
    );
}

std::size_t MotionPlannerDebugMsgs::hash() const noexcept
{
    std::size_t result = 17;
    result = result * 31 + std::hash<PathInfo>()(pathInfo);
    result = result * 31 + std::hash<GraphInfo>()(graphInfo);
    return result;
}

 } // namespace av
 } // namespace msg


CEREAL_BIND_TO_ARCHIVES(av::msg::MotionPlannerDebugMsgs)




 namespace av {
 namespace msg {


MotionPlannerMsgs::MotionPlannerMsgs() = default;

MotionPlannerMsgs::MotionPlannerMsgs(
    plan_ctrl::PlannerHeader header_arg ,
    Path path_arg ,
    RouteProgress routeProgress_arg ,
    MotionPlannerDebugMsgs debugMsgs_arg 
    ):
    header { std::move(header_arg) } // NOLINT
      ,path { std::move(path_arg) } // NOLINT
      ,routeProgress { std::move(routeProgress_arg) } // NOLINT
      ,debugMsgs { std::move(debugMsgs_arg) } // NOLINT
      {
}

MotionPlannerMsgs::MotionPlannerMsgs(const MotionPlannerMsgs&) = default;

// NOLINTNEXTLINE
MotionPlannerMsgs::MotionPlannerMsgs(MotionPlannerMsgs&&) = default;

MotionPlannerMsgs& MotionPlannerMsgs::operator=(const MotionPlannerMsgs&) = default;

// NOLINTNEXTLINE
MotionPlannerMsgs& MotionPlannerMsgs::operator=(MotionPlannerMsgs&&) = default;

void MotionPlannerMsgs::toCsv(std::ostream& out) const
{
    out.precision(16);
    out << std::boolalpha;
    out << header;out << ",";
    out << path;out << ",";
    out << routeProgress;out << ",";
    out << debugMsgs;
    
}

std::stringstream MotionPlannerMsgs::toStringstream() const
{
    std::stringstream out;
    toCsv(out);
    return out;
}

template<class Archive>
    void MotionPlannerMsgs::serialize(Archive& ar, unsigned int /*version*/)
{
    ar& CEREAL_NVP(header);
    ar& CEREAL_NVP(path);
    ar& CEREAL_NVP(routeProgress);
    ar& CEREAL_NVP(debugMsgs);
}


template void MotionPlannerMsgs::serialize(cereal::BinaryInputArchive& ar, unsigned int version);
template void MotionPlannerMsgs::serialize(cereal::BinaryOutputArchive& ar, unsigned int version);
template void MotionPlannerMsgs::serialize(cereal::IntrospectionArchive& ar, unsigned int version);
template void MotionPlannerMsgs::serialize(cereal::JSONInputArchive& ar, unsigned int version);
template void MotionPlannerMsgs::serialize(cereal::JSONOutputArchive& ar, unsigned int version);
template void MotionPlannerMsgs::serialize(cereal::PortableBinaryInputArchive& ar, unsigned int version);
template void MotionPlannerMsgs::serialize(cereal::PortableBinaryOutputArchive& ar, unsigned int version);

std::ostream& operator<< (std::ostream& os, const MotionPlannerMsgs& motionplannermsgs)
{
    motionplannermsgs.toCsv(os);
    return os;
}

bool operator== (const MotionPlannerMsgs &lhs, const MotionPlannerMsgs &rhs)
{
    return (  lhs.header== rhs.header &&
     lhs.path== rhs.path &&
     lhs.routeProgress== rhs.routeProgress &&
     lhs.debugMsgs== rhs.debugMsgs 
    );
}

std::size_t MotionPlannerMsgs::hash() const noexcept
{
    std::size_t result = 17;
    result = result * 31 + std::hash<plan_ctrl::PlannerHeader>()(header);
    result = result * 31 + std::hash<Path>()(path);
    result = result * 31 + std::hash<RouteProgress>()(routeProgress);
    result = result * 31 + std::hash<MotionPlannerDebugMsgs>()(debugMsgs);
    return result;
}

 } // namespace av
 } // namespace msg


CEREAL_BIND_TO_ARCHIVES(av::msg::MotionPlannerMsgs)

