#include <interfaces/MultiIntersectionTLStatus/MultiIntersectionTLStatus.h>

#include <iterator>
#include <type_traits>

#include <cereal/cereal.hpp>
#include <cereal/archives/json.hpp>
#include <cereal/archives/binary.hpp>

#include <cereal/types/map.hpp>
#include <cereal/types/vector.hpp>
#include <cereal/types/array.hpp>
#include <cereal/types/string.hpp>
#include <cereal/types/complex.hpp>

#include <ODatum/IntrospectionArchive.h>
#include <ODatum/Hash.h>





 namespace av {
 namespace msg {


MultiIntersectionTLStatus::MultiIntersectionTLStatus() = default;

MultiIntersectionTLStatus::MultiIntersectionTLStatus(
    int64_t stamp_arg ,
    std::vector<IntersectionTLStatus> multi_intersection_status_arg 
    ):
    stamp { std::move(stamp_arg) } // NOLINT
      ,multi_intersection_status { std::move(multi_intersection_status_arg) } // NOLINT
      {
}

MultiIntersectionTLStatus::MultiIntersectionTLStatus(const MultiIntersectionTLStatus&) = default;

// NOLINTNEXTLINE
MultiIntersectionTLStatus::MultiIntersectionTLStatus(MultiIntersectionTLStatus&&) = default;

MultiIntersectionTLStatus& MultiIntersectionTLStatus::operator=(const MultiIntersectionTLStatus&) = default;

// NOLINTNEXTLINE
MultiIntersectionTLStatus& MultiIntersectionTLStatus::operator=(MultiIntersectionTLStatus&&) = default;

void MultiIntersectionTLStatus::toCsv(std::ostream& out) const
{
    out.precision(16);
    out << std::boolalpha;
    out << stamp;out << ",";
    // Enclose sequence in double quotations and curly brackets to keep CSV line length uniform
        
        out << "\"{{";
        
        if (!multi_intersection_status.empty())
        {
            std::copy(multi_intersection_status.begin(), multi_intersection_status.end() - 1, std::ostream_iterator<IntersectionTLStatus>(out, "},{"));
            std::copy(multi_intersection_status.end()-1, multi_intersection_status.end(), std::ostream_iterator<IntersectionTLStatus>(out, ""));
        }
        out << "}}\"";
    
}

std::stringstream MultiIntersectionTLStatus::toStringstream() const
{
    std::stringstream out;
    toCsv(out);
    return out;
}

template<class Archive>
    void MultiIntersectionTLStatus::serialize(Archive& ar, [[maybe_unused]] unsigned int version)
    {

        ar& CEREAL_NVP(stamp);
        ar& CEREAL_NVP(multi_intersection_status);
    }


template void MultiIntersectionTLStatus::serialize(cereal::BinaryInputArchive& ar, unsigned int version);
template void MultiIntersectionTLStatus::serialize(cereal::BinaryOutputArchive& ar, unsigned int version);
template void MultiIntersectionTLStatus::serialize(cereal::IntrospectionArchive& ar, unsigned int version);
template void MultiIntersectionTLStatus::serialize(cereal::JSONInputArchive& ar, unsigned int version);
template void MultiIntersectionTLStatus::serialize(cereal::JSONOutputArchive& ar, unsigned int version);
template void MultiIntersectionTLStatus::serialize(cereal::PortableBinaryInputArchive& ar, unsigned int version);
template void MultiIntersectionTLStatus::serialize(cereal::PortableBinaryOutputArchive& ar, unsigned int version);

std::ostream& operator<< (std::ostream& os, const MultiIntersectionTLStatus& multiintersectiontlstatus)
{
    multiintersectiontlstatus.toCsv(os);
    return os;
}

bool operator== (const MultiIntersectionTLStatus &lhs, const MultiIntersectionTLStatus &rhs)
{
    return (  lhs.stamp== rhs.stamp &&
     lhs.multi_intersection_status== rhs.multi_intersection_status 
    );
}

std::size_t MultiIntersectionTLStatus::hash() const noexcept
{
    std::size_t result = 17;
    result = result * 31 + std::hash<int64_t>()(stamp);
    result = result * 31 + hashVector(multi_intersection_status);
    return result;
}

 } // namespace av
 } // namespace msg


CEREAL_BIND_TO_ARCHIVES(av::msg::MultiIntersectionTLStatus)

