#include <interfaces/MultiTypeSpeedConstraints/MultiTypeSpeedConstraints.h>

#include <iterator>
#include <type_traits>

#include <cereal/cereal.hpp>
#include <cereal/archives/json.hpp>
#include <cereal/archives/binary.hpp>

#include <cereal/types/map.hpp>
#include <cereal/types/vector.hpp>
#include <cereal/types/array.hpp>
#include <cereal/types/string.hpp>
#include <cereal/types/complex.hpp>

#include <ODatum/IntrospectionArchive.h>
#include <ODatum/Hash.h>





 namespace av {
 namespace msg {


MultiTypeSpeedConstraints::MultiTypeSpeedConstraints() = default;

MultiTypeSpeedConstraints::MultiTypeSpeedConstraints(
    int64_t utime_ms_arg ,
    std::vector<double> stations_m_arg ,
    std::vector<double> fusedSpeedConstraints_mps_arg ,
    std::vector<double> speedLimitConstraint_mps_arg ,
    std::vector<double> turningConstraint_mps_arg ,
    std::vector<double> stopsignConstraint_mps_arg ,
    std::vector<double> proximityConstraint_mps_arg ,
    std::vector<double> crosswalkConstraint_mps_arg 
    ):
    utime_ms { std::move(utime_ms_arg) } // NOLINT
      ,stations_m { std::move(stations_m_arg) } // NOLINT
      ,fusedSpeedConstraints_mps { std::move(fusedSpeedConstraints_mps_arg) } // NOLINT
      ,speedLimitConstraint_mps { std::move(speedLimitConstraint_mps_arg) } // NOLINT
      ,turningConstraint_mps { std::move(turningConstraint_mps_arg) } // NOLINT
      ,stopsignConstraint_mps { std::move(stopsignConstraint_mps_arg) } // NOLINT
      ,proximityConstraint_mps { std::move(proximityConstraint_mps_arg) } // NOLINT
      ,crosswalkConstraint_mps { std::move(crosswalkConstraint_mps_arg) } // NOLINT
      {
}

MultiTypeSpeedConstraints::MultiTypeSpeedConstraints(const MultiTypeSpeedConstraints&) = default;

// NOLINTNEXTLINE
MultiTypeSpeedConstraints::MultiTypeSpeedConstraints(MultiTypeSpeedConstraints&&) = default;

MultiTypeSpeedConstraints& MultiTypeSpeedConstraints::operator=(const MultiTypeSpeedConstraints&) = default;

// NOLINTNEXTLINE
MultiTypeSpeedConstraints& MultiTypeSpeedConstraints::operator=(MultiTypeSpeedConstraints&&) = default;

void MultiTypeSpeedConstraints::toCsv(std::ostream& out) const
{
    out.precision(16);
    out << std::boolalpha;
    out << utime_ms;out << ",";
    // Enclose sequence in double quotations and curly brackets to keep CSV line length uniform
        
        out << "\"{{";
        
        if (!stations_m.empty())
        {
            std::copy(stations_m.begin(), stations_m.end() - 1, std::ostream_iterator<double>(out, "},{"));
            std::copy(stations_m.end()-1, stations_m.end(), std::ostream_iterator<double>(out, ""));
        }
        out << "}}\"";out << ",";
    // Enclose sequence in double quotations and curly brackets to keep CSV line length uniform
        
        out << "\"{{";
        
        if (!fusedSpeedConstraints_mps.empty())
        {
            std::copy(fusedSpeedConstraints_mps.begin(), fusedSpeedConstraints_mps.end() - 1, std::ostream_iterator<double>(out, "},{"));
            std::copy(fusedSpeedConstraints_mps.end()-1, fusedSpeedConstraints_mps.end(), std::ostream_iterator<double>(out, ""));
        }
        out << "}}\"";out << ",";
    // Enclose sequence in double quotations and curly brackets to keep CSV line length uniform
        
        out << "\"{{";
        
        if (!speedLimitConstraint_mps.empty())
        {
            std::copy(speedLimitConstraint_mps.begin(), speedLimitConstraint_mps.end() - 1, std::ostream_iterator<double>(out, "},{"));
            std::copy(speedLimitConstraint_mps.end()-1, speedLimitConstraint_mps.end(), std::ostream_iterator<double>(out, ""));
        }
        out << "}}\"";out << ",";
    // Enclose sequence in double quotations and curly brackets to keep CSV line length uniform
        
        out << "\"{{";
        
        if (!turningConstraint_mps.empty())
        {
            std::copy(turningConstraint_mps.begin(), turningConstraint_mps.end() - 1, std::ostream_iterator<double>(out, "},{"));
            std::copy(turningConstraint_mps.end()-1, turningConstraint_mps.end(), std::ostream_iterator<double>(out, ""));
        }
        out << "}}\"";out << ",";
    // Enclose sequence in double quotations and curly brackets to keep CSV line length uniform
        
        out << "\"{{";
        
        if (!stopsignConstraint_mps.empty())
        {
            std::copy(stopsignConstraint_mps.begin(), stopsignConstraint_mps.end() - 1, std::ostream_iterator<double>(out, "},{"));
            std::copy(stopsignConstraint_mps.end()-1, stopsignConstraint_mps.end(), std::ostream_iterator<double>(out, ""));
        }
        out << "}}\"";out << ",";
    // Enclose sequence in double quotations and curly brackets to keep CSV line length uniform
        
        out << "\"{{";
        
        if (!proximityConstraint_mps.empty())
        {
            std::copy(proximityConstraint_mps.begin(), proximityConstraint_mps.end() - 1, std::ostream_iterator<double>(out, "},{"));
            std::copy(proximityConstraint_mps.end()-1, proximityConstraint_mps.end(), std::ostream_iterator<double>(out, ""));
        }
        out << "}}\"";out << ",";
    // Enclose sequence in double quotations and curly brackets to keep CSV line length uniform
        
        out << "\"{{";
        
        if (!crosswalkConstraint_mps.empty())
        {
            std::copy(crosswalkConstraint_mps.begin(), crosswalkConstraint_mps.end() - 1, std::ostream_iterator<double>(out, "},{"));
            std::copy(crosswalkConstraint_mps.end()-1, crosswalkConstraint_mps.end(), std::ostream_iterator<double>(out, ""));
        }
        out << "}}\"";
    
}

std::stringstream MultiTypeSpeedConstraints::toStringstream() const
{
    std::stringstream out;
    toCsv(out);
    return out;
}

template<class Archive>
    void MultiTypeSpeedConstraints::serialize(Archive& ar, [[maybe_unused]] unsigned int version)
    {

        ar& CEREAL_NVP(utime_ms);
        ar& CEREAL_NVP(stations_m);
        ar& CEREAL_NVP(fusedSpeedConstraints_mps);
        ar& CEREAL_NVP(speedLimitConstraint_mps);
        ar& CEREAL_NVP(turningConstraint_mps);
        ar& CEREAL_NVP(stopsignConstraint_mps);
        ar& CEREAL_NVP(proximityConstraint_mps);
        ar& CEREAL_NVP(crosswalkConstraint_mps);
    }


template void MultiTypeSpeedConstraints::serialize(cereal::BinaryInputArchive& ar, unsigned int version);
template void MultiTypeSpeedConstraints::serialize(cereal::BinaryOutputArchive& ar, unsigned int version);
template void MultiTypeSpeedConstraints::serialize(cereal::IntrospectionArchive& ar, unsigned int version);
template void MultiTypeSpeedConstraints::serialize(cereal::JSONInputArchive& ar, unsigned int version);
template void MultiTypeSpeedConstraints::serialize(cereal::JSONOutputArchive& ar, unsigned int version);
template void MultiTypeSpeedConstraints::serialize(cereal::PortableBinaryInputArchive& ar, unsigned int version);
template void MultiTypeSpeedConstraints::serialize(cereal::PortableBinaryOutputArchive& ar, unsigned int version);

std::ostream& operator<< (std::ostream& os, const MultiTypeSpeedConstraints& multitypespeedconstraints)
{
    multitypespeedconstraints.toCsv(os);
    return os;
}

bool operator== (const MultiTypeSpeedConstraints &lhs, const MultiTypeSpeedConstraints &rhs)
{
    return (  lhs.utime_ms== rhs.utime_ms &&
     lhs.stations_m== rhs.stations_m &&
     lhs.fusedSpeedConstraints_mps== rhs.fusedSpeedConstraints_mps &&
     lhs.speedLimitConstraint_mps== rhs.speedLimitConstraint_mps &&
     lhs.turningConstraint_mps== rhs.turningConstraint_mps &&
     lhs.stopsignConstraint_mps== rhs.stopsignConstraint_mps &&
     lhs.proximityConstraint_mps== rhs.proximityConstraint_mps &&
     lhs.crosswalkConstraint_mps== rhs.crosswalkConstraint_mps 
    );
}

std::size_t MultiTypeSpeedConstraints::hash() const noexcept
{
    std::size_t result = 17;
    result = result * 31 + std::hash<int64_t>()(utime_ms);
    result = result * 31 + hashVector(stations_m);
    result = result * 31 + hashVector(fusedSpeedConstraints_mps);
    result = result * 31 + hashVector(speedLimitConstraint_mps);
    result = result * 31 + hashVector(turningConstraint_mps);
    result = result * 31 + hashVector(stopsignConstraint_mps);
    result = result * 31 + hashVector(proximityConstraint_mps);
    result = result * 31 + hashVector(crosswalkConstraint_mps);
    return result;
}

 } // namespace av
 } // namespace msg


CEREAL_BIND_TO_ARCHIVES(av::msg::MultiTypeSpeedConstraints)

