#include <interfaces/NetworkMetricsAlertMessage/NetworkMetricsAlertMessage.h>

#include <iterator>
#include <type_traits>

#include <cereal/cereal.hpp>
#include <cereal/archives/json.hpp>
#include <cereal/archives/binary.hpp>

#include <cereal/types/map.hpp>
#include <cereal/types/vector.hpp>
#include <cereal/types/array.hpp>
#include <cereal/types/string.hpp>
#include <cereal/types/complex.hpp>

#include <ODatum/IntrospectionArchive.h>
#include <ODatum/Hash.h>





 namespace av {
 namespace msg {


NetworkMetricsAlertMessage::NetworkMetricsAlertMessage() = default;

NetworkMetricsAlertMessage::NetworkMetricsAlertMessage(
    std::string taskName_arg ,
    std::string hostName_arg ,
    std::string channelName_arg ,
    TimePoint timeDetected_arg ,
    NetworkMetricsAlertMessageCategory category_arg ,
    int expectedFrequencyHz_arg ,
    int numCyclesMissed_arg 
    ):
    taskName { std::move(taskName_arg) } // NOLINT
      ,hostName { std::move(hostName_arg) } // NOLINT
      ,channelName { std::move(channelName_arg) } // NOLINT
      ,timeDetected { std::move(timeDetected_arg) } // NOLINT
      ,category { std::move(category_arg) } // NOLINT
      ,expectedFrequencyHz { std::move(expectedFrequencyHz_arg) } // NOLINT
      ,numCyclesMissed { std::move(numCyclesMissed_arg) } // NOLINT
      {
}

NetworkMetricsAlertMessage::NetworkMetricsAlertMessage(const NetworkMetricsAlertMessage&) = default;

// NOLINTNEXTLINE
NetworkMetricsAlertMessage::NetworkMetricsAlertMessage(NetworkMetricsAlertMessage&&) = default;

NetworkMetricsAlertMessage& NetworkMetricsAlertMessage::operator=(const NetworkMetricsAlertMessage&) = default;

// NOLINTNEXTLINE
NetworkMetricsAlertMessage& NetworkMetricsAlertMessage::operator=(NetworkMetricsAlertMessage&&) = default;

void NetworkMetricsAlertMessage::toCsv(std::ostream& out) const
{
    out.precision(16);
    out << std::boolalpha;
    out << "\"";
        out << taskName;
        out << "\"";out << ",";
    out << "\"";
        out << hostName;
        out << "\"";out << ",";
    out << "\"";
        out << channelName;
        out << "\"";out << ",";
    out << timeDetected;out << ",";
    out << static_cast<uint32_t>(category);out << ",";
    out << expectedFrequencyHz;out << ",";
    out << numCyclesMissed;
    
}

std::stringstream NetworkMetricsAlertMessage::toStringstream() const
{
    std::stringstream out;
    toCsv(out);
    return out;
}

template<class Archive>
    void NetworkMetricsAlertMessage::serialize(Archive& ar, [[maybe_unused]] unsigned int version)
    {

        ar& CEREAL_NVP(taskName);
        ar& CEREAL_NVP(hostName);
        ar& CEREAL_NVP(channelName);
        ar& CEREAL_NVP(timeDetected);
        ar& CEREAL_NVP(category);
        ar& CEREAL_NVP(expectedFrequencyHz);
        ar& CEREAL_NVP(numCyclesMissed);
    }


template void NetworkMetricsAlertMessage::serialize(cereal::BinaryInputArchive& ar, unsigned int version);
template void NetworkMetricsAlertMessage::serialize(cereal::BinaryOutputArchive& ar, unsigned int version);
template void NetworkMetricsAlertMessage::serialize(cereal::IntrospectionArchive& ar, unsigned int version);
template void NetworkMetricsAlertMessage::serialize(cereal::JSONInputArchive& ar, unsigned int version);
template void NetworkMetricsAlertMessage::serialize(cereal::JSONOutputArchive& ar, unsigned int version);
template void NetworkMetricsAlertMessage::serialize(cereal::PortableBinaryInputArchive& ar, unsigned int version);
template void NetworkMetricsAlertMessage::serialize(cereal::PortableBinaryOutputArchive& ar, unsigned int version);

std::ostream& operator<< (std::ostream& os, const NetworkMetricsAlertMessage& networkmetricsalertmessage)
{
    networkmetricsalertmessage.toCsv(os);
    return os;
}

bool operator== (const NetworkMetricsAlertMessage &lhs, const NetworkMetricsAlertMessage &rhs)
{
    return (  lhs.taskName== rhs.taskName &&
     lhs.hostName== rhs.hostName &&
     lhs.channelName== rhs.channelName &&
     lhs.timeDetected== rhs.timeDetected &&
     lhs.category== rhs.category &&
     lhs.expectedFrequencyHz== rhs.expectedFrequencyHz &&
     lhs.numCyclesMissed== rhs.numCyclesMissed 
    );
}

std::size_t NetworkMetricsAlertMessage::hash() const noexcept
{
    std::size_t result = 17;
    result = result * 31 + std::hash<std::string>()(taskName);
    result = result * 31 + std::hash<std::string>()(hostName);
    result = result * 31 + std::hash<std::string>()(channelName);
    result = result * 31 + 0;
    result = result * 31 + std::hash<NetworkMetricsAlertMessageCategory>()(category);
    result = result * 31 + std::hash<int>()(expectedFrequencyHz);
    result = result * 31 + std::hash<int>()(numCyclesMissed);
    return result;
}

 } // namespace av
 } // namespace msg


CEREAL_BIND_TO_ARCHIVES(av::msg::NetworkMetricsAlertMessage)

