#include <interfaces/NetworkMetricsMessage/NetworkMetricsMessage.h>

#include <iterator>
#include <type_traits>

#include <cereal/cereal.hpp>
#include <cereal/archives/json.hpp>
#include <cereal/archives/binary.hpp>

#include <cereal/types/map.hpp>
#include <cereal/types/vector.hpp>
#include <cereal/types/array.hpp>
#include <cereal/types/string.hpp>
#include <cereal/types/complex.hpp>

#include <ODatum/IntrospectionArchive.h>
#include <ODatum/Hash.h>


 namespace av {
 namespace msg {


NetworkMetricsStatsPerValue::NetworkMetricsStatsPerValue() = default;

NetworkMetricsStatsPerValue::NetworkMetricsStatsPerValue(
    bool valid_arg ,
    double min_arg ,
    double max_arg ,
    double mean_arg ,
    double variance_arg ,
    double jitter_arg 
    ):
    valid { std::move(valid_arg) } // NOLINT
      ,
    min { std::move(min_arg) } // NOLINT
      ,
    max { std::move(max_arg) } // NOLINT
      ,
    mean { std::move(mean_arg) } // NOLINT
      ,
    variance { std::move(variance_arg) } // NOLINT
      ,
    jitter { std::move(jitter_arg) } // NOLINT
      
    {
}

NetworkMetricsStatsPerValue::NetworkMetricsStatsPerValue(const NetworkMetricsStatsPerValue&) = default;

// NOLINTNEXTLINE
NetworkMetricsStatsPerValue::NetworkMetricsStatsPerValue(NetworkMetricsStatsPerValue&&) = default;

NetworkMetricsStatsPerValue& NetworkMetricsStatsPerValue::operator=(const NetworkMetricsStatsPerValue&) = default;

// NOLINTNEXTLINE
NetworkMetricsStatsPerValue& NetworkMetricsStatsPerValue::operator=(NetworkMetricsStatsPerValue&&) = default;

void NetworkMetricsStatsPerValue::toCsv(std::ostream& out) const
{
    out.precision(16);
    out << std::boolalpha;
    out << valid;out << ",";
    out << min;out << ",";
    out << max;out << ",";
    out << mean;out << ",";
    out << variance;out << ",";
    out << jitter;
    
}

std::stringstream NetworkMetricsStatsPerValue::toStringstream() const
{
    std::stringstream out;
    toCsv(out);
    return out;
}

template<class Archive>
    void NetworkMetricsStatsPerValue::serialize(Archive& ar, unsigned int version)
{
    if (version >= 1) {
        ar & CEREAL_NVP(valid);
        ar & CEREAL_NVP(min);
        ar & CEREAL_NVP(max);
        ar & CEREAL_NVP(mean);
        ar & CEREAL_NVP(variance);
    }

    if (version >= 2) {
        ar & CEREAL_NVP(jitter);
    }
}



template void NetworkMetricsStatsPerValue::serialize(cereal::BinaryInputArchive& ar, unsigned int version);
template void NetworkMetricsStatsPerValue::serialize(cereal::BinaryOutputArchive& ar, unsigned int version);
template void NetworkMetricsStatsPerValue::serialize(cereal::IntrospectionArchive& ar, unsigned int version);
template void NetworkMetricsStatsPerValue::serialize(cereal::JSONInputArchive& ar, unsigned int version);
template void NetworkMetricsStatsPerValue::serialize(cereal::JSONOutputArchive& ar, unsigned int version);
template void NetworkMetricsStatsPerValue::serialize(cereal::PortableBinaryInputArchive& ar, unsigned int version);
template void NetworkMetricsStatsPerValue::serialize(cereal::PortableBinaryOutputArchive& ar, unsigned int version);

std::ostream& operator<< (std::ostream& os, const NetworkMetricsStatsPerValue& networkmetricsstatspervalue)
{
    networkmetricsstatspervalue.toCsv(os);
    return os;
}

bool operator== (const NetworkMetricsStatsPerValue &lhs, const NetworkMetricsStatsPerValue &rhs)
{
    return (  lhs.valid== rhs.valid &&
     lhs.min== rhs.min &&
     lhs.max== rhs.max &&
     lhs.mean== rhs.mean &&
     lhs.variance== rhs.variance &&
     lhs.jitter== rhs.jitter 
    );
}

std::size_t NetworkMetricsStatsPerValue::hash() const noexcept
{
    std::size_t result = 17;
    result = result * 31 + std::hash<bool>()(valid);
    result = result * 31 + std::hash<double>()(min);
    result = result * 31 + std::hash<double>()(max);
    result = result * 31 + std::hash<double>()(mean);
    result = result * 31 + std::hash<double>()(variance);
    result = result * 31 + std::hash<double>()(jitter);
    return result;
}

 } // namespace av
 } // namespace msg


CEREAL_BIND_TO_ARCHIVES(av::msg::NetworkMetricsStatsPerValue)




 namespace av {
 namespace msg {


NetworkMetricsMessage::NetworkMetricsMessage() = default;

NetworkMetricsMessage::NetworkMetricsMessage(
    TimePoint windowStart_arg ,
    Duration windowLength_arg ,
    uint64_t count_arg ,
    NetworkMetricsStatsPerValue latency_arg ,
    NetworkMetricsStatsPerValue bytes_arg ,
    std::string taskName_arg ,
    std::string hostName_arg 
    ):
    windowStart { std::move(windowStart_arg) } // NOLINT
      ,windowLength { std::move(windowLength_arg) } // NOLINT
      ,count { std::move(count_arg) } // NOLINT
      ,latency { std::move(latency_arg) } // NOLINT
      ,bytes { std::move(bytes_arg) } // NOLINT
      ,taskName { std::move(taskName_arg) } // NOLINT
      ,hostName { std::move(hostName_arg) } // NOLINT
      {
}

NetworkMetricsMessage::NetworkMetricsMessage(const NetworkMetricsMessage&) = default;

// NOLINTNEXTLINE
NetworkMetricsMessage::NetworkMetricsMessage(NetworkMetricsMessage&&) = default;

NetworkMetricsMessage& NetworkMetricsMessage::operator=(const NetworkMetricsMessage&) = default;

// NOLINTNEXTLINE
NetworkMetricsMessage& NetworkMetricsMessage::operator=(NetworkMetricsMessage&&) = default;

void NetworkMetricsMessage::toCsv(std::ostream& out) const
{
    out.precision(16);
    out << std::boolalpha;
    out << windowStart;out << ",";
    out << windowLength;out << ",";
    out << count;out << ",";
    out << latency;out << ",";
    out << bytes;out << ",";
    out << "\"";
        out << taskName;
        out << "\"";out << ",";
    out << "\"";
        out << hostName;
        out << "\"";
    
}

std::stringstream NetworkMetricsMessage::toStringstream() const
{
    std::stringstream out;
    toCsv(out);
    return out;
}

template<class Archive>
    void NetworkMetricsMessage::serialize(Archive& ar, unsigned int version)
{
    if (version >= 1) {
        ar & CEREAL_NVP(windowStart);
        ar & CEREAL_NVP(windowLength);
        ar & CEREAL_NVP(count);
        ar & CEREAL_NVP(latency);
        ar & CEREAL_NVP(bytes);
    }

    if (version >= 2) {
        ar & CEREAL_NVP(taskName);
        ar & CEREAL_NVP(hostName);
    }
}



template void NetworkMetricsMessage::serialize(cereal::BinaryInputArchive& ar, unsigned int version);
template void NetworkMetricsMessage::serialize(cereal::BinaryOutputArchive& ar, unsigned int version);
template void NetworkMetricsMessage::serialize(cereal::IntrospectionArchive& ar, unsigned int version);
template void NetworkMetricsMessage::serialize(cereal::JSONInputArchive& ar, unsigned int version);
template void NetworkMetricsMessage::serialize(cereal::JSONOutputArchive& ar, unsigned int version);
template void NetworkMetricsMessage::serialize(cereal::PortableBinaryInputArchive& ar, unsigned int version);
template void NetworkMetricsMessage::serialize(cereal::PortableBinaryOutputArchive& ar, unsigned int version);

std::ostream& operator<< (std::ostream& os, const NetworkMetricsMessage& networkmetricsmessage)
{
    networkmetricsmessage.toCsv(os);
    return os;
}

bool operator== (const NetworkMetricsMessage &lhs, const NetworkMetricsMessage &rhs)
{
    return (  lhs.windowStart== rhs.windowStart &&
     lhs.windowLength== rhs.windowLength &&
     lhs.count== rhs.count &&
     lhs.latency== rhs.latency &&
     lhs.bytes== rhs.bytes &&
     lhs.taskName== rhs.taskName &&
     lhs.hostName== rhs.hostName 
    );
}

std::size_t NetworkMetricsMessage::hash() const noexcept
{
    std::size_t result = 17;
    result = result * 31 + 0;
    result = result * 31 + 0;
    result = result * 31 + std::hash<uint64_t>()(count);
    result = result * 31 + std::hash<NetworkMetricsStatsPerValue>()(latency);
    result = result * 31 + std::hash<NetworkMetricsStatsPerValue>()(bytes);
    result = result * 31 + std::hash<std::string>()(taskName);
    result = result * 31 + std::hash<std::string>()(hostName);
    return result;
}

 } // namespace av
 } // namespace msg


CEREAL_BIND_TO_ARCHIVES(av::msg::NetworkMetricsMessage)

