#include <interfaces/NetworkStatistics/NetworkStatistics.h>

#include <iterator>
#include <type_traits>

#include <cereal/cereal.hpp>
#include <cereal/archives/json.hpp>
#include <cereal/archives/binary.hpp>

#include <cereal/types/map.hpp>
#include <cereal/types/vector.hpp>
#include <cereal/types/array.hpp>
#include <cereal/types/string.hpp>
#include <cereal/types/complex.hpp>

#include <ODatum/IntrospectionArchive.h>
#include <ODatum/Hash.h>


 namespace av {
 namespace msg {


InterfaceStatPair::InterfaceStatPair() = default;

InterfaceStatPair::InterfaceStatPair(
    std::string key_arg ,
    int64_t value_arg 
    ):
    key { std::move(key_arg) } // NOLINT
      ,
    value { std::move(value_arg) } // NOLINT
      
    {
}

InterfaceStatPair::InterfaceStatPair(const InterfaceStatPair&) = default;

// NOLINTNEXTLINE
InterfaceStatPair::InterfaceStatPair(InterfaceStatPair&&) = default;

InterfaceStatPair& InterfaceStatPair::operator=(const InterfaceStatPair&) = default;

// NOLINTNEXTLINE
InterfaceStatPair& InterfaceStatPair::operator=(InterfaceStatPair&&) = default;

void InterfaceStatPair::toCsv(std::ostream& out) const
{
    out.precision(16);
    out << std::boolalpha;
    out << "\"";
        out << key;
        out << "\"";out << ",";
    out << value;
    
}

std::stringstream InterfaceStatPair::toStringstream() const
{
    std::stringstream out;
    toCsv(out);
    return out;
}

template<class Archive>
    void InterfaceStatPair::serialize(Archive& ar, [[maybe_unused]] unsigned int version)
    {

        ar& CEREAL_NVP(key);
        ar& CEREAL_NVP(value);
    }


template void InterfaceStatPair::serialize(cereal::BinaryInputArchive& ar, unsigned int version);
template void InterfaceStatPair::serialize(cereal::BinaryOutputArchive& ar, unsigned int version);
template void InterfaceStatPair::serialize(cereal::IntrospectionArchive& ar, unsigned int version);
template void InterfaceStatPair::serialize(cereal::JSONInputArchive& ar, unsigned int version);
template void InterfaceStatPair::serialize(cereal::JSONOutputArchive& ar, unsigned int version);
template void InterfaceStatPair::serialize(cereal::PortableBinaryInputArchive& ar, unsigned int version);
template void InterfaceStatPair::serialize(cereal::PortableBinaryOutputArchive& ar, unsigned int version);

std::ostream& operator<< (std::ostream& os, const InterfaceStatPair& interfacestatpair)
{
    interfacestatpair.toCsv(os);
    return os;
}

bool operator== (const InterfaceStatPair &lhs, const InterfaceStatPair &rhs)
{
    return (  lhs.key== rhs.key &&
     lhs.value== rhs.value 
    );
}

std::size_t InterfaceStatPair::hash() const noexcept
{
    std::size_t result = 17;
    result = result * 31 + std::hash<std::string>()(key);
    result = result * 31 + std::hash<int64_t>()(value);
    return result;
}

 } // namespace av
 } // namespace msg


CEREAL_BIND_TO_ARCHIVES(av::msg::InterfaceStatPair)

 namespace av {
 namespace msg {


InterfaceStats::InterfaceStats() = default;

InterfaceStats::InterfaceStats(
    std::string name_arg ,
    std::vector<InterfaceStatPair> values_arg 
    ):
    name { std::move(name_arg) } // NOLINT
      ,
    values { std::move(values_arg) } // NOLINT
      
    {
}

InterfaceStats::InterfaceStats(const InterfaceStats&) = default;

// NOLINTNEXTLINE
InterfaceStats::InterfaceStats(InterfaceStats&&) = default;

InterfaceStats& InterfaceStats::operator=(const InterfaceStats&) = default;

// NOLINTNEXTLINE
InterfaceStats& InterfaceStats::operator=(InterfaceStats&&) = default;

void InterfaceStats::toCsv(std::ostream& out) const
{
    out.precision(16);
    out << std::boolalpha;
    out << "\"";
        out << name;
        out << "\"";out << ",";
    // Enclose sequence in double quotations and curly brackets to keep CSV line length uniform
        
        out << "\"{{";
        
        if (!values.empty())
        {
            std::copy(values.begin(), values.end() - 1, std::ostream_iterator<InterfaceStatPair>(out, "},{"));
            std::copy(values.end()-1, values.end(), std::ostream_iterator<InterfaceStatPair>(out, ""));
        }
        out << "}}\"";
    
}

std::stringstream InterfaceStats::toStringstream() const
{
    std::stringstream out;
    toCsv(out);
    return out;
}

template<class Archive>
    void InterfaceStats::serialize(Archive& ar, unsigned int version)
{
    if (version == 1) {
        ar& CEREAL_NVP(name);

        // oldValues will be deserialized as std::map
        std::map<std::string, ssize_t> oldValues;
        ar& cereal::make_nvp("values", oldValues);

        // Convert to serializableMap
        values.reserve(oldValues.size());
        for (auto&& key_val : oldValues) {
            values.emplace_back(std::move(key_val.first), std::move(key_val.second));
        }
    }

    if (version >= 2) {
        ar& CEREAL_NVP(name);
        ar& CEREAL_NVP(values);
    }
}



template void InterfaceStats::serialize(cereal::BinaryInputArchive& ar, unsigned int version);
template void InterfaceStats::serialize(cereal::BinaryOutputArchive& ar, unsigned int version);
template void InterfaceStats::serialize(cereal::IntrospectionArchive& ar, unsigned int version);
template void InterfaceStats::serialize(cereal::JSONInputArchive& ar, unsigned int version);
template void InterfaceStats::serialize(cereal::JSONOutputArchive& ar, unsigned int version);
template void InterfaceStats::serialize(cereal::PortableBinaryInputArchive& ar, unsigned int version);
template void InterfaceStats::serialize(cereal::PortableBinaryOutputArchive& ar, unsigned int version);

std::ostream& operator<< (std::ostream& os, const InterfaceStats& interfacestats)
{
    interfacestats.toCsv(os);
    return os;
}

bool operator== (const InterfaceStats &lhs, const InterfaceStats &rhs)
{
    return (  lhs.name== rhs.name &&
     lhs.values== rhs.values 
    );
}

std::size_t InterfaceStats::hash() const noexcept
{
    std::size_t result = 17;
    result = result * 31 + std::hash<std::string>()(name);
    result = result * 31 + hashVector(values);
    return result;
}

 } // namespace av
 } // namespace msg


CEREAL_BIND_TO_ARCHIVES(av::msg::InterfaceStats)

 namespace av {
 namespace msg {


UDPStats::UDPStats() = default;

UDPStats::UDPStats(
    uint64_t received_arg ,
    uint64_t unknownPort_arg ,
    uint64_t inErrors_arg ,
    uint64_t sent_arg ,
    uint64_t receiveBufErrors_arg ,
    uint64_t sendBufErrors_arg ,
    uint64_t checksumErrors_arg ,
    uint64_t ignoredMulti_arg 
    ):
    received { std::move(received_arg) } // NOLINT
      ,
    unknownPort { std::move(unknownPort_arg) } // NOLINT
      ,
    inErrors { std::move(inErrors_arg) } // NOLINT
      ,
    sent { std::move(sent_arg) } // NOLINT
      ,
    receiveBufErrors { std::move(receiveBufErrors_arg) } // NOLINT
      ,
    sendBufErrors { std::move(sendBufErrors_arg) } // NOLINT
      ,
    checksumErrors { std::move(checksumErrors_arg) } // NOLINT
      ,
    ignoredMulti { std::move(ignoredMulti_arg) } // NOLINT
      
    {
}

UDPStats::UDPStats(const UDPStats&) = default;

// NOLINTNEXTLINE
UDPStats::UDPStats(UDPStats&&) = default;

UDPStats& UDPStats::operator=(const UDPStats&) = default;

// NOLINTNEXTLINE
UDPStats& UDPStats::operator=(UDPStats&&) = default;

void UDPStats::toCsv(std::ostream& out) const
{
    out.precision(16);
    out << std::boolalpha;
    out << received;out << ",";
    out << unknownPort;out << ",";
    out << inErrors;out << ",";
    out << sent;out << ",";
    out << receiveBufErrors;out << ",";
    out << sendBufErrors;out << ",";
    out << checksumErrors;out << ",";
    out << ignoredMulti;
    
}

std::stringstream UDPStats::toStringstream() const
{
    std::stringstream out;
    toCsv(out);
    return out;
}

template<class Archive>
    void UDPStats::serialize(Archive& ar, [[maybe_unused]] unsigned int version)
    {

        ar& CEREAL_NVP(received);
        ar& CEREAL_NVP(unknownPort);
        ar& CEREAL_NVP(inErrors);
        ar& CEREAL_NVP(sent);
        ar& CEREAL_NVP(receiveBufErrors);
        ar& CEREAL_NVP(sendBufErrors);
        ar& CEREAL_NVP(checksumErrors);
        ar& CEREAL_NVP(ignoredMulti);
    }


template void UDPStats::serialize(cereal::BinaryInputArchive& ar, unsigned int version);
template void UDPStats::serialize(cereal::BinaryOutputArchive& ar, unsigned int version);
template void UDPStats::serialize(cereal::IntrospectionArchive& ar, unsigned int version);
template void UDPStats::serialize(cereal::JSONInputArchive& ar, unsigned int version);
template void UDPStats::serialize(cereal::JSONOutputArchive& ar, unsigned int version);
template void UDPStats::serialize(cereal::PortableBinaryInputArchive& ar, unsigned int version);
template void UDPStats::serialize(cereal::PortableBinaryOutputArchive& ar, unsigned int version);

std::ostream& operator<< (std::ostream& os, const UDPStats& udpstats)
{
    udpstats.toCsv(os);
    return os;
}

bool operator== (const UDPStats &lhs, const UDPStats &rhs)
{
    return (  lhs.received== rhs.received &&
     lhs.unknownPort== rhs.unknownPort &&
     lhs.inErrors== rhs.inErrors &&
     lhs.sent== rhs.sent &&
     lhs.receiveBufErrors== rhs.receiveBufErrors &&
     lhs.sendBufErrors== rhs.sendBufErrors &&
     lhs.checksumErrors== rhs.checksumErrors &&
     lhs.ignoredMulti== rhs.ignoredMulti 
    );
}

std::size_t UDPStats::hash() const noexcept
{
    std::size_t result = 17;
    result = result * 31 + std::hash<uint64_t>()(received);
    result = result * 31 + std::hash<uint64_t>()(unknownPort);
    result = result * 31 + std::hash<uint64_t>()(inErrors);
    result = result * 31 + std::hash<uint64_t>()(sent);
    result = result * 31 + std::hash<uint64_t>()(receiveBufErrors);
    result = result * 31 + std::hash<uint64_t>()(sendBufErrors);
    result = result * 31 + std::hash<uint64_t>()(checksumErrors);
    result = result * 31 + std::hash<uint64_t>()(ignoredMulti);
    return result;
}

 } // namespace av
 } // namespace msg


CEREAL_BIND_TO_ARCHIVES(av::msg::UDPStats)

 namespace av {
 namespace msg {


TCPStats::TCPStats() = default;

TCPStats::TCPStats(
    uint64_t retransSegments_arg ,
    uint64_t inErrors_arg ,
    uint64_t checksumErrors_arg 
    ):
    retransSegments { std::move(retransSegments_arg) } // NOLINT
      ,
    inErrors { std::move(inErrors_arg) } // NOLINT
      ,
    checksumErrors { std::move(checksumErrors_arg) } // NOLINT
      
    {
}

TCPStats::TCPStats(const TCPStats&) = default;

// NOLINTNEXTLINE
TCPStats::TCPStats(TCPStats&&) = default;

TCPStats& TCPStats::operator=(const TCPStats&) = default;

// NOLINTNEXTLINE
TCPStats& TCPStats::operator=(TCPStats&&) = default;

void TCPStats::toCsv(std::ostream& out) const
{
    out.precision(16);
    out << std::boolalpha;
    out << retransSegments;out << ",";
    out << inErrors;out << ",";
    out << checksumErrors;
    
}

std::stringstream TCPStats::toStringstream() const
{
    std::stringstream out;
    toCsv(out);
    return out;
}

template<class Archive>
    void TCPStats::serialize(Archive& ar, [[maybe_unused]] unsigned int version)
    {

        ar& CEREAL_NVP(retransSegments);
        ar& CEREAL_NVP(inErrors);
        ar& CEREAL_NVP(checksumErrors);
    }


template void TCPStats::serialize(cereal::BinaryInputArchive& ar, unsigned int version);
template void TCPStats::serialize(cereal::BinaryOutputArchive& ar, unsigned int version);
template void TCPStats::serialize(cereal::IntrospectionArchive& ar, unsigned int version);
template void TCPStats::serialize(cereal::JSONInputArchive& ar, unsigned int version);
template void TCPStats::serialize(cereal::JSONOutputArchive& ar, unsigned int version);
template void TCPStats::serialize(cereal::PortableBinaryInputArchive& ar, unsigned int version);
template void TCPStats::serialize(cereal::PortableBinaryOutputArchive& ar, unsigned int version);

std::ostream& operator<< (std::ostream& os, const TCPStats& tcpstats)
{
    tcpstats.toCsv(os);
    return os;
}

bool operator== (const TCPStats &lhs, const TCPStats &rhs)
{
    return (  lhs.retransSegments== rhs.retransSegments &&
     lhs.inErrors== rhs.inErrors &&
     lhs.checksumErrors== rhs.checksumErrors 
    );
}

std::size_t TCPStats::hash() const noexcept
{
    std::size_t result = 17;
    result = result * 31 + std::hash<uint64_t>()(retransSegments);
    result = result * 31 + std::hash<uint64_t>()(inErrors);
    result = result * 31 + std::hash<uint64_t>()(checksumErrors);
    return result;
}

 } // namespace av
 } // namespace msg


CEREAL_BIND_TO_ARCHIVES(av::msg::TCPStats)




 namespace av {
 namespace msg {


NetworkStatistics::NetworkStatistics() = default;

NetworkStatistics::NetworkStatistics(
    TimePoint timestamp_arg ,
    std::string hostname_arg ,
    std::vector<InterfaceStats> interfaceStats_arg ,
    UDPStats udpStats_arg ,
    TCPStats tcpStats_arg 
    ):
    timestamp { std::move(timestamp_arg) } // NOLINT
      ,hostname { std::move(hostname_arg) } // NOLINT
      ,interfaceStats { std::move(interfaceStats_arg) } // NOLINT
      ,udpStats { std::move(udpStats_arg) } // NOLINT
      ,tcpStats { std::move(tcpStats_arg) } // NOLINT
      {
}

NetworkStatistics::NetworkStatistics(const NetworkStatistics&) = default;

// NOLINTNEXTLINE
NetworkStatistics::NetworkStatistics(NetworkStatistics&&) = default;

NetworkStatistics& NetworkStatistics::operator=(const NetworkStatistics&) = default;

// NOLINTNEXTLINE
NetworkStatistics& NetworkStatistics::operator=(NetworkStatistics&&) = default;

void NetworkStatistics::toCsv(std::ostream& out) const
{
    out.precision(16);
    out << std::boolalpha;
    out << timestamp;out << ",";
    out << "\"";
        out << hostname;
        out << "\"";out << ",";
    // Enclose sequence in double quotations and curly brackets to keep CSV line length uniform
        
        out << "\"{{";
        
        if (!interfaceStats.empty())
        {
            std::copy(interfaceStats.begin(), interfaceStats.end() - 1, std::ostream_iterator<InterfaceStats>(out, "},{"));
            std::copy(interfaceStats.end()-1, interfaceStats.end(), std::ostream_iterator<InterfaceStats>(out, ""));
        }
        out << "}}\"";out << ",";
    out << udpStats;out << ",";
    out << tcpStats;
    
}

std::stringstream NetworkStatistics::toStringstream() const
{
    std::stringstream out;
    toCsv(out);
    return out;
}

template<class Archive>
    void NetworkStatistics::serialize(Archive& ar, unsigned int version)
{
    if (version == 1) {
        otimePoint otp;
        ar& cereal::make_nvp("timestamp", otp);
        timestamp = av::TimePoint(std::chrono::seconds(otp.tv_sec) + std::chrono::nanoseconds(otp.tv_nsec));
    }
    else {
        ar& CEREAL_NVP(timestamp);
    }
    
    if (version >= 1) {
        ar& CEREAL_NVP(hostname);
        ar& CEREAL_NVP(interfaceStats);
    }
    
    if (version >= 2)
    {
        ar& CEREAL_NVP(udpStats);
        ar& CEREAL_NVP(tcpStats);
    }
}

}} // to close av::msg namespace temporary

struct NetworkStatisticsMessage : public av::msg::NetworkStatistics {};
CEREAL_CLASS_VERSION(NetworkStatisticsMessage, 3)
CEREAL_REGISTER_TYPE(NetworkStatisticsMessage)

namespace av {
namespace msg {



template void NetworkStatistics::serialize(cereal::BinaryInputArchive& ar, unsigned int version);
template void NetworkStatistics::serialize(cereal::BinaryOutputArchive& ar, unsigned int version);
template void NetworkStatistics::serialize(cereal::IntrospectionArchive& ar, unsigned int version);
template void NetworkStatistics::serialize(cereal::JSONInputArchive& ar, unsigned int version);
template void NetworkStatistics::serialize(cereal::JSONOutputArchive& ar, unsigned int version);
template void NetworkStatistics::serialize(cereal::PortableBinaryInputArchive& ar, unsigned int version);
template void NetworkStatistics::serialize(cereal::PortableBinaryOutputArchive& ar, unsigned int version);

std::ostream& operator<< (std::ostream& os, const NetworkStatistics& networkstatistics)
{
    networkstatistics.toCsv(os);
    return os;
}

bool operator== (const NetworkStatistics &lhs, const NetworkStatistics &rhs)
{
    return (  lhs.timestamp== rhs.timestamp &&
     lhs.hostname== rhs.hostname &&
     lhs.interfaceStats== rhs.interfaceStats &&
     lhs.udpStats== rhs.udpStats &&
     lhs.tcpStats== rhs.tcpStats 
    );
}

std::size_t NetworkStatistics::hash() const noexcept
{
    std::size_t result = 17;
    result = result * 31 + 0;
    result = result * 31 + std::hash<std::string>()(hostname);
    result = result * 31 + hashVector(interfaceStats);
    result = result * 31 + std::hash<UDPStats>()(udpStats);
    result = result * 31 + std::hash<TCPStats>()(tcpStats);
    return result;
}

 } // namespace av
 } // namespace msg


CEREAL_BIND_TO_ARCHIVES(av::msg::NetworkStatistics)

