#include <interfaces/OcclusionMapContainer/OcclusionMapContainer.h>

#include <iterator>
#include <type_traits>

#include <cereal/cereal.hpp>
#include <cereal/archives/json.hpp>
#include <cereal/archives/binary.hpp>

#include <cereal/types/map.hpp>
#include <cereal/types/vector.hpp>
#include <cereal/types/array.hpp>
#include <cereal/types/string.hpp>
#include <cereal/types/complex.hpp>

#include <ODatum/IntrospectionArchive.h>
#include <ODatum/Hash.h>





 namespace av {
 namespace msg {
 namespace tc {


OcclusionMapContainer::OcclusionMapContainer() = default;

OcclusionMapContainer::OcclusionMapContainer(
    int64_t timeUs_arg ,
    av::geometry::Vector2d position_arg ,
    double cellSize_arg ,
    uint32_t width_arg ,
    uint32_t height_arg ,
    std::vector<uint8_t> data_arg 
    ):
    timeUs { std::move(timeUs_arg) } // NOLINT
      ,position { std::move(position_arg) } // NOLINT
      ,cellSize { std::move(cellSize_arg) } // NOLINT
      ,width { std::move(width_arg) } // NOLINT
      ,height { std::move(height_arg) } // NOLINT
      ,data { std::move(data_arg) } // NOLINT
      {
}

OcclusionMapContainer::OcclusionMapContainer(const OcclusionMapContainer&) = default;

// NOLINTNEXTLINE
OcclusionMapContainer::OcclusionMapContainer(OcclusionMapContainer&&) = default;

OcclusionMapContainer& OcclusionMapContainer::operator=(const OcclusionMapContainer&) = default;

// NOLINTNEXTLINE
OcclusionMapContainer& OcclusionMapContainer::operator=(OcclusionMapContainer&&) = default;

void OcclusionMapContainer::toCsv(std::ostream& out) const
{
    out.precision(16);
    out << std::boolalpha;
    out << timeUs;out << ",";
    out << position;out << ",";
    out << cellSize;out << ",";
    out << width;out << ",";
    out << height;out << ",";
    // Enclose sequence in double quotations and curly brackets to keep CSV line length uniform
        
        out << "\"{{";
        
        if (!data.empty())
        {
            std::copy(data.begin(), data.end() - 1, std::ostream_iterator<uint8_t>(out, "},{"));
            std::copy(data.end()-1, data.end(), std::ostream_iterator<uint8_t>(out, ""));
        }
        out << "}}\"";
    
}

std::stringstream OcclusionMapContainer::toStringstream() const
{
    std::stringstream out;
    toCsv(out);
    return out;
}

template<class Archive>
    void OcclusionMapContainer::serialize(Archive& ar, [[maybe_unused]] unsigned int version)
    {

        ar& CEREAL_NVP(timeUs);
        ar& CEREAL_NVP(position);
        ar& CEREAL_NVP(cellSize);
        ar& CEREAL_NVP(width);
        ar& CEREAL_NVP(height);
        ar& CEREAL_NVP(data);
    }


template void OcclusionMapContainer::serialize(cereal::BinaryInputArchive& ar, unsigned int version);
template void OcclusionMapContainer::serialize(cereal::BinaryOutputArchive& ar, unsigned int version);
template void OcclusionMapContainer::serialize(cereal::IntrospectionArchive& ar, unsigned int version);
template void OcclusionMapContainer::serialize(cereal::JSONInputArchive& ar, unsigned int version);
template void OcclusionMapContainer::serialize(cereal::JSONOutputArchive& ar, unsigned int version);
template void OcclusionMapContainer::serialize(cereal::PortableBinaryInputArchive& ar, unsigned int version);
template void OcclusionMapContainer::serialize(cereal::PortableBinaryOutputArchive& ar, unsigned int version);

std::ostream& operator<< (std::ostream& os, const OcclusionMapContainer& occlusionmapcontainer)
{
    occlusionmapcontainer.toCsv(os);
    return os;
}

bool operator== (const OcclusionMapContainer &lhs, const OcclusionMapContainer &rhs)
{
    return (  lhs.timeUs== rhs.timeUs &&
     lhs.position== rhs.position &&
     lhs.cellSize== rhs.cellSize &&
     lhs.width== rhs.width &&
     lhs.height== rhs.height &&
     lhs.data== rhs.data 
    );
}

std::size_t OcclusionMapContainer::hash() const noexcept
{
    std::size_t result = 17;
    result = result * 31 + std::hash<int64_t>()(timeUs);
    result = result * 31 + std::hash<av::geometry::Vector2d>()(position);
    result = result * 31 + std::hash<double>()(cellSize);
    result = result * 31 + std::hash<uint32_t>()(width);
    result = result * 31 + std::hash<uint32_t>()(height);
    result = result * 31 + hashVector(data);
    return result;
}

 } // namespace av
 } // namespace msg
 } // namespace tc


CEREAL_BIND_TO_ARCHIVES(av::msg::tc::OcclusionMapContainer)

