#include <interfaces/Odometry/Odometry.h>

#include <iterator>
#include <type_traits>

#include <cereal/cereal.hpp>
#include <cereal/archives/json.hpp>
#include <cereal/archives/binary.hpp>

#include <cereal/types/map.hpp>
#include <cereal/types/vector.hpp>
#include <cereal/types/array.hpp>
#include <cereal/types/string.hpp>
#include <cereal/types/complex.hpp>

#include <ODatum/IntrospectionArchive.h>
#include <ODatum/Hash.h>





 namespace av {
 namespace msg {


Odometry::Odometry() = default;

Odometry::Odometry(
    TimePoint frontDriverTimeOfValidity_arg ,
    double frontDriver_mps_arg ,
    TimePoint frontPassengerTimeOfValidity_arg ,
    double frontPassenger_mps_arg ,
    TimePoint rearDriverTimeOfValidity_arg ,
    double rearDriver_mps_arg ,
    TimePoint rearPassengerTimeOfValidity_arg ,
    double rearPassenger_mps_arg ,
    TimePoint odoYawRateTimeOfValidity_arg ,
    double odoYawRate_arg ,
    TimePoint odoSteeringWheelAngleTimeOfValidity_arg ,
    double odoSteeringWheelAngle_arg 
    ):
    frontDriverTimeOfValidity { std::move(frontDriverTimeOfValidity_arg) } // NOLINT
      ,frontDriver_mps { std::move(frontDriver_mps_arg) } // NOLINT
      ,frontPassengerTimeOfValidity { std::move(frontPassengerTimeOfValidity_arg) } // NOLINT
      ,frontPassenger_mps { std::move(frontPassenger_mps_arg) } // NOLINT
      ,rearDriverTimeOfValidity { std::move(rearDriverTimeOfValidity_arg) } // NOLINT
      ,rearDriver_mps { std::move(rearDriver_mps_arg) } // NOLINT
      ,rearPassengerTimeOfValidity { std::move(rearPassengerTimeOfValidity_arg) } // NOLINT
      ,rearPassenger_mps { std::move(rearPassenger_mps_arg) } // NOLINT
      ,odoYawRateTimeOfValidity { std::move(odoYawRateTimeOfValidity_arg) } // NOLINT
      ,odoYawRate { std::move(odoYawRate_arg) } // NOLINT
      ,odoSteeringWheelAngleTimeOfValidity { std::move(odoSteeringWheelAngleTimeOfValidity_arg) } // NOLINT
      ,odoSteeringWheelAngle { std::move(odoSteeringWheelAngle_arg) } // NOLINT
      {
}

Odometry::Odometry(const Odometry&) = default;

// NOLINTNEXTLINE
Odometry::Odometry(Odometry&&) = default;

Odometry& Odometry::operator=(const Odometry&) = default;

// NOLINTNEXTLINE
Odometry& Odometry::operator=(Odometry&&) = default;

void Odometry::toCsv(std::ostream& out) const
{
    out.precision(16);
    out << std::boolalpha;
    out << frontDriverTimeOfValidity;out << ",";
    out << frontDriver_mps;out << ",";
    out << frontPassengerTimeOfValidity;out << ",";
    out << frontPassenger_mps;out << ",";
    out << rearDriverTimeOfValidity;out << ",";
    out << rearDriver_mps;out << ",";
    out << rearPassengerTimeOfValidity;out << ",";
    out << rearPassenger_mps;out << ",";
    out << odoYawRateTimeOfValidity;out << ",";
    out << odoYawRate;out << ",";
    out << odoSteeringWheelAngleTimeOfValidity;out << ",";
    out << odoSteeringWheelAngle;
    
}

std::stringstream Odometry::toStringstream() const
{
    std::stringstream out;
    toCsv(out);
    return out;
}

template<class Archive>
    void Odometry::serialize(Archive& ar, unsigned int version)
{
    if (version == 1)
    {
        otimePoint fdOtp;
        ar& cereal::make_nvp("frontDriverTimeOfValidity", fdOtp);
        frontDriverTimeOfValidity = av::TimePoint(std::chrono::seconds(fdOtp.tv_sec) + std::chrono::nanoseconds(fdOtp.tv_nsec));

        ar& CEREAL_NVP(frontDriver_mps);

        otimePoint fpOtp;
        ar& cereal::make_nvp("frontPassengerTimeOfValidity", fpOtp);
        frontPassengerTimeOfValidity = av::TimePoint(std::chrono::seconds(fpOtp.tv_sec) + std::chrono::nanoseconds(fpOtp.tv_nsec));

        ar& CEREAL_NVP(frontPassenger_mps);

        otimePoint rdOtp;
        ar& cereal::make_nvp("rearDriverTimeOfValidity", rdOtp);
        rearDriverTimeOfValidity = av::TimePoint(std::chrono::seconds(rdOtp.tv_sec) + std::chrono::nanoseconds(rdOtp.tv_nsec));

        ar& CEREAL_NVP(rearDriver_mps);

        otimePoint rpOtp;
        ar& cereal::make_nvp("rearPassengerTimeOfValidity", rpOtp);
        rearPassengerTimeOfValidity = av::TimePoint(std::chrono::seconds(rpOtp.tv_sec) + std::chrono::nanoseconds(rpOtp.tv_nsec));

        ar& CEREAL_NVP(rearPassenger_mps);

        otimePoint yawOtp;
        ar& cereal::make_nvp("odoYawRateTimeOfValidity", yawOtp);
        odoYawRateTimeOfValidity = av::TimePoint(std::chrono::seconds(yawOtp.tv_sec) + std::chrono::nanoseconds(yawOtp.tv_nsec));

        ar& CEREAL_NVP(odoYawRate);

        otimePoint steerOtp;
        ar& cereal::make_nvp("odoSteeringWheelAngleTimeOfValidity", steerOtp);
        odoSteeringWheelAngleTimeOfValidity = av::TimePoint(std::chrono::seconds(steerOtp.tv_sec) + std::chrono::nanoseconds(steerOtp.tv_nsec));

        ar& CEREAL_NVP(odoSteeringWheelAngle);
    }
    
    if (version >= 2)
    {
        ar& CEREAL_NVP(frontDriverTimeOfValidity);
        ar& CEREAL_NVP(frontDriver_mps);
        ar& CEREAL_NVP(frontPassengerTimeOfValidity);
        ar& CEREAL_NVP(frontPassenger_mps);

        ar& CEREAL_NVP(rearDriverTimeOfValidity);
        ar& CEREAL_NVP(rearDriver_mps);
        ar& CEREAL_NVP(rearPassengerTimeOfValidity);
        ar& CEREAL_NVP(rearPassenger_mps);

        ar& CEREAL_NVP(odoYawRateTimeOfValidity);
        ar& CEREAL_NVP(odoYawRate);
        ar& CEREAL_NVP(odoSteeringWheelAngleTimeOfValidity);
        ar& CEREAL_NVP(odoSteeringWheelAngle);
    }
}

}} // to close av::msg namespace temporary

struct OdometryMessage : public av::msg::Odometry{};
CEREAL_CLASS_VERSION(OdometryMessage, 2)
CEREAL_REGISTER_TYPE(OdometryMessage)

namespace av {
namespace msg {


template void Odometry::serialize(cereal::BinaryInputArchive& ar, unsigned int version);
template void Odometry::serialize(cereal::BinaryOutputArchive& ar, unsigned int version);
template void Odometry::serialize(cereal::IntrospectionArchive& ar, unsigned int version);
template void Odometry::serialize(cereal::JSONInputArchive& ar, unsigned int version);
template void Odometry::serialize(cereal::JSONOutputArchive& ar, unsigned int version);
template void Odometry::serialize(cereal::PortableBinaryInputArchive& ar, unsigned int version);
template void Odometry::serialize(cereal::PortableBinaryOutputArchive& ar, unsigned int version);

std::ostream& operator<< (std::ostream& os, const Odometry& odometry)
{
    odometry.toCsv(os);
    return os;
}

bool operator== (const Odometry &lhs, const Odometry &rhs)
{
    return (  lhs.frontDriverTimeOfValidity== rhs.frontDriverTimeOfValidity &&
     lhs.frontDriver_mps== rhs.frontDriver_mps &&
     lhs.frontPassengerTimeOfValidity== rhs.frontPassengerTimeOfValidity &&
     lhs.frontPassenger_mps== rhs.frontPassenger_mps &&
     lhs.rearDriverTimeOfValidity== rhs.rearDriverTimeOfValidity &&
     lhs.rearDriver_mps== rhs.rearDriver_mps &&
     lhs.rearPassengerTimeOfValidity== rhs.rearPassengerTimeOfValidity &&
     lhs.rearPassenger_mps== rhs.rearPassenger_mps &&
     lhs.odoYawRateTimeOfValidity== rhs.odoYawRateTimeOfValidity &&
     lhs.odoYawRate== rhs.odoYawRate &&
     lhs.odoSteeringWheelAngleTimeOfValidity== rhs.odoSteeringWheelAngleTimeOfValidity &&
     lhs.odoSteeringWheelAngle== rhs.odoSteeringWheelAngle 
    );
}

std::size_t Odometry::hash() const noexcept
{
    std::size_t result = 17;
    result = result * 31 + 0;
    result = result * 31 + std::hash<double>()(frontDriver_mps);
    result = result * 31 + 0;
    result = result * 31 + std::hash<double>()(frontPassenger_mps);
    result = result * 31 + 0;
    result = result * 31 + std::hash<double>()(rearDriver_mps);
    result = result * 31 + 0;
    result = result * 31 + std::hash<double>()(rearPassenger_mps);
    result = result * 31 + 0;
    result = result * 31 + std::hash<double>()(odoYawRate);
    result = result * 31 + 0;
    result = result * 31 + std::hash<double>()(odoSteeringWheelAngle);
    return result;
}

 } // namespace av
 } // namespace msg


CEREAL_BIND_TO_ARCHIVES(av::msg::Odometry)

