#include <interfaces/OptionalRouteProgress/OptionalRouteProgress.h>

#include <iterator>
#include <type_traits>

#include <cereal/cereal.hpp>
#include <cereal/archives/json.hpp>
#include <cereal/archives/binary.hpp>

#include <cereal/types/map.hpp>
#include <cereal/types/vector.hpp>
#include <cereal/types/array.hpp>
#include <cereal/types/string.hpp>
#include <cereal/types/complex.hpp>

#include <ODatum/IntrospectionArchive.h>
#include <ODatum/Hash.h>


 namespace av {
 namespace msg {


MapIdSequence::MapIdSequence() = default;

MapIdSequence::MapIdSequence(
    std::vector<MapId> mapIdSequence_arg 
    ):
    mapIdSequence { std::move(mapIdSequence_arg) } // NOLINT
      
    {
}

MapIdSequence::MapIdSequence(const MapIdSequence&) = default;

// NOLINTNEXTLINE
MapIdSequence::MapIdSequence(MapIdSequence&&) = default;

MapIdSequence& MapIdSequence::operator=(const MapIdSequence&) = default;

// NOLINTNEXTLINE
MapIdSequence& MapIdSequence::operator=(MapIdSequence&&) = default;

void MapIdSequence::toCsv(std::ostream& out) const
{
    out.precision(16);
    out << std::boolalpha;
    // Enclose sequence in double quotations and curly brackets to keep CSV line length uniform
        
        out << "\"{{";
        
        if (!mapIdSequence.empty())
        {
            std::copy(mapIdSequence.begin(), mapIdSequence.end() - 1, std::ostream_iterator<MapId>(out, "},{"));
            std::copy(mapIdSequence.end()-1, mapIdSequence.end(), std::ostream_iterator<MapId>(out, ""));
        }
        out << "}}\"";
    
}

std::stringstream MapIdSequence::toStringstream() const
{
    std::stringstream out;
    toCsv(out);
    return out;
}

template<class Archive>
    void MapIdSequence::serialize(Archive& ar, [[maybe_unused]] unsigned int version)
    {

        ar& CEREAL_NVP(mapIdSequence);
    }


template void MapIdSequence::serialize(cereal::BinaryInputArchive& ar, unsigned int version);
template void MapIdSequence::serialize(cereal::BinaryOutputArchive& ar, unsigned int version);
template void MapIdSequence::serialize(cereal::IntrospectionArchive& ar, unsigned int version);
template void MapIdSequence::serialize(cereal::JSONInputArchive& ar, unsigned int version);
template void MapIdSequence::serialize(cereal::JSONOutputArchive& ar, unsigned int version);
template void MapIdSequence::serialize(cereal::PortableBinaryInputArchive& ar, unsigned int version);
template void MapIdSequence::serialize(cereal::PortableBinaryOutputArchive& ar, unsigned int version);

std::ostream& operator<< (std::ostream& os, const MapIdSequence& mapidsequence)
{
    mapidsequence.toCsv(os);
    return os;
}

bool operator== (const MapIdSequence &lhs, const MapIdSequence &rhs)
{
    return (  lhs.mapIdSequence== rhs.mapIdSequence 
    );
}

std::size_t MapIdSequence::hash() const noexcept
{
    std::size_t result = 17;
    result = result * 31 + hashVector(mapIdSequence);
    return result;
}

 } // namespace av
 } // namespace msg


CEREAL_BIND_TO_ARCHIVES(av::msg::MapIdSequence)

 namespace av {
 namespace msg {


RouteProgress::RouteProgress() = default;

RouteProgress::RouteProgress(
    plan_ctrl::PlannerHeader header_arg ,
    MapId currentRoadblock_arg ,
    MapIdSequence currentRoute_arg ,
    MapIdSequence upcomingBaselineRoute_arg ,
    msg::SE2 routeGoalPose_arg ,
    double distanceToRouteGoal_arg ,
    msg::GUID missionGoalId_arg 
    ):
    header { std::move(header_arg) } // NOLINT
      ,
    currentRoadblock { std::move(currentRoadblock_arg) } // NOLINT
      ,
    currentRoute { std::move(currentRoute_arg) } // NOLINT
      ,
    upcomingBaselineRoute { std::move(upcomingBaselineRoute_arg) } // NOLINT
      ,
    routeGoalPose { std::move(routeGoalPose_arg) } // NOLINT
      ,
    distanceToRouteGoal { std::move(distanceToRouteGoal_arg) } // NOLINT
      ,
    missionGoalId { std::move(missionGoalId_arg) } // NOLINT
      
    {
}

RouteProgress::RouteProgress(const RouteProgress&) = default;

// NOLINTNEXTLINE
RouteProgress::RouteProgress(RouteProgress&&) = default;

RouteProgress& RouteProgress::operator=(const RouteProgress&) = default;

// NOLINTNEXTLINE
RouteProgress& RouteProgress::operator=(RouteProgress&&) = default;

void RouteProgress::toCsv(std::ostream& out) const
{
    out.precision(16);
    out << std::boolalpha;
    out << header;out << ",";
    out << currentRoadblock;out << ",";
    out << currentRoute;out << ",";
    out << upcomingBaselineRoute;out << ",";
    out << routeGoalPose;out << ",";
    out << distanceToRouteGoal;out << ",";
    out << missionGoalId;
    
}

std::stringstream RouteProgress::toStringstream() const
{
    std::stringstream out;
    toCsv(out);
    return out;
}

template<class Archive>
    void RouteProgress::serialize(Archive& ar, unsigned int version)
{
    if(version >= 1)
    {
        ar& CEREAL_NVP(header);
        ar& CEREAL_NVP(currentRoadblock);
        ar& CEREAL_NVP(currentRoute);
    }
    if(version >= 2)
    {
        ar& CEREAL_NVP(routeGoalPose);
        ar& CEREAL_NVP(distanceToRouteGoal);
    }
    if(version >= 3)
    {
        ar& CEREAL_NVP(missionGoalId);
    }
    if(version >= 4){
        ar& CEREAL_NVP(upcomingBaselineRoute);
    }
}


template void RouteProgress::serialize(cereal::BinaryInputArchive& ar, unsigned int version);
template void RouteProgress::serialize(cereal::BinaryOutputArchive& ar, unsigned int version);
template void RouteProgress::serialize(cereal::IntrospectionArchive& ar, unsigned int version);
template void RouteProgress::serialize(cereal::JSONInputArchive& ar, unsigned int version);
template void RouteProgress::serialize(cereal::JSONOutputArchive& ar, unsigned int version);
template void RouteProgress::serialize(cereal::PortableBinaryInputArchive& ar, unsigned int version);
template void RouteProgress::serialize(cereal::PortableBinaryOutputArchive& ar, unsigned int version);

std::ostream& operator<< (std::ostream& os, const RouteProgress& routeprogress)
{
    routeprogress.toCsv(os);
    return os;
}

bool operator== (const RouteProgress &lhs, const RouteProgress &rhs)
{
    return (  lhs.header== rhs.header &&
     lhs.currentRoadblock== rhs.currentRoadblock &&
     lhs.currentRoute== rhs.currentRoute &&
     lhs.upcomingBaselineRoute== rhs.upcomingBaselineRoute &&
     lhs.routeGoalPose== rhs.routeGoalPose &&
     lhs.distanceToRouteGoal== rhs.distanceToRouteGoal &&
     lhs.missionGoalId== rhs.missionGoalId 
    );
}

std::size_t RouteProgress::hash() const noexcept
{
    std::size_t result = 17;
    result = result * 31 + std::hash<plan_ctrl::PlannerHeader>()(header);
    result = result * 31 + std::hash<MapId>()(currentRoadblock);
    result = result * 31 + std::hash<MapIdSequence>()(currentRoute);
    result = result * 31 + std::hash<MapIdSequence>()(upcomingBaselineRoute);
    result = result * 31 + std::hash<msg::SE2>()(routeGoalPose);
    result = result * 31 + std::hash<double>()(distanceToRouteGoal);
    result = result * 31 + std::hash<msg::GUID>()(missionGoalId);
    return result;
}

 } // namespace av
 } // namespace msg


CEREAL_BIND_TO_ARCHIVES(av::msg::RouteProgress)




 namespace av {
 namespace msg {


OptionalRouteProgress::OptionalRouteProgress() = default;

OptionalRouteProgress::OptionalRouteProgress(
    bool hasValue_arg ,
    RouteProgress value_arg 
    ):
    hasValue { std::move(hasValue_arg) } // NOLINT
      ,value { std::move(value_arg) } // NOLINT
      {
}

OptionalRouteProgress::OptionalRouteProgress(const OptionalRouteProgress&) = default;

// NOLINTNEXTLINE
OptionalRouteProgress::OptionalRouteProgress(OptionalRouteProgress&&) = default;

OptionalRouteProgress& OptionalRouteProgress::operator=(const OptionalRouteProgress&) = default;

// NOLINTNEXTLINE
OptionalRouteProgress& OptionalRouteProgress::operator=(OptionalRouteProgress&&) = default;

void OptionalRouteProgress::toCsv(std::ostream& out) const
{
    out.precision(16);
    out << std::boolalpha;
    out << hasValue;out << ",";
    out << value;
    
}

std::stringstream OptionalRouteProgress::toStringstream() const
{
    std::stringstream out;
    toCsv(out);
    return out;
}

template<class Archive>
    void OptionalRouteProgress::serialize(Archive& ar, [[maybe_unused]] unsigned int version)
    {

        ar& CEREAL_NVP(hasValue);
        ar& CEREAL_NVP(value);
    }


template void OptionalRouteProgress::serialize(cereal::BinaryInputArchive& ar, unsigned int version);
template void OptionalRouteProgress::serialize(cereal::BinaryOutputArchive& ar, unsigned int version);
template void OptionalRouteProgress::serialize(cereal::IntrospectionArchive& ar, unsigned int version);
template void OptionalRouteProgress::serialize(cereal::JSONInputArchive& ar, unsigned int version);
template void OptionalRouteProgress::serialize(cereal::JSONOutputArchive& ar, unsigned int version);
template void OptionalRouteProgress::serialize(cereal::PortableBinaryInputArchive& ar, unsigned int version);
template void OptionalRouteProgress::serialize(cereal::PortableBinaryOutputArchive& ar, unsigned int version);

std::ostream& operator<< (std::ostream& os, const OptionalRouteProgress& optionalrouteprogress)
{
    optionalrouteprogress.toCsv(os);
    return os;
}

bool operator== (const OptionalRouteProgress &lhs, const OptionalRouteProgress &rhs)
{
    return (  lhs.hasValue== rhs.hasValue &&
     lhs.value== rhs.value 
    );
}

std::size_t OptionalRouteProgress::hash() const noexcept
{
    std::size_t result = 17;
    result = result * 31 + std::hash<bool>()(hasValue);
    result = result * 31 + std::hash<RouteProgress>()(value);
    return result;
}

 } // namespace av
 } // namespace msg


CEREAL_BIND_TO_ARCHIVES(av::msg::OptionalRouteProgress)

