#include <interfaces/PDU2CurrMonitoring/PDU2CurrMonitoring.h>

#include <iterator>
#include <type_traits>

#include <cereal/cereal.hpp>
#include <cereal/archives/json.hpp>
#include <cereal/archives/binary.hpp>

#include <cereal/types/map.hpp>
#include <cereal/types/vector.hpp>
#include <cereal/types/array.hpp>
#include <cereal/types/string.hpp>
#include <cereal/types/complex.hpp>

#include <ODatum/IntrospectionArchive.h>
#include <ODatum/Hash.h>





 namespace av {
 namespace msg {


PDU2CurrMonitoring::PDU2CurrMonitoring() = default;

PDU2CurrMonitoring::PDU2CurrMonitoring(
    TimePoint timestamp_arg ,
    float RadarRRCurrent_mA_arg ,
    float RadarRLCurrent_mA_arg ,
    float LidarBCurrent_mA_arg 
    ):
    timestamp { std::move(timestamp_arg) } // NOLINT
      ,RadarRRCurrent_mA { std::move(RadarRRCurrent_mA_arg) } // NOLINT
      ,RadarRLCurrent_mA { std::move(RadarRLCurrent_mA_arg) } // NOLINT
      ,LidarBCurrent_mA { std::move(LidarBCurrent_mA_arg) } // NOLINT
      {
}

PDU2CurrMonitoring::PDU2CurrMonitoring(const PDU2CurrMonitoring&) = default;

// NOLINTNEXTLINE
PDU2CurrMonitoring::PDU2CurrMonitoring(PDU2CurrMonitoring&&) = default;

PDU2CurrMonitoring& PDU2CurrMonitoring::operator=(const PDU2CurrMonitoring&) = default;

// NOLINTNEXTLINE
PDU2CurrMonitoring& PDU2CurrMonitoring::operator=(PDU2CurrMonitoring&&) = default;

void PDU2CurrMonitoring::toCsv(std::ostream& out) const
{
    out.precision(16);
    out << std::boolalpha;
    out << timestamp;out << ",";
    out << RadarRRCurrent_mA;out << ",";
    out << RadarRLCurrent_mA;out << ",";
    out << LidarBCurrent_mA;
    
}

std::stringstream PDU2CurrMonitoring::toStringstream() const
{
    std::stringstream out;
    toCsv(out);
    return out;
}

template<class Archive>
    void PDU2CurrMonitoring::serialize(Archive& ar, [[maybe_unused]] unsigned int version)
    {

        ar& CEREAL_NVP(timestamp);
        ar& CEREAL_NVP(RadarRRCurrent_mA);
        ar& CEREAL_NVP(RadarRLCurrent_mA);
        ar& CEREAL_NVP(LidarBCurrent_mA);
    }


template void PDU2CurrMonitoring::serialize(cereal::BinaryInputArchive& ar, unsigned int version);
template void PDU2CurrMonitoring::serialize(cereal::BinaryOutputArchive& ar, unsigned int version);
template void PDU2CurrMonitoring::serialize(cereal::IntrospectionArchive& ar, unsigned int version);
template void PDU2CurrMonitoring::serialize(cereal::JSONInputArchive& ar, unsigned int version);
template void PDU2CurrMonitoring::serialize(cereal::JSONOutputArchive& ar, unsigned int version);
template void PDU2CurrMonitoring::serialize(cereal::PortableBinaryInputArchive& ar, unsigned int version);
template void PDU2CurrMonitoring::serialize(cereal::PortableBinaryOutputArchive& ar, unsigned int version);

std::ostream& operator<< (std::ostream& os, const PDU2CurrMonitoring& pdu2currmonitoring)
{
    pdu2currmonitoring.toCsv(os);
    return os;
}

bool operator== (const PDU2CurrMonitoring &lhs, const PDU2CurrMonitoring &rhs)
{
    return (  lhs.timestamp== rhs.timestamp &&
     lhs.RadarRRCurrent_mA== rhs.RadarRRCurrent_mA &&
     lhs.RadarRLCurrent_mA== rhs.RadarRLCurrent_mA &&
     lhs.LidarBCurrent_mA== rhs.LidarBCurrent_mA 
    );
}

std::size_t PDU2CurrMonitoring::hash() const noexcept
{
    std::size_t result = 17;
    result = result * 31 + 0;
    result = result * 31 + std::hash<float>()(RadarRRCurrent_mA);
    result = result * 31 + std::hash<float>()(RadarRLCurrent_mA);
    result = result * 31 + std::hash<float>()(LidarBCurrent_mA);
    return result;
}

 } // namespace av
 } // namespace msg


CEREAL_BIND_TO_ARCHIVES(av::msg::PDU2CurrMonitoring)

