#include <interfaces/PDUStatusMessage/PDUStatusMessage.h>

#include <iterator>
#include <type_traits>

#include <cereal/cereal.hpp>
#include <cereal/archives/json.hpp>
#include <cereal/archives/binary.hpp>

#include <cereal/types/map.hpp>
#include <cereal/types/vector.hpp>
#include <cereal/types/array.hpp>
#include <cereal/types/string.hpp>
#include <cereal/types/complex.hpp>

#include <ODatum/IntrospectionArchive.h>
#include <ODatum/Hash.h>


 namespace av {
 namespace msg {


ParameterCompact::ParameterCompact() = default;

ParameterCompact::ParameterCompact(
    std::string parameterName_arg ,
    ParameterCompactStatus status_arg ,
    ParameterCompactType type_arg ,
    std::string value_arg 
    ):
    parameterName { std::move(parameterName_arg) } // NOLINT
      ,
    status { std::move(status_arg) } // NOLINT
      ,
    type { std::move(type_arg) } // NOLINT
      ,
    value { std::move(value_arg) } // NOLINT
      
    {
}

ParameterCompact::ParameterCompact(const ParameterCompact&) = default;

// NOLINTNEXTLINE
ParameterCompact::ParameterCompact(ParameterCompact&&) = default;

ParameterCompact& ParameterCompact::operator=(const ParameterCompact&) = default;

// NOLINTNEXTLINE
ParameterCompact& ParameterCompact::operator=(ParameterCompact&&) = default;

void ParameterCompact::toCsv(std::ostream& out) const
{
    out.precision(16);
    out << std::boolalpha;
    out << "\"";
        out << parameterName;
        out << "\"";out << ",";
    out << static_cast<uint32_t>(status);out << ",";
    out << static_cast<uint32_t>(type);out << ",";
    out << "\"";
        out << value;
        out << "\"";
    
}

std::stringstream ParameterCompact::toStringstream() const
{
    std::stringstream out;
    toCsv(out);
    return out;
}

template<class Archive>
    void ParameterCompact::serialize(Archive& ar, [[maybe_unused]] unsigned int version)
    {

        ar& CEREAL_NVP(parameterName);
        ar& CEREAL_NVP(status);
        ar& CEREAL_NVP(type);
        ar& CEREAL_NVP(value);
    }


template void ParameterCompact::serialize(cereal::BinaryInputArchive& ar, unsigned int version);
template void ParameterCompact::serialize(cereal::BinaryOutputArchive& ar, unsigned int version);
template void ParameterCompact::serialize(cereal::IntrospectionArchive& ar, unsigned int version);
template void ParameterCompact::serialize(cereal::JSONInputArchive& ar, unsigned int version);
template void ParameterCompact::serialize(cereal::JSONOutputArchive& ar, unsigned int version);
template void ParameterCompact::serialize(cereal::PortableBinaryInputArchive& ar, unsigned int version);
template void ParameterCompact::serialize(cereal::PortableBinaryOutputArchive& ar, unsigned int version);

std::ostream& operator<< (std::ostream& os, const ParameterCompact& parametercompact)
{
    parametercompact.toCsv(os);
    return os;
}

bool operator== (const ParameterCompact &lhs, const ParameterCompact &rhs)
{
    return (  lhs.parameterName== rhs.parameterName &&
     lhs.status== rhs.status &&
     lhs.type== rhs.type &&
     lhs.value== rhs.value 
    );
}

std::size_t ParameterCompact::hash() const noexcept
{
    std::size_t result = 17;
    result = result * 31 + std::hash<std::string>()(parameterName);
    result = result * 31 + std::hash<ParameterCompactStatus>()(status);
    result = result * 31 + std::hash<ParameterCompactType>()(type);
    result = result * 31 + std::hash<std::string>()(value);
    return result;
}

 } // namespace av
 } // namespace msg


CEREAL_BIND_TO_ARCHIVES(av::msg::ParameterCompact)




 namespace av {
 namespace msg {


PDUStatusMessage::PDUStatusMessage() = default;

PDUStatusMessage::PDUStatusMessage(
    std::string moduleName_arg ,
    std::vector<ParameterCompact> parameterList_arg ,
    uint64_t timestamp_arg 
    ):
    moduleName { std::move(moduleName_arg) } // NOLINT
      ,parameterList { std::move(parameterList_arg) } // NOLINT
      ,timestamp { std::move(timestamp_arg) } // NOLINT
      {
}

PDUStatusMessage::PDUStatusMessage(const PDUStatusMessage&) = default;

// NOLINTNEXTLINE
PDUStatusMessage::PDUStatusMessage(PDUStatusMessage&&) = default;

PDUStatusMessage& PDUStatusMessage::operator=(const PDUStatusMessage&) = default;

// NOLINTNEXTLINE
PDUStatusMessage& PDUStatusMessage::operator=(PDUStatusMessage&&) = default;

void PDUStatusMessage::toCsv(std::ostream& out) const
{
    out.precision(16);
    out << std::boolalpha;
    out << "\"";
        out << moduleName;
        out << "\"";out << ",";
    // Enclose sequence in double quotations and curly brackets to keep CSV line length uniform
        
        out << "\"{{";
        
        if (!parameterList.empty())
        {
            std::copy(parameterList.begin(), parameterList.end() - 1, std::ostream_iterator<ParameterCompact>(out, "},{"));
            std::copy(parameterList.end()-1, parameterList.end(), std::ostream_iterator<ParameterCompact>(out, ""));
        }
        out << "}}\"";out << ",";
    out << timestamp;
    
}

std::stringstream PDUStatusMessage::toStringstream() const
{
    std::stringstream out;
    toCsv(out);
    return out;
}

template<class Archive>
    void PDUStatusMessage::serialize(Archive& ar, [[maybe_unused]] unsigned int version)
    {

        ar& CEREAL_NVP(moduleName);
        ar& CEREAL_NVP(parameterList);
        ar& CEREAL_NVP(timestamp);
    }


template void PDUStatusMessage::serialize(cereal::BinaryInputArchive& ar, unsigned int version);
template void PDUStatusMessage::serialize(cereal::BinaryOutputArchive& ar, unsigned int version);
template void PDUStatusMessage::serialize(cereal::IntrospectionArchive& ar, unsigned int version);
template void PDUStatusMessage::serialize(cereal::JSONInputArchive& ar, unsigned int version);
template void PDUStatusMessage::serialize(cereal::JSONOutputArchive& ar, unsigned int version);
template void PDUStatusMessage::serialize(cereal::PortableBinaryInputArchive& ar, unsigned int version);
template void PDUStatusMessage::serialize(cereal::PortableBinaryOutputArchive& ar, unsigned int version);

std::ostream& operator<< (std::ostream& os, const PDUStatusMessage& pdustatusmessage)
{
    pdustatusmessage.toCsv(os);
    return os;
}

bool operator== (const PDUStatusMessage &lhs, const PDUStatusMessage &rhs)
{
    return (  lhs.moduleName== rhs.moduleName &&
     lhs.parameterList== rhs.parameterList &&
     lhs.timestamp== rhs.timestamp 
    );
}

std::size_t PDUStatusMessage::hash() const noexcept
{
    std::size_t result = 17;
    result = result * 31 + std::hash<std::string>()(moduleName);
    result = result * 31 + hashVector(parameterList);
    result = result * 31 + std::hash<uint64_t>()(timestamp);
    return result;
}

 } // namespace av
 } // namespace msg


CEREAL_BIND_TO_ARCHIVES(av::msg::PDUStatusMessage)

