#include <interfaces/Parameter/Parameter.h>

#include <iterator>
#include <type_traits>

#include <cereal/cereal.hpp>
#include <cereal/archives/json.hpp>
#include <cereal/archives/binary.hpp>

#include <cereal/types/map.hpp>
#include <cereal/types/vector.hpp>
#include <cereal/types/array.hpp>
#include <cereal/types/string.hpp>
#include <cereal/types/complex.hpp>

#include <ODatum/IntrospectionArchive.h>
#include <ODatum/Hash.h>





 namespace av {
 namespace msg {


Parameter::Parameter() = default;

Parameter::Parameter(
    int64_t timestamp_arg ,
    std::string moduleName_arg ,
    std::string faultName_arg ,
    std::string parameterName_arg ,
    ParameterStatus status_arg ,
    ParameterType type_arg ,
    std::string measured_value_arg ,
    std::string threshold_value_arg ,
    bool measuredValueAvailable_arg ,
    bool thresholdValueAvailable_arg 
    ):
    timestamp { std::move(timestamp_arg) } // NOLINT
      ,moduleName { std::move(moduleName_arg) } // NOLINT
      ,faultName { std::move(faultName_arg) } // NOLINT
      ,parameterName { std::move(parameterName_arg) } // NOLINT
      ,status { std::move(status_arg) } // NOLINT
      ,type { std::move(type_arg) } // NOLINT
      ,measured_value { std::move(measured_value_arg) } // NOLINT
      ,threshold_value { std::move(threshold_value_arg) } // NOLINT
      ,measuredValueAvailable { std::move(measuredValueAvailable_arg) } // NOLINT
      ,thresholdValueAvailable { std::move(thresholdValueAvailable_arg) } // NOLINT
      {
}

Parameter::Parameter(const Parameter&) = default;

// NOLINTNEXTLINE
Parameter::Parameter(Parameter&&) = default;

Parameter& Parameter::operator=(const Parameter&) = default;

// NOLINTNEXTLINE
Parameter& Parameter::operator=(Parameter&&) = default;

void Parameter::toCsv(std::ostream& out) const
{
    out.precision(16);
    out << std::boolalpha;
    out << timestamp;out << ",";
    out << "\"";
        out << moduleName;
        out << "\"";out << ",";
    out << "\"";
        out << faultName;
        out << "\"";out << ",";
    out << "\"";
        out << parameterName;
        out << "\"";out << ",";
    out << static_cast<uint32_t>(status);out << ",";
    out << static_cast<uint32_t>(type);out << ",";
    out << "\"";
        out << measured_value;
        out << "\"";out << ",";
    out << "\"";
        out << threshold_value;
        out << "\"";out << ",";
    out << measuredValueAvailable;out << ",";
    out << thresholdValueAvailable;
    
}

std::stringstream Parameter::toStringstream() const
{
    std::stringstream out;
    toCsv(out);
    return out;
}

template<class Archive>
    void Parameter::serialize(Archive& ar, unsigned int version)
{
    if (version == 1)
    {   
        uint64_t uTimestamp{0};
        ar(cereal::make_nvp("timestamp", uTimestamp));
        timestamp = static_cast<int64_t>(uTimestamp);
        ar(CEREAL_NVP(moduleName));
        ar& CEREAL_NVP(parameterName);
        ar& CEREAL_NVP(status);
        ar& CEREAL_NVP(type);
        ar(cereal::make_nvp("value", measured_value));
    }
    else{ 
      if (version == 2)
      {
          uint64_t uTimestamp{0};
          ar(cereal::make_nvp("timestamp", uTimestamp));
          timestamp = static_cast<int64_t>(uTimestamp);
      }
      else
      {
          ar(CEREAL_NVP(timestamp));
      }
      ar(CEREAL_NVP(moduleName));
      ar(CEREAL_NVP(faultName));
      if (version >= 4) 
      {
        ar(CEREAL_NVP(parameterName));
      }
      ar& CEREAL_NVP(status);
      ar& CEREAL_NVP(type);
      ar(CEREAL_NVP(measured_value));
      ar(CEREAL_NVP(threshold_value));
      if (version >= 4) 
      {
        ar(CEREAL_NVP(measuredValueAvailable));
        ar(CEREAL_NVP(thresholdValueAvailable));
      }
    }
}



template void Parameter::serialize(cereal::BinaryInputArchive& ar, unsigned int version);
template void Parameter::serialize(cereal::BinaryOutputArchive& ar, unsigned int version);
template void Parameter::serialize(cereal::IntrospectionArchive& ar, unsigned int version);
template void Parameter::serialize(cereal::JSONInputArchive& ar, unsigned int version);
template void Parameter::serialize(cereal::JSONOutputArchive& ar, unsigned int version);
template void Parameter::serialize(cereal::PortableBinaryInputArchive& ar, unsigned int version);
template void Parameter::serialize(cereal::PortableBinaryOutputArchive& ar, unsigned int version);

std::ostream& operator<< (std::ostream& os, const Parameter& parameter)
{
    parameter.toCsv(os);
    return os;
}

bool operator== (const Parameter &lhs, const Parameter &rhs)
{
    return (  lhs.timestamp== rhs.timestamp &&
     lhs.moduleName== rhs.moduleName &&
     lhs.faultName== rhs.faultName &&
     lhs.parameterName== rhs.parameterName &&
     lhs.status== rhs.status &&
     lhs.type== rhs.type &&
     lhs.measured_value== rhs.measured_value &&
     lhs.threshold_value== rhs.threshold_value &&
     lhs.measuredValueAvailable== rhs.measuredValueAvailable &&
     lhs.thresholdValueAvailable== rhs.thresholdValueAvailable 
    );
}

std::size_t Parameter::hash() const noexcept
{
    std::size_t result = 17;
    result = result * 31 + std::hash<int64_t>()(timestamp);
    result = result * 31 + std::hash<std::string>()(moduleName);
    result = result * 31 + std::hash<std::string>()(faultName);
    result = result * 31 + std::hash<std::string>()(parameterName);
    result = result * 31 + std::hash<ParameterStatus>()(status);
    result = result * 31 + std::hash<ParameterType>()(type);
    result = result * 31 + std::hash<std::string>()(measured_value);
    result = result * 31 + std::hash<std::string>()(threshold_value);
    result = result * 31 + std::hash<bool>()(measuredValueAvailable);
    result = result * 31 + std::hash<bool>()(thresholdValueAvailable);
    return result;
}

 } // namespace av
 } // namespace msg


CEREAL_BIND_TO_ARCHIVES(av::msg::Parameter)

