#include <interfaces/ParameterMessage/ParameterMessage.h>

#include <iterator>
#include <type_traits>

#include <cereal/cereal.hpp>
#include <cereal/archives/json.hpp>
#include <cereal/archives/binary.hpp>

#include <cereal/types/map.hpp>
#include <cereal/types/vector.hpp>
#include <cereal/types/array.hpp>
#include <cereal/types/string.hpp>
#include <cereal/types/complex.hpp>

#include <ODatum/IntrospectionArchive.h>
#include <ODatum/Hash.h>





 namespace av {
 namespace msg {


ParameterMessage::ParameterMessage() = default;

ParameterMessage::ParameterMessage(
    TimePoint timestamp_arg ,
    std::vector<Parameter> parameters_arg 
    ):
    timestamp { std::move(timestamp_arg) } // NOLINT
      ,parameters { std::move(parameters_arg) } // NOLINT
      {
}

ParameterMessage::ParameterMessage(const ParameterMessage&) = default;

// NOLINTNEXTLINE
ParameterMessage::ParameterMessage(ParameterMessage&&) = default;

ParameterMessage& ParameterMessage::operator=(const ParameterMessage&) = default;

// NOLINTNEXTLINE
ParameterMessage& ParameterMessage::operator=(ParameterMessage&&) = default;

void ParameterMessage::toCsv(std::ostream& out) const
{
    out.precision(16);
    out << std::boolalpha;
    out << timestamp;out << ",";
    // Enclose sequence in double quotations and curly brackets to keep CSV line length uniform
        
        out << "\"{{";
        
        if (!parameters.empty())
        {
            std::copy(parameters.begin(), parameters.end() - 1, std::ostream_iterator<Parameter>(out, "},{"));
            std::copy(parameters.end()-1, parameters.end(), std::ostream_iterator<Parameter>(out, ""));
        }
        out << "}}\"";
    
}

std::stringstream ParameterMessage::toStringstream() const
{
    std::stringstream out;
    toCsv(out);
    return out;
}

template<class Archive>
    void ParameterMessage::serialize(Archive& ar, [[maybe_unused]] unsigned int version)
    {

        ar& CEREAL_NVP(timestamp);
        ar& CEREAL_NVP(parameters);
    }


template void ParameterMessage::serialize(cereal::BinaryInputArchive& ar, unsigned int version);
template void ParameterMessage::serialize(cereal::BinaryOutputArchive& ar, unsigned int version);
template void ParameterMessage::serialize(cereal::IntrospectionArchive& ar, unsigned int version);
template void ParameterMessage::serialize(cereal::JSONInputArchive& ar, unsigned int version);
template void ParameterMessage::serialize(cereal::JSONOutputArchive& ar, unsigned int version);
template void ParameterMessage::serialize(cereal::PortableBinaryInputArchive& ar, unsigned int version);
template void ParameterMessage::serialize(cereal::PortableBinaryOutputArchive& ar, unsigned int version);

std::ostream& operator<< (std::ostream& os, const ParameterMessage& parametermessage)
{
    parametermessage.toCsv(os);
    return os;
}

bool operator== (const ParameterMessage &lhs, const ParameterMessage &rhs)
{
    return (  lhs.timestamp== rhs.timestamp &&
     lhs.parameters== rhs.parameters 
    );
}

std::size_t ParameterMessage::hash() const noexcept
{
    std::size_t result = 17;
    result = result * 31 + 0;
    result = result * 31 + hashVector(parameters);
    return result;
}

 } // namespace av
 } // namespace msg


CEREAL_BIND_TO_ARCHIVES(av::msg::ParameterMessage)

