#include <interfaces/PathInfoWithDirection/PathInfoWithDirection.h>

#include <iterator>
#include <type_traits>

#include <cereal/cereal.hpp>
#include <cereal/archives/json.hpp>
#include <cereal/archives/binary.hpp>

#include <cereal/types/map.hpp>
#include <cereal/types/vector.hpp>
#include <cereal/types/array.hpp>
#include <cereal/types/string.hpp>
#include <cereal/types/complex.hpp>

#include <ODatum/IntrospectionArchive.h>
#include <ODatum/Hash.h>


 namespace av {
 namespace msg {
 namespace plan_ctrl {


PathStateWithDirection::PathStateWithDirection() = default;

PathStateWithDirection::PathStateWithDirection(
    double s_m_arg ,
    double x_m_arg ,
    double y_m_arg ,
    double heading_rad_arg ,
    double curvature_k_arg ,
    double delta_rad_arg ,
    double delta_rate_radps_arg ,
    double direction_arg 
    ):
    s_m { std::move(s_m_arg) } // NOLINT
      ,
    x_m { std::move(x_m_arg) } // NOLINT
      ,
    y_m { std::move(y_m_arg) } // NOLINT
      ,
    heading_rad { std::move(heading_rad_arg) } // NOLINT
      ,
    curvature_k { std::move(curvature_k_arg) } // NOLINT
      ,
    delta_rad { std::move(delta_rad_arg) } // NOLINT
      ,
    delta_rate_radps { std::move(delta_rate_radps_arg) } // NOLINT
      ,
    direction { std::move(direction_arg) } // NOLINT
      
    {
}

PathStateWithDirection::PathStateWithDirection(const PathStateWithDirection&) = default;

// NOLINTNEXTLINE
PathStateWithDirection::PathStateWithDirection(PathStateWithDirection&&) = default;

PathStateWithDirection& PathStateWithDirection::operator=(const PathStateWithDirection&) = default;

// NOLINTNEXTLINE
PathStateWithDirection& PathStateWithDirection::operator=(PathStateWithDirection&&) = default;

void PathStateWithDirection::toCsv(std::ostream& out) const
{
    out.precision(16);
    out << std::boolalpha;
    out << s_m;out << ",";
    out << x_m;out << ",";
    out << y_m;out << ",";
    out << heading_rad;out << ",";
    out << curvature_k;out << ",";
    out << delta_rad;out << ",";
    out << delta_rate_radps;out << ",";
    out << direction;
    
}

std::stringstream PathStateWithDirection::toStringstream() const
{
    std::stringstream out;
    toCsv(out);
    return out;
}

template<class Archive>
    void PathStateWithDirection::serialize(Archive& ar, [[maybe_unused]] unsigned int version)
    {

        ar& CEREAL_NVP(s_m);
        ar& CEREAL_NVP(x_m);
        ar& CEREAL_NVP(y_m);
        ar& CEREAL_NVP(heading_rad);
        ar& CEREAL_NVP(curvature_k);
        ar& CEREAL_NVP(delta_rad);
        ar& CEREAL_NVP(delta_rate_radps);
        ar& CEREAL_NVP(direction);
    }


template void PathStateWithDirection::serialize(cereal::BinaryInputArchive& ar, unsigned int version);
template void PathStateWithDirection::serialize(cereal::BinaryOutputArchive& ar, unsigned int version);
template void PathStateWithDirection::serialize(cereal::IntrospectionArchive& ar, unsigned int version);
template void PathStateWithDirection::serialize(cereal::JSONInputArchive& ar, unsigned int version);
template void PathStateWithDirection::serialize(cereal::JSONOutputArchive& ar, unsigned int version);
template void PathStateWithDirection::serialize(cereal::PortableBinaryInputArchive& ar, unsigned int version);
template void PathStateWithDirection::serialize(cereal::PortableBinaryOutputArchive& ar, unsigned int version);

std::ostream& operator<< (std::ostream& os, const PathStateWithDirection& pathstatewithdirection)
{
    pathstatewithdirection.toCsv(os);
    return os;
}

bool operator== (const PathStateWithDirection &lhs, const PathStateWithDirection &rhs)
{
    return (  lhs.s_m== rhs.s_m &&
     lhs.x_m== rhs.x_m &&
     lhs.y_m== rhs.y_m &&
     lhs.heading_rad== rhs.heading_rad &&
     lhs.curvature_k== rhs.curvature_k &&
     lhs.delta_rad== rhs.delta_rad &&
     lhs.delta_rate_radps== rhs.delta_rate_radps &&
     lhs.direction== rhs.direction 
    );
}

std::size_t PathStateWithDirection::hash() const noexcept
{
    std::size_t result = 17;
    result = result * 31 + std::hash<double>()(s_m);
    result = result * 31 + std::hash<double>()(x_m);
    result = result * 31 + std::hash<double>()(y_m);
    result = result * 31 + std::hash<double>()(heading_rad);
    result = result * 31 + std::hash<double>()(curvature_k);
    result = result * 31 + std::hash<double>()(delta_rad);
    result = result * 31 + std::hash<double>()(delta_rate_radps);
    result = result * 31 + std::hash<double>()(direction);
    return result;
}

 } // namespace av
 } // namespace msg
 } // namespace plan_ctrl


CEREAL_BIND_TO_ARCHIVES(av::msg::plan_ctrl::PathStateWithDirection)




 namespace av {
 namespace msg {
 namespace plan_ctrl {


PathInfoWithDirection::PathInfoWithDirection() = default;

PathInfoWithDirection::PathInfoWithDirection(
    std::vector<PathStateWithDirection> pathStatesWithDirection_arg ,
    int64_t timestamp_us_arg ,
    int64_t counter_arg 
    ):
    pathStatesWithDirection { std::move(pathStatesWithDirection_arg) } // NOLINT
      ,timestamp_us { std::move(timestamp_us_arg) } // NOLINT
      ,counter { std::move(counter_arg) } // NOLINT
      {
}

PathInfoWithDirection::PathInfoWithDirection(const PathInfoWithDirection&) = default;

// NOLINTNEXTLINE
PathInfoWithDirection::PathInfoWithDirection(PathInfoWithDirection&&) = default;

PathInfoWithDirection& PathInfoWithDirection::operator=(const PathInfoWithDirection&) = default;

// NOLINTNEXTLINE
PathInfoWithDirection& PathInfoWithDirection::operator=(PathInfoWithDirection&&) = default;

void PathInfoWithDirection::toCsv(std::ostream& out) const
{
    out.precision(16);
    out << std::boolalpha;
    // Enclose sequence in double quotations and curly brackets to keep CSV line length uniform
        
        out << "\"{{";
        
        if (!pathStatesWithDirection.empty())
        {
            std::copy(pathStatesWithDirection.begin(), pathStatesWithDirection.end() - 1, std::ostream_iterator<PathStateWithDirection>(out, "},{"));
            std::copy(pathStatesWithDirection.end()-1, pathStatesWithDirection.end(), std::ostream_iterator<PathStateWithDirection>(out, ""));
        }
        out << "}}\"";out << ",";
    out << timestamp_us;out << ",";
    out << counter;
    
}

std::stringstream PathInfoWithDirection::toStringstream() const
{
    std::stringstream out;
    toCsv(out);
    return out;
}

template<class Archive>
    void PathInfoWithDirection::serialize(Archive& ar, [[maybe_unused]] unsigned int version)
    {

        ar& CEREAL_NVP(pathStatesWithDirection);
        ar& CEREAL_NVP(timestamp_us);
        ar& CEREAL_NVP(counter);
    }


template void PathInfoWithDirection::serialize(cereal::BinaryInputArchive& ar, unsigned int version);
template void PathInfoWithDirection::serialize(cereal::BinaryOutputArchive& ar, unsigned int version);
template void PathInfoWithDirection::serialize(cereal::IntrospectionArchive& ar, unsigned int version);
template void PathInfoWithDirection::serialize(cereal::JSONInputArchive& ar, unsigned int version);
template void PathInfoWithDirection::serialize(cereal::JSONOutputArchive& ar, unsigned int version);
template void PathInfoWithDirection::serialize(cereal::PortableBinaryInputArchive& ar, unsigned int version);
template void PathInfoWithDirection::serialize(cereal::PortableBinaryOutputArchive& ar, unsigned int version);

std::ostream& operator<< (std::ostream& os, const PathInfoWithDirection& pathinfowithdirection)
{
    pathinfowithdirection.toCsv(os);
    return os;
}

bool operator== (const PathInfoWithDirection &lhs, const PathInfoWithDirection &rhs)
{
    return (  lhs.pathStatesWithDirection== rhs.pathStatesWithDirection &&
     lhs.timestamp_us== rhs.timestamp_us &&
     lhs.counter== rhs.counter 
    );
}

std::size_t PathInfoWithDirection::hash() const noexcept
{
    std::size_t result = 17;
    result = result * 31 + hashVector(pathStatesWithDirection);
    result = result * 31 + std::hash<int64_t>()(timestamp_us);
    result = result * 31 + std::hash<int64_t>()(counter);
    return result;
}

 } // namespace av
 } // namespace msg
 } // namespace plan_ctrl


CEREAL_BIND_TO_ARCHIVES(av::msg::plan_ctrl::PathInfoWithDirection)

