#include <interfaces/PathTrackerTaskChannel/PathTrackerTaskChannel.h>

#include <iterator>
#include <type_traits>

#include <cereal/cereal.hpp>
#include <cereal/archives/json.hpp>
#include <cereal/archives/binary.hpp>

#include <cereal/types/map.hpp>
#include <cereal/types/vector.hpp>
#include <cereal/types/array.hpp>
#include <cereal/types/string.hpp>
#include <cereal/types/complex.hpp>

#include <ODatum/IntrospectionArchive.h>
#include <ODatum/Hash.h>


 namespace av {
 namespace msg {


PathTrackerVehicleState::PathTrackerVehicleState() = default;

PathTrackerVehicleState::PathTrackerVehicleState(
    TimePoint timestamp_arg ,
    double refpoint_offset_arg ,
    double pos_x_arg ,
    double pos_y_arg ,
    double pos_theta_arg ,
    double ref_pos_x_arg ,
    double ref_pos_y_arg ,
    double ref_pos_theta_arg ,
    std::array<double,3> vel_arg ,
    std::array<double,3> acc_arg ,
    std::array<double,3> rotation_arg ,
    std::array<double,3> rotationRates_arg ,
    double curvature_arg ,
    double roadWheelAngle_arg ,
    bool isAutonomous_arg 
    ):
    timestamp { std::move(timestamp_arg) } // NOLINT
      ,
    refpoint_offset { std::move(refpoint_offset_arg) } // NOLINT
      ,
    pos_x { std::move(pos_x_arg) } // NOLINT
      ,
    pos_y { std::move(pos_y_arg) } // NOLINT
      ,
    pos_theta { std::move(pos_theta_arg) } // NOLINT
      ,
    ref_pos_x { std::move(ref_pos_x_arg) } // NOLINT
      ,
    ref_pos_y { std::move(ref_pos_y_arg) } // NOLINT
      ,
    ref_pos_theta { std::move(ref_pos_theta_arg) } // NOLINT
      ,
    vel { std::move(vel_arg) } // NOLINT
      ,
    acc { std::move(acc_arg) } // NOLINT
      ,
    rotation { std::move(rotation_arg) } // NOLINT
      ,
    rotationRates { std::move(rotationRates_arg) } // NOLINT
      ,
    curvature { std::move(curvature_arg) } // NOLINT
      ,
    roadWheelAngle { std::move(roadWheelAngle_arg) } // NOLINT
      ,
    isAutonomous { std::move(isAutonomous_arg) } // NOLINT
      
    {
}

PathTrackerVehicleState::PathTrackerVehicleState(const PathTrackerVehicleState&) = default;

// NOLINTNEXTLINE
PathTrackerVehicleState::PathTrackerVehicleState(PathTrackerVehicleState&&) = default;

PathTrackerVehicleState& PathTrackerVehicleState::operator=(const PathTrackerVehicleState&) = default;

// NOLINTNEXTLINE
PathTrackerVehicleState& PathTrackerVehicleState::operator=(PathTrackerVehicleState&&) = default;

void PathTrackerVehicleState::toCsv(std::ostream& out) const
{
    out.precision(16);
    out << std::boolalpha;
    out << timestamp;out << ",";
    out << refpoint_offset;out << ",";
    out << pos_x;out << ",";
    out << pos_y;out << ",";
    out << pos_theta;out << ",";
    out << ref_pos_x;out << ",";
    out << ref_pos_y;out << ",";
    out << ref_pos_theta;out << ",";
    // Enclose array in double quotations and curly brackets to keep CSV line length uniform
        
        out << "\"{{";
        
        if (!vel.empty())
        {
            std::copy(vel.begin(), vel.end() - 1, std::ostream_iterator<double>(out, "},{"));
            out << vel.back();
        }
        out << "}}\"";out << ",";
    // Enclose array in double quotations and curly brackets to keep CSV line length uniform
        
        out << "\"{{";
        
        if (!acc.empty())
        {
            std::copy(acc.begin(), acc.end() - 1, std::ostream_iterator<double>(out, "},{"));
            out << acc.back();
        }
        out << "}}\"";out << ",";
    // Enclose array in double quotations and curly brackets to keep CSV line length uniform
        
        out << "\"{{";
        
        if (!rotation.empty())
        {
            std::copy(rotation.begin(), rotation.end() - 1, std::ostream_iterator<double>(out, "},{"));
            out << rotation.back();
        }
        out << "}}\"";out << ",";
    // Enclose array in double quotations and curly brackets to keep CSV line length uniform
        
        out << "\"{{";
        
        if (!rotationRates.empty())
        {
            std::copy(rotationRates.begin(), rotationRates.end() - 1, std::ostream_iterator<double>(out, "},{"));
            out << rotationRates.back();
        }
        out << "}}\"";out << ",";
    out << curvature;out << ",";
    out << roadWheelAngle;out << ",";
    out << isAutonomous;
    
}

std::stringstream PathTrackerVehicleState::toStringstream() const
{
    std::stringstream out;
    toCsv(out);
    return out;
}

template<class Archive>
    void PathTrackerVehicleState::serialize(Archive& ar, unsigned int version)
{
    if (version >= 1) {
        ar& CEREAL_NVP(timestamp);
        ar& CEREAL_NVP(refpoint_offset);
        ar& CEREAL_NVP(pos_x) & CEREAL_NVP(pos_y) & CEREAL_NVP(pos_theta);
        ar& CEREAL_NVP(ref_pos_x) & CEREAL_NVP(ref_pos_y) & CEREAL_NVP(ref_pos_theta);
        ar& CEREAL_NVP(vel);
        ar& CEREAL_NVP(acc);
        ar& CEREAL_NVP(rotation);
        ar& CEREAL_NVP(rotationRates);
        ar& CEREAL_NVP(curvature);
        ar& CEREAL_NVP(roadWheelAngle);
    }
    if (version >= 2) {
        ar& CEREAL_NVP(isAutonomous);
    }
}



template void PathTrackerVehicleState::serialize(cereal::BinaryInputArchive& ar, unsigned int version);
template void PathTrackerVehicleState::serialize(cereal::BinaryOutputArchive& ar, unsigned int version);
template void PathTrackerVehicleState::serialize(cereal::IntrospectionArchive& ar, unsigned int version);
template void PathTrackerVehicleState::serialize(cereal::JSONInputArchive& ar, unsigned int version);
template void PathTrackerVehicleState::serialize(cereal::JSONOutputArchive& ar, unsigned int version);
template void PathTrackerVehicleState::serialize(cereal::PortableBinaryInputArchive& ar, unsigned int version);
template void PathTrackerVehicleState::serialize(cereal::PortableBinaryOutputArchive& ar, unsigned int version);

std::ostream& operator<< (std::ostream& os, const PathTrackerVehicleState& pathtrackervehiclestate)
{
    pathtrackervehiclestate.toCsv(os);
    return os;
}

bool operator== (const PathTrackerVehicleState &lhs, const PathTrackerVehicleState &rhs)
{
    return (  lhs.timestamp== rhs.timestamp &&
     lhs.refpoint_offset== rhs.refpoint_offset &&
     lhs.pos_x== rhs.pos_x &&
     lhs.pos_y== rhs.pos_y &&
     lhs.pos_theta== rhs.pos_theta &&
     lhs.ref_pos_x== rhs.ref_pos_x &&
     lhs.ref_pos_y== rhs.ref_pos_y &&
     lhs.ref_pos_theta== rhs.ref_pos_theta &&
     lhs.vel== rhs.vel &&
     lhs.acc== rhs.acc &&
     lhs.rotation== rhs.rotation &&
     lhs.rotationRates== rhs.rotationRates &&
     lhs.curvature== rhs.curvature &&
     lhs.roadWheelAngle== rhs.roadWheelAngle &&
     lhs.isAutonomous== rhs.isAutonomous 
    );
}

std::size_t PathTrackerVehicleState::hash() const noexcept
{
    std::size_t result = 17;
    result = result * 31 + 0;
    result = result * 31 + std::hash<double>()(refpoint_offset);
    result = result * 31 + std::hash<double>()(pos_x);
    result = result * 31 + std::hash<double>()(pos_y);
    result = result * 31 + std::hash<double>()(pos_theta);
    result = result * 31 + std::hash<double>()(ref_pos_x);
    result = result * 31 + std::hash<double>()(ref_pos_y);
    result = result * 31 + std::hash<double>()(ref_pos_theta);
    result = result * 31 + hashVector(vel);
    result = result * 31 + hashVector(acc);
    result = result * 31 + hashVector(rotation);
    result = result * 31 + hashVector(rotationRates);
    result = result * 31 + std::hash<double>()(curvature);
    result = result * 31 + std::hash<double>()(roadWheelAngle);
    result = result * 31 + std::hash<bool>()(isAutonomous);
    return result;
}

 } // namespace av
 } // namespace msg


CEREAL_BIND_TO_ARCHIVES(av::msg::PathTrackerVehicleState)

 namespace av {
 namespace msg {


AutoReadyCommand::AutoReadyCommand() = default;

AutoReadyCommand::AutoReadyCommand(
    double timestamp_arg ,
    bool isReady_arg 
    ):
    timestamp { std::move(timestamp_arg) } // NOLINT
      ,
    isReady { std::move(isReady_arg) } // NOLINT
      
    {
}

AutoReadyCommand::AutoReadyCommand(const AutoReadyCommand&) = default;

// NOLINTNEXTLINE
AutoReadyCommand::AutoReadyCommand(AutoReadyCommand&&) = default;

AutoReadyCommand& AutoReadyCommand::operator=(const AutoReadyCommand&) = default;

// NOLINTNEXTLINE
AutoReadyCommand& AutoReadyCommand::operator=(AutoReadyCommand&&) = default;

void AutoReadyCommand::toCsv(std::ostream& out) const
{
    out.precision(16);
    out << std::boolalpha;
    out << timestamp;out << ",";
    out << isReady;
    
}

std::stringstream AutoReadyCommand::toStringstream() const
{
    std::stringstream out;
    toCsv(out);
    return out;
}

template<class Archive>
    void AutoReadyCommand::serialize(Archive& ar, [[maybe_unused]] unsigned int version)
    {

        ar& CEREAL_NVP(timestamp);
        ar& CEREAL_NVP(isReady);
    }


template void AutoReadyCommand::serialize(cereal::BinaryInputArchive& ar, unsigned int version);
template void AutoReadyCommand::serialize(cereal::BinaryOutputArchive& ar, unsigned int version);
template void AutoReadyCommand::serialize(cereal::IntrospectionArchive& ar, unsigned int version);
template void AutoReadyCommand::serialize(cereal::JSONInputArchive& ar, unsigned int version);
template void AutoReadyCommand::serialize(cereal::JSONOutputArchive& ar, unsigned int version);
template void AutoReadyCommand::serialize(cereal::PortableBinaryInputArchive& ar, unsigned int version);
template void AutoReadyCommand::serialize(cereal::PortableBinaryOutputArchive& ar, unsigned int version);

std::ostream& operator<< (std::ostream& os, const AutoReadyCommand& autoreadycommand)
{
    autoreadycommand.toCsv(os);
    return os;
}

bool operator== (const AutoReadyCommand &lhs, const AutoReadyCommand &rhs)
{
    return (  lhs.timestamp== rhs.timestamp &&
     lhs.isReady== rhs.isReady 
    );
}

std::size_t AutoReadyCommand::hash() const noexcept
{
    std::size_t result = 17;
    result = result * 31 + std::hash<double>()(timestamp);
    result = result * 31 + std::hash<bool>()(isReady);
    return result;
}

 } // namespace av
 } // namespace msg


CEREAL_BIND_TO_ARCHIVES(av::msg::AutoReadyCommand)

 namespace av {
 namespace msg {


OptimizerErrorMessages::OptimizerErrorMessages() = default;

OptimizerErrorMessages::OptimizerErrorMessages(
    bool pathOptimizerSolverFailure_arg ,
    bool speedOptimizerSolverFailure_arg ,
    bool safeStopTriggered_arg 
    ):
    pathOptimizerSolverFailure { std::move(pathOptimizerSolverFailure_arg) } // NOLINT
      ,
    speedOptimizerSolverFailure { std::move(speedOptimizerSolverFailure_arg) } // NOLINT
      ,
    safeStopTriggered { std::move(safeStopTriggered_arg) } // NOLINT
      
    {
}

OptimizerErrorMessages::OptimizerErrorMessages(const OptimizerErrorMessages&) = default;

// NOLINTNEXTLINE
OptimizerErrorMessages::OptimizerErrorMessages(OptimizerErrorMessages&&) = default;

OptimizerErrorMessages& OptimizerErrorMessages::operator=(const OptimizerErrorMessages&) = default;

// NOLINTNEXTLINE
OptimizerErrorMessages& OptimizerErrorMessages::operator=(OptimizerErrorMessages&&) = default;

void OptimizerErrorMessages::toCsv(std::ostream& out) const
{
    out.precision(16);
    out << std::boolalpha;
    out << pathOptimizerSolverFailure;out << ",";
    out << speedOptimizerSolverFailure;out << ",";
    out << safeStopTriggered;
    
}

std::stringstream OptimizerErrorMessages::toStringstream() const
{
    std::stringstream out;
    toCsv(out);
    return out;
}

template<class Archive>
    void OptimizerErrorMessages::serialize(Archive& ar, [[maybe_unused]] unsigned int version)
    {

        ar& CEREAL_NVP(pathOptimizerSolverFailure);
        ar& CEREAL_NVP(speedOptimizerSolverFailure);
        ar& CEREAL_NVP(safeStopTriggered);
    }


template void OptimizerErrorMessages::serialize(cereal::BinaryInputArchive& ar, unsigned int version);
template void OptimizerErrorMessages::serialize(cereal::BinaryOutputArchive& ar, unsigned int version);
template void OptimizerErrorMessages::serialize(cereal::IntrospectionArchive& ar, unsigned int version);
template void OptimizerErrorMessages::serialize(cereal::JSONInputArchive& ar, unsigned int version);
template void OptimizerErrorMessages::serialize(cereal::JSONOutputArchive& ar, unsigned int version);
template void OptimizerErrorMessages::serialize(cereal::PortableBinaryInputArchive& ar, unsigned int version);
template void OptimizerErrorMessages::serialize(cereal::PortableBinaryOutputArchive& ar, unsigned int version);

std::ostream& operator<< (std::ostream& os, const OptimizerErrorMessages& optimizererrormessages)
{
    optimizererrormessages.toCsv(os);
    return os;
}

bool operator== (const OptimizerErrorMessages &lhs, const OptimizerErrorMessages &rhs)
{
    return (  lhs.pathOptimizerSolverFailure== rhs.pathOptimizerSolverFailure &&
     lhs.speedOptimizerSolverFailure== rhs.speedOptimizerSolverFailure &&
     lhs.safeStopTriggered== rhs.safeStopTriggered 
    );
}

std::size_t OptimizerErrorMessages::hash() const noexcept
{
    std::size_t result = 17;
    result = result * 31 + std::hash<bool>()(pathOptimizerSolverFailure);
    result = result * 31 + std::hash<bool>()(speedOptimizerSolverFailure);
    result = result * 31 + std::hash<bool>()(safeStopTriggered);
    return result;
}

 } // namespace av
 } // namespace msg


CEREAL_BIND_TO_ARCHIVES(av::msg::OptimizerErrorMessages)

 namespace av {
 namespace msg {


PathTrackerPreviewState::PathTrackerPreviewState() = default;

PathTrackerPreviewState::PathTrackerPreviewState(
    double time_s_arg ,
    double curvature_k_arg ,
    double velocity_mps_arg ,
    double minCrossTrackConstraint_m_arg ,
    double maxCrossTrackConstraint_m_arg ,
    double xPosition_m_arg ,
    double yPosition_m_arg ,
    double heading_rad_arg 
    ):
    time_s { std::move(time_s_arg) } // NOLINT
      ,
    curvature_k { std::move(curvature_k_arg) } // NOLINT
      ,
    velocity_mps { std::move(velocity_mps_arg) } // NOLINT
      ,
    minCrossTrackConstraint_m { std::move(minCrossTrackConstraint_m_arg) } // NOLINT
      ,
    maxCrossTrackConstraint_m { std::move(maxCrossTrackConstraint_m_arg) } // NOLINT
      ,
    xPosition_m { std::move(xPosition_m_arg) } // NOLINT
      ,
    yPosition_m { std::move(yPosition_m_arg) } // NOLINT
      ,
    heading_rad { std::move(heading_rad_arg) } // NOLINT
      
    {
}

PathTrackerPreviewState::PathTrackerPreviewState(const PathTrackerPreviewState&) = default;

// NOLINTNEXTLINE
PathTrackerPreviewState::PathTrackerPreviewState(PathTrackerPreviewState&&) = default;

PathTrackerPreviewState& PathTrackerPreviewState::operator=(const PathTrackerPreviewState&) = default;

// NOLINTNEXTLINE
PathTrackerPreviewState& PathTrackerPreviewState::operator=(PathTrackerPreviewState&&) = default;

void PathTrackerPreviewState::toCsv(std::ostream& out) const
{
    out.precision(16);
    out << std::boolalpha;
    out << time_s;out << ",";
    out << curvature_k;out << ",";
    out << velocity_mps;out << ",";
    out << minCrossTrackConstraint_m;out << ",";
    out << maxCrossTrackConstraint_m;out << ",";
    out << xPosition_m;out << ",";
    out << yPosition_m;out << ",";
    out << heading_rad;
    
}

std::stringstream PathTrackerPreviewState::toStringstream() const
{
    std::stringstream out;
    toCsv(out);
    return out;
}

template<class Archive>
    void PathTrackerPreviewState::serialize(Archive& ar, unsigned int version)
{
    if (version >= 1) {
        ar& CEREAL_NVP(time_s);
        ar& CEREAL_NVP(curvature_k);
        ar& CEREAL_NVP(velocity_mps);
        ar& CEREAL_NVP(minCrossTrackConstraint_m);
        ar& CEREAL_NVP(maxCrossTrackConstraint_m);
    }
    if (version >= 2) {
        ar& CEREAL_NVP(xPosition_m);
        ar& CEREAL_NVP(yPosition_m);
        ar& CEREAL_NVP(heading_rad);
    }
}



template void PathTrackerPreviewState::serialize(cereal::BinaryInputArchive& ar, unsigned int version);
template void PathTrackerPreviewState::serialize(cereal::BinaryOutputArchive& ar, unsigned int version);
template void PathTrackerPreviewState::serialize(cereal::IntrospectionArchive& ar, unsigned int version);
template void PathTrackerPreviewState::serialize(cereal::JSONInputArchive& ar, unsigned int version);
template void PathTrackerPreviewState::serialize(cereal::JSONOutputArchive& ar, unsigned int version);
template void PathTrackerPreviewState::serialize(cereal::PortableBinaryInputArchive& ar, unsigned int version);
template void PathTrackerPreviewState::serialize(cereal::PortableBinaryOutputArchive& ar, unsigned int version);

std::ostream& operator<< (std::ostream& os, const PathTrackerPreviewState& pathtrackerpreviewstate)
{
    pathtrackerpreviewstate.toCsv(os);
    return os;
}

bool operator== (const PathTrackerPreviewState &lhs, const PathTrackerPreviewState &rhs)
{
    return (  lhs.time_s== rhs.time_s &&
     lhs.curvature_k== rhs.curvature_k &&
     lhs.velocity_mps== rhs.velocity_mps &&
     lhs.minCrossTrackConstraint_m== rhs.minCrossTrackConstraint_m &&
     lhs.maxCrossTrackConstraint_m== rhs.maxCrossTrackConstraint_m &&
     lhs.xPosition_m== rhs.xPosition_m &&
     lhs.yPosition_m== rhs.yPosition_m &&
     lhs.heading_rad== rhs.heading_rad 
    );
}

std::size_t PathTrackerPreviewState::hash() const noexcept
{
    std::size_t result = 17;
    result = result * 31 + std::hash<double>()(time_s);
    result = result * 31 + std::hash<double>()(curvature_k);
    result = result * 31 + std::hash<double>()(velocity_mps);
    result = result * 31 + std::hash<double>()(minCrossTrackConstraint_m);
    result = result * 31 + std::hash<double>()(maxCrossTrackConstraint_m);
    result = result * 31 + std::hash<double>()(xPosition_m);
    result = result * 31 + std::hash<double>()(yPosition_m);
    result = result * 31 + std::hash<double>()(heading_rad);
    return result;
}

 } // namespace av
 } // namespace msg


CEREAL_BIND_TO_ARCHIVES(av::msg::PathTrackerPreviewState)

 namespace av {
 namespace msg {


PathTrackerReference::PathTrackerReference() = default;

PathTrackerReference::PathTrackerReference(
    double crossTrackError_m_arg ,
    double crossTrackErrorRate_mps_arg ,
    double headingError_rad_arg ,
    double headingErrorFiltered_rad_arg ,
    double headingErrorRate_radps_arg ,
    double curvature_k_arg ,
    double curvatureFirstDerivative_kpm_arg ,
    double curvatureSecondDerivative_kpm2_arg ,
    double maxCurvatureInWindow_k_arg ,
    std::vector<PathTrackerPreviewState> pathTrackerPreview_arg 
    ):
    crossTrackError_m { std::move(crossTrackError_m_arg) } // NOLINT
      ,
    crossTrackErrorRate_mps { std::move(crossTrackErrorRate_mps_arg) } // NOLINT
      ,
    headingError_rad { std::move(headingError_rad_arg) } // NOLINT
      ,
    headingErrorFiltered_rad { std::move(headingErrorFiltered_rad_arg) } // NOLINT
      ,
    headingErrorRate_radps { std::move(headingErrorRate_radps_arg) } // NOLINT
      ,
    curvature_k { std::move(curvature_k_arg) } // NOLINT
      ,
    curvatureFirstDerivative_kpm { std::move(curvatureFirstDerivative_kpm_arg) } // NOLINT
      ,
    curvatureSecondDerivative_kpm2 { std::move(curvatureSecondDerivative_kpm2_arg) } // NOLINT
      ,
    maxCurvatureInWindow_k { std::move(maxCurvatureInWindow_k_arg) } // NOLINT
      ,
    pathTrackerPreview { std::move(pathTrackerPreview_arg) } // NOLINT
      
    {
}

PathTrackerReference::PathTrackerReference(const PathTrackerReference&) = default;

// NOLINTNEXTLINE
PathTrackerReference::PathTrackerReference(PathTrackerReference&&) = default;

PathTrackerReference& PathTrackerReference::operator=(const PathTrackerReference&) = default;

// NOLINTNEXTLINE
PathTrackerReference& PathTrackerReference::operator=(PathTrackerReference&&) = default;

void PathTrackerReference::toCsv(std::ostream& out) const
{
    out.precision(16);
    out << std::boolalpha;
    out << crossTrackError_m;out << ",";
    out << crossTrackErrorRate_mps;out << ",";
    out << headingError_rad;out << ",";
    out << headingErrorFiltered_rad;out << ",";
    out << headingErrorRate_radps;out << ",";
    out << curvature_k;out << ",";
    out << curvatureFirstDerivative_kpm;out << ",";
    out << curvatureSecondDerivative_kpm2;out << ",";
    out << maxCurvatureInWindow_k;out << ",";
    // Enclose sequence in double quotations and curly brackets to keep CSV line length uniform
        
        out << "\"{{";
        
        if (!pathTrackerPreview.empty())
        {
            std::copy(pathTrackerPreview.begin(), pathTrackerPreview.end() - 1, std::ostream_iterator<PathTrackerPreviewState>(out, "},{"));
            std::copy(pathTrackerPreview.end()-1, pathTrackerPreview.end(), std::ostream_iterator<PathTrackerPreviewState>(out, ""));
        }
        out << "}}\"";
    
}

std::stringstream PathTrackerReference::toStringstream() const
{
    std::stringstream out;
    toCsv(out);
    return out;
}

template<class Archive>
    void PathTrackerReference::serialize(Archive& ar, [[maybe_unused]] unsigned int version)
    {

        ar& CEREAL_NVP(crossTrackError_m);
        ar& CEREAL_NVP(crossTrackErrorRate_mps);
        ar& CEREAL_NVP(headingError_rad);
        ar& CEREAL_NVP(headingErrorFiltered_rad);
        ar& CEREAL_NVP(headingErrorRate_radps);
        ar& CEREAL_NVP(curvature_k);
        ar& CEREAL_NVP(curvatureFirstDerivative_kpm);
        ar& CEREAL_NVP(curvatureSecondDerivative_kpm2);
        ar& CEREAL_NVP(maxCurvatureInWindow_k);
        ar& CEREAL_NVP(pathTrackerPreview);
    }


template void PathTrackerReference::serialize(cereal::BinaryInputArchive& ar, unsigned int version);
template void PathTrackerReference::serialize(cereal::BinaryOutputArchive& ar, unsigned int version);
template void PathTrackerReference::serialize(cereal::IntrospectionArchive& ar, unsigned int version);
template void PathTrackerReference::serialize(cereal::JSONInputArchive& ar, unsigned int version);
template void PathTrackerReference::serialize(cereal::JSONOutputArchive& ar, unsigned int version);
template void PathTrackerReference::serialize(cereal::PortableBinaryInputArchive& ar, unsigned int version);
template void PathTrackerReference::serialize(cereal::PortableBinaryOutputArchive& ar, unsigned int version);

std::ostream& operator<< (std::ostream& os, const PathTrackerReference& pathtrackerreference)
{
    pathtrackerreference.toCsv(os);
    return os;
}

bool operator== (const PathTrackerReference &lhs, const PathTrackerReference &rhs)
{
    return (  lhs.crossTrackError_m== rhs.crossTrackError_m &&
     lhs.crossTrackErrorRate_mps== rhs.crossTrackErrorRate_mps &&
     lhs.headingError_rad== rhs.headingError_rad &&
     lhs.headingErrorFiltered_rad== rhs.headingErrorFiltered_rad &&
     lhs.headingErrorRate_radps== rhs.headingErrorRate_radps &&
     lhs.curvature_k== rhs.curvature_k &&
     lhs.curvatureFirstDerivative_kpm== rhs.curvatureFirstDerivative_kpm &&
     lhs.curvatureSecondDerivative_kpm2== rhs.curvatureSecondDerivative_kpm2 &&
     lhs.maxCurvatureInWindow_k== rhs.maxCurvatureInWindow_k &&
     lhs.pathTrackerPreview== rhs.pathTrackerPreview 
    );
}

std::size_t PathTrackerReference::hash() const noexcept
{
    std::size_t result = 17;
    result = result * 31 + std::hash<double>()(crossTrackError_m);
    result = result * 31 + std::hash<double>()(crossTrackErrorRate_mps);
    result = result * 31 + std::hash<double>()(headingError_rad);
    result = result * 31 + std::hash<double>()(headingErrorFiltered_rad);
    result = result * 31 + std::hash<double>()(headingErrorRate_radps);
    result = result * 31 + std::hash<double>()(curvature_k);
    result = result * 31 + std::hash<double>()(curvatureFirstDerivative_kpm);
    result = result * 31 + std::hash<double>()(curvatureSecondDerivative_kpm2);
    result = result * 31 + std::hash<double>()(maxCurvatureInWindow_k);
    result = result * 31 + hashVector(pathTrackerPreview);
    return result;
}

 } // namespace av
 } // namespace msg


CEREAL_BIND_TO_ARCHIVES(av::msg::PathTrackerReference)

 namespace av {
 namespace msg {


PathTrackerTaskInputs::PathTrackerTaskInputs() = default;

PathTrackerTaskInputs::PathTrackerTaskInputs(
    PathTrackerReference reference_arg ,
    PathTrackerVehicleState vehicleState_arg ,
    AutoReadyCommand autoReadyCommand_arg ,
    OptimizerErrorMessages optimizerErrorMessages_arg 
    ):
    reference { std::move(reference_arg) } // NOLINT
      ,
    vehicleState { std::move(vehicleState_arg) } // NOLINT
      ,
    autoReadyCommand { std::move(autoReadyCommand_arg) } // NOLINT
      ,
    optimizerErrorMessages { std::move(optimizerErrorMessages_arg) } // NOLINT
      
    {
}

PathTrackerTaskInputs::PathTrackerTaskInputs(const PathTrackerTaskInputs&) = default;

// NOLINTNEXTLINE
PathTrackerTaskInputs::PathTrackerTaskInputs(PathTrackerTaskInputs&&) = default;

PathTrackerTaskInputs& PathTrackerTaskInputs::operator=(const PathTrackerTaskInputs&) = default;

// NOLINTNEXTLINE
PathTrackerTaskInputs& PathTrackerTaskInputs::operator=(PathTrackerTaskInputs&&) = default;

void PathTrackerTaskInputs::toCsv(std::ostream& out) const
{
    out.precision(16);
    out << std::boolalpha;
    out << reference;out << ",";
    out << vehicleState;out << ",";
    out << autoReadyCommand;out << ",";
    out << optimizerErrorMessages;
    
}

std::stringstream PathTrackerTaskInputs::toStringstream() const
{
    std::stringstream out;
    toCsv(out);
    return out;
}

template<class Archive>
    void PathTrackerTaskInputs::serialize(Archive& ar, unsigned int version)
{
    if (version >= 1) {
        ar& CEREAL_NVP(reference);
        ar& CEREAL_NVP(vehicleState);
        ar& CEREAL_NVP(autoReadyCommand);
        ar& CEREAL_NVP(optimizerErrorMessages);
    }

    if (version == 2) {
        ar& CEREAL_NVP(reference);
        ar& CEREAL_NVP(vehicleState);
    }
}



template void PathTrackerTaskInputs::serialize(cereal::BinaryInputArchive& ar, unsigned int version);
template void PathTrackerTaskInputs::serialize(cereal::BinaryOutputArchive& ar, unsigned int version);
template void PathTrackerTaskInputs::serialize(cereal::IntrospectionArchive& ar, unsigned int version);
template void PathTrackerTaskInputs::serialize(cereal::JSONInputArchive& ar, unsigned int version);
template void PathTrackerTaskInputs::serialize(cereal::JSONOutputArchive& ar, unsigned int version);
template void PathTrackerTaskInputs::serialize(cereal::PortableBinaryInputArchive& ar, unsigned int version);
template void PathTrackerTaskInputs::serialize(cereal::PortableBinaryOutputArchive& ar, unsigned int version);

std::ostream& operator<< (std::ostream& os, const PathTrackerTaskInputs& pathtrackertaskinputs)
{
    pathtrackertaskinputs.toCsv(os);
    return os;
}

bool operator== (const PathTrackerTaskInputs &lhs, const PathTrackerTaskInputs &rhs)
{
    return (  lhs.reference== rhs.reference &&
     lhs.vehicleState== rhs.vehicleState &&
     lhs.autoReadyCommand== rhs.autoReadyCommand &&
     lhs.optimizerErrorMessages== rhs.optimizerErrorMessages 
    );
}

std::size_t PathTrackerTaskInputs::hash() const noexcept
{
    std::size_t result = 17;
    result = result * 31 + std::hash<PathTrackerReference>()(reference);
    result = result * 31 + std::hash<PathTrackerVehicleState>()(vehicleState);
    result = result * 31 + std::hash<AutoReadyCommand>()(autoReadyCommand);
    result = result * 31 + std::hash<OptimizerErrorMessages>()(optimizerErrorMessages);
    return result;
}

 } // namespace av
 } // namespace msg


CEREAL_BIND_TO_ARCHIVES(av::msg::PathTrackerTaskInputs)




 namespace av {
 namespace msg {


PathTrackerTaskChannel::PathTrackerTaskChannel() = default;

PathTrackerTaskChannel::PathTrackerTaskChannel(
    PathTrackerTaskInputs pathTrackerTaskInputs_arg 
    ):
    pathTrackerTaskInputs { std::move(pathTrackerTaskInputs_arg) } // NOLINT
      {
}

PathTrackerTaskChannel::PathTrackerTaskChannel(const PathTrackerTaskChannel&) = default;

// NOLINTNEXTLINE
PathTrackerTaskChannel::PathTrackerTaskChannel(PathTrackerTaskChannel&&) = default;

PathTrackerTaskChannel& PathTrackerTaskChannel::operator=(const PathTrackerTaskChannel&) = default;

// NOLINTNEXTLINE
PathTrackerTaskChannel& PathTrackerTaskChannel::operator=(PathTrackerTaskChannel&&) = default;

void PathTrackerTaskChannel::toCsv(std::ostream& out) const
{
    out.precision(16);
    out << std::boolalpha;
    out << pathTrackerTaskInputs;
    
}

std::stringstream PathTrackerTaskChannel::toStringstream() const
{
    std::stringstream out;
    toCsv(out);
    return out;
}

template<class Archive>
    void PathTrackerTaskChannel::serialize(Archive& ar, unsigned int version)
    {
        if (version >= 1) {
            ar& CEREAL_NVP(pathTrackerTaskInputs);
        }
    }

}} // to close av::msg namespace temporary

struct PathTrackerTaskChannel : public av::msg::PathTrackerTaskChannel {};
CEREAL_CLASS_VERSION(PathTrackerTaskChannel, 2)
CEREAL_REGISTER_TYPE(PathTrackerTaskChannel)

namespace av {
namespace msg {



template void PathTrackerTaskChannel::serialize(cereal::BinaryInputArchive& ar, unsigned int version);
template void PathTrackerTaskChannel::serialize(cereal::BinaryOutputArchive& ar, unsigned int version);
template void PathTrackerTaskChannel::serialize(cereal::IntrospectionArchive& ar, unsigned int version);
template void PathTrackerTaskChannel::serialize(cereal::JSONInputArchive& ar, unsigned int version);
template void PathTrackerTaskChannel::serialize(cereal::JSONOutputArchive& ar, unsigned int version);
template void PathTrackerTaskChannel::serialize(cereal::PortableBinaryInputArchive& ar, unsigned int version);
template void PathTrackerTaskChannel::serialize(cereal::PortableBinaryOutputArchive& ar, unsigned int version);

std::ostream& operator<< (std::ostream& os, const PathTrackerTaskChannel& pathtrackertaskchannel)
{
    pathtrackertaskchannel.toCsv(os);
    return os;
}

bool operator== (const PathTrackerTaskChannel &lhs, const PathTrackerTaskChannel &rhs)
{
    return (  lhs.pathTrackerTaskInputs== rhs.pathTrackerTaskInputs 
    );
}

std::size_t PathTrackerTaskChannel::hash() const noexcept
{
    std::size_t result = 17;
    result = result * 31 + std::hash<PathTrackerTaskInputs>()(pathTrackerTaskInputs);
    return result;
}

 } // namespace av
 } // namespace msg


CEREAL_BIND_TO_ARCHIVES(av::msg::PathTrackerTaskChannel)

