#include <interfaces/PerceptionCommonTypes/PerceptionCommonTypes.h>

#include <iterator>
#include <type_traits>

#include <cereal/cereal.hpp>
#include <cereal/archives/json.hpp>
#include <cereal/archives/binary.hpp>

#include <cereal/types/map.hpp>
#include <cereal/types/vector.hpp>
#include <cereal/types/array.hpp>
#include <cereal/types/string.hpp>
#include <cereal/types/complex.hpp>

#include <ODatum/IntrospectionArchive.h>
#include <ODatum/Hash.h>


 namespace av {
 namespace msg {


PercElapsedTime::PercElapsedTime() = default;

PercElapsedTime::PercElapsedTime(
    uint64_t ref_tstamp_arg ,
    std::string node_name_arg ,
    double node_delay_arg ,
    std::string sender_arg 
    ):
    ref_tstamp { std::move(ref_tstamp_arg) } // NOLINT
      ,
    node_name { std::move(node_name_arg) } // NOLINT
      ,
    node_delay { std::move(node_delay_arg) } // NOLINT
      ,
    sender { std::move(sender_arg) } // NOLINT
      
    {
}

PercElapsedTime::PercElapsedTime(const PercElapsedTime&) = default;

// NOLINTNEXTLINE
PercElapsedTime::PercElapsedTime(PercElapsedTime&&) = default;

PercElapsedTime& PercElapsedTime::operator=(const PercElapsedTime&) = default;

// NOLINTNEXTLINE
PercElapsedTime& PercElapsedTime::operator=(PercElapsedTime&&) = default;

void PercElapsedTime::toCsv(std::ostream& out) const
{
    out.precision(16);
    out << std::boolalpha;
    out << ref_tstamp;out << ",";
    out << "\"";
        out << node_name;
        out << "\"";out << ",";
    out << node_delay;out << ",";
    out << "\"";
        out << sender;
        out << "\"";
    
}

std::stringstream PercElapsedTime::toStringstream() const
{
    std::stringstream out;
    toCsv(out);
    return out;
}

template<class Archive>
    void PercElapsedTime::serialize(Archive& ar, [[maybe_unused]] unsigned int version)
    {

        ar& CEREAL_NVP(ref_tstamp);
        ar& CEREAL_NVP(node_name);
        ar& CEREAL_NVP(node_delay);
        ar& CEREAL_NVP(sender);
    }


template void PercElapsedTime::serialize(cereal::BinaryInputArchive& ar, unsigned int version);
template void PercElapsedTime::serialize(cereal::BinaryOutputArchive& ar, unsigned int version);
template void PercElapsedTime::serialize(cereal::IntrospectionArchive& ar, unsigned int version);
template void PercElapsedTime::serialize(cereal::JSONInputArchive& ar, unsigned int version);
template void PercElapsedTime::serialize(cereal::JSONOutputArchive& ar, unsigned int version);
template void PercElapsedTime::serialize(cereal::PortableBinaryInputArchive& ar, unsigned int version);
template void PercElapsedTime::serialize(cereal::PortableBinaryOutputArchive& ar, unsigned int version);

std::ostream& operator<< (std::ostream& os, const PercElapsedTime& percelapsedtime)
{
    percelapsedtime.toCsv(os);
    return os;
}

bool operator== (const PercElapsedTime &lhs, const PercElapsedTime &rhs)
{
    return (  lhs.ref_tstamp== rhs.ref_tstamp &&
     lhs.node_name== rhs.node_name &&
     lhs.node_delay== rhs.node_delay &&
     lhs.sender== rhs.sender 
    );
}

std::size_t PercElapsedTime::hash() const noexcept
{
    std::size_t result = 17;
    result = result * 31 + std::hash<uint64_t>()(ref_tstamp);
    result = result * 31 + std::hash<std::string>()(node_name);
    result = result * 31 + std::hash<double>()(node_delay);
    result = result * 31 + std::hash<std::string>()(sender);
    return result;
}

 } // namespace av
 } // namespace msg


CEREAL_BIND_TO_ARCHIVES(av::msg::PercElapsedTime)

 namespace av {
 namespace msg {


TrackMeasSerialPair::TrackMeasSerialPair() = default;

TrackMeasSerialPair::TrackMeasSerialPair(
    int32_t track_serial_arg ,
    int32_t meas_serial_arg 
    ):
    track_serial { std::move(track_serial_arg) } // NOLINT
      ,
    meas_serial { std::move(meas_serial_arg) } // NOLINT
      
    {
}

TrackMeasSerialPair::TrackMeasSerialPair(const TrackMeasSerialPair&) = default;

// NOLINTNEXTLINE
TrackMeasSerialPair::TrackMeasSerialPair(TrackMeasSerialPair&&) = default;

TrackMeasSerialPair& TrackMeasSerialPair::operator=(const TrackMeasSerialPair&) = default;

// NOLINTNEXTLINE
TrackMeasSerialPair& TrackMeasSerialPair::operator=(TrackMeasSerialPair&&) = default;

void TrackMeasSerialPair::toCsv(std::ostream& out) const
{
    out.precision(16);
    out << std::boolalpha;
    out << track_serial;out << ",";
    out << meas_serial;
    
}

std::stringstream TrackMeasSerialPair::toStringstream() const
{
    std::stringstream out;
    toCsv(out);
    return out;
}

template<class Archive>
    void TrackMeasSerialPair::serialize(Archive& ar, [[maybe_unused]] unsigned int version)
    {

        ar& CEREAL_NVP(track_serial);
        ar& CEREAL_NVP(meas_serial);
    }


template void TrackMeasSerialPair::serialize(cereal::BinaryInputArchive& ar, unsigned int version);
template void TrackMeasSerialPair::serialize(cereal::BinaryOutputArchive& ar, unsigned int version);
template void TrackMeasSerialPair::serialize(cereal::IntrospectionArchive& ar, unsigned int version);
template void TrackMeasSerialPair::serialize(cereal::JSONInputArchive& ar, unsigned int version);
template void TrackMeasSerialPair::serialize(cereal::JSONOutputArchive& ar, unsigned int version);
template void TrackMeasSerialPair::serialize(cereal::PortableBinaryInputArchive& ar, unsigned int version);
template void TrackMeasSerialPair::serialize(cereal::PortableBinaryOutputArchive& ar, unsigned int version);

std::ostream& operator<< (std::ostream& os, const TrackMeasSerialPair& trackmeasserialpair)
{
    trackmeasserialpair.toCsv(os);
    return os;
}

bool operator== (const TrackMeasSerialPair &lhs, const TrackMeasSerialPair &rhs)
{
    return (  lhs.track_serial== rhs.track_serial &&
     lhs.meas_serial== rhs.meas_serial 
    );
}

std::size_t TrackMeasSerialPair::hash() const noexcept
{
    std::size_t result = 17;
    result = result * 31 + std::hash<int32_t>()(track_serial);
    result = result * 31 + std::hash<int32_t>()(meas_serial);
    return result;
}

 } // namespace av
 } // namespace msg


CEREAL_BIND_TO_ARCHIVES(av::msg::TrackMeasSerialPair)


