#include <interfaces/PlannedPathWithBounds/PlannedPathWithBounds.h>

#include <iterator>
#include <type_traits>

#include <cereal/cereal.hpp>
#include <cereal/archives/json.hpp>
#include <cereal/archives/binary.hpp>

#include <cereal/types/map.hpp>
#include <cereal/types/vector.hpp>
#include <cereal/types/array.hpp>
#include <cereal/types/string.hpp>
#include <cereal/types/complex.hpp>

#include <ODatum/IntrospectionArchive.h>
#include <ODatum/Hash.h>


 namespace av {
 namespace msg {


SpeedConstraintMapInfo::SpeedConstraintMapInfo() = default;

SpeedConstraintMapInfo::SpeedConstraintMapInfo(
    std::string id_arg ,
    MapType mapType_arg 
    ):
    id { std::move(id_arg) } // NOLINT
      ,
    mapType { std::move(mapType_arg) } // NOLINT
      
    {
}

SpeedConstraintMapInfo::SpeedConstraintMapInfo(const SpeedConstraintMapInfo&) = default;

// NOLINTNEXTLINE
SpeedConstraintMapInfo::SpeedConstraintMapInfo(SpeedConstraintMapInfo&&) = default;

SpeedConstraintMapInfo& SpeedConstraintMapInfo::operator=(const SpeedConstraintMapInfo&) = default;

// NOLINTNEXTLINE
SpeedConstraintMapInfo& SpeedConstraintMapInfo::operator=(SpeedConstraintMapInfo&&) = default;

void SpeedConstraintMapInfo::toCsv(std::ostream& out) const
{
    out.precision(16);
    out << std::boolalpha;
    out << "\"";
        out << id;
        out << "\"";out << ",";
    out << static_cast<uint32_t>(mapType);
    
}

std::stringstream SpeedConstraintMapInfo::toStringstream() const
{
    std::stringstream out;
    toCsv(out);
    return out;
}

template<class Archive>
    void SpeedConstraintMapInfo::serialize(Archive& ar, [[maybe_unused]] unsigned int version)
    {

        ar& CEREAL_NVP(id);
        ar& CEREAL_NVP(mapType);
    }


template void SpeedConstraintMapInfo::serialize(cereal::BinaryInputArchive& ar, unsigned int version);
template void SpeedConstraintMapInfo::serialize(cereal::BinaryOutputArchive& ar, unsigned int version);
template void SpeedConstraintMapInfo::serialize(cereal::IntrospectionArchive& ar, unsigned int version);
template void SpeedConstraintMapInfo::serialize(cereal::JSONInputArchive& ar, unsigned int version);
template void SpeedConstraintMapInfo::serialize(cereal::JSONOutputArchive& ar, unsigned int version);
template void SpeedConstraintMapInfo::serialize(cereal::PortableBinaryInputArchive& ar, unsigned int version);
template void SpeedConstraintMapInfo::serialize(cereal::PortableBinaryOutputArchive& ar, unsigned int version);

std::ostream& operator<< (std::ostream& os, const SpeedConstraintMapInfo& speedconstraintmapinfo)
{
    speedconstraintmapinfo.toCsv(os);
    return os;
}

bool operator== (const SpeedConstraintMapInfo &lhs, const SpeedConstraintMapInfo &rhs)
{
    return (  lhs.id== rhs.id &&
     lhs.mapType== rhs.mapType 
    );
}

std::size_t SpeedConstraintMapInfo::hash() const noexcept
{
    std::size_t result = 17;
    result = result * 31 + std::hash<std::string>()(id);
    result = result * 31 + std::hash<MapType>()(mapType);
    return result;
}

 } // namespace av
 } // namespace msg


CEREAL_BIND_TO_ARCHIVES(av::msg::SpeedConstraintMapInfo)

 namespace av {
 namespace msg {


SpeedConstraintTrackInfo::SpeedConstraintTrackInfo() = default;

SpeedConstraintTrackInfo::SpeedConstraintTrackInfo(
    int64_t id_arg ,
    double x_arg ,
    double y_arg 
    ):
    id { std::move(id_arg) } // NOLINT
      ,
    x { std::move(x_arg) } // NOLINT
      ,
    y { std::move(y_arg) } // NOLINT
      
    {
}

SpeedConstraintTrackInfo::SpeedConstraintTrackInfo(const SpeedConstraintTrackInfo&) = default;

// NOLINTNEXTLINE
SpeedConstraintTrackInfo::SpeedConstraintTrackInfo(SpeedConstraintTrackInfo&&) = default;

SpeedConstraintTrackInfo& SpeedConstraintTrackInfo::operator=(const SpeedConstraintTrackInfo&) = default;

// NOLINTNEXTLINE
SpeedConstraintTrackInfo& SpeedConstraintTrackInfo::operator=(SpeedConstraintTrackInfo&&) = default;

void SpeedConstraintTrackInfo::toCsv(std::ostream& out) const
{
    out.precision(16);
    out << std::boolalpha;
    out << id;out << ",";
    out << x;out << ",";
    out << y;
    
}

std::stringstream SpeedConstraintTrackInfo::toStringstream() const
{
    std::stringstream out;
    toCsv(out);
    return out;
}

template<class Archive>
    void SpeedConstraintTrackInfo::serialize(Archive& ar, [[maybe_unused]] unsigned int version)
    {

        ar& CEREAL_NVP(id);
        ar& CEREAL_NVP(x);
        ar& CEREAL_NVP(y);
    }


template void SpeedConstraintTrackInfo::serialize(cereal::BinaryInputArchive& ar, unsigned int version);
template void SpeedConstraintTrackInfo::serialize(cereal::BinaryOutputArchive& ar, unsigned int version);
template void SpeedConstraintTrackInfo::serialize(cereal::IntrospectionArchive& ar, unsigned int version);
template void SpeedConstraintTrackInfo::serialize(cereal::JSONInputArchive& ar, unsigned int version);
template void SpeedConstraintTrackInfo::serialize(cereal::JSONOutputArchive& ar, unsigned int version);
template void SpeedConstraintTrackInfo::serialize(cereal::PortableBinaryInputArchive& ar, unsigned int version);
template void SpeedConstraintTrackInfo::serialize(cereal::PortableBinaryOutputArchive& ar, unsigned int version);

std::ostream& operator<< (std::ostream& os, const SpeedConstraintTrackInfo& speedconstrainttrackinfo)
{
    speedconstrainttrackinfo.toCsv(os);
    return os;
}

bool operator== (const SpeedConstraintTrackInfo &lhs, const SpeedConstraintTrackInfo &rhs)
{
    return (  lhs.id== rhs.id &&
     lhs.x== rhs.x &&
     lhs.y== rhs.y 
    );
}

std::size_t SpeedConstraintTrackInfo::hash() const noexcept
{
    std::size_t result = 17;
    result = result * 31 + std::hash<int64_t>()(id);
    result = result * 31 + std::hash<double>()(x);
    result = result * 31 + std::hash<double>()(y);
    return result;
}

 } // namespace av
 } // namespace msg


CEREAL_BIND_TO_ARCHIVES(av::msg::SpeedConstraintTrackInfo)

 namespace av {
 namespace msg {


SingleContour::SingleContour() = default;

SingleContour::SingleContour(
    ContinuityType continuityType_arg ,
    uint8_t constraintType_arg ,
    BoundType boundType_arg ,
    EnforcementType enforcement_arg ,
    double sMin_arg ,
    double sMax_arg ,
    double sSet_arg ,
    double constrVar_arg ,
    double rate_arg ,
    bool trackExist_arg ,
    SpeedConstraintTrackInfo trackInfo_arg ,
    bool mapObjectExist_arg ,
    SpeedConstraintMapInfo mapInfo_arg 
    ):
    continuityType { std::move(continuityType_arg) } // NOLINT
      ,
    constraintType { std::move(constraintType_arg) } // NOLINT
      ,
    boundType { std::move(boundType_arg) } // NOLINT
      ,
    enforcement { std::move(enforcement_arg) } // NOLINT
      ,
    sMin { std::move(sMin_arg) } // NOLINT
      ,
    sMax { std::move(sMax_arg) } // NOLINT
      ,
    sSet { std::move(sSet_arg) } // NOLINT
      ,
    constrVar { std::move(constrVar_arg) } // NOLINT
      ,
    rate { std::move(rate_arg) } // NOLINT
      ,
    trackExist { std::move(trackExist_arg) } // NOLINT
      ,
    trackInfo { std::move(trackInfo_arg) } // NOLINT
      ,
    mapObjectExist { std::move(mapObjectExist_arg) } // NOLINT
      ,
    mapInfo { std::move(mapInfo_arg) } // NOLINT
      
    {
}

SingleContour::SingleContour(const SingleContour&) = default;

// NOLINTNEXTLINE
SingleContour::SingleContour(SingleContour&&) = default;

SingleContour& SingleContour::operator=(const SingleContour&) = default;

// NOLINTNEXTLINE
SingleContour& SingleContour::operator=(SingleContour&&) = default;

void SingleContour::toCsv(std::ostream& out) const
{
    out.precision(16);
    out << std::boolalpha;
    out << static_cast<uint32_t>(continuityType);out << ",";
    // Cast to uint16_t to print as integer instead of char
        out << static_cast<uint16_t>(constraintType);out << ",";
    out << static_cast<uint32_t>(boundType);out << ",";
    out << static_cast<uint32_t>(enforcement);out << ",";
    out << sMin;out << ",";
    out << sMax;out << ",";
    out << sSet;out << ",";
    out << constrVar;out << ",";
    out << rate;out << ",";
    out << trackExist;out << ",";
    out << trackInfo;out << ",";
    out << mapObjectExist;out << ",";
    out << mapInfo;
    
}

std::stringstream SingleContour::toStringstream() const
{
    std::stringstream out;
    toCsv(out);
    return out;
}

template<class Archive>
    void SingleContour::serialize(Archive& ar, unsigned int version)
{
    if(version >= 1)
    {
        ar& CEREAL_NVP(continuityType);
        ar& CEREAL_NVP(constraintType);
        ar& CEREAL_NVP(boundType);
        ar& CEREAL_NVP(enforcement);

        ar& CEREAL_NVP(sMin);
        ar& CEREAL_NVP(sMax);
        ar& CEREAL_NVP(sSet);
        ar& CEREAL_NVP(constrVar);
        ar& CEREAL_NVP(rate);

        ar& CEREAL_NVP(trackExist);
        ar& CEREAL_NVP(trackInfo);
        ar& CEREAL_NVP(mapObjectExist);
        ar& CEREAL_NVP(mapInfo);
    }
}


template void SingleContour::serialize(cereal::BinaryInputArchive& ar, unsigned int version);
template void SingleContour::serialize(cereal::BinaryOutputArchive& ar, unsigned int version);
template void SingleContour::serialize(cereal::IntrospectionArchive& ar, unsigned int version);
template void SingleContour::serialize(cereal::JSONInputArchive& ar, unsigned int version);
template void SingleContour::serialize(cereal::JSONOutputArchive& ar, unsigned int version);
template void SingleContour::serialize(cereal::PortableBinaryInputArchive& ar, unsigned int version);
template void SingleContour::serialize(cereal::PortableBinaryOutputArchive& ar, unsigned int version);

std::ostream& operator<< (std::ostream& os, const SingleContour& singlecontour)
{
    singlecontour.toCsv(os);
    return os;
}

bool operator== (const SingleContour &lhs, const SingleContour &rhs)
{
    return (  lhs.continuityType== rhs.continuityType &&
     lhs.constraintType== rhs.constraintType &&
     lhs.boundType== rhs.boundType &&
     lhs.enforcement== rhs.enforcement &&
     lhs.sMin== rhs.sMin &&
     lhs.sMax== rhs.sMax &&
     lhs.sSet== rhs.sSet &&
     lhs.constrVar== rhs.constrVar &&
     lhs.rate== rhs.rate &&
     lhs.trackExist== rhs.trackExist &&
     lhs.trackInfo== rhs.trackInfo &&
     lhs.mapObjectExist== rhs.mapObjectExist &&
     lhs.mapInfo== rhs.mapInfo 
    );
}

std::size_t SingleContour::hash() const noexcept
{
    std::size_t result = 17;
    result = result * 31 + std::hash<ContinuityType>()(continuityType);
    result = result * 31 + std::hash<uint8_t>()(constraintType);
    result = result * 31 + std::hash<BoundType>()(boundType);
    result = result * 31 + std::hash<EnforcementType>()(enforcement);
    result = result * 31 + std::hash<double>()(sMin);
    result = result * 31 + std::hash<double>()(sMax);
    result = result * 31 + std::hash<double>()(sSet);
    result = result * 31 + std::hash<double>()(constrVar);
    result = result * 31 + std::hash<double>()(rate);
    result = result * 31 + std::hash<bool>()(trackExist);
    result = result * 31 + std::hash<SpeedConstraintTrackInfo>()(trackInfo);
    result = result * 31 + std::hash<bool>()(mapObjectExist);
    result = result * 31 + std::hash<SpeedConstraintMapInfo>()(mapInfo);
    return result;
}

 } // namespace av
 } // namespace msg


CEREAL_BIND_TO_ARCHIVES(av::msg::SingleContour)

 namespace av {
 namespace msg {


HardAndSoftBounds::HardAndSoftBounds() = default;

HardAndSoftBounds::HardAndSoftBounds(
    double hardMax_arg ,
    double softMax_arg ,
    double hardMin_arg ,
    double softMin_arg 
    ):
    hardMax { std::move(hardMax_arg) } // NOLINT
      ,
    softMax { std::move(softMax_arg) } // NOLINT
      ,
    hardMin { std::move(hardMin_arg) } // NOLINT
      ,
    softMin { std::move(softMin_arg) } // NOLINT
      
    {
}

HardAndSoftBounds::HardAndSoftBounds(const HardAndSoftBounds&) = default;

// NOLINTNEXTLINE
HardAndSoftBounds::HardAndSoftBounds(HardAndSoftBounds&&) = default;

HardAndSoftBounds& HardAndSoftBounds::operator=(const HardAndSoftBounds&) = default;

// NOLINTNEXTLINE
HardAndSoftBounds& HardAndSoftBounds::operator=(HardAndSoftBounds&&) = default;

void HardAndSoftBounds::toCsv(std::ostream& out) const
{
    out.precision(16);
    out << std::boolalpha;
    out << hardMax;out << ",";
    out << softMax;out << ",";
    out << hardMin;out << ",";
    out << softMin;
    
}

std::stringstream HardAndSoftBounds::toStringstream() const
{
    std::stringstream out;
    toCsv(out);
    return out;
}

template<class Archive>
    void HardAndSoftBounds::serialize(Archive& ar, [[maybe_unused]] unsigned int version)
    {

        ar& CEREAL_NVP(hardMax);
        ar& CEREAL_NVP(softMax);
        ar& CEREAL_NVP(hardMin);
        ar& CEREAL_NVP(softMin);
    }


template void HardAndSoftBounds::serialize(cereal::BinaryInputArchive& ar, unsigned int version);
template void HardAndSoftBounds::serialize(cereal::BinaryOutputArchive& ar, unsigned int version);
template void HardAndSoftBounds::serialize(cereal::IntrospectionArchive& ar, unsigned int version);
template void HardAndSoftBounds::serialize(cereal::JSONInputArchive& ar, unsigned int version);
template void HardAndSoftBounds::serialize(cereal::JSONOutputArchive& ar, unsigned int version);
template void HardAndSoftBounds::serialize(cereal::PortableBinaryInputArchive& ar, unsigned int version);
template void HardAndSoftBounds::serialize(cereal::PortableBinaryOutputArchive& ar, unsigned int version);

std::ostream& operator<< (std::ostream& os, const HardAndSoftBounds& hardandsoftbounds)
{
    hardandsoftbounds.toCsv(os);
    return os;
}

bool operator== (const HardAndSoftBounds &lhs, const HardAndSoftBounds &rhs)
{
    return (  lhs.hardMax== rhs.hardMax &&
     lhs.softMax== rhs.softMax &&
     lhs.hardMin== rhs.hardMin &&
     lhs.softMin== rhs.softMin 
    );
}

std::size_t HardAndSoftBounds::hash() const noexcept
{
    std::size_t result = 17;
    result = result * 31 + std::hash<double>()(hardMax);
    result = result * 31 + std::hash<double>()(softMax);
    result = result * 31 + std::hash<double>()(hardMin);
    result = result * 31 + std::hash<double>()(softMin);
    return result;
}

 } // namespace av
 } // namespace msg


CEREAL_BIND_TO_ARCHIVES(av::msg::HardAndSoftBounds)

 namespace av {
 namespace msg {


HardAndSoftBoundsSpatialConstraintType::HardAndSoftBoundsSpatialConstraintType() = default;

HardAndSoftBoundsSpatialConstraintType::HardAndSoftBoundsSpatialConstraintType(
    SpatialConstraintType hardMax_arg ,
    SpatialConstraintType softMax_arg ,
    SpatialConstraintType hardMin_arg ,
    SpatialConstraintType softMin_arg 
    ):
    hardMax { std::move(hardMax_arg) } // NOLINT
      ,
    softMax { std::move(softMax_arg) } // NOLINT
      ,
    hardMin { std::move(hardMin_arg) } // NOLINT
      ,
    softMin { std::move(softMin_arg) } // NOLINT
      
    {
}

HardAndSoftBoundsSpatialConstraintType::HardAndSoftBoundsSpatialConstraintType(const HardAndSoftBoundsSpatialConstraintType&) = default;

// NOLINTNEXTLINE
HardAndSoftBoundsSpatialConstraintType::HardAndSoftBoundsSpatialConstraintType(HardAndSoftBoundsSpatialConstraintType&&) = default;

HardAndSoftBoundsSpatialConstraintType& HardAndSoftBoundsSpatialConstraintType::operator=(const HardAndSoftBoundsSpatialConstraintType&) = default;

// NOLINTNEXTLINE
HardAndSoftBoundsSpatialConstraintType& HardAndSoftBoundsSpatialConstraintType::operator=(HardAndSoftBoundsSpatialConstraintType&&) = default;

void HardAndSoftBoundsSpatialConstraintType::toCsv(std::ostream& out) const
{
    out.precision(16);
    out << std::boolalpha;
    out << static_cast<uint32_t>(hardMax);out << ",";
    out << static_cast<uint32_t>(softMax);out << ",";
    out << static_cast<uint32_t>(hardMin);out << ",";
    out << static_cast<uint32_t>(softMin);
    
}

std::stringstream HardAndSoftBoundsSpatialConstraintType::toStringstream() const
{
    std::stringstream out;
    toCsv(out);
    return out;
}

template<class Archive>
    void HardAndSoftBoundsSpatialConstraintType::serialize(Archive& ar, [[maybe_unused]] unsigned int version)
    {

        ar& CEREAL_NVP(hardMax);
        ar& CEREAL_NVP(softMax);
        ar& CEREAL_NVP(hardMin);
        ar& CEREAL_NVP(softMin);
    }


template void HardAndSoftBoundsSpatialConstraintType::serialize(cereal::BinaryInputArchive& ar, unsigned int version);
template void HardAndSoftBoundsSpatialConstraintType::serialize(cereal::BinaryOutputArchive& ar, unsigned int version);
template void HardAndSoftBoundsSpatialConstraintType::serialize(cereal::IntrospectionArchive& ar, unsigned int version);
template void HardAndSoftBoundsSpatialConstraintType::serialize(cereal::JSONInputArchive& ar, unsigned int version);
template void HardAndSoftBoundsSpatialConstraintType::serialize(cereal::JSONOutputArchive& ar, unsigned int version);
template void HardAndSoftBoundsSpatialConstraintType::serialize(cereal::PortableBinaryInputArchive& ar, unsigned int version);
template void HardAndSoftBoundsSpatialConstraintType::serialize(cereal::PortableBinaryOutputArchive& ar, unsigned int version);

std::ostream& operator<< (std::ostream& os, const HardAndSoftBoundsSpatialConstraintType& hardandsoftboundsspatialconstrainttype)
{
    hardandsoftboundsspatialconstrainttype.toCsv(os);
    return os;
}

bool operator== (const HardAndSoftBoundsSpatialConstraintType &lhs, const HardAndSoftBoundsSpatialConstraintType &rhs)
{
    return (  lhs.hardMax== rhs.hardMax &&
     lhs.softMax== rhs.softMax &&
     lhs.hardMin== rhs.hardMin &&
     lhs.softMin== rhs.softMin 
    );
}

std::size_t HardAndSoftBoundsSpatialConstraintType::hash() const noexcept
{
    std::size_t result = 17;
    result = result * 31 + std::hash<SpatialConstraintType>()(hardMax);
    result = result * 31 + std::hash<SpatialConstraintType>()(softMax);
    result = result * 31 + std::hash<SpatialConstraintType>()(hardMin);
    result = result * 31 + std::hash<SpatialConstraintType>()(softMin);
    return result;
}

 } // namespace av
 } // namespace msg


CEREAL_BIND_TO_ARCHIVES(av::msg::HardAndSoftBoundsSpatialConstraintType)

 namespace av {
 namespace msg {


HardAndSoftConstraints::HardAndSoftConstraints() = default;

HardAndSoftConstraints::HardAndSoftConstraints(
    double sRef_arg ,
    std::vector<SingleContour> hardMax_arg ,
    std::vector<SingleContour> softMax_arg ,
    std::vector<SingleContour> hardMin_arg ,
    std::vector<SingleContour> softMin_arg 
    ):
    sRef { std::move(sRef_arg) } // NOLINT
      ,
    hardMax { std::move(hardMax_arg) } // NOLINT
      ,
    softMax { std::move(softMax_arg) } // NOLINT
      ,
    hardMin { std::move(hardMin_arg) } // NOLINT
      ,
    softMin { std::move(softMin_arg) } // NOLINT
      
    {
}

HardAndSoftConstraints::HardAndSoftConstraints(const HardAndSoftConstraints&) = default;

// NOLINTNEXTLINE
HardAndSoftConstraints::HardAndSoftConstraints(HardAndSoftConstraints&&) = default;

HardAndSoftConstraints& HardAndSoftConstraints::operator=(const HardAndSoftConstraints&) = default;

// NOLINTNEXTLINE
HardAndSoftConstraints& HardAndSoftConstraints::operator=(HardAndSoftConstraints&&) = default;

void HardAndSoftConstraints::toCsv(std::ostream& out) const
{
    out.precision(16);
    out << std::boolalpha;
    out << sRef;out << ",";
    // Enclose sequence in double quotations and curly brackets to keep CSV line length uniform
        
        out << "\"{{";
        
        if (!hardMax.empty())
        {
            std::copy(hardMax.begin(), hardMax.end() - 1, std::ostream_iterator<SingleContour>(out, "},{"));
            std::copy(hardMax.end()-1, hardMax.end(), std::ostream_iterator<SingleContour>(out, ""));
        }
        out << "}}\"";out << ",";
    // Enclose sequence in double quotations and curly brackets to keep CSV line length uniform
        
        out << "\"{{";
        
        if (!softMax.empty())
        {
            std::copy(softMax.begin(), softMax.end() - 1, std::ostream_iterator<SingleContour>(out, "},{"));
            std::copy(softMax.end()-1, softMax.end(), std::ostream_iterator<SingleContour>(out, ""));
        }
        out << "}}\"";out << ",";
    // Enclose sequence in double quotations and curly brackets to keep CSV line length uniform
        
        out << "\"{{";
        
        if (!hardMin.empty())
        {
            std::copy(hardMin.begin(), hardMin.end() - 1, std::ostream_iterator<SingleContour>(out, "},{"));
            std::copy(hardMin.end()-1, hardMin.end(), std::ostream_iterator<SingleContour>(out, ""));
        }
        out << "}}\"";out << ",";
    // Enclose sequence in double quotations and curly brackets to keep CSV line length uniform
        
        out << "\"{{";
        
        if (!softMin.empty())
        {
            std::copy(softMin.begin(), softMin.end() - 1, std::ostream_iterator<SingleContour>(out, "},{"));
            std::copy(softMin.end()-1, softMin.end(), std::ostream_iterator<SingleContour>(out, ""));
        }
        out << "}}\"";
    
}

std::stringstream HardAndSoftConstraints::toStringstream() const
{
    std::stringstream out;
    toCsv(out);
    return out;
}

template<class Archive>
    void HardAndSoftConstraints::serialize(Archive& ar, [[maybe_unused]] unsigned int version)
    {

        ar& CEREAL_NVP(sRef);
        ar& CEREAL_NVP(hardMax);
        ar& CEREAL_NVP(softMax);
        ar& CEREAL_NVP(hardMin);
        ar& CEREAL_NVP(softMin);
    }


template void HardAndSoftConstraints::serialize(cereal::BinaryInputArchive& ar, unsigned int version);
template void HardAndSoftConstraints::serialize(cereal::BinaryOutputArchive& ar, unsigned int version);
template void HardAndSoftConstraints::serialize(cereal::IntrospectionArchive& ar, unsigned int version);
template void HardAndSoftConstraints::serialize(cereal::JSONInputArchive& ar, unsigned int version);
template void HardAndSoftConstraints::serialize(cereal::JSONOutputArchive& ar, unsigned int version);
template void HardAndSoftConstraints::serialize(cereal::PortableBinaryInputArchive& ar, unsigned int version);
template void HardAndSoftConstraints::serialize(cereal::PortableBinaryOutputArchive& ar, unsigned int version);

std::ostream& operator<< (std::ostream& os, const HardAndSoftConstraints& hardandsoftconstraints)
{
    hardandsoftconstraints.toCsv(os);
    return os;
}

bool operator== (const HardAndSoftConstraints &lhs, const HardAndSoftConstraints &rhs)
{
    return (  lhs.sRef== rhs.sRef &&
     lhs.hardMax== rhs.hardMax &&
     lhs.softMax== rhs.softMax &&
     lhs.hardMin== rhs.hardMin &&
     lhs.softMin== rhs.softMin 
    );
}

std::size_t HardAndSoftConstraints::hash() const noexcept
{
    std::size_t result = 17;
    result = result * 31 + std::hash<double>()(sRef);
    result = result * 31 + hashVector(hardMax);
    result = result * 31 + hashVector(softMax);
    result = result * 31 + hashVector(hardMin);
    result = result * 31 + hashVector(softMin);
    return result;
}

 } // namespace av
 } // namespace msg


CEREAL_BIND_TO_ARCHIVES(av::msg::HardAndSoftConstraints)

 namespace av {
 namespace msg {


StaticLongitudinalSquaredProximityRelation::StaticLongitudinalSquaredProximityRelation() = default;

StaticLongitudinalSquaredProximityRelation::StaticLongitudinalSquaredProximityRelation(
    double ego_max_decel_arg ,
    double comfort_decel_arg 
    ):
    ego_max_decel { std::move(ego_max_decel_arg) } // NOLINT
      ,
    comfort_decel { std::move(comfort_decel_arg) } // NOLINT
      
    {
}

StaticLongitudinalSquaredProximityRelation::StaticLongitudinalSquaredProximityRelation(const StaticLongitudinalSquaredProximityRelation&) = default;

// NOLINTNEXTLINE
StaticLongitudinalSquaredProximityRelation::StaticLongitudinalSquaredProximityRelation(StaticLongitudinalSquaredProximityRelation&&) = default;

StaticLongitudinalSquaredProximityRelation& StaticLongitudinalSquaredProximityRelation::operator=(const StaticLongitudinalSquaredProximityRelation&) = default;

// NOLINTNEXTLINE
StaticLongitudinalSquaredProximityRelation& StaticLongitudinalSquaredProximityRelation::operator=(StaticLongitudinalSquaredProximityRelation&&) = default;

void StaticLongitudinalSquaredProximityRelation::toCsv(std::ostream& out) const
{
    out.precision(16);
    out << std::boolalpha;
    out << ego_max_decel;out << ",";
    out << comfort_decel;
    
}

std::stringstream StaticLongitudinalSquaredProximityRelation::toStringstream() const
{
    std::stringstream out;
    toCsv(out);
    return out;
}

template<class Archive>
    void StaticLongitudinalSquaredProximityRelation::serialize(Archive& ar, [[maybe_unused]] unsigned int version)
    {

        ar& CEREAL_NVP(ego_max_decel);
        ar& CEREAL_NVP(comfort_decel);
    }


template void StaticLongitudinalSquaredProximityRelation::serialize(cereal::BinaryInputArchive& ar, unsigned int version);
template void StaticLongitudinalSquaredProximityRelation::serialize(cereal::BinaryOutputArchive& ar, unsigned int version);
template void StaticLongitudinalSquaredProximityRelation::serialize(cereal::IntrospectionArchive& ar, unsigned int version);
template void StaticLongitudinalSquaredProximityRelation::serialize(cereal::JSONInputArchive& ar, unsigned int version);
template void StaticLongitudinalSquaredProximityRelation::serialize(cereal::JSONOutputArchive& ar, unsigned int version);
template void StaticLongitudinalSquaredProximityRelation::serialize(cereal::PortableBinaryInputArchive& ar, unsigned int version);
template void StaticLongitudinalSquaredProximityRelation::serialize(cereal::PortableBinaryOutputArchive& ar, unsigned int version);

std::ostream& operator<< (std::ostream& os, const StaticLongitudinalSquaredProximityRelation& staticlongitudinalsquaredproximityrelation)
{
    staticlongitudinalsquaredproximityrelation.toCsv(os);
    return os;
}

bool operator== (const StaticLongitudinalSquaredProximityRelation &lhs, const StaticLongitudinalSquaredProximityRelation &rhs)
{
    return (  lhs.ego_max_decel== rhs.ego_max_decel &&
     lhs.comfort_decel== rhs.comfort_decel 
    );
}

std::size_t StaticLongitudinalSquaredProximityRelation::hash() const noexcept
{
    std::size_t result = 17;
    result = result * 31 + std::hash<double>()(ego_max_decel);
    result = result * 31 + std::hash<double>()(comfort_decel);
    return result;
}

 } // namespace av
 } // namespace msg


CEREAL_BIND_TO_ARCHIVES(av::msg::StaticLongitudinalSquaredProximityRelation)

 namespace av {
 namespace msg {


LateralSigmoidProximityRelation::LateralSigmoidProximityRelation() = default;

LateralSigmoidProximityRelation::LateralSigmoidProximityRelation(
    double minimum_clearance_tube_arg ,
    double sigmoid_a_arg ,
    double sigmoid_c_arg ,
    double car_half_width_arg 
    ):
    minimum_clearance_tube { std::move(minimum_clearance_tube_arg) } // NOLINT
      ,
    sigmoid_a { std::move(sigmoid_a_arg) } // NOLINT
      ,
    sigmoid_c { std::move(sigmoid_c_arg) } // NOLINT
      ,
    car_half_width { std::move(car_half_width_arg) } // NOLINT
      
    {
}

LateralSigmoidProximityRelation::LateralSigmoidProximityRelation(const LateralSigmoidProximityRelation&) = default;

// NOLINTNEXTLINE
LateralSigmoidProximityRelation::LateralSigmoidProximityRelation(LateralSigmoidProximityRelation&&) = default;

LateralSigmoidProximityRelation& LateralSigmoidProximityRelation::operator=(const LateralSigmoidProximityRelation&) = default;

// NOLINTNEXTLINE
LateralSigmoidProximityRelation& LateralSigmoidProximityRelation::operator=(LateralSigmoidProximityRelation&&) = default;

void LateralSigmoidProximityRelation::toCsv(std::ostream& out) const
{
    out.precision(16);
    out << std::boolalpha;
    out << minimum_clearance_tube;out << ",";
    out << sigmoid_a;out << ",";
    out << sigmoid_c;out << ",";
    out << car_half_width;
    
}

std::stringstream LateralSigmoidProximityRelation::toStringstream() const
{
    std::stringstream out;
    toCsv(out);
    return out;
}

template<class Archive>
    void LateralSigmoidProximityRelation::serialize(Archive& ar, [[maybe_unused]] unsigned int version)
    {

        ar& CEREAL_NVP(minimum_clearance_tube);
        ar& CEREAL_NVP(sigmoid_a);
        ar& CEREAL_NVP(sigmoid_c);
        ar& CEREAL_NVP(car_half_width);
    }


template void LateralSigmoidProximityRelation::serialize(cereal::BinaryInputArchive& ar, unsigned int version);
template void LateralSigmoidProximityRelation::serialize(cereal::BinaryOutputArchive& ar, unsigned int version);
template void LateralSigmoidProximityRelation::serialize(cereal::IntrospectionArchive& ar, unsigned int version);
template void LateralSigmoidProximityRelation::serialize(cereal::JSONInputArchive& ar, unsigned int version);
template void LateralSigmoidProximityRelation::serialize(cereal::JSONOutputArchive& ar, unsigned int version);
template void LateralSigmoidProximityRelation::serialize(cereal::PortableBinaryInputArchive& ar, unsigned int version);
template void LateralSigmoidProximityRelation::serialize(cereal::PortableBinaryOutputArchive& ar, unsigned int version);

std::ostream& operator<< (std::ostream& os, const LateralSigmoidProximityRelation& lateralsigmoidproximityrelation)
{
    lateralsigmoidproximityrelation.toCsv(os);
    return os;
}

bool operator== (const LateralSigmoidProximityRelation &lhs, const LateralSigmoidProximityRelation &rhs)
{
    return (  lhs.minimum_clearance_tube== rhs.minimum_clearance_tube &&
     lhs.sigmoid_a== rhs.sigmoid_a &&
     lhs.sigmoid_c== rhs.sigmoid_c &&
     lhs.car_half_width== rhs.car_half_width 
    );
}

std::size_t LateralSigmoidProximityRelation::hash() const noexcept
{
    std::size_t result = 17;
    result = result * 31 + std::hash<double>()(minimum_clearance_tube);
    result = result * 31 + std::hash<double>()(sigmoid_a);
    result = result * 31 + std::hash<double>()(sigmoid_c);
    result = result * 31 + std::hash<double>()(car_half_width);
    return result;
}

 } // namespace av
 } // namespace msg


CEREAL_BIND_TO_ARCHIVES(av::msg::LateralSigmoidProximityRelation)

 namespace av {
 namespace msg {


ObjectProximityParams::ObjectProximityParams() = default;

ObjectProximityParams::ObjectProximityParams(
    double front_distance_buffer_arg ,
    double minimum_clearance_tube_radius_arg ,
    double ego_max_decel_arg ,
    double comfort_decel_arg ,
    SpeedConstraintType speed_constraint_type_arg ,
    StaticLongitudinalSquaredProximityRelation longitudinal_proximity_relation_arg ,
    LateralSigmoidProximityRelation lateral_proximity_relation_arg ,
    double perpendicular_to_path_range_arg ,
    double perpendicular_tracks_speed_threshold_arg ,
    double frontbumper_lookback_m_arg 
    ):
    front_distance_buffer { std::move(front_distance_buffer_arg) } // NOLINT
      ,
    minimum_clearance_tube_radius { std::move(minimum_clearance_tube_radius_arg) } // NOLINT
      ,
    ego_max_decel { std::move(ego_max_decel_arg) } // NOLINT
      ,
    comfort_decel { std::move(comfort_decel_arg) } // NOLINT
      ,
    speed_constraint_type { std::move(speed_constraint_type_arg) } // NOLINT
      ,
    longitudinal_proximity_relation { std::move(longitudinal_proximity_relation_arg) } // NOLINT
      ,
    lateral_proximity_relation { std::move(lateral_proximity_relation_arg) } // NOLINT
      ,
    perpendicular_to_path_range { std::move(perpendicular_to_path_range_arg) } // NOLINT
      ,
    perpendicular_tracks_speed_threshold { std::move(perpendicular_tracks_speed_threshold_arg) } // NOLINT
      ,
    frontbumper_lookback_m { std::move(frontbumper_lookback_m_arg) } // NOLINT
      
    {
}

ObjectProximityParams::ObjectProximityParams(const ObjectProximityParams&) = default;

// NOLINTNEXTLINE
ObjectProximityParams::ObjectProximityParams(ObjectProximityParams&&) = default;

ObjectProximityParams& ObjectProximityParams::operator=(const ObjectProximityParams&) = default;

// NOLINTNEXTLINE
ObjectProximityParams& ObjectProximityParams::operator=(ObjectProximityParams&&) = default;

void ObjectProximityParams::toCsv(std::ostream& out) const
{
    out.precision(16);
    out << std::boolalpha;
    out << front_distance_buffer;out << ",";
    out << minimum_clearance_tube_radius;out << ",";
    out << ego_max_decel;out << ",";
    out << comfort_decel;out << ",";
    out << static_cast<uint32_t>(speed_constraint_type);out << ",";
    out << longitudinal_proximity_relation;out << ",";
    out << lateral_proximity_relation;out << ",";
    out << perpendicular_to_path_range;out << ",";
    out << perpendicular_tracks_speed_threshold;out << ",";
    out << frontbumper_lookback_m;
    
}

std::stringstream ObjectProximityParams::toStringstream() const
{
    std::stringstream out;
    toCsv(out);
    return out;
}

template<class Archive>
    void ObjectProximityParams::serialize(Archive& ar, [[maybe_unused]] unsigned int version)
    {

        ar& CEREAL_NVP(front_distance_buffer);
        ar& CEREAL_NVP(minimum_clearance_tube_radius);
        ar& CEREAL_NVP(ego_max_decel);
        ar& CEREAL_NVP(comfort_decel);
        ar& CEREAL_NVP(speed_constraint_type);
        ar& CEREAL_NVP(longitudinal_proximity_relation);
        ar& CEREAL_NVP(lateral_proximity_relation);
        ar& CEREAL_NVP(perpendicular_to_path_range);
        ar& CEREAL_NVP(perpendicular_tracks_speed_threshold);
        ar& CEREAL_NVP(frontbumper_lookback_m);
    }


template void ObjectProximityParams::serialize(cereal::BinaryInputArchive& ar, unsigned int version);
template void ObjectProximityParams::serialize(cereal::BinaryOutputArchive& ar, unsigned int version);
template void ObjectProximityParams::serialize(cereal::IntrospectionArchive& ar, unsigned int version);
template void ObjectProximityParams::serialize(cereal::JSONInputArchive& ar, unsigned int version);
template void ObjectProximityParams::serialize(cereal::JSONOutputArchive& ar, unsigned int version);
template void ObjectProximityParams::serialize(cereal::PortableBinaryInputArchive& ar, unsigned int version);
template void ObjectProximityParams::serialize(cereal::PortableBinaryOutputArchive& ar, unsigned int version);

std::ostream& operator<< (std::ostream& os, const ObjectProximityParams& objectproximityparams)
{
    objectproximityparams.toCsv(os);
    return os;
}

bool operator== (const ObjectProximityParams &lhs, const ObjectProximityParams &rhs)
{
    return (  lhs.front_distance_buffer== rhs.front_distance_buffer &&
     lhs.minimum_clearance_tube_radius== rhs.minimum_clearance_tube_radius &&
     lhs.ego_max_decel== rhs.ego_max_decel &&
     lhs.comfort_decel== rhs.comfort_decel &&
     lhs.speed_constraint_type== rhs.speed_constraint_type &&
     lhs.longitudinal_proximity_relation== rhs.longitudinal_proximity_relation &&
     lhs.lateral_proximity_relation== rhs.lateral_proximity_relation &&
     lhs.perpendicular_to_path_range== rhs.perpendicular_to_path_range &&
     lhs.perpendicular_tracks_speed_threshold== rhs.perpendicular_tracks_speed_threshold &&
     lhs.frontbumper_lookback_m== rhs.frontbumper_lookback_m 
    );
}

std::size_t ObjectProximityParams::hash() const noexcept
{
    std::size_t result = 17;
    result = result * 31 + std::hash<double>()(front_distance_buffer);
    result = result * 31 + std::hash<double>()(minimum_clearance_tube_radius);
    result = result * 31 + std::hash<double>()(ego_max_decel);
    result = result * 31 + std::hash<double>()(comfort_decel);
    result = result * 31 + std::hash<SpeedConstraintType>()(speed_constraint_type);
    result = result * 31 + std::hash<StaticLongitudinalSquaredProximityRelation>()(longitudinal_proximity_relation);
    result = result * 31 + std::hash<LateralSigmoidProximityRelation>()(lateral_proximity_relation);
    result = result * 31 + std::hash<double>()(perpendicular_to_path_range);
    result = result * 31 + std::hash<double>()(perpendicular_tracks_speed_threshold);
    result = result * 31 + std::hash<double>()(frontbumper_lookback_m);
    return result;
}

 } // namespace av
 } // namespace msg


CEREAL_BIND_TO_ARCHIVES(av::msg::ObjectProximityParams)

 namespace av {
 namespace msg {


EdgeClearanceInfo::EdgeClearanceInfo() = default;

EdgeClearanceInfo::EdgeClearanceInfo(
    double startDistanceAlongTransition_arg ,
    double endDistanceAlongTransition_arg ,
    double lateralDistanceToTransition_arg ,
    double longitudinalSpeed_arg ,
    double transitionLength_arg ,
    double headingDiffToTransition_arg 
    ):
    startDistanceAlongTransition { std::move(startDistanceAlongTransition_arg) } // NOLINT
      ,
    endDistanceAlongTransition { std::move(endDistanceAlongTransition_arg) } // NOLINT
      ,
    lateralDistanceToTransition { std::move(lateralDistanceToTransition_arg) } // NOLINT
      ,
    longitudinalSpeed { std::move(longitudinalSpeed_arg) } // NOLINT
      ,
    transitionLength { std::move(transitionLength_arg) } // NOLINT
      ,
    headingDiffToTransition { std::move(headingDiffToTransition_arg) } // NOLINT
      
    {
}

EdgeClearanceInfo::EdgeClearanceInfo(const EdgeClearanceInfo&) = default;

// NOLINTNEXTLINE
EdgeClearanceInfo::EdgeClearanceInfo(EdgeClearanceInfo&&) = default;

EdgeClearanceInfo& EdgeClearanceInfo::operator=(const EdgeClearanceInfo&) = default;

// NOLINTNEXTLINE
EdgeClearanceInfo& EdgeClearanceInfo::operator=(EdgeClearanceInfo&&) = default;

void EdgeClearanceInfo::toCsv(std::ostream& out) const
{
    out.precision(16);
    out << std::boolalpha;
    out << startDistanceAlongTransition;out << ",";
    out << endDistanceAlongTransition;out << ",";
    out << lateralDistanceToTransition;out << ",";
    out << longitudinalSpeed;out << ",";
    out << transitionLength;out << ",";
    out << headingDiffToTransition;
    
}

std::stringstream EdgeClearanceInfo::toStringstream() const
{
    std::stringstream out;
    toCsv(out);
    return out;
}

template<class Archive>
    void EdgeClearanceInfo::serialize(Archive& ar, [[maybe_unused]] unsigned int version)
    {

        ar& CEREAL_NVP(startDistanceAlongTransition);
        ar& CEREAL_NVP(endDistanceAlongTransition);
        ar& CEREAL_NVP(lateralDistanceToTransition);
        ar& CEREAL_NVP(longitudinalSpeed);
        ar& CEREAL_NVP(transitionLength);
        ar& CEREAL_NVP(headingDiffToTransition);
    }


template void EdgeClearanceInfo::serialize(cereal::BinaryInputArchive& ar, unsigned int version);
template void EdgeClearanceInfo::serialize(cereal::BinaryOutputArchive& ar, unsigned int version);
template void EdgeClearanceInfo::serialize(cereal::IntrospectionArchive& ar, unsigned int version);
template void EdgeClearanceInfo::serialize(cereal::JSONInputArchive& ar, unsigned int version);
template void EdgeClearanceInfo::serialize(cereal::JSONOutputArchive& ar, unsigned int version);
template void EdgeClearanceInfo::serialize(cereal::PortableBinaryInputArchive& ar, unsigned int version);
template void EdgeClearanceInfo::serialize(cereal::PortableBinaryOutputArchive& ar, unsigned int version);

std::ostream& operator<< (std::ostream& os, const EdgeClearanceInfo& edgeclearanceinfo)
{
    edgeclearanceinfo.toCsv(os);
    return os;
}

bool operator== (const EdgeClearanceInfo &lhs, const EdgeClearanceInfo &rhs)
{
    return (  lhs.startDistanceAlongTransition== rhs.startDistanceAlongTransition &&
     lhs.endDistanceAlongTransition== rhs.endDistanceAlongTransition &&
     lhs.lateralDistanceToTransition== rhs.lateralDistanceToTransition &&
     lhs.longitudinalSpeed== rhs.longitudinalSpeed &&
     lhs.transitionLength== rhs.transitionLength &&
     lhs.headingDiffToTransition== rhs.headingDiffToTransition 
    );
}

std::size_t EdgeClearanceInfo::hash() const noexcept
{
    std::size_t result = 17;
    result = result * 31 + std::hash<double>()(startDistanceAlongTransition);
    result = result * 31 + std::hash<double>()(endDistanceAlongTransition);
    result = result * 31 + std::hash<double>()(lateralDistanceToTransition);
    result = result * 31 + std::hash<double>()(longitudinalSpeed);
    result = result * 31 + std::hash<double>()(transitionLength);
    result = result * 31 + std::hash<double>()(headingDiffToTransition);
    return result;
}

 } // namespace av
 } // namespace msg


CEREAL_BIND_TO_ARCHIVES(av::msg::EdgeClearanceInfo)

 namespace av {
 namespace msg {


PlaceHolder::PlaceHolder() = default;

PlaceHolder::PlaceHolder(
    ObjectProximityParams proximityParams_arg ,
    EdgeClearanceInfo edgeClearanceInfo_arg 
    ):
    proximityParams { std::move(proximityParams_arg) } // NOLINT
      ,
    edgeClearanceInfo { std::move(edgeClearanceInfo_arg) } // NOLINT
      
    {
}

PlaceHolder::PlaceHolder(const PlaceHolder&) = default;

// NOLINTNEXTLINE
PlaceHolder::PlaceHolder(PlaceHolder&&) = default;

PlaceHolder& PlaceHolder::operator=(const PlaceHolder&) = default;

// NOLINTNEXTLINE
PlaceHolder& PlaceHolder::operator=(PlaceHolder&&) = default;

void PlaceHolder::toCsv(std::ostream& out) const
{
    out.precision(16);
    out << std::boolalpha;
    out << proximityParams;out << ",";
    out << edgeClearanceInfo;
    
}

std::stringstream PlaceHolder::toStringstream() const
{
    std::stringstream out;
    toCsv(out);
    return out;
}

template<class Archive>
    void PlaceHolder::serialize(Archive& ar, [[maybe_unused]] unsigned int version)
    {

        ar& CEREAL_NVP(proximityParams);
        ar& CEREAL_NVP(edgeClearanceInfo);
    }


template void PlaceHolder::serialize(cereal::BinaryInputArchive& ar, unsigned int version);
template void PlaceHolder::serialize(cereal::BinaryOutputArchive& ar, unsigned int version);
template void PlaceHolder::serialize(cereal::IntrospectionArchive& ar, unsigned int version);
template void PlaceHolder::serialize(cereal::JSONInputArchive& ar, unsigned int version);
template void PlaceHolder::serialize(cereal::JSONOutputArchive& ar, unsigned int version);
template void PlaceHolder::serialize(cereal::PortableBinaryInputArchive& ar, unsigned int version);
template void PlaceHolder::serialize(cereal::PortableBinaryOutputArchive& ar, unsigned int version);

std::ostream& operator<< (std::ostream& os, const PlaceHolder& placeholder)
{
    placeholder.toCsv(os);
    return os;
}

bool operator== (const PlaceHolder &lhs, const PlaceHolder &rhs)
{
    return (  lhs.proximityParams== rhs.proximityParams &&
     lhs.edgeClearanceInfo== rhs.edgeClearanceInfo 
    );
}

std::size_t PlaceHolder::hash() const noexcept
{
    std::size_t result = 17;
    result = result * 31 + std::hash<ObjectProximityParams>()(proximityParams);
    result = result * 31 + std::hash<EdgeClearanceInfo>()(edgeClearanceInfo);
    return result;
}

 } // namespace av
 } // namespace msg


CEREAL_BIND_TO_ARCHIVES(av::msg::PlaceHolder)

 namespace av {
 namespace msg {


ProximityBiasingParameters::ProximityBiasingParameters() = default;

ProximityBiasingParameters::ProximityBiasingParameters(
    SpeedConstraintType speedConstraintType_arg ,
    double longitudinalProximityRelationComfortDecel_arg ,
    double longitudinalProximityRelationEgoMaxDecel_arg ,
    double lateralProximityRelationMinimumClearanceTube_arg ,
    double lateralProximityRelationParamA_arg ,
    double lateralProximityRelationParamC_arg ,
    double lateralProximityRelationCarHalfWidth_arg 
    ):
    speedConstraintType { std::move(speedConstraintType_arg) } // NOLINT
      ,
    longitudinalProximityRelationComfortDecel { std::move(longitudinalProximityRelationComfortDecel_arg) } // NOLINT
      ,
    longitudinalProximityRelationEgoMaxDecel { std::move(longitudinalProximityRelationEgoMaxDecel_arg) } // NOLINT
      ,
    lateralProximityRelationMinimumClearanceTube { std::move(lateralProximityRelationMinimumClearanceTube_arg) } // NOLINT
      ,
    lateralProximityRelationParamA { std::move(lateralProximityRelationParamA_arg) } // NOLINT
      ,
    lateralProximityRelationParamC { std::move(lateralProximityRelationParamC_arg) } // NOLINT
      ,
    lateralProximityRelationCarHalfWidth { std::move(lateralProximityRelationCarHalfWidth_arg) } // NOLINT
      
    {
}

ProximityBiasingParameters::ProximityBiasingParameters(const ProximityBiasingParameters&) = default;

// NOLINTNEXTLINE
ProximityBiasingParameters::ProximityBiasingParameters(ProximityBiasingParameters&&) = default;

ProximityBiasingParameters& ProximityBiasingParameters::operator=(const ProximityBiasingParameters&) = default;

// NOLINTNEXTLINE
ProximityBiasingParameters& ProximityBiasingParameters::operator=(ProximityBiasingParameters&&) = default;

void ProximityBiasingParameters::toCsv(std::ostream& out) const
{
    out.precision(16);
    out << std::boolalpha;
    out << static_cast<uint32_t>(speedConstraintType);out << ",";
    out << longitudinalProximityRelationComfortDecel;out << ",";
    out << longitudinalProximityRelationEgoMaxDecel;out << ",";
    out << lateralProximityRelationMinimumClearanceTube;out << ",";
    out << lateralProximityRelationParamA;out << ",";
    out << lateralProximityRelationParamC;out << ",";
    out << lateralProximityRelationCarHalfWidth;
    
}

std::stringstream ProximityBiasingParameters::toStringstream() const
{
    std::stringstream out;
    toCsv(out);
    return out;
}

template<class Archive>
    void ProximityBiasingParameters::serialize(Archive& ar, [[maybe_unused]] unsigned int version)
    {

        ar& CEREAL_NVP(speedConstraintType);
        ar& CEREAL_NVP(longitudinalProximityRelationComfortDecel);
        ar& CEREAL_NVP(longitudinalProximityRelationEgoMaxDecel);
        ar& CEREAL_NVP(lateralProximityRelationMinimumClearanceTube);
        ar& CEREAL_NVP(lateralProximityRelationParamA);
        ar& CEREAL_NVP(lateralProximityRelationParamC);
        ar& CEREAL_NVP(lateralProximityRelationCarHalfWidth);
    }


template void ProximityBiasingParameters::serialize(cereal::BinaryInputArchive& ar, unsigned int version);
template void ProximityBiasingParameters::serialize(cereal::BinaryOutputArchive& ar, unsigned int version);
template void ProximityBiasingParameters::serialize(cereal::IntrospectionArchive& ar, unsigned int version);
template void ProximityBiasingParameters::serialize(cereal::JSONInputArchive& ar, unsigned int version);
template void ProximityBiasingParameters::serialize(cereal::JSONOutputArchive& ar, unsigned int version);
template void ProximityBiasingParameters::serialize(cereal::PortableBinaryInputArchive& ar, unsigned int version);
template void ProximityBiasingParameters::serialize(cereal::PortableBinaryOutputArchive& ar, unsigned int version);

std::ostream& operator<< (std::ostream& os, const ProximityBiasingParameters& proximitybiasingparameters)
{
    proximitybiasingparameters.toCsv(os);
    return os;
}

bool operator== (const ProximityBiasingParameters &lhs, const ProximityBiasingParameters &rhs)
{
    return (  lhs.speedConstraintType== rhs.speedConstraintType &&
     lhs.longitudinalProximityRelationComfortDecel== rhs.longitudinalProximityRelationComfortDecel &&
     lhs.longitudinalProximityRelationEgoMaxDecel== rhs.longitudinalProximityRelationEgoMaxDecel &&
     lhs.lateralProximityRelationMinimumClearanceTube== rhs.lateralProximityRelationMinimumClearanceTube &&
     lhs.lateralProximityRelationParamA== rhs.lateralProximityRelationParamA &&
     lhs.lateralProximityRelationParamC== rhs.lateralProximityRelationParamC &&
     lhs.lateralProximityRelationCarHalfWidth== rhs.lateralProximityRelationCarHalfWidth 
    );
}

std::size_t ProximityBiasingParameters::hash() const noexcept
{
    std::size_t result = 17;
    result = result * 31 + std::hash<SpeedConstraintType>()(speedConstraintType);
    result = result * 31 + std::hash<double>()(longitudinalProximityRelationComfortDecel);
    result = result * 31 + std::hash<double>()(longitudinalProximityRelationEgoMaxDecel);
    result = result * 31 + std::hash<double>()(lateralProximityRelationMinimumClearanceTube);
    result = result * 31 + std::hash<double>()(lateralProximityRelationParamA);
    result = result * 31 + std::hash<double>()(lateralProximityRelationParamC);
    result = result * 31 + std::hash<double>()(lateralProximityRelationCarHalfWidth);
    return result;
}

 } // namespace av
 } // namespace msg


CEREAL_BIND_TO_ARCHIVES(av::msg::ProximityBiasingParameters)

 namespace av {
 namespace msg {


LinearContour::LinearContour() = default;

LinearContour::LinearContour(
    ContinuityType continuityType_arg ,
    uint8_t constraintType_arg ,
    double sMin_arg ,
    double sMax_arg ,
    double sSet_arg ,
    double constrVar_arg ,
    double rate_arg ,
    bool trackExist_arg ,
    SpeedConstraintTrackInfo trackInfo_arg ,
    bool mapObjectExist_arg ,
    SpeedConstraintMapInfo mapInfo_arg ,
    bool biasingInfoExists_arg ,
    PlaceHolder biasingInfo_arg 
    ):
    continuityType { std::move(continuityType_arg) } // NOLINT
      ,
    constraintType { std::move(constraintType_arg) } // NOLINT
      ,
    sMin { std::move(sMin_arg) } // NOLINT
      ,
    sMax { std::move(sMax_arg) } // NOLINT
      ,
    sSet { std::move(sSet_arg) } // NOLINT
      ,
    constrVar { std::move(constrVar_arg) } // NOLINT
      ,
    rate { std::move(rate_arg) } // NOLINT
      ,
    trackExist { std::move(trackExist_arg) } // NOLINT
      ,
    trackInfo { std::move(trackInfo_arg) } // NOLINT
      ,
    mapObjectExist { std::move(mapObjectExist_arg) } // NOLINT
      ,
    mapInfo { std::move(mapInfo_arg) } // NOLINT
      ,
    biasingInfoExists { std::move(biasingInfoExists_arg) } // NOLINT
      ,
    biasingInfo { std::move(biasingInfo_arg) } // NOLINT
      
    {
}

LinearContour::LinearContour(const LinearContour&) = default;

// NOLINTNEXTLINE
LinearContour::LinearContour(LinearContour&&) = default;

LinearContour& LinearContour::operator=(const LinearContour&) = default;

// NOLINTNEXTLINE
LinearContour& LinearContour::operator=(LinearContour&&) = default;

void LinearContour::toCsv(std::ostream& out) const
{
    out.precision(16);
    out << std::boolalpha;
    out << static_cast<uint32_t>(continuityType);out << ",";
    // Cast to uint16_t to print as integer instead of char
        out << static_cast<uint16_t>(constraintType);out << ",";
    out << sMin;out << ",";
    out << sMax;out << ",";
    out << sSet;out << ",";
    out << constrVar;out << ",";
    out << rate;out << ",";
    out << trackExist;out << ",";
    out << trackInfo;out << ",";
    out << mapObjectExist;out << ",";
    out << mapInfo;out << ",";
    out << biasingInfoExists;out << ",";
    out << biasingInfo;
    
}

std::stringstream LinearContour::toStringstream() const
{
    std::stringstream out;
    toCsv(out);
    return out;
}

template<class Archive>
    void LinearContour::serialize(Archive& ar, [[maybe_unused]] unsigned int version)
    {

        ar& CEREAL_NVP(continuityType);
        ar& CEREAL_NVP(constraintType);
        ar& CEREAL_NVP(sMin);
        ar& CEREAL_NVP(sMax);
        ar& CEREAL_NVP(sSet);
        ar& CEREAL_NVP(constrVar);
        ar& CEREAL_NVP(rate);
        ar& CEREAL_NVP(trackExist);
        ar& CEREAL_NVP(trackInfo);
        ar& CEREAL_NVP(mapObjectExist);
        ar& CEREAL_NVP(mapInfo);
        ar& CEREAL_NVP(biasingInfoExists);
        ar& CEREAL_NVP(biasingInfo);
    }


template void LinearContour::serialize(cereal::BinaryInputArchive& ar, unsigned int version);
template void LinearContour::serialize(cereal::BinaryOutputArchive& ar, unsigned int version);
template void LinearContour::serialize(cereal::IntrospectionArchive& ar, unsigned int version);
template void LinearContour::serialize(cereal::JSONInputArchive& ar, unsigned int version);
template void LinearContour::serialize(cereal::JSONOutputArchive& ar, unsigned int version);
template void LinearContour::serialize(cereal::PortableBinaryInputArchive& ar, unsigned int version);
template void LinearContour::serialize(cereal::PortableBinaryOutputArchive& ar, unsigned int version);

std::ostream& operator<< (std::ostream& os, const LinearContour& linearcontour)
{
    linearcontour.toCsv(os);
    return os;
}

bool operator== (const LinearContour &lhs, const LinearContour &rhs)
{
    return (  lhs.continuityType== rhs.continuityType &&
     lhs.constraintType== rhs.constraintType &&
     lhs.sMin== rhs.sMin &&
     lhs.sMax== rhs.sMax &&
     lhs.sSet== rhs.sSet &&
     lhs.constrVar== rhs.constrVar &&
     lhs.rate== rhs.rate &&
     lhs.trackExist== rhs.trackExist &&
     lhs.trackInfo== rhs.trackInfo &&
     lhs.mapObjectExist== rhs.mapObjectExist &&
     lhs.mapInfo== rhs.mapInfo &&
     lhs.biasingInfoExists== rhs.biasingInfoExists &&
     lhs.biasingInfo== rhs.biasingInfo 
    );
}

std::size_t LinearContour::hash() const noexcept
{
    std::size_t result = 17;
    result = result * 31 + std::hash<ContinuityType>()(continuityType);
    result = result * 31 + std::hash<uint8_t>()(constraintType);
    result = result * 31 + std::hash<double>()(sMin);
    result = result * 31 + std::hash<double>()(sMax);
    result = result * 31 + std::hash<double>()(sSet);
    result = result * 31 + std::hash<double>()(constrVar);
    result = result * 31 + std::hash<double>()(rate);
    result = result * 31 + std::hash<bool>()(trackExist);
    result = result * 31 + std::hash<SpeedConstraintTrackInfo>()(trackInfo);
    result = result * 31 + std::hash<bool>()(mapObjectExist);
    result = result * 31 + std::hash<SpeedConstraintMapInfo>()(mapInfo);
    result = result * 31 + std::hash<bool>()(biasingInfoExists);
    result = result * 31 + std::hash<PlaceHolder>()(biasingInfo);
    return result;
}

 } // namespace av
 } // namespace msg


CEREAL_BIND_TO_ARCHIVES(av::msg::LinearContour)

 namespace av {
 namespace msg {


QuadraticContour::QuadraticContour() = default;

QuadraticContour::QuadraticContour(
    ContinuityType continuityType_arg ,
    uint8_t constraintType_arg ,
    double sMin_arg ,
    double sMax_arg ,
    double sSet_arg ,
    double constrVar_arg ,
    double constrRate_arg ,
    double acceleration_arg ,
    bool trackExist_arg ,
    SpeedConstraintTrackInfo trackInfo_arg ,
    bool mapObjectExist_arg ,
    SpeedConstraintMapInfo mapInfo_arg ,
    bool biasingInfoExists_arg ,
    PlaceHolder biasingInfo_arg 
    ):
    continuityType { std::move(continuityType_arg) } // NOLINT
      ,
    constraintType { std::move(constraintType_arg) } // NOLINT
      ,
    sMin { std::move(sMin_arg) } // NOLINT
      ,
    sMax { std::move(sMax_arg) } // NOLINT
      ,
    sSet { std::move(sSet_arg) } // NOLINT
      ,
    constrVar { std::move(constrVar_arg) } // NOLINT
      ,
    constrRate { std::move(constrRate_arg) } // NOLINT
      ,
    acceleration { std::move(acceleration_arg) } // NOLINT
      ,
    trackExist { std::move(trackExist_arg) } // NOLINT
      ,
    trackInfo { std::move(trackInfo_arg) } // NOLINT
      ,
    mapObjectExist { std::move(mapObjectExist_arg) } // NOLINT
      ,
    mapInfo { std::move(mapInfo_arg) } // NOLINT
      ,
    biasingInfoExists { std::move(biasingInfoExists_arg) } // NOLINT
      ,
    biasingInfo { std::move(biasingInfo_arg) } // NOLINT
      
    {
}

QuadraticContour::QuadraticContour(const QuadraticContour&) = default;

// NOLINTNEXTLINE
QuadraticContour::QuadraticContour(QuadraticContour&&) = default;

QuadraticContour& QuadraticContour::operator=(const QuadraticContour&) = default;

// NOLINTNEXTLINE
QuadraticContour& QuadraticContour::operator=(QuadraticContour&&) = default;

void QuadraticContour::toCsv(std::ostream& out) const
{
    out.precision(16);
    out << std::boolalpha;
    out << static_cast<uint32_t>(continuityType);out << ",";
    // Cast to uint16_t to print as integer instead of char
        out << static_cast<uint16_t>(constraintType);out << ",";
    out << sMin;out << ",";
    out << sMax;out << ",";
    out << sSet;out << ",";
    out << constrVar;out << ",";
    out << constrRate;out << ",";
    out << acceleration;out << ",";
    out << trackExist;out << ",";
    out << trackInfo;out << ",";
    out << mapObjectExist;out << ",";
    out << mapInfo;out << ",";
    out << biasingInfoExists;out << ",";
    out << biasingInfo;
    
}

std::stringstream QuadraticContour::toStringstream() const
{
    std::stringstream out;
    toCsv(out);
    return out;
}

template<class Archive>
    void QuadraticContour::serialize(Archive& ar, unsigned int version)
{
    if (version >= 1) {
        ar& CEREAL_NVP(continuityType);
        ar& CEREAL_NVP(constraintType);

        ar& CEREAL_NVP(sMin);
        ar& CEREAL_NVP(sMax);
        ar& CEREAL_NVP(sSet);
        ar& CEREAL_NVP(constrVar);
        ar& CEREAL_NVP(constrRate);
        ar& CEREAL_NVP(acceleration);

        ar& CEREAL_NVP(trackExist);
        ar& CEREAL_NVP(trackInfo);
        ar& CEREAL_NVP(mapObjectExist);
        ar& CEREAL_NVP(mapInfo);
    }
    if (version >= 2) {
        ar& CEREAL_NVP(biasingInfoExists);
        ar& CEREAL_NVP(biasingInfo);
    }
}


template void QuadraticContour::serialize(cereal::BinaryInputArchive& ar, unsigned int version);
template void QuadraticContour::serialize(cereal::BinaryOutputArchive& ar, unsigned int version);
template void QuadraticContour::serialize(cereal::IntrospectionArchive& ar, unsigned int version);
template void QuadraticContour::serialize(cereal::JSONInputArchive& ar, unsigned int version);
template void QuadraticContour::serialize(cereal::JSONOutputArchive& ar, unsigned int version);
template void QuadraticContour::serialize(cereal::PortableBinaryInputArchive& ar, unsigned int version);
template void QuadraticContour::serialize(cereal::PortableBinaryOutputArchive& ar, unsigned int version);

std::ostream& operator<< (std::ostream& os, const QuadraticContour& quadraticcontour)
{
    quadraticcontour.toCsv(os);
    return os;
}

bool operator== (const QuadraticContour &lhs, const QuadraticContour &rhs)
{
    return (  lhs.continuityType== rhs.continuityType &&
     lhs.constraintType== rhs.constraintType &&
     lhs.sMin== rhs.sMin &&
     lhs.sMax== rhs.sMax &&
     lhs.sSet== rhs.sSet &&
     lhs.constrVar== rhs.constrVar &&
     lhs.constrRate== rhs.constrRate &&
     lhs.acceleration== rhs.acceleration &&
     lhs.trackExist== rhs.trackExist &&
     lhs.trackInfo== rhs.trackInfo &&
     lhs.mapObjectExist== rhs.mapObjectExist &&
     lhs.mapInfo== rhs.mapInfo &&
     lhs.biasingInfoExists== rhs.biasingInfoExists &&
     lhs.biasingInfo== rhs.biasingInfo 
    );
}

std::size_t QuadraticContour::hash() const noexcept
{
    std::size_t result = 17;
    result = result * 31 + std::hash<ContinuityType>()(continuityType);
    result = result * 31 + std::hash<uint8_t>()(constraintType);
    result = result * 31 + std::hash<double>()(sMin);
    result = result * 31 + std::hash<double>()(sMax);
    result = result * 31 + std::hash<double>()(sSet);
    result = result * 31 + std::hash<double>()(constrVar);
    result = result * 31 + std::hash<double>()(constrRate);
    result = result * 31 + std::hash<double>()(acceleration);
    result = result * 31 + std::hash<bool>()(trackExist);
    result = result * 31 + std::hash<SpeedConstraintTrackInfo>()(trackInfo);
    result = result * 31 + std::hash<bool>()(mapObjectExist);
    result = result * 31 + std::hash<SpeedConstraintMapInfo>()(mapInfo);
    result = result * 31 + std::hash<bool>()(biasingInfoExists);
    result = result * 31 + std::hash<PlaceHolder>()(biasingInfo);
    return result;
}

 } // namespace av
 } // namespace msg


CEREAL_BIND_TO_ARCHIVES(av::msg::QuadraticContour)

 namespace av {
 namespace msg {


HardAndSoftLinearConstraints::HardAndSoftLinearConstraints() = default;

HardAndSoftLinearConstraints::HardAndSoftLinearConstraints(
    double sRef_arg ,
    std::vector<LinearContour> hardUpper_arg ,
    std::vector<LinearContour> softUpper_arg ,
    std::vector<LinearContour> hardLower_arg ,
    std::vector<LinearContour> softLower_arg 
    ):
    sRef { std::move(sRef_arg) } // NOLINT
      ,
    hardUpper { std::move(hardUpper_arg) } // NOLINT
      ,
    softUpper { std::move(softUpper_arg) } // NOLINT
      ,
    hardLower { std::move(hardLower_arg) } // NOLINT
      ,
    softLower { std::move(softLower_arg) } // NOLINT
      
    {
}

HardAndSoftLinearConstraints::HardAndSoftLinearConstraints(const HardAndSoftLinearConstraints&) = default;

// NOLINTNEXTLINE
HardAndSoftLinearConstraints::HardAndSoftLinearConstraints(HardAndSoftLinearConstraints&&) = default;

HardAndSoftLinearConstraints& HardAndSoftLinearConstraints::operator=(const HardAndSoftLinearConstraints&) = default;

// NOLINTNEXTLINE
HardAndSoftLinearConstraints& HardAndSoftLinearConstraints::operator=(HardAndSoftLinearConstraints&&) = default;

void HardAndSoftLinearConstraints::toCsv(std::ostream& out) const
{
    out.precision(16);
    out << std::boolalpha;
    out << sRef;out << ",";
    // Enclose sequence in double quotations and curly brackets to keep CSV line length uniform
        
        out << "\"{{";
        
        if (!hardUpper.empty())
        {
            std::copy(hardUpper.begin(), hardUpper.end() - 1, std::ostream_iterator<LinearContour>(out, "},{"));
            std::copy(hardUpper.end()-1, hardUpper.end(), std::ostream_iterator<LinearContour>(out, ""));
        }
        out << "}}\"";out << ",";
    // Enclose sequence in double quotations and curly brackets to keep CSV line length uniform
        
        out << "\"{{";
        
        if (!softUpper.empty())
        {
            std::copy(softUpper.begin(), softUpper.end() - 1, std::ostream_iterator<LinearContour>(out, "},{"));
            std::copy(softUpper.end()-1, softUpper.end(), std::ostream_iterator<LinearContour>(out, ""));
        }
        out << "}}\"";out << ",";
    // Enclose sequence in double quotations and curly brackets to keep CSV line length uniform
        
        out << "\"{{";
        
        if (!hardLower.empty())
        {
            std::copy(hardLower.begin(), hardLower.end() - 1, std::ostream_iterator<LinearContour>(out, "},{"));
            std::copy(hardLower.end()-1, hardLower.end(), std::ostream_iterator<LinearContour>(out, ""));
        }
        out << "}}\"";out << ",";
    // Enclose sequence in double quotations and curly brackets to keep CSV line length uniform
        
        out << "\"{{";
        
        if (!softLower.empty())
        {
            std::copy(softLower.begin(), softLower.end() - 1, std::ostream_iterator<LinearContour>(out, "},{"));
            std::copy(softLower.end()-1, softLower.end(), std::ostream_iterator<LinearContour>(out, ""));
        }
        out << "}}\"";
    
}

std::stringstream HardAndSoftLinearConstraints::toStringstream() const
{
    std::stringstream out;
    toCsv(out);
    return out;
}

template<class Archive>
    void HardAndSoftLinearConstraints::serialize(Archive& ar, [[maybe_unused]] unsigned int version)
    {

        ar& CEREAL_NVP(sRef);
        ar& CEREAL_NVP(hardUpper);
        ar& CEREAL_NVP(softUpper);
        ar& CEREAL_NVP(hardLower);
        ar& CEREAL_NVP(softLower);
    }


template void HardAndSoftLinearConstraints::serialize(cereal::BinaryInputArchive& ar, unsigned int version);
template void HardAndSoftLinearConstraints::serialize(cereal::BinaryOutputArchive& ar, unsigned int version);
template void HardAndSoftLinearConstraints::serialize(cereal::IntrospectionArchive& ar, unsigned int version);
template void HardAndSoftLinearConstraints::serialize(cereal::JSONInputArchive& ar, unsigned int version);
template void HardAndSoftLinearConstraints::serialize(cereal::JSONOutputArchive& ar, unsigned int version);
template void HardAndSoftLinearConstraints::serialize(cereal::PortableBinaryInputArchive& ar, unsigned int version);
template void HardAndSoftLinearConstraints::serialize(cereal::PortableBinaryOutputArchive& ar, unsigned int version);

std::ostream& operator<< (std::ostream& os, const HardAndSoftLinearConstraints& hardandsoftlinearconstraints)
{
    hardandsoftlinearconstraints.toCsv(os);
    return os;
}

bool operator== (const HardAndSoftLinearConstraints &lhs, const HardAndSoftLinearConstraints &rhs)
{
    return (  lhs.sRef== rhs.sRef &&
     lhs.hardUpper== rhs.hardUpper &&
     lhs.softUpper== rhs.softUpper &&
     lhs.hardLower== rhs.hardLower &&
     lhs.softLower== rhs.softLower 
    );
}

std::size_t HardAndSoftLinearConstraints::hash() const noexcept
{
    std::size_t result = 17;
    result = result * 31 + std::hash<double>()(sRef);
    result = result * 31 + hashVector(hardUpper);
    result = result * 31 + hashVector(softUpper);
    result = result * 31 + hashVector(hardLower);
    result = result * 31 + hashVector(softLower);
    return result;
}

 } // namespace av
 } // namespace msg


CEREAL_BIND_TO_ARCHIVES(av::msg::HardAndSoftLinearConstraints)

 namespace av {
 namespace msg {


HardAndSoftQuadraticConstraints::HardAndSoftQuadraticConstraints() = default;

HardAndSoftQuadraticConstraints::HardAndSoftQuadraticConstraints(
    double sRef_arg ,
    std::vector<QuadraticContour> hardUpper_arg ,
    std::vector<QuadraticContour> softUpper_arg ,
    std::vector<QuadraticContour> hardLower_arg ,
    std::vector<QuadraticContour> softLower_arg 
    ):
    sRef { std::move(sRef_arg) } // NOLINT
      ,
    hardUpper { std::move(hardUpper_arg) } // NOLINT
      ,
    softUpper { std::move(softUpper_arg) } // NOLINT
      ,
    hardLower { std::move(hardLower_arg) } // NOLINT
      ,
    softLower { std::move(softLower_arg) } // NOLINT
      
    {
}

HardAndSoftQuadraticConstraints::HardAndSoftQuadraticConstraints(const HardAndSoftQuadraticConstraints&) = default;

// NOLINTNEXTLINE
HardAndSoftQuadraticConstraints::HardAndSoftQuadraticConstraints(HardAndSoftQuadraticConstraints&&) = default;

HardAndSoftQuadraticConstraints& HardAndSoftQuadraticConstraints::operator=(const HardAndSoftQuadraticConstraints&) = default;

// NOLINTNEXTLINE
HardAndSoftQuadraticConstraints& HardAndSoftQuadraticConstraints::operator=(HardAndSoftQuadraticConstraints&&) = default;

void HardAndSoftQuadraticConstraints::toCsv(std::ostream& out) const
{
    out.precision(16);
    out << std::boolalpha;
    out << sRef;out << ",";
    // Enclose sequence in double quotations and curly brackets to keep CSV line length uniform
        
        out << "\"{{";
        
        if (!hardUpper.empty())
        {
            std::copy(hardUpper.begin(), hardUpper.end() - 1, std::ostream_iterator<QuadraticContour>(out, "},{"));
            std::copy(hardUpper.end()-1, hardUpper.end(), std::ostream_iterator<QuadraticContour>(out, ""));
        }
        out << "}}\"";out << ",";
    // Enclose sequence in double quotations and curly brackets to keep CSV line length uniform
        
        out << "\"{{";
        
        if (!softUpper.empty())
        {
            std::copy(softUpper.begin(), softUpper.end() - 1, std::ostream_iterator<QuadraticContour>(out, "},{"));
            std::copy(softUpper.end()-1, softUpper.end(), std::ostream_iterator<QuadraticContour>(out, ""));
        }
        out << "}}\"";out << ",";
    // Enclose sequence in double quotations and curly brackets to keep CSV line length uniform
        
        out << "\"{{";
        
        if (!hardLower.empty())
        {
            std::copy(hardLower.begin(), hardLower.end() - 1, std::ostream_iterator<QuadraticContour>(out, "},{"));
            std::copy(hardLower.end()-1, hardLower.end(), std::ostream_iterator<QuadraticContour>(out, ""));
        }
        out << "}}\"";out << ",";
    // Enclose sequence in double quotations and curly brackets to keep CSV line length uniform
        
        out << "\"{{";
        
        if (!softLower.empty())
        {
            std::copy(softLower.begin(), softLower.end() - 1, std::ostream_iterator<QuadraticContour>(out, "},{"));
            std::copy(softLower.end()-1, softLower.end(), std::ostream_iterator<QuadraticContour>(out, ""));
        }
        out << "}}\"";
    
}

std::stringstream HardAndSoftQuadraticConstraints::toStringstream() const
{
    std::stringstream out;
    toCsv(out);
    return out;
}

template<class Archive>
    void HardAndSoftQuadraticConstraints::serialize(Archive& ar, [[maybe_unused]] unsigned int version)
    {

        ar& CEREAL_NVP(sRef);
        ar& CEREAL_NVP(hardUpper);
        ar& CEREAL_NVP(softUpper);
        ar& CEREAL_NVP(hardLower);
        ar& CEREAL_NVP(softLower);
    }


template void HardAndSoftQuadraticConstraints::serialize(cereal::BinaryInputArchive& ar, unsigned int version);
template void HardAndSoftQuadraticConstraints::serialize(cereal::BinaryOutputArchive& ar, unsigned int version);
template void HardAndSoftQuadraticConstraints::serialize(cereal::IntrospectionArchive& ar, unsigned int version);
template void HardAndSoftQuadraticConstraints::serialize(cereal::JSONInputArchive& ar, unsigned int version);
template void HardAndSoftQuadraticConstraints::serialize(cereal::JSONOutputArchive& ar, unsigned int version);
template void HardAndSoftQuadraticConstraints::serialize(cereal::PortableBinaryInputArchive& ar, unsigned int version);
template void HardAndSoftQuadraticConstraints::serialize(cereal::PortableBinaryOutputArchive& ar, unsigned int version);

std::ostream& operator<< (std::ostream& os, const HardAndSoftQuadraticConstraints& hardandsoftquadraticconstraints)
{
    hardandsoftquadraticconstraints.toCsv(os);
    return os;
}

bool operator== (const HardAndSoftQuadraticConstraints &lhs, const HardAndSoftQuadraticConstraints &rhs)
{
    return (  lhs.sRef== rhs.sRef &&
     lhs.hardUpper== rhs.hardUpper &&
     lhs.softUpper== rhs.softUpper &&
     lhs.hardLower== rhs.hardLower &&
     lhs.softLower== rhs.softLower 
    );
}

std::size_t HardAndSoftQuadraticConstraints::hash() const noexcept
{
    std::size_t result = 17;
    result = result * 31 + std::hash<double>()(sRef);
    result = result * 31 + hashVector(hardUpper);
    result = result * 31 + hashVector(softUpper);
    result = result * 31 + hashVector(hardLower);
    result = result * 31 + hashVector(softLower);
    return result;
}

 } // namespace av
 } // namespace msg


CEREAL_BIND_TO_ARCHIVES(av::msg::HardAndSoftQuadraticConstraints)

 namespace av {
 namespace msg {


SpeedBounds::SpeedBounds() = default;

SpeedBounds::SpeedBounds(
    HardAndSoftBounds fused_mps_arg ,
    SingleContour hardMax_dominant_arg ,
    SingleContour softMax_dominant_arg ,
    SingleContour hardMin_dominant_arg ,
    SingleContour softMin_dominant_arg ,
    HardAndSoftBounds profile_mps_arg ,
    HardAndSoftBounds proximity_mps_arg ,
    HardAndSoftBounds pedestrian_mps_arg ,
    HardAndSoftBounds generic_mps_arg ,
    HardAndSoftBounds logical_mps_arg ,
    HardAndSoftBounds merging_mps_arg ,
    LinearContour hardUpper_dominant_arg ,
    LinearContour softUpper_dominant_arg ,
    LinearContour hardLower_dominant_arg ,
    LinearContour softLower_dominant_arg 
    ):
    fused_mps { std::move(fused_mps_arg) } // NOLINT
      ,
    hardMax_dominant { std::move(hardMax_dominant_arg) } // NOLINT
      ,
    softMax_dominant { std::move(softMax_dominant_arg) } // NOLINT
      ,
    hardMin_dominant { std::move(hardMin_dominant_arg) } // NOLINT
      ,
    softMin_dominant { std::move(softMin_dominant_arg) } // NOLINT
      ,
    profile_mps { std::move(profile_mps_arg) } // NOLINT
      ,
    proximity_mps { std::move(proximity_mps_arg) } // NOLINT
      ,
    pedestrian_mps { std::move(pedestrian_mps_arg) } // NOLINT
      ,
    generic_mps { std::move(generic_mps_arg) } // NOLINT
      ,
    logical_mps { std::move(logical_mps_arg) } // NOLINT
      ,
    merging_mps { std::move(merging_mps_arg) } // NOLINT
      ,
    hardUpper_dominant { std::move(hardUpper_dominant_arg) } // NOLINT
      ,
    softUpper_dominant { std::move(softUpper_dominant_arg) } // NOLINT
      ,
    hardLower_dominant { std::move(hardLower_dominant_arg) } // NOLINT
      ,
    softLower_dominant { std::move(softLower_dominant_arg) } // NOLINT
      
    {
}

SpeedBounds::SpeedBounds(const SpeedBounds&) = default;

// NOLINTNEXTLINE
SpeedBounds::SpeedBounds(SpeedBounds&&) = default;

SpeedBounds& SpeedBounds::operator=(const SpeedBounds&) = default;

// NOLINTNEXTLINE
SpeedBounds& SpeedBounds::operator=(SpeedBounds&&) = default;

void SpeedBounds::toCsv(std::ostream& out) const
{
    out.precision(16);
    out << std::boolalpha;
    out << fused_mps;out << ",";
    out << hardMax_dominant;out << ",";
    out << softMax_dominant;out << ",";
    out << hardMin_dominant;out << ",";
    out << softMin_dominant;out << ",";
    out << profile_mps;out << ",";
    out << proximity_mps;out << ",";
    out << pedestrian_mps;out << ",";
    out << generic_mps;out << ",";
    out << logical_mps;out << ",";
    out << merging_mps;out << ",";
    out << hardUpper_dominant;out << ",";
    out << softUpper_dominant;out << ",";
    out << hardLower_dominant;out << ",";
    out << softLower_dominant;
    
}

std::stringstream SpeedBounds::toStringstream() const
{
    std::stringstream out;
    toCsv(out);
    return out;
}

template<class Archive>
    void SpeedBounds::serialize(Archive& ar, unsigned int version)
{
    if (version >= 1)
	{
        ar& CEREAL_NVP(fused_mps);
        ar& CEREAL_NVP(hardMax_dominant);
        ar& CEREAL_NVP(softMax_dominant);
        ar& CEREAL_NVP(hardMin_dominant);
        ar& CEREAL_NVP(softMin_dominant);
        ar& CEREAL_NVP(profile_mps);
        ar& CEREAL_NVP(proximity_mps);
        ar& CEREAL_NVP(pedestrian_mps);
        ar& CEREAL_NVP(generic_mps);
        ar& CEREAL_NVP(logical_mps);
        ar& CEREAL_NVP(merging_mps);
    }

    if(version >= 2)
    {
        ar& CEREAL_NVP(hardUpper_dominant);
        ar& CEREAL_NVP(softUpper_dominant);
        ar& CEREAL_NVP(hardLower_dominant);
        ar& CEREAL_NVP(softLower_dominant);
    }
}


template void SpeedBounds::serialize(cereal::BinaryInputArchive& ar, unsigned int version);
template void SpeedBounds::serialize(cereal::BinaryOutputArchive& ar, unsigned int version);
template void SpeedBounds::serialize(cereal::IntrospectionArchive& ar, unsigned int version);
template void SpeedBounds::serialize(cereal::JSONInputArchive& ar, unsigned int version);
template void SpeedBounds::serialize(cereal::JSONOutputArchive& ar, unsigned int version);
template void SpeedBounds::serialize(cereal::PortableBinaryInputArchive& ar, unsigned int version);
template void SpeedBounds::serialize(cereal::PortableBinaryOutputArchive& ar, unsigned int version);

std::ostream& operator<< (std::ostream& os, const SpeedBounds& speedbounds)
{
    speedbounds.toCsv(os);
    return os;
}

bool operator== (const SpeedBounds &lhs, const SpeedBounds &rhs)
{
    return (  lhs.fused_mps== rhs.fused_mps &&
     lhs.hardMax_dominant== rhs.hardMax_dominant &&
     lhs.softMax_dominant== rhs.softMax_dominant &&
     lhs.hardMin_dominant== rhs.hardMin_dominant &&
     lhs.softMin_dominant== rhs.softMin_dominant &&
     lhs.profile_mps== rhs.profile_mps &&
     lhs.proximity_mps== rhs.proximity_mps &&
     lhs.pedestrian_mps== rhs.pedestrian_mps &&
     lhs.generic_mps== rhs.generic_mps &&
     lhs.logical_mps== rhs.logical_mps &&
     lhs.merging_mps== rhs.merging_mps &&
     lhs.hardUpper_dominant== rhs.hardUpper_dominant &&
     lhs.softUpper_dominant== rhs.softUpper_dominant &&
     lhs.hardLower_dominant== rhs.hardLower_dominant &&
     lhs.softLower_dominant== rhs.softLower_dominant 
    );
}

std::size_t SpeedBounds::hash() const noexcept
{
    std::size_t result = 17;
    result = result * 31 + std::hash<HardAndSoftBounds>()(fused_mps);
    result = result * 31 + std::hash<SingleContour>()(hardMax_dominant);
    result = result * 31 + std::hash<SingleContour>()(softMax_dominant);
    result = result * 31 + std::hash<SingleContour>()(hardMin_dominant);
    result = result * 31 + std::hash<SingleContour>()(softMin_dominant);
    result = result * 31 + std::hash<HardAndSoftBounds>()(profile_mps);
    result = result * 31 + std::hash<HardAndSoftBounds>()(proximity_mps);
    result = result * 31 + std::hash<HardAndSoftBounds>()(pedestrian_mps);
    result = result * 31 + std::hash<HardAndSoftBounds>()(generic_mps);
    result = result * 31 + std::hash<HardAndSoftBounds>()(logical_mps);
    result = result * 31 + std::hash<HardAndSoftBounds>()(merging_mps);
    result = result * 31 + std::hash<LinearContour>()(hardUpper_dominant);
    result = result * 31 + std::hash<LinearContour>()(softUpper_dominant);
    result = result * 31 + std::hash<LinearContour>()(hardLower_dominant);
    result = result * 31 + std::hash<LinearContour>()(softLower_dominant);
    return result;
}

 } // namespace av
 } // namespace msg


CEREAL_BIND_TO_ARCHIVES(av::msg::SpeedBounds)

 namespace av {
 namespace msg {


MultiTypeSpeedSquaredConstraints::MultiTypeSpeedSquaredConstraints() = default;

MultiTypeSpeedSquaredConstraints::MultiTypeSpeedSquaredConstraints(
    HardAndSoftConstraints profileConstraint_arg ,
    HardAndSoftConstraints proximityConstraint_arg ,
    HardAndSoftConstraints pedestrianConstraint_arg ,
    HardAndSoftConstraints genericConstraint_arg ,
    HardAndSoftConstraints logicalConstraint_arg ,
    HardAndSoftConstraints mergingConstraint_arg ,
    HardAndSoftLinearConstraints linearProfileConstraint_arg ,
    HardAndSoftLinearConstraints linearProximityConstraint_arg ,
    HardAndSoftLinearConstraints linearPedestrianConstraint_arg ,
    HardAndSoftLinearConstraints linearGenericConstraint_arg ,
    HardAndSoftLinearConstraints linearLogicalConstraint_arg ,
    HardAndSoftLinearConstraints linearMergingConstraint_arg 
    ):
    profileConstraint { std::move(profileConstraint_arg) } // NOLINT
      ,
    proximityConstraint { std::move(proximityConstraint_arg) } // NOLINT
      ,
    pedestrianConstraint { std::move(pedestrianConstraint_arg) } // NOLINT
      ,
    genericConstraint { std::move(genericConstraint_arg) } // NOLINT
      ,
    logicalConstraint { std::move(logicalConstraint_arg) } // NOLINT
      ,
    mergingConstraint { std::move(mergingConstraint_arg) } // NOLINT
      ,
    linearProfileConstraint { std::move(linearProfileConstraint_arg) } // NOLINT
      ,
    linearProximityConstraint { std::move(linearProximityConstraint_arg) } // NOLINT
      ,
    linearPedestrianConstraint { std::move(linearPedestrianConstraint_arg) } // NOLINT
      ,
    linearGenericConstraint { std::move(linearGenericConstraint_arg) } // NOLINT
      ,
    linearLogicalConstraint { std::move(linearLogicalConstraint_arg) } // NOLINT
      ,
    linearMergingConstraint { std::move(linearMergingConstraint_arg) } // NOLINT
      
    {
}

MultiTypeSpeedSquaredConstraints::MultiTypeSpeedSquaredConstraints(const MultiTypeSpeedSquaredConstraints&) = default;

// NOLINTNEXTLINE
MultiTypeSpeedSquaredConstraints::MultiTypeSpeedSquaredConstraints(MultiTypeSpeedSquaredConstraints&&) = default;

MultiTypeSpeedSquaredConstraints& MultiTypeSpeedSquaredConstraints::operator=(const MultiTypeSpeedSquaredConstraints&) = default;

// NOLINTNEXTLINE
MultiTypeSpeedSquaredConstraints& MultiTypeSpeedSquaredConstraints::operator=(MultiTypeSpeedSquaredConstraints&&) = default;

void MultiTypeSpeedSquaredConstraints::toCsv(std::ostream& out) const
{
    out.precision(16);
    out << std::boolalpha;
    out << profileConstraint;out << ",";
    out << proximityConstraint;out << ",";
    out << pedestrianConstraint;out << ",";
    out << genericConstraint;out << ",";
    out << logicalConstraint;out << ",";
    out << mergingConstraint;out << ",";
    out << linearProfileConstraint;out << ",";
    out << linearProximityConstraint;out << ",";
    out << linearPedestrianConstraint;out << ",";
    out << linearGenericConstraint;out << ",";
    out << linearLogicalConstraint;out << ",";
    out << linearMergingConstraint;
    
}

std::stringstream MultiTypeSpeedSquaredConstraints::toStringstream() const
{
    std::stringstream out;
    toCsv(out);
    return out;
}

template<class Archive>
    void MultiTypeSpeedSquaredConstraints::serialize(Archive& ar, unsigned int version)
{
    if (version >= 1)
	{
        ar& CEREAL_NVP(profileConstraint);
        ar& CEREAL_NVP(proximityConstraint);
        ar& CEREAL_NVP(pedestrianConstraint);
        ar& CEREAL_NVP(genericConstraint);
        ar& CEREAL_NVP(logicalConstraint);
        ar& CEREAL_NVP(mergingConstraint);
    }

    if(version >= 2)
    {
        ar& CEREAL_NVP(linearProfileConstraint);
        ar& CEREAL_NVP(linearProximityConstraint);
        ar& CEREAL_NVP(linearPedestrianConstraint);
        ar& CEREAL_NVP(linearGenericConstraint);
        ar& CEREAL_NVP(linearLogicalConstraint);
        ar& CEREAL_NVP(linearMergingConstraint);
    }
}


template void MultiTypeSpeedSquaredConstraints::serialize(cereal::BinaryInputArchive& ar, unsigned int version);
template void MultiTypeSpeedSquaredConstraints::serialize(cereal::BinaryOutputArchive& ar, unsigned int version);
template void MultiTypeSpeedSquaredConstraints::serialize(cereal::IntrospectionArchive& ar, unsigned int version);
template void MultiTypeSpeedSquaredConstraints::serialize(cereal::JSONInputArchive& ar, unsigned int version);
template void MultiTypeSpeedSquaredConstraints::serialize(cereal::JSONOutputArchive& ar, unsigned int version);
template void MultiTypeSpeedSquaredConstraints::serialize(cereal::PortableBinaryInputArchive& ar, unsigned int version);
template void MultiTypeSpeedSquaredConstraints::serialize(cereal::PortableBinaryOutputArchive& ar, unsigned int version);

std::ostream& operator<< (std::ostream& os, const MultiTypeSpeedSquaredConstraints& multitypespeedsquaredconstraints)
{
    multitypespeedsquaredconstraints.toCsv(os);
    return os;
}

bool operator== (const MultiTypeSpeedSquaredConstraints &lhs, const MultiTypeSpeedSquaredConstraints &rhs)
{
    return (  lhs.profileConstraint== rhs.profileConstraint &&
     lhs.proximityConstraint== rhs.proximityConstraint &&
     lhs.pedestrianConstraint== rhs.pedestrianConstraint &&
     lhs.genericConstraint== rhs.genericConstraint &&
     lhs.logicalConstraint== rhs.logicalConstraint &&
     lhs.mergingConstraint== rhs.mergingConstraint &&
     lhs.linearProfileConstraint== rhs.linearProfileConstraint &&
     lhs.linearProximityConstraint== rhs.linearProximityConstraint &&
     lhs.linearPedestrianConstraint== rhs.linearPedestrianConstraint &&
     lhs.linearGenericConstraint== rhs.linearGenericConstraint &&
     lhs.linearLogicalConstraint== rhs.linearLogicalConstraint &&
     lhs.linearMergingConstraint== rhs.linearMergingConstraint 
    );
}

std::size_t MultiTypeSpeedSquaredConstraints::hash() const noexcept
{
    std::size_t result = 17;
    result = result * 31 + std::hash<HardAndSoftConstraints>()(profileConstraint);
    result = result * 31 + std::hash<HardAndSoftConstraints>()(proximityConstraint);
    result = result * 31 + std::hash<HardAndSoftConstraints>()(pedestrianConstraint);
    result = result * 31 + std::hash<HardAndSoftConstraints>()(genericConstraint);
    result = result * 31 + std::hash<HardAndSoftConstraints>()(logicalConstraint);
    result = result * 31 + std::hash<HardAndSoftConstraints>()(mergingConstraint);
    result = result * 31 + std::hash<HardAndSoftLinearConstraints>()(linearProfileConstraint);
    result = result * 31 + std::hash<HardAndSoftLinearConstraints>()(linearProximityConstraint);
    result = result * 31 + std::hash<HardAndSoftLinearConstraints>()(linearPedestrianConstraint);
    result = result * 31 + std::hash<HardAndSoftLinearConstraints>()(linearGenericConstraint);
    result = result * 31 + std::hash<HardAndSoftLinearConstraints>()(linearLogicalConstraint);
    result = result * 31 + std::hash<HardAndSoftLinearConstraints>()(linearMergingConstraint);
    return result;
}

 } // namespace av
 } // namespace msg


CEREAL_BIND_TO_ARCHIVES(av::msg::MultiTypeSpeedSquaredConstraints)

 namespace av {
 namespace msg {


MultiTypeSpatialConstraints::MultiTypeSpatialConstraints() = default;

MultiTypeSpatialConstraints::MultiTypeSpatialConstraints(
    HardAndSoftLinearConstraints singleLaneConstraint_arg ,
    HardAndSoftLinearConstraints multiLaneConstraint_arg ,
    HardAndSoftLinearConstraints intersectionConstraint_arg ,
    HardAndSoftLinearConstraints trackConstraint_arg 
    ):
    singleLaneConstraint { std::move(singleLaneConstraint_arg) } // NOLINT
      ,
    multiLaneConstraint { std::move(multiLaneConstraint_arg) } // NOLINT
      ,
    intersectionConstraint { std::move(intersectionConstraint_arg) } // NOLINT
      ,
    trackConstraint { std::move(trackConstraint_arg) } // NOLINT
      
    {
}

MultiTypeSpatialConstraints::MultiTypeSpatialConstraints(const MultiTypeSpatialConstraints&) = default;

// NOLINTNEXTLINE
MultiTypeSpatialConstraints::MultiTypeSpatialConstraints(MultiTypeSpatialConstraints&&) = default;

MultiTypeSpatialConstraints& MultiTypeSpatialConstraints::operator=(const MultiTypeSpatialConstraints&) = default;

// NOLINTNEXTLINE
MultiTypeSpatialConstraints& MultiTypeSpatialConstraints::operator=(MultiTypeSpatialConstraints&&) = default;

void MultiTypeSpatialConstraints::toCsv(std::ostream& out) const
{
    out.precision(16);
    out << std::boolalpha;
    out << singleLaneConstraint;out << ",";
    out << multiLaneConstraint;out << ",";
    out << intersectionConstraint;out << ",";
    out << trackConstraint;
    
}

std::stringstream MultiTypeSpatialConstraints::toStringstream() const
{
    std::stringstream out;
    toCsv(out);
    return out;
}

template<class Archive>
    void MultiTypeSpatialConstraints::serialize(Archive& ar, [[maybe_unused]] unsigned int version)
    {

        ar& CEREAL_NVP(singleLaneConstraint);
        ar& CEREAL_NVP(multiLaneConstraint);
        ar& CEREAL_NVP(intersectionConstraint);
        ar& CEREAL_NVP(trackConstraint);
    }


template void MultiTypeSpatialConstraints::serialize(cereal::BinaryInputArchive& ar, unsigned int version);
template void MultiTypeSpatialConstraints::serialize(cereal::BinaryOutputArchive& ar, unsigned int version);
template void MultiTypeSpatialConstraints::serialize(cereal::IntrospectionArchive& ar, unsigned int version);
template void MultiTypeSpatialConstraints::serialize(cereal::JSONInputArchive& ar, unsigned int version);
template void MultiTypeSpatialConstraints::serialize(cereal::JSONOutputArchive& ar, unsigned int version);
template void MultiTypeSpatialConstraints::serialize(cereal::PortableBinaryInputArchive& ar, unsigned int version);
template void MultiTypeSpatialConstraints::serialize(cereal::PortableBinaryOutputArchive& ar, unsigned int version);

std::ostream& operator<< (std::ostream& os, const MultiTypeSpatialConstraints& multitypespatialconstraints)
{
    multitypespatialconstraints.toCsv(os);
    return os;
}

bool operator== (const MultiTypeSpatialConstraints &lhs, const MultiTypeSpatialConstraints &rhs)
{
    return (  lhs.singleLaneConstraint== rhs.singleLaneConstraint &&
     lhs.multiLaneConstraint== rhs.multiLaneConstraint &&
     lhs.intersectionConstraint== rhs.intersectionConstraint &&
     lhs.trackConstraint== rhs.trackConstraint 
    );
}

std::size_t MultiTypeSpatialConstraints::hash() const noexcept
{
    std::size_t result = 17;
    result = result * 31 + std::hash<HardAndSoftLinearConstraints>()(singleLaneConstraint);
    result = result * 31 + std::hash<HardAndSoftLinearConstraints>()(multiLaneConstraint);
    result = result * 31 + std::hash<HardAndSoftLinearConstraints>()(intersectionConstraint);
    result = result * 31 + std::hash<HardAndSoftLinearConstraints>()(trackConstraint);
    return result;
}

 } // namespace av
 } // namespace msg


CEREAL_BIND_TO_ARCHIVES(av::msg::MultiTypeSpatialConstraints)




 namespace av {
 namespace msg {


PlannedPathWithBounds::PlannedPathWithBounds() = default;

PlannedPathWithBounds::PlannedPathWithBounds(
    plan_ctrl::PlannerHeader header_arg ,
    Path pathWitness_arg ,
    double currentS_m_arg ,
    std::vector<SingleContour> speedSquaredWitness_arg ,
    HardAndSoftConstraints spatialConstraints_arg ,
    HardAndSoftConstraints speedSquaredConstraints_arg ,
    HardAndSoftLinearConstraints linearSpatialConstraints_arg ,
    HardAndSoftLinearConstraints multilaneSpatialConstraints_arg ,
    HardAndSoftLinearConstraints linearSpeedSquaredConstraints_arg ,
    HardAndSoftQuadraticConstraints STConstraints_arg ,
    SpeedBounds currentSpeedConstraint_arg ,
    MultiTypeSpeedSquaredConstraints multiTypeSpeedConstraint_arg ,
    MultiTypeSpatialConstraints multiTypeSpatialConstraint_arg ,
    std::vector<LinearContour> linearSpeedSquaredWitness_arg 
    ):
    header { std::move(header_arg) } // NOLINT
      ,pathWitness { std::move(pathWitness_arg) } // NOLINT
      ,currentS_m { std::move(currentS_m_arg) } // NOLINT
      ,speedSquaredWitness { std::move(speedSquaredWitness_arg) } // NOLINT
      ,spatialConstraints { std::move(spatialConstraints_arg) } // NOLINT
      ,speedSquaredConstraints { std::move(speedSquaredConstraints_arg) } // NOLINT
      ,linearSpatialConstraints { std::move(linearSpatialConstraints_arg) } // NOLINT
      ,multilaneSpatialConstraints { std::move(multilaneSpatialConstraints_arg) } // NOLINT
      ,linearSpeedSquaredConstraints { std::move(linearSpeedSquaredConstraints_arg) } // NOLINT
      ,STConstraints { std::move(STConstraints_arg) } // NOLINT
      ,currentSpeedConstraint { std::move(currentSpeedConstraint_arg) } // NOLINT
      ,multiTypeSpeedConstraint { std::move(multiTypeSpeedConstraint_arg) } // NOLINT
      ,multiTypeSpatialConstraint { std::move(multiTypeSpatialConstraint_arg) } // NOLINT
      ,linearSpeedSquaredWitness { std::move(linearSpeedSquaredWitness_arg) } // NOLINT
      {
}

PlannedPathWithBounds::PlannedPathWithBounds(const PlannedPathWithBounds&) = default;

// NOLINTNEXTLINE
PlannedPathWithBounds::PlannedPathWithBounds(PlannedPathWithBounds&&) = default;

PlannedPathWithBounds& PlannedPathWithBounds::operator=(const PlannedPathWithBounds&) = default;

// NOLINTNEXTLINE
PlannedPathWithBounds& PlannedPathWithBounds::operator=(PlannedPathWithBounds&&) = default;

void PlannedPathWithBounds::toCsv(std::ostream& out) const
{
    out.precision(16);
    out << std::boolalpha;
    out << header;out << ",";
    out << pathWitness;out << ",";
    out << currentS_m;out << ",";
    // Enclose sequence in double quotations and curly brackets to keep CSV line length uniform
        
        out << "\"{{";
        
        if (!speedSquaredWitness.empty())
        {
            std::copy(speedSquaredWitness.begin(), speedSquaredWitness.end() - 1, std::ostream_iterator<SingleContour>(out, "},{"));
            std::copy(speedSquaredWitness.end()-1, speedSquaredWitness.end(), std::ostream_iterator<SingleContour>(out, ""));
        }
        out << "}}\"";out << ",";
    out << spatialConstraints;out << ",";
    out << speedSquaredConstraints;out << ",";
    out << linearSpatialConstraints;out << ",";
    out << multilaneSpatialConstraints;out << ",";
    out << linearSpeedSquaredConstraints;out << ",";
    out << STConstraints;out << ",";
    out << currentSpeedConstraint;out << ",";
    out << multiTypeSpeedConstraint;out << ",";
    out << multiTypeSpatialConstraint;out << ",";
    // Enclose sequence in double quotations and curly brackets to keep CSV line length uniform
        
        out << "\"{{";
        
        if (!linearSpeedSquaredWitness.empty())
        {
            std::copy(linearSpeedSquaredWitness.begin(), linearSpeedSquaredWitness.end() - 1, std::ostream_iterator<LinearContour>(out, "},{"));
            std::copy(linearSpeedSquaredWitness.end()-1, linearSpeedSquaredWitness.end(), std::ostream_iterator<LinearContour>(out, ""));
        }
        out << "}}\"";
    
}

std::stringstream PlannedPathWithBounds::toStringstream() const
{
    std::stringstream out;
    toCsv(out);
    return out;
}

template<class Archive>
    void PlannedPathWithBounds::serialize(Archive& ar, unsigned int version)
{
    if (version >= 1)
	{
		ar& CEREAL_NVP(header);
		ar& CEREAL_NVP(pathWitness);
		ar& CEREAL_NVP(speedSquaredWitness);
		ar& CEREAL_NVP(currentS_m);
		ar& CEREAL_NVP(spatialConstraints);
		ar& CEREAL_NVP(speedSquaredConstraints);
		ar& CEREAL_NVP(currentSpeedConstraint);
		ar& CEREAL_NVP(multiTypeSpeedConstraint);
	}
    if(version >= 2)
    {
		ar& CEREAL_NVP(linearSpatialConstraints);
		ar& CEREAL_NVP(multilaneSpatialConstraints);
		ar& CEREAL_NVP(linearSpeedSquaredConstraints);
		ar& CEREAL_NVP(STConstraints);
		ar& CEREAL_NVP(multiTypeSpatialConstraint);
		ar& CEREAL_NVP(linearSpeedSquaredWitness);
    }
}



template void PlannedPathWithBounds::serialize(cereal::BinaryInputArchive& ar, unsigned int version);
template void PlannedPathWithBounds::serialize(cereal::BinaryOutputArchive& ar, unsigned int version);
template void PlannedPathWithBounds::serialize(cereal::IntrospectionArchive& ar, unsigned int version);
template void PlannedPathWithBounds::serialize(cereal::JSONInputArchive& ar, unsigned int version);
template void PlannedPathWithBounds::serialize(cereal::JSONOutputArchive& ar, unsigned int version);
template void PlannedPathWithBounds::serialize(cereal::PortableBinaryInputArchive& ar, unsigned int version);
template void PlannedPathWithBounds::serialize(cereal::PortableBinaryOutputArchive& ar, unsigned int version);

std::ostream& operator<< (std::ostream& os, const PlannedPathWithBounds& plannedpathwithbounds)
{
    plannedpathwithbounds.toCsv(os);
    return os;
}

bool operator== (const PlannedPathWithBounds &lhs, const PlannedPathWithBounds &rhs)
{
    return (  lhs.header== rhs.header &&
     lhs.pathWitness== rhs.pathWitness &&
     lhs.currentS_m== rhs.currentS_m &&
     lhs.speedSquaredWitness== rhs.speedSquaredWitness &&
     lhs.spatialConstraints== rhs.spatialConstraints &&
     lhs.speedSquaredConstraints== rhs.speedSquaredConstraints &&
     lhs.linearSpatialConstraints== rhs.linearSpatialConstraints &&
     lhs.multilaneSpatialConstraints== rhs.multilaneSpatialConstraints &&
     lhs.linearSpeedSquaredConstraints== rhs.linearSpeedSquaredConstraints &&
     lhs.STConstraints== rhs.STConstraints &&
     lhs.currentSpeedConstraint== rhs.currentSpeedConstraint &&
     lhs.multiTypeSpeedConstraint== rhs.multiTypeSpeedConstraint &&
     lhs.multiTypeSpatialConstraint== rhs.multiTypeSpatialConstraint &&
     lhs.linearSpeedSquaredWitness== rhs.linearSpeedSquaredWitness 
    );
}

std::size_t PlannedPathWithBounds::hash() const noexcept
{
    std::size_t result = 17;
    result = result * 31 + std::hash<plan_ctrl::PlannerHeader>()(header);
    result = result * 31 + std::hash<Path>()(pathWitness);
    result = result * 31 + std::hash<double>()(currentS_m);
    result = result * 31 + hashVector(speedSquaredWitness);
    result = result * 31 + std::hash<HardAndSoftConstraints>()(spatialConstraints);
    result = result * 31 + std::hash<HardAndSoftConstraints>()(speedSquaredConstraints);
    result = result * 31 + std::hash<HardAndSoftLinearConstraints>()(linearSpatialConstraints);
    result = result * 31 + std::hash<HardAndSoftLinearConstraints>()(multilaneSpatialConstraints);
    result = result * 31 + std::hash<HardAndSoftLinearConstraints>()(linearSpeedSquaredConstraints);
    result = result * 31 + std::hash<HardAndSoftQuadraticConstraints>()(STConstraints);
    result = result * 31 + std::hash<SpeedBounds>()(currentSpeedConstraint);
    result = result * 31 + std::hash<MultiTypeSpeedSquaredConstraints>()(multiTypeSpeedConstraint);
    result = result * 31 + std::hash<MultiTypeSpatialConstraints>()(multiTypeSpatialConstraint);
    result = result * 31 + hashVector(linearSpeedSquaredWitness);
    return result;
}

 } // namespace av
 } // namespace msg


CEREAL_BIND_TO_ARCHIVES(av::msg::PlannedPathWithBounds)

