#include <interfaces/PlannerHeader/PlannerHeader.h>

#include <iterator>
#include <type_traits>

#include <cereal/cereal.hpp>
#include <cereal/archives/json.hpp>
#include <cereal/archives/binary.hpp>

#include <cereal/types/map.hpp>
#include <cereal/types/vector.hpp>
#include <cereal/types/array.hpp>
#include <cereal/types/string.hpp>
#include <cereal/types/complex.hpp>

#include <ODatum/IntrospectionArchive.h>
#include <ODatum/Hash.h>





 namespace av {
 namespace msg {
 namespace plan_ctrl {


PlannerHeader::PlannerHeader() = default;

PlannerHeader::PlannerHeader(
    int64_t timestamp_arg 
    ):
    timestamp { std::move(timestamp_arg) } // NOLINT
      {
}

PlannerHeader::PlannerHeader(const PlannerHeader&) = default;

// NOLINTNEXTLINE
PlannerHeader::PlannerHeader(PlannerHeader&&) = default;

PlannerHeader& PlannerHeader::operator=(const PlannerHeader&) = default;

// NOLINTNEXTLINE
PlannerHeader& PlannerHeader::operator=(PlannerHeader&&) = default;

void PlannerHeader::toCsv(std::ostream& out) const
{
    out.precision(16);
    out << std::boolalpha;
    out << timestamp;
    
}

std::stringstream PlannerHeader::toStringstream() const
{
    std::stringstream out;
    toCsv(out);
    return out;
}

template<class Archive>
    void PlannerHeader::serialize(Archive& ar, [[maybe_unused]] unsigned int version)
    {

        ar& CEREAL_NVP(timestamp);
    }


template void PlannerHeader::serialize(cereal::BinaryInputArchive& ar, unsigned int version);
template void PlannerHeader::serialize(cereal::BinaryOutputArchive& ar, unsigned int version);
template void PlannerHeader::serialize(cereal::IntrospectionArchive& ar, unsigned int version);
template void PlannerHeader::serialize(cereal::JSONInputArchive& ar, unsigned int version);
template void PlannerHeader::serialize(cereal::JSONOutputArchive& ar, unsigned int version);
template void PlannerHeader::serialize(cereal::PortableBinaryInputArchive& ar, unsigned int version);
template void PlannerHeader::serialize(cereal::PortableBinaryOutputArchive& ar, unsigned int version);

std::ostream& operator<< (std::ostream& os, const PlannerHeader& plannerheader)
{
    plannerheader.toCsv(os);
    return os;
}

bool operator== (const PlannerHeader &lhs, const PlannerHeader &rhs)
{
    return (  lhs.timestamp== rhs.timestamp 
    );
}

std::size_t PlannerHeader::hash() const noexcept
{
    std::size_t result = 17;
    result = result * 31 + std::hash<int64_t>()(timestamp);
    return result;
}

 } // namespace av
 } // namespace msg
 } // namespace plan_ctrl


CEREAL_BIND_TO_ARCHIVES(av::msg::plan_ctrl::PlannerHeader)

