#include <interfaces/PlanningControlHighLevelStatus/PlanningControlHighLevelStatus.h>

#include <iterator>
#include <type_traits>

#include <cereal/cereal.hpp>
#include <cereal/archives/json.hpp>
#include <cereal/archives/binary.hpp>

#include <cereal/types/map.hpp>
#include <cereal/types/vector.hpp>
#include <cereal/types/array.hpp>
#include <cereal/types/string.hpp>
#include <cereal/types/complex.hpp>

#include <ODatum/IntrospectionArchive.h>
#include <ODatum/Hash.h>


 namespace av {
 namespace msg {
 namespace plan_ctrl {


OptionalDuration::OptionalDuration() = default;

OptionalDuration::OptionalDuration(
    bool hasValue_arg ,
    Duration value_arg 
    ):
    hasValue { std::move(hasValue_arg) } // NOLINT
      ,
    value { std::move(value_arg) } // NOLINT
      
    {
}

OptionalDuration::OptionalDuration(const OptionalDuration&) = default;

// NOLINTNEXTLINE
OptionalDuration::OptionalDuration(OptionalDuration&&) = default;

OptionalDuration& OptionalDuration::operator=(const OptionalDuration&) = default;

// NOLINTNEXTLINE
OptionalDuration& OptionalDuration::operator=(OptionalDuration&&) = default;

void OptionalDuration::toCsv(std::ostream& out) const
{
    out.precision(16);
    out << std::boolalpha;
    out << hasValue;out << ",";
    out << value;
    
}

std::stringstream OptionalDuration::toStringstream() const
{
    std::stringstream out;
    toCsv(out);
    return out;
}

template<class Archive>
    void OptionalDuration::serialize(Archive& ar, [[maybe_unused]] unsigned int version)
    {

        ar& CEREAL_NVP(hasValue);
        ar& CEREAL_NVP(value);
    }


template void OptionalDuration::serialize(cereal::BinaryInputArchive& ar, unsigned int version);
template void OptionalDuration::serialize(cereal::BinaryOutputArchive& ar, unsigned int version);
template void OptionalDuration::serialize(cereal::IntrospectionArchive& ar, unsigned int version);
template void OptionalDuration::serialize(cereal::JSONInputArchive& ar, unsigned int version);
template void OptionalDuration::serialize(cereal::JSONOutputArchive& ar, unsigned int version);
template void OptionalDuration::serialize(cereal::PortableBinaryInputArchive& ar, unsigned int version);
template void OptionalDuration::serialize(cereal::PortableBinaryOutputArchive& ar, unsigned int version);

std::ostream& operator<< (std::ostream& os, const OptionalDuration& optionalduration)
{
    optionalduration.toCsv(os);
    return os;
}

bool operator== (const OptionalDuration &lhs, const OptionalDuration &rhs)
{
    return (  lhs.hasValue== rhs.hasValue &&
     lhs.value== rhs.value 
    );
}

std::size_t OptionalDuration::hash() const noexcept
{
    std::size_t result = 17;
    result = result * 31 + std::hash<bool>()(hasValue);
    result = result * 31 + 0;
    return result;
}

 } // namespace av
 } // namespace msg
 } // namespace plan_ctrl


CEREAL_BIND_TO_ARCHIVES(av::msg::plan_ctrl::OptionalDuration)

 namespace av {
 namespace msg {
 namespace plan_ctrl {


LaneChangeMetrics::LaneChangeMetrics() = default;

LaneChangeMetrics::LaneChangeMetrics(
    OptionalDuration rangeOfTimeToHit_arg ,
    OptionalDuration timeToCollisionLeadVehicle_arg ,
    OptionalDuration timeToCollisionLagVehicle_arg 
    ):
    rangeOfTimeToHit { std::move(rangeOfTimeToHit_arg) } // NOLINT
      ,
    timeToCollisionLeadVehicle { std::move(timeToCollisionLeadVehicle_arg) } // NOLINT
      ,
    timeToCollisionLagVehicle { std::move(timeToCollisionLagVehicle_arg) } // NOLINT
      
    {
}

LaneChangeMetrics::LaneChangeMetrics(const LaneChangeMetrics&) = default;

// NOLINTNEXTLINE
LaneChangeMetrics::LaneChangeMetrics(LaneChangeMetrics&&) = default;

LaneChangeMetrics& LaneChangeMetrics::operator=(const LaneChangeMetrics&) = default;

// NOLINTNEXTLINE
LaneChangeMetrics& LaneChangeMetrics::operator=(LaneChangeMetrics&&) = default;

void LaneChangeMetrics::toCsv(std::ostream& out) const
{
    out.precision(16);
    out << std::boolalpha;
    out << rangeOfTimeToHit;out << ",";
    out << timeToCollisionLeadVehicle;out << ",";
    out << timeToCollisionLagVehicle;
    
}

std::stringstream LaneChangeMetrics::toStringstream() const
{
    std::stringstream out;
    toCsv(out);
    return out;
}

template<class Archive>
    void LaneChangeMetrics::serialize(Archive& ar, [[maybe_unused]] unsigned int version)
    {

        ar& CEREAL_NVP(rangeOfTimeToHit);
        ar& CEREAL_NVP(timeToCollisionLeadVehicle);
        ar& CEREAL_NVP(timeToCollisionLagVehicle);
    }


template void LaneChangeMetrics::serialize(cereal::BinaryInputArchive& ar, unsigned int version);
template void LaneChangeMetrics::serialize(cereal::BinaryOutputArchive& ar, unsigned int version);
template void LaneChangeMetrics::serialize(cereal::IntrospectionArchive& ar, unsigned int version);
template void LaneChangeMetrics::serialize(cereal::JSONInputArchive& ar, unsigned int version);
template void LaneChangeMetrics::serialize(cereal::JSONOutputArchive& ar, unsigned int version);
template void LaneChangeMetrics::serialize(cereal::PortableBinaryInputArchive& ar, unsigned int version);
template void LaneChangeMetrics::serialize(cereal::PortableBinaryOutputArchive& ar, unsigned int version);

std::ostream& operator<< (std::ostream& os, const LaneChangeMetrics& lanechangemetrics)
{
    lanechangemetrics.toCsv(os);
    return os;
}

bool operator== (const LaneChangeMetrics &lhs, const LaneChangeMetrics &rhs)
{
    return (  lhs.rangeOfTimeToHit== rhs.rangeOfTimeToHit &&
     lhs.timeToCollisionLeadVehicle== rhs.timeToCollisionLeadVehicle &&
     lhs.timeToCollisionLagVehicle== rhs.timeToCollisionLagVehicle 
    );
}

std::size_t LaneChangeMetrics::hash() const noexcept
{
    std::size_t result = 17;
    result = result * 31 + std::hash<OptionalDuration>()(rangeOfTimeToHit);
    result = result * 31 + std::hash<OptionalDuration>()(timeToCollisionLeadVehicle);
    result = result * 31 + std::hash<OptionalDuration>()(timeToCollisionLagVehicle);
    return result;
}

 } // namespace av
 } // namespace msg
 } // namespace plan_ctrl


CEREAL_BIND_TO_ARCHIVES(av::msg::plan_ctrl::LaneChangeMetrics)




 namespace av {
 namespace msg {
 namespace plan_ctrl {


PlanningControlHighLevelStatus::PlanningControlHighLevelStatus() = default;

PlanningControlHighLevelStatus::PlanningControlHighLevelStatus(
    TimePoint timestamp_arg ,
    Path path_arg ,
    CROSSWALK_STATUS crosswalkStatus_arg ,
    bool traversingCrossWalk_arg ,
    double crosswalkDistance_m_arg ,
    TRAFFIC_LIGHT_STATUS trafficLightStatus_arg ,
    bool traversingTrafficLight_arg ,
    double trafficLightDistance_m_arg ,
    UNPROTECTED_TURN_STATUS unprotectedTurnStatus_arg ,
    bool traversingUnprotectedTurn_arg ,
    LANE_CHANGE_STATUS planningLaneChangeStatus_arg ,
    LANE_CHANGE_STATUS controlLaneChangeStatus_arg ,
    LaneChangeMetrics laneChangeMetrics_arg ,
    PUDO_STATUS pudoStatus_arg ,
    std::string currentExtendedPudo_arg ,
    bool isConstrainedByStopLine_arg ,
    TimePoint trackedObjectsTimestamp_arg 
    ):
    timestamp { std::move(timestamp_arg) } // NOLINT
      ,path { std::move(path_arg) } // NOLINT
      ,crosswalkStatus { std::move(crosswalkStatus_arg) } // NOLINT
      ,traversingCrossWalk { std::move(traversingCrossWalk_arg) } // NOLINT
      ,crosswalkDistance_m { std::move(crosswalkDistance_m_arg) } // NOLINT
      ,trafficLightStatus { std::move(trafficLightStatus_arg) } // NOLINT
      ,traversingTrafficLight { std::move(traversingTrafficLight_arg) } // NOLINT
      ,trafficLightDistance_m { std::move(trafficLightDistance_m_arg) } // NOLINT
      ,unprotectedTurnStatus { std::move(unprotectedTurnStatus_arg) } // NOLINT
      ,traversingUnprotectedTurn { std::move(traversingUnprotectedTurn_arg) } // NOLINT
      ,planningLaneChangeStatus { std::move(planningLaneChangeStatus_arg) } // NOLINT
      ,controlLaneChangeStatus { std::move(controlLaneChangeStatus_arg) } // NOLINT
      ,laneChangeMetrics { std::move(laneChangeMetrics_arg) } // NOLINT
      ,pudoStatus { std::move(pudoStatus_arg) } // NOLINT
      ,currentExtendedPudo { std::move(currentExtendedPudo_arg) } // NOLINT
      ,isConstrainedByStopLine { std::move(isConstrainedByStopLine_arg) } // NOLINT
      ,trackedObjectsTimestamp { std::move(trackedObjectsTimestamp_arg) } // NOLINT
      {
}

PlanningControlHighLevelStatus::PlanningControlHighLevelStatus(const PlanningControlHighLevelStatus&) = default;

// NOLINTNEXTLINE
PlanningControlHighLevelStatus::PlanningControlHighLevelStatus(PlanningControlHighLevelStatus&&) = default;

PlanningControlHighLevelStatus& PlanningControlHighLevelStatus::operator=(const PlanningControlHighLevelStatus&) = default;

// NOLINTNEXTLINE
PlanningControlHighLevelStatus& PlanningControlHighLevelStatus::operator=(PlanningControlHighLevelStatus&&) = default;

void PlanningControlHighLevelStatus::toCsv(std::ostream& out) const
{
    out.precision(16);
    out << std::boolalpha;
    out << timestamp;out << ",";
    out << path;out << ",";
    out << static_cast<uint32_t>(crosswalkStatus);out << ",";
    out << traversingCrossWalk;out << ",";
    out << crosswalkDistance_m;out << ",";
    out << static_cast<uint32_t>(trafficLightStatus);out << ",";
    out << traversingTrafficLight;out << ",";
    out << trafficLightDistance_m;out << ",";
    out << static_cast<uint32_t>(unprotectedTurnStatus);out << ",";
    out << traversingUnprotectedTurn;out << ",";
    out << static_cast<uint32_t>(planningLaneChangeStatus);out << ",";
    out << static_cast<uint32_t>(controlLaneChangeStatus);out << ",";
    out << laneChangeMetrics;out << ",";
    out << static_cast<uint32_t>(pudoStatus);out << ",";
    out << "\"";
        out << currentExtendedPudo;
        out << "\"";out << ",";
    out << isConstrainedByStopLine;out << ",";
    out << trackedObjectsTimestamp;
    
}

std::stringstream PlanningControlHighLevelStatus::toStringstream() const
{
    std::stringstream out;
    toCsv(out);
    return out;
}

template<class Archive>
    void PlanningControlHighLevelStatus::serialize(Archive& ar, unsigned int version)
{
    if(version >= 1)
    {
        ar& CEREAL_NVP(timestamp);
        ar& CEREAL_NVP(path);
        ar& CEREAL_NVP(crosswalkStatus);
        ar& CEREAL_NVP(traversingCrossWalk);
        ar& CEREAL_NVP(crosswalkDistance_m);
        ar& CEREAL_NVP(trafficLightStatus);
        ar& CEREAL_NVP(traversingTrafficLight);
        ar& CEREAL_NVP(trafficLightDistance_m);
        ar& CEREAL_NVP(unprotectedTurnStatus);
        ar& CEREAL_NVP(traversingUnprotectedTurn);

        ar& CEREAL_NVP(planningLaneChangeStatus);
        ar& CEREAL_NVP(controlLaneChangeStatus);
        if(version < 4)
        {
            Duration timeToCollisionLeadVehicle;
            ar& CEREAL_NVP(timeToCollisionLeadVehicle);
            // This value is set to false because before version 4, a valid value for this variable was never published
            laneChangeMetrics.timeToCollisionLeadVehicle.hasValue = false;
            
            Duration timeToCollisionLagVehicle;
            ar& CEREAL_NVP(timeToCollisionLagVehicle);
            // This value is set to false because before version 4, a valid value for this variable was never published
            laneChangeMetrics.timeToCollisionLagVehicle.hasValue = false;
        }
        ar& CEREAL_NVP(pudoStatus);
        ar& CEREAL_NVP(isConstrainedByStopLine);
    }
    if(version >= 2)
    {
        ar& CEREAL_NVP(currentExtendedPudo);
    }
    if(version == 3)
    {
        Duration rangeOfTimeToHit;
        ar& CEREAL_NVP(rangeOfTimeToHit);
        // This value is set to true because before version 4, rangeOfTimeToHit was always published but the validity
        // was guaranteed only when lane change status was available.
        laneChangeMetrics.timeToCollisionLeadVehicle.hasValue = true;
        laneChangeMetrics.timeToCollisionLeadVehicle.value = rangeOfTimeToHit;
    }
    if(version >= 4)
    {
        ar& CEREAL_NVP(laneChangeMetrics);
    }
    if(version >= 5)
    {
        ar& CEREAL_NVP(trackedObjectsTimestamp);
    }
}


template void PlanningControlHighLevelStatus::serialize(cereal::BinaryInputArchive& ar, unsigned int version);
template void PlanningControlHighLevelStatus::serialize(cereal::BinaryOutputArchive& ar, unsigned int version);
template void PlanningControlHighLevelStatus::serialize(cereal::IntrospectionArchive& ar, unsigned int version);
template void PlanningControlHighLevelStatus::serialize(cereal::JSONInputArchive& ar, unsigned int version);
template void PlanningControlHighLevelStatus::serialize(cereal::JSONOutputArchive& ar, unsigned int version);
template void PlanningControlHighLevelStatus::serialize(cereal::PortableBinaryInputArchive& ar, unsigned int version);
template void PlanningControlHighLevelStatus::serialize(cereal::PortableBinaryOutputArchive& ar, unsigned int version);

std::ostream& operator<< (std::ostream& os, const PlanningControlHighLevelStatus& planningcontrolhighlevelstatus)
{
    planningcontrolhighlevelstatus.toCsv(os);
    return os;
}

bool operator== (const PlanningControlHighLevelStatus &lhs, const PlanningControlHighLevelStatus &rhs)
{
    return (  lhs.timestamp== rhs.timestamp &&
     lhs.path== rhs.path &&
     lhs.crosswalkStatus== rhs.crosswalkStatus &&
     lhs.traversingCrossWalk== rhs.traversingCrossWalk &&
     lhs.crosswalkDistance_m== rhs.crosswalkDistance_m &&
     lhs.trafficLightStatus== rhs.trafficLightStatus &&
     lhs.traversingTrafficLight== rhs.traversingTrafficLight &&
     lhs.trafficLightDistance_m== rhs.trafficLightDistance_m &&
     lhs.unprotectedTurnStatus== rhs.unprotectedTurnStatus &&
     lhs.traversingUnprotectedTurn== rhs.traversingUnprotectedTurn &&
     lhs.planningLaneChangeStatus== rhs.planningLaneChangeStatus &&
     lhs.controlLaneChangeStatus== rhs.controlLaneChangeStatus &&
     lhs.laneChangeMetrics== rhs.laneChangeMetrics &&
     lhs.pudoStatus== rhs.pudoStatus &&
     lhs.currentExtendedPudo== rhs.currentExtendedPudo &&
     lhs.isConstrainedByStopLine== rhs.isConstrainedByStopLine &&
     lhs.trackedObjectsTimestamp== rhs.trackedObjectsTimestamp 
    );
}

std::size_t PlanningControlHighLevelStatus::hash() const noexcept
{
    std::size_t result = 17;
    result = result * 31 + 0;
    result = result * 31 + std::hash<Path>()(path);
    result = result * 31 + std::hash<CROSSWALK_STATUS>()(crosswalkStatus);
    result = result * 31 + std::hash<bool>()(traversingCrossWalk);
    result = result * 31 + std::hash<double>()(crosswalkDistance_m);
    result = result * 31 + std::hash<TRAFFIC_LIGHT_STATUS>()(trafficLightStatus);
    result = result * 31 + std::hash<bool>()(traversingTrafficLight);
    result = result * 31 + std::hash<double>()(trafficLightDistance_m);
    result = result * 31 + std::hash<UNPROTECTED_TURN_STATUS>()(unprotectedTurnStatus);
    result = result * 31 + std::hash<bool>()(traversingUnprotectedTurn);
    result = result * 31 + std::hash<LANE_CHANGE_STATUS>()(planningLaneChangeStatus);
    result = result * 31 + std::hash<LANE_CHANGE_STATUS>()(controlLaneChangeStatus);
    result = result * 31 + std::hash<LaneChangeMetrics>()(laneChangeMetrics);
    result = result * 31 + std::hash<PUDO_STATUS>()(pudoStatus);
    result = result * 31 + std::hash<std::string>()(currentExtendedPudo);
    result = result * 31 + std::hash<bool>()(isConstrainedByStopLine);
    result = result * 31 + 0;
    return result;
}

 } // namespace av
 } // namespace msg
 } // namespace plan_ctrl


CEREAL_BIND_TO_ARCHIVES(av::msg::plan_ctrl::PlanningControlHighLevelStatus)

