#include <interfaces/PnSDebug/PnSDebug.h>

#include <iterator>
#include <type_traits>

#include <cereal/cereal.hpp>
#include <cereal/archives/json.hpp>
#include <cereal/archives/binary.hpp>

#include <cereal/types/map.hpp>
#include <cereal/types/vector.hpp>
#include <cereal/types/array.hpp>
#include <cereal/types/string.hpp>
#include <cereal/types/complex.hpp>

#include <ODatum/IntrospectionArchive.h>
#include <ODatum/Hash.h>


 namespace av {
 namespace msg {
 namespace plan_ctrl {


SingleRealizationIterationInfo::SingleRealizationIterationInfo() = default;

SingleRealizationIterationInfo::SingleRealizationIterationInfo(
    SolverInfo MPCInfo_arg ,
    std::vector<NamedVectorOfDouble> horizonDecisionVariables_arg ,
    std::vector<NamedVectorOfDouble> horizonSolverParameters_arg ,
    std::vector<NamedVectorOfDouble> horizonAdditionalParameters_arg 
    ):
    MPCInfo { std::move(MPCInfo_arg) } // NOLINT
      ,
    horizonDecisionVariables { std::move(horizonDecisionVariables_arg) } // NOLINT
      ,
    horizonSolverParameters { std::move(horizonSolverParameters_arg) } // NOLINT
      ,
    horizonAdditionalParameters { std::move(horizonAdditionalParameters_arg) } // NOLINT
      
    {
}

SingleRealizationIterationInfo::SingleRealizationIterationInfo(const SingleRealizationIterationInfo&) = default;

// NOLINTNEXTLINE
SingleRealizationIterationInfo::SingleRealizationIterationInfo(SingleRealizationIterationInfo&&) = default;

SingleRealizationIterationInfo& SingleRealizationIterationInfo::operator=(const SingleRealizationIterationInfo&) = default;

// NOLINTNEXTLINE
SingleRealizationIterationInfo& SingleRealizationIterationInfo::operator=(SingleRealizationIterationInfo&&) = default;

void SingleRealizationIterationInfo::toCsv(std::ostream& out) const
{
    out.precision(16);
    out << std::boolalpha;
    out << MPCInfo;out << ",";
    // Enclose sequence in double quotations and curly brackets to keep CSV line length uniform
        
        out << "\"{{";
        
        if (!horizonDecisionVariables.empty())
        {
            std::copy(horizonDecisionVariables.begin(), horizonDecisionVariables.end() - 1, std::ostream_iterator<NamedVectorOfDouble>(out, "},{"));
            std::copy(horizonDecisionVariables.end()-1, horizonDecisionVariables.end(), std::ostream_iterator<NamedVectorOfDouble>(out, ""));
        }
        out << "}}\"";out << ",";
    // Enclose sequence in double quotations and curly brackets to keep CSV line length uniform
        
        out << "\"{{";
        
        if (!horizonSolverParameters.empty())
        {
            std::copy(horizonSolverParameters.begin(), horizonSolverParameters.end() - 1, std::ostream_iterator<NamedVectorOfDouble>(out, "},{"));
            std::copy(horizonSolverParameters.end()-1, horizonSolverParameters.end(), std::ostream_iterator<NamedVectorOfDouble>(out, ""));
        }
        out << "}}\"";out << ",";
    // Enclose sequence in double quotations and curly brackets to keep CSV line length uniform
        
        out << "\"{{";
        
        if (!horizonAdditionalParameters.empty())
        {
            std::copy(horizonAdditionalParameters.begin(), horizonAdditionalParameters.end() - 1, std::ostream_iterator<NamedVectorOfDouble>(out, "},{"));
            std::copy(horizonAdditionalParameters.end()-1, horizonAdditionalParameters.end(), std::ostream_iterator<NamedVectorOfDouble>(out, ""));
        }
        out << "}}\"";
    
}

std::stringstream SingleRealizationIterationInfo::toStringstream() const
{
    std::stringstream out;
    toCsv(out);
    return out;
}

template<class Archive>
    void SingleRealizationIterationInfo::serialize(Archive& ar, [[maybe_unused]] unsigned int version)
    {

        ar& CEREAL_NVP(MPCInfo);
        ar& CEREAL_NVP(horizonDecisionVariables);
        ar& CEREAL_NVP(horizonSolverParameters);
        ar& CEREAL_NVP(horizonAdditionalParameters);
    }


template void SingleRealizationIterationInfo::serialize(cereal::BinaryInputArchive& ar, unsigned int version);
template void SingleRealizationIterationInfo::serialize(cereal::BinaryOutputArchive& ar, unsigned int version);
template void SingleRealizationIterationInfo::serialize(cereal::IntrospectionArchive& ar, unsigned int version);
template void SingleRealizationIterationInfo::serialize(cereal::JSONInputArchive& ar, unsigned int version);
template void SingleRealizationIterationInfo::serialize(cereal::JSONOutputArchive& ar, unsigned int version);
template void SingleRealizationIterationInfo::serialize(cereal::PortableBinaryInputArchive& ar, unsigned int version);
template void SingleRealizationIterationInfo::serialize(cereal::PortableBinaryOutputArchive& ar, unsigned int version);

std::ostream& operator<< (std::ostream& os, const SingleRealizationIterationInfo& singlerealizationiterationinfo)
{
    singlerealizationiterationinfo.toCsv(os);
    return os;
}

bool operator== (const SingleRealizationIterationInfo &lhs, const SingleRealizationIterationInfo &rhs)
{
    return (  lhs.MPCInfo== rhs.MPCInfo &&
     lhs.horizonDecisionVariables== rhs.horizonDecisionVariables &&
     lhs.horizonSolverParameters== rhs.horizonSolverParameters &&
     lhs.horizonAdditionalParameters== rhs.horizonAdditionalParameters 
    );
}

std::size_t SingleRealizationIterationInfo::hash() const noexcept
{
    std::size_t result = 17;
    result = result * 31 + std::hash<SolverInfo>()(MPCInfo);
    result = result * 31 + hashVector(horizonDecisionVariables);
    result = result * 31 + hashVector(horizonSolverParameters);
    result = result * 31 + hashVector(horizonAdditionalParameters);
    return result;
}

 } // namespace av
 } // namespace msg
 } // namespace plan_ctrl


CEREAL_BIND_TO_ARCHIVES(av::msg::plan_ctrl::SingleRealizationIterationInfo)

 namespace av {
 namespace msg {
 namespace plan_ctrl {


SingleRealizerInfo::SingleRealizerInfo() = default;

SingleRealizerInfo::SingleRealizerInfo(
    PathPointMassModelTrajectory optimizedTrajectory_arg ,
    bool validTrajectory_arg ,
    std::vector<SingleRealizationIterationInfo> MPCInfos_arg ,
    uint64_t realizerIndex_arg 
    ):
    optimizedTrajectory { std::move(optimizedTrajectory_arg) } // NOLINT
      ,
    validTrajectory { std::move(validTrajectory_arg) } // NOLINT
      ,
    MPCInfos { std::move(MPCInfos_arg) } // NOLINT
      ,
    realizerIndex { std::move(realizerIndex_arg) } // NOLINT
      
    {
}

SingleRealizerInfo::SingleRealizerInfo(const SingleRealizerInfo&) = default;

// NOLINTNEXTLINE
SingleRealizerInfo::SingleRealizerInfo(SingleRealizerInfo&&) = default;

SingleRealizerInfo& SingleRealizerInfo::operator=(const SingleRealizerInfo&) = default;

// NOLINTNEXTLINE
SingleRealizerInfo& SingleRealizerInfo::operator=(SingleRealizerInfo&&) = default;

void SingleRealizerInfo::toCsv(std::ostream& out) const
{
    out.precision(16);
    out << std::boolalpha;
    out << optimizedTrajectory;out << ",";
    out << validTrajectory;out << ",";
    // Enclose sequence in double quotations and curly brackets to keep CSV line length uniform
        
        out << "\"{{";
        
        if (!MPCInfos.empty())
        {
            std::copy(MPCInfos.begin(), MPCInfos.end() - 1, std::ostream_iterator<SingleRealizationIterationInfo>(out, "},{"));
            std::copy(MPCInfos.end()-1, MPCInfos.end(), std::ostream_iterator<SingleRealizationIterationInfo>(out, ""));
        }
        out << "}}\"";out << ",";
    out << realizerIndex;
    
}

std::stringstream SingleRealizerInfo::toStringstream() const
{
    std::stringstream out;
    toCsv(out);
    return out;
}

template<class Archive>
    void SingleRealizerInfo::serialize(Archive& ar, [[maybe_unused]] unsigned int version)
    {

        ar& CEREAL_NVP(optimizedTrajectory);
        ar& CEREAL_NVP(validTrajectory);
        ar& CEREAL_NVP(MPCInfos);
        ar& CEREAL_NVP(realizerIndex);
    }


template void SingleRealizerInfo::serialize(cereal::BinaryInputArchive& ar, unsigned int version);
template void SingleRealizerInfo::serialize(cereal::BinaryOutputArchive& ar, unsigned int version);
template void SingleRealizerInfo::serialize(cereal::IntrospectionArchive& ar, unsigned int version);
template void SingleRealizerInfo::serialize(cereal::JSONInputArchive& ar, unsigned int version);
template void SingleRealizerInfo::serialize(cereal::JSONOutputArchive& ar, unsigned int version);
template void SingleRealizerInfo::serialize(cereal::PortableBinaryInputArchive& ar, unsigned int version);
template void SingleRealizerInfo::serialize(cereal::PortableBinaryOutputArchive& ar, unsigned int version);

std::ostream& operator<< (std::ostream& os, const SingleRealizerInfo& singlerealizerinfo)
{
    singlerealizerinfo.toCsv(os);
    return os;
}

bool operator== (const SingleRealizerInfo &lhs, const SingleRealizerInfo &rhs)
{
    return (  lhs.optimizedTrajectory== rhs.optimizedTrajectory &&
     lhs.validTrajectory== rhs.validTrajectory &&
     lhs.MPCInfos== rhs.MPCInfos &&
     lhs.realizerIndex== rhs.realizerIndex 
    );
}

std::size_t SingleRealizerInfo::hash() const noexcept
{
    std::size_t result = 17;
    result = result * 31 + std::hash<PathPointMassModelTrajectory>()(optimizedTrajectory);
    result = result * 31 + std::hash<bool>()(validTrajectory);
    result = result * 31 + hashVector(MPCInfos);
    result = result * 31 + std::hash<uint64_t>()(realizerIndex);
    return result;
}

 } // namespace av
 } // namespace msg
 } // namespace plan_ctrl


CEREAL_BIND_TO_ARCHIVES(av::msg::plan_ctrl::SingleRealizerInfo)

 namespace av {
 namespace msg {
 namespace plan_ctrl {


OptionalTrackedObjects::OptionalTrackedObjects() = default;

OptionalTrackedObjects::OptionalTrackedObjects(
    bool hasValue_arg ,
    tracker::tracked_objects_avt trackedObjects_arg 
    ):
    hasValue { std::move(hasValue_arg) } // NOLINT
      ,
    trackedObjects { std::move(trackedObjects_arg) } // NOLINT
      
    {
}

OptionalTrackedObjects::OptionalTrackedObjects(const OptionalTrackedObjects&) = default;

// NOLINTNEXTLINE
OptionalTrackedObjects::OptionalTrackedObjects(OptionalTrackedObjects&&) = default;

OptionalTrackedObjects& OptionalTrackedObjects::operator=(const OptionalTrackedObjects&) = default;

// NOLINTNEXTLINE
OptionalTrackedObjects& OptionalTrackedObjects::operator=(OptionalTrackedObjects&&) = default;

void OptionalTrackedObjects::toCsv(std::ostream& out) const
{
    out.precision(16);
    out << std::boolalpha;
    out << hasValue;out << ",";
    out << trackedObjects;
    
}

std::stringstream OptionalTrackedObjects::toStringstream() const
{
    std::stringstream out;
    toCsv(out);
    return out;
}

template<class Archive>
    void OptionalTrackedObjects::serialize(Archive& ar, [[maybe_unused]] unsigned int version)
    {

        ar& CEREAL_NVP(hasValue);
        ar& CEREAL_NVP(trackedObjects);
    }


template void OptionalTrackedObjects::serialize(cereal::BinaryInputArchive& ar, unsigned int version);
template void OptionalTrackedObjects::serialize(cereal::BinaryOutputArchive& ar, unsigned int version);
template void OptionalTrackedObjects::serialize(cereal::IntrospectionArchive& ar, unsigned int version);
template void OptionalTrackedObjects::serialize(cereal::JSONInputArchive& ar, unsigned int version);
template void OptionalTrackedObjects::serialize(cereal::JSONOutputArchive& ar, unsigned int version);
template void OptionalTrackedObjects::serialize(cereal::PortableBinaryInputArchive& ar, unsigned int version);
template void OptionalTrackedObjects::serialize(cereal::PortableBinaryOutputArchive& ar, unsigned int version);

std::ostream& operator<< (std::ostream& os, const OptionalTrackedObjects& optionaltrackedobjects)
{
    optionaltrackedobjects.toCsv(os);
    return os;
}

bool operator== (const OptionalTrackedObjects &lhs, const OptionalTrackedObjects &rhs)
{
    return (  lhs.hasValue== rhs.hasValue &&
     lhs.trackedObjects== rhs.trackedObjects 
    );
}

std::size_t OptionalTrackedObjects::hash() const noexcept
{
    std::size_t result = 17;
    result = result * 31 + std::hash<bool>()(hasValue);
    result = result * 31 + std::hash<tracker::tracked_objects_avt>()(trackedObjects);
    return result;
}

 } // namespace av
 } // namespace msg
 } // namespace plan_ctrl


CEREAL_BIND_TO_ARCHIVES(av::msg::plan_ctrl::OptionalTrackedObjects)

 namespace av {
 namespace msg {
 namespace plan_ctrl {


OptionalTrackedEntities::OptionalTrackedEntities() = default;

OptionalTrackedEntities::OptionalTrackedEntities(
    bool hasValue_arg ,
    av::msg::tracker::TrackedEntities trackedEntities_arg 
    ):
    hasValue { std::move(hasValue_arg) } // NOLINT
      ,
    trackedEntities { std::move(trackedEntities_arg) } // NOLINT
      
    {
}

OptionalTrackedEntities::OptionalTrackedEntities(const OptionalTrackedEntities&) = default;

// NOLINTNEXTLINE
OptionalTrackedEntities::OptionalTrackedEntities(OptionalTrackedEntities&&) = default;

OptionalTrackedEntities& OptionalTrackedEntities::operator=(const OptionalTrackedEntities&) = default;

// NOLINTNEXTLINE
OptionalTrackedEntities& OptionalTrackedEntities::operator=(OptionalTrackedEntities&&) = default;

void OptionalTrackedEntities::toCsv(std::ostream& out) const
{
    out.precision(16);
    out << std::boolalpha;
    out << hasValue;out << ",";
    out << trackedEntities;
    
}

std::stringstream OptionalTrackedEntities::toStringstream() const
{
    std::stringstream out;
    toCsv(out);
    return out;
}

template<class Archive>
    void OptionalTrackedEntities::serialize(Archive& ar, [[maybe_unused]] unsigned int version)
    {

        ar& CEREAL_NVP(hasValue);
        ar& CEREAL_NVP(trackedEntities);
    }


template void OptionalTrackedEntities::serialize(cereal::BinaryInputArchive& ar, unsigned int version);
template void OptionalTrackedEntities::serialize(cereal::BinaryOutputArchive& ar, unsigned int version);
template void OptionalTrackedEntities::serialize(cereal::IntrospectionArchive& ar, unsigned int version);
template void OptionalTrackedEntities::serialize(cereal::JSONInputArchive& ar, unsigned int version);
template void OptionalTrackedEntities::serialize(cereal::JSONOutputArchive& ar, unsigned int version);
template void OptionalTrackedEntities::serialize(cereal::PortableBinaryInputArchive& ar, unsigned int version);
template void OptionalTrackedEntities::serialize(cereal::PortableBinaryOutputArchive& ar, unsigned int version);

std::ostream& operator<< (std::ostream& os, const OptionalTrackedEntities& optionaltrackedentities)
{
    optionaltrackedentities.toCsv(os);
    return os;
}

bool operator== (const OptionalTrackedEntities &lhs, const OptionalTrackedEntities &rhs)
{
    return (  lhs.hasValue== rhs.hasValue &&
     lhs.trackedEntities== rhs.trackedEntities 
    );
}

std::size_t OptionalTrackedEntities::hash() const noexcept
{
    std::size_t result = 17;
    result = result * 31 + std::hash<bool>()(hasValue);
    result = result * 31 + std::hash<av::msg::tracker::TrackedEntities>()(trackedEntities);
    return result;
}

 } // namespace av
 } // namespace msg
 } // namespace plan_ctrl


CEREAL_BIND_TO_ARCHIVES(av::msg::plan_ctrl::OptionalTrackedEntities)

 namespace av {
 namespace msg {
 namespace plan_ctrl {


PnSCache::PnSCache() = default;

PnSCache::PnSCache(
    TrajectorySelectionCache trajectorySelection_arg ,
    LastSelectedRealizationCache lastSelectedRealization_arg 
    ):
    trajectorySelection { std::move(trajectorySelection_arg) } // NOLINT
      ,
    lastSelectedRealization { std::move(lastSelectedRealization_arg) } // NOLINT
      
    {
}

PnSCache::PnSCache(const PnSCache&) = default;

// NOLINTNEXTLINE
PnSCache::PnSCache(PnSCache&&) = default;

PnSCache& PnSCache::operator=(const PnSCache&) = default;

// NOLINTNEXTLINE
PnSCache& PnSCache::operator=(PnSCache&&) = default;

void PnSCache::toCsv(std::ostream& out) const
{
    out.precision(16);
    out << std::boolalpha;
    out << trajectorySelection;out << ",";
    out << lastSelectedRealization;
    
}

std::stringstream PnSCache::toStringstream() const
{
    std::stringstream out;
    toCsv(out);
    return out;
}

template<class Archive>
    void PnSCache::serialize(Archive& ar, unsigned int version)
{
    if (version >= 1)
    {
        ar& CEREAL_NVP(trajectorySelection);
    }
    if (version == 2)
    {
        ar& CEREAL_NVP(lastSelectedRealization);
    }
}


template void PnSCache::serialize(cereal::BinaryInputArchive& ar, unsigned int version);
template void PnSCache::serialize(cereal::BinaryOutputArchive& ar, unsigned int version);
template void PnSCache::serialize(cereal::IntrospectionArchive& ar, unsigned int version);
template void PnSCache::serialize(cereal::JSONInputArchive& ar, unsigned int version);
template void PnSCache::serialize(cereal::JSONOutputArchive& ar, unsigned int version);
template void PnSCache::serialize(cereal::PortableBinaryInputArchive& ar, unsigned int version);
template void PnSCache::serialize(cereal::PortableBinaryOutputArchive& ar, unsigned int version);

std::ostream& operator<< (std::ostream& os, const PnSCache& pnscache)
{
    pnscache.toCsv(os);
    return os;
}

bool operator== (const PnSCache &lhs, const PnSCache &rhs)
{
    return (  lhs.trajectorySelection== rhs.trajectorySelection &&
     lhs.lastSelectedRealization== rhs.lastSelectedRealization 
    );
}

std::size_t PnSCache::hash() const noexcept
{
    std::size_t result = 17;
    result = result * 31 + std::hash<TrajectorySelectionCache>()(trajectorySelection);
    result = result * 31 + std::hash<LastSelectedRealizationCache>()(lastSelectedRealization);
    return result;
}

 } // namespace av
 } // namespace msg
 } // namespace plan_ctrl


CEREAL_BIND_TO_ARCHIVES(av::msg::plan_ctrl::PnSCache)

 namespace av {
 namespace msg {
 namespace plan_ctrl {


AppliedFilterInfo::AppliedFilterInfo() = default;

AppliedFilterInfo::AppliedFilterInfo(
    std::string filterLabel_arg ,
    AppliedFilterType filterType_arg ,
    bool rejected_arg ,
    std::string reason_arg ,
    std::string debug_arg 
    ):
    filterLabel { std::move(filterLabel_arg) } // NOLINT
      ,
    filterType { std::move(filterType_arg) } // NOLINT
      ,
    rejected { std::move(rejected_arg) } // NOLINT
      ,
    reason { std::move(reason_arg) } // NOLINT
      ,
    debug { std::move(debug_arg) } // NOLINT
      
    {
}

AppliedFilterInfo::AppliedFilterInfo(const AppliedFilterInfo&) = default;

// NOLINTNEXTLINE
AppliedFilterInfo::AppliedFilterInfo(AppliedFilterInfo&&) = default;

AppliedFilterInfo& AppliedFilterInfo::operator=(const AppliedFilterInfo&) = default;

// NOLINTNEXTLINE
AppliedFilterInfo& AppliedFilterInfo::operator=(AppliedFilterInfo&&) = default;

void AppliedFilterInfo::toCsv(std::ostream& out) const
{
    out.precision(16);
    out << std::boolalpha;
    out << "\"";
        out << filterLabel;
        out << "\"";out << ",";
    out << static_cast<uint32_t>(filterType);out << ",";
    out << rejected;out << ",";
    out << "\"";
        out << reason;
        out << "\"";out << ",";
    out << "\"";
        out << debug;
        out << "\"";
    
}

std::stringstream AppliedFilterInfo::toStringstream() const
{
    std::stringstream out;
    toCsv(out);
    return out;
}

template<class Archive>
    void AppliedFilterInfo::serialize(Archive& ar, [[maybe_unused]] unsigned int version)
    {

        ar& CEREAL_NVP(filterLabel);
        ar& CEREAL_NVP(filterType);
        ar& CEREAL_NVP(rejected);
        ar& CEREAL_NVP(reason);
        ar& CEREAL_NVP(debug);
    }


template void AppliedFilterInfo::serialize(cereal::BinaryInputArchive& ar, unsigned int version);
template void AppliedFilterInfo::serialize(cereal::BinaryOutputArchive& ar, unsigned int version);
template void AppliedFilterInfo::serialize(cereal::IntrospectionArchive& ar, unsigned int version);
template void AppliedFilterInfo::serialize(cereal::JSONInputArchive& ar, unsigned int version);
template void AppliedFilterInfo::serialize(cereal::JSONOutputArchive& ar, unsigned int version);
template void AppliedFilterInfo::serialize(cereal::PortableBinaryInputArchive& ar, unsigned int version);
template void AppliedFilterInfo::serialize(cereal::PortableBinaryOutputArchive& ar, unsigned int version);

std::ostream& operator<< (std::ostream& os, const AppliedFilterInfo& appliedfilterinfo)
{
    appliedfilterinfo.toCsv(os);
    return os;
}

bool operator== (const AppliedFilterInfo &lhs, const AppliedFilterInfo &rhs)
{
    return (  lhs.filterLabel== rhs.filterLabel &&
     lhs.filterType== rhs.filterType &&
     lhs.rejected== rhs.rejected &&
     lhs.reason== rhs.reason &&
     lhs.debug== rhs.debug 
    );
}

std::size_t AppliedFilterInfo::hash() const noexcept
{
    std::size_t result = 17;
    result = result * 31 + std::hash<std::string>()(filterLabel);
    result = result * 31 + std::hash<AppliedFilterType>()(filterType);
    result = result * 31 + std::hash<bool>()(rejected);
    result = result * 31 + std::hash<std::string>()(reason);
    result = result * 31 + std::hash<std::string>()(debug);
    return result;
}

 } // namespace av
 } // namespace msg
 } // namespace plan_ctrl


CEREAL_BIND_TO_ARCHIVES(av::msg::plan_ctrl::AppliedFilterInfo)

 namespace av {
 namespace msg {
 namespace plan_ctrl {


PnSFilterVisualizationInfo::PnSFilterVisualizationInfo() = default;

PnSFilterVisualizationInfo::PnSFilterVisualizationInfo(
    int64_t agentID_arg ,
    std::string agentType_arg ,
    std::vector<AppliedFilterInfo> appliedFilters_arg ,
    std::vector<AppliedFilterInfo> activeFilters_arg 
    ):
    agentID { std::move(agentID_arg) } // NOLINT
      ,
    agentType { std::move(agentType_arg) } // NOLINT
      ,
    appliedFilters { std::move(appliedFilters_arg) } // NOLINT
      ,
    activeFilters { std::move(activeFilters_arg) } // NOLINT
      
    {
}

PnSFilterVisualizationInfo::PnSFilterVisualizationInfo(const PnSFilterVisualizationInfo&) = default;

// NOLINTNEXTLINE
PnSFilterVisualizationInfo::PnSFilterVisualizationInfo(PnSFilterVisualizationInfo&&) = default;

PnSFilterVisualizationInfo& PnSFilterVisualizationInfo::operator=(const PnSFilterVisualizationInfo&) = default;

// NOLINTNEXTLINE
PnSFilterVisualizationInfo& PnSFilterVisualizationInfo::operator=(PnSFilterVisualizationInfo&&) = default;

void PnSFilterVisualizationInfo::toCsv(std::ostream& out) const
{
    out.precision(16);
    out << std::boolalpha;
    out << agentID;out << ",";
    out << "\"";
        out << agentType;
        out << "\"";out << ",";
    // Enclose sequence in double quotations and curly brackets to keep CSV line length uniform
        
        out << "\"{{";
        
        if (!appliedFilters.empty())
        {
            std::copy(appliedFilters.begin(), appliedFilters.end() - 1, std::ostream_iterator<AppliedFilterInfo>(out, "},{"));
            std::copy(appliedFilters.end()-1, appliedFilters.end(), std::ostream_iterator<AppliedFilterInfo>(out, ""));
        }
        out << "}}\"";out << ",";
    // Enclose sequence in double quotations and curly brackets to keep CSV line length uniform
        
        out << "\"{{";
        
        if (!activeFilters.empty())
        {
            std::copy(activeFilters.begin(), activeFilters.end() - 1, std::ostream_iterator<AppliedFilterInfo>(out, "},{"));
            std::copy(activeFilters.end()-1, activeFilters.end(), std::ostream_iterator<AppliedFilterInfo>(out, ""));
        }
        out << "}}\"";
    
}

std::stringstream PnSFilterVisualizationInfo::toStringstream() const
{
    std::stringstream out;
    toCsv(out);
    return out;
}

template<class Archive>
    void PnSFilterVisualizationInfo::serialize(Archive& ar, [[maybe_unused]] unsigned int version)
    {

        ar& CEREAL_NVP(agentID);
        ar& CEREAL_NVP(agentType);
        ar& CEREAL_NVP(appliedFilters);
        ar& CEREAL_NVP(activeFilters);
    }


template void PnSFilterVisualizationInfo::serialize(cereal::BinaryInputArchive& ar, unsigned int version);
template void PnSFilterVisualizationInfo::serialize(cereal::BinaryOutputArchive& ar, unsigned int version);
template void PnSFilterVisualizationInfo::serialize(cereal::IntrospectionArchive& ar, unsigned int version);
template void PnSFilterVisualizationInfo::serialize(cereal::JSONInputArchive& ar, unsigned int version);
template void PnSFilterVisualizationInfo::serialize(cereal::JSONOutputArchive& ar, unsigned int version);
template void PnSFilterVisualizationInfo::serialize(cereal::PortableBinaryInputArchive& ar, unsigned int version);
template void PnSFilterVisualizationInfo::serialize(cereal::PortableBinaryOutputArchive& ar, unsigned int version);

std::ostream& operator<< (std::ostream& os, const PnSFilterVisualizationInfo& pnsfiltervisualizationinfo)
{
    pnsfiltervisualizationinfo.toCsv(os);
    return os;
}

bool operator== (const PnSFilterVisualizationInfo &lhs, const PnSFilterVisualizationInfo &rhs)
{
    return (  lhs.agentID== rhs.agentID &&
     lhs.agentType== rhs.agentType &&
     lhs.appliedFilters== rhs.appliedFilters &&
     lhs.activeFilters== rhs.activeFilters 
    );
}

std::size_t PnSFilterVisualizationInfo::hash() const noexcept
{
    std::size_t result = 17;
    result = result * 31 + std::hash<int64_t>()(agentID);
    result = result * 31 + std::hash<std::string>()(agentType);
    result = result * 31 + hashVector(appliedFilters);
    result = result * 31 + hashVector(activeFilters);
    return result;
}

 } // namespace av
 } // namespace msg
 } // namespace plan_ctrl


CEREAL_BIND_TO_ARCHIVES(av::msg::plan_ctrl::PnSFilterVisualizationInfo)

 namespace av {
 namespace msg {
 namespace plan_ctrl {


TrajectorySelectorVisualizationInfo::TrajectorySelectorVisualizationInfo() = default;

TrajectorySelectorVisualizationInfo::TrajectorySelectorVisualizationInfo(
    TrajectoryOrigin origin_arg ,
    std::string trajectoryDescription_arg 
    ):
    origin { std::move(origin_arg) } // NOLINT
      ,
    trajectoryDescription { std::move(trajectoryDescription_arg) } // NOLINT
      
    {
}

TrajectorySelectorVisualizationInfo::TrajectorySelectorVisualizationInfo(const TrajectorySelectorVisualizationInfo&) = default;

// NOLINTNEXTLINE
TrajectorySelectorVisualizationInfo::TrajectorySelectorVisualizationInfo(TrajectorySelectorVisualizationInfo&&) = default;

TrajectorySelectorVisualizationInfo& TrajectorySelectorVisualizationInfo::operator=(const TrajectorySelectorVisualizationInfo&) = default;

// NOLINTNEXTLINE
TrajectorySelectorVisualizationInfo& TrajectorySelectorVisualizationInfo::operator=(TrajectorySelectorVisualizationInfo&&) = default;

void TrajectorySelectorVisualizationInfo::toCsv(std::ostream& out) const
{
    out.precision(16);
    out << std::boolalpha;
    out << static_cast<uint32_t>(origin);out << ",";
    out << "\"";
        out << trajectoryDescription;
        out << "\"";
    
}

std::stringstream TrajectorySelectorVisualizationInfo::toStringstream() const
{
    std::stringstream out;
    toCsv(out);
    return out;
}

template<class Archive>
    void TrajectorySelectorVisualizationInfo::serialize(Archive& ar, [[maybe_unused]] unsigned int version)
    {

        ar& CEREAL_NVP(origin);
        ar& CEREAL_NVP(trajectoryDescription);
    }


template void TrajectorySelectorVisualizationInfo::serialize(cereal::BinaryInputArchive& ar, unsigned int version);
template void TrajectorySelectorVisualizationInfo::serialize(cereal::BinaryOutputArchive& ar, unsigned int version);
template void TrajectorySelectorVisualizationInfo::serialize(cereal::IntrospectionArchive& ar, unsigned int version);
template void TrajectorySelectorVisualizationInfo::serialize(cereal::JSONInputArchive& ar, unsigned int version);
template void TrajectorySelectorVisualizationInfo::serialize(cereal::JSONOutputArchive& ar, unsigned int version);
template void TrajectorySelectorVisualizationInfo::serialize(cereal::PortableBinaryInputArchive& ar, unsigned int version);
template void TrajectorySelectorVisualizationInfo::serialize(cereal::PortableBinaryOutputArchive& ar, unsigned int version);

std::ostream& operator<< (std::ostream& os, const TrajectorySelectorVisualizationInfo& trajectoryselectorvisualizationinfo)
{
    trajectoryselectorvisualizationinfo.toCsv(os);
    return os;
}

bool operator== (const TrajectorySelectorVisualizationInfo &lhs, const TrajectorySelectorVisualizationInfo &rhs)
{
    return (  lhs.origin== rhs.origin &&
     lhs.trajectoryDescription== rhs.trajectoryDescription 
    );
}

std::size_t TrajectorySelectorVisualizationInfo::hash() const noexcept
{
    std::size_t result = 17;
    result = result * 31 + std::hash<TrajectoryOrigin>()(origin);
    result = result * 31 + std::hash<std::string>()(trajectoryDescription);
    return result;
}

 } // namespace av
 } // namespace msg
 } // namespace plan_ctrl


CEREAL_BIND_TO_ARCHIVES(av::msg::plan_ctrl::TrajectorySelectorVisualizationInfo)

 namespace av {
 namespace msg {
 namespace plan_ctrl {


OptionalTrafficLightAheadOfEgo::OptionalTrafficLightAheadOfEgo() = default;

OptionalTrafficLightAheadOfEgo::OptionalTrafficLightAheadOfEgo(
    bool hasValue_arg ,
    IntersectionTLStatus trafficLightAheadOfEgo_arg 
    ):
    hasValue { std::move(hasValue_arg) } // NOLINT
      ,
    trafficLightAheadOfEgo { std::move(trafficLightAheadOfEgo_arg) } // NOLINT
      
    {
}

OptionalTrafficLightAheadOfEgo::OptionalTrafficLightAheadOfEgo(const OptionalTrafficLightAheadOfEgo&) = default;

// NOLINTNEXTLINE
OptionalTrafficLightAheadOfEgo::OptionalTrafficLightAheadOfEgo(OptionalTrafficLightAheadOfEgo&&) = default;

OptionalTrafficLightAheadOfEgo& OptionalTrafficLightAheadOfEgo::operator=(const OptionalTrafficLightAheadOfEgo&) = default;

// NOLINTNEXTLINE
OptionalTrafficLightAheadOfEgo& OptionalTrafficLightAheadOfEgo::operator=(OptionalTrafficLightAheadOfEgo&&) = default;

void OptionalTrafficLightAheadOfEgo::toCsv(std::ostream& out) const
{
    out.precision(16);
    out << std::boolalpha;
    out << hasValue;out << ",";
    out << trafficLightAheadOfEgo;
    
}

std::stringstream OptionalTrafficLightAheadOfEgo::toStringstream() const
{
    std::stringstream out;
    toCsv(out);
    return out;
}

template<class Archive>
    void OptionalTrafficLightAheadOfEgo::serialize(Archive& ar, [[maybe_unused]] unsigned int version)
    {

        ar& CEREAL_NVP(hasValue);
        ar& CEREAL_NVP(trafficLightAheadOfEgo);
    }


template void OptionalTrafficLightAheadOfEgo::serialize(cereal::BinaryInputArchive& ar, unsigned int version);
template void OptionalTrafficLightAheadOfEgo::serialize(cereal::BinaryOutputArchive& ar, unsigned int version);
template void OptionalTrafficLightAheadOfEgo::serialize(cereal::IntrospectionArchive& ar, unsigned int version);
template void OptionalTrafficLightAheadOfEgo::serialize(cereal::JSONInputArchive& ar, unsigned int version);
template void OptionalTrafficLightAheadOfEgo::serialize(cereal::JSONOutputArchive& ar, unsigned int version);
template void OptionalTrafficLightAheadOfEgo::serialize(cereal::PortableBinaryInputArchive& ar, unsigned int version);
template void OptionalTrafficLightAheadOfEgo::serialize(cereal::PortableBinaryOutputArchive& ar, unsigned int version);

std::ostream& operator<< (std::ostream& os, const OptionalTrafficLightAheadOfEgo& optionaltrafficlightaheadofego)
{
    optionaltrafficlightaheadofego.toCsv(os);
    return os;
}

bool operator== (const OptionalTrafficLightAheadOfEgo &lhs, const OptionalTrafficLightAheadOfEgo &rhs)
{
    return (  lhs.hasValue== rhs.hasValue &&
     lhs.trafficLightAheadOfEgo== rhs.trafficLightAheadOfEgo 
    );
}

std::size_t OptionalTrafficLightAheadOfEgo::hash() const noexcept
{
    std::size_t result = 17;
    result = result * 31 + std::hash<bool>()(hasValue);
    result = result * 31 + std::hash<IntersectionTLStatus>()(trafficLightAheadOfEgo);
    return result;
}

 } // namespace av
 } // namespace msg
 } // namespace plan_ctrl


CEREAL_BIND_TO_ARCHIVES(av::msg::plan_ctrl::OptionalTrafficLightAheadOfEgo)




 namespace av {
 namespace msg {
 namespace plan_ctrl {


PnSDebug::PnSDebug() = default;

PnSDebug::PnSDebug(
    int64_t timeStamp_us_arg ,
    MapParams mapParams_arg ,
    EgoStateExtended x0_arg ,
    RoutePlanWithStaticConstraints routePlan_arg ,
    AvtestWorld testWorld_arg ,
    AvtestDynamicWorld dynamicWorld_arg ,
    PathPointMassModelTrajectory committedTrajectory_arg ,
    route_obstacles obstacles_arg ,
    std::vector<WorldTrackDecisions> foundOptions_arg ,
    std::vector<HomotopyOverPlan> homotopies_arg ,
    std::vector<ClusterInfo> clusterInfos_arg ,
    std::vector<SingleRealizerInfo> realizerInfos_arg ,
    int64_t selectedTrajectoryIndex_arg ,
    PathPointMassModelTrajectory selectedTrajectory_arg ,
    PnSCache pnsCache_arg ,
    LaneChangeEarlyWarning earlyWarning_arg ,
    std::vector<PnSFilterVisualizationInfo> pnsFilterVisualizationInfos_arg ,
    OptionalTrackedEntities trackedEntities_arg ,
    int64_t cycleCounter_arg ,
    MultiIntersectionTLStatus multiIntersectionTLStatus_arg ,
    TrajectorySelectorVisualizationInfo trajectorySelectorVisualizationInfo_arg ,
    std::vector<VectorOfGlobalStaticReference> globalStaticReferences_arg ,
    OptionalTrafficLightAheadOfEgo trafficLightAheadOfEgo_arg 
    ):
    timeStamp_us { std::move(timeStamp_us_arg) } // NOLINT
      ,mapParams { std::move(mapParams_arg) } // NOLINT
      ,x0 { std::move(x0_arg) } // NOLINT
      ,routePlan { std::move(routePlan_arg) } // NOLINT
      ,testWorld { std::move(testWorld_arg) } // NOLINT
      ,dynamicWorld { std::move(dynamicWorld_arg) } // NOLINT
      ,committedTrajectory { std::move(committedTrajectory_arg) } // NOLINT
      ,obstacles { std::move(obstacles_arg) } // NOLINT
      ,foundOptions { std::move(foundOptions_arg) } // NOLINT
      ,homotopies { std::move(homotopies_arg) } // NOLINT
      ,clusterInfos { std::move(clusterInfos_arg) } // NOLINT
      ,realizerInfos { std::move(realizerInfos_arg) } // NOLINT
      ,selectedTrajectoryIndex { std::move(selectedTrajectoryIndex_arg) } // NOLINT
      ,selectedTrajectory { std::move(selectedTrajectory_arg) } // NOLINT
      ,pnsCache { std::move(pnsCache_arg) } // NOLINT
      ,earlyWarning { std::move(earlyWarning_arg) } // NOLINT
      ,pnsFilterVisualizationInfos { std::move(pnsFilterVisualizationInfos_arg) } // NOLINT
      ,trackedEntities { std::move(trackedEntities_arg) } // NOLINT
      ,cycleCounter { std::move(cycleCounter_arg) } // NOLINT
      ,multiIntersectionTLStatus { std::move(multiIntersectionTLStatus_arg) } // NOLINT
      ,trajectorySelectorVisualizationInfo { std::move(trajectorySelectorVisualizationInfo_arg) } // NOLINT
      ,globalStaticReferences { std::move(globalStaticReferences_arg) } // NOLINT
      ,trafficLightAheadOfEgo { std::move(trafficLightAheadOfEgo_arg) } // NOLINT
      {
}

PnSDebug::PnSDebug(const PnSDebug&) = default;

// NOLINTNEXTLINE
PnSDebug::PnSDebug(PnSDebug&&) = default;

PnSDebug& PnSDebug::operator=(const PnSDebug&) = default;

// NOLINTNEXTLINE
PnSDebug& PnSDebug::operator=(PnSDebug&&) = default;

void PnSDebug::toCsv(std::ostream& out) const
{
    out.precision(16);
    out << std::boolalpha;
    out << timeStamp_us;out << ",";
    out << mapParams;out << ",";
    out << x0;out << ",";
    out << routePlan;out << ",";
    out << testWorld;out << ",";
    out << dynamicWorld;out << ",";
    out << committedTrajectory;out << ",";
    out << obstacles;out << ",";
    // Enclose sequence in double quotations and curly brackets to keep CSV line length uniform
        
        out << "\"{{";
        
        if (!foundOptions.empty())
        {
            std::copy(foundOptions.begin(), foundOptions.end() - 1, std::ostream_iterator<WorldTrackDecisions>(out, "},{"));
            std::copy(foundOptions.end()-1, foundOptions.end(), std::ostream_iterator<WorldTrackDecisions>(out, ""));
        }
        out << "}}\"";out << ",";
    // Enclose sequence in double quotations and curly brackets to keep CSV line length uniform
        
        out << "\"{{";
        
        if (!homotopies.empty())
        {
            std::copy(homotopies.begin(), homotopies.end() - 1, std::ostream_iterator<HomotopyOverPlan>(out, "},{"));
            std::copy(homotopies.end()-1, homotopies.end(), std::ostream_iterator<HomotopyOverPlan>(out, ""));
        }
        out << "}}\"";out << ",";
    // Enclose sequence in double quotations and curly brackets to keep CSV line length uniform
        
        out << "\"{{";
        
        if (!clusterInfos.empty())
        {
            std::copy(clusterInfos.begin(), clusterInfos.end() - 1, std::ostream_iterator<ClusterInfo>(out, "},{"));
            std::copy(clusterInfos.end()-1, clusterInfos.end(), std::ostream_iterator<ClusterInfo>(out, ""));
        }
        out << "}}\"";out << ",";
    // Enclose sequence in double quotations and curly brackets to keep CSV line length uniform
        
        out << "\"{{";
        
        if (!realizerInfos.empty())
        {
            std::copy(realizerInfos.begin(), realizerInfos.end() - 1, std::ostream_iterator<SingleRealizerInfo>(out, "},{"));
            std::copy(realizerInfos.end()-1, realizerInfos.end(), std::ostream_iterator<SingleRealizerInfo>(out, ""));
        }
        out << "}}\"";out << ",";
    out << selectedTrajectoryIndex;out << ",";
    out << selectedTrajectory;out << ",";
    out << pnsCache;out << ",";
    out << earlyWarning;out << ",";
    // Enclose sequence in double quotations and curly brackets to keep CSV line length uniform
        
        out << "\"{{";
        
        if (!pnsFilterVisualizationInfos.empty())
        {
            std::copy(pnsFilterVisualizationInfos.begin(), pnsFilterVisualizationInfos.end() - 1, std::ostream_iterator<PnSFilterVisualizationInfo>(out, "},{"));
            std::copy(pnsFilterVisualizationInfos.end()-1, pnsFilterVisualizationInfos.end(), std::ostream_iterator<PnSFilterVisualizationInfo>(out, ""));
        }
        out << "}}\"";out << ",";
    out << trackedEntities;out << ",";
    out << cycleCounter;out << ",";
    out << multiIntersectionTLStatus;out << ",";
    out << trajectorySelectorVisualizationInfo;out << ",";
    // Enclose sequence in double quotations and curly brackets to keep CSV line length uniform
        
        out << "\"{{";
        
        if (!globalStaticReferences.empty())
        {
            std::copy(globalStaticReferences.begin(), globalStaticReferences.end() - 1, std::ostream_iterator<VectorOfGlobalStaticReference>(out, "},{"));
            std::copy(globalStaticReferences.end()-1, globalStaticReferences.end(), std::ostream_iterator<VectorOfGlobalStaticReference>(out, ""));
        }
        out << "}}\"";out << ",";
    out << trafficLightAheadOfEgo;
    
}

std::stringstream PnSDebug::toStringstream() const
{
    std::stringstream out;
    toCsv(out);
    return out;
}

template<class Archive>
    void PnSDebug::serialize(Archive& ar, unsigned int version)
{
    if (version >= 1)
	{
		ar& CEREAL_NVP(timeStamp_us);
        ar& CEREAL_NVP(mapParams);
		ar& CEREAL_NVP(x0);
		ar& CEREAL_NVP(routePlan);
		ar& CEREAL_NVP(testWorld);
		ar& CEREAL_NVP(dynamicWorld);
		ar& CEREAL_NVP(committedTrajectory);
		ar& CEREAL_NVP(obstacles);
        ar& CEREAL_NVP(foundOptions);
		ar& CEREAL_NVP(homotopies);
		ar& CEREAL_NVP(realizerInfos);
		ar& CEREAL_NVP(selectedTrajectoryIndex);
		ar& CEREAL_NVP(selectedTrajectory);
	}
    if(version >= 2)
    {
        ar& CEREAL_NVP(clusterInfos);
    }
    if (version >= 3 && version <= 6)
    {
        // dummy code in order to deserialize the old data type
        OptionalTrackedObjects oldTrackedObjects;
        ar& CEREAL_NVP(oldTrackedObjects);
        trackedEntities.hasValue = false;
    }
    if(version >= 4)
    {
        ar& CEREAL_NVP(pnsCache);
    }
    if(version >= 5)
    {
        ar& CEREAL_NVP(earlyWarning);
    }
    if(version >= 6)
    {
        ar& CEREAL_NVP(pnsFilterVisualizationInfos);
    }
    if(version >= 7)
    {
        ar& CEREAL_NVP(trackedEntities);
    }
    if(version >= 8)
    {
        ar& CEREAL_NVP(cycleCounter);
    }
    if(version >= 9)
    {
        ar& CEREAL_NVP(trajectorySelectorVisualizationInfo);
    }
    if(version >= 10)
    {
        ar& CEREAL_NVP(multiIntersectionTLStatus);
    }
    if(version >= 12)
    {
        ar& CEREAL_NVP(globalStaticReferences);
    }
    if(version >= 13)
    {
        ar& CEREAL_NVP(trafficLightAheadOfEgo);
    }
}



template void PnSDebug::serialize(cereal::BinaryInputArchive& ar, unsigned int version);
template void PnSDebug::serialize(cereal::BinaryOutputArchive& ar, unsigned int version);
template void PnSDebug::serialize(cereal::IntrospectionArchive& ar, unsigned int version);
template void PnSDebug::serialize(cereal::JSONInputArchive& ar, unsigned int version);
template void PnSDebug::serialize(cereal::JSONOutputArchive& ar, unsigned int version);
template void PnSDebug::serialize(cereal::PortableBinaryInputArchive& ar, unsigned int version);
template void PnSDebug::serialize(cereal::PortableBinaryOutputArchive& ar, unsigned int version);

std::ostream& operator<< (std::ostream& os, const PnSDebug& pnsdebug)
{
    pnsdebug.toCsv(os);
    return os;
}

bool operator== (const PnSDebug &lhs, const PnSDebug &rhs)
{
    return (  lhs.timeStamp_us== rhs.timeStamp_us &&
     lhs.mapParams== rhs.mapParams &&
     lhs.x0== rhs.x0 &&
     lhs.routePlan== rhs.routePlan &&
     lhs.testWorld== rhs.testWorld &&
     lhs.dynamicWorld== rhs.dynamicWorld &&
     lhs.committedTrajectory== rhs.committedTrajectory &&
     lhs.obstacles== rhs.obstacles &&
     lhs.foundOptions== rhs.foundOptions &&
     lhs.homotopies== rhs.homotopies &&
     lhs.clusterInfos== rhs.clusterInfos &&
     lhs.realizerInfos== rhs.realizerInfos &&
     lhs.selectedTrajectoryIndex== rhs.selectedTrajectoryIndex &&
     lhs.selectedTrajectory== rhs.selectedTrajectory &&
     lhs.pnsCache== rhs.pnsCache &&
     lhs.earlyWarning== rhs.earlyWarning &&
     lhs.pnsFilterVisualizationInfos== rhs.pnsFilterVisualizationInfos &&
     lhs.trackedEntities== rhs.trackedEntities &&
     lhs.cycleCounter== rhs.cycleCounter &&
     lhs.multiIntersectionTLStatus== rhs.multiIntersectionTLStatus &&
     lhs.trajectorySelectorVisualizationInfo== rhs.trajectorySelectorVisualizationInfo &&
     lhs.globalStaticReferences== rhs.globalStaticReferences &&
     lhs.trafficLightAheadOfEgo== rhs.trafficLightAheadOfEgo 
    );
}

std::size_t PnSDebug::hash() const noexcept
{
    std::size_t result = 17;
    result = result * 31 + std::hash<int64_t>()(timeStamp_us);
    result = result * 31 + std::hash<MapParams>()(mapParams);
    result = result * 31 + std::hash<EgoStateExtended>()(x0);
    result = result * 31 + std::hash<RoutePlanWithStaticConstraints>()(routePlan);
    result = result * 31 + std::hash<AvtestWorld>()(testWorld);
    result = result * 31 + std::hash<AvtestDynamicWorld>()(dynamicWorld);
    result = result * 31 + std::hash<PathPointMassModelTrajectory>()(committedTrajectory);
    result = result * 31 + std::hash<route_obstacles>()(obstacles);
    result = result * 31 + hashVector(foundOptions);
    result = result * 31 + hashVector(homotopies);
    result = result * 31 + hashVector(clusterInfos);
    result = result * 31 + hashVector(realizerInfos);
    result = result * 31 + std::hash<int64_t>()(selectedTrajectoryIndex);
    result = result * 31 + std::hash<PathPointMassModelTrajectory>()(selectedTrajectory);
    result = result * 31 + std::hash<PnSCache>()(pnsCache);
    result = result * 31 + std::hash<LaneChangeEarlyWarning>()(earlyWarning);
    result = result * 31 + hashVector(pnsFilterVisualizationInfos);
    result = result * 31 + std::hash<OptionalTrackedEntities>()(trackedEntities);
    result = result * 31 + std::hash<int64_t>()(cycleCounter);
    result = result * 31 + std::hash<MultiIntersectionTLStatus>()(multiIntersectionTLStatus);
    result = result * 31 + std::hash<TrajectorySelectorVisualizationInfo>()(trajectorySelectorVisualizationInfo);
    result = result * 31 + hashVector(globalStaticReferences);
    result = result * 31 + std::hash<OptionalTrafficLightAheadOfEgo>()(trafficLightAheadOfEgo);
    return result;
}

 } // namespace av
 } // namespace msg
 } // namespace plan_ctrl


CEREAL_BIND_TO_ARCHIVES(av::msg::plan_ctrl::PnSDebug)

