#include <interfaces/Point/Point.h>

#include <iterator>
#include <type_traits>

#include <cereal/cereal.hpp>
#include <cereal/archives/json.hpp>
#include <cereal/archives/binary.hpp>

#include <cereal/types/map.hpp>
#include <cereal/types/vector.hpp>
#include <cereal/types/array.hpp>
#include <cereal/types/string.hpp>
#include <cereal/types/complex.hpp>

#include <ODatum/IntrospectionArchive.h>
#include <ODatum/Hash.h>


 namespace av {


Point::Point() = default;

Point::Point(
    double x_arg ,
    double y_arg ,
    double z_arg 
    ):
    x { std::move(x_arg) } // NOLINT
      ,
    y { std::move(y_arg) } // NOLINT
      ,
    z { std::move(z_arg) } // NOLINT
      
    {
}

Point::Point(const Point&) = default;

// NOLINTNEXTLINE
Point::Point(Point&&) = default;

Point& Point::operator=(const Point&) = default;

// NOLINTNEXTLINE
Point& Point::operator=(Point&&) = default;

void Point::toCsv(std::ostream& out) const
{
    out.precision(16);
    out << std::boolalpha;
    out << x;out << ",";
    out << y;out << ",";
    out << z;
    
}

std::stringstream Point::toStringstream() const
{
    std::stringstream out;
    toCsv(out);
    return out;
}

template<class Archive>
    void Point::serialize(Archive& ar, [[maybe_unused]] unsigned int version)
    {

        ar& CEREAL_NVP(x);
        ar& CEREAL_NVP(y);
        ar& CEREAL_NVP(z);
    }


template void Point::serialize(cereal::BinaryInputArchive& ar, unsigned int version);
template void Point::serialize(cereal::BinaryOutputArchive& ar, unsigned int version);
template void Point::serialize(cereal::IntrospectionArchive& ar, unsigned int version);
template void Point::serialize(cereal::JSONInputArchive& ar, unsigned int version);
template void Point::serialize(cereal::JSONOutputArchive& ar, unsigned int version);
template void Point::serialize(cereal::PortableBinaryInputArchive& ar, unsigned int version);
template void Point::serialize(cereal::PortableBinaryOutputArchive& ar, unsigned int version);

std::ostream& operator<< (std::ostream& os, const Point& point)
{
    point.toCsv(os);
    return os;
}

bool operator== (const Point &lhs, const Point &rhs)
{
    return (  lhs.x== rhs.x &&
     lhs.y== rhs.y &&
     lhs.z== rhs.z 
    );
}

std::size_t Point::hash() const noexcept
{
    std::size_t result = 17;
    result = result * 31 + std::hash<double>()(x);
    result = result * 31 + std::hash<double>()(y);
    result = result * 31 + std::hash<double>()(z);
    return result;
}

 } // namespace av


CEREAL_BIND_TO_ARCHIVES(av::Point)


