#include <interfaces/Point2D/Point2D.h>

#include <iterator>
#include <type_traits>

#include <cereal/cereal.hpp>
#include <cereal/archives/json.hpp>
#include <cereal/archives/binary.hpp>

#include <cereal/types/map.hpp>
#include <cereal/types/vector.hpp>
#include <cereal/types/array.hpp>
#include <cereal/types/string.hpp>
#include <cereal/types/complex.hpp>

#include <ODatum/IntrospectionArchive.h>
#include <ODatum/Hash.h>


 namespace av {
 namespace msg {


Point2D::Point2D() = default;

Point2D::Point2D(
    double x_arg ,
    double y_arg 
    ):
    x { std::move(x_arg) } // NOLINT
      ,
    y { std::move(y_arg) } // NOLINT
      
    {
}

Point2D::Point2D(const Point2D&) = default;

// NOLINTNEXTLINE
Point2D::Point2D(Point2D&&) = default;

Point2D& Point2D::operator=(const Point2D&) = default;

// NOLINTNEXTLINE
Point2D& Point2D::operator=(Point2D&&) = default;

void Point2D::toCsv(std::ostream& out) const
{
    out.precision(16);
    out << std::boolalpha;
    out << x;out << ",";
    out << y;
    
}

std::stringstream Point2D::toStringstream() const
{
    std::stringstream out;
    toCsv(out);
    return out;
}

template<class Archive>
    void Point2D::serialize(Archive& ar, [[maybe_unused]] unsigned int version)
    {

        ar& CEREAL_NVP(x);
        ar& CEREAL_NVP(y);
    }


template void Point2D::serialize(cereal::BinaryInputArchive& ar, unsigned int version);
template void Point2D::serialize(cereal::BinaryOutputArchive& ar, unsigned int version);
template void Point2D::serialize(cereal::IntrospectionArchive& ar, unsigned int version);
template void Point2D::serialize(cereal::JSONInputArchive& ar, unsigned int version);
template void Point2D::serialize(cereal::JSONOutputArchive& ar, unsigned int version);
template void Point2D::serialize(cereal::PortableBinaryInputArchive& ar, unsigned int version);
template void Point2D::serialize(cereal::PortableBinaryOutputArchive& ar, unsigned int version);

std::ostream& operator<< (std::ostream& os, const Point2D& point2d)
{
    point2d.toCsv(os);
    return os;
}

bool operator== (const Point2D &lhs, const Point2D &rhs)
{
    return (  lhs.x== rhs.x &&
     lhs.y== rhs.y 
    );
}

std::size_t Point2D::hash() const noexcept
{
    std::size_t result = 17;
    result = result * 31 + std::hash<double>()(x);
    result = result * 31 + std::hash<double>()(y);
    return result;
}

 } // namespace av
 } // namespace msg


CEREAL_BIND_TO_ARCHIVES(av::msg::Point2D)


