#include <interfaces/Point3D/Point3D.h>

#include <iterator>
#include <type_traits>

#include <cereal/cereal.hpp>
#include <cereal/archives/json.hpp>
#include <cereal/archives/binary.hpp>

#include <cereal/types/map.hpp>
#include <cereal/types/vector.hpp>
#include <cereal/types/array.hpp>
#include <cereal/types/string.hpp>
#include <cereal/types/complex.hpp>

#include <ODatum/IntrospectionArchive.h>
#include <ODatum/Hash.h>


 namespace av {
 namespace horizon {


Point3D::Point3D() = default;

Point3D::Point3D(
    double x_arg ,
    double y_arg ,
    double z_arg 
    ):
    x { std::move(x_arg) } // NOLINT
      ,
    y { std::move(y_arg) } // NOLINT
      ,
    z { std::move(z_arg) } // NOLINT
      
    {
}

Point3D::Point3D(const Point3D&) = default;

// NOLINTNEXTLINE
Point3D::Point3D(Point3D&&) = default;

Point3D& Point3D::operator=(const Point3D&) = default;

// NOLINTNEXTLINE
Point3D& Point3D::operator=(Point3D&&) = default;

void Point3D::toCsv(std::ostream& out) const
{
    out.precision(16);
    out << std::boolalpha;
    out << x;out << ",";
    out << y;out << ",";
    out << z;
    
}

std::stringstream Point3D::toStringstream() const
{
    std::stringstream out;
    toCsv(out);
    return out;
}

template<class Archive>
    void Point3D::serialize(Archive& ar, [[maybe_unused]] unsigned int version)
    {

        ar& CEREAL_NVP(x);
        ar& CEREAL_NVP(y);
        ar& CEREAL_NVP(z);
    }


template void Point3D::serialize(cereal::BinaryInputArchive& ar, unsigned int version);
template void Point3D::serialize(cereal::BinaryOutputArchive& ar, unsigned int version);
template void Point3D::serialize(cereal::IntrospectionArchive& ar, unsigned int version);
template void Point3D::serialize(cereal::JSONInputArchive& ar, unsigned int version);
template void Point3D::serialize(cereal::JSONOutputArchive& ar, unsigned int version);
template void Point3D::serialize(cereal::PortableBinaryInputArchive& ar, unsigned int version);
template void Point3D::serialize(cereal::PortableBinaryOutputArchive& ar, unsigned int version);

std::ostream& operator<< (std::ostream& os, const Point3D& point3d)
{
    point3d.toCsv(os);
    return os;
}

bool operator== (const Point3D &lhs, const Point3D &rhs)
{
    return (  lhs.x== rhs.x &&
     lhs.y== rhs.y &&
     lhs.z== rhs.z 
    );
}

std::size_t Point3D::hash() const noexcept
{
    std::size_t result = 17;
    result = result * 31 + std::hash<double>()(x);
    result = result * 31 + std::hash<double>()(y);
    result = result * 31 + std::hash<double>()(z);
    return result;
}

 } // namespace av
 } // namespace horizon


CEREAL_BIND_TO_ARCHIVES(av::horizon::Point3D)


