#include <interfaces/Pose/Pose.h>

#include <iterator>
#include <type_traits>

#include <cereal/cereal.hpp>
#include <cereal/archives/json.hpp>
#include <cereal/archives/binary.hpp>

#include <cereal/types/map.hpp>
#include <cereal/types/vector.hpp>
#include <cereal/types/array.hpp>
#include <cereal/types/string.hpp>
#include <cereal/types/complex.hpp>

#include <ODatum/IntrospectionArchive.h>
#include <ODatum/Hash.h>





 namespace av {
 namespace geometry {


Pose::Pose() = default;

Pose::Pose(
    Vector3d position_arg ,
    Quaterniond rotation_arg 
    ):
    position { std::move(position_arg) } // NOLINT
      ,rotation { std::move(rotation_arg) } // NOLINT
      {
}

Pose::Pose(const Pose&) = default;

// NOLINTNEXTLINE
Pose::Pose(Pose&&) = default;

Pose& Pose::operator=(const Pose&) = default;

// NOLINTNEXTLINE
Pose& Pose::operator=(Pose&&) = default;

void Pose::toCsv(std::ostream& out) const
{
    out.precision(16);
    out << std::boolalpha;
    out << position;out << ",";
    out << rotation;
    
}

std::stringstream Pose::toStringstream() const
{
    std::stringstream out;
    toCsv(out);
    return out;
}

template<class Archive>
    void Pose::serialize(Archive& ar, [[maybe_unused]] unsigned int version)
    {

        ar& CEREAL_NVP(position);
        ar& CEREAL_NVP(rotation);
    }


template void Pose::serialize(cereal::BinaryInputArchive& ar, unsigned int version);
template void Pose::serialize(cereal::BinaryOutputArchive& ar, unsigned int version);
template void Pose::serialize(cereal::IntrospectionArchive& ar, unsigned int version);
template void Pose::serialize(cereal::JSONInputArchive& ar, unsigned int version);
template void Pose::serialize(cereal::JSONOutputArchive& ar, unsigned int version);
template void Pose::serialize(cereal::PortableBinaryInputArchive& ar, unsigned int version);
template void Pose::serialize(cereal::PortableBinaryOutputArchive& ar, unsigned int version);

std::ostream& operator<< (std::ostream& os, const Pose& pose)
{
    pose.toCsv(os);
    return os;
}

bool operator== (const Pose &lhs, const Pose &rhs)
{
    return (  lhs.position== rhs.position &&
     lhs.rotation== rhs.rotation 
    );
}

std::size_t Pose::hash() const noexcept
{
    std::size_t result = 17;
    result = result * 31 + std::hash<Vector3d>()(position);
    result = result * 31 + std::hash<Quaterniond>()(rotation);
    return result;
}

 } // namespace av
 } // namespace geometry


CEREAL_BIND_TO_ARCHIVES(av::geometry::Pose)

