#include <interfaces/Pose2d/Pose2d.h>

#include <iterator>
#include <type_traits>

#include <cereal/cereal.hpp>
#include <cereal/archives/json.hpp>
#include <cereal/archives/binary.hpp>

#include <cereal/types/map.hpp>
#include <cereal/types/vector.hpp>
#include <cereal/types/array.hpp>
#include <cereal/types/string.hpp>
#include <cereal/types/complex.hpp>

#include <ODatum/IntrospectionArchive.h>
#include <ODatum/Hash.h>


 namespace av {
 namespace geometry {


Pose2d::Pose2d() = default;

Pose2d::Pose2d(
    av::geometry::Vector2d position_arg ,
    double heading_arg 
    ):
    position { std::move(position_arg) } // NOLINT
      ,
    heading { std::move(heading_arg) } // NOLINT
      
    {
}

Pose2d::Pose2d(const Pose2d&) = default;

// NOLINTNEXTLINE
Pose2d::Pose2d(Pose2d&&) = default;

Pose2d& Pose2d::operator=(const Pose2d&) = default;

// NOLINTNEXTLINE
Pose2d& Pose2d::operator=(Pose2d&&) = default;

void Pose2d::toCsv(std::ostream& out) const
{
    out.precision(16);
    out << std::boolalpha;
    out << position;out << ",";
    out << heading;
    
}

std::stringstream Pose2d::toStringstream() const
{
    std::stringstream out;
    toCsv(out);
    return out;
}

template<class Archive>
    void Pose2d::serialize(Archive& ar, [[maybe_unused]] unsigned int version)
    {

        ar& CEREAL_NVP(position);
        ar& CEREAL_NVP(heading);
    }


template void Pose2d::serialize(cereal::BinaryInputArchive& ar, unsigned int version);
template void Pose2d::serialize(cereal::BinaryOutputArchive& ar, unsigned int version);
template void Pose2d::serialize(cereal::IntrospectionArchive& ar, unsigned int version);
template void Pose2d::serialize(cereal::JSONInputArchive& ar, unsigned int version);
template void Pose2d::serialize(cereal::JSONOutputArchive& ar, unsigned int version);
template void Pose2d::serialize(cereal::PortableBinaryInputArchive& ar, unsigned int version);
template void Pose2d::serialize(cereal::PortableBinaryOutputArchive& ar, unsigned int version);

std::ostream& operator<< (std::ostream& os, const Pose2d& pose2d)
{
    pose2d.toCsv(os);
    return os;
}

bool operator== (const Pose2d &lhs, const Pose2d &rhs)
{
    return (  lhs.position== rhs.position &&
     lhs.heading== rhs.heading 
    );
}

std::size_t Pose2d::hash() const noexcept
{
    std::size_t result = 17;
    result = result * 31 + std::hash<av::geometry::Vector2d>()(position);
    result = result * 31 + std::hash<double>()(heading);
    return result;
}

 } // namespace av
 } // namespace geometry


CEREAL_BIND_TO_ARCHIVES(av::geometry::Pose2d)


