#include <interfaces/ProcessControlCommand/ProcessControlCommand.h>

#include <iterator>
#include <type_traits>

#include <cereal/cereal.hpp>
#include <cereal/archives/json.hpp>
#include <cereal/archives/binary.hpp>

#include <cereal/types/map.hpp>
#include <cereal/types/vector.hpp>
#include <cereal/types/array.hpp>
#include <cereal/types/string.hpp>
#include <cereal/types/complex.hpp>

#include <ODatum/IntrospectionArchive.h>
#include <ODatum/Hash.h>





 namespace av {
 namespace msg {


ProcessControlCommand::ProcessControlCommand() = default;

ProcessControlCommand::ProcessControlCommand(
    MessageId messageType__arg ,
    std::string sender__arg ,
    std::string message__arg ,
    std::string recipient__arg 
    ):
    messageType_ { std::move(messageType__arg) } // NOLINT
      ,sender_ { std::move(sender__arg) } // NOLINT
      ,message_ { std::move(message__arg) } // NOLINT
      ,recipient_ { std::move(recipient__arg) } // NOLINT
      {
}

ProcessControlCommand::ProcessControlCommand(const ProcessControlCommand&) = default;

// NOLINTNEXTLINE
ProcessControlCommand::ProcessControlCommand(ProcessControlCommand&&) = default;

ProcessControlCommand& ProcessControlCommand::operator=(const ProcessControlCommand&) = default;

// NOLINTNEXTLINE
ProcessControlCommand& ProcessControlCommand::operator=(ProcessControlCommand&&) = default;

void ProcessControlCommand::toCsv(std::ostream& out) const
{
    out.precision(16);
    out << std::boolalpha;
    out << static_cast<uint32_t>(messageType_);out << ",";
    out << "\"";
        out << sender_;
        out << "\"";out << ",";
    out << "\"";
        out << message_;
        out << "\"";out << ",";
    out << "\"";
        out << recipient_;
        out << "\"";
    
}

std::stringstream ProcessControlCommand::toStringstream() const
{
    std::stringstream out;
    toCsv(out);
    return out;
}

template<class Archive>
    void ProcessControlCommand::serialize(Archive& ar, [[maybe_unused]] unsigned int version)
    {

        ar& CEREAL_NVP(messageType_);
        ar& CEREAL_NVP(sender_);
        ar& CEREAL_NVP(message_);
        ar& CEREAL_NVP(recipient_);
    }


template void ProcessControlCommand::serialize(cereal::BinaryInputArchive& ar, unsigned int version);
template void ProcessControlCommand::serialize(cereal::BinaryOutputArchive& ar, unsigned int version);
template void ProcessControlCommand::serialize(cereal::IntrospectionArchive& ar, unsigned int version);
template void ProcessControlCommand::serialize(cereal::JSONInputArchive& ar, unsigned int version);
template void ProcessControlCommand::serialize(cereal::JSONOutputArchive& ar, unsigned int version);
template void ProcessControlCommand::serialize(cereal::PortableBinaryInputArchive& ar, unsigned int version);
template void ProcessControlCommand::serialize(cereal::PortableBinaryOutputArchive& ar, unsigned int version);

std::ostream& operator<< (std::ostream& os, const ProcessControlCommand& processcontrolcommand)
{
    processcontrolcommand.toCsv(os);
    return os;
}

bool operator== (const ProcessControlCommand &lhs, const ProcessControlCommand &rhs)
{
    return (  lhs.messageType_== rhs.messageType_ &&
     lhs.sender_== rhs.sender_ &&
     lhs.message_== rhs.message_ &&
     lhs.recipient_== rhs.recipient_ 
    );
}

std::size_t ProcessControlCommand::hash() const noexcept
{
    std::size_t result = 17;
    result = result * 31 + std::hash<MessageId>()(messageType_);
    result = result * 31 + std::hash<std::string>()(sender_);
    result = result * 31 + std::hash<std::string>()(message_);
    result = result * 31 + std::hash<std::string>()(recipient_);
    return result;
}

 } // namespace av
 } // namespace msg


CEREAL_BIND_TO_ARCHIVES(av::msg::ProcessControlCommand)

