#include <interfaces/ProcessHealthMessage/ProcessHealthMessage.h>

#include <iterator>
#include <type_traits>

#include <cereal/cereal.hpp>
#include <cereal/archives/json.hpp>
#include <cereal/archives/binary.hpp>

#include <cereal/types/map.hpp>
#include <cereal/types/vector.hpp>
#include <cereal/types/array.hpp>
#include <cereal/types/string.hpp>
#include <cereal/types/complex.hpp>

#include <ODatum/IntrospectionArchive.h>
#include <ODatum/Hash.h>





 namespace av {
 namespace msg {


ProcessHealthMessage::ProcessHealthMessage() = default;

ProcessHealthMessage::ProcessHealthMessage(
    std::string source_arg ,
    std::string status_arg ,
    double cycleRate_hz_arg ,
    double intendedCycleRate_hz_arg ,
    TimePoint timeStamp_arg ,
    std::string host_arg ,
    int32_t processID_arg 
    ):
    source { std::move(source_arg) } // NOLINT
      ,status { std::move(status_arg) } // NOLINT
      ,cycleRate_hz { std::move(cycleRate_hz_arg) } // NOLINT
      ,intendedCycleRate_hz { std::move(intendedCycleRate_hz_arg) } // NOLINT
      ,timeStamp { std::move(timeStamp_arg) } // NOLINT
      ,host { std::move(host_arg) } // NOLINT
      ,processID { std::move(processID_arg) } // NOLINT
      {
}

ProcessHealthMessage::ProcessHealthMessage(const ProcessHealthMessage&) = default;

// NOLINTNEXTLINE
ProcessHealthMessage::ProcessHealthMessage(ProcessHealthMessage&&) = default;

ProcessHealthMessage& ProcessHealthMessage::operator=(const ProcessHealthMessage&) = default;

// NOLINTNEXTLINE
ProcessHealthMessage& ProcessHealthMessage::operator=(ProcessHealthMessage&&) = default;

void ProcessHealthMessage::toCsv(std::ostream& out) const
{
    out.precision(16);
    out << std::boolalpha;
    out << "\"";
        out << source;
        out << "\"";out << ",";
    out << "\"";
        out << status;
        out << "\"";out << ",";
    out << cycleRate_hz;out << ",";
    out << intendedCycleRate_hz;out << ",";
    out << timeStamp;out << ",";
    out << "\"";
        out << host;
        out << "\"";out << ",";
    out << processID;
    
}

std::stringstream ProcessHealthMessage::toStringstream() const
{
    std::stringstream out;
    toCsv(out);
    return out;
}

template<class Archive>
    void ProcessHealthMessage::serialize(Archive& ar, unsigned int version)
{
    ar& CEREAL_NVP(source);
    ar& CEREAL_NVP(status);
    ar& CEREAL_NVP(cycleRate_hz);
    ar& CEREAL_NVP(intendedCycleRate_hz);
    
    if (version < 2)
    {
        otimePoint otp;
        ar& cereal::make_nvp("timeStamp", otp);
        timeStamp = av::TimePoint(std::chrono::seconds(otp.tv_sec) + std::chrono::nanoseconds(otp.tv_nsec));
    }
    else
    {
        ar& CEREAL_NVP(timeStamp);
    }

    ar& CEREAL_NVP(host);
    ar& CEREAL_NVP(processID);

    if (version < 2)
    {
        // booldata is not used. Deprecated in version 2. Just discard the data, it's empty anyway.
        std::map<std::string, bool> oldBoolData;
        ar& cereal::make_nvp("boolData", oldBoolData);
    }
}

}} // to close av::msg namespace temporary

struct ProcessHealthMessage : public av::msg::ProcessHealthMessage {};
CEREAL_CLASS_VERSION(ProcessHealthMessage, 2)
CEREAL_REGISTER_TYPE(ProcessHealthMessage)

namespace av {
namespace msg {



template void ProcessHealthMessage::serialize(cereal::BinaryInputArchive& ar, unsigned int version);
template void ProcessHealthMessage::serialize(cereal::BinaryOutputArchive& ar, unsigned int version);
template void ProcessHealthMessage::serialize(cereal::IntrospectionArchive& ar, unsigned int version);
template void ProcessHealthMessage::serialize(cereal::JSONInputArchive& ar, unsigned int version);
template void ProcessHealthMessage::serialize(cereal::JSONOutputArchive& ar, unsigned int version);
template void ProcessHealthMessage::serialize(cereal::PortableBinaryInputArchive& ar, unsigned int version);
template void ProcessHealthMessage::serialize(cereal::PortableBinaryOutputArchive& ar, unsigned int version);

std::ostream& operator<< (std::ostream& os, const ProcessHealthMessage& processhealthmessage)
{
    processhealthmessage.toCsv(os);
    return os;
}

bool operator== (const ProcessHealthMessage &lhs, const ProcessHealthMessage &rhs)
{
    return (  lhs.source== rhs.source &&
     lhs.status== rhs.status &&
     lhs.cycleRate_hz== rhs.cycleRate_hz &&
     lhs.intendedCycleRate_hz== rhs.intendedCycleRate_hz &&
     lhs.timeStamp== rhs.timeStamp &&
     lhs.host== rhs.host &&
     lhs.processID== rhs.processID 
    );
}

std::size_t ProcessHealthMessage::hash() const noexcept
{
    std::size_t result = 17;
    result = result * 31 + std::hash<std::string>()(source);
    result = result * 31 + std::hash<std::string>()(status);
    result = result * 31 + std::hash<double>()(cycleRate_hz);
    result = result * 31 + std::hash<double>()(intendedCycleRate_hz);
    result = result * 31 + 0;
    result = result * 31 + std::hash<std::string>()(host);
    result = result * 31 + std::hash<int32_t>()(processID);
    return result;
}

 } // namespace av
 } // namespace msg


CEREAL_BIND_TO_ARCHIVES(av::msg::ProcessHealthMessage)

