#include <interfaces/PublishListMessage/PublishListMessage.h>

#include <iterator>
#include <type_traits>

#include <cereal/cereal.hpp>
#include <cereal/archives/json.hpp>
#include <cereal/archives/binary.hpp>

#include <cereal/types/map.hpp>
#include <cereal/types/vector.hpp>
#include <cereal/types/array.hpp>
#include <cereal/types/string.hpp>
#include <cereal/types/complex.hpp>

#include <ODatum/IntrospectionArchive.h>
#include <ODatum/Hash.h>





 namespace av {
 namespace msg {


PublishListMessage::PublishListMessage() = default;

PublishListMessage::PublishListMessage(
    std::string host_arg ,
    std::vector<std::string> channels_arg 
    ):
    host { std::move(host_arg) } // NOLINT
      ,channels { std::move(channels_arg) } // NOLINT
      {
}

PublishListMessage::PublishListMessage(const PublishListMessage&) = default;

// NOLINTNEXTLINE
PublishListMessage::PublishListMessage(PublishListMessage&&) = default;

PublishListMessage& PublishListMessage::operator=(const PublishListMessage&) = default;

// NOLINTNEXTLINE
PublishListMessage& PublishListMessage::operator=(PublishListMessage&&) = default;

void PublishListMessage::toCsv(std::ostream& out) const
{
    out.precision(16);
    out << std::boolalpha;
    out << "\"";
        out << host;
        out << "\"";out << ",";
    // Enclose sequence in double quotations and curly brackets to keep CSV line length uniform
        
        out << "\"{{";
        
        if (!channels.empty())
        {
            std::copy(channels.begin(), channels.end() - 1, std::ostream_iterator<std::string>(out, "},{"));
            std::copy(channels.end()-1, channels.end(), std::ostream_iterator<std::string>(out, ""));
        }
        out << "}}\"";
    
}

std::stringstream PublishListMessage::toStringstream() const
{
    std::stringstream out;
    toCsv(out);
    return out;
}

template<class Archive>
    void PublishListMessage::serialize(Archive& ar, [[maybe_unused]] unsigned int version)
    {

        ar& CEREAL_NVP(host);
        ar& CEREAL_NVP(channels);
    }


template void PublishListMessage::serialize(cereal::BinaryInputArchive& ar, unsigned int version);
template void PublishListMessage::serialize(cereal::BinaryOutputArchive& ar, unsigned int version);
template void PublishListMessage::serialize(cereal::IntrospectionArchive& ar, unsigned int version);
template void PublishListMessage::serialize(cereal::JSONInputArchive& ar, unsigned int version);
template void PublishListMessage::serialize(cereal::JSONOutputArchive& ar, unsigned int version);
template void PublishListMessage::serialize(cereal::PortableBinaryInputArchive& ar, unsigned int version);
template void PublishListMessage::serialize(cereal::PortableBinaryOutputArchive& ar, unsigned int version);

std::ostream& operator<< (std::ostream& os, const PublishListMessage& publishlistmessage)
{
    publishlistmessage.toCsv(os);
    return os;
}

bool operator== (const PublishListMessage &lhs, const PublishListMessage &rhs)
{
    return (  lhs.host== rhs.host &&
     lhs.channels== rhs.channels 
    );
}

std::size_t PublishListMessage::hash() const noexcept
{
    std::size_t result = 17;
    result = result * 31 + std::hash<std::string>()(host);
    result = result * 31 + hashVector(channels);
    return result;
}

 } // namespace av
 } // namespace msg


CEREAL_BIND_TO_ARCHIVES(av::msg::PublishListMessage)

