#include <interfaces/Quaterniond/Quaterniond.h>

#include <iterator>
#include <type_traits>

#include <cereal/cereal.hpp>
#include <cereal/archives/json.hpp>
#include <cereal/archives/binary.hpp>

#include <cereal/types/map.hpp>
#include <cereal/types/vector.hpp>
#include <cereal/types/array.hpp>
#include <cereal/types/string.hpp>
#include <cereal/types/complex.hpp>

#include <ODatum/IntrospectionArchive.h>
#include <ODatum/Hash.h>





 namespace av {
 namespace geometry {


Quaterniond::Quaterniond() = default;

Quaterniond::Quaterniond(
    double x_arg ,
    double y_arg ,
    double z_arg ,
    double w_arg 
    ):
    x { std::move(x_arg) } // NOLINT
      ,y { std::move(y_arg) } // NOLINT
      ,z { std::move(z_arg) } // NOLINT
      ,w { std::move(w_arg) } // NOLINT
      {
}

Quaterniond::Quaterniond(const Quaterniond&) = default;

// NOLINTNEXTLINE
Quaterniond::Quaterniond(Quaterniond&&) = default;

Quaterniond& Quaterniond::operator=(const Quaterniond&) = default;

// NOLINTNEXTLINE
Quaterniond& Quaterniond::operator=(Quaterniond&&) = default;

void Quaterniond::toCsv(std::ostream& out) const
{
    out.precision(16);
    out << std::boolalpha;
    out << x;out << ",";
    out << y;out << ",";
    out << z;out << ",";
    out << w;
    
}

std::stringstream Quaterniond::toStringstream() const
{
    std::stringstream out;
    toCsv(out);
    return out;
}

template<class Archive>
    void Quaterniond::serialize(Archive& ar, [[maybe_unused]] unsigned int version)
    {

        ar& CEREAL_NVP(x);
        ar& CEREAL_NVP(y);
        ar& CEREAL_NVP(z);
        ar& CEREAL_NVP(w);
    }


template void Quaterniond::serialize(cereal::BinaryInputArchive& ar, unsigned int version);
template void Quaterniond::serialize(cereal::BinaryOutputArchive& ar, unsigned int version);
template void Quaterniond::serialize(cereal::IntrospectionArchive& ar, unsigned int version);
template void Quaterniond::serialize(cereal::JSONInputArchive& ar, unsigned int version);
template void Quaterniond::serialize(cereal::JSONOutputArchive& ar, unsigned int version);
template void Quaterniond::serialize(cereal::PortableBinaryInputArchive& ar, unsigned int version);
template void Quaterniond::serialize(cereal::PortableBinaryOutputArchive& ar, unsigned int version);

std::ostream& operator<< (std::ostream& os, const Quaterniond& quaterniond)
{
    quaterniond.toCsv(os);
    return os;
}

bool operator== (const Quaterniond &lhs, const Quaterniond &rhs)
{
    return (  lhs.x== rhs.x &&
     lhs.y== rhs.y &&
     lhs.z== rhs.z &&
     lhs.w== rhs.w 
    );
}

std::size_t Quaterniond::hash() const noexcept
{
    std::size_t result = 17;
    result = result * 31 + std::hash<double>()(x);
    result = result * 31 + std::hash<double>()(y);
    result = result * 31 + std::hash<double>()(z);
    result = result * 31 + std::hash<double>()(w);
    return result;
}

 } // namespace av
 } // namespace geometry


CEREAL_BIND_TO_ARCHIVES(av::geometry::Quaterniond)

