#include <interfaces/RawDataPackage/RawDataPackage.h>

#include <iterator>
#include <type_traits>

#include <cereal/cereal.hpp>
#include <cereal/archives/json.hpp>
#include <cereal/archives/binary.hpp>

#include <cereal/types/map.hpp>
#include <cereal/types/vector.hpp>
#include <cereal/types/array.hpp>
#include <cereal/types/string.hpp>
#include <cereal/types/complex.hpp>

#include <ODatum/IntrospectionArchive.h>
#include <ODatum/Hash.h>





 namespace av {
 namespace msg {


RawDataPackage::RawDataPackage() = default;

RawDataPackage::RawDataPackage(
    TimePoint timeStamp_arg ,
    RawDataPackageType type_arg ,
    std::vector<uint8_t> data_arg 
    ):
    timeStamp { std::move(timeStamp_arg) } // NOLINT
      ,type { std::move(type_arg) } // NOLINT
      ,data { std::move(data_arg) } // NOLINT
      {
}

RawDataPackage::RawDataPackage(const RawDataPackage&) = default;

// NOLINTNEXTLINE
RawDataPackage::RawDataPackage(RawDataPackage&&) = default;

RawDataPackage& RawDataPackage::operator=(const RawDataPackage&) = default;

// NOLINTNEXTLINE
RawDataPackage& RawDataPackage::operator=(RawDataPackage&&) = default;

void RawDataPackage::toCsv(std::ostream& out) const
{
    out.precision(16);
    out << std::boolalpha;
    out << timeStamp;out << ",";
    out << static_cast<uint32_t>(type);out << ",";
    // Enclose sequence in double quotations and curly brackets to keep CSV line length uniform
        
        out << "\"{{";
        
        if (!data.empty())
        {
            std::copy(data.begin(), data.end() - 1, std::ostream_iterator<uint8_t>(out, "},{"));
            std::copy(data.end()-1, data.end(), std::ostream_iterator<uint8_t>(out, ""));
        }
        out << "}}\"";
    
}

std::stringstream RawDataPackage::toStringstream() const
{
    std::stringstream out;
    toCsv(out);
    return out;
}

template<class Archive>
    void RawDataPackage::serialize(Archive& ar, unsigned int version)
{
    if (version < 2)
    {
        otimePoint otp;
        ar& cereal::make_nvp("timeStamp", otp);
        timeStamp = av::TimePoint(std::chrono::seconds(otp.tv_sec) + std::chrono::nanoseconds(otp.tv_nsec));
    }
    else
    {
        ar& CEREAL_NVP(timeStamp);
    }

    ar& CEREAL_NVP(type);
    ar& CEREAL_NVP(data);
}

}} // to close av::msg namespace temporary

struct RawDataPackage : public av::msg::RawDataPackage {};
CEREAL_CLASS_VERSION(RawDataPackage, 2)
CEREAL_REGISTER_TYPE(RawDataPackage)

namespace av {
namespace msg {





template void RawDataPackage::serialize(cereal::BinaryInputArchive& ar, unsigned int version);
template void RawDataPackage::serialize(cereal::BinaryOutputArchive& ar, unsigned int version);
template void RawDataPackage::serialize(cereal::IntrospectionArchive& ar, unsigned int version);
template void RawDataPackage::serialize(cereal::JSONInputArchive& ar, unsigned int version);
template void RawDataPackage::serialize(cereal::JSONOutputArchive& ar, unsigned int version);
template void RawDataPackage::serialize(cereal::PortableBinaryInputArchive& ar, unsigned int version);
template void RawDataPackage::serialize(cereal::PortableBinaryOutputArchive& ar, unsigned int version);

std::ostream& operator<< (std::ostream& os, const RawDataPackage& rawdatapackage)
{
    rawdatapackage.toCsv(os);
    return os;
}

bool operator== (const RawDataPackage &lhs, const RawDataPackage &rhs)
{
    return (  lhs.timeStamp== rhs.timeStamp &&
     lhs.type== rhs.type &&
     lhs.data== rhs.data 
    );
}

std::size_t RawDataPackage::hash() const noexcept
{
    std::size_t result = 17;
    result = result * 31 + 0;
    result = result * 31 + std::hash<RawDataPackageType>()(type);
    result = result * 31 + hashVector(data);
    return result;
}

 } // namespace av
 } // namespace msg


CEREAL_BIND_TO_ARCHIVES(av::msg::RawDataPackage)

