#include <interfaces/RoutePlan/RoutePlan.h>

#include <iterator>
#include <type_traits>

#include <cereal/cereal.hpp>
#include <cereal/archives/json.hpp>
#include <cereal/archives/binary.hpp>

#include <cereal/types/map.hpp>
#include <cereal/types/vector.hpp>
#include <cereal/types/array.hpp>
#include <cereal/types/string.hpp>
#include <cereal/types/complex.hpp>

#include <ODatum/IntrospectionArchive.h>
#include <ODatum/Hash.h>


 namespace av {
 namespace msg {
 namespace plan_ctrl {


PathRepresentation::PathRepresentation() = default;

PathRepresentation::PathRepresentation(
    int64_t laneConnector_arg ,
    bool isLaneConnectorValid_arg ,
    int64_t laneFrom_arg ,
    bool isLaneFromValid_arg 
    ):
    laneConnector { std::move(laneConnector_arg) } // NOLINT
      ,
    isLaneConnectorValid { std::move(isLaneConnectorValid_arg) } // NOLINT
      ,
    laneFrom { std::move(laneFrom_arg) } // NOLINT
      ,
    isLaneFromValid { std::move(isLaneFromValid_arg) } // NOLINT
      
    {
}

PathRepresentation::PathRepresentation(const PathRepresentation&) = default;

// NOLINTNEXTLINE
PathRepresentation::PathRepresentation(PathRepresentation&&) = default;

PathRepresentation& PathRepresentation::operator=(const PathRepresentation&) = default;

// NOLINTNEXTLINE
PathRepresentation& PathRepresentation::operator=(PathRepresentation&&) = default;

void PathRepresentation::toCsv(std::ostream& out) const
{
    out.precision(16);
    out << std::boolalpha;
    out << laneConnector;out << ",";
    out << isLaneConnectorValid;out << ",";
    out << laneFrom;out << ",";
    out << isLaneFromValid;
    
}

std::stringstream PathRepresentation::toStringstream() const
{
    std::stringstream out;
    toCsv(out);
    return out;
}

template<class Archive>
    void PathRepresentation::serialize(Archive& ar, [[maybe_unused]] unsigned int version)
    {

        ar& CEREAL_NVP(laneConnector);
        ar& CEREAL_NVP(isLaneConnectorValid);
        ar& CEREAL_NVP(laneFrom);
        ar& CEREAL_NVP(isLaneFromValid);
    }


template void PathRepresentation::serialize(cereal::BinaryInputArchive& ar, unsigned int version);
template void PathRepresentation::serialize(cereal::BinaryOutputArchive& ar, unsigned int version);
template void PathRepresentation::serialize(cereal::IntrospectionArchive& ar, unsigned int version);
template void PathRepresentation::serialize(cereal::JSONInputArchive& ar, unsigned int version);
template void PathRepresentation::serialize(cereal::JSONOutputArchive& ar, unsigned int version);
template void PathRepresentation::serialize(cereal::PortableBinaryInputArchive& ar, unsigned int version);
template void PathRepresentation::serialize(cereal::PortableBinaryOutputArchive& ar, unsigned int version);

std::ostream& operator<< (std::ostream& os, const PathRepresentation& pathrepresentation)
{
    pathrepresentation.toCsv(os);
    return os;
}

bool operator== (const PathRepresentation &lhs, const PathRepresentation &rhs)
{
    return (  lhs.laneConnector== rhs.laneConnector &&
     lhs.isLaneConnectorValid== rhs.isLaneConnectorValid &&
     lhs.laneFrom== rhs.laneFrom &&
     lhs.isLaneFromValid== rhs.isLaneFromValid 
    );
}

std::size_t PathRepresentation::hash() const noexcept
{
    std::size_t result = 17;
    result = result * 31 + std::hash<int64_t>()(laneConnector);
    result = result * 31 + std::hash<bool>()(isLaneConnectorValid);
    result = result * 31 + std::hash<int64_t>()(laneFrom);
    result = result * 31 + std::hash<bool>()(isLaneFromValid);
    return result;
}

 } // namespace av
 } // namespace msg
 } // namespace plan_ctrl


CEREAL_BIND_TO_ARCHIVES(av::msg::plan_ctrl::PathRepresentation)

 namespace av {
 namespace msg {
 namespace plan_ctrl {


PathWithRoute::PathWithRoute() = default;

PathWithRoute::PathWithRoute(
    Path path_arg ,
    std::vector<PathRepresentation> laneSequence_arg 
    ):
    path { std::move(path_arg) } // NOLINT
      ,
    laneSequence { std::move(laneSequence_arg) } // NOLINT
      
    {
}

PathWithRoute::PathWithRoute(const PathWithRoute&) = default;

// NOLINTNEXTLINE
PathWithRoute::PathWithRoute(PathWithRoute&&) = default;

PathWithRoute& PathWithRoute::operator=(const PathWithRoute&) = default;

// NOLINTNEXTLINE
PathWithRoute& PathWithRoute::operator=(PathWithRoute&&) = default;

void PathWithRoute::toCsv(std::ostream& out) const
{
    out.precision(16);
    out << std::boolalpha;
    out << path;out << ",";
    // Enclose sequence in double quotations and curly brackets to keep CSV line length uniform
        
        out << "\"{{";
        
        if (!laneSequence.empty())
        {
            std::copy(laneSequence.begin(), laneSequence.end() - 1, std::ostream_iterator<PathRepresentation>(out, "},{"));
            std::copy(laneSequence.end()-1, laneSequence.end(), std::ostream_iterator<PathRepresentation>(out, ""));
        }
        out << "}}\"";
    
}

std::stringstream PathWithRoute::toStringstream() const
{
    std::stringstream out;
    toCsv(out);
    return out;
}

template<class Archive>
    void PathWithRoute::serialize(Archive& ar, [[maybe_unused]] unsigned int version)
    {

        ar& CEREAL_NVP(path);
        ar& CEREAL_NVP(laneSequence);
    }


template void PathWithRoute::serialize(cereal::BinaryInputArchive& ar, unsigned int version);
template void PathWithRoute::serialize(cereal::BinaryOutputArchive& ar, unsigned int version);
template void PathWithRoute::serialize(cereal::IntrospectionArchive& ar, unsigned int version);
template void PathWithRoute::serialize(cereal::JSONInputArchive& ar, unsigned int version);
template void PathWithRoute::serialize(cereal::JSONOutputArchive& ar, unsigned int version);
template void PathWithRoute::serialize(cereal::PortableBinaryInputArchive& ar, unsigned int version);
template void PathWithRoute::serialize(cereal::PortableBinaryOutputArchive& ar, unsigned int version);

std::ostream& operator<< (std::ostream& os, const PathWithRoute& pathwithroute)
{
    pathwithroute.toCsv(os);
    return os;
}

bool operator== (const PathWithRoute &lhs, const PathWithRoute &rhs)
{
    return (  lhs.path== rhs.path &&
     lhs.laneSequence== rhs.laneSequence 
    );
}

std::size_t PathWithRoute::hash() const noexcept
{
    std::size_t result = 17;
    result = result * 31 + std::hash<Path>()(path);
    result = result * 31 + hashVector(laneSequence);
    return result;
}

 } // namespace av
 } // namespace msg
 } // namespace plan_ctrl


CEREAL_BIND_TO_ARCHIVES(av::msg::plan_ctrl::PathWithRoute)

 namespace av {
 namespace msg {
 namespace plan_ctrl {


OptionalPaths::OptionalPaths() = default;

OptionalPaths::OptionalPaths(
    PathWithRoute path_arg ,
    double station_interval_in_anchor_max_arg ,
    double station_interval_in_anchor_min_arg ,
    bool isDesiredLane_arg ,
    bool isOppositeDirection_arg ,
    bool isLeft_arg 
    ):
    path { std::move(path_arg) } // NOLINT
      ,
    station_interval_in_anchor_max { std::move(station_interval_in_anchor_max_arg) } // NOLINT
      ,
    station_interval_in_anchor_min { std::move(station_interval_in_anchor_min_arg) } // NOLINT
      ,
    isDesiredLane { std::move(isDesiredLane_arg) } // NOLINT
      ,
    isOppositeDirection { std::move(isOppositeDirection_arg) } // NOLINT
      ,
    isLeft { std::move(isLeft_arg) } // NOLINT
      
    {
}

OptionalPaths::OptionalPaths(const OptionalPaths&) = default;

// NOLINTNEXTLINE
OptionalPaths::OptionalPaths(OptionalPaths&&) = default;

OptionalPaths& OptionalPaths::operator=(const OptionalPaths&) = default;

// NOLINTNEXTLINE
OptionalPaths& OptionalPaths::operator=(OptionalPaths&&) = default;

void OptionalPaths::toCsv(std::ostream& out) const
{
    out.precision(16);
    out << std::boolalpha;
    out << path;out << ",";
    out << station_interval_in_anchor_max;out << ",";
    out << station_interval_in_anchor_min;out << ",";
    out << isDesiredLane;out << ",";
    out << isOppositeDirection;out << ",";
    out << isLeft;
    
}

std::stringstream OptionalPaths::toStringstream() const
{
    std::stringstream out;
    toCsv(out);
    return out;
}

template<class Archive>
    void OptionalPaths::serialize(Archive& ar, [[maybe_unused]] unsigned int version)
    {

        ar& CEREAL_NVP(path);
        ar& CEREAL_NVP(station_interval_in_anchor_max);
        ar& CEREAL_NVP(station_interval_in_anchor_min);
        ar& CEREAL_NVP(isDesiredLane);
        ar& CEREAL_NVP(isOppositeDirection);
        ar& CEREAL_NVP(isLeft);
    }


template void OptionalPaths::serialize(cereal::BinaryInputArchive& ar, unsigned int version);
template void OptionalPaths::serialize(cereal::BinaryOutputArchive& ar, unsigned int version);
template void OptionalPaths::serialize(cereal::IntrospectionArchive& ar, unsigned int version);
template void OptionalPaths::serialize(cereal::JSONInputArchive& ar, unsigned int version);
template void OptionalPaths::serialize(cereal::JSONOutputArchive& ar, unsigned int version);
template void OptionalPaths::serialize(cereal::PortableBinaryInputArchive& ar, unsigned int version);
template void OptionalPaths::serialize(cereal::PortableBinaryOutputArchive& ar, unsigned int version);

std::ostream& operator<< (std::ostream& os, const OptionalPaths& optionalpaths)
{
    optionalpaths.toCsv(os);
    return os;
}

bool operator== (const OptionalPaths &lhs, const OptionalPaths &rhs)
{
    return (  lhs.path== rhs.path &&
     lhs.station_interval_in_anchor_max== rhs.station_interval_in_anchor_max &&
     lhs.station_interval_in_anchor_min== rhs.station_interval_in_anchor_min &&
     lhs.isDesiredLane== rhs.isDesiredLane &&
     lhs.isOppositeDirection== rhs.isOppositeDirection &&
     lhs.isLeft== rhs.isLeft 
    );
}

std::size_t OptionalPaths::hash() const noexcept
{
    std::size_t result = 17;
    result = result * 31 + std::hash<PathWithRoute>()(path);
    result = result * 31 + std::hash<double>()(station_interval_in_anchor_max);
    result = result * 31 + std::hash<double>()(station_interval_in_anchor_min);
    result = result * 31 + std::hash<bool>()(isDesiredLane);
    result = result * 31 + std::hash<bool>()(isOppositeDirection);
    result = result * 31 + std::hash<bool>()(isLeft);
    return result;
}

 } // namespace av
 } // namespace msg
 } // namespace plan_ctrl


CEREAL_BIND_TO_ARCHIVES(av::msg::plan_ctrl::OptionalPaths)




 namespace av {
 namespace msg {
 namespace plan_ctrl {


RoutePlan::RoutePlan() = default;

RoutePlan::RoutePlan(
    int64_t timestamp_arg ,
    PathWithRoute anchor_path_arg ,
    std::vector<OptionalPaths> adjacent_paths_arg ,
    std::vector<int64_t> roadBlocksRoute_arg ,
    std::vector<int64_t> laneRoute_arg 
    ):
    timestamp { std::move(timestamp_arg) } // NOLINT
      ,anchor_path { std::move(anchor_path_arg) } // NOLINT
      ,adjacent_paths { std::move(adjacent_paths_arg) } // NOLINT
      ,roadBlocksRoute { std::move(roadBlocksRoute_arg) } // NOLINT
      ,laneRoute { std::move(laneRoute_arg) } // NOLINT
      {
}

RoutePlan::RoutePlan(const RoutePlan&) = default;

// NOLINTNEXTLINE
RoutePlan::RoutePlan(RoutePlan&&) = default;

RoutePlan& RoutePlan::operator=(const RoutePlan&) = default;

// NOLINTNEXTLINE
RoutePlan& RoutePlan::operator=(RoutePlan&&) = default;

void RoutePlan::toCsv(std::ostream& out) const
{
    out.precision(16);
    out << std::boolalpha;
    out << timestamp;out << ",";
    out << anchor_path;out << ",";
    // Enclose sequence in double quotations and curly brackets to keep CSV line length uniform
        
        out << "\"{{";
        
        if (!adjacent_paths.empty())
        {
            std::copy(adjacent_paths.begin(), adjacent_paths.end() - 1, std::ostream_iterator<OptionalPaths>(out, "},{"));
            std::copy(adjacent_paths.end()-1, adjacent_paths.end(), std::ostream_iterator<OptionalPaths>(out, ""));
        }
        out << "}}\"";out << ",";
    // Enclose sequence in double quotations and curly brackets to keep CSV line length uniform
        
        out << "\"{{";
        
        if (!roadBlocksRoute.empty())
        {
            std::copy(roadBlocksRoute.begin(), roadBlocksRoute.end() - 1, std::ostream_iterator<int64_t>(out, "},{"));
            std::copy(roadBlocksRoute.end()-1, roadBlocksRoute.end(), std::ostream_iterator<int64_t>(out, ""));
        }
        out << "}}\"";out << ",";
    // Enclose sequence in double quotations and curly brackets to keep CSV line length uniform
        
        out << "\"{{";
        
        if (!laneRoute.empty())
        {
            std::copy(laneRoute.begin(), laneRoute.end() - 1, std::ostream_iterator<int64_t>(out, "},{"));
            std::copy(laneRoute.end()-1, laneRoute.end(), std::ostream_iterator<int64_t>(out, ""));
        }
        out << "}}\"";
    
}

std::stringstream RoutePlan::toStringstream() const
{
    std::stringstream out;
    toCsv(out);
    return out;
}

template<class Archive>
    void RoutePlan::serialize(Archive& ar, [[maybe_unused]] unsigned int version)
    {

        ar& CEREAL_NVP(timestamp);
        ar& CEREAL_NVP(anchor_path);
        ar& CEREAL_NVP(adjacent_paths);
        ar& CEREAL_NVP(roadBlocksRoute);
        ar& CEREAL_NVP(laneRoute);
    }


template void RoutePlan::serialize(cereal::BinaryInputArchive& ar, unsigned int version);
template void RoutePlan::serialize(cereal::BinaryOutputArchive& ar, unsigned int version);
template void RoutePlan::serialize(cereal::IntrospectionArchive& ar, unsigned int version);
template void RoutePlan::serialize(cereal::JSONInputArchive& ar, unsigned int version);
template void RoutePlan::serialize(cereal::JSONOutputArchive& ar, unsigned int version);
template void RoutePlan::serialize(cereal::PortableBinaryInputArchive& ar, unsigned int version);
template void RoutePlan::serialize(cereal::PortableBinaryOutputArchive& ar, unsigned int version);

std::ostream& operator<< (std::ostream& os, const RoutePlan& routeplan)
{
    routeplan.toCsv(os);
    return os;
}

bool operator== (const RoutePlan &lhs, const RoutePlan &rhs)
{
    return (  lhs.timestamp== rhs.timestamp &&
     lhs.anchor_path== rhs.anchor_path &&
     lhs.adjacent_paths== rhs.adjacent_paths &&
     lhs.roadBlocksRoute== rhs.roadBlocksRoute &&
     lhs.laneRoute== rhs.laneRoute 
    );
}

std::size_t RoutePlan::hash() const noexcept
{
    std::size_t result = 17;
    result = result * 31 + std::hash<int64_t>()(timestamp);
    result = result * 31 + std::hash<PathWithRoute>()(anchor_path);
    result = result * 31 + hashVector(adjacent_paths);
    result = result * 31 + hashVector(roadBlocksRoute);
    result = result * 31 + hashVector(laneRoute);
    return result;
}

 } // namespace av
 } // namespace msg
 } // namespace plan_ctrl


CEREAL_BIND_TO_ARCHIVES(av::msg::plan_ctrl::RoutePlan)

