#include <interfaces/RoutePlanWithStaticConstraints/RoutePlanWithStaticConstraints.h>

#include <iterator>
#include <type_traits>

#include <cereal/cereal.hpp>
#include <cereal/archives/json.hpp>
#include <cereal/archives/binary.hpp>

#include <cereal/types/map.hpp>
#include <cereal/types/vector.hpp>
#include <cereal/types/array.hpp>
#include <cereal/types/string.hpp>
#include <cereal/types/complex.hpp>

#include <ODatum/IntrospectionArchive.h>
#include <ODatum/Hash.h>





 namespace av {
 namespace msg {
 namespace plan_ctrl {


RoutePlanWithStaticConstraints::RoutePlanWithStaticConstraints() = default;

RoutePlanWithStaticConstraints::RoutePlanWithStaticConstraints(
    PlannerHeader header_arg ,
    RoutePlan route_plan_arg ,
    HardAndSoftLinearConstraints static_spatial_constraints_arg ,
    std::vector<HardAndSoftLinearConstraints> static_spatial_constraints_left_arg ,
    std::vector<HardAndSoftLinearConstraints> static_spatial_constraints_right_arg ,
    HardAndSoftQuadraticConstraints static_station_constraints_arg ,
    HardAndSoftLinearConstraints static_speed_squared_constraints_arg 
    ):
    header { std::move(header_arg) } // NOLINT
      ,route_plan { std::move(route_plan_arg) } // NOLINT
      ,static_spatial_constraints { std::move(static_spatial_constraints_arg) } // NOLINT
      ,static_spatial_constraints_left { std::move(static_spatial_constraints_left_arg) } // NOLINT
      ,static_spatial_constraints_right { std::move(static_spatial_constraints_right_arg) } // NOLINT
      ,static_station_constraints { std::move(static_station_constraints_arg) } // NOLINT
      ,static_speed_squared_constraints { std::move(static_speed_squared_constraints_arg) } // NOLINT
      {
}

RoutePlanWithStaticConstraints::RoutePlanWithStaticConstraints(const RoutePlanWithStaticConstraints&) = default;

// NOLINTNEXTLINE
RoutePlanWithStaticConstraints::RoutePlanWithStaticConstraints(RoutePlanWithStaticConstraints&&) = default;

RoutePlanWithStaticConstraints& RoutePlanWithStaticConstraints::operator=(const RoutePlanWithStaticConstraints&) = default;

// NOLINTNEXTLINE
RoutePlanWithStaticConstraints& RoutePlanWithStaticConstraints::operator=(RoutePlanWithStaticConstraints&&) = default;

void RoutePlanWithStaticConstraints::toCsv(std::ostream& out) const
{
    out.precision(16);
    out << std::boolalpha;
    out << header;out << ",";
    out << route_plan;out << ",";
    out << static_spatial_constraints;out << ",";
    // Enclose sequence in double quotations and curly brackets to keep CSV line length uniform
        
        out << "\"{{";
        
        if (!static_spatial_constraints_left.empty())
        {
            std::copy(static_spatial_constraints_left.begin(), static_spatial_constraints_left.end() - 1, std::ostream_iterator<HardAndSoftLinearConstraints>(out, "},{"));
            std::copy(static_spatial_constraints_left.end()-1, static_spatial_constraints_left.end(), std::ostream_iterator<HardAndSoftLinearConstraints>(out, ""));
        }
        out << "}}\"";out << ",";
    // Enclose sequence in double quotations and curly brackets to keep CSV line length uniform
        
        out << "\"{{";
        
        if (!static_spatial_constraints_right.empty())
        {
            std::copy(static_spatial_constraints_right.begin(), static_spatial_constraints_right.end() - 1, std::ostream_iterator<HardAndSoftLinearConstraints>(out, "},{"));
            std::copy(static_spatial_constraints_right.end()-1, static_spatial_constraints_right.end(), std::ostream_iterator<HardAndSoftLinearConstraints>(out, ""));
        }
        out << "}}\"";out << ",";
    out << static_station_constraints;out << ",";
    out << static_speed_squared_constraints;
    
}

std::stringstream RoutePlanWithStaticConstraints::toStringstream() const
{
    std::stringstream out;
    toCsv(out);
    return out;
}

template<class Archive>
    void RoutePlanWithStaticConstraints::serialize(Archive& ar, unsigned int version)
{
    if(version == 1)
    {
        ar& CEREAL_NVP(header);
        ar& CEREAL_NVP(route_plan);
        ar& CEREAL_NVP(static_spatial_constraints);
        ar& CEREAL_NVP(static_station_constraints);
        ar& CEREAL_NVP(static_speed_squared_constraints);
    }
    if(version >= 2)
    {
        ar& CEREAL_NVP(header);
        ar& CEREAL_NVP(route_plan);
        ar& CEREAL_NVP(static_spatial_constraints);
        ar& CEREAL_NVP(static_spatial_constraints_left);
        ar& CEREAL_NVP(static_spatial_constraints_right);
        ar& CEREAL_NVP(static_station_constraints);
        ar& CEREAL_NVP(static_speed_squared_constraints);
    }
}



template void RoutePlanWithStaticConstraints::serialize(cereal::BinaryInputArchive& ar, unsigned int version);
template void RoutePlanWithStaticConstraints::serialize(cereal::BinaryOutputArchive& ar, unsigned int version);
template void RoutePlanWithStaticConstraints::serialize(cereal::IntrospectionArchive& ar, unsigned int version);
template void RoutePlanWithStaticConstraints::serialize(cereal::JSONInputArchive& ar, unsigned int version);
template void RoutePlanWithStaticConstraints::serialize(cereal::JSONOutputArchive& ar, unsigned int version);
template void RoutePlanWithStaticConstraints::serialize(cereal::PortableBinaryInputArchive& ar, unsigned int version);
template void RoutePlanWithStaticConstraints::serialize(cereal::PortableBinaryOutputArchive& ar, unsigned int version);

std::ostream& operator<< (std::ostream& os, const RoutePlanWithStaticConstraints& routeplanwithstaticconstraints)
{
    routeplanwithstaticconstraints.toCsv(os);
    return os;
}

bool operator== (const RoutePlanWithStaticConstraints &lhs, const RoutePlanWithStaticConstraints &rhs)
{
    return (  lhs.header== rhs.header &&
     lhs.route_plan== rhs.route_plan &&
     lhs.static_spatial_constraints== rhs.static_spatial_constraints &&
     lhs.static_spatial_constraints_left== rhs.static_spatial_constraints_left &&
     lhs.static_spatial_constraints_right== rhs.static_spatial_constraints_right &&
     lhs.static_station_constraints== rhs.static_station_constraints &&
     lhs.static_speed_squared_constraints== rhs.static_speed_squared_constraints 
    );
}

std::size_t RoutePlanWithStaticConstraints::hash() const noexcept
{
    std::size_t result = 17;
    result = result * 31 + std::hash<PlannerHeader>()(header);
    result = result * 31 + std::hash<RoutePlan>()(route_plan);
    result = result * 31 + std::hash<HardAndSoftLinearConstraints>()(static_spatial_constraints);
    result = result * 31 + hashVector(static_spatial_constraints_left);
    result = result * 31 + hashVector(static_spatial_constraints_right);
    result = result * 31 + std::hash<HardAndSoftQuadraticConstraints>()(static_station_constraints);
    result = result * 31 + std::hash<HardAndSoftLinearConstraints>()(static_speed_squared_constraints);
    return result;
}

 } // namespace av
 } // namespace msg
 } // namespace plan_ctrl


CEREAL_BIND_TO_ARCHIVES(av::msg::plan_ctrl::RoutePlanWithStaticConstraints)

