#include <interfaces/RoutingRequest/RoutingRequest.h>

#include <iterator>
#include <type_traits>

#include <cereal/cereal.hpp>
#include <cereal/archives/json.hpp>
#include <cereal/archives/binary.hpp>

#include <cereal/types/map.hpp>
#include <cereal/types/vector.hpp>
#include <cereal/types/array.hpp>
#include <cereal/types/string.hpp>
#include <cereal/types/complex.hpp>

#include <ODatum/IntrospectionArchive.h>
#include <ODatum/Hash.h>


 namespace av {
 namespace msg {


RoutingSearchParameters::RoutingSearchParameters() = default;

RoutingSearchParameters::RoutingSearchParameters(
    double radius_arg ,
    uint32_t limit_arg 
    ):
    radius { std::move(radius_arg) } // NOLINT
      ,
    limit { std::move(limit_arg) } // NOLINT
      
    {
}

RoutingSearchParameters::RoutingSearchParameters(const RoutingSearchParameters&) = default;

// NOLINTNEXTLINE
RoutingSearchParameters::RoutingSearchParameters(RoutingSearchParameters&&) = default;

RoutingSearchParameters& RoutingSearchParameters::operator=(const RoutingSearchParameters&) = default;

// NOLINTNEXTLINE
RoutingSearchParameters& RoutingSearchParameters::operator=(RoutingSearchParameters&&) = default;

void RoutingSearchParameters::toCsv(std::ostream& out) const
{
    out.precision(16);
    out << std::boolalpha;
    out << radius;out << ",";
    out << limit;
    
}

std::stringstream RoutingSearchParameters::toStringstream() const
{
    std::stringstream out;
    toCsv(out);
    return out;
}

template<class Archive>
    void RoutingSearchParameters::serialize(Archive& ar, [[maybe_unused]] unsigned int version)
    {

        ar& CEREAL_NVP(radius);
        ar& CEREAL_NVP(limit);
    }


template void RoutingSearchParameters::serialize(cereal::BinaryInputArchive& ar, unsigned int version);
template void RoutingSearchParameters::serialize(cereal::BinaryOutputArchive& ar, unsigned int version);
template void RoutingSearchParameters::serialize(cereal::IntrospectionArchive& ar, unsigned int version);
template void RoutingSearchParameters::serialize(cereal::JSONInputArchive& ar, unsigned int version);
template void RoutingSearchParameters::serialize(cereal::JSONOutputArchive& ar, unsigned int version);
template void RoutingSearchParameters::serialize(cereal::PortableBinaryInputArchive& ar, unsigned int version);
template void RoutingSearchParameters::serialize(cereal::PortableBinaryOutputArchive& ar, unsigned int version);

std::ostream& operator<< (std::ostream& os, const RoutingSearchParameters& routingsearchparameters)
{
    routingsearchparameters.toCsv(os);
    return os;
}

bool operator== (const RoutingSearchParameters &lhs, const RoutingSearchParameters &rhs)
{
    return (  lhs.radius== rhs.radius &&
     lhs.limit== rhs.limit 
    );
}

std::size_t RoutingSearchParameters::hash() const noexcept
{
    std::size_t result = 17;
    result = result * 31 + std::hash<double>()(radius);
    result = result * 31 + std::hash<uint32_t>()(limit);
    return result;
}

 } // namespace av
 } // namespace msg


CEREAL_BIND_TO_ARCHIVES(av::msg::RoutingSearchParameters)




 namespace av {
 namespace msg {


RoutingRequest::RoutingRequest() = default;

RoutingRequest::RoutingRequest(
    av::msg::TimePoint timestamp_arg ,
    msg::GUID requestId_arg ,
    int32_t utmZone_arg ,
    RoutingTargetType targetType_arg ,
    av::msg::SE2 targetPose_arg ,
    av::msg::SE2 startPose_arg ,
    std::vector<Point2D> wayPoints_arg ,
    RoutingSearchParameters searchParameters_arg 
    ):
    timestamp { std::move(timestamp_arg) } // NOLINT
      ,requestId { std::move(requestId_arg) } // NOLINT
      ,utmZone { std::move(utmZone_arg) } // NOLINT
      ,targetType { std::move(targetType_arg) } // NOLINT
      ,targetPose { std::move(targetPose_arg) } // NOLINT
      ,startPose { std::move(startPose_arg) } // NOLINT
      ,wayPoints { std::move(wayPoints_arg) } // NOLINT
      ,searchParameters { std::move(searchParameters_arg) } // NOLINT
      {
}

RoutingRequest::RoutingRequest(const RoutingRequest&) = default;

// NOLINTNEXTLINE
RoutingRequest::RoutingRequest(RoutingRequest&&) = default;

RoutingRequest& RoutingRequest::operator=(const RoutingRequest&) = default;

// NOLINTNEXTLINE
RoutingRequest& RoutingRequest::operator=(RoutingRequest&&) = default;

void RoutingRequest::toCsv(std::ostream& out) const
{
    out.precision(16);
    out << std::boolalpha;
    out << timestamp;out << ",";
    out << requestId;out << ",";
    out << utmZone;out << ",";
    out << static_cast<uint32_t>(targetType);out << ",";
    out << targetPose;out << ",";
    out << startPose;out << ",";
    // Enclose sequence in double quotations and curly brackets to keep CSV line length uniform
        
        out << "\"{{";
        
        if (!wayPoints.empty())
        {
            std::copy(wayPoints.begin(), wayPoints.end() - 1, std::ostream_iterator<Point2D>(out, "},{"));
            std::copy(wayPoints.end()-1, wayPoints.end(), std::ostream_iterator<Point2D>(out, ""));
        }
        out << "}}\"";out << ",";
    out << searchParameters;
    
}

std::stringstream RoutingRequest::toStringstream() const
{
    std::stringstream out;
    toCsv(out);
    return out;
}

template<class Archive>
    void RoutingRequest::serialize(Archive& ar, [[maybe_unused]] unsigned int version)
    {

        ar& CEREAL_NVP(timestamp);
        ar& CEREAL_NVP(requestId);
        ar& CEREAL_NVP(utmZone);
        ar& CEREAL_NVP(targetType);
        ar& CEREAL_NVP(targetPose);
        ar& CEREAL_NVP(startPose);
        ar& CEREAL_NVP(wayPoints);
        ar& CEREAL_NVP(searchParameters);
    }


template void RoutingRequest::serialize(cereal::BinaryInputArchive& ar, unsigned int version);
template void RoutingRequest::serialize(cereal::BinaryOutputArchive& ar, unsigned int version);
template void RoutingRequest::serialize(cereal::IntrospectionArchive& ar, unsigned int version);
template void RoutingRequest::serialize(cereal::JSONInputArchive& ar, unsigned int version);
template void RoutingRequest::serialize(cereal::JSONOutputArchive& ar, unsigned int version);
template void RoutingRequest::serialize(cereal::PortableBinaryInputArchive& ar, unsigned int version);
template void RoutingRequest::serialize(cereal::PortableBinaryOutputArchive& ar, unsigned int version);

std::ostream& operator<< (std::ostream& os, const RoutingRequest& routingrequest)
{
    routingrequest.toCsv(os);
    return os;
}

bool operator== (const RoutingRequest &lhs, const RoutingRequest &rhs)
{
    return (  lhs.timestamp== rhs.timestamp &&
     lhs.requestId== rhs.requestId &&
     lhs.utmZone== rhs.utmZone &&
     lhs.targetType== rhs.targetType &&
     lhs.targetPose== rhs.targetPose &&
     lhs.startPose== rhs.startPose &&
     lhs.wayPoints== rhs.wayPoints &&
     lhs.searchParameters== rhs.searchParameters 
    );
}

std::size_t RoutingRequest::hash() const noexcept
{
    std::size_t result = 17;
    result = result * 31 + std::hash<av::msg::TimePoint>()(timestamp);
    result = result * 31 + std::hash<msg::GUID>()(requestId);
    result = result * 31 + std::hash<int32_t>()(utmZone);
    result = result * 31 + std::hash<RoutingTargetType>()(targetType);
    result = result * 31 + std::hash<av::msg::SE2>()(targetPose);
    result = result * 31 + std::hash<av::msg::SE2>()(startPose);
    result = result * 31 + hashVector(wayPoints);
    result = result * 31 + std::hash<RoutingSearchParameters>()(searchParameters);
    return result;
}

 } // namespace av
 } // namespace msg


CEREAL_BIND_TO_ARCHIVES(av::msg::RoutingRequest)

