#include <interfaces/RoutingResponse/RoutingResponse.h>

#include <iterator>
#include <type_traits>

#include <cereal/cereal.hpp>
#include <cereal/archives/json.hpp>
#include <cereal/archives/binary.hpp>

#include <cereal/types/map.hpp>
#include <cereal/types/vector.hpp>
#include <cereal/types/array.hpp>
#include <cereal/types/string.hpp>
#include <cereal/types/complex.hpp>

#include <ODatum/IntrospectionArchive.h>
#include <ODatum/Hash.h>


 namespace av {
 namespace msg {


RoutingItemMeta::RoutingItemMeta() = default;

RoutingItemMeta::RoutingItemMeta(
    uint64_t MaxStandingTimeSec_arg 
    ):
    MaxStandingTimeSec { std::move(MaxStandingTimeSec_arg) } // NOLINT
      
    {
}

RoutingItemMeta::RoutingItemMeta(const RoutingItemMeta&) = default;

// NOLINTNEXTLINE
RoutingItemMeta::RoutingItemMeta(RoutingItemMeta&&) = default;

RoutingItemMeta& RoutingItemMeta::operator=(const RoutingItemMeta&) = default;

// NOLINTNEXTLINE
RoutingItemMeta& RoutingItemMeta::operator=(RoutingItemMeta&&) = default;

void RoutingItemMeta::toCsv(std::ostream& out) const
{
    out.precision(16);
    out << std::boolalpha;
    out << MaxStandingTimeSec;
    
}

std::stringstream RoutingItemMeta::toStringstream() const
{
    std::stringstream out;
    toCsv(out);
    return out;
}

template<class Archive>
    void RoutingItemMeta::serialize(Archive& ar, [[maybe_unused]] unsigned int version)
    {

        ar& CEREAL_NVP(MaxStandingTimeSec);
    }


template void RoutingItemMeta::serialize(cereal::BinaryInputArchive& ar, unsigned int version);
template void RoutingItemMeta::serialize(cereal::BinaryOutputArchive& ar, unsigned int version);
template void RoutingItemMeta::serialize(cereal::IntrospectionArchive& ar, unsigned int version);
template void RoutingItemMeta::serialize(cereal::JSONInputArchive& ar, unsigned int version);
template void RoutingItemMeta::serialize(cereal::JSONOutputArchive& ar, unsigned int version);
template void RoutingItemMeta::serialize(cereal::PortableBinaryInputArchive& ar, unsigned int version);
template void RoutingItemMeta::serialize(cereal::PortableBinaryOutputArchive& ar, unsigned int version);

std::ostream& operator<< (std::ostream& os, const RoutingItemMeta& routingitemmeta)
{
    routingitemmeta.toCsv(os);
    return os;
}

bool operator== (const RoutingItemMeta &lhs, const RoutingItemMeta &rhs)
{
    return (  lhs.MaxStandingTimeSec== rhs.MaxStandingTimeSec 
    );
}

std::size_t RoutingItemMeta::hash() const noexcept
{
    std::size_t result = 17;
    result = result * 31 + std::hash<uint64_t>()(MaxStandingTimeSec);
    return result;
}

 } // namespace av
 } // namespace msg


CEREAL_BIND_TO_ARCHIVES(av::msg::RoutingItemMeta)

 namespace av {
 namespace msg {


RoutingItem::RoutingItem() = default;

RoutingItem::RoutingItem(
    av::msg::SE2 startPose_arg ,
    av::msg::SE2 targetPose_arg ,
    std::vector<MapId> roadblocks_arg ,
    RoutingItemMeta meta_arg 
    ):
    startPose { std::move(startPose_arg) } // NOLINT
      ,
    targetPose { std::move(targetPose_arg) } // NOLINT
      ,
    roadblocks { std::move(roadblocks_arg) } // NOLINT
      ,
    meta { std::move(meta_arg) } // NOLINT
      
    {
}

RoutingItem::RoutingItem(const RoutingItem&) = default;

// NOLINTNEXTLINE
RoutingItem::RoutingItem(RoutingItem&&) = default;

RoutingItem& RoutingItem::operator=(const RoutingItem&) = default;

// NOLINTNEXTLINE
RoutingItem& RoutingItem::operator=(RoutingItem&&) = default;

void RoutingItem::toCsv(std::ostream& out) const
{
    out.precision(16);
    out << std::boolalpha;
    out << startPose;out << ",";
    out << targetPose;out << ",";
    // Enclose sequence in double quotations and curly brackets to keep CSV line length uniform
        
        out << "\"{{";
        
        if (!roadblocks.empty())
        {
            std::copy(roadblocks.begin(), roadblocks.end() - 1, std::ostream_iterator<MapId>(out, "},{"));
            std::copy(roadblocks.end()-1, roadblocks.end(), std::ostream_iterator<MapId>(out, ""));
        }
        out << "}}\"";out << ",";
    out << meta;
    
}

std::stringstream RoutingItem::toStringstream() const
{
    std::stringstream out;
    toCsv(out);
    return out;
}

template<class Archive>
    void RoutingItem::serialize(Archive& ar, [[maybe_unused]] unsigned int version)
    {

        ar& CEREAL_NVP(startPose);
        ar& CEREAL_NVP(targetPose);
        ar& CEREAL_NVP(roadblocks);
        ar& CEREAL_NVP(meta);
    }


template void RoutingItem::serialize(cereal::BinaryInputArchive& ar, unsigned int version);
template void RoutingItem::serialize(cereal::BinaryOutputArchive& ar, unsigned int version);
template void RoutingItem::serialize(cereal::IntrospectionArchive& ar, unsigned int version);
template void RoutingItem::serialize(cereal::JSONInputArchive& ar, unsigned int version);
template void RoutingItem::serialize(cereal::JSONOutputArchive& ar, unsigned int version);
template void RoutingItem::serialize(cereal::PortableBinaryInputArchive& ar, unsigned int version);
template void RoutingItem::serialize(cereal::PortableBinaryOutputArchive& ar, unsigned int version);

std::ostream& operator<< (std::ostream& os, const RoutingItem& routingitem)
{
    routingitem.toCsv(os);
    return os;
}

bool operator== (const RoutingItem &lhs, const RoutingItem &rhs)
{
    return (  lhs.startPose== rhs.startPose &&
     lhs.targetPose== rhs.targetPose &&
     lhs.roadblocks== rhs.roadblocks &&
     lhs.meta== rhs.meta 
    );
}

std::size_t RoutingItem::hash() const noexcept
{
    std::size_t result = 17;
    result = result * 31 + std::hash<av::msg::SE2>()(startPose);
    result = result * 31 + std::hash<av::msg::SE2>()(targetPose);
    result = result * 31 + hashVector(roadblocks);
    result = result * 31 + std::hash<RoutingItemMeta>()(meta);
    return result;
}

 } // namespace av
 } // namespace msg


CEREAL_BIND_TO_ARCHIVES(av::msg::RoutingItem)

 namespace av {
 namespace msg {


RoutingResponseStatus::RoutingResponseStatus() = default;

RoutingResponseStatus::RoutingResponseStatus(
    RoutingResponseState state_arg ,
    std::string details_arg 
    ):
    state { std::move(state_arg) } // NOLINT
      ,
    details { std::move(details_arg) } // NOLINT
      
    {
}

RoutingResponseStatus::RoutingResponseStatus(const RoutingResponseStatus&) = default;

// NOLINTNEXTLINE
RoutingResponseStatus::RoutingResponseStatus(RoutingResponseStatus&&) = default;

RoutingResponseStatus& RoutingResponseStatus::operator=(const RoutingResponseStatus&) = default;

// NOLINTNEXTLINE
RoutingResponseStatus& RoutingResponseStatus::operator=(RoutingResponseStatus&&) = default;

void RoutingResponseStatus::toCsv(std::ostream& out) const
{
    out.precision(16);
    out << std::boolalpha;
    out << static_cast<uint32_t>(state);out << ",";
    out << "\"";
        out << details;
        out << "\"";
    
}

std::stringstream RoutingResponseStatus::toStringstream() const
{
    std::stringstream out;
    toCsv(out);
    return out;
}

template<class Archive>
    void RoutingResponseStatus::serialize(Archive& ar, [[maybe_unused]] unsigned int version)
    {

        ar& CEREAL_NVP(state);
        ar& CEREAL_NVP(details);
    }


template void RoutingResponseStatus::serialize(cereal::BinaryInputArchive& ar, unsigned int version);
template void RoutingResponseStatus::serialize(cereal::BinaryOutputArchive& ar, unsigned int version);
template void RoutingResponseStatus::serialize(cereal::IntrospectionArchive& ar, unsigned int version);
template void RoutingResponseStatus::serialize(cereal::JSONInputArchive& ar, unsigned int version);
template void RoutingResponseStatus::serialize(cereal::JSONOutputArchive& ar, unsigned int version);
template void RoutingResponseStatus::serialize(cereal::PortableBinaryInputArchive& ar, unsigned int version);
template void RoutingResponseStatus::serialize(cereal::PortableBinaryOutputArchive& ar, unsigned int version);

std::ostream& operator<< (std::ostream& os, const RoutingResponseStatus& routingresponsestatus)
{
    routingresponsestatus.toCsv(os);
    return os;
}

bool operator== (const RoutingResponseStatus &lhs, const RoutingResponseStatus &rhs)
{
    return (  lhs.state== rhs.state &&
     lhs.details== rhs.details 
    );
}

std::size_t RoutingResponseStatus::hash() const noexcept
{
    std::size_t result = 17;
    result = result * 31 + std::hash<RoutingResponseState>()(state);
    result = result * 31 + std::hash<std::string>()(details);
    return result;
}

 } // namespace av
 } // namespace msg


CEREAL_BIND_TO_ARCHIVES(av::msg::RoutingResponseStatus)




 namespace av {
 namespace msg {


RoutingResponse::RoutingResponse() = default;

RoutingResponse::RoutingResponse(
    av::msg::TimePoint timestamp_arg ,
    msg::GUID requestId_arg ,
    int32_t utmZone_arg ,
    RoutingResponseStatus status_arg ,
    std::vector<RoutingItem> items_arg 
    ):
    timestamp { std::move(timestamp_arg) } // NOLINT
      ,requestId { std::move(requestId_arg) } // NOLINT
      ,utmZone { std::move(utmZone_arg) } // NOLINT
      ,status { std::move(status_arg) } // NOLINT
      ,items { std::move(items_arg) } // NOLINT
      {
}

RoutingResponse::RoutingResponse(const RoutingResponse&) = default;

// NOLINTNEXTLINE
RoutingResponse::RoutingResponse(RoutingResponse&&) = default;

RoutingResponse& RoutingResponse::operator=(const RoutingResponse&) = default;

// NOLINTNEXTLINE
RoutingResponse& RoutingResponse::operator=(RoutingResponse&&) = default;

void RoutingResponse::toCsv(std::ostream& out) const
{
    out.precision(16);
    out << std::boolalpha;
    out << timestamp;out << ",";
    out << requestId;out << ",";
    out << utmZone;out << ",";
    out << status;out << ",";
    // Enclose sequence in double quotations and curly brackets to keep CSV line length uniform
        
        out << "\"{{";
        
        if (!items.empty())
        {
            std::copy(items.begin(), items.end() - 1, std::ostream_iterator<RoutingItem>(out, "},{"));
            std::copy(items.end()-1, items.end(), std::ostream_iterator<RoutingItem>(out, ""));
        }
        out << "}}\"";
    
}

std::stringstream RoutingResponse::toStringstream() const
{
    std::stringstream out;
    toCsv(out);
    return out;
}

template<class Archive>
    void RoutingResponse::serialize(Archive& ar, [[maybe_unused]] unsigned int version)
    {

        ar& CEREAL_NVP(timestamp);
        ar& CEREAL_NVP(requestId);
        ar& CEREAL_NVP(utmZone);
        ar& CEREAL_NVP(status);
        ar& CEREAL_NVP(items);
    }


template void RoutingResponse::serialize(cereal::BinaryInputArchive& ar, unsigned int version);
template void RoutingResponse::serialize(cereal::BinaryOutputArchive& ar, unsigned int version);
template void RoutingResponse::serialize(cereal::IntrospectionArchive& ar, unsigned int version);
template void RoutingResponse::serialize(cereal::JSONInputArchive& ar, unsigned int version);
template void RoutingResponse::serialize(cereal::JSONOutputArchive& ar, unsigned int version);
template void RoutingResponse::serialize(cereal::PortableBinaryInputArchive& ar, unsigned int version);
template void RoutingResponse::serialize(cereal::PortableBinaryOutputArchive& ar, unsigned int version);

std::ostream& operator<< (std::ostream& os, const RoutingResponse& routingresponse)
{
    routingresponse.toCsv(os);
    return os;
}

bool operator== (const RoutingResponse &lhs, const RoutingResponse &rhs)
{
    return (  lhs.timestamp== rhs.timestamp &&
     lhs.requestId== rhs.requestId &&
     lhs.utmZone== rhs.utmZone &&
     lhs.status== rhs.status &&
     lhs.items== rhs.items 
    );
}

std::size_t RoutingResponse::hash() const noexcept
{
    std::size_t result = 17;
    result = result * 31 + std::hash<av::msg::TimePoint>()(timestamp);
    result = result * 31 + std::hash<msg::GUID>()(requestId);
    result = result * 31 + std::hash<int32_t>()(utmZone);
    result = result * 31 + std::hash<RoutingResponseStatus>()(status);
    result = result * 31 + hashVector(items);
    return result;
}

 } // namespace av
 } // namespace msg


CEREAL_BIND_TO_ARCHIVES(av::msg::RoutingResponse)

