#include <interfaces/RvaCameraSceneUIEvent/RvaCameraSceneUIEvent.h>

#include <iterator>
#include <type_traits>

#include <cereal/cereal.hpp>
#include <cereal/archives/json.hpp>
#include <cereal/archives/binary.hpp>

#include <cereal/types/map.hpp>
#include <cereal/types/vector.hpp>
#include <cereal/types/array.hpp>
#include <cereal/types/string.hpp>
#include <cereal/types/complex.hpp>

#include <ODatum/IntrospectionArchive.h>
#include <ODatum/Hash.h>





 namespace av {
 namespace msg {
 namespace rva {
 namespace hmi_metric {


RvaCameraSceneUIEvent::RvaCameraSceneUIEvent() = default;

RvaCameraSceneUIEvent::RvaCameraSceneUIEvent(
    RvaCameraSceneUIEventType type_arg ,
    bool keyboardUsed_arg 
    ):
    type { std::move(type_arg) } // NOLINT
      ,keyboardUsed { std::move(keyboardUsed_arg) } // NOLINT
      {
}

RvaCameraSceneUIEvent::RvaCameraSceneUIEvent(const RvaCameraSceneUIEvent&) = default;

// NOLINTNEXTLINE
RvaCameraSceneUIEvent::RvaCameraSceneUIEvent(RvaCameraSceneUIEvent&&) = default;

RvaCameraSceneUIEvent& RvaCameraSceneUIEvent::operator=(const RvaCameraSceneUIEvent&) = default;

// NOLINTNEXTLINE
RvaCameraSceneUIEvent& RvaCameraSceneUIEvent::operator=(RvaCameraSceneUIEvent&&) = default;

void RvaCameraSceneUIEvent::toCsv(std::ostream& out) const
{
    out.precision(16);
    out << std::boolalpha;
    out << static_cast<uint32_t>(type);out << ",";
    out << keyboardUsed;
    
}

std::stringstream RvaCameraSceneUIEvent::toStringstream() const
{
    std::stringstream out;
    toCsv(out);
    return out;
}

template<class Archive>
    void RvaCameraSceneUIEvent::serialize(Archive& ar, [[maybe_unused]] unsigned int version)
    {

        ar& CEREAL_NVP(type);
        ar& CEREAL_NVP(keyboardUsed);
    }


template void RvaCameraSceneUIEvent::serialize(cereal::BinaryInputArchive& ar, unsigned int version);
template void RvaCameraSceneUIEvent::serialize(cereal::BinaryOutputArchive& ar, unsigned int version);
template void RvaCameraSceneUIEvent::serialize(cereal::IntrospectionArchive& ar, unsigned int version);
template void RvaCameraSceneUIEvent::serialize(cereal::JSONInputArchive& ar, unsigned int version);
template void RvaCameraSceneUIEvent::serialize(cereal::JSONOutputArchive& ar, unsigned int version);
template void RvaCameraSceneUIEvent::serialize(cereal::PortableBinaryInputArchive& ar, unsigned int version);
template void RvaCameraSceneUIEvent::serialize(cereal::PortableBinaryOutputArchive& ar, unsigned int version);

std::ostream& operator<< (std::ostream& os, const RvaCameraSceneUIEvent& rvacamerasceneuievent)
{
    rvacamerasceneuievent.toCsv(os);
    return os;
}

bool operator== (const RvaCameraSceneUIEvent &lhs, const RvaCameraSceneUIEvent &rhs)
{
    return (  lhs.type== rhs.type &&
     lhs.keyboardUsed== rhs.keyboardUsed 
    );
}

std::size_t RvaCameraSceneUIEvent::hash() const noexcept
{
    std::size_t result = 17;
    result = result * 31 + std::hash<RvaCameraSceneUIEventType>()(type);
    result = result * 31 + std::hash<bool>()(keyboardUsed);
    return result;
}

 } // namespace av
 } // namespace msg
 } // namespace rva
 } // namespace hmi_metric


CEREAL_BIND_TO_ARCHIVES(av::msg::rva::hmi_metric::RvaCameraSceneUIEvent)

