#include <interfaces/RvaCameraStreamViewUIEvent/RvaCameraStreamViewUIEvent.h>

#include <iterator>
#include <type_traits>

#include <cereal/cereal.hpp>
#include <cereal/archives/json.hpp>
#include <cereal/archives/binary.hpp>

#include <cereal/types/map.hpp>
#include <cereal/types/vector.hpp>
#include <cereal/types/array.hpp>
#include <cereal/types/string.hpp>
#include <cereal/types/complex.hpp>

#include <ODatum/IntrospectionArchive.h>
#include <ODatum/Hash.h>





 namespace av {
 namespace msg {
 namespace rva {
 namespace hmi_metric {


RvaCameraStreamViewUIEvent::RvaCameraStreamViewUIEvent() = default;

RvaCameraStreamViewUIEvent::RvaCameraStreamViewUIEvent(
    RvaCameraStreamViewUIEventType type_arg ,
    std::string cameraId_arg ,
    std::string oldCameraId_arg ,
    bool keyboardUsed_arg 
    ):
    type { std::move(type_arg) } // NOLINT
      ,cameraId { std::move(cameraId_arg) } // NOLINT
      ,oldCameraId { std::move(oldCameraId_arg) } // NOLINT
      ,keyboardUsed { std::move(keyboardUsed_arg) } // NOLINT
      {
}

RvaCameraStreamViewUIEvent::RvaCameraStreamViewUIEvent(const RvaCameraStreamViewUIEvent&) = default;

// NOLINTNEXTLINE
RvaCameraStreamViewUIEvent::RvaCameraStreamViewUIEvent(RvaCameraStreamViewUIEvent&&) = default;

RvaCameraStreamViewUIEvent& RvaCameraStreamViewUIEvent::operator=(const RvaCameraStreamViewUIEvent&) = default;

// NOLINTNEXTLINE
RvaCameraStreamViewUIEvent& RvaCameraStreamViewUIEvent::operator=(RvaCameraStreamViewUIEvent&&) = default;

void RvaCameraStreamViewUIEvent::toCsv(std::ostream& out) const
{
    out.precision(16);
    out << std::boolalpha;
    out << static_cast<uint32_t>(type);out << ",";
    out << "\"";
        out << cameraId;
        out << "\"";out << ",";
    out << "\"";
        out << oldCameraId;
        out << "\"";out << ",";
    out << keyboardUsed;
    
}

std::stringstream RvaCameraStreamViewUIEvent::toStringstream() const
{
    std::stringstream out;
    toCsv(out);
    return out;
}

template<class Archive>
    void RvaCameraStreamViewUIEvent::serialize(Archive& ar, [[maybe_unused]] unsigned int version)
    {

        ar& CEREAL_NVP(type);
        ar& CEREAL_NVP(cameraId);
        ar& CEREAL_NVP(oldCameraId);
        ar& CEREAL_NVP(keyboardUsed);
    }


template void RvaCameraStreamViewUIEvent::serialize(cereal::BinaryInputArchive& ar, unsigned int version);
template void RvaCameraStreamViewUIEvent::serialize(cereal::BinaryOutputArchive& ar, unsigned int version);
template void RvaCameraStreamViewUIEvent::serialize(cereal::IntrospectionArchive& ar, unsigned int version);
template void RvaCameraStreamViewUIEvent::serialize(cereal::JSONInputArchive& ar, unsigned int version);
template void RvaCameraStreamViewUIEvent::serialize(cereal::JSONOutputArchive& ar, unsigned int version);
template void RvaCameraStreamViewUIEvent::serialize(cereal::PortableBinaryInputArchive& ar, unsigned int version);
template void RvaCameraStreamViewUIEvent::serialize(cereal::PortableBinaryOutputArchive& ar, unsigned int version);

std::ostream& operator<< (std::ostream& os, const RvaCameraStreamViewUIEvent& rvacamerastreamviewuievent)
{
    rvacamerastreamviewuievent.toCsv(os);
    return os;
}

bool operator== (const RvaCameraStreamViewUIEvent &lhs, const RvaCameraStreamViewUIEvent &rhs)
{
    return (  lhs.type== rhs.type &&
     lhs.cameraId== rhs.cameraId &&
     lhs.oldCameraId== rhs.oldCameraId &&
     lhs.keyboardUsed== rhs.keyboardUsed 
    );
}

std::size_t RvaCameraStreamViewUIEvent::hash() const noexcept
{
    std::size_t result = 17;
    result = result * 31 + std::hash<RvaCameraStreamViewUIEventType>()(type);
    result = result * 31 + std::hash<std::string>()(cameraId);
    result = result * 31 + std::hash<std::string>()(oldCameraId);
    result = result * 31 + std::hash<bool>()(keyboardUsed);
    return result;
}

 } // namespace av
 } // namespace msg
 } // namespace rva
 } // namespace hmi_metric


CEREAL_BIND_TO_ARCHIVES(av::msg::rva::hmi_metric::RvaCameraStreamViewUIEvent)

