#include <interfaces/RvaHmiMetric/RvaHmiMetric.h>

#include <iterator>
#include <type_traits>

#include <cereal/cereal.hpp>
#include <cereal/archives/json.hpp>
#include <cereal/archives/binary.hpp>

#include <cereal/types/map.hpp>
#include <cereal/types/vector.hpp>
#include <cereal/types/array.hpp>
#include <cereal/types/string.hpp>
#include <cereal/types/complex.hpp>

#include <ODatum/IntrospectionArchive.h>
#include <ODatum/Hash.h>





 namespace av {
 namespace msg {
 namespace rva {
 namespace hmi_metric {


RvaHmiMetric::RvaHmiMetric() = default;

RvaHmiMetric::RvaHmiMetric(
    TimePoint timestamp_arg ,
    RvaHmiMetricType activeEventType_arg ,
    RvaAssistanceRequestEvent assistanceRequestEvent_arg ,
    RvaCameraSceneUIEvent cameraSceneUIEvent_arg ,
    RvaCameraStreamViewUIEvent cameraStreamViewUIEvent_arg ,
    RvaPathPlanningUIEvent pathPlanningUIEvent_arg ,
    RvaPathPlanningStatusEvent pathPlanningStatusEvent_arg ,
    RvaSensorCleaningRequestUIEvent sensorCleaningRequestUIEvent_arg ,
    RvaSensorCleaningStatusEvent sensorCleaningStatusEvent_arg ,
    RvaWaitAndGoUIEvent waitAndGoEvent_arg ,
    RvaZoomUIEvent zoomUIEvent_arg 
    ):
    timestamp { std::move(timestamp_arg) } // NOLINT
      ,activeEventType { std::move(activeEventType_arg) } // NOLINT
      ,assistanceRequestEvent { std::move(assistanceRequestEvent_arg) } // NOLINT
      ,cameraSceneUIEvent { std::move(cameraSceneUIEvent_arg) } // NOLINT
      ,cameraStreamViewUIEvent { std::move(cameraStreamViewUIEvent_arg) } // NOLINT
      ,pathPlanningUIEvent { std::move(pathPlanningUIEvent_arg) } // NOLINT
      ,pathPlanningStatusEvent { std::move(pathPlanningStatusEvent_arg) } // NOLINT
      ,sensorCleaningRequestUIEvent { std::move(sensorCleaningRequestUIEvent_arg) } // NOLINT
      ,sensorCleaningStatusEvent { std::move(sensorCleaningStatusEvent_arg) } // NOLINT
      ,waitAndGoEvent { std::move(waitAndGoEvent_arg) } // NOLINT
      ,zoomUIEvent { std::move(zoomUIEvent_arg) } // NOLINT
      {
}

RvaHmiMetric::RvaHmiMetric(const RvaHmiMetric&) = default;

// NOLINTNEXTLINE
RvaHmiMetric::RvaHmiMetric(RvaHmiMetric&&) = default;

RvaHmiMetric& RvaHmiMetric::operator=(const RvaHmiMetric&) = default;

// NOLINTNEXTLINE
RvaHmiMetric& RvaHmiMetric::operator=(RvaHmiMetric&&) = default;

void RvaHmiMetric::toCsv(std::ostream& out) const
{
    out.precision(16);
    out << std::boolalpha;
    out << timestamp;out << ",";
    out << static_cast<uint32_t>(activeEventType);out << ",";
    out << assistanceRequestEvent;out << ",";
    out << cameraSceneUIEvent;out << ",";
    out << cameraStreamViewUIEvent;out << ",";
    out << pathPlanningUIEvent;out << ",";
    out << pathPlanningStatusEvent;out << ",";
    out << sensorCleaningRequestUIEvent;out << ",";
    out << sensorCleaningStatusEvent;out << ",";
    out << waitAndGoEvent;out << ",";
    out << zoomUIEvent;
    
}

std::stringstream RvaHmiMetric::toStringstream() const
{
    std::stringstream out;
    toCsv(out);
    return out;
}

template<class Archive>
    void RvaHmiMetric::serialize(Archive& ar, [[maybe_unused]] unsigned int version)
    {

        ar& CEREAL_NVP(timestamp);
        ar& CEREAL_NVP(activeEventType);
        ar& CEREAL_NVP(assistanceRequestEvent);
        ar& CEREAL_NVP(cameraSceneUIEvent);
        ar& CEREAL_NVP(cameraStreamViewUIEvent);
        ar& CEREAL_NVP(pathPlanningUIEvent);
        ar& CEREAL_NVP(pathPlanningStatusEvent);
        ar& CEREAL_NVP(sensorCleaningRequestUIEvent);
        ar& CEREAL_NVP(sensorCleaningStatusEvent);
        ar& CEREAL_NVP(waitAndGoEvent);
        ar& CEREAL_NVP(zoomUIEvent);
    }


template void RvaHmiMetric::serialize(cereal::BinaryInputArchive& ar, unsigned int version);
template void RvaHmiMetric::serialize(cereal::BinaryOutputArchive& ar, unsigned int version);
template void RvaHmiMetric::serialize(cereal::IntrospectionArchive& ar, unsigned int version);
template void RvaHmiMetric::serialize(cereal::JSONInputArchive& ar, unsigned int version);
template void RvaHmiMetric::serialize(cereal::JSONOutputArchive& ar, unsigned int version);
template void RvaHmiMetric::serialize(cereal::PortableBinaryInputArchive& ar, unsigned int version);
template void RvaHmiMetric::serialize(cereal::PortableBinaryOutputArchive& ar, unsigned int version);

std::ostream& operator<< (std::ostream& os, const RvaHmiMetric& rvahmimetric)
{
    rvahmimetric.toCsv(os);
    return os;
}

bool operator== (const RvaHmiMetric &lhs, const RvaHmiMetric &rhs)
{
    return (  lhs.timestamp== rhs.timestamp &&
     lhs.activeEventType== rhs.activeEventType &&
     lhs.assistanceRequestEvent== rhs.assistanceRequestEvent &&
     lhs.cameraSceneUIEvent== rhs.cameraSceneUIEvent &&
     lhs.cameraStreamViewUIEvent== rhs.cameraStreamViewUIEvent &&
     lhs.pathPlanningUIEvent== rhs.pathPlanningUIEvent &&
     lhs.pathPlanningStatusEvent== rhs.pathPlanningStatusEvent &&
     lhs.sensorCleaningRequestUIEvent== rhs.sensorCleaningRequestUIEvent &&
     lhs.sensorCleaningStatusEvent== rhs.sensorCleaningStatusEvent &&
     lhs.waitAndGoEvent== rhs.waitAndGoEvent &&
     lhs.zoomUIEvent== rhs.zoomUIEvent 
    );
}

std::size_t RvaHmiMetric::hash() const noexcept
{
    std::size_t result = 17;
    result = result * 31 + 0;
    result = result * 31 + std::hash<RvaHmiMetricType>()(activeEventType);
    result = result * 31 + std::hash<RvaAssistanceRequestEvent>()(assistanceRequestEvent);
    result = result * 31 + std::hash<RvaCameraSceneUIEvent>()(cameraSceneUIEvent);
    result = result * 31 + std::hash<RvaCameraStreamViewUIEvent>()(cameraStreamViewUIEvent);
    result = result * 31 + std::hash<RvaPathPlanningUIEvent>()(pathPlanningUIEvent);
    result = result * 31 + std::hash<RvaPathPlanningStatusEvent>()(pathPlanningStatusEvent);
    result = result * 31 + std::hash<RvaSensorCleaningRequestUIEvent>()(sensorCleaningRequestUIEvent);
    result = result * 31 + std::hash<RvaSensorCleaningStatusEvent>()(sensorCleaningStatusEvent);
    result = result * 31 + std::hash<RvaWaitAndGoUIEvent>()(waitAndGoEvent);
    result = result * 31 + std::hash<RvaZoomUIEvent>()(zoomUIEvent);
    return result;
}

 } // namespace av
 } // namespace msg
 } // namespace rva
 } // namespace hmi_metric


CEREAL_BIND_TO_ARCHIVES(av::msg::rva::hmi_metric::RvaHmiMetric)

