#include <interfaces/RvaPathMsg/RvaPathMsg.h>

#include <iterator>
#include <type_traits>

#include <cereal/cereal.hpp>
#include <cereal/archives/json.hpp>
#include <cereal/archives/binary.hpp>

#include <cereal/types/map.hpp>
#include <cereal/types/vector.hpp>
#include <cereal/types/array.hpp>
#include <cereal/types/string.hpp>
#include <cereal/types/complex.hpp>

#include <ODatum/IntrospectionArchive.h>
#include <ODatum/Hash.h>





 namespace av {
 namespace msg {


RvaPathMsg::RvaPathMsg() = default;

RvaPathMsg::RvaPathMsg(
    TimePoint timestamp_arg ,
    std::vector<DubinsTransition> transitions_arg ,
    int64_t id_arg 
    ):
    timestamp { std::move(timestamp_arg) } // NOLINT
      ,transitions { std::move(transitions_arg) } // NOLINT
      ,id { std::move(id_arg) } // NOLINT
      {
}

RvaPathMsg::RvaPathMsg(const RvaPathMsg&) = default;

// NOLINTNEXTLINE
RvaPathMsg::RvaPathMsg(RvaPathMsg&&) = default;

RvaPathMsg& RvaPathMsg::operator=(const RvaPathMsg&) = default;

// NOLINTNEXTLINE
RvaPathMsg& RvaPathMsg::operator=(RvaPathMsg&&) = default;

void RvaPathMsg::toCsv(std::ostream& out) const
{
    out.precision(16);
    out << std::boolalpha;
    out << timestamp;out << ",";
    // Enclose sequence in double quotations and curly brackets to keep CSV line length uniform
        
        out << "\"{{";
        
        if (!transitions.empty())
        {
            std::copy(transitions.begin(), transitions.end() - 1, std::ostream_iterator<DubinsTransition>(out, "},{"));
            std::copy(transitions.end()-1, transitions.end(), std::ostream_iterator<DubinsTransition>(out, ""));
        }
        out << "}}\"";out << ",";
    out << id;
    
}

std::stringstream RvaPathMsg::toStringstream() const
{
    std::stringstream out;
    toCsv(out);
    return out;
}

template<class Archive>
    void RvaPathMsg::serialize(Archive& ar, unsigned int version)
{
    if(version >= 1)
    {
        ar& CEREAL_NVP(timestamp);
        ar& CEREAL_NVP(transitions);
    }
    if(version >= 2)
    {
        ar& CEREAL_NVP(id);
    }
}


template void RvaPathMsg::serialize(cereal::BinaryInputArchive& ar, unsigned int version);
template void RvaPathMsg::serialize(cereal::BinaryOutputArchive& ar, unsigned int version);
template void RvaPathMsg::serialize(cereal::IntrospectionArchive& ar, unsigned int version);
template void RvaPathMsg::serialize(cereal::JSONInputArchive& ar, unsigned int version);
template void RvaPathMsg::serialize(cereal::JSONOutputArchive& ar, unsigned int version);
template void RvaPathMsg::serialize(cereal::PortableBinaryInputArchive& ar, unsigned int version);
template void RvaPathMsg::serialize(cereal::PortableBinaryOutputArchive& ar, unsigned int version);

std::ostream& operator<< (std::ostream& os, const RvaPathMsg& rvapathmsg)
{
    rvapathmsg.toCsv(os);
    return os;
}

bool operator== (const RvaPathMsg &lhs, const RvaPathMsg &rhs)
{
    return (  lhs.timestamp== rhs.timestamp &&
     lhs.transitions== rhs.transitions &&
     lhs.id== rhs.id 
    );
}

std::size_t RvaPathMsg::hash() const noexcept
{
    std::size_t result = 17;
    result = result * 31 + 0;
    result = result * 31 + hashVector(transitions);
    result = result * 31 + std::hash<int64_t>()(id);
    return result;
}

 } // namespace av
 } // namespace msg


CEREAL_BIND_TO_ARCHIVES(av::msg::RvaPathMsg)

