#include <interfaces/RvaPathPlanningUIEvent/RvaPathPlanningUIEvent.h>

#include <iterator>
#include <type_traits>

#include <cereal/cereal.hpp>
#include <cereal/archives/json.hpp>
#include <cereal/archives/binary.hpp>

#include <cereal/types/map.hpp>
#include <cereal/types/vector.hpp>
#include <cereal/types/array.hpp>
#include <cereal/types/string.hpp>
#include <cereal/types/complex.hpp>

#include <ODatum/IntrospectionArchive.h>
#include <ODatum/Hash.h>





 namespace av {
 namespace msg {
 namespace rva {
 namespace hmi_metric {


RvaPathPlanningUIEvent::RvaPathPlanningUIEvent() = default;

RvaPathPlanningUIEvent::RvaPathPlanningUIEvent(
    RvaPathPlanningUIEventType type_arg ,
    double pathLength_arg ,
    uint32_t numWaypoints_arg ,
    bool keyboardUsed_arg 
    ):
    type { std::move(type_arg) } // NOLINT
      ,pathLength { std::move(pathLength_arg) } // NOLINT
      ,numWaypoints { std::move(numWaypoints_arg) } // NOLINT
      ,keyboardUsed { std::move(keyboardUsed_arg) } // NOLINT
      {
}

RvaPathPlanningUIEvent::RvaPathPlanningUIEvent(const RvaPathPlanningUIEvent&) = default;

// NOLINTNEXTLINE
RvaPathPlanningUIEvent::RvaPathPlanningUIEvent(RvaPathPlanningUIEvent&&) = default;

RvaPathPlanningUIEvent& RvaPathPlanningUIEvent::operator=(const RvaPathPlanningUIEvent&) = default;

// NOLINTNEXTLINE
RvaPathPlanningUIEvent& RvaPathPlanningUIEvent::operator=(RvaPathPlanningUIEvent&&) = default;

void RvaPathPlanningUIEvent::toCsv(std::ostream& out) const
{
    out.precision(16);
    out << std::boolalpha;
    out << static_cast<uint32_t>(type);out << ",";
    out << pathLength;out << ",";
    out << numWaypoints;out << ",";
    out << keyboardUsed;
    
}

std::stringstream RvaPathPlanningUIEvent::toStringstream() const
{
    std::stringstream out;
    toCsv(out);
    return out;
}

template<class Archive>
    void RvaPathPlanningUIEvent::serialize(Archive& ar, [[maybe_unused]] unsigned int version)
    {

        ar& CEREAL_NVP(type);
        ar& CEREAL_NVP(pathLength);
        ar& CEREAL_NVP(numWaypoints);
        ar& CEREAL_NVP(keyboardUsed);
    }


template void RvaPathPlanningUIEvent::serialize(cereal::BinaryInputArchive& ar, unsigned int version);
template void RvaPathPlanningUIEvent::serialize(cereal::BinaryOutputArchive& ar, unsigned int version);
template void RvaPathPlanningUIEvent::serialize(cereal::IntrospectionArchive& ar, unsigned int version);
template void RvaPathPlanningUIEvent::serialize(cereal::JSONInputArchive& ar, unsigned int version);
template void RvaPathPlanningUIEvent::serialize(cereal::JSONOutputArchive& ar, unsigned int version);
template void RvaPathPlanningUIEvent::serialize(cereal::PortableBinaryInputArchive& ar, unsigned int version);
template void RvaPathPlanningUIEvent::serialize(cereal::PortableBinaryOutputArchive& ar, unsigned int version);

std::ostream& operator<< (std::ostream& os, const RvaPathPlanningUIEvent& rvapathplanninguievent)
{
    rvapathplanninguievent.toCsv(os);
    return os;
}

bool operator== (const RvaPathPlanningUIEvent &lhs, const RvaPathPlanningUIEvent &rhs)
{
    return (  lhs.type== rhs.type &&
     lhs.pathLength== rhs.pathLength &&
     lhs.numWaypoints== rhs.numWaypoints &&
     lhs.keyboardUsed== rhs.keyboardUsed 
    );
}

std::size_t RvaPathPlanningUIEvent::hash() const noexcept
{
    std::size_t result = 17;
    result = result * 31 + std::hash<RvaPathPlanningUIEventType>()(type);
    result = result * 31 + std::hash<double>()(pathLength);
    result = result * 31 + std::hash<uint32_t>()(numWaypoints);
    result = result * 31 + std::hash<bool>()(keyboardUsed);
    return result;
}

 } // namespace av
 } // namespace msg
 } // namespace rva
 } // namespace hmi_metric


CEREAL_BIND_TO_ARCHIVES(av::msg::rva::hmi_metric::RvaPathPlanningUIEvent)

