#include <interfaces/RvaStatusMsg/RvaStatusMsg.h>

#include <iterator>
#include <type_traits>

#include <cereal/cereal.hpp>
#include <cereal/archives/json.hpp>
#include <cereal/archives/binary.hpp>

#include <cereal/types/map.hpp>
#include <cereal/types/vector.hpp>
#include <cereal/types/array.hpp>
#include <cereal/types/string.hpp>
#include <cereal/types/complex.hpp>

#include <ODatum/IntrospectionArchive.h>
#include <ODatum/Hash.h>





 namespace av {
 namespace msg {


RvaStatusMsg::RvaStatusMsg() = default;

RvaStatusMsg::RvaStatusMsg(
    TimePoint timestamp_arg ,
    bool rvaResponseAccepted_arg ,
    bool rvaWaitAndGoAccepted_arg ,
    bool hasTrajectoryStatus_arg ,
    bool trajectoryAccepted_arg ,
    bool trajectorySelected_arg ,
    bool trajectoryCompleted_arg ,
    int64_t pathId_arg ,
    bool hasWaitAndGoStatus_arg ,
    bool waitAndGoAccepted_arg 
    ):
    timestamp { std::move(timestamp_arg) } // NOLINT
      ,rvaResponseAccepted { std::move(rvaResponseAccepted_arg) } // NOLINT
      ,rvaWaitAndGoAccepted { std::move(rvaWaitAndGoAccepted_arg) } // NOLINT
      ,hasTrajectoryStatus { std::move(hasTrajectoryStatus_arg) } // NOLINT
      ,trajectoryAccepted { std::move(trajectoryAccepted_arg) } // NOLINT
      ,trajectorySelected { std::move(trajectorySelected_arg) } // NOLINT
      ,trajectoryCompleted { std::move(trajectoryCompleted_arg) } // NOLINT
      ,pathId { std::move(pathId_arg) } // NOLINT
      ,hasWaitAndGoStatus { std::move(hasWaitAndGoStatus_arg) } // NOLINT
      ,waitAndGoAccepted { std::move(waitAndGoAccepted_arg) } // NOLINT
      {
}

RvaStatusMsg::RvaStatusMsg(const RvaStatusMsg&) = default;

// NOLINTNEXTLINE
RvaStatusMsg::RvaStatusMsg(RvaStatusMsg&&) = default;

RvaStatusMsg& RvaStatusMsg::operator=(const RvaStatusMsg&) = default;

// NOLINTNEXTLINE
RvaStatusMsg& RvaStatusMsg::operator=(RvaStatusMsg&&) = default;

void RvaStatusMsg::toCsv(std::ostream& out) const
{
    out.precision(16);
    out << std::boolalpha;
    out << timestamp;out << ",";
    out << rvaResponseAccepted;out << ",";
    out << rvaWaitAndGoAccepted;out << ",";
    out << hasTrajectoryStatus;out << ",";
    out << trajectoryAccepted;out << ",";
    out << trajectorySelected;out << ",";
    out << trajectoryCompleted;out << ",";
    out << pathId;out << ",";
    out << hasWaitAndGoStatus;out << ",";
    out << waitAndGoAccepted;
    
}

std::stringstream RvaStatusMsg::toStringstream() const
{
    std::stringstream out;
    toCsv(out);
    return out;
}

template<class Archive>
    void RvaStatusMsg::serialize(Archive& ar, unsigned int version)
{
    if(version >= 1)
    {
        ar& CEREAL_NVP(timestamp);
        ar& CEREAL_NVP(rvaResponseAccepted);
    }
    if(version >= 2)
    {
        ar& CEREAL_NVP(rvaWaitAndGoAccepted);
    }
    if(version >= 3)
    {
        ar& CEREAL_NVP(hasTrajectoryStatus);
        ar& CEREAL_NVP(trajectoryAccepted);
        ar& CEREAL_NVP(trajectorySelected);
        ar& CEREAL_NVP(hasWaitAndGoStatus);
        ar& CEREAL_NVP(waitAndGoAccepted);
    }
    if(version >= 4)
    {
        ar& CEREAL_NVP(pathId);
    }
    if(version >= 5)
    {
        ar& CEREAL_NVP(trajectoryCompleted);
    }
}


template void RvaStatusMsg::serialize(cereal::BinaryInputArchive& ar, unsigned int version);
template void RvaStatusMsg::serialize(cereal::BinaryOutputArchive& ar, unsigned int version);
template void RvaStatusMsg::serialize(cereal::IntrospectionArchive& ar, unsigned int version);
template void RvaStatusMsg::serialize(cereal::JSONInputArchive& ar, unsigned int version);
template void RvaStatusMsg::serialize(cereal::JSONOutputArchive& ar, unsigned int version);
template void RvaStatusMsg::serialize(cereal::PortableBinaryInputArchive& ar, unsigned int version);
template void RvaStatusMsg::serialize(cereal::PortableBinaryOutputArchive& ar, unsigned int version);

std::ostream& operator<< (std::ostream& os, const RvaStatusMsg& rvastatusmsg)
{
    rvastatusmsg.toCsv(os);
    return os;
}

bool operator== (const RvaStatusMsg &lhs, const RvaStatusMsg &rhs)
{
    return (  lhs.timestamp== rhs.timestamp &&
     lhs.rvaResponseAccepted== rhs.rvaResponseAccepted &&
     lhs.rvaWaitAndGoAccepted== rhs.rvaWaitAndGoAccepted &&
     lhs.hasTrajectoryStatus== rhs.hasTrajectoryStatus &&
     lhs.trajectoryAccepted== rhs.trajectoryAccepted &&
     lhs.trajectorySelected== rhs.trajectorySelected &&
     lhs.trajectoryCompleted== rhs.trajectoryCompleted &&
     lhs.pathId== rhs.pathId &&
     lhs.hasWaitAndGoStatus== rhs.hasWaitAndGoStatus &&
     lhs.waitAndGoAccepted== rhs.waitAndGoAccepted 
    );
}

std::size_t RvaStatusMsg::hash() const noexcept
{
    std::size_t result = 17;
    result = result * 31 + 0;
    result = result * 31 + std::hash<bool>()(rvaResponseAccepted);
    result = result * 31 + std::hash<bool>()(rvaWaitAndGoAccepted);
    result = result * 31 + std::hash<bool>()(hasTrajectoryStatus);
    result = result * 31 + std::hash<bool>()(trajectoryAccepted);
    result = result * 31 + std::hash<bool>()(trajectorySelected);
    result = result * 31 + std::hash<bool>()(trajectoryCompleted);
    result = result * 31 + std::hash<int64_t>()(pathId);
    result = result * 31 + std::hash<bool>()(hasWaitAndGoStatus);
    result = result * 31 + std::hash<bool>()(waitAndGoAccepted);
    return result;
}

 } // namespace av
 } // namespace msg


CEREAL_BIND_TO_ARCHIVES(av::msg::RvaStatusMsg)

