#include <interfaces/SE2/SE2.h>

#include <iterator>
#include <type_traits>

#include <cereal/cereal.hpp>
#include <cereal/archives/json.hpp>
#include <cereal/archives/binary.hpp>

#include <cereal/types/map.hpp>
#include <cereal/types/vector.hpp>
#include <cereal/types/array.hpp>
#include <cereal/types/string.hpp>
#include <cereal/types/complex.hpp>

#include <ODatum/IntrospectionArchive.h>
#include <ODatum/Hash.h>


 namespace av {
 namespace msg {


SE2::SE2() = default;

SE2::SE2(
    double x_arg ,
    double y_arg ,
    double theta_arg 
    ):
    x { std::move(x_arg) } // NOLINT
      ,
    y { std::move(y_arg) } // NOLINT
      ,
    theta { std::move(theta_arg) } // NOLINT
      
    {
}

SE2::SE2(const SE2&) = default;

// NOLINTNEXTLINE
SE2::SE2(SE2&&) = default;

SE2& SE2::operator=(const SE2&) = default;

// NOLINTNEXTLINE
SE2& SE2::operator=(SE2&&) = default;

void SE2::toCsv(std::ostream& out) const
{
    out.precision(16);
    out << std::boolalpha;
    out << x;out << ",";
    out << y;out << ",";
    out << theta;
    
}

std::stringstream SE2::toStringstream() const
{
    std::stringstream out;
    toCsv(out);
    return out;
}

template<class Archive>
    void SE2::serialize(Archive& ar, [[maybe_unused]] unsigned int version)
    {

        ar& CEREAL_NVP(x);
        ar& CEREAL_NVP(y);
        ar& CEREAL_NVP(theta);
    }


template void SE2::serialize(cereal::BinaryInputArchive& ar, unsigned int version);
template void SE2::serialize(cereal::BinaryOutputArchive& ar, unsigned int version);
template void SE2::serialize(cereal::IntrospectionArchive& ar, unsigned int version);
template void SE2::serialize(cereal::JSONInputArchive& ar, unsigned int version);
template void SE2::serialize(cereal::JSONOutputArchive& ar, unsigned int version);
template void SE2::serialize(cereal::PortableBinaryInputArchive& ar, unsigned int version);
template void SE2::serialize(cereal::PortableBinaryOutputArchive& ar, unsigned int version);

std::ostream& operator<< (std::ostream& os, const SE2& se2)
{
    se2.toCsv(os);
    return os;
}

bool operator== (const SE2 &lhs, const SE2 &rhs)
{
    return (  lhs.x== rhs.x &&
     lhs.y== rhs.y &&
     lhs.theta== rhs.theta 
    );
}

std::size_t SE2::hash() const noexcept
{
    std::size_t result = 17;
    result = result * 31 + std::hash<double>()(x);
    result = result * 31 + std::hash<double>()(y);
    result = result * 31 + std::hash<double>()(theta);
    return result;
}

 } // namespace av
 } // namespace msg


CEREAL_BIND_TO_ARCHIVES(av::msg::SE2)


