#include <interfaces/SH1_Outputs_avt/SH1_Outputs_avt.h>

#include <iterator>
#include <type_traits>

#include <cereal/cereal.hpp>
#include <cereal/archives/json.hpp>
#include <cereal/archives/binary.hpp>

#include <cereal/types/map.hpp>
#include <cereal/types/vector.hpp>
#include <cereal/types/array.hpp>
#include <cereal/types/string.hpp>
#include <cereal/types/complex.hpp>

#include <ODatum/IntrospectionArchive.h>
#include <ODatum/Hash.h>





 namespace av {
 namespace msg {


SH1_Outputs_avt::SH1_Outputs_avt() = default;

SH1_Outputs_avt::SH1_Outputs_avt(
    uint8_t sts_BSMState_arg ,
    uint8_t sts_ECUStatusSH1_arg ,
    uint8_t B_DecelRequestSH1_arg ,
    uint8_t B_ThrottleOffSH1_arg ,
    uint8_t B_BrakePreChargeSH1_arg ,
    uint8_t ALIV_SH1_Outputs_arg ,
    uint8_t CRC_SH1_Outputs_arg 
    ):
    sts_BSMState { std::move(sts_BSMState_arg) } // NOLINT
      ,sts_ECUStatusSH1 { std::move(sts_ECUStatusSH1_arg) } // NOLINT
      ,B_DecelRequestSH1 { std::move(B_DecelRequestSH1_arg) } // NOLINT
      ,B_ThrottleOffSH1 { std::move(B_ThrottleOffSH1_arg) } // NOLINT
      ,B_BrakePreChargeSH1 { std::move(B_BrakePreChargeSH1_arg) } // NOLINT
      ,ALIV_SH1_Outputs { std::move(ALIV_SH1_Outputs_arg) } // NOLINT
      ,CRC_SH1_Outputs { std::move(CRC_SH1_Outputs_arg) } // NOLINT
      {
}

SH1_Outputs_avt::SH1_Outputs_avt(const SH1_Outputs_avt&) = default;

// NOLINTNEXTLINE
SH1_Outputs_avt::SH1_Outputs_avt(SH1_Outputs_avt&&) = default;

SH1_Outputs_avt& SH1_Outputs_avt::operator=(const SH1_Outputs_avt&) = default;

// NOLINTNEXTLINE
SH1_Outputs_avt& SH1_Outputs_avt::operator=(SH1_Outputs_avt&&) = default;

void SH1_Outputs_avt::toCsv(std::ostream& out) const
{
    out.precision(16);
    out << std::boolalpha;
    // Cast to uint16_t to print as integer instead of char
        out << static_cast<uint16_t>(sts_BSMState);out << ",";
    // Cast to uint16_t to print as integer instead of char
        out << static_cast<uint16_t>(sts_ECUStatusSH1);out << ",";
    // Cast to uint16_t to print as integer instead of char
        out << static_cast<uint16_t>(B_DecelRequestSH1);out << ",";
    // Cast to uint16_t to print as integer instead of char
        out << static_cast<uint16_t>(B_ThrottleOffSH1);out << ",";
    // Cast to uint16_t to print as integer instead of char
        out << static_cast<uint16_t>(B_BrakePreChargeSH1);out << ",";
    // Cast to uint16_t to print as integer instead of char
        out << static_cast<uint16_t>(ALIV_SH1_Outputs);out << ",";
    // Cast to uint16_t to print as integer instead of char
        out << static_cast<uint16_t>(CRC_SH1_Outputs);
    
}

std::stringstream SH1_Outputs_avt::toStringstream() const
{
    std::stringstream out;
    toCsv(out);
    return out;
}

template<class Archive>
    void SH1_Outputs_avt::serialize(Archive& ar, [[maybe_unused]] unsigned int version)
    {

        ar& CEREAL_NVP(sts_BSMState);
        ar& CEREAL_NVP(sts_ECUStatusSH1);
        ar& CEREAL_NVP(B_DecelRequestSH1);
        ar& CEREAL_NVP(B_ThrottleOffSH1);
        ar& CEREAL_NVP(B_BrakePreChargeSH1);
        ar& CEREAL_NVP(ALIV_SH1_Outputs);
        ar& CEREAL_NVP(CRC_SH1_Outputs);
    }


template void SH1_Outputs_avt::serialize(cereal::BinaryInputArchive& ar, unsigned int version);
template void SH1_Outputs_avt::serialize(cereal::BinaryOutputArchive& ar, unsigned int version);
template void SH1_Outputs_avt::serialize(cereal::IntrospectionArchive& ar, unsigned int version);
template void SH1_Outputs_avt::serialize(cereal::JSONInputArchive& ar, unsigned int version);
template void SH1_Outputs_avt::serialize(cereal::JSONOutputArchive& ar, unsigned int version);
template void SH1_Outputs_avt::serialize(cereal::PortableBinaryInputArchive& ar, unsigned int version);
template void SH1_Outputs_avt::serialize(cereal::PortableBinaryOutputArchive& ar, unsigned int version);

std::ostream& operator<< (std::ostream& os, const SH1_Outputs_avt& sh1_outputs_avt)
{
    sh1_outputs_avt.toCsv(os);
    return os;
}

bool operator== (const SH1_Outputs_avt &lhs, const SH1_Outputs_avt &rhs)
{
    return (  lhs.sts_BSMState== rhs.sts_BSMState &&
     lhs.sts_ECUStatusSH1== rhs.sts_ECUStatusSH1 &&
     lhs.B_DecelRequestSH1== rhs.B_DecelRequestSH1 &&
     lhs.B_ThrottleOffSH1== rhs.B_ThrottleOffSH1 &&
     lhs.B_BrakePreChargeSH1== rhs.B_BrakePreChargeSH1 &&
     lhs.ALIV_SH1_Outputs== rhs.ALIV_SH1_Outputs &&
     lhs.CRC_SH1_Outputs== rhs.CRC_SH1_Outputs 
    );
}

std::size_t SH1_Outputs_avt::hash() const noexcept
{
    std::size_t result = 17;
    result = result * 31 + std::hash<uint8_t>()(sts_BSMState);
    result = result * 31 + std::hash<uint8_t>()(sts_ECUStatusSH1);
    result = result * 31 + std::hash<uint8_t>()(B_DecelRequestSH1);
    result = result * 31 + std::hash<uint8_t>()(B_ThrottleOffSH1);
    result = result * 31 + std::hash<uint8_t>()(B_BrakePreChargeSH1);
    result = result * 31 + std::hash<uint8_t>()(ALIV_SH1_Outputs);
    result = result * 31 + std::hash<uint8_t>()(CRC_SH1_Outputs);
    return result;
}

 } // namespace av
 } // namespace msg


CEREAL_BIND_TO_ARCHIVES(av::msg::SH1_Outputs_avt)

