#include <interfaces/SafeStopControlPoint/SafeStopControlPoint.h>

#include <iterator>
#include <type_traits>

#include <cereal/cereal.hpp>
#include <cereal/archives/json.hpp>
#include <cereal/archives/binary.hpp>

#include <cereal/types/map.hpp>
#include <cereal/types/vector.hpp>
#include <cereal/types/array.hpp>
#include <cereal/types/string.hpp>
#include <cereal/types/complex.hpp>

#include <ODatum/IntrospectionArchive.h>
#include <ODatum/Hash.h>





 namespace av {
 namespace msg {


SafeStopControlPoint::SafeStopControlPoint() = default;

SafeStopControlPoint::SafeStopControlPoint(
    int64_t timeStamp_arg ,
    double crossError_arg ,
    double curvature_k_arg ,
    double headingError_arg 
    ):
    timeStamp { std::move(timeStamp_arg) } // NOLINT
      ,crossError { std::move(crossError_arg) } // NOLINT
      ,curvature_k { std::move(curvature_k_arg) } // NOLINT
      ,headingError { std::move(headingError_arg) } // NOLINT
      {
}

SafeStopControlPoint::SafeStopControlPoint(const SafeStopControlPoint&) = default;

// NOLINTNEXTLINE
SafeStopControlPoint::SafeStopControlPoint(SafeStopControlPoint&&) = default;

SafeStopControlPoint& SafeStopControlPoint::operator=(const SafeStopControlPoint&) = default;

// NOLINTNEXTLINE
SafeStopControlPoint& SafeStopControlPoint::operator=(SafeStopControlPoint&&) = default;

void SafeStopControlPoint::toCsv(std::ostream& out) const
{
    out.precision(16);
    out << std::boolalpha;
    out << timeStamp;out << ",";
    out << crossError;out << ",";
    out << curvature_k;out << ",";
    out << headingError;
    
}

std::stringstream SafeStopControlPoint::toStringstream() const
{
    std::stringstream out;
    toCsv(out);
    return out;
}

template<class Archive>
    void SafeStopControlPoint::serialize(Archive& ar, [[maybe_unused]] unsigned int version)
    {

        ar& CEREAL_NVP(timeStamp);
        ar& CEREAL_NVP(crossError);
        ar& CEREAL_NVP(curvature_k);
        ar& CEREAL_NVP(headingError);
    }


template void SafeStopControlPoint::serialize(cereal::BinaryInputArchive& ar, unsigned int version);
template void SafeStopControlPoint::serialize(cereal::BinaryOutputArchive& ar, unsigned int version);
template void SafeStopControlPoint::serialize(cereal::IntrospectionArchive& ar, unsigned int version);
template void SafeStopControlPoint::serialize(cereal::JSONInputArchive& ar, unsigned int version);
template void SafeStopControlPoint::serialize(cereal::JSONOutputArchive& ar, unsigned int version);
template void SafeStopControlPoint::serialize(cereal::PortableBinaryInputArchive& ar, unsigned int version);
template void SafeStopControlPoint::serialize(cereal::PortableBinaryOutputArchive& ar, unsigned int version);

std::ostream& operator<< (std::ostream& os, const SafeStopControlPoint& safestopcontrolpoint)
{
    safestopcontrolpoint.toCsv(os);
    return os;
}

bool operator== (const SafeStopControlPoint &lhs, const SafeStopControlPoint &rhs)
{
    return (  lhs.timeStamp== rhs.timeStamp &&
     lhs.crossError== rhs.crossError &&
     lhs.curvature_k== rhs.curvature_k &&
     lhs.headingError== rhs.headingError 
    );
}

std::size_t SafeStopControlPoint::hash() const noexcept
{
    std::size_t result = 17;
    result = result * 31 + std::hash<int64_t>()(timeStamp);
    result = result * 31 + std::hash<double>()(crossError);
    result = result * 31 + std::hash<double>()(curvature_k);
    result = result * 31 + std::hash<double>()(headingError);
    return result;
}

 } // namespace av
 } // namespace msg


CEREAL_BIND_TO_ARCHIVES(av::msg::SafeStopControlPoint)

