#include <interfaces/SaferCommand/SaferCommand.h>

#include <iterator>
#include <type_traits>

#include <cereal/cereal.hpp>
#include <cereal/archives/json.hpp>
#include <cereal/archives/binary.hpp>

#include <cereal/types/map.hpp>
#include <cereal/types/vector.hpp>
#include <cereal/types/array.hpp>
#include <cereal/types/string.hpp>
#include <cereal/types/complex.hpp>

#include <ODatum/IntrospectionArchive.h>
#include <ODatum/Hash.h>





 namespace av {
 namespace msg {


SaferCommand::SaferCommand() = default;

SaferCommand::SaferCommand(
    MessageId messageType__arg ,
    std::string sender__arg ,
    std::string message__arg ,
    std::string recipient__arg ,
    std::string launchTime__arg ,
    std::string taskName__arg 
    ):
    messageType_ { std::move(messageType__arg) } // NOLINT
      ,sender_ { std::move(sender__arg) } // NOLINT
      ,message_ { std::move(message__arg) } // NOLINT
      ,recipient_ { std::move(recipient__arg) } // NOLINT
      ,launchTime_ { std::move(launchTime__arg) } // NOLINT
      ,taskName_ { std::move(taskName__arg) } // NOLINT
      {
}

SaferCommand::SaferCommand(const SaferCommand&) = default;

// NOLINTNEXTLINE
SaferCommand::SaferCommand(SaferCommand&&) = default;

SaferCommand& SaferCommand::operator=(const SaferCommand&) = default;

// NOLINTNEXTLINE
SaferCommand& SaferCommand::operator=(SaferCommand&&) = default;

void SaferCommand::toCsv(std::ostream& out) const
{
    out.precision(16);
    out << std::boolalpha;
    out << static_cast<uint32_t>(messageType_);out << ",";
    out << "\"";
        out << sender_;
        out << "\"";out << ",";
    out << "\"";
        out << message_;
        out << "\"";out << ",";
    out << "\"";
        out << recipient_;
        out << "\"";out << ",";
    out << "\"";
        out << launchTime_;
        out << "\"";out << ",";
    out << "\"";
        out << taskName_;
        out << "\"";
    
}

std::stringstream SaferCommand::toStringstream() const
{
    std::stringstream out;
    toCsv(out);
    return out;
}

template<class Archive>
    void SaferCommand::serialize(Archive& ar, [[maybe_unused]] unsigned int version)
    {

        ar& CEREAL_NVP(messageType_);
        ar& CEREAL_NVP(sender_);
        ar& CEREAL_NVP(message_);
        ar& CEREAL_NVP(recipient_);
        ar& CEREAL_NVP(launchTime_);
        ar& CEREAL_NVP(taskName_);
    }


template void SaferCommand::serialize(cereal::BinaryInputArchive& ar, unsigned int version);
template void SaferCommand::serialize(cereal::BinaryOutputArchive& ar, unsigned int version);
template void SaferCommand::serialize(cereal::IntrospectionArchive& ar, unsigned int version);
template void SaferCommand::serialize(cereal::JSONInputArchive& ar, unsigned int version);
template void SaferCommand::serialize(cereal::JSONOutputArchive& ar, unsigned int version);
template void SaferCommand::serialize(cereal::PortableBinaryInputArchive& ar, unsigned int version);
template void SaferCommand::serialize(cereal::PortableBinaryOutputArchive& ar, unsigned int version);

std::ostream& operator<< (std::ostream& os, const SaferCommand& safercommand)
{
    safercommand.toCsv(os);
    return os;
}

bool operator== (const SaferCommand &lhs, const SaferCommand &rhs)
{
    return (  lhs.messageType_== rhs.messageType_ &&
     lhs.sender_== rhs.sender_ &&
     lhs.message_== rhs.message_ &&
     lhs.recipient_== rhs.recipient_ &&
     lhs.launchTime_== rhs.launchTime_ &&
     lhs.taskName_== rhs.taskName_ 
    );
}

std::size_t SaferCommand::hash() const noexcept
{
    std::size_t result = 17;
    result = result * 31 + std::hash<MessageId>()(messageType_);
    result = result * 31 + std::hash<std::string>()(sender_);
    result = result * 31 + std::hash<std::string>()(message_);
    result = result * 31 + std::hash<std::string>()(recipient_);
    result = result * 31 + std::hash<std::string>()(launchTime_);
    result = result * 31 + std::hash<std::string>()(taskName_);
    return result;
}

 } // namespace av
 } // namespace msg


CEREAL_BIND_TO_ARCHIVES(av::msg::SaferCommand)

