#include <interfaces/SaferResponse/SaferResponse.h>

#include <iterator>
#include <type_traits>

#include <cereal/cereal.hpp>
#include <cereal/archives/json.hpp>
#include <cereal/archives/binary.hpp>

#include <cereal/types/map.hpp>
#include <cereal/types/vector.hpp>
#include <cereal/types/array.hpp>
#include <cereal/types/string.hpp>
#include <cereal/types/complex.hpp>

#include <ODatum/IntrospectionArchive.h>
#include <ODatum/Hash.h>





 namespace av {
 namespace msg {


SaferResponse::SaferResponse() = default;

SaferResponse::SaferResponse(
    MessageId messageType__arg ,
    std::string sender__arg ,
    std::string message__arg ,
    std::string recipient__arg ,
    std::string instanceName__arg ,
    SaferResponseId responseType__arg 
    ):
    messageType_ { std::move(messageType__arg) } // NOLINT
      ,sender_ { std::move(sender__arg) } // NOLINT
      ,message_ { std::move(message__arg) } // NOLINT
      ,recipient_ { std::move(recipient__arg) } // NOLINT
      ,instanceName_ { std::move(instanceName__arg) } // NOLINT
      ,responseType_ { std::move(responseType__arg) } // NOLINT
      {
}

SaferResponse::SaferResponse(const SaferResponse&) = default;

// NOLINTNEXTLINE
SaferResponse::SaferResponse(SaferResponse&&) = default;

SaferResponse& SaferResponse::operator=(const SaferResponse&) = default;

// NOLINTNEXTLINE
SaferResponse& SaferResponse::operator=(SaferResponse&&) = default;

void SaferResponse::toCsv(std::ostream& out) const
{
    out.precision(16);
    out << std::boolalpha;
    out << static_cast<uint32_t>(messageType_);out << ",";
    out << "\"";
        out << sender_;
        out << "\"";out << ",";
    out << "\"";
        out << message_;
        out << "\"";out << ",";
    out << "\"";
        out << recipient_;
        out << "\"";out << ",";
    out << "\"";
        out << instanceName_;
        out << "\"";out << ",";
    out << static_cast<uint32_t>(responseType_);
    
}

std::stringstream SaferResponse::toStringstream() const
{
    std::stringstream out;
    toCsv(out);
    return out;
}

template<class Archive>
    void SaferResponse::serialize(Archive& ar, [[maybe_unused]] unsigned int version)
    {

        ar& CEREAL_NVP(messageType_);
        ar& CEREAL_NVP(sender_);
        ar& CEREAL_NVP(message_);
        ar& CEREAL_NVP(recipient_);
        ar& CEREAL_NVP(instanceName_);
        ar& CEREAL_NVP(responseType_);
    }


template void SaferResponse::serialize(cereal::BinaryInputArchive& ar, unsigned int version);
template void SaferResponse::serialize(cereal::BinaryOutputArchive& ar, unsigned int version);
template void SaferResponse::serialize(cereal::IntrospectionArchive& ar, unsigned int version);
template void SaferResponse::serialize(cereal::JSONInputArchive& ar, unsigned int version);
template void SaferResponse::serialize(cereal::JSONOutputArchive& ar, unsigned int version);
template void SaferResponse::serialize(cereal::PortableBinaryInputArchive& ar, unsigned int version);
template void SaferResponse::serialize(cereal::PortableBinaryOutputArchive& ar, unsigned int version);

std::ostream& operator<< (std::ostream& os, const SaferResponse& saferresponse)
{
    saferresponse.toCsv(os);
    return os;
}

bool operator== (const SaferResponse &lhs, const SaferResponse &rhs)
{
    return (  lhs.messageType_== rhs.messageType_ &&
     lhs.sender_== rhs.sender_ &&
     lhs.message_== rhs.message_ &&
     lhs.recipient_== rhs.recipient_ &&
     lhs.instanceName_== rhs.instanceName_ &&
     lhs.responseType_== rhs.responseType_ 
    );
}

std::size_t SaferResponse::hash() const noexcept
{
    std::size_t result = 17;
    result = result * 31 + std::hash<MessageId>()(messageType_);
    result = result * 31 + std::hash<std::string>()(sender_);
    result = result * 31 + std::hash<std::string>()(message_);
    result = result * 31 + std::hash<std::string>()(recipient_);
    result = result * 31 + std::hash<std::string>()(instanceName_);
    result = result * 31 + std::hash<SaferResponseId>()(responseType_);
    return result;
}

 } // namespace av
 } // namespace msg


CEREAL_BIND_TO_ARCHIVES(av::msg::SaferResponse)

