#include <interfaces/SampleMessage/SampleMessage.h>

#include <iterator>
#include <type_traits>

#include <cereal/cereal.hpp>
#include <cereal/archives/json.hpp>
#include <cereal/archives/binary.hpp>

#include <cereal/types/map.hpp>
#include <cereal/types/vector.hpp>
#include <cereal/types/array.hpp>
#include <cereal/types/string.hpp>
#include <cereal/types/complex.hpp>

#include <ODatum/IntrospectionArchive.h>
#include <ODatum/Hash.h>





 namespace av {
 namespace msg {


SampleMessage::SampleMessage() = default;

SampleMessage::SampleMessage(
    int value_arg 
    ):
    value { std::move(value_arg) } // NOLINT
      {
}

SampleMessage::SampleMessage(const SampleMessage&) = default;

// NOLINTNEXTLINE
SampleMessage::SampleMessage(SampleMessage&&) = default;

SampleMessage& SampleMessage::operator=(const SampleMessage&) = default;

// NOLINTNEXTLINE
SampleMessage& SampleMessage::operator=(SampleMessage&&) = default;

void SampleMessage::toCsv(std::ostream& out) const
{
    out.precision(16);
    out << std::boolalpha;
    out << value;
    
}

std::stringstream SampleMessage::toStringstream() const
{
    std::stringstream out;
    toCsv(out);
    return out;
}

template<class Archive>
    void SampleMessage::serialize(Archive& ar, [[maybe_unused]] unsigned int version)
    {

        ar& CEREAL_NVP(value);
    }


template void SampleMessage::serialize(cereal::BinaryInputArchive& ar, unsigned int version);
template void SampleMessage::serialize(cereal::BinaryOutputArchive& ar, unsigned int version);
template void SampleMessage::serialize(cereal::IntrospectionArchive& ar, unsigned int version);
template void SampleMessage::serialize(cereal::JSONInputArchive& ar, unsigned int version);
template void SampleMessage::serialize(cereal::JSONOutputArchive& ar, unsigned int version);
template void SampleMessage::serialize(cereal::PortableBinaryInputArchive& ar, unsigned int version);
template void SampleMessage::serialize(cereal::PortableBinaryOutputArchive& ar, unsigned int version);

std::ostream& operator<< (std::ostream& os, const SampleMessage& samplemessage)
{
    samplemessage.toCsv(os);
    return os;
}

bool operator== (const SampleMessage &lhs, const SampleMessage &rhs)
{
    return (  lhs.value== rhs.value 
    );
}

std::size_t SampleMessage::hash() const noexcept
{
    std::size_t result = 17;
    result = result * 31 + std::hash<int>()(value);
    return result;
}

 } // namespace av
 } // namespace msg


CEREAL_BIND_TO_ARCHIVES(av::msg::SampleMessage)

