#include <interfaces/ScsRecordingCommand/ScsRecordingCommand.h>

#include <iterator>
#include <type_traits>

#include <cereal/cereal.hpp>
#include <cereal/archives/json.hpp>
#include <cereal/archives/binary.hpp>

#include <cereal/types/map.hpp>
#include <cereal/types/vector.hpp>
#include <cereal/types/array.hpp>
#include <cereal/types/string.hpp>
#include <cereal/types/complex.hpp>

#include <ODatum/IntrospectionArchive.h>
#include <ODatum/Hash.h>





 namespace av {
 namespace msg {


ScsRecordingCommand::ScsRecordingCommand() = default;

ScsRecordingCommand::ScsRecordingCommand(
    ScsRecordingCommandType command_arg ,
    std::string logName_arg 
    ):
    command { std::move(command_arg) } // NOLINT
      ,logName { std::move(logName_arg) } // NOLINT
      {
}

ScsRecordingCommand::ScsRecordingCommand(const ScsRecordingCommand&) = default;

// NOLINTNEXTLINE
ScsRecordingCommand::ScsRecordingCommand(ScsRecordingCommand&&) = default;

ScsRecordingCommand& ScsRecordingCommand::operator=(const ScsRecordingCommand&) = default;

// NOLINTNEXTLINE
ScsRecordingCommand& ScsRecordingCommand::operator=(ScsRecordingCommand&&) = default;

void ScsRecordingCommand::toCsv(std::ostream& out) const
{
    out.precision(16);
    out << std::boolalpha;
    out << static_cast<uint32_t>(command);out << ",";
    out << "\"";
        out << logName;
        out << "\"";
    
}

std::stringstream ScsRecordingCommand::toStringstream() const
{
    std::stringstream out;
    toCsv(out);
    return out;
}

template<class Archive>
    void ScsRecordingCommand::serialize(Archive& ar, [[maybe_unused]] unsigned int version)
    {

        ar& CEREAL_NVP(command);
        ar& CEREAL_NVP(logName);
    }


template void ScsRecordingCommand::serialize(cereal::BinaryInputArchive& ar, unsigned int version);
template void ScsRecordingCommand::serialize(cereal::BinaryOutputArchive& ar, unsigned int version);
template void ScsRecordingCommand::serialize(cereal::IntrospectionArchive& ar, unsigned int version);
template void ScsRecordingCommand::serialize(cereal::JSONInputArchive& ar, unsigned int version);
template void ScsRecordingCommand::serialize(cereal::JSONOutputArchive& ar, unsigned int version);
template void ScsRecordingCommand::serialize(cereal::PortableBinaryInputArchive& ar, unsigned int version);
template void ScsRecordingCommand::serialize(cereal::PortableBinaryOutputArchive& ar, unsigned int version);

std::ostream& operator<< (std::ostream& os, const ScsRecordingCommand& scsrecordingcommand)
{
    scsrecordingcommand.toCsv(os);
    return os;
}

bool operator== (const ScsRecordingCommand &lhs, const ScsRecordingCommand &rhs)
{
    return (  lhs.command== rhs.command &&
     lhs.logName== rhs.logName 
    );
}

std::size_t ScsRecordingCommand::hash() const noexcept
{
    std::size_t result = 17;
    result = result * 31 + std::hash<ScsRecordingCommandType>()(command);
    result = result * 31 + std::hash<std::string>()(logName);
    return result;
}

 } // namespace av
 } // namespace msg


CEREAL_BIND_TO_ARCHIVES(av::msg::ScsRecordingCommand)

