#include <interfaces/SelectedRealizationCache/SelectedRealizationCache.h>

#include <iterator>
#include <type_traits>

#include <cereal/cereal.hpp>
#include <cereal/archives/json.hpp>
#include <cereal/archives/binary.hpp>

#include <cereal/types/map.hpp>
#include <cereal/types/vector.hpp>
#include <cereal/types/array.hpp>
#include <cereal/types/string.hpp>
#include <cereal/types/complex.hpp>

#include <ODatum/IntrospectionArchive.h>
#include <ODatum/Hash.h>


 namespace av {
 namespace msg {


LastSelectedRealization::LastSelectedRealization() = default;

LastSelectedRealization::LastSelectedRealization(
    PathPointMassModelTrajectory realization_arg ,
    TimePoint timestamp_arg ,
    bool isBlocked_arg 
    ):
    realization { std::move(realization_arg) } // NOLINT
      ,
    timestamp { std::move(timestamp_arg) } // NOLINT
      ,
    isBlocked { std::move(isBlocked_arg) } // NOLINT
      
    {
}

LastSelectedRealization::LastSelectedRealization(const LastSelectedRealization&) = default;

// NOLINTNEXTLINE
LastSelectedRealization::LastSelectedRealization(LastSelectedRealization&&) = default;

LastSelectedRealization& LastSelectedRealization::operator=(const LastSelectedRealization&) = default;

// NOLINTNEXTLINE
LastSelectedRealization& LastSelectedRealization::operator=(LastSelectedRealization&&) = default;

void LastSelectedRealization::toCsv(std::ostream& out) const
{
    out.precision(16);
    out << std::boolalpha;
    out << realization;out << ",";
    out << timestamp;out << ",";
    out << isBlocked;
    
}

std::stringstream LastSelectedRealization::toStringstream() const
{
    std::stringstream out;
    toCsv(out);
    return out;
}

template<class Archive>
    void LastSelectedRealization::serialize(Archive& ar, [[maybe_unused]] unsigned int version)
    {

        ar& CEREAL_NVP(realization);
        ar& CEREAL_NVP(timestamp);
        ar& CEREAL_NVP(isBlocked);
    }


template void LastSelectedRealization::serialize(cereal::BinaryInputArchive& ar, unsigned int version);
template void LastSelectedRealization::serialize(cereal::BinaryOutputArchive& ar, unsigned int version);
template void LastSelectedRealization::serialize(cereal::IntrospectionArchive& ar, unsigned int version);
template void LastSelectedRealization::serialize(cereal::JSONInputArchive& ar, unsigned int version);
template void LastSelectedRealization::serialize(cereal::JSONOutputArchive& ar, unsigned int version);
template void LastSelectedRealization::serialize(cereal::PortableBinaryInputArchive& ar, unsigned int version);
template void LastSelectedRealization::serialize(cereal::PortableBinaryOutputArchive& ar, unsigned int version);

std::ostream& operator<< (std::ostream& os, const LastSelectedRealization& lastselectedrealization)
{
    lastselectedrealization.toCsv(os);
    return os;
}

bool operator== (const LastSelectedRealization &lhs, const LastSelectedRealization &rhs)
{
    return (  lhs.realization== rhs.realization &&
     lhs.timestamp== rhs.timestamp &&
     lhs.isBlocked== rhs.isBlocked 
    );
}

std::size_t LastSelectedRealization::hash() const noexcept
{
    std::size_t result = 17;
    result = result * 31 + std::hash<PathPointMassModelTrajectory>()(realization);
    result = result * 31 + 0;
    result = result * 31 + std::hash<bool>()(isBlocked);
    return result;
}

 } // namespace av
 } // namespace msg


CEREAL_BIND_TO_ARCHIVES(av::msg::LastSelectedRealization)

 namespace av {
 namespace msg {


LastSelectedRealizationCache::LastSelectedRealizationCache() = default;

LastSelectedRealizationCache::LastSelectedRealizationCache(
    bool hasLaneKeepRealization_arg ,
    LastSelectedRealization laneKeepRealization_arg ,
    bool hasLeftLaneChangeRealization_arg ,
    LastSelectedRealization leftLaneChangeRealization_arg ,
    bool hasRightLaneChangeRealization_arg ,
    LastSelectedRealization rightLaneChangeRealization_arg 
    ):
    hasLaneKeepRealization { std::move(hasLaneKeepRealization_arg) } // NOLINT
      ,
    laneKeepRealization { std::move(laneKeepRealization_arg) } // NOLINT
      ,
    hasLeftLaneChangeRealization { std::move(hasLeftLaneChangeRealization_arg) } // NOLINT
      ,
    leftLaneChangeRealization { std::move(leftLaneChangeRealization_arg) } // NOLINT
      ,
    hasRightLaneChangeRealization { std::move(hasRightLaneChangeRealization_arg) } // NOLINT
      ,
    rightLaneChangeRealization { std::move(rightLaneChangeRealization_arg) } // NOLINT
      
    {
}

LastSelectedRealizationCache::LastSelectedRealizationCache(const LastSelectedRealizationCache&) = default;

// NOLINTNEXTLINE
LastSelectedRealizationCache::LastSelectedRealizationCache(LastSelectedRealizationCache&&) = default;

LastSelectedRealizationCache& LastSelectedRealizationCache::operator=(const LastSelectedRealizationCache&) = default;

// NOLINTNEXTLINE
LastSelectedRealizationCache& LastSelectedRealizationCache::operator=(LastSelectedRealizationCache&&) = default;

void LastSelectedRealizationCache::toCsv(std::ostream& out) const
{
    out.precision(16);
    out << std::boolalpha;
    out << hasLaneKeepRealization;out << ",";
    out << laneKeepRealization;out << ",";
    out << hasLeftLaneChangeRealization;out << ",";
    out << leftLaneChangeRealization;out << ",";
    out << hasRightLaneChangeRealization;out << ",";
    out << rightLaneChangeRealization;
    
}

std::stringstream LastSelectedRealizationCache::toStringstream() const
{
    std::stringstream out;
    toCsv(out);
    return out;
}

template<class Archive>
    void LastSelectedRealizationCache::serialize(Archive& ar, [[maybe_unused]] unsigned int version)
    {

        ar& CEREAL_NVP(hasLaneKeepRealization);
        ar& CEREAL_NVP(laneKeepRealization);
        ar& CEREAL_NVP(hasLeftLaneChangeRealization);
        ar& CEREAL_NVP(leftLaneChangeRealization);
        ar& CEREAL_NVP(hasRightLaneChangeRealization);
        ar& CEREAL_NVP(rightLaneChangeRealization);
    }


template void LastSelectedRealizationCache::serialize(cereal::BinaryInputArchive& ar, unsigned int version);
template void LastSelectedRealizationCache::serialize(cereal::BinaryOutputArchive& ar, unsigned int version);
template void LastSelectedRealizationCache::serialize(cereal::IntrospectionArchive& ar, unsigned int version);
template void LastSelectedRealizationCache::serialize(cereal::JSONInputArchive& ar, unsigned int version);
template void LastSelectedRealizationCache::serialize(cereal::JSONOutputArchive& ar, unsigned int version);
template void LastSelectedRealizationCache::serialize(cereal::PortableBinaryInputArchive& ar, unsigned int version);
template void LastSelectedRealizationCache::serialize(cereal::PortableBinaryOutputArchive& ar, unsigned int version);

std::ostream& operator<< (std::ostream& os, const LastSelectedRealizationCache& lastselectedrealizationcache)
{
    lastselectedrealizationcache.toCsv(os);
    return os;
}

bool operator== (const LastSelectedRealizationCache &lhs, const LastSelectedRealizationCache &rhs)
{
    return (  lhs.hasLaneKeepRealization== rhs.hasLaneKeepRealization &&
     lhs.laneKeepRealization== rhs.laneKeepRealization &&
     lhs.hasLeftLaneChangeRealization== rhs.hasLeftLaneChangeRealization &&
     lhs.leftLaneChangeRealization== rhs.leftLaneChangeRealization &&
     lhs.hasRightLaneChangeRealization== rhs.hasRightLaneChangeRealization &&
     lhs.rightLaneChangeRealization== rhs.rightLaneChangeRealization 
    );
}

std::size_t LastSelectedRealizationCache::hash() const noexcept
{
    std::size_t result = 17;
    result = result * 31 + std::hash<bool>()(hasLaneKeepRealization);
    result = result * 31 + std::hash<LastSelectedRealization>()(laneKeepRealization);
    result = result * 31 + std::hash<bool>()(hasLeftLaneChangeRealization);
    result = result * 31 + std::hash<LastSelectedRealization>()(leftLaneChangeRealization);
    result = result * 31 + std::hash<bool>()(hasRightLaneChangeRealization);
    result = result * 31 + std::hash<LastSelectedRealization>()(rightLaneChangeRealization);
    return result;
}

 } // namespace av
 } // namespace msg


CEREAL_BIND_TO_ARCHIVES(av::msg::LastSelectedRealizationCache)


