#include <interfaces/SensorAnomalyStatusList/SensorAnomalyStatusList.h>

#include <iterator>
#include <type_traits>

#include <cereal/cereal.hpp>
#include <cereal/archives/json.hpp>
#include <cereal/archives/binary.hpp>

#include <cereal/types/map.hpp>
#include <cereal/types/vector.hpp>
#include <cereal/types/array.hpp>
#include <cereal/types/string.hpp>
#include <cereal/types/complex.hpp>

#include <ODatum/IntrospectionArchive.h>
#include <ODatum/Hash.h>





 namespace av {
 namespace msg {


SensorAnomalyStatusList::SensorAnomalyStatusList() = default;

SensorAnomalyStatusList::SensorAnomalyStatusList(
    av::msg::TimePoint timeStamp_arg ,
    std::vector<av::msg::BlockageStatus> lidarBlockageStatus_arg ,
    std::vector<av::msg::BlockageStatus> cameraBlockageStatus_arg 
    ):
    timeStamp { std::move(timeStamp_arg) } // NOLINT
      ,lidarBlockageStatus { std::move(lidarBlockageStatus_arg) } // NOLINT
      ,cameraBlockageStatus { std::move(cameraBlockageStatus_arg) } // NOLINT
      {
}

SensorAnomalyStatusList::SensorAnomalyStatusList(const SensorAnomalyStatusList&) = default;

// NOLINTNEXTLINE
SensorAnomalyStatusList::SensorAnomalyStatusList(SensorAnomalyStatusList&&) = default;

SensorAnomalyStatusList& SensorAnomalyStatusList::operator=(const SensorAnomalyStatusList&) = default;

// NOLINTNEXTLINE
SensorAnomalyStatusList& SensorAnomalyStatusList::operator=(SensorAnomalyStatusList&&) = default;

void SensorAnomalyStatusList::toCsv(std::ostream& out) const
{
    out.precision(16);
    out << std::boolalpha;
    out << timeStamp;out << ",";
    // Enclose sequence in double quotations and curly brackets to keep CSV line length uniform
        
        out << "\"{{";
        
        if (!lidarBlockageStatus.empty())
        {
            std::copy(lidarBlockageStatus.begin(), lidarBlockageStatus.end() - 1, std::ostream_iterator<av::msg::BlockageStatus>(out, "},{"));
            std::copy(lidarBlockageStatus.end()-1, lidarBlockageStatus.end(), std::ostream_iterator<av::msg::BlockageStatus>(out, ""));
        }
        out << "}}\"";out << ",";
    // Enclose sequence in double quotations and curly brackets to keep CSV line length uniform
        
        out << "\"{{";
        
        if (!cameraBlockageStatus.empty())
        {
            std::copy(cameraBlockageStatus.begin(), cameraBlockageStatus.end() - 1, std::ostream_iterator<av::msg::BlockageStatus>(out, "},{"));
            std::copy(cameraBlockageStatus.end()-1, cameraBlockageStatus.end(), std::ostream_iterator<av::msg::BlockageStatus>(out, ""));
        }
        out << "}}\"";
    
}

std::stringstream SensorAnomalyStatusList::toStringstream() const
{
    std::stringstream out;
    toCsv(out);
    return out;
}

template<class Archive>
    void SensorAnomalyStatusList::serialize(Archive& ar, [[maybe_unused]] unsigned int version)
    {

        ar& CEREAL_NVP(timeStamp);
        ar& CEREAL_NVP(lidarBlockageStatus);
        ar& CEREAL_NVP(cameraBlockageStatus);
    }


template void SensorAnomalyStatusList::serialize(cereal::BinaryInputArchive& ar, unsigned int version);
template void SensorAnomalyStatusList::serialize(cereal::BinaryOutputArchive& ar, unsigned int version);
template void SensorAnomalyStatusList::serialize(cereal::IntrospectionArchive& ar, unsigned int version);
template void SensorAnomalyStatusList::serialize(cereal::JSONInputArchive& ar, unsigned int version);
template void SensorAnomalyStatusList::serialize(cereal::JSONOutputArchive& ar, unsigned int version);
template void SensorAnomalyStatusList::serialize(cereal::PortableBinaryInputArchive& ar, unsigned int version);
template void SensorAnomalyStatusList::serialize(cereal::PortableBinaryOutputArchive& ar, unsigned int version);

std::ostream& operator<< (std::ostream& os, const SensorAnomalyStatusList& sensoranomalystatuslist)
{
    sensoranomalystatuslist.toCsv(os);
    return os;
}

bool operator== (const SensorAnomalyStatusList &lhs, const SensorAnomalyStatusList &rhs)
{
    return (  lhs.timeStamp== rhs.timeStamp &&
     lhs.lidarBlockageStatus== rhs.lidarBlockageStatus &&
     lhs.cameraBlockageStatus== rhs.cameraBlockageStatus 
    );
}

std::size_t SensorAnomalyStatusList::hash() const noexcept
{
    std::size_t result = 17;
    result = result * 31 + std::hash<av::msg::TimePoint>()(timeStamp);
    result = result * 31 + hashVector(lidarBlockageStatus);
    result = result * 31 + hashVector(cameraBlockageStatus);
    return result;
}

 } // namespace av
 } // namespace msg


CEREAL_BIND_TO_ARCHIVES(av::msg::SensorAnomalyStatusList)

