#include <interfaces/SensorCleaningFeedback/SensorCleaningFeedback.h>

#include <iterator>
#include <type_traits>

#include <cereal/cereal.hpp>
#include <cereal/archives/json.hpp>
#include <cereal/archives/binary.hpp>

#include <cereal/types/map.hpp>
#include <cereal/types/vector.hpp>
#include <cereal/types/array.hpp>
#include <cereal/types/string.hpp>
#include <cereal/types/complex.hpp>

#include <ODatum/IntrospectionArchive.h>
#include <ODatum/Hash.h>


 namespace av {
 namespace msg {
 namespace scm {


CleaningStatus::CleaningStatus() = default;

CleaningStatus::CleaningStatus(
    std::string sensorId_arg ,
    CleaningType cleaningType_arg ,
    CleaningState cleaningState_arg ,
    uint16_t repetitionCnt_arg 
    ):
    sensorId { std::move(sensorId_arg) } // NOLINT
      ,
    cleaningType { std::move(cleaningType_arg) } // NOLINT
      ,
    cleaningState { std::move(cleaningState_arg) } // NOLINT
      ,
    repetitionCnt { std::move(repetitionCnt_arg) } // NOLINT
      
    {
}

CleaningStatus::CleaningStatus(const CleaningStatus&) = default;

// NOLINTNEXTLINE
CleaningStatus::CleaningStatus(CleaningStatus&&) = default;

CleaningStatus& CleaningStatus::operator=(const CleaningStatus&) = default;

// NOLINTNEXTLINE
CleaningStatus& CleaningStatus::operator=(CleaningStatus&&) = default;

void CleaningStatus::toCsv(std::ostream& out) const
{
    out.precision(16);
    out << std::boolalpha;
    out << "\"";
        out << sensorId;
        out << "\"";out << ",";
    out << cleaningType;out << ",";
    out << static_cast<uint32_t>(cleaningState);out << ",";
    out << repetitionCnt;
    
}

std::stringstream CleaningStatus::toStringstream() const
{
    std::stringstream out;
    toCsv(out);
    return out;
}

template<class Archive>
    void CleaningStatus::serialize(Archive& ar, [[maybe_unused]] unsigned int version)
    {

        ar& CEREAL_NVP(sensorId);
        ar& CEREAL_NVP(cleaningType);
        ar& CEREAL_NVP(cleaningState);
        ar& CEREAL_NVP(repetitionCnt);
    }


template void CleaningStatus::serialize(cereal::BinaryInputArchive& ar, unsigned int version);
template void CleaningStatus::serialize(cereal::BinaryOutputArchive& ar, unsigned int version);
template void CleaningStatus::serialize(cereal::IntrospectionArchive& ar, unsigned int version);
template void CleaningStatus::serialize(cereal::JSONInputArchive& ar, unsigned int version);
template void CleaningStatus::serialize(cereal::JSONOutputArchive& ar, unsigned int version);
template void CleaningStatus::serialize(cereal::PortableBinaryInputArchive& ar, unsigned int version);
template void CleaningStatus::serialize(cereal::PortableBinaryOutputArchive& ar, unsigned int version);

std::ostream& operator<< (std::ostream& os, const CleaningStatus& cleaningstatus)
{
    cleaningstatus.toCsv(os);
    return os;
}

bool operator== (const CleaningStatus &lhs, const CleaningStatus &rhs)
{
    return (  lhs.sensorId== rhs.sensorId &&
     lhs.cleaningType== rhs.cleaningType &&
     lhs.cleaningState== rhs.cleaningState &&
     lhs.repetitionCnt== rhs.repetitionCnt 
    );
}

std::size_t CleaningStatus::hash() const noexcept
{
    std::size_t result = 17;
    result = result * 31 + std::hash<std::string>()(sensorId);
    result = result * 31 + std::hash<CleaningType>()(cleaningType);
    result = result * 31 + std::hash<CleaningState>()(cleaningState);
    result = result * 31 + std::hash<uint16_t>()(repetitionCnt);
    return result;
}

 } // namespace av
 } // namespace msg
 } // namespace scm


CEREAL_BIND_TO_ARCHIVES(av::msg::scm::CleaningStatus)




 namespace av {
 namespace msg {
 namespace scm {


SensorCleaningFeedback::SensorCleaningFeedback() = default;

SensorCleaningFeedback::SensorCleaningFeedback(
    TimePoint timestamp_arg ,
    std::vector<CleaningStatus> cleaningStatus_arg 
    ):
    timestamp { std::move(timestamp_arg) } // NOLINT
      ,cleaningStatus { std::move(cleaningStatus_arg) } // NOLINT
      {
}

SensorCleaningFeedback::SensorCleaningFeedback(const SensorCleaningFeedback&) = default;

// NOLINTNEXTLINE
SensorCleaningFeedback::SensorCleaningFeedback(SensorCleaningFeedback&&) = default;

SensorCleaningFeedback& SensorCleaningFeedback::operator=(const SensorCleaningFeedback&) = default;

// NOLINTNEXTLINE
SensorCleaningFeedback& SensorCleaningFeedback::operator=(SensorCleaningFeedback&&) = default;

void SensorCleaningFeedback::toCsv(std::ostream& out) const
{
    out.precision(16);
    out << std::boolalpha;
    out << timestamp;out << ",";
    // Enclose sequence in double quotations and curly brackets to keep CSV line length uniform
        
        out << "\"{{";
        
        if (!cleaningStatus.empty())
        {
            std::copy(cleaningStatus.begin(), cleaningStatus.end() - 1, std::ostream_iterator<CleaningStatus>(out, "},{"));
            std::copy(cleaningStatus.end()-1, cleaningStatus.end(), std::ostream_iterator<CleaningStatus>(out, ""));
        }
        out << "}}\"";
    
}

std::stringstream SensorCleaningFeedback::toStringstream() const
{
    std::stringstream out;
    toCsv(out);
    return out;
}

template<class Archive>
    void SensorCleaningFeedback::serialize(Archive& ar, [[maybe_unused]] unsigned int version)
    {

        ar& CEREAL_NVP(timestamp);
        ar& CEREAL_NVP(cleaningStatus);
    }


template void SensorCleaningFeedback::serialize(cereal::BinaryInputArchive& ar, unsigned int version);
template void SensorCleaningFeedback::serialize(cereal::BinaryOutputArchive& ar, unsigned int version);
template void SensorCleaningFeedback::serialize(cereal::IntrospectionArchive& ar, unsigned int version);
template void SensorCleaningFeedback::serialize(cereal::JSONInputArchive& ar, unsigned int version);
template void SensorCleaningFeedback::serialize(cereal::JSONOutputArchive& ar, unsigned int version);
template void SensorCleaningFeedback::serialize(cereal::PortableBinaryInputArchive& ar, unsigned int version);
template void SensorCleaningFeedback::serialize(cereal::PortableBinaryOutputArchive& ar, unsigned int version);

std::ostream& operator<< (std::ostream& os, const SensorCleaningFeedback& sensorcleaningfeedback)
{
    sensorcleaningfeedback.toCsv(os);
    return os;
}

bool operator== (const SensorCleaningFeedback &lhs, const SensorCleaningFeedback &rhs)
{
    return (  lhs.timestamp== rhs.timestamp &&
     lhs.cleaningStatus== rhs.cleaningStatus 
    );
}

std::size_t SensorCleaningFeedback::hash() const noexcept
{
    std::size_t result = 17;
    result = result * 31 + 0;
    result = result * 31 + hashVector(cleaningStatus);
    return result;
}

 } // namespace av
 } // namespace msg
 } // namespace scm


CEREAL_BIND_TO_ARCHIVES(av::msg::scm::SensorCleaningFeedback)

