#include <interfaces/SensorCleaningRequest/SensorCleaningRequest.h>

#include <iterator>
#include <type_traits>

#include <cereal/cereal.hpp>
#include <cereal/archives/json.hpp>
#include <cereal/archives/binary.hpp>

#include <cereal/types/map.hpp>
#include <cereal/types/vector.hpp>
#include <cereal/types/array.hpp>
#include <cereal/types/string.hpp>
#include <cereal/types/complex.hpp>

#include <ODatum/IntrospectionArchive.h>
#include <ODatum/Hash.h>


 namespace av {
 namespace msg {
 namespace scm {


CleaningInfo::CleaningInfo() = default;

CleaningInfo::CleaningInfo(
    std::string sensorId_arg ,
    bool contaminated_arg ,
    DecisionType decisionType_arg ,
    CleaningType cleaningType_arg 
    ):
    sensorId { std::move(sensorId_arg) } // NOLINT
      ,
    contaminated { std::move(contaminated_arg) } // NOLINT
      ,
    decisionType { std::move(decisionType_arg) } // NOLINT
      ,
    cleaningType { std::move(cleaningType_arg) } // NOLINT
      
    {
}

CleaningInfo::CleaningInfo(const CleaningInfo&) = default;

// NOLINTNEXTLINE
CleaningInfo::CleaningInfo(CleaningInfo&&) = default;

CleaningInfo& CleaningInfo::operator=(const CleaningInfo&) = default;

// NOLINTNEXTLINE
CleaningInfo& CleaningInfo::operator=(CleaningInfo&&) = default;

void CleaningInfo::toCsv(std::ostream& out) const
{
    out.precision(16);
    out << std::boolalpha;
    out << "\"";
        out << sensorId;
        out << "\"";out << ",";
    out << contaminated;out << ",";
    out << static_cast<uint32_t>(decisionType);out << ",";
    out << cleaningType;
    
}

std::stringstream CleaningInfo::toStringstream() const
{
    std::stringstream out;
    toCsv(out);
    return out;
}

template<class Archive>
    void CleaningInfo::serialize(Archive& ar, [[maybe_unused]] unsigned int version)
    {

        ar& CEREAL_NVP(sensorId);
        ar& CEREAL_NVP(contaminated);
        ar& CEREAL_NVP(decisionType);
        ar& CEREAL_NVP(cleaningType);
    }


template void CleaningInfo::serialize(cereal::BinaryInputArchive& ar, unsigned int version);
template void CleaningInfo::serialize(cereal::BinaryOutputArchive& ar, unsigned int version);
template void CleaningInfo::serialize(cereal::IntrospectionArchive& ar, unsigned int version);
template void CleaningInfo::serialize(cereal::JSONInputArchive& ar, unsigned int version);
template void CleaningInfo::serialize(cereal::JSONOutputArchive& ar, unsigned int version);
template void CleaningInfo::serialize(cereal::PortableBinaryInputArchive& ar, unsigned int version);
template void CleaningInfo::serialize(cereal::PortableBinaryOutputArchive& ar, unsigned int version);

std::ostream& operator<< (std::ostream& os, const CleaningInfo& cleaninginfo)
{
    cleaninginfo.toCsv(os);
    return os;
}

bool operator== (const CleaningInfo &lhs, const CleaningInfo &rhs)
{
    return (  lhs.sensorId== rhs.sensorId &&
     lhs.contaminated== rhs.contaminated &&
     lhs.decisionType== rhs.decisionType &&
     lhs.cleaningType== rhs.cleaningType 
    );
}

std::size_t CleaningInfo::hash() const noexcept
{
    std::size_t result = 17;
    result = result * 31 + std::hash<std::string>()(sensorId);
    result = result * 31 + std::hash<bool>()(contaminated);
    result = result * 31 + std::hash<DecisionType>()(decisionType);
    result = result * 31 + std::hash<CleaningType>()(cleaningType);
    return result;
}

 } // namespace av
 } // namespace msg
 } // namespace scm


CEREAL_BIND_TO_ARCHIVES(av::msg::scm::CleaningInfo)




 namespace av {
 namespace msg {
 namespace scm {


SensorCleaningRequest::SensorCleaningRequest() = default;

SensorCleaningRequest::SensorCleaningRequest(
    TimePoint timestamp_arg ,
    std::vector<CleaningInfo> cleaningInfo_arg 
    ):
    timestamp { std::move(timestamp_arg) } // NOLINT
      ,cleaningInfo { std::move(cleaningInfo_arg) } // NOLINT
      {
}

SensorCleaningRequest::SensorCleaningRequest(const SensorCleaningRequest&) = default;

// NOLINTNEXTLINE
SensorCleaningRequest::SensorCleaningRequest(SensorCleaningRequest&&) = default;

SensorCleaningRequest& SensorCleaningRequest::operator=(const SensorCleaningRequest&) = default;

// NOLINTNEXTLINE
SensorCleaningRequest& SensorCleaningRequest::operator=(SensorCleaningRequest&&) = default;

void SensorCleaningRequest::toCsv(std::ostream& out) const
{
    out.precision(16);
    out << std::boolalpha;
    out << timestamp;out << ",";
    // Enclose sequence in double quotations and curly brackets to keep CSV line length uniform
        
        out << "\"{{";
        
        if (!cleaningInfo.empty())
        {
            std::copy(cleaningInfo.begin(), cleaningInfo.end() - 1, std::ostream_iterator<CleaningInfo>(out, "},{"));
            std::copy(cleaningInfo.end()-1, cleaningInfo.end(), std::ostream_iterator<CleaningInfo>(out, ""));
        }
        out << "}}\"";
    
}

std::stringstream SensorCleaningRequest::toStringstream() const
{
    std::stringstream out;
    toCsv(out);
    return out;
}

template<class Archive>
    void SensorCleaningRequest::serialize(Archive& ar, [[maybe_unused]] unsigned int version)
    {

        ar& CEREAL_NVP(timestamp);
        ar& CEREAL_NVP(cleaningInfo);
    }


template void SensorCleaningRequest::serialize(cereal::BinaryInputArchive& ar, unsigned int version);
template void SensorCleaningRequest::serialize(cereal::BinaryOutputArchive& ar, unsigned int version);
template void SensorCleaningRequest::serialize(cereal::IntrospectionArchive& ar, unsigned int version);
template void SensorCleaningRequest::serialize(cereal::JSONInputArchive& ar, unsigned int version);
template void SensorCleaningRequest::serialize(cereal::JSONOutputArchive& ar, unsigned int version);
template void SensorCleaningRequest::serialize(cereal::PortableBinaryInputArchive& ar, unsigned int version);
template void SensorCleaningRequest::serialize(cereal::PortableBinaryOutputArchive& ar, unsigned int version);

std::ostream& operator<< (std::ostream& os, const SensorCleaningRequest& sensorcleaningrequest)
{
    sensorcleaningrequest.toCsv(os);
    return os;
}

bool operator== (const SensorCleaningRequest &lhs, const SensorCleaningRequest &rhs)
{
    return (  lhs.timestamp== rhs.timestamp &&
     lhs.cleaningInfo== rhs.cleaningInfo 
    );
}

std::size_t SensorCleaningRequest::hash() const noexcept
{
    std::size_t result = 17;
    result = result * 31 + 0;
    result = result * 31 + hashVector(cleaningInfo);
    return result;
}

 } // namespace av
 } // namespace msg
 } // namespace scm


CEREAL_BIND_TO_ARCHIVES(av::msg::scm::SensorCleaningRequest)

