#include <interfaces/SimVehicleInitState/SimVehicleInitState.h>

#include <iterator>
#include <type_traits>

#include <cereal/cereal.hpp>
#include <cereal/archives/json.hpp>
#include <cereal/archives/binary.hpp>

#include <cereal/types/map.hpp>
#include <cereal/types/vector.hpp>
#include <cereal/types/array.hpp>
#include <cereal/types/string.hpp>
#include <cereal/types/complex.hpp>

#include <ODatum/IntrospectionArchive.h>
#include <ODatum/Hash.h>





 namespace av {
 namespace msg {


SimVehicleInitState::SimVehicleInitState() = default;

SimVehicleInitState::SimVehicleInitState(
    bool isLatLon_arg ,
    av::geometry::RecPose3D pose_arg ,
    char utmZone_arg ,
    double speedMps_arg 
    ):
    isLatLon { std::move(isLatLon_arg) } // NOLINT
      ,pose { std::move(pose_arg) } // NOLINT
      ,utmZone { std::move(utmZone_arg) } // NOLINT
      ,speedMps { std::move(speedMps_arg) } // NOLINT
      {
}

SimVehicleInitState::SimVehicleInitState(const SimVehicleInitState&) = default;

// NOLINTNEXTLINE
SimVehicleInitState::SimVehicleInitState(SimVehicleInitState&&) = default;

SimVehicleInitState& SimVehicleInitState::operator=(const SimVehicleInitState&) = default;

// NOLINTNEXTLINE
SimVehicleInitState& SimVehicleInitState::operator=(SimVehicleInitState&&) = default;

void SimVehicleInitState::toCsv(std::ostream& out) const
{
    out.precision(16);
    out << std::boolalpha;
    out << isLatLon;out << ",";
    out << pose;out << ",";
    // Avoid appending null terminator to stream
        if (utmZone != '\0')
        {
            out << utmZone;
        }out << ",";
    out << speedMps;
    
}

std::stringstream SimVehicleInitState::toStringstream() const
{
    std::stringstream out;
    toCsv(out);
    return out;
}

template<class Archive>
    void SimVehicleInitState::serialize(Archive& ar, unsigned int version)
{
    ar& CEREAL_NVP(isLatLon);
    ar& CEREAL_NVP(pose);

    if (version >= 2)
    {
        ar& CEREAL_NVP(utmZone);
        ar& CEREAL_NVP(speedMps);
    }
}


template void SimVehicleInitState::serialize(cereal::BinaryInputArchive& ar, unsigned int version);
template void SimVehicleInitState::serialize(cereal::BinaryOutputArchive& ar, unsigned int version);
template void SimVehicleInitState::serialize(cereal::IntrospectionArchive& ar, unsigned int version);
template void SimVehicleInitState::serialize(cereal::JSONInputArchive& ar, unsigned int version);
template void SimVehicleInitState::serialize(cereal::JSONOutputArchive& ar, unsigned int version);
template void SimVehicleInitState::serialize(cereal::PortableBinaryInputArchive& ar, unsigned int version);
template void SimVehicleInitState::serialize(cereal::PortableBinaryOutputArchive& ar, unsigned int version);

std::ostream& operator<< (std::ostream& os, const SimVehicleInitState& simvehicleinitstate)
{
    simvehicleinitstate.toCsv(os);
    return os;
}

bool operator== (const SimVehicleInitState &lhs, const SimVehicleInitState &rhs)
{
    return (  lhs.isLatLon== rhs.isLatLon &&
     lhs.pose== rhs.pose &&
     lhs.utmZone== rhs.utmZone &&
     lhs.speedMps== rhs.speedMps 
    );
}

std::size_t SimVehicleInitState::hash() const noexcept
{
    std::size_t result = 17;
    result = result * 31 + std::hash<bool>()(isLatLon);
    result = result * 31 + std::hash<av::geometry::RecPose3D>()(pose);
    result = result * 31 + std::hash<char>()(utmZone);
    result = result * 31 + std::hash<double>()(speedMps);
    return result;
}

 } // namespace av
 } // namespace msg


CEREAL_BIND_TO_ARCHIVES(av::msg::SimVehicleInitState)

