#include <interfaces/SimulationCtrlGenericObject/SimulationCtrlGenericObject.h>

#include <iterator>
#include <type_traits>

#include <cereal/cereal.hpp>
#include <cereal/archives/json.hpp>
#include <cereal/archives/binary.hpp>

#include <cereal/types/map.hpp>
#include <cereal/types/vector.hpp>
#include <cereal/types/array.hpp>
#include <cereal/types/string.hpp>
#include <cereal/types/complex.hpp>

#include <ODatum/IntrospectionArchive.h>
#include <ODatum/Hash.h>





 namespace simulation {


SimulationCtrlGenericObject::SimulationCtrlGenericObject() = default;

SimulationCtrlGenericObject::SimulationCtrlGenericObject(
    std::vector<double> start_arg ,
    std::vector<double> end_arg ,
    bool shouldDelete_arg 
    ):
    start { std::move(start_arg) } // NOLINT
      ,end { std::move(end_arg) } // NOLINT
      ,shouldDelete { std::move(shouldDelete_arg) } // NOLINT
      {
}

SimulationCtrlGenericObject::SimulationCtrlGenericObject(const SimulationCtrlGenericObject&) = default;

// NOLINTNEXTLINE
SimulationCtrlGenericObject::SimulationCtrlGenericObject(SimulationCtrlGenericObject&&) = default;

SimulationCtrlGenericObject& SimulationCtrlGenericObject::operator=(const SimulationCtrlGenericObject&) = default;

// NOLINTNEXTLINE
SimulationCtrlGenericObject& SimulationCtrlGenericObject::operator=(SimulationCtrlGenericObject&&) = default;

void SimulationCtrlGenericObject::toCsv(std::ostream& out) const
{
    out.precision(16);
    out << std::boolalpha;
    // Enclose sequence in double quotations and curly brackets to keep CSV line length uniform
        
        out << "\"{{";
        
        if (!start.empty())
        {
            std::copy(start.begin(), start.end() - 1, std::ostream_iterator<double>(out, "},{"));
            std::copy(start.end()-1, start.end(), std::ostream_iterator<double>(out, ""));
        }
        out << "}}\"";out << ",";
    // Enclose sequence in double quotations and curly brackets to keep CSV line length uniform
        
        out << "\"{{";
        
        if (!end.empty())
        {
            std::copy(end.begin(), end.end() - 1, std::ostream_iterator<double>(out, "},{"));
            std::copy(end.end()-1, end.end(), std::ostream_iterator<double>(out, ""));
        }
        out << "}}\"";out << ",";
    out << shouldDelete;
    
}

std::stringstream SimulationCtrlGenericObject::toStringstream() const
{
    std::stringstream out;
    toCsv(out);
    return out;
}

template<class Archive>
    void SimulationCtrlGenericObject::serialize(Archive& ar, [[maybe_unused]] unsigned int version)
    {

        ar& CEREAL_NVP(start);
        ar& CEREAL_NVP(end);
        ar& CEREAL_NVP(shouldDelete);
    }


template void SimulationCtrlGenericObject::serialize(cereal::BinaryInputArchive& ar, unsigned int version);
template void SimulationCtrlGenericObject::serialize(cereal::BinaryOutputArchive& ar, unsigned int version);
template void SimulationCtrlGenericObject::serialize(cereal::IntrospectionArchive& ar, unsigned int version);
template void SimulationCtrlGenericObject::serialize(cereal::JSONInputArchive& ar, unsigned int version);
template void SimulationCtrlGenericObject::serialize(cereal::JSONOutputArchive& ar, unsigned int version);
template void SimulationCtrlGenericObject::serialize(cereal::PortableBinaryInputArchive& ar, unsigned int version);
template void SimulationCtrlGenericObject::serialize(cereal::PortableBinaryOutputArchive& ar, unsigned int version);

std::ostream& operator<< (std::ostream& os, const SimulationCtrlGenericObject& simulationctrlgenericobject)
{
    simulationctrlgenericobject.toCsv(os);
    return os;
}

bool operator== (const SimulationCtrlGenericObject &lhs, const SimulationCtrlGenericObject &rhs)
{
    return (  lhs.start== rhs.start &&
     lhs.end== rhs.end &&
     lhs.shouldDelete== rhs.shouldDelete 
    );
}

std::size_t SimulationCtrlGenericObject::hash() const noexcept
{
    std::size_t result = 17;
    result = result * 31 + hashVector(start);
    result = result * 31 + hashVector(end);
    result = result * 31 + std::hash<bool>()(shouldDelete);
    return result;
}

 } // namespace simulation


CEREAL_BIND_TO_ARCHIVES(simulation::SimulationCtrlGenericObject)

