#include <interfaces/SimulationCtrlTeleport/SimulationCtrlTeleport.h>

#include <iterator>
#include <type_traits>

#include <cereal/cereal.hpp>
#include <cereal/archives/json.hpp>
#include <cereal/archives/binary.hpp>

#include <cereal/types/map.hpp>
#include <cereal/types/vector.hpp>
#include <cereal/types/array.hpp>
#include <cereal/types/string.hpp>
#include <cereal/types/complex.hpp>

#include <ODatum/IntrospectionArchive.h>
#include <ODatum/Hash.h>





 namespace simulation {


SimulationCtrlTeleport::SimulationCtrlTeleport() = default;

SimulationCtrlTeleport::SimulationCtrlTeleport(
    std::vector<double> pos_arg ,
    double heading_arg 
    ):
    pos { std::move(pos_arg) } // NOLINT
      ,heading { std::move(heading_arg) } // NOLINT
      {
}

SimulationCtrlTeleport::SimulationCtrlTeleport(const SimulationCtrlTeleport&) = default;

// NOLINTNEXTLINE
SimulationCtrlTeleport::SimulationCtrlTeleport(SimulationCtrlTeleport&&) = default;

SimulationCtrlTeleport& SimulationCtrlTeleport::operator=(const SimulationCtrlTeleport&) = default;

// NOLINTNEXTLINE
SimulationCtrlTeleport& SimulationCtrlTeleport::operator=(SimulationCtrlTeleport&&) = default;

void SimulationCtrlTeleport::toCsv(std::ostream& out) const
{
    out.precision(16);
    out << std::boolalpha;
    // Enclose sequence in double quotations and curly brackets to keep CSV line length uniform
        
        out << "\"{{";
        
        if (!pos.empty())
        {
            std::copy(pos.begin(), pos.end() - 1, std::ostream_iterator<double>(out, "},{"));
            std::copy(pos.end()-1, pos.end(), std::ostream_iterator<double>(out, ""));
        }
        out << "}}\"";out << ",";
    out << heading;
    
}

std::stringstream SimulationCtrlTeleport::toStringstream() const
{
    std::stringstream out;
    toCsv(out);
    return out;
}

template<class Archive>
    void SimulationCtrlTeleport::serialize(Archive& ar, [[maybe_unused]] unsigned int version)
    {

        ar& CEREAL_NVP(pos);
        ar& CEREAL_NVP(heading);
    }


template void SimulationCtrlTeleport::serialize(cereal::BinaryInputArchive& ar, unsigned int version);
template void SimulationCtrlTeleport::serialize(cereal::BinaryOutputArchive& ar, unsigned int version);
template void SimulationCtrlTeleport::serialize(cereal::IntrospectionArchive& ar, unsigned int version);
template void SimulationCtrlTeleport::serialize(cereal::JSONInputArchive& ar, unsigned int version);
template void SimulationCtrlTeleport::serialize(cereal::JSONOutputArchive& ar, unsigned int version);
template void SimulationCtrlTeleport::serialize(cereal::PortableBinaryInputArchive& ar, unsigned int version);
template void SimulationCtrlTeleport::serialize(cereal::PortableBinaryOutputArchive& ar, unsigned int version);

std::ostream& operator<< (std::ostream& os, const SimulationCtrlTeleport& simulationctrlteleport)
{
    simulationctrlteleport.toCsv(os);
    return os;
}

bool operator== (const SimulationCtrlTeleport &lhs, const SimulationCtrlTeleport &rhs)
{
    return (  lhs.pos== rhs.pos &&
     lhs.heading== rhs.heading 
    );
}

std::size_t SimulationCtrlTeleport::hash() const noexcept
{
    std::size_t result = 17;
    result = result * 31 + hashVector(pos);
    result = result * 31 + std::hash<double>()(heading);
    return result;
}

 } // namespace simulation


CEREAL_BIND_TO_ARCHIVES(simulation::SimulationCtrlTeleport)

