#include <interfaces/SimulationStateCommands/SimulationStateCommands.h>

#include <iterator>
#include <type_traits>

#include <cereal/cereal.hpp>
#include <cereal/archives/json.hpp>
#include <cereal/archives/binary.hpp>

#include <cereal/types/map.hpp>
#include <cereal/types/vector.hpp>
#include <cereal/types/array.hpp>
#include <cereal/types/string.hpp>
#include <cereal/types/complex.hpp>

#include <ODatum/IntrospectionArchive.h>
#include <ODatum/Hash.h>





 namespace simulation {


SimulationStateCommands::SimulationStateCommands() = default;

SimulationStateCommands::SimulationStateCommands(
    SIMULATION_STATE_COMMANDS command_arg ,
    std::string testcaseFilePath_arg 
    ):
    command { std::move(command_arg) } // NOLINT
      ,testcaseFilePath { std::move(testcaseFilePath_arg) } // NOLINT
      {
}

SimulationStateCommands::SimulationStateCommands(const SimulationStateCommands&) = default;

// NOLINTNEXTLINE
SimulationStateCommands::SimulationStateCommands(SimulationStateCommands&&) = default;

SimulationStateCommands& SimulationStateCommands::operator=(const SimulationStateCommands&) = default;

// NOLINTNEXTLINE
SimulationStateCommands& SimulationStateCommands::operator=(SimulationStateCommands&&) = default;

void SimulationStateCommands::toCsv(std::ostream& out) const
{
    out.precision(16);
    out << std::boolalpha;
    out << static_cast<uint32_t>(command);out << ",";
    out << "\"";
        out << testcaseFilePath;
        out << "\"";
    
}

std::stringstream SimulationStateCommands::toStringstream() const
{
    std::stringstream out;
    toCsv(out);
    return out;
}

template<class Archive>
    void SimulationStateCommands::serialize(Archive& ar, unsigned int version)
{
	if (version >= 1)
	{
		ar& CEREAL_NVP(command);
	}
	if (version >= 2)
	{
		ar& CEREAL_NVP(testcaseFilePath);
	}
}



template void SimulationStateCommands::serialize(cereal::BinaryInputArchive& ar, unsigned int version);
template void SimulationStateCommands::serialize(cereal::BinaryOutputArchive& ar, unsigned int version);
template void SimulationStateCommands::serialize(cereal::IntrospectionArchive& ar, unsigned int version);
template void SimulationStateCommands::serialize(cereal::JSONInputArchive& ar, unsigned int version);
template void SimulationStateCommands::serialize(cereal::JSONOutputArchive& ar, unsigned int version);
template void SimulationStateCommands::serialize(cereal::PortableBinaryInputArchive& ar, unsigned int version);
template void SimulationStateCommands::serialize(cereal::PortableBinaryOutputArchive& ar, unsigned int version);

std::ostream& operator<< (std::ostream& os, const SimulationStateCommands& simulationstatecommands)
{
    simulationstatecommands.toCsv(os);
    return os;
}

bool operator== (const SimulationStateCommands &lhs, const SimulationStateCommands &rhs)
{
    return (  lhs.command== rhs.command &&
     lhs.testcaseFilePath== rhs.testcaseFilePath 
    );
}

std::size_t SimulationStateCommands::hash() const noexcept
{
    std::size_t result = 17;
    result = result * 31 + std::hash<SIMULATION_STATE_COMMANDS>()(command);
    result = result * 31 + std::hash<std::string>()(testcaseFilePath);
    return result;
}

 } // namespace simulation


CEREAL_BIND_TO_ARCHIVES(simulation::SimulationStateCommands)

