#include <interfaces/SmoothedLocalizationPoseList/SmoothedLocalizationPoseList.h>

#include <iterator>
#include <type_traits>

#include <cereal/cereal.hpp>
#include <cereal/archives/json.hpp>
#include <cereal/archives/binary.hpp>

#include <cereal/types/map.hpp>
#include <cereal/types/vector.hpp>
#include <cereal/types/array.hpp>
#include <cereal/types/string.hpp>
#include <cereal/types/complex.hpp>

#include <ODatum/IntrospectionArchive.h>
#include <ODatum/Hash.h>


 namespace av {
 namespace msg {


SmoothedLocalizationPose::SmoothedLocalizationPose() = default;

SmoothedLocalizationPose::SmoothedLocalizationPose(
    TimePoint timeStamp_arg ,
    av::geometry::Pose poseENU_arg ,
    av::geometry::Vector3d velocityENU_mps_arg ,
    LOCALIZATION_POSE_HEALTH health_arg 
    ):
    timeStamp { std::move(timeStamp_arg) } // NOLINT
      ,
    poseENU { std::move(poseENU_arg) } // NOLINT
      ,
    velocityENU_mps { std::move(velocityENU_mps_arg) } // NOLINT
      ,
    health { std::move(health_arg) } // NOLINT
      
    {
}

SmoothedLocalizationPose::SmoothedLocalizationPose(const SmoothedLocalizationPose&) = default;

// NOLINTNEXTLINE
SmoothedLocalizationPose::SmoothedLocalizationPose(SmoothedLocalizationPose&&) = default;

SmoothedLocalizationPose& SmoothedLocalizationPose::operator=(const SmoothedLocalizationPose&) = default;

// NOLINTNEXTLINE
SmoothedLocalizationPose& SmoothedLocalizationPose::operator=(SmoothedLocalizationPose&&) = default;

void SmoothedLocalizationPose::toCsv(std::ostream& out) const
{
    out.precision(16);
    out << std::boolalpha;
    out << timeStamp;out << ",";
    out << poseENU;out << ",";
    out << velocityENU_mps;out << ",";
    out << static_cast<uint32_t>(health);
    
}

std::stringstream SmoothedLocalizationPose::toStringstream() const
{
    std::stringstream out;
    toCsv(out);
    return out;
}

template<class Archive>
    void SmoothedLocalizationPose::serialize(Archive& ar, [[maybe_unused]] unsigned int version)
    {

        ar& CEREAL_NVP(timeStamp);
        ar& CEREAL_NVP(poseENU);
        ar& CEREAL_NVP(velocityENU_mps);
        ar& CEREAL_NVP(health);
    }


template void SmoothedLocalizationPose::serialize(cereal::BinaryInputArchive& ar, unsigned int version);
template void SmoothedLocalizationPose::serialize(cereal::BinaryOutputArchive& ar, unsigned int version);
template void SmoothedLocalizationPose::serialize(cereal::IntrospectionArchive& ar, unsigned int version);
template void SmoothedLocalizationPose::serialize(cereal::JSONInputArchive& ar, unsigned int version);
template void SmoothedLocalizationPose::serialize(cereal::JSONOutputArchive& ar, unsigned int version);
template void SmoothedLocalizationPose::serialize(cereal::PortableBinaryInputArchive& ar, unsigned int version);
template void SmoothedLocalizationPose::serialize(cereal::PortableBinaryOutputArchive& ar, unsigned int version);

std::ostream& operator<< (std::ostream& os, const SmoothedLocalizationPose& smoothedlocalizationpose)
{
    smoothedlocalizationpose.toCsv(os);
    return os;
}

bool operator== (const SmoothedLocalizationPose &lhs, const SmoothedLocalizationPose &rhs)
{
    return (  lhs.timeStamp== rhs.timeStamp &&
     lhs.poseENU== rhs.poseENU &&
     lhs.velocityENU_mps== rhs.velocityENU_mps &&
     lhs.health== rhs.health 
    );
}

std::size_t SmoothedLocalizationPose::hash() const noexcept
{
    std::size_t result = 17;
    result = result * 31 + 0;
    result = result * 31 + std::hash<av::geometry::Pose>()(poseENU);
    result = result * 31 + std::hash<av::geometry::Vector3d>()(velocityENU_mps);
    result = result * 31 + std::hash<LOCALIZATION_POSE_HEALTH>()(health);
    return result;
}

 } // namespace av
 } // namespace msg


CEREAL_BIND_TO_ARCHIVES(av::msg::SmoothedLocalizationPose)




 namespace av {
 namespace msg {


SmoothedLocalizationPoseList::SmoothedLocalizationPoseList() = default;

SmoothedLocalizationPoseList::SmoothedLocalizationPoseList(
    TimePoint timeStamp_arg ,
    std::vector<SmoothedLocalizationPose> poses_arg 
    ):
    timeStamp { std::move(timeStamp_arg) } // NOLINT
      ,poses { std::move(poses_arg) } // NOLINT
      {
}

SmoothedLocalizationPoseList::SmoothedLocalizationPoseList(const SmoothedLocalizationPoseList&) = default;

// NOLINTNEXTLINE
SmoothedLocalizationPoseList::SmoothedLocalizationPoseList(SmoothedLocalizationPoseList&&) = default;

SmoothedLocalizationPoseList& SmoothedLocalizationPoseList::operator=(const SmoothedLocalizationPoseList&) = default;

// NOLINTNEXTLINE
SmoothedLocalizationPoseList& SmoothedLocalizationPoseList::operator=(SmoothedLocalizationPoseList&&) = default;

void SmoothedLocalizationPoseList::toCsv(std::ostream& out) const
{
    out.precision(16);
    out << std::boolalpha;
    out << timeStamp;out << ",";
    // Enclose sequence in double quotations and curly brackets to keep CSV line length uniform
        
        out << "\"{{";
        
        if (!poses.empty())
        {
            std::copy(poses.begin(), poses.end() - 1, std::ostream_iterator<SmoothedLocalizationPose>(out, "},{"));
            std::copy(poses.end()-1, poses.end(), std::ostream_iterator<SmoothedLocalizationPose>(out, ""));
        }
        out << "}}\"";
    
}

std::stringstream SmoothedLocalizationPoseList::toStringstream() const
{
    std::stringstream out;
    toCsv(out);
    return out;
}

template<class Archive>
    void SmoothedLocalizationPoseList::serialize(Archive& ar, [[maybe_unused]] unsigned int version)
    {

        ar& CEREAL_NVP(timeStamp);
        ar& CEREAL_NVP(poses);
    }


template void SmoothedLocalizationPoseList::serialize(cereal::BinaryInputArchive& ar, unsigned int version);
template void SmoothedLocalizationPoseList::serialize(cereal::BinaryOutputArchive& ar, unsigned int version);
template void SmoothedLocalizationPoseList::serialize(cereal::IntrospectionArchive& ar, unsigned int version);
template void SmoothedLocalizationPoseList::serialize(cereal::JSONInputArchive& ar, unsigned int version);
template void SmoothedLocalizationPoseList::serialize(cereal::JSONOutputArchive& ar, unsigned int version);
template void SmoothedLocalizationPoseList::serialize(cereal::PortableBinaryInputArchive& ar, unsigned int version);
template void SmoothedLocalizationPoseList::serialize(cereal::PortableBinaryOutputArchive& ar, unsigned int version);

std::ostream& operator<< (std::ostream& os, const SmoothedLocalizationPoseList& smoothedlocalizationposelist)
{
    smoothedlocalizationposelist.toCsv(os);
    return os;
}

bool operator== (const SmoothedLocalizationPoseList &lhs, const SmoothedLocalizationPoseList &rhs)
{
    return (  lhs.timeStamp== rhs.timeStamp &&
     lhs.poses== rhs.poses 
    );
}

std::size_t SmoothedLocalizationPoseList::hash() const noexcept
{
    std::size_t result = 17;
    result = result * 31 + 0;
    result = result * 31 + hashVector(poses);
    return result;
}

 } // namespace av
 } // namespace msg


CEREAL_BIND_TO_ARCHIVES(av::msg::SmoothedLocalizationPoseList)

