#include <interfaces/SoundGeneration/SoundGeneration.h>

#include <iterator>
#include <type_traits>

#include <cereal/cereal.hpp>
#include <cereal/archives/json.hpp>
#include <cereal/archives/binary.hpp>

#include <cereal/types/map.hpp>
#include <cereal/types/vector.hpp>
#include <cereal/types/array.hpp>
#include <cereal/types/string.hpp>
#include <cereal/types/complex.hpp>

#include <ODatum/IntrospectionArchive.h>
#include <ODatum/Hash.h>





 namespace av {
 namespace msg {


SoundGeneration::SoundGeneration() = default;

SoundGeneration::SoundGeneration(
    std::string soundMessage_arg 
    ):
    soundMessage { std::move(soundMessage_arg) } // NOLINT
      {
}

SoundGeneration::SoundGeneration(const SoundGeneration&) = default;

// NOLINTNEXTLINE
SoundGeneration::SoundGeneration(SoundGeneration&&) = default;

SoundGeneration& SoundGeneration::operator=(const SoundGeneration&) = default;

// NOLINTNEXTLINE
SoundGeneration& SoundGeneration::operator=(SoundGeneration&&) = default;

void SoundGeneration::toCsv(std::ostream& out) const
{
    out.precision(16);
    out << std::boolalpha;
    out << "\"";
        out << soundMessage;
        out << "\"";
    
}

std::stringstream SoundGeneration::toStringstream() const
{
    std::stringstream out;
    toCsv(out);
    return out;
}

template<class Archive>
    void SoundGeneration::serialize(Archive& ar, [[maybe_unused]] unsigned int version)
    {

        ar& CEREAL_NVP(soundMessage);
    }


template void SoundGeneration::serialize(cereal::BinaryInputArchive& ar, unsigned int version);
template void SoundGeneration::serialize(cereal::BinaryOutputArchive& ar, unsigned int version);
template void SoundGeneration::serialize(cereal::IntrospectionArchive& ar, unsigned int version);
template void SoundGeneration::serialize(cereal::JSONInputArchive& ar, unsigned int version);
template void SoundGeneration::serialize(cereal::JSONOutputArchive& ar, unsigned int version);
template void SoundGeneration::serialize(cereal::PortableBinaryInputArchive& ar, unsigned int version);
template void SoundGeneration::serialize(cereal::PortableBinaryOutputArchive& ar, unsigned int version);

std::ostream& operator<< (std::ostream& os, const SoundGeneration& soundgeneration)
{
    soundgeneration.toCsv(os);
    return os;
}

bool operator== (const SoundGeneration &lhs, const SoundGeneration &rhs)
{
    return (  lhs.soundMessage== rhs.soundMessage 
    );
}

std::size_t SoundGeneration::hash() const noexcept
{
    std::size_t result = 17;
    result = result * 31 + std::hash<std::string>()(soundMessage);
    return result;
}

 } // namespace av
 } // namespace msg


CEREAL_BIND_TO_ARCHIVES(av::msg::SoundGeneration)

