#include <interfaces/SpeedControlCommand/SpeedControlCommand.h>

#include <iterator>
#include <type_traits>

#include <cereal/cereal.hpp>
#include <cereal/archives/json.hpp>
#include <cereal/archives/binary.hpp>

#include <cereal/types/map.hpp>
#include <cereal/types/vector.hpp>
#include <cereal/types/array.hpp>
#include <cereal/types/string.hpp>
#include <cereal/types/complex.hpp>

#include <ODatum/IntrospectionArchive.h>
#include <ODatum/Hash.h>





 namespace av {
 namespace msg {


SpeedControlCommand::SpeedControlCommand() = default;

SpeedControlCommand::SpeedControlCommand(
    TimePoint timeStamp_arg ,
    double speed_mps_arg ,
    double acceleration_mps2_arg 
    ):
    timeStamp { std::move(timeStamp_arg) } // NOLINT
      ,speed_mps { std::move(speed_mps_arg) } // NOLINT
      ,acceleration_mps2 { std::move(acceleration_mps2_arg) } // NOLINT
      {
}

SpeedControlCommand::SpeedControlCommand(const SpeedControlCommand&) = default;

// NOLINTNEXTLINE
SpeedControlCommand::SpeedControlCommand(SpeedControlCommand&&) = default;

SpeedControlCommand& SpeedControlCommand::operator=(const SpeedControlCommand&) = default;

// NOLINTNEXTLINE
SpeedControlCommand& SpeedControlCommand::operator=(SpeedControlCommand&&) = default;

void SpeedControlCommand::toCsv(std::ostream& out) const
{
    out.precision(16);
    out << std::boolalpha;
    out << timeStamp;out << ",";
    out << speed_mps;out << ",";
    out << acceleration_mps2;
    
}

std::stringstream SpeedControlCommand::toStringstream() const
{
    std::stringstream out;
    toCsv(out);
    return out;
}

template<class Archive>
    void SpeedControlCommand::serialize(Archive& ar, unsigned int version)
{
	if (version == 1) {
		ar& CEREAL_NVP(timeStamp);
		ar& CEREAL_NVP(static_cast<float>(speed_mps));
		ar& CEREAL_NVP(static_cast<float>(acceleration_mps2));
	}

	if (version >= 2) // breaking changes
	{
		ar& CEREAL_NVP(timeStamp);
		ar& CEREAL_NVP(speed_mps);
		ar& CEREAL_NVP(acceleration_mps2);
	}
}

}} // to close av::msg namespace temporary

struct SpeedControlCommand : public av::msg::SpeedControlCommand{};
CEREAL_CLASS_VERSION(SpeedControlCommand, 3)
CEREAL_REGISTER_TYPE(SpeedControlCommand)

namespace av {
namespace msg {


template void SpeedControlCommand::serialize(cereal::BinaryInputArchive& ar, unsigned int version);
template void SpeedControlCommand::serialize(cereal::BinaryOutputArchive& ar, unsigned int version);
template void SpeedControlCommand::serialize(cereal::IntrospectionArchive& ar, unsigned int version);
template void SpeedControlCommand::serialize(cereal::JSONInputArchive& ar, unsigned int version);
template void SpeedControlCommand::serialize(cereal::JSONOutputArchive& ar, unsigned int version);
template void SpeedControlCommand::serialize(cereal::PortableBinaryInputArchive& ar, unsigned int version);
template void SpeedControlCommand::serialize(cereal::PortableBinaryOutputArchive& ar, unsigned int version);

std::ostream& operator<< (std::ostream& os, const SpeedControlCommand& speedcontrolcommand)
{
    speedcontrolcommand.toCsv(os);
    return os;
}

bool operator== (const SpeedControlCommand &lhs, const SpeedControlCommand &rhs)
{
    return (  lhs.timeStamp== rhs.timeStamp &&
     lhs.speed_mps== rhs.speed_mps &&
     lhs.acceleration_mps2== rhs.acceleration_mps2 
    );
}

std::size_t SpeedControlCommand::hash() const noexcept
{
    std::size_t result = 17;
    result = result * 31 + 0;
    result = result * 31 + std::hash<double>()(speed_mps);
    result = result * 31 + std::hash<double>()(acceleration_mps2);
    return result;
}

 } // namespace av
 } // namespace msg


CEREAL_BIND_TO_ARCHIVES(av::msg::SpeedControlCommand)

