#include <interfaces/SteerControlCommand/SteerControlCommand.h>

#include <iterator>
#include <type_traits>

#include <cereal/cereal.hpp>
#include <cereal/archives/json.hpp>
#include <cereal/archives/binary.hpp>

#include <cereal/types/map.hpp>
#include <cereal/types/vector.hpp>
#include <cereal/types/array.hpp>
#include <cereal/types/string.hpp>
#include <cereal/types/complex.hpp>

#include <ODatum/IntrospectionArchive.h>
#include <ODatum/Hash.h>





 namespace av {
 namespace msg {


SteerControlCommand::SteerControlCommand() = default;

SteerControlCommand::SteerControlCommand(
    TimePoint timeStamp_arg ,
    double roadWheelAngle_rad_arg ,
    bool isValidFeedForward_arg ,
    double roadWheelAngleRate_radps_arg ,
    double roadWheelAngleAccel_radpss_arg 
    ):
    timeStamp { std::move(timeStamp_arg) } // NOLINT
      ,roadWheelAngle_rad { std::move(roadWheelAngle_rad_arg) } // NOLINT
      ,isValidFeedForward { std::move(isValidFeedForward_arg) } // NOLINT
      ,roadWheelAngleRate_radps { std::move(roadWheelAngleRate_radps_arg) } // NOLINT
      ,roadWheelAngleAccel_radpss { std::move(roadWheelAngleAccel_radpss_arg) } // NOLINT
      {
}

SteerControlCommand::SteerControlCommand(const SteerControlCommand&) = default;

// NOLINTNEXTLINE
SteerControlCommand::SteerControlCommand(SteerControlCommand&&) = default;

SteerControlCommand& SteerControlCommand::operator=(const SteerControlCommand&) = default;

// NOLINTNEXTLINE
SteerControlCommand& SteerControlCommand::operator=(SteerControlCommand&&) = default;

void SteerControlCommand::toCsv(std::ostream& out) const
{
    out.precision(16);
    out << std::boolalpha;
    out << timeStamp;out << ",";
    out << roadWheelAngle_rad;out << ",";
    out << isValidFeedForward;out << ",";
    out << roadWheelAngleRate_radps;out << ",";
    out << roadWheelAngleAccel_radpss;
    
}

std::stringstream SteerControlCommand::toStringstream() const
{
    std::stringstream out;
    toCsv(out);
    return out;
}

template<class Archive>
    void SteerControlCommand::serialize(Archive& ar, unsigned int version)
{
    if (version < 4)
    {
        otimePoint otp;
        ar& cereal::make_nvp("timeStamp", otp);
        timeStamp = av::TimePoint(std::chrono::seconds(otp.tv_sec) + std::chrono::nanoseconds(otp.tv_nsec));
    } else {
        ar& CEREAL_NVP(timeStamp);
    }

    if (version < 3) {
        ar& CEREAL_NVP(static_cast<float>(roadWheelAngle_rad));
        ar& CEREAL_NVP(static_cast<float>(roadWheelAngleRate_radps));
    }

    if (version >= 3) // breaking changes
    {
        ar& CEREAL_NVP(roadWheelAngle_rad);
        ar& CEREAL_NVP(isValidFeedForward);
        ar& CEREAL_NVP(roadWheelAngleRate_radps);
        ar& CEREAL_NVP(roadWheelAngleAccel_radpss);
    }
}

}} // to close av::msg namespace temporary

struct SteerControlCommand : public av::msg::SteerControlCommand{};
CEREAL_CLASS_VERSION(SteerControlCommand, 4)
CEREAL_REGISTER_TYPE(SteerControlCommand)

namespace av {
namespace msg {



template void SteerControlCommand::serialize(cereal::BinaryInputArchive& ar, unsigned int version);
template void SteerControlCommand::serialize(cereal::BinaryOutputArchive& ar, unsigned int version);
template void SteerControlCommand::serialize(cereal::IntrospectionArchive& ar, unsigned int version);
template void SteerControlCommand::serialize(cereal::JSONInputArchive& ar, unsigned int version);
template void SteerControlCommand::serialize(cereal::JSONOutputArchive& ar, unsigned int version);
template void SteerControlCommand::serialize(cereal::PortableBinaryInputArchive& ar, unsigned int version);
template void SteerControlCommand::serialize(cereal::PortableBinaryOutputArchive& ar, unsigned int version);

std::ostream& operator<< (std::ostream& os, const SteerControlCommand& steercontrolcommand)
{
    steercontrolcommand.toCsv(os);
    return os;
}

bool operator== (const SteerControlCommand &lhs, const SteerControlCommand &rhs)
{
    return (  lhs.timeStamp== rhs.timeStamp &&
     lhs.roadWheelAngle_rad== rhs.roadWheelAngle_rad &&
     lhs.isValidFeedForward== rhs.isValidFeedForward &&
     lhs.roadWheelAngleRate_radps== rhs.roadWheelAngleRate_radps &&
     lhs.roadWheelAngleAccel_radpss== rhs.roadWheelAngleAccel_radpss 
    );
}

std::size_t SteerControlCommand::hash() const noexcept
{
    std::size_t result = 17;
    result = result * 31 + 0;
    result = result * 31 + std::hash<double>()(roadWheelAngle_rad);
    result = result * 31 + std::hash<bool>()(isValidFeedForward);
    result = result * 31 + std::hash<double>()(roadWheelAngleRate_radps);
    result = result * 31 + std::hash<double>()(roadWheelAngleAccel_radpss);
    return result;
}

 } // namespace av
 } // namespace msg


CEREAL_BIND_TO_ARCHIVES(av::msg::SteerControlCommand)

