#include <interfaces/SystemHealthMessage_avt/SystemHealthMessage_avt.h>

#include <iterator>
#include <type_traits>

#include <cereal/cereal.hpp>
#include <cereal/archives/json.hpp>
#include <cereal/archives/binary.hpp>

#include <cereal/types/map.hpp>
#include <cereal/types/vector.hpp>
#include <cereal/types/array.hpp>
#include <cereal/types/string.hpp>
#include <cereal/types/complex.hpp>

#include <ODatum/IntrospectionArchive.h>
#include <ODatum/Hash.h>


 namespace av {
 namespace msg {


SystemMemInfo::SystemMemInfo() = default;

SystemMemInfo::SystemMemInfo(
    uint64_t total_kb_arg ,
    uint64_t free_kb_arg ,
    uint64_t available_kb_arg ,
    uint64_t buffers_kb_arg ,
    uint64_t cached_kb_arg 
    ):
    total_kb { std::move(total_kb_arg) } // NOLINT
      ,
    free_kb { std::move(free_kb_arg) } // NOLINT
      ,
    available_kb { std::move(available_kb_arg) } // NOLINT
      ,
    buffers_kb { std::move(buffers_kb_arg) } // NOLINT
      ,
    cached_kb { std::move(cached_kb_arg) } // NOLINT
      
    {
}

SystemMemInfo::SystemMemInfo(const SystemMemInfo&) = default;

// NOLINTNEXTLINE
SystemMemInfo::SystemMemInfo(SystemMemInfo&&) = default;

SystemMemInfo& SystemMemInfo::operator=(const SystemMemInfo&) = default;

// NOLINTNEXTLINE
SystemMemInfo& SystemMemInfo::operator=(SystemMemInfo&&) = default;

void SystemMemInfo::toCsv(std::ostream& out) const
{
    out.precision(16);
    out << std::boolalpha;
    out << total_kb;out << ",";
    out << free_kb;out << ",";
    out << available_kb;out << ",";
    out << buffers_kb;out << ",";
    out << cached_kb;
    
}

std::stringstream SystemMemInfo::toStringstream() const
{
    std::stringstream out;
    toCsv(out);
    return out;
}

template<class Archive>
    void SystemMemInfo::serialize(Archive& ar, [[maybe_unused]] unsigned int version)
    {

        ar& CEREAL_NVP(total_kb);
        ar& CEREAL_NVP(free_kb);
        ar& CEREAL_NVP(available_kb);
        ar& CEREAL_NVP(buffers_kb);
        ar& CEREAL_NVP(cached_kb);
    }


template void SystemMemInfo::serialize(cereal::BinaryInputArchive& ar, unsigned int version);
template void SystemMemInfo::serialize(cereal::BinaryOutputArchive& ar, unsigned int version);
template void SystemMemInfo::serialize(cereal::IntrospectionArchive& ar, unsigned int version);
template void SystemMemInfo::serialize(cereal::JSONInputArchive& ar, unsigned int version);
template void SystemMemInfo::serialize(cereal::JSONOutputArchive& ar, unsigned int version);
template void SystemMemInfo::serialize(cereal::PortableBinaryInputArchive& ar, unsigned int version);
template void SystemMemInfo::serialize(cereal::PortableBinaryOutputArchive& ar, unsigned int version);

std::ostream& operator<< (std::ostream& os, const SystemMemInfo& systemmeminfo)
{
    systemmeminfo.toCsv(os);
    return os;
}

bool operator== (const SystemMemInfo &lhs, const SystemMemInfo &rhs)
{
    return (  lhs.total_kb== rhs.total_kb &&
     lhs.free_kb== rhs.free_kb &&
     lhs.available_kb== rhs.available_kb &&
     lhs.buffers_kb== rhs.buffers_kb &&
     lhs.cached_kb== rhs.cached_kb 
    );
}

std::size_t SystemMemInfo::hash() const noexcept
{
    std::size_t result = 17;
    result = result * 31 + std::hash<uint64_t>()(total_kb);
    result = result * 31 + std::hash<uint64_t>()(free_kb);
    result = result * 31 + std::hash<uint64_t>()(available_kb);
    result = result * 31 + std::hash<uint64_t>()(buffers_kb);
    result = result * 31 + std::hash<uint64_t>()(cached_kb);
    return result;
}

 } // namespace av
 } // namespace msg


CEREAL_BIND_TO_ARCHIVES(av::msg::SystemMemInfo)

 namespace av {
 namespace msg {


GpuDeviceData::GpuDeviceData() = default;

GpuDeviceData::GpuDeviceData(
    int32_t utilization__arg ,
    int64_t memoryUsage__arg ,
    int64_t vramApertureUsage__arg ,
    int32_t temperature__arg ,
    int32_t powerUsage__arg ,
    int32_t graphicsClockSpeed__arg ,
    int32_t smClockSpeed__arg ,
    int32_t memClockSpeed__arg ,
    int32_t videoClockSpeed__arg 
    ):
    utilization_ { std::move(utilization__arg) } // NOLINT
      ,
    memoryUsage_ { std::move(memoryUsage__arg) } // NOLINT
      ,
    vramApertureUsage_ { std::move(vramApertureUsage__arg) } // NOLINT
      ,
    temperature_ { std::move(temperature__arg) } // NOLINT
      ,
    powerUsage_ { std::move(powerUsage__arg) } // NOLINT
      ,
    graphicsClockSpeed_ { std::move(graphicsClockSpeed__arg) } // NOLINT
      ,
    smClockSpeed_ { std::move(smClockSpeed__arg) } // NOLINT
      ,
    memClockSpeed_ { std::move(memClockSpeed__arg) } // NOLINT
      ,
    videoClockSpeed_ { std::move(videoClockSpeed__arg) } // NOLINT
      
    {
}

GpuDeviceData::GpuDeviceData(const GpuDeviceData&) = default;

// NOLINTNEXTLINE
GpuDeviceData::GpuDeviceData(GpuDeviceData&&) = default;

GpuDeviceData& GpuDeviceData::operator=(const GpuDeviceData&) = default;

// NOLINTNEXTLINE
GpuDeviceData& GpuDeviceData::operator=(GpuDeviceData&&) = default;

void GpuDeviceData::toCsv(std::ostream& out) const
{
    out.precision(16);
    out << std::boolalpha;
    out << utilization_;out << ",";
    out << memoryUsage_;out << ",";
    out << vramApertureUsage_;out << ",";
    out << temperature_;out << ",";
    out << powerUsage_;out << ",";
    out << graphicsClockSpeed_;out << ",";
    out << smClockSpeed_;out << ",";
    out << memClockSpeed_;out << ",";
    out << videoClockSpeed_;
    
}

std::stringstream GpuDeviceData::toStringstream() const
{
    std::stringstream out;
    toCsv(out);
    return out;
}

template<class Archive>
    void GpuDeviceData::serialize(Archive& ar, [[maybe_unused]] unsigned int version)
    {

        ar& CEREAL_NVP(utilization_);
        ar& CEREAL_NVP(memoryUsage_);
        ar& CEREAL_NVP(vramApertureUsage_);
        ar& CEREAL_NVP(temperature_);
        ar& CEREAL_NVP(powerUsage_);
        ar& CEREAL_NVP(graphicsClockSpeed_);
        ar& CEREAL_NVP(smClockSpeed_);
        ar& CEREAL_NVP(memClockSpeed_);
        ar& CEREAL_NVP(videoClockSpeed_);
    }


template void GpuDeviceData::serialize(cereal::BinaryInputArchive& ar, unsigned int version);
template void GpuDeviceData::serialize(cereal::BinaryOutputArchive& ar, unsigned int version);
template void GpuDeviceData::serialize(cereal::IntrospectionArchive& ar, unsigned int version);
template void GpuDeviceData::serialize(cereal::JSONInputArchive& ar, unsigned int version);
template void GpuDeviceData::serialize(cereal::JSONOutputArchive& ar, unsigned int version);
template void GpuDeviceData::serialize(cereal::PortableBinaryInputArchive& ar, unsigned int version);
template void GpuDeviceData::serialize(cereal::PortableBinaryOutputArchive& ar, unsigned int version);

std::ostream& operator<< (std::ostream& os, const GpuDeviceData& gpudevicedata)
{
    gpudevicedata.toCsv(os);
    return os;
}

bool operator== (const GpuDeviceData &lhs, const GpuDeviceData &rhs)
{
    return (  lhs.utilization_== rhs.utilization_ &&
     lhs.memoryUsage_== rhs.memoryUsage_ &&
     lhs.vramApertureUsage_== rhs.vramApertureUsage_ &&
     lhs.temperature_== rhs.temperature_ &&
     lhs.powerUsage_== rhs.powerUsage_ &&
     lhs.graphicsClockSpeed_== rhs.graphicsClockSpeed_ &&
     lhs.smClockSpeed_== rhs.smClockSpeed_ &&
     lhs.memClockSpeed_== rhs.memClockSpeed_ &&
     lhs.videoClockSpeed_== rhs.videoClockSpeed_ 
    );
}

std::size_t GpuDeviceData::hash() const noexcept
{
    std::size_t result = 17;
    result = result * 31 + std::hash<int32_t>()(utilization_);
    result = result * 31 + std::hash<int64_t>()(memoryUsage_);
    result = result * 31 + std::hash<int64_t>()(vramApertureUsage_);
    result = result * 31 + std::hash<int32_t>()(temperature_);
    result = result * 31 + std::hash<int32_t>()(powerUsage_);
    result = result * 31 + std::hash<int32_t>()(graphicsClockSpeed_);
    result = result * 31 + std::hash<int32_t>()(smClockSpeed_);
    result = result * 31 + std::hash<int32_t>()(memClockSpeed_);
    result = result * 31 + std::hash<int32_t>()(videoClockSpeed_);
    return result;
}

 } // namespace av
 } // namespace msg


CEREAL_BIND_TO_ARCHIVES(av::msg::GpuDeviceData)

 namespace av {
 namespace msg {


GpuResource::GpuResource() = default;

GpuResource::GpuResource(
    std::string uuid_arg ,
    GpuDeviceData resource_arg 
    ):
    uuid { std::move(uuid_arg) } // NOLINT
      ,
    resource { std::move(resource_arg) } // NOLINT
      
    {
}

GpuResource::GpuResource(const GpuResource&) = default;

// NOLINTNEXTLINE
GpuResource::GpuResource(GpuResource&&) = default;

GpuResource& GpuResource::operator=(const GpuResource&) = default;

// NOLINTNEXTLINE
GpuResource& GpuResource::operator=(GpuResource&&) = default;

void GpuResource::toCsv(std::ostream& out) const
{
    out.precision(16);
    out << std::boolalpha;
    out << "\"";
        out << uuid;
        out << "\"";out << ",";
    out << resource;
    
}

std::stringstream GpuResource::toStringstream() const
{
    std::stringstream out;
    toCsv(out);
    return out;
}

template<class Archive>
    void GpuResource::serialize(Archive& ar, [[maybe_unused]] unsigned int version)
    {

        ar& CEREAL_NVP(uuid);
        ar& CEREAL_NVP(resource);
    }


template void GpuResource::serialize(cereal::BinaryInputArchive& ar, unsigned int version);
template void GpuResource::serialize(cereal::BinaryOutputArchive& ar, unsigned int version);
template void GpuResource::serialize(cereal::IntrospectionArchive& ar, unsigned int version);
template void GpuResource::serialize(cereal::JSONInputArchive& ar, unsigned int version);
template void GpuResource::serialize(cereal::JSONOutputArchive& ar, unsigned int version);
template void GpuResource::serialize(cereal::PortableBinaryInputArchive& ar, unsigned int version);
template void GpuResource::serialize(cereal::PortableBinaryOutputArchive& ar, unsigned int version);

std::ostream& operator<< (std::ostream& os, const GpuResource& gpuresource)
{
    gpuresource.toCsv(os);
    return os;
}

bool operator== (const GpuResource &lhs, const GpuResource &rhs)
{
    return (  lhs.uuid== rhs.uuid &&
     lhs.resource== rhs.resource 
    );
}

std::size_t GpuResource::hash() const noexcept
{
    std::size_t result = 17;
    result = result * 31 + std::hash<std::string>()(uuid);
    result = result * 31 + std::hash<GpuDeviceData>()(resource);
    return result;
}

 } // namespace av
 } // namespace msg


CEREAL_BIND_TO_ARCHIVES(av::msg::GpuResource)

 namespace av {
 namespace msg {


SystemResourceData::SystemResourceData() = default;

SystemResourceData::SystemResourceData(
    std::string hostname__arg ,
    std::vector<double> cpuUtilization__arg ,
    double cpuLoad__arg ,
    uint64_t memUtilization__arg ,
    double gpuUtilization__arg ,
    std::vector<double> cpuFreq__arg ,
    std::vector<GpuResource> gpuResources__arg ,
    SystemMemInfo memInfo__arg ,
    double cpuTemperature__arg 
    ):
    hostname_ { std::move(hostname__arg) } // NOLINT
      ,
    cpuUtilization_ { std::move(cpuUtilization__arg) } // NOLINT
      ,
    cpuLoad_ { std::move(cpuLoad__arg) } // NOLINT
      ,
    memUtilization_ { std::move(memUtilization__arg) } // NOLINT
      ,
    gpuUtilization_ { std::move(gpuUtilization__arg) } // NOLINT
      ,
    cpuFreq_ { std::move(cpuFreq__arg) } // NOLINT
      ,
    gpuResources_ { std::move(gpuResources__arg) } // NOLINT
      ,
    memInfo_ { std::move(memInfo__arg) } // NOLINT
      ,
    cpuTemperature_ { std::move(cpuTemperature__arg) } // NOLINT
      
    {
}

SystemResourceData::SystemResourceData(const SystemResourceData&) = default;

// NOLINTNEXTLINE
SystemResourceData::SystemResourceData(SystemResourceData&&) = default;

SystemResourceData& SystemResourceData::operator=(const SystemResourceData&) = default;

// NOLINTNEXTLINE
SystemResourceData& SystemResourceData::operator=(SystemResourceData&&) = default;

void SystemResourceData::toCsv(std::ostream& out) const
{
    out.precision(16);
    out << std::boolalpha;
    out << "\"";
        out << hostname_;
        out << "\"";out << ",";
    // Enclose sequence in double quotations and curly brackets to keep CSV line length uniform
        
        out << "\"{{";
        
        if (!cpuUtilization_.empty())
        {
            std::copy(cpuUtilization_.begin(), cpuUtilization_.end() - 1, std::ostream_iterator<double>(out, "},{"));
            std::copy(cpuUtilization_.end()-1, cpuUtilization_.end(), std::ostream_iterator<double>(out, ""));
        }
        out << "}}\"";out << ",";
    out << cpuLoad_;out << ",";
    out << memUtilization_;out << ",";
    out << gpuUtilization_;out << ",";
    // Enclose sequence in double quotations and curly brackets to keep CSV line length uniform
        
        out << "\"{{";
        
        if (!cpuFreq_.empty())
        {
            std::copy(cpuFreq_.begin(), cpuFreq_.end() - 1, std::ostream_iterator<double>(out, "},{"));
            std::copy(cpuFreq_.end()-1, cpuFreq_.end(), std::ostream_iterator<double>(out, ""));
        }
        out << "}}\"";out << ",";
    // Enclose sequence in double quotations and curly brackets to keep CSV line length uniform
        
        out << "\"{{";
        
        if (!gpuResources_.empty())
        {
            std::copy(gpuResources_.begin(), gpuResources_.end() - 1, std::ostream_iterator<GpuResource>(out, "},{"));
            std::copy(gpuResources_.end()-1, gpuResources_.end(), std::ostream_iterator<GpuResource>(out, ""));
        }
        out << "}}\"";out << ",";
    out << memInfo_;out << ",";
    out << cpuTemperature_;
    
}

std::stringstream SystemResourceData::toStringstream() const
{
    std::stringstream out;
    toCsv(out);
    return out;
}

template<class Archive>
    void SystemResourceData::serialize(Archive& ar, const unsigned int version)
{
    if (version >= 1)
    {
        ar& CEREAL_NVP(hostname_);
        ar& CEREAL_NVP(cpuUtilization_);
        ar& CEREAL_NVP(cpuLoad_);
        ar& CEREAL_NVP(memUtilization_);
        ar& CEREAL_NVP(gpuUtilization_);
    }

    if (version >= 2)
    {
        ar& CEREAL_NVP(cpuFreq_);
    }

    if (version >= 3)
    {
        ar& CEREAL_NVP(gpuResources_);
    }

    if (version >= 4u)
    {
        ar& CEREAL_NVP(memInfo_);
    }

    if (version >= 5u)
    {
        ar& CEREAL_NVP(cpuTemperature_);
    }
}



template void SystemResourceData::serialize(cereal::BinaryInputArchive& ar, unsigned int version);
template void SystemResourceData::serialize(cereal::BinaryOutputArchive& ar, unsigned int version);
template void SystemResourceData::serialize(cereal::IntrospectionArchive& ar, unsigned int version);
template void SystemResourceData::serialize(cereal::JSONInputArchive& ar, unsigned int version);
template void SystemResourceData::serialize(cereal::JSONOutputArchive& ar, unsigned int version);
template void SystemResourceData::serialize(cereal::PortableBinaryInputArchive& ar, unsigned int version);
template void SystemResourceData::serialize(cereal::PortableBinaryOutputArchive& ar, unsigned int version);

std::ostream& operator<< (std::ostream& os, const SystemResourceData& systemresourcedata)
{
    systemresourcedata.toCsv(os);
    return os;
}

bool operator== (const SystemResourceData &lhs, const SystemResourceData &rhs)
{
    return (  lhs.hostname_== rhs.hostname_ &&
     lhs.cpuUtilization_== rhs.cpuUtilization_ &&
     lhs.cpuLoad_== rhs.cpuLoad_ &&
     lhs.memUtilization_== rhs.memUtilization_ &&
     lhs.gpuUtilization_== rhs.gpuUtilization_ &&
     lhs.cpuFreq_== rhs.cpuFreq_ &&
     lhs.gpuResources_== rhs.gpuResources_ &&
     lhs.memInfo_== rhs.memInfo_ &&
     lhs.cpuTemperature_== rhs.cpuTemperature_ 
    );
}

std::size_t SystemResourceData::hash() const noexcept
{
    std::size_t result = 17;
    result = result * 31 + std::hash<std::string>()(hostname_);
    result = result * 31 + hashVector(cpuUtilization_);
    result = result * 31 + std::hash<double>()(cpuLoad_);
    result = result * 31 + std::hash<uint64_t>()(memUtilization_);
    result = result * 31 + std::hash<double>()(gpuUtilization_);
    result = result * 31 + hashVector(cpuFreq_);
    result = result * 31 + hashVector(gpuResources_);
    result = result * 31 + std::hash<SystemMemInfo>()(memInfo_);
    result = result * 31 + std::hash<double>()(cpuTemperature_);
    return result;
}

 } // namespace av
 } // namespace msg


CEREAL_BIND_TO_ARCHIVES(av::msg::SystemResourceData)

 namespace av {
 namespace msg {


GpuProcessData::GpuProcessData() = default;

GpuProcessData::GpuProcessData(
    int64_t gpuMemoryUsage__arg ,
    int32_t decoderUtil__arg ,
    int32_t encoderUtil__arg ,
    int32_t framebufferMemoryUtil__arg ,
    int32_t streamMultiprocessorUtil__arg 
    ):
    gpuMemoryUsage_ { std::move(gpuMemoryUsage__arg) } // NOLINT
      ,
    decoderUtil_ { std::move(decoderUtil__arg) } // NOLINT
      ,
    encoderUtil_ { std::move(encoderUtil__arg) } // NOLINT
      ,
    framebufferMemoryUtil_ { std::move(framebufferMemoryUtil__arg) } // NOLINT
      ,
    streamMultiprocessorUtil_ { std::move(streamMultiprocessorUtil__arg) } // NOLINT
      
    {
}

GpuProcessData::GpuProcessData(const GpuProcessData&) = default;

// NOLINTNEXTLINE
GpuProcessData::GpuProcessData(GpuProcessData&&) = default;

GpuProcessData& GpuProcessData::operator=(const GpuProcessData&) = default;

// NOLINTNEXTLINE
GpuProcessData& GpuProcessData::operator=(GpuProcessData&&) = default;

void GpuProcessData::toCsv(std::ostream& out) const
{
    out.precision(16);
    out << std::boolalpha;
    out << gpuMemoryUsage_;out << ",";
    out << decoderUtil_;out << ",";
    out << encoderUtil_;out << ",";
    out << framebufferMemoryUtil_;out << ",";
    out << streamMultiprocessorUtil_;
    
}

std::stringstream GpuProcessData::toStringstream() const
{
    std::stringstream out;
    toCsv(out);
    return out;
}

template<class Archive>
    void GpuProcessData::serialize(Archive& ar, [[maybe_unused]] unsigned int version)
    {

        ar& CEREAL_NVP(gpuMemoryUsage_);
        ar& CEREAL_NVP(decoderUtil_);
        ar& CEREAL_NVP(encoderUtil_);
        ar& CEREAL_NVP(framebufferMemoryUtil_);
        ar& CEREAL_NVP(streamMultiprocessorUtil_);
    }


template void GpuProcessData::serialize(cereal::BinaryInputArchive& ar, unsigned int version);
template void GpuProcessData::serialize(cereal::BinaryOutputArchive& ar, unsigned int version);
template void GpuProcessData::serialize(cereal::IntrospectionArchive& ar, unsigned int version);
template void GpuProcessData::serialize(cereal::JSONInputArchive& ar, unsigned int version);
template void GpuProcessData::serialize(cereal::JSONOutputArchive& ar, unsigned int version);
template void GpuProcessData::serialize(cereal::PortableBinaryInputArchive& ar, unsigned int version);
template void GpuProcessData::serialize(cereal::PortableBinaryOutputArchive& ar, unsigned int version);

std::ostream& operator<< (std::ostream& os, const GpuProcessData& gpuprocessdata)
{
    gpuprocessdata.toCsv(os);
    return os;
}

bool operator== (const GpuProcessData &lhs, const GpuProcessData &rhs)
{
    return (  lhs.gpuMemoryUsage_== rhs.gpuMemoryUsage_ &&
     lhs.decoderUtil_== rhs.decoderUtil_ &&
     lhs.encoderUtil_== rhs.encoderUtil_ &&
     lhs.framebufferMemoryUtil_== rhs.framebufferMemoryUtil_ &&
     lhs.streamMultiprocessorUtil_== rhs.streamMultiprocessorUtil_ 
    );
}

std::size_t GpuProcessData::hash() const noexcept
{
    std::size_t result = 17;
    result = result * 31 + std::hash<int64_t>()(gpuMemoryUsage_);
    result = result * 31 + std::hash<int32_t>()(decoderUtil_);
    result = result * 31 + std::hash<int32_t>()(encoderUtil_);
    result = result * 31 + std::hash<int32_t>()(framebufferMemoryUtil_);
    result = result * 31 + std::hash<int32_t>()(streamMultiprocessorUtil_);
    return result;
}

 } // namespace av
 } // namespace msg


CEREAL_BIND_TO_ARCHIVES(av::msg::GpuProcessData)

 namespace av {
 namespace msg {


TaskHealthData::TaskHealthData() = default;

TaskHealthData::TaskHealthData(
    std::string instanceName__arg ,
    std::string executableName__arg ,
    double cpuUtilization__arg ,
    uint64_t memUtilizationVirtual__arg ,
    uint64_t memUtilizationRSS__arg ,
    GpuProcessData gpuData__arg ,
    int32_t status__arg 
    ):
    instanceName_ { std::move(instanceName__arg) } // NOLINT
      ,
    executableName_ { std::move(executableName__arg) } // NOLINT
      ,
    cpuUtilization_ { std::move(cpuUtilization__arg) } // NOLINT
      ,
    memUtilizationVirtual_ { std::move(memUtilizationVirtual__arg) } // NOLINT
      ,
    memUtilizationRSS_ { std::move(memUtilizationRSS__arg) } // NOLINT
      ,
    gpuData_ { std::move(gpuData__arg) } // NOLINT
      ,
    status_ { std::move(status__arg) } // NOLINT
      
    {
}

TaskHealthData::TaskHealthData(const TaskHealthData&) = default;

// NOLINTNEXTLINE
TaskHealthData::TaskHealthData(TaskHealthData&&) = default;

TaskHealthData& TaskHealthData::operator=(const TaskHealthData&) = default;

// NOLINTNEXTLINE
TaskHealthData& TaskHealthData::operator=(TaskHealthData&&) = default;

void TaskHealthData::toCsv(std::ostream& out) const
{
    out.precision(16);
    out << std::boolalpha;
    out << "\"";
        out << instanceName_;
        out << "\"";out << ",";
    out << "\"";
        out << executableName_;
        out << "\"";out << ",";
    out << cpuUtilization_;out << ",";
    out << memUtilizationVirtual_;out << ",";
    out << memUtilizationRSS_;out << ",";
    out << gpuData_;out << ",";
    out << status_;
    
}

std::stringstream TaskHealthData::toStringstream() const
{
    std::stringstream out;
    toCsv(out);
    return out;
}

template<class Archive>
    void TaskHealthData::serialize(Archive& ar, const unsigned int version)
{
    if (version >= 1u)
    {
        ar& CEREAL_NVP(instanceName_);
        ar& CEREAL_NVP(executableName_);
        ar& CEREAL_NVP(cpuUtilization_);
        ar& CEREAL_NVP(memUtilizationVirtual_);
        ar& CEREAL_NVP(status_);
    }

    if (version >= 2u)
    {
        ar& CEREAL_NVP(memUtilizationRSS_);
    }
    
    if (version >= 3u)
    {
        ar& CEREAL_NVP(gpuData_);
    }
}



template void TaskHealthData::serialize(cereal::BinaryInputArchive& ar, unsigned int version);
template void TaskHealthData::serialize(cereal::BinaryOutputArchive& ar, unsigned int version);
template void TaskHealthData::serialize(cereal::IntrospectionArchive& ar, unsigned int version);
template void TaskHealthData::serialize(cereal::JSONInputArchive& ar, unsigned int version);
template void TaskHealthData::serialize(cereal::JSONOutputArchive& ar, unsigned int version);
template void TaskHealthData::serialize(cereal::PortableBinaryInputArchive& ar, unsigned int version);
template void TaskHealthData::serialize(cereal::PortableBinaryOutputArchive& ar, unsigned int version);

std::ostream& operator<< (std::ostream& os, const TaskHealthData& taskhealthdata)
{
    taskhealthdata.toCsv(os);
    return os;
}

bool operator== (const TaskHealthData &lhs, const TaskHealthData &rhs)
{
    return (  lhs.instanceName_== rhs.instanceName_ &&
     lhs.executableName_== rhs.executableName_ &&
     lhs.cpuUtilization_== rhs.cpuUtilization_ &&
     lhs.memUtilizationVirtual_== rhs.memUtilizationVirtual_ &&
     lhs.memUtilizationRSS_== rhs.memUtilizationRSS_ &&
     lhs.gpuData_== rhs.gpuData_ &&
     lhs.status_== rhs.status_ 
    );
}

std::size_t TaskHealthData::hash() const noexcept
{
    std::size_t result = 17;
    result = result * 31 + std::hash<std::string>()(instanceName_);
    result = result * 31 + std::hash<std::string>()(executableName_);
    result = result * 31 + std::hash<double>()(cpuUtilization_);
    result = result * 31 + std::hash<uint64_t>()(memUtilizationVirtual_);
    result = result * 31 + std::hash<uint64_t>()(memUtilizationRSS_);
    result = result * 31 + std::hash<GpuProcessData>()(gpuData_);
    result = result * 31 + std::hash<int32_t>()(status_);
    return result;
}

 } // namespace av
 } // namespace msg


CEREAL_BIND_TO_ARCHIVES(av::msg::TaskHealthData)




 namespace av {
 namespace msg {


SystemHealthMessage_avt::SystemHealthMessage_avt() = default;

SystemHealthMessage_avt::SystemHealthMessage_avt(
    SystemResourceData systemResources__arg ,
    std::vector<TaskHealthData> taskHealth__arg ,
    int64_t time__arg 
    ):
    systemResources_ { std::move(systemResources__arg) } // NOLINT
      ,taskHealth_ { std::move(taskHealth__arg) } // NOLINT
      ,time_ { std::move(time__arg) } // NOLINT
      {
}

SystemHealthMessage_avt::SystemHealthMessage_avt(const SystemHealthMessage_avt&) = default;

// NOLINTNEXTLINE
SystemHealthMessage_avt::SystemHealthMessage_avt(SystemHealthMessage_avt&&) = default;

SystemHealthMessage_avt& SystemHealthMessage_avt::operator=(const SystemHealthMessage_avt&) = default;

// NOLINTNEXTLINE
SystemHealthMessage_avt& SystemHealthMessage_avt::operator=(SystemHealthMessage_avt&&) = default;

void SystemHealthMessage_avt::toCsv(std::ostream& out) const
{
    out.precision(16);
    out << std::boolalpha;
    out << systemResources_;out << ",";
    // Enclose sequence in double quotations and curly brackets to keep CSV line length uniform
        
        out << "\"{{";
        
        if (!taskHealth_.empty())
        {
            std::copy(taskHealth_.begin(), taskHealth_.end() - 1, std::ostream_iterator<TaskHealthData>(out, "},{"));
            std::copy(taskHealth_.end()-1, taskHealth_.end(), std::ostream_iterator<TaskHealthData>(out, ""));
        }
        out << "}}\"";out << ",";
    out << time_;
    
}

std::stringstream SystemHealthMessage_avt::toStringstream() const
{
    std::stringstream out;
    toCsv(out);
    return out;
}

template<class Archive>
    void SystemHealthMessage_avt::serialize(Archive& ar, const unsigned int /*version*/)
{
    ar& CEREAL_NVP(systemResources_);
    ar& CEREAL_NVP(taskHealth_);
    ar& CEREAL_NVP(time_);
}



template void SystemHealthMessage_avt::serialize(cereal::BinaryInputArchive& ar, unsigned int version);
template void SystemHealthMessage_avt::serialize(cereal::BinaryOutputArchive& ar, unsigned int version);
template void SystemHealthMessage_avt::serialize(cereal::IntrospectionArchive& ar, unsigned int version);
template void SystemHealthMessage_avt::serialize(cereal::JSONInputArchive& ar, unsigned int version);
template void SystemHealthMessage_avt::serialize(cereal::JSONOutputArchive& ar, unsigned int version);
template void SystemHealthMessage_avt::serialize(cereal::PortableBinaryInputArchive& ar, unsigned int version);
template void SystemHealthMessage_avt::serialize(cereal::PortableBinaryOutputArchive& ar, unsigned int version);

std::ostream& operator<< (std::ostream& os, const SystemHealthMessage_avt& systemhealthmessage_avt)
{
    systemhealthmessage_avt.toCsv(os);
    return os;
}

bool operator== (const SystemHealthMessage_avt &lhs, const SystemHealthMessage_avt &rhs)
{
    return (  lhs.systemResources_== rhs.systemResources_ &&
     lhs.taskHealth_== rhs.taskHealth_ &&
     lhs.time_== rhs.time_ 
    );
}

std::size_t SystemHealthMessage_avt::hash() const noexcept
{
    std::size_t result = 17;
    result = result * 31 + std::hash<SystemResourceData>()(systemResources_);
    result = result * 31 + hashVector(taskHealth_);
    result = result * 31 + std::hash<int64_t>()(time_);
    return result;
}

 } // namespace av
 } // namespace msg


CEREAL_BIND_TO_ARCHIVES(av::msg::SystemHealthMessage_avt)

