#include <interfaces/TimePoint/TimePoint.h>

#include <iterator>
#include <type_traits>

#include <cereal/cereal.hpp>
#include <cereal/archives/json.hpp>
#include <cereal/archives/binary.hpp>

#include <cereal/types/map.hpp>
#include <cereal/types/vector.hpp>
#include <cereal/types/array.hpp>
#include <cereal/types/string.hpp>
#include <cereal/types/complex.hpp>

#include <ODatum/IntrospectionArchive.h>
#include <ODatum/Hash.h>





 namespace av {
 namespace msg {


TimePoint::TimePoint() = default;

TimePoint::TimePoint(
    int64_t privateData_arg 
    ):
    privateData { std::move(privateData_arg) } // NOLINT
      {
}

TimePoint::TimePoint(const TimePoint&) = default;

// NOLINTNEXTLINE
TimePoint::TimePoint(TimePoint&&) = default;

TimePoint& TimePoint::operator=(const TimePoint&) = default;

// NOLINTNEXTLINE
TimePoint& TimePoint::operator=(TimePoint&&) = default;

void TimePoint::toCsv(std::ostream& out) const
{
    out.precision(16);
    out << std::boolalpha;
    out << privateData;
    
}

std::stringstream TimePoint::toStringstream() const
{
    std::stringstream out;
    toCsv(out);
    return out;
}

template<class Archive>
    void TimePoint::serialize(Archive& ar, [[maybe_unused]] unsigned int version)
{
    ar& CEREAL_NVP(privateData);
}

// Custom logic to allow implicit conversion between av::TimePoint and av::msg::TimePoint

// Constructor from av::TimePoint to av::msg::TimePoint
TimePoint::TimePoint(const av::TimePoint& timepoint)
    : privateData{timepoint.time_since_epoch().count()}
{
}

// Assignment operator from av::TimePoint to av::msg::TimePoint
TimePoint& TimePoint::operator=(const av::TimePoint& timepoint)
{
    privateData = timepoint.time_since_epoch().count();
    return *this;
}

// Implicit conversion function from av::msg::TimePoint to av::TimePoint
TimePoint::operator av::TimePoint() const
{
    return av::TimePoint(av::Duration(privateData));
}

// Implicit conversion function from av::msg::TimePoint to av::Duration
TimePoint::operator av::Duration() const
{
    return av::Duration(privateData);
}



template void TimePoint::serialize(cereal::BinaryInputArchive& ar, unsigned int version);
template void TimePoint::serialize(cereal::BinaryOutputArchive& ar, unsigned int version);
template void TimePoint::serialize(cereal::IntrospectionArchive& ar, unsigned int version);
template void TimePoint::serialize(cereal::JSONInputArchive& ar, unsigned int version);
template void TimePoint::serialize(cereal::JSONOutputArchive& ar, unsigned int version);
template void TimePoint::serialize(cereal::PortableBinaryInputArchive& ar, unsigned int version);
template void TimePoint::serialize(cereal::PortableBinaryOutputArchive& ar, unsigned int version);

std::ostream& operator<< (std::ostream& os, const TimePoint& timepoint)
{
    timepoint.toCsv(os);
    return os;
}

bool operator== (const TimePoint &lhs, const TimePoint &rhs)
{
    return (  lhs.privateData== rhs.privateData 
    );
}

std::size_t TimePoint::hash() const noexcept
{
    std::size_t result = 17;
    result = result * 31 + std::hash<int64_t>()(privateData);
    return result;
}

 } // namespace av
 } // namespace msg


CEREAL_BIND_TO_ARCHIVES(av::msg::TimePoint)

