#include <interfaces/TimeSync/TimeSync.h>

#include <iterator>
#include <type_traits>

#include <cereal/cereal.hpp>
#include <cereal/archives/json.hpp>
#include <cereal/archives/binary.hpp>

#include <cereal/types/map.hpp>
#include <cereal/types/vector.hpp>
#include <cereal/types/array.hpp>
#include <cereal/types/string.hpp>
#include <cereal/types/complex.hpp>

#include <ODatum/IntrospectionArchive.h>
#include <ODatum/Hash.h>





 namespace av {
 namespace msg {


TimeSync::TimeSync() = default;

TimeSync::TimeSync(
    uint32_t requestId_arg ,
    std::string hostName_arg ,
    TimePoint requestTimeStamp_arg ,
    TimePoint replyTimeStamp_arg 
    ):
    requestId { std::move(requestId_arg) } // NOLINT
      ,hostName { std::move(hostName_arg) } // NOLINT
      ,requestTimeStamp { std::move(requestTimeStamp_arg) } // NOLINT
      ,replyTimeStamp { std::move(replyTimeStamp_arg) } // NOLINT
      {
}

TimeSync::TimeSync(const TimeSync&) = default;

// NOLINTNEXTLINE
TimeSync::TimeSync(TimeSync&&) = default;

TimeSync& TimeSync::operator=(const TimeSync&) = default;

// NOLINTNEXTLINE
TimeSync& TimeSync::operator=(TimeSync&&) = default;

void TimeSync::toCsv(std::ostream& out) const
{
    out.precision(16);
    out << std::boolalpha;
    out << requestId;out << ",";
    out << "\"";
        out << hostName;
        out << "\"";out << ",";
    out << requestTimeStamp;out << ",";
    out << replyTimeStamp;
    
}

std::stringstream TimeSync::toStringstream() const
{
    std::stringstream out;
    toCsv(out);
    return out;
}

template<class Archive>
    void TimeSync::serialize(Archive& ar, unsigned int /*version*/)
{
	ar& CEREAL_NVP(requestId);
	ar& CEREAL_NVP(hostName);
	
	ar& CEREAL_NVP(requestTimeStamp);
	ar& CEREAL_NVP(replyTimeStamp);
}


template void TimeSync::serialize(cereal::BinaryInputArchive& ar, unsigned int version);
template void TimeSync::serialize(cereal::BinaryOutputArchive& ar, unsigned int version);
template void TimeSync::serialize(cereal::IntrospectionArchive& ar, unsigned int version);
template void TimeSync::serialize(cereal::JSONInputArchive& ar, unsigned int version);
template void TimeSync::serialize(cereal::JSONOutputArchive& ar, unsigned int version);
template void TimeSync::serialize(cereal::PortableBinaryInputArchive& ar, unsigned int version);
template void TimeSync::serialize(cereal::PortableBinaryOutputArchive& ar, unsigned int version);

std::ostream& operator<< (std::ostream& os, const TimeSync& timesync)
{
    timesync.toCsv(os);
    return os;
}

bool operator== (const TimeSync &lhs, const TimeSync &rhs)
{
    return (  lhs.requestId== rhs.requestId &&
     lhs.hostName== rhs.hostName &&
     lhs.requestTimeStamp== rhs.requestTimeStamp &&
     lhs.replyTimeStamp== rhs.replyTimeStamp 
    );
}

std::size_t TimeSync::hash() const noexcept
{
    std::size_t result = 17;
    result = result * 31 + std::hash<uint32_t>()(requestId);
    result = result * 31 + std::hash<std::string>()(hostName);
    result = result * 31 + 0;
    result = result * 31 + 0;
    return result;
}

 } // namespace av
 } // namespace msg


CEREAL_BIND_TO_ARCHIVES(av::msg::TimeSync)

