#include <interfaces/Timestamp128/Timestamp128.h>

#include <iterator>
#include <type_traits>

#include <cereal/cereal.hpp>
#include <cereal/archives/json.hpp>
#include <cereal/archives/binary.hpp>

#include <cereal/types/map.hpp>
#include <cereal/types/vector.hpp>
#include <cereal/types/array.hpp>
#include <cereal/types/string.hpp>
#include <cereal/types/complex.hpp>

#include <ODatum/IntrospectionArchive.h>
#include <ODatum/Hash.h>




Timestamp128::Timestamp128() = default;

Timestamp128::Timestamp128(
    int64_t sec_arg ,
    int64_t nsec_arg 
    ):
    sec { std::move(sec_arg) } // NOLINT
      ,
    nsec { std::move(nsec_arg) } // NOLINT
      
    {
}

Timestamp128::Timestamp128(const Timestamp128&) = default;

// NOLINTNEXTLINE
Timestamp128::Timestamp128(Timestamp128&&) = default;

Timestamp128& Timestamp128::operator=(const Timestamp128&) = default;

// NOLINTNEXTLINE
Timestamp128& Timestamp128::operator=(Timestamp128&&) = default;

void Timestamp128::toCsv(std::ostream& out) const
{
    out.precision(16);
    out << std::boolalpha;
    out << sec;out << ",";
    out << nsec;
    
}

std::stringstream Timestamp128::toStringstream() const
{
    std::stringstream out;
    toCsv(out);
    return out;
}

template<class Archive>
    void Timestamp128::serialize(Archive& ar, [[maybe_unused]] unsigned int version)
    {

        ar& CEREAL_NVP(sec);
        ar& CEREAL_NVP(nsec);
    }


template void Timestamp128::serialize(cereal::BinaryInputArchive& ar, unsigned int version);
template void Timestamp128::serialize(cereal::BinaryOutputArchive& ar, unsigned int version);
template void Timestamp128::serialize(cereal::IntrospectionArchive& ar, unsigned int version);
template void Timestamp128::serialize(cereal::JSONInputArchive& ar, unsigned int version);
template void Timestamp128::serialize(cereal::JSONOutputArchive& ar, unsigned int version);
template void Timestamp128::serialize(cereal::PortableBinaryInputArchive& ar, unsigned int version);
template void Timestamp128::serialize(cereal::PortableBinaryOutputArchive& ar, unsigned int version);

std::ostream& operator<< (std::ostream& os, const Timestamp128& timestamp128)
{
    timestamp128.toCsv(os);
    return os;
}

bool operator== (const Timestamp128 &lhs, const Timestamp128 &rhs)
{
    return (  lhs.sec== rhs.sec &&
     lhs.nsec== rhs.nsec 
    );
}

std::size_t Timestamp128::hash() const noexcept
{
    std::size_t result = 17;
    result = result * 31 + std::hash<int64_t>()(sec);
    result = result * 31 + std::hash<int64_t>()(nsec);
    return result;
}



CEREAL_BIND_TO_ARCHIVES(Timestamp128)


