#include <interfaces/TkCrashState/TkCrashState.h>

#include <iterator>
#include <type_traits>

#include <cereal/cereal.hpp>
#include <cereal/archives/json.hpp>
#include <cereal/archives/binary.hpp>

#include <cereal/types/map.hpp>
#include <cereal/types/vector.hpp>
#include <cereal/types/array.hpp>
#include <cereal/types/string.hpp>
#include <cereal/types/complex.hpp>

#include <ODatum/IntrospectionArchive.h>
#include <ODatum/Hash.h>





 namespace av {
 namespace msg {


TkCrashState::TkCrashState() = default;

TkCrashState::TkCrashState(
    bool status_arg ,
    double confidence_arg ,
    TK_MODULE module_arg 
    ):
    status { std::move(status_arg) } // NOLINT
      ,confidence { std::move(confidence_arg) } // NOLINT
      ,module { std::move(module_arg) } // NOLINT
      {
}

TkCrashState::TkCrashState(const TkCrashState&) = default;

// NOLINTNEXTLINE
TkCrashState::TkCrashState(TkCrashState&&) = default;

TkCrashState& TkCrashState::operator=(const TkCrashState&) = default;

// NOLINTNEXTLINE
TkCrashState& TkCrashState::operator=(TkCrashState&&) = default;

void TkCrashState::toCsv(std::ostream& out) const
{
    out.precision(16);
    out << std::boolalpha;
    out << status;out << ",";
    out << confidence;out << ",";
    out << static_cast<uint32_t>(module);
    
}

std::stringstream TkCrashState::toStringstream() const
{
    std::stringstream out;
    toCsv(out);
    return out;
}

template<class Archive>
    void TkCrashState::serialize(Archive& ar, [[maybe_unused]] unsigned int version)
    {

        ar& CEREAL_NVP(status);
        ar& CEREAL_NVP(confidence);
        ar& CEREAL_NVP(module);
    }


template void TkCrashState::serialize(cereal::BinaryInputArchive& ar, unsigned int version);
template void TkCrashState::serialize(cereal::BinaryOutputArchive& ar, unsigned int version);
template void TkCrashState::serialize(cereal::IntrospectionArchive& ar, unsigned int version);
template void TkCrashState::serialize(cereal::JSONInputArchive& ar, unsigned int version);
template void TkCrashState::serialize(cereal::JSONOutputArchive& ar, unsigned int version);
template void TkCrashState::serialize(cereal::PortableBinaryInputArchive& ar, unsigned int version);
template void TkCrashState::serialize(cereal::PortableBinaryOutputArchive& ar, unsigned int version);

std::ostream& operator<< (std::ostream& os, const TkCrashState& tkcrashstate)
{
    tkcrashstate.toCsv(os);
    return os;
}

bool operator== (const TkCrashState &lhs, const TkCrashState &rhs)
{
    return (  lhs.status== rhs.status &&
     lhs.confidence== rhs.confidence &&
     lhs.module== rhs.module 
    );
}

std::size_t TkCrashState::hash() const noexcept
{
    std::size_t result = 17;
    result = result * 31 + std::hash<bool>()(status);
    result = result * 31 + std::hash<double>()(confidence);
    result = result * 31 + std::hash<TK_MODULE>()(module);
    return result;
}

 } // namespace av
 } // namespace msg


CEREAL_BIND_TO_ARCHIVES(av::msg::TkCrashState)

