#include <interfaces/TkInertialTcnMessage/TkInertialTcnMessage.h>

#include <iterator>
#include <type_traits>

#include <cereal/cereal.hpp>
#include <cereal/archives/json.hpp>
#include <cereal/archives/binary.hpp>

#include <cereal/types/map.hpp>
#include <cereal/types/vector.hpp>
#include <cereal/types/array.hpp>
#include <cereal/types/string.hpp>
#include <cereal/types/complex.hpp>

#include <ODatum/IntrospectionArchive.h>
#include <ODatum/Hash.h>


 namespace av {
 namespace msg {


TkInertialTcnInput::TkInertialTcnInput() = default;

TkInertialTcnInput::TkInertialTcnInput(
    TimePoint timestamp_arg ,
    double accel_x_arg ,
    double accel_y_arg ,
    double accel_z_arg 
    ):
    timestamp { std::move(timestamp_arg) } // NOLINT
      ,
    accel_x { std::move(accel_x_arg) } // NOLINT
      ,
    accel_y { std::move(accel_y_arg) } // NOLINT
      ,
    accel_z { std::move(accel_z_arg) } // NOLINT
      
    {
}

TkInertialTcnInput::TkInertialTcnInput(const TkInertialTcnInput&) = default;

// NOLINTNEXTLINE
TkInertialTcnInput::TkInertialTcnInput(TkInertialTcnInput&&) = default;

TkInertialTcnInput& TkInertialTcnInput::operator=(const TkInertialTcnInput&) = default;

// NOLINTNEXTLINE
TkInertialTcnInput& TkInertialTcnInput::operator=(TkInertialTcnInput&&) = default;

void TkInertialTcnInput::toCsv(std::ostream& out) const
{
    out.precision(16);
    out << std::boolalpha;
    out << timestamp;out << ",";
    out << accel_x;out << ",";
    out << accel_y;out << ",";
    out << accel_z;
    
}

std::stringstream TkInertialTcnInput::toStringstream() const
{
    std::stringstream out;
    toCsv(out);
    return out;
}

template<class Archive>
    void TkInertialTcnInput::serialize(Archive& ar, [[maybe_unused]] unsigned int version)
    {

        ar& CEREAL_NVP(timestamp);
        ar& CEREAL_NVP(accel_x);
        ar& CEREAL_NVP(accel_y);
        ar& CEREAL_NVP(accel_z);
    }


template void TkInertialTcnInput::serialize(cereal::BinaryInputArchive& ar, unsigned int version);
template void TkInertialTcnInput::serialize(cereal::BinaryOutputArchive& ar, unsigned int version);
template void TkInertialTcnInput::serialize(cereal::IntrospectionArchive& ar, unsigned int version);
template void TkInertialTcnInput::serialize(cereal::JSONInputArchive& ar, unsigned int version);
template void TkInertialTcnInput::serialize(cereal::JSONOutputArchive& ar, unsigned int version);
template void TkInertialTcnInput::serialize(cereal::PortableBinaryInputArchive& ar, unsigned int version);
template void TkInertialTcnInput::serialize(cereal::PortableBinaryOutputArchive& ar, unsigned int version);

std::ostream& operator<< (std::ostream& os, const TkInertialTcnInput& tkinertialtcninput)
{
    tkinertialtcninput.toCsv(os);
    return os;
}

bool operator== (const TkInertialTcnInput &lhs, const TkInertialTcnInput &rhs)
{
    return (  lhs.timestamp== rhs.timestamp &&
     lhs.accel_x== rhs.accel_x &&
     lhs.accel_y== rhs.accel_y &&
     lhs.accel_z== rhs.accel_z 
    );
}

std::size_t TkInertialTcnInput::hash() const noexcept
{
    std::size_t result = 17;
    result = result * 31 + 0;
    result = result * 31 + std::hash<double>()(accel_x);
    result = result * 31 + std::hash<double>()(accel_y);
    result = result * 31 + std::hash<double>()(accel_z);
    return result;
}

 } // namespace av
 } // namespace msg


CEREAL_BIND_TO_ARCHIVES(av::msg::TkInertialTcnInput)




 namespace av {
 namespace msg {


TkInertialTcnMessage::TkInertialTcnMessage() = default;

TkInertialTcnMessage::TkInertialTcnMessage(
    TimePoint timestamp_arg ,
    TkCrashState state_arg ,
    TkObjectInfo ego_last_known_arg ,
    std::vector<TkInertialTcnInput> inputs_arg 
    ):
    timestamp { std::move(timestamp_arg) } // NOLINT
      ,state { std::move(state_arg) } // NOLINT
      ,ego_last_known { std::move(ego_last_known_arg) } // NOLINT
      ,inputs { std::move(inputs_arg) } // NOLINT
      {
}

TkInertialTcnMessage::TkInertialTcnMessage(const TkInertialTcnMessage&) = default;

// NOLINTNEXTLINE
TkInertialTcnMessage::TkInertialTcnMessage(TkInertialTcnMessage&&) = default;

TkInertialTcnMessage& TkInertialTcnMessage::operator=(const TkInertialTcnMessage&) = default;

// NOLINTNEXTLINE
TkInertialTcnMessage& TkInertialTcnMessage::operator=(TkInertialTcnMessage&&) = default;

void TkInertialTcnMessage::toCsv(std::ostream& out) const
{
    out.precision(16);
    out << std::boolalpha;
    out << timestamp;out << ",";
    out << state;out << ",";
    out << ego_last_known;out << ",";
    // Enclose sequence in double quotations and curly brackets to keep CSV line length uniform
        
        out << "\"{{";
        
        if (!inputs.empty())
        {
            std::copy(inputs.begin(), inputs.end() - 1, std::ostream_iterator<TkInertialTcnInput>(out, "},{"));
            std::copy(inputs.end()-1, inputs.end(), std::ostream_iterator<TkInertialTcnInput>(out, ""));
        }
        out << "}}\"";
    
}

std::stringstream TkInertialTcnMessage::toStringstream() const
{
    std::stringstream out;
    toCsv(out);
    return out;
}

template<class Archive>
    void TkInertialTcnMessage::serialize(Archive& ar, [[maybe_unused]] unsigned int version)
    {

        ar& CEREAL_NVP(timestamp);
        ar& CEREAL_NVP(state);
        ar& CEREAL_NVP(ego_last_known);
        ar& CEREAL_NVP(inputs);
    }


template void TkInertialTcnMessage::serialize(cereal::BinaryInputArchive& ar, unsigned int version);
template void TkInertialTcnMessage::serialize(cereal::BinaryOutputArchive& ar, unsigned int version);
template void TkInertialTcnMessage::serialize(cereal::IntrospectionArchive& ar, unsigned int version);
template void TkInertialTcnMessage::serialize(cereal::JSONInputArchive& ar, unsigned int version);
template void TkInertialTcnMessage::serialize(cereal::JSONOutputArchive& ar, unsigned int version);
template void TkInertialTcnMessage::serialize(cereal::PortableBinaryInputArchive& ar, unsigned int version);
template void TkInertialTcnMessage::serialize(cereal::PortableBinaryOutputArchive& ar, unsigned int version);

std::ostream& operator<< (std::ostream& os, const TkInertialTcnMessage& tkinertialtcnmessage)
{
    tkinertialtcnmessage.toCsv(os);
    return os;
}

bool operator== (const TkInertialTcnMessage &lhs, const TkInertialTcnMessage &rhs)
{
    return (  lhs.timestamp== rhs.timestamp &&
     lhs.state== rhs.state &&
     lhs.ego_last_known== rhs.ego_last_known &&
     lhs.inputs== rhs.inputs 
    );
}

std::size_t TkInertialTcnMessage::hash() const noexcept
{
    std::size_t result = 17;
    result = result * 31 + 0;
    result = result * 31 + std::hash<TkCrashState>()(state);
    result = result * 31 + std::hash<TkObjectInfo>()(ego_last_known);
    result = result * 31 + hashVector(inputs);
    return result;
}

 } // namespace av
 } // namespace msg


CEREAL_BIND_TO_ARCHIVES(av::msg::TkInertialTcnMessage)

