#include <interfaces/TkInformationFusionMessage/TkInformationFusionMessage.h>

#include <iterator>
#include <type_traits>

#include <cereal/cereal.hpp>
#include <cereal/archives/json.hpp>
#include <cereal/archives/binary.hpp>

#include <cereal/types/map.hpp>
#include <cereal/types/vector.hpp>
#include <cereal/types/array.hpp>
#include <cereal/types/string.hpp>
#include <cereal/types/complex.hpp>

#include <ODatum/IntrospectionArchive.h>
#include <ODatum/Hash.h>


 namespace av {
 namespace msg {


TkFusionObservation::TkFusionObservation() = default;

TkFusionObservation::TkFusionObservation(
    TimePoint timestamp_arg ,
    TkCrashState state_arg 
    ):
    timestamp { std::move(timestamp_arg) } // NOLINT
      ,
    state { std::move(state_arg) } // NOLINT
      
    {
}

TkFusionObservation::TkFusionObservation(const TkFusionObservation&) = default;

// NOLINTNEXTLINE
TkFusionObservation::TkFusionObservation(TkFusionObservation&&) = default;

TkFusionObservation& TkFusionObservation::operator=(const TkFusionObservation&) = default;

// NOLINTNEXTLINE
TkFusionObservation& TkFusionObservation::operator=(TkFusionObservation&&) = default;

void TkFusionObservation::toCsv(std::ostream& out) const
{
    out.precision(16);
    out << std::boolalpha;
    out << timestamp;out << ",";
    out << state;
    
}

std::stringstream TkFusionObservation::toStringstream() const
{
    std::stringstream out;
    toCsv(out);
    return out;
}

template<class Archive>
    void TkFusionObservation::serialize(Archive& ar, [[maybe_unused]] unsigned int version)
    {

        ar& CEREAL_NVP(timestamp);
        ar& CEREAL_NVP(state);
    }


template void TkFusionObservation::serialize(cereal::BinaryInputArchive& ar, unsigned int version);
template void TkFusionObservation::serialize(cereal::BinaryOutputArchive& ar, unsigned int version);
template void TkFusionObservation::serialize(cereal::IntrospectionArchive& ar, unsigned int version);
template void TkFusionObservation::serialize(cereal::JSONInputArchive& ar, unsigned int version);
template void TkFusionObservation::serialize(cereal::JSONOutputArchive& ar, unsigned int version);
template void TkFusionObservation::serialize(cereal::PortableBinaryInputArchive& ar, unsigned int version);
template void TkFusionObservation::serialize(cereal::PortableBinaryOutputArchive& ar, unsigned int version);

std::ostream& operator<< (std::ostream& os, const TkFusionObservation& tkfusionobservation)
{
    tkfusionobservation.toCsv(os);
    return os;
}

bool operator== (const TkFusionObservation &lhs, const TkFusionObservation &rhs)
{
    return (  lhs.timestamp== rhs.timestamp &&
     lhs.state== rhs.state 
    );
}

std::size_t TkFusionObservation::hash() const noexcept
{
    std::size_t result = 17;
    result = result * 31 + 0;
    result = result * 31 + std::hash<TkCrashState>()(state);
    return result;
}

 } // namespace av
 } // namespace msg


CEREAL_BIND_TO_ARCHIVES(av::msg::TkFusionObservation)




 namespace av {
 namespace msg {


TkInformationFusionMessage::TkInformationFusionMessage() = default;

TkInformationFusionMessage::TkInformationFusionMessage(
    TimePoint timestamp_arg ,
    TkCrashState state_arg ,
    TkObjectInfo ego_last_known_arg ,
    std::vector<TkFusionObservation> observations_arg 
    ):
    timestamp { std::move(timestamp_arg) } // NOLINT
      ,state { std::move(state_arg) } // NOLINT
      ,ego_last_known { std::move(ego_last_known_arg) } // NOLINT
      ,observations { std::move(observations_arg) } // NOLINT
      {
}

TkInformationFusionMessage::TkInformationFusionMessage(const TkInformationFusionMessage&) = default;

// NOLINTNEXTLINE
TkInformationFusionMessage::TkInformationFusionMessage(TkInformationFusionMessage&&) = default;

TkInformationFusionMessage& TkInformationFusionMessage::operator=(const TkInformationFusionMessage&) = default;

// NOLINTNEXTLINE
TkInformationFusionMessage& TkInformationFusionMessage::operator=(TkInformationFusionMessage&&) = default;

void TkInformationFusionMessage::toCsv(std::ostream& out) const
{
    out.precision(16);
    out << std::boolalpha;
    out << timestamp;out << ",";
    out << state;out << ",";
    out << ego_last_known;out << ",";
    // Enclose sequence in double quotations and curly brackets to keep CSV line length uniform
        
        out << "\"{{";
        
        if (!observations.empty())
        {
            std::copy(observations.begin(), observations.end() - 1, std::ostream_iterator<TkFusionObservation>(out, "},{"));
            std::copy(observations.end()-1, observations.end(), std::ostream_iterator<TkFusionObservation>(out, ""));
        }
        out << "}}\"";
    
}

std::stringstream TkInformationFusionMessage::toStringstream() const
{
    std::stringstream out;
    toCsv(out);
    return out;
}

template<class Archive>
    void TkInformationFusionMessage::serialize(Archive& ar, [[maybe_unused]] unsigned int version)
    {

        ar& CEREAL_NVP(timestamp);
        ar& CEREAL_NVP(state);
        ar& CEREAL_NVP(ego_last_known);
        ar& CEREAL_NVP(observations);
    }


template void TkInformationFusionMessage::serialize(cereal::BinaryInputArchive& ar, unsigned int version);
template void TkInformationFusionMessage::serialize(cereal::BinaryOutputArchive& ar, unsigned int version);
template void TkInformationFusionMessage::serialize(cereal::IntrospectionArchive& ar, unsigned int version);
template void TkInformationFusionMessage::serialize(cereal::JSONInputArchive& ar, unsigned int version);
template void TkInformationFusionMessage::serialize(cereal::JSONOutputArchive& ar, unsigned int version);
template void TkInformationFusionMessage::serialize(cereal::PortableBinaryInputArchive& ar, unsigned int version);
template void TkInformationFusionMessage::serialize(cereal::PortableBinaryOutputArchive& ar, unsigned int version);

std::ostream& operator<< (std::ostream& os, const TkInformationFusionMessage& tkinformationfusionmessage)
{
    tkinformationfusionmessage.toCsv(os);
    return os;
}

bool operator== (const TkInformationFusionMessage &lhs, const TkInformationFusionMessage &rhs)
{
    return (  lhs.timestamp== rhs.timestamp &&
     lhs.state== rhs.state &&
     lhs.ego_last_known== rhs.ego_last_known &&
     lhs.observations== rhs.observations 
    );
}

std::size_t TkInformationFusionMessage::hash() const noexcept
{
    std::size_t result = 17;
    result = result * 31 + 0;
    result = result * 31 + std::hash<TkCrashState>()(state);
    result = result * 31 + std::hash<TkObjectInfo>()(ego_last_known);
    result = result * 31 + hashVector(observations);
    return result;
}

 } // namespace av
 } // namespace msg


CEREAL_BIND_TO_ARCHIVES(av::msg::TkInformationFusionMessage)

