#include <interfaces/TkNftMessage/TkNftMessage.h>

#include <iterator>
#include <type_traits>

#include <cereal/cereal.hpp>
#include <cereal/archives/json.hpp>
#include <cereal/archives/binary.hpp>

#include <cereal/types/map.hpp>
#include <cereal/types/vector.hpp>
#include <cereal/types/array.hpp>
#include <cereal/types/string.hpp>
#include <cereal/types/complex.hpp>

#include <ODatum/IntrospectionArchive.h>
#include <ODatum/Hash.h>


 namespace av {
 namespace msg {


TkObjectViolationMap::TkObjectViolationMap() = default;

TkObjectViolationMap::TkObjectViolationMap(
    TkObjectInfo object_arg ,
    double violation_score_arg 
    ):
    object { std::move(object_arg) } // NOLINT
      ,
    violation_score { std::move(violation_score_arg) } // NOLINT
      
    {
}

TkObjectViolationMap::TkObjectViolationMap(const TkObjectViolationMap&) = default;

// NOLINTNEXTLINE
TkObjectViolationMap::TkObjectViolationMap(TkObjectViolationMap&&) = default;

TkObjectViolationMap& TkObjectViolationMap::operator=(const TkObjectViolationMap&) = default;

// NOLINTNEXTLINE
TkObjectViolationMap& TkObjectViolationMap::operator=(TkObjectViolationMap&&) = default;

void TkObjectViolationMap::toCsv(std::ostream& out) const
{
    out.precision(16);
    out << std::boolalpha;
    out << object;out << ",";
    out << violation_score;
    
}

std::stringstream TkObjectViolationMap::toStringstream() const
{
    std::stringstream out;
    toCsv(out);
    return out;
}

template<class Archive>
    void TkObjectViolationMap::serialize(Archive& ar, [[maybe_unused]] unsigned int version)
    {

        ar& CEREAL_NVP(object);
        ar& CEREAL_NVP(violation_score);
    }


template void TkObjectViolationMap::serialize(cereal::BinaryInputArchive& ar, unsigned int version);
template void TkObjectViolationMap::serialize(cereal::BinaryOutputArchive& ar, unsigned int version);
template void TkObjectViolationMap::serialize(cereal::IntrospectionArchive& ar, unsigned int version);
template void TkObjectViolationMap::serialize(cereal::JSONInputArchive& ar, unsigned int version);
template void TkObjectViolationMap::serialize(cereal::JSONOutputArchive& ar, unsigned int version);
template void TkObjectViolationMap::serialize(cereal::PortableBinaryInputArchive& ar, unsigned int version);
template void TkObjectViolationMap::serialize(cereal::PortableBinaryOutputArchive& ar, unsigned int version);

std::ostream& operator<< (std::ostream& os, const TkObjectViolationMap& tkobjectviolationmap)
{
    tkobjectviolationmap.toCsv(os);
    return os;
}

bool operator== (const TkObjectViolationMap &lhs, const TkObjectViolationMap &rhs)
{
    return (  lhs.object== rhs.object &&
     lhs.violation_score== rhs.violation_score 
    );
}

std::size_t TkObjectViolationMap::hash() const noexcept
{
    std::size_t result = 17;
    result = result * 31 + std::hash<TkObjectInfo>()(object);
    result = result * 31 + std::hash<double>()(violation_score);
    return result;
}

 } // namespace av
 } // namespace msg


CEREAL_BIND_TO_ARCHIVES(av::msg::TkObjectViolationMap)




 namespace av {
 namespace msg {


TkNftMessage::TkNftMessage() = default;

TkNftMessage::TkNftMessage(
    TimePoint timestamp_arg ,
    TkCrashState state_arg ,
    double max_violation_score_arg ,
    TkObjectInfo ego_last_known_arg ,
    std::vector<TkObjectViolationMap> colliding_objects_arg 
    ):
    timestamp { std::move(timestamp_arg) } // NOLINT
      ,state { std::move(state_arg) } // NOLINT
      ,max_violation_score { std::move(max_violation_score_arg) } // NOLINT
      ,ego_last_known { std::move(ego_last_known_arg) } // NOLINT
      ,colliding_objects { std::move(colliding_objects_arg) } // NOLINT
      {
}

TkNftMessage::TkNftMessage(const TkNftMessage&) = default;

// NOLINTNEXTLINE
TkNftMessage::TkNftMessage(TkNftMessage&&) = default;

TkNftMessage& TkNftMessage::operator=(const TkNftMessage&) = default;

// NOLINTNEXTLINE
TkNftMessage& TkNftMessage::operator=(TkNftMessage&&) = default;

void TkNftMessage::toCsv(std::ostream& out) const
{
    out.precision(16);
    out << std::boolalpha;
    out << timestamp;out << ",";
    out << state;out << ",";
    out << max_violation_score;out << ",";
    out << ego_last_known;out << ",";
    // Enclose sequence in double quotations and curly brackets to keep CSV line length uniform
        
        out << "\"{{";
        
        if (!colliding_objects.empty())
        {
            std::copy(colliding_objects.begin(), colliding_objects.end() - 1, std::ostream_iterator<TkObjectViolationMap>(out, "},{"));
            std::copy(colliding_objects.end()-1, colliding_objects.end(), std::ostream_iterator<TkObjectViolationMap>(out, ""));
        }
        out << "}}\"";
    
}

std::stringstream TkNftMessage::toStringstream() const
{
    std::stringstream out;
    toCsv(out);
    return out;
}

template<class Archive>
    void TkNftMessage::serialize(Archive& ar, [[maybe_unused]] unsigned int version)
    {

        ar& CEREAL_NVP(timestamp);
        ar& CEREAL_NVP(state);
        ar& CEREAL_NVP(max_violation_score);
        ar& CEREAL_NVP(ego_last_known);
        ar& CEREAL_NVP(colliding_objects);
    }


template void TkNftMessage::serialize(cereal::BinaryInputArchive& ar, unsigned int version);
template void TkNftMessage::serialize(cereal::BinaryOutputArchive& ar, unsigned int version);
template void TkNftMessage::serialize(cereal::IntrospectionArchive& ar, unsigned int version);
template void TkNftMessage::serialize(cereal::JSONInputArchive& ar, unsigned int version);
template void TkNftMessage::serialize(cereal::JSONOutputArchive& ar, unsigned int version);
template void TkNftMessage::serialize(cereal::PortableBinaryInputArchive& ar, unsigned int version);
template void TkNftMessage::serialize(cereal::PortableBinaryOutputArchive& ar, unsigned int version);

std::ostream& operator<< (std::ostream& os, const TkNftMessage& tknftmessage)
{
    tknftmessage.toCsv(os);
    return os;
}

bool operator== (const TkNftMessage &lhs, const TkNftMessage &rhs)
{
    return (  lhs.timestamp== rhs.timestamp &&
     lhs.state== rhs.state &&
     lhs.max_violation_score== rhs.max_violation_score &&
     lhs.ego_last_known== rhs.ego_last_known &&
     lhs.colliding_objects== rhs.colliding_objects 
    );
}

std::size_t TkNftMessage::hash() const noexcept
{
    std::size_t result = 17;
    result = result * 31 + 0;
    result = result * 31 + std::hash<TkCrashState>()(state);
    result = result * 31 + std::hash<double>()(max_violation_score);
    result = result * 31 + std::hash<TkObjectInfo>()(ego_last_known);
    result = result * 31 + hashVector(colliding_objects);
    return result;
}

 } // namespace av
 } // namespace msg


CEREAL_BIND_TO_ARCHIVES(av::msg::TkNftMessage)

