#include <interfaces/TkObjectInfo/TkObjectInfo.h>

#include <iterator>
#include <type_traits>

#include <cereal/cereal.hpp>
#include <cereal/archives/json.hpp>
#include <cereal/archives/binary.hpp>

#include <cereal/types/map.hpp>
#include <cereal/types/vector.hpp>
#include <cereal/types/array.hpp>
#include <cereal/types/string.hpp>
#include <cereal/types/complex.hpp>

#include <ODatum/IntrospectionArchive.h>
#include <ODatum/Hash.h>





 namespace av {
 namespace msg {


TkObjectInfo::TkObjectInfo() = default;

TkObjectInfo::TkObjectInfo(
    int64_t id_arg ,
    TK_OBJECT_TYPE type_arg ,
    std::string frame_id_arg ,
    double x_arg ,
    double y_arg ,
    double yaw_arg ,
    double vx_arg ,
    double vy_arg 
    ):
    id { std::move(id_arg) } // NOLINT
      ,type { std::move(type_arg) } // NOLINT
      ,frame_id { std::move(frame_id_arg) } // NOLINT
      ,x { std::move(x_arg) } // NOLINT
      ,y { std::move(y_arg) } // NOLINT
      ,yaw { std::move(yaw_arg) } // NOLINT
      ,vx { std::move(vx_arg) } // NOLINT
      ,vy { std::move(vy_arg) } // NOLINT
      {
}

TkObjectInfo::TkObjectInfo(const TkObjectInfo&) = default;

// NOLINTNEXTLINE
TkObjectInfo::TkObjectInfo(TkObjectInfo&&) = default;

TkObjectInfo& TkObjectInfo::operator=(const TkObjectInfo&) = default;

// NOLINTNEXTLINE
TkObjectInfo& TkObjectInfo::operator=(TkObjectInfo&&) = default;

void TkObjectInfo::toCsv(std::ostream& out) const
{
    out.precision(16);
    out << std::boolalpha;
    out << id;out << ",";
    out << static_cast<uint32_t>(type);out << ",";
    out << "\"";
        out << frame_id;
        out << "\"";out << ",";
    out << x;out << ",";
    out << y;out << ",";
    out << yaw;out << ",";
    out << vx;out << ",";
    out << vy;
    
}

std::stringstream TkObjectInfo::toStringstream() const
{
    std::stringstream out;
    toCsv(out);
    return out;
}

template<class Archive>
    void TkObjectInfo::serialize(Archive& ar, [[maybe_unused]] unsigned int version)
    {

        ar& CEREAL_NVP(id);
        ar& CEREAL_NVP(type);
        ar& CEREAL_NVP(frame_id);
        ar& CEREAL_NVP(x);
        ar& CEREAL_NVP(y);
        ar& CEREAL_NVP(yaw);
        ar& CEREAL_NVP(vx);
        ar& CEREAL_NVP(vy);
    }


template void TkObjectInfo::serialize(cereal::BinaryInputArchive& ar, unsigned int version);
template void TkObjectInfo::serialize(cereal::BinaryOutputArchive& ar, unsigned int version);
template void TkObjectInfo::serialize(cereal::IntrospectionArchive& ar, unsigned int version);
template void TkObjectInfo::serialize(cereal::JSONInputArchive& ar, unsigned int version);
template void TkObjectInfo::serialize(cereal::JSONOutputArchive& ar, unsigned int version);
template void TkObjectInfo::serialize(cereal::PortableBinaryInputArchive& ar, unsigned int version);
template void TkObjectInfo::serialize(cereal::PortableBinaryOutputArchive& ar, unsigned int version);

std::ostream& operator<< (std::ostream& os, const TkObjectInfo& tkobjectinfo)
{
    tkobjectinfo.toCsv(os);
    return os;
}

bool operator== (const TkObjectInfo &lhs, const TkObjectInfo &rhs)
{
    return (  lhs.id== rhs.id &&
     lhs.type== rhs.type &&
     lhs.frame_id== rhs.frame_id &&
     lhs.x== rhs.x &&
     lhs.y== rhs.y &&
     lhs.yaw== rhs.yaw &&
     lhs.vx== rhs.vx &&
     lhs.vy== rhs.vy 
    );
}

std::size_t TkObjectInfo::hash() const noexcept
{
    std::size_t result = 17;
    result = result * 31 + std::hash<int64_t>()(id);
    result = result * 31 + std::hash<TK_OBJECT_TYPE>()(type);
    result = result * 31 + std::hash<std::string>()(frame_id);
    result = result * 31 + std::hash<double>()(x);
    result = result * 31 + std::hash<double>()(y);
    result = result * 31 + std::hash<double>()(yaw);
    result = result * 31 + std::hash<double>()(vx);
    result = result * 31 + std::hash<double>()(vy);
    return result;
}

 } // namespace av
 } // namespace msg


CEREAL_BIND_TO_ARCHIVES(av::msg::TkObjectInfo)

