#include <interfaces/TrackedEntities/TrackedEntities.h>

#include <iterator>
#include <type_traits>

#include <cereal/cereal.hpp>
#include <cereal/archives/json.hpp>
#include <cereal/archives/binary.hpp>

#include <cereal/types/map.hpp>
#include <cereal/types/vector.hpp>
#include <cereal/types/array.hpp>
#include <cereal/types/string.hpp>
#include <cereal/types/complex.hpp>

#include <ODatum/IntrospectionArchive.h>
#include <ODatum/Hash.h>


 namespace av {
 namespace msg {
 namespace tracker {


TrackedPose::TrackedPose() = default;

TrackedPose::TrackedPose(
    double x_meters_arg ,
    double y_meters_arg ,
    double z_meters_arg ,
    double heading_radians_arg 
    ):
    x_meters { std::move(x_meters_arg) } // NOLINT
      ,
    y_meters { std::move(y_meters_arg) } // NOLINT
      ,
    z_meters { std::move(z_meters_arg) } // NOLINT
      ,
    heading_radians { std::move(heading_radians_arg) } // NOLINT
      
    {
}

TrackedPose::TrackedPose(const TrackedPose&) = default;

// NOLINTNEXTLINE
TrackedPose::TrackedPose(TrackedPose&&) = default;

TrackedPose& TrackedPose::operator=(const TrackedPose&) = default;

// NOLINTNEXTLINE
TrackedPose& TrackedPose::operator=(TrackedPose&&) = default;

void TrackedPose::toCsv(std::ostream& out) const
{
    out.precision(16);
    out << std::boolalpha;
    out << x_meters;out << ",";
    out << y_meters;out << ",";
    out << z_meters;out << ",";
    out << heading_radians;
    
}

std::stringstream TrackedPose::toStringstream() const
{
    std::stringstream out;
    toCsv(out);
    return out;
}

template<class Archive>
    void TrackedPose::serialize(Archive& ar, [[maybe_unused]] unsigned int version)
    {

        ar& CEREAL_NVP(x_meters);
        ar& CEREAL_NVP(y_meters);
        ar& CEREAL_NVP(z_meters);
        ar& CEREAL_NVP(heading_radians);
    }


template void TrackedPose::serialize(cereal::BinaryInputArchive& ar, unsigned int version);
template void TrackedPose::serialize(cereal::BinaryOutputArchive& ar, unsigned int version);
template void TrackedPose::serialize(cereal::IntrospectionArchive& ar, unsigned int version);
template void TrackedPose::serialize(cereal::JSONInputArchive& ar, unsigned int version);
template void TrackedPose::serialize(cereal::JSONOutputArchive& ar, unsigned int version);
template void TrackedPose::serialize(cereal::PortableBinaryInputArchive& ar, unsigned int version);
template void TrackedPose::serialize(cereal::PortableBinaryOutputArchive& ar, unsigned int version);

std::ostream& operator<< (std::ostream& os, const TrackedPose& trackedpose)
{
    trackedpose.toCsv(os);
    return os;
}

bool operator== (const TrackedPose &lhs, const TrackedPose &rhs)
{
    return (  lhs.x_meters== rhs.x_meters &&
     lhs.y_meters== rhs.y_meters &&
     lhs.z_meters== rhs.z_meters &&
     lhs.heading_radians== rhs.heading_radians 
    );
}

std::size_t TrackedPose::hash() const noexcept
{
    std::size_t result = 17;
    result = result * 31 + std::hash<double>()(x_meters);
    result = result * 31 + std::hash<double>()(y_meters);
    result = result * 31 + std::hash<double>()(z_meters);
    result = result * 31 + std::hash<double>()(heading_radians);
    return result;
}

 } // namespace av
 } // namespace msg
 } // namespace tracker


CEREAL_BIND_TO_ARCHIVES(av::msg::tracker::TrackedPose)

 namespace av {
 namespace msg {
 namespace tracker {


Vertex::Vertex() = default;

Vertex::Vertex(
    double x_arg ,
    double y_arg 
    ):
    x { std::move(x_arg) } // NOLINT
      ,
    y { std::move(y_arg) } // NOLINT
      
    {
}

Vertex::Vertex(const Vertex&) = default;

// NOLINTNEXTLINE
Vertex::Vertex(Vertex&&) = default;

Vertex& Vertex::operator=(const Vertex&) = default;

// NOLINTNEXTLINE
Vertex& Vertex::operator=(Vertex&&) = default;

void Vertex::toCsv(std::ostream& out) const
{
    out.precision(16);
    out << std::boolalpha;
    out << x;out << ",";
    out << y;
    
}

std::stringstream Vertex::toStringstream() const
{
    std::stringstream out;
    toCsv(out);
    return out;
}

template<class Archive>
    void Vertex::serialize(Archive& ar, [[maybe_unused]] unsigned int version)
    {

        ar& CEREAL_NVP(x);
        ar& CEREAL_NVP(y);
    }


template void Vertex::serialize(cereal::BinaryInputArchive& ar, unsigned int version);
template void Vertex::serialize(cereal::BinaryOutputArchive& ar, unsigned int version);
template void Vertex::serialize(cereal::IntrospectionArchive& ar, unsigned int version);
template void Vertex::serialize(cereal::JSONInputArchive& ar, unsigned int version);
template void Vertex::serialize(cereal::JSONOutputArchive& ar, unsigned int version);
template void Vertex::serialize(cereal::PortableBinaryInputArchive& ar, unsigned int version);
template void Vertex::serialize(cereal::PortableBinaryOutputArchive& ar, unsigned int version);

std::ostream& operator<< (std::ostream& os, const Vertex& vertex)
{
    vertex.toCsv(os);
    return os;
}

bool operator== (const Vertex &lhs, const Vertex &rhs)
{
    return (  lhs.x== rhs.x &&
     lhs.y== rhs.y 
    );
}

std::size_t Vertex::hash() const noexcept
{
    std::size_t result = 17;
    result = result * 31 + std::hash<double>()(x);
    result = result * 31 + std::hash<double>()(y);
    return result;
}

 } // namespace av
 } // namespace msg
 } // namespace tracker


CEREAL_BIND_TO_ARCHIVES(av::msg::tracker::Vertex)

 namespace av {
 namespace msg {
 namespace tracker {


ConvexPolygon::ConvexPolygon() = default;

ConvexPolygon::ConvexPolygon(
    bool isBox_arg ,
    std::vector<Vertex> points_arg ,
    double height_arg 
    ):
    isBox { std::move(isBox_arg) } // NOLINT
      ,
    points { std::move(points_arg) } // NOLINT
      ,
    height { std::move(height_arg) } // NOLINT
      
    {
}

ConvexPolygon::ConvexPolygon(const ConvexPolygon&) = default;

// NOLINTNEXTLINE
ConvexPolygon::ConvexPolygon(ConvexPolygon&&) = default;

ConvexPolygon& ConvexPolygon::operator=(const ConvexPolygon&) = default;

// NOLINTNEXTLINE
ConvexPolygon& ConvexPolygon::operator=(ConvexPolygon&&) = default;

void ConvexPolygon::toCsv(std::ostream& out) const
{
    out.precision(16);
    out << std::boolalpha;
    out << isBox;out << ",";
    // Enclose sequence in double quotations and curly brackets to keep CSV line length uniform
        
        out << "\"{{";
        
        if (!points.empty())
        {
            std::copy(points.begin(), points.end() - 1, std::ostream_iterator<Vertex>(out, "},{"));
            std::copy(points.end()-1, points.end(), std::ostream_iterator<Vertex>(out, ""));
        }
        out << "}}\"";out << ",";
    out << height;
    
}

std::stringstream ConvexPolygon::toStringstream() const
{
    std::stringstream out;
    toCsv(out);
    return out;
}

template<class Archive>
    void ConvexPolygon::serialize(Archive& ar, const unsigned int version)
{
    ar& CEREAL_NVP(isBox);
    ar& CEREAL_NVP(points);
    if (version >= 2) {
        ar& CEREAL_NVP(height);
    }
}



template void ConvexPolygon::serialize(cereal::BinaryInputArchive& ar, unsigned int version);
template void ConvexPolygon::serialize(cereal::BinaryOutputArchive& ar, unsigned int version);
template void ConvexPolygon::serialize(cereal::IntrospectionArchive& ar, unsigned int version);
template void ConvexPolygon::serialize(cereal::JSONInputArchive& ar, unsigned int version);
template void ConvexPolygon::serialize(cereal::JSONOutputArchive& ar, unsigned int version);
template void ConvexPolygon::serialize(cereal::PortableBinaryInputArchive& ar, unsigned int version);
template void ConvexPolygon::serialize(cereal::PortableBinaryOutputArchive& ar, unsigned int version);

std::ostream& operator<< (std::ostream& os, const ConvexPolygon& convexpolygon)
{
    convexpolygon.toCsv(os);
    return os;
}

bool operator== (const ConvexPolygon &lhs, const ConvexPolygon &rhs)
{
    return (  lhs.isBox== rhs.isBox &&
     lhs.points== rhs.points &&
     lhs.height== rhs.height 
    );
}

std::size_t ConvexPolygon::hash() const noexcept
{
    std::size_t result = 17;
    result = result * 31 + std::hash<bool>()(isBox);
    result = result * 31 + hashVector(points);
    result = result * 31 + std::hash<double>()(height);
    return result;
}

 } // namespace av
 } // namespace msg
 } // namespace tracker


CEREAL_BIND_TO_ARCHIVES(av::msg::tracker::ConvexPolygon)

 namespace av {
 namespace msg {
 namespace tracker {


TrackedEntity::TrackedEntity() = default;

TrackedEntity::TrackedEntity(
    int64_t trackId_arg ,
    int64_t timeFirstDetection_arg ,
    int64_t timeLastDetection_arg ,
    TrackedEntityType classType_arg ,
    std::vector<double> classTypeProbabilities_arg ,
    double confidence_arg ,
    double probabilityActive_arg ,
    TrackedPose pose_arg ,
    prediction::Matrix2D poseCov_arg ,
    Velocity velocity_arg ,
    prediction::Matrix2D velocityCov_arg ,
    std::vector<ConvexPolygon> convexPolygons_arg ,
    std::vector<prediction::Trajectory> trajectories_arg ,
    prediction::Acceleration acceleration_arg ,
    double angularVelocityRadps_arg 
    ):
    trackId { std::move(trackId_arg) } // NOLINT
      ,
    timeFirstDetection { std::move(timeFirstDetection_arg) } // NOLINT
      ,
    timeLastDetection { std::move(timeLastDetection_arg) } // NOLINT
      ,
    classType { std::move(classType_arg) } // NOLINT
      ,
    classTypeProbabilities { std::move(classTypeProbabilities_arg) } // NOLINT
      ,
    confidence { std::move(confidence_arg) } // NOLINT
      ,
    probabilityActive { std::move(probabilityActive_arg) } // NOLINT
      ,
    pose { std::move(pose_arg) } // NOLINT
      ,
    poseCov { std::move(poseCov_arg) } // NOLINT
      ,
    velocity { std::move(velocity_arg) } // NOLINT
      ,
    velocityCov { std::move(velocityCov_arg) } // NOLINT
      ,
    convexPolygons { std::move(convexPolygons_arg) } // NOLINT
      ,
    trajectories { std::move(trajectories_arg) } // NOLINT
      ,
    acceleration { std::move(acceleration_arg) } // NOLINT
      ,
    angularVelocityRadps { std::move(angularVelocityRadps_arg) } // NOLINT
      
    {
}

TrackedEntity::TrackedEntity(const TrackedEntity&) = default;

// NOLINTNEXTLINE
TrackedEntity::TrackedEntity(TrackedEntity&&) = default;

TrackedEntity& TrackedEntity::operator=(const TrackedEntity&) = default;

// NOLINTNEXTLINE
TrackedEntity& TrackedEntity::operator=(TrackedEntity&&) = default;

void TrackedEntity::toCsv(std::ostream& out) const
{
    out.precision(16);
    out << std::boolalpha;
    out << trackId;out << ",";
    out << timeFirstDetection;out << ",";
    out << timeLastDetection;out << ",";
    out << static_cast<uint32_t>(classType);out << ",";
    // Enclose sequence in double quotations and curly brackets to keep CSV line length uniform
        
        out << "\"{{";
        
        if (!classTypeProbabilities.empty())
        {
            std::copy(classTypeProbabilities.begin(), classTypeProbabilities.end() - 1, std::ostream_iterator<double>(out, "},{"));
            std::copy(classTypeProbabilities.end()-1, classTypeProbabilities.end(), std::ostream_iterator<double>(out, ""));
        }
        out << "}}\"";out << ",";
    out << confidence;out << ",";
    out << probabilityActive;out << ",";
    out << pose;out << ",";
    out << poseCov;out << ",";
    out << velocity;out << ",";
    out << velocityCov;out << ",";
    // Enclose sequence in double quotations and curly brackets to keep CSV line length uniform
        
        out << "\"{{";
        
        if (!convexPolygons.empty())
        {
            std::copy(convexPolygons.begin(), convexPolygons.end() - 1, std::ostream_iterator<ConvexPolygon>(out, "},{"));
            std::copy(convexPolygons.end()-1, convexPolygons.end(), std::ostream_iterator<ConvexPolygon>(out, ""));
        }
        out << "}}\"";out << ",";
    // Enclose sequence in double quotations and curly brackets to keep CSV line length uniform
        
        out << "\"{{";
        
        if (!trajectories.empty())
        {
            std::copy(trajectories.begin(), trajectories.end() - 1, std::ostream_iterator<prediction::Trajectory>(out, "},{"));
            std::copy(trajectories.end()-1, trajectories.end(), std::ostream_iterator<prediction::Trajectory>(out, ""));
        }
        out << "}}\"";out << ",";
    out << acceleration;out << ",";
    out << angularVelocityRadps;
    
}

std::stringstream TrackedEntity::toStringstream() const
{
    std::stringstream out;
    toCsv(out);
    return out;
}

template<class Archive>
    void TrackedEntity::serialize(Archive& ar, unsigned int version)
{
	if (version >= 1)
	{
		ar& CEREAL_NVP(trackId);
		ar& CEREAL_NVP(timeFirstDetection);
		ar& CEREAL_NVP(timeLastDetection);
		ar& CEREAL_NVP(classType);		

		ar& CEREAL_NVP(classTypeProbabilities);
		ar& CEREAL_NVP(confidence);
		ar& CEREAL_NVP(probabilityActive);
		ar& CEREAL_NVP(pose);

		ar& CEREAL_NVP(poseCov);
		ar& CEREAL_NVP(velocity);
		ar& CEREAL_NVP(velocityCov);
		ar& CEREAL_NVP(convexPolygons);

		ar& CEREAL_NVP(trajectories);

	}
	if (version >= 2)
	{
		ar& CEREAL_NVP(acceleration);
		ar& CEREAL_NVP(angularVelocityRadps);
	}
}



template void TrackedEntity::serialize(cereal::BinaryInputArchive& ar, unsigned int version);
template void TrackedEntity::serialize(cereal::BinaryOutputArchive& ar, unsigned int version);
template void TrackedEntity::serialize(cereal::IntrospectionArchive& ar, unsigned int version);
template void TrackedEntity::serialize(cereal::JSONInputArchive& ar, unsigned int version);
template void TrackedEntity::serialize(cereal::JSONOutputArchive& ar, unsigned int version);
template void TrackedEntity::serialize(cereal::PortableBinaryInputArchive& ar, unsigned int version);
template void TrackedEntity::serialize(cereal::PortableBinaryOutputArchive& ar, unsigned int version);

std::ostream& operator<< (std::ostream& os, const TrackedEntity& trackedentity)
{
    trackedentity.toCsv(os);
    return os;
}

bool operator== (const TrackedEntity &lhs, const TrackedEntity &rhs)
{
    return (  lhs.trackId== rhs.trackId &&
     lhs.timeFirstDetection== rhs.timeFirstDetection &&
     lhs.timeLastDetection== rhs.timeLastDetection &&
     lhs.classType== rhs.classType &&
     lhs.classTypeProbabilities== rhs.classTypeProbabilities &&
     lhs.confidence== rhs.confidence &&
     lhs.probabilityActive== rhs.probabilityActive &&
     lhs.pose== rhs.pose &&
     lhs.poseCov== rhs.poseCov &&
     lhs.velocity== rhs.velocity &&
     lhs.velocityCov== rhs.velocityCov &&
     lhs.convexPolygons== rhs.convexPolygons &&
     lhs.trajectories== rhs.trajectories &&
     lhs.acceleration== rhs.acceleration &&
     lhs.angularVelocityRadps== rhs.angularVelocityRadps 
    );
}

std::size_t TrackedEntity::hash() const noexcept
{
    std::size_t result = 17;
    result = result * 31 + std::hash<int64_t>()(trackId);
    result = result * 31 + std::hash<int64_t>()(timeFirstDetection);
    result = result * 31 + std::hash<int64_t>()(timeLastDetection);
    result = result * 31 + std::hash<TrackedEntityType>()(classType);
    result = result * 31 + hashVector(classTypeProbabilities);
    result = result * 31 + std::hash<double>()(confidence);
    result = result * 31 + std::hash<double>()(probabilityActive);
    result = result * 31 + std::hash<TrackedPose>()(pose);
    result = result * 31 + std::hash<prediction::Matrix2D>()(poseCov);
    result = result * 31 + std::hash<Velocity>()(velocity);
    result = result * 31 + std::hash<prediction::Matrix2D>()(velocityCov);
    result = result * 31 + hashVector(convexPolygons);
    result = result * 31 + hashVector(trajectories);
    result = result * 31 + std::hash<prediction::Acceleration>()(acceleration);
    result = result * 31 + std::hash<double>()(angularVelocityRadps);
    return result;
}

 } // namespace av
 } // namespace msg
 } // namespace tracker


CEREAL_BIND_TO_ARCHIVES(av::msg::tracker::TrackedEntity)




 namespace av {
 namespace msg {
 namespace tracker {


TrackedEntities::TrackedEntities() = default;

TrackedEntities::TrackedEntities(
    int64_t timeStamp_arg ,
    std::string frameId_arg ,
    std::vector<TrackedEntity> trackedEntities_arg 
    ):
    timeStamp { std::move(timeStamp_arg) } // NOLINT
      ,frameId { std::move(frameId_arg) } // NOLINT
      ,trackedEntities { std::move(trackedEntities_arg) } // NOLINT
      {
}

TrackedEntities::TrackedEntities(const TrackedEntities&) = default;

// NOLINTNEXTLINE
TrackedEntities::TrackedEntities(TrackedEntities&&) = default;

TrackedEntities& TrackedEntities::operator=(const TrackedEntities&) = default;

// NOLINTNEXTLINE
TrackedEntities& TrackedEntities::operator=(TrackedEntities&&) = default;

void TrackedEntities::toCsv(std::ostream& out) const
{
    out.precision(16);
    out << std::boolalpha;
    out << timeStamp;out << ",";
    out << "\"";
        out << frameId;
        out << "\"";out << ",";
    // Enclose sequence in double quotations and curly brackets to keep CSV line length uniform
        
        out << "\"{{";
        
        if (!trackedEntities.empty())
        {
            std::copy(trackedEntities.begin(), trackedEntities.end() - 1, std::ostream_iterator<TrackedEntity>(out, "},{"));
            std::copy(trackedEntities.end()-1, trackedEntities.end(), std::ostream_iterator<TrackedEntity>(out, ""));
        }
        out << "}}\"";
    
}

std::stringstream TrackedEntities::toStringstream() const
{
    std::stringstream out;
    toCsv(out);
    return out;
}

template<class Archive>
    void TrackedEntities::serialize(Archive& ar, [[maybe_unused]] unsigned int version)
    {

        ar& CEREAL_NVP(timeStamp);
        ar& CEREAL_NVP(frameId);
        ar& CEREAL_NVP(trackedEntities);
    }


template void TrackedEntities::serialize(cereal::BinaryInputArchive& ar, unsigned int version);
template void TrackedEntities::serialize(cereal::BinaryOutputArchive& ar, unsigned int version);
template void TrackedEntities::serialize(cereal::IntrospectionArchive& ar, unsigned int version);
template void TrackedEntities::serialize(cereal::JSONInputArchive& ar, unsigned int version);
template void TrackedEntities::serialize(cereal::JSONOutputArchive& ar, unsigned int version);
template void TrackedEntities::serialize(cereal::PortableBinaryInputArchive& ar, unsigned int version);
template void TrackedEntities::serialize(cereal::PortableBinaryOutputArchive& ar, unsigned int version);

std::ostream& operator<< (std::ostream& os, const TrackedEntities& trackedentities)
{
    trackedentities.toCsv(os);
    return os;
}

bool operator== (const TrackedEntities &lhs, const TrackedEntities &rhs)
{
    return (  lhs.timeStamp== rhs.timeStamp &&
     lhs.frameId== rhs.frameId &&
     lhs.trackedEntities== rhs.trackedEntities 
    );
}

std::size_t TrackedEntities::hash() const noexcept
{
    std::size_t result = 17;
    result = result * 31 + std::hash<int64_t>()(timeStamp);
    result = result * 31 + std::hash<std::string>()(frameId);
    result = result * 31 + hashVector(trackedEntities);
    return result;
}

 } // namespace av
 } // namespace msg
 } // namespace tracker


CEREAL_BIND_TO_ARCHIVES(av::msg::tracker::TrackedEntities)

