#include <interfaces/TrackedObjectSet/TrackedObjectSet.h>

#include <iterator>
#include <type_traits>

#include <cereal/cereal.hpp>
#include <cereal/archives/json.hpp>
#include <cereal/archives/binary.hpp>

#include <cereal/types/map.hpp>
#include <cereal/types/vector.hpp>
#include <cereal/types/array.hpp>
#include <cereal/types/string.hpp>
#include <cereal/types/complex.hpp>

#include <ODatum/IntrospectionArchive.h>
#include <ODatum/Hash.h>


 namespace av {
 namespace msg {
 namespace tc {


Polygon2d::Polygon2d() = default;

Polygon2d::Polygon2d(
    std::vector<av::geometry::Vector2d> points_arg 
    ):
    points { std::move(points_arg) } // NOLINT
      
    {
}

Polygon2d::Polygon2d(const Polygon2d&) = default;

// NOLINTNEXTLINE
Polygon2d::Polygon2d(Polygon2d&&) = default;

Polygon2d& Polygon2d::operator=(const Polygon2d&) = default;

// NOLINTNEXTLINE
Polygon2d& Polygon2d::operator=(Polygon2d&&) = default;

void Polygon2d::toCsv(std::ostream& out) const
{
    out.precision(16);
    out << std::boolalpha;
    // Enclose sequence in double quotations and curly brackets to keep CSV line length uniform
        
        out << "\"{{";
        
        if (!points.empty())
        {
            std::copy(points.begin(), points.end() - 1, std::ostream_iterator<av::geometry::Vector2d>(out, "},{"));
            std::copy(points.end()-1, points.end(), std::ostream_iterator<av::geometry::Vector2d>(out, ""));
        }
        out << "}}\"";
    
}

std::stringstream Polygon2d::toStringstream() const
{
    std::stringstream out;
    toCsv(out);
    return out;
}

template<class Archive>
    void Polygon2d::serialize(Archive& ar, [[maybe_unused]] unsigned int version)
    {

        ar& CEREAL_NVP(points);
    }


template void Polygon2d::serialize(cereal::BinaryInputArchive& ar, unsigned int version);
template void Polygon2d::serialize(cereal::BinaryOutputArchive& ar, unsigned int version);
template void Polygon2d::serialize(cereal::IntrospectionArchive& ar, unsigned int version);
template void Polygon2d::serialize(cereal::JSONInputArchive& ar, unsigned int version);
template void Polygon2d::serialize(cereal::JSONOutputArchive& ar, unsigned int version);
template void Polygon2d::serialize(cereal::PortableBinaryInputArchive& ar, unsigned int version);
template void Polygon2d::serialize(cereal::PortableBinaryOutputArchive& ar, unsigned int version);

std::ostream& operator<< (std::ostream& os, const Polygon2d& polygon2d)
{
    polygon2d.toCsv(os);
    return os;
}

bool operator== (const Polygon2d &lhs, const Polygon2d &rhs)
{
    return (  lhs.points== rhs.points 
    );
}

std::size_t Polygon2d::hash() const noexcept
{
    std::size_t result = 17;
    result = result * 31 + hashVector(points);
    return result;
}

 } // namespace av
 } // namespace msg
 } // namespace tc


CEREAL_BIND_TO_ARCHIVES(av::msg::tc::Polygon2d)

 namespace av {
 namespace msg {
 namespace tc {


TrackedObject::TrackedObject() = default;

TrackedObject::TrackedObject(
    uint64_t id_arg ,
    av::geometry::Pose2d pose_arg ,
    av::geometry::Vector2d velocity_arg ,
    double boundingRadius_arg ,
    Polygon2d boundingPoly_arg 
    ):
    id { std::move(id_arg) } // NOLINT
      ,
    pose { std::move(pose_arg) } // NOLINT
      ,
    velocity { std::move(velocity_arg) } // NOLINT
      ,
    boundingRadius { std::move(boundingRadius_arg) } // NOLINT
      ,
    boundingPoly { std::move(boundingPoly_arg) } // NOLINT
      
    {
}

TrackedObject::TrackedObject(const TrackedObject&) = default;

// NOLINTNEXTLINE
TrackedObject::TrackedObject(TrackedObject&&) = default;

TrackedObject& TrackedObject::operator=(const TrackedObject&) = default;

// NOLINTNEXTLINE
TrackedObject& TrackedObject::operator=(TrackedObject&&) = default;

void TrackedObject::toCsv(std::ostream& out) const
{
    out.precision(16);
    out << std::boolalpha;
    out << id;out << ",";
    out << pose;out << ",";
    out << velocity;out << ",";
    out << boundingRadius;out << ",";
    out << boundingPoly;
    
}

std::stringstream TrackedObject::toStringstream() const
{
    std::stringstream out;
    toCsv(out);
    return out;
}

template<class Archive>
    void TrackedObject::serialize(Archive& ar, [[maybe_unused]] unsigned int version)
    {

        ar& CEREAL_NVP(id);
        ar& CEREAL_NVP(pose);
        ar& CEREAL_NVP(velocity);
        ar& CEREAL_NVP(boundingRadius);
        ar& CEREAL_NVP(boundingPoly);
    }


template void TrackedObject::serialize(cereal::BinaryInputArchive& ar, unsigned int version);
template void TrackedObject::serialize(cereal::BinaryOutputArchive& ar, unsigned int version);
template void TrackedObject::serialize(cereal::IntrospectionArchive& ar, unsigned int version);
template void TrackedObject::serialize(cereal::JSONInputArchive& ar, unsigned int version);
template void TrackedObject::serialize(cereal::JSONOutputArchive& ar, unsigned int version);
template void TrackedObject::serialize(cereal::PortableBinaryInputArchive& ar, unsigned int version);
template void TrackedObject::serialize(cereal::PortableBinaryOutputArchive& ar, unsigned int version);

std::ostream& operator<< (std::ostream& os, const TrackedObject& trackedobject)
{
    trackedobject.toCsv(os);
    return os;
}

bool operator== (const TrackedObject &lhs, const TrackedObject &rhs)
{
    return (  lhs.id== rhs.id &&
     lhs.pose== rhs.pose &&
     lhs.velocity== rhs.velocity &&
     lhs.boundingRadius== rhs.boundingRadius &&
     lhs.boundingPoly== rhs.boundingPoly 
    );
}

std::size_t TrackedObject::hash() const noexcept
{
    std::size_t result = 17;
    result = result * 31 + std::hash<uint64_t>()(id);
    result = result * 31 + std::hash<av::geometry::Pose2d>()(pose);
    result = result * 31 + std::hash<av::geometry::Vector2d>()(velocity);
    result = result * 31 + std::hash<double>()(boundingRadius);
    result = result * 31 + std::hash<Polygon2d>()(boundingPoly);
    return result;
}

 } // namespace av
 } // namespace msg
 } // namespace tc


CEREAL_BIND_TO_ARCHIVES(av::msg::tc::TrackedObject)




 namespace av {
 namespace msg {
 namespace tc {


TrackedObjectSet::TrackedObjectSet() = default;

TrackedObjectSet::TrackedObjectSet(
    int64_t timeUs_arg ,
    std::vector<TrackedObject> staticObjects_arg ,
    std::vector<TrackedObject> vehicles_arg ,
    std::vector<TrackedObject> bicycles_arg ,
    std::vector<TrackedObject> pedestrians_arg ,
    std::vector<TrackedObject> otherDynamicObjects_arg 
    ):
    timeUs { std::move(timeUs_arg) } // NOLINT
      ,staticObjects { std::move(staticObjects_arg) } // NOLINT
      ,vehicles { std::move(vehicles_arg) } // NOLINT
      ,bicycles { std::move(bicycles_arg) } // NOLINT
      ,pedestrians { std::move(pedestrians_arg) } // NOLINT
      ,otherDynamicObjects { std::move(otherDynamicObjects_arg) } // NOLINT
      {
}

TrackedObjectSet::TrackedObjectSet(const TrackedObjectSet&) = default;

// NOLINTNEXTLINE
TrackedObjectSet::TrackedObjectSet(TrackedObjectSet&&) = default;

TrackedObjectSet& TrackedObjectSet::operator=(const TrackedObjectSet&) = default;

// NOLINTNEXTLINE
TrackedObjectSet& TrackedObjectSet::operator=(TrackedObjectSet&&) = default;

void TrackedObjectSet::toCsv(std::ostream& out) const
{
    out.precision(16);
    out << std::boolalpha;
    out << timeUs;out << ",";
    // Enclose sequence in double quotations and curly brackets to keep CSV line length uniform
        
        out << "\"{{";
        
        if (!staticObjects.empty())
        {
            std::copy(staticObjects.begin(), staticObjects.end() - 1, std::ostream_iterator<TrackedObject>(out, "},{"));
            std::copy(staticObjects.end()-1, staticObjects.end(), std::ostream_iterator<TrackedObject>(out, ""));
        }
        out << "}}\"";out << ",";
    // Enclose sequence in double quotations and curly brackets to keep CSV line length uniform
        
        out << "\"{{";
        
        if (!vehicles.empty())
        {
            std::copy(vehicles.begin(), vehicles.end() - 1, std::ostream_iterator<TrackedObject>(out, "},{"));
            std::copy(vehicles.end()-1, vehicles.end(), std::ostream_iterator<TrackedObject>(out, ""));
        }
        out << "}}\"";out << ",";
    // Enclose sequence in double quotations and curly brackets to keep CSV line length uniform
        
        out << "\"{{";
        
        if (!bicycles.empty())
        {
            std::copy(bicycles.begin(), bicycles.end() - 1, std::ostream_iterator<TrackedObject>(out, "},{"));
            std::copy(bicycles.end()-1, bicycles.end(), std::ostream_iterator<TrackedObject>(out, ""));
        }
        out << "}}\"";out << ",";
    // Enclose sequence in double quotations and curly brackets to keep CSV line length uniform
        
        out << "\"{{";
        
        if (!pedestrians.empty())
        {
            std::copy(pedestrians.begin(), pedestrians.end() - 1, std::ostream_iterator<TrackedObject>(out, "},{"));
            std::copy(pedestrians.end()-1, pedestrians.end(), std::ostream_iterator<TrackedObject>(out, ""));
        }
        out << "}}\"";out << ",";
    // Enclose sequence in double quotations and curly brackets to keep CSV line length uniform
        
        out << "\"{{";
        
        if (!otherDynamicObjects.empty())
        {
            std::copy(otherDynamicObjects.begin(), otherDynamicObjects.end() - 1, std::ostream_iterator<TrackedObject>(out, "},{"));
            std::copy(otherDynamicObjects.end()-1, otherDynamicObjects.end(), std::ostream_iterator<TrackedObject>(out, ""));
        }
        out << "}}\"";
    
}

std::stringstream TrackedObjectSet::toStringstream() const
{
    std::stringstream out;
    toCsv(out);
    return out;
}

template<class Archive>
    void TrackedObjectSet::serialize(Archive& ar, [[maybe_unused]] unsigned int version)
    {

        ar& CEREAL_NVP(timeUs);
        ar& CEREAL_NVP(staticObjects);
        ar& CEREAL_NVP(vehicles);
        ar& CEREAL_NVP(bicycles);
        ar& CEREAL_NVP(pedestrians);
        ar& CEREAL_NVP(otherDynamicObjects);
    }


template void TrackedObjectSet::serialize(cereal::BinaryInputArchive& ar, unsigned int version);
template void TrackedObjectSet::serialize(cereal::BinaryOutputArchive& ar, unsigned int version);
template void TrackedObjectSet::serialize(cereal::IntrospectionArchive& ar, unsigned int version);
template void TrackedObjectSet::serialize(cereal::JSONInputArchive& ar, unsigned int version);
template void TrackedObjectSet::serialize(cereal::JSONOutputArchive& ar, unsigned int version);
template void TrackedObjectSet::serialize(cereal::PortableBinaryInputArchive& ar, unsigned int version);
template void TrackedObjectSet::serialize(cereal::PortableBinaryOutputArchive& ar, unsigned int version);

std::ostream& operator<< (std::ostream& os, const TrackedObjectSet& trackedobjectset)
{
    trackedobjectset.toCsv(os);
    return os;
}

bool operator== (const TrackedObjectSet &lhs, const TrackedObjectSet &rhs)
{
    return (  lhs.timeUs== rhs.timeUs &&
     lhs.staticObjects== rhs.staticObjects &&
     lhs.vehicles== rhs.vehicles &&
     lhs.bicycles== rhs.bicycles &&
     lhs.pedestrians== rhs.pedestrians &&
     lhs.otherDynamicObjects== rhs.otherDynamicObjects 
    );
}

std::size_t TrackedObjectSet::hash() const noexcept
{
    std::size_t result = 17;
    result = result * 31 + std::hash<int64_t>()(timeUs);
    result = result * 31 + hashVector(staticObjects);
    result = result * 31 + hashVector(vehicles);
    result = result * 31 + hashVector(bicycles);
    result = result * 31 + hashVector(pedestrians);
    result = result * 31 + hashVector(otherDynamicObjects);
    return result;
}

 } // namespace av
 } // namespace msg
 } // namespace tc


CEREAL_BIND_TO_ARCHIVES(av::msg::tc::TrackedObjectSet)

