#include <interfaces/TrafficLightNetworkMessage/TrafficLightNetworkMessage.h>

#include <iterator>
#include <type_traits>

#include <cereal/cereal.hpp>
#include <cereal/archives/json.hpp>
#include <cereal/archives/binary.hpp>

#include <cereal/types/map.hpp>
#include <cereal/types/vector.hpp>
#include <cereal/types/array.hpp>
#include <cereal/types/string.hpp>
#include <cereal/types/complex.hpp>

#include <ODatum/IntrospectionArchive.h>
#include <ODatum/Hash.h>


 namespace av {
 namespace msg {
 namespace tln {


point2D::point2D() = default;

point2D::point2D(
    double x_arg ,
    double y_arg 
    ):
    x { std::move(x_arg) } // NOLINT
      ,
    y { std::move(y_arg) } // NOLINT
      
    {
}

point2D::point2D(const point2D&) = default;

// NOLINTNEXTLINE
point2D::point2D(point2D&&) = default;

point2D& point2D::operator=(const point2D&) = default;

// NOLINTNEXTLINE
point2D& point2D::operator=(point2D&&) = default;

void point2D::toCsv(std::ostream& out) const
{
    out.precision(16);
    out << std::boolalpha;
    out << x;out << ",";
    out << y;
    
}

std::stringstream point2D::toStringstream() const
{
    std::stringstream out;
    toCsv(out);
    return out;
}

template<class Archive>
    void point2D::serialize(Archive& ar, [[maybe_unused]] unsigned int version)
    {

        ar& CEREAL_NVP(x);
        ar& CEREAL_NVP(y);
    }


template void point2D::serialize(cereal::BinaryInputArchive& ar, unsigned int version);
template void point2D::serialize(cereal::BinaryOutputArchive& ar, unsigned int version);
template void point2D::serialize(cereal::IntrospectionArchive& ar, unsigned int version);
template void point2D::serialize(cereal::JSONInputArchive& ar, unsigned int version);
template void point2D::serialize(cereal::JSONOutputArchive& ar, unsigned int version);
template void point2D::serialize(cereal::PortableBinaryInputArchive& ar, unsigned int version);
template void point2D::serialize(cereal::PortableBinaryOutputArchive& ar, unsigned int version);

std::ostream& operator<< (std::ostream& os, const point2D& point2d)
{
    point2d.toCsv(os);
    return os;
}

bool operator== (const point2D &lhs, const point2D &rhs)
{
    return (  lhs.x== rhs.x &&
     lhs.y== rhs.y 
    );
}

std::size_t point2D::hash() const noexcept
{
    std::size_t result = 17;
    result = result * 31 + std::hash<double>()(x);
    result = result * 31 + std::hash<double>()(y);
    return result;
}

 } // namespace av
 } // namespace msg
 } // namespace tln


CEREAL_BIND_TO_ARCHIVES(av::msg::tln::point2D)

 namespace av {
 namespace msg {
 namespace tln {


TLNBoundingBox::TLNBoundingBox() = default;

TLNBoundingBox::TLNBoundingBox(
    point2D topLeftCorner_arg ,
    double width_arg ,
    double height_arg ,
    double attributeScore_arg 
    ):
    topLeftCorner { std::move(topLeftCorner_arg) } // NOLINT
      ,
    width { std::move(width_arg) } // NOLINT
      ,
    height { std::move(height_arg) } // NOLINT
      ,
    attributeScore { std::move(attributeScore_arg) } // NOLINT
      
    {
}

TLNBoundingBox::TLNBoundingBox(const TLNBoundingBox&) = default;

// NOLINTNEXTLINE
TLNBoundingBox::TLNBoundingBox(TLNBoundingBox&&) = default;

TLNBoundingBox& TLNBoundingBox::operator=(const TLNBoundingBox&) = default;

// NOLINTNEXTLINE
TLNBoundingBox& TLNBoundingBox::operator=(TLNBoundingBox&&) = default;

void TLNBoundingBox::toCsv(std::ostream& out) const
{
    out.precision(16);
    out << std::boolalpha;
    out << topLeftCorner;out << ",";
    out << width;out << ",";
    out << height;out << ",";
    out << attributeScore;
    
}

std::stringstream TLNBoundingBox::toStringstream() const
{
    std::stringstream out;
    toCsv(out);
    return out;
}

template<class Archive>
    void TLNBoundingBox::serialize(Archive& ar, [[maybe_unused]] unsigned int version)
    {

        ar& CEREAL_NVP(topLeftCorner);
        ar& CEREAL_NVP(width);
        ar& CEREAL_NVP(height);
        ar& CEREAL_NVP(attributeScore);
    }


template void TLNBoundingBox::serialize(cereal::BinaryInputArchive& ar, unsigned int version);
template void TLNBoundingBox::serialize(cereal::BinaryOutputArchive& ar, unsigned int version);
template void TLNBoundingBox::serialize(cereal::IntrospectionArchive& ar, unsigned int version);
template void TLNBoundingBox::serialize(cereal::JSONInputArchive& ar, unsigned int version);
template void TLNBoundingBox::serialize(cereal::JSONOutputArchive& ar, unsigned int version);
template void TLNBoundingBox::serialize(cereal::PortableBinaryInputArchive& ar, unsigned int version);
template void TLNBoundingBox::serialize(cereal::PortableBinaryOutputArchive& ar, unsigned int version);

std::ostream& operator<< (std::ostream& os, const TLNBoundingBox& tlnboundingbox)
{
    tlnboundingbox.toCsv(os);
    return os;
}

bool operator== (const TLNBoundingBox &lhs, const TLNBoundingBox &rhs)
{
    return (  lhs.topLeftCorner== rhs.topLeftCorner &&
     lhs.width== rhs.width &&
     lhs.height== rhs.height &&
     lhs.attributeScore== rhs.attributeScore 
    );
}

std::size_t TLNBoundingBox::hash() const noexcept
{
    std::size_t result = 17;
    result = result * 31 + std::hash<point2D>()(topLeftCorner);
    result = result * 31 + std::hash<double>()(width);
    result = result * 31 + std::hash<double>()(height);
    result = result * 31 + std::hash<double>()(attributeScore);
    return result;
}

 } // namespace av
 } // namespace msg
 } // namespace tln


CEREAL_BIND_TO_ARCHIVES(av::msg::tln::TLNBoundingBox)

 namespace av {
 namespace msg {
 namespace tln {


TLNLens::TLNLens() = default;

TLNLens::TLNLens(
    point2D lensCenter_arg ,
    LIGHT_COLOR color_arg ,
    LIGHT_SHAPE shape_arg ,
    double attributeScore_arg 
    ):
    lensCenter { std::move(lensCenter_arg) } // NOLINT
      ,
    color { std::move(color_arg) } // NOLINT
      ,
    shape { std::move(shape_arg) } // NOLINT
      ,
    attributeScore { std::move(attributeScore_arg) } // NOLINT
      
    {
}

TLNLens::TLNLens(const TLNLens&) = default;

// NOLINTNEXTLINE
TLNLens::TLNLens(TLNLens&&) = default;

TLNLens& TLNLens::operator=(const TLNLens&) = default;

// NOLINTNEXTLINE
TLNLens& TLNLens::operator=(TLNLens&&) = default;

void TLNLens::toCsv(std::ostream& out) const
{
    out.precision(16);
    out << std::boolalpha;
    out << lensCenter;out << ",";
    out << static_cast<uint32_t>(color);out << ",";
    out << static_cast<uint32_t>(shape);out << ",";
    out << attributeScore;
    
}

std::stringstream TLNLens::toStringstream() const
{
    std::stringstream out;
    toCsv(out);
    return out;
}

template<class Archive>
    void TLNLens::serialize(Archive& ar, [[maybe_unused]] unsigned int version)
    {

        ar& CEREAL_NVP(lensCenter);
        ar& CEREAL_NVP(color);
        ar& CEREAL_NVP(shape);
        ar& CEREAL_NVP(attributeScore);
    }


template void TLNLens::serialize(cereal::BinaryInputArchive& ar, unsigned int version);
template void TLNLens::serialize(cereal::BinaryOutputArchive& ar, unsigned int version);
template void TLNLens::serialize(cereal::IntrospectionArchive& ar, unsigned int version);
template void TLNLens::serialize(cereal::JSONInputArchive& ar, unsigned int version);
template void TLNLens::serialize(cereal::JSONOutputArchive& ar, unsigned int version);
template void TLNLens::serialize(cereal::PortableBinaryInputArchive& ar, unsigned int version);
template void TLNLens::serialize(cereal::PortableBinaryOutputArchive& ar, unsigned int version);

std::ostream& operator<< (std::ostream& os, const TLNLens& tlnlens)
{
    tlnlens.toCsv(os);
    return os;
}

bool operator== (const TLNLens &lhs, const TLNLens &rhs)
{
    return (  lhs.lensCenter== rhs.lensCenter &&
     lhs.color== rhs.color &&
     lhs.shape== rhs.shape &&
     lhs.attributeScore== rhs.attributeScore 
    );
}

std::size_t TLNLens::hash() const noexcept
{
    std::size_t result = 17;
    result = result * 31 + std::hash<point2D>()(lensCenter);
    result = result * 31 + std::hash<LIGHT_COLOR>()(color);
    result = result * 31 + std::hash<LIGHT_SHAPE>()(shape);
    result = result * 31 + std::hash<double>()(attributeScore);
    return result;
}

 } // namespace av
 } // namespace msg
 } // namespace tln


CEREAL_BIND_TO_ARCHIVES(av::msg::tln::TLNLens)

 namespace av {
 namespace msg {
 namespace tln {


TLNObject::TLNObject() = default;

TLNObject::TLNObject(
    TLN_OBJECT_TYPE type_arg ,
    TLN_OBJECT_ORIENTATION orientation_arg ,
    double confidenceLevel_arg ,
    TLNBoundingBox boundingBox_arg ,
    std::vector<TLNLens> lenses_arg 
    ):
    type { std::move(type_arg) } // NOLINT
      ,
    orientation { std::move(orientation_arg) } // NOLINT
      ,
    confidenceLevel { std::move(confidenceLevel_arg) } // NOLINT
      ,
    boundingBox { std::move(boundingBox_arg) } // NOLINT
      ,
    lenses { std::move(lenses_arg) } // NOLINT
      
    {
}

TLNObject::TLNObject(const TLNObject&) = default;

// NOLINTNEXTLINE
TLNObject::TLNObject(TLNObject&&) = default;

TLNObject& TLNObject::operator=(const TLNObject&) = default;

// NOLINTNEXTLINE
TLNObject& TLNObject::operator=(TLNObject&&) = default;

void TLNObject::toCsv(std::ostream& out) const
{
    out.precision(16);
    out << std::boolalpha;
    out << static_cast<uint32_t>(type);out << ",";
    out << static_cast<uint32_t>(orientation);out << ",";
    out << confidenceLevel;out << ",";
    out << boundingBox;out << ",";
    // Enclose sequence in double quotations and curly brackets to keep CSV line length uniform
        
        out << "\"{{";
        
        if (!lenses.empty())
        {
            std::copy(lenses.begin(), lenses.end() - 1, std::ostream_iterator<TLNLens>(out, "},{"));
            std::copy(lenses.end()-1, lenses.end(), std::ostream_iterator<TLNLens>(out, ""));
        }
        out << "}}\"";
    
}

std::stringstream TLNObject::toStringstream() const
{
    std::stringstream out;
    toCsv(out);
    return out;
}

template<class Archive>
    void TLNObject::serialize(Archive& ar, [[maybe_unused]] unsigned int version)
    {

        ar& CEREAL_NVP(type);
        ar& CEREAL_NVP(orientation);
        ar& CEREAL_NVP(confidenceLevel);
        ar& CEREAL_NVP(boundingBox);
        ar& CEREAL_NVP(lenses);
    }


template void TLNObject::serialize(cereal::BinaryInputArchive& ar, unsigned int version);
template void TLNObject::serialize(cereal::BinaryOutputArchive& ar, unsigned int version);
template void TLNObject::serialize(cereal::IntrospectionArchive& ar, unsigned int version);
template void TLNObject::serialize(cereal::JSONInputArchive& ar, unsigned int version);
template void TLNObject::serialize(cereal::JSONOutputArchive& ar, unsigned int version);
template void TLNObject::serialize(cereal::PortableBinaryInputArchive& ar, unsigned int version);
template void TLNObject::serialize(cereal::PortableBinaryOutputArchive& ar, unsigned int version);

std::ostream& operator<< (std::ostream& os, const TLNObject& tlnobject)
{
    tlnobject.toCsv(os);
    return os;
}

bool operator== (const TLNObject &lhs, const TLNObject &rhs)
{
    return (  lhs.type== rhs.type &&
     lhs.orientation== rhs.orientation &&
     lhs.confidenceLevel== rhs.confidenceLevel &&
     lhs.boundingBox== rhs.boundingBox &&
     lhs.lenses== rhs.lenses 
    );
}

std::size_t TLNObject::hash() const noexcept
{
    std::size_t result = 17;
    result = result * 31 + std::hash<TLN_OBJECT_TYPE>()(type);
    result = result * 31 + std::hash<TLN_OBJECT_ORIENTATION>()(orientation);
    result = result * 31 + std::hash<double>()(confidenceLevel);
    result = result * 31 + std::hash<TLNBoundingBox>()(boundingBox);
    result = result * 31 + hashVector(lenses);
    return result;
}

 } // namespace av
 } // namespace msg
 } // namespace tln


CEREAL_BIND_TO_ARCHIVES(av::msg::tln::TLNObject)




 namespace av {
 namespace msg {
 namespace tln {


TrafficLightNetworkMessage::TrafficLightNetworkMessage() = default;

TrafficLightNetworkMessage::TrafficLightNetworkMessage(
    int64_t timestamp_arg ,
    std::string sensorId_arg ,
    std::vector<TLNObject> objects_arg ,
    TLN_MESSAGE_TYPE type_arg ,
    std::string cnn_model_arg 
    ):
    timestamp { std::move(timestamp_arg) } // NOLINT
      ,sensorId { std::move(sensorId_arg) } // NOLINT
      ,objects { std::move(objects_arg) } // NOLINT
      ,type { std::move(type_arg) } // NOLINT
      ,cnn_model { std::move(cnn_model_arg) } // NOLINT
      {
}

TrafficLightNetworkMessage::TrafficLightNetworkMessage(const TrafficLightNetworkMessage&) = default;

// NOLINTNEXTLINE
TrafficLightNetworkMessage::TrafficLightNetworkMessage(TrafficLightNetworkMessage&&) = default;

TrafficLightNetworkMessage& TrafficLightNetworkMessage::operator=(const TrafficLightNetworkMessage&) = default;

// NOLINTNEXTLINE
TrafficLightNetworkMessage& TrafficLightNetworkMessage::operator=(TrafficLightNetworkMessage&&) = default;

void TrafficLightNetworkMessage::toCsv(std::ostream& out) const
{
    out.precision(16);
    out << std::boolalpha;
    out << timestamp;out << ",";
    out << "\"";
        out << sensorId;
        out << "\"";out << ",";
    // Enclose sequence in double quotations and curly brackets to keep CSV line length uniform
        
        out << "\"{{";
        
        if (!objects.empty())
        {
            std::copy(objects.begin(), objects.end() - 1, std::ostream_iterator<TLNObject>(out, "},{"));
            std::copy(objects.end()-1, objects.end(), std::ostream_iterator<TLNObject>(out, ""));
        }
        out << "}}\"";out << ",";
    out << static_cast<uint32_t>(type);out << ",";
    out << "\"";
        out << cnn_model;
        out << "\"";
    
}

std::stringstream TrafficLightNetworkMessage::toStringstream() const
{
    std::stringstream out;
    toCsv(out);
    return out;
}

template<class Archive>
    void TrafficLightNetworkMessage::serialize(Archive& ar, [[maybe_unused]] unsigned int version)
    {

        ar& CEREAL_NVP(timestamp);
        ar& CEREAL_NVP(sensorId);
        ar& CEREAL_NVP(objects);
        ar& CEREAL_NVP(type);
        ar& CEREAL_NVP(cnn_model);
    }


template void TrafficLightNetworkMessage::serialize(cereal::BinaryInputArchive& ar, unsigned int version);
template void TrafficLightNetworkMessage::serialize(cereal::BinaryOutputArchive& ar, unsigned int version);
template void TrafficLightNetworkMessage::serialize(cereal::IntrospectionArchive& ar, unsigned int version);
template void TrafficLightNetworkMessage::serialize(cereal::JSONInputArchive& ar, unsigned int version);
template void TrafficLightNetworkMessage::serialize(cereal::JSONOutputArchive& ar, unsigned int version);
template void TrafficLightNetworkMessage::serialize(cereal::PortableBinaryInputArchive& ar, unsigned int version);
template void TrafficLightNetworkMessage::serialize(cereal::PortableBinaryOutputArchive& ar, unsigned int version);

std::ostream& operator<< (std::ostream& os, const TrafficLightNetworkMessage& trafficlightnetworkmessage)
{
    trafficlightnetworkmessage.toCsv(os);
    return os;
}

bool operator== (const TrafficLightNetworkMessage &lhs, const TrafficLightNetworkMessage &rhs)
{
    return (  lhs.timestamp== rhs.timestamp &&
     lhs.sensorId== rhs.sensorId &&
     lhs.objects== rhs.objects &&
     lhs.type== rhs.type &&
     lhs.cnn_model== rhs.cnn_model 
    );
}

std::size_t TrafficLightNetworkMessage::hash() const noexcept
{
    std::size_t result = 17;
    result = result * 31 + std::hash<int64_t>()(timestamp);
    result = result * 31 + std::hash<std::string>()(sensorId);
    result = result * 31 + hashVector(objects);
    result = result * 31 + std::hash<TLN_MESSAGE_TYPE>()(type);
    result = result * 31 + std::hash<std::string>()(cnn_model);
    return result;
}

 } // namespace av
 } // namespace msg
 } // namespace tln


CEREAL_BIND_TO_ARCHIVES(av::msg::tln::TrafficLightNetworkMessage)

