#include <interfaces/TransformStamped/TransformStamped.h>

#include <iterator>
#include <type_traits>

#include <cereal/cereal.hpp>
#include <cereal/archives/json.hpp>
#include <cereal/archives/binary.hpp>

#include <cereal/types/map.hpp>
#include <cereal/types/vector.hpp>
#include <cereal/types/array.hpp>
#include <cereal/types/string.hpp>
#include <cereal/types/complex.hpp>

#include <ODatum/IntrospectionArchive.h>
#include <ODatum/Hash.h>





 namespace av {
 namespace msg {


TransformStamped::TransformStamped() = default;

TransformStamped::TransformStamped(
    TimePoint timeStamp_arg ,
    std::string parentFrame_arg ,
    std::string childFrame_arg ,
    std::array<double,3> translation_arg ,
    std::array<double,4> rotation_arg ,
    bool isStatic_arg 
    ):
    timeStamp { std::move(timeStamp_arg) } // NOLINT
      ,parentFrame { std::move(parentFrame_arg) } // NOLINT
      ,childFrame { std::move(childFrame_arg) } // NOLINT
      ,translation { std::move(translation_arg) } // NOLINT
      ,rotation { std::move(rotation_arg) } // NOLINT
      ,isStatic { std::move(isStatic_arg) } // NOLINT
      {
}

TransformStamped::TransformStamped(const TransformStamped&) = default;

// NOLINTNEXTLINE
TransformStamped::TransformStamped(TransformStamped&&) = default;

TransformStamped& TransformStamped::operator=(const TransformStamped&) = default;

// NOLINTNEXTLINE
TransformStamped& TransformStamped::operator=(TransformStamped&&) = default;

void TransformStamped::toCsv(std::ostream& out) const
{
    out.precision(16);
    out << std::boolalpha;
    out << timeStamp;out << ",";
    out << "\"";
        out << parentFrame;
        out << "\"";out << ",";
    out << "\"";
        out << childFrame;
        out << "\"";out << ",";
    // Enclose array in double quotations and curly brackets to keep CSV line length uniform
        
        out << "\"{{";
        
        if (!translation.empty())
        {
            std::copy(translation.begin(), translation.end() - 1, std::ostream_iterator<double>(out, "},{"));
            out << translation.back();
        }
        out << "}}\"";out << ",";
    // Enclose array in double quotations and curly brackets to keep CSV line length uniform
        
        out << "\"{{";
        
        if (!rotation.empty())
        {
            std::copy(rotation.begin(), rotation.end() - 1, std::ostream_iterator<double>(out, "},{"));
            out << rotation.back();
        }
        out << "}}\"";out << ",";
    out << isStatic;
    
}

std::stringstream TransformStamped::toStringstream() const
{
    std::stringstream out;
    toCsv(out);
    return out;
}

template<class Archive>
    void TransformStamped::serialize(Archive& ar, unsigned int version)
{
    if (version < 3)
    {
        otimePoint otp;
        ar& cereal::make_nvp("timeStamp", otp);
        timeStamp = av::TimePoint(std::chrono::seconds(otp.tv_sec) + std::chrono::nanoseconds(otp.tv_nsec));
    }
    else
    {
        ar& CEREAL_NVP(timeStamp);
    }
    
    ar& CEREAL_NVP(parentFrame);
    ar& CEREAL_NVP(childFrame);
    ar& CEREAL_NVP(translation);
    ar& CEREAL_NVP(rotation);

    if (version > 1) {
        ar& CEREAL_NVP(isStatic);
    }
}

}} // to close av::msg namespace temporary

struct TransformStamped : public av::msg::TransformStamped {};
CEREAL_CLASS_VERSION(TransformStamped, 3)
CEREAL_REGISTER_TYPE(TransformStamped)

namespace av {
namespace msg {



template void TransformStamped::serialize(cereal::BinaryInputArchive& ar, unsigned int version);
template void TransformStamped::serialize(cereal::BinaryOutputArchive& ar, unsigned int version);
template void TransformStamped::serialize(cereal::IntrospectionArchive& ar, unsigned int version);
template void TransformStamped::serialize(cereal::JSONInputArchive& ar, unsigned int version);
template void TransformStamped::serialize(cereal::JSONOutputArchive& ar, unsigned int version);
template void TransformStamped::serialize(cereal::PortableBinaryInputArchive& ar, unsigned int version);
template void TransformStamped::serialize(cereal::PortableBinaryOutputArchive& ar, unsigned int version);

std::ostream& operator<< (std::ostream& os, const TransformStamped& transformstamped)
{
    transformstamped.toCsv(os);
    return os;
}

bool operator== (const TransformStamped &lhs, const TransformStamped &rhs)
{
    return (  lhs.timeStamp== rhs.timeStamp &&
     lhs.parentFrame== rhs.parentFrame &&
     lhs.childFrame== rhs.childFrame &&
     lhs.translation== rhs.translation &&
     lhs.rotation== rhs.rotation &&
     lhs.isStatic== rhs.isStatic 
    );
}

std::size_t TransformStamped::hash() const noexcept
{
    std::size_t result = 17;
    result = result * 31 + 0;
    result = result * 31 + std::hash<std::string>()(parentFrame);
    result = result * 31 + std::hash<std::string>()(childFrame);
    result = result * 31 + hashVector(translation);
    result = result * 31 + hashVector(rotation);
    result = result * 31 + std::hash<bool>()(isStatic);
    return result;
}

 } // namespace av
 } // namespace msg


CEREAL_BIND_TO_ARCHIVES(av::msg::TransformStamped)

