#include <interfaces/TransformStampedList/TransformStampedList.h>

#include <iterator>
#include <type_traits>

#include <cereal/cereal.hpp>
#include <cereal/archives/json.hpp>
#include <cereal/archives/binary.hpp>

#include <cereal/types/map.hpp>
#include <cereal/types/vector.hpp>
#include <cereal/types/array.hpp>
#include <cereal/types/string.hpp>
#include <cereal/types/complex.hpp>

#include <ODatum/IntrospectionArchive.h>
#include <ODatum/Hash.h>





 namespace av {
 namespace msg {


TransformStampedList::TransformStampedList() = default;

TransformStampedList::TransformStampedList(
    std::vector<TransformStamped> transforms_arg 
    ):
    transforms { std::move(transforms_arg) } // NOLINT
      {
}

TransformStampedList::TransformStampedList(const TransformStampedList&) = default;

// NOLINTNEXTLINE
TransformStampedList::TransformStampedList(TransformStampedList&&) = default;

TransformStampedList& TransformStampedList::operator=(const TransformStampedList&) = default;

// NOLINTNEXTLINE
TransformStampedList& TransformStampedList::operator=(TransformStampedList&&) = default;

void TransformStampedList::toCsv(std::ostream& out) const
{
    out.precision(16);
    out << std::boolalpha;
    // Enclose sequence in double quotations and curly brackets to keep CSV line length uniform
        
        out << "\"{{";
        
        if (!transforms.empty())
        {
            std::copy(transforms.begin(), transforms.end() - 1, std::ostream_iterator<TransformStamped>(out, "},{"));
            std::copy(transforms.end()-1, transforms.end(), std::ostream_iterator<TransformStamped>(out, ""));
        }
        out << "}}\"";
    
}

std::stringstream TransformStampedList::toStringstream() const
{
    std::stringstream out;
    toCsv(out);
    return out;
}

template<class Archive>
    void TransformStampedList::serialize(Archive& ar, unsigned int version)
{
        ar& CEREAL_NVP(transforms);
}

}} // to close av::msg namespace temporary

struct TransformStampedList : public av::msg::TransformStampedList {};
CEREAL_CLASS_VERSION(TransformStampedList, 2)
CEREAL_REGISTER_TYPE(TransformStampedList)

namespace av {
namespace msg {


template void TransformStampedList::serialize(cereal::BinaryInputArchive& ar, unsigned int version);
template void TransformStampedList::serialize(cereal::BinaryOutputArchive& ar, unsigned int version);
template void TransformStampedList::serialize(cereal::IntrospectionArchive& ar, unsigned int version);
template void TransformStampedList::serialize(cereal::JSONInputArchive& ar, unsigned int version);
template void TransformStampedList::serialize(cereal::JSONOutputArchive& ar, unsigned int version);
template void TransformStampedList::serialize(cereal::PortableBinaryInputArchive& ar, unsigned int version);
template void TransformStampedList::serialize(cereal::PortableBinaryOutputArchive& ar, unsigned int version);

std::ostream& operator<< (std::ostream& os, const TransformStampedList& transformstampedlist)
{
    transformstampedlist.toCsv(os);
    return os;
}

bool operator== (const TransformStampedList &lhs, const TransformStampedList &rhs)
{
    return (  lhs.transforms== rhs.transforms 
    );
}

std::size_t TransformStampedList::hash() const noexcept
{
    std::size_t result = 17;
    result = result * 31 + hashVector(transforms);
    return result;
}

 } // namespace av
 } // namespace msg


CEREAL_BIND_TO_ARCHIVES(av::msg::TransformStampedList)

