#include <interfaces/TrimbleGSOFState/TrimbleGSOFState.h>

#include <iterator>
#include <type_traits>

#include <cereal/cereal.hpp>
#include <cereal/archives/json.hpp>
#include <cereal/archives/binary.hpp>

#include <cereal/types/map.hpp>
#include <cereal/types/vector.hpp>
#include <cereal/types/array.hpp>
#include <cereal/types/string.hpp>
#include <cereal/types/complex.hpp>

#include <ODatum/IntrospectionArchive.h>
#include <ODatum/Hash.h>





 namespace av {
 namespace msg {


TrimbleGSOFState::TrimbleGSOFState() = default;

TrimbleGSOFState::TrimbleGSOFState(
    TimePoint timeStamp_arg ,
    bool utcOffsetValid_arg ,
    TimePoint gpsTimeStamp_arg ,
    bool gpsTimeValid_arg ,
    TimePoint sysTimeStamp_arg ,
    uint64_t msecs_t1_arg ,
    uint16_t weekNumber_t1_arg ,
    int32_t nSVs_t1_arg ,
    int32_t flags1_t1_arg ,
    int32_t flags2_t1_arg ,
    int32_t initNumber_t1_arg ,
    double lat_t2_arg ,
    double lon_t2_arg ,
    double height_t2_arg ,
    double X_t3_arg ,
    double Y_t3_arg ,
    double Z_t3_arg ,
    double deltaEast_t7_arg ,
    double deltaNorth_t7_arg ,
    double deltaUp_t7_arg ,
    int32_t flags_t8_arg ,
    float velocity_t8_arg ,
    float heading_t8_arg ,
    float vertical_t8_arg ,
    float pdop_t9_arg ,
    float hdop_t9_arg ,
    float vdop_t9_arg ,
    float tdop_t9_arg ,
    float positionRMS_t12_arg ,
    float sigmaEast_t12_arg ,
    float sigmaNorth_t12_arg ,
    float covarEastNorth_t12_arg ,
    float sigmaUp_t12_arg ,
    float simiMajorAxis_t12_arg ,
    float simiMinorAxis_t12_arg ,
    float orientation_t12_arg ,
    float unitVariance_t12_arg ,
    uint64_t msecs_t16_arg ,
    uint16_t weekNumber_t16_arg ,
    uint16_t utcOffset_t16_arg ,
    uint8_t flags_t16_arg ,
    double gpsTime_t27_arg ,
    uint8_t flags_t27_arg ,
    bool flags_pitch_valid_t27_arg ,
    bool flags_roll_valid_t27_arg ,
    uint8_t nSVs_t27_arg ,
    uint8_t mode_t27_arg ,
    double pitch_t27_arg ,
    double yaw_t27_arg ,
    double roll_t27_arg ,
    double range_t27_arg ,
    double pdop_t27_arg ,
    float pitch_var_t27_arg ,
    float yaw_var_t27_arg ,
    float roll_var_t27_arg ,
    float pitch_yaw_covar_t27_arg ,
    float pitch_roll_covar_t27_arg ,
    float yaw_roll_covar_t27_arg ,
    float range_var_t27_arg ,
    float positionType_t38_arg ,
    int32_t solutionFlags_t38_arg ,
    bool solutionFlags_rtk_fix_t38_arg ,
    int32_t rtkCondition_t38_arg ,
    uint64_t correctionAge_t38_arg ,
    int32_t networkFlags_t38_arg ,
    int32_t networkFlags2_t38_arg ,
    int32_t frameFlag_t38_arg ,
    uint16_t itrfEpoch_t38_arg ,
    int32_t techtonicPlate_t38_arg ,
    uint64_t minutesLeft_t38_arg ,
    int32_t poleWobbleStatusFlag_t38_arg ,
    float poleWobbleDistance_t38_arg 
    ):
    timeStamp { std::move(timeStamp_arg) } // NOLINT
      ,utcOffsetValid { std::move(utcOffsetValid_arg) } // NOLINT
      ,gpsTimeStamp { std::move(gpsTimeStamp_arg) } // NOLINT
      ,gpsTimeValid { std::move(gpsTimeValid_arg) } // NOLINT
      ,sysTimeStamp { std::move(sysTimeStamp_arg) } // NOLINT
      ,msecs_t1 { std::move(msecs_t1_arg) } // NOLINT
      ,weekNumber_t1 { std::move(weekNumber_t1_arg) } // NOLINT
      ,nSVs_t1 { std::move(nSVs_t1_arg) } // NOLINT
      ,flags1_t1 { std::move(flags1_t1_arg) } // NOLINT
      ,flags2_t1 { std::move(flags2_t1_arg) } // NOLINT
      ,initNumber_t1 { std::move(initNumber_t1_arg) } // NOLINT
      ,lat_t2 { std::move(lat_t2_arg) } // NOLINT
      ,lon_t2 { std::move(lon_t2_arg) } // NOLINT
      ,height_t2 { std::move(height_t2_arg) } // NOLINT
      ,X_t3 { std::move(X_t3_arg) } // NOLINT
      ,Y_t3 { std::move(Y_t3_arg) } // NOLINT
      ,Z_t3 { std::move(Z_t3_arg) } // NOLINT
      ,deltaEast_t7 { std::move(deltaEast_t7_arg) } // NOLINT
      ,deltaNorth_t7 { std::move(deltaNorth_t7_arg) } // NOLINT
      ,deltaUp_t7 { std::move(deltaUp_t7_arg) } // NOLINT
      ,flags_t8 { std::move(flags_t8_arg) } // NOLINT
      ,velocity_t8 { std::move(velocity_t8_arg) } // NOLINT
      ,heading_t8 { std::move(heading_t8_arg) } // NOLINT
      ,vertical_t8 { std::move(vertical_t8_arg) } // NOLINT
      ,pdop_t9 { std::move(pdop_t9_arg) } // NOLINT
      ,hdop_t9 { std::move(hdop_t9_arg) } // NOLINT
      ,vdop_t9 { std::move(vdop_t9_arg) } // NOLINT
      ,tdop_t9 { std::move(tdop_t9_arg) } // NOLINT
      ,positionRMS_t12 { std::move(positionRMS_t12_arg) } // NOLINT
      ,sigmaEast_t12 { std::move(sigmaEast_t12_arg) } // NOLINT
      ,sigmaNorth_t12 { std::move(sigmaNorth_t12_arg) } // NOLINT
      ,covarEastNorth_t12 { std::move(covarEastNorth_t12_arg) } // NOLINT
      ,sigmaUp_t12 { std::move(sigmaUp_t12_arg) } // NOLINT
      ,simiMajorAxis_t12 { std::move(simiMajorAxis_t12_arg) } // NOLINT
      ,simiMinorAxis_t12 { std::move(simiMinorAxis_t12_arg) } // NOLINT
      ,orientation_t12 { std::move(orientation_t12_arg) } // NOLINT
      ,unitVariance_t12 { std::move(unitVariance_t12_arg) } // NOLINT
      ,msecs_t16 { std::move(msecs_t16_arg) } // NOLINT
      ,weekNumber_t16 { std::move(weekNumber_t16_arg) } // NOLINT
      ,utcOffset_t16 { std::move(utcOffset_t16_arg) } // NOLINT
      ,flags_t16 { std::move(flags_t16_arg) } // NOLINT
      ,gpsTime_t27 { std::move(gpsTime_t27_arg) } // NOLINT
      ,flags_t27 { std::move(flags_t27_arg) } // NOLINT
      ,flags_pitch_valid_t27 { std::move(flags_pitch_valid_t27_arg) } // NOLINT
      ,flags_roll_valid_t27 { std::move(flags_roll_valid_t27_arg) } // NOLINT
      ,nSVs_t27 { std::move(nSVs_t27_arg) } // NOLINT
      ,mode_t27 { std::move(mode_t27_arg) } // NOLINT
      ,pitch_t27 { std::move(pitch_t27_arg) } // NOLINT
      ,yaw_t27 { std::move(yaw_t27_arg) } // NOLINT
      ,roll_t27 { std::move(roll_t27_arg) } // NOLINT
      ,range_t27 { std::move(range_t27_arg) } // NOLINT
      ,pdop_t27 { std::move(pdop_t27_arg) } // NOLINT
      ,pitch_var_t27 { std::move(pitch_var_t27_arg) } // NOLINT
      ,yaw_var_t27 { std::move(yaw_var_t27_arg) } // NOLINT
      ,roll_var_t27 { std::move(roll_var_t27_arg) } // NOLINT
      ,pitch_yaw_covar_t27 { std::move(pitch_yaw_covar_t27_arg) } // NOLINT
      ,pitch_roll_covar_t27 { std::move(pitch_roll_covar_t27_arg) } // NOLINT
      ,yaw_roll_covar_t27 { std::move(yaw_roll_covar_t27_arg) } // NOLINT
      ,range_var_t27 { std::move(range_var_t27_arg) } // NOLINT
      ,positionType_t38 { std::move(positionType_t38_arg) } // NOLINT
      ,solutionFlags_t38 { std::move(solutionFlags_t38_arg) } // NOLINT
      ,solutionFlags_rtk_fix_t38 { std::move(solutionFlags_rtk_fix_t38_arg) } // NOLINT
      ,rtkCondition_t38 { std::move(rtkCondition_t38_arg) } // NOLINT
      ,correctionAge_t38 { std::move(correctionAge_t38_arg) } // NOLINT
      ,networkFlags_t38 { std::move(networkFlags_t38_arg) } // NOLINT
      ,networkFlags2_t38 { std::move(networkFlags2_t38_arg) } // NOLINT
      ,frameFlag_t38 { std::move(frameFlag_t38_arg) } // NOLINT
      ,itrfEpoch_t38 { std::move(itrfEpoch_t38_arg) } // NOLINT
      ,techtonicPlate_t38 { std::move(techtonicPlate_t38_arg) } // NOLINT
      ,minutesLeft_t38 { std::move(minutesLeft_t38_arg) } // NOLINT
      ,poleWobbleStatusFlag_t38 { std::move(poleWobbleStatusFlag_t38_arg) } // NOLINT
      ,poleWobbleDistance_t38 { std::move(poleWobbleDistance_t38_arg) } // NOLINT
      {
}

TrimbleGSOFState::TrimbleGSOFState(const TrimbleGSOFState&) = default;

// NOLINTNEXTLINE
TrimbleGSOFState::TrimbleGSOFState(TrimbleGSOFState&&) = default;

TrimbleGSOFState& TrimbleGSOFState::operator=(const TrimbleGSOFState&) = default;

// NOLINTNEXTLINE
TrimbleGSOFState& TrimbleGSOFState::operator=(TrimbleGSOFState&&) = default;

void TrimbleGSOFState::toCsv(std::ostream& out) const
{
    out.precision(16);
    out << std::boolalpha;
    out << timeStamp;out << ",";
    out << utcOffsetValid;out << ",";
    out << gpsTimeStamp;out << ",";
    out << gpsTimeValid;out << ",";
    out << sysTimeStamp;out << ",";
    out << msecs_t1;out << ",";
    out << weekNumber_t1;out << ",";
    out << nSVs_t1;out << ",";
    out << flags1_t1;out << ",";
    out << flags2_t1;out << ",";
    out << initNumber_t1;out << ",";
    out << lat_t2;out << ",";
    out << lon_t2;out << ",";
    out << height_t2;out << ",";
    out << X_t3;out << ",";
    out << Y_t3;out << ",";
    out << Z_t3;out << ",";
    out << deltaEast_t7;out << ",";
    out << deltaNorth_t7;out << ",";
    out << deltaUp_t7;out << ",";
    out << flags_t8;out << ",";
    out << velocity_t8;out << ",";
    out << heading_t8;out << ",";
    out << vertical_t8;out << ",";
    out << pdop_t9;out << ",";
    out << hdop_t9;out << ",";
    out << vdop_t9;out << ",";
    out << tdop_t9;out << ",";
    out << positionRMS_t12;out << ",";
    out << sigmaEast_t12;out << ",";
    out << sigmaNorth_t12;out << ",";
    out << covarEastNorth_t12;out << ",";
    out << sigmaUp_t12;out << ",";
    out << simiMajorAxis_t12;out << ",";
    out << simiMinorAxis_t12;out << ",";
    out << orientation_t12;out << ",";
    out << unitVariance_t12;out << ",";
    out << msecs_t16;out << ",";
    out << weekNumber_t16;out << ",";
    out << utcOffset_t16;out << ",";
    // Cast to uint16_t to print as integer instead of char
        out << static_cast<uint16_t>(flags_t16);out << ",";
    out << gpsTime_t27;out << ",";
    // Cast to uint16_t to print as integer instead of char
        out << static_cast<uint16_t>(flags_t27);out << ",";
    out << flags_pitch_valid_t27;out << ",";
    out << flags_roll_valid_t27;out << ",";
    // Cast to uint16_t to print as integer instead of char
        out << static_cast<uint16_t>(nSVs_t27);out << ",";
    // Cast to uint16_t to print as integer instead of char
        out << static_cast<uint16_t>(mode_t27);out << ",";
    out << pitch_t27;out << ",";
    out << yaw_t27;out << ",";
    out << roll_t27;out << ",";
    out << range_t27;out << ",";
    out << pdop_t27;out << ",";
    out << pitch_var_t27;out << ",";
    out << yaw_var_t27;out << ",";
    out << roll_var_t27;out << ",";
    out << pitch_yaw_covar_t27;out << ",";
    out << pitch_roll_covar_t27;out << ",";
    out << yaw_roll_covar_t27;out << ",";
    out << range_var_t27;out << ",";
    out << positionType_t38;out << ",";
    out << solutionFlags_t38;out << ",";
    out << solutionFlags_rtk_fix_t38;out << ",";
    out << rtkCondition_t38;out << ",";
    out << correctionAge_t38;out << ",";
    out << networkFlags_t38;out << ",";
    out << networkFlags2_t38;out << ",";
    out << frameFlag_t38;out << ",";
    out << itrfEpoch_t38;out << ",";
    out << techtonicPlate_t38;out << ",";
    out << minutesLeft_t38;out << ",";
    out << poleWobbleStatusFlag_t38;out << ",";
    out << poleWobbleDistance_t38;
    
}

std::stringstream TrimbleGSOFState::toStringstream() const
{
    std::stringstream out;
    toCsv(out);
    return out;
}

template<class Archive>
    void TrimbleGSOFState::serialize(Archive& ar, unsigned int version)
{
    if (version >= 5) {
        ar& CEREAL_NVP(timeStamp);

        ar& CEREAL_NVP(gpsTimeStamp);
        ar& CEREAL_NVP(sysTimeStamp);

        ar& CEREAL_NVP(utcOffsetValid);
        ar& CEREAL_NVP(gpsTimeValid);
    }
    else if (version == 4) {
        otimePoint otp;
        ar& cereal::make_nvp("timeStamp", otp);
        timeStamp = av::TimePoint(std::chrono::seconds(otp.tv_sec) + std::chrono::nanoseconds(otp.tv_nsec));

        otimePoint gpsOtp;
        ar& cereal::make_nvp("gpsTimeStamp", gpsOtp);
        gpsTimeStamp = av::TimePoint(std::chrono::seconds(gpsOtp.tv_sec) + std::chrono::nanoseconds(gpsOtp.tv_nsec));

        otimePoint sysOtp;
        ar& cereal::make_nvp("sysTimeStamp", sysOtp);
        sysTimeStamp = av::TimePoint(std::chrono::seconds(sysOtp.tv_sec) + std::chrono::nanoseconds(sysOtp.tv_nsec));

        ar& CEREAL_NVP(utcOffsetValid);
        ar& CEREAL_NVP(gpsTimeValid);
    }

    // Type 1 record
    ar& CEREAL_NVP(msecs_t1);
    ar& CEREAL_NVP(weekNumber_t1);
    ar& CEREAL_NVP(nSVs_t1);
    ar& CEREAL_NVP(flags1_t1);
    ar& CEREAL_NVP(flags2_t1);
    ar& CEREAL_NVP(initNumber_t1);

    // Type 2 record
    ar& CEREAL_NVP(lat_t2);
    ar& CEREAL_NVP(lon_t2);
    ar& CEREAL_NVP(height_t2);

    // Type 3 record
    ar& CEREAL_NVP(X_t3);
    ar& CEREAL_NVP(Y_t3);
    ar& CEREAL_NVP(Z_t3);

    // Type 8 record
    ar& CEREAL_NVP(flags_t8);
    ar& CEREAL_NVP(velocity_t8);
    ar& CEREAL_NVP(heading_t8);
    ar& CEREAL_NVP(vertical_t8);

    // Type 12 record
    ar& CEREAL_NVP(positionRMS_t12);
    ar& CEREAL_NVP(sigmaEast_t12);
    ar& CEREAL_NVP(sigmaNorth_t12);
    ar& CEREAL_NVP(covarEastNorth_t12);
    ar& CEREAL_NVP(sigmaUp_t12);
    ar& CEREAL_NVP(simiMajorAxis_t12);
    ar& CEREAL_NVP(simiMinorAxis_t12);
    ar& CEREAL_NVP(orientation_t12);
    ar& CEREAL_NVP(unitVariance_t12);

    // Type 27 record
    ar& CEREAL_NVP(gpsTime_t27);
    ar& CEREAL_NVP(flags_t27);
    if (version >= 3) {
        ar& CEREAL_NVP(flags_pitch_valid_t27);
        ar& CEREAL_NVP(flags_roll_valid_t27);
    }
    ar& CEREAL_NVP(nSVs_t27);
    ar& CEREAL_NVP(mode_t27);
    ar& CEREAL_NVP(pitch_t27);
    ar& CEREAL_NVP(yaw_t27);
    ar& CEREAL_NVP(roll_t27);
    ar& CEREAL_NVP(range_t27);
    ar& CEREAL_NVP(pdop_t27);

    ar& CEREAL_NVP(pitch_var_t27);
    ar& CEREAL_NVP(yaw_var_t27);
    ar& CEREAL_NVP(roll_var_t27);
    ar& CEREAL_NVP(pitch_yaw_covar_t27);
    ar& CEREAL_NVP(pitch_roll_covar_t27);
    ar& CEREAL_NVP(yaw_roll_covar_t27);
    ar& CEREAL_NVP(range_var_t27);

    // Type 38 record
    ar& CEREAL_NVP(positionType_t38);
    ar& CEREAL_NVP(solutionFlags_t38);
    if (version >= 3) {
        ar& CEREAL_NVP(solutionFlags_rtk_fix_t38);
    }
    ar& CEREAL_NVP(rtkCondition_t38);
    ar& CEREAL_NVP(correctionAge_t38);
    ar& CEREAL_NVP(networkFlags_t38);
    ar& CEREAL_NVP(networkFlags2_t38);
    ar& CEREAL_NVP(frameFlag_t38);
    ar& CEREAL_NVP(itrfEpoch_t38);
    ar& CEREAL_NVP(techtonicPlate_t38);
    ar& CEREAL_NVP(minutesLeft_t38);
    ar& CEREAL_NVP(poleWobbleStatusFlag_t38);
    ar& CEREAL_NVP(poleWobbleDistance_t38);

    if (version >= 2) {
        // Type 7 record (TPlane ENU)
        ar& CEREAL_NVP(deltaEast_t7);
        ar& CEREAL_NVP(deltaNorth_t7);
        ar& CEREAL_NVP(deltaUp_t7);

        // Type 9 record (DOP Info)
        ar& CEREAL_NVP(pdop_t9);
        ar& CEREAL_NVP(hdop_t9);
        ar& CEREAL_NVP(vdop_t9);
        ar& CEREAL_NVP(tdop_t9);

        // Type 16 record (Current Time UTC)
        ar& CEREAL_NVP(msecs_t16);
        ar& CEREAL_NVP(weekNumber_t16);
        ar& CEREAL_NVP(utcOffset_t16);
        ar& CEREAL_NVP(flags_t16);
    }
}
}} // close av::msg namespace

struct TrimbleGSOFMessage : public av::msg::TrimbleGSOFState {};
CEREAL_CLASS_VERSION(TrimbleGSOFMessage, 5)
CEREAL_REGISTER_TYPE(TrimbleGSOFMessage)

namespace av {
namespace msg {


template void TrimbleGSOFState::serialize(cereal::BinaryInputArchive& ar, unsigned int version);
template void TrimbleGSOFState::serialize(cereal::BinaryOutputArchive& ar, unsigned int version);
template void TrimbleGSOFState::serialize(cereal::IntrospectionArchive& ar, unsigned int version);
template void TrimbleGSOFState::serialize(cereal::JSONInputArchive& ar, unsigned int version);
template void TrimbleGSOFState::serialize(cereal::JSONOutputArchive& ar, unsigned int version);
template void TrimbleGSOFState::serialize(cereal::PortableBinaryInputArchive& ar, unsigned int version);
template void TrimbleGSOFState::serialize(cereal::PortableBinaryOutputArchive& ar, unsigned int version);

std::ostream& operator<< (std::ostream& os, const TrimbleGSOFState& trimblegsofstate)
{
    trimblegsofstate.toCsv(os);
    return os;
}

bool operator== (const TrimbleGSOFState &lhs, const TrimbleGSOFState &rhs)
{
    return (  lhs.timeStamp== rhs.timeStamp &&
     lhs.utcOffsetValid== rhs.utcOffsetValid &&
     lhs.gpsTimeStamp== rhs.gpsTimeStamp &&
     lhs.gpsTimeValid== rhs.gpsTimeValid &&
     lhs.sysTimeStamp== rhs.sysTimeStamp &&
     lhs.msecs_t1== rhs.msecs_t1 &&
     lhs.weekNumber_t1== rhs.weekNumber_t1 &&
     lhs.nSVs_t1== rhs.nSVs_t1 &&
     lhs.flags1_t1== rhs.flags1_t1 &&
     lhs.flags2_t1== rhs.flags2_t1 &&
     lhs.initNumber_t1== rhs.initNumber_t1 &&
     lhs.lat_t2== rhs.lat_t2 &&
     lhs.lon_t2== rhs.lon_t2 &&
     lhs.height_t2== rhs.height_t2 &&
     lhs.X_t3== rhs.X_t3 &&
     lhs.Y_t3== rhs.Y_t3 &&
     lhs.Z_t3== rhs.Z_t3 &&
     lhs.deltaEast_t7== rhs.deltaEast_t7 &&
     lhs.deltaNorth_t7== rhs.deltaNorth_t7 &&
     lhs.deltaUp_t7== rhs.deltaUp_t7 &&
     lhs.flags_t8== rhs.flags_t8 &&
     lhs.velocity_t8== rhs.velocity_t8 &&
     lhs.heading_t8== rhs.heading_t8 &&
     lhs.vertical_t8== rhs.vertical_t8 &&
     lhs.pdop_t9== rhs.pdop_t9 &&
     lhs.hdop_t9== rhs.hdop_t9 &&
     lhs.vdop_t9== rhs.vdop_t9 &&
     lhs.tdop_t9== rhs.tdop_t9 &&
     lhs.positionRMS_t12== rhs.positionRMS_t12 &&
     lhs.sigmaEast_t12== rhs.sigmaEast_t12 &&
     lhs.sigmaNorth_t12== rhs.sigmaNorth_t12 &&
     lhs.covarEastNorth_t12== rhs.covarEastNorth_t12 &&
     lhs.sigmaUp_t12== rhs.sigmaUp_t12 &&
     lhs.simiMajorAxis_t12== rhs.simiMajorAxis_t12 &&
     lhs.simiMinorAxis_t12== rhs.simiMinorAxis_t12 &&
     lhs.orientation_t12== rhs.orientation_t12 &&
     lhs.unitVariance_t12== rhs.unitVariance_t12 &&
     lhs.msecs_t16== rhs.msecs_t16 &&
     lhs.weekNumber_t16== rhs.weekNumber_t16 &&
     lhs.utcOffset_t16== rhs.utcOffset_t16 &&
     lhs.flags_t16== rhs.flags_t16 &&
     lhs.gpsTime_t27== rhs.gpsTime_t27 &&
     lhs.flags_t27== rhs.flags_t27 &&
     lhs.flags_pitch_valid_t27== rhs.flags_pitch_valid_t27 &&
     lhs.flags_roll_valid_t27== rhs.flags_roll_valid_t27 &&
     lhs.nSVs_t27== rhs.nSVs_t27 &&
     lhs.mode_t27== rhs.mode_t27 &&
     lhs.pitch_t27== rhs.pitch_t27 &&
     lhs.yaw_t27== rhs.yaw_t27 &&
     lhs.roll_t27== rhs.roll_t27 &&
     lhs.range_t27== rhs.range_t27 &&
     lhs.pdop_t27== rhs.pdop_t27 &&
     lhs.pitch_var_t27== rhs.pitch_var_t27 &&
     lhs.yaw_var_t27== rhs.yaw_var_t27 &&
     lhs.roll_var_t27== rhs.roll_var_t27 &&
     lhs.pitch_yaw_covar_t27== rhs.pitch_yaw_covar_t27 &&
     lhs.pitch_roll_covar_t27== rhs.pitch_roll_covar_t27 &&
     lhs.yaw_roll_covar_t27== rhs.yaw_roll_covar_t27 &&
     lhs.range_var_t27== rhs.range_var_t27 &&
     lhs.positionType_t38== rhs.positionType_t38 &&
     lhs.solutionFlags_t38== rhs.solutionFlags_t38 &&
     lhs.solutionFlags_rtk_fix_t38== rhs.solutionFlags_rtk_fix_t38 &&
     lhs.rtkCondition_t38== rhs.rtkCondition_t38 &&
     lhs.correctionAge_t38== rhs.correctionAge_t38 &&
     lhs.networkFlags_t38== rhs.networkFlags_t38 &&
     lhs.networkFlags2_t38== rhs.networkFlags2_t38 &&
     lhs.frameFlag_t38== rhs.frameFlag_t38 &&
     lhs.itrfEpoch_t38== rhs.itrfEpoch_t38 &&
     lhs.techtonicPlate_t38== rhs.techtonicPlate_t38 &&
     lhs.minutesLeft_t38== rhs.minutesLeft_t38 &&
     lhs.poleWobbleStatusFlag_t38== rhs.poleWobbleStatusFlag_t38 &&
     lhs.poleWobbleDistance_t38== rhs.poleWobbleDistance_t38 
    );
}

std::size_t TrimbleGSOFState::hash() const noexcept
{
    std::size_t result = 17;
    result = result * 31 + 0;
    result = result * 31 + std::hash<bool>()(utcOffsetValid);
    result = result * 31 + 0;
    result = result * 31 + std::hash<bool>()(gpsTimeValid);
    result = result * 31 + 0;
    result = result * 31 + std::hash<uint64_t>()(msecs_t1);
    result = result * 31 + std::hash<uint16_t>()(weekNumber_t1);
    result = result * 31 + std::hash<int32_t>()(nSVs_t1);
    result = result * 31 + std::hash<int32_t>()(flags1_t1);
    result = result * 31 + std::hash<int32_t>()(flags2_t1);
    result = result * 31 + std::hash<int32_t>()(initNumber_t1);
    result = result * 31 + std::hash<double>()(lat_t2);
    result = result * 31 + std::hash<double>()(lon_t2);
    result = result * 31 + std::hash<double>()(height_t2);
    result = result * 31 + std::hash<double>()(X_t3);
    result = result * 31 + std::hash<double>()(Y_t3);
    result = result * 31 + std::hash<double>()(Z_t3);
    result = result * 31 + std::hash<double>()(deltaEast_t7);
    result = result * 31 + std::hash<double>()(deltaNorth_t7);
    result = result * 31 + std::hash<double>()(deltaUp_t7);
    result = result * 31 + std::hash<int32_t>()(flags_t8);
    result = result * 31 + std::hash<float>()(velocity_t8);
    result = result * 31 + std::hash<float>()(heading_t8);
    result = result * 31 + std::hash<float>()(vertical_t8);
    result = result * 31 + std::hash<float>()(pdop_t9);
    result = result * 31 + std::hash<float>()(hdop_t9);
    result = result * 31 + std::hash<float>()(vdop_t9);
    result = result * 31 + std::hash<float>()(tdop_t9);
    result = result * 31 + std::hash<float>()(positionRMS_t12);
    result = result * 31 + std::hash<float>()(sigmaEast_t12);
    result = result * 31 + std::hash<float>()(sigmaNorth_t12);
    result = result * 31 + std::hash<float>()(covarEastNorth_t12);
    result = result * 31 + std::hash<float>()(sigmaUp_t12);
    result = result * 31 + std::hash<float>()(simiMajorAxis_t12);
    result = result * 31 + std::hash<float>()(simiMinorAxis_t12);
    result = result * 31 + std::hash<float>()(orientation_t12);
    result = result * 31 + std::hash<float>()(unitVariance_t12);
    result = result * 31 + std::hash<uint64_t>()(msecs_t16);
    result = result * 31 + std::hash<uint16_t>()(weekNumber_t16);
    result = result * 31 + std::hash<uint16_t>()(utcOffset_t16);
    result = result * 31 + std::hash<uint8_t>()(flags_t16);
    result = result * 31 + std::hash<double>()(gpsTime_t27);
    result = result * 31 + std::hash<uint8_t>()(flags_t27);
    result = result * 31 + std::hash<bool>()(flags_pitch_valid_t27);
    result = result * 31 + std::hash<bool>()(flags_roll_valid_t27);
    result = result * 31 + std::hash<uint8_t>()(nSVs_t27);
    result = result * 31 + std::hash<uint8_t>()(mode_t27);
    result = result * 31 + std::hash<double>()(pitch_t27);
    result = result * 31 + std::hash<double>()(yaw_t27);
    result = result * 31 + std::hash<double>()(roll_t27);
    result = result * 31 + std::hash<double>()(range_t27);
    result = result * 31 + std::hash<double>()(pdop_t27);
    result = result * 31 + std::hash<float>()(pitch_var_t27);
    result = result * 31 + std::hash<float>()(yaw_var_t27);
    result = result * 31 + std::hash<float>()(roll_var_t27);
    result = result * 31 + std::hash<float>()(pitch_yaw_covar_t27);
    result = result * 31 + std::hash<float>()(pitch_roll_covar_t27);
    result = result * 31 + std::hash<float>()(yaw_roll_covar_t27);
    result = result * 31 + std::hash<float>()(range_var_t27);
    result = result * 31 + std::hash<float>()(positionType_t38);
    result = result * 31 + std::hash<int32_t>()(solutionFlags_t38);
    result = result * 31 + std::hash<bool>()(solutionFlags_rtk_fix_t38);
    result = result * 31 + std::hash<int32_t>()(rtkCondition_t38);
    result = result * 31 + std::hash<uint64_t>()(correctionAge_t38);
    result = result * 31 + std::hash<int32_t>()(networkFlags_t38);
    result = result * 31 + std::hash<int32_t>()(networkFlags2_t38);
    result = result * 31 + std::hash<int32_t>()(frameFlag_t38);
    result = result * 31 + std::hash<uint16_t>()(itrfEpoch_t38);
    result = result * 31 + std::hash<int32_t>()(techtonicPlate_t38);
    result = result * 31 + std::hash<uint64_t>()(minutesLeft_t38);
    result = result * 31 + std::hash<int32_t>()(poleWobbleStatusFlag_t38);
    result = result * 31 + std::hash<float>()(poleWobbleDistance_t38);
    return result;
}

 } // namespace av
 } // namespace msg


CEREAL_BIND_TO_ARCHIVES(av::msg::TrimbleGSOFState)

