#include <interfaces/TurnSignalCommand/TurnSignalCommand.h>

#include <iterator>
#include <type_traits>

#include <cereal/cereal.hpp>
#include <cereal/archives/json.hpp>
#include <cereal/archives/binary.hpp>

#include <cereal/types/map.hpp>
#include <cereal/types/vector.hpp>
#include <cereal/types/array.hpp>
#include <cereal/types/string.hpp>
#include <cereal/types/complex.hpp>

#include <ODatum/IntrospectionArchive.h>
#include <ODatum/Hash.h>





 namespace av {
 namespace msg {


TurnSignalCommand::TurnSignalCommand() = default;

TurnSignalCommand::TurnSignalCommand(
    TimePoint timeStamp_arg ,
    TurnSignalCommands commandId_arg 
    ):
    timeStamp { std::move(timeStamp_arg) } // NOLINT
      ,commandId { std::move(commandId_arg) } // NOLINT
      {
}

TurnSignalCommand::TurnSignalCommand(const TurnSignalCommand&) = default;

// NOLINTNEXTLINE
TurnSignalCommand::TurnSignalCommand(TurnSignalCommand&&) = default;

TurnSignalCommand& TurnSignalCommand::operator=(const TurnSignalCommand&) = default;

// NOLINTNEXTLINE
TurnSignalCommand& TurnSignalCommand::operator=(TurnSignalCommand&&) = default;

void TurnSignalCommand::toCsv(std::ostream& out) const
{
    out.precision(16);
    out << std::boolalpha;
    out << timeStamp;out << ",";
    out << static_cast<uint32_t>(commandId);
    
}

std::stringstream TurnSignalCommand::toStringstream() const
{
    std::stringstream out;
    toCsv(out);
    return out;
}

template<class Archive>
    void TurnSignalCommand::serialize(Archive& ar, unsigned int version)
{
    if (version < 2)
    {
        // This expects the manually defined version 1 TurnSignalCommand message and cannot parse the version 1
        // turn_signal_command_avt message (which was never published)
        otimePoint otp;
        ar& cereal::make_nvp("timeStamp", otp);
        timeStamp = av::TimePoint(std::chrono::seconds(otp.tv_sec) + std::chrono::nanoseconds(otp.tv_nsec));
    }
    else
    {
        ar& CEREAL_NVP(timeStamp);
    }

    ar& CEREAL_NVP(commandId);
}

}} // close av::msg namespace

struct TurnSignalCommand : public av::msg::TurnSignalCommand {};
CEREAL_CLASS_VERSION(TurnSignalCommand, 2)
CEREAL_REGISTER_TYPE(TurnSignalCommand)

namespace av {
namespace msg {


template void TurnSignalCommand::serialize(cereal::BinaryInputArchive& ar, unsigned int version);
template void TurnSignalCommand::serialize(cereal::BinaryOutputArchive& ar, unsigned int version);
template void TurnSignalCommand::serialize(cereal::IntrospectionArchive& ar, unsigned int version);
template void TurnSignalCommand::serialize(cereal::JSONInputArchive& ar, unsigned int version);
template void TurnSignalCommand::serialize(cereal::JSONOutputArchive& ar, unsigned int version);
template void TurnSignalCommand::serialize(cereal::PortableBinaryInputArchive& ar, unsigned int version);
template void TurnSignalCommand::serialize(cereal::PortableBinaryOutputArchive& ar, unsigned int version);

std::ostream& operator<< (std::ostream& os, const TurnSignalCommand& turnsignalcommand)
{
    turnsignalcommand.toCsv(os);
    return os;
}

bool operator== (const TurnSignalCommand &lhs, const TurnSignalCommand &rhs)
{
    return (  lhs.timeStamp== rhs.timeStamp &&
     lhs.commandId== rhs.commandId 
    );
}

std::size_t TurnSignalCommand::hash() const noexcept
{
    std::size_t result = 17;
    result = result * 31 + 0;
    result = result * 31 + std::hash<TurnSignalCommands>()(commandId);
    return result;
}

 } // namespace av
 } // namespace msg


CEREAL_BIND_TO_ARCHIVES(av::msg::TurnSignalCommand)

