#include <interfaces/UserRequest/UserRequest.h>

#include <iterator>
#include <type_traits>

#include <cereal/cereal.hpp>
#include <cereal/archives/json.hpp>
#include <cereal/archives/binary.hpp>

#include <cereal/types/map.hpp>
#include <cereal/types/vector.hpp>
#include <cereal/types/array.hpp>
#include <cereal/types/string.hpp>
#include <cereal/types/complex.hpp>

#include <ODatum/IntrospectionArchive.h>
#include <ODatum/Hash.h>





 namespace av {
 namespace msg {


UserRequest::UserRequest() = default;

UserRequest::UserRequest(
    UserReqType type_arg ,
    std::string taskName_arg ,
    std::string hostName_arg 
    ):
    type { std::move(type_arg) } // NOLINT
      ,taskName { std::move(taskName_arg) } // NOLINT
      ,hostName { std::move(hostName_arg) } // NOLINT
      {
}

UserRequest::UserRequest(const UserRequest&) = default;

// NOLINTNEXTLINE
UserRequest::UserRequest(UserRequest&&) = default;

UserRequest& UserRequest::operator=(const UserRequest&) = default;

// NOLINTNEXTLINE
UserRequest& UserRequest::operator=(UserRequest&&) = default;

void UserRequest::toCsv(std::ostream& out) const
{
    out.precision(16);
    out << std::boolalpha;
    out << static_cast<uint32_t>(type);out << ",";
    out << "\"";
        out << taskName;
        out << "\"";out << ",";
    out << "\"";
        out << hostName;
        out << "\"";
    
}

std::stringstream UserRequest::toStringstream() const
{
    std::stringstream out;
    toCsv(out);
    return out;
}

template<class Archive>
    void UserRequest::serialize(Archive& ar, [[maybe_unused]] unsigned int version)
{
    ar& CEREAL_NVP(type);
    ar& CEREAL_NVP(taskName);
    ar& CEREAL_NVP(hostName);
}

}} // to close av::msg namespace temporary

struct UserRequest : public av::msg::UserRequest {};
CEREAL_CLASS_VERSION(UserRequest, 2)
CEREAL_REGISTER_TYPE(UserRequest)

namespace av {
namespace msg {



template void UserRequest::serialize(cereal::BinaryInputArchive& ar, unsigned int version);
template void UserRequest::serialize(cereal::BinaryOutputArchive& ar, unsigned int version);
template void UserRequest::serialize(cereal::IntrospectionArchive& ar, unsigned int version);
template void UserRequest::serialize(cereal::JSONInputArchive& ar, unsigned int version);
template void UserRequest::serialize(cereal::JSONOutputArchive& ar, unsigned int version);
template void UserRequest::serialize(cereal::PortableBinaryInputArchive& ar, unsigned int version);
template void UserRequest::serialize(cereal::PortableBinaryOutputArchive& ar, unsigned int version);

std::ostream& operator<< (std::ostream& os, const UserRequest& userrequest)
{
    userrequest.toCsv(os);
    return os;
}

bool operator== (const UserRequest &lhs, const UserRequest &rhs)
{
    return (  lhs.type== rhs.type &&
     lhs.taskName== rhs.taskName &&
     lhs.hostName== rhs.hostName 
    );
}

std::size_t UserRequest::hash() const noexcept
{
    std::size_t result = 17;
    result = result * 31 + std::hash<UserReqType>()(type);
    result = result * 31 + std::hash<std::string>()(taskName);
    result = result * 31 + std::hash<std::string>()(hostName);
    return result;
}

 } // namespace av
 } // namespace msg


CEREAL_BIND_TO_ARCHIVES(av::msg::UserRequest)

