#include <interfaces/ValidationTrigger/ValidationTrigger.h>

#include <iterator>
#include <type_traits>

#include <cereal/cereal.hpp>
#include <cereal/archives/json.hpp>
#include <cereal/archives/binary.hpp>

#include <cereal/types/map.hpp>
#include <cereal/types/vector.hpp>
#include <cereal/types/array.hpp>
#include <cereal/types/string.hpp>
#include <cereal/types/complex.hpp>

#include <ODatum/IntrospectionArchive.h>
#include <ODatum/Hash.h>





 namespace av {
 namespace msg {
 namespace calibration {


ValidationTrigger::ValidationTrigger() = default;

ValidationTrigger::ValidationTrigger(
    ValidationModalityType modality_arg ,
    std::vector<std::string> sensorNames_arg ,
    av::msg::TimePoint time_arg 
    ):
    modality { std::move(modality_arg) } // NOLINT
      ,sensorNames { std::move(sensorNames_arg) } // NOLINT
      ,time { std::move(time_arg) } // NOLINT
      {
}

ValidationTrigger::ValidationTrigger(const ValidationTrigger&) = default;

// NOLINTNEXTLINE
ValidationTrigger::ValidationTrigger(ValidationTrigger&&) = default;

ValidationTrigger& ValidationTrigger::operator=(const ValidationTrigger&) = default;

// NOLINTNEXTLINE
ValidationTrigger& ValidationTrigger::operator=(ValidationTrigger&&) = default;

void ValidationTrigger::toCsv(std::ostream& out) const
{
    out.precision(16);
    out << std::boolalpha;
    out << static_cast<uint32_t>(modality);out << ",";
    // Enclose sequence in double quotations and curly brackets to keep CSV line length uniform
        
        out << "\"{{";
        
        if (!sensorNames.empty())
        {
            std::copy(sensorNames.begin(), sensorNames.end() - 1, std::ostream_iterator<std::string>(out, "},{"));
            std::copy(sensorNames.end()-1, sensorNames.end(), std::ostream_iterator<std::string>(out, ""));
        }
        out << "}}\"";out << ",";
    out << time;
    
}

std::stringstream ValidationTrigger::toStringstream() const
{
    std::stringstream out;
    toCsv(out);
    return out;
}

template<class Archive>
    void ValidationTrigger::serialize(Archive& ar, [[maybe_unused]] unsigned int version)
    {

        ar& CEREAL_NVP(modality);
        ar& CEREAL_NVP(sensorNames);
        ar& CEREAL_NVP(time);
    }


template void ValidationTrigger::serialize(cereal::BinaryInputArchive& ar, unsigned int version);
template void ValidationTrigger::serialize(cereal::BinaryOutputArchive& ar, unsigned int version);
template void ValidationTrigger::serialize(cereal::IntrospectionArchive& ar, unsigned int version);
template void ValidationTrigger::serialize(cereal::JSONInputArchive& ar, unsigned int version);
template void ValidationTrigger::serialize(cereal::JSONOutputArchive& ar, unsigned int version);
template void ValidationTrigger::serialize(cereal::PortableBinaryInputArchive& ar, unsigned int version);
template void ValidationTrigger::serialize(cereal::PortableBinaryOutputArchive& ar, unsigned int version);

std::ostream& operator<< (std::ostream& os, const ValidationTrigger& validationtrigger)
{
    validationtrigger.toCsv(os);
    return os;
}

bool operator== (const ValidationTrigger &lhs, const ValidationTrigger &rhs)
{
    return (  lhs.modality== rhs.modality &&
     lhs.sensorNames== rhs.sensorNames &&
     lhs.time== rhs.time 
    );
}

std::size_t ValidationTrigger::hash() const noexcept
{
    std::size_t result = 17;
    result = result * 31 + std::hash<ValidationModalityType>()(modality);
    result = result * 31 + hashVector(sensorNames);
    result = result * 31 + std::hash<av::msg::TimePoint>()(time);
    return result;
}

 } // namespace av
 } // namespace msg
 } // namespace calibration


CEREAL_BIND_TO_ARCHIVES(av::msg::calibration::ValidationTrigger)

