#include <interfaces/Vector2d/Vector2d.h>

#include <iterator>
#include <type_traits>

#include <cereal/cereal.hpp>
#include <cereal/archives/json.hpp>
#include <cereal/archives/binary.hpp>

#include <cereal/types/map.hpp>
#include <cereal/types/vector.hpp>
#include <cereal/types/array.hpp>
#include <cereal/types/string.hpp>
#include <cereal/types/complex.hpp>

#include <ODatum/IntrospectionArchive.h>
#include <ODatum/Hash.h>


 namespace av {
 namespace geometry {


Vector2d::Vector2d() = default;

Vector2d::Vector2d(
    float x_arg ,
    float y_arg 
    ):
    x { std::move(x_arg) } // NOLINT
      ,
    y { std::move(y_arg) } // NOLINT
      
    {
}

Vector2d::Vector2d(const Vector2d&) = default;

// NOLINTNEXTLINE
Vector2d::Vector2d(Vector2d&&) = default;

Vector2d& Vector2d::operator=(const Vector2d&) = default;

// NOLINTNEXTLINE
Vector2d& Vector2d::operator=(Vector2d&&) = default;

void Vector2d::toCsv(std::ostream& out) const
{
    out.precision(16);
    out << std::boolalpha;
    out << x;out << ",";
    out << y;
    
}

std::stringstream Vector2d::toStringstream() const
{
    std::stringstream out;
    toCsv(out);
    return out;
}

template<class Archive>
    void Vector2d::serialize(Archive& ar, [[maybe_unused]] unsigned int version)
    {

        ar& CEREAL_NVP(x);
        ar& CEREAL_NVP(y);
    }


template void Vector2d::serialize(cereal::BinaryInputArchive& ar, unsigned int version);
template void Vector2d::serialize(cereal::BinaryOutputArchive& ar, unsigned int version);
template void Vector2d::serialize(cereal::IntrospectionArchive& ar, unsigned int version);
template void Vector2d::serialize(cereal::JSONInputArchive& ar, unsigned int version);
template void Vector2d::serialize(cereal::JSONOutputArchive& ar, unsigned int version);
template void Vector2d::serialize(cereal::PortableBinaryInputArchive& ar, unsigned int version);
template void Vector2d::serialize(cereal::PortableBinaryOutputArchive& ar, unsigned int version);

std::ostream& operator<< (std::ostream& os, const Vector2d& vector2d)
{
    vector2d.toCsv(os);
    return os;
}

bool operator== (const Vector2d &lhs, const Vector2d &rhs)
{
    return (  lhs.x== rhs.x &&
     lhs.y== rhs.y 
    );
}

std::size_t Vector2d::hash() const noexcept
{
    std::size_t result = 17;
    result = result * 31 + std::hash<float>()(x);
    result = result * 31 + std::hash<float>()(y);
    return result;
}

 } // namespace av
 } // namespace geometry


CEREAL_BIND_TO_ARCHIVES(av::geometry::Vector2d)


