#include <interfaces/Vector3d/Vector3d.h>

#include <iterator>
#include <type_traits>

#include <cereal/cereal.hpp>
#include <cereal/archives/json.hpp>
#include <cereal/archives/binary.hpp>

#include <cereal/types/map.hpp>
#include <cereal/types/vector.hpp>
#include <cereal/types/array.hpp>
#include <cereal/types/string.hpp>
#include <cereal/types/complex.hpp>

#include <ODatum/IntrospectionArchive.h>
#include <ODatum/Hash.h>


 namespace av {
 namespace geometry {


Vector3d::Vector3d() = default;

Vector3d::Vector3d(
    double x_arg ,
    double y_arg ,
    double z_arg 
    ):
    x { std::move(x_arg) } // NOLINT
      ,
    y { std::move(y_arg) } // NOLINT
      ,
    z { std::move(z_arg) } // NOLINT
      
    {
}

Vector3d::Vector3d(const Vector3d&) = default;

// NOLINTNEXTLINE
Vector3d::Vector3d(Vector3d&&) = default;

Vector3d& Vector3d::operator=(const Vector3d&) = default;

// NOLINTNEXTLINE
Vector3d& Vector3d::operator=(Vector3d&&) = default;

void Vector3d::toCsv(std::ostream& out) const
{
    out.precision(16);
    out << std::boolalpha;
    out << x;out << ",";
    out << y;out << ",";
    out << z;
    
}

std::stringstream Vector3d::toStringstream() const
{
    std::stringstream out;
    toCsv(out);
    return out;
}

template<class Archive>
    void Vector3d::serialize(Archive& ar, [[maybe_unused]] unsigned int version)
    {

        ar& CEREAL_NVP(x);
        ar& CEREAL_NVP(y);
        ar& CEREAL_NVP(z);
    }


template void Vector3d::serialize(cereal::BinaryInputArchive& ar, unsigned int version);
template void Vector3d::serialize(cereal::BinaryOutputArchive& ar, unsigned int version);
template void Vector3d::serialize(cereal::IntrospectionArchive& ar, unsigned int version);
template void Vector3d::serialize(cereal::JSONInputArchive& ar, unsigned int version);
template void Vector3d::serialize(cereal::JSONOutputArchive& ar, unsigned int version);
template void Vector3d::serialize(cereal::PortableBinaryInputArchive& ar, unsigned int version);
template void Vector3d::serialize(cereal::PortableBinaryOutputArchive& ar, unsigned int version);

std::ostream& operator<< (std::ostream& os, const Vector3d& vector3d)
{
    vector3d.toCsv(os);
    return os;
}

bool operator== (const Vector3d &lhs, const Vector3d &rhs)
{
    return (  lhs.x== rhs.x &&
     lhs.y== rhs.y &&
     lhs.z== rhs.z 
    );
}

std::size_t Vector3d::hash() const noexcept
{
    std::size_t result = 17;
    result = result * 31 + std::hash<double>()(x);
    result = result * 31 + std::hash<double>()(y);
    result = result * 31 + std::hash<double>()(z);
    return result;
}

 } // namespace av
 } // namespace geometry


CEREAL_BIND_TO_ARCHIVES(av::geometry::Vector3d)


