#include <interfaces/VectorOfGlobalStaticReference/VectorOfGlobalStaticReference.h>

#include <iterator>
#include <type_traits>

#include <cereal/cereal.hpp>
#include <cereal/archives/json.hpp>
#include <cereal/archives/binary.hpp>

#include <cereal/types/map.hpp>
#include <cereal/types/vector.hpp>
#include <cereal/types/array.hpp>
#include <cereal/types/string.hpp>
#include <cereal/types/complex.hpp>

#include <ODatum/IntrospectionArchive.h>
#include <ODatum/Hash.h>


 namespace av {
 namespace msg {


SampledTransition::SampledTransition() = default;

SampledTransition::SampledTransition(
    TimePoint time_arg ,
    double progressReference_arg ,
    double progressFrontBumper_arg ,
    double progressRearAxle_arg ,
    SE2 pose_arg ,
    double lateralReference_arg ,
    double curvature_arg ,
    double speed_arg 
    ):
    time { std::move(time_arg) } // NOLINT
      ,
    progressReference { std::move(progressReference_arg) } // NOLINT
      ,
    progressFrontBumper { std::move(progressFrontBumper_arg) } // NOLINT
      ,
    progressRearAxle { std::move(progressRearAxle_arg) } // NOLINT
      ,
    pose { std::move(pose_arg) } // NOLINT
      ,
    lateralReference { std::move(lateralReference_arg) } // NOLINT
      ,
    curvature { std::move(curvature_arg) } // NOLINT
      ,
    speed { std::move(speed_arg) } // NOLINT
      
    {
}

SampledTransition::SampledTransition(const SampledTransition&) = default;

// NOLINTNEXTLINE
SampledTransition::SampledTransition(SampledTransition&&) = default;

SampledTransition& SampledTransition::operator=(const SampledTransition&) = default;

// NOLINTNEXTLINE
SampledTransition& SampledTransition::operator=(SampledTransition&&) = default;

void SampledTransition::toCsv(std::ostream& out) const
{
    out.precision(16);
    out << std::boolalpha;
    out << time;out << ",";
    out << progressReference;out << ",";
    out << progressFrontBumper;out << ",";
    out << progressRearAxle;out << ",";
    out << pose;out << ",";
    out << lateralReference;out << ",";
    out << curvature;out << ",";
    out << speed;
    
}

std::stringstream SampledTransition::toStringstream() const
{
    std::stringstream out;
    toCsv(out);
    return out;
}

template<class Archive>
    void SampledTransition::serialize(Archive& ar, [[maybe_unused]] unsigned int version)
    {

        ar& CEREAL_NVP(time);
        ar& CEREAL_NVP(progressReference);
        ar& CEREAL_NVP(progressFrontBumper);
        ar& CEREAL_NVP(progressRearAxle);
        ar& CEREAL_NVP(pose);
        ar& CEREAL_NVP(lateralReference);
        ar& CEREAL_NVP(curvature);
        ar& CEREAL_NVP(speed);
    }


template void SampledTransition::serialize(cereal::BinaryInputArchive& ar, unsigned int version);
template void SampledTransition::serialize(cereal::BinaryOutputArchive& ar, unsigned int version);
template void SampledTransition::serialize(cereal::IntrospectionArchive& ar, unsigned int version);
template void SampledTransition::serialize(cereal::JSONInputArchive& ar, unsigned int version);
template void SampledTransition::serialize(cereal::JSONOutputArchive& ar, unsigned int version);
template void SampledTransition::serialize(cereal::PortableBinaryInputArchive& ar, unsigned int version);
template void SampledTransition::serialize(cereal::PortableBinaryOutputArchive& ar, unsigned int version);

std::ostream& operator<< (std::ostream& os, const SampledTransition& sampledtransition)
{
    sampledtransition.toCsv(os);
    return os;
}

bool operator== (const SampledTransition &lhs, const SampledTransition &rhs)
{
    return (  lhs.time== rhs.time &&
     lhs.progressReference== rhs.progressReference &&
     lhs.progressFrontBumper== rhs.progressFrontBumper &&
     lhs.progressRearAxle== rhs.progressRearAxle &&
     lhs.pose== rhs.pose &&
     lhs.lateralReference== rhs.lateralReference &&
     lhs.curvature== rhs.curvature &&
     lhs.speed== rhs.speed 
    );
}

std::size_t SampledTransition::hash() const noexcept
{
    std::size_t result = 17;
    result = result * 31 + 0;
    result = result * 31 + std::hash<double>()(progressReference);
    result = result * 31 + std::hash<double>()(progressFrontBumper);
    result = result * 31 + std::hash<double>()(progressRearAxle);
    result = result * 31 + std::hash<SE2>()(pose);
    result = result * 31 + std::hash<double>()(lateralReference);
    result = result * 31 + std::hash<double>()(curvature);
    result = result * 31 + std::hash<double>()(speed);
    return result;
}

 } // namespace av
 } // namespace msg


CEREAL_BIND_TO_ARCHIVES(av::msg::SampledTransition)

 namespace av {
 namespace msg {


VTConstraint::VTConstraint() = default;

VTConstraint::VTConstraint(
    TimePoint time_arg ,
    double s_arg ,
    double minSpeed_arg ,
    double accelAtMinSpeed_arg ,
    double maxSpeed_arg ,
    double accelAtMaxSpeed_arg 
    ):
    time { std::move(time_arg) } // NOLINT
      ,
    s { std::move(s_arg) } // NOLINT
      ,
    minSpeed { std::move(minSpeed_arg) } // NOLINT
      ,
    accelAtMinSpeed { std::move(accelAtMinSpeed_arg) } // NOLINT
      ,
    maxSpeed { std::move(maxSpeed_arg) } // NOLINT
      ,
    accelAtMaxSpeed { std::move(accelAtMaxSpeed_arg) } // NOLINT
      
    {
}

VTConstraint::VTConstraint(const VTConstraint&) = default;

// NOLINTNEXTLINE
VTConstraint::VTConstraint(VTConstraint&&) = default;

VTConstraint& VTConstraint::operator=(const VTConstraint&) = default;

// NOLINTNEXTLINE
VTConstraint& VTConstraint::operator=(VTConstraint&&) = default;

void VTConstraint::toCsv(std::ostream& out) const
{
    out.precision(16);
    out << std::boolalpha;
    out << time;out << ",";
    out << s;out << ",";
    out << minSpeed;out << ",";
    out << accelAtMinSpeed;out << ",";
    out << maxSpeed;out << ",";
    out << accelAtMaxSpeed;
    
}

std::stringstream VTConstraint::toStringstream() const
{
    std::stringstream out;
    toCsv(out);
    return out;
}

template<class Archive>
    void VTConstraint::serialize(Archive& ar, [[maybe_unused]] unsigned int version)
    {

        ar& CEREAL_NVP(time);
        ar& CEREAL_NVP(s);
        ar& CEREAL_NVP(minSpeed);
        ar& CEREAL_NVP(accelAtMinSpeed);
        ar& CEREAL_NVP(maxSpeed);
        ar& CEREAL_NVP(accelAtMaxSpeed);
    }


template void VTConstraint::serialize(cereal::BinaryInputArchive& ar, unsigned int version);
template void VTConstraint::serialize(cereal::BinaryOutputArchive& ar, unsigned int version);
template void VTConstraint::serialize(cereal::IntrospectionArchive& ar, unsigned int version);
template void VTConstraint::serialize(cereal::JSONInputArchive& ar, unsigned int version);
template void VTConstraint::serialize(cereal::JSONOutputArchive& ar, unsigned int version);
template void VTConstraint::serialize(cereal::PortableBinaryInputArchive& ar, unsigned int version);
template void VTConstraint::serialize(cereal::PortableBinaryOutputArchive& ar, unsigned int version);

std::ostream& operator<< (std::ostream& os, const VTConstraint& vtconstraint)
{
    vtconstraint.toCsv(os);
    return os;
}

bool operator== (const VTConstraint &lhs, const VTConstraint &rhs)
{
    return (  lhs.time== rhs.time &&
     lhs.s== rhs.s &&
     lhs.minSpeed== rhs.minSpeed &&
     lhs.accelAtMinSpeed== rhs.accelAtMinSpeed &&
     lhs.maxSpeed== rhs.maxSpeed &&
     lhs.accelAtMaxSpeed== rhs.accelAtMaxSpeed 
    );
}

std::size_t VTConstraint::hash() const noexcept
{
    std::size_t result = 17;
    result = result * 31 + 0;
    result = result * 31 + std::hash<double>()(s);
    result = result * 31 + std::hash<double>()(minSpeed);
    result = result * 31 + std::hash<double>()(accelAtMinSpeed);
    result = result * 31 + std::hash<double>()(maxSpeed);
    result = result * 31 + std::hash<double>()(accelAtMaxSpeed);
    return result;
}

 } // namespace av
 } // namespace msg


CEREAL_BIND_TO_ARCHIVES(av::msg::VTConstraint)

 namespace av {
 namespace msg {


SampledSTConstraint::SampledSTConstraint() = default;

SampledSTConstraint::SampledSTConstraint(
    TimePoint time_arg ,
    double station_arg ,
    HardAndSoftBounds stationBounds_arg ,
    double longitudinalSpeed_arg ,
    bool anchorLane_arg 
    ):
    time { std::move(time_arg) } // NOLINT
      ,
    station { std::move(station_arg) } // NOLINT
      ,
    stationBounds { std::move(stationBounds_arg) } // NOLINT
      ,
    longitudinalSpeed { std::move(longitudinalSpeed_arg) } // NOLINT
      ,
    anchorLane { std::move(anchorLane_arg) } // NOLINT
      
    {
}

SampledSTConstraint::SampledSTConstraint(const SampledSTConstraint&) = default;

// NOLINTNEXTLINE
SampledSTConstraint::SampledSTConstraint(SampledSTConstraint&&) = default;

SampledSTConstraint& SampledSTConstraint::operator=(const SampledSTConstraint&) = default;

// NOLINTNEXTLINE
SampledSTConstraint& SampledSTConstraint::operator=(SampledSTConstraint&&) = default;

void SampledSTConstraint::toCsv(std::ostream& out) const
{
    out.precision(16);
    out << std::boolalpha;
    out << time;out << ",";
    out << station;out << ",";
    out << stationBounds;out << ",";
    out << longitudinalSpeed;out << ",";
    out << anchorLane;
    
}

std::stringstream SampledSTConstraint::toStringstream() const
{
    std::stringstream out;
    toCsv(out);
    return out;
}

template<class Archive>
    void SampledSTConstraint::serialize(Archive& ar, unsigned int version)
{
    ar& CEREAL_NVP(time);
    ar& CEREAL_NVP(station);
    ar& CEREAL_NVP(stationBounds);
    if (version >= 2){
        ar& CEREAL_NVP(longitudinalSpeed);
    }
    ar& CEREAL_NVP(anchorLane);
}


template void SampledSTConstraint::serialize(cereal::BinaryInputArchive& ar, unsigned int version);
template void SampledSTConstraint::serialize(cereal::BinaryOutputArchive& ar, unsigned int version);
template void SampledSTConstraint::serialize(cereal::IntrospectionArchive& ar, unsigned int version);
template void SampledSTConstraint::serialize(cereal::JSONInputArchive& ar, unsigned int version);
template void SampledSTConstraint::serialize(cereal::JSONOutputArchive& ar, unsigned int version);
template void SampledSTConstraint::serialize(cereal::PortableBinaryInputArchive& ar, unsigned int version);
template void SampledSTConstraint::serialize(cereal::PortableBinaryOutputArchive& ar, unsigned int version);

std::ostream& operator<< (std::ostream& os, const SampledSTConstraint& sampledstconstraint)
{
    sampledstconstraint.toCsv(os);
    return os;
}

bool operator== (const SampledSTConstraint &lhs, const SampledSTConstraint &rhs)
{
    return (  lhs.time== rhs.time &&
     lhs.station== rhs.station &&
     lhs.stationBounds== rhs.stationBounds &&
     lhs.longitudinalSpeed== rhs.longitudinalSpeed &&
     lhs.anchorLane== rhs.anchorLane 
    );
}

std::size_t SampledSTConstraint::hash() const noexcept
{
    std::size_t result = 17;
    result = result * 31 + 0;
    result = result * 31 + std::hash<double>()(station);
    result = result * 31 + std::hash<HardAndSoftBounds>()(stationBounds);
    result = result * 31 + std::hash<double>()(longitudinalSpeed);
    result = result * 31 + std::hash<bool>()(anchorLane);
    return result;
}

 } // namespace av
 } // namespace msg


CEREAL_BIND_TO_ARCHIVES(av::msg::SampledSTConstraint)

 namespace av {
 namespace msg {


LateralTube::LateralTube() = default;

LateralTube::LateralTube(
    HardAndSoftBounds tubeRefpoint_arg ,
    HardAndSoftBounds tubeFrontAxle_arg ,
    HardAndSoftBounds tubeRearAxle_arg ,
    HardAndSoftBoundsSpatialConstraintType constraintType_arg 
    ):
    tubeRefpoint { std::move(tubeRefpoint_arg) } // NOLINT
      ,
    tubeFrontAxle { std::move(tubeFrontAxle_arg) } // NOLINT
      ,
    tubeRearAxle { std::move(tubeRearAxle_arg) } // NOLINT
      ,
    constraintType { std::move(constraintType_arg) } // NOLINT
      
    {
}

LateralTube::LateralTube(const LateralTube&) = default;

// NOLINTNEXTLINE
LateralTube::LateralTube(LateralTube&&) = default;

LateralTube& LateralTube::operator=(const LateralTube&) = default;

// NOLINTNEXTLINE
LateralTube& LateralTube::operator=(LateralTube&&) = default;

void LateralTube::toCsv(std::ostream& out) const
{
    out.precision(16);
    out << std::boolalpha;
    out << tubeRefpoint;out << ",";
    out << tubeFrontAxle;out << ",";
    out << tubeRearAxle;out << ",";
    out << constraintType;
    
}

std::stringstream LateralTube::toStringstream() const
{
    std::stringstream out;
    toCsv(out);
    return out;
}

template<class Archive>
    void LateralTube::serialize(Archive& ar, [[maybe_unused]] unsigned int version)
    {

        ar& CEREAL_NVP(tubeRefpoint);
        ar& CEREAL_NVP(tubeFrontAxle);
        ar& CEREAL_NVP(tubeRearAxle);
        ar& CEREAL_NVP(constraintType);
    }


template void LateralTube::serialize(cereal::BinaryInputArchive& ar, unsigned int version);
template void LateralTube::serialize(cereal::BinaryOutputArchive& ar, unsigned int version);
template void LateralTube::serialize(cereal::IntrospectionArchive& ar, unsigned int version);
template void LateralTube::serialize(cereal::JSONInputArchive& ar, unsigned int version);
template void LateralTube::serialize(cereal::JSONOutputArchive& ar, unsigned int version);
template void LateralTube::serialize(cereal::PortableBinaryInputArchive& ar, unsigned int version);
template void LateralTube::serialize(cereal::PortableBinaryOutputArchive& ar, unsigned int version);

std::ostream& operator<< (std::ostream& os, const LateralTube& lateraltube)
{
    lateraltube.toCsv(os);
    return os;
}

bool operator== (const LateralTube &lhs, const LateralTube &rhs)
{
    return (  lhs.tubeRefpoint== rhs.tubeRefpoint &&
     lhs.tubeFrontAxle== rhs.tubeFrontAxle &&
     lhs.tubeRearAxle== rhs.tubeRearAxle &&
     lhs.constraintType== rhs.constraintType 
    );
}

std::size_t LateralTube::hash() const noexcept
{
    std::size_t result = 17;
    result = result * 31 + std::hash<HardAndSoftBounds>()(tubeRefpoint);
    result = result * 31 + std::hash<HardAndSoftBounds>()(tubeFrontAxle);
    result = result * 31 + std::hash<HardAndSoftBounds>()(tubeRearAxle);
    result = result * 31 + std::hash<HardAndSoftBoundsSpatialConstraintType>()(constraintType);
    return result;
}

 } // namespace av
 } // namespace msg


CEREAL_BIND_TO_ARCHIVES(av::msg::LateralTube)

 namespace av {
 namespace msg {


TubeAsSpline::TubeAsSpline() = default;

TubeAsSpline::TubeAsSpline(
    std::vector<double> controlPointsHardUpper_arg ,
    std::vector<double> controlPointsHardLower_arg ,
    std::vector<double> controlPointsSoftUpper_arg ,
    std::vector<double> controlPointsSoftLower_arg 
    ):
    controlPointsHardUpper { std::move(controlPointsHardUpper_arg) } // NOLINT
      ,
    controlPointsHardLower { std::move(controlPointsHardLower_arg) } // NOLINT
      ,
    controlPointsSoftUpper { std::move(controlPointsSoftUpper_arg) } // NOLINT
      ,
    controlPointsSoftLower { std::move(controlPointsSoftLower_arg) } // NOLINT
      
    {
}

TubeAsSpline::TubeAsSpline(const TubeAsSpline&) = default;

// NOLINTNEXTLINE
TubeAsSpline::TubeAsSpline(TubeAsSpline&&) = default;

TubeAsSpline& TubeAsSpline::operator=(const TubeAsSpline&) = default;

// NOLINTNEXTLINE
TubeAsSpline& TubeAsSpline::operator=(TubeAsSpline&&) = default;

void TubeAsSpline::toCsv(std::ostream& out) const
{
    out.precision(16);
    out << std::boolalpha;
    // Enclose sequence in double quotations and curly brackets to keep CSV line length uniform
        
        out << "\"{{";
        
        if (!controlPointsHardUpper.empty())
        {
            std::copy(controlPointsHardUpper.begin(), controlPointsHardUpper.end() - 1, std::ostream_iterator<double>(out, "},{"));
            std::copy(controlPointsHardUpper.end()-1, controlPointsHardUpper.end(), std::ostream_iterator<double>(out, ""));
        }
        out << "}}\"";out << ",";
    // Enclose sequence in double quotations and curly brackets to keep CSV line length uniform
        
        out << "\"{{";
        
        if (!controlPointsHardLower.empty())
        {
            std::copy(controlPointsHardLower.begin(), controlPointsHardLower.end() - 1, std::ostream_iterator<double>(out, "},{"));
            std::copy(controlPointsHardLower.end()-1, controlPointsHardLower.end(), std::ostream_iterator<double>(out, ""));
        }
        out << "}}\"";out << ",";
    // Enclose sequence in double quotations and curly brackets to keep CSV line length uniform
        
        out << "\"{{";
        
        if (!controlPointsSoftUpper.empty())
        {
            std::copy(controlPointsSoftUpper.begin(), controlPointsSoftUpper.end() - 1, std::ostream_iterator<double>(out, "},{"));
            std::copy(controlPointsSoftUpper.end()-1, controlPointsSoftUpper.end(), std::ostream_iterator<double>(out, ""));
        }
        out << "}}\"";out << ",";
    // Enclose sequence in double quotations and curly brackets to keep CSV line length uniform
        
        out << "\"{{";
        
        if (!controlPointsSoftLower.empty())
        {
            std::copy(controlPointsSoftLower.begin(), controlPointsSoftLower.end() - 1, std::ostream_iterator<double>(out, "},{"));
            std::copy(controlPointsSoftLower.end()-1, controlPointsSoftLower.end(), std::ostream_iterator<double>(out, ""));
        }
        out << "}}\"";
    
}

std::stringstream TubeAsSpline::toStringstream() const
{
    std::stringstream out;
    toCsv(out);
    return out;
}

template<class Archive>
    void TubeAsSpline::serialize(Archive& ar, [[maybe_unused]] unsigned int version)
    {

        ar& CEREAL_NVP(controlPointsHardUpper);
        ar& CEREAL_NVP(controlPointsHardLower);
        ar& CEREAL_NVP(controlPointsSoftUpper);
        ar& CEREAL_NVP(controlPointsSoftLower);
    }


template void TubeAsSpline::serialize(cereal::BinaryInputArchive& ar, unsigned int version);
template void TubeAsSpline::serialize(cereal::BinaryOutputArchive& ar, unsigned int version);
template void TubeAsSpline::serialize(cereal::IntrospectionArchive& ar, unsigned int version);
template void TubeAsSpline::serialize(cereal::JSONInputArchive& ar, unsigned int version);
template void TubeAsSpline::serialize(cereal::JSONOutputArchive& ar, unsigned int version);
template void TubeAsSpline::serialize(cereal::PortableBinaryInputArchive& ar, unsigned int version);
template void TubeAsSpline::serialize(cereal::PortableBinaryOutputArchive& ar, unsigned int version);

std::ostream& operator<< (std::ostream& os, const TubeAsSpline& tubeasspline)
{
    tubeasspline.toCsv(os);
    return os;
}

bool operator== (const TubeAsSpline &lhs, const TubeAsSpline &rhs)
{
    return (  lhs.controlPointsHardUpper== rhs.controlPointsHardUpper &&
     lhs.controlPointsHardLower== rhs.controlPointsHardLower &&
     lhs.controlPointsSoftUpper== rhs.controlPointsSoftUpper &&
     lhs.controlPointsSoftLower== rhs.controlPointsSoftLower 
    );
}

std::size_t TubeAsSpline::hash() const noexcept
{
    std::size_t result = 17;
    result = result * 31 + hashVector(controlPointsHardUpper);
    result = result * 31 + hashVector(controlPointsHardLower);
    result = result * 31 + hashVector(controlPointsSoftUpper);
    result = result * 31 + hashVector(controlPointsSoftLower);
    return result;
}

 } // namespace av
 } // namespace msg


CEREAL_BIND_TO_ARCHIVES(av::msg::TubeAsSpline)

 namespace av {
 namespace msg {


TrackConstraintInfo::TrackConstraintInfo() = default;

TrackConstraintInfo::TrackConstraintInfo(
    EdgeClearanceInfo clearanceInfo_arg ,
    ProximityBiasingParameters params_arg 
    ):
    clearanceInfo { std::move(clearanceInfo_arg) } // NOLINT
      ,
    params { std::move(params_arg) } // NOLINT
      
    {
}

TrackConstraintInfo::TrackConstraintInfo(const TrackConstraintInfo&) = default;

// NOLINTNEXTLINE
TrackConstraintInfo::TrackConstraintInfo(TrackConstraintInfo&&) = default;

TrackConstraintInfo& TrackConstraintInfo::operator=(const TrackConstraintInfo&) = default;

// NOLINTNEXTLINE
TrackConstraintInfo& TrackConstraintInfo::operator=(TrackConstraintInfo&&) = default;

void TrackConstraintInfo::toCsv(std::ostream& out) const
{
    out.precision(16);
    out << std::boolalpha;
    out << clearanceInfo;out << ",";
    out << params;
    
}

std::stringstream TrackConstraintInfo::toStringstream() const
{
    std::stringstream out;
    toCsv(out);
    return out;
}

template<class Archive>
    void TrackConstraintInfo::serialize(Archive& ar, [[maybe_unused]] unsigned int version)
    {

        ar& CEREAL_NVP(clearanceInfo);
        ar& CEREAL_NVP(params);
    }


template void TrackConstraintInfo::serialize(cereal::BinaryInputArchive& ar, unsigned int version);
template void TrackConstraintInfo::serialize(cereal::BinaryOutputArchive& ar, unsigned int version);
template void TrackConstraintInfo::serialize(cereal::IntrospectionArchive& ar, unsigned int version);
template void TrackConstraintInfo::serialize(cereal::JSONInputArchive& ar, unsigned int version);
template void TrackConstraintInfo::serialize(cereal::JSONOutputArchive& ar, unsigned int version);
template void TrackConstraintInfo::serialize(cereal::PortableBinaryInputArchive& ar, unsigned int version);
template void TrackConstraintInfo::serialize(cereal::PortableBinaryOutputArchive& ar, unsigned int version);

std::ostream& operator<< (std::ostream& os, const TrackConstraintInfo& trackconstraintinfo)
{
    trackconstraintinfo.toCsv(os);
    return os;
}

bool operator== (const TrackConstraintInfo &lhs, const TrackConstraintInfo &rhs)
{
    return (  lhs.clearanceInfo== rhs.clearanceInfo &&
     lhs.params== rhs.params 
    );
}

std::size_t TrackConstraintInfo::hash() const noexcept
{
    std::size_t result = 17;
    result = result * 31 + std::hash<EdgeClearanceInfo>()(clearanceInfo);
    result = result * 31 + std::hash<ProximityBiasingParameters>()(params);
    return result;
}

 } // namespace av
 } // namespace msg


CEREAL_BIND_TO_ARCHIVES(av::msg::TrackConstraintInfo)

 namespace av {
 namespace msg {


BiasingConstraints::BiasingConstraints() = default;

BiasingConstraints::BiasingConstraints(
    double stationAlongPath_arg ,
    bool hasValue_arg ,
    TrackConstraintInfo biasingInfo_arg 
    ):
    stationAlongPath { std::move(stationAlongPath_arg) } // NOLINT
      ,
    hasValue { std::move(hasValue_arg) } // NOLINT
      ,
    biasingInfo { std::move(biasingInfo_arg) } // NOLINT
      
    {
}

BiasingConstraints::BiasingConstraints(const BiasingConstraints&) = default;

// NOLINTNEXTLINE
BiasingConstraints::BiasingConstraints(BiasingConstraints&&) = default;

BiasingConstraints& BiasingConstraints::operator=(const BiasingConstraints&) = default;

// NOLINTNEXTLINE
BiasingConstraints& BiasingConstraints::operator=(BiasingConstraints&&) = default;

void BiasingConstraints::toCsv(std::ostream& out) const
{
    out.precision(16);
    out << std::boolalpha;
    out << stationAlongPath;out << ",";
    out << hasValue;out << ",";
    out << biasingInfo;
    
}

std::stringstream BiasingConstraints::toStringstream() const
{
    std::stringstream out;
    toCsv(out);
    return out;
}

template<class Archive>
    void BiasingConstraints::serialize(Archive& ar, [[maybe_unused]] unsigned int version)
    {

        ar& CEREAL_NVP(stationAlongPath);
        ar& CEREAL_NVP(hasValue);
        ar& CEREAL_NVP(biasingInfo);
    }


template void BiasingConstraints::serialize(cereal::BinaryInputArchive& ar, unsigned int version);
template void BiasingConstraints::serialize(cereal::BinaryOutputArchive& ar, unsigned int version);
template void BiasingConstraints::serialize(cereal::IntrospectionArchive& ar, unsigned int version);
template void BiasingConstraints::serialize(cereal::JSONInputArchive& ar, unsigned int version);
template void BiasingConstraints::serialize(cereal::JSONOutputArchive& ar, unsigned int version);
template void BiasingConstraints::serialize(cereal::PortableBinaryInputArchive& ar, unsigned int version);
template void BiasingConstraints::serialize(cereal::PortableBinaryOutputArchive& ar, unsigned int version);

std::ostream& operator<< (std::ostream& os, const BiasingConstraints& biasingconstraints)
{
    biasingconstraints.toCsv(os);
    return os;
}

bool operator== (const BiasingConstraints &lhs, const BiasingConstraints &rhs)
{
    return (  lhs.stationAlongPath== rhs.stationAlongPath &&
     lhs.hasValue== rhs.hasValue &&
     lhs.biasingInfo== rhs.biasingInfo 
    );
}

std::size_t BiasingConstraints::hash() const noexcept
{
    std::size_t result = 17;
    result = result * 31 + std::hash<double>()(stationAlongPath);
    result = result * 31 + std::hash<bool>()(hasValue);
    result = result * 31 + std::hash<TrackConstraintInfo>()(biasingInfo);
    return result;
}

 } // namespace av
 } // namespace msg


CEREAL_BIND_TO_ARCHIVES(av::msg::BiasingConstraints)

 namespace av {
 namespace msg {


MostConstrainingTrack::MostConstrainingTrack() = default;

MostConstrainingTrack::MostConstrainingTrack(
    MostConstrainingTrackType type_arg ,
    int64_t trackId_arg ,
    int64_t mapId_arg ,
    std::string debug_arg 
    ):
    type { std::move(type_arg) } // NOLINT
      ,
    trackId { std::move(trackId_arg) } // NOLINT
      ,
    mapId { std::move(mapId_arg) } // NOLINT
      ,
    debug { std::move(debug_arg) } // NOLINT
      
    {
}

MostConstrainingTrack::MostConstrainingTrack(const MostConstrainingTrack&) = default;

// NOLINTNEXTLINE
MostConstrainingTrack::MostConstrainingTrack(MostConstrainingTrack&&) = default;

MostConstrainingTrack& MostConstrainingTrack::operator=(const MostConstrainingTrack&) = default;

// NOLINTNEXTLINE
MostConstrainingTrack& MostConstrainingTrack::operator=(MostConstrainingTrack&&) = default;

void MostConstrainingTrack::toCsv(std::ostream& out) const
{
    out.precision(16);
    out << std::boolalpha;
    out << static_cast<uint32_t>(type);out << ",";
    out << trackId;out << ",";
    out << mapId;out << ",";
    out << "\"";
        out << debug;
        out << "\"";
    
}

std::stringstream MostConstrainingTrack::toStringstream() const
{
    std::stringstream out;
    toCsv(out);
    return out;
}

template<class Archive>
    void MostConstrainingTrack::serialize(Archive& ar, [[maybe_unused]] unsigned int version)
    {

        ar& CEREAL_NVP(type);
        ar& CEREAL_NVP(trackId);
        ar& CEREAL_NVP(mapId);
        ar& CEREAL_NVP(debug);
    }


template void MostConstrainingTrack::serialize(cereal::BinaryInputArchive& ar, unsigned int version);
template void MostConstrainingTrack::serialize(cereal::BinaryOutputArchive& ar, unsigned int version);
template void MostConstrainingTrack::serialize(cereal::IntrospectionArchive& ar, unsigned int version);
template void MostConstrainingTrack::serialize(cereal::JSONInputArchive& ar, unsigned int version);
template void MostConstrainingTrack::serialize(cereal::JSONOutputArchive& ar, unsigned int version);
template void MostConstrainingTrack::serialize(cereal::PortableBinaryInputArchive& ar, unsigned int version);
template void MostConstrainingTrack::serialize(cereal::PortableBinaryOutputArchive& ar, unsigned int version);

std::ostream& operator<< (std::ostream& os, const MostConstrainingTrack& mostconstrainingtrack)
{
    mostconstrainingtrack.toCsv(os);
    return os;
}

bool operator== (const MostConstrainingTrack &lhs, const MostConstrainingTrack &rhs)
{
    return (  lhs.type== rhs.type &&
     lhs.trackId== rhs.trackId &&
     lhs.mapId== rhs.mapId &&
     lhs.debug== rhs.debug 
    );
}

std::size_t MostConstrainingTrack::hash() const noexcept
{
    std::size_t result = 17;
    result = result * 31 + std::hash<MostConstrainingTrackType>()(type);
    result = result * 31 + std::hash<int64_t>()(trackId);
    result = result * 31 + std::hash<int64_t>()(mapId);
    result = result * 31 + std::hash<std::string>()(debug);
    return result;
}

 } // namespace av
 } // namespace msg


CEREAL_BIND_TO_ARCHIVES(av::msg::MostConstrainingTrack)

 namespace av {
 namespace msg {


OptionalLateralReferenceAsCubicSpline::OptionalLateralReferenceAsCubicSpline() = default;

OptionalLateralReferenceAsCubicSpline::OptionalLateralReferenceAsCubicSpline(
    bool hasValue_arg ,
    double spanLength_arg ,
    double startStation_arg ,
    std::vector<double> controlPoints_arg 
    ):
    hasValue { std::move(hasValue_arg) } // NOLINT
      ,
    spanLength { std::move(spanLength_arg) } // NOLINT
      ,
    startStation { std::move(startStation_arg) } // NOLINT
      ,
    controlPoints { std::move(controlPoints_arg) } // NOLINT
      
    {
}

OptionalLateralReferenceAsCubicSpline::OptionalLateralReferenceAsCubicSpline(const OptionalLateralReferenceAsCubicSpline&) = default;

// NOLINTNEXTLINE
OptionalLateralReferenceAsCubicSpline::OptionalLateralReferenceAsCubicSpline(OptionalLateralReferenceAsCubicSpline&&) = default;

OptionalLateralReferenceAsCubicSpline& OptionalLateralReferenceAsCubicSpline::operator=(const OptionalLateralReferenceAsCubicSpline&) = default;

// NOLINTNEXTLINE
OptionalLateralReferenceAsCubicSpline& OptionalLateralReferenceAsCubicSpline::operator=(OptionalLateralReferenceAsCubicSpline&&) = default;

void OptionalLateralReferenceAsCubicSpline::toCsv(std::ostream& out) const
{
    out.precision(16);
    out << std::boolalpha;
    out << hasValue;out << ",";
    out << spanLength;out << ",";
    out << startStation;out << ",";
    // Enclose sequence in double quotations and curly brackets to keep CSV line length uniform
        
        out << "\"{{";
        
        if (!controlPoints.empty())
        {
            std::copy(controlPoints.begin(), controlPoints.end() - 1, std::ostream_iterator<double>(out, "},{"));
            std::copy(controlPoints.end()-1, controlPoints.end(), std::ostream_iterator<double>(out, ""));
        }
        out << "}}\"";
    
}

std::stringstream OptionalLateralReferenceAsCubicSpline::toStringstream() const
{
    std::stringstream out;
    toCsv(out);
    return out;
}

template<class Archive>
    void OptionalLateralReferenceAsCubicSpline::serialize(Archive& ar, [[maybe_unused]] unsigned int version)
    {

        ar& CEREAL_NVP(hasValue);
        ar& CEREAL_NVP(spanLength);
        ar& CEREAL_NVP(startStation);
        ar& CEREAL_NVP(controlPoints);
    }


template void OptionalLateralReferenceAsCubicSpline::serialize(cereal::BinaryInputArchive& ar, unsigned int version);
template void OptionalLateralReferenceAsCubicSpline::serialize(cereal::BinaryOutputArchive& ar, unsigned int version);
template void OptionalLateralReferenceAsCubicSpline::serialize(cereal::IntrospectionArchive& ar, unsigned int version);
template void OptionalLateralReferenceAsCubicSpline::serialize(cereal::JSONInputArchive& ar, unsigned int version);
template void OptionalLateralReferenceAsCubicSpline::serialize(cereal::JSONOutputArchive& ar, unsigned int version);
template void OptionalLateralReferenceAsCubicSpline::serialize(cereal::PortableBinaryInputArchive& ar, unsigned int version);
template void OptionalLateralReferenceAsCubicSpline::serialize(cereal::PortableBinaryOutputArchive& ar, unsigned int version);

std::ostream& operator<< (std::ostream& os, const OptionalLateralReferenceAsCubicSpline& optionallateralreferenceascubicspline)
{
    optionallateralreferenceascubicspline.toCsv(os);
    return os;
}

bool operator== (const OptionalLateralReferenceAsCubicSpline &lhs, const OptionalLateralReferenceAsCubicSpline &rhs)
{
    return (  lhs.hasValue== rhs.hasValue &&
     lhs.spanLength== rhs.spanLength &&
     lhs.startStation== rhs.startStation &&
     lhs.controlPoints== rhs.controlPoints 
    );
}

std::size_t OptionalLateralReferenceAsCubicSpline::hash() const noexcept
{
    std::size_t result = 17;
    result = result * 31 + std::hash<bool>()(hasValue);
    result = result * 31 + std::hash<double>()(spanLength);
    result = result * 31 + std::hash<double>()(startStation);
    result = result * 31 + hashVector(controlPoints);
    return result;
}

 } // namespace av
 } // namespace msg


CEREAL_BIND_TO_ARCHIVES(av::msg::OptionalLateralReferenceAsCubicSpline)

 namespace av {
 namespace msg {


PairOfVectors::PairOfVectors() = default;

PairOfVectors::PairOfVectors(
    std::vector<double> first_arg ,
    std::vector<double> second_arg 
    ):
    first { std::move(first_arg) } // NOLINT
      ,
    second { std::move(second_arg) } // NOLINT
      
    {
}

PairOfVectors::PairOfVectors(const PairOfVectors&) = default;

// NOLINTNEXTLINE
PairOfVectors::PairOfVectors(PairOfVectors&&) = default;

PairOfVectors& PairOfVectors::operator=(const PairOfVectors&) = default;

// NOLINTNEXTLINE
PairOfVectors& PairOfVectors::operator=(PairOfVectors&&) = default;

void PairOfVectors::toCsv(std::ostream& out) const
{
    out.precision(16);
    out << std::boolalpha;
    // Enclose sequence in double quotations and curly brackets to keep CSV line length uniform
        
        out << "\"{{";
        
        if (!first.empty())
        {
            std::copy(first.begin(), first.end() - 1, std::ostream_iterator<double>(out, "},{"));
            std::copy(first.end()-1, first.end(), std::ostream_iterator<double>(out, ""));
        }
        out << "}}\"";out << ",";
    // Enclose sequence in double quotations and curly brackets to keep CSV line length uniform
        
        out << "\"{{";
        
        if (!second.empty())
        {
            std::copy(second.begin(), second.end() - 1, std::ostream_iterator<double>(out, "},{"));
            std::copy(second.end()-1, second.end(), std::ostream_iterator<double>(out, ""));
        }
        out << "}}\"";
    
}

std::stringstream PairOfVectors::toStringstream() const
{
    std::stringstream out;
    toCsv(out);
    return out;
}

template<class Archive>
    void PairOfVectors::serialize(Archive& ar, [[maybe_unused]] unsigned int version)
    {

        ar& CEREAL_NVP(first);
        ar& CEREAL_NVP(second);
    }


template void PairOfVectors::serialize(cereal::BinaryInputArchive& ar, unsigned int version);
template void PairOfVectors::serialize(cereal::BinaryOutputArchive& ar, unsigned int version);
template void PairOfVectors::serialize(cereal::IntrospectionArchive& ar, unsigned int version);
template void PairOfVectors::serialize(cereal::JSONInputArchive& ar, unsigned int version);
template void PairOfVectors::serialize(cereal::JSONOutputArchive& ar, unsigned int version);
template void PairOfVectors::serialize(cereal::PortableBinaryInputArchive& ar, unsigned int version);
template void PairOfVectors::serialize(cereal::PortableBinaryOutputArchive& ar, unsigned int version);

std::ostream& operator<< (std::ostream& os, const PairOfVectors& pairofvectors)
{
    pairofvectors.toCsv(os);
    return os;
}

bool operator== (const PairOfVectors &lhs, const PairOfVectors &rhs)
{
    return (  lhs.first== rhs.first &&
     lhs.second== rhs.second 
    );
}

std::size_t PairOfVectors::hash() const noexcept
{
    std::size_t result = 17;
    result = result * 31 + hashVector(first);
    result = result * 31 + hashVector(second);
    return result;
}

 } // namespace av
 } // namespace msg


CEREAL_BIND_TO_ARCHIVES(av::msg::PairOfVectors)

 namespace av {
 namespace msg {


OptionalPairOfVectors::OptionalPairOfVectors() = default;

OptionalPairOfVectors::OptionalPairOfVectors(
    bool hasValue_arg ,
    PairOfVectors value_arg 
    ):
    hasValue { std::move(hasValue_arg) } // NOLINT
      ,
    value { std::move(value_arg) } // NOLINT
      
    {
}

OptionalPairOfVectors::OptionalPairOfVectors(const OptionalPairOfVectors&) = default;

// NOLINTNEXTLINE
OptionalPairOfVectors::OptionalPairOfVectors(OptionalPairOfVectors&&) = default;

OptionalPairOfVectors& OptionalPairOfVectors::operator=(const OptionalPairOfVectors&) = default;

// NOLINTNEXTLINE
OptionalPairOfVectors& OptionalPairOfVectors::operator=(OptionalPairOfVectors&&) = default;

void OptionalPairOfVectors::toCsv(std::ostream& out) const
{
    out.precision(16);
    out << std::boolalpha;
    out << hasValue;out << ",";
    out << value;
    
}

std::stringstream OptionalPairOfVectors::toStringstream() const
{
    std::stringstream out;
    toCsv(out);
    return out;
}

template<class Archive>
    void OptionalPairOfVectors::serialize(Archive& ar, [[maybe_unused]] unsigned int version)
    {

        ar& CEREAL_NVP(hasValue);
        ar& CEREAL_NVP(value);
    }


template void OptionalPairOfVectors::serialize(cereal::BinaryInputArchive& ar, unsigned int version);
template void OptionalPairOfVectors::serialize(cereal::BinaryOutputArchive& ar, unsigned int version);
template void OptionalPairOfVectors::serialize(cereal::IntrospectionArchive& ar, unsigned int version);
template void OptionalPairOfVectors::serialize(cereal::JSONInputArchive& ar, unsigned int version);
template void OptionalPairOfVectors::serialize(cereal::JSONOutputArchive& ar, unsigned int version);
template void OptionalPairOfVectors::serialize(cereal::PortableBinaryInputArchive& ar, unsigned int version);
template void OptionalPairOfVectors::serialize(cereal::PortableBinaryOutputArchive& ar, unsigned int version);

std::ostream& operator<< (std::ostream& os, const OptionalPairOfVectors& optionalpairofvectors)
{
    optionalpairofvectors.toCsv(os);
    return os;
}

bool operator== (const OptionalPairOfVectors &lhs, const OptionalPairOfVectors &rhs)
{
    return (  lhs.hasValue== rhs.hasValue &&
     lhs.value== rhs.value 
    );
}

std::size_t OptionalPairOfVectors::hash() const noexcept
{
    std::size_t result = 17;
    result = result * 31 + std::hash<bool>()(hasValue);
    result = result * 31 + std::hash<PairOfVectors>()(value);
    return result;
}

 } // namespace av
 } // namespace msg


CEREAL_BIND_TO_ARCHIVES(av::msg::OptionalPairOfVectors)

 namespace av {
 namespace msg {


CubicTransitionBSpline::CubicTransitionBSpline() = default;

CubicTransitionBSpline::CubicTransitionBSpline(
    PairOfVectors cartesianControlPoints_arg ,
    PairOfVectors mapProgressToStation_arg ,
    PairOfVectors mapStationToProgress_arg ,
    OptionalPairOfVectors mapStationToLateralOffset_arg 
    ):
    cartesianControlPoints { std::move(cartesianControlPoints_arg) } // NOLINT
      ,
    mapProgressToStation { std::move(mapProgressToStation_arg) } // NOLINT
      ,
    mapStationToProgress { std::move(mapStationToProgress_arg) } // NOLINT
      ,
    mapStationToLateralOffset { std::move(mapStationToLateralOffset_arg) } // NOLINT
      
    {
}

CubicTransitionBSpline::CubicTransitionBSpline(const CubicTransitionBSpline&) = default;

// NOLINTNEXTLINE
CubicTransitionBSpline::CubicTransitionBSpline(CubicTransitionBSpline&&) = default;

CubicTransitionBSpline& CubicTransitionBSpline::operator=(const CubicTransitionBSpline&) = default;

// NOLINTNEXTLINE
CubicTransitionBSpline& CubicTransitionBSpline::operator=(CubicTransitionBSpline&&) = default;

void CubicTransitionBSpline::toCsv(std::ostream& out) const
{
    out.precision(16);
    out << std::boolalpha;
    out << cartesianControlPoints;out << ",";
    out << mapProgressToStation;out << ",";
    out << mapStationToProgress;out << ",";
    out << mapStationToLateralOffset;
    
}

std::stringstream CubicTransitionBSpline::toStringstream() const
{
    std::stringstream out;
    toCsv(out);
    return out;
}

template<class Archive>
    void CubicTransitionBSpline::serialize(Archive& ar, unsigned int version)
{
    ar& CEREAL_NVP(cartesianControlPoints);
    ar& CEREAL_NVP(mapProgressToStation);
    ar& CEREAL_NVP(mapStationToProgress);
    if (version >= 2)
    {
        ar& CEREAL_NVP(mapStationToLateralOffset);
    }
}



template void CubicTransitionBSpline::serialize(cereal::BinaryInputArchive& ar, unsigned int version);
template void CubicTransitionBSpline::serialize(cereal::BinaryOutputArchive& ar, unsigned int version);
template void CubicTransitionBSpline::serialize(cereal::IntrospectionArchive& ar, unsigned int version);
template void CubicTransitionBSpline::serialize(cereal::JSONInputArchive& ar, unsigned int version);
template void CubicTransitionBSpline::serialize(cereal::JSONOutputArchive& ar, unsigned int version);
template void CubicTransitionBSpline::serialize(cereal::PortableBinaryInputArchive& ar, unsigned int version);
template void CubicTransitionBSpline::serialize(cereal::PortableBinaryOutputArchive& ar, unsigned int version);

std::ostream& operator<< (std::ostream& os, const CubicTransitionBSpline& cubictransitionbspline)
{
    cubictransitionbspline.toCsv(os);
    return os;
}

bool operator== (const CubicTransitionBSpline &lhs, const CubicTransitionBSpline &rhs)
{
    return (  lhs.cartesianControlPoints== rhs.cartesianControlPoints &&
     lhs.mapProgressToStation== rhs.mapProgressToStation &&
     lhs.mapStationToProgress== rhs.mapStationToProgress &&
     lhs.mapStationToLateralOffset== rhs.mapStationToLateralOffset 
    );
}

std::size_t CubicTransitionBSpline::hash() const noexcept
{
    std::size_t result = 17;
    result = result * 31 + std::hash<PairOfVectors>()(cartesianControlPoints);
    result = result * 31 + std::hash<PairOfVectors>()(mapProgressToStation);
    result = result * 31 + std::hash<PairOfVectors>()(mapStationToProgress);
    result = result * 31 + std::hash<OptionalPairOfVectors>()(mapStationToLateralOffset);
    return result;
}

 } // namespace av
 } // namespace msg


CEREAL_BIND_TO_ARCHIVES(av::msg::CubicTransitionBSpline)

 namespace av {
 namespace msg {


GlobalStaticReferenceVariableMembers::GlobalStaticReferenceVariableMembers() = default;

GlobalStaticReferenceVariableMembers::GlobalStaticReferenceVariableMembers(
    SampledTransition reference_arg ,
    VTConstraint vtConstraint_arg ,
    SampledSTConstraint stConstraint_arg ,
    LateralTube sampledTubeConstraint_arg ,
    TubeAsSpline splineTubeConstraint_arg ,
    BiasingConstraints biasingLeft_arg ,
    BiasingConstraints biasingRight_arg ,
    std::vector<MostConstrainingTrack> mostConstrainingTracks_arg 
    ):
    reference { std::move(reference_arg) } // NOLINT
      ,
    vtConstraint { std::move(vtConstraint_arg) } // NOLINT
      ,
    stConstraint { std::move(stConstraint_arg) } // NOLINT
      ,
    sampledTubeConstraint { std::move(sampledTubeConstraint_arg) } // NOLINT
      ,
    splineTubeConstraint { std::move(splineTubeConstraint_arg) } // NOLINT
      ,
    biasingLeft { std::move(biasingLeft_arg) } // NOLINT
      ,
    biasingRight { std::move(biasingRight_arg) } // NOLINT
      ,
    mostConstrainingTracks { std::move(mostConstrainingTracks_arg) } // NOLINT
      
    {
}

GlobalStaticReferenceVariableMembers::GlobalStaticReferenceVariableMembers(const GlobalStaticReferenceVariableMembers&) = default;

// NOLINTNEXTLINE
GlobalStaticReferenceVariableMembers::GlobalStaticReferenceVariableMembers(GlobalStaticReferenceVariableMembers&&) = default;

GlobalStaticReferenceVariableMembers& GlobalStaticReferenceVariableMembers::operator=(const GlobalStaticReferenceVariableMembers&) = default;

// NOLINTNEXTLINE
GlobalStaticReferenceVariableMembers& GlobalStaticReferenceVariableMembers::operator=(GlobalStaticReferenceVariableMembers&&) = default;

void GlobalStaticReferenceVariableMembers::toCsv(std::ostream& out) const
{
    out.precision(16);
    out << std::boolalpha;
    out << reference;out << ",";
    out << vtConstraint;out << ",";
    out << stConstraint;out << ",";
    out << sampledTubeConstraint;out << ",";
    out << splineTubeConstraint;out << ",";
    out << biasingLeft;out << ",";
    out << biasingRight;out << ",";
    // Enclose sequence in double quotations and curly brackets to keep CSV line length uniform
        
        out << "\"{{";
        
        if (!mostConstrainingTracks.empty())
        {
            std::copy(mostConstrainingTracks.begin(), mostConstrainingTracks.end() - 1, std::ostream_iterator<MostConstrainingTrack>(out, "},{"));
            std::copy(mostConstrainingTracks.end()-1, mostConstrainingTracks.end(), std::ostream_iterator<MostConstrainingTrack>(out, ""));
        }
        out << "}}\"";
    
}

std::stringstream GlobalStaticReferenceVariableMembers::toStringstream() const
{
    std::stringstream out;
    toCsv(out);
    return out;
}

template<class Archive>
    void GlobalStaticReferenceVariableMembers::serialize(Archive& ar, unsigned int version)
{
    if (version >= 1) {
        ar& CEREAL_NVP(reference);
        ar& CEREAL_NVP(vtConstraint);
        ar& CEREAL_NVP(stConstraint);
    }
    if (version == 1) {
        ar& cereal::make_nvp("tubeConstraint", sampledTubeConstraint); // name change from "tubeConstraint" in version 2
    }
    if (version >= 2) {
        ar& CEREAL_NVP(sampledTubeConstraint);
        ar& CEREAL_NVP(splineTubeConstraint);
    }
    if (version >= 1) {
        ar& CEREAL_NVP(biasingLeft);
        ar& CEREAL_NVP(biasingRight);
        ar& CEREAL_NVP(mostConstrainingTracks);
    }
}



template void GlobalStaticReferenceVariableMembers::serialize(cereal::BinaryInputArchive& ar, unsigned int version);
template void GlobalStaticReferenceVariableMembers::serialize(cereal::BinaryOutputArchive& ar, unsigned int version);
template void GlobalStaticReferenceVariableMembers::serialize(cereal::IntrospectionArchive& ar, unsigned int version);
template void GlobalStaticReferenceVariableMembers::serialize(cereal::JSONInputArchive& ar, unsigned int version);
template void GlobalStaticReferenceVariableMembers::serialize(cereal::JSONOutputArchive& ar, unsigned int version);
template void GlobalStaticReferenceVariableMembers::serialize(cereal::PortableBinaryInputArchive& ar, unsigned int version);
template void GlobalStaticReferenceVariableMembers::serialize(cereal::PortableBinaryOutputArchive& ar, unsigned int version);

std::ostream& operator<< (std::ostream& os, const GlobalStaticReferenceVariableMembers& globalstaticreferencevariablemembers)
{
    globalstaticreferencevariablemembers.toCsv(os);
    return os;
}

bool operator== (const GlobalStaticReferenceVariableMembers &lhs, const GlobalStaticReferenceVariableMembers &rhs)
{
    return (  lhs.reference== rhs.reference &&
     lhs.vtConstraint== rhs.vtConstraint &&
     lhs.stConstraint== rhs.stConstraint &&
     lhs.sampledTubeConstraint== rhs.sampledTubeConstraint &&
     lhs.splineTubeConstraint== rhs.splineTubeConstraint &&
     lhs.biasingLeft== rhs.biasingLeft &&
     lhs.biasingRight== rhs.biasingRight &&
     lhs.mostConstrainingTracks== rhs.mostConstrainingTracks 
    );
}

std::size_t GlobalStaticReferenceVariableMembers::hash() const noexcept
{
    std::size_t result = 17;
    result = result * 31 + std::hash<SampledTransition>()(reference);
    result = result * 31 + std::hash<VTConstraint>()(vtConstraint);
    result = result * 31 + std::hash<SampledSTConstraint>()(stConstraint);
    result = result * 31 + std::hash<LateralTube>()(sampledTubeConstraint);
    result = result * 31 + std::hash<TubeAsSpline>()(splineTubeConstraint);
    result = result * 31 + std::hash<BiasingConstraints>()(biasingLeft);
    result = result * 31 + std::hash<BiasingConstraints>()(biasingRight);
    result = result * 31 + hashVector(mostConstrainingTracks);
    return result;
}

 } // namespace av
 } // namespace msg


CEREAL_BIND_TO_ARCHIVES(av::msg::GlobalStaticReferenceVariableMembers)

 namespace av {
 namespace msg {


GlobalStaticReferenceConstantMembers::GlobalStaticReferenceConstantMembers() = default;

GlobalStaticReferenceConstantMembers::GlobalStaticReferenceConstantMembers(
    OptionalLateralReferenceAsCubicSpline optionalCommittedPath_arg ,
    CubicTransitionBSpline transitionSpline_arg 
    ):
    optionalCommittedPath { std::move(optionalCommittedPath_arg) } // NOLINT
      ,
    transitionSpline { std::move(transitionSpline_arg) } // NOLINT
      
    {
}

GlobalStaticReferenceConstantMembers::GlobalStaticReferenceConstantMembers(const GlobalStaticReferenceConstantMembers&) = default;

// NOLINTNEXTLINE
GlobalStaticReferenceConstantMembers::GlobalStaticReferenceConstantMembers(GlobalStaticReferenceConstantMembers&&) = default;

GlobalStaticReferenceConstantMembers& GlobalStaticReferenceConstantMembers::operator=(const GlobalStaticReferenceConstantMembers&) = default;

// NOLINTNEXTLINE
GlobalStaticReferenceConstantMembers& GlobalStaticReferenceConstantMembers::operator=(GlobalStaticReferenceConstantMembers&&) = default;

void GlobalStaticReferenceConstantMembers::toCsv(std::ostream& out) const
{
    out.precision(16);
    out << std::boolalpha;
    out << optionalCommittedPath;out << ",";
    out << transitionSpline;
    
}

std::stringstream GlobalStaticReferenceConstantMembers::toStringstream() const
{
    std::stringstream out;
    toCsv(out);
    return out;
}

template<class Archive>
    void GlobalStaticReferenceConstantMembers::serialize(Archive& ar, [[maybe_unused]] unsigned int version)
    {

        ar& CEREAL_NVP(optionalCommittedPath);
        ar& CEREAL_NVP(transitionSpline);
    }


template void GlobalStaticReferenceConstantMembers::serialize(cereal::BinaryInputArchive& ar, unsigned int version);
template void GlobalStaticReferenceConstantMembers::serialize(cereal::BinaryOutputArchive& ar, unsigned int version);
template void GlobalStaticReferenceConstantMembers::serialize(cereal::IntrospectionArchive& ar, unsigned int version);
template void GlobalStaticReferenceConstantMembers::serialize(cereal::JSONInputArchive& ar, unsigned int version);
template void GlobalStaticReferenceConstantMembers::serialize(cereal::JSONOutputArchive& ar, unsigned int version);
template void GlobalStaticReferenceConstantMembers::serialize(cereal::PortableBinaryInputArchive& ar, unsigned int version);
template void GlobalStaticReferenceConstantMembers::serialize(cereal::PortableBinaryOutputArchive& ar, unsigned int version);

std::ostream& operator<< (std::ostream& os, const GlobalStaticReferenceConstantMembers& globalstaticreferenceconstantmembers)
{
    globalstaticreferenceconstantmembers.toCsv(os);
    return os;
}

bool operator== (const GlobalStaticReferenceConstantMembers &lhs, const GlobalStaticReferenceConstantMembers &rhs)
{
    return (  lhs.optionalCommittedPath== rhs.optionalCommittedPath &&
     lhs.transitionSpline== rhs.transitionSpline 
    );
}

std::size_t GlobalStaticReferenceConstantMembers::hash() const noexcept
{
    std::size_t result = 17;
    result = result * 31 + std::hash<OptionalLateralReferenceAsCubicSpline>()(optionalCommittedPath);
    result = result * 31 + std::hash<CubicTransitionBSpline>()(transitionSpline);
    return result;
}

 } // namespace av
 } // namespace msg


CEREAL_BIND_TO_ARCHIVES(av::msg::GlobalStaticReferenceConstantMembers)




 namespace av {
 namespace msg {


VectorOfGlobalStaticReference::VectorOfGlobalStaticReference() = default;

VectorOfGlobalStaticReference::VectorOfGlobalStaticReference(
    GlobalStaticReferenceConstantMembers constantMembers_arg ,
    std::vector<GlobalStaticReferenceVariableMembers> variableMembers_arg 
    ):
    constantMembers { std::move(constantMembers_arg) } // NOLINT
      ,variableMembers { std::move(variableMembers_arg) } // NOLINT
      {
}

VectorOfGlobalStaticReference::VectorOfGlobalStaticReference(const VectorOfGlobalStaticReference&) = default;

// NOLINTNEXTLINE
VectorOfGlobalStaticReference::VectorOfGlobalStaticReference(VectorOfGlobalStaticReference&&) = default;

VectorOfGlobalStaticReference& VectorOfGlobalStaticReference::operator=(const VectorOfGlobalStaticReference&) = default;

// NOLINTNEXTLINE
VectorOfGlobalStaticReference& VectorOfGlobalStaticReference::operator=(VectorOfGlobalStaticReference&&) = default;

void VectorOfGlobalStaticReference::toCsv(std::ostream& out) const
{
    out.precision(16);
    out << std::boolalpha;
    out << constantMembers;out << ",";
    // Enclose sequence in double quotations and curly brackets to keep CSV line length uniform
        
        out << "\"{{";
        
        if (!variableMembers.empty())
        {
            std::copy(variableMembers.begin(), variableMembers.end() - 1, std::ostream_iterator<GlobalStaticReferenceVariableMembers>(out, "},{"));
            std::copy(variableMembers.end()-1, variableMembers.end(), std::ostream_iterator<GlobalStaticReferenceVariableMembers>(out, ""));
        }
        out << "}}\"";
    
}

std::stringstream VectorOfGlobalStaticReference::toStringstream() const
{
    std::stringstream out;
    toCsv(out);
    return out;
}

template<class Archive>
    void VectorOfGlobalStaticReference::serialize(Archive& ar, [[maybe_unused]] unsigned int version)
    {

        ar& CEREAL_NVP(constantMembers);
        ar& CEREAL_NVP(variableMembers);
    }


template void VectorOfGlobalStaticReference::serialize(cereal::BinaryInputArchive& ar, unsigned int version);
template void VectorOfGlobalStaticReference::serialize(cereal::BinaryOutputArchive& ar, unsigned int version);
template void VectorOfGlobalStaticReference::serialize(cereal::IntrospectionArchive& ar, unsigned int version);
template void VectorOfGlobalStaticReference::serialize(cereal::JSONInputArchive& ar, unsigned int version);
template void VectorOfGlobalStaticReference::serialize(cereal::JSONOutputArchive& ar, unsigned int version);
template void VectorOfGlobalStaticReference::serialize(cereal::PortableBinaryInputArchive& ar, unsigned int version);
template void VectorOfGlobalStaticReference::serialize(cereal::PortableBinaryOutputArchive& ar, unsigned int version);

std::ostream& operator<< (std::ostream& os, const VectorOfGlobalStaticReference& vectorofglobalstaticreference)
{
    vectorofglobalstaticreference.toCsv(os);
    return os;
}

bool operator== (const VectorOfGlobalStaticReference &lhs, const VectorOfGlobalStaticReference &rhs)
{
    return (  lhs.constantMembers== rhs.constantMembers &&
     lhs.variableMembers== rhs.variableMembers 
    );
}

std::size_t VectorOfGlobalStaticReference::hash() const noexcept
{
    std::size_t result = 17;
    result = result * 31 + std::hash<GlobalStaticReferenceConstantMembers>()(constantMembers);
    result = result * 31 + hashVector(variableMembers);
    return result;
}

 } // namespace av
 } // namespace msg


CEREAL_BIND_TO_ARCHIVES(av::msg::VectorOfGlobalStaticReference)

