#include <interfaces/Velocity/Velocity.h>

#include <iterator>
#include <type_traits>

#include <cereal/cereal.hpp>
#include <cereal/archives/json.hpp>
#include <cereal/archives/binary.hpp>

#include <cereal/types/map.hpp>
#include <cereal/types/vector.hpp>
#include <cereal/types/array.hpp>
#include <cereal/types/string.hpp>
#include <cereal/types/complex.hpp>

#include <ODatum/IntrospectionArchive.h>
#include <ODatum/Hash.h>


 namespace av {
 namespace msg {


Velocity::Velocity() = default;

Velocity::Velocity(
    double velocityX_m_s_arg ,
    double velocityY_m_s_arg ,
    double velocityZ_m_s_arg 
    ):
    velocityX_m_s { std::move(velocityX_m_s_arg) } // NOLINT
      ,
    velocityY_m_s { std::move(velocityY_m_s_arg) } // NOLINT
      ,
    velocityZ_m_s { std::move(velocityZ_m_s_arg) } // NOLINT
      
    {
}

Velocity::Velocity(const Velocity&) = default;

// NOLINTNEXTLINE
Velocity::Velocity(Velocity&&) = default;

Velocity& Velocity::operator=(const Velocity&) = default;

// NOLINTNEXTLINE
Velocity& Velocity::operator=(Velocity&&) = default;

void Velocity::toCsv(std::ostream& out) const
{
    out.precision(16);
    out << std::boolalpha;
    out << velocityX_m_s;out << ",";
    out << velocityY_m_s;out << ",";
    out << velocityZ_m_s;
    
}

std::stringstream Velocity::toStringstream() const
{
    std::stringstream out;
    toCsv(out);
    return out;
}

template<class Archive>
    void Velocity::serialize(Archive& ar, [[maybe_unused]] unsigned int version)
    {

        ar& CEREAL_NVP(velocityX_m_s);
        ar& CEREAL_NVP(velocityY_m_s);
        ar& CEREAL_NVP(velocityZ_m_s);
    }


template void Velocity::serialize(cereal::BinaryInputArchive& ar, unsigned int version);
template void Velocity::serialize(cereal::BinaryOutputArchive& ar, unsigned int version);
template void Velocity::serialize(cereal::IntrospectionArchive& ar, unsigned int version);
template void Velocity::serialize(cereal::JSONInputArchive& ar, unsigned int version);
template void Velocity::serialize(cereal::JSONOutputArchive& ar, unsigned int version);
template void Velocity::serialize(cereal::PortableBinaryInputArchive& ar, unsigned int version);
template void Velocity::serialize(cereal::PortableBinaryOutputArchive& ar, unsigned int version);

std::ostream& operator<< (std::ostream& os, const Velocity& velocity)
{
    velocity.toCsv(os);
    return os;
}

bool operator== (const Velocity &lhs, const Velocity &rhs)
{
    return (  lhs.velocityX_m_s== rhs.velocityX_m_s &&
     lhs.velocityY_m_s== rhs.velocityY_m_s &&
     lhs.velocityZ_m_s== rhs.velocityZ_m_s 
    );
}

std::size_t Velocity::hash() const noexcept
{
    std::size_t result = 17;
    result = result * 31 + std::hash<double>()(velocityX_m_s);
    result = result * 31 + std::hash<double>()(velocityY_m_s);
    result = result * 31 + std::hash<double>()(velocityZ_m_s);
    return result;
}

 } // namespace av
 } // namespace msg


CEREAL_BIND_TO_ARCHIVES(av::msg::Velocity)


