#include <interfaces/VertexRoute/VertexRoute.h>

#include <iterator>
#include <type_traits>

#include <cereal/cereal.hpp>
#include <cereal/archives/json.hpp>
#include <cereal/archives/binary.hpp>

#include <cereal/types/map.hpp>
#include <cereal/types/vector.hpp>
#include <cereal/types/array.hpp>
#include <cereal/types/string.hpp>
#include <cereal/types/complex.hpp>

#include <ODatum/IntrospectionArchive.h>
#include <ODatum/Hash.h>





 namespace av {
 namespace msg {


VertexRoute::VertexRoute() = default;

VertexRoute::VertexRoute(
    uint32_t numVertices_arg ,
    std::vector<GraphVertex> vertices_arg 
    ):
    numVertices { std::move(numVertices_arg) } // NOLINT
      ,vertices { std::move(vertices_arg) } // NOLINT
      {
}

VertexRoute::VertexRoute(const VertexRoute&) = default;

// NOLINTNEXTLINE
VertexRoute::VertexRoute(VertexRoute&&) = default;

VertexRoute& VertexRoute::operator=(const VertexRoute&) = default;

// NOLINTNEXTLINE
VertexRoute& VertexRoute::operator=(VertexRoute&&) = default;

void VertexRoute::toCsv(std::ostream& out) const
{
    out.precision(16);
    out << std::boolalpha;
    out << numVertices;out << ",";
    // Enclose sequence in double quotations and curly brackets to keep CSV line length uniform
        
        out << "\"{{";
        
        if (!vertices.empty())
        {
            std::copy(vertices.begin(), vertices.end() - 1, std::ostream_iterator<GraphVertex>(out, "},{"));
            std::copy(vertices.end()-1, vertices.end(), std::ostream_iterator<GraphVertex>(out, ""));
        }
        out << "}}\"";
    
}

std::stringstream VertexRoute::toStringstream() const
{
    std::stringstream out;
    toCsv(out);
    return out;
}

template<class Archive>
    void VertexRoute::serialize(Archive& ar, [[maybe_unused]] unsigned int version)
    {

        ar& CEREAL_NVP(numVertices);
        ar& CEREAL_NVP(vertices);
    }


template void VertexRoute::serialize(cereal::BinaryInputArchive& ar, unsigned int version);
template void VertexRoute::serialize(cereal::BinaryOutputArchive& ar, unsigned int version);
template void VertexRoute::serialize(cereal::IntrospectionArchive& ar, unsigned int version);
template void VertexRoute::serialize(cereal::JSONInputArchive& ar, unsigned int version);
template void VertexRoute::serialize(cereal::JSONOutputArchive& ar, unsigned int version);
template void VertexRoute::serialize(cereal::PortableBinaryInputArchive& ar, unsigned int version);
template void VertexRoute::serialize(cereal::PortableBinaryOutputArchive& ar, unsigned int version);

std::ostream& operator<< (std::ostream& os, const VertexRoute& vertexroute)
{
    vertexroute.toCsv(os);
    return os;
}

bool operator== (const VertexRoute &lhs, const VertexRoute &rhs)
{
    return (  lhs.numVertices== rhs.numVertices &&
     lhs.vertices== rhs.vertices 
    );
}

std::size_t VertexRoute::hash() const noexcept
{
    std::size_t result = 17;
    result = result * 31 + std::hash<uint32_t>()(numVertices);
    result = result * 31 + hashVector(vertices);
    return result;
}

 } // namespace av
 } // namespace msg


CEREAL_BIND_TO_ARCHIVES(av::msg::VertexRoute)

