#include <interfaces/VicBodyStatusMessage/VicBodyStatusMessage.h>

#include <iterator>
#include <type_traits>

#include <cereal/cereal.hpp>
#include <cereal/archives/json.hpp>
#include <cereal/archives/binary.hpp>

#include <cereal/types/map.hpp>
#include <cereal/types/vector.hpp>
#include <cereal/types/array.hpp>
#include <cereal/types/string.hpp>
#include <cereal/types/complex.hpp>

#include <ODatum/IntrospectionArchive.h>
#include <ODatum/Hash.h>


 namespace av {
 namespace msg {


DoorStatus::DoorStatus() = default;

DoorStatus::DoorStatus(
    DoorState rearRightDoorState_arg ,
    DoorState rearLeftDoorState_arg ,
    DoorState frontPassengerDoorState_arg ,
    DoorState driverDoorState_arg 
    ):
    rearRightDoorState { std::move(rearRightDoorState_arg) } // NOLINT
      ,
    rearLeftDoorState { std::move(rearLeftDoorState_arg) } // NOLINT
      ,
    frontPassengerDoorState { std::move(frontPassengerDoorState_arg) } // NOLINT
      ,
    driverDoorState { std::move(driverDoorState_arg) } // NOLINT
      
    {
}

DoorStatus::DoorStatus(const DoorStatus&) = default;

// NOLINTNEXTLINE
DoorStatus::DoorStatus(DoorStatus&&) = default;

DoorStatus& DoorStatus::operator=(const DoorStatus&) = default;

// NOLINTNEXTLINE
DoorStatus& DoorStatus::operator=(DoorStatus&&) = default;

void DoorStatus::toCsv(std::ostream& out) const
{
    out.precision(16);
    out << std::boolalpha;
    out << static_cast<uint32_t>(rearRightDoorState);out << ",";
    out << static_cast<uint32_t>(rearLeftDoorState);out << ",";
    out << static_cast<uint32_t>(frontPassengerDoorState);out << ",";
    out << static_cast<uint32_t>(driverDoorState);
    
}

std::stringstream DoorStatus::toStringstream() const
{
    std::stringstream out;
    toCsv(out);
    return out;
}

template<class Archive>
    void DoorStatus::serialize(Archive& ar, [[maybe_unused]] unsigned int version)
    {

        ar& CEREAL_NVP(rearRightDoorState);
        ar& CEREAL_NVP(rearLeftDoorState);
        ar& CEREAL_NVP(frontPassengerDoorState);
        ar& CEREAL_NVP(driverDoorState);
    }


template void DoorStatus::serialize(cereal::BinaryInputArchive& ar, unsigned int version);
template void DoorStatus::serialize(cereal::BinaryOutputArchive& ar, unsigned int version);
template void DoorStatus::serialize(cereal::IntrospectionArchive& ar, unsigned int version);
template void DoorStatus::serialize(cereal::JSONInputArchive& ar, unsigned int version);
template void DoorStatus::serialize(cereal::JSONOutputArchive& ar, unsigned int version);
template void DoorStatus::serialize(cereal::PortableBinaryInputArchive& ar, unsigned int version);
template void DoorStatus::serialize(cereal::PortableBinaryOutputArchive& ar, unsigned int version);

std::ostream& operator<< (std::ostream& os, const DoorStatus& doorstatus)
{
    doorstatus.toCsv(os);
    return os;
}

bool operator== (const DoorStatus &lhs, const DoorStatus &rhs)
{
    return (  lhs.rearRightDoorState== rhs.rearRightDoorState &&
     lhs.rearLeftDoorState== rhs.rearLeftDoorState &&
     lhs.frontPassengerDoorState== rhs.frontPassengerDoorState &&
     lhs.driverDoorState== rhs.driverDoorState 
    );
}

std::size_t DoorStatus::hash() const noexcept
{
    std::size_t result = 17;
    result = result * 31 + std::hash<DoorState>()(rearRightDoorState);
    result = result * 31 + std::hash<DoorState>()(rearLeftDoorState);
    result = result * 31 + std::hash<DoorState>()(frontPassengerDoorState);
    result = result * 31 + std::hash<DoorState>()(driverDoorState);
    return result;
}

 } // namespace av
 } // namespace msg


CEREAL_BIND_TO_ARCHIVES(av::msg::DoorStatus)

 namespace av {
 namespace msg {


LockStatus::LockStatus() = default;

LockStatus::LockStatus(
    LockState rearRightLockState_arg ,
    LockState rearLeftLockState_arg ,
    LockState frontPassengerLockState_arg ,
    LockState driverLockState_arg 
    ):
    rearRightLockState { std::move(rearRightLockState_arg) } // NOLINT
      ,
    rearLeftLockState { std::move(rearLeftLockState_arg) } // NOLINT
      ,
    frontPassengerLockState { std::move(frontPassengerLockState_arg) } // NOLINT
      ,
    driverLockState { std::move(driverLockState_arg) } // NOLINT
      
    {
}

LockStatus::LockStatus(const LockStatus&) = default;

// NOLINTNEXTLINE
LockStatus::LockStatus(LockStatus&&) = default;

LockStatus& LockStatus::operator=(const LockStatus&) = default;

// NOLINTNEXTLINE
LockStatus& LockStatus::operator=(LockStatus&&) = default;

void LockStatus::toCsv(std::ostream& out) const
{
    out.precision(16);
    out << std::boolalpha;
    out << static_cast<uint32_t>(rearRightLockState);out << ",";
    out << static_cast<uint32_t>(rearLeftLockState);out << ",";
    out << static_cast<uint32_t>(frontPassengerLockState);out << ",";
    out << static_cast<uint32_t>(driverLockState);
    
}

std::stringstream LockStatus::toStringstream() const
{
    std::stringstream out;
    toCsv(out);
    return out;
}

template<class Archive>
    void LockStatus::serialize(Archive& ar, [[maybe_unused]] unsigned int version)
    {

        ar& CEREAL_NVP(rearRightLockState);
        ar& CEREAL_NVP(rearLeftLockState);
        ar& CEREAL_NVP(frontPassengerLockState);
        ar& CEREAL_NVP(driverLockState);
    }


template void LockStatus::serialize(cereal::BinaryInputArchive& ar, unsigned int version);
template void LockStatus::serialize(cereal::BinaryOutputArchive& ar, unsigned int version);
template void LockStatus::serialize(cereal::IntrospectionArchive& ar, unsigned int version);
template void LockStatus::serialize(cereal::JSONInputArchive& ar, unsigned int version);
template void LockStatus::serialize(cereal::JSONOutputArchive& ar, unsigned int version);
template void LockStatus::serialize(cereal::PortableBinaryInputArchive& ar, unsigned int version);
template void LockStatus::serialize(cereal::PortableBinaryOutputArchive& ar, unsigned int version);

std::ostream& operator<< (std::ostream& os, const LockStatus& lockstatus)
{
    lockstatus.toCsv(os);
    return os;
}

bool operator== (const LockStatus &lhs, const LockStatus &rhs)
{
    return (  lhs.rearRightLockState== rhs.rearRightLockState &&
     lhs.rearLeftLockState== rhs.rearLeftLockState &&
     lhs.frontPassengerLockState== rhs.frontPassengerLockState &&
     lhs.driverLockState== rhs.driverLockState 
    );
}

std::size_t LockStatus::hash() const noexcept
{
    std::size_t result = 17;
    result = result * 31 + std::hash<LockState>()(rearRightLockState);
    result = result * 31 + std::hash<LockState>()(rearLeftLockState);
    result = result * 31 + std::hash<LockState>()(frontPassengerLockState);
    result = result * 31 + std::hash<LockState>()(driverLockState);
    return result;
}

 } // namespace av
 } // namespace msg


CEREAL_BIND_TO_ARCHIVES(av::msg::LockStatus)

 namespace av {
 namespace msg {


SeatOccupancyStatus::SeatOccupancyStatus() = default;

SeatOccupancyStatus::SeatOccupancyStatus(
    SeatOccupancyState driverSeatOccupancyState_arg ,
    SeatOccupancyState frontPassengerSeatOccupancyState_arg ,
    SeatOccupancyState rearMiddleSeatOccupancyState_arg ,
    SeatOccupancyState rearLeftSeatOccupancyState_arg ,
    SeatOccupancyState rearRightSeatOccupancyState_arg 
    ):
    driverSeatOccupancyState { std::move(driverSeatOccupancyState_arg) } // NOLINT
      ,
    frontPassengerSeatOccupancyState { std::move(frontPassengerSeatOccupancyState_arg) } // NOLINT
      ,
    rearMiddleSeatOccupancyState { std::move(rearMiddleSeatOccupancyState_arg) } // NOLINT
      ,
    rearLeftSeatOccupancyState { std::move(rearLeftSeatOccupancyState_arg) } // NOLINT
      ,
    rearRightSeatOccupancyState { std::move(rearRightSeatOccupancyState_arg) } // NOLINT
      
    {
}

SeatOccupancyStatus::SeatOccupancyStatus(const SeatOccupancyStatus&) = default;

// NOLINTNEXTLINE
SeatOccupancyStatus::SeatOccupancyStatus(SeatOccupancyStatus&&) = default;

SeatOccupancyStatus& SeatOccupancyStatus::operator=(const SeatOccupancyStatus&) = default;

// NOLINTNEXTLINE
SeatOccupancyStatus& SeatOccupancyStatus::operator=(SeatOccupancyStatus&&) = default;

void SeatOccupancyStatus::toCsv(std::ostream& out) const
{
    out.precision(16);
    out << std::boolalpha;
    out << static_cast<uint32_t>(driverSeatOccupancyState);out << ",";
    out << static_cast<uint32_t>(frontPassengerSeatOccupancyState);out << ",";
    out << static_cast<uint32_t>(rearMiddleSeatOccupancyState);out << ",";
    out << static_cast<uint32_t>(rearLeftSeatOccupancyState);out << ",";
    out << static_cast<uint32_t>(rearRightSeatOccupancyState);
    
}

std::stringstream SeatOccupancyStatus::toStringstream() const
{
    std::stringstream out;
    toCsv(out);
    return out;
}

template<class Archive>
    void SeatOccupancyStatus::serialize(Archive& ar, [[maybe_unused]] unsigned int version)
    {

        ar& CEREAL_NVP(driverSeatOccupancyState);
        ar& CEREAL_NVP(frontPassengerSeatOccupancyState);
        ar& CEREAL_NVP(rearMiddleSeatOccupancyState);
        ar& CEREAL_NVP(rearLeftSeatOccupancyState);
        ar& CEREAL_NVP(rearRightSeatOccupancyState);
    }


template void SeatOccupancyStatus::serialize(cereal::BinaryInputArchive& ar, unsigned int version);
template void SeatOccupancyStatus::serialize(cereal::BinaryOutputArchive& ar, unsigned int version);
template void SeatOccupancyStatus::serialize(cereal::IntrospectionArchive& ar, unsigned int version);
template void SeatOccupancyStatus::serialize(cereal::JSONInputArchive& ar, unsigned int version);
template void SeatOccupancyStatus::serialize(cereal::JSONOutputArchive& ar, unsigned int version);
template void SeatOccupancyStatus::serialize(cereal::PortableBinaryInputArchive& ar, unsigned int version);
template void SeatOccupancyStatus::serialize(cereal::PortableBinaryOutputArchive& ar, unsigned int version);

std::ostream& operator<< (std::ostream& os, const SeatOccupancyStatus& seatoccupancystatus)
{
    seatoccupancystatus.toCsv(os);
    return os;
}

bool operator== (const SeatOccupancyStatus &lhs, const SeatOccupancyStatus &rhs)
{
    return (  lhs.driverSeatOccupancyState== rhs.driverSeatOccupancyState &&
     lhs.frontPassengerSeatOccupancyState== rhs.frontPassengerSeatOccupancyState &&
     lhs.rearMiddleSeatOccupancyState== rhs.rearMiddleSeatOccupancyState &&
     lhs.rearLeftSeatOccupancyState== rhs.rearLeftSeatOccupancyState &&
     lhs.rearRightSeatOccupancyState== rhs.rearRightSeatOccupancyState 
    );
}

std::size_t SeatOccupancyStatus::hash() const noexcept
{
    std::size_t result = 17;
    result = result * 31 + std::hash<SeatOccupancyState>()(driverSeatOccupancyState);
    result = result * 31 + std::hash<SeatOccupancyState>()(frontPassengerSeatOccupancyState);
    result = result * 31 + std::hash<SeatOccupancyState>()(rearMiddleSeatOccupancyState);
    result = result * 31 + std::hash<SeatOccupancyState>()(rearLeftSeatOccupancyState);
    result = result * 31 + std::hash<SeatOccupancyState>()(rearRightSeatOccupancyState);
    return result;
}

 } // namespace av
 } // namespace msg


CEREAL_BIND_TO_ARCHIVES(av::msg::SeatOccupancyStatus)

 namespace av {
 namespace msg {


SeatPositionStatus::SeatPositionStatus() = default;

SeatPositionStatus::SeatPositionStatus(
    SeatLimitState driverSeatSideLimitState_arg ,
    SeatLimitState driverSeatReclineLimitState_arg ,
    uint8_t driverSeatHeightPositionMillimeters_arg 
    ):
    driverSeatSideLimitState { std::move(driverSeatSideLimitState_arg) } // NOLINT
      ,
    driverSeatReclineLimitState { std::move(driverSeatReclineLimitState_arg) } // NOLINT
      ,
    driverSeatHeightPositionMillimeters { std::move(driverSeatHeightPositionMillimeters_arg) } // NOLINT
      
    {
}

SeatPositionStatus::SeatPositionStatus(const SeatPositionStatus&) = default;

// NOLINTNEXTLINE
SeatPositionStatus::SeatPositionStatus(SeatPositionStatus&&) = default;

SeatPositionStatus& SeatPositionStatus::operator=(const SeatPositionStatus&) = default;

// NOLINTNEXTLINE
SeatPositionStatus& SeatPositionStatus::operator=(SeatPositionStatus&&) = default;

void SeatPositionStatus::toCsv(std::ostream& out) const
{
    out.precision(16);
    out << std::boolalpha;
    out << static_cast<uint32_t>(driverSeatSideLimitState);out << ",";
    out << static_cast<uint32_t>(driverSeatReclineLimitState);out << ",";
    // Cast to uint16_t to print as integer instead of char
        out << static_cast<uint16_t>(driverSeatHeightPositionMillimeters);
    
}

std::stringstream SeatPositionStatus::toStringstream() const
{
    std::stringstream out;
    toCsv(out);
    return out;
}

template<class Archive>
    void SeatPositionStatus::serialize(Archive& ar, [[maybe_unused]] unsigned int version)
    {

        ar& CEREAL_NVP(driverSeatSideLimitState);
        ar& CEREAL_NVP(driverSeatReclineLimitState);
        ar& CEREAL_NVP(driverSeatHeightPositionMillimeters);
    }


template void SeatPositionStatus::serialize(cereal::BinaryInputArchive& ar, unsigned int version);
template void SeatPositionStatus::serialize(cereal::BinaryOutputArchive& ar, unsigned int version);
template void SeatPositionStatus::serialize(cereal::IntrospectionArchive& ar, unsigned int version);
template void SeatPositionStatus::serialize(cereal::JSONInputArchive& ar, unsigned int version);
template void SeatPositionStatus::serialize(cereal::JSONOutputArchive& ar, unsigned int version);
template void SeatPositionStatus::serialize(cereal::PortableBinaryInputArchive& ar, unsigned int version);
template void SeatPositionStatus::serialize(cereal::PortableBinaryOutputArchive& ar, unsigned int version);

std::ostream& operator<< (std::ostream& os, const SeatPositionStatus& seatpositionstatus)
{
    seatpositionstatus.toCsv(os);
    return os;
}

bool operator== (const SeatPositionStatus &lhs, const SeatPositionStatus &rhs)
{
    return (  lhs.driverSeatSideLimitState== rhs.driverSeatSideLimitState &&
     lhs.driverSeatReclineLimitState== rhs.driverSeatReclineLimitState &&
     lhs.driverSeatHeightPositionMillimeters== rhs.driverSeatHeightPositionMillimeters 
    );
}

std::size_t SeatPositionStatus::hash() const noexcept
{
    std::size_t result = 17;
    result = result * 31 + std::hash<SeatLimitState>()(driverSeatSideLimitState);
    result = result * 31 + std::hash<SeatLimitState>()(driverSeatReclineLimitState);
    result = result * 31 + std::hash<uint8_t>()(driverSeatHeightPositionMillimeters);
    return result;
}

 } // namespace av
 } // namespace msg


CEREAL_BIND_TO_ARCHIVES(av::msg::SeatPositionStatus)

 namespace av {
 namespace msg {


SeatbeltStatus::SeatbeltStatus() = default;

SeatbeltStatus::SeatbeltStatus(
    SeatbeltState driverSeatbeltState_arg ,
    SeatbeltState frontPassengerSeatbeltState_arg ,
    SeatbeltState rearMiddleSeatbeltState_arg ,
    SeatbeltState rearLeftSeatbeltState_arg ,
    SeatbeltState rearRightSeatbeltState_arg 
    ):
    driverSeatbeltState { std::move(driverSeatbeltState_arg) } // NOLINT
      ,
    frontPassengerSeatbeltState { std::move(frontPassengerSeatbeltState_arg) } // NOLINT
      ,
    rearMiddleSeatbeltState { std::move(rearMiddleSeatbeltState_arg) } // NOLINT
      ,
    rearLeftSeatbeltState { std::move(rearLeftSeatbeltState_arg) } // NOLINT
      ,
    rearRightSeatbeltState { std::move(rearRightSeatbeltState_arg) } // NOLINT
      
    {
}

SeatbeltStatus::SeatbeltStatus(const SeatbeltStatus&) = default;

// NOLINTNEXTLINE
SeatbeltStatus::SeatbeltStatus(SeatbeltStatus&&) = default;

SeatbeltStatus& SeatbeltStatus::operator=(const SeatbeltStatus&) = default;

// NOLINTNEXTLINE
SeatbeltStatus& SeatbeltStatus::operator=(SeatbeltStatus&&) = default;

void SeatbeltStatus::toCsv(std::ostream& out) const
{
    out.precision(16);
    out << std::boolalpha;
    out << static_cast<uint32_t>(driverSeatbeltState);out << ",";
    out << static_cast<uint32_t>(frontPassengerSeatbeltState);out << ",";
    out << static_cast<uint32_t>(rearMiddleSeatbeltState);out << ",";
    out << static_cast<uint32_t>(rearLeftSeatbeltState);out << ",";
    out << static_cast<uint32_t>(rearRightSeatbeltState);
    
}

std::stringstream SeatbeltStatus::toStringstream() const
{
    std::stringstream out;
    toCsv(out);
    return out;
}

template<class Archive>
    void SeatbeltStatus::serialize(Archive& ar, [[maybe_unused]] unsigned int version)
    {

        ar& CEREAL_NVP(driverSeatbeltState);
        ar& CEREAL_NVP(frontPassengerSeatbeltState);
        ar& CEREAL_NVP(rearMiddleSeatbeltState);
        ar& CEREAL_NVP(rearLeftSeatbeltState);
        ar& CEREAL_NVP(rearRightSeatbeltState);
    }


template void SeatbeltStatus::serialize(cereal::BinaryInputArchive& ar, unsigned int version);
template void SeatbeltStatus::serialize(cereal::BinaryOutputArchive& ar, unsigned int version);
template void SeatbeltStatus::serialize(cereal::IntrospectionArchive& ar, unsigned int version);
template void SeatbeltStatus::serialize(cereal::JSONInputArchive& ar, unsigned int version);
template void SeatbeltStatus::serialize(cereal::JSONOutputArchive& ar, unsigned int version);
template void SeatbeltStatus::serialize(cereal::PortableBinaryInputArchive& ar, unsigned int version);
template void SeatbeltStatus::serialize(cereal::PortableBinaryOutputArchive& ar, unsigned int version);

std::ostream& operator<< (std::ostream& os, const SeatbeltStatus& seatbeltstatus)
{
    seatbeltstatus.toCsv(os);
    return os;
}

bool operator== (const SeatbeltStatus &lhs, const SeatbeltStatus &rhs)
{
    return (  lhs.driverSeatbeltState== rhs.driverSeatbeltState &&
     lhs.frontPassengerSeatbeltState== rhs.frontPassengerSeatbeltState &&
     lhs.rearMiddleSeatbeltState== rhs.rearMiddleSeatbeltState &&
     lhs.rearLeftSeatbeltState== rhs.rearLeftSeatbeltState &&
     lhs.rearRightSeatbeltState== rhs.rearRightSeatbeltState 
    );
}

std::size_t SeatbeltStatus::hash() const noexcept
{
    std::size_t result = 17;
    result = result * 31 + std::hash<SeatbeltState>()(driverSeatbeltState);
    result = result * 31 + std::hash<SeatbeltState>()(frontPassengerSeatbeltState);
    result = result * 31 + std::hash<SeatbeltState>()(rearMiddleSeatbeltState);
    result = result * 31 + std::hash<SeatbeltState>()(rearLeftSeatbeltState);
    result = result * 31 + std::hash<SeatbeltState>()(rearRightSeatbeltState);
    return result;
}

 } // namespace av
 } // namespace msg


CEREAL_BIND_TO_ARCHIVES(av::msg::SeatbeltStatus)

 namespace av {
 namespace msg {


WindowStatus::WindowStatus() = default;

WindowStatus::WindowStatus(
    WindowState rearRightWindowState_arg ,
    WindowState rearLeftWindowState_arg ,
    WindowState driverWindowState_arg ,
    WindowState frontPassengerWindowState_arg 
    ):
    rearRightWindowState { std::move(rearRightWindowState_arg) } // NOLINT
      ,
    rearLeftWindowState { std::move(rearLeftWindowState_arg) } // NOLINT
      ,
    driverWindowState { std::move(driverWindowState_arg) } // NOLINT
      ,
    frontPassengerWindowState { std::move(frontPassengerWindowState_arg) } // NOLINT
      
    {
}

WindowStatus::WindowStatus(const WindowStatus&) = default;

// NOLINTNEXTLINE
WindowStatus::WindowStatus(WindowStatus&&) = default;

WindowStatus& WindowStatus::operator=(const WindowStatus&) = default;

// NOLINTNEXTLINE
WindowStatus& WindowStatus::operator=(WindowStatus&&) = default;

void WindowStatus::toCsv(std::ostream& out) const
{
    out.precision(16);
    out << std::boolalpha;
    out << static_cast<uint32_t>(rearRightWindowState);out << ",";
    out << static_cast<uint32_t>(rearLeftWindowState);out << ",";
    out << static_cast<uint32_t>(driverWindowState);out << ",";
    out << static_cast<uint32_t>(frontPassengerWindowState);
    
}

std::stringstream WindowStatus::toStringstream() const
{
    std::stringstream out;
    toCsv(out);
    return out;
}

template<class Archive>
    void WindowStatus::serialize(Archive& ar, [[maybe_unused]] unsigned int version)
    {

        ar& CEREAL_NVP(rearRightWindowState);
        ar& CEREAL_NVP(rearLeftWindowState);
        ar& CEREAL_NVP(driverWindowState);
        ar& CEREAL_NVP(frontPassengerWindowState);
    }


template void WindowStatus::serialize(cereal::BinaryInputArchive& ar, unsigned int version);
template void WindowStatus::serialize(cereal::BinaryOutputArchive& ar, unsigned int version);
template void WindowStatus::serialize(cereal::IntrospectionArchive& ar, unsigned int version);
template void WindowStatus::serialize(cereal::JSONInputArchive& ar, unsigned int version);
template void WindowStatus::serialize(cereal::JSONOutputArchive& ar, unsigned int version);
template void WindowStatus::serialize(cereal::PortableBinaryInputArchive& ar, unsigned int version);
template void WindowStatus::serialize(cereal::PortableBinaryOutputArchive& ar, unsigned int version);

std::ostream& operator<< (std::ostream& os, const WindowStatus& windowstatus)
{
    windowstatus.toCsv(os);
    return os;
}

bool operator== (const WindowStatus &lhs, const WindowStatus &rhs)
{
    return (  lhs.rearRightWindowState== rhs.rearRightWindowState &&
     lhs.rearLeftWindowState== rhs.rearLeftWindowState &&
     lhs.driverWindowState== rhs.driverWindowState &&
     lhs.frontPassengerWindowState== rhs.frontPassengerWindowState 
    );
}

std::size_t WindowStatus::hash() const noexcept
{
    std::size_t result = 17;
    result = result * 31 + std::hash<WindowState>()(rearRightWindowState);
    result = result * 31 + std::hash<WindowState>()(rearLeftWindowState);
    result = result * 31 + std::hash<WindowState>()(driverWindowState);
    result = result * 31 + std::hash<WindowState>()(frontPassengerWindowState);
    return result;
}

 } // namespace av
 } // namespace msg


CEREAL_BIND_TO_ARCHIVES(av::msg::WindowStatus)

 namespace av {
 namespace msg {


WindshieldStatus::WindshieldStatus() = default;

WindshieldStatus::WindshieldStatus(
    WindshieldDefogState windshieldDefogState_arg ,
    WindshieldDefrostState windshieldDefrostState_arg 
    ):
    windshieldDefogState { std::move(windshieldDefogState_arg) } // NOLINT
      ,
    windshieldDefrostState { std::move(windshieldDefrostState_arg) } // NOLINT
      
    {
}

WindshieldStatus::WindshieldStatus(const WindshieldStatus&) = default;

// NOLINTNEXTLINE
WindshieldStatus::WindshieldStatus(WindshieldStatus&&) = default;

WindshieldStatus& WindshieldStatus::operator=(const WindshieldStatus&) = default;

// NOLINTNEXTLINE
WindshieldStatus& WindshieldStatus::operator=(WindshieldStatus&&) = default;

void WindshieldStatus::toCsv(std::ostream& out) const
{
    out.precision(16);
    out << std::boolalpha;
    out << static_cast<uint32_t>(windshieldDefogState);out << ",";
    out << static_cast<uint32_t>(windshieldDefrostState);
    
}

std::stringstream WindshieldStatus::toStringstream() const
{
    std::stringstream out;
    toCsv(out);
    return out;
}

template<class Archive>
    void WindshieldStatus::serialize(Archive& ar, [[maybe_unused]] unsigned int version)
    {

        ar& CEREAL_NVP(windshieldDefogState);
        ar& CEREAL_NVP(windshieldDefrostState);
    }


template void WindshieldStatus::serialize(cereal::BinaryInputArchive& ar, unsigned int version);
template void WindshieldStatus::serialize(cereal::BinaryOutputArchive& ar, unsigned int version);
template void WindshieldStatus::serialize(cereal::IntrospectionArchive& ar, unsigned int version);
template void WindshieldStatus::serialize(cereal::JSONInputArchive& ar, unsigned int version);
template void WindshieldStatus::serialize(cereal::JSONOutputArchive& ar, unsigned int version);
template void WindshieldStatus::serialize(cereal::PortableBinaryInputArchive& ar, unsigned int version);
template void WindshieldStatus::serialize(cereal::PortableBinaryOutputArchive& ar, unsigned int version);

std::ostream& operator<< (std::ostream& os, const WindshieldStatus& windshieldstatus)
{
    windshieldstatus.toCsv(os);
    return os;
}

bool operator== (const WindshieldStatus &lhs, const WindshieldStatus &rhs)
{
    return (  lhs.windshieldDefogState== rhs.windshieldDefogState &&
     lhs.windshieldDefrostState== rhs.windshieldDefrostState 
    );
}

std::size_t WindshieldStatus::hash() const noexcept
{
    std::size_t result = 17;
    result = result * 31 + std::hash<WindshieldDefogState>()(windshieldDefogState);
    result = result * 31 + std::hash<WindshieldDefrostState>()(windshieldDefrostState);
    return result;
}

 } // namespace av
 } // namespace msg


CEREAL_BIND_TO_ARCHIVES(av::msg::WindshieldStatus)

 namespace av {
 namespace msg {


HvacStatus::HvacStatus() = default;

HvacStatus::HvacStatus(
    int32_t hvacTemperatureDriverSide_arg ,
    int32_t hvacTemperaturePassengerSide_arg ,
    RecircState hvacCirculationState_arg ,
    AirflowDirectionState airflowDirectionState_arg ,
    AirflowSpeedState airflowSpeedState_arg 
    ):
    hvacTemperatureDriverSide { std::move(hvacTemperatureDriverSide_arg) } // NOLINT
      ,
    hvacTemperaturePassengerSide { std::move(hvacTemperaturePassengerSide_arg) } // NOLINT
      ,
    hvacCirculationState { std::move(hvacCirculationState_arg) } // NOLINT
      ,
    airflowDirectionState { std::move(airflowDirectionState_arg) } // NOLINT
      ,
    airflowSpeedState { std::move(airflowSpeedState_arg) } // NOLINT
      
    {
}

HvacStatus::HvacStatus(const HvacStatus&) = default;

// NOLINTNEXTLINE
HvacStatus::HvacStatus(HvacStatus&&) = default;

HvacStatus& HvacStatus::operator=(const HvacStatus&) = default;

// NOLINTNEXTLINE
HvacStatus& HvacStatus::operator=(HvacStatus&&) = default;

void HvacStatus::toCsv(std::ostream& out) const
{
    out.precision(16);
    out << std::boolalpha;
    out << hvacTemperatureDriverSide;out << ",";
    out << hvacTemperaturePassengerSide;out << ",";
    out << static_cast<uint32_t>(hvacCirculationState);out << ",";
    out << static_cast<uint32_t>(airflowDirectionState);out << ",";
    out << static_cast<uint32_t>(airflowSpeedState);
    
}

std::stringstream HvacStatus::toStringstream() const
{
    std::stringstream out;
    toCsv(out);
    return out;
}

template<class Archive>
    void HvacStatus::serialize(Archive& ar, [[maybe_unused]] unsigned int version)
    {

        ar& CEREAL_NVP(hvacTemperatureDriverSide);
        ar& CEREAL_NVP(hvacTemperaturePassengerSide);
        ar& CEREAL_NVP(hvacCirculationState);
        ar& CEREAL_NVP(airflowDirectionState);
        ar& CEREAL_NVP(airflowSpeedState);
    }


template void HvacStatus::serialize(cereal::BinaryInputArchive& ar, unsigned int version);
template void HvacStatus::serialize(cereal::BinaryOutputArchive& ar, unsigned int version);
template void HvacStatus::serialize(cereal::IntrospectionArchive& ar, unsigned int version);
template void HvacStatus::serialize(cereal::JSONInputArchive& ar, unsigned int version);
template void HvacStatus::serialize(cereal::JSONOutputArchive& ar, unsigned int version);
template void HvacStatus::serialize(cereal::PortableBinaryInputArchive& ar, unsigned int version);
template void HvacStatus::serialize(cereal::PortableBinaryOutputArchive& ar, unsigned int version);

std::ostream& operator<< (std::ostream& os, const HvacStatus& hvacstatus)
{
    hvacstatus.toCsv(os);
    return os;
}

bool operator== (const HvacStatus &lhs, const HvacStatus &rhs)
{
    return (  lhs.hvacTemperatureDriverSide== rhs.hvacTemperatureDriverSide &&
     lhs.hvacTemperaturePassengerSide== rhs.hvacTemperaturePassengerSide &&
     lhs.hvacCirculationState== rhs.hvacCirculationState &&
     lhs.airflowDirectionState== rhs.airflowDirectionState &&
     lhs.airflowSpeedState== rhs.airflowSpeedState 
    );
}

std::size_t HvacStatus::hash() const noexcept
{
    std::size_t result = 17;
    result = result * 31 + std::hash<int32_t>()(hvacTemperatureDriverSide);
    result = result * 31 + std::hash<int32_t>()(hvacTemperaturePassengerSide);
    result = result * 31 + std::hash<RecircState>()(hvacCirculationState);
    result = result * 31 + std::hash<AirflowDirectionState>()(airflowDirectionState);
    result = result * 31 + std::hash<AirflowSpeedState>()(airflowSpeedState);
    return result;
}

 } // namespace av
 } // namespace msg


CEREAL_BIND_TO_ARCHIVES(av::msg::HvacStatus)

 namespace av {
 namespace msg {


LampStatus::LampStatus() = default;

LampStatus::LampStatus(
    LampState frontRowCeillingSpotLampState_arg ,
    LampState rearLeftCeillingSpotLampState_arg ,
    LampState rearRightCeillingSpotLampState_arg ,
    LampState ceilingMoodLampState_arg ,
    LampState frontFootwellMoodLamp_arg ,
    LampState rearFootwellMoodLamp_arg 
    ):
    frontRowCeillingSpotLampState { std::move(frontRowCeillingSpotLampState_arg) } // NOLINT
      ,
    rearLeftCeillingSpotLampState { std::move(rearLeftCeillingSpotLampState_arg) } // NOLINT
      ,
    rearRightCeillingSpotLampState { std::move(rearRightCeillingSpotLampState_arg) } // NOLINT
      ,
    ceilingMoodLampState { std::move(ceilingMoodLampState_arg) } // NOLINT
      ,
    frontFootwellMoodLamp { std::move(frontFootwellMoodLamp_arg) } // NOLINT
      ,
    rearFootwellMoodLamp { std::move(rearFootwellMoodLamp_arg) } // NOLINT
      
    {
}

LampStatus::LampStatus(const LampStatus&) = default;

// NOLINTNEXTLINE
LampStatus::LampStatus(LampStatus&&) = default;

LampStatus& LampStatus::operator=(const LampStatus&) = default;

// NOLINTNEXTLINE
LampStatus& LampStatus::operator=(LampStatus&&) = default;

void LampStatus::toCsv(std::ostream& out) const
{
    out.precision(16);
    out << std::boolalpha;
    out << static_cast<uint32_t>(frontRowCeillingSpotLampState);out << ",";
    out << static_cast<uint32_t>(rearLeftCeillingSpotLampState);out << ",";
    out << static_cast<uint32_t>(rearRightCeillingSpotLampState);out << ",";
    out << static_cast<uint32_t>(ceilingMoodLampState);out << ",";
    out << static_cast<uint32_t>(frontFootwellMoodLamp);out << ",";
    out << static_cast<uint32_t>(rearFootwellMoodLamp);
    
}

std::stringstream LampStatus::toStringstream() const
{
    std::stringstream out;
    toCsv(out);
    return out;
}

template<class Archive>
    void LampStatus::serialize(Archive& ar, [[maybe_unused]] unsigned int version)
    {

        ar& CEREAL_NVP(frontRowCeillingSpotLampState);
        ar& CEREAL_NVP(rearLeftCeillingSpotLampState);
        ar& CEREAL_NVP(rearRightCeillingSpotLampState);
        ar& CEREAL_NVP(ceilingMoodLampState);
        ar& CEREAL_NVP(frontFootwellMoodLamp);
        ar& CEREAL_NVP(rearFootwellMoodLamp);
    }


template void LampStatus::serialize(cereal::BinaryInputArchive& ar, unsigned int version);
template void LampStatus::serialize(cereal::BinaryOutputArchive& ar, unsigned int version);
template void LampStatus::serialize(cereal::IntrospectionArchive& ar, unsigned int version);
template void LampStatus::serialize(cereal::JSONInputArchive& ar, unsigned int version);
template void LampStatus::serialize(cereal::JSONOutputArchive& ar, unsigned int version);
template void LampStatus::serialize(cereal::PortableBinaryInputArchive& ar, unsigned int version);
template void LampStatus::serialize(cereal::PortableBinaryOutputArchive& ar, unsigned int version);

std::ostream& operator<< (std::ostream& os, const LampStatus& lampstatus)
{
    lampstatus.toCsv(os);
    return os;
}

bool operator== (const LampStatus &lhs, const LampStatus &rhs)
{
    return (  lhs.frontRowCeillingSpotLampState== rhs.frontRowCeillingSpotLampState &&
     lhs.rearLeftCeillingSpotLampState== rhs.rearLeftCeillingSpotLampState &&
     lhs.rearRightCeillingSpotLampState== rhs.rearRightCeillingSpotLampState &&
     lhs.ceilingMoodLampState== rhs.ceilingMoodLampState &&
     lhs.frontFootwellMoodLamp== rhs.frontFootwellMoodLamp &&
     lhs.rearFootwellMoodLamp== rhs.rearFootwellMoodLamp 
    );
}

std::size_t LampStatus::hash() const noexcept
{
    std::size_t result = 17;
    result = result * 31 + std::hash<LampState>()(frontRowCeillingSpotLampState);
    result = result * 31 + std::hash<LampState>()(rearLeftCeillingSpotLampState);
    result = result * 31 + std::hash<LampState>()(rearRightCeillingSpotLampState);
    result = result * 31 + std::hash<LampState>()(ceilingMoodLampState);
    result = result * 31 + std::hash<LampState>()(frontFootwellMoodLamp);
    result = result * 31 + std::hash<LampState>()(rearFootwellMoodLamp);
    return result;
}

 } // namespace av
 } // namespace msg


CEREAL_BIND_TO_ARCHIVES(av::msg::LampStatus)

 namespace av {
 namespace msg {


VehicleStatus::VehicleStatus() = default;

VehicleStatus::VehicleStatus(
    float odometerKm_arg ,
    int32_t fuelRangeKm_arg ,
    float fuelLevelPercent_arg ,
    int32_t serviceFaults_arg ,
    WarningLampState acuWarningLampStatus_arg ,
    WarningLampState auxBatteryWarningLampStatus_arg ,
    WarningLampState auxBatteryAlarmStatus_arg ,
    WarningLampState aldcServiceLampStatus_arg ,
    WarningLampState bmsServiceLampStatus_arg ,
    LowTempState evLowTemperatureStatus_arg ,
    HighTempState evHighTemperatureStatus_arg 
    ):
    odometerKm { std::move(odometerKm_arg) } // NOLINT
      ,
    fuelRangeKm { std::move(fuelRangeKm_arg) } // NOLINT
      ,
    fuelLevelPercent { std::move(fuelLevelPercent_arg) } // NOLINT
      ,
    serviceFaults { std::move(serviceFaults_arg) } // NOLINT
      ,
    acuWarningLampStatus { std::move(acuWarningLampStatus_arg) } // NOLINT
      ,
    auxBatteryWarningLampStatus { std::move(auxBatteryWarningLampStatus_arg) } // NOLINT
      ,
    auxBatteryAlarmStatus { std::move(auxBatteryAlarmStatus_arg) } // NOLINT
      ,
    aldcServiceLampStatus { std::move(aldcServiceLampStatus_arg) } // NOLINT
      ,
    bmsServiceLampStatus { std::move(bmsServiceLampStatus_arg) } // NOLINT
      ,
    evLowTemperatureStatus { std::move(evLowTemperatureStatus_arg) } // NOLINT
      ,
    evHighTemperatureStatus { std::move(evHighTemperatureStatus_arg) } // NOLINT
      
    {
}

VehicleStatus::VehicleStatus(const VehicleStatus&) = default;

// NOLINTNEXTLINE
VehicleStatus::VehicleStatus(VehicleStatus&&) = default;

VehicleStatus& VehicleStatus::operator=(const VehicleStatus&) = default;

// NOLINTNEXTLINE
VehicleStatus& VehicleStatus::operator=(VehicleStatus&&) = default;

void VehicleStatus::toCsv(std::ostream& out) const
{
    out.precision(16);
    out << std::boolalpha;
    out << odometerKm;out << ",";
    out << fuelRangeKm;out << ",";
    out << fuelLevelPercent;out << ",";
    out << serviceFaults;out << ",";
    out << static_cast<uint32_t>(acuWarningLampStatus);out << ",";
    out << static_cast<uint32_t>(auxBatteryWarningLampStatus);out << ",";
    out << static_cast<uint32_t>(auxBatteryAlarmStatus);out << ",";
    out << static_cast<uint32_t>(aldcServiceLampStatus);out << ",";
    out << static_cast<uint32_t>(bmsServiceLampStatus);out << ",";
    out << static_cast<uint32_t>(evLowTemperatureStatus);out << ",";
    out << static_cast<uint32_t>(evHighTemperatureStatus);
    
}

std::stringstream VehicleStatus::toStringstream() const
{
    std::stringstream out;
    toCsv(out);
    return out;
}

template<class Archive>
    void VehicleStatus::serialize(Archive& ar, [[maybe_unused]] unsigned int version)
    {

        ar& CEREAL_NVP(odometerKm);
        ar& CEREAL_NVP(fuelRangeKm);
        ar& CEREAL_NVP(fuelLevelPercent);
        ar& CEREAL_NVP(serviceFaults);
        ar& CEREAL_NVP(acuWarningLampStatus);
        ar& CEREAL_NVP(auxBatteryWarningLampStatus);
        ar& CEREAL_NVP(auxBatteryAlarmStatus);
        ar& CEREAL_NVP(aldcServiceLampStatus);
        ar& CEREAL_NVP(bmsServiceLampStatus);
        ar& CEREAL_NVP(evLowTemperatureStatus);
        ar& CEREAL_NVP(evHighTemperatureStatus);
    }


template void VehicleStatus::serialize(cereal::BinaryInputArchive& ar, unsigned int version);
template void VehicleStatus::serialize(cereal::BinaryOutputArchive& ar, unsigned int version);
template void VehicleStatus::serialize(cereal::IntrospectionArchive& ar, unsigned int version);
template void VehicleStatus::serialize(cereal::JSONInputArchive& ar, unsigned int version);
template void VehicleStatus::serialize(cereal::JSONOutputArchive& ar, unsigned int version);
template void VehicleStatus::serialize(cereal::PortableBinaryInputArchive& ar, unsigned int version);
template void VehicleStatus::serialize(cereal::PortableBinaryOutputArchive& ar, unsigned int version);

std::ostream& operator<< (std::ostream& os, const VehicleStatus& vehiclestatus)
{
    vehiclestatus.toCsv(os);
    return os;
}

bool operator== (const VehicleStatus &lhs, const VehicleStatus &rhs)
{
    return (  lhs.odometerKm== rhs.odometerKm &&
     lhs.fuelRangeKm== rhs.fuelRangeKm &&
     lhs.fuelLevelPercent== rhs.fuelLevelPercent &&
     lhs.serviceFaults== rhs.serviceFaults &&
     lhs.acuWarningLampStatus== rhs.acuWarningLampStatus &&
     lhs.auxBatteryWarningLampStatus== rhs.auxBatteryWarningLampStatus &&
     lhs.auxBatteryAlarmStatus== rhs.auxBatteryAlarmStatus &&
     lhs.aldcServiceLampStatus== rhs.aldcServiceLampStatus &&
     lhs.bmsServiceLampStatus== rhs.bmsServiceLampStatus &&
     lhs.evLowTemperatureStatus== rhs.evLowTemperatureStatus &&
     lhs.evHighTemperatureStatus== rhs.evHighTemperatureStatus 
    );
}

std::size_t VehicleStatus::hash() const noexcept
{
    std::size_t result = 17;
    result = result * 31 + std::hash<float>()(odometerKm);
    result = result * 31 + std::hash<int32_t>()(fuelRangeKm);
    result = result * 31 + std::hash<float>()(fuelLevelPercent);
    result = result * 31 + std::hash<int32_t>()(serviceFaults);
    result = result * 31 + std::hash<WarningLampState>()(acuWarningLampStatus);
    result = result * 31 + std::hash<WarningLampState>()(auxBatteryWarningLampStatus);
    result = result * 31 + std::hash<WarningLampState>()(auxBatteryAlarmStatus);
    result = result * 31 + std::hash<WarningLampState>()(aldcServiceLampStatus);
    result = result * 31 + std::hash<WarningLampState>()(bmsServiceLampStatus);
    result = result * 31 + std::hash<LowTempState>()(evLowTemperatureStatus);
    result = result * 31 + std::hash<HighTempState>()(evHighTemperatureStatus);
    return result;
}

 } // namespace av
 } // namespace msg


CEREAL_BIND_TO_ARCHIVES(av::msg::VehicleStatus)

 namespace av {
 namespace msg {


TyreStatus::TyreStatus() = default;

TyreStatus::TyreStatus(
    uint8_t rearRightTyrePressure_arg ,
    uint8_t rearLeftTyrePressure_arg ,
    uint8_t frontPassengerTyrePressure_arg ,
    uint8_t driverTyrePressure_arg 
    ):
    rearRightTyrePressure { std::move(rearRightTyrePressure_arg) } // NOLINT
      ,
    rearLeftTyrePressure { std::move(rearLeftTyrePressure_arg) } // NOLINT
      ,
    frontPassengerTyrePressure { std::move(frontPassengerTyrePressure_arg) } // NOLINT
      ,
    driverTyrePressure { std::move(driverTyrePressure_arg) } // NOLINT
      
    {
}

TyreStatus::TyreStatus(const TyreStatus&) = default;

// NOLINTNEXTLINE
TyreStatus::TyreStatus(TyreStatus&&) = default;

TyreStatus& TyreStatus::operator=(const TyreStatus&) = default;

// NOLINTNEXTLINE
TyreStatus& TyreStatus::operator=(TyreStatus&&) = default;

void TyreStatus::toCsv(std::ostream& out) const
{
    out.precision(16);
    out << std::boolalpha;
    // Cast to uint16_t to print as integer instead of char
        out << static_cast<uint16_t>(rearRightTyrePressure);out << ",";
    // Cast to uint16_t to print as integer instead of char
        out << static_cast<uint16_t>(rearLeftTyrePressure);out << ",";
    // Cast to uint16_t to print as integer instead of char
        out << static_cast<uint16_t>(frontPassengerTyrePressure);out << ",";
    // Cast to uint16_t to print as integer instead of char
        out << static_cast<uint16_t>(driverTyrePressure);
    
}

std::stringstream TyreStatus::toStringstream() const
{
    std::stringstream out;
    toCsv(out);
    return out;
}

template<class Archive>
    void TyreStatus::serialize(Archive& ar, [[maybe_unused]] unsigned int version)
    {

        ar& CEREAL_NVP(rearRightTyrePressure);
        ar& CEREAL_NVP(rearLeftTyrePressure);
        ar& CEREAL_NVP(frontPassengerTyrePressure);
        ar& CEREAL_NVP(driverTyrePressure);
    }


template void TyreStatus::serialize(cereal::BinaryInputArchive& ar, unsigned int version);
template void TyreStatus::serialize(cereal::BinaryOutputArchive& ar, unsigned int version);
template void TyreStatus::serialize(cereal::IntrospectionArchive& ar, unsigned int version);
template void TyreStatus::serialize(cereal::JSONInputArchive& ar, unsigned int version);
template void TyreStatus::serialize(cereal::JSONOutputArchive& ar, unsigned int version);
template void TyreStatus::serialize(cereal::PortableBinaryInputArchive& ar, unsigned int version);
template void TyreStatus::serialize(cereal::PortableBinaryOutputArchive& ar, unsigned int version);

std::ostream& operator<< (std::ostream& os, const TyreStatus& tyrestatus)
{
    tyrestatus.toCsv(os);
    return os;
}

bool operator== (const TyreStatus &lhs, const TyreStatus &rhs)
{
    return (  lhs.rearRightTyrePressure== rhs.rearRightTyrePressure &&
     lhs.rearLeftTyrePressure== rhs.rearLeftTyrePressure &&
     lhs.frontPassengerTyrePressure== rhs.frontPassengerTyrePressure &&
     lhs.driverTyrePressure== rhs.driverTyrePressure 
    );
}

std::size_t TyreStatus::hash() const noexcept
{
    std::size_t result = 17;
    result = result * 31 + std::hash<uint8_t>()(rearRightTyrePressure);
    result = result * 31 + std::hash<uint8_t>()(rearLeftTyrePressure);
    result = result * 31 + std::hash<uint8_t>()(frontPassengerTyrePressure);
    result = result * 31 + std::hash<uint8_t>()(driverTyrePressure);
    return result;
}

 } // namespace av
 } // namespace msg


CEREAL_BIND_TO_ARCHIVES(av::msg::TyreStatus)

 namespace av {
 namespace msg {


LedStripStatus::LedStripStatus() = default;

LedStripStatus::LedStripStatus(
    uint8_t colorRed_arg ,
    uint8_t colorGreen_arg ,
    uint8_t colorBlue_arg ,
    LedStripState animationPattern_arg ,
    LedStripState brightness_arg ,
    LedStripState animationFrequency_arg ,
    LedStripState animationSpeed_arg ,
    LampState PowerState_arg 
    ):
    colorRed { std::move(colorRed_arg) } // NOLINT
      ,
    colorGreen { std::move(colorGreen_arg) } // NOLINT
      ,
    colorBlue { std::move(colorBlue_arg) } // NOLINT
      ,
    animationPattern { std::move(animationPattern_arg) } // NOLINT
      ,
    brightness { std::move(brightness_arg) } // NOLINT
      ,
    animationFrequency { std::move(animationFrequency_arg) } // NOLINT
      ,
    animationSpeed { std::move(animationSpeed_arg) } // NOLINT
      ,
    PowerState { std::move(PowerState_arg) } // NOLINT
      
    {
}

LedStripStatus::LedStripStatus(const LedStripStatus&) = default;

// NOLINTNEXTLINE
LedStripStatus::LedStripStatus(LedStripStatus&&) = default;

LedStripStatus& LedStripStatus::operator=(const LedStripStatus&) = default;

// NOLINTNEXTLINE
LedStripStatus& LedStripStatus::operator=(LedStripStatus&&) = default;

void LedStripStatus::toCsv(std::ostream& out) const
{
    out.precision(16);
    out << std::boolalpha;
    // Cast to uint16_t to print as integer instead of char
        out << static_cast<uint16_t>(colorRed);out << ",";
    // Cast to uint16_t to print as integer instead of char
        out << static_cast<uint16_t>(colorGreen);out << ",";
    // Cast to uint16_t to print as integer instead of char
        out << static_cast<uint16_t>(colorBlue);out << ",";
    out << static_cast<uint32_t>(animationPattern);out << ",";
    out << static_cast<uint32_t>(brightness);out << ",";
    out << static_cast<uint32_t>(animationFrequency);out << ",";
    out << static_cast<uint32_t>(animationSpeed);out << ",";
    out << static_cast<uint32_t>(PowerState);
    
}

std::stringstream LedStripStatus::toStringstream() const
{
    std::stringstream out;
    toCsv(out);
    return out;
}

template<class Archive>
    void LedStripStatus::serialize(Archive& ar, [[maybe_unused]] unsigned int version)
    {

        ar& CEREAL_NVP(colorRed);
        ar& CEREAL_NVP(colorGreen);
        ar& CEREAL_NVP(colorBlue);
        ar& CEREAL_NVP(animationPattern);
        ar& CEREAL_NVP(brightness);
        ar& CEREAL_NVP(animationFrequency);
        ar& CEREAL_NVP(animationSpeed);
        ar& CEREAL_NVP(PowerState);
    }


template void LedStripStatus::serialize(cereal::BinaryInputArchive& ar, unsigned int version);
template void LedStripStatus::serialize(cereal::BinaryOutputArchive& ar, unsigned int version);
template void LedStripStatus::serialize(cereal::IntrospectionArchive& ar, unsigned int version);
template void LedStripStatus::serialize(cereal::JSONInputArchive& ar, unsigned int version);
template void LedStripStatus::serialize(cereal::JSONOutputArchive& ar, unsigned int version);
template void LedStripStatus::serialize(cereal::PortableBinaryInputArchive& ar, unsigned int version);
template void LedStripStatus::serialize(cereal::PortableBinaryOutputArchive& ar, unsigned int version);

std::ostream& operator<< (std::ostream& os, const LedStripStatus& ledstripstatus)
{
    ledstripstatus.toCsv(os);
    return os;
}

bool operator== (const LedStripStatus &lhs, const LedStripStatus &rhs)
{
    return (  lhs.colorRed== rhs.colorRed &&
     lhs.colorGreen== rhs.colorGreen &&
     lhs.colorBlue== rhs.colorBlue &&
     lhs.animationPattern== rhs.animationPattern &&
     lhs.brightness== rhs.brightness &&
     lhs.animationFrequency== rhs.animationFrequency &&
     lhs.animationSpeed== rhs.animationSpeed &&
     lhs.PowerState== rhs.PowerState 
    );
}

std::size_t LedStripStatus::hash() const noexcept
{
    std::size_t result = 17;
    result = result * 31 + std::hash<uint8_t>()(colorRed);
    result = result * 31 + std::hash<uint8_t>()(colorGreen);
    result = result * 31 + std::hash<uint8_t>()(colorBlue);
    result = result * 31 + std::hash<LedStripState>()(animationPattern);
    result = result * 31 + std::hash<LedStripState>()(brightness);
    result = result * 31 + std::hash<LedStripState>()(animationFrequency);
    result = result * 31 + std::hash<LedStripState>()(animationSpeed);
    result = result * 31 + std::hash<LampState>()(PowerState);
    return result;
}

 } // namespace av
 } // namespace msg


CEREAL_BIND_TO_ARCHIVES(av::msg::LedStripStatus)

 namespace av {
 namespace msg {


LedStripsStatus::LedStripsStatus() = default;

LedStripsStatus::LedStripsStatus(
    LedStripStatus frontPassengerDoorLedStripStatus_arg ,
    LedStripStatus driverDoorLedStripStatus_arg ,
    LedStripStatus rearLeftDoorLedStripStatus_arg ,
    LedStripStatus rearRightDoorLedStripStatus_arg 
    ):
    frontPassengerDoorLedStripStatus { std::move(frontPassengerDoorLedStripStatus_arg) } // NOLINT
      ,
    driverDoorLedStripStatus { std::move(driverDoorLedStripStatus_arg) } // NOLINT
      ,
    rearLeftDoorLedStripStatus { std::move(rearLeftDoorLedStripStatus_arg) } // NOLINT
      ,
    rearRightDoorLedStripStatus { std::move(rearRightDoorLedStripStatus_arg) } // NOLINT
      
    {
}

LedStripsStatus::LedStripsStatus(const LedStripsStatus&) = default;

// NOLINTNEXTLINE
LedStripsStatus::LedStripsStatus(LedStripsStatus&&) = default;

LedStripsStatus& LedStripsStatus::operator=(const LedStripsStatus&) = default;

// NOLINTNEXTLINE
LedStripsStatus& LedStripsStatus::operator=(LedStripsStatus&&) = default;

void LedStripsStatus::toCsv(std::ostream& out) const
{
    out.precision(16);
    out << std::boolalpha;
    out << frontPassengerDoorLedStripStatus;out << ",";
    out << driverDoorLedStripStatus;out << ",";
    out << rearLeftDoorLedStripStatus;out << ",";
    out << rearRightDoorLedStripStatus;
    
}

std::stringstream LedStripsStatus::toStringstream() const
{
    std::stringstream out;
    toCsv(out);
    return out;
}

template<class Archive>
    void LedStripsStatus::serialize(Archive& ar, [[maybe_unused]] unsigned int version)
    {

        ar& CEREAL_NVP(frontPassengerDoorLedStripStatus);
        ar& CEREAL_NVP(driverDoorLedStripStatus);
        ar& CEREAL_NVP(rearLeftDoorLedStripStatus);
        ar& CEREAL_NVP(rearRightDoorLedStripStatus);
    }


template void LedStripsStatus::serialize(cereal::BinaryInputArchive& ar, unsigned int version);
template void LedStripsStatus::serialize(cereal::BinaryOutputArchive& ar, unsigned int version);
template void LedStripsStatus::serialize(cereal::IntrospectionArchive& ar, unsigned int version);
template void LedStripsStatus::serialize(cereal::JSONInputArchive& ar, unsigned int version);
template void LedStripsStatus::serialize(cereal::JSONOutputArchive& ar, unsigned int version);
template void LedStripsStatus::serialize(cereal::PortableBinaryInputArchive& ar, unsigned int version);
template void LedStripsStatus::serialize(cereal::PortableBinaryOutputArchive& ar, unsigned int version);

std::ostream& operator<< (std::ostream& os, const LedStripsStatus& ledstripsstatus)
{
    ledstripsstatus.toCsv(os);
    return os;
}

bool operator== (const LedStripsStatus &lhs, const LedStripsStatus &rhs)
{
    return (  lhs.frontPassengerDoorLedStripStatus== rhs.frontPassengerDoorLedStripStatus &&
     lhs.driverDoorLedStripStatus== rhs.driverDoorLedStripStatus &&
     lhs.rearLeftDoorLedStripStatus== rhs.rearLeftDoorLedStripStatus &&
     lhs.rearRightDoorLedStripStatus== rhs.rearRightDoorLedStripStatus 
    );
}

std::size_t LedStripsStatus::hash() const noexcept
{
    std::size_t result = 17;
    result = result * 31 + std::hash<LedStripStatus>()(frontPassengerDoorLedStripStatus);
    result = result * 31 + std::hash<LedStripStatus>()(driverDoorLedStripStatus);
    result = result * 31 + std::hash<LedStripStatus>()(rearLeftDoorLedStripStatus);
    result = result * 31 + std::hash<LedStripStatus>()(rearRightDoorLedStripStatus);
    return result;
}

 } // namespace av
 } // namespace msg


CEREAL_BIND_TO_ARCHIVES(av::msg::LedStripsStatus)




 namespace av {
 namespace msg {


VicBodyStatusMessage::VicBodyStatusMessage() = default;

VicBodyStatusMessage::VicBodyStatusMessage(
    DoorStatus doorStatus_arg ,
    LockStatus lockStatus_arg ,
    SeatOccupancyStatus seatOccupancyStatus_arg ,
    SeatPositionStatus driverSeatPositionStatus_arg ,
    SeatbeltStatus seatbeltsStatus_arg ,
    WindowStatus windowStatus_arg ,
    WindshieldStatus windshieldStatus_arg ,
    HvacStatus hvacStatus_arg ,
    LampStatus lampStatus_arg ,
    VehicleStatus vehicleStatus_arg ,
    TyreStatus tyreStatus_arg ,
    LedStripsStatus ledStripsStatus_arg 
    ):
    doorStatus { std::move(doorStatus_arg) } // NOLINT
      ,lockStatus { std::move(lockStatus_arg) } // NOLINT
      ,seatOccupancyStatus { std::move(seatOccupancyStatus_arg) } // NOLINT
      ,driverSeatPositionStatus { std::move(driverSeatPositionStatus_arg) } // NOLINT
      ,seatbeltsStatus { std::move(seatbeltsStatus_arg) } // NOLINT
      ,windowStatus { std::move(windowStatus_arg) } // NOLINT
      ,windshieldStatus { std::move(windshieldStatus_arg) } // NOLINT
      ,hvacStatus { std::move(hvacStatus_arg) } // NOLINT
      ,lampStatus { std::move(lampStatus_arg) } // NOLINT
      ,vehicleStatus { std::move(vehicleStatus_arg) } // NOLINT
      ,tyreStatus { std::move(tyreStatus_arg) } // NOLINT
      ,ledStripsStatus { std::move(ledStripsStatus_arg) } // NOLINT
      {
}

VicBodyStatusMessage::VicBodyStatusMessage(const VicBodyStatusMessage&) = default;

// NOLINTNEXTLINE
VicBodyStatusMessage::VicBodyStatusMessage(VicBodyStatusMessage&&) = default;

VicBodyStatusMessage& VicBodyStatusMessage::operator=(const VicBodyStatusMessage&) = default;

// NOLINTNEXTLINE
VicBodyStatusMessage& VicBodyStatusMessage::operator=(VicBodyStatusMessage&&) = default;

void VicBodyStatusMessage::toCsv(std::ostream& out) const
{
    out.precision(16);
    out << std::boolalpha;
    out << doorStatus;out << ",";
    out << lockStatus;out << ",";
    out << seatOccupancyStatus;out << ",";
    out << driverSeatPositionStatus;out << ",";
    out << seatbeltsStatus;out << ",";
    out << windowStatus;out << ",";
    out << windshieldStatus;out << ",";
    out << hvacStatus;out << ",";
    out << lampStatus;out << ",";
    out << vehicleStatus;out << ",";
    out << tyreStatus;out << ",";
    out << ledStripsStatus;
    
}

std::stringstream VicBodyStatusMessage::toStringstream() const
{
    std::stringstream out;
    toCsv(out);
    return out;
}

template<class Archive>
    void VicBodyStatusMessage::serialize(Archive& ar, [[maybe_unused]] unsigned int version)
    {

        ar& CEREAL_NVP(doorStatus);
        ar& CEREAL_NVP(lockStatus);
        ar& CEREAL_NVP(seatOccupancyStatus);
        ar& CEREAL_NVP(driverSeatPositionStatus);
        ar& CEREAL_NVP(seatbeltsStatus);
        ar& CEREAL_NVP(windowStatus);
        ar& CEREAL_NVP(windshieldStatus);
        ar& CEREAL_NVP(hvacStatus);
        ar& CEREAL_NVP(lampStatus);
        ar& CEREAL_NVP(vehicleStatus);
        ar& CEREAL_NVP(tyreStatus);
        ar& CEREAL_NVP(ledStripsStatus);
    }


template void VicBodyStatusMessage::serialize(cereal::BinaryInputArchive& ar, unsigned int version);
template void VicBodyStatusMessage::serialize(cereal::BinaryOutputArchive& ar, unsigned int version);
template void VicBodyStatusMessage::serialize(cereal::IntrospectionArchive& ar, unsigned int version);
template void VicBodyStatusMessage::serialize(cereal::JSONInputArchive& ar, unsigned int version);
template void VicBodyStatusMessage::serialize(cereal::JSONOutputArchive& ar, unsigned int version);
template void VicBodyStatusMessage::serialize(cereal::PortableBinaryInputArchive& ar, unsigned int version);
template void VicBodyStatusMessage::serialize(cereal::PortableBinaryOutputArchive& ar, unsigned int version);

std::ostream& operator<< (std::ostream& os, const VicBodyStatusMessage& vicbodystatusmessage)
{
    vicbodystatusmessage.toCsv(os);
    return os;
}

bool operator== (const VicBodyStatusMessage &lhs, const VicBodyStatusMessage &rhs)
{
    return (  lhs.doorStatus== rhs.doorStatus &&
     lhs.lockStatus== rhs.lockStatus &&
     lhs.seatOccupancyStatus== rhs.seatOccupancyStatus &&
     lhs.driverSeatPositionStatus== rhs.driverSeatPositionStatus &&
     lhs.seatbeltsStatus== rhs.seatbeltsStatus &&
     lhs.windowStatus== rhs.windowStatus &&
     lhs.windshieldStatus== rhs.windshieldStatus &&
     lhs.hvacStatus== rhs.hvacStatus &&
     lhs.lampStatus== rhs.lampStatus &&
     lhs.vehicleStatus== rhs.vehicleStatus &&
     lhs.tyreStatus== rhs.tyreStatus &&
     lhs.ledStripsStatus== rhs.ledStripsStatus 
    );
}

std::size_t VicBodyStatusMessage::hash() const noexcept
{
    std::size_t result = 17;
    result = result * 31 + std::hash<DoorStatus>()(doorStatus);
    result = result * 31 + std::hash<LockStatus>()(lockStatus);
    result = result * 31 + std::hash<SeatOccupancyStatus>()(seatOccupancyStatus);
    result = result * 31 + std::hash<SeatPositionStatus>()(driverSeatPositionStatus);
    result = result * 31 + std::hash<SeatbeltStatus>()(seatbeltsStatus);
    result = result * 31 + std::hash<WindowStatus>()(windowStatus);
    result = result * 31 + std::hash<WindshieldStatus>()(windshieldStatus);
    result = result * 31 + std::hash<HvacStatus>()(hvacStatus);
    result = result * 31 + std::hash<LampStatus>()(lampStatus);
    result = result * 31 + std::hash<VehicleStatus>()(vehicleStatus);
    result = result * 31 + std::hash<TyreStatus>()(tyreStatus);
    result = result * 31 + std::hash<LedStripsStatus>()(ledStripsStatus);
    return result;
}

 } // namespace av
 } // namespace msg


CEREAL_BIND_TO_ARCHIVES(av::msg::VicBodyStatusMessage)

