#include <interfaces/VicProtocolVersion/VicProtocolVersion.h>

#include <iterator>
#include <type_traits>

#include <cereal/cereal.hpp>
#include <cereal/archives/json.hpp>
#include <cereal/archives/binary.hpp>

#include <cereal/types/map.hpp>
#include <cereal/types/vector.hpp>
#include <cereal/types/array.hpp>
#include <cereal/types/string.hpp>
#include <cereal/types/complex.hpp>

#include <ODatum/IntrospectionArchive.h>
#include <ODatum/Hash.h>





 namespace av {
 namespace msg {


VicProtocolVersion::VicProtocolVersion() = default;

VicProtocolVersion::VicProtocolVersion(
    uint8_t vicProtocolMajor_arg ,
    uint8_t vicProtocolMinor_arg ,
    std::string vicProtocolString_arg 
    ):
    vicProtocolMajor { std::move(vicProtocolMajor_arg) } // NOLINT
      ,vicProtocolMinor { std::move(vicProtocolMinor_arg) } // NOLINT
      ,vicProtocolString { std::move(vicProtocolString_arg) } // NOLINT
      {
}

VicProtocolVersion::VicProtocolVersion(const VicProtocolVersion&) = default;

// NOLINTNEXTLINE
VicProtocolVersion::VicProtocolVersion(VicProtocolVersion&&) = default;

VicProtocolVersion& VicProtocolVersion::operator=(const VicProtocolVersion&) = default;

// NOLINTNEXTLINE
VicProtocolVersion& VicProtocolVersion::operator=(VicProtocolVersion&&) = default;

void VicProtocolVersion::toCsv(std::ostream& out) const
{
    out.precision(16);
    out << std::boolalpha;
    // Cast to uint16_t to print as integer instead of char
        out << static_cast<uint16_t>(vicProtocolMajor);out << ",";
    // Cast to uint16_t to print as integer instead of char
        out << static_cast<uint16_t>(vicProtocolMinor);out << ",";
    out << "\"";
        out << vicProtocolString;
        out << "\"";
    
}

std::stringstream VicProtocolVersion::toStringstream() const
{
    std::stringstream out;
    toCsv(out);
    return out;
}

template<class Archive>
    void VicProtocolVersion::serialize(Archive& ar, [[maybe_unused]] unsigned int version)
    {

        ar& CEREAL_NVP(vicProtocolMajor);
        ar& CEREAL_NVP(vicProtocolMinor);
        ar& CEREAL_NVP(vicProtocolString);
    }


template void VicProtocolVersion::serialize(cereal::BinaryInputArchive& ar, unsigned int version);
template void VicProtocolVersion::serialize(cereal::BinaryOutputArchive& ar, unsigned int version);
template void VicProtocolVersion::serialize(cereal::IntrospectionArchive& ar, unsigned int version);
template void VicProtocolVersion::serialize(cereal::JSONInputArchive& ar, unsigned int version);
template void VicProtocolVersion::serialize(cereal::JSONOutputArchive& ar, unsigned int version);
template void VicProtocolVersion::serialize(cereal::PortableBinaryInputArchive& ar, unsigned int version);
template void VicProtocolVersion::serialize(cereal::PortableBinaryOutputArchive& ar, unsigned int version);

std::ostream& operator<< (std::ostream& os, const VicProtocolVersion& vicprotocolversion)
{
    vicprotocolversion.toCsv(os);
    return os;
}

bool operator== (const VicProtocolVersion &lhs, const VicProtocolVersion &rhs)
{
    return (  lhs.vicProtocolMajor== rhs.vicProtocolMajor &&
     lhs.vicProtocolMinor== rhs.vicProtocolMinor &&
     lhs.vicProtocolString== rhs.vicProtocolString 
    );
}

std::size_t VicProtocolVersion::hash() const noexcept
{
    std::size_t result = 17;
    result = result * 31 + std::hash<uint8_t>()(vicProtocolMajor);
    result = result * 31 + std::hash<uint8_t>()(vicProtocolMinor);
    result = result * 31 + std::hash<std::string>()(vicProtocolString);
    return result;
}

 } // namespace av
 } // namespace msg


CEREAL_BIND_TO_ARCHIVES(av::msg::VicProtocolVersion)

