#include <interfaces/VicVehicleDriveInterface/VicVehicleDriveInterface.h>

#include <iterator>
#include <type_traits>

#include <cereal/cereal.hpp>
#include <cereal/archives/json.hpp>
#include <cereal/archives/binary.hpp>

#include <cereal/types/map.hpp>
#include <cereal/types/vector.hpp>
#include <cereal/types/array.hpp>
#include <cereal/types/string.hpp>
#include <cereal/types/complex.hpp>

#include <ODatum/IntrospectionArchive.h>
#include <ODatum/Hash.h>





 namespace av {
 namespace msg {


VicVehicleDriveInterface::VicVehicleDriveInterface() = default;

VicVehicleDriveInterface::VicVehicleDriveInterface(
    TimePoint timestamp_arg ,
    double emergencyParkingBrakeApplied_arg ,
    double emergencyParkingBrakeReleased_arg ,
    VicMode modeManager_arg ,
    double transmissionInDrive_arg ,
    double transmissionInPark_arg 
    ):
    timestamp { std::move(timestamp_arg) } // NOLINT
      ,emergencyParkingBrakeApplied { std::move(emergencyParkingBrakeApplied_arg) } // NOLINT
      ,emergencyParkingBrakeReleased { std::move(emergencyParkingBrakeReleased_arg) } // NOLINT
      ,modeManager { std::move(modeManager_arg) } // NOLINT
      ,transmissionInDrive { std::move(transmissionInDrive_arg) } // NOLINT
      ,transmissionInPark { std::move(transmissionInPark_arg) } // NOLINT
      {
}

VicVehicleDriveInterface::VicVehicleDriveInterface(const VicVehicleDriveInterface&) = default;

// NOLINTNEXTLINE
VicVehicleDriveInterface::VicVehicleDriveInterface(VicVehicleDriveInterface&&) = default;

VicVehicleDriveInterface& VicVehicleDriveInterface::operator=(const VicVehicleDriveInterface&) = default;

// NOLINTNEXTLINE
VicVehicleDriveInterface& VicVehicleDriveInterface::operator=(VicVehicleDriveInterface&&) = default;

void VicVehicleDriveInterface::toCsv(std::ostream& out) const
{
    out.precision(16);
    out << std::boolalpha;
    out << timestamp;out << ",";
    out << emergencyParkingBrakeApplied;out << ",";
    out << emergencyParkingBrakeReleased;out << ",";
    out << static_cast<uint32_t>(modeManager);out << ",";
    out << transmissionInDrive;out << ",";
    out << transmissionInPark;
    
}

std::stringstream VicVehicleDriveInterface::toStringstream() const
{
    std::stringstream out;
    toCsv(out);
    return out;
}

template<class Archive>
    void VicVehicleDriveInterface::serialize(Archive& ar, [[maybe_unused]] unsigned int version)
    {

        ar& CEREAL_NVP(timestamp);
        ar& CEREAL_NVP(emergencyParkingBrakeApplied);
        ar& CEREAL_NVP(emergencyParkingBrakeReleased);
        ar& CEREAL_NVP(modeManager);
        ar& CEREAL_NVP(transmissionInDrive);
        ar& CEREAL_NVP(transmissionInPark);
    }


template void VicVehicleDriveInterface::serialize(cereal::BinaryInputArchive& ar, unsigned int version);
template void VicVehicleDriveInterface::serialize(cereal::BinaryOutputArchive& ar, unsigned int version);
template void VicVehicleDriveInterface::serialize(cereal::IntrospectionArchive& ar, unsigned int version);
template void VicVehicleDriveInterface::serialize(cereal::JSONInputArchive& ar, unsigned int version);
template void VicVehicleDriveInterface::serialize(cereal::JSONOutputArchive& ar, unsigned int version);
template void VicVehicleDriveInterface::serialize(cereal::PortableBinaryInputArchive& ar, unsigned int version);
template void VicVehicleDriveInterface::serialize(cereal::PortableBinaryOutputArchive& ar, unsigned int version);

std::ostream& operator<< (std::ostream& os, const VicVehicleDriveInterface& vicvehicledriveinterface)
{
    vicvehicledriveinterface.toCsv(os);
    return os;
}

bool operator== (const VicVehicleDriveInterface &lhs, const VicVehicleDriveInterface &rhs)
{
    return (  lhs.timestamp== rhs.timestamp &&
     lhs.emergencyParkingBrakeApplied== rhs.emergencyParkingBrakeApplied &&
     lhs.emergencyParkingBrakeReleased== rhs.emergencyParkingBrakeReleased &&
     lhs.modeManager== rhs.modeManager &&
     lhs.transmissionInDrive== rhs.transmissionInDrive &&
     lhs.transmissionInPark== rhs.transmissionInPark 
    );
}

std::size_t VicVehicleDriveInterface::hash() const noexcept
{
    std::size_t result = 17;
    result = result * 31 + 0;
    result = result * 31 + std::hash<double>()(emergencyParkingBrakeApplied);
    result = result * 31 + std::hash<double>()(emergencyParkingBrakeReleased);
    result = result * 31 + std::hash<VicMode>()(modeManager);
    result = result * 31 + std::hash<double>()(transmissionInDrive);
    result = result * 31 + std::hash<double>()(transmissionInPark);
    return result;
}

 } // namespace av
 } // namespace msg


CEREAL_BIND_TO_ARCHIVES(av::msg::VicVehicleDriveInterface)

