#include <interfaces/VisionMessage/VisionMessage.h>

#include <iterator>
#include <type_traits>

#include <cereal/cereal.hpp>
#include <cereal/archives/json.hpp>
#include <cereal/archives/binary.hpp>

#include <cereal/types/map.hpp>
#include <cereal/types/vector.hpp>
#include <cereal/types/array.hpp>
#include <cereal/types/string.hpp>
#include <cereal/types/complex.hpp>

#include <ODatum/IntrospectionArchive.h>
#include <ODatum/Hash.h>


 namespace av {
 namespace msg {
 namespace vision {


point_2d::point_2d() = default;

point_2d::point_2d(
    double x_arg ,
    double y_arg 
    ):
    x { std::move(x_arg) } // NOLINT
      ,
    y { std::move(y_arg) } // NOLINT
      
    {
}

point_2d::point_2d(const point_2d&) = default;

// NOLINTNEXTLINE
point_2d::point_2d(point_2d&&) = default;

point_2d& point_2d::operator=(const point_2d&) = default;

// NOLINTNEXTLINE
point_2d& point_2d::operator=(point_2d&&) = default;

void point_2d::toCsv(std::ostream& out) const
{
    out.precision(16);
    out << std::boolalpha;
    out << x;out << ",";
    out << y;
    
}

std::stringstream point_2d::toStringstream() const
{
    std::stringstream out;
    toCsv(out);
    return out;
}

template<class Archive>
    void point_2d::serialize(Archive& ar, [[maybe_unused]] unsigned int version)
    {

        ar& CEREAL_NVP(x);
        ar& CEREAL_NVP(y);
    }


template void point_2d::serialize(cereal::BinaryInputArchive& ar, unsigned int version);
template void point_2d::serialize(cereal::BinaryOutputArchive& ar, unsigned int version);
template void point_2d::serialize(cereal::IntrospectionArchive& ar, unsigned int version);
template void point_2d::serialize(cereal::JSONInputArchive& ar, unsigned int version);
template void point_2d::serialize(cereal::JSONOutputArchive& ar, unsigned int version);
template void point_2d::serialize(cereal::PortableBinaryInputArchive& ar, unsigned int version);
template void point_2d::serialize(cereal::PortableBinaryOutputArchive& ar, unsigned int version);

std::ostream& operator<< (std::ostream& os, const point_2d& point_2d)
{
    point_2d.toCsv(os);
    return os;
}

bool operator== (const point_2d &lhs, const point_2d &rhs)
{
    return (  lhs.x== rhs.x &&
     lhs.y== rhs.y 
    );
}

std::size_t point_2d::hash() const noexcept
{
    std::size_t result = 17;
    result = result * 31 + std::hash<double>()(x);
    result = result * 31 + std::hash<double>()(y);
    return result;
}

 } // namespace av
 } // namespace msg
 } // namespace vision


CEREAL_BIND_TO_ARCHIVES(av::msg::vision::point_2d)

 namespace av {
 namespace msg {
 namespace vision {


size_2d::size_2d() = default;

size_2d::size_2d(
    double x_arg ,
    double y_arg 
    ):
    x { std::move(x_arg) } // NOLINT
      ,
    y { std::move(y_arg) } // NOLINT
      
    {
}

size_2d::size_2d(const size_2d&) = default;

// NOLINTNEXTLINE
size_2d::size_2d(size_2d&&) = default;

size_2d& size_2d::operator=(const size_2d&) = default;

// NOLINTNEXTLINE
size_2d& size_2d::operator=(size_2d&&) = default;

void size_2d::toCsv(std::ostream& out) const
{
    out.precision(16);
    out << std::boolalpha;
    out << x;out << ",";
    out << y;
    
}

std::stringstream size_2d::toStringstream() const
{
    std::stringstream out;
    toCsv(out);
    return out;
}

template<class Archive>
    void size_2d::serialize(Archive& ar, [[maybe_unused]] unsigned int version)
    {

        ar& CEREAL_NVP(x);
        ar& CEREAL_NVP(y);
    }


template void size_2d::serialize(cereal::BinaryInputArchive& ar, unsigned int version);
template void size_2d::serialize(cereal::BinaryOutputArchive& ar, unsigned int version);
template void size_2d::serialize(cereal::IntrospectionArchive& ar, unsigned int version);
template void size_2d::serialize(cereal::JSONInputArchive& ar, unsigned int version);
template void size_2d::serialize(cereal::JSONOutputArchive& ar, unsigned int version);
template void size_2d::serialize(cereal::PortableBinaryInputArchive& ar, unsigned int version);
template void size_2d::serialize(cereal::PortableBinaryOutputArchive& ar, unsigned int version);

std::ostream& operator<< (std::ostream& os, const size_2d& size_2d)
{
    size_2d.toCsv(os);
    return os;
}

bool operator== (const size_2d &lhs, const size_2d &rhs)
{
    return (  lhs.x== rhs.x &&
     lhs.y== rhs.y 
    );
}

std::size_t size_2d::hash() const noexcept
{
    std::size_t result = 17;
    result = result * 31 + std::hash<double>()(x);
    result = result * 31 + std::hash<double>()(y);
    return result;
}

 } // namespace av
 } // namespace msg
 } // namespace vision


CEREAL_BIND_TO_ARCHIVES(av::msg::vision::size_2d)

 namespace av {
 namespace msg {
 namespace vision {


point_2i::point_2i() = default;

point_2i::point_2i(
    int32_t x_arg ,
    int32_t y_arg 
    ):
    x { std::move(x_arg) } // NOLINT
      ,
    y { std::move(y_arg) } // NOLINT
      
    {
}

point_2i::point_2i(const point_2i&) = default;

// NOLINTNEXTLINE
point_2i::point_2i(point_2i&&) = default;

point_2i& point_2i::operator=(const point_2i&) = default;

// NOLINTNEXTLINE
point_2i& point_2i::operator=(point_2i&&) = default;

void point_2i::toCsv(std::ostream& out) const
{
    out.precision(16);
    out << std::boolalpha;
    out << x;out << ",";
    out << y;
    
}

std::stringstream point_2i::toStringstream() const
{
    std::stringstream out;
    toCsv(out);
    return out;
}

template<class Archive>
    void point_2i::serialize(Archive& ar, [[maybe_unused]] unsigned int version)
    {

        ar& CEREAL_NVP(x);
        ar& CEREAL_NVP(y);
    }


template void point_2i::serialize(cereal::BinaryInputArchive& ar, unsigned int version);
template void point_2i::serialize(cereal::BinaryOutputArchive& ar, unsigned int version);
template void point_2i::serialize(cereal::IntrospectionArchive& ar, unsigned int version);
template void point_2i::serialize(cereal::JSONInputArchive& ar, unsigned int version);
template void point_2i::serialize(cereal::JSONOutputArchive& ar, unsigned int version);
template void point_2i::serialize(cereal::PortableBinaryInputArchive& ar, unsigned int version);
template void point_2i::serialize(cereal::PortableBinaryOutputArchive& ar, unsigned int version);

std::ostream& operator<< (std::ostream& os, const point_2i& point_2i)
{
    point_2i.toCsv(os);
    return os;
}

bool operator== (const point_2i &lhs, const point_2i &rhs)
{
    return (  lhs.x== rhs.x &&
     lhs.y== rhs.y 
    );
}

std::size_t point_2i::hash() const noexcept
{
    std::size_t result = 17;
    result = result * 31 + std::hash<int32_t>()(x);
    result = result * 31 + std::hash<int32_t>()(y);
    return result;
}

 } // namespace av
 } // namespace msg
 } // namespace vision


CEREAL_BIND_TO_ARCHIVES(av::msg::vision::point_2i)

 namespace av {
 namespace msg {
 namespace vision {


size_2i::size_2i() = default;

size_2i::size_2i(
    int32_t x_arg ,
    int32_t y_arg 
    ):
    x { std::move(x_arg) } // NOLINT
      ,
    y { std::move(y_arg) } // NOLINT
      
    {
}

size_2i::size_2i(const size_2i&) = default;

// NOLINTNEXTLINE
size_2i::size_2i(size_2i&&) = default;

size_2i& size_2i::operator=(const size_2i&) = default;

// NOLINTNEXTLINE
size_2i& size_2i::operator=(size_2i&&) = default;

void size_2i::toCsv(std::ostream& out) const
{
    out.precision(16);
    out << std::boolalpha;
    out << x;out << ",";
    out << y;
    
}

std::stringstream size_2i::toStringstream() const
{
    std::stringstream out;
    toCsv(out);
    return out;
}

template<class Archive>
    void size_2i::serialize(Archive& ar, [[maybe_unused]] unsigned int version)
    {

        ar& CEREAL_NVP(x);
        ar& CEREAL_NVP(y);
    }


template void size_2i::serialize(cereal::BinaryInputArchive& ar, unsigned int version);
template void size_2i::serialize(cereal::BinaryOutputArchive& ar, unsigned int version);
template void size_2i::serialize(cereal::IntrospectionArchive& ar, unsigned int version);
template void size_2i::serialize(cereal::JSONInputArchive& ar, unsigned int version);
template void size_2i::serialize(cereal::JSONOutputArchive& ar, unsigned int version);
template void size_2i::serialize(cereal::PortableBinaryInputArchive& ar, unsigned int version);
template void size_2i::serialize(cereal::PortableBinaryOutputArchive& ar, unsigned int version);

std::ostream& operator<< (std::ostream& os, const size_2i& size_2i)
{
    size_2i.toCsv(os);
    return os;
}

bool operator== (const size_2i &lhs, const size_2i &rhs)
{
    return (  lhs.x== rhs.x &&
     lhs.y== rhs.y 
    );
}

std::size_t size_2i::hash() const noexcept
{
    std::size_t result = 17;
    result = result * 31 + std::hash<int32_t>()(x);
    result = result * 31 + std::hash<int32_t>()(y);
    return result;
}

 } // namespace av
 } // namespace msg
 } // namespace vision


CEREAL_BIND_TO_ARCHIVES(av::msg::vision::size_2i)

 namespace av {
 namespace msg {
 namespace vision {


rectangle_2d::rectangle_2d() = default;

rectangle_2d::rectangle_2d(
    point_2d top_left_corner_arg ,
    size_2d size_arg 
    ):
    top_left_corner { std::move(top_left_corner_arg) } // NOLINT
      ,
    size { std::move(size_arg) } // NOLINT
      
    {
}

rectangle_2d::rectangle_2d(const rectangle_2d&) = default;

// NOLINTNEXTLINE
rectangle_2d::rectangle_2d(rectangle_2d&&) = default;

rectangle_2d& rectangle_2d::operator=(const rectangle_2d&) = default;

// NOLINTNEXTLINE
rectangle_2d& rectangle_2d::operator=(rectangle_2d&&) = default;

void rectangle_2d::toCsv(std::ostream& out) const
{
    out.precision(16);
    out << std::boolalpha;
    out << top_left_corner;out << ",";
    out << size;
    
}

std::stringstream rectangle_2d::toStringstream() const
{
    std::stringstream out;
    toCsv(out);
    return out;
}

template<class Archive>
    void rectangle_2d::serialize(Archive& ar, [[maybe_unused]] unsigned int version)
    {

        ar& CEREAL_NVP(top_left_corner);
        ar& CEREAL_NVP(size);
    }


template void rectangle_2d::serialize(cereal::BinaryInputArchive& ar, unsigned int version);
template void rectangle_2d::serialize(cereal::BinaryOutputArchive& ar, unsigned int version);
template void rectangle_2d::serialize(cereal::IntrospectionArchive& ar, unsigned int version);
template void rectangle_2d::serialize(cereal::JSONInputArchive& ar, unsigned int version);
template void rectangle_2d::serialize(cereal::JSONOutputArchive& ar, unsigned int version);
template void rectangle_2d::serialize(cereal::PortableBinaryInputArchive& ar, unsigned int version);
template void rectangle_2d::serialize(cereal::PortableBinaryOutputArchive& ar, unsigned int version);

std::ostream& operator<< (std::ostream& os, const rectangle_2d& rectangle_2d)
{
    rectangle_2d.toCsv(os);
    return os;
}

bool operator== (const rectangle_2d &lhs, const rectangle_2d &rhs)
{
    return (  lhs.top_left_corner== rhs.top_left_corner &&
     lhs.size== rhs.size 
    );
}

std::size_t rectangle_2d::hash() const noexcept
{
    std::size_t result = 17;
    result = result * 31 + std::hash<point_2d>()(top_left_corner);
    result = result * 31 + std::hash<size_2d>()(size);
    return result;
}

 } // namespace av
 } // namespace msg
 } // namespace vision


CEREAL_BIND_TO_ARCHIVES(av::msg::vision::rectangle_2d)

 namespace av {
 namespace msg {
 namespace vision {


rectangle_2i::rectangle_2i() = default;

rectangle_2i::rectangle_2i(
    point_2i top_left_corner_arg ,
    size_2i size_arg 
    ):
    top_left_corner { std::move(top_left_corner_arg) } // NOLINT
      ,
    size { std::move(size_arg) } // NOLINT
      
    {
}

rectangle_2i::rectangle_2i(const rectangle_2i&) = default;

// NOLINTNEXTLINE
rectangle_2i::rectangle_2i(rectangle_2i&&) = default;

rectangle_2i& rectangle_2i::operator=(const rectangle_2i&) = default;

// NOLINTNEXTLINE
rectangle_2i& rectangle_2i::operator=(rectangle_2i&&) = default;

void rectangle_2i::toCsv(std::ostream& out) const
{
    out.precision(16);
    out << std::boolalpha;
    out << top_left_corner;out << ",";
    out << size;
    
}

std::stringstream rectangle_2i::toStringstream() const
{
    std::stringstream out;
    toCsv(out);
    return out;
}

template<class Archive>
    void rectangle_2i::serialize(Archive& ar, [[maybe_unused]] unsigned int version)
    {

        ar& CEREAL_NVP(top_left_corner);
        ar& CEREAL_NVP(size);
    }


template void rectangle_2i::serialize(cereal::BinaryInputArchive& ar, unsigned int version);
template void rectangle_2i::serialize(cereal::BinaryOutputArchive& ar, unsigned int version);
template void rectangle_2i::serialize(cereal::IntrospectionArchive& ar, unsigned int version);
template void rectangle_2i::serialize(cereal::JSONInputArchive& ar, unsigned int version);
template void rectangle_2i::serialize(cereal::JSONOutputArchive& ar, unsigned int version);
template void rectangle_2i::serialize(cereal::PortableBinaryInputArchive& ar, unsigned int version);
template void rectangle_2i::serialize(cereal::PortableBinaryOutputArchive& ar, unsigned int version);

std::ostream& operator<< (std::ostream& os, const rectangle_2i& rectangle_2i)
{
    rectangle_2i.toCsv(os);
    return os;
}

bool operator== (const rectangle_2i &lhs, const rectangle_2i &rhs)
{
    return (  lhs.top_left_corner== rhs.top_left_corner &&
     lhs.size== rhs.size 
    );
}

std::size_t rectangle_2i::hash() const noexcept
{
    std::size_t result = 17;
    result = result * 31 + std::hash<point_2i>()(top_left_corner);
    result = result * 31 + std::hash<size_2i>()(size);
    return result;
}

 } // namespace av
 } // namespace msg
 } // namespace vision


CEREAL_BIND_TO_ARCHIVES(av::msg::vision::rectangle_2i)

 namespace av {
 namespace msg {
 namespace vision {


cuboid_2i::cuboid_2i() = default;

cuboid_2i::cuboid_2i(
    std::vector<point_2i> corners_arg 
    ):
    corners { std::move(corners_arg) } // NOLINT
      
    {
}

cuboid_2i::cuboid_2i(const cuboid_2i&) = default;

// NOLINTNEXTLINE
cuboid_2i::cuboid_2i(cuboid_2i&&) = default;

cuboid_2i& cuboid_2i::operator=(const cuboid_2i&) = default;

// NOLINTNEXTLINE
cuboid_2i& cuboid_2i::operator=(cuboid_2i&&) = default;

void cuboid_2i::toCsv(std::ostream& out) const
{
    out.precision(16);
    out << std::boolalpha;
    // Enclose sequence in double quotations and curly brackets to keep CSV line length uniform
        
        out << "\"{{";
        
        if (!corners.empty())
        {
            std::copy(corners.begin(), corners.end() - 1, std::ostream_iterator<point_2i>(out, "},{"));
            std::copy(corners.end()-1, corners.end(), std::ostream_iterator<point_2i>(out, ""));
        }
        out << "}}\"";
    
}

std::stringstream cuboid_2i::toStringstream() const
{
    std::stringstream out;
    toCsv(out);
    return out;
}

template<class Archive>
    void cuboid_2i::serialize(Archive& ar, [[maybe_unused]] unsigned int version)
    {

        ar& CEREAL_NVP(corners);
    }


template void cuboid_2i::serialize(cereal::BinaryInputArchive& ar, unsigned int version);
template void cuboid_2i::serialize(cereal::BinaryOutputArchive& ar, unsigned int version);
template void cuboid_2i::serialize(cereal::IntrospectionArchive& ar, unsigned int version);
template void cuboid_2i::serialize(cereal::JSONInputArchive& ar, unsigned int version);
template void cuboid_2i::serialize(cereal::JSONOutputArchive& ar, unsigned int version);
template void cuboid_2i::serialize(cereal::PortableBinaryInputArchive& ar, unsigned int version);
template void cuboid_2i::serialize(cereal::PortableBinaryOutputArchive& ar, unsigned int version);

std::ostream& operator<< (std::ostream& os, const cuboid_2i& cuboid_2i)
{
    cuboid_2i.toCsv(os);
    return os;
}

bool operator== (const cuboid_2i &lhs, const cuboid_2i &rhs)
{
    return (  lhs.corners== rhs.corners 
    );
}

std::size_t cuboid_2i::hash() const noexcept
{
    std::size_t result = 17;
    result = result * 31 + hashVector(corners);
    return result;
}

 } // namespace av
 } // namespace msg
 } // namespace vision


CEREAL_BIND_TO_ARCHIVES(av::msg::vision::cuboid_2i)

 namespace av {
 namespace msg {
 namespace vision {


roi::roi() = default;

roi::roi(
    rectangle_2d roi_rect_arg ,
    bool is_used_arg 
    ):
    roi_rect { std::move(roi_rect_arg) } // NOLINT
      ,
    is_used { std::move(is_used_arg) } // NOLINT
      
    {
}

roi::roi(const roi&) = default;

// NOLINTNEXTLINE
roi::roi(roi&&) = default;

roi& roi::operator=(const roi&) = default;

// NOLINTNEXTLINE
roi& roi::operator=(roi&&) = default;

void roi::toCsv(std::ostream& out) const
{
    out.precision(16);
    out << std::boolalpha;
    out << roi_rect;out << ",";
    out << is_used;
    
}

std::stringstream roi::toStringstream() const
{
    std::stringstream out;
    toCsv(out);
    return out;
}

template<class Archive>
    void roi::serialize(Archive& ar, [[maybe_unused]] unsigned int version)
    {

        ar& CEREAL_NVP(roi_rect);
        ar& CEREAL_NVP(is_used);
    }


template void roi::serialize(cereal::BinaryInputArchive& ar, unsigned int version);
template void roi::serialize(cereal::BinaryOutputArchive& ar, unsigned int version);
template void roi::serialize(cereal::IntrospectionArchive& ar, unsigned int version);
template void roi::serialize(cereal::JSONInputArchive& ar, unsigned int version);
template void roi::serialize(cereal::JSONOutputArchive& ar, unsigned int version);
template void roi::serialize(cereal::PortableBinaryInputArchive& ar, unsigned int version);
template void roi::serialize(cereal::PortableBinaryOutputArchive& ar, unsigned int version);

std::ostream& operator<< (std::ostream& os, const roi& roi)
{
    roi.toCsv(os);
    return os;
}

bool operator== (const roi &lhs, const roi &rhs)
{
    return (  lhs.roi_rect== rhs.roi_rect &&
     lhs.is_used== rhs.is_used 
    );
}

std::size_t roi::hash() const noexcept
{
    std::size_t result = 17;
    result = result * 31 + std::hash<rectangle_2d>()(roi_rect);
    result = result * 31 + std::hash<bool>()(is_used);
    return result;
}

 } // namespace av
 } // namespace msg
 } // namespace vision


CEREAL_BIND_TO_ARCHIVES(av::msg::vision::roi)

 namespace av {
 namespace msg {
 namespace vision {


vector_1d::vector_1d() = default;

vector_1d::vector_1d(
    std::vector<double> data_arg 
    ):
    data { std::move(data_arg) } // NOLINT
      
    {
}

vector_1d::vector_1d(const vector_1d&) = default;

// NOLINTNEXTLINE
vector_1d::vector_1d(vector_1d&&) = default;

vector_1d& vector_1d::operator=(const vector_1d&) = default;

// NOLINTNEXTLINE
vector_1d& vector_1d::operator=(vector_1d&&) = default;

void vector_1d::toCsv(std::ostream& out) const
{
    out.precision(16);
    out << std::boolalpha;
    // Enclose sequence in double quotations and curly brackets to keep CSV line length uniform
        
        out << "\"{{";
        
        if (!data.empty())
        {
            std::copy(data.begin(), data.end() - 1, std::ostream_iterator<double>(out, "},{"));
            std::copy(data.end()-1, data.end(), std::ostream_iterator<double>(out, ""));
        }
        out << "}}\"";
    
}

std::stringstream vector_1d::toStringstream() const
{
    std::stringstream out;
    toCsv(out);
    return out;
}

template<class Archive>
    void vector_1d::serialize(Archive& ar, [[maybe_unused]] unsigned int version)
    {

        ar& CEREAL_NVP(data);
    }


template void vector_1d::serialize(cereal::BinaryInputArchive& ar, unsigned int version);
template void vector_1d::serialize(cereal::BinaryOutputArchive& ar, unsigned int version);
template void vector_1d::serialize(cereal::IntrospectionArchive& ar, unsigned int version);
template void vector_1d::serialize(cereal::JSONInputArchive& ar, unsigned int version);
template void vector_1d::serialize(cereal::JSONOutputArchive& ar, unsigned int version);
template void vector_1d::serialize(cereal::PortableBinaryInputArchive& ar, unsigned int version);
template void vector_1d::serialize(cereal::PortableBinaryOutputArchive& ar, unsigned int version);

std::ostream& operator<< (std::ostream& os, const vector_1d& vector_1d)
{
    vector_1d.toCsv(os);
    return os;
}

bool operator== (const vector_1d &lhs, const vector_1d &rhs)
{
    return (  lhs.data== rhs.data 
    );
}

std::size_t vector_1d::hash() const noexcept
{
    std::size_t result = 17;
    result = result * 31 + hashVector(data);
    return result;
}

 } // namespace av
 } // namespace msg
 } // namespace vision


CEREAL_BIND_TO_ARCHIVES(av::msg::vision::vector_1d)

 namespace av {
 namespace msg {
 namespace vision {


matrix_2d::matrix_2d() = default;

matrix_2d::matrix_2d(
    std::vector<vector_1d> rows_arg 
    ):
    rows { std::move(rows_arg) } // NOLINT
      
    {
}

matrix_2d::matrix_2d(const matrix_2d&) = default;

// NOLINTNEXTLINE
matrix_2d::matrix_2d(matrix_2d&&) = default;

matrix_2d& matrix_2d::operator=(const matrix_2d&) = default;

// NOLINTNEXTLINE
matrix_2d& matrix_2d::operator=(matrix_2d&&) = default;

void matrix_2d::toCsv(std::ostream& out) const
{
    out.precision(16);
    out << std::boolalpha;
    // Enclose sequence in double quotations and curly brackets to keep CSV line length uniform
        
        out << "\"{{";
        
        if (!rows.empty())
        {
            std::copy(rows.begin(), rows.end() - 1, std::ostream_iterator<vector_1d>(out, "},{"));
            std::copy(rows.end()-1, rows.end(), std::ostream_iterator<vector_1d>(out, ""));
        }
        out << "}}\"";
    
}

std::stringstream matrix_2d::toStringstream() const
{
    std::stringstream out;
    toCsv(out);
    return out;
}

template<class Archive>
    void matrix_2d::serialize(Archive& ar, [[maybe_unused]] unsigned int version)
    {

        ar& CEREAL_NVP(rows);
    }


template void matrix_2d::serialize(cereal::BinaryInputArchive& ar, unsigned int version);
template void matrix_2d::serialize(cereal::BinaryOutputArchive& ar, unsigned int version);
template void matrix_2d::serialize(cereal::IntrospectionArchive& ar, unsigned int version);
template void matrix_2d::serialize(cereal::JSONInputArchive& ar, unsigned int version);
template void matrix_2d::serialize(cereal::JSONOutputArchive& ar, unsigned int version);
template void matrix_2d::serialize(cereal::PortableBinaryInputArchive& ar, unsigned int version);
template void matrix_2d::serialize(cereal::PortableBinaryOutputArchive& ar, unsigned int version);

std::ostream& operator<< (std::ostream& os, const matrix_2d& matrix_2d)
{
    matrix_2d.toCsv(os);
    return os;
}

bool operator== (const matrix_2d &lhs, const matrix_2d &rhs)
{
    return (  lhs.rows== rhs.rows 
    );
}

std::size_t matrix_2d::hash() const noexcept
{
    std::size_t result = 17;
    result = result * 31 + hashVector(rows);
    return result;
}

 } // namespace av
 } // namespace msg
 } // namespace vision


CEREAL_BIND_TO_ARCHIVES(av::msg::vision::matrix_2d)

 namespace av {
 namespace msg {
 namespace vision {


image_space_info::image_space_info() = default;

image_space_info::image_space_info(
    rectangle_2i bbox_arg ,
    cuboid_2i bcuboid_arg ,
    point_2d btm_cent_mean_arg ,
    matrix_2d btm_cent_cov_arg 
    ):
    bbox { std::move(bbox_arg) } // NOLINT
      ,
    bcuboid { std::move(bcuboid_arg) } // NOLINT
      ,
    btm_cent_mean { std::move(btm_cent_mean_arg) } // NOLINT
      ,
    btm_cent_cov { std::move(btm_cent_cov_arg) } // NOLINT
      
    {
}

image_space_info::image_space_info(const image_space_info&) = default;

// NOLINTNEXTLINE
image_space_info::image_space_info(image_space_info&&) = default;

image_space_info& image_space_info::operator=(const image_space_info&) = default;

// NOLINTNEXTLINE
image_space_info& image_space_info::operator=(image_space_info&&) = default;

void image_space_info::toCsv(std::ostream& out) const
{
    out.precision(16);
    out << std::boolalpha;
    out << bbox;out << ",";
    out << bcuboid;out << ",";
    out << btm_cent_mean;out << ",";
    out << btm_cent_cov;
    
}

std::stringstream image_space_info::toStringstream() const
{
    std::stringstream out;
    toCsv(out);
    return out;
}

template<class Archive>
    void image_space_info::serialize(Archive& ar, [[maybe_unused]] unsigned int version)
    {

        ar& CEREAL_NVP(bbox);
        ar& CEREAL_NVP(bcuboid);
        ar& CEREAL_NVP(btm_cent_mean);
        ar& CEREAL_NVP(btm_cent_cov);
    }


template void image_space_info::serialize(cereal::BinaryInputArchive& ar, unsigned int version);
template void image_space_info::serialize(cereal::BinaryOutputArchive& ar, unsigned int version);
template void image_space_info::serialize(cereal::IntrospectionArchive& ar, unsigned int version);
template void image_space_info::serialize(cereal::JSONInputArchive& ar, unsigned int version);
template void image_space_info::serialize(cereal::JSONOutputArchive& ar, unsigned int version);
template void image_space_info::serialize(cereal::PortableBinaryInputArchive& ar, unsigned int version);
template void image_space_info::serialize(cereal::PortableBinaryOutputArchive& ar, unsigned int version);

std::ostream& operator<< (std::ostream& os, const image_space_info& image_space_info)
{
    image_space_info.toCsv(os);
    return os;
}

bool operator== (const image_space_info &lhs, const image_space_info &rhs)
{
    return (  lhs.bbox== rhs.bbox &&
     lhs.bcuboid== rhs.bcuboid &&
     lhs.btm_cent_mean== rhs.btm_cent_mean &&
     lhs.btm_cent_cov== rhs.btm_cent_cov 
    );
}

std::size_t image_space_info::hash() const noexcept
{
    std::size_t result = 17;
    result = result * 31 + std::hash<rectangle_2i>()(bbox);
    result = result * 31 + std::hash<cuboid_2i>()(bcuboid);
    result = result * 31 + std::hash<point_2d>()(btm_cent_mean);
    result = result * 31 + std::hash<matrix_2d>()(btm_cent_cov);
    return result;
}

 } // namespace av
 } // namespace msg
 } // namespace vision


CEREAL_BIND_TO_ARCHIVES(av::msg::vision::image_space_info)

 namespace av {
 namespace msg {
 namespace vision {


camera_space_info::camera_space_info() = default;

camera_space_info::camera_space_info(
    Point center_arg ,
    Point size_arg ,
    Point orientation_arg ,
    Point velocity_arg 
    ):
    center { std::move(center_arg) } // NOLINT
      ,
    size { std::move(size_arg) } // NOLINT
      ,
    orientation { std::move(orientation_arg) } // NOLINT
      ,
    velocity { std::move(velocity_arg) } // NOLINT
      
    {
}

camera_space_info::camera_space_info(const camera_space_info&) = default;

// NOLINTNEXTLINE
camera_space_info::camera_space_info(camera_space_info&&) = default;

camera_space_info& camera_space_info::operator=(const camera_space_info&) = default;

// NOLINTNEXTLINE
camera_space_info& camera_space_info::operator=(camera_space_info&&) = default;

void camera_space_info::toCsv(std::ostream& out) const
{
    out.precision(16);
    out << std::boolalpha;
    out << center;out << ",";
    out << size;out << ",";
    out << orientation;out << ",";
    out << velocity;
    
}

std::stringstream camera_space_info::toStringstream() const
{
    std::stringstream out;
    toCsv(out);
    return out;
}

template<class Archive>
    void camera_space_info::serialize(Archive& ar, unsigned int version)
{
  ar& CEREAL_NVP(center);
  ar& CEREAL_NVP(size);
  ar& CEREAL_NVP(orientation);
  if (version > 1) 
  {
     ar& CEREAL_NVP(velocity);
  }
}



template void camera_space_info::serialize(cereal::BinaryInputArchive& ar, unsigned int version);
template void camera_space_info::serialize(cereal::BinaryOutputArchive& ar, unsigned int version);
template void camera_space_info::serialize(cereal::IntrospectionArchive& ar, unsigned int version);
template void camera_space_info::serialize(cereal::JSONInputArchive& ar, unsigned int version);
template void camera_space_info::serialize(cereal::JSONOutputArchive& ar, unsigned int version);
template void camera_space_info::serialize(cereal::PortableBinaryInputArchive& ar, unsigned int version);
template void camera_space_info::serialize(cereal::PortableBinaryOutputArchive& ar, unsigned int version);

std::ostream& operator<< (std::ostream& os, const camera_space_info& camera_space_info)
{
    camera_space_info.toCsv(os);
    return os;
}

bool operator== (const camera_space_info &lhs, const camera_space_info &rhs)
{
    return (  lhs.center== rhs.center &&
     lhs.size== rhs.size &&
     lhs.orientation== rhs.orientation &&
     lhs.velocity== rhs.velocity 
    );
}

std::size_t camera_space_info::hash() const noexcept
{
    std::size_t result = 17;
    result = result * 31 + std::hash<Point>()(center);
    result = result * 31 + std::hash<Point>()(size);
    result = result * 31 + std::hash<Point>()(orientation);
    result = result * 31 + std::hash<Point>()(velocity);
    return result;
}

 } // namespace av
 } // namespace msg
 } // namespace vision


CEREAL_BIND_TO_ARCHIVES(av::msg::vision::camera_space_info)

 namespace av {
 namespace msg {
 namespace vision {


baselink_space_info::baselink_space_info() = default;

baselink_space_info::baselink_space_info(
    bool ipm_applicable_arg ,
    double yaw_arg ,
    double length_arg ,
    double width_arg ,
    double height_arg ,
    Point pos_mean_arg ,
    matrix_2d pos_cov_arg 
    ):
    ipm_applicable { std::move(ipm_applicable_arg) } // NOLINT
      ,
    yaw { std::move(yaw_arg) } // NOLINT
      ,
    length { std::move(length_arg) } // NOLINT
      ,
    width { std::move(width_arg) } // NOLINT
      ,
    height { std::move(height_arg) } // NOLINT
      ,
    pos_mean { std::move(pos_mean_arg) } // NOLINT
      ,
    pos_cov { std::move(pos_cov_arg) } // NOLINT
      
    {
}

baselink_space_info::baselink_space_info(const baselink_space_info&) = default;

// NOLINTNEXTLINE
baselink_space_info::baselink_space_info(baselink_space_info&&) = default;

baselink_space_info& baselink_space_info::operator=(const baselink_space_info&) = default;

// NOLINTNEXTLINE
baselink_space_info& baselink_space_info::operator=(baselink_space_info&&) = default;

void baselink_space_info::toCsv(std::ostream& out) const
{
    out.precision(16);
    out << std::boolalpha;
    out << ipm_applicable;out << ",";
    out << yaw;out << ",";
    out << length;out << ",";
    out << width;out << ",";
    out << height;out << ",";
    out << pos_mean;out << ",";
    out << pos_cov;
    
}

std::stringstream baselink_space_info::toStringstream() const
{
    std::stringstream out;
    toCsv(out);
    return out;
}

template<class Archive>
    void baselink_space_info::serialize(Archive& ar, [[maybe_unused]] unsigned int version)
    {

        ar& CEREAL_NVP(ipm_applicable);
        ar& CEREAL_NVP(yaw);
        ar& CEREAL_NVP(length);
        ar& CEREAL_NVP(width);
        ar& CEREAL_NVP(height);
        ar& CEREAL_NVP(pos_mean);
        ar& CEREAL_NVP(pos_cov);
    }


template void baselink_space_info::serialize(cereal::BinaryInputArchive& ar, unsigned int version);
template void baselink_space_info::serialize(cereal::BinaryOutputArchive& ar, unsigned int version);
template void baselink_space_info::serialize(cereal::IntrospectionArchive& ar, unsigned int version);
template void baselink_space_info::serialize(cereal::JSONInputArchive& ar, unsigned int version);
template void baselink_space_info::serialize(cereal::JSONOutputArchive& ar, unsigned int version);
template void baselink_space_info::serialize(cereal::PortableBinaryInputArchive& ar, unsigned int version);
template void baselink_space_info::serialize(cereal::PortableBinaryOutputArchive& ar, unsigned int version);

std::ostream& operator<< (std::ostream& os, const baselink_space_info& baselink_space_info)
{
    baselink_space_info.toCsv(os);
    return os;
}

bool operator== (const baselink_space_info &lhs, const baselink_space_info &rhs)
{
    return (  lhs.ipm_applicable== rhs.ipm_applicable &&
     lhs.yaw== rhs.yaw &&
     lhs.length== rhs.length &&
     lhs.width== rhs.width &&
     lhs.height== rhs.height &&
     lhs.pos_mean== rhs.pos_mean &&
     lhs.pos_cov== rhs.pos_cov 
    );
}

std::size_t baselink_space_info::hash() const noexcept
{
    std::size_t result = 17;
    result = result * 31 + std::hash<bool>()(ipm_applicable);
    result = result * 31 + std::hash<double>()(yaw);
    result = result * 31 + std::hash<double>()(length);
    result = result * 31 + std::hash<double>()(width);
    result = result * 31 + std::hash<double>()(height);
    result = result * 31 + std::hash<Point>()(pos_mean);
    result = result * 31 + std::hash<matrix_2d>()(pos_cov);
    return result;
}

 } // namespace av
 } // namespace msg
 } // namespace vision


CEREAL_BIND_TO_ARCHIVES(av::msg::vision::baselink_space_info)

 namespace av {
 namespace msg {
 namespace vision {


object_attributes::object_attributes() = default;

object_attributes::object_attributes(
    RIDER_STATUS rider_status_arg ,
    VEHICLE_STATUS vehicle_status_arg 
    ):
    rider_status { std::move(rider_status_arg) } // NOLINT
      ,
    vehicle_status { std::move(vehicle_status_arg) } // NOLINT
      
    {
}

object_attributes::object_attributes(const object_attributes&) = default;

// NOLINTNEXTLINE
object_attributes::object_attributes(object_attributes&&) = default;

object_attributes& object_attributes::operator=(const object_attributes&) = default;

// NOLINTNEXTLINE
object_attributes& object_attributes::operator=(object_attributes&&) = default;

void object_attributes::toCsv(std::ostream& out) const
{
    out.precision(16);
    out << std::boolalpha;
    out << static_cast<uint32_t>(rider_status);out << ",";
    out << static_cast<uint32_t>(vehicle_status);
    
}

std::stringstream object_attributes::toStringstream() const
{
    std::stringstream out;
    toCsv(out);
    return out;
}

template<class Archive>
    void object_attributes::serialize(Archive& ar, [[maybe_unused]] unsigned int version)
    {

        ar& CEREAL_NVP(rider_status);
        ar& CEREAL_NVP(vehicle_status);
    }


template void object_attributes::serialize(cereal::BinaryInputArchive& ar, unsigned int version);
template void object_attributes::serialize(cereal::BinaryOutputArchive& ar, unsigned int version);
template void object_attributes::serialize(cereal::IntrospectionArchive& ar, unsigned int version);
template void object_attributes::serialize(cereal::JSONInputArchive& ar, unsigned int version);
template void object_attributes::serialize(cereal::JSONOutputArchive& ar, unsigned int version);
template void object_attributes::serialize(cereal::PortableBinaryInputArchive& ar, unsigned int version);
template void object_attributes::serialize(cereal::PortableBinaryOutputArchive& ar, unsigned int version);

std::ostream& operator<< (std::ostream& os, const object_attributes& object_attributes)
{
    object_attributes.toCsv(os);
    return os;
}

bool operator== (const object_attributes &lhs, const object_attributes &rhs)
{
    return (  lhs.rider_status== rhs.rider_status &&
     lhs.vehicle_status== rhs.vehicle_status 
    );
}

std::size_t object_attributes::hash() const noexcept
{
    std::size_t result = 17;
    result = result * 31 + std::hash<RIDER_STATUS>()(rider_status);
    result = result * 31 + std::hash<VEHICLE_STATUS>()(vehicle_status);
    return result;
}

 } // namespace av
 } // namespace msg
 } // namespace vision


CEREAL_BIND_TO_ARCHIVES(av::msg::vision::object_attributes)

 namespace av {
 namespace msg {
 namespace vision {


vector_1f::vector_1f() = default;

vector_1f::vector_1f(
    std::vector<float> data_arg 
    ):
    data { std::move(data_arg) } // NOLINT
      
    {
}

vector_1f::vector_1f(const vector_1f&) = default;

// NOLINTNEXTLINE
vector_1f::vector_1f(vector_1f&&) = default;

vector_1f& vector_1f::operator=(const vector_1f&) = default;

// NOLINTNEXTLINE
vector_1f& vector_1f::operator=(vector_1f&&) = default;

void vector_1f::toCsv(std::ostream& out) const
{
    out.precision(16);
    out << std::boolalpha;
    // Enclose sequence in double quotations and curly brackets to keep CSV line length uniform
        
        out << "\"{{";
        
        if (!data.empty())
        {
            std::copy(data.begin(), data.end() - 1, std::ostream_iterator<float>(out, "},{"));
            std::copy(data.end()-1, data.end(), std::ostream_iterator<float>(out, ""));
        }
        out << "}}\"";
    
}

std::stringstream vector_1f::toStringstream() const
{
    std::stringstream out;
    toCsv(out);
    return out;
}

template<class Archive>
    void vector_1f::serialize(Archive& ar, [[maybe_unused]] unsigned int version)
    {

        ar& CEREAL_NVP(data);
    }


template void vector_1f::serialize(cereal::BinaryInputArchive& ar, unsigned int version);
template void vector_1f::serialize(cereal::BinaryOutputArchive& ar, unsigned int version);
template void vector_1f::serialize(cereal::IntrospectionArchive& ar, unsigned int version);
template void vector_1f::serialize(cereal::JSONInputArchive& ar, unsigned int version);
template void vector_1f::serialize(cereal::JSONOutputArchive& ar, unsigned int version);
template void vector_1f::serialize(cereal::PortableBinaryInputArchive& ar, unsigned int version);
template void vector_1f::serialize(cereal::PortableBinaryOutputArchive& ar, unsigned int version);

std::ostream& operator<< (std::ostream& os, const vector_1f& vector_1f)
{
    vector_1f.toCsv(os);
    return os;
}

bool operator== (const vector_1f &lhs, const vector_1f &rhs)
{
    return (  lhs.data== rhs.data 
    );
}

std::size_t vector_1f::hash() const noexcept
{
    std::size_t result = 17;
    result = result * 31 + hashVector(data);
    return result;
}

 } // namespace av
 } // namespace msg
 } // namespace vision


CEREAL_BIND_TO_ARCHIVES(av::msg::vision::vector_1f)

 namespace av {
 namespace msg {
 namespace vision {


visual_feature_mapping::visual_feature_mapping() = default;

visual_feature_mapping::visual_feature_mapping(
    VISUAL_FEATURE_TYPE feature_type_arg ,
    std::vector<float> features_arg 
    ):
    feature_type { std::move(feature_type_arg) } // NOLINT
      ,
    features { std::move(features_arg) } // NOLINT
      
    {
}

visual_feature_mapping::visual_feature_mapping(const visual_feature_mapping&) = default;

// NOLINTNEXTLINE
visual_feature_mapping::visual_feature_mapping(visual_feature_mapping&&) = default;

visual_feature_mapping& visual_feature_mapping::operator=(const visual_feature_mapping&) = default;

// NOLINTNEXTLINE
visual_feature_mapping& visual_feature_mapping::operator=(visual_feature_mapping&&) = default;

void visual_feature_mapping::toCsv(std::ostream& out) const
{
    out.precision(16);
    out << std::boolalpha;
    out << static_cast<uint32_t>(feature_type);out << ",";
    // Enclose sequence in double quotations and curly brackets to keep CSV line length uniform
        
        out << "\"{{";
        
        if (!features.empty())
        {
            std::copy(features.begin(), features.end() - 1, std::ostream_iterator<float>(out, "},{"));
            std::copy(features.end()-1, features.end(), std::ostream_iterator<float>(out, ""));
        }
        out << "}}\"";
    
}

std::stringstream visual_feature_mapping::toStringstream() const
{
    std::stringstream out;
    toCsv(out);
    return out;
}

template<class Archive>
    void visual_feature_mapping::serialize(Archive& ar, [[maybe_unused]] unsigned int version)
    {

        ar& CEREAL_NVP(feature_type);
        ar& CEREAL_NVP(features);
    }


template void visual_feature_mapping::serialize(cereal::BinaryInputArchive& ar, unsigned int version);
template void visual_feature_mapping::serialize(cereal::BinaryOutputArchive& ar, unsigned int version);
template void visual_feature_mapping::serialize(cereal::IntrospectionArchive& ar, unsigned int version);
template void visual_feature_mapping::serialize(cereal::JSONInputArchive& ar, unsigned int version);
template void visual_feature_mapping::serialize(cereal::JSONOutputArchive& ar, unsigned int version);
template void visual_feature_mapping::serialize(cereal::PortableBinaryInputArchive& ar, unsigned int version);
template void visual_feature_mapping::serialize(cereal::PortableBinaryOutputArchive& ar, unsigned int version);

std::ostream& operator<< (std::ostream& os, const visual_feature_mapping& visual_feature_mapping)
{
    visual_feature_mapping.toCsv(os);
    return os;
}

bool operator== (const visual_feature_mapping &lhs, const visual_feature_mapping &rhs)
{
    return (  lhs.feature_type== rhs.feature_type &&
     lhs.features== rhs.features 
    );
}

std::size_t visual_feature_mapping::hash() const noexcept
{
    std::size_t result = 17;
    result = result * 31 + std::hash<VISUAL_FEATURE_TYPE>()(feature_type);
    result = result * 31 + hashVector(features);
    return result;
}

 } // namespace av
 } // namespace msg
 } // namespace vision


CEREAL_BIND_TO_ARCHIVES(av::msg::vision::visual_feature_mapping)

 namespace av {
 namespace msg {
 namespace vision {


vision_object::vision_object() = default;

vision_object::vision_object(
    OBJECT_TYPE object_type_arg ,
    double confidence_arg ,
    std::vector<double> probabilities_arg ,
    image_space_info img_space_info_arg ,
    camera_space_info cam_space_info_arg ,
    baselink_space_info bl_space_info_arg ,
    object_attributes object_attrs_arg ,
    std::vector<visual_feature_mapping> visual_features_arg ,
    bool camera_space_info_applicable_arg 
    ):
    object_type { std::move(object_type_arg) } // NOLINT
      ,
    confidence { std::move(confidence_arg) } // NOLINT
      ,
    probabilities { std::move(probabilities_arg) } // NOLINT
      ,
    img_space_info { std::move(img_space_info_arg) } // NOLINT
      ,
    cam_space_info { std::move(cam_space_info_arg) } // NOLINT
      ,
    bl_space_info { std::move(bl_space_info_arg) } // NOLINT
      ,
    object_attrs { std::move(object_attrs_arg) } // NOLINT
      ,
    visual_features { std::move(visual_features_arg) } // NOLINT
      ,
    camera_space_info_applicable { std::move(camera_space_info_applicable_arg) } // NOLINT
      
    {
}

vision_object::vision_object(const vision_object&) = default;

// NOLINTNEXTLINE
vision_object::vision_object(vision_object&&) = default;

vision_object& vision_object::operator=(const vision_object&) = default;

// NOLINTNEXTLINE
vision_object& vision_object::operator=(vision_object&&) = default;

void vision_object::toCsv(std::ostream& out) const
{
    out.precision(16);
    out << std::boolalpha;
    out << static_cast<uint32_t>(object_type);out << ",";
    out << confidence;out << ",";
    // Enclose sequence in double quotations and curly brackets to keep CSV line length uniform
        
        out << "\"{{";
        
        if (!probabilities.empty())
        {
            std::copy(probabilities.begin(), probabilities.end() - 1, std::ostream_iterator<double>(out, "},{"));
            std::copy(probabilities.end()-1, probabilities.end(), std::ostream_iterator<double>(out, ""));
        }
        out << "}}\"";out << ",";
    out << img_space_info;out << ",";
    out << cam_space_info;out << ",";
    out << bl_space_info;out << ",";
    out << object_attrs;out << ",";
    // Enclose sequence in double quotations and curly brackets to keep CSV line length uniform
        
        out << "\"{{";
        
        if (!visual_features.empty())
        {
            std::copy(visual_features.begin(), visual_features.end() - 1, std::ostream_iterator<visual_feature_mapping>(out, "},{"));
            std::copy(visual_features.end()-1, visual_features.end(), std::ostream_iterator<visual_feature_mapping>(out, ""));
        }
        out << "}}\"";out << ",";
    out << camera_space_info_applicable;
    
}

std::stringstream vision_object::toStringstream() const
{
    std::stringstream out;
    toCsv(out);
    return out;
}

template<class Archive>
    void vision_object::serialize(Archive& ar, [[maybe_unused]] unsigned int version)
    {

        ar& CEREAL_NVP(object_type);
        ar& CEREAL_NVP(confidence);
        ar& CEREAL_NVP(probabilities);
        ar& CEREAL_NVP(img_space_info);
        ar& CEREAL_NVP(cam_space_info);
        ar& CEREAL_NVP(bl_space_info);
        ar& CEREAL_NVP(object_attrs);
        ar& CEREAL_NVP(visual_features);
        ar& CEREAL_NVP(camera_space_info_applicable);
    }


template void vision_object::serialize(cereal::BinaryInputArchive& ar, unsigned int version);
template void vision_object::serialize(cereal::BinaryOutputArchive& ar, unsigned int version);
template void vision_object::serialize(cereal::IntrospectionArchive& ar, unsigned int version);
template void vision_object::serialize(cereal::JSONInputArchive& ar, unsigned int version);
template void vision_object::serialize(cereal::JSONOutputArchive& ar, unsigned int version);
template void vision_object::serialize(cereal::PortableBinaryInputArchive& ar, unsigned int version);
template void vision_object::serialize(cereal::PortableBinaryOutputArchive& ar, unsigned int version);

std::ostream& operator<< (std::ostream& os, const vision_object& vision_object)
{
    vision_object.toCsv(os);
    return os;
}

bool operator== (const vision_object &lhs, const vision_object &rhs)
{
    return (  lhs.object_type== rhs.object_type &&
     lhs.confidence== rhs.confidence &&
     lhs.probabilities== rhs.probabilities &&
     lhs.img_space_info== rhs.img_space_info &&
     lhs.cam_space_info== rhs.cam_space_info &&
     lhs.bl_space_info== rhs.bl_space_info &&
     lhs.object_attrs== rhs.object_attrs &&
     lhs.visual_features== rhs.visual_features &&
     lhs.camera_space_info_applicable== rhs.camera_space_info_applicable 
    );
}

std::size_t vision_object::hash() const noexcept
{
    std::size_t result = 17;
    result = result * 31 + std::hash<OBJECT_TYPE>()(object_type);
    result = result * 31 + std::hash<double>()(confidence);
    result = result * 31 + hashVector(probabilities);
    result = result * 31 + std::hash<image_space_info>()(img_space_info);
    result = result * 31 + std::hash<camera_space_info>()(cam_space_info);
    result = result * 31 + std::hash<baselink_space_info>()(bl_space_info);
    result = result * 31 + std::hash<object_attributes>()(object_attrs);
    result = result * 31 + hashVector(visual_features);
    result = result * 31 + std::hash<bool>()(camera_space_info_applicable);
    return result;
}

 } // namespace av
 } // namespace msg
 } // namespace vision


CEREAL_BIND_TO_ARCHIVES(av::msg::vision::vision_object)




 namespace av {
 namespace msg {
 namespace vision {


VisionMessage::VisionMessage() = default;

VisionMessage::VisionMessage(
    int64_t timestamp_arg ,
    int32_t num_object_types_arg ,
    int32_t num_segment_types_arg ,
    std::string frame_id_arg ,
    DEVICE_TYPE device_type_arg ,
    int16_t device_version_arg ,
    std::vector<vision_object> vision_objects_arg ,
    roi seg_mask_roi_arg ,
    CompressedImageMessage seg_mask_arg ,
    std::string isn_model_arg ,
    av::camera::H264FrameMeta h264frame_metadata_arg 
    ):
    timestamp { std::move(timestamp_arg) } // NOLINT
      ,num_object_types { std::move(num_object_types_arg) } // NOLINT
      ,num_segment_types { std::move(num_segment_types_arg) } // NOLINT
      ,frame_id { std::move(frame_id_arg) } // NOLINT
      ,device_type { std::move(device_type_arg) } // NOLINT
      ,device_version { std::move(device_version_arg) } // NOLINT
      ,vision_objects { std::move(vision_objects_arg) } // NOLINT
      ,seg_mask_roi { std::move(seg_mask_roi_arg) } // NOLINT
      ,seg_mask { std::move(seg_mask_arg) } // NOLINT
      ,isn_model { std::move(isn_model_arg) } // NOLINT
      ,h264frame_metadata { std::move(h264frame_metadata_arg) } // NOLINT
      {
}

VisionMessage::VisionMessage(const VisionMessage&) = default;

// NOLINTNEXTLINE
VisionMessage::VisionMessage(VisionMessage&&) = default;

VisionMessage& VisionMessage::operator=(const VisionMessage&) = default;

// NOLINTNEXTLINE
VisionMessage& VisionMessage::operator=(VisionMessage&&) = default;

void VisionMessage::toCsv(std::ostream& out) const
{
    out.precision(16);
    out << std::boolalpha;
    out << timestamp;out << ",";
    out << num_object_types;out << ",";
    out << num_segment_types;out << ",";
    out << "\"";
        out << frame_id;
        out << "\"";out << ",";
    out << static_cast<uint32_t>(device_type);out << ",";
    out << device_version;out << ",";
    // Enclose sequence in double quotations and curly brackets to keep CSV line length uniform
        
        out << "\"{{";
        
        if (!vision_objects.empty())
        {
            std::copy(vision_objects.begin(), vision_objects.end() - 1, std::ostream_iterator<vision_object>(out, "},{"));
            std::copy(vision_objects.end()-1, vision_objects.end(), std::ostream_iterator<vision_object>(out, ""));
        }
        out << "}}\"";out << ",";
    out << seg_mask_roi;out << ",";
    out << seg_mask;out << ",";
    out << "\"";
        out << isn_model;
        out << "\"";out << ",";
    out << h264frame_metadata;
    
}

std::stringstream VisionMessage::toStringstream() const
{
    std::stringstream out;
    toCsv(out);
    return out;
}

template<class Archive>
    void VisionMessage::serialize(Archive& ar, [[maybe_unused]] unsigned int version)
    {

        ar& CEREAL_NVP(timestamp);
        ar& CEREAL_NVP(num_object_types);
        ar& CEREAL_NVP(num_segment_types);
        ar& CEREAL_NVP(frame_id);
        ar& CEREAL_NVP(device_type);
        ar& CEREAL_NVP(device_version);
        ar& CEREAL_NVP(vision_objects);
        ar& CEREAL_NVP(seg_mask_roi);
        ar& CEREAL_NVP(seg_mask);
        ar& CEREAL_NVP(isn_model);
        ar& CEREAL_NVP(h264frame_metadata);
    }


template void VisionMessage::serialize(cereal::BinaryInputArchive& ar, unsigned int version);
template void VisionMessage::serialize(cereal::BinaryOutputArchive& ar, unsigned int version);
template void VisionMessage::serialize(cereal::IntrospectionArchive& ar, unsigned int version);
template void VisionMessage::serialize(cereal::JSONInputArchive& ar, unsigned int version);
template void VisionMessage::serialize(cereal::JSONOutputArchive& ar, unsigned int version);
template void VisionMessage::serialize(cereal::PortableBinaryInputArchive& ar, unsigned int version);
template void VisionMessage::serialize(cereal::PortableBinaryOutputArchive& ar, unsigned int version);

std::ostream& operator<< (std::ostream& os, const VisionMessage& visionmessage)
{
    visionmessage.toCsv(os);
    return os;
}

bool operator== (const VisionMessage &lhs, const VisionMessage &rhs)
{
    return (  lhs.timestamp== rhs.timestamp &&
     lhs.num_object_types== rhs.num_object_types &&
     lhs.num_segment_types== rhs.num_segment_types &&
     lhs.frame_id== rhs.frame_id &&
     lhs.device_type== rhs.device_type &&
     lhs.device_version== rhs.device_version &&
     lhs.vision_objects== rhs.vision_objects &&
     lhs.seg_mask_roi== rhs.seg_mask_roi &&
     lhs.seg_mask== rhs.seg_mask &&
     lhs.isn_model== rhs.isn_model &&
     lhs.h264frame_metadata== rhs.h264frame_metadata 
    );
}

std::size_t VisionMessage::hash() const noexcept
{
    std::size_t result = 17;
    result = result * 31 + std::hash<int64_t>()(timestamp);
    result = result * 31 + std::hash<int32_t>()(num_object_types);
    result = result * 31 + std::hash<int32_t>()(num_segment_types);
    result = result * 31 + std::hash<std::string>()(frame_id);
    result = result * 31 + std::hash<DEVICE_TYPE>()(device_type);
    result = result * 31 + std::hash<int16_t>()(device_version);
    result = result * 31 + hashVector(vision_objects);
    result = result * 31 + std::hash<roi>()(seg_mask_roi);
    result = result * 31 + std::hash<CompressedImageMessage>()(seg_mask);
    result = result * 31 + std::hash<std::string>()(isn_model);
    result = result * 31 + std::hash<av::camera::H264FrameMeta>()(h264frame_metadata);
    return result;
}

 } // namespace av
 } // namespace msg
 } // namespace vision


CEREAL_BIND_TO_ARCHIVES(av::msg::vision::VisionMessage)

