#include <interfaces/VpbChipHealth/VpbChipHealth.h>

#include <iterator>
#include <type_traits>

#include <cereal/cereal.hpp>
#include <cereal/archives/json.hpp>
#include <cereal/archives/binary.hpp>

#include <cereal/types/map.hpp>
#include <cereal/types/vector.hpp>
#include <cereal/types/array.hpp>
#include <cereal/types/string.hpp>
#include <cereal/types/complex.hpp>

#include <ODatum/IntrospectionArchive.h>
#include <ODatum/Hash.h>





 namespace av {
 namespace msg {
 namespace camera {


VpbChipHealth::VpbChipHealth() = default;

VpbChipHealth::VpbChipHealth(
    float rate_sec_arg ,
    std::string chipId_arg ,
    TimeSyncStatus timeSyncStatus_arg ,
    std::array<float,10> thermalStatus_degreesC_arg ,
    float rtosDspFrequency_hz_arg ,
    float rtosCpuUsage_percentage_arg ,
    float rtosCachedRamUsage_percentage_arg ,
    float rtosNonCachedRamUsage_percentage_arg ,
    std::array<float,3> linuxCpuUsage_percentage_arg ,
    float linuxRamUsage_percentage_arg 
    ):
    rate_sec { std::move(rate_sec_arg) } // NOLINT
      ,chipId { std::move(chipId_arg) } // NOLINT
      ,timeSyncStatus { std::move(timeSyncStatus_arg) } // NOLINT
      ,thermalStatus_degreesC { std::move(thermalStatus_degreesC_arg) } // NOLINT
      ,rtosDspFrequency_hz { std::move(rtosDspFrequency_hz_arg) } // NOLINT
      ,rtosCpuUsage_percentage { std::move(rtosCpuUsage_percentage_arg) } // NOLINT
      ,rtosCachedRamUsage_percentage { std::move(rtosCachedRamUsage_percentage_arg) } // NOLINT
      ,rtosNonCachedRamUsage_percentage { std::move(rtosNonCachedRamUsage_percentage_arg) } // NOLINT
      ,linuxCpuUsage_percentage { std::move(linuxCpuUsage_percentage_arg) } // NOLINT
      ,linuxRamUsage_percentage { std::move(linuxRamUsage_percentage_arg) } // NOLINT
      {
}

VpbChipHealth::VpbChipHealth(const VpbChipHealth&) = default;

// NOLINTNEXTLINE
VpbChipHealth::VpbChipHealth(VpbChipHealth&&) = default;

VpbChipHealth& VpbChipHealth::operator=(const VpbChipHealth&) = default;

// NOLINTNEXTLINE
VpbChipHealth& VpbChipHealth::operator=(VpbChipHealth&&) = default;

void VpbChipHealth::toCsv(std::ostream& out) const
{
    out.precision(16);
    out << std::boolalpha;
    out << rate_sec;out << ",";
    out << "\"";
        out << chipId;
        out << "\"";out << ",";
    out << static_cast<uint32_t>(timeSyncStatus);out << ",";
    // Enclose array in double quotations and curly brackets to keep CSV line length uniform
        
        out << "\"{{";
        
        if (!thermalStatus_degreesC.empty())
        {
            std::copy(thermalStatus_degreesC.begin(), thermalStatus_degreesC.end() - 1, std::ostream_iterator<float>(out, "},{"));
            out << thermalStatus_degreesC.back();
        }
        out << "}}\"";out << ",";
    out << rtosDspFrequency_hz;out << ",";
    out << rtosCpuUsage_percentage;out << ",";
    out << rtosCachedRamUsage_percentage;out << ",";
    out << rtosNonCachedRamUsage_percentage;out << ",";
    // Enclose array in double quotations and curly brackets to keep CSV line length uniform
        
        out << "\"{{";
        
        if (!linuxCpuUsage_percentage.empty())
        {
            std::copy(linuxCpuUsage_percentage.begin(), linuxCpuUsage_percentage.end() - 1, std::ostream_iterator<float>(out, "},{"));
            out << linuxCpuUsage_percentage.back();
        }
        out << "}}\"";out << ",";
    out << linuxRamUsage_percentage;
    
}

std::stringstream VpbChipHealth::toStringstream() const
{
    std::stringstream out;
    toCsv(out);
    return out;
}

template<class Archive>
    void VpbChipHealth::serialize(Archive& ar, [[maybe_unused]] unsigned int version)
    {

        ar& CEREAL_NVP(rate_sec);
        ar& CEREAL_NVP(chipId);
        ar& CEREAL_NVP(timeSyncStatus);
        ar& CEREAL_NVP(thermalStatus_degreesC);
        ar& CEREAL_NVP(rtosDspFrequency_hz);
        ar& CEREAL_NVP(rtosCpuUsage_percentage);
        ar& CEREAL_NVP(rtosCachedRamUsage_percentage);
        ar& CEREAL_NVP(rtosNonCachedRamUsage_percentage);
        ar& CEREAL_NVP(linuxCpuUsage_percentage);
        ar& CEREAL_NVP(linuxRamUsage_percentage);
    }


template void VpbChipHealth::serialize(cereal::BinaryInputArchive& ar, unsigned int version);
template void VpbChipHealth::serialize(cereal::BinaryOutputArchive& ar, unsigned int version);
template void VpbChipHealth::serialize(cereal::IntrospectionArchive& ar, unsigned int version);
template void VpbChipHealth::serialize(cereal::JSONInputArchive& ar, unsigned int version);
template void VpbChipHealth::serialize(cereal::JSONOutputArchive& ar, unsigned int version);
template void VpbChipHealth::serialize(cereal::PortableBinaryInputArchive& ar, unsigned int version);
template void VpbChipHealth::serialize(cereal::PortableBinaryOutputArchive& ar, unsigned int version);

std::ostream& operator<< (std::ostream& os, const VpbChipHealth& vpbchiphealth)
{
    vpbchiphealth.toCsv(os);
    return os;
}

bool operator== (const VpbChipHealth &lhs, const VpbChipHealth &rhs)
{
    return (  lhs.rate_sec== rhs.rate_sec &&
     lhs.chipId== rhs.chipId &&
     lhs.timeSyncStatus== rhs.timeSyncStatus &&
     lhs.thermalStatus_degreesC== rhs.thermalStatus_degreesC &&
     lhs.rtosDspFrequency_hz== rhs.rtosDspFrequency_hz &&
     lhs.rtosCpuUsage_percentage== rhs.rtosCpuUsage_percentage &&
     lhs.rtosCachedRamUsage_percentage== rhs.rtosCachedRamUsage_percentage &&
     lhs.rtosNonCachedRamUsage_percentage== rhs.rtosNonCachedRamUsage_percentage &&
     lhs.linuxCpuUsage_percentage== rhs.linuxCpuUsage_percentage &&
     lhs.linuxRamUsage_percentage== rhs.linuxRamUsage_percentage 
    );
}

std::size_t VpbChipHealth::hash() const noexcept
{
    std::size_t result = 17;
    result = result * 31 + std::hash<float>()(rate_sec);
    result = result * 31 + std::hash<std::string>()(chipId);
    result = result * 31 + std::hash<TimeSyncStatus>()(timeSyncStatus);
    result = result * 31 + hashVector(thermalStatus_degreesC);
    result = result * 31 + std::hash<float>()(rtosDspFrequency_hz);
    result = result * 31 + std::hash<float>()(rtosCpuUsage_percentage);
    result = result * 31 + std::hash<float>()(rtosCachedRamUsage_percentage);
    result = result * 31 + std::hash<float>()(rtosNonCachedRamUsage_percentage);
    result = result * 31 + hashVector(linuxCpuUsage_percentage);
    result = result * 31 + std::hash<float>()(linuxRamUsage_percentage);
    return result;
}

 } // namespace av
 } // namespace msg
 } // namespace camera


CEREAL_BIND_TO_ARCHIVES(av::msg::camera::VpbChipHealth)

