#include <interfaces/VpbIsnHealth/VpbIsnHealth.h>

#include <iterator>
#include <type_traits>

#include <cereal/cereal.hpp>
#include <cereal/archives/json.hpp>
#include <cereal/archives/binary.hpp>

#include <cereal/types/map.hpp>
#include <cereal/types/vector.hpp>
#include <cereal/types/array.hpp>
#include <cereal/types/string.hpp>
#include <cereal/types/complex.hpp>

#include <ODatum/IntrospectionArchive.h>
#include <ODatum/Hash.h>





 namespace av {
 namespace msg {
 namespace camera {


VpbIsnHealth::VpbIsnHealth() = default;

VpbIsnHealth::VpbIsnHealth(
    float publishRate_hz_arg ,
    std::string cameraId_arg ,
    std::string isnModelVersion_arg ,
    IsnStatus isnStatus_arg ,
    uint64_t capPts_arg ,
    uint64_t modelInferenceRuntime_ms_arg ,
    uint64_t picInfoRuntime_ms_arg ,
    uint64_t box2dGenerationRuntime_ms_arg ,
    uint64_t box3dGenerationRuntime_ms_arg ,
    uint64_t segmaskProcessRuntime_ms_arg ,
    uint64_t similarityEmbeddingRuntime_ms_arg ,
    uint64_t pipelineLatency_ms_arg 
    ):
    publishRate_hz { std::move(publishRate_hz_arg) } // NOLINT
      ,cameraId { std::move(cameraId_arg) } // NOLINT
      ,isnModelVersion { std::move(isnModelVersion_arg) } // NOLINT
      ,isnStatus { std::move(isnStatus_arg) } // NOLINT
      ,capPts { std::move(capPts_arg) } // NOLINT
      ,modelInferenceRuntime_ms { std::move(modelInferenceRuntime_ms_arg) } // NOLINT
      ,picInfoRuntime_ms { std::move(picInfoRuntime_ms_arg) } // NOLINT
      ,box2dGenerationRuntime_ms { std::move(box2dGenerationRuntime_ms_arg) } // NOLINT
      ,box3dGenerationRuntime_ms { std::move(box3dGenerationRuntime_ms_arg) } // NOLINT
      ,segmaskProcessRuntime_ms { std::move(segmaskProcessRuntime_ms_arg) } // NOLINT
      ,similarityEmbeddingRuntime_ms { std::move(similarityEmbeddingRuntime_ms_arg) } // NOLINT
      ,pipelineLatency_ms { std::move(pipelineLatency_ms_arg) } // NOLINT
      {
}

VpbIsnHealth::VpbIsnHealth(const VpbIsnHealth&) = default;

// NOLINTNEXTLINE
VpbIsnHealth::VpbIsnHealth(VpbIsnHealth&&) = default;

VpbIsnHealth& VpbIsnHealth::operator=(const VpbIsnHealth&) = default;

// NOLINTNEXTLINE
VpbIsnHealth& VpbIsnHealth::operator=(VpbIsnHealth&&) = default;

void VpbIsnHealth::toCsv(std::ostream& out) const
{
    out.precision(16);
    out << std::boolalpha;
    out << publishRate_hz;out << ",";
    out << "\"";
        out << cameraId;
        out << "\"";out << ",";
    out << "\"";
        out << isnModelVersion;
        out << "\"";out << ",";
    out << static_cast<uint32_t>(isnStatus);out << ",";
    out << capPts;out << ",";
    out << modelInferenceRuntime_ms;out << ",";
    out << picInfoRuntime_ms;out << ",";
    out << box2dGenerationRuntime_ms;out << ",";
    out << box3dGenerationRuntime_ms;out << ",";
    out << segmaskProcessRuntime_ms;out << ",";
    out << similarityEmbeddingRuntime_ms;out << ",";
    out << pipelineLatency_ms;
    
}

std::stringstream VpbIsnHealth::toStringstream() const
{
    std::stringstream out;
    toCsv(out);
    return out;
}

template<class Archive>
    void VpbIsnHealth::serialize(Archive& ar, [[maybe_unused]] unsigned int version)
    {

        ar& CEREAL_NVP(publishRate_hz);
        ar& CEREAL_NVP(cameraId);
        ar& CEREAL_NVP(isnModelVersion);
        ar& CEREAL_NVP(isnStatus);
        ar& CEREAL_NVP(capPts);
        ar& CEREAL_NVP(modelInferenceRuntime_ms);
        ar& CEREAL_NVP(picInfoRuntime_ms);
        ar& CEREAL_NVP(box2dGenerationRuntime_ms);
        ar& CEREAL_NVP(box3dGenerationRuntime_ms);
        ar& CEREAL_NVP(segmaskProcessRuntime_ms);
        ar& CEREAL_NVP(similarityEmbeddingRuntime_ms);
        ar& CEREAL_NVP(pipelineLatency_ms);
    }


template void VpbIsnHealth::serialize(cereal::BinaryInputArchive& ar, unsigned int version);
template void VpbIsnHealth::serialize(cereal::BinaryOutputArchive& ar, unsigned int version);
template void VpbIsnHealth::serialize(cereal::IntrospectionArchive& ar, unsigned int version);
template void VpbIsnHealth::serialize(cereal::JSONInputArchive& ar, unsigned int version);
template void VpbIsnHealth::serialize(cereal::JSONOutputArchive& ar, unsigned int version);
template void VpbIsnHealth::serialize(cereal::PortableBinaryInputArchive& ar, unsigned int version);
template void VpbIsnHealth::serialize(cereal::PortableBinaryOutputArchive& ar, unsigned int version);

std::ostream& operator<< (std::ostream& os, const VpbIsnHealth& vpbisnhealth)
{
    vpbisnhealth.toCsv(os);
    return os;
}

bool operator== (const VpbIsnHealth &lhs, const VpbIsnHealth &rhs)
{
    return (  lhs.publishRate_hz== rhs.publishRate_hz &&
     lhs.cameraId== rhs.cameraId &&
     lhs.isnModelVersion== rhs.isnModelVersion &&
     lhs.isnStatus== rhs.isnStatus &&
     lhs.capPts== rhs.capPts &&
     lhs.modelInferenceRuntime_ms== rhs.modelInferenceRuntime_ms &&
     lhs.picInfoRuntime_ms== rhs.picInfoRuntime_ms &&
     lhs.box2dGenerationRuntime_ms== rhs.box2dGenerationRuntime_ms &&
     lhs.box3dGenerationRuntime_ms== rhs.box3dGenerationRuntime_ms &&
     lhs.segmaskProcessRuntime_ms== rhs.segmaskProcessRuntime_ms &&
     lhs.similarityEmbeddingRuntime_ms== rhs.similarityEmbeddingRuntime_ms &&
     lhs.pipelineLatency_ms== rhs.pipelineLatency_ms 
    );
}

std::size_t VpbIsnHealth::hash() const noexcept
{
    std::size_t result = 17;
    result = result * 31 + std::hash<float>()(publishRate_hz);
    result = result * 31 + std::hash<std::string>()(cameraId);
    result = result * 31 + std::hash<std::string>()(isnModelVersion);
    result = result * 31 + std::hash<IsnStatus>()(isnStatus);
    result = result * 31 + std::hash<uint64_t>()(capPts);
    result = result * 31 + std::hash<uint64_t>()(modelInferenceRuntime_ms);
    result = result * 31 + std::hash<uint64_t>()(picInfoRuntime_ms);
    result = result * 31 + std::hash<uint64_t>()(box2dGenerationRuntime_ms);
    result = result * 31 + std::hash<uint64_t>()(box3dGenerationRuntime_ms);
    result = result * 31 + std::hash<uint64_t>()(segmaskProcessRuntime_ms);
    result = result * 31 + std::hash<uint64_t>()(similarityEmbeddingRuntime_ms);
    result = result * 31 + std::hash<uint64_t>()(pipelineLatency_ms);
    return result;
}

 } // namespace av
 } // namespace msg
 } // namespace camera


CEREAL_BIND_TO_ARCHIVES(av::msg::camera::VpbIsnHealth)

