#include <interfaces/VpbStreamHealth/VpbStreamHealth.h>

#include <iterator>
#include <type_traits>

#include <cereal/cereal.hpp>
#include <cereal/archives/json.hpp>
#include <cereal/archives/binary.hpp>

#include <cereal/types/map.hpp>
#include <cereal/types/vector.hpp>
#include <cereal/types/array.hpp>
#include <cereal/types/string.hpp>
#include <cereal/types/complex.hpp>

#include <ODatum/IntrospectionArchive.h>
#include <ODatum/Hash.h>





 namespace av {
 namespace msg {
 namespace camera {


VpbStreamHealth::VpbStreamHealth() = default;

VpbStreamHealth::VpbStreamHealth(
    float rate_seconds_arg ,
    std::string cameraId_arg ,
    StreamStatus streamStatus_arg ,
    float readerFrameRate_fps_arg ,
    uint64_t irqCount_arg ,
    uint64_t encodedFramesReceived_arg ,
    uint64_t framesSent_arg ,
    uint64_t bytesSent_arg ,
    uint64_t noMetaFrames_arg ,
    uint64_t avgLatency_us_arg 
    ):
    rate_seconds { std::move(rate_seconds_arg) } // NOLINT
      ,cameraId { std::move(cameraId_arg) } // NOLINT
      ,streamStatus { std::move(streamStatus_arg) } // NOLINT
      ,readerFrameRate_fps { std::move(readerFrameRate_fps_arg) } // NOLINT
      ,irqCount { std::move(irqCount_arg) } // NOLINT
      ,encodedFramesReceived { std::move(encodedFramesReceived_arg) } // NOLINT
      ,framesSent { std::move(framesSent_arg) } // NOLINT
      ,bytesSent { std::move(bytesSent_arg) } // NOLINT
      ,noMetaFrames { std::move(noMetaFrames_arg) } // NOLINT
      ,avgLatency_us { std::move(avgLatency_us_arg) } // NOLINT
      {
}

VpbStreamHealth::VpbStreamHealth(const VpbStreamHealth&) = default;

// NOLINTNEXTLINE
VpbStreamHealth::VpbStreamHealth(VpbStreamHealth&&) = default;

VpbStreamHealth& VpbStreamHealth::operator=(const VpbStreamHealth&) = default;

// NOLINTNEXTLINE
VpbStreamHealth& VpbStreamHealth::operator=(VpbStreamHealth&&) = default;

void VpbStreamHealth::toCsv(std::ostream& out) const
{
    out.precision(16);
    out << std::boolalpha;
    out << rate_seconds;out << ",";
    out << "\"";
        out << cameraId;
        out << "\"";out << ",";
    out << static_cast<uint32_t>(streamStatus);out << ",";
    out << readerFrameRate_fps;out << ",";
    out << irqCount;out << ",";
    out << encodedFramesReceived;out << ",";
    out << framesSent;out << ",";
    out << bytesSent;out << ",";
    out << noMetaFrames;out << ",";
    out << avgLatency_us;
    
}

std::stringstream VpbStreamHealth::toStringstream() const
{
    std::stringstream out;
    toCsv(out);
    return out;
}

template<class Archive>
    void VpbStreamHealth::serialize(Archive& ar, unsigned int version)
{
    ar& CEREAL_NVP(rate_seconds);
    ar& CEREAL_NVP(cameraId);

    // Stream performance
    ar& CEREAL_NVP(streamStatus);

    ar& CEREAL_NVP(readerFrameRate_fps);
    ar& CEREAL_NVP(irqCount);
    if (version >= 2)
    {
        ar& CEREAL_NVP(encodedFramesReceived);
    }
    ar& CEREAL_NVP(framesSent);
    ar& CEREAL_NVP(bytesSent);
    ar& CEREAL_NVP(noMetaFrames);
    ar& CEREAL_NVP(avgLatency_us);
}


template void VpbStreamHealth::serialize(cereal::BinaryInputArchive& ar, unsigned int version);
template void VpbStreamHealth::serialize(cereal::BinaryOutputArchive& ar, unsigned int version);
template void VpbStreamHealth::serialize(cereal::IntrospectionArchive& ar, unsigned int version);
template void VpbStreamHealth::serialize(cereal::JSONInputArchive& ar, unsigned int version);
template void VpbStreamHealth::serialize(cereal::JSONOutputArchive& ar, unsigned int version);
template void VpbStreamHealth::serialize(cereal::PortableBinaryInputArchive& ar, unsigned int version);
template void VpbStreamHealth::serialize(cereal::PortableBinaryOutputArchive& ar, unsigned int version);

std::ostream& operator<< (std::ostream& os, const VpbStreamHealth& vpbstreamhealth)
{
    vpbstreamhealth.toCsv(os);
    return os;
}

bool operator== (const VpbStreamHealth &lhs, const VpbStreamHealth &rhs)
{
    return (  lhs.rate_seconds== rhs.rate_seconds &&
     lhs.cameraId== rhs.cameraId &&
     lhs.streamStatus== rhs.streamStatus &&
     lhs.readerFrameRate_fps== rhs.readerFrameRate_fps &&
     lhs.irqCount== rhs.irqCount &&
     lhs.encodedFramesReceived== rhs.encodedFramesReceived &&
     lhs.framesSent== rhs.framesSent &&
     lhs.bytesSent== rhs.bytesSent &&
     lhs.noMetaFrames== rhs.noMetaFrames &&
     lhs.avgLatency_us== rhs.avgLatency_us 
    );
}

std::size_t VpbStreamHealth::hash() const noexcept
{
    std::size_t result = 17;
    result = result * 31 + std::hash<float>()(rate_seconds);
    result = result * 31 + std::hash<std::string>()(cameraId);
    result = result * 31 + std::hash<StreamStatus>()(streamStatus);
    result = result * 31 + std::hash<float>()(readerFrameRate_fps);
    result = result * 31 + std::hash<uint64_t>()(irqCount);
    result = result * 31 + std::hash<uint64_t>()(encodedFramesReceived);
    result = result * 31 + std::hash<uint64_t>()(framesSent);
    result = result * 31 + std::hash<uint64_t>()(bytesSent);
    result = result * 31 + std::hash<uint64_t>()(noMetaFrames);
    result = result * 31 + std::hash<uint64_t>()(avgLatency_us);
    return result;
}

 } // namespace av
 } // namespace msg
 } // namespace camera


CEREAL_BIND_TO_ARCHIVES(av::msg::camera::VpbStreamHealth)

