#include <interfaces/WitnessTrajWithBounds/WitnessTrajWithBounds.h>

#include <iterator>
#include <type_traits>

#include <cereal/cereal.hpp>
#include <cereal/archives/json.hpp>
#include <cereal/archives/binary.hpp>

#include <cereal/types/map.hpp>
#include <cereal/types/vector.hpp>
#include <cereal/types/array.hpp>
#include <cereal/types/string.hpp>
#include <cereal/types/complex.hpp>

#include <ODatum/IntrospectionArchive.h>
#include <ODatum/Hash.h>


 namespace av {
 namespace msg {


WitnessTrajState::WitnessTrajState() = default;

WitnessTrajState::WitnessTrajState(
    double x_m_arg ,
    double y_m_arg ,
    double yaw_rad_arg ,
    double curvature_k_arg ,
    double velocity_mps_arg ,
    double accel_mps2_arg ,
    double refpointOffset_m_arg 
    ):
    x_m { std::move(x_m_arg) } // NOLINT
      ,
    y_m { std::move(y_m_arg) } // NOLINT
      ,
    yaw_rad { std::move(yaw_rad_arg) } // NOLINT
      ,
    curvature_k { std::move(curvature_k_arg) } // NOLINT
      ,
    velocity_mps { std::move(velocity_mps_arg) } // NOLINT
      ,
    accel_mps2 { std::move(accel_mps2_arg) } // NOLINT
      ,
    refpointOffset_m { std::move(refpointOffset_m_arg) } // NOLINT
      
    {
}

WitnessTrajState::WitnessTrajState(const WitnessTrajState&) = default;

// NOLINTNEXTLINE
WitnessTrajState::WitnessTrajState(WitnessTrajState&&) = default;

WitnessTrajState& WitnessTrajState::operator=(const WitnessTrajState&) = default;

// NOLINTNEXTLINE
WitnessTrajState& WitnessTrajState::operator=(WitnessTrajState&&) = default;

void WitnessTrajState::toCsv(std::ostream& out) const
{
    out.precision(16);
    out << std::boolalpha;
    out << x_m;out << ",";
    out << y_m;out << ",";
    out << yaw_rad;out << ",";
    out << curvature_k;out << ",";
    out << velocity_mps;out << ",";
    out << accel_mps2;out << ",";
    out << refpointOffset_m;
    
}

std::stringstream WitnessTrajState::toStringstream() const
{
    std::stringstream out;
    toCsv(out);
    return out;
}

template<class Archive>
    void WitnessTrajState::serialize(Archive& ar, [[maybe_unused]] unsigned int version)
    {

        ar& CEREAL_NVP(x_m);
        ar& CEREAL_NVP(y_m);
        ar& CEREAL_NVP(yaw_rad);
        ar& CEREAL_NVP(curvature_k);
        ar& CEREAL_NVP(velocity_mps);
        ar& CEREAL_NVP(accel_mps2);
        ar& CEREAL_NVP(refpointOffset_m);
    }


template void WitnessTrajState::serialize(cereal::BinaryInputArchive& ar, unsigned int version);
template void WitnessTrajState::serialize(cereal::BinaryOutputArchive& ar, unsigned int version);
template void WitnessTrajState::serialize(cereal::IntrospectionArchive& ar, unsigned int version);
template void WitnessTrajState::serialize(cereal::JSONInputArchive& ar, unsigned int version);
template void WitnessTrajState::serialize(cereal::JSONOutputArchive& ar, unsigned int version);
template void WitnessTrajState::serialize(cereal::PortableBinaryInputArchive& ar, unsigned int version);
template void WitnessTrajState::serialize(cereal::PortableBinaryOutputArchive& ar, unsigned int version);

std::ostream& operator<< (std::ostream& os, const WitnessTrajState& witnesstrajstate)
{
    witnesstrajstate.toCsv(os);
    return os;
}

bool operator== (const WitnessTrajState &lhs, const WitnessTrajState &rhs)
{
    return (  lhs.x_m== rhs.x_m &&
     lhs.y_m== rhs.y_m &&
     lhs.yaw_rad== rhs.yaw_rad &&
     lhs.curvature_k== rhs.curvature_k &&
     lhs.velocity_mps== rhs.velocity_mps &&
     lhs.accel_mps2== rhs.accel_mps2 &&
     lhs.refpointOffset_m== rhs.refpointOffset_m 
    );
}

std::size_t WitnessTrajState::hash() const noexcept
{
    std::size_t result = 17;
    result = result * 31 + std::hash<double>()(x_m);
    result = result * 31 + std::hash<double>()(y_m);
    result = result * 31 + std::hash<double>()(yaw_rad);
    result = result * 31 + std::hash<double>()(curvature_k);
    result = result * 31 + std::hash<double>()(velocity_mps);
    result = result * 31 + std::hash<double>()(accel_mps2);
    result = result * 31 + std::hash<double>()(refpointOffset_m);
    return result;
}

 } // namespace av
 } // namespace msg


CEREAL_BIND_TO_ARCHIVES(av::msg::WitnessTrajState)

 namespace av {
 namespace msg {


ConstraintBox::ConstraintBox() = default;

ConstraintBox::ConstraintBox(
    double upper_arg ,
    double lower_arg 
    ):
    upper { std::move(upper_arg) } // NOLINT
      ,
    lower { std::move(lower_arg) } // NOLINT
      
    {
}

ConstraintBox::ConstraintBox(const ConstraintBox&) = default;

// NOLINTNEXTLINE
ConstraintBox::ConstraintBox(ConstraintBox&&) = default;

ConstraintBox& ConstraintBox::operator=(const ConstraintBox&) = default;

// NOLINTNEXTLINE
ConstraintBox& ConstraintBox::operator=(ConstraintBox&&) = default;

void ConstraintBox::toCsv(std::ostream& out) const
{
    out.precision(16);
    out << std::boolalpha;
    out << upper;out << ",";
    out << lower;
    
}

std::stringstream ConstraintBox::toStringstream() const
{
    std::stringstream out;
    toCsv(out);
    return out;
}

template<class Archive>
    void ConstraintBox::serialize(Archive& ar, [[maybe_unused]] unsigned int version)
    {

        ar& CEREAL_NVP(upper);
        ar& CEREAL_NVP(lower);
    }


template void ConstraintBox::serialize(cereal::BinaryInputArchive& ar, unsigned int version);
template void ConstraintBox::serialize(cereal::BinaryOutputArchive& ar, unsigned int version);
template void ConstraintBox::serialize(cereal::IntrospectionArchive& ar, unsigned int version);
template void ConstraintBox::serialize(cereal::JSONInputArchive& ar, unsigned int version);
template void ConstraintBox::serialize(cereal::JSONOutputArchive& ar, unsigned int version);
template void ConstraintBox::serialize(cereal::PortableBinaryInputArchive& ar, unsigned int version);
template void ConstraintBox::serialize(cereal::PortableBinaryOutputArchive& ar, unsigned int version);

std::ostream& operator<< (std::ostream& os, const ConstraintBox& constraintbox)
{
    constraintbox.toCsv(os);
    return os;
}

bool operator== (const ConstraintBox &lhs, const ConstraintBox &rhs)
{
    return (  lhs.upper== rhs.upper &&
     lhs.lower== rhs.lower 
    );
}

std::size_t ConstraintBox::hash() const noexcept
{
    std::size_t result = 17;
    result = result * 31 + std::hash<double>()(upper);
    result = result * 31 + std::hash<double>()(lower);
    return result;
}

 } // namespace av
 } // namespace msg


CEREAL_BIND_TO_ARCHIVES(av::msg::ConstraintBox)




 namespace av {
 namespace msg {


WitnessTrajWithBounds::WitnessTrajWithBounds() = default;

WitnessTrajWithBounds::WitnessTrajWithBounds(
    int64_t time_us_arg ,
    double timestep_s_arg ,
    int32_t numStates_arg ,
    std::vector<WitnessTrajState> states_arg ,
    std::vector<ConstraintBox> lateralTubeConstraints_m_arg ,
    std::vector<ConstraintBox> yawConstraints_rad_arg ,
    std::vector<ConstraintBox> tracksLonVel_mps2_arg ,
    std::vector<ConstraintBox> tracksLateralDistance_m_arg ,
    int32_t numConstraintsLeft_arg ,
    std::vector<SE2> leftConstraints_arg ,
    int32_t numConstraintsRight_arg ,
    std::vector<SE2> rightConstraints_arg 
    ):
    time_us { std::move(time_us_arg) } // NOLINT
      ,timestep_s { std::move(timestep_s_arg) } // NOLINT
      ,numStates { std::move(numStates_arg) } // NOLINT
      ,states { std::move(states_arg) } // NOLINT
      ,lateralTubeConstraints_m { std::move(lateralTubeConstraints_m_arg) } // NOLINT
      ,yawConstraints_rad { std::move(yawConstraints_rad_arg) } // NOLINT
      ,tracksLonVel_mps2 { std::move(tracksLonVel_mps2_arg) } // NOLINT
      ,tracksLateralDistance_m { std::move(tracksLateralDistance_m_arg) } // NOLINT
      ,numConstraintsLeft { std::move(numConstraintsLeft_arg) } // NOLINT
      ,leftConstraints { std::move(leftConstraints_arg) } // NOLINT
      ,numConstraintsRight { std::move(numConstraintsRight_arg) } // NOLINT
      ,rightConstraints { std::move(rightConstraints_arg) } // NOLINT
      {
}

WitnessTrajWithBounds::WitnessTrajWithBounds(const WitnessTrajWithBounds&) = default;

// NOLINTNEXTLINE
WitnessTrajWithBounds::WitnessTrajWithBounds(WitnessTrajWithBounds&&) = default;

WitnessTrajWithBounds& WitnessTrajWithBounds::operator=(const WitnessTrajWithBounds&) = default;

// NOLINTNEXTLINE
WitnessTrajWithBounds& WitnessTrajWithBounds::operator=(WitnessTrajWithBounds&&) = default;

void WitnessTrajWithBounds::toCsv(std::ostream& out) const
{
    out.precision(16);
    out << std::boolalpha;
    out << time_us;out << ",";
    out << timestep_s;out << ",";
    out << numStates;out << ",";
    // Enclose sequence in double quotations and curly brackets to keep CSV line length uniform
        
        out << "\"{{";
        
        if (!states.empty())
        {
            std::copy(states.begin(), states.end() - 1, std::ostream_iterator<WitnessTrajState>(out, "},{"));
            std::copy(states.end()-1, states.end(), std::ostream_iterator<WitnessTrajState>(out, ""));
        }
        out << "}}\"";out << ",";
    // Enclose sequence in double quotations and curly brackets to keep CSV line length uniform
        
        out << "\"{{";
        
        if (!lateralTubeConstraints_m.empty())
        {
            std::copy(lateralTubeConstraints_m.begin(), lateralTubeConstraints_m.end() - 1, std::ostream_iterator<ConstraintBox>(out, "},{"));
            std::copy(lateralTubeConstraints_m.end()-1, lateralTubeConstraints_m.end(), std::ostream_iterator<ConstraintBox>(out, ""));
        }
        out << "}}\"";out << ",";
    // Enclose sequence in double quotations and curly brackets to keep CSV line length uniform
        
        out << "\"{{";
        
        if (!yawConstraints_rad.empty())
        {
            std::copy(yawConstraints_rad.begin(), yawConstraints_rad.end() - 1, std::ostream_iterator<ConstraintBox>(out, "},{"));
            std::copy(yawConstraints_rad.end()-1, yawConstraints_rad.end(), std::ostream_iterator<ConstraintBox>(out, ""));
        }
        out << "}}\"";out << ",";
    // Enclose sequence in double quotations and curly brackets to keep CSV line length uniform
        
        out << "\"{{";
        
        if (!tracksLonVel_mps2.empty())
        {
            std::copy(tracksLonVel_mps2.begin(), tracksLonVel_mps2.end() - 1, std::ostream_iterator<ConstraintBox>(out, "},{"));
            std::copy(tracksLonVel_mps2.end()-1, tracksLonVel_mps2.end(), std::ostream_iterator<ConstraintBox>(out, ""));
        }
        out << "}}\"";out << ",";
    // Enclose sequence in double quotations and curly brackets to keep CSV line length uniform
        
        out << "\"{{";
        
        if (!tracksLateralDistance_m.empty())
        {
            std::copy(tracksLateralDistance_m.begin(), tracksLateralDistance_m.end() - 1, std::ostream_iterator<ConstraintBox>(out, "},{"));
            std::copy(tracksLateralDistance_m.end()-1, tracksLateralDistance_m.end(), std::ostream_iterator<ConstraintBox>(out, ""));
        }
        out << "}}\"";out << ",";
    out << numConstraintsLeft;out << ",";
    // Enclose sequence in double quotations and curly brackets to keep CSV line length uniform
        
        out << "\"{{";
        
        if (!leftConstraints.empty())
        {
            std::copy(leftConstraints.begin(), leftConstraints.end() - 1, std::ostream_iterator<SE2>(out, "},{"));
            std::copy(leftConstraints.end()-1, leftConstraints.end(), std::ostream_iterator<SE2>(out, ""));
        }
        out << "}}\"";out << ",";
    out << numConstraintsRight;out << ",";
    // Enclose sequence in double quotations and curly brackets to keep CSV line length uniform
        
        out << "\"{{";
        
        if (!rightConstraints.empty())
        {
            std::copy(rightConstraints.begin(), rightConstraints.end() - 1, std::ostream_iterator<SE2>(out, "},{"));
            std::copy(rightConstraints.end()-1, rightConstraints.end(), std::ostream_iterator<SE2>(out, ""));
        }
        out << "}}\"";
    
}

std::stringstream WitnessTrajWithBounds::toStringstream() const
{
    std::stringstream out;
    toCsv(out);
    return out;
}

template<class Archive>
    void WitnessTrajWithBounds::serialize(Archive& ar, unsigned int version)
{
    if (version >= 1) {
        ar& CEREAL_NVP(time_us);
        ar& CEREAL_NVP(timestep_s);
        ar& CEREAL_NVP(numStates);
        ar& CEREAL_NVP(states);
        ar& CEREAL_NVP(lateralTubeConstraints_m);
        ar& CEREAL_NVP(yawConstraints_rad);
        ar& CEREAL_NVP(tracksLonVel_mps2);
        ar& CEREAL_NVP(tracksLateralDistance_m);
        ar& CEREAL_NVP(numConstraintsLeft);
        ar& CEREAL_NVP(leftConstraints);
        ar& CEREAL_NVP(numConstraintsRight);
        ar& CEREAL_NVP(rightConstraints);
    }
}

}}  // to close av::msg namespace temporary

struct WitnessTrajWithBounds : public av::msg::WitnessTrajWithBounds {};
CEREAL_CLASS_VERSION(WitnessTrajWithBounds, 1)
CEREAL_REGISTER_TYPE(WitnessTrajWithBounds)

namespace av {
namespace msg {



template void WitnessTrajWithBounds::serialize(cereal::BinaryInputArchive& ar, unsigned int version);
template void WitnessTrajWithBounds::serialize(cereal::BinaryOutputArchive& ar, unsigned int version);
template void WitnessTrajWithBounds::serialize(cereal::IntrospectionArchive& ar, unsigned int version);
template void WitnessTrajWithBounds::serialize(cereal::JSONInputArchive& ar, unsigned int version);
template void WitnessTrajWithBounds::serialize(cereal::JSONOutputArchive& ar, unsigned int version);
template void WitnessTrajWithBounds::serialize(cereal::PortableBinaryInputArchive& ar, unsigned int version);
template void WitnessTrajWithBounds::serialize(cereal::PortableBinaryOutputArchive& ar, unsigned int version);

std::ostream& operator<< (std::ostream& os, const WitnessTrajWithBounds& witnesstrajwithbounds)
{
    witnesstrajwithbounds.toCsv(os);
    return os;
}

bool operator== (const WitnessTrajWithBounds &lhs, const WitnessTrajWithBounds &rhs)
{
    return (  lhs.time_us== rhs.time_us &&
     lhs.timestep_s== rhs.timestep_s &&
     lhs.numStates== rhs.numStates &&
     lhs.states== rhs.states &&
     lhs.lateralTubeConstraints_m== rhs.lateralTubeConstraints_m &&
     lhs.yawConstraints_rad== rhs.yawConstraints_rad &&
     lhs.tracksLonVel_mps2== rhs.tracksLonVel_mps2 &&
     lhs.tracksLateralDistance_m== rhs.tracksLateralDistance_m &&
     lhs.numConstraintsLeft== rhs.numConstraintsLeft &&
     lhs.leftConstraints== rhs.leftConstraints &&
     lhs.numConstraintsRight== rhs.numConstraintsRight &&
     lhs.rightConstraints== rhs.rightConstraints 
    );
}

std::size_t WitnessTrajWithBounds::hash() const noexcept
{
    std::size_t result = 17;
    result = result * 31 + std::hash<int64_t>()(time_us);
    result = result * 31 + std::hash<double>()(timestep_s);
    result = result * 31 + std::hash<int32_t>()(numStates);
    result = result * 31 + hashVector(states);
    result = result * 31 + hashVector(lateralTubeConstraints_m);
    result = result * 31 + hashVector(yawConstraints_rad);
    result = result * 31 + hashVector(tracksLonVel_mps2);
    result = result * 31 + hashVector(tracksLateralDistance_m);
    result = result * 31 + std::hash<int32_t>()(numConstraintsLeft);
    result = result * 31 + hashVector(leftConstraints);
    result = result * 31 + std::hash<int32_t>()(numConstraintsRight);
    result = result * 31 + hashVector(rightConstraints);
    return result;
}

 } // namespace av
 } // namespace msg


CEREAL_BIND_TO_ARCHIVES(av::msg::WitnessTrajWithBounds)

