#include <interfaces/WorldManager/WorldManager.h>

#include <iterator>
#include <type_traits>

#include <cereal/cereal.hpp>
#include <cereal/archives/json.hpp>
#include <cereal/archives/binary.hpp>

#include <cereal/types/map.hpp>
#include <cereal/types/vector.hpp>
#include <cereal/types/array.hpp>
#include <cereal/types/string.hpp>
#include <cereal/types/complex.hpp>

#include <ODatum/IntrospectionArchive.h>
#include <ODatum/Hash.h>


 namespace av {
 namespace msg {
 namespace world_types {


TrackInfoBase::TrackInfoBase() = default;

TrackInfoBase::TrackInfoBase(
    int64_t id_arg ,
    bool isReal_arg ,
    bool isActive_arg ,
    bool isConverged_arg ,
    av::msg::SE2 centerPose_arg ,
    av::msg::SE2 frontPose_arg ,
    av::msg::SE2 rearPose_arg ,
    av::msg::SE2 leftCornerPose_arg ,
    av::msg::SE2 rightCornerPose_arg ,
    double length_arg ,
    double width_arg ,
    double vx_arg ,
    double vy_arg ,
    double speed_arg ,
    double accel_arg ,
    double yawRate_arg ,
    std::vector<double> typeBelief_arg ,
    double beliefThreshold_arg 
    ):
    id { std::move(id_arg) } // NOLINT
      ,
    isReal { std::move(isReal_arg) } // NOLINT
      ,
    isActive { std::move(isActive_arg) } // NOLINT
      ,
    isConverged { std::move(isConverged_arg) } // NOLINT
      ,
    centerPose { std::move(centerPose_arg) } // NOLINT
      ,
    frontPose { std::move(frontPose_arg) } // NOLINT
      ,
    rearPose { std::move(rearPose_arg) } // NOLINT
      ,
    leftCornerPose { std::move(leftCornerPose_arg) } // NOLINT
      ,
    rightCornerPose { std::move(rightCornerPose_arg) } // NOLINT
      ,
    length { std::move(length_arg) } // NOLINT
      ,
    width { std::move(width_arg) } // NOLINT
      ,
    vx { std::move(vx_arg) } // NOLINT
      ,
    vy { std::move(vy_arg) } // NOLINT
      ,
    speed { std::move(speed_arg) } // NOLINT
      ,
    accel { std::move(accel_arg) } // NOLINT
      ,
    yawRate { std::move(yawRate_arg) } // NOLINT
      ,
    typeBelief { std::move(typeBelief_arg) } // NOLINT
      ,
    beliefThreshold { std::move(beliefThreshold_arg) } // NOLINT
      
    {
}

TrackInfoBase::TrackInfoBase(const TrackInfoBase&) = default;

// NOLINTNEXTLINE
TrackInfoBase::TrackInfoBase(TrackInfoBase&&) = default;

TrackInfoBase& TrackInfoBase::operator=(const TrackInfoBase&) = default;

// NOLINTNEXTLINE
TrackInfoBase& TrackInfoBase::operator=(TrackInfoBase&&) = default;

void TrackInfoBase::toCsv(std::ostream& out) const
{
    out.precision(16);
    out << std::boolalpha;
    out << id;out << ",";
    out << isReal;out << ",";
    out << isActive;out << ",";
    out << isConverged;out << ",";
    out << centerPose;out << ",";
    out << frontPose;out << ",";
    out << rearPose;out << ",";
    out << leftCornerPose;out << ",";
    out << rightCornerPose;out << ",";
    out << length;out << ",";
    out << width;out << ",";
    out << vx;out << ",";
    out << vy;out << ",";
    out << speed;out << ",";
    out << accel;out << ",";
    out << yawRate;out << ",";
    // Enclose sequence in double quotations and curly brackets to keep CSV line length uniform
        
        out << "\"{{";
        
        if (!typeBelief.empty())
        {
            std::copy(typeBelief.begin(), typeBelief.end() - 1, std::ostream_iterator<double>(out, "},{"));
            std::copy(typeBelief.end()-1, typeBelief.end(), std::ostream_iterator<double>(out, ""));
        }
        out << "}}\"";out << ",";
    out << beliefThreshold;
    
}

std::stringstream TrackInfoBase::toStringstream() const
{
    std::stringstream out;
    toCsv(out);
    return out;
}

template<class Archive>
    void TrackInfoBase::serialize(Archive& ar, [[maybe_unused]] unsigned int version)
    {

        ar& CEREAL_NVP(id);
        ar& CEREAL_NVP(isReal);
        ar& CEREAL_NVP(isActive);
        ar& CEREAL_NVP(isConverged);
        ar& CEREAL_NVP(centerPose);
        ar& CEREAL_NVP(frontPose);
        ar& CEREAL_NVP(rearPose);
        ar& CEREAL_NVP(leftCornerPose);
        ar& CEREAL_NVP(rightCornerPose);
        ar& CEREAL_NVP(length);
        ar& CEREAL_NVP(width);
        ar& CEREAL_NVP(vx);
        ar& CEREAL_NVP(vy);
        ar& CEREAL_NVP(speed);
        ar& CEREAL_NVP(accel);
        ar& CEREAL_NVP(yawRate);
        ar& CEREAL_NVP(typeBelief);
        ar& CEREAL_NVP(beliefThreshold);
    }


template void TrackInfoBase::serialize(cereal::BinaryInputArchive& ar, unsigned int version);
template void TrackInfoBase::serialize(cereal::BinaryOutputArchive& ar, unsigned int version);
template void TrackInfoBase::serialize(cereal::IntrospectionArchive& ar, unsigned int version);
template void TrackInfoBase::serialize(cereal::JSONInputArchive& ar, unsigned int version);
template void TrackInfoBase::serialize(cereal::JSONOutputArchive& ar, unsigned int version);
template void TrackInfoBase::serialize(cereal::PortableBinaryInputArchive& ar, unsigned int version);
template void TrackInfoBase::serialize(cereal::PortableBinaryOutputArchive& ar, unsigned int version);

std::ostream& operator<< (std::ostream& os, const TrackInfoBase& trackinfobase)
{
    trackinfobase.toCsv(os);
    return os;
}

bool operator== (const TrackInfoBase &lhs, const TrackInfoBase &rhs)
{
    return (  lhs.id== rhs.id &&
     lhs.isReal== rhs.isReal &&
     lhs.isActive== rhs.isActive &&
     lhs.isConverged== rhs.isConverged &&
     lhs.centerPose== rhs.centerPose &&
     lhs.frontPose== rhs.frontPose &&
     lhs.rearPose== rhs.rearPose &&
     lhs.leftCornerPose== rhs.leftCornerPose &&
     lhs.rightCornerPose== rhs.rightCornerPose &&
     lhs.length== rhs.length &&
     lhs.width== rhs.width &&
     lhs.vx== rhs.vx &&
     lhs.vy== rhs.vy &&
     lhs.speed== rhs.speed &&
     lhs.accel== rhs.accel &&
     lhs.yawRate== rhs.yawRate &&
     lhs.typeBelief== rhs.typeBelief &&
     lhs.beliefThreshold== rhs.beliefThreshold 
    );
}

std::size_t TrackInfoBase::hash() const noexcept
{
    std::size_t result = 17;
    result = result * 31 + std::hash<int64_t>()(id);
    result = result * 31 + std::hash<bool>()(isReal);
    result = result * 31 + std::hash<bool>()(isActive);
    result = result * 31 + std::hash<bool>()(isConverged);
    result = result * 31 + std::hash<av::msg::SE2>()(centerPose);
    result = result * 31 + std::hash<av::msg::SE2>()(frontPose);
    result = result * 31 + std::hash<av::msg::SE2>()(rearPose);
    result = result * 31 + std::hash<av::msg::SE2>()(leftCornerPose);
    result = result * 31 + std::hash<av::msg::SE2>()(rightCornerPose);
    result = result * 31 + std::hash<double>()(length);
    result = result * 31 + std::hash<double>()(width);
    result = result * 31 + std::hash<double>()(vx);
    result = result * 31 + std::hash<double>()(vy);
    result = result * 31 + std::hash<double>()(speed);
    result = result * 31 + std::hash<double>()(accel);
    result = result * 31 + std::hash<double>()(yawRate);
    result = result * 31 + hashVector(typeBelief);
    result = result * 31 + std::hash<double>()(beliefThreshold);
    return result;
}

 } // namespace av
 } // namespace msg
 } // namespace world_types


CEREAL_BIND_TO_ARCHIVES(av::msg::world_types::TrackInfoBase)

 namespace av {
 namespace msg {
 namespace world_types {


TurnSignal::TurnSignal() = default;

TurnSignal::TurnSignal(
    TurnSignalType signal_arg ,
    double secOn_arg 
    ):
    signal { std::move(signal_arg) } // NOLINT
      ,
    secOn { std::move(secOn_arg) } // NOLINT
      
    {
}

TurnSignal::TurnSignal(const TurnSignal&) = default;

// NOLINTNEXTLINE
TurnSignal::TurnSignal(TurnSignal&&) = default;

TurnSignal& TurnSignal::operator=(const TurnSignal&) = default;

// NOLINTNEXTLINE
TurnSignal& TurnSignal::operator=(TurnSignal&&) = default;

void TurnSignal::toCsv(std::ostream& out) const
{
    out.precision(16);
    out << std::boolalpha;
    out << static_cast<uint32_t>(signal);out << ",";
    out << secOn;
    
}

std::stringstream TurnSignal::toStringstream() const
{
    std::stringstream out;
    toCsv(out);
    return out;
}

template<class Archive>
    void TurnSignal::serialize(Archive& ar, [[maybe_unused]] unsigned int version)
    {

        ar& CEREAL_NVP(signal);
        ar& CEREAL_NVP(secOn);
    }


template void TurnSignal::serialize(cereal::BinaryInputArchive& ar, unsigned int version);
template void TurnSignal::serialize(cereal::BinaryOutputArchive& ar, unsigned int version);
template void TurnSignal::serialize(cereal::IntrospectionArchive& ar, unsigned int version);
template void TurnSignal::serialize(cereal::JSONInputArchive& ar, unsigned int version);
template void TurnSignal::serialize(cereal::JSONOutputArchive& ar, unsigned int version);
template void TurnSignal::serialize(cereal::PortableBinaryInputArchive& ar, unsigned int version);
template void TurnSignal::serialize(cereal::PortableBinaryOutputArchive& ar, unsigned int version);

std::ostream& operator<< (std::ostream& os, const TurnSignal& turnsignal)
{
    turnsignal.toCsv(os);
    return os;
}

bool operator== (const TurnSignal &lhs, const TurnSignal &rhs)
{
    return (  lhs.signal== rhs.signal &&
     lhs.secOn== rhs.secOn 
    );
}

std::size_t TurnSignal::hash() const noexcept
{
    std::size_t result = 17;
    result = result * 31 + std::hash<TurnSignalType>()(signal);
    result = result * 31 + std::hash<double>()(secOn);
    return result;
}

 } // namespace av
 } // namespace msg
 } // namespace world_types


CEREAL_BIND_TO_ARCHIVES(av::msg::world_types::TurnSignal)

 namespace av {
 namespace msg {
 namespace world_types {


AccelConstraint::AccelConstraint() = default;

AccelConstraint::AccelConstraint(
    std::string type_arg ,
    double accel_arg ,
    std::string id_arg 
    ):
    type { std::move(type_arg) } // NOLINT
      ,
    accel { std::move(accel_arg) } // NOLINT
      ,
    id { std::move(id_arg) } // NOLINT
      
    {
}

AccelConstraint::AccelConstraint(const AccelConstraint&) = default;

// NOLINTNEXTLINE
AccelConstraint::AccelConstraint(AccelConstraint&&) = default;

AccelConstraint& AccelConstraint::operator=(const AccelConstraint&) = default;

// NOLINTNEXTLINE
AccelConstraint& AccelConstraint::operator=(AccelConstraint&&) = default;

void AccelConstraint::toCsv(std::ostream& out) const
{
    out.precision(16);
    out << std::boolalpha;
    out << "\"";
        out << type;
        out << "\"";out << ",";
    out << accel;out << ",";
    out << "\"";
        out << id;
        out << "\"";
    
}

std::stringstream AccelConstraint::toStringstream() const
{
    std::stringstream out;
    toCsv(out);
    return out;
}

template<class Archive>
    void AccelConstraint::serialize(Archive& ar, [[maybe_unused]] unsigned int version)
    {

        ar& CEREAL_NVP(type);
        ar& CEREAL_NVP(accel);
        ar& CEREAL_NVP(id);
    }


template void AccelConstraint::serialize(cereal::BinaryInputArchive& ar, unsigned int version);
template void AccelConstraint::serialize(cereal::BinaryOutputArchive& ar, unsigned int version);
template void AccelConstraint::serialize(cereal::IntrospectionArchive& ar, unsigned int version);
template void AccelConstraint::serialize(cereal::JSONInputArchive& ar, unsigned int version);
template void AccelConstraint::serialize(cereal::JSONOutputArchive& ar, unsigned int version);
template void AccelConstraint::serialize(cereal::PortableBinaryInputArchive& ar, unsigned int version);
template void AccelConstraint::serialize(cereal::PortableBinaryOutputArchive& ar, unsigned int version);

std::ostream& operator<< (std::ostream& os, const AccelConstraint& accelconstraint)
{
    accelconstraint.toCsv(os);
    return os;
}

bool operator== (const AccelConstraint &lhs, const AccelConstraint &rhs)
{
    return (  lhs.type== rhs.type &&
     lhs.accel== rhs.accel &&
     lhs.id== rhs.id 
    );
}

std::size_t AccelConstraint::hash() const noexcept
{
    std::size_t result = 17;
    result = result * 31 + std::hash<std::string>()(type);
    result = result * 31 + std::hash<double>()(accel);
    result = result * 31 + std::hash<std::string>()(id);
    return result;
}

 } // namespace av
 } // namespace msg
 } // namespace world_types


CEREAL_BIND_TO_ARCHIVES(av::msg::world_types::AccelConstraint)

 namespace av {
 namespace msg {
 namespace world_types {


LaneGroup::LaneGroup() = default;

LaneGroup::LaneGroup(
    std::vector<std::string> lanes_arg 
    ):
    lanes { std::move(lanes_arg) } // NOLINT
      
    {
}

LaneGroup::LaneGroup(const LaneGroup&) = default;

// NOLINTNEXTLINE
LaneGroup::LaneGroup(LaneGroup&&) = default;

LaneGroup& LaneGroup::operator=(const LaneGroup&) = default;

// NOLINTNEXTLINE
LaneGroup& LaneGroup::operator=(LaneGroup&&) = default;

void LaneGroup::toCsv(std::ostream& out) const
{
    out.precision(16);
    out << std::boolalpha;
    // Enclose sequence in double quotations and curly brackets to keep CSV line length uniform
        
        out << "\"{{";
        
        if (!lanes.empty())
        {
            std::copy(lanes.begin(), lanes.end() - 1, std::ostream_iterator<std::string>(out, "},{"));
            std::copy(lanes.end()-1, lanes.end(), std::ostream_iterator<std::string>(out, ""));
        }
        out << "}}\"";
    
}

std::stringstream LaneGroup::toStringstream() const
{
    std::stringstream out;
    toCsv(out);
    return out;
}

template<class Archive>
    void LaneGroup::serialize(Archive& ar, [[maybe_unused]] unsigned int version)
    {

        ar& CEREAL_NVP(lanes);
    }


template void LaneGroup::serialize(cereal::BinaryInputArchive& ar, unsigned int version);
template void LaneGroup::serialize(cereal::BinaryOutputArchive& ar, unsigned int version);
template void LaneGroup::serialize(cereal::IntrospectionArchive& ar, unsigned int version);
template void LaneGroup::serialize(cereal::JSONInputArchive& ar, unsigned int version);
template void LaneGroup::serialize(cereal::JSONOutputArchive& ar, unsigned int version);
template void LaneGroup::serialize(cereal::PortableBinaryInputArchive& ar, unsigned int version);
template void LaneGroup::serialize(cereal::PortableBinaryOutputArchive& ar, unsigned int version);

std::ostream& operator<< (std::ostream& os, const LaneGroup& lanegroup)
{
    lanegroup.toCsv(os);
    return os;
}

bool operator== (const LaneGroup &lhs, const LaneGroup &rhs)
{
    return (  lhs.lanes== rhs.lanes 
    );
}

std::size_t LaneGroup::hash() const noexcept
{
    std::size_t result = 17;
    result = result * 31 + hashVector(lanes);
    return result;
}

 } // namespace av
 } // namespace msg
 } // namespace world_types


CEREAL_BIND_TO_ARCHIVES(av::msg::world_types::LaneGroup)

 namespace av {
 namespace msg {
 namespace world_types {


TrafficLight::TrafficLight() = default;

TrafficLight::TrafficLight(
    int8_t fov_arg ,
    int8_t state_arg ,
    int8_t confidenceLevel_arg 
    ):
    fov { std::move(fov_arg) } // NOLINT
      ,
    state { std::move(state_arg) } // NOLINT
      ,
    confidenceLevel { std::move(confidenceLevel_arg) } // NOLINT
      
    {
}

TrafficLight::TrafficLight(const TrafficLight&) = default;

// NOLINTNEXTLINE
TrafficLight::TrafficLight(TrafficLight&&) = default;

TrafficLight& TrafficLight::operator=(const TrafficLight&) = default;

// NOLINTNEXTLINE
TrafficLight& TrafficLight::operator=(TrafficLight&&) = default;

void TrafficLight::toCsv(std::ostream& out) const
{
    out.precision(16);
    out << std::boolalpha;
    // Cast to int16_t to print as integer instead of char
        out << static_cast<int16_t>(fov);out << ",";
    // Cast to int16_t to print as integer instead of char
        out << static_cast<int16_t>(state);out << ",";
    // Cast to int16_t to print as integer instead of char
        out << static_cast<int16_t>(confidenceLevel);
    
}

std::stringstream TrafficLight::toStringstream() const
{
    std::stringstream out;
    toCsv(out);
    return out;
}

template<class Archive>
    void TrafficLight::serialize(Archive& ar, [[maybe_unused]] unsigned int version)
    {

        ar& CEREAL_NVP(fov);
        ar& CEREAL_NVP(state);
        ar& CEREAL_NVP(confidenceLevel);
    }


template void TrafficLight::serialize(cereal::BinaryInputArchive& ar, unsigned int version);
template void TrafficLight::serialize(cereal::BinaryOutputArchive& ar, unsigned int version);
template void TrafficLight::serialize(cereal::IntrospectionArchive& ar, unsigned int version);
template void TrafficLight::serialize(cereal::JSONInputArchive& ar, unsigned int version);
template void TrafficLight::serialize(cereal::JSONOutputArchive& ar, unsigned int version);
template void TrafficLight::serialize(cereal::PortableBinaryInputArchive& ar, unsigned int version);
template void TrafficLight::serialize(cereal::PortableBinaryOutputArchive& ar, unsigned int version);

std::ostream& operator<< (std::ostream& os, const TrafficLight& trafficlight)
{
    trafficlight.toCsv(os);
    return os;
}

bool operator== (const TrafficLight &lhs, const TrafficLight &rhs)
{
    return (  lhs.fov== rhs.fov &&
     lhs.state== rhs.state &&
     lhs.confidenceLevel== rhs.confidenceLevel 
    );
}

std::size_t TrafficLight::hash() const noexcept
{
    std::size_t result = 17;
    result = result * 31 + std::hash<int8_t>()(fov);
    result = result * 31 + std::hash<int8_t>()(state);
    result = result * 31 + std::hash<int8_t>()(confidenceLevel);
    return result;
}

 } // namespace av
 } // namespace msg
 } // namespace world_types


CEREAL_BIND_TO_ARCHIVES(av::msg::world_types::TrafficLight)

 namespace av {
 namespace msg {
 namespace world_types {


DistanceAndIds::DistanceAndIds() = default;

DistanceAndIds::DistanceAndIds(
    double distance_arg ,
    int64_t track1Id_arg ,
    int64_t track2Id_arg 
    ):
    distance { std::move(distance_arg) } // NOLINT
      ,
    track1Id { std::move(track1Id_arg) } // NOLINT
      ,
    track2Id { std::move(track2Id_arg) } // NOLINT
      
    {
}

DistanceAndIds::DistanceAndIds(const DistanceAndIds&) = default;

// NOLINTNEXTLINE
DistanceAndIds::DistanceAndIds(DistanceAndIds&&) = default;

DistanceAndIds& DistanceAndIds::operator=(const DistanceAndIds&) = default;

// NOLINTNEXTLINE
DistanceAndIds& DistanceAndIds::operator=(DistanceAndIds&&) = default;

void DistanceAndIds::toCsv(std::ostream& out) const
{
    out.precision(16);
    out << std::boolalpha;
    out << distance;out << ",";
    out << track1Id;out << ",";
    out << track2Id;
    
}

std::stringstream DistanceAndIds::toStringstream() const
{
    std::stringstream out;
    toCsv(out);
    return out;
}

template<class Archive>
    void DistanceAndIds::serialize(Archive& ar, [[maybe_unused]] unsigned int version)
    {

        ar& CEREAL_NVP(distance);
        ar& CEREAL_NVP(track1Id);
        ar& CEREAL_NVP(track2Id);
    }


template void DistanceAndIds::serialize(cereal::BinaryInputArchive& ar, unsigned int version);
template void DistanceAndIds::serialize(cereal::BinaryOutputArchive& ar, unsigned int version);
template void DistanceAndIds::serialize(cereal::IntrospectionArchive& ar, unsigned int version);
template void DistanceAndIds::serialize(cereal::JSONInputArchive& ar, unsigned int version);
template void DistanceAndIds::serialize(cereal::JSONOutputArchive& ar, unsigned int version);
template void DistanceAndIds::serialize(cereal::PortableBinaryInputArchive& ar, unsigned int version);
template void DistanceAndIds::serialize(cereal::PortableBinaryOutputArchive& ar, unsigned int version);

std::ostream& operator<< (std::ostream& os, const DistanceAndIds& distanceandids)
{
    distanceandids.toCsv(os);
    return os;
}

bool operator== (const DistanceAndIds &lhs, const DistanceAndIds &rhs)
{
    return (  lhs.distance== rhs.distance &&
     lhs.track1Id== rhs.track1Id &&
     lhs.track2Id== rhs.track2Id 
    );
}

std::size_t DistanceAndIds::hash() const noexcept
{
    std::size_t result = 17;
    result = result * 31 + std::hash<double>()(distance);
    result = result * 31 + std::hash<int64_t>()(track1Id);
    result = result * 31 + std::hash<int64_t>()(track2Id);
    return result;
}

 } // namespace av
 } // namespace msg
 } // namespace world_types


CEREAL_BIND_TO_ARCHIVES(av::msg::world_types::DistanceAndIds)

 namespace av {
 namespace msg {
 namespace world_types {


TimeToCollisionAndIds::TimeToCollisionAndIds() = default;

TimeToCollisionAndIds::TimeToCollisionAndIds(
    double timeToCollision_arg ,
    int64_t track1Id_arg ,
    int64_t track2Id_arg 
    ):
    timeToCollision { std::move(timeToCollision_arg) } // NOLINT
      ,
    track1Id { std::move(track1Id_arg) } // NOLINT
      ,
    track2Id { std::move(track2Id_arg) } // NOLINT
      
    {
}

TimeToCollisionAndIds::TimeToCollisionAndIds(const TimeToCollisionAndIds&) = default;

// NOLINTNEXTLINE
TimeToCollisionAndIds::TimeToCollisionAndIds(TimeToCollisionAndIds&&) = default;

TimeToCollisionAndIds& TimeToCollisionAndIds::operator=(const TimeToCollisionAndIds&) = default;

// NOLINTNEXTLINE
TimeToCollisionAndIds& TimeToCollisionAndIds::operator=(TimeToCollisionAndIds&&) = default;

void TimeToCollisionAndIds::toCsv(std::ostream& out) const
{
    out.precision(16);
    out << std::boolalpha;
    out << timeToCollision;out << ",";
    out << track1Id;out << ",";
    out << track2Id;
    
}

std::stringstream TimeToCollisionAndIds::toStringstream() const
{
    std::stringstream out;
    toCsv(out);
    return out;
}

template<class Archive>
    void TimeToCollisionAndIds::serialize(Archive& ar, [[maybe_unused]] unsigned int version)
    {

        ar& CEREAL_NVP(timeToCollision);
        ar& CEREAL_NVP(track1Id);
        ar& CEREAL_NVP(track2Id);
    }


template void TimeToCollisionAndIds::serialize(cereal::BinaryInputArchive& ar, unsigned int version);
template void TimeToCollisionAndIds::serialize(cereal::BinaryOutputArchive& ar, unsigned int version);
template void TimeToCollisionAndIds::serialize(cereal::IntrospectionArchive& ar, unsigned int version);
template void TimeToCollisionAndIds::serialize(cereal::JSONInputArchive& ar, unsigned int version);
template void TimeToCollisionAndIds::serialize(cereal::JSONOutputArchive& ar, unsigned int version);
template void TimeToCollisionAndIds::serialize(cereal::PortableBinaryInputArchive& ar, unsigned int version);
template void TimeToCollisionAndIds::serialize(cereal::PortableBinaryOutputArchive& ar, unsigned int version);

std::ostream& operator<< (std::ostream& os, const TimeToCollisionAndIds& timetocollisionandids)
{
    timetocollisionandids.toCsv(os);
    return os;
}

bool operator== (const TimeToCollisionAndIds &lhs, const TimeToCollisionAndIds &rhs)
{
    return (  lhs.timeToCollision== rhs.timeToCollision &&
     lhs.track1Id== rhs.track1Id &&
     lhs.track2Id== rhs.track2Id 
    );
}

std::size_t TimeToCollisionAndIds::hash() const noexcept
{
    std::size_t result = 17;
    result = result * 31 + std::hash<double>()(timeToCollision);
    result = result * 31 + std::hash<int64_t>()(track1Id);
    result = result * 31 + std::hash<int64_t>()(track2Id);
    return result;
}

 } // namespace av
 } // namespace msg
 } // namespace world_types


CEREAL_BIND_TO_ARCHIVES(av::msg::world_types::TimeToCollisionAndIds)

 namespace av {
 namespace msg {
 namespace world_types {


TimeId::TimeId() = default;

TimeId::TimeId(
    double time_arg ,
    int64_t trackId_arg 
    ):
    time { std::move(time_arg) } // NOLINT
      ,
    trackId { std::move(trackId_arg) } // NOLINT
      
    {
}

TimeId::TimeId(const TimeId&) = default;

// NOLINTNEXTLINE
TimeId::TimeId(TimeId&&) = default;

TimeId& TimeId::operator=(const TimeId&) = default;

// NOLINTNEXTLINE
TimeId& TimeId::operator=(TimeId&&) = default;

void TimeId::toCsv(std::ostream& out) const
{
    out.precision(16);
    out << std::boolalpha;
    out << time;out << ",";
    out << trackId;
    
}

std::stringstream TimeId::toStringstream() const
{
    std::stringstream out;
    toCsv(out);
    return out;
}

template<class Archive>
    void TimeId::serialize(Archive& ar, [[maybe_unused]] unsigned int version)
    {

        ar& CEREAL_NVP(time);
        ar& CEREAL_NVP(trackId);
    }


template void TimeId::serialize(cereal::BinaryInputArchive& ar, unsigned int version);
template void TimeId::serialize(cereal::BinaryOutputArchive& ar, unsigned int version);
template void TimeId::serialize(cereal::IntrospectionArchive& ar, unsigned int version);
template void TimeId::serialize(cereal::JSONInputArchive& ar, unsigned int version);
template void TimeId::serialize(cereal::JSONOutputArchive& ar, unsigned int version);
template void TimeId::serialize(cereal::PortableBinaryInputArchive& ar, unsigned int version);
template void TimeId::serialize(cereal::PortableBinaryOutputArchive& ar, unsigned int version);

std::ostream& operator<< (std::ostream& os, const TimeId& timeid)
{
    timeid.toCsv(os);
    return os;
}

bool operator== (const TimeId &lhs, const TimeId &rhs)
{
    return (  lhs.time== rhs.time &&
     lhs.trackId== rhs.trackId 
    );
}

std::size_t TimeId::hash() const noexcept
{
    std::size_t result = 17;
    result = result * 31 + std::hash<double>()(time);
    result = result * 31 + std::hash<int64_t>()(trackId);
    return result;
}

 } // namespace av
 } // namespace msg
 } // namespace world_types


CEREAL_BIND_TO_ARCHIVES(av::msg::world_types::TimeId)

 namespace av {
 namespace msg {
 namespace world_types {


Route::Route() = default;

Route::Route(
    bool isOppositeDirection_arg ,
    std::vector<av::msg::world_types::LaneGroup> groups_arg ,
    av::msg::SE2 currentGoal_arg 
    ):
    isOppositeDirection { std::move(isOppositeDirection_arg) } // NOLINT
      ,
    groups { std::move(groups_arg) } // NOLINT
      ,
    currentGoal { std::move(currentGoal_arg) } // NOLINT
      
    {
}

Route::Route(const Route&) = default;

// NOLINTNEXTLINE
Route::Route(Route&&) = default;

Route& Route::operator=(const Route&) = default;

// NOLINTNEXTLINE
Route& Route::operator=(Route&&) = default;

void Route::toCsv(std::ostream& out) const
{
    out.precision(16);
    out << std::boolalpha;
    out << isOppositeDirection;out << ",";
    // Enclose sequence in double quotations and curly brackets to keep CSV line length uniform
        
        out << "\"{{";
        
        if (!groups.empty())
        {
            std::copy(groups.begin(), groups.end() - 1, std::ostream_iterator<av::msg::world_types::LaneGroup>(out, "},{"));
            std::copy(groups.end()-1, groups.end(), std::ostream_iterator<av::msg::world_types::LaneGroup>(out, ""));
        }
        out << "}}\"";out << ",";
    out << currentGoal;
    
}

std::stringstream Route::toStringstream() const
{
    std::stringstream out;
    toCsv(out);
    return out;
}

template<class Archive>
    void Route::serialize(Archive& ar, [[maybe_unused]] unsigned int version)
    {

        ar& CEREAL_NVP(isOppositeDirection);
        ar& CEREAL_NVP(groups);
        ar& CEREAL_NVP(currentGoal);
    }


template void Route::serialize(cereal::BinaryInputArchive& ar, unsigned int version);
template void Route::serialize(cereal::BinaryOutputArchive& ar, unsigned int version);
template void Route::serialize(cereal::IntrospectionArchive& ar, unsigned int version);
template void Route::serialize(cereal::JSONInputArchive& ar, unsigned int version);
template void Route::serialize(cereal::JSONOutputArchive& ar, unsigned int version);
template void Route::serialize(cereal::PortableBinaryInputArchive& ar, unsigned int version);
template void Route::serialize(cereal::PortableBinaryOutputArchive& ar, unsigned int version);

std::ostream& operator<< (std::ostream& os, const Route& route)
{
    route.toCsv(os);
    return os;
}

bool operator== (const Route &lhs, const Route &rhs)
{
    return (  lhs.isOppositeDirection== rhs.isOppositeDirection &&
     lhs.groups== rhs.groups &&
     lhs.currentGoal== rhs.currentGoal 
    );
}

std::size_t Route::hash() const noexcept
{
    std::size_t result = 17;
    result = result * 31 + std::hash<bool>()(isOppositeDirection);
    result = result * 31 + hashVector(groups);
    result = result * 31 + std::hash<av::msg::SE2>()(currentGoal);
    return result;
}

 } // namespace av
 } // namespace msg
 } // namespace world_types


CEREAL_BIND_TO_ARCHIVES(av::msg::world_types::Route)

 namespace av {
 namespace msg {
 namespace world_types {


TrafficLights::TrafficLights() = default;

TrafficLights::TrafficLights(
    int64_t utime_arg ,
    std::string intersectionId_arg ,
    std::string fromRoadblockId_arg ,
    av::msg::world_types::TrafficLight mainLightInfo_arg ,
    std::vector<std::string> toRoadblockIds_arg ,
    std::vector<av::msg::world_types::TrafficLight> dedicatedLightInfo_arg 
    ):
    utime { std::move(utime_arg) } // NOLINT
      ,
    intersectionId { std::move(intersectionId_arg) } // NOLINT
      ,
    fromRoadblockId { std::move(fromRoadblockId_arg) } // NOLINT
      ,
    mainLightInfo { std::move(mainLightInfo_arg) } // NOLINT
      ,
    toRoadblockIds { std::move(toRoadblockIds_arg) } // NOLINT
      ,
    dedicatedLightInfo { std::move(dedicatedLightInfo_arg) } // NOLINT
      
    {
}

TrafficLights::TrafficLights(const TrafficLights&) = default;

// NOLINTNEXTLINE
TrafficLights::TrafficLights(TrafficLights&&) = default;

TrafficLights& TrafficLights::operator=(const TrafficLights&) = default;

// NOLINTNEXTLINE
TrafficLights& TrafficLights::operator=(TrafficLights&&) = default;

void TrafficLights::toCsv(std::ostream& out) const
{
    out.precision(16);
    out << std::boolalpha;
    out << utime;out << ",";
    out << "\"";
        out << intersectionId;
        out << "\"";out << ",";
    out << "\"";
        out << fromRoadblockId;
        out << "\"";out << ",";
    out << mainLightInfo;out << ",";
    // Enclose sequence in double quotations and curly brackets to keep CSV line length uniform
        
        out << "\"{{";
        
        if (!toRoadblockIds.empty())
        {
            std::copy(toRoadblockIds.begin(), toRoadblockIds.end() - 1, std::ostream_iterator<std::string>(out, "},{"));
            std::copy(toRoadblockIds.end()-1, toRoadblockIds.end(), std::ostream_iterator<std::string>(out, ""));
        }
        out << "}}\"";out << ",";
    // Enclose sequence in double quotations and curly brackets to keep CSV line length uniform
        
        out << "\"{{";
        
        if (!dedicatedLightInfo.empty())
        {
            std::copy(dedicatedLightInfo.begin(), dedicatedLightInfo.end() - 1, std::ostream_iterator<av::msg::world_types::TrafficLight>(out, "},{"));
            std::copy(dedicatedLightInfo.end()-1, dedicatedLightInfo.end(), std::ostream_iterator<av::msg::world_types::TrafficLight>(out, ""));
        }
        out << "}}\"";
    
}

std::stringstream TrafficLights::toStringstream() const
{
    std::stringstream out;
    toCsv(out);
    return out;
}

template<class Archive>
    void TrafficLights::serialize(Archive& ar, [[maybe_unused]] unsigned int version)
    {

        ar& CEREAL_NVP(utime);
        ar& CEREAL_NVP(intersectionId);
        ar& CEREAL_NVP(fromRoadblockId);
        ar& CEREAL_NVP(mainLightInfo);
        ar& CEREAL_NVP(toRoadblockIds);
        ar& CEREAL_NVP(dedicatedLightInfo);
    }


template void TrafficLights::serialize(cereal::BinaryInputArchive& ar, unsigned int version);
template void TrafficLights::serialize(cereal::BinaryOutputArchive& ar, unsigned int version);
template void TrafficLights::serialize(cereal::IntrospectionArchive& ar, unsigned int version);
template void TrafficLights::serialize(cereal::JSONInputArchive& ar, unsigned int version);
template void TrafficLights::serialize(cereal::JSONOutputArchive& ar, unsigned int version);
template void TrafficLights::serialize(cereal::PortableBinaryInputArchive& ar, unsigned int version);
template void TrafficLights::serialize(cereal::PortableBinaryOutputArchive& ar, unsigned int version);

std::ostream& operator<< (std::ostream& os, const TrafficLights& trafficlights)
{
    trafficlights.toCsv(os);
    return os;
}

bool operator== (const TrafficLights &lhs, const TrafficLights &rhs)
{
    return (  lhs.utime== rhs.utime &&
     lhs.intersectionId== rhs.intersectionId &&
     lhs.fromRoadblockId== rhs.fromRoadblockId &&
     lhs.mainLightInfo== rhs.mainLightInfo &&
     lhs.toRoadblockIds== rhs.toRoadblockIds &&
     lhs.dedicatedLightInfo== rhs.dedicatedLightInfo 
    );
}

std::size_t TrafficLights::hash() const noexcept
{
    std::size_t result = 17;
    result = result * 31 + std::hash<int64_t>()(utime);
    result = result * 31 + std::hash<std::string>()(intersectionId);
    result = result * 31 + std::hash<std::string>()(fromRoadblockId);
    result = result * 31 + std::hash<av::msg::world_types::TrafficLight>()(mainLightInfo);
    result = result * 31 + hashVector(toRoadblockIds);
    result = result * 31 + hashVector(dedicatedLightInfo);
    return result;
}

 } // namespace av
 } // namespace msg
 } // namespace world_types


CEREAL_BIND_TO_ARCHIVES(av::msg::world_types::TrafficLights)

 namespace av {
 namespace msg {
 namespace world_types {


WorldTrack::WorldTrack() = default;

WorldTrack::WorldTrack(
    av::msg::world_types::TrackInfoBase trackBase_arg ,
    av::msg::world_types::TurnSignal turnSignal_arg ,
    av::msg::world_types::Route route_arg ,
    std::vector<av::msg::SE2> waypoint_arg ,
    std::vector<std::string> adjacentLanes_arg ,
    int64_t trackAhead_arg ,
    std::vector<int64_t> tracksBlocked_arg ,
    std::vector<av::msg::world_types::AccelConstraint> accelConstraints_arg 
    ):
    trackBase { std::move(trackBase_arg) } // NOLINT
      ,
    turnSignal { std::move(turnSignal_arg) } // NOLINT
      ,
    route { std::move(route_arg) } // NOLINT
      ,
    waypoint { std::move(waypoint_arg) } // NOLINT
      ,
    adjacentLanes { std::move(adjacentLanes_arg) } // NOLINT
      ,
    trackAhead { std::move(trackAhead_arg) } // NOLINT
      ,
    tracksBlocked { std::move(tracksBlocked_arg) } // NOLINT
      ,
    accelConstraints { std::move(accelConstraints_arg) } // NOLINT
      
    {
}

WorldTrack::WorldTrack(const WorldTrack&) = default;

// NOLINTNEXTLINE
WorldTrack::WorldTrack(WorldTrack&&) = default;

WorldTrack& WorldTrack::operator=(const WorldTrack&) = default;

// NOLINTNEXTLINE
WorldTrack& WorldTrack::operator=(WorldTrack&&) = default;

void WorldTrack::toCsv(std::ostream& out) const
{
    out.precision(16);
    out << std::boolalpha;
    out << trackBase;out << ",";
    out << turnSignal;out << ",";
    out << route;out << ",";
    // Enclose sequence in double quotations and curly brackets to keep CSV line length uniform
        
        out << "\"{{";
        
        if (!waypoint.empty())
        {
            std::copy(waypoint.begin(), waypoint.end() - 1, std::ostream_iterator<av::msg::SE2>(out, "},{"));
            std::copy(waypoint.end()-1, waypoint.end(), std::ostream_iterator<av::msg::SE2>(out, ""));
        }
        out << "}}\"";out << ",";
    // Enclose sequence in double quotations and curly brackets to keep CSV line length uniform
        
        out << "\"{{";
        
        if (!adjacentLanes.empty())
        {
            std::copy(adjacentLanes.begin(), adjacentLanes.end() - 1, std::ostream_iterator<std::string>(out, "},{"));
            std::copy(adjacentLanes.end()-1, adjacentLanes.end(), std::ostream_iterator<std::string>(out, ""));
        }
        out << "}}\"";out << ",";
    out << trackAhead;out << ",";
    // Enclose sequence in double quotations and curly brackets to keep CSV line length uniform
        
        out << "\"{{";
        
        if (!tracksBlocked.empty())
        {
            std::copy(tracksBlocked.begin(), tracksBlocked.end() - 1, std::ostream_iterator<int64_t>(out, "},{"));
            std::copy(tracksBlocked.end()-1, tracksBlocked.end(), std::ostream_iterator<int64_t>(out, ""));
        }
        out << "}}\"";out << ",";
    // Enclose sequence in double quotations and curly brackets to keep CSV line length uniform
        
        out << "\"{{";
        
        if (!accelConstraints.empty())
        {
            std::copy(accelConstraints.begin(), accelConstraints.end() - 1, std::ostream_iterator<av::msg::world_types::AccelConstraint>(out, "},{"));
            std::copy(accelConstraints.end()-1, accelConstraints.end(), std::ostream_iterator<av::msg::world_types::AccelConstraint>(out, ""));
        }
        out << "}}\"";
    
}

std::stringstream WorldTrack::toStringstream() const
{
    std::stringstream out;
    toCsv(out);
    return out;
}

template<class Archive>
    void WorldTrack::serialize(Archive& ar, [[maybe_unused]] unsigned int version)
    {

        ar& CEREAL_NVP(trackBase);
        ar& CEREAL_NVP(turnSignal);
        ar& CEREAL_NVP(route);
        ar& CEREAL_NVP(waypoint);
        ar& CEREAL_NVP(adjacentLanes);
        ar& CEREAL_NVP(trackAhead);
        ar& CEREAL_NVP(tracksBlocked);
        ar& CEREAL_NVP(accelConstraints);
    }


template void WorldTrack::serialize(cereal::BinaryInputArchive& ar, unsigned int version);
template void WorldTrack::serialize(cereal::BinaryOutputArchive& ar, unsigned int version);
template void WorldTrack::serialize(cereal::IntrospectionArchive& ar, unsigned int version);
template void WorldTrack::serialize(cereal::JSONInputArchive& ar, unsigned int version);
template void WorldTrack::serialize(cereal::JSONOutputArchive& ar, unsigned int version);
template void WorldTrack::serialize(cereal::PortableBinaryInputArchive& ar, unsigned int version);
template void WorldTrack::serialize(cereal::PortableBinaryOutputArchive& ar, unsigned int version);

std::ostream& operator<< (std::ostream& os, const WorldTrack& worldtrack)
{
    worldtrack.toCsv(os);
    return os;
}

bool operator== (const WorldTrack &lhs, const WorldTrack &rhs)
{
    return (  lhs.trackBase== rhs.trackBase &&
     lhs.turnSignal== rhs.turnSignal &&
     lhs.route== rhs.route &&
     lhs.waypoint== rhs.waypoint &&
     lhs.adjacentLanes== rhs.adjacentLanes &&
     lhs.trackAhead== rhs.trackAhead &&
     lhs.tracksBlocked== rhs.tracksBlocked &&
     lhs.accelConstraints== rhs.accelConstraints 
    );
}

std::size_t WorldTrack::hash() const noexcept
{
    std::size_t result = 17;
    result = result * 31 + std::hash<av::msg::world_types::TrackInfoBase>()(trackBase);
    result = result * 31 + std::hash<av::msg::world_types::TurnSignal>()(turnSignal);
    result = result * 31 + std::hash<av::msg::world_types::Route>()(route);
    result = result * 31 + hashVector(waypoint);
    result = result * 31 + hashVector(adjacentLanes);
    result = result * 31 + std::hash<int64_t>()(trackAhead);
    result = result * 31 + hashVector(tracksBlocked);
    result = result * 31 + hashVector(accelConstraints);
    return result;
}

 } // namespace av
 } // namespace msg
 } // namespace world_types


CEREAL_BIND_TO_ARCHIVES(av::msg::world_types::WorldTrack)

 namespace av {
 namespace msg {
 namespace world_types {


World::World() = default;

World::World(
    int64_t utime_arg ,
    int64_t egoId_arg ,
    std::vector<av::msg::world_types::WorldTrack> tracks_arg ,
    av::msg::world_types::TrafficLights trafficLights_arg 
    ):
    utime { std::move(utime_arg) } // NOLINT
      ,
    egoId { std::move(egoId_arg) } // NOLINT
      ,
    tracks { std::move(tracks_arg) } // NOLINT
      ,
    trafficLights { std::move(trafficLights_arg) } // NOLINT
      
    {
}

World::World(const World&) = default;

// NOLINTNEXTLINE
World::World(World&&) = default;

World& World::operator=(const World&) = default;

// NOLINTNEXTLINE
World& World::operator=(World&&) = default;

void World::toCsv(std::ostream& out) const
{
    out.precision(16);
    out << std::boolalpha;
    out << utime;out << ",";
    out << egoId;out << ",";
    // Enclose sequence in double quotations and curly brackets to keep CSV line length uniform
        
        out << "\"{{";
        
        if (!tracks.empty())
        {
            std::copy(tracks.begin(), tracks.end() - 1, std::ostream_iterator<av::msg::world_types::WorldTrack>(out, "},{"));
            std::copy(tracks.end()-1, tracks.end(), std::ostream_iterator<av::msg::world_types::WorldTrack>(out, ""));
        }
        out << "}}\"";out << ",";
    out << trafficLights;
    
}

std::stringstream World::toStringstream() const
{
    std::stringstream out;
    toCsv(out);
    return out;
}

template<class Archive>
    void World::serialize(Archive& ar, [[maybe_unused]] unsigned int version)
    {

        ar& CEREAL_NVP(utime);
        ar& CEREAL_NVP(egoId);
        ar& CEREAL_NVP(tracks);
        ar& CEREAL_NVP(trafficLights);
    }


template void World::serialize(cereal::BinaryInputArchive& ar, unsigned int version);
template void World::serialize(cereal::BinaryOutputArchive& ar, unsigned int version);
template void World::serialize(cereal::IntrospectionArchive& ar, unsigned int version);
template void World::serialize(cereal::JSONInputArchive& ar, unsigned int version);
template void World::serialize(cereal::JSONOutputArchive& ar, unsigned int version);
template void World::serialize(cereal::PortableBinaryInputArchive& ar, unsigned int version);
template void World::serialize(cereal::PortableBinaryOutputArchive& ar, unsigned int version);

std::ostream& operator<< (std::ostream& os, const World& world)
{
    world.toCsv(os);
    return os;
}

bool operator== (const World &lhs, const World &rhs)
{
    return (  lhs.utime== rhs.utime &&
     lhs.egoId== rhs.egoId &&
     lhs.tracks== rhs.tracks &&
     lhs.trafficLights== rhs.trafficLights 
    );
}

std::size_t World::hash() const noexcept
{
    std::size_t result = 17;
    result = result * 31 + std::hash<int64_t>()(utime);
    result = result * 31 + std::hash<int64_t>()(egoId);
    result = result * 31 + hashVector(tracks);
    result = result * 31 + std::hash<av::msg::world_types::TrafficLights>()(trafficLights);
    return result;
}

 } // namespace av
 } // namespace msg
 } // namespace world_types


CEREAL_BIND_TO_ARCHIVES(av::msg::world_types::World)

 namespace av {
 namespace msg {
 namespace world_types {


WorldTrajectory::WorldTrajectory() = default;

WorldTrajectory::WorldTrajectory(
    std::vector<av::msg::world_types::World> trajectory_arg 
    ):
    trajectory { std::move(trajectory_arg) } // NOLINT
      
    {
}

WorldTrajectory::WorldTrajectory(const WorldTrajectory&) = default;

// NOLINTNEXTLINE
WorldTrajectory::WorldTrajectory(WorldTrajectory&&) = default;

WorldTrajectory& WorldTrajectory::operator=(const WorldTrajectory&) = default;

// NOLINTNEXTLINE
WorldTrajectory& WorldTrajectory::operator=(WorldTrajectory&&) = default;

void WorldTrajectory::toCsv(std::ostream& out) const
{
    out.precision(16);
    out << std::boolalpha;
    // Enclose sequence in double quotations and curly brackets to keep CSV line length uniform
        
        out << "\"{{";
        
        if (!trajectory.empty())
        {
            std::copy(trajectory.begin(), trajectory.end() - 1, std::ostream_iterator<av::msg::world_types::World>(out, "},{"));
            std::copy(trajectory.end()-1, trajectory.end(), std::ostream_iterator<av::msg::world_types::World>(out, ""));
        }
        out << "}}\"";
    
}

std::stringstream WorldTrajectory::toStringstream() const
{
    std::stringstream out;
    toCsv(out);
    return out;
}

template<class Archive>
    void WorldTrajectory::serialize(Archive& ar, [[maybe_unused]] unsigned int version)
    {

        ar& CEREAL_NVP(trajectory);
    }


template void WorldTrajectory::serialize(cereal::BinaryInputArchive& ar, unsigned int version);
template void WorldTrajectory::serialize(cereal::BinaryOutputArchive& ar, unsigned int version);
template void WorldTrajectory::serialize(cereal::IntrospectionArchive& ar, unsigned int version);
template void WorldTrajectory::serialize(cereal::JSONInputArchive& ar, unsigned int version);
template void WorldTrajectory::serialize(cereal::JSONOutputArchive& ar, unsigned int version);
template void WorldTrajectory::serialize(cereal::PortableBinaryInputArchive& ar, unsigned int version);
template void WorldTrajectory::serialize(cereal::PortableBinaryOutputArchive& ar, unsigned int version);

std::ostream& operator<< (std::ostream& os, const WorldTrajectory& worldtrajectory)
{
    worldtrajectory.toCsv(os);
    return os;
}

bool operator== (const WorldTrajectory &lhs, const WorldTrajectory &rhs)
{
    return (  lhs.trajectory== rhs.trajectory 
    );
}

std::size_t WorldTrajectory::hash() const noexcept
{
    std::size_t result = 17;
    result = result * 31 + hashVector(trajectory);
    return result;
}

 } // namespace av
 } // namespace msg
 } // namespace world_types


CEREAL_BIND_TO_ARCHIVES(av::msg::world_types::WorldTrajectory)




 namespace av {
 namespace msg {


WorldManager::WorldManager() = default;

WorldManager::WorldManager(
    int8_t rollingCounter_arg ,
    av::msg::world_types::World current_arg ,
    std::vector<av::msg::world_types::WorldTrajectory> predictions_arg ,
    std::vector<av::msg::NamedVectorOfDouble> debugVariables_arg 
    ):
    rollingCounter { std::move(rollingCounter_arg) } // NOLINT
      ,current { std::move(current_arg) } // NOLINT
      ,predictions { std::move(predictions_arg) } // NOLINT
      ,debugVariables { std::move(debugVariables_arg) } // NOLINT
      {
}

WorldManager::WorldManager(const WorldManager&) = default;

// NOLINTNEXTLINE
WorldManager::WorldManager(WorldManager&&) = default;

WorldManager& WorldManager::operator=(const WorldManager&) = default;

// NOLINTNEXTLINE
WorldManager& WorldManager::operator=(WorldManager&&) = default;

void WorldManager::toCsv(std::ostream& out) const
{
    out.precision(16);
    out << std::boolalpha;
    // Cast to int16_t to print as integer instead of char
        out << static_cast<int16_t>(rollingCounter);out << ",";
    out << current;out << ",";
    // Enclose sequence in double quotations and curly brackets to keep CSV line length uniform
        
        out << "\"{{";
        
        if (!predictions.empty())
        {
            std::copy(predictions.begin(), predictions.end() - 1, std::ostream_iterator<av::msg::world_types::WorldTrajectory>(out, "},{"));
            std::copy(predictions.end()-1, predictions.end(), std::ostream_iterator<av::msg::world_types::WorldTrajectory>(out, ""));
        }
        out << "}}\"";out << ",";
    // Enclose sequence in double quotations and curly brackets to keep CSV line length uniform
        
        out << "\"{{";
        
        if (!debugVariables.empty())
        {
            std::copy(debugVariables.begin(), debugVariables.end() - 1, std::ostream_iterator<av::msg::NamedVectorOfDouble>(out, "},{"));
            std::copy(debugVariables.end()-1, debugVariables.end(), std::ostream_iterator<av::msg::NamedVectorOfDouble>(out, ""));
        }
        out << "}}\"";
    
}

std::stringstream WorldManager::toStringstream() const
{
    std::stringstream out;
    toCsv(out);
    return out;
}

template<class Archive>
    void WorldManager::serialize(Archive& ar, const unsigned int version)
{
    if (version >= 1) {
        ar& CEREAL_NVP(rollingCounter);
        ar& CEREAL_NVP(current);
        ar& CEREAL_NVP(predictions);
    }
    if (version >= 2) {
        ar& CEREAL_NVP(debugVariables);
    }
}


template void WorldManager::serialize(cereal::BinaryInputArchive& ar, unsigned int version);
template void WorldManager::serialize(cereal::BinaryOutputArchive& ar, unsigned int version);
template void WorldManager::serialize(cereal::IntrospectionArchive& ar, unsigned int version);
template void WorldManager::serialize(cereal::JSONInputArchive& ar, unsigned int version);
template void WorldManager::serialize(cereal::JSONOutputArchive& ar, unsigned int version);
template void WorldManager::serialize(cereal::PortableBinaryInputArchive& ar, unsigned int version);
template void WorldManager::serialize(cereal::PortableBinaryOutputArchive& ar, unsigned int version);

std::ostream& operator<< (std::ostream& os, const WorldManager& worldmanager)
{
    worldmanager.toCsv(os);
    return os;
}

bool operator== (const WorldManager &lhs, const WorldManager &rhs)
{
    return (  lhs.rollingCounter== rhs.rollingCounter &&
     lhs.current== rhs.current &&
     lhs.predictions== rhs.predictions &&
     lhs.debugVariables== rhs.debugVariables 
    );
}

std::size_t WorldManager::hash() const noexcept
{
    std::size_t result = 17;
    result = result * 31 + std::hash<int8_t>()(rollingCounter);
    result = result * 31 + std::hash<av::msg::world_types::World>()(current);
    result = result * 31 + hashVector(predictions);
    result = result * 31 + hashVector(debugVariables);
    return result;
}

 } // namespace av
 } // namespace msg


CEREAL_BIND_TO_ARCHIVES(av::msg::WorldManager)

