#include <interfaces/WorldTrackDecisions/WorldTrackDecisions.h>

#include <iterator>
#include <type_traits>

#include <cereal/cereal.hpp>
#include <cereal/archives/json.hpp>
#include <cereal/archives/binary.hpp>

#include <cereal/types/map.hpp>
#include <cereal/types/vector.hpp>
#include <cereal/types/array.hpp>
#include <cereal/types/string.hpp>
#include <cereal/types/complex.hpp>

#include <ODatum/IntrospectionArchive.h>
#include <ODatum/Hash.h>


 namespace av {
 namespace msg {
 namespace plan_ctrl {


TimeLimit::TimeLimit() = default;

TimeLimit::TimeLimit(
    int64_t upper_arg ,
    int64_t lower_arg 
    ):
    upper { std::move(upper_arg) } // NOLINT
      ,
    lower { std::move(lower_arg) } // NOLINT
      
    {
}

TimeLimit::TimeLimit(const TimeLimit&) = default;

// NOLINTNEXTLINE
TimeLimit::TimeLimit(TimeLimit&&) = default;

TimeLimit& TimeLimit::operator=(const TimeLimit&) = default;

// NOLINTNEXTLINE
TimeLimit& TimeLimit::operator=(TimeLimit&&) = default;

void TimeLimit::toCsv(std::ostream& out) const
{
    out.precision(16);
    out << std::boolalpha;
    out << upper;out << ",";
    out << lower;
    
}

std::stringstream TimeLimit::toStringstream() const
{
    std::stringstream out;
    toCsv(out);
    return out;
}

template<class Archive>
    void TimeLimit::serialize(Archive& ar, [[maybe_unused]] unsigned int version)
    {

        ar& CEREAL_NVP(upper);
        ar& CEREAL_NVP(lower);
    }


template void TimeLimit::serialize(cereal::BinaryInputArchive& ar, unsigned int version);
template void TimeLimit::serialize(cereal::BinaryOutputArchive& ar, unsigned int version);
template void TimeLimit::serialize(cereal::IntrospectionArchive& ar, unsigned int version);
template void TimeLimit::serialize(cereal::JSONInputArchive& ar, unsigned int version);
template void TimeLimit::serialize(cereal::JSONOutputArchive& ar, unsigned int version);
template void TimeLimit::serialize(cereal::PortableBinaryInputArchive& ar, unsigned int version);
template void TimeLimit::serialize(cereal::PortableBinaryOutputArchive& ar, unsigned int version);

std::ostream& operator<< (std::ostream& os, const TimeLimit& timelimit)
{
    timelimit.toCsv(os);
    return os;
}

bool operator== (const TimeLimit &lhs, const TimeLimit &rhs)
{
    return (  lhs.upper== rhs.upper &&
     lhs.lower== rhs.lower 
    );
}

std::size_t TimeLimit::hash() const noexcept
{
    std::size_t result = 17;
    result = result * 31 + std::hash<int64_t>()(upper);
    result = result * 31 + std::hash<int64_t>()(lower);
    return result;
}

 } // namespace av
 } // namespace msg
 } // namespace plan_ctrl


CEREAL_BIND_TO_ARCHIVES(av::msg::plan_ctrl::TimeLimit)

 namespace av {
 namespace msg {
 namespace plan_ctrl {


TrackDecision::TrackDecision() = default;

TrackDecision::TrackDecision(
    SIDE_DECISION side_arg ,
    TimeLimit side_time_limit_arg ,
    PASSING_DECISION passing_arg ,
    uint64_t obstacle_id_arg ,
    std::vector<uint64_t> track_ids_arg 
    ):
    side { std::move(side_arg) } // NOLINT
      ,
    side_time_limit { std::move(side_time_limit_arg) } // NOLINT
      ,
    passing { std::move(passing_arg) } // NOLINT
      ,
    obstacle_id { std::move(obstacle_id_arg) } // NOLINT
      ,
    track_ids { std::move(track_ids_arg) } // NOLINT
      
    {
}

TrackDecision::TrackDecision(const TrackDecision&) = default;

// NOLINTNEXTLINE
TrackDecision::TrackDecision(TrackDecision&&) = default;

TrackDecision& TrackDecision::operator=(const TrackDecision&) = default;

// NOLINTNEXTLINE
TrackDecision& TrackDecision::operator=(TrackDecision&&) = default;

void TrackDecision::toCsv(std::ostream& out) const
{
    out.precision(16);
    out << std::boolalpha;
    out << static_cast<uint32_t>(side);out << ",";
    out << side_time_limit;out << ",";
    out << static_cast<uint32_t>(passing);out << ",";
    out << obstacle_id;out << ",";
    // Enclose sequence in double quotations and curly brackets to keep CSV line length uniform
        
        out << "\"{{";
        
        if (!track_ids.empty())
        {
            std::copy(track_ids.begin(), track_ids.end() - 1, std::ostream_iterator<uint64_t>(out, "},{"));
            std::copy(track_ids.end()-1, track_ids.end(), std::ostream_iterator<uint64_t>(out, ""));
        }
        out << "}}\"";
    
}

std::stringstream TrackDecision::toStringstream() const
{
    std::stringstream out;
    toCsv(out);
    return out;
}

template<class Archive>
    void TrackDecision::serialize(Archive& ar, unsigned int version)
{
	if (version >= 1)
	{
		ar& CEREAL_NVP(side);
		ar& CEREAL_NVP(side_time_limit);
		ar& CEREAL_NVP(passing);
		ar& CEREAL_NVP(obstacle_id);		
	}
	if (version >= 2)
	{
		ar& CEREAL_NVP(track_ids);
	}
}



template void TrackDecision::serialize(cereal::BinaryInputArchive& ar, unsigned int version);
template void TrackDecision::serialize(cereal::BinaryOutputArchive& ar, unsigned int version);
template void TrackDecision::serialize(cereal::IntrospectionArchive& ar, unsigned int version);
template void TrackDecision::serialize(cereal::JSONInputArchive& ar, unsigned int version);
template void TrackDecision::serialize(cereal::JSONOutputArchive& ar, unsigned int version);
template void TrackDecision::serialize(cereal::PortableBinaryInputArchive& ar, unsigned int version);
template void TrackDecision::serialize(cereal::PortableBinaryOutputArchive& ar, unsigned int version);

std::ostream& operator<< (std::ostream& os, const TrackDecision& trackdecision)
{
    trackdecision.toCsv(os);
    return os;
}

bool operator== (const TrackDecision &lhs, const TrackDecision &rhs)
{
    return (  lhs.side== rhs.side &&
     lhs.side_time_limit== rhs.side_time_limit &&
     lhs.passing== rhs.passing &&
     lhs.obstacle_id== rhs.obstacle_id &&
     lhs.track_ids== rhs.track_ids 
    );
}

std::size_t TrackDecision::hash() const noexcept
{
    std::size_t result = 17;
    result = result * 31 + std::hash<SIDE_DECISION>()(side);
    result = result * 31 + std::hash<TimeLimit>()(side_time_limit);
    result = result * 31 + std::hash<PASSING_DECISION>()(passing);
    result = result * 31 + std::hash<uint64_t>()(obstacle_id);
    result = result * 31 + hashVector(track_ids);
    return result;
}

 } // namespace av
 } // namespace msg
 } // namespace plan_ctrl


CEREAL_BIND_TO_ARCHIVES(av::msg::plan_ctrl::TrackDecision)




 namespace av {
 namespace msg {
 namespace plan_ctrl {


WorldTrackDecisions::WorldTrackDecisions() = default;

WorldTrackDecisions::WorldTrackDecisions(
    std::vector<TrackDecision> decisions_arg 
    ):
    decisions { std::move(decisions_arg) } // NOLINT
      {
}

WorldTrackDecisions::WorldTrackDecisions(const WorldTrackDecisions&) = default;

// NOLINTNEXTLINE
WorldTrackDecisions::WorldTrackDecisions(WorldTrackDecisions&&) = default;

WorldTrackDecisions& WorldTrackDecisions::operator=(const WorldTrackDecisions&) = default;

// NOLINTNEXTLINE
WorldTrackDecisions& WorldTrackDecisions::operator=(WorldTrackDecisions&&) = default;

void WorldTrackDecisions::toCsv(std::ostream& out) const
{
    out.precision(16);
    out << std::boolalpha;
    // Enclose sequence in double quotations and curly brackets to keep CSV line length uniform
        
        out << "\"{{";
        
        if (!decisions.empty())
        {
            std::copy(decisions.begin(), decisions.end() - 1, std::ostream_iterator<TrackDecision>(out, "},{"));
            std::copy(decisions.end()-1, decisions.end(), std::ostream_iterator<TrackDecision>(out, ""));
        }
        out << "}}\"";
    
}

std::stringstream WorldTrackDecisions::toStringstream() const
{
    std::stringstream out;
    toCsv(out);
    return out;
}

template<class Archive>
    void WorldTrackDecisions::serialize(Archive& ar, [[maybe_unused]] unsigned int version)
    {

        ar& CEREAL_NVP(decisions);
    }


template void WorldTrackDecisions::serialize(cereal::BinaryInputArchive& ar, unsigned int version);
template void WorldTrackDecisions::serialize(cereal::BinaryOutputArchive& ar, unsigned int version);
template void WorldTrackDecisions::serialize(cereal::IntrospectionArchive& ar, unsigned int version);
template void WorldTrackDecisions::serialize(cereal::JSONInputArchive& ar, unsigned int version);
template void WorldTrackDecisions::serialize(cereal::JSONOutputArchive& ar, unsigned int version);
template void WorldTrackDecisions::serialize(cereal::PortableBinaryInputArchive& ar, unsigned int version);
template void WorldTrackDecisions::serialize(cereal::PortableBinaryOutputArchive& ar, unsigned int version);

std::ostream& operator<< (std::ostream& os, const WorldTrackDecisions& worldtrackdecisions)
{
    worldtrackdecisions.toCsv(os);
    return os;
}

bool operator== (const WorldTrackDecisions &lhs, const WorldTrackDecisions &rhs)
{
    return (  lhs.decisions== rhs.decisions 
    );
}

std::size_t WorldTrackDecisions::hash() const noexcept
{
    std::size_t result = 17;
    result = result * 31 + hashVector(decisions);
    return result;
}

 } // namespace av
 } // namespace msg
 } // namespace plan_ctrl


CEREAL_BIND_TO_ARCHIVES(av::msg::plan_ctrl::WorldTrackDecisions)

