#include <interfaces/av_state_avt/av_state_avt.h>

#include <iterator>
#include <type_traits>

#include <cereal/cereal.hpp>
#include <cereal/archives/json.hpp>
#include <cereal/archives/binary.hpp>

#include <cereal/types/map.hpp>
#include <cereal/types/vector.hpp>
#include <cereal/types/array.hpp>
#include <cereal/types/string.hpp>
#include <cereal/types/complex.hpp>

#include <ODatum/IntrospectionArchive.h>
#include <ODatum/Hash.h>





 namespace av {
 namespace msg {


av_state_avt::av_state_avt() = default;

av_state_avt::av_state_avt(
    int64_t timestamp_arg ,
    av_status_avt avStatus_arg ,
    TransmissionStates currentGear_arg ,
    TurnSignal turnSignal_arg ,
    bool brakePressed_arg ,
    bool reverseLight_arg ,
    double brakeTorque_arg ,
    double roadWheelAngle_arg ,
    double steeringWheelAngle_arg ,
    double throttleTorque_arg ,
    double vehicleSpeed_arg ,
    double yawRate_arg ,
    double odometer_arg ,
    int8_t numTirePressure_arg ,
    std::vector<float> tirePressure_arg ,
    int16_t fuelRange_arg ,
    float fuelLevel_arg ,
    int32_t serviceFaults_arg ,
    double crossTrackError_arg ,
    double headingError_arg ,
    double curvatureRequest_arg ,
    double stationError_arg ,
    double commandedSpeed_arg ,
    double feedbackSpeed_arg ,
    double commandedAcceleration_arg ,
    double actualAcceleration_arg ,
    double desiredAcceleration_arg ,
    double commandedRoadWheelAngle_arg ,
    double compensatedHeadingError_arg ,
    double temporalProjectionIndex_arg ,
    double spatialProjectionIndex_arg 
    ):
    timestamp { std::move(timestamp_arg) } // NOLINT
      ,avStatus { std::move(avStatus_arg) } // NOLINT
      ,currentGear { std::move(currentGear_arg) } // NOLINT
      ,turnSignal { std::move(turnSignal_arg) } // NOLINT
      ,brakePressed { std::move(brakePressed_arg) } // NOLINT
      ,reverseLight { std::move(reverseLight_arg) } // NOLINT
      ,brakeTorque { std::move(brakeTorque_arg) } // NOLINT
      ,roadWheelAngle { std::move(roadWheelAngle_arg) } // NOLINT
      ,steeringWheelAngle { std::move(steeringWheelAngle_arg) } // NOLINT
      ,throttleTorque { std::move(throttleTorque_arg) } // NOLINT
      ,vehicleSpeed { std::move(vehicleSpeed_arg) } // NOLINT
      ,yawRate { std::move(yawRate_arg) } // NOLINT
      ,odometer { std::move(odometer_arg) } // NOLINT
      ,numTirePressure { std::move(numTirePressure_arg) } // NOLINT
      ,tirePressure { std::move(tirePressure_arg) } // NOLINT
      ,fuelRange { std::move(fuelRange_arg) } // NOLINT
      ,fuelLevel { std::move(fuelLevel_arg) } // NOLINT
      ,serviceFaults { std::move(serviceFaults_arg) } // NOLINT
      ,crossTrackError { std::move(crossTrackError_arg) } // NOLINT
      ,headingError { std::move(headingError_arg) } // NOLINT
      ,curvatureRequest { std::move(curvatureRequest_arg) } // NOLINT
      ,stationError { std::move(stationError_arg) } // NOLINT
      ,commandedSpeed { std::move(commandedSpeed_arg) } // NOLINT
      ,feedbackSpeed { std::move(feedbackSpeed_arg) } // NOLINT
      ,commandedAcceleration { std::move(commandedAcceleration_arg) } // NOLINT
      ,actualAcceleration { std::move(actualAcceleration_arg) } // NOLINT
      ,desiredAcceleration { std::move(desiredAcceleration_arg) } // NOLINT
      ,commandedRoadWheelAngle { std::move(commandedRoadWheelAngle_arg) } // NOLINT
      ,compensatedHeadingError { std::move(compensatedHeadingError_arg) } // NOLINT
      ,temporalProjectionIndex { std::move(temporalProjectionIndex_arg) } // NOLINT
      ,spatialProjectionIndex { std::move(spatialProjectionIndex_arg) } // NOLINT
      {
}

av_state_avt::av_state_avt(const av_state_avt&) = default;

// NOLINTNEXTLINE
av_state_avt::av_state_avt(av_state_avt&&) = default;

av_state_avt& av_state_avt::operator=(const av_state_avt&) = default;

// NOLINTNEXTLINE
av_state_avt& av_state_avt::operator=(av_state_avt&&) = default;

void av_state_avt::toCsv(std::ostream& out) const
{
    out.precision(16);
    out << std::boolalpha;
    out << timestamp;out << ",";
    out << avStatus;out << ",";
    out << static_cast<uint32_t>(currentGear);out << ",";
    out << static_cast<uint32_t>(turnSignal);out << ",";
    out << brakePressed;out << ",";
    out << reverseLight;out << ",";
    out << brakeTorque;out << ",";
    out << roadWheelAngle;out << ",";
    out << steeringWheelAngle;out << ",";
    out << throttleTorque;out << ",";
    out << vehicleSpeed;out << ",";
    out << yawRate;out << ",";
    out << odometer;out << ",";
    // Cast to int16_t to print as integer instead of char
        out << static_cast<int16_t>(numTirePressure);out << ",";
    // Enclose sequence in double quotations and curly brackets to keep CSV line length uniform
        
        out << "\"{{";
        
        if (!tirePressure.empty())
        {
            std::copy(tirePressure.begin(), tirePressure.end() - 1, std::ostream_iterator<float>(out, "},{"));
            std::copy(tirePressure.end()-1, tirePressure.end(), std::ostream_iterator<float>(out, ""));
        }
        out << "}}\"";out << ",";
    out << fuelRange;out << ",";
    out << fuelLevel;out << ",";
    out << serviceFaults;out << ",";
    out << crossTrackError;out << ",";
    out << headingError;out << ",";
    out << curvatureRequest;out << ",";
    out << stationError;out << ",";
    out << commandedSpeed;out << ",";
    out << feedbackSpeed;out << ",";
    out << commandedAcceleration;out << ",";
    out << actualAcceleration;out << ",";
    out << desiredAcceleration;out << ",";
    out << commandedRoadWheelAngle;out << ",";
    out << compensatedHeadingError;out << ",";
    out << temporalProjectionIndex;out << ",";
    out << spatialProjectionIndex;
    
}

std::stringstream av_state_avt::toStringstream() const
{
    std::stringstream out;
    toCsv(out);
    return out;
}

template<class Archive>
    void av_state_avt::serialize(Archive& ar, unsigned int version)
{
    if (version >= 1) {
        ar& CEREAL_NVP(timestamp);
        ar& CEREAL_NVP(avStatus);
        ar& CEREAL_NVP(currentGear);
        ar& CEREAL_NVP(turnSignal);
        ar& CEREAL_NVP(brakePressed);
        ar& CEREAL_NVP(reverseLight);
        ar& CEREAL_NVP(brakeTorque);
        ar& CEREAL_NVP(roadWheelAngle);
        ar& CEREAL_NVP(steeringWheelAngle);
        ar& CEREAL_NVP(throttleTorque);
        ar& CEREAL_NVP(vehicleSpeed);
        ar& CEREAL_NVP(yawRate);
    }
    if (version >= 2) {
        ar& CEREAL_NVP(odometer);
        ar& CEREAL_NVP(numTirePressure);
        ar& CEREAL_NVP(tirePressure);
        ar& CEREAL_NVP(fuelRange);
        ar& CEREAL_NVP(fuelLevel);
        ar& CEREAL_NVP(serviceFaults);
    }
    if (version >= 3) {
        ar& CEREAL_NVP(crossTrackError);
        ar& CEREAL_NVP(headingError);
        ar& CEREAL_NVP(curvatureRequest);
        ar& CEREAL_NVP(stationError);
        ar& CEREAL_NVP(commandedSpeed);
        ar& CEREAL_NVP(feedbackSpeed);
        ar& CEREAL_NVP(commandedAcceleration);
        ar& CEREAL_NVP(actualAcceleration);
        ar& CEREAL_NVP(desiredAcceleration);
    }
    if (version == 3) {
        double commandedSteeringAngle;
        double actualSteeringAngle;
        ar& CEREAL_NVP(commandedSteeringAngle);
        ar& CEREAL_NVP(actualSteeringAngle);
    }
    if (version >= 4) {
        ar& CEREAL_NVP(commandedRoadWheelAngle);
    }
    if (version >= 5) {
        ar& CEREAL_NVP(compensatedHeadingError);
        ar& CEREAL_NVP(temporalProjectionIndex);
        ar& CEREAL_NVP(spatialProjectionIndex);
    }
}



template void av_state_avt::serialize(cereal::BinaryInputArchive& ar, unsigned int version);
template void av_state_avt::serialize(cereal::BinaryOutputArchive& ar, unsigned int version);
template void av_state_avt::serialize(cereal::IntrospectionArchive& ar, unsigned int version);
template void av_state_avt::serialize(cereal::JSONInputArchive& ar, unsigned int version);
template void av_state_avt::serialize(cereal::JSONOutputArchive& ar, unsigned int version);
template void av_state_avt::serialize(cereal::PortableBinaryInputArchive& ar, unsigned int version);
template void av_state_avt::serialize(cereal::PortableBinaryOutputArchive& ar, unsigned int version);

std::ostream& operator<< (std::ostream& os, const av_state_avt& av_state_avt)
{
    av_state_avt.toCsv(os);
    return os;
}

bool operator== (const av_state_avt &lhs, const av_state_avt &rhs)
{
    return (  lhs.timestamp== rhs.timestamp &&
     lhs.avStatus== rhs.avStatus &&
     lhs.currentGear== rhs.currentGear &&
     lhs.turnSignal== rhs.turnSignal &&
     lhs.brakePressed== rhs.brakePressed &&
     lhs.reverseLight== rhs.reverseLight &&
     lhs.brakeTorque== rhs.brakeTorque &&
     lhs.roadWheelAngle== rhs.roadWheelAngle &&
     lhs.steeringWheelAngle== rhs.steeringWheelAngle &&
     lhs.throttleTorque== rhs.throttleTorque &&
     lhs.vehicleSpeed== rhs.vehicleSpeed &&
     lhs.yawRate== rhs.yawRate &&
     lhs.odometer== rhs.odometer &&
     lhs.numTirePressure== rhs.numTirePressure &&
     lhs.tirePressure== rhs.tirePressure &&
     lhs.fuelRange== rhs.fuelRange &&
     lhs.fuelLevel== rhs.fuelLevel &&
     lhs.serviceFaults== rhs.serviceFaults &&
     lhs.crossTrackError== rhs.crossTrackError &&
     lhs.headingError== rhs.headingError &&
     lhs.curvatureRequest== rhs.curvatureRequest &&
     lhs.stationError== rhs.stationError &&
     lhs.commandedSpeed== rhs.commandedSpeed &&
     lhs.feedbackSpeed== rhs.feedbackSpeed &&
     lhs.commandedAcceleration== rhs.commandedAcceleration &&
     lhs.actualAcceleration== rhs.actualAcceleration &&
     lhs.desiredAcceleration== rhs.desiredAcceleration &&
     lhs.commandedRoadWheelAngle== rhs.commandedRoadWheelAngle &&
     lhs.compensatedHeadingError== rhs.compensatedHeadingError &&
     lhs.temporalProjectionIndex== rhs.temporalProjectionIndex &&
     lhs.spatialProjectionIndex== rhs.spatialProjectionIndex 
    );
}

std::size_t av_state_avt::hash() const noexcept
{
    std::size_t result = 17;
    result = result * 31 + std::hash<int64_t>()(timestamp);
    result = result * 31 + std::hash<av_status_avt>()(avStatus);
    result = result * 31 + std::hash<TransmissionStates>()(currentGear);
    result = result * 31 + std::hash<TurnSignal>()(turnSignal);
    result = result * 31 + std::hash<bool>()(brakePressed);
    result = result * 31 + std::hash<bool>()(reverseLight);
    result = result * 31 + std::hash<double>()(brakeTorque);
    result = result * 31 + std::hash<double>()(roadWheelAngle);
    result = result * 31 + std::hash<double>()(steeringWheelAngle);
    result = result * 31 + std::hash<double>()(throttleTorque);
    result = result * 31 + std::hash<double>()(vehicleSpeed);
    result = result * 31 + std::hash<double>()(yawRate);
    result = result * 31 + std::hash<double>()(odometer);
    result = result * 31 + std::hash<int8_t>()(numTirePressure);
    result = result * 31 + hashVector(tirePressure);
    result = result * 31 + std::hash<int16_t>()(fuelRange);
    result = result * 31 + std::hash<float>()(fuelLevel);
    result = result * 31 + std::hash<int32_t>()(serviceFaults);
    result = result * 31 + std::hash<double>()(crossTrackError);
    result = result * 31 + std::hash<double>()(headingError);
    result = result * 31 + std::hash<double>()(curvatureRequest);
    result = result * 31 + std::hash<double>()(stationError);
    result = result * 31 + std::hash<double>()(commandedSpeed);
    result = result * 31 + std::hash<double>()(feedbackSpeed);
    result = result * 31 + std::hash<double>()(commandedAcceleration);
    result = result * 31 + std::hash<double>()(actualAcceleration);
    result = result * 31 + std::hash<double>()(desiredAcceleration);
    result = result * 31 + std::hash<double>()(commandedRoadWheelAngle);
    result = result * 31 + std::hash<double>()(compensatedHeadingError);
    result = result * 31 + std::hash<double>()(temporalProjectionIndex);
    result = result * 31 + std::hash<double>()(spatialProjectionIndex);
    return result;
}

 } // namespace av
 } // namespace msg


CEREAL_BIND_TO_ARCHIVES(av::msg::av_state_avt)

