#include <interfaces/control_perf_status_avt/control_perf_status_avt.h>

#include <iterator>
#include <type_traits>

#include <cereal/cereal.hpp>
#include <cereal/archives/json.hpp>
#include <cereal/archives/binary.hpp>

#include <cereal/types/map.hpp>
#include <cereal/types/vector.hpp>
#include <cereal/types/array.hpp>
#include <cereal/types/string.hpp>
#include <cereal/types/complex.hpp>

#include <ODatum/IntrospectionArchive.h>
#include <ODatum/Hash.h>





 namespace av {
 namespace msg {


control_perf_status_avt::control_perf_status_avt() = default;

control_perf_status_avt::control_perf_status_avt(
    int64_t timeStamp_arg ,
    av::msg::SE2 pos_arg ,
    double lateralErrorRef_m_arg ,
    double lateralErrorFC_m_arg ,
    double lateralErrorRC_m_arg ,
    double laneClearanceFL_m_arg ,
    double laneClearanceFR_m_arg ,
    double laneClearanceRL_m_arg ,
    double laneClearanceRR_m_arg 
    ):
    timeStamp { std::move(timeStamp_arg) } // NOLINT
      ,pos { std::move(pos_arg) } // NOLINT
      ,lateralErrorRef_m { std::move(lateralErrorRef_m_arg) } // NOLINT
      ,lateralErrorFC_m { std::move(lateralErrorFC_m_arg) } // NOLINT
      ,lateralErrorRC_m { std::move(lateralErrorRC_m_arg) } // NOLINT
      ,laneClearanceFL_m { std::move(laneClearanceFL_m_arg) } // NOLINT
      ,laneClearanceFR_m { std::move(laneClearanceFR_m_arg) } // NOLINT
      ,laneClearanceRL_m { std::move(laneClearanceRL_m_arg) } // NOLINT
      ,laneClearanceRR_m { std::move(laneClearanceRR_m_arg) } // NOLINT
      {
}

control_perf_status_avt::control_perf_status_avt(const control_perf_status_avt&) = default;

// NOLINTNEXTLINE
control_perf_status_avt::control_perf_status_avt(control_perf_status_avt&&) = default;

control_perf_status_avt& control_perf_status_avt::operator=(const control_perf_status_avt&) = default;

// NOLINTNEXTLINE
control_perf_status_avt& control_perf_status_avt::operator=(control_perf_status_avt&&) = default;

void control_perf_status_avt::toCsv(std::ostream& out) const
{
    out.precision(16);
    out << std::boolalpha;
    out << timeStamp;out << ",";
    out << pos;out << ",";
    out << lateralErrorRef_m;out << ",";
    out << lateralErrorFC_m;out << ",";
    out << lateralErrorRC_m;out << ",";
    out << laneClearanceFL_m;out << ",";
    out << laneClearanceFR_m;out << ",";
    out << laneClearanceRL_m;out << ",";
    out << laneClearanceRR_m;
    
}

std::stringstream control_perf_status_avt::toStringstream() const
{
    std::stringstream out;
    toCsv(out);
    return out;
}

template<class Archive>
    void control_perf_status_avt::serialize(Archive& ar, [[maybe_unused]] unsigned int version)
    {

        ar& CEREAL_NVP(timeStamp);
        ar& CEREAL_NVP(pos);
        ar& CEREAL_NVP(lateralErrorRef_m);
        ar& CEREAL_NVP(lateralErrorFC_m);
        ar& CEREAL_NVP(lateralErrorRC_m);
        ar& CEREAL_NVP(laneClearanceFL_m);
        ar& CEREAL_NVP(laneClearanceFR_m);
        ar& CEREAL_NVP(laneClearanceRL_m);
        ar& CEREAL_NVP(laneClearanceRR_m);
    }


template void control_perf_status_avt::serialize(cereal::BinaryInputArchive& ar, unsigned int version);
template void control_perf_status_avt::serialize(cereal::BinaryOutputArchive& ar, unsigned int version);
template void control_perf_status_avt::serialize(cereal::IntrospectionArchive& ar, unsigned int version);
template void control_perf_status_avt::serialize(cereal::JSONInputArchive& ar, unsigned int version);
template void control_perf_status_avt::serialize(cereal::JSONOutputArchive& ar, unsigned int version);
template void control_perf_status_avt::serialize(cereal::PortableBinaryInputArchive& ar, unsigned int version);
template void control_perf_status_avt::serialize(cereal::PortableBinaryOutputArchive& ar, unsigned int version);

std::ostream& operator<< (std::ostream& os, const control_perf_status_avt& control_perf_status_avt)
{
    control_perf_status_avt.toCsv(os);
    return os;
}

bool operator== (const control_perf_status_avt &lhs, const control_perf_status_avt &rhs)
{
    return (  lhs.timeStamp== rhs.timeStamp &&
     lhs.pos== rhs.pos &&
     lhs.lateralErrorRef_m== rhs.lateralErrorRef_m &&
     lhs.lateralErrorFC_m== rhs.lateralErrorFC_m &&
     lhs.lateralErrorRC_m== rhs.lateralErrorRC_m &&
     lhs.laneClearanceFL_m== rhs.laneClearanceFL_m &&
     lhs.laneClearanceFR_m== rhs.laneClearanceFR_m &&
     lhs.laneClearanceRL_m== rhs.laneClearanceRL_m &&
     lhs.laneClearanceRR_m== rhs.laneClearanceRR_m 
    );
}

std::size_t control_perf_status_avt::hash() const noexcept
{
    std::size_t result = 17;
    result = result * 31 + std::hash<int64_t>()(timeStamp);
    result = result * 31 + std::hash<av::msg::SE2>()(pos);
    result = result * 31 + std::hash<double>()(lateralErrorRef_m);
    result = result * 31 + std::hash<double>()(lateralErrorFC_m);
    result = result * 31 + std::hash<double>()(lateralErrorRC_m);
    result = result * 31 + std::hash<double>()(laneClearanceFL_m);
    result = result * 31 + std::hash<double>()(laneClearanceFR_m);
    result = result * 31 + std::hash<double>()(laneClearanceRL_m);
    result = result * 31 + std::hash<double>()(laneClearanceRR_m);
    return result;
}

 } // namespace av
 } // namespace msg


CEREAL_BIND_TO_ARCHIVES(av::msg::control_perf_status_avt)

