#include <interfaces/controller_status_message/controller_status_message.h>

#include <iterator>
#include <type_traits>

#include <cereal/cereal.hpp>
#include <cereal/archives/json.hpp>
#include <cereal/archives/binary.hpp>

#include <cereal/types/map.hpp>
#include <cereal/types/vector.hpp>
#include <cereal/types/array.hpp>
#include <cereal/types/string.hpp>
#include <cereal/types/complex.hpp>

#include <ODatum/IntrospectionArchive.h>
#include <ODatum/Hash.h>


 namespace av {
 namespace msg {


NamedDouble::NamedDouble() = default;

NamedDouble::NamedDouble(
    std::string name_arg ,
    double value_arg 
    ):
    name { std::move(name_arg) } // NOLINT
      ,
    value { std::move(value_arg) } // NOLINT
      
    {
}

NamedDouble::NamedDouble(const NamedDouble&) = default;

// NOLINTNEXTLINE
NamedDouble::NamedDouble(NamedDouble&&) = default;

NamedDouble& NamedDouble::operator=(const NamedDouble&) = default;

// NOLINTNEXTLINE
NamedDouble& NamedDouble::operator=(NamedDouble&&) = default;

void NamedDouble::toCsv(std::ostream& out) const
{
    out.precision(16);
    out << std::boolalpha;
    out << "\"";
        out << name;
        out << "\"";out << ",";
    out << value;
    
}

std::stringstream NamedDouble::toStringstream() const
{
    std::stringstream out;
    toCsv(out);
    return out;
}

template<class Archive>
    void NamedDouble::serialize(Archive& ar, [[maybe_unused]] unsigned int version)
    {

        ar& CEREAL_NVP(name);
        ar& CEREAL_NVP(value);
    }


template void NamedDouble::serialize(cereal::BinaryInputArchive& ar, unsigned int version);
template void NamedDouble::serialize(cereal::BinaryOutputArchive& ar, unsigned int version);
template void NamedDouble::serialize(cereal::IntrospectionArchive& ar, unsigned int version);
template void NamedDouble::serialize(cereal::JSONInputArchive& ar, unsigned int version);
template void NamedDouble::serialize(cereal::JSONOutputArchive& ar, unsigned int version);
template void NamedDouble::serialize(cereal::PortableBinaryInputArchive& ar, unsigned int version);
template void NamedDouble::serialize(cereal::PortableBinaryOutputArchive& ar, unsigned int version);

std::ostream& operator<< (std::ostream& os, const NamedDouble& nameddouble)
{
    nameddouble.toCsv(os);
    return os;
}

bool operator== (const NamedDouble &lhs, const NamedDouble &rhs)
{
    return (  lhs.name== rhs.name &&
     lhs.value== rhs.value 
    );
}

std::size_t NamedDouble::hash() const noexcept
{
    std::size_t result = 17;
    result = result * 31 + std::hash<std::string>()(name);
    result = result * 31 + std::hash<double>()(value);
    return result;
}

 } // namespace av
 } // namespace msg


CEREAL_BIND_TO_ARCHIVES(av::msg::NamedDouble)

 namespace av {
 namespace msg {


VectorOfNamedDouble::VectorOfNamedDouble() = default;

VectorOfNamedDouble::VectorOfNamedDouble(
    std::string name_arg ,
    std::vector<NamedDouble> values_arg 
    ):
    name { std::move(name_arg) } // NOLINT
      ,
    values { std::move(values_arg) } // NOLINT
      
    {
}

VectorOfNamedDouble::VectorOfNamedDouble(const VectorOfNamedDouble&) = default;

// NOLINTNEXTLINE
VectorOfNamedDouble::VectorOfNamedDouble(VectorOfNamedDouble&&) = default;

VectorOfNamedDouble& VectorOfNamedDouble::operator=(const VectorOfNamedDouble&) = default;

// NOLINTNEXTLINE
VectorOfNamedDouble& VectorOfNamedDouble::operator=(VectorOfNamedDouble&&) = default;

void VectorOfNamedDouble::toCsv(std::ostream& out) const
{
    out.precision(16);
    out << std::boolalpha;
    out << "\"";
        out << name;
        out << "\"";out << ",";
    // Enclose sequence in double quotations and curly brackets to keep CSV line length uniform
        
        out << "\"{{";
        
        if (!values.empty())
        {
            std::copy(values.begin(), values.end() - 1, std::ostream_iterator<NamedDouble>(out, "},{"));
            std::copy(values.end()-1, values.end(), std::ostream_iterator<NamedDouble>(out, ""));
        }
        out << "}}\"";
    
}

std::stringstream VectorOfNamedDouble::toStringstream() const
{
    std::stringstream out;
    toCsv(out);
    return out;
}

template<class Archive>
    void VectorOfNamedDouble::serialize(Archive& ar, [[maybe_unused]] unsigned int version)
    {

        ar& CEREAL_NVP(name);
        ar& CEREAL_NVP(values);
    }


template void VectorOfNamedDouble::serialize(cereal::BinaryInputArchive& ar, unsigned int version);
template void VectorOfNamedDouble::serialize(cereal::BinaryOutputArchive& ar, unsigned int version);
template void VectorOfNamedDouble::serialize(cereal::IntrospectionArchive& ar, unsigned int version);
template void VectorOfNamedDouble::serialize(cereal::JSONInputArchive& ar, unsigned int version);
template void VectorOfNamedDouble::serialize(cereal::JSONOutputArchive& ar, unsigned int version);
template void VectorOfNamedDouble::serialize(cereal::PortableBinaryInputArchive& ar, unsigned int version);
template void VectorOfNamedDouble::serialize(cereal::PortableBinaryOutputArchive& ar, unsigned int version);

std::ostream& operator<< (std::ostream& os, const VectorOfNamedDouble& vectorofnameddouble)
{
    vectorofnameddouble.toCsv(os);
    return os;
}

bool operator== (const VectorOfNamedDouble &lhs, const VectorOfNamedDouble &rhs)
{
    return (  lhs.name== rhs.name &&
     lhs.values== rhs.values 
    );
}

std::size_t VectorOfNamedDouble::hash() const noexcept
{
    std::size_t result = 17;
    result = result * 31 + std::hash<std::string>()(name);
    result = result * 31 + hashVector(values);
    return result;
}

 } // namespace av
 } // namespace msg


CEREAL_BIND_TO_ARCHIVES(av::msg::VectorOfNamedDouble)

 namespace av {
 namespace msg {


NamedVectorOfDouble::NamedVectorOfDouble() = default;

NamedVectorOfDouble::NamedVectorOfDouble(
    std::string name_arg ,
    std::vector<double> values_arg 
    ):
    name { std::move(name_arg) } // NOLINT
      ,
    values { std::move(values_arg) } // NOLINT
      
    {
}

NamedVectorOfDouble::NamedVectorOfDouble(const NamedVectorOfDouble&) = default;

// NOLINTNEXTLINE
NamedVectorOfDouble::NamedVectorOfDouble(NamedVectorOfDouble&&) = default;

NamedVectorOfDouble& NamedVectorOfDouble::operator=(const NamedVectorOfDouble&) = default;

// NOLINTNEXTLINE
NamedVectorOfDouble& NamedVectorOfDouble::operator=(NamedVectorOfDouble&&) = default;

void NamedVectorOfDouble::toCsv(std::ostream& out) const
{
    out.precision(16);
    out << std::boolalpha;
    out << "\"";
        out << name;
        out << "\"";out << ",";
    // Enclose sequence in double quotations and curly brackets to keep CSV line length uniform
        
        out << "\"{{";
        
        if (!values.empty())
        {
            std::copy(values.begin(), values.end() - 1, std::ostream_iterator<double>(out, "},{"));
            std::copy(values.end()-1, values.end(), std::ostream_iterator<double>(out, ""));
        }
        out << "}}\"";
    
}

std::stringstream NamedVectorOfDouble::toStringstream() const
{
    std::stringstream out;
    toCsv(out);
    return out;
}

template<class Archive>
    void NamedVectorOfDouble::serialize(Archive& ar, [[maybe_unused]] unsigned int version)
    {

        ar& CEREAL_NVP(name);
        ar& CEREAL_NVP(values);
    }


template void NamedVectorOfDouble::serialize(cereal::BinaryInputArchive& ar, unsigned int version);
template void NamedVectorOfDouble::serialize(cereal::BinaryOutputArchive& ar, unsigned int version);
template void NamedVectorOfDouble::serialize(cereal::IntrospectionArchive& ar, unsigned int version);
template void NamedVectorOfDouble::serialize(cereal::JSONInputArchive& ar, unsigned int version);
template void NamedVectorOfDouble::serialize(cereal::JSONOutputArchive& ar, unsigned int version);
template void NamedVectorOfDouble::serialize(cereal::PortableBinaryInputArchive& ar, unsigned int version);
template void NamedVectorOfDouble::serialize(cereal::PortableBinaryOutputArchive& ar, unsigned int version);

std::ostream& operator<< (std::ostream& os, const NamedVectorOfDouble& namedvectorofdouble)
{
    namedvectorofdouble.toCsv(os);
    return os;
}

bool operator== (const NamedVectorOfDouble &lhs, const NamedVectorOfDouble &rhs)
{
    return (  lhs.name== rhs.name &&
     lhs.values== rhs.values 
    );
}

std::size_t NamedVectorOfDouble::hash() const noexcept
{
    std::size_t result = 17;
    result = result * 31 + std::hash<std::string>()(name);
    result = result * 31 + hashVector(values);
    return result;
}

 } // namespace av
 } // namespace msg


CEREAL_BIND_TO_ARCHIVES(av::msg::NamedVectorOfDouble)

 namespace av {
 namespace msg {


SolverInfo::SolverInfo() = default;

SolverInfo::SolverInfo(
    int exitFlag_arg ,
    int iterations_arg ,
    int it2opt_arg ,
    double residualEq_arg ,
    double residualIneq_arg ,
    double rsNorm_arg ,
    double rCompNorm_arg ,
    double primalObjective_arg ,
    double dualObjective_arg ,
    double dGap_arg ,
    double rdGap_arg ,
    double mu_arg ,
    double muAff_arg ,
    double sigma_arg ,
    int lsitAff_arg ,
    int lsitCc_arg ,
    double stepAff_arg ,
    double stepCc_arg ,
    double solveTime_arg ,
    double fEvalsTime_arg 
    ):
    exitFlag { std::move(exitFlag_arg) } // NOLINT
      ,
    iterations { std::move(iterations_arg) } // NOLINT
      ,
    it2opt { std::move(it2opt_arg) } // NOLINT
      ,
    residualEq { std::move(residualEq_arg) } // NOLINT
      ,
    residualIneq { std::move(residualIneq_arg) } // NOLINT
      ,
    rsNorm { std::move(rsNorm_arg) } // NOLINT
      ,
    rCompNorm { std::move(rCompNorm_arg) } // NOLINT
      ,
    primalObjective { std::move(primalObjective_arg) } // NOLINT
      ,
    dualObjective { std::move(dualObjective_arg) } // NOLINT
      ,
    dGap { std::move(dGap_arg) } // NOLINT
      ,
    rdGap { std::move(rdGap_arg) } // NOLINT
      ,
    mu { std::move(mu_arg) } // NOLINT
      ,
    muAff { std::move(muAff_arg) } // NOLINT
      ,
    sigma { std::move(sigma_arg) } // NOLINT
      ,
    lsitAff { std::move(lsitAff_arg) } // NOLINT
      ,
    lsitCc { std::move(lsitCc_arg) } // NOLINT
      ,
    stepAff { std::move(stepAff_arg) } // NOLINT
      ,
    stepCc { std::move(stepCc_arg) } // NOLINT
      ,
    solveTime { std::move(solveTime_arg) } // NOLINT
      ,
    fEvalsTime { std::move(fEvalsTime_arg) } // NOLINT
      
    {
}

SolverInfo::SolverInfo(const SolverInfo&) = default;

// NOLINTNEXTLINE
SolverInfo::SolverInfo(SolverInfo&&) = default;

SolverInfo& SolverInfo::operator=(const SolverInfo&) = default;

// NOLINTNEXTLINE
SolverInfo& SolverInfo::operator=(SolverInfo&&) = default;

void SolverInfo::toCsv(std::ostream& out) const
{
    out.precision(16);
    out << std::boolalpha;
    out << exitFlag;out << ",";
    out << iterations;out << ",";
    out << it2opt;out << ",";
    out << residualEq;out << ",";
    out << residualIneq;out << ",";
    out << rsNorm;out << ",";
    out << rCompNorm;out << ",";
    out << primalObjective;out << ",";
    out << dualObjective;out << ",";
    out << dGap;out << ",";
    out << rdGap;out << ",";
    out << mu;out << ",";
    out << muAff;out << ",";
    out << sigma;out << ",";
    out << lsitAff;out << ",";
    out << lsitCc;out << ",";
    out << stepAff;out << ",";
    out << stepCc;out << ",";
    out << solveTime;out << ",";
    out << fEvalsTime;
    
}

std::stringstream SolverInfo::toStringstream() const
{
    std::stringstream out;
    toCsv(out);
    return out;
}

template<class Archive>
    void SolverInfo::serialize(Archive& ar, [[maybe_unused]] unsigned int version)
    {

        ar& CEREAL_NVP(exitFlag);
        ar& CEREAL_NVP(iterations);
        ar& CEREAL_NVP(it2opt);
        ar& CEREAL_NVP(residualEq);
        ar& CEREAL_NVP(residualIneq);
        ar& CEREAL_NVP(rsNorm);
        ar& CEREAL_NVP(rCompNorm);
        ar& CEREAL_NVP(primalObjective);
        ar& CEREAL_NVP(dualObjective);
        ar& CEREAL_NVP(dGap);
        ar& CEREAL_NVP(rdGap);
        ar& CEREAL_NVP(mu);
        ar& CEREAL_NVP(muAff);
        ar& CEREAL_NVP(sigma);
        ar& CEREAL_NVP(lsitAff);
        ar& CEREAL_NVP(lsitCc);
        ar& CEREAL_NVP(stepAff);
        ar& CEREAL_NVP(stepCc);
        ar& CEREAL_NVP(solveTime);
        ar& CEREAL_NVP(fEvalsTime);
    }


template void SolverInfo::serialize(cereal::BinaryInputArchive& ar, unsigned int version);
template void SolverInfo::serialize(cereal::BinaryOutputArchive& ar, unsigned int version);
template void SolverInfo::serialize(cereal::IntrospectionArchive& ar, unsigned int version);
template void SolverInfo::serialize(cereal::JSONInputArchive& ar, unsigned int version);
template void SolverInfo::serialize(cereal::JSONOutputArchive& ar, unsigned int version);
template void SolverInfo::serialize(cereal::PortableBinaryInputArchive& ar, unsigned int version);
template void SolverInfo::serialize(cereal::PortableBinaryOutputArchive& ar, unsigned int version);

std::ostream& operator<< (std::ostream& os, const SolverInfo& solverinfo)
{
    solverinfo.toCsv(os);
    return os;
}

bool operator== (const SolverInfo &lhs, const SolverInfo &rhs)
{
    return (  lhs.exitFlag== rhs.exitFlag &&
     lhs.iterations== rhs.iterations &&
     lhs.it2opt== rhs.it2opt &&
     lhs.residualEq== rhs.residualEq &&
     lhs.residualIneq== rhs.residualIneq &&
     lhs.rsNorm== rhs.rsNorm &&
     lhs.rCompNorm== rhs.rCompNorm &&
     lhs.primalObjective== rhs.primalObjective &&
     lhs.dualObjective== rhs.dualObjective &&
     lhs.dGap== rhs.dGap &&
     lhs.rdGap== rhs.rdGap &&
     lhs.mu== rhs.mu &&
     lhs.muAff== rhs.muAff &&
     lhs.sigma== rhs.sigma &&
     lhs.lsitAff== rhs.lsitAff &&
     lhs.lsitCc== rhs.lsitCc &&
     lhs.stepAff== rhs.stepAff &&
     lhs.stepCc== rhs.stepCc &&
     lhs.solveTime== rhs.solveTime &&
     lhs.fEvalsTime== rhs.fEvalsTime 
    );
}

std::size_t SolverInfo::hash() const noexcept
{
    std::size_t result = 17;
    result = result * 31 + std::hash<int>()(exitFlag);
    result = result * 31 + std::hash<int>()(iterations);
    result = result * 31 + std::hash<int>()(it2opt);
    result = result * 31 + std::hash<double>()(residualEq);
    result = result * 31 + std::hash<double>()(residualIneq);
    result = result * 31 + std::hash<double>()(rsNorm);
    result = result * 31 + std::hash<double>()(rCompNorm);
    result = result * 31 + std::hash<double>()(primalObjective);
    result = result * 31 + std::hash<double>()(dualObjective);
    result = result * 31 + std::hash<double>()(dGap);
    result = result * 31 + std::hash<double>()(rdGap);
    result = result * 31 + std::hash<double>()(mu);
    result = result * 31 + std::hash<double>()(muAff);
    result = result * 31 + std::hash<double>()(sigma);
    result = result * 31 + std::hash<int>()(lsitAff);
    result = result * 31 + std::hash<int>()(lsitCc);
    result = result * 31 + std::hash<double>()(stepAff);
    result = result * 31 + std::hash<double>()(stepCc);
    result = result * 31 + std::hash<double>()(solveTime);
    result = result * 31 + std::hash<double>()(fEvalsTime);
    return result;
}

 } // namespace av
 } // namespace msg


CEREAL_BIND_TO_ARCHIVES(av::msg::SolverInfo)




 namespace av {
 namespace msg {


controller_status_message::controller_status_message() = default;

controller_status_message::controller_status_message(
    int64_t timeStamp_arg ,
    VectorOfNamedDouble x0_arg ,
    VectorOfNamedDouble u0_arg ,
    VectorOfNamedDouble xPropagated_arg ,
    std::vector<NamedVectorOfDouble> horizonOptimizationVariables_arg ,
    std::vector<NamedVectorOfDouble> horizonSolverParameters_arg ,
    std::vector<NamedVectorOfDouble> horizonAdditionalParameters_arg ,
    SolverInfo solverInfo_arg 
    ):
    timeStamp { std::move(timeStamp_arg) } // NOLINT
      ,x0 { std::move(x0_arg) } // NOLINT
      ,u0 { std::move(u0_arg) } // NOLINT
      ,xPropagated { std::move(xPropagated_arg) } // NOLINT
      ,horizonOptimizationVariables { std::move(horizonOptimizationVariables_arg) } // NOLINT
      ,horizonSolverParameters { std::move(horizonSolverParameters_arg) } // NOLINT
      ,horizonAdditionalParameters { std::move(horizonAdditionalParameters_arg) } // NOLINT
      ,solverInfo { std::move(solverInfo_arg) } // NOLINT
      {
}

controller_status_message::controller_status_message(const controller_status_message&) = default;

// NOLINTNEXTLINE
controller_status_message::controller_status_message(controller_status_message&&) = default;

controller_status_message& controller_status_message::operator=(const controller_status_message&) = default;

// NOLINTNEXTLINE
controller_status_message& controller_status_message::operator=(controller_status_message&&) = default;

void controller_status_message::toCsv(std::ostream& out) const
{
    out.precision(16);
    out << std::boolalpha;
    out << timeStamp;out << ",";
    out << x0;out << ",";
    out << u0;out << ",";
    out << xPropagated;out << ",";
    // Enclose sequence in double quotations and curly brackets to keep CSV line length uniform
        
        out << "\"{{";
        
        if (!horizonOptimizationVariables.empty())
        {
            std::copy(horizonOptimizationVariables.begin(), horizonOptimizationVariables.end() - 1, std::ostream_iterator<NamedVectorOfDouble>(out, "},{"));
            std::copy(horizonOptimizationVariables.end()-1, horizonOptimizationVariables.end(), std::ostream_iterator<NamedVectorOfDouble>(out, ""));
        }
        out << "}}\"";out << ",";
    // Enclose sequence in double quotations and curly brackets to keep CSV line length uniform
        
        out << "\"{{";
        
        if (!horizonSolverParameters.empty())
        {
            std::copy(horizonSolverParameters.begin(), horizonSolverParameters.end() - 1, std::ostream_iterator<NamedVectorOfDouble>(out, "},{"));
            std::copy(horizonSolverParameters.end()-1, horizonSolverParameters.end(), std::ostream_iterator<NamedVectorOfDouble>(out, ""));
        }
        out << "}}\"";out << ",";
    // Enclose sequence in double quotations and curly brackets to keep CSV line length uniform
        
        out << "\"{{";
        
        if (!horizonAdditionalParameters.empty())
        {
            std::copy(horizonAdditionalParameters.begin(), horizonAdditionalParameters.end() - 1, std::ostream_iterator<NamedVectorOfDouble>(out, "},{"));
            std::copy(horizonAdditionalParameters.end()-1, horizonAdditionalParameters.end(), std::ostream_iterator<NamedVectorOfDouble>(out, ""));
        }
        out << "}}\"";out << ",";
    out << solverInfo;
    
}

std::stringstream controller_status_message::toStringstream() const
{
    std::stringstream out;
    toCsv(out);
    return out;
}

template<class Archive>
    void controller_status_message::serialize(Archive& ar, [[maybe_unused]] unsigned int version)
    {

        ar& CEREAL_NVP(timeStamp);
        ar& CEREAL_NVP(x0);
        ar& CEREAL_NVP(u0);
        ar& CEREAL_NVP(xPropagated);
        ar& CEREAL_NVP(horizonOptimizationVariables);
        ar& CEREAL_NVP(horizonSolverParameters);
        ar& CEREAL_NVP(horizonAdditionalParameters);
        ar& CEREAL_NVP(solverInfo);
    }


template void controller_status_message::serialize(cereal::BinaryInputArchive& ar, unsigned int version);
template void controller_status_message::serialize(cereal::BinaryOutputArchive& ar, unsigned int version);
template void controller_status_message::serialize(cereal::IntrospectionArchive& ar, unsigned int version);
template void controller_status_message::serialize(cereal::JSONInputArchive& ar, unsigned int version);
template void controller_status_message::serialize(cereal::JSONOutputArchive& ar, unsigned int version);
template void controller_status_message::serialize(cereal::PortableBinaryInputArchive& ar, unsigned int version);
template void controller_status_message::serialize(cereal::PortableBinaryOutputArchive& ar, unsigned int version);

std::ostream& operator<< (std::ostream& os, const controller_status_message& controller_status_message)
{
    controller_status_message.toCsv(os);
    return os;
}

bool operator== (const controller_status_message &lhs, const controller_status_message &rhs)
{
    return (  lhs.timeStamp== rhs.timeStamp &&
     lhs.x0== rhs.x0 &&
     lhs.u0== rhs.u0 &&
     lhs.xPropagated== rhs.xPropagated &&
     lhs.horizonOptimizationVariables== rhs.horizonOptimizationVariables &&
     lhs.horizonSolverParameters== rhs.horizonSolverParameters &&
     lhs.horizonAdditionalParameters== rhs.horizonAdditionalParameters &&
     lhs.solverInfo== rhs.solverInfo 
    );
}

std::size_t controller_status_message::hash() const noexcept
{
    std::size_t result = 17;
    result = result * 31 + std::hash<int64_t>()(timeStamp);
    result = result * 31 + std::hash<VectorOfNamedDouble>()(x0);
    result = result * 31 + std::hash<VectorOfNamedDouble>()(u0);
    result = result * 31 + std::hash<VectorOfNamedDouble>()(xPropagated);
    result = result * 31 + hashVector(horizonOptimizationVariables);
    result = result * 31 + hashVector(horizonSolverParameters);
    result = result * 31 + hashVector(horizonAdditionalParameters);
    result = result * 31 + std::hash<SolverInfo>()(solverInfo);
    return result;
}

 } // namespace av
 } // namespace msg


CEREAL_BIND_TO_ARCHIVES(av::msg::controller_status_message)

