#include <interfaces/imu_state_avt/imu_state_avt.h>

#include <iterator>
#include <type_traits>

#include <cereal/cereal.hpp>
#include <cereal/archives/json.hpp>
#include <cereal/archives/binary.hpp>

#include <cereal/types/map.hpp>
#include <cereal/types/vector.hpp>
#include <cereal/types/array.hpp>
#include <cereal/types/string.hpp>
#include <cereal/types/complex.hpp>

#include <ODatum/IntrospectionArchive.h>
#include <ODatum/Hash.h>





 namespace av {
 namespace msg {


imu_state_avt::imu_state_avt() = default;

imu_state_avt::imu_state_avt(
    int64_t timeStamp_arg ,
    geometry::RecPoint3D accel_arg ,
    geometry::RecPoint3D gyro_arg ,
    geometry::RecPoint3D mag_arg ,
    geometry::RecPoint3D deltaAccel_arg ,
    geometry::RecPoint3D deltaGyro_arg 
    ):
    timeStamp { std::move(timeStamp_arg) } // NOLINT
      ,accel { std::move(accel_arg) } // NOLINT
      ,gyro { std::move(gyro_arg) } // NOLINT
      ,mag { std::move(mag_arg) } // NOLINT
      ,deltaAccel { std::move(deltaAccel_arg) } // NOLINT
      ,deltaGyro { std::move(deltaGyro_arg) } // NOLINT
      {
}

imu_state_avt::imu_state_avt(const imu_state_avt&) = default;

// NOLINTNEXTLINE
imu_state_avt::imu_state_avt(imu_state_avt&&) = default;

imu_state_avt& imu_state_avt::operator=(const imu_state_avt&) = default;

// NOLINTNEXTLINE
imu_state_avt& imu_state_avt::operator=(imu_state_avt&&) = default;

void imu_state_avt::toCsv(std::ostream& out) const
{
    out.precision(16);
    out << std::boolalpha;
    out << timeStamp;out << ",";
    out << accel;out << ",";
    out << gyro;out << ",";
    out << mag;out << ",";
    out << deltaAccel;out << ",";
    out << deltaGyro;
    
}

std::stringstream imu_state_avt::toStringstream() const
{
    std::stringstream out;
    toCsv(out);
    return out;
}

template<class Archive>
    void imu_state_avt::serialize(Archive& ar, [[maybe_unused]] unsigned int version)
    {

        ar& CEREAL_NVP(timeStamp);
        ar& CEREAL_NVP(accel);
        ar& CEREAL_NVP(gyro);
        ar& CEREAL_NVP(mag);
        ar& CEREAL_NVP(deltaAccel);
        ar& CEREAL_NVP(deltaGyro);
    }


template void imu_state_avt::serialize(cereal::BinaryInputArchive& ar, unsigned int version);
template void imu_state_avt::serialize(cereal::BinaryOutputArchive& ar, unsigned int version);
template void imu_state_avt::serialize(cereal::IntrospectionArchive& ar, unsigned int version);
template void imu_state_avt::serialize(cereal::JSONInputArchive& ar, unsigned int version);
template void imu_state_avt::serialize(cereal::JSONOutputArchive& ar, unsigned int version);
template void imu_state_avt::serialize(cereal::PortableBinaryInputArchive& ar, unsigned int version);
template void imu_state_avt::serialize(cereal::PortableBinaryOutputArchive& ar, unsigned int version);

std::ostream& operator<< (std::ostream& os, const imu_state_avt& imu_state_avt)
{
    imu_state_avt.toCsv(os);
    return os;
}

bool operator== (const imu_state_avt &lhs, const imu_state_avt &rhs)
{
    return (  lhs.timeStamp== rhs.timeStamp &&
     lhs.accel== rhs.accel &&
     lhs.gyro== rhs.gyro &&
     lhs.mag== rhs.mag &&
     lhs.deltaAccel== rhs.deltaAccel &&
     lhs.deltaGyro== rhs.deltaGyro 
    );
}

std::size_t imu_state_avt::hash() const noexcept
{
    std::size_t result = 17;
    result = result * 31 + std::hash<int64_t>()(timeStamp);
    result = result * 31 + std::hash<geometry::RecPoint3D>()(accel);
    result = result * 31 + std::hash<geometry::RecPoint3D>()(gyro);
    result = result * 31 + std::hash<geometry::RecPoint3D>()(mag);
    result = result * 31 + std::hash<geometry::RecPoint3D>()(deltaAccel);
    result = result * 31 + std::hash<geometry::RecPoint3D>()(deltaGyro);
    return result;
}

 } // namespace av
 } // namespace msg


CEREAL_BIND_TO_ARCHIVES(av::msg::imu_state_avt)

