#include <interfaces/otimePoint/otimePoint.h>

#include <iterator>
#include <type_traits>

#include <cereal/cereal.hpp>
#include <cereal/archives/json.hpp>
#include <cereal/archives/binary.hpp>

#include <cereal/types/map.hpp>
#include <cereal/types/vector.hpp>
#include <cereal/types/array.hpp>
#include <cereal/types/string.hpp>
#include <cereal/types/complex.hpp>

#include <ODatum/IntrospectionArchive.h>
#include <ODatum/Hash.h>


 namespace av {
 namespace msg {


otimePoint::otimePoint() = default;

otimePoint::otimePoint(
    int64_t tv_sec_arg ,
    int64_t tv_nsec_arg 
    ):
    tv_sec { std::move(tv_sec_arg) } // NOLINT
      ,
    tv_nsec { std::move(tv_nsec_arg) } // NOLINT
      
    {
}

otimePoint::otimePoint(const otimePoint&) = default;

// NOLINTNEXTLINE
otimePoint::otimePoint(otimePoint&&) = default;

otimePoint& otimePoint::operator=(const otimePoint&) = default;

// NOLINTNEXTLINE
otimePoint& otimePoint::operator=(otimePoint&&) = default;

void otimePoint::toCsv(std::ostream& out) const
{
    out.precision(16);
    out << std::boolalpha;
    out << tv_sec;out << ",";
    out << tv_nsec;
    
}

std::stringstream otimePoint::toStringstream() const
{
    std::stringstream out;
    toCsv(out);
    return out;
}

template<class Archive>
    void otimePoint::serialize(Archive& ar, [[maybe_unused]] unsigned int version)
    {

        ar& CEREAL_NVP(tv_sec);
        ar& CEREAL_NVP(tv_nsec);
    }


template void otimePoint::serialize(cereal::BinaryInputArchive& ar, unsigned int version);
template void otimePoint::serialize(cereal::BinaryOutputArchive& ar, unsigned int version);
template void otimePoint::serialize(cereal::IntrospectionArchive& ar, unsigned int version);
template void otimePoint::serialize(cereal::JSONInputArchive& ar, unsigned int version);
template void otimePoint::serialize(cereal::JSONOutputArchive& ar, unsigned int version);
template void otimePoint::serialize(cereal::PortableBinaryInputArchive& ar, unsigned int version);
template void otimePoint::serialize(cereal::PortableBinaryOutputArchive& ar, unsigned int version);

std::ostream& operator<< (std::ostream& os, const otimePoint& otimepoint)
{
    otimepoint.toCsv(os);
    return os;
}

bool operator== (const otimePoint &lhs, const otimePoint &rhs)
{
    return (  lhs.tv_sec== rhs.tv_sec &&
     lhs.tv_nsec== rhs.tv_nsec 
    );
}

std::size_t otimePoint::hash() const noexcept
{
    std::size_t result = 17;
    result = result * 31 + std::hash<int64_t>()(tv_sec);
    result = result * 31 + std::hash<int64_t>()(tv_nsec);
    return result;
}

 } // namespace av
 } // namespace msg


CEREAL_BIND_TO_ARCHIVES(av::msg::otimePoint)


