#include <interfaces/path_iterator_error_avt/path_iterator_error_avt.h>

#include <iterator>
#include <type_traits>

#include <cereal/cereal.hpp>
#include <cereal/archives/json.hpp>
#include <cereal/archives/binary.hpp>

#include <cereal/types/map.hpp>
#include <cereal/types/vector.hpp>
#include <cereal/types/array.hpp>
#include <cereal/types/string.hpp>
#include <cereal/types/complex.hpp>

#include <ODatum/IntrospectionArchive.h>
#include <ODatum/Hash.h>





 namespace av {
 namespace msg {


path_iterator_error_avt::path_iterator_error_avt() = default;

path_iterator_error_avt::path_iterator_error_avt(
    bool isValid_arg ,
    double crossError_arg ,
    double crossErrorRD_arg ,
    double headingError_arg ,
    double curvature_arg ,
    double dk_arg ,
    double ddk_arg ,
    double speed_arg ,
    double suggestDec_arg ,
    double acceleration_arg ,
    double maxCurvature_arg ,
    std::vector<double> previewCurvatures_arg ,
    bool usePreviewCurvature_arg ,
    double linearFactor_arg ,
    int64_t timeStamp_arg 
    ):
    isValid { std::move(isValid_arg) } // NOLINT
      ,crossError { std::move(crossError_arg) } // NOLINT
      ,crossErrorRD { std::move(crossErrorRD_arg) } // NOLINT
      ,headingError { std::move(headingError_arg) } // NOLINT
      ,curvature { std::move(curvature_arg) } // NOLINT
      ,dk { std::move(dk_arg) } // NOLINT
      ,ddk { std::move(ddk_arg) } // NOLINT
      ,speed { std::move(speed_arg) } // NOLINT
      ,suggestDec { std::move(suggestDec_arg) } // NOLINT
      ,acceleration { std::move(acceleration_arg) } // NOLINT
      ,maxCurvature { std::move(maxCurvature_arg) } // NOLINT
      ,previewCurvatures { std::move(previewCurvatures_arg) } // NOLINT
      ,usePreviewCurvature { std::move(usePreviewCurvature_arg) } // NOLINT
      ,linearFactor { std::move(linearFactor_arg) } // NOLINT
      ,timeStamp { std::move(timeStamp_arg) } // NOLINT
      {
}

path_iterator_error_avt::path_iterator_error_avt(const path_iterator_error_avt&) = default;

// NOLINTNEXTLINE
path_iterator_error_avt::path_iterator_error_avt(path_iterator_error_avt&&) = default;

path_iterator_error_avt& path_iterator_error_avt::operator=(const path_iterator_error_avt&) = default;

// NOLINTNEXTLINE
path_iterator_error_avt& path_iterator_error_avt::operator=(path_iterator_error_avt&&) = default;

void path_iterator_error_avt::toCsv(std::ostream& out) const
{
    out.precision(16);
    out << std::boolalpha;
    out << isValid;out << ",";
    out << crossError;out << ",";
    out << crossErrorRD;out << ",";
    out << headingError;out << ",";
    out << curvature;out << ",";
    out << dk;out << ",";
    out << ddk;out << ",";
    out << speed;out << ",";
    out << suggestDec;out << ",";
    out << acceleration;out << ",";
    out << maxCurvature;out << ",";
    // Enclose sequence in double quotations and curly brackets to keep CSV line length uniform
        
        out << "\"{{";
        
        if (!previewCurvatures.empty())
        {
            std::copy(previewCurvatures.begin(), previewCurvatures.end() - 1, std::ostream_iterator<double>(out, "},{"));
            std::copy(previewCurvatures.end()-1, previewCurvatures.end(), std::ostream_iterator<double>(out, ""));
        }
        out << "}}\"";out << ",";
    out << usePreviewCurvature;out << ",";
    out << linearFactor;out << ",";
    out << timeStamp;
    
}

std::stringstream path_iterator_error_avt::toStringstream() const
{
    std::stringstream out;
    toCsv(out);
    return out;
}

template<class Archive>
    void path_iterator_error_avt::serialize(Archive& ar, [[maybe_unused]] unsigned int version)
    {

        ar& CEREAL_NVP(isValid);
        ar& CEREAL_NVP(crossError);
        ar& CEREAL_NVP(crossErrorRD);
        ar& CEREAL_NVP(headingError);
        ar& CEREAL_NVP(curvature);
        ar& CEREAL_NVP(dk);
        ar& CEREAL_NVP(ddk);
        ar& CEREAL_NVP(speed);
        ar& CEREAL_NVP(suggestDec);
        ar& CEREAL_NVP(acceleration);
        ar& CEREAL_NVP(maxCurvature);
        ar& CEREAL_NVP(previewCurvatures);
        ar& CEREAL_NVP(usePreviewCurvature);
        ar& CEREAL_NVP(linearFactor);
        ar& CEREAL_NVP(timeStamp);
    }


template void path_iterator_error_avt::serialize(cereal::BinaryInputArchive& ar, unsigned int version);
template void path_iterator_error_avt::serialize(cereal::BinaryOutputArchive& ar, unsigned int version);
template void path_iterator_error_avt::serialize(cereal::IntrospectionArchive& ar, unsigned int version);
template void path_iterator_error_avt::serialize(cereal::JSONInputArchive& ar, unsigned int version);
template void path_iterator_error_avt::serialize(cereal::JSONOutputArchive& ar, unsigned int version);
template void path_iterator_error_avt::serialize(cereal::PortableBinaryInputArchive& ar, unsigned int version);
template void path_iterator_error_avt::serialize(cereal::PortableBinaryOutputArchive& ar, unsigned int version);

std::ostream& operator<< (std::ostream& os, const path_iterator_error_avt& path_iterator_error_avt)
{
    path_iterator_error_avt.toCsv(os);
    return os;
}

bool operator== (const path_iterator_error_avt &lhs, const path_iterator_error_avt &rhs)
{
    return (  lhs.isValid== rhs.isValid &&
     lhs.crossError== rhs.crossError &&
     lhs.crossErrorRD== rhs.crossErrorRD &&
     lhs.headingError== rhs.headingError &&
     lhs.curvature== rhs.curvature &&
     lhs.dk== rhs.dk &&
     lhs.ddk== rhs.ddk &&
     lhs.speed== rhs.speed &&
     lhs.suggestDec== rhs.suggestDec &&
     lhs.acceleration== rhs.acceleration &&
     lhs.maxCurvature== rhs.maxCurvature &&
     lhs.previewCurvatures== rhs.previewCurvatures &&
     lhs.usePreviewCurvature== rhs.usePreviewCurvature &&
     lhs.linearFactor== rhs.linearFactor &&
     lhs.timeStamp== rhs.timeStamp 
    );
}

std::size_t path_iterator_error_avt::hash() const noexcept
{
    std::size_t result = 17;
    result = result * 31 + std::hash<bool>()(isValid);
    result = result * 31 + std::hash<double>()(crossError);
    result = result * 31 + std::hash<double>()(crossErrorRD);
    result = result * 31 + std::hash<double>()(headingError);
    result = result * 31 + std::hash<double>()(curvature);
    result = result * 31 + std::hash<double>()(dk);
    result = result * 31 + std::hash<double>()(ddk);
    result = result * 31 + std::hash<double>()(speed);
    result = result * 31 + std::hash<double>()(suggestDec);
    result = result * 31 + std::hash<double>()(acceleration);
    result = result * 31 + std::hash<double>()(maxCurvature);
    result = result * 31 + hashVector(previewCurvatures);
    result = result * 31 + std::hash<bool>()(usePreviewCurvature);
    result = result * 31 + std::hash<double>()(linearFactor);
    result = result * 31 + std::hash<int64_t>()(timeStamp);
    return result;
}

 } // namespace av
 } // namespace msg


CEREAL_BIND_TO_ARCHIVES(av::msg::path_iterator_error_avt)

