#include <interfaces/playback_log_details_avt/playback_log_details_avt.h>

#include <iterator>
#include <type_traits>

#include <cereal/cereal.hpp>
#include <cereal/archives/json.hpp>
#include <cereal/archives/binary.hpp>

#include <cereal/types/map.hpp>
#include <cereal/types/vector.hpp>
#include <cereal/types/array.hpp>
#include <cereal/types/string.hpp>
#include <cereal/types/complex.hpp>

#include <ODatum/IntrospectionArchive.h>
#include <ODatum/Hash.h>


 namespace av {
 namespace msg {


TimePair::TimePair() = default;

TimePair::TimePair(
    int64_t start_arg ,
    int64_t end_arg 
    ):
    start { std::move(start_arg) } // NOLINT
      ,
    end { std::move(end_arg) } // NOLINT
      
    {
}

TimePair::TimePair(const TimePair&) = default;

// NOLINTNEXTLINE
TimePair::TimePair(TimePair&&) = default;

TimePair& TimePair::operator=(const TimePair&) = default;

// NOLINTNEXTLINE
TimePair& TimePair::operator=(TimePair&&) = default;

void TimePair::toCsv(std::ostream& out) const
{
    out.precision(16);
    out << std::boolalpha;
    out << start;out << ",";
    out << end;
    
}

std::stringstream TimePair::toStringstream() const
{
    std::stringstream out;
    toCsv(out);
    return out;
}

template<class Archive>
    void TimePair::serialize(Archive& ar, [[maybe_unused]] unsigned int version)
    {

        ar& CEREAL_NVP(start);
        ar& CEREAL_NVP(end);
    }


template void TimePair::serialize(cereal::BinaryInputArchive& ar, unsigned int version);
template void TimePair::serialize(cereal::BinaryOutputArchive& ar, unsigned int version);
template void TimePair::serialize(cereal::IntrospectionArchive& ar, unsigned int version);
template void TimePair::serialize(cereal::JSONInputArchive& ar, unsigned int version);
template void TimePair::serialize(cereal::JSONOutputArchive& ar, unsigned int version);
template void TimePair::serialize(cereal::PortableBinaryInputArchive& ar, unsigned int version);
template void TimePair::serialize(cereal::PortableBinaryOutputArchive& ar, unsigned int version);

std::ostream& operator<< (std::ostream& os, const TimePair& timepair)
{
    timepair.toCsv(os);
    return os;
}

bool operator== (const TimePair &lhs, const TimePair &rhs)
{
    return (  lhs.start== rhs.start &&
     lhs.end== rhs.end 
    );
}

std::size_t TimePair::hash() const noexcept
{
    std::size_t result = 17;
    result = result * 31 + std::hash<int64_t>()(start);
    result = result * 31 + std::hash<int64_t>()(end);
    return result;
}

 } // namespace av
 } // namespace msg


CEREAL_BIND_TO_ARCHIVES(av::msg::TimePair)

 namespace av {
 namespace msg {


logExtent::logExtent() = default;

logExtent::logExtent(
    std::string key_arg ,
    TimePair value_arg 
    ):
    key { std::move(key_arg) } // NOLINT
      ,
    value { std::move(value_arg) } // NOLINT
      
    {
}

logExtent::logExtent(const logExtent&) = default;

// NOLINTNEXTLINE
logExtent::logExtent(logExtent&&) = default;

logExtent& logExtent::operator=(const logExtent&) = default;

// NOLINTNEXTLINE
logExtent& logExtent::operator=(logExtent&&) = default;

void logExtent::toCsv(std::ostream& out) const
{
    out.precision(16);
    out << std::boolalpha;
    out << "\"";
        out << key;
        out << "\"";out << ",";
    out << value;
    
}

std::stringstream logExtent::toStringstream() const
{
    std::stringstream out;
    toCsv(out);
    return out;
}

template<class Archive>
    void logExtent::serialize(Archive& ar, [[maybe_unused]] unsigned int version)
    {

        ar& CEREAL_NVP(key);
        ar& CEREAL_NVP(value);
    }


template void logExtent::serialize(cereal::BinaryInputArchive& ar, unsigned int version);
template void logExtent::serialize(cereal::BinaryOutputArchive& ar, unsigned int version);
template void logExtent::serialize(cereal::IntrospectionArchive& ar, unsigned int version);
template void logExtent::serialize(cereal::JSONInputArchive& ar, unsigned int version);
template void logExtent::serialize(cereal::JSONOutputArchive& ar, unsigned int version);
template void logExtent::serialize(cereal::PortableBinaryInputArchive& ar, unsigned int version);
template void logExtent::serialize(cereal::PortableBinaryOutputArchive& ar, unsigned int version);

std::ostream& operator<< (std::ostream& os, const logExtent& logextent)
{
    logextent.toCsv(os);
    return os;
}

bool operator== (const logExtent &lhs, const logExtent &rhs)
{
    return (  lhs.key== rhs.key &&
     lhs.value== rhs.value 
    );
}

std::size_t logExtent::hash() const noexcept
{
    std::size_t result = 17;
    result = result * 31 + std::hash<std::string>()(key);
    result = result * 31 + std::hash<TimePair>()(value);
    return result;
}

 } // namespace av
 } // namespace msg


CEREAL_BIND_TO_ARCHIVES(av::msg::logExtent)




 namespace av {
 namespace msg {


playback_log_details_avt::playback_log_details_avt() = default;

playback_log_details_avt::playback_log_details_avt(
    int64_t sessionStartTime__arg ,
    std::vector<logExtent> logExtents_arg 
    ):
    sessionStartTime_ { std::move(sessionStartTime__arg) } // NOLINT
      ,logExtents { std::move(logExtents_arg) } // NOLINT
      {
}

playback_log_details_avt::playback_log_details_avt(const playback_log_details_avt&) = default;

// NOLINTNEXTLINE
playback_log_details_avt::playback_log_details_avt(playback_log_details_avt&&) = default;

playback_log_details_avt& playback_log_details_avt::operator=(const playback_log_details_avt&) = default;

// NOLINTNEXTLINE
playback_log_details_avt& playback_log_details_avt::operator=(playback_log_details_avt&&) = default;

void playback_log_details_avt::toCsv(std::ostream& out) const
{
    out.precision(16);
    out << std::boolalpha;
    out << sessionStartTime_;out << ",";
    // Enclose sequence in double quotations and curly brackets to keep CSV line length uniform
        
        out << "\"{{";
        
        if (!logExtents.empty())
        {
            std::copy(logExtents.begin(), logExtents.end() - 1, std::ostream_iterator<logExtent>(out, "},{"));
            std::copy(logExtents.end()-1, logExtents.end(), std::ostream_iterator<logExtent>(out, ""));
        }
        out << "}}\"";
    
}

std::stringstream playback_log_details_avt::toStringstream() const
{
    std::stringstream out;
    toCsv(out);
    return out;
}

template<class Archive>
    void playback_log_details_avt::serialize(Archive& ar, [[maybe_unused]] unsigned int version)
    {

        ar& CEREAL_NVP(sessionStartTime_);
        ar& CEREAL_NVP(logExtents);
    }


template void playback_log_details_avt::serialize(cereal::BinaryInputArchive& ar, unsigned int version);
template void playback_log_details_avt::serialize(cereal::BinaryOutputArchive& ar, unsigned int version);
template void playback_log_details_avt::serialize(cereal::IntrospectionArchive& ar, unsigned int version);
template void playback_log_details_avt::serialize(cereal::JSONInputArchive& ar, unsigned int version);
template void playback_log_details_avt::serialize(cereal::JSONOutputArchive& ar, unsigned int version);
template void playback_log_details_avt::serialize(cereal::PortableBinaryInputArchive& ar, unsigned int version);
template void playback_log_details_avt::serialize(cereal::PortableBinaryOutputArchive& ar, unsigned int version);

std::ostream& operator<< (std::ostream& os, const playback_log_details_avt& playback_log_details_avt)
{
    playback_log_details_avt.toCsv(os);
    return os;
}

bool operator== (const playback_log_details_avt &lhs, const playback_log_details_avt &rhs)
{
    return (  lhs.sessionStartTime_== rhs.sessionStartTime_ &&
     lhs.logExtents== rhs.logExtents 
    );
}

std::size_t playback_log_details_avt::hash() const noexcept
{
    std::size_t result = 17;
    result = result * 31 + std::hash<int64_t>()(sessionStartTime_);
    result = result * 31 + hashVector(logExtents);
    return result;
}

 } // namespace av
 } // namespace msg


CEREAL_BIND_TO_ARCHIVES(av::msg::playback_log_details_avt)

