#include <interfaces/pose_list_avt/pose_list_avt.h>

#include <iterator>
#include <type_traits>

#include <cereal/cereal.hpp>
#include <cereal/archives/json.hpp>
#include <cereal/archives/binary.hpp>

#include <cereal/types/map.hpp>
#include <cereal/types/vector.hpp>
#include <cereal/types/array.hpp>
#include <cereal/types/string.hpp>
#include <cereal/types/complex.hpp>

#include <ODatum/IntrospectionArchive.h>
#include <ODatum/Hash.h>





 namespace av {
 namespace msg {


pose_list_avt::pose_list_avt() = default;

pose_list_avt::pose_list_avt(
    int64_t timeStamp_arg ,
    std::vector<av::geometry::Pose> poses_arg 
    ):
    timeStamp { std::move(timeStamp_arg) } // NOLINT
      ,poses { std::move(poses_arg) } // NOLINT
      {
}

pose_list_avt::pose_list_avt(const pose_list_avt&) = default;

// NOLINTNEXTLINE
pose_list_avt::pose_list_avt(pose_list_avt&&) = default;

pose_list_avt& pose_list_avt::operator=(const pose_list_avt&) = default;

// NOLINTNEXTLINE
pose_list_avt& pose_list_avt::operator=(pose_list_avt&&) = default;

void pose_list_avt::toCsv(std::ostream& out) const
{
    out.precision(16);
    out << std::boolalpha;
    out << timeStamp;out << ",";
    // Enclose sequence in double quotations and curly brackets to keep CSV line length uniform
        
        out << "\"{{";
        
        if (!poses.empty())
        {
            std::copy(poses.begin(), poses.end() - 1, std::ostream_iterator<av::geometry::Pose>(out, "},{"));
            std::copy(poses.end()-1, poses.end(), std::ostream_iterator<av::geometry::Pose>(out, ""));
        }
        out << "}}\"";
    
}

std::stringstream pose_list_avt::toStringstream() const
{
    std::stringstream out;
    toCsv(out);
    return out;
}

template<class Archive>
    void pose_list_avt::serialize(Archive& ar, [[maybe_unused]] unsigned int version)
    {

        ar& CEREAL_NVP(timeStamp);
        ar& CEREAL_NVP(poses);
    }


template void pose_list_avt::serialize(cereal::BinaryInputArchive& ar, unsigned int version);
template void pose_list_avt::serialize(cereal::BinaryOutputArchive& ar, unsigned int version);
template void pose_list_avt::serialize(cereal::IntrospectionArchive& ar, unsigned int version);
template void pose_list_avt::serialize(cereal::JSONInputArchive& ar, unsigned int version);
template void pose_list_avt::serialize(cereal::JSONOutputArchive& ar, unsigned int version);
template void pose_list_avt::serialize(cereal::PortableBinaryInputArchive& ar, unsigned int version);
template void pose_list_avt::serialize(cereal::PortableBinaryOutputArchive& ar, unsigned int version);

std::ostream& operator<< (std::ostream& os, const pose_list_avt& pose_list_avt)
{
    pose_list_avt.toCsv(os);
    return os;
}

bool operator== (const pose_list_avt &lhs, const pose_list_avt &rhs)
{
    return (  lhs.timeStamp== rhs.timeStamp &&
     lhs.poses== rhs.poses 
    );
}

std::size_t pose_list_avt::hash() const noexcept
{
    std::size_t result = 17;
    result = result * 31 + std::hash<int64_t>()(timeStamp);
    result = result * 31 + hashVector(poses);
    return result;
}

 } // namespace av
 } // namespace msg


CEREAL_BIND_TO_ARCHIVES(av::msg::pose_list_avt)

