#include <interfaces/rec_geometry_avt/rec_geometry_avt.h>

#include <iterator>
#include <type_traits>

#include <cereal/cereal.hpp>
#include <cereal/archives/json.hpp>
#include <cereal/archives/binary.hpp>

#include <cereal/types/map.hpp>
#include <cereal/types/vector.hpp>
#include <cereal/types/array.hpp>
#include <cereal/types/string.hpp>
#include <cereal/types/complex.hpp>

#include <ODatum/IntrospectionArchive.h>
#include <ODatum/Hash.h>


 namespace av {
 namespace geometry {


RecRadians::RecRadians() = default;

RecRadians::RecRadians(
    double rad_arg 
    ):
    rad { std::move(rad_arg) } // NOLINT
      
    {
}

RecRadians::RecRadians(const RecRadians&) = default;

// NOLINTNEXTLINE
RecRadians::RecRadians(RecRadians&&) = default;

RecRadians& RecRadians::operator=(const RecRadians&) = default;

// NOLINTNEXTLINE
RecRadians& RecRadians::operator=(RecRadians&&) = default;

void RecRadians::toCsv(std::ostream& out) const
{
    out.precision(16);
    out << std::boolalpha;
    out << rad;
    
}

std::stringstream RecRadians::toStringstream() const
{
    std::stringstream out;
    toCsv(out);
    return out;
}

template<class Archive>
    void RecRadians::serialize(Archive& ar, [[maybe_unused]] unsigned int version)
    {

        ar& CEREAL_NVP(rad);
    }


template void RecRadians::serialize(cereal::BinaryInputArchive& ar, unsigned int version);
template void RecRadians::serialize(cereal::BinaryOutputArchive& ar, unsigned int version);
template void RecRadians::serialize(cereal::IntrospectionArchive& ar, unsigned int version);
template void RecRadians::serialize(cereal::JSONInputArchive& ar, unsigned int version);
template void RecRadians::serialize(cereal::JSONOutputArchive& ar, unsigned int version);
template void RecRadians::serialize(cereal::PortableBinaryInputArchive& ar, unsigned int version);
template void RecRadians::serialize(cereal::PortableBinaryOutputArchive& ar, unsigned int version);

std::ostream& operator<< (std::ostream& os, const RecRadians& recradians)
{
    recradians.toCsv(os);
    return os;
}

bool operator== (const RecRadians &lhs, const RecRadians &rhs)
{
    return (  lhs.rad== rhs.rad 
    );
}

std::size_t RecRadians::hash() const noexcept
{
    std::size_t result = 17;
    result = result * 31 + std::hash<double>()(rad);
    return result;
}

 } // namespace av
 } // namespace geometry


CEREAL_BIND_TO_ARCHIVES(av::geometry::RecRadians)

 namespace av {
 namespace geometry {


RecPoint2D::RecPoint2D() = default;

RecPoint2D::RecPoint2D(
    double x_arg ,
    double y_arg 
    ):
    x { std::move(x_arg) } // NOLINT
      ,
    y { std::move(y_arg) } // NOLINT
      
    {
}

RecPoint2D::RecPoint2D(const RecPoint2D&) = default;

// NOLINTNEXTLINE
RecPoint2D::RecPoint2D(RecPoint2D&&) = default;

RecPoint2D& RecPoint2D::operator=(const RecPoint2D&) = default;

// NOLINTNEXTLINE
RecPoint2D& RecPoint2D::operator=(RecPoint2D&&) = default;

void RecPoint2D::toCsv(std::ostream& out) const
{
    out.precision(16);
    out << std::boolalpha;
    out << x;out << ",";
    out << y;
    
}

std::stringstream RecPoint2D::toStringstream() const
{
    std::stringstream out;
    toCsv(out);
    return out;
}

template<class Archive>
    void RecPoint2D::serialize(Archive& ar, [[maybe_unused]] unsigned int version)
    {

        ar& CEREAL_NVP(x);
        ar& CEREAL_NVP(y);
    }


template void RecPoint2D::serialize(cereal::BinaryInputArchive& ar, unsigned int version);
template void RecPoint2D::serialize(cereal::BinaryOutputArchive& ar, unsigned int version);
template void RecPoint2D::serialize(cereal::IntrospectionArchive& ar, unsigned int version);
template void RecPoint2D::serialize(cereal::JSONInputArchive& ar, unsigned int version);
template void RecPoint2D::serialize(cereal::JSONOutputArchive& ar, unsigned int version);
template void RecPoint2D::serialize(cereal::PortableBinaryInputArchive& ar, unsigned int version);
template void RecPoint2D::serialize(cereal::PortableBinaryOutputArchive& ar, unsigned int version);

std::ostream& operator<< (std::ostream& os, const RecPoint2D& recpoint2d)
{
    recpoint2d.toCsv(os);
    return os;
}

bool operator== (const RecPoint2D &lhs, const RecPoint2D &rhs)
{
    return (  lhs.x== rhs.x &&
     lhs.y== rhs.y 
    );
}

std::size_t RecPoint2D::hash() const noexcept
{
    std::size_t result = 17;
    result = result * 31 + std::hash<double>()(x);
    result = result * 31 + std::hash<double>()(y);
    return result;
}

 } // namespace av
 } // namespace geometry


CEREAL_BIND_TO_ARCHIVES(av::geometry::RecPoint2D)

 namespace av {
 namespace geometry {


RecPoint3D::RecPoint3D() = default;

RecPoint3D::RecPoint3D(
    double x_arg ,
    double y_arg ,
    double z_arg 
    ):
    x { std::move(x_arg) } // NOLINT
      ,
    y { std::move(y_arg) } // NOLINT
      ,
    z { std::move(z_arg) } // NOLINT
      
    {
}

RecPoint3D::RecPoint3D(const RecPoint3D&) = default;

// NOLINTNEXTLINE
RecPoint3D::RecPoint3D(RecPoint3D&&) = default;

RecPoint3D& RecPoint3D::operator=(const RecPoint3D&) = default;

// NOLINTNEXTLINE
RecPoint3D& RecPoint3D::operator=(RecPoint3D&&) = default;

void RecPoint3D::toCsv(std::ostream& out) const
{
    out.precision(16);
    out << std::boolalpha;
    out << x;out << ",";
    out << y;out << ",";
    out << z;
    
}

std::stringstream RecPoint3D::toStringstream() const
{
    std::stringstream out;
    toCsv(out);
    return out;
}

template<class Archive>
    void RecPoint3D::serialize(Archive& ar, [[maybe_unused]] unsigned int version)
    {

        ar& CEREAL_NVP(x);
        ar& CEREAL_NVP(y);
        ar& CEREAL_NVP(z);
    }


template void RecPoint3D::serialize(cereal::BinaryInputArchive& ar, unsigned int version);
template void RecPoint3D::serialize(cereal::BinaryOutputArchive& ar, unsigned int version);
template void RecPoint3D::serialize(cereal::IntrospectionArchive& ar, unsigned int version);
template void RecPoint3D::serialize(cereal::JSONInputArchive& ar, unsigned int version);
template void RecPoint3D::serialize(cereal::JSONOutputArchive& ar, unsigned int version);
template void RecPoint3D::serialize(cereal::PortableBinaryInputArchive& ar, unsigned int version);
template void RecPoint3D::serialize(cereal::PortableBinaryOutputArchive& ar, unsigned int version);

std::ostream& operator<< (std::ostream& os, const RecPoint3D& recpoint3d)
{
    recpoint3d.toCsv(os);
    return os;
}

bool operator== (const RecPoint3D &lhs, const RecPoint3D &rhs)
{
    return (  lhs.x== rhs.x &&
     lhs.y== rhs.y &&
     lhs.z== rhs.z 
    );
}

std::size_t RecPoint3D::hash() const noexcept
{
    std::size_t result = 17;
    result = result * 31 + std::hash<double>()(x);
    result = result * 31 + std::hash<double>()(y);
    result = result * 31 + std::hash<double>()(z);
    return result;
}

 } // namespace av
 } // namespace geometry


CEREAL_BIND_TO_ARCHIVES(av::geometry::RecPoint3D)

 namespace av {
 namespace geometry {


RecVector2D::RecVector2D() = default;

RecVector2D::RecVector2D(
    double x_arg ,
    double y_arg 
    ):
    x { std::move(x_arg) } // NOLINT
      ,
    y { std::move(y_arg) } // NOLINT
      
    {
}

RecVector2D::RecVector2D(const RecVector2D&) = default;

// NOLINTNEXTLINE
RecVector2D::RecVector2D(RecVector2D&&) = default;

RecVector2D& RecVector2D::operator=(const RecVector2D&) = default;

// NOLINTNEXTLINE
RecVector2D& RecVector2D::operator=(RecVector2D&&) = default;

void RecVector2D::toCsv(std::ostream& out) const
{
    out.precision(16);
    out << std::boolalpha;
    out << x;out << ",";
    out << y;
    
}

std::stringstream RecVector2D::toStringstream() const
{
    std::stringstream out;
    toCsv(out);
    return out;
}

template<class Archive>
    void RecVector2D::serialize(Archive& ar, [[maybe_unused]] unsigned int version)
    {

        ar& CEREAL_NVP(x);
        ar& CEREAL_NVP(y);
    }


template void RecVector2D::serialize(cereal::BinaryInputArchive& ar, unsigned int version);
template void RecVector2D::serialize(cereal::BinaryOutputArchive& ar, unsigned int version);
template void RecVector2D::serialize(cereal::IntrospectionArchive& ar, unsigned int version);
template void RecVector2D::serialize(cereal::JSONInputArchive& ar, unsigned int version);
template void RecVector2D::serialize(cereal::JSONOutputArchive& ar, unsigned int version);
template void RecVector2D::serialize(cereal::PortableBinaryInputArchive& ar, unsigned int version);
template void RecVector2D::serialize(cereal::PortableBinaryOutputArchive& ar, unsigned int version);

std::ostream& operator<< (std::ostream& os, const RecVector2D& recvector2d)
{
    recvector2d.toCsv(os);
    return os;
}

bool operator== (const RecVector2D &lhs, const RecVector2D &rhs)
{
    return (  lhs.x== rhs.x &&
     lhs.y== rhs.y 
    );
}

std::size_t RecVector2D::hash() const noexcept
{
    std::size_t result = 17;
    result = result * 31 + std::hash<double>()(x);
    result = result * 31 + std::hash<double>()(y);
    return result;
}

 } // namespace av
 } // namespace geometry


CEREAL_BIND_TO_ARCHIVES(av::geometry::RecVector2D)

 namespace av {
 namespace geometry {


RecVector3D::RecVector3D() = default;

RecVector3D::RecVector3D(
    double x_arg ,
    double y_arg ,
    double z_arg 
    ):
    x { std::move(x_arg) } // NOLINT
      ,
    y { std::move(y_arg) } // NOLINT
      ,
    z { std::move(z_arg) } // NOLINT
      
    {
}

RecVector3D::RecVector3D(const RecVector3D&) = default;

// NOLINTNEXTLINE
RecVector3D::RecVector3D(RecVector3D&&) = default;

RecVector3D& RecVector3D::operator=(const RecVector3D&) = default;

// NOLINTNEXTLINE
RecVector3D& RecVector3D::operator=(RecVector3D&&) = default;

void RecVector3D::toCsv(std::ostream& out) const
{
    out.precision(16);
    out << std::boolalpha;
    out << x;out << ",";
    out << y;out << ",";
    out << z;
    
}

std::stringstream RecVector3D::toStringstream() const
{
    std::stringstream out;
    toCsv(out);
    return out;
}

template<class Archive>
    void RecVector3D::serialize(Archive& ar, [[maybe_unused]] unsigned int version)
    {

        ar& CEREAL_NVP(x);
        ar& CEREAL_NVP(y);
        ar& CEREAL_NVP(z);
    }


template void RecVector3D::serialize(cereal::BinaryInputArchive& ar, unsigned int version);
template void RecVector3D::serialize(cereal::BinaryOutputArchive& ar, unsigned int version);
template void RecVector3D::serialize(cereal::IntrospectionArchive& ar, unsigned int version);
template void RecVector3D::serialize(cereal::JSONInputArchive& ar, unsigned int version);
template void RecVector3D::serialize(cereal::JSONOutputArchive& ar, unsigned int version);
template void RecVector3D::serialize(cereal::PortableBinaryInputArchive& ar, unsigned int version);
template void RecVector3D::serialize(cereal::PortableBinaryOutputArchive& ar, unsigned int version);

std::ostream& operator<< (std::ostream& os, const RecVector3D& recvector3d)
{
    recvector3d.toCsv(os);
    return os;
}

bool operator== (const RecVector3D &lhs, const RecVector3D &rhs)
{
    return (  lhs.x== rhs.x &&
     lhs.y== rhs.y &&
     lhs.z== rhs.z 
    );
}

std::size_t RecVector3D::hash() const noexcept
{
    std::size_t result = 17;
    result = result * 31 + std::hash<double>()(x);
    result = result * 31 + std::hash<double>()(y);
    result = result * 31 + std::hash<double>()(z);
    return result;
}

 } // namespace av
 } // namespace geometry


CEREAL_BIND_TO_ARCHIVES(av::geometry::RecVector3D)

 namespace av {
 namespace geometry {


RecPose2D::RecPose2D() = default;

RecPose2D::RecPose2D(
    double x_arg ,
    double y_arg ,
    RecRadians rotZ_arg 
    ):
    x { std::move(x_arg) } // NOLINT
      ,
    y { std::move(y_arg) } // NOLINT
      ,
    rotZ { std::move(rotZ_arg) } // NOLINT
      
    {
}

RecPose2D::RecPose2D(const RecPose2D&) = default;

// NOLINTNEXTLINE
RecPose2D::RecPose2D(RecPose2D&&) = default;

RecPose2D& RecPose2D::operator=(const RecPose2D&) = default;

// NOLINTNEXTLINE
RecPose2D& RecPose2D::operator=(RecPose2D&&) = default;

void RecPose2D::toCsv(std::ostream& out) const
{
    out.precision(16);
    out << std::boolalpha;
    out << x;out << ",";
    out << y;out << ",";
    out << rotZ;
    
}

std::stringstream RecPose2D::toStringstream() const
{
    std::stringstream out;
    toCsv(out);
    return out;
}

template<class Archive>
    void RecPose2D::serialize(Archive& ar, [[maybe_unused]] unsigned int version)
    {

        ar& CEREAL_NVP(x);
        ar& CEREAL_NVP(y);
        ar& CEREAL_NVP(rotZ);
    }


template void RecPose2D::serialize(cereal::BinaryInputArchive& ar, unsigned int version);
template void RecPose2D::serialize(cereal::BinaryOutputArchive& ar, unsigned int version);
template void RecPose2D::serialize(cereal::IntrospectionArchive& ar, unsigned int version);
template void RecPose2D::serialize(cereal::JSONInputArchive& ar, unsigned int version);
template void RecPose2D::serialize(cereal::JSONOutputArchive& ar, unsigned int version);
template void RecPose2D::serialize(cereal::PortableBinaryInputArchive& ar, unsigned int version);
template void RecPose2D::serialize(cereal::PortableBinaryOutputArchive& ar, unsigned int version);

std::ostream& operator<< (std::ostream& os, const RecPose2D& recpose2d)
{
    recpose2d.toCsv(os);
    return os;
}

bool operator== (const RecPose2D &lhs, const RecPose2D &rhs)
{
    return (  lhs.x== rhs.x &&
     lhs.y== rhs.y &&
     lhs.rotZ== rhs.rotZ 
    );
}

std::size_t RecPose2D::hash() const noexcept
{
    std::size_t result = 17;
    result = result * 31 + std::hash<double>()(x);
    result = result * 31 + std::hash<double>()(y);
    result = result * 31 + std::hash<RecRadians>()(rotZ);
    return result;
}

 } // namespace av
 } // namespace geometry


CEREAL_BIND_TO_ARCHIVES(av::geometry::RecPose2D)

 namespace av {
 namespace geometry {


RecDifferentialPose2D::RecDifferentialPose2D() = default;

RecDifferentialPose2D::RecDifferentialPose2D(
    double inx_arg ,
    double iny_arg ,
    RecRadians inw_arg 
    ):
    inx { std::move(inx_arg) } // NOLINT
      ,
    iny { std::move(iny_arg) } // NOLINT
      ,
    inw { std::move(inw_arg) } // NOLINT
      
    {
}

RecDifferentialPose2D::RecDifferentialPose2D(const RecDifferentialPose2D&) = default;

// NOLINTNEXTLINE
RecDifferentialPose2D::RecDifferentialPose2D(RecDifferentialPose2D&&) = default;

RecDifferentialPose2D& RecDifferentialPose2D::operator=(const RecDifferentialPose2D&) = default;

// NOLINTNEXTLINE
RecDifferentialPose2D& RecDifferentialPose2D::operator=(RecDifferentialPose2D&&) = default;

void RecDifferentialPose2D::toCsv(std::ostream& out) const
{
    out.precision(16);
    out << std::boolalpha;
    out << inx;out << ",";
    out << iny;out << ",";
    out << inw;
    
}

std::stringstream RecDifferentialPose2D::toStringstream() const
{
    std::stringstream out;
    toCsv(out);
    return out;
}

template<class Archive>
    void RecDifferentialPose2D::serialize(Archive& ar, [[maybe_unused]] unsigned int version)
    {

        ar& CEREAL_NVP(inx);
        ar& CEREAL_NVP(iny);
        ar& CEREAL_NVP(inw);
    }


template void RecDifferentialPose2D::serialize(cereal::BinaryInputArchive& ar, unsigned int version);
template void RecDifferentialPose2D::serialize(cereal::BinaryOutputArchive& ar, unsigned int version);
template void RecDifferentialPose2D::serialize(cereal::IntrospectionArchive& ar, unsigned int version);
template void RecDifferentialPose2D::serialize(cereal::JSONInputArchive& ar, unsigned int version);
template void RecDifferentialPose2D::serialize(cereal::JSONOutputArchive& ar, unsigned int version);
template void RecDifferentialPose2D::serialize(cereal::PortableBinaryInputArchive& ar, unsigned int version);
template void RecDifferentialPose2D::serialize(cereal::PortableBinaryOutputArchive& ar, unsigned int version);

std::ostream& operator<< (std::ostream& os, const RecDifferentialPose2D& recdifferentialpose2d)
{
    recdifferentialpose2d.toCsv(os);
    return os;
}

bool operator== (const RecDifferentialPose2D &lhs, const RecDifferentialPose2D &rhs)
{
    return (  lhs.inx== rhs.inx &&
     lhs.iny== rhs.iny &&
     lhs.inw== rhs.inw 
    );
}

std::size_t RecDifferentialPose2D::hash() const noexcept
{
    std::size_t result = 17;
    result = result * 31 + std::hash<double>()(inx);
    result = result * 31 + std::hash<double>()(iny);
    result = result * 31 + std::hash<RecRadians>()(inw);
    return result;
}

 } // namespace av
 } // namespace geometry


CEREAL_BIND_TO_ARCHIVES(av::geometry::RecDifferentialPose2D)

 namespace av {
 namespace geometry {


RecPose3D::RecPose3D() = default;

RecPose3D::RecPose3D(
    double x_arg ,
    double y_arg ,
    double z_arg ,
    RecRadians rot1_arg ,
    RecRadians rot2_arg ,
    RecRadians rot3_arg ,
    EULER_ANGLE_ORDER angleOrder_arg 
    ):
    x { std::move(x_arg) } // NOLINT
      ,
    y { std::move(y_arg) } // NOLINT
      ,
    z { std::move(z_arg) } // NOLINT
      ,
    rot1 { std::move(rot1_arg) } // NOLINT
      ,
    rot2 { std::move(rot2_arg) } // NOLINT
      ,
    rot3 { std::move(rot3_arg) } // NOLINT
      ,
    angleOrder { std::move(angleOrder_arg) } // NOLINT
      
    {
}

RecPose3D::RecPose3D(const RecPose3D&) = default;

// NOLINTNEXTLINE
RecPose3D::RecPose3D(RecPose3D&&) = default;

RecPose3D& RecPose3D::operator=(const RecPose3D&) = default;

// NOLINTNEXTLINE
RecPose3D& RecPose3D::operator=(RecPose3D&&) = default;

void RecPose3D::toCsv(std::ostream& out) const
{
    out.precision(16);
    out << std::boolalpha;
    out << x;out << ",";
    out << y;out << ",";
    out << z;out << ",";
    out << rot1;out << ",";
    out << rot2;out << ",";
    out << rot3;out << ",";
    out << static_cast<uint32_t>(angleOrder);
    
}

std::stringstream RecPose3D::toStringstream() const
{
    std::stringstream out;
    toCsv(out);
    return out;
}

template<class Archive>
    void RecPose3D::serialize(Archive& ar, [[maybe_unused]] unsigned int version)
    {

        ar& CEREAL_NVP(x);
        ar& CEREAL_NVP(y);
        ar& CEREAL_NVP(z);
        ar& CEREAL_NVP(rot1);
        ar& CEREAL_NVP(rot2);
        ar& CEREAL_NVP(rot3);
        ar& CEREAL_NVP(angleOrder);
    }


template void RecPose3D::serialize(cereal::BinaryInputArchive& ar, unsigned int version);
template void RecPose3D::serialize(cereal::BinaryOutputArchive& ar, unsigned int version);
template void RecPose3D::serialize(cereal::IntrospectionArchive& ar, unsigned int version);
template void RecPose3D::serialize(cereal::JSONInputArchive& ar, unsigned int version);
template void RecPose3D::serialize(cereal::JSONOutputArchive& ar, unsigned int version);
template void RecPose3D::serialize(cereal::PortableBinaryInputArchive& ar, unsigned int version);
template void RecPose3D::serialize(cereal::PortableBinaryOutputArchive& ar, unsigned int version);

std::ostream& operator<< (std::ostream& os, const RecPose3D& recpose3d)
{
    recpose3d.toCsv(os);
    return os;
}

bool operator== (const RecPose3D &lhs, const RecPose3D &rhs)
{
    return (  lhs.x== rhs.x &&
     lhs.y== rhs.y &&
     lhs.z== rhs.z &&
     lhs.rot1== rhs.rot1 &&
     lhs.rot2== rhs.rot2 &&
     lhs.rot3== rhs.rot3 &&
     lhs.angleOrder== rhs.angleOrder 
    );
}

std::size_t RecPose3D::hash() const noexcept
{
    std::size_t result = 17;
    result = result * 31 + std::hash<double>()(x);
    result = result * 31 + std::hash<double>()(y);
    result = result * 31 + std::hash<double>()(z);
    result = result * 31 + std::hash<RecRadians>()(rot1);
    result = result * 31 + std::hash<RecRadians>()(rot2);
    result = result * 31 + std::hash<RecRadians>()(rot3);
    result = result * 31 + std::hash<EULER_ANGLE_ORDER>()(angleOrder);
    return result;
}

 } // namespace av
 } // namespace geometry


CEREAL_BIND_TO_ARCHIVES(av::geometry::RecPose3D)

 namespace av {
 namespace geometry {


RecDifferentialPose3D::RecDifferentialPose3D() = default;

RecDifferentialPose3D::RecDifferentialPose3D(
    double x_arg ,
    double y_arg ,
    double z_arg ,
    RecRadians rot1_arg ,
    RecRadians rot2_arg ,
    RecRadians rot3_arg 
    ):
    x { std::move(x_arg) } // NOLINT
      ,
    y { std::move(y_arg) } // NOLINT
      ,
    z { std::move(z_arg) } // NOLINT
      ,
    rot1 { std::move(rot1_arg) } // NOLINT
      ,
    rot2 { std::move(rot2_arg) } // NOLINT
      ,
    rot3 { std::move(rot3_arg) } // NOLINT
      
    {
}

RecDifferentialPose3D::RecDifferentialPose3D(const RecDifferentialPose3D&) = default;

// NOLINTNEXTLINE
RecDifferentialPose3D::RecDifferentialPose3D(RecDifferentialPose3D&&) = default;

RecDifferentialPose3D& RecDifferentialPose3D::operator=(const RecDifferentialPose3D&) = default;

// NOLINTNEXTLINE
RecDifferentialPose3D& RecDifferentialPose3D::operator=(RecDifferentialPose3D&&) = default;

void RecDifferentialPose3D::toCsv(std::ostream& out) const
{
    out.precision(16);
    out << std::boolalpha;
    out << x;out << ",";
    out << y;out << ",";
    out << z;out << ",";
    out << rot1;out << ",";
    out << rot2;out << ",";
    out << rot3;
    
}

std::stringstream RecDifferentialPose3D::toStringstream() const
{
    std::stringstream out;
    toCsv(out);
    return out;
}

template<class Archive>
    void RecDifferentialPose3D::serialize(Archive& ar, [[maybe_unused]] unsigned int version)
    {

        ar& CEREAL_NVP(x);
        ar& CEREAL_NVP(y);
        ar& CEREAL_NVP(z);
        ar& CEREAL_NVP(rot1);
        ar& CEREAL_NVP(rot2);
        ar& CEREAL_NVP(rot3);
    }


template void RecDifferentialPose3D::serialize(cereal::BinaryInputArchive& ar, unsigned int version);
template void RecDifferentialPose3D::serialize(cereal::BinaryOutputArchive& ar, unsigned int version);
template void RecDifferentialPose3D::serialize(cereal::IntrospectionArchive& ar, unsigned int version);
template void RecDifferentialPose3D::serialize(cereal::JSONInputArchive& ar, unsigned int version);
template void RecDifferentialPose3D::serialize(cereal::JSONOutputArchive& ar, unsigned int version);
template void RecDifferentialPose3D::serialize(cereal::PortableBinaryInputArchive& ar, unsigned int version);
template void RecDifferentialPose3D::serialize(cereal::PortableBinaryOutputArchive& ar, unsigned int version);

std::ostream& operator<< (std::ostream& os, const RecDifferentialPose3D& recdifferentialpose3d)
{
    recdifferentialpose3d.toCsv(os);
    return os;
}

bool operator== (const RecDifferentialPose3D &lhs, const RecDifferentialPose3D &rhs)
{
    return (  lhs.x== rhs.x &&
     lhs.y== rhs.y &&
     lhs.z== rhs.z &&
     lhs.rot1== rhs.rot1 &&
     lhs.rot2== rhs.rot2 &&
     lhs.rot3== rhs.rot3 
    );
}

std::size_t RecDifferentialPose3D::hash() const noexcept
{
    std::size_t result = 17;
    result = result * 31 + std::hash<double>()(x);
    result = result * 31 + std::hash<double>()(y);
    result = result * 31 + std::hash<double>()(z);
    result = result * 31 + std::hash<RecRadians>()(rot1);
    result = result * 31 + std::hash<RecRadians>()(rot2);
    result = result * 31 + std::hash<RecRadians>()(rot3);
    return result;
}

 } // namespace av
 } // namespace geometry


CEREAL_BIND_TO_ARCHIVES(av::geometry::RecDifferentialPose3D)

 namespace av {
 namespace geometry {


RecQuaternion::RecQuaternion() = default;

RecQuaternion::RecQuaternion(
    std::array<std::string,4> quat_arg 
    ):
    quat { std::move(quat_arg) } // NOLINT
      
    {
}

RecQuaternion::RecQuaternion(const RecQuaternion&) = default;

// NOLINTNEXTLINE
RecQuaternion::RecQuaternion(RecQuaternion&&) = default;

RecQuaternion& RecQuaternion::operator=(const RecQuaternion&) = default;

// NOLINTNEXTLINE
RecQuaternion& RecQuaternion::operator=(RecQuaternion&&) = default;

void RecQuaternion::toCsv(std::ostream& out) const
{
    out.precision(16);
    out << std::boolalpha;
    // Enclose array in double quotations and curly brackets to keep CSV line length uniform
        
        out << "\"{{";
        
        if (!quat.empty())
        {
            std::copy(quat.begin(), quat.end() - 1, std::ostream_iterator<std::string>(out, "},{"));
            out << quat.back();
        }
        out << "}}\"";
    
}

std::stringstream RecQuaternion::toStringstream() const
{
    std::stringstream out;
    toCsv(out);
    return out;
}

template<class Archive>
    void RecQuaternion::serialize(Archive& ar, [[maybe_unused]] unsigned int version)
    {

        ar& CEREAL_NVP(quat);
    }


template void RecQuaternion::serialize(cereal::BinaryInputArchive& ar, unsigned int version);
template void RecQuaternion::serialize(cereal::BinaryOutputArchive& ar, unsigned int version);
template void RecQuaternion::serialize(cereal::IntrospectionArchive& ar, unsigned int version);
template void RecQuaternion::serialize(cereal::JSONInputArchive& ar, unsigned int version);
template void RecQuaternion::serialize(cereal::JSONOutputArchive& ar, unsigned int version);
template void RecQuaternion::serialize(cereal::PortableBinaryInputArchive& ar, unsigned int version);
template void RecQuaternion::serialize(cereal::PortableBinaryOutputArchive& ar, unsigned int version);

std::ostream& operator<< (std::ostream& os, const RecQuaternion& recquaternion)
{
    recquaternion.toCsv(os);
    return os;
}

bool operator== (const RecQuaternion &lhs, const RecQuaternion &rhs)
{
    return (  lhs.quat== rhs.quat 
    );
}

std::size_t RecQuaternion::hash() const noexcept
{
    std::size_t result = 17;
    result = result * 31 + hashVector(quat);
    return result;
}

 } // namespace av
 } // namespace geometry


CEREAL_BIND_TO_ARCHIVES(av::geometry::RecQuaternion)

 namespace av {
 namespace geometry {


RecLineSegment2D::RecLineSegment2D() = default;

RecLineSegment2D::RecLineSegment2D(
    RecPoint2D p1__arg ,
    RecPoint2D p2__arg ,
    RecVector2D vv__arg 
    ):
    p1_ { std::move(p1__arg) } // NOLINT
      ,
    p2_ { std::move(p2__arg) } // NOLINT
      ,
    vv_ { std::move(vv__arg) } // NOLINT
      
    {
}

RecLineSegment2D::RecLineSegment2D(const RecLineSegment2D&) = default;

// NOLINTNEXTLINE
RecLineSegment2D::RecLineSegment2D(RecLineSegment2D&&) = default;

RecLineSegment2D& RecLineSegment2D::operator=(const RecLineSegment2D&) = default;

// NOLINTNEXTLINE
RecLineSegment2D& RecLineSegment2D::operator=(RecLineSegment2D&&) = default;

void RecLineSegment2D::toCsv(std::ostream& out) const
{
    out.precision(16);
    out << std::boolalpha;
    out << p1_;out << ",";
    out << p2_;out << ",";
    out << vv_;
    
}

std::stringstream RecLineSegment2D::toStringstream() const
{
    std::stringstream out;
    toCsv(out);
    return out;
}

template<class Archive>
    void RecLineSegment2D::serialize(Archive& ar, [[maybe_unused]] unsigned int version)
    {

        ar& CEREAL_NVP(p1_);
        ar& CEREAL_NVP(p2_);
        ar& CEREAL_NVP(vv_);
    }


template void RecLineSegment2D::serialize(cereal::BinaryInputArchive& ar, unsigned int version);
template void RecLineSegment2D::serialize(cereal::BinaryOutputArchive& ar, unsigned int version);
template void RecLineSegment2D::serialize(cereal::IntrospectionArchive& ar, unsigned int version);
template void RecLineSegment2D::serialize(cereal::JSONInputArchive& ar, unsigned int version);
template void RecLineSegment2D::serialize(cereal::JSONOutputArchive& ar, unsigned int version);
template void RecLineSegment2D::serialize(cereal::PortableBinaryInputArchive& ar, unsigned int version);
template void RecLineSegment2D::serialize(cereal::PortableBinaryOutputArchive& ar, unsigned int version);

std::ostream& operator<< (std::ostream& os, const RecLineSegment2D& reclinesegment2d)
{
    reclinesegment2d.toCsv(os);
    return os;
}

bool operator== (const RecLineSegment2D &lhs, const RecLineSegment2D &rhs)
{
    return (  lhs.p1_== rhs.p1_ &&
     lhs.p2_== rhs.p2_ &&
     lhs.vv_== rhs.vv_ 
    );
}

std::size_t RecLineSegment2D::hash() const noexcept
{
    std::size_t result = 17;
    result = result * 31 + std::hash<RecPoint2D>()(p1_);
    result = result * 31 + std::hash<RecPoint2D>()(p2_);
    result = result * 31 + std::hash<RecVector2D>()(vv_);
    return result;
}

 } // namespace av
 } // namespace geometry


CEREAL_BIND_TO_ARCHIVES(av::geometry::RecLineSegment2D)

 namespace av {
 namespace geometry {


RecLineSegment3D::RecLineSegment3D() = default;

RecLineSegment3D::RecLineSegment3D(
    RecPoint3D p1_arg ,
    RecPoint3D p2_arg 
    ):
    p1 { std::move(p1_arg) } // NOLINT
      ,
    p2 { std::move(p2_arg) } // NOLINT
      
    {
}

RecLineSegment3D::RecLineSegment3D(const RecLineSegment3D&) = default;

// NOLINTNEXTLINE
RecLineSegment3D::RecLineSegment3D(RecLineSegment3D&&) = default;

RecLineSegment3D& RecLineSegment3D::operator=(const RecLineSegment3D&) = default;

// NOLINTNEXTLINE
RecLineSegment3D& RecLineSegment3D::operator=(RecLineSegment3D&&) = default;

void RecLineSegment3D::toCsv(std::ostream& out) const
{
    out.precision(16);
    out << std::boolalpha;
    out << p1;out << ",";
    out << p2;
    
}

std::stringstream RecLineSegment3D::toStringstream() const
{
    std::stringstream out;
    toCsv(out);
    return out;
}

template<class Archive>
    void RecLineSegment3D::serialize(Archive& ar, [[maybe_unused]] unsigned int version)
    {

        ar& CEREAL_NVP(p1);
        ar& CEREAL_NVP(p2);
    }


template void RecLineSegment3D::serialize(cereal::BinaryInputArchive& ar, unsigned int version);
template void RecLineSegment3D::serialize(cereal::BinaryOutputArchive& ar, unsigned int version);
template void RecLineSegment3D::serialize(cereal::IntrospectionArchive& ar, unsigned int version);
template void RecLineSegment3D::serialize(cereal::JSONInputArchive& ar, unsigned int version);
template void RecLineSegment3D::serialize(cereal::JSONOutputArchive& ar, unsigned int version);
template void RecLineSegment3D::serialize(cereal::PortableBinaryInputArchive& ar, unsigned int version);
template void RecLineSegment3D::serialize(cereal::PortableBinaryOutputArchive& ar, unsigned int version);

std::ostream& operator<< (std::ostream& os, const RecLineSegment3D& reclinesegment3d)
{
    reclinesegment3d.toCsv(os);
    return os;
}

bool operator== (const RecLineSegment3D &lhs, const RecLineSegment3D &rhs)
{
    return (  lhs.p1== rhs.p1 &&
     lhs.p2== rhs.p2 
    );
}

std::size_t RecLineSegment3D::hash() const noexcept
{
    std::size_t result = 17;
    result = result * 31 + std::hash<RecPoint3D>()(p1);
    result = result * 31 + std::hash<RecPoint3D>()(p2);
    return result;
}

 } // namespace av
 } // namespace geometry


CEREAL_BIND_TO_ARCHIVES(av::geometry::RecLineSegment3D)

 namespace av {
 namespace geometry {


RecPolygon2D::RecPolygon2D() = default;

RecPolygon2D::RecPolygon2D(
    int numVertices_arg ,
    std::vector<RecPoint2D> verticesP_arg ,
    RecPoint2D center_arg ,
    RecPoint2D min_arg ,
    RecPoint2D max_arg ,
    std::vector<double> angleToVertexP_arg 
    ):
    numVertices { std::move(numVertices_arg) } // NOLINT
      ,
    verticesP { std::move(verticesP_arg) } // NOLINT
      ,
    center { std::move(center_arg) } // NOLINT
      ,
    min { std::move(min_arg) } // NOLINT
      ,
    max { std::move(max_arg) } // NOLINT
      ,
    angleToVertexP { std::move(angleToVertexP_arg) } // NOLINT
      
    {
}

RecPolygon2D::RecPolygon2D(const RecPolygon2D&) = default;

// NOLINTNEXTLINE
RecPolygon2D::RecPolygon2D(RecPolygon2D&&) = default;

RecPolygon2D& RecPolygon2D::operator=(const RecPolygon2D&) = default;

// NOLINTNEXTLINE
RecPolygon2D& RecPolygon2D::operator=(RecPolygon2D&&) = default;

void RecPolygon2D::toCsv(std::ostream& out) const
{
    out.precision(16);
    out << std::boolalpha;
    out << numVertices;out << ",";
    // Enclose sequence in double quotations and curly brackets to keep CSV line length uniform
        
        out << "\"{{";
        
        if (!verticesP.empty())
        {
            std::copy(verticesP.begin(), verticesP.end() - 1, std::ostream_iterator<RecPoint2D>(out, "},{"));
            std::copy(verticesP.end()-1, verticesP.end(), std::ostream_iterator<RecPoint2D>(out, ""));
        }
        out << "}}\"";out << ",";
    out << center;out << ",";
    out << min;out << ",";
    out << max;out << ",";
    // Enclose sequence in double quotations and curly brackets to keep CSV line length uniform
        
        out << "\"{{";
        
        if (!angleToVertexP.empty())
        {
            std::copy(angleToVertexP.begin(), angleToVertexP.end() - 1, std::ostream_iterator<double>(out, "},{"));
            std::copy(angleToVertexP.end()-1, angleToVertexP.end(), std::ostream_iterator<double>(out, ""));
        }
        out << "}}\"";
    
}

std::stringstream RecPolygon2D::toStringstream() const
{
    std::stringstream out;
    toCsv(out);
    return out;
}

template<class Archive>
    void RecPolygon2D::serialize(Archive& ar, [[maybe_unused]] unsigned int version)
    {

        ar& CEREAL_NVP(numVertices);
        ar& CEREAL_NVP(verticesP);
        ar& CEREAL_NVP(center);
        ar& CEREAL_NVP(min);
        ar& CEREAL_NVP(max);
        ar& CEREAL_NVP(angleToVertexP);
    }


template void RecPolygon2D::serialize(cereal::BinaryInputArchive& ar, unsigned int version);
template void RecPolygon2D::serialize(cereal::BinaryOutputArchive& ar, unsigned int version);
template void RecPolygon2D::serialize(cereal::IntrospectionArchive& ar, unsigned int version);
template void RecPolygon2D::serialize(cereal::JSONInputArchive& ar, unsigned int version);
template void RecPolygon2D::serialize(cereal::JSONOutputArchive& ar, unsigned int version);
template void RecPolygon2D::serialize(cereal::PortableBinaryInputArchive& ar, unsigned int version);
template void RecPolygon2D::serialize(cereal::PortableBinaryOutputArchive& ar, unsigned int version);

std::ostream& operator<< (std::ostream& os, const RecPolygon2D& recpolygon2d)
{
    recpolygon2d.toCsv(os);
    return os;
}

bool operator== (const RecPolygon2D &lhs, const RecPolygon2D &rhs)
{
    return (  lhs.numVertices== rhs.numVertices &&
     lhs.verticesP== rhs.verticesP &&
     lhs.center== rhs.center &&
     lhs.min== rhs.min &&
     lhs.max== rhs.max &&
     lhs.angleToVertexP== rhs.angleToVertexP 
    );
}

std::size_t RecPolygon2D::hash() const noexcept
{
    std::size_t result = 17;
    result = result * 31 + std::hash<int>()(numVertices);
    result = result * 31 + hashVector(verticesP);
    result = result * 31 + std::hash<RecPoint2D>()(center);
    result = result * 31 + std::hash<RecPoint2D>()(min);
    result = result * 31 + std::hash<RecPoint2D>()(max);
    result = result * 31 + hashVector(angleToVertexP);
    return result;
}

 } // namespace av
 } // namespace geometry


CEREAL_BIND_TO_ARCHIVES(av::geometry::RecPolygon2D)




 namespace av {
 namespace geometry {


rec_geometry_avt::rec_geometry_avt() = default;

rec_geometry_avt::rec_geometry_avt(
    RecRadians RecRadians_var_arg ,
    RecPoint2D RecPoint2D_var_arg ,
    RecPoint3D RecPoint3D_var_arg ,
    RecVector2D RecVector2D_var_arg ,
    RecVector3D RecVector3D_var_arg ,
    RecPose2D RecPose2D_var_arg ,
    RecDifferentialPose2D RecDifferentialPose2D_var_arg ,
    EULER_ANGLE_ORDER EulerAngleOrder_var_arg ,
    RecPose3D RecPose3D_var_arg ,
    RecDifferentialPose3D RecDifferentialPose3D_var_arg ,
    RecQuaternion RecQuaternion_var_arg ,
    RecLineSegment2D RecLineSegment2D_var_arg ,
    RecLineSegment3D RecLineSegment3D_var_arg ,
    RecPolygon2D RecPolygon2D_var_arg 
    ):
    RecRadians_var { std::move(RecRadians_var_arg) } // NOLINT
      ,RecPoint2D_var { std::move(RecPoint2D_var_arg) } // NOLINT
      ,RecPoint3D_var { std::move(RecPoint3D_var_arg) } // NOLINT
      ,RecVector2D_var { std::move(RecVector2D_var_arg) } // NOLINT
      ,RecVector3D_var { std::move(RecVector3D_var_arg) } // NOLINT
      ,RecPose2D_var { std::move(RecPose2D_var_arg) } // NOLINT
      ,RecDifferentialPose2D_var { std::move(RecDifferentialPose2D_var_arg) } // NOLINT
      ,EulerAngleOrder_var { std::move(EulerAngleOrder_var_arg) } // NOLINT
      ,RecPose3D_var { std::move(RecPose3D_var_arg) } // NOLINT
      ,RecDifferentialPose3D_var { std::move(RecDifferentialPose3D_var_arg) } // NOLINT
      ,RecQuaternion_var { std::move(RecQuaternion_var_arg) } // NOLINT
      ,RecLineSegment2D_var { std::move(RecLineSegment2D_var_arg) } // NOLINT
      ,RecLineSegment3D_var { std::move(RecLineSegment3D_var_arg) } // NOLINT
      ,RecPolygon2D_var { std::move(RecPolygon2D_var_arg) } // NOLINT
      {
}

rec_geometry_avt::rec_geometry_avt(const rec_geometry_avt&) = default;

// NOLINTNEXTLINE
rec_geometry_avt::rec_geometry_avt(rec_geometry_avt&&) = default;

rec_geometry_avt& rec_geometry_avt::operator=(const rec_geometry_avt&) = default;

// NOLINTNEXTLINE
rec_geometry_avt& rec_geometry_avt::operator=(rec_geometry_avt&&) = default;

void rec_geometry_avt::toCsv(std::ostream& out) const
{
    out.precision(16);
    out << std::boolalpha;
    out << RecRadians_var;out << ",";
    out << RecPoint2D_var;out << ",";
    out << RecPoint3D_var;out << ",";
    out << RecVector2D_var;out << ",";
    out << RecVector3D_var;out << ",";
    out << RecPose2D_var;out << ",";
    out << RecDifferentialPose2D_var;out << ",";
    out << static_cast<uint32_t>(EulerAngleOrder_var);out << ",";
    out << RecPose3D_var;out << ",";
    out << RecDifferentialPose3D_var;out << ",";
    out << RecQuaternion_var;out << ",";
    out << RecLineSegment2D_var;out << ",";
    out << RecLineSegment3D_var;out << ",";
    out << RecPolygon2D_var;
    
}

std::stringstream rec_geometry_avt::toStringstream() const
{
    std::stringstream out;
    toCsv(out);
    return out;
}

template<class Archive>
    void rec_geometry_avt::serialize(Archive& ar, [[maybe_unused]] unsigned int version)
    {

        ar& CEREAL_NVP(RecRadians_var);
        ar& CEREAL_NVP(RecPoint2D_var);
        ar& CEREAL_NVP(RecPoint3D_var);
        ar& CEREAL_NVP(RecVector2D_var);
        ar& CEREAL_NVP(RecVector3D_var);
        ar& CEREAL_NVP(RecPose2D_var);
        ar& CEREAL_NVP(RecDifferentialPose2D_var);
        ar& CEREAL_NVP(EulerAngleOrder_var);
        ar& CEREAL_NVP(RecPose3D_var);
        ar& CEREAL_NVP(RecDifferentialPose3D_var);
        ar& CEREAL_NVP(RecQuaternion_var);
        ar& CEREAL_NVP(RecLineSegment2D_var);
        ar& CEREAL_NVP(RecLineSegment3D_var);
        ar& CEREAL_NVP(RecPolygon2D_var);
    }


template void rec_geometry_avt::serialize(cereal::BinaryInputArchive& ar, unsigned int version);
template void rec_geometry_avt::serialize(cereal::BinaryOutputArchive& ar, unsigned int version);
template void rec_geometry_avt::serialize(cereal::IntrospectionArchive& ar, unsigned int version);
template void rec_geometry_avt::serialize(cereal::JSONInputArchive& ar, unsigned int version);
template void rec_geometry_avt::serialize(cereal::JSONOutputArchive& ar, unsigned int version);
template void rec_geometry_avt::serialize(cereal::PortableBinaryInputArchive& ar, unsigned int version);
template void rec_geometry_avt::serialize(cereal::PortableBinaryOutputArchive& ar, unsigned int version);

std::ostream& operator<< (std::ostream& os, const rec_geometry_avt& rec_geometry_avt)
{
    rec_geometry_avt.toCsv(os);
    return os;
}

bool operator== (const rec_geometry_avt &lhs, const rec_geometry_avt &rhs)
{
    return (  lhs.RecRadians_var== rhs.RecRadians_var &&
     lhs.RecPoint2D_var== rhs.RecPoint2D_var &&
     lhs.RecPoint3D_var== rhs.RecPoint3D_var &&
     lhs.RecVector2D_var== rhs.RecVector2D_var &&
     lhs.RecVector3D_var== rhs.RecVector3D_var &&
     lhs.RecPose2D_var== rhs.RecPose2D_var &&
     lhs.RecDifferentialPose2D_var== rhs.RecDifferentialPose2D_var &&
     lhs.EulerAngleOrder_var== rhs.EulerAngleOrder_var &&
     lhs.RecPose3D_var== rhs.RecPose3D_var &&
     lhs.RecDifferentialPose3D_var== rhs.RecDifferentialPose3D_var &&
     lhs.RecQuaternion_var== rhs.RecQuaternion_var &&
     lhs.RecLineSegment2D_var== rhs.RecLineSegment2D_var &&
     lhs.RecLineSegment3D_var== rhs.RecLineSegment3D_var &&
     lhs.RecPolygon2D_var== rhs.RecPolygon2D_var 
    );
}

std::size_t rec_geometry_avt::hash() const noexcept
{
    std::size_t result = 17;
    result = result * 31 + std::hash<RecRadians>()(RecRadians_var);
    result = result * 31 + std::hash<RecPoint2D>()(RecPoint2D_var);
    result = result * 31 + std::hash<RecPoint3D>()(RecPoint3D_var);
    result = result * 31 + std::hash<RecVector2D>()(RecVector2D_var);
    result = result * 31 + std::hash<RecVector3D>()(RecVector3D_var);
    result = result * 31 + std::hash<RecPose2D>()(RecPose2D_var);
    result = result * 31 + std::hash<RecDifferentialPose2D>()(RecDifferentialPose2D_var);
    result = result * 31 + std::hash<EULER_ANGLE_ORDER>()(EulerAngleOrder_var);
    result = result * 31 + std::hash<RecPose3D>()(RecPose3D_var);
    result = result * 31 + std::hash<RecDifferentialPose3D>()(RecDifferentialPose3D_var);
    result = result * 31 + std::hash<RecQuaternion>()(RecQuaternion_var);
    result = result * 31 + std::hash<RecLineSegment2D>()(RecLineSegment2D_var);
    result = result * 31 + std::hash<RecLineSegment3D>()(RecLineSegment3D_var);
    result = result * 31 + std::hash<RecPolygon2D>()(RecPolygon2D_var);
    return result;
}

 } // namespace av
 } // namespace geometry


CEREAL_BIND_TO_ARCHIVES(av::geometry::rec_geometry_avt)

