#include <interfaces/route_obstacles/route_obstacles.h>

#include <iterator>
#include <type_traits>

#include <cereal/cereal.hpp>
#include <cereal/archives/json.hpp>
#include <cereal/archives/binary.hpp>

#include <cereal/types/map.hpp>
#include <cereal/types/vector.hpp>
#include <cereal/types/array.hpp>
#include <cereal/types/string.hpp>
#include <cereal/types/complex.hpp>

#include <ODatum/IntrospectionArchive.h>
#include <ODatum/Hash.h>


 namespace av {
 namespace msg {
 namespace plan_ctrl {


track_info::track_info() = default;

track_info::track_info(
    int64_t id_arg ,
    TRACK_TYPE type_arg 
    ):
    id { std::move(id_arg) } // NOLINT
      ,
    type { std::move(type_arg) } // NOLINT
      
    {
}

track_info::track_info(const track_info&) = default;

// NOLINTNEXTLINE
track_info::track_info(track_info&&) = default;

track_info& track_info::operator=(const track_info&) = default;

// NOLINTNEXTLINE
track_info& track_info::operator=(track_info&&) = default;

void track_info::toCsv(std::ostream& out) const
{
    out.precision(16);
    out << std::boolalpha;
    out << id;out << ",";
    out << static_cast<uint32_t>(type);
    
}

std::stringstream track_info::toStringstream() const
{
    std::stringstream out;
    toCsv(out);
    return out;
}

template<class Archive>
    void track_info::serialize(Archive& ar, [[maybe_unused]] unsigned int version)
    {

        ar& CEREAL_NVP(id);
        ar& CEREAL_NVP(type);
    }


template void track_info::serialize(cereal::BinaryInputArchive& ar, unsigned int version);
template void track_info::serialize(cereal::BinaryOutputArchive& ar, unsigned int version);
template void track_info::serialize(cereal::IntrospectionArchive& ar, unsigned int version);
template void track_info::serialize(cereal::JSONInputArchive& ar, unsigned int version);
template void track_info::serialize(cereal::JSONOutputArchive& ar, unsigned int version);
template void track_info::serialize(cereal::PortableBinaryInputArchive& ar, unsigned int version);
template void track_info::serialize(cereal::PortableBinaryOutputArchive& ar, unsigned int version);

std::ostream& operator<< (std::ostream& os, const track_info& track_info)
{
    track_info.toCsv(os);
    return os;
}

bool operator== (const track_info &lhs, const track_info &rhs)
{
    return (  lhs.id== rhs.id &&
     lhs.type== rhs.type 
    );
}

std::size_t track_info::hash() const noexcept
{
    std::size_t result = 17;
    result = result * 31 + std::hash<int64_t>()(id);
    result = result * 31 + std::hash<TRACK_TYPE>()(type);
    return result;
}

 } // namespace av
 } // namespace msg
 } // namespace plan_ctrl


CEREAL_BIND_TO_ARCHIVES(av::msg::plan_ctrl::track_info)

 namespace av {
 namespace msg {
 namespace plan_ctrl {


point_2d::point_2d() = default;

point_2d::point_2d(
    double x_arg ,
    double y_arg 
    ):
    x { std::move(x_arg) } // NOLINT
      ,
    y { std::move(y_arg) } // NOLINT
      
    {
}

point_2d::point_2d(const point_2d&) = default;

// NOLINTNEXTLINE
point_2d::point_2d(point_2d&&) = default;

point_2d& point_2d::operator=(const point_2d&) = default;

// NOLINTNEXTLINE
point_2d& point_2d::operator=(point_2d&&) = default;

void point_2d::toCsv(std::ostream& out) const
{
    out.precision(16);
    out << std::boolalpha;
    out << x;out << ",";
    out << y;
    
}

std::stringstream point_2d::toStringstream() const
{
    std::stringstream out;
    toCsv(out);
    return out;
}

template<class Archive>
    void point_2d::serialize(Archive& ar, [[maybe_unused]] unsigned int version)
    {

        ar& CEREAL_NVP(x);
        ar& CEREAL_NVP(y);
    }


template void point_2d::serialize(cereal::BinaryInputArchive& ar, unsigned int version);
template void point_2d::serialize(cereal::BinaryOutputArchive& ar, unsigned int version);
template void point_2d::serialize(cereal::IntrospectionArchive& ar, unsigned int version);
template void point_2d::serialize(cereal::JSONInputArchive& ar, unsigned int version);
template void point_2d::serialize(cereal::JSONOutputArchive& ar, unsigned int version);
template void point_2d::serialize(cereal::PortableBinaryInputArchive& ar, unsigned int version);
template void point_2d::serialize(cereal::PortableBinaryOutputArchive& ar, unsigned int version);

std::ostream& operator<< (std::ostream& os, const point_2d& point_2d)
{
    point_2d.toCsv(os);
    return os;
}

bool operator== (const point_2d &lhs, const point_2d &rhs)
{
    return (  lhs.x== rhs.x &&
     lhs.y== rhs.y 
    );
}

std::size_t point_2d::hash() const noexcept
{
    std::size_t result = 17;
    result = result * 31 + std::hash<double>()(x);
    result = result * 31 + std::hash<double>()(y);
    return result;
}

 } // namespace av
 } // namespace msg
 } // namespace plan_ctrl


CEREAL_BIND_TO_ARCHIVES(av::msg::plan_ctrl::point_2d)

 namespace av {
 namespace msg {
 namespace plan_ctrl {


spatio_temporal_interval::spatio_temporal_interval() = default;

spatio_temporal_interval::spatio_temporal_interval(
    double clearance_left_annotation_in_m_arg ,
    double clearance_right_annotation_in_m_arg ,
    point_2d cartesian_point_at_projection_start_annotation_arg ,
    point_2d cartesian_point_at_projection_end_annotation_arg ,
    double time_point_in_sec_arg ,
    double transition_start_in_m_arg ,
    double transition_end_in_m_arg 
    ):
    clearance_left_annotation_in_m { std::move(clearance_left_annotation_in_m_arg) } // NOLINT
      ,
    clearance_right_annotation_in_m { std::move(clearance_right_annotation_in_m_arg) } // NOLINT
      ,
    cartesian_point_at_projection_start_annotation { std::move(cartesian_point_at_projection_start_annotation_arg) } // NOLINT
      ,
    cartesian_point_at_projection_end_annotation { std::move(cartesian_point_at_projection_end_annotation_arg) } // NOLINT
      ,
    time_point_in_sec { std::move(time_point_in_sec_arg) } // NOLINT
      ,
    transition_start_in_m { std::move(transition_start_in_m_arg) } // NOLINT
      ,
    transition_end_in_m { std::move(transition_end_in_m_arg) } // NOLINT
      
    {
}

spatio_temporal_interval::spatio_temporal_interval(const spatio_temporal_interval&) = default;

// NOLINTNEXTLINE
spatio_temporal_interval::spatio_temporal_interval(spatio_temporal_interval&&) = default;

spatio_temporal_interval& spatio_temporal_interval::operator=(const spatio_temporal_interval&) = default;

// NOLINTNEXTLINE
spatio_temporal_interval& spatio_temporal_interval::operator=(spatio_temporal_interval&&) = default;

void spatio_temporal_interval::toCsv(std::ostream& out) const
{
    out.precision(16);
    out << std::boolalpha;
    out << clearance_left_annotation_in_m;out << ",";
    out << clearance_right_annotation_in_m;out << ",";
    out << cartesian_point_at_projection_start_annotation;out << ",";
    out << cartesian_point_at_projection_end_annotation;out << ",";
    out << time_point_in_sec;out << ",";
    out << transition_start_in_m;out << ",";
    out << transition_end_in_m;
    
}

std::stringstream spatio_temporal_interval::toStringstream() const
{
    std::stringstream out;
    toCsv(out);
    return out;
}

template<class Archive>
    void spatio_temporal_interval::serialize(Archive& ar, [[maybe_unused]] unsigned int version)
    {

        ar& CEREAL_NVP(clearance_left_annotation_in_m);
        ar& CEREAL_NVP(clearance_right_annotation_in_m);
        ar& CEREAL_NVP(cartesian_point_at_projection_start_annotation);
        ar& CEREAL_NVP(cartesian_point_at_projection_end_annotation);
        ar& CEREAL_NVP(time_point_in_sec);
        ar& CEREAL_NVP(transition_start_in_m);
        ar& CEREAL_NVP(transition_end_in_m);
    }


template void spatio_temporal_interval::serialize(cereal::BinaryInputArchive& ar, unsigned int version);
template void spatio_temporal_interval::serialize(cereal::BinaryOutputArchive& ar, unsigned int version);
template void spatio_temporal_interval::serialize(cereal::IntrospectionArchive& ar, unsigned int version);
template void spatio_temporal_interval::serialize(cereal::JSONInputArchive& ar, unsigned int version);
template void spatio_temporal_interval::serialize(cereal::JSONOutputArchive& ar, unsigned int version);
template void spatio_temporal_interval::serialize(cereal::PortableBinaryInputArchive& ar, unsigned int version);
template void spatio_temporal_interval::serialize(cereal::PortableBinaryOutputArchive& ar, unsigned int version);

std::ostream& operator<< (std::ostream& os, const spatio_temporal_interval& spatio_temporal_interval)
{
    spatio_temporal_interval.toCsv(os);
    return os;
}

bool operator== (const spatio_temporal_interval &lhs, const spatio_temporal_interval &rhs)
{
    return (  lhs.clearance_left_annotation_in_m== rhs.clearance_left_annotation_in_m &&
     lhs.clearance_right_annotation_in_m== rhs.clearance_right_annotation_in_m &&
     lhs.cartesian_point_at_projection_start_annotation== rhs.cartesian_point_at_projection_start_annotation &&
     lhs.cartesian_point_at_projection_end_annotation== rhs.cartesian_point_at_projection_end_annotation &&
     lhs.time_point_in_sec== rhs.time_point_in_sec &&
     lhs.transition_start_in_m== rhs.transition_start_in_m &&
     lhs.transition_end_in_m== rhs.transition_end_in_m 
    );
}

std::size_t spatio_temporal_interval::hash() const noexcept
{
    std::size_t result = 17;
    result = result * 31 + std::hash<double>()(clearance_left_annotation_in_m);
    result = result * 31 + std::hash<double>()(clearance_right_annotation_in_m);
    result = result * 31 + std::hash<point_2d>()(cartesian_point_at_projection_start_annotation);
    result = result * 31 + std::hash<point_2d>()(cartesian_point_at_projection_end_annotation);
    result = result * 31 + std::hash<double>()(time_point_in_sec);
    result = result * 31 + std::hash<double>()(transition_start_in_m);
    result = result * 31 + std::hash<double>()(transition_end_in_m);
    return result;
}

 } // namespace av
 } // namespace msg
 } // namespace plan_ctrl


CEREAL_BIND_TO_ARCHIVES(av::msg::plan_ctrl::spatio_temporal_interval)

 namespace av {
 namespace msg {
 namespace plan_ctrl {


spatio_temporal_obstacle::spatio_temporal_obstacle() = default;

spatio_temporal_obstacle::spatio_temporal_obstacle(
    OBSTACLE_TYPE type_arg ,
    std::vector<spatio_temporal_interval> station_intervals_arg ,
    double min_clearance_left_annotation_in_m_arg ,
    double min_clearance_right_annotation_in_m_arg 
    ):
    type { std::move(type_arg) } // NOLINT
      ,
    station_intervals { std::move(station_intervals_arg) } // NOLINT
      ,
    min_clearance_left_annotation_in_m { std::move(min_clearance_left_annotation_in_m_arg) } // NOLINT
      ,
    min_clearance_right_annotation_in_m { std::move(min_clearance_right_annotation_in_m_arg) } // NOLINT
      
    {
}

spatio_temporal_obstacle::spatio_temporal_obstacle(const spatio_temporal_obstacle&) = default;

// NOLINTNEXTLINE
spatio_temporal_obstacle::spatio_temporal_obstacle(spatio_temporal_obstacle&&) = default;

spatio_temporal_obstacle& spatio_temporal_obstacle::operator=(const spatio_temporal_obstacle&) = default;

// NOLINTNEXTLINE
spatio_temporal_obstacle& spatio_temporal_obstacle::operator=(spatio_temporal_obstacle&&) = default;

void spatio_temporal_obstacle::toCsv(std::ostream& out) const
{
    out.precision(16);
    out << std::boolalpha;
    out << static_cast<uint32_t>(type);out << ",";
    // Enclose sequence in double quotations and curly brackets to keep CSV line length uniform
        
        out << "\"{{";
        
        if (!station_intervals.empty())
        {
            std::copy(station_intervals.begin(), station_intervals.end() - 1, std::ostream_iterator<spatio_temporal_interval>(out, "},{"));
            std::copy(station_intervals.end()-1, station_intervals.end(), std::ostream_iterator<spatio_temporal_interval>(out, ""));
        }
        out << "}}\"";out << ",";
    out << min_clearance_left_annotation_in_m;out << ",";
    out << min_clearance_right_annotation_in_m;
    
}

std::stringstream spatio_temporal_obstacle::toStringstream() const
{
    std::stringstream out;
    toCsv(out);
    return out;
}

template<class Archive>
    void spatio_temporal_obstacle::serialize(Archive& ar, [[maybe_unused]] unsigned int version)
    {

        ar& CEREAL_NVP(type);
        ar& CEREAL_NVP(station_intervals);
        ar& CEREAL_NVP(min_clearance_left_annotation_in_m);
        ar& CEREAL_NVP(min_clearance_right_annotation_in_m);
    }


template void spatio_temporal_obstacle::serialize(cereal::BinaryInputArchive& ar, unsigned int version);
template void spatio_temporal_obstacle::serialize(cereal::BinaryOutputArchive& ar, unsigned int version);
template void spatio_temporal_obstacle::serialize(cereal::IntrospectionArchive& ar, unsigned int version);
template void spatio_temporal_obstacle::serialize(cereal::JSONInputArchive& ar, unsigned int version);
template void spatio_temporal_obstacle::serialize(cereal::JSONOutputArchive& ar, unsigned int version);
template void spatio_temporal_obstacle::serialize(cereal::PortableBinaryInputArchive& ar, unsigned int version);
template void spatio_temporal_obstacle::serialize(cereal::PortableBinaryOutputArchive& ar, unsigned int version);

std::ostream& operator<< (std::ostream& os, const spatio_temporal_obstacle& spatio_temporal_obstacle)
{
    spatio_temporal_obstacle.toCsv(os);
    return os;
}

bool operator== (const spatio_temporal_obstacle &lhs, const spatio_temporal_obstacle &rhs)
{
    return (  lhs.type== rhs.type &&
     lhs.station_intervals== rhs.station_intervals &&
     lhs.min_clearance_left_annotation_in_m== rhs.min_clearance_left_annotation_in_m &&
     lhs.min_clearance_right_annotation_in_m== rhs.min_clearance_right_annotation_in_m 
    );
}

std::size_t spatio_temporal_obstacle::hash() const noexcept
{
    std::size_t result = 17;
    result = result * 31 + std::hash<OBSTACLE_TYPE>()(type);
    result = result * 31 + hashVector(station_intervals);
    result = result * 31 + std::hash<double>()(min_clearance_left_annotation_in_m);
    result = result * 31 + std::hash<double>()(min_clearance_right_annotation_in_m);
    return result;
}

 } // namespace av
 } // namespace msg
 } // namespace plan_ctrl


CEREAL_BIND_TO_ARCHIVES(av::msg::plan_ctrl::spatio_temporal_obstacle)

 namespace av {
 namespace msg {
 namespace plan_ctrl {


track_obstacle::track_obstacle() = default;

track_obstacle::track_obstacle(
    spatio_temporal_obstacle obstacle_arg ,
    track_info info_arg 
    ):
    obstacle { std::move(obstacle_arg) } // NOLINT
      ,
    info { std::move(info_arg) } // NOLINT
      
    {
}

track_obstacle::track_obstacle(const track_obstacle&) = default;

// NOLINTNEXTLINE
track_obstacle::track_obstacle(track_obstacle&&) = default;

track_obstacle& track_obstacle::operator=(const track_obstacle&) = default;

// NOLINTNEXTLINE
track_obstacle& track_obstacle::operator=(track_obstacle&&) = default;

void track_obstacle::toCsv(std::ostream& out) const
{
    out.precision(16);
    out << std::boolalpha;
    out << obstacle;out << ",";
    out << info;
    
}

std::stringstream track_obstacle::toStringstream() const
{
    std::stringstream out;
    toCsv(out);
    return out;
}

template<class Archive>
    void track_obstacle::serialize(Archive& ar, [[maybe_unused]] unsigned int version)
    {

        ar& CEREAL_NVP(obstacle);
        ar& CEREAL_NVP(info);
    }


template void track_obstacle::serialize(cereal::BinaryInputArchive& ar, unsigned int version);
template void track_obstacle::serialize(cereal::BinaryOutputArchive& ar, unsigned int version);
template void track_obstacle::serialize(cereal::IntrospectionArchive& ar, unsigned int version);
template void track_obstacle::serialize(cereal::JSONInputArchive& ar, unsigned int version);
template void track_obstacle::serialize(cereal::JSONOutputArchive& ar, unsigned int version);
template void track_obstacle::serialize(cereal::PortableBinaryInputArchive& ar, unsigned int version);
template void track_obstacle::serialize(cereal::PortableBinaryOutputArchive& ar, unsigned int version);

std::ostream& operator<< (std::ostream& os, const track_obstacle& track_obstacle)
{
    track_obstacle.toCsv(os);
    return os;
}

bool operator== (const track_obstacle &lhs, const track_obstacle &rhs)
{
    return (  lhs.obstacle== rhs.obstacle &&
     lhs.info== rhs.info 
    );
}

std::size_t track_obstacle::hash() const noexcept
{
    std::size_t result = 17;
    result = result * 31 + std::hash<spatio_temporal_obstacle>()(obstacle);
    result = result * 31 + std::hash<track_info>()(info);
    return result;
}

 } // namespace av
 } // namespace msg
 } // namespace plan_ctrl


CEREAL_BIND_TO_ARCHIVES(av::msg::plan_ctrl::track_obstacle)

 namespace av {
 namespace msg {
 namespace plan_ctrl {


spatial_obstacle::spatial_obstacle() = default;

spatial_obstacle::spatial_obstacle(
    OBSTACLE_TYPE type_arg ,
    double transition_start_in_m_arg ,
    double transition_end_in_m_arg ,
    double clearance_left_in_m_arg ,
    double clearance_right_in_m_arg 
    ):
    type { std::move(type_arg) } // NOLINT
      ,
    transition_start_in_m { std::move(transition_start_in_m_arg) } // NOLINT
      ,
    transition_end_in_m { std::move(transition_end_in_m_arg) } // NOLINT
      ,
    clearance_left_in_m { std::move(clearance_left_in_m_arg) } // NOLINT
      ,
    clearance_right_in_m { std::move(clearance_right_in_m_arg) } // NOLINT
      
    {
}

spatial_obstacle::spatial_obstacle(const spatial_obstacle&) = default;

// NOLINTNEXTLINE
spatial_obstacle::spatial_obstacle(spatial_obstacle&&) = default;

spatial_obstacle& spatial_obstacle::operator=(const spatial_obstacle&) = default;

// NOLINTNEXTLINE
spatial_obstacle& spatial_obstacle::operator=(spatial_obstacle&&) = default;

void spatial_obstacle::toCsv(std::ostream& out) const
{
    out.precision(16);
    out << std::boolalpha;
    out << static_cast<uint32_t>(type);out << ",";
    out << transition_start_in_m;out << ",";
    out << transition_end_in_m;out << ",";
    out << clearance_left_in_m;out << ",";
    out << clearance_right_in_m;
    
}

std::stringstream spatial_obstacle::toStringstream() const
{
    std::stringstream out;
    toCsv(out);
    return out;
}

template<class Archive>
    void spatial_obstacle::serialize(Archive& ar, [[maybe_unused]] unsigned int version)
    {

        ar& CEREAL_NVP(type);
        ar& CEREAL_NVP(transition_start_in_m);
        ar& CEREAL_NVP(transition_end_in_m);
        ar& CEREAL_NVP(clearance_left_in_m);
        ar& CEREAL_NVP(clearance_right_in_m);
    }


template void spatial_obstacle::serialize(cereal::BinaryInputArchive& ar, unsigned int version);
template void spatial_obstacle::serialize(cereal::BinaryOutputArchive& ar, unsigned int version);
template void spatial_obstacle::serialize(cereal::IntrospectionArchive& ar, unsigned int version);
template void spatial_obstacle::serialize(cereal::JSONInputArchive& ar, unsigned int version);
template void spatial_obstacle::serialize(cereal::JSONOutputArchive& ar, unsigned int version);
template void spatial_obstacle::serialize(cereal::PortableBinaryInputArchive& ar, unsigned int version);
template void spatial_obstacle::serialize(cereal::PortableBinaryOutputArchive& ar, unsigned int version);

std::ostream& operator<< (std::ostream& os, const spatial_obstacle& spatial_obstacle)
{
    spatial_obstacle.toCsv(os);
    return os;
}

bool operator== (const spatial_obstacle &lhs, const spatial_obstacle &rhs)
{
    return (  lhs.type== rhs.type &&
     lhs.transition_start_in_m== rhs.transition_start_in_m &&
     lhs.transition_end_in_m== rhs.transition_end_in_m &&
     lhs.clearance_left_in_m== rhs.clearance_left_in_m &&
     lhs.clearance_right_in_m== rhs.clearance_right_in_m 
    );
}

std::size_t spatial_obstacle::hash() const noexcept
{
    std::size_t result = 17;
    result = result * 31 + std::hash<OBSTACLE_TYPE>()(type);
    result = result * 31 + std::hash<double>()(transition_start_in_m);
    result = result * 31 + std::hash<double>()(transition_end_in_m);
    result = result * 31 + std::hash<double>()(clearance_left_in_m);
    result = result * 31 + std::hash<double>()(clearance_right_in_m);
    return result;
}

 } // namespace av
 } // namespace msg
 } // namespace plan_ctrl


CEREAL_BIND_TO_ARCHIVES(av::msg::plan_ctrl::spatial_obstacle)

 namespace av {
 namespace msg {
 namespace plan_ctrl {


hull_obstacle::hull_obstacle() = default;

hull_obstacle::hull_obstacle(
    spatial_obstacle obstacle_arg ,
    track_info info_arg 
    ):
    obstacle { std::move(obstacle_arg) } // NOLINT
      ,
    info { std::move(info_arg) } // NOLINT
      
    {
}

hull_obstacle::hull_obstacle(const hull_obstacle&) = default;

// NOLINTNEXTLINE
hull_obstacle::hull_obstacle(hull_obstacle&&) = default;

hull_obstacle& hull_obstacle::operator=(const hull_obstacle&) = default;

// NOLINTNEXTLINE
hull_obstacle& hull_obstacle::operator=(hull_obstacle&&) = default;

void hull_obstacle::toCsv(std::ostream& out) const
{
    out.precision(16);
    out << std::boolalpha;
    out << obstacle;out << ",";
    out << info;
    
}

std::stringstream hull_obstacle::toStringstream() const
{
    std::stringstream out;
    toCsv(out);
    return out;
}

template<class Archive>
    void hull_obstacle::serialize(Archive& ar, [[maybe_unused]] unsigned int version)
    {

        ar& CEREAL_NVP(obstacle);
        ar& CEREAL_NVP(info);
    }


template void hull_obstacle::serialize(cereal::BinaryInputArchive& ar, unsigned int version);
template void hull_obstacle::serialize(cereal::BinaryOutputArchive& ar, unsigned int version);
template void hull_obstacle::serialize(cereal::IntrospectionArchive& ar, unsigned int version);
template void hull_obstacle::serialize(cereal::JSONInputArchive& ar, unsigned int version);
template void hull_obstacle::serialize(cereal::JSONOutputArchive& ar, unsigned int version);
template void hull_obstacle::serialize(cereal::PortableBinaryInputArchive& ar, unsigned int version);
template void hull_obstacle::serialize(cereal::PortableBinaryOutputArchive& ar, unsigned int version);

std::ostream& operator<< (std::ostream& os, const hull_obstacle& hull_obstacle)
{
    hull_obstacle.toCsv(os);
    return os;
}

bool operator== (const hull_obstacle &lhs, const hull_obstacle &rhs)
{
    return (  lhs.obstacle== rhs.obstacle &&
     lhs.info== rhs.info 
    );
}

std::size_t hull_obstacle::hash() const noexcept
{
    std::size_t result = 17;
    result = result * 31 + std::hash<spatial_obstacle>()(obstacle);
    result = result * 31 + std::hash<track_info>()(info);
    return result;
}

 } // namespace av
 } // namespace msg
 } // namespace plan_ctrl


CEREAL_BIND_TO_ARCHIVES(av::msg::plan_ctrl::hull_obstacle)

 namespace av {
 namespace msg {
 namespace plan_ctrl {


st_obstacle_set::st_obstacle_set() = default;

st_obstacle_set::st_obstacle_set(
    std::vector<track_obstacle> obstacles_arg ,
    std::vector<hull_obstacle> static_obstacles_arg 
    ):
    obstacles { std::move(obstacles_arg) } // NOLINT
      ,
    static_obstacles { std::move(static_obstacles_arg) } // NOLINT
      
    {
}

st_obstacle_set::st_obstacle_set(const st_obstacle_set&) = default;

// NOLINTNEXTLINE
st_obstacle_set::st_obstacle_set(st_obstacle_set&&) = default;

st_obstacle_set& st_obstacle_set::operator=(const st_obstacle_set&) = default;

// NOLINTNEXTLINE
st_obstacle_set& st_obstacle_set::operator=(st_obstacle_set&&) = default;

void st_obstacle_set::toCsv(std::ostream& out) const
{
    out.precision(16);
    out << std::boolalpha;
    // Enclose sequence in double quotations and curly brackets to keep CSV line length uniform
        
        out << "\"{{";
        
        if (!obstacles.empty())
        {
            std::copy(obstacles.begin(), obstacles.end() - 1, std::ostream_iterator<track_obstacle>(out, "},{"));
            std::copy(obstacles.end()-1, obstacles.end(), std::ostream_iterator<track_obstacle>(out, ""));
        }
        out << "}}\"";out << ",";
    // Enclose sequence in double quotations and curly brackets to keep CSV line length uniform
        
        out << "\"{{";
        
        if (!static_obstacles.empty())
        {
            std::copy(static_obstacles.begin(), static_obstacles.end() - 1, std::ostream_iterator<hull_obstacle>(out, "},{"));
            std::copy(static_obstacles.end()-1, static_obstacles.end(), std::ostream_iterator<hull_obstacle>(out, ""));
        }
        out << "}}\"";
    
}

std::stringstream st_obstacle_set::toStringstream() const
{
    std::stringstream out;
    toCsv(out);
    return out;
}

template<class Archive>
    void st_obstacle_set::serialize(Archive& ar, [[maybe_unused]] unsigned int version)
    {

        ar& CEREAL_NVP(obstacles);
        ar& CEREAL_NVP(static_obstacles);
    }


template void st_obstacle_set::serialize(cereal::BinaryInputArchive& ar, unsigned int version);
template void st_obstacle_set::serialize(cereal::BinaryOutputArchive& ar, unsigned int version);
template void st_obstacle_set::serialize(cereal::IntrospectionArchive& ar, unsigned int version);
template void st_obstacle_set::serialize(cereal::JSONInputArchive& ar, unsigned int version);
template void st_obstacle_set::serialize(cereal::JSONOutputArchive& ar, unsigned int version);
template void st_obstacle_set::serialize(cereal::PortableBinaryInputArchive& ar, unsigned int version);
template void st_obstacle_set::serialize(cereal::PortableBinaryOutputArchive& ar, unsigned int version);

std::ostream& operator<< (std::ostream& os, const st_obstacle_set& st_obstacle_set)
{
    st_obstacle_set.toCsv(os);
    return os;
}

bool operator== (const st_obstacle_set &lhs, const st_obstacle_set &rhs)
{
    return (  lhs.obstacles== rhs.obstacles &&
     lhs.static_obstacles== rhs.static_obstacles 
    );
}

std::size_t st_obstacle_set::hash() const noexcept
{
    std::size_t result = 17;
    result = result * 31 + hashVector(obstacles);
    result = result * 31 + hashVector(static_obstacles);
    return result;
}

 } // namespace av
 } // namespace msg
 } // namespace plan_ctrl


CEREAL_BIND_TO_ARCHIVES(av::msg::plan_ctrl::st_obstacle_set)




 namespace av {
 namespace msg {
 namespace plan_ctrl {


route_obstacles::route_obstacles() = default;

route_obstacles::route_obstacles(
    st_obstacle_set anchor_lane_obstacles_arg ,
    std::vector<st_obstacle_set> adjacent_lanes_obstacles_arg 
    ):
    anchor_lane_obstacles { std::move(anchor_lane_obstacles_arg) } // NOLINT
      ,adjacent_lanes_obstacles { std::move(adjacent_lanes_obstacles_arg) } // NOLINT
      {
}

route_obstacles::route_obstacles(const route_obstacles&) = default;

// NOLINTNEXTLINE
route_obstacles::route_obstacles(route_obstacles&&) = default;

route_obstacles& route_obstacles::operator=(const route_obstacles&) = default;

// NOLINTNEXTLINE
route_obstacles& route_obstacles::operator=(route_obstacles&&) = default;

void route_obstacles::toCsv(std::ostream& out) const
{
    out.precision(16);
    out << std::boolalpha;
    out << anchor_lane_obstacles;out << ",";
    // Enclose sequence in double quotations and curly brackets to keep CSV line length uniform
        
        out << "\"{{";
        
        if (!adjacent_lanes_obstacles.empty())
        {
            std::copy(adjacent_lanes_obstacles.begin(), adjacent_lanes_obstacles.end() - 1, std::ostream_iterator<st_obstacle_set>(out, "},{"));
            std::copy(adjacent_lanes_obstacles.end()-1, adjacent_lanes_obstacles.end(), std::ostream_iterator<st_obstacle_set>(out, ""));
        }
        out << "}}\"";
    
}

std::stringstream route_obstacles::toStringstream() const
{
    std::stringstream out;
    toCsv(out);
    return out;
}

template<class Archive>
    void route_obstacles::serialize(Archive& ar, [[maybe_unused]] unsigned int version)
    {

        ar& CEREAL_NVP(anchor_lane_obstacles);
        ar& CEREAL_NVP(adjacent_lanes_obstacles);
    }


template void route_obstacles::serialize(cereal::BinaryInputArchive& ar, unsigned int version);
template void route_obstacles::serialize(cereal::BinaryOutputArchive& ar, unsigned int version);
template void route_obstacles::serialize(cereal::IntrospectionArchive& ar, unsigned int version);
template void route_obstacles::serialize(cereal::JSONInputArchive& ar, unsigned int version);
template void route_obstacles::serialize(cereal::JSONOutputArchive& ar, unsigned int version);
template void route_obstacles::serialize(cereal::PortableBinaryInputArchive& ar, unsigned int version);
template void route_obstacles::serialize(cereal::PortableBinaryOutputArchive& ar, unsigned int version);

std::ostream& operator<< (std::ostream& os, const route_obstacles& route_obstacles)
{
    route_obstacles.toCsv(os);
    return os;
}

bool operator== (const route_obstacles &lhs, const route_obstacles &rhs)
{
    return (  lhs.anchor_lane_obstacles== rhs.anchor_lane_obstacles &&
     lhs.adjacent_lanes_obstacles== rhs.adjacent_lanes_obstacles 
    );
}

std::size_t route_obstacles::hash() const noexcept
{
    std::size_t result = 17;
    result = result * 31 + std::hash<st_obstacle_set>()(anchor_lane_obstacles);
    result = result * 31 + hashVector(adjacent_lanes_obstacles);
    return result;
}

 } // namespace av
 } // namespace msg
 } // namespace plan_ctrl


CEREAL_BIND_TO_ARCHIVES(av::msg::plan_ctrl::route_obstacles)

