#include <interfaces/sam_message/sam_message.h>

#include <iterator>
#include <type_traits>

#include <cereal/cereal.hpp>
#include <cereal/archives/json.hpp>
#include <cereal/archives/binary.hpp>

#include <cereal/types/map.hpp>
#include <cereal/types/vector.hpp>
#include <cereal/types/array.hpp>
#include <cereal/types/string.hpp>
#include <cereal/types/complex.hpp>

#include <ODatum/IntrospectionArchive.h>
#include <ODatum/Hash.h>


 namespace av {
 namespace msg {


pedestrian_proximity_violation_record::pedestrian_proximity_violation_record() = default;

pedestrian_proximity_violation_record::pedestrian_proximity_violation_record(
    double distance_arg ,
    double violation_score_arg 
    ):
    distance { std::move(distance_arg) } // NOLINT
      ,
    violation_score { std::move(violation_score_arg) } // NOLINT
      
    {
}

pedestrian_proximity_violation_record::pedestrian_proximity_violation_record(const pedestrian_proximity_violation_record&) = default;

// NOLINTNEXTLINE
pedestrian_proximity_violation_record::pedestrian_proximity_violation_record(pedestrian_proximity_violation_record&&) = default;

pedestrian_proximity_violation_record& pedestrian_proximity_violation_record::operator=(const pedestrian_proximity_violation_record&) = default;

// NOLINTNEXTLINE
pedestrian_proximity_violation_record& pedestrian_proximity_violation_record::operator=(pedestrian_proximity_violation_record&&) = default;

void pedestrian_proximity_violation_record::toCsv(std::ostream& out) const
{
    out.precision(16);
    out << std::boolalpha;
    out << distance;out << ",";
    out << violation_score;
    
}

std::stringstream pedestrian_proximity_violation_record::toStringstream() const
{
    std::stringstream out;
    toCsv(out);
    return out;
}

template<class Archive>
    void pedestrian_proximity_violation_record::serialize(Archive& ar, [[maybe_unused]] unsigned int version)
    {

        ar& CEREAL_NVP(distance);
        ar& CEREAL_NVP(violation_score);
    }


template void pedestrian_proximity_violation_record::serialize(cereal::BinaryInputArchive& ar, unsigned int version);
template void pedestrian_proximity_violation_record::serialize(cereal::BinaryOutputArchive& ar, unsigned int version);
template void pedestrian_proximity_violation_record::serialize(cereal::IntrospectionArchive& ar, unsigned int version);
template void pedestrian_proximity_violation_record::serialize(cereal::JSONInputArchive& ar, unsigned int version);
template void pedestrian_proximity_violation_record::serialize(cereal::JSONOutputArchive& ar, unsigned int version);
template void pedestrian_proximity_violation_record::serialize(cereal::PortableBinaryInputArchive& ar, unsigned int version);
template void pedestrian_proximity_violation_record::serialize(cereal::PortableBinaryOutputArchive& ar, unsigned int version);

std::ostream& operator<< (std::ostream& os, const pedestrian_proximity_violation_record& pedestrian_proximity_violation_record)
{
    pedestrian_proximity_violation_record.toCsv(os);
    return os;
}

bool operator== (const pedestrian_proximity_violation_record &lhs, const pedestrian_proximity_violation_record &rhs)
{
    return (  lhs.distance== rhs.distance &&
     lhs.violation_score== rhs.violation_score 
    );
}

std::size_t pedestrian_proximity_violation_record::hash() const noexcept
{
    std::size_t result = 17;
    result = result * 31 + std::hash<double>()(distance);
    result = result * 31 + std::hash<double>()(violation_score);
    return result;
}

 } // namespace av
 } // namespace msg


CEREAL_BIND_TO_ARCHIVES(av::msg::pedestrian_proximity_violation_record)

 namespace av {
 namespace msg {


pedestrian_proximity_violation::pedestrian_proximity_violation() = default;

pedestrian_proximity_violation::pedestrian_proximity_violation(
    SAMRC rc_arg ,
    std::vector<pedestrian_proximity_violation_record> records_arg 
    ):
    rc { std::move(rc_arg) } // NOLINT
      ,
    records { std::move(records_arg) } // NOLINT
      
    {
}

pedestrian_proximity_violation::pedestrian_proximity_violation(const pedestrian_proximity_violation&) = default;

// NOLINTNEXTLINE
pedestrian_proximity_violation::pedestrian_proximity_violation(pedestrian_proximity_violation&&) = default;

pedestrian_proximity_violation& pedestrian_proximity_violation::operator=(const pedestrian_proximity_violation&) = default;

// NOLINTNEXTLINE
pedestrian_proximity_violation& pedestrian_proximity_violation::operator=(pedestrian_proximity_violation&&) = default;

void pedestrian_proximity_violation::toCsv(std::ostream& out) const
{
    out.precision(16);
    out << std::boolalpha;
    out << static_cast<uint32_t>(rc);out << ",";
    // Enclose sequence in double quotations and curly brackets to keep CSV line length uniform
        
        out << "\"{{";
        
        if (!records.empty())
        {
            std::copy(records.begin(), records.end() - 1, std::ostream_iterator<pedestrian_proximity_violation_record>(out, "},{"));
            std::copy(records.end()-1, records.end(), std::ostream_iterator<pedestrian_proximity_violation_record>(out, ""));
        }
        out << "}}\"";
    
}

std::stringstream pedestrian_proximity_violation::toStringstream() const
{
    std::stringstream out;
    toCsv(out);
    return out;
}

template<class Archive>
    void pedestrian_proximity_violation::serialize(Archive& ar, [[maybe_unused]] unsigned int version)
    {

        ar& CEREAL_NVP(rc);
        ar& CEREAL_NVP(records);
    }


template void pedestrian_proximity_violation::serialize(cereal::BinaryInputArchive& ar, unsigned int version);
template void pedestrian_proximity_violation::serialize(cereal::BinaryOutputArchive& ar, unsigned int version);
template void pedestrian_proximity_violation::serialize(cereal::IntrospectionArchive& ar, unsigned int version);
template void pedestrian_proximity_violation::serialize(cereal::JSONInputArchive& ar, unsigned int version);
template void pedestrian_proximity_violation::serialize(cereal::JSONOutputArchive& ar, unsigned int version);
template void pedestrian_proximity_violation::serialize(cereal::PortableBinaryInputArchive& ar, unsigned int version);
template void pedestrian_proximity_violation::serialize(cereal::PortableBinaryOutputArchive& ar, unsigned int version);

std::ostream& operator<< (std::ostream& os, const pedestrian_proximity_violation& pedestrian_proximity_violation)
{
    pedestrian_proximity_violation.toCsv(os);
    return os;
}

bool operator== (const pedestrian_proximity_violation &lhs, const pedestrian_proximity_violation &rhs)
{
    return (  lhs.rc== rhs.rc &&
     lhs.records== rhs.records 
    );
}

std::size_t pedestrian_proximity_violation::hash() const noexcept
{
    std::size_t result = 17;
    result = result * 31 + std::hash<SAMRC>()(rc);
    result = result * 31 + hashVector(records);
    return result;
}

 } // namespace av
 } // namespace msg


CEREAL_BIND_TO_ARCHIVES(av::msg::pedestrian_proximity_violation)

 namespace av {
 namespace msg {


pedestrian_proximity::pedestrian_proximity() = default;

pedestrian_proximity::pedestrian_proximity(
    SAMRC rc_arg ,
    std::vector<uint64_t> on_road_ids_arg ,
    std::vector<uint64_t> off_road_ids_arg 
    ):
    rc { std::move(rc_arg) } // NOLINT
      ,
    on_road_ids { std::move(on_road_ids_arg) } // NOLINT
      ,
    off_road_ids { std::move(off_road_ids_arg) } // NOLINT
      
    {
}

pedestrian_proximity::pedestrian_proximity(const pedestrian_proximity&) = default;

// NOLINTNEXTLINE
pedestrian_proximity::pedestrian_proximity(pedestrian_proximity&&) = default;

pedestrian_proximity& pedestrian_proximity::operator=(const pedestrian_proximity&) = default;

// NOLINTNEXTLINE
pedestrian_proximity& pedestrian_proximity::operator=(pedestrian_proximity&&) = default;

void pedestrian_proximity::toCsv(std::ostream& out) const
{
    out.precision(16);
    out << std::boolalpha;
    out << static_cast<uint32_t>(rc);out << ",";
    // Enclose sequence in double quotations and curly brackets to keep CSV line length uniform
        
        out << "\"{{";
        
        if (!on_road_ids.empty())
        {
            std::copy(on_road_ids.begin(), on_road_ids.end() - 1, std::ostream_iterator<uint64_t>(out, "},{"));
            std::copy(on_road_ids.end()-1, on_road_ids.end(), std::ostream_iterator<uint64_t>(out, ""));
        }
        out << "}}\"";out << ",";
    // Enclose sequence in double quotations and curly brackets to keep CSV line length uniform
        
        out << "\"{{";
        
        if (!off_road_ids.empty())
        {
            std::copy(off_road_ids.begin(), off_road_ids.end() - 1, std::ostream_iterator<uint64_t>(out, "},{"));
            std::copy(off_road_ids.end()-1, off_road_ids.end(), std::ostream_iterator<uint64_t>(out, ""));
        }
        out << "}}\"";
    
}

std::stringstream pedestrian_proximity::toStringstream() const
{
    std::stringstream out;
    toCsv(out);
    return out;
}

template<class Archive>
    void pedestrian_proximity::serialize(Archive& ar, [[maybe_unused]] unsigned int version)
    {

        ar& CEREAL_NVP(rc);
        ar& CEREAL_NVP(on_road_ids);
        ar& CEREAL_NVP(off_road_ids);
    }


template void pedestrian_proximity::serialize(cereal::BinaryInputArchive& ar, unsigned int version);
template void pedestrian_proximity::serialize(cereal::BinaryOutputArchive& ar, unsigned int version);
template void pedestrian_proximity::serialize(cereal::IntrospectionArchive& ar, unsigned int version);
template void pedestrian_proximity::serialize(cereal::JSONInputArchive& ar, unsigned int version);
template void pedestrian_proximity::serialize(cereal::JSONOutputArchive& ar, unsigned int version);
template void pedestrian_proximity::serialize(cereal::PortableBinaryInputArchive& ar, unsigned int version);
template void pedestrian_proximity::serialize(cereal::PortableBinaryOutputArchive& ar, unsigned int version);

std::ostream& operator<< (std::ostream& os, const pedestrian_proximity& pedestrian_proximity)
{
    pedestrian_proximity.toCsv(os);
    return os;
}

bool operator== (const pedestrian_proximity &lhs, const pedestrian_proximity &rhs)
{
    return (  lhs.rc== rhs.rc &&
     lhs.on_road_ids== rhs.on_road_ids &&
     lhs.off_road_ids== rhs.off_road_ids 
    );
}

std::size_t pedestrian_proximity::hash() const noexcept
{
    std::size_t result = 17;
    result = result * 31 + std::hash<SAMRC>()(rc);
    result = result * 31 + hashVector(on_road_ids);
    result = result * 31 + hashVector(off_road_ids);
    return result;
}

 } // namespace av
 } // namespace msg


CEREAL_BIND_TO_ARCHIVES(av::msg::pedestrian_proximity)




 namespace av {
 namespace msg {


sam_message::sam_message() = default;

sam_message::sam_message(
    int64_t timestamp_arg ,
    int64_t localization_pose_timestamp_arg ,
    int64_t tracked_object_timestamp_arg ,
    int64_t planned_path_timestamp_arg ,
    int64_t intersection_progress_metric_object_id_arg ,
    SAMRC intersection_progress_metric_object_id_rc_arg ,
    double intersection_progress_metric_distance_enter_arg ,
    SAMRC intersection_progress_metric_distance_enter_rc_arg ,
    double intersection_progress_metric_distance_exit_arg ,
    SAMRC intersection_progress_metric_distance_exit_rc_arg ,
    uint8_t car_following_detection_metric_size_arg ,
    SAMRC car_following_detection_metric_size_rc_arg ,
    double minimum_car_following_distance_metric_distance_arg ,
    SAMRC minimum_car_following_distance_metric_distance_rc_arg ,
    pedestrian_proximity pedestrian_proximity_metric_result_arg ,
    pedestrian_proximity_violation on_road_pedestrian_proximity_violation_metric_result_arg ,
    pedestrian_proximity_violation off_road_pedestrian_proximity_violation_metric_result_arg 
    ):
    timestamp { std::move(timestamp_arg) } // NOLINT
      ,localization_pose_timestamp { std::move(localization_pose_timestamp_arg) } // NOLINT
      ,tracked_object_timestamp { std::move(tracked_object_timestamp_arg) } // NOLINT
      ,planned_path_timestamp { std::move(planned_path_timestamp_arg) } // NOLINT
      ,intersection_progress_metric_object_id { std::move(intersection_progress_metric_object_id_arg) } // NOLINT
      ,intersection_progress_metric_object_id_rc { std::move(intersection_progress_metric_object_id_rc_arg) } // NOLINT
      ,intersection_progress_metric_distance_enter { std::move(intersection_progress_metric_distance_enter_arg) } // NOLINT
      ,intersection_progress_metric_distance_enter_rc { std::move(intersection_progress_metric_distance_enter_rc_arg) } // NOLINT
      ,intersection_progress_metric_distance_exit { std::move(intersection_progress_metric_distance_exit_arg) } // NOLINT
      ,intersection_progress_metric_distance_exit_rc { std::move(intersection_progress_metric_distance_exit_rc_arg) } // NOLINT
      ,car_following_detection_metric_size { std::move(car_following_detection_metric_size_arg) } // NOLINT
      ,car_following_detection_metric_size_rc { std::move(car_following_detection_metric_size_rc_arg) } // NOLINT
      ,minimum_car_following_distance_metric_distance { std::move(minimum_car_following_distance_metric_distance_arg) } // NOLINT
      ,minimum_car_following_distance_metric_distance_rc { std::move(minimum_car_following_distance_metric_distance_rc_arg) } // NOLINT
      ,pedestrian_proximity_metric_result { std::move(pedestrian_proximity_metric_result_arg) } // NOLINT
      ,on_road_pedestrian_proximity_violation_metric_result { std::move(on_road_pedestrian_proximity_violation_metric_result_arg) } // NOLINT
      ,off_road_pedestrian_proximity_violation_metric_result { std::move(off_road_pedestrian_proximity_violation_metric_result_arg) } // NOLINT
      {
}

sam_message::sam_message(const sam_message&) = default;

// NOLINTNEXTLINE
sam_message::sam_message(sam_message&&) = default;

sam_message& sam_message::operator=(const sam_message&) = default;

// NOLINTNEXTLINE
sam_message& sam_message::operator=(sam_message&&) = default;

void sam_message::toCsv(std::ostream& out) const
{
    out.precision(16);
    out << std::boolalpha;
    out << timestamp;out << ",";
    out << localization_pose_timestamp;out << ",";
    out << tracked_object_timestamp;out << ",";
    out << planned_path_timestamp;out << ",";
    out << intersection_progress_metric_object_id;out << ",";
    out << static_cast<uint32_t>(intersection_progress_metric_object_id_rc);out << ",";
    out << intersection_progress_metric_distance_enter;out << ",";
    out << static_cast<uint32_t>(intersection_progress_metric_distance_enter_rc);out << ",";
    out << intersection_progress_metric_distance_exit;out << ",";
    out << static_cast<uint32_t>(intersection_progress_metric_distance_exit_rc);out << ",";
    // Cast to uint16_t to print as integer instead of char
        out << static_cast<uint16_t>(car_following_detection_metric_size);out << ",";
    out << static_cast<uint32_t>(car_following_detection_metric_size_rc);out << ",";
    out << minimum_car_following_distance_metric_distance;out << ",";
    out << static_cast<uint32_t>(minimum_car_following_distance_metric_distance_rc);out << ",";
    out << pedestrian_proximity_metric_result;out << ",";
    out << on_road_pedestrian_proximity_violation_metric_result;out << ",";
    out << off_road_pedestrian_proximity_violation_metric_result;
    
}

std::stringstream sam_message::toStringstream() const
{
    std::stringstream out;
    toCsv(out);
    return out;
}

template<class Archive>
    void sam_message::serialize(Archive& ar, [[maybe_unused]] unsigned int version)
    {

        ar& CEREAL_NVP(timestamp);
        ar& CEREAL_NVP(localization_pose_timestamp);
        ar& CEREAL_NVP(tracked_object_timestamp);
        ar& CEREAL_NVP(planned_path_timestamp);
        ar& CEREAL_NVP(intersection_progress_metric_object_id);
        ar& CEREAL_NVP(intersection_progress_metric_object_id_rc);
        ar& CEREAL_NVP(intersection_progress_metric_distance_enter);
        ar& CEREAL_NVP(intersection_progress_metric_distance_enter_rc);
        ar& CEREAL_NVP(intersection_progress_metric_distance_exit);
        ar& CEREAL_NVP(intersection_progress_metric_distance_exit_rc);
        ar& CEREAL_NVP(car_following_detection_metric_size);
        ar& CEREAL_NVP(car_following_detection_metric_size_rc);
        ar& CEREAL_NVP(minimum_car_following_distance_metric_distance);
        ar& CEREAL_NVP(minimum_car_following_distance_metric_distance_rc);
        ar& CEREAL_NVP(pedestrian_proximity_metric_result);
        ar& CEREAL_NVP(on_road_pedestrian_proximity_violation_metric_result);
        ar& CEREAL_NVP(off_road_pedestrian_proximity_violation_metric_result);
    }


template void sam_message::serialize(cereal::BinaryInputArchive& ar, unsigned int version);
template void sam_message::serialize(cereal::BinaryOutputArchive& ar, unsigned int version);
template void sam_message::serialize(cereal::IntrospectionArchive& ar, unsigned int version);
template void sam_message::serialize(cereal::JSONInputArchive& ar, unsigned int version);
template void sam_message::serialize(cereal::JSONOutputArchive& ar, unsigned int version);
template void sam_message::serialize(cereal::PortableBinaryInputArchive& ar, unsigned int version);
template void sam_message::serialize(cereal::PortableBinaryOutputArchive& ar, unsigned int version);

std::ostream& operator<< (std::ostream& os, const sam_message& sam_message)
{
    sam_message.toCsv(os);
    return os;
}

bool operator== (const sam_message &lhs, const sam_message &rhs)
{
    return (  lhs.timestamp== rhs.timestamp &&
     lhs.localization_pose_timestamp== rhs.localization_pose_timestamp &&
     lhs.tracked_object_timestamp== rhs.tracked_object_timestamp &&
     lhs.planned_path_timestamp== rhs.planned_path_timestamp &&
     lhs.intersection_progress_metric_object_id== rhs.intersection_progress_metric_object_id &&
     lhs.intersection_progress_metric_object_id_rc== rhs.intersection_progress_metric_object_id_rc &&
     lhs.intersection_progress_metric_distance_enter== rhs.intersection_progress_metric_distance_enter &&
     lhs.intersection_progress_metric_distance_enter_rc== rhs.intersection_progress_metric_distance_enter_rc &&
     lhs.intersection_progress_metric_distance_exit== rhs.intersection_progress_metric_distance_exit &&
     lhs.intersection_progress_metric_distance_exit_rc== rhs.intersection_progress_metric_distance_exit_rc &&
     lhs.car_following_detection_metric_size== rhs.car_following_detection_metric_size &&
     lhs.car_following_detection_metric_size_rc== rhs.car_following_detection_metric_size_rc &&
     lhs.minimum_car_following_distance_metric_distance== rhs.minimum_car_following_distance_metric_distance &&
     lhs.minimum_car_following_distance_metric_distance_rc== rhs.minimum_car_following_distance_metric_distance_rc &&
     lhs.pedestrian_proximity_metric_result== rhs.pedestrian_proximity_metric_result &&
     lhs.on_road_pedestrian_proximity_violation_metric_result== rhs.on_road_pedestrian_proximity_violation_metric_result &&
     lhs.off_road_pedestrian_proximity_violation_metric_result== rhs.off_road_pedestrian_proximity_violation_metric_result 
    );
}

std::size_t sam_message::hash() const noexcept
{
    std::size_t result = 17;
    result = result * 31 + std::hash<int64_t>()(timestamp);
    result = result * 31 + std::hash<int64_t>()(localization_pose_timestamp);
    result = result * 31 + std::hash<int64_t>()(tracked_object_timestamp);
    result = result * 31 + std::hash<int64_t>()(planned_path_timestamp);
    result = result * 31 + std::hash<int64_t>()(intersection_progress_metric_object_id);
    result = result * 31 + std::hash<SAMRC>()(intersection_progress_metric_object_id_rc);
    result = result * 31 + std::hash<double>()(intersection_progress_metric_distance_enter);
    result = result * 31 + std::hash<SAMRC>()(intersection_progress_metric_distance_enter_rc);
    result = result * 31 + std::hash<double>()(intersection_progress_metric_distance_exit);
    result = result * 31 + std::hash<SAMRC>()(intersection_progress_metric_distance_exit_rc);
    result = result * 31 + std::hash<uint8_t>()(car_following_detection_metric_size);
    result = result * 31 + std::hash<SAMRC>()(car_following_detection_metric_size_rc);
    result = result * 31 + std::hash<double>()(minimum_car_following_distance_metric_distance);
    result = result * 31 + std::hash<SAMRC>()(minimum_car_following_distance_metric_distance_rc);
    result = result * 31 + std::hash<pedestrian_proximity>()(pedestrian_proximity_metric_result);
    result = result * 31 + std::hash<pedestrian_proximity_violation>()(on_road_pedestrian_proximity_violation_metric_result);
    result = result * 31 + std::hash<pedestrian_proximity_violation>()(off_road_pedestrian_proximity_violation_metric_result);
    return result;
}

 } // namespace av
 } // namespace msg


CEREAL_BIND_TO_ARCHIVES(av::msg::sam_message)

