#include <interfaces/sensor_reader_command_avt/sensor_reader_command_avt.h>

#include <iterator>
#include <type_traits>

#include <cereal/cereal.hpp>
#include <cereal/archives/json.hpp>
#include <cereal/archives/binary.hpp>

#include <cereal/types/map.hpp>
#include <cereal/types/vector.hpp>
#include <cereal/types/array.hpp>
#include <cereal/types/string.hpp>
#include <cereal/types/complex.hpp>

#include <ODatum/IntrospectionArchive.h>
#include <ODatum/Hash.h>





 namespace av {
 namespace msg {


sensor_reader_command_avt::sensor_reader_command_avt() = default;

sensor_reader_command_avt::sensor_reader_command_avt(
    int64_t sequenceCounter_arg ,
    uint64_t timeStamp_arg ,
    SENSOR_TYPE sensorType_arg ,
    COMMAND_TYPE cmdType_arg ,
    std::vector<uint8_t> cmdValue_arg 
    ):
    sequenceCounter { std::move(sequenceCounter_arg) } // NOLINT
      ,timeStamp { std::move(timeStamp_arg) } // NOLINT
      ,sensorType { std::move(sensorType_arg) } // NOLINT
      ,cmdType { std::move(cmdType_arg) } // NOLINT
      ,cmdValue { std::move(cmdValue_arg) } // NOLINT
      {
}

sensor_reader_command_avt::sensor_reader_command_avt(const sensor_reader_command_avt&) = default;

// NOLINTNEXTLINE
sensor_reader_command_avt::sensor_reader_command_avt(sensor_reader_command_avt&&) = default;

sensor_reader_command_avt& sensor_reader_command_avt::operator=(const sensor_reader_command_avt&) = default;

// NOLINTNEXTLINE
sensor_reader_command_avt& sensor_reader_command_avt::operator=(sensor_reader_command_avt&&) = default;

void sensor_reader_command_avt::toCsv(std::ostream& out) const
{
    out.precision(16);
    out << std::boolalpha;
    out << sequenceCounter;out << ",";
    out << timeStamp;out << ",";
    out << static_cast<uint32_t>(sensorType);out << ",";
    out << static_cast<uint32_t>(cmdType);out << ",";
    // Enclose sequence in double quotations and curly brackets to keep CSV line length uniform
        
        out << "\"{{";
        
        if (!cmdValue.empty())
        {
            std::copy(cmdValue.begin(), cmdValue.end() - 1, std::ostream_iterator<uint8_t>(out, "},{"));
            std::copy(cmdValue.end()-1, cmdValue.end(), std::ostream_iterator<uint8_t>(out, ""));
        }
        out << "}}\"";
    
}

std::stringstream sensor_reader_command_avt::toStringstream() const
{
    std::stringstream out;
    toCsv(out);
    return out;
}

template<class Archive>
    void sensor_reader_command_avt::serialize(Archive& ar, [[maybe_unused]] unsigned int version)
    {

        ar& CEREAL_NVP(sequenceCounter);
        ar& CEREAL_NVP(timeStamp);
        ar& CEREAL_NVP(sensorType);
        ar& CEREAL_NVP(cmdType);
        ar& CEREAL_NVP(cmdValue);
    }


template void sensor_reader_command_avt::serialize(cereal::BinaryInputArchive& ar, unsigned int version);
template void sensor_reader_command_avt::serialize(cereal::BinaryOutputArchive& ar, unsigned int version);
template void sensor_reader_command_avt::serialize(cereal::IntrospectionArchive& ar, unsigned int version);
template void sensor_reader_command_avt::serialize(cereal::JSONInputArchive& ar, unsigned int version);
template void sensor_reader_command_avt::serialize(cereal::JSONOutputArchive& ar, unsigned int version);
template void sensor_reader_command_avt::serialize(cereal::PortableBinaryInputArchive& ar, unsigned int version);
template void sensor_reader_command_avt::serialize(cereal::PortableBinaryOutputArchive& ar, unsigned int version);

std::ostream& operator<< (std::ostream& os, const sensor_reader_command_avt& sensor_reader_command_avt)
{
    sensor_reader_command_avt.toCsv(os);
    return os;
}

bool operator== (const sensor_reader_command_avt &lhs, const sensor_reader_command_avt &rhs)
{
    return (  lhs.sequenceCounter== rhs.sequenceCounter &&
     lhs.timeStamp== rhs.timeStamp &&
     lhs.sensorType== rhs.sensorType &&
     lhs.cmdType== rhs.cmdType &&
     lhs.cmdValue== rhs.cmdValue 
    );
}

std::size_t sensor_reader_command_avt::hash() const noexcept
{
    std::size_t result = 17;
    result = result * 31 + std::hash<int64_t>()(sequenceCounter);
    result = result * 31 + std::hash<uint64_t>()(timeStamp);
    result = result * 31 + std::hash<SENSOR_TYPE>()(sensorType);
    result = result * 31 + std::hash<COMMAND_TYPE>()(cmdType);
    result = result * 31 + hashVector(cmdValue);
    return result;
}

 } // namespace av
 } // namespace msg


CEREAL_BIND_TO_ARCHIVES(av::msg::sensor_reader_command_avt)

