#include <interfaces/target_prediction_avt/target_prediction_avt.h>

#include <iterator>
#include <type_traits>

#include <cereal/cereal.hpp>
#include <cereal/archives/json.hpp>
#include <cereal/archives/binary.hpp>

#include <cereal/types/map.hpp>
#include <cereal/types/vector.hpp>
#include <cereal/types/array.hpp>
#include <cereal/types/string.hpp>
#include <cereal/types/complex.hpp>

#include <ODatum/IntrospectionArchive.h>
#include <ODatum/Hash.h>


 namespace av {
 namespace msg {
 namespace prediction {


Position::Position() = default;

Position::Position(
    double x_m_arg ,
    double y_m_arg ,
    double z_m_arg 
    ):
    x_m { std::move(x_m_arg) } // NOLINT
      ,
    y_m { std::move(y_m_arg) } // NOLINT
      ,
    z_m { std::move(z_m_arg) } // NOLINT
      
    {
}

Position::Position(const Position&) = default;

// NOLINTNEXTLINE
Position::Position(Position&&) = default;

Position& Position::operator=(const Position&) = default;

// NOLINTNEXTLINE
Position& Position::operator=(Position&&) = default;

void Position::toCsv(std::ostream& out) const
{
    out.precision(16);
    out << std::boolalpha;
    out << x_m;out << ",";
    out << y_m;out << ",";
    out << z_m;
    
}

std::stringstream Position::toStringstream() const
{
    std::stringstream out;
    toCsv(out);
    return out;
}

template<class Archive>
    void Position::serialize(Archive& ar, [[maybe_unused]] unsigned int version)
    {

        ar& CEREAL_NVP(x_m);
        ar& CEREAL_NVP(y_m);
        ar& CEREAL_NVP(z_m);
    }


template void Position::serialize(cereal::BinaryInputArchive& ar, unsigned int version);
template void Position::serialize(cereal::BinaryOutputArchive& ar, unsigned int version);
template void Position::serialize(cereal::IntrospectionArchive& ar, unsigned int version);
template void Position::serialize(cereal::JSONInputArchive& ar, unsigned int version);
template void Position::serialize(cereal::JSONOutputArchive& ar, unsigned int version);
template void Position::serialize(cereal::PortableBinaryInputArchive& ar, unsigned int version);
template void Position::serialize(cereal::PortableBinaryOutputArchive& ar, unsigned int version);

std::ostream& operator<< (std::ostream& os, const Position& position)
{
    position.toCsv(os);
    return os;
}

bool operator== (const Position &lhs, const Position &rhs)
{
    return (  lhs.x_m== rhs.x_m &&
     lhs.y_m== rhs.y_m &&
     lhs.z_m== rhs.z_m 
    );
}

std::size_t Position::hash() const noexcept
{
    std::size_t result = 17;
    result = result * 31 + std::hash<double>()(x_m);
    result = result * 31 + std::hash<double>()(y_m);
    result = result * 31 + std::hash<double>()(z_m);
    return result;
}

 } // namespace av
 } // namespace msg
 } // namespace prediction


CEREAL_BIND_TO_ARCHIVES(av::msg::prediction::Position)

 namespace av {
 namespace msg {
 namespace prediction {


Size::Size() = default;

Size::Size(
    double length_m_arg ,
    double width_m_arg ,
    double height_m_arg 
    ):
    length_m { std::move(length_m_arg) } // NOLINT
      ,
    width_m { std::move(width_m_arg) } // NOLINT
      ,
    height_m { std::move(height_m_arg) } // NOLINT
      
    {
}

Size::Size(const Size&) = default;

// NOLINTNEXTLINE
Size::Size(Size&&) = default;

Size& Size::operator=(const Size&) = default;

// NOLINTNEXTLINE
Size& Size::operator=(Size&&) = default;

void Size::toCsv(std::ostream& out) const
{
    out.precision(16);
    out << std::boolalpha;
    out << length_m;out << ",";
    out << width_m;out << ",";
    out << height_m;
    
}

std::stringstream Size::toStringstream() const
{
    std::stringstream out;
    toCsv(out);
    return out;
}

template<class Archive>
    void Size::serialize(Archive& ar, [[maybe_unused]] unsigned int version)
    {

        ar& CEREAL_NVP(length_m);
        ar& CEREAL_NVP(width_m);
        ar& CEREAL_NVP(height_m);
    }


template void Size::serialize(cereal::BinaryInputArchive& ar, unsigned int version);
template void Size::serialize(cereal::BinaryOutputArchive& ar, unsigned int version);
template void Size::serialize(cereal::IntrospectionArchive& ar, unsigned int version);
template void Size::serialize(cereal::JSONInputArchive& ar, unsigned int version);
template void Size::serialize(cereal::JSONOutputArchive& ar, unsigned int version);
template void Size::serialize(cereal::PortableBinaryInputArchive& ar, unsigned int version);
template void Size::serialize(cereal::PortableBinaryOutputArchive& ar, unsigned int version);

std::ostream& operator<< (std::ostream& os, const Size& size)
{
    size.toCsv(os);
    return os;
}

bool operator== (const Size &lhs, const Size &rhs)
{
    return (  lhs.length_m== rhs.length_m &&
     lhs.width_m== rhs.width_m &&
     lhs.height_m== rhs.height_m 
    );
}

std::size_t Size::hash() const noexcept
{
    std::size_t result = 17;
    result = result * 31 + std::hash<double>()(length_m);
    result = result * 31 + std::hash<double>()(width_m);
    result = result * 31 + std::hash<double>()(height_m);
    return result;
}

 } // namespace av
 } // namespace msg
 } // namespace prediction


CEREAL_BIND_TO_ARCHIVES(av::msg::prediction::Size)

 namespace av {
 namespace msg {
 namespace prediction {


Acceleration::Acceleration() = default;

Acceleration::Acceleration(
    double accelerationX_mps2_arg ,
    double accelerationY_mps2_arg ,
    double accelerationZ_mps2_arg 
    ):
    accelerationX_mps2 { std::move(accelerationX_mps2_arg) } // NOLINT
      ,
    accelerationY_mps2 { std::move(accelerationY_mps2_arg) } // NOLINT
      ,
    accelerationZ_mps2 { std::move(accelerationZ_mps2_arg) } // NOLINT
      
    {
}

Acceleration::Acceleration(const Acceleration&) = default;

// NOLINTNEXTLINE
Acceleration::Acceleration(Acceleration&&) = default;

Acceleration& Acceleration::operator=(const Acceleration&) = default;

// NOLINTNEXTLINE
Acceleration& Acceleration::operator=(Acceleration&&) = default;

void Acceleration::toCsv(std::ostream& out) const
{
    out.precision(16);
    out << std::boolalpha;
    out << accelerationX_mps2;out << ",";
    out << accelerationY_mps2;out << ",";
    out << accelerationZ_mps2;
    
}

std::stringstream Acceleration::toStringstream() const
{
    std::stringstream out;
    toCsv(out);
    return out;
}

template<class Archive>
    void Acceleration::serialize(Archive& ar, [[maybe_unused]] unsigned int version)
    {

        ar& CEREAL_NVP(accelerationX_mps2);
        ar& CEREAL_NVP(accelerationY_mps2);
        ar& CEREAL_NVP(accelerationZ_mps2);
    }


template void Acceleration::serialize(cereal::BinaryInputArchive& ar, unsigned int version);
template void Acceleration::serialize(cereal::BinaryOutputArchive& ar, unsigned int version);
template void Acceleration::serialize(cereal::IntrospectionArchive& ar, unsigned int version);
template void Acceleration::serialize(cereal::JSONInputArchive& ar, unsigned int version);
template void Acceleration::serialize(cereal::JSONOutputArchive& ar, unsigned int version);
template void Acceleration::serialize(cereal::PortableBinaryInputArchive& ar, unsigned int version);
template void Acceleration::serialize(cereal::PortableBinaryOutputArchive& ar, unsigned int version);

std::ostream& operator<< (std::ostream& os, const Acceleration& acceleration)
{
    acceleration.toCsv(os);
    return os;
}

bool operator== (const Acceleration &lhs, const Acceleration &rhs)
{
    return (  lhs.accelerationX_mps2== rhs.accelerationX_mps2 &&
     lhs.accelerationY_mps2== rhs.accelerationY_mps2 &&
     lhs.accelerationZ_mps2== rhs.accelerationZ_mps2 
    );
}

std::size_t Acceleration::hash() const noexcept
{
    std::size_t result = 17;
    result = result * 31 + std::hash<double>()(accelerationX_mps2);
    result = result * 31 + std::hash<double>()(accelerationY_mps2);
    result = result * 31 + std::hash<double>()(accelerationZ_mps2);
    return result;
}

 } // namespace av
 } // namespace msg
 } // namespace prediction


CEREAL_BIND_TO_ARCHIVES(av::msg::prediction::Acceleration)

 namespace av {
 namespace msg {
 namespace prediction {


Vector1D::Vector1D() = default;

Vector1D::Vector1D(
    std::vector<double> data_arg 
    ):
    data { std::move(data_arg) } // NOLINT
      
    {
}

Vector1D::Vector1D(const Vector1D&) = default;

// NOLINTNEXTLINE
Vector1D::Vector1D(Vector1D&&) = default;

Vector1D& Vector1D::operator=(const Vector1D&) = default;

// NOLINTNEXTLINE
Vector1D& Vector1D::operator=(Vector1D&&) = default;

void Vector1D::toCsv(std::ostream& out) const
{
    out.precision(16);
    out << std::boolalpha;
    // Enclose sequence in double quotations and curly brackets to keep CSV line length uniform
        
        out << "\"{{";
        
        if (!data.empty())
        {
            std::copy(data.begin(), data.end() - 1, std::ostream_iterator<double>(out, "},{"));
            std::copy(data.end()-1, data.end(), std::ostream_iterator<double>(out, ""));
        }
        out << "}}\"";
    
}

std::stringstream Vector1D::toStringstream() const
{
    std::stringstream out;
    toCsv(out);
    return out;
}

template<class Archive>
    void Vector1D::serialize(Archive& ar, [[maybe_unused]] unsigned int version)
    {

        ar& CEREAL_NVP(data);
    }


template void Vector1D::serialize(cereal::BinaryInputArchive& ar, unsigned int version);
template void Vector1D::serialize(cereal::BinaryOutputArchive& ar, unsigned int version);
template void Vector1D::serialize(cereal::IntrospectionArchive& ar, unsigned int version);
template void Vector1D::serialize(cereal::JSONInputArchive& ar, unsigned int version);
template void Vector1D::serialize(cereal::JSONOutputArchive& ar, unsigned int version);
template void Vector1D::serialize(cereal::PortableBinaryInputArchive& ar, unsigned int version);
template void Vector1D::serialize(cereal::PortableBinaryOutputArchive& ar, unsigned int version);

std::ostream& operator<< (std::ostream& os, const Vector1D& vector1d)
{
    vector1d.toCsv(os);
    return os;
}

bool operator== (const Vector1D &lhs, const Vector1D &rhs)
{
    return (  lhs.data== rhs.data 
    );
}

std::size_t Vector1D::hash() const noexcept
{
    std::size_t result = 17;
    result = result * 31 + hashVector(data);
    return result;
}

 } // namespace av
 } // namespace msg
 } // namespace prediction


CEREAL_BIND_TO_ARCHIVES(av::msg::prediction::Vector1D)

 namespace av {
 namespace msg {
 namespace prediction {


Matrix2D::Matrix2D() = default;

Matrix2D::Matrix2D(
    std::vector<Vector1D> rows_arg 
    ):
    rows { std::move(rows_arg) } // NOLINT
      
    {
}

Matrix2D::Matrix2D(const Matrix2D&) = default;

// NOLINTNEXTLINE
Matrix2D::Matrix2D(Matrix2D&&) = default;

Matrix2D& Matrix2D::operator=(const Matrix2D&) = default;

// NOLINTNEXTLINE
Matrix2D& Matrix2D::operator=(Matrix2D&&) = default;

void Matrix2D::toCsv(std::ostream& out) const
{
    out.precision(16);
    out << std::boolalpha;
    // Enclose sequence in double quotations and curly brackets to keep CSV line length uniform
        
        out << "\"{{";
        
        if (!rows.empty())
        {
            std::copy(rows.begin(), rows.end() - 1, std::ostream_iterator<Vector1D>(out, "},{"));
            std::copy(rows.end()-1, rows.end(), std::ostream_iterator<Vector1D>(out, ""));
        }
        out << "}}\"";
    
}

std::stringstream Matrix2D::toStringstream() const
{
    std::stringstream out;
    toCsv(out);
    return out;
}

template<class Archive>
    void Matrix2D::serialize(Archive& ar, [[maybe_unused]] unsigned int version)
    {

        ar& CEREAL_NVP(rows);
    }


template void Matrix2D::serialize(cereal::BinaryInputArchive& ar, unsigned int version);
template void Matrix2D::serialize(cereal::BinaryOutputArchive& ar, unsigned int version);
template void Matrix2D::serialize(cereal::IntrospectionArchive& ar, unsigned int version);
template void Matrix2D::serialize(cereal::JSONInputArchive& ar, unsigned int version);
template void Matrix2D::serialize(cereal::JSONOutputArchive& ar, unsigned int version);
template void Matrix2D::serialize(cereal::PortableBinaryInputArchive& ar, unsigned int version);
template void Matrix2D::serialize(cereal::PortableBinaryOutputArchive& ar, unsigned int version);

std::ostream& operator<< (std::ostream& os, const Matrix2D& matrix2d)
{
    matrix2d.toCsv(os);
    return os;
}

bool operator== (const Matrix2D &lhs, const Matrix2D &rhs)
{
    return (  lhs.rows== rhs.rows 
    );
}

std::size_t Matrix2D::hash() const noexcept
{
    std::size_t result = 17;
    result = result * 31 + hashVector(rows);
    return result;
}

 } // namespace av
 } // namespace msg
 } // namespace prediction


CEREAL_BIND_TO_ARCHIVES(av::msg::prediction::Matrix2D)

 namespace av {
 namespace msg {
 namespace prediction {


AgentAttributes::AgentAttributes() = default;

AgentAttributes::AgentAttributes(
    int32_t type_arg ,
    std::vector<double> probs_arg ,
    double confidence_arg ,
    double prob_active_arg ,
    Size size_arg ,
    double angular_velocity_radps_arg ,
    Acceleration acceleration_arg ,
    std::vector<float> similarity_embedding_arg 
    ):
    type { std::move(type_arg) } // NOLINT
      ,
    probs { std::move(probs_arg) } // NOLINT
      ,
    confidence { std::move(confidence_arg) } // NOLINT
      ,
    prob_active { std::move(prob_active_arg) } // NOLINT
      ,
    size { std::move(size_arg) } // NOLINT
      ,
    angular_velocity_radps { std::move(angular_velocity_radps_arg) } // NOLINT
      ,
    acceleration { std::move(acceleration_arg) } // NOLINT
      ,
    similarity_embedding { std::move(similarity_embedding_arg) } // NOLINT
      
    {
}

AgentAttributes::AgentAttributes(const AgentAttributes&) = default;

// NOLINTNEXTLINE
AgentAttributes::AgentAttributes(AgentAttributes&&) = default;

AgentAttributes& AgentAttributes::operator=(const AgentAttributes&) = default;

// NOLINTNEXTLINE
AgentAttributes& AgentAttributes::operator=(AgentAttributes&&) = default;

void AgentAttributes::toCsv(std::ostream& out) const
{
    out.precision(16);
    out << std::boolalpha;
    out << type;out << ",";
    // Enclose sequence in double quotations and curly brackets to keep CSV line length uniform
        
        out << "\"{{";
        
        if (!probs.empty())
        {
            std::copy(probs.begin(), probs.end() - 1, std::ostream_iterator<double>(out, "},{"));
            std::copy(probs.end()-1, probs.end(), std::ostream_iterator<double>(out, ""));
        }
        out << "}}\"";out << ",";
    out << confidence;out << ",";
    out << prob_active;out << ",";
    out << size;out << ",";
    out << angular_velocity_radps;out << ",";
    out << acceleration;out << ",";
    // Enclose sequence in double quotations and curly brackets to keep CSV line length uniform
        
        out << "\"{{";
        
        if (!similarity_embedding.empty())
        {
            std::copy(similarity_embedding.begin(), similarity_embedding.end() - 1, std::ostream_iterator<float>(out, "},{"));
            std::copy(similarity_embedding.end()-1, similarity_embedding.end(), std::ostream_iterator<float>(out, ""));
        }
        out << "}}\"";
    
}

std::stringstream AgentAttributes::toStringstream() const
{
    std::stringstream out;
    toCsv(out);
    return out;
}

template<class Archive>
    void AgentAttributes::serialize(Archive& ar, const unsigned int version)
{
    if (version >= 1)
    {
        ar& CEREAL_NVP(type);
        ar& CEREAL_NVP(probs);
        ar& CEREAL_NVP(confidence);
        ar& CEREAL_NVP(prob_active);
        ar& CEREAL_NVP(size);
    }

    if (version >= 2)
    {
        ar& CEREAL_NVP(angular_velocity_radps);
        ar& CEREAL_NVP(acceleration);
        ar& CEREAL_NVP(similarity_embedding);
    }
}


template void AgentAttributes::serialize(cereal::BinaryInputArchive& ar, unsigned int version);
template void AgentAttributes::serialize(cereal::BinaryOutputArchive& ar, unsigned int version);
template void AgentAttributes::serialize(cereal::IntrospectionArchive& ar, unsigned int version);
template void AgentAttributes::serialize(cereal::JSONInputArchive& ar, unsigned int version);
template void AgentAttributes::serialize(cereal::JSONOutputArchive& ar, unsigned int version);
template void AgentAttributes::serialize(cereal::PortableBinaryInputArchive& ar, unsigned int version);
template void AgentAttributes::serialize(cereal::PortableBinaryOutputArchive& ar, unsigned int version);

std::ostream& operator<< (std::ostream& os, const AgentAttributes& agentattributes)
{
    agentattributes.toCsv(os);
    return os;
}

bool operator== (const AgentAttributes &lhs, const AgentAttributes &rhs)
{
    return (  lhs.type== rhs.type &&
     lhs.probs== rhs.probs &&
     lhs.confidence== rhs.confidence &&
     lhs.prob_active== rhs.prob_active &&
     lhs.size== rhs.size &&
     lhs.angular_velocity_radps== rhs.angular_velocity_radps &&
     lhs.acceleration== rhs.acceleration &&
     lhs.similarity_embedding== rhs.similarity_embedding 
    );
}

std::size_t AgentAttributes::hash() const noexcept
{
    std::size_t result = 17;
    result = result * 31 + std::hash<int32_t>()(type);
    result = result * 31 + hashVector(probs);
    result = result * 31 + std::hash<double>()(confidence);
    result = result * 31 + std::hash<double>()(prob_active);
    result = result * 31 + std::hash<Size>()(size);
    result = result * 31 + std::hash<double>()(angular_velocity_radps);
    result = result * 31 + std::hash<Acceleration>()(acceleration);
    result = result * 31 + hashVector(similarity_embedding);
    return result;
}

 } // namespace av
 } // namespace msg
 } // namespace prediction


CEREAL_BIND_TO_ARCHIVES(av::msg::prediction::AgentAttributes)

 namespace av {
 namespace msg {
 namespace prediction {


Waypoint::Waypoint() = default;

Waypoint::Waypoint(
    int64_t time_usecs_arg ,
    Position position_m_arg ,
    Matrix2D position_cov_m2_arg ,
    double heading_rad_arg ,
    Velocity velocity_mps_arg 
    ):
    time_usecs { std::move(time_usecs_arg) } // NOLINT
      ,
    position_m { std::move(position_m_arg) } // NOLINT
      ,
    position_cov_m2 { std::move(position_cov_m2_arg) } // NOLINT
      ,
    heading_rad { std::move(heading_rad_arg) } // NOLINT
      ,
    velocity_mps { std::move(velocity_mps_arg) } // NOLINT
      
    {
}

Waypoint::Waypoint(const Waypoint&) = default;

// NOLINTNEXTLINE
Waypoint::Waypoint(Waypoint&&) = default;

Waypoint& Waypoint::operator=(const Waypoint&) = default;

// NOLINTNEXTLINE
Waypoint& Waypoint::operator=(Waypoint&&) = default;

void Waypoint::toCsv(std::ostream& out) const
{
    out.precision(16);
    out << std::boolalpha;
    out << time_usecs;out << ",";
    out << position_m;out << ",";
    out << position_cov_m2;out << ",";
    out << heading_rad;out << ",";
    out << velocity_mps;
    
}

std::stringstream Waypoint::toStringstream() const
{
    std::stringstream out;
    toCsv(out);
    return out;
}

template<class Archive>
    void Waypoint::serialize(Archive& ar, [[maybe_unused]] unsigned int version)
    {

        ar& CEREAL_NVP(time_usecs);
        ar& CEREAL_NVP(position_m);
        ar& CEREAL_NVP(position_cov_m2);
        ar& CEREAL_NVP(heading_rad);
        ar& CEREAL_NVP(velocity_mps);
    }


template void Waypoint::serialize(cereal::BinaryInputArchive& ar, unsigned int version);
template void Waypoint::serialize(cereal::BinaryOutputArchive& ar, unsigned int version);
template void Waypoint::serialize(cereal::IntrospectionArchive& ar, unsigned int version);
template void Waypoint::serialize(cereal::JSONInputArchive& ar, unsigned int version);
template void Waypoint::serialize(cereal::JSONOutputArchive& ar, unsigned int version);
template void Waypoint::serialize(cereal::PortableBinaryInputArchive& ar, unsigned int version);
template void Waypoint::serialize(cereal::PortableBinaryOutputArchive& ar, unsigned int version);

std::ostream& operator<< (std::ostream& os, const Waypoint& waypoint)
{
    waypoint.toCsv(os);
    return os;
}

bool operator== (const Waypoint &lhs, const Waypoint &rhs)
{
    return (  lhs.time_usecs== rhs.time_usecs &&
     lhs.position_m== rhs.position_m &&
     lhs.position_cov_m2== rhs.position_cov_m2 &&
     lhs.heading_rad== rhs.heading_rad &&
     lhs.velocity_mps== rhs.velocity_mps 
    );
}

std::size_t Waypoint::hash() const noexcept
{
    std::size_t result = 17;
    result = result * 31 + std::hash<int64_t>()(time_usecs);
    result = result * 31 + std::hash<Position>()(position_m);
    result = result * 31 + std::hash<Matrix2D>()(position_cov_m2);
    result = result * 31 + std::hash<double>()(heading_rad);
    result = result * 31 + std::hash<Velocity>()(velocity_mps);
    return result;
}

 } // namespace av
 } // namespace msg
 } // namespace prediction


CEREAL_BIND_TO_ARCHIVES(av::msg::prediction::Waypoint)

 namespace av {
 namespace msg {
 namespace prediction {


Trajectory::Trajectory() = default;

Trajectory::Trajectory(
    double probability_arg ,
    std::vector<Waypoint> waypoints_arg 
    ):
    probability { std::move(probability_arg) } // NOLINT
      ,
    waypoints { std::move(waypoints_arg) } // NOLINT
      
    {
}

Trajectory::Trajectory(const Trajectory&) = default;

// NOLINTNEXTLINE
Trajectory::Trajectory(Trajectory&&) = default;

Trajectory& Trajectory::operator=(const Trajectory&) = default;

// NOLINTNEXTLINE
Trajectory& Trajectory::operator=(Trajectory&&) = default;

void Trajectory::toCsv(std::ostream& out) const
{
    out.precision(16);
    out << std::boolalpha;
    out << probability;out << ",";
    // Enclose sequence in double quotations and curly brackets to keep CSV line length uniform
        
        out << "\"{{";
        
        if (!waypoints.empty())
        {
            std::copy(waypoints.begin(), waypoints.end() - 1, std::ostream_iterator<Waypoint>(out, "},{"));
            std::copy(waypoints.end()-1, waypoints.end(), std::ostream_iterator<Waypoint>(out, ""));
        }
        out << "}}\"";
    
}

std::stringstream Trajectory::toStringstream() const
{
    std::stringstream out;
    toCsv(out);
    return out;
}

template<class Archive>
    void Trajectory::serialize(Archive& ar, [[maybe_unused]] unsigned int version)
    {

        ar& CEREAL_NVP(probability);
        ar& CEREAL_NVP(waypoints);
    }


template void Trajectory::serialize(cereal::BinaryInputArchive& ar, unsigned int version);
template void Trajectory::serialize(cereal::BinaryOutputArchive& ar, unsigned int version);
template void Trajectory::serialize(cereal::IntrospectionArchive& ar, unsigned int version);
template void Trajectory::serialize(cereal::JSONInputArchive& ar, unsigned int version);
template void Trajectory::serialize(cereal::JSONOutputArchive& ar, unsigned int version);
template void Trajectory::serialize(cereal::PortableBinaryInputArchive& ar, unsigned int version);
template void Trajectory::serialize(cereal::PortableBinaryOutputArchive& ar, unsigned int version);

std::ostream& operator<< (std::ostream& os, const Trajectory& trajectory)
{
    trajectory.toCsv(os);
    return os;
}

bool operator== (const Trajectory &lhs, const Trajectory &rhs)
{
    return (  lhs.probability== rhs.probability &&
     lhs.waypoints== rhs.waypoints 
    );
}

std::size_t Trajectory::hash() const noexcept
{
    std::size_t result = 17;
    result = result * 31 + std::hash<double>()(probability);
    result = result * 31 + hashVector(waypoints);
    return result;
}

 } // namespace av
 } // namespace msg
 } // namespace prediction


CEREAL_BIND_TO_ARCHIVES(av::msg::prediction::Trajectory)

 namespace av {
 namespace msg {
 namespace prediction {


Agent::Agent() = default;

Agent::Agent(
    AgentAttributes attributes_arg ,
    std::vector<prediction::Trajectory> trajectories_arg 
    ):
    attributes { std::move(attributes_arg) } // NOLINT
      ,
    trajectories { std::move(trajectories_arg) } // NOLINT
      
    {
}

Agent::Agent(const Agent&) = default;

// NOLINTNEXTLINE
Agent::Agent(Agent&&) = default;

Agent& Agent::operator=(const Agent&) = default;

// NOLINTNEXTLINE
Agent& Agent::operator=(Agent&&) = default;

void Agent::toCsv(std::ostream& out) const
{
    out.precision(16);
    out << std::boolalpha;
    out << attributes;out << ",";
    // Enclose sequence in double quotations and curly brackets to keep CSV line length uniform
        
        out << "\"{{";
        
        if (!trajectories.empty())
        {
            std::copy(trajectories.begin(), trajectories.end() - 1, std::ostream_iterator<prediction::Trajectory>(out, "},{"));
            std::copy(trajectories.end()-1, trajectories.end(), std::ostream_iterator<prediction::Trajectory>(out, ""));
        }
        out << "}}\"";
    
}

std::stringstream Agent::toStringstream() const
{
    std::stringstream out;
    toCsv(out);
    return out;
}

template<class Archive>
    void Agent::serialize(Archive& ar, [[maybe_unused]] unsigned int version)
    {

        ar& CEREAL_NVP(attributes);
        ar& CEREAL_NVP(trajectories);
    }


template void Agent::serialize(cereal::BinaryInputArchive& ar, unsigned int version);
template void Agent::serialize(cereal::BinaryOutputArchive& ar, unsigned int version);
template void Agent::serialize(cereal::IntrospectionArchive& ar, unsigned int version);
template void Agent::serialize(cereal::JSONInputArchive& ar, unsigned int version);
template void Agent::serialize(cereal::JSONOutputArchive& ar, unsigned int version);
template void Agent::serialize(cereal::PortableBinaryInputArchive& ar, unsigned int version);
template void Agent::serialize(cereal::PortableBinaryOutputArchive& ar, unsigned int version);

std::ostream& operator<< (std::ostream& os, const Agent& agent)
{
    agent.toCsv(os);
    return os;
}

bool operator== (const Agent &lhs, const Agent &rhs)
{
    return (  lhs.attributes== rhs.attributes &&
     lhs.trajectories== rhs.trajectories 
    );
}

std::size_t Agent::hash() const noexcept
{
    std::size_t result = 17;
    result = result * 31 + std::hash<AgentAttributes>()(attributes);
    result = result * 31 + hashVector(trajectories);
    return result;
}

 } // namespace av
 } // namespace msg
 } // namespace prediction


CEREAL_BIND_TO_ARCHIVES(av::msg::prediction::Agent)




 namespace av {
 namespace msg {
 namespace prediction {


target_prediction_avt::target_prediction_avt() = default;

target_prediction_avt::target_prediction_avt(
    int64_t time_usecs_arg ,
    std::string frame_id_arg ,
    float detection_range_m_arg ,
    std::vector<Agent> agents_arg 
    ):
    time_usecs { std::move(time_usecs_arg) } // NOLINT
      ,frame_id { std::move(frame_id_arg) } // NOLINT
      ,detection_range_m { std::move(detection_range_m_arg) } // NOLINT
      ,agents { std::move(agents_arg) } // NOLINT
      {
}

target_prediction_avt::target_prediction_avt(const target_prediction_avt&) = default;

// NOLINTNEXTLINE
target_prediction_avt::target_prediction_avt(target_prediction_avt&&) = default;

target_prediction_avt& target_prediction_avt::operator=(const target_prediction_avt&) = default;

// NOLINTNEXTLINE
target_prediction_avt& target_prediction_avt::operator=(target_prediction_avt&&) = default;

void target_prediction_avt::toCsv(std::ostream& out) const
{
    out.precision(16);
    out << std::boolalpha;
    out << time_usecs;out << ",";
    out << "\"";
        out << frame_id;
        out << "\"";out << ",";
    out << detection_range_m;out << ",";
    // Enclose sequence in double quotations and curly brackets to keep CSV line length uniform
        
        out << "\"{{";
        
        if (!agents.empty())
        {
            std::copy(agents.begin(), agents.end() - 1, std::ostream_iterator<Agent>(out, "},{"));
            std::copy(agents.end()-1, agents.end(), std::ostream_iterator<Agent>(out, ""));
        }
        out << "}}\"";
    
}

std::stringstream target_prediction_avt::toStringstream() const
{
    std::stringstream out;
    toCsv(out);
    return out;
}

template<class Archive>
    void target_prediction_avt::serialize(Archive& ar, [[maybe_unused]] unsigned int version)
    {

        ar& CEREAL_NVP(time_usecs);
        ar& CEREAL_NVP(frame_id);
        ar& CEREAL_NVP(detection_range_m);
        ar& CEREAL_NVP(agents);
    }


template void target_prediction_avt::serialize(cereal::BinaryInputArchive& ar, unsigned int version);
template void target_prediction_avt::serialize(cereal::BinaryOutputArchive& ar, unsigned int version);
template void target_prediction_avt::serialize(cereal::IntrospectionArchive& ar, unsigned int version);
template void target_prediction_avt::serialize(cereal::JSONInputArchive& ar, unsigned int version);
template void target_prediction_avt::serialize(cereal::JSONOutputArchive& ar, unsigned int version);
template void target_prediction_avt::serialize(cereal::PortableBinaryInputArchive& ar, unsigned int version);
template void target_prediction_avt::serialize(cereal::PortableBinaryOutputArchive& ar, unsigned int version);

std::ostream& operator<< (std::ostream& os, const target_prediction_avt& target_prediction_avt)
{
    target_prediction_avt.toCsv(os);
    return os;
}

bool operator== (const target_prediction_avt &lhs, const target_prediction_avt &rhs)
{
    return (  lhs.time_usecs== rhs.time_usecs &&
     lhs.frame_id== rhs.frame_id &&
     lhs.detection_range_m== rhs.detection_range_m &&
     lhs.agents== rhs.agents 
    );
}

std::size_t target_prediction_avt::hash() const noexcept
{
    std::size_t result = 17;
    result = result * 31 + std::hash<int64_t>()(time_usecs);
    result = result * 31 + std::hash<std::string>()(frame_id);
    result = result * 31 + std::hash<float>()(detection_range_m);
    result = result * 31 + hashVector(agents);
    return result;
}

 } // namespace av
 } // namespace msg
 } // namespace prediction


CEREAL_BIND_TO_ARCHIVES(av::msg::prediction::target_prediction_avt)

