#include <interfaces/test_msg/test_msg.h>

#include <iterator>
#include <type_traits>

#include <cereal/cereal.hpp>
#include <cereal/archives/json.hpp>
#include <cereal/archives/binary.hpp>

#include <cereal/types/map.hpp>
#include <cereal/types/vector.hpp>
#include <cereal/types/array.hpp>
#include <cereal/types/string.hpp>
#include <cereal/types/complex.hpp>

#include <ODatum/IntrospectionArchive.h>
#include <ODatum/Hash.h>


 namespace test {
 namespace msg {


custom::custom() = default;

custom::custom(
    int8_t int8_var_arg ,
    int16_t int16_var_arg ,
    uint8_t uint8_var_arg ,
    uint64_t uint64_var_arg ,
    double double_var_arg ,
    char char_var_arg ,
    std::string string_var_arg ,
    bool boolean_var_arg ,
    std::vector<int64_t> seq_int64_var_arg ,
    std::array<bool,2> arr_bool_var_arg ,
    std::array<std::string,3> arr_char_var_arg 
    ):
    int8_var { std::move(int8_var_arg) } // NOLINT
      ,
    int16_var { std::move(int16_var_arg) } // NOLINT
      ,
    uint8_var { std::move(uint8_var_arg) } // NOLINT
      ,
    uint64_var { std::move(uint64_var_arg) } // NOLINT
      ,
    double_var { std::move(double_var_arg) } // NOLINT
      ,
    char_var { std::move(char_var_arg) } // NOLINT
      ,
    string_var { std::move(string_var_arg) } // NOLINT
      ,
    boolean_var { std::move(boolean_var_arg) } // NOLINT
      ,
    seq_int64_var { std::move(seq_int64_var_arg) } // NOLINT
      ,
    arr_bool_var { std::move(arr_bool_var_arg) } // NOLINT
      ,
    arr_char_var { std::move(arr_char_var_arg) } // NOLINT
      
    {
}

custom::custom(const custom&) = default;

// NOLINTNEXTLINE
custom::custom(custom&&) = default;

custom& custom::operator=(const custom&) = default;

// NOLINTNEXTLINE
custom& custom::operator=(custom&&) = default;

void custom::toCsv(std::ostream& out) const
{
    out.precision(16);
    out << std::boolalpha;
    // Cast to int16_t to print as integer instead of char
        out << static_cast<int16_t>(int8_var);out << ",";
    out << int16_var;out << ",";
    // Cast to uint16_t to print as integer instead of char
        out << static_cast<uint16_t>(uint8_var);out << ",";
    out << uint64_var;out << ",";
    out << double_var;out << ",";
    // Avoid appending null terminator to stream
        if (char_var != '\0')
        {
            out << char_var;
        }out << ",";
    out << "\"";
        out << string_var;
        out << "\"";out << ",";
    out << boolean_var;out << ",";
    // Enclose sequence in double quotations and curly brackets to keep CSV line length uniform
        
        out << "\"{{";
        
        if (!seq_int64_var.empty())
        {
            std::copy(seq_int64_var.begin(), seq_int64_var.end() - 1, std::ostream_iterator<int64_t>(out, "},{"));
            std::copy(seq_int64_var.end()-1, seq_int64_var.end(), std::ostream_iterator<int64_t>(out, ""));
        }
        out << "}}\"";out << ",";
    // Enclose array in double quotations and curly brackets to keep CSV line length uniform
        
        out << "\"{{";
        
        if (!arr_bool_var.empty())
        {
            std::copy(arr_bool_var.begin(), arr_bool_var.end() - 1, std::ostream_iterator<bool>(out, "},{"));
            out << arr_bool_var.back();
        }
        out << "}}\"";out << ",";
    // Enclose array in double quotations and curly brackets to keep CSV line length uniform
        
        out << "\"{{";
        
        if (!arr_char_var.empty())
        {
            std::copy(arr_char_var.begin(), arr_char_var.end() - 1, std::ostream_iterator<std::string>(out, "},{"));
            out << arr_char_var.back();
        }
        out << "}}\"";
    
}

std::stringstream custom::toStringstream() const
{
    std::stringstream out;
    toCsv(out);
    return out;
}

template<class Archive>
    void custom::serialize(Archive& ar, [[maybe_unused]] unsigned int version)
    {

        ar& CEREAL_NVP(int8_var);
        ar& CEREAL_NVP(int16_var);
        ar& CEREAL_NVP(uint8_var);
        ar& CEREAL_NVP(uint64_var);
        ar& CEREAL_NVP(double_var);
        ar& CEREAL_NVP(char_var);
        ar& CEREAL_NVP(string_var);
        ar& CEREAL_NVP(boolean_var);
        ar& CEREAL_NVP(seq_int64_var);
        ar& CEREAL_NVP(arr_bool_var);
        ar& CEREAL_NVP(arr_char_var);
    }


template void custom::serialize(cereal::BinaryInputArchive& ar, unsigned int version);
template void custom::serialize(cereal::BinaryOutputArchive& ar, unsigned int version);
template void custom::serialize(cereal::IntrospectionArchive& ar, unsigned int version);
template void custom::serialize(cereal::JSONInputArchive& ar, unsigned int version);
template void custom::serialize(cereal::JSONOutputArchive& ar, unsigned int version);
template void custom::serialize(cereal::PortableBinaryInputArchive& ar, unsigned int version);
template void custom::serialize(cereal::PortableBinaryOutputArchive& ar, unsigned int version);

std::ostream& operator<< (std::ostream& os, const custom& custom)
{
    custom.toCsv(os);
    return os;
}

bool operator== (const custom &lhs, const custom &rhs)
{
    return (  lhs.int8_var== rhs.int8_var &&
     lhs.int16_var== rhs.int16_var &&
     lhs.uint8_var== rhs.uint8_var &&
     lhs.uint64_var== rhs.uint64_var &&
     lhs.double_var== rhs.double_var &&
     lhs.char_var== rhs.char_var &&
     lhs.string_var== rhs.string_var &&
     lhs.boolean_var== rhs.boolean_var &&
     lhs.seq_int64_var== rhs.seq_int64_var &&
     lhs.arr_bool_var== rhs.arr_bool_var &&
     lhs.arr_char_var== rhs.arr_char_var 
    );
}

std::size_t custom::hash() const noexcept
{
    std::size_t result = 17;
    result = result * 31 + std::hash<int8_t>()(int8_var);
    result = result * 31 + std::hash<int16_t>()(int16_var);
    result = result * 31 + std::hash<uint8_t>()(uint8_var);
    result = result * 31 + std::hash<uint64_t>()(uint64_var);
    result = result * 31 + std::hash<double>()(double_var);
    result = result * 31 + std::hash<char>()(char_var);
    result = result * 31 + std::hash<std::string>()(string_var);
    result = result * 31 + std::hash<bool>()(boolean_var);
    result = result * 31 + hashVector(seq_int64_var);
    result = result * 31 + hashVector(arr_bool_var);
    result = result * 31 + hashVector(arr_char_var);
    return result;
}

 } // namespace test
 } // namespace msg


CEREAL_BIND_TO_ARCHIVES(test::msg::custom)

 namespace test {
 namespace msg {


Pair::Pair() = default;

Pair::Pair(
    int key_arg ,
    double value_arg 
    ):
    key { std::move(key_arg) } // NOLINT
      ,
    value { std::move(value_arg) } // NOLINT
      
    {
}

Pair::Pair(const Pair&) = default;

// NOLINTNEXTLINE
Pair::Pair(Pair&&) = default;

Pair& Pair::operator=(const Pair&) = default;

// NOLINTNEXTLINE
Pair& Pair::operator=(Pair&&) = default;

void Pair::toCsv(std::ostream& out) const
{
    out.precision(16);
    out << std::boolalpha;
    out << key;out << ",";
    out << value;
    
}

std::stringstream Pair::toStringstream() const
{
    std::stringstream out;
    toCsv(out);
    return out;
}

template<class Archive>
    void Pair::serialize(Archive& ar, [[maybe_unused]] unsigned int version)
    {

        ar& CEREAL_NVP(key);
        ar& CEREAL_NVP(value);
    }


template void Pair::serialize(cereal::BinaryInputArchive& ar, unsigned int version);
template void Pair::serialize(cereal::BinaryOutputArchive& ar, unsigned int version);
template void Pair::serialize(cereal::IntrospectionArchive& ar, unsigned int version);
template void Pair::serialize(cereal::JSONInputArchive& ar, unsigned int version);
template void Pair::serialize(cereal::JSONOutputArchive& ar, unsigned int version);
template void Pair::serialize(cereal::PortableBinaryInputArchive& ar, unsigned int version);
template void Pair::serialize(cereal::PortableBinaryOutputArchive& ar, unsigned int version);

std::ostream& operator<< (std::ostream& os, const Pair& pair)
{
    pair.toCsv(os);
    return os;
}

bool operator== (const Pair &lhs, const Pair &rhs)
{
    return (  lhs.key== rhs.key &&
     lhs.value== rhs.value 
    );
}

std::size_t Pair::hash() const noexcept
{
    std::size_t result = 17;
    result = result * 31 + std::hash<int>()(key);
    result = result * 31 + std::hash<double>()(value);
    return result;
}

 } // namespace test
 } // namespace msg


CEREAL_BIND_TO_ARCHIVES(test::msg::Pair)




 namespace test {
 namespace msg {


test_msg::test_msg() = default;

test_msg::test_msg(
    custom custom_var_arg ,
    std::vector<custom> seq_custom_var_arg ,
    int8_t int8_var_arg ,
    int16_t int16_var_arg ,
    uint8_t uint8_var_arg ,
    uint64_t uint64_var_arg ,
    double double_var_arg ,
    char char_var_arg ,
    std::string string_var_arg ,
    bool boolean_var_arg ,
    std::vector<int64_t> seq_int64_var_arg ,
    std::array<bool,2> arr_bool_var_arg ,
    std::array<std::string,3> arr_char_var_arg ,
    std::vector<Pair> map_arg 
    ):
    custom_var { std::move(custom_var_arg) } // NOLINT
      ,seq_custom_var { std::move(seq_custom_var_arg) } // NOLINT
      ,int8_var { std::move(int8_var_arg) } // NOLINT
      ,int16_var { std::move(int16_var_arg) } // NOLINT
      ,uint8_var { std::move(uint8_var_arg) } // NOLINT
      ,uint64_var { std::move(uint64_var_arg) } // NOLINT
      ,double_var { std::move(double_var_arg) } // NOLINT
      ,char_var { std::move(char_var_arg) } // NOLINT
      ,string_var { std::move(string_var_arg) } // NOLINT
      ,boolean_var { std::move(boolean_var_arg) } // NOLINT
      ,seq_int64_var { std::move(seq_int64_var_arg) } // NOLINT
      ,arr_bool_var { std::move(arr_bool_var_arg) } // NOLINT
      ,arr_char_var { std::move(arr_char_var_arg) } // NOLINT
      ,map { std::move(map_arg) } // NOLINT
      {
}

test_msg::test_msg(const test_msg&) = default;

// NOLINTNEXTLINE
test_msg::test_msg(test_msg&&) = default;

test_msg& test_msg::operator=(const test_msg&) = default;

// NOLINTNEXTLINE
test_msg& test_msg::operator=(test_msg&&) = default;

void test_msg::toCsv(std::ostream& out) const
{
    out.precision(16);
    out << std::boolalpha;
    out << custom_var;out << ",";
    // Enclose sequence in double quotations and curly brackets to keep CSV line length uniform
        
        out << "\"{{";
        
        if (!seq_custom_var.empty())
        {
            std::copy(seq_custom_var.begin(), seq_custom_var.end() - 1, std::ostream_iterator<custom>(out, "},{"));
            std::copy(seq_custom_var.end()-1, seq_custom_var.end(), std::ostream_iterator<custom>(out, ""));
        }
        out << "}}\"";out << ",";
    // Cast to int16_t to print as integer instead of char
        out << static_cast<int16_t>(int8_var);out << ",";
    out << int16_var;out << ",";
    // Cast to uint16_t to print as integer instead of char
        out << static_cast<uint16_t>(uint8_var);out << ",";
    out << uint64_var;out << ",";
    out << double_var;out << ",";
    // Avoid appending null terminator to stream
        if (char_var != '\0')
        {
            out << char_var;
        }out << ",";
    out << "\"";
        out << string_var;
        out << "\"";out << ",";
    out << boolean_var;out << ",";
    // Enclose sequence in double quotations and curly brackets to keep CSV line length uniform
        
        out << "\"{{";
        
        if (!seq_int64_var.empty())
        {
            std::copy(seq_int64_var.begin(), seq_int64_var.end() - 1, std::ostream_iterator<int64_t>(out, "},{"));
            std::copy(seq_int64_var.end()-1, seq_int64_var.end(), std::ostream_iterator<int64_t>(out, ""));
        }
        out << "}}\"";out << ",";
    // Enclose array in double quotations and curly brackets to keep CSV line length uniform
        
        out << "\"{{";
        
        if (!arr_bool_var.empty())
        {
            std::copy(arr_bool_var.begin(), arr_bool_var.end() - 1, std::ostream_iterator<bool>(out, "},{"));
            out << arr_bool_var.back();
        }
        out << "}}\"";out << ",";
    // Enclose array in double quotations and curly brackets to keep CSV line length uniform
        
        out << "\"{{";
        
        if (!arr_char_var.empty())
        {
            std::copy(arr_char_var.begin(), arr_char_var.end() - 1, std::ostream_iterator<std::string>(out, "},{"));
            out << arr_char_var.back();
        }
        out << "}}\"";out << ",";
    // Enclose sequence in double quotations and curly brackets to keep CSV line length uniform
        
        out << "\"{{";
        
        if (!map.empty())
        {
            std::copy(map.begin(), map.end() - 1, std::ostream_iterator<Pair>(out, "},{"));
            std::copy(map.end()-1, map.end(), std::ostream_iterator<Pair>(out, ""));
        }
        out << "}}\"";
    
}

std::stringstream test_msg::toStringstream() const
{
    std::stringstream out;
    toCsv(out);
    return out;
}

template<class Archive>
    void test_msg::serialize(Archive& ar, [[maybe_unused]] unsigned int version)
    {

        ar& CEREAL_NVP(custom_var);
        ar& CEREAL_NVP(seq_custom_var);
        ar& CEREAL_NVP(int8_var);
        ar& CEREAL_NVP(int16_var);
        ar& CEREAL_NVP(uint8_var);
        ar& CEREAL_NVP(uint64_var);
        ar& CEREAL_NVP(double_var);
        ar& CEREAL_NVP(char_var);
        ar& CEREAL_NVP(string_var);
        ar& CEREAL_NVP(boolean_var);
        ar& CEREAL_NVP(seq_int64_var);
        ar& CEREAL_NVP(arr_bool_var);
        ar& CEREAL_NVP(arr_char_var);
        ar& CEREAL_NVP(map);
    }


template void test_msg::serialize(cereal::BinaryInputArchive& ar, unsigned int version);
template void test_msg::serialize(cereal::BinaryOutputArchive& ar, unsigned int version);
template void test_msg::serialize(cereal::IntrospectionArchive& ar, unsigned int version);
template void test_msg::serialize(cereal::JSONInputArchive& ar, unsigned int version);
template void test_msg::serialize(cereal::JSONOutputArchive& ar, unsigned int version);
template void test_msg::serialize(cereal::PortableBinaryInputArchive& ar, unsigned int version);
template void test_msg::serialize(cereal::PortableBinaryOutputArchive& ar, unsigned int version);

std::ostream& operator<< (std::ostream& os, const test_msg& test_msg)
{
    test_msg.toCsv(os);
    return os;
}

bool operator== (const test_msg &lhs, const test_msg &rhs)
{
    return (  lhs.custom_var== rhs.custom_var &&
     lhs.seq_custom_var== rhs.seq_custom_var &&
     lhs.int8_var== rhs.int8_var &&
     lhs.int16_var== rhs.int16_var &&
     lhs.uint8_var== rhs.uint8_var &&
     lhs.uint64_var== rhs.uint64_var &&
     lhs.double_var== rhs.double_var &&
     lhs.char_var== rhs.char_var &&
     lhs.string_var== rhs.string_var &&
     lhs.boolean_var== rhs.boolean_var &&
     lhs.seq_int64_var== rhs.seq_int64_var &&
     lhs.arr_bool_var== rhs.arr_bool_var &&
     lhs.arr_char_var== rhs.arr_char_var &&
     lhs.map== rhs.map 
    );
}

std::size_t test_msg::hash() const noexcept
{
    std::size_t result = 17;
    result = result * 31 + std::hash<custom>()(custom_var);
    result = result * 31 + hashVector(seq_custom_var);
    result = result * 31 + std::hash<int8_t>()(int8_var);
    result = result * 31 + std::hash<int16_t>()(int16_var);
    result = result * 31 + std::hash<uint8_t>()(uint8_var);
    result = result * 31 + std::hash<uint64_t>()(uint64_var);
    result = result * 31 + std::hash<double>()(double_var);
    result = result * 31 + std::hash<char>()(char_var);
    result = result * 31 + std::hash<std::string>()(string_var);
    result = result * 31 + std::hash<bool>()(boolean_var);
    result = result * 31 + hashVector(seq_int64_var);
    result = result * 31 + hashVector(arr_bool_var);
    result = result * 31 + hashVector(arr_char_var);
    result = result * 31 + hashVector(map);
    return result;
}

 } // namespace test
 } // namespace msg


CEREAL_BIND_TO_ARCHIVES(test::msg::test_msg)

