#include <interfaces/tracked_objects_avt/tracked_objects_avt.h>

#include <iterator>
#include <type_traits>

#include <cereal/cereal.hpp>
#include <cereal/archives/json.hpp>
#include <cereal/archives/binary.hpp>

#include <cereal/types/map.hpp>
#include <cereal/types/vector.hpp>
#include <cereal/types/array.hpp>
#include <cereal/types/string.hpp>
#include <cereal/types/complex.hpp>

#include <ODatum/IntrospectionArchive.h>
#include <ODatum/Hash.h>


 namespace av {
 namespace msg {
 namespace tracker {


BoxDimensions3D::BoxDimensions3D() = default;

BoxDimensions3D::BoxDimensions3D(
    float length_meters_arg ,
    float width_meters_arg ,
    float height_meters_arg 
    ):
    length_meters { std::move(length_meters_arg) } // NOLINT
      ,
    width_meters { std::move(width_meters_arg) } // NOLINT
      ,
    height_meters { std::move(height_meters_arg) } // NOLINT
      
    {
}

BoxDimensions3D::BoxDimensions3D(const BoxDimensions3D&) = default;

// NOLINTNEXTLINE
BoxDimensions3D::BoxDimensions3D(BoxDimensions3D&&) = default;

BoxDimensions3D& BoxDimensions3D::operator=(const BoxDimensions3D&) = default;

// NOLINTNEXTLINE
BoxDimensions3D& BoxDimensions3D::operator=(BoxDimensions3D&&) = default;

void BoxDimensions3D::toCsv(std::ostream& out) const
{
    out.precision(16);
    out << std::boolalpha;
    out << length_meters;out << ",";
    out << width_meters;out << ",";
    out << height_meters;
    
}

std::stringstream BoxDimensions3D::toStringstream() const
{
    std::stringstream out;
    toCsv(out);
    return out;
}

template<class Archive>
    void BoxDimensions3D::serialize(Archive& ar, [[maybe_unused]] unsigned int version)
    {

        ar& CEREAL_NVP(length_meters);
        ar& CEREAL_NVP(width_meters);
        ar& CEREAL_NVP(height_meters);
    }


template void BoxDimensions3D::serialize(cereal::BinaryInputArchive& ar, unsigned int version);
template void BoxDimensions3D::serialize(cereal::BinaryOutputArchive& ar, unsigned int version);
template void BoxDimensions3D::serialize(cereal::IntrospectionArchive& ar, unsigned int version);
template void BoxDimensions3D::serialize(cereal::JSONInputArchive& ar, unsigned int version);
template void BoxDimensions3D::serialize(cereal::JSONOutputArchive& ar, unsigned int version);
template void BoxDimensions3D::serialize(cereal::PortableBinaryInputArchive& ar, unsigned int version);
template void BoxDimensions3D::serialize(cereal::PortableBinaryOutputArchive& ar, unsigned int version);

std::ostream& operator<< (std::ostream& os, const BoxDimensions3D& boxdimensions3d)
{
    boxdimensions3d.toCsv(os);
    return os;
}

bool operator== (const BoxDimensions3D &lhs, const BoxDimensions3D &rhs)
{
    return (  lhs.length_meters== rhs.length_meters &&
     lhs.width_meters== rhs.width_meters &&
     lhs.height_meters== rhs.height_meters 
    );
}

std::size_t BoxDimensions3D::hash() const noexcept
{
    std::size_t result = 17;
    result = result * 31 + std::hash<float>()(length_meters);
    result = result * 31 + std::hash<float>()(width_meters);
    result = result * 31 + std::hash<float>()(height_meters);
    return result;
}

 } // namespace av
 } // namespace msg
 } // namespace tracker


CEREAL_BIND_TO_ARCHIVES(av::msg::tracker::BoxDimensions3D)

 namespace av {
 namespace msg {
 namespace tracker {


KinematicMean::KinematicMean() = default;

KinematicMean::KinematicMean(
    double x_meters_arg ,
    double y_meters_arg ,
    float heading_radians_arg ,
    float vx_meters_per_second_arg ,
    float vy_meters_per_second_arg 
    ):
    x_meters { std::move(x_meters_arg) } // NOLINT
      ,
    y_meters { std::move(y_meters_arg) } // NOLINT
      ,
    heading_radians { std::move(heading_radians_arg) } // NOLINT
      ,
    vx_meters_per_second { std::move(vx_meters_per_second_arg) } // NOLINT
      ,
    vy_meters_per_second { std::move(vy_meters_per_second_arg) } // NOLINT
      
    {
}

KinematicMean::KinematicMean(const KinematicMean&) = default;

// NOLINTNEXTLINE
KinematicMean::KinematicMean(KinematicMean&&) = default;

KinematicMean& KinematicMean::operator=(const KinematicMean&) = default;

// NOLINTNEXTLINE
KinematicMean& KinematicMean::operator=(KinematicMean&&) = default;

void KinematicMean::toCsv(std::ostream& out) const
{
    out.precision(16);
    out << std::boolalpha;
    out << x_meters;out << ",";
    out << y_meters;out << ",";
    out << heading_radians;out << ",";
    out << vx_meters_per_second;out << ",";
    out << vy_meters_per_second;
    
}

std::stringstream KinematicMean::toStringstream() const
{
    std::stringstream out;
    toCsv(out);
    return out;
}

template<class Archive>
    void KinematicMean::serialize(Archive& ar, [[maybe_unused]] unsigned int version)
    {

        ar& CEREAL_NVP(x_meters);
        ar& CEREAL_NVP(y_meters);
        ar& CEREAL_NVP(heading_radians);
        ar& CEREAL_NVP(vx_meters_per_second);
        ar& CEREAL_NVP(vy_meters_per_second);
    }


template void KinematicMean::serialize(cereal::BinaryInputArchive& ar, unsigned int version);
template void KinematicMean::serialize(cereal::BinaryOutputArchive& ar, unsigned int version);
template void KinematicMean::serialize(cereal::IntrospectionArchive& ar, unsigned int version);
template void KinematicMean::serialize(cereal::JSONInputArchive& ar, unsigned int version);
template void KinematicMean::serialize(cereal::JSONOutputArchive& ar, unsigned int version);
template void KinematicMean::serialize(cereal::PortableBinaryInputArchive& ar, unsigned int version);
template void KinematicMean::serialize(cereal::PortableBinaryOutputArchive& ar, unsigned int version);

std::ostream& operator<< (std::ostream& os, const KinematicMean& kinematicmean)
{
    kinematicmean.toCsv(os);
    return os;
}

bool operator== (const KinematicMean &lhs, const KinematicMean &rhs)
{
    return (  lhs.x_meters== rhs.x_meters &&
     lhs.y_meters== rhs.y_meters &&
     lhs.heading_radians== rhs.heading_radians &&
     lhs.vx_meters_per_second== rhs.vx_meters_per_second &&
     lhs.vy_meters_per_second== rhs.vy_meters_per_second 
    );
}

std::size_t KinematicMean::hash() const noexcept
{
    std::size_t result = 17;
    result = result * 31 + std::hash<double>()(x_meters);
    result = result * 31 + std::hash<double>()(y_meters);
    result = result * 31 + std::hash<float>()(heading_radians);
    result = result * 31 + std::hash<float>()(vx_meters_per_second);
    result = result * 31 + std::hash<float>()(vy_meters_per_second);
    return result;
}

 } // namespace av
 } // namespace msg
 } // namespace tracker


CEREAL_BIND_TO_ARCHIVES(av::msg::tracker::KinematicMean)

 namespace av {
 namespace msg {
 namespace tracker {


KinematicCovariance::KinematicCovariance() = default;

KinematicCovariance::KinematicCovariance(
    float x_x_arg ,
    float x_y_arg ,
    float y_y_arg 
    ):
    x_x { std::move(x_x_arg) } // NOLINT
      ,
    x_y { std::move(x_y_arg) } // NOLINT
      ,
    y_y { std::move(y_y_arg) } // NOLINT
      
    {
}

KinematicCovariance::KinematicCovariance(const KinematicCovariance&) = default;

// NOLINTNEXTLINE
KinematicCovariance::KinematicCovariance(KinematicCovariance&&) = default;

KinematicCovariance& KinematicCovariance::operator=(const KinematicCovariance&) = default;

// NOLINTNEXTLINE
KinematicCovariance& KinematicCovariance::operator=(KinematicCovariance&&) = default;

void KinematicCovariance::toCsv(std::ostream& out) const
{
    out.precision(16);
    out << std::boolalpha;
    out << x_x;out << ",";
    out << x_y;out << ",";
    out << y_y;
    
}

std::stringstream KinematicCovariance::toStringstream() const
{
    std::stringstream out;
    toCsv(out);
    return out;
}

template<class Archive>
    void KinematicCovariance::serialize(Archive& ar, [[maybe_unused]] unsigned int version)
    {

        ar& CEREAL_NVP(x_x);
        ar& CEREAL_NVP(x_y);
        ar& CEREAL_NVP(y_y);
    }


template void KinematicCovariance::serialize(cereal::BinaryInputArchive& ar, unsigned int version);
template void KinematicCovariance::serialize(cereal::BinaryOutputArchive& ar, unsigned int version);
template void KinematicCovariance::serialize(cereal::IntrospectionArchive& ar, unsigned int version);
template void KinematicCovariance::serialize(cereal::JSONInputArchive& ar, unsigned int version);
template void KinematicCovariance::serialize(cereal::JSONOutputArchive& ar, unsigned int version);
template void KinematicCovariance::serialize(cereal::PortableBinaryInputArchive& ar, unsigned int version);
template void KinematicCovariance::serialize(cereal::PortableBinaryOutputArchive& ar, unsigned int version);

std::ostream& operator<< (std::ostream& os, const KinematicCovariance& kinematiccovariance)
{
    kinematiccovariance.toCsv(os);
    return os;
}

bool operator== (const KinematicCovariance &lhs, const KinematicCovariance &rhs)
{
    return (  lhs.x_x== rhs.x_x &&
     lhs.x_y== rhs.x_y &&
     lhs.y_y== rhs.y_y 
    );
}

std::size_t KinematicCovariance::hash() const noexcept
{
    std::size_t result = 17;
    result = result * 31 + std::hash<float>()(x_x);
    result = result * 31 + std::hash<float>()(x_y);
    result = result * 31 + std::hash<float>()(y_y);
    return result;
}

 } // namespace av
 } // namespace msg
 } // namespace tracker


CEREAL_BIND_TO_ARCHIVES(av::msg::tracker::KinematicCovariance)

 namespace av {
 namespace msg {
 namespace tracker {


Waypoint::Waypoint() = default;

Waypoint::Waypoint(
    int64_t utime_microseconds_arg ,
    KinematicMean kinematic_mean_arg ,
    KinematicCovariance kinematic_covariance_arg 
    ):
    utime_microseconds { std::move(utime_microseconds_arg) } // NOLINT
      ,
    kinematic_mean { std::move(kinematic_mean_arg) } // NOLINT
      ,
    kinematic_covariance { std::move(kinematic_covariance_arg) } // NOLINT
      
    {
}

Waypoint::Waypoint(const Waypoint&) = default;

// NOLINTNEXTLINE
Waypoint::Waypoint(Waypoint&&) = default;

Waypoint& Waypoint::operator=(const Waypoint&) = default;

// NOLINTNEXTLINE
Waypoint& Waypoint::operator=(Waypoint&&) = default;

void Waypoint::toCsv(std::ostream& out) const
{
    out.precision(16);
    out << std::boolalpha;
    out << utime_microseconds;out << ",";
    out << kinematic_mean;out << ",";
    out << kinematic_covariance;
    
}

std::stringstream Waypoint::toStringstream() const
{
    std::stringstream out;
    toCsv(out);
    return out;
}

template<class Archive>
    void Waypoint::serialize(Archive& ar, [[maybe_unused]] unsigned int version)
    {

        ar& CEREAL_NVP(utime_microseconds);
        ar& CEREAL_NVP(kinematic_mean);
        ar& CEREAL_NVP(kinematic_covariance);
    }


template void Waypoint::serialize(cereal::BinaryInputArchive& ar, unsigned int version);
template void Waypoint::serialize(cereal::BinaryOutputArchive& ar, unsigned int version);
template void Waypoint::serialize(cereal::IntrospectionArchive& ar, unsigned int version);
template void Waypoint::serialize(cereal::JSONInputArchive& ar, unsigned int version);
template void Waypoint::serialize(cereal::JSONOutputArchive& ar, unsigned int version);
template void Waypoint::serialize(cereal::PortableBinaryInputArchive& ar, unsigned int version);
template void Waypoint::serialize(cereal::PortableBinaryOutputArchive& ar, unsigned int version);

std::ostream& operator<< (std::ostream& os, const Waypoint& waypoint)
{
    waypoint.toCsv(os);
    return os;
}

bool operator== (const Waypoint &lhs, const Waypoint &rhs)
{
    return (  lhs.utime_microseconds== rhs.utime_microseconds &&
     lhs.kinematic_mean== rhs.kinematic_mean &&
     lhs.kinematic_covariance== rhs.kinematic_covariance 
    );
}

std::size_t Waypoint::hash() const noexcept
{
    std::size_t result = 17;
    result = result * 31 + std::hash<int64_t>()(utime_microseconds);
    result = result * 31 + std::hash<KinematicMean>()(kinematic_mean);
    result = result * 31 + std::hash<KinematicCovariance>()(kinematic_covariance);
    return result;
}

 } // namespace av
 } // namespace msg
 } // namespace tracker


CEREAL_BIND_TO_ARCHIVES(av::msg::tracker::Waypoint)

 namespace av {
 namespace msg {
 namespace tracker {


Trajectory::Trajectory() = default;

Trajectory::Trajectory(
    float probability_arg ,
    std::vector<Waypoint> waypoints_arg 
    ):
    probability { std::move(probability_arg) } // NOLINT
      ,
    waypoints { std::move(waypoints_arg) } // NOLINT
      
    {
}

Trajectory::Trajectory(const Trajectory&) = default;

// NOLINTNEXTLINE
Trajectory::Trajectory(Trajectory&&) = default;

Trajectory& Trajectory::operator=(const Trajectory&) = default;

// NOLINTNEXTLINE
Trajectory& Trajectory::operator=(Trajectory&&) = default;

void Trajectory::toCsv(std::ostream& out) const
{
    out.precision(16);
    out << std::boolalpha;
    out << probability;out << ",";
    // Enclose sequence in double quotations and curly brackets to keep CSV line length uniform
        
        out << "\"{{";
        
        if (!waypoints.empty())
        {
            std::copy(waypoints.begin(), waypoints.end() - 1, std::ostream_iterator<Waypoint>(out, "},{"));
            std::copy(waypoints.end()-1, waypoints.end(), std::ostream_iterator<Waypoint>(out, ""));
        }
        out << "}}\"";
    
}

std::stringstream Trajectory::toStringstream() const
{
    std::stringstream out;
    toCsv(out);
    return out;
}

template<class Archive>
    void Trajectory::serialize(Archive& ar, [[maybe_unused]] unsigned int version)
    {

        ar& CEREAL_NVP(probability);
        ar& CEREAL_NVP(waypoints);
    }


template void Trajectory::serialize(cereal::BinaryInputArchive& ar, unsigned int version);
template void Trajectory::serialize(cereal::BinaryOutputArchive& ar, unsigned int version);
template void Trajectory::serialize(cereal::IntrospectionArchive& ar, unsigned int version);
template void Trajectory::serialize(cereal::JSONInputArchive& ar, unsigned int version);
template void Trajectory::serialize(cereal::JSONOutputArchive& ar, unsigned int version);
template void Trajectory::serialize(cereal::PortableBinaryInputArchive& ar, unsigned int version);
template void Trajectory::serialize(cereal::PortableBinaryOutputArchive& ar, unsigned int version);

std::ostream& operator<< (std::ostream& os, const Trajectory& trajectory)
{
    trajectory.toCsv(os);
    return os;
}

bool operator== (const Trajectory &lhs, const Trajectory &rhs)
{
    return (  lhs.probability== rhs.probability &&
     lhs.waypoints== rhs.waypoints 
    );
}

std::size_t Trajectory::hash() const noexcept
{
    std::size_t result = 17;
    result = result * 31 + std::hash<float>()(probability);
    result = result * 31 + hashVector(waypoints);
    return result;
}

 } // namespace av
 } // namespace msg
 } // namespace tracker


CEREAL_BIND_TO_ARCHIVES(av::msg::tracker::Trajectory)

 namespace av {
 namespace msg {
 namespace tracker {


Attributes::Attributes() = default;

Attributes::Attributes(
    int64_t id_arg ,
    OBJECT_TYPE class_type_arg ,
    std::array<float,6> class_type_probabilities_arg ,
    float existence_confidence_arg ,
    float probability_active_arg ,
    BoxDimensions3D dimensions_arg 
    ):
    id { std::move(id_arg) } // NOLINT
      ,
    class_type { std::move(class_type_arg) } // NOLINT
      ,
    class_type_probabilities { std::move(class_type_probabilities_arg) } // NOLINT
      ,
    existence_confidence { std::move(existence_confidence_arg) } // NOLINT
      ,
    probability_active { std::move(probability_active_arg) } // NOLINT
      ,
    dimensions { std::move(dimensions_arg) } // NOLINT
      
    {
}

Attributes::Attributes(const Attributes&) = default;

// NOLINTNEXTLINE
Attributes::Attributes(Attributes&&) = default;

Attributes& Attributes::operator=(const Attributes&) = default;

// NOLINTNEXTLINE
Attributes& Attributes::operator=(Attributes&&) = default;

void Attributes::toCsv(std::ostream& out) const
{
    out.precision(16);
    out << std::boolalpha;
    out << id;out << ",";
    out << static_cast<uint32_t>(class_type);out << ",";
    // Enclose array in double quotations and curly brackets to keep CSV line length uniform
        
        out << "\"{{";
        
        if (!class_type_probabilities.empty())
        {
            std::copy(class_type_probabilities.begin(), class_type_probabilities.end() - 1, std::ostream_iterator<float>(out, "},{"));
            out << class_type_probabilities.back();
        }
        out << "}}\"";out << ",";
    out << existence_confidence;out << ",";
    out << probability_active;out << ",";
    out << dimensions;
    
}

std::stringstream Attributes::toStringstream() const
{
    std::stringstream out;
    toCsv(out);
    return out;
}

template<class Archive>
    void Attributes::serialize(Archive& ar, [[maybe_unused]] unsigned int version)
    {

        ar& CEREAL_NVP(id);
        ar& CEREAL_NVP(class_type);
        ar& CEREAL_NVP(class_type_probabilities);
        ar& CEREAL_NVP(existence_confidence);
        ar& CEREAL_NVP(probability_active);
        ar& CEREAL_NVP(dimensions);
    }


template void Attributes::serialize(cereal::BinaryInputArchive& ar, unsigned int version);
template void Attributes::serialize(cereal::BinaryOutputArchive& ar, unsigned int version);
template void Attributes::serialize(cereal::IntrospectionArchive& ar, unsigned int version);
template void Attributes::serialize(cereal::JSONInputArchive& ar, unsigned int version);
template void Attributes::serialize(cereal::JSONOutputArchive& ar, unsigned int version);
template void Attributes::serialize(cereal::PortableBinaryInputArchive& ar, unsigned int version);
template void Attributes::serialize(cereal::PortableBinaryOutputArchive& ar, unsigned int version);

std::ostream& operator<< (std::ostream& os, const Attributes& attributes)
{
    attributes.toCsv(os);
    return os;
}

bool operator== (const Attributes &lhs, const Attributes &rhs)
{
    return (  lhs.id== rhs.id &&
     lhs.class_type== rhs.class_type &&
     lhs.class_type_probabilities== rhs.class_type_probabilities &&
     lhs.existence_confidence== rhs.existence_confidence &&
     lhs.probability_active== rhs.probability_active &&
     lhs.dimensions== rhs.dimensions 
    );
}

std::size_t Attributes::hash() const noexcept
{
    std::size_t result = 17;
    result = result * 31 + std::hash<int64_t>()(id);
    result = result * 31 + std::hash<OBJECT_TYPE>()(class_type);
    result = result * 31 + hashVector(class_type_probabilities);
    result = result * 31 + std::hash<float>()(existence_confidence);
    result = result * 31 + std::hash<float>()(probability_active);
    result = result * 31 + std::hash<BoxDimensions3D>()(dimensions);
    return result;
}

 } // namespace av
 } // namespace msg
 } // namespace tracker


CEREAL_BIND_TO_ARCHIVES(av::msg::tracker::Attributes)

 namespace av {
 namespace msg {
 namespace tracker {


Track::Track() = default;

Track::Track(
    Attributes attributes_arg ,
    std::vector<Trajectory> trajectories_arg 
    ):
    attributes { std::move(attributes_arg) } // NOLINT
      ,
    trajectories { std::move(trajectories_arg) } // NOLINT
      
    {
}

Track::Track(const Track&) = default;

// NOLINTNEXTLINE
Track::Track(Track&&) = default;

Track& Track::operator=(const Track&) = default;

// NOLINTNEXTLINE
Track& Track::operator=(Track&&) = default;

void Track::toCsv(std::ostream& out) const
{
    out.precision(16);
    out << std::boolalpha;
    out << attributes;out << ",";
    // Enclose sequence in double quotations and curly brackets to keep CSV line length uniform
        
        out << "\"{{";
        
        if (!trajectories.empty())
        {
            std::copy(trajectories.begin(), trajectories.end() - 1, std::ostream_iterator<Trajectory>(out, "},{"));
            std::copy(trajectories.end()-1, trajectories.end(), std::ostream_iterator<Trajectory>(out, ""));
        }
        out << "}}\"";
    
}

std::stringstream Track::toStringstream() const
{
    std::stringstream out;
    toCsv(out);
    return out;
}

template<class Archive>
    void Track::serialize(Archive& ar, [[maybe_unused]] unsigned int version)
    {

        ar& CEREAL_NVP(attributes);
        ar& CEREAL_NVP(trajectories);
    }


template void Track::serialize(cereal::BinaryInputArchive& ar, unsigned int version);
template void Track::serialize(cereal::BinaryOutputArchive& ar, unsigned int version);
template void Track::serialize(cereal::IntrospectionArchive& ar, unsigned int version);
template void Track::serialize(cereal::JSONInputArchive& ar, unsigned int version);
template void Track::serialize(cereal::JSONOutputArchive& ar, unsigned int version);
template void Track::serialize(cereal::PortableBinaryInputArchive& ar, unsigned int version);
template void Track::serialize(cereal::PortableBinaryOutputArchive& ar, unsigned int version);

std::ostream& operator<< (std::ostream& os, const Track& track)
{
    track.toCsv(os);
    return os;
}

bool operator== (const Track &lhs, const Track &rhs)
{
    return (  lhs.attributes== rhs.attributes &&
     lhs.trajectories== rhs.trajectories 
    );
}

std::size_t Track::hash() const noexcept
{
    std::size_t result = 17;
    result = result * 31 + std::hash<Attributes>()(attributes);
    result = result * 31 + hashVector(trajectories);
    return result;
}

 } // namespace av
 } // namespace msg
 } // namespace tracker


CEREAL_BIND_TO_ARCHIVES(av::msg::tracker::Track)




 namespace av {
 namespace msg {
 namespace tracker {


tracked_objects_avt::tracked_objects_avt() = default;

tracked_objects_avt::tracked_objects_avt(
    int64_t utime_microseconds_arg ,
    std::string frame_id_arg ,
    std::vector<Track> vehicles_arg ,
    std::vector<Track> pedestrians_arg ,
    std::vector<Track> bicyclists_arg 
    ):
    utime_microseconds { std::move(utime_microseconds_arg) } // NOLINT
      ,frame_id { std::move(frame_id_arg) } // NOLINT
      ,vehicles { std::move(vehicles_arg) } // NOLINT
      ,pedestrians { std::move(pedestrians_arg) } // NOLINT
      ,bicyclists { std::move(bicyclists_arg) } // NOLINT
      {
}

tracked_objects_avt::tracked_objects_avt(const tracked_objects_avt&) = default;

// NOLINTNEXTLINE
tracked_objects_avt::tracked_objects_avt(tracked_objects_avt&&) = default;

tracked_objects_avt& tracked_objects_avt::operator=(const tracked_objects_avt&) = default;

// NOLINTNEXTLINE
tracked_objects_avt& tracked_objects_avt::operator=(tracked_objects_avt&&) = default;

void tracked_objects_avt::toCsv(std::ostream& out) const
{
    out.precision(16);
    out << std::boolalpha;
    out << utime_microseconds;out << ",";
    out << "\"";
        out << frame_id;
        out << "\"";out << ",";
    // Enclose sequence in double quotations and curly brackets to keep CSV line length uniform
        
        out << "\"{{";
        
        if (!vehicles.empty())
        {
            std::copy(vehicles.begin(), vehicles.end() - 1, std::ostream_iterator<Track>(out, "},{"));
            std::copy(vehicles.end()-1, vehicles.end(), std::ostream_iterator<Track>(out, ""));
        }
        out << "}}\"";out << ",";
    // Enclose sequence in double quotations and curly brackets to keep CSV line length uniform
        
        out << "\"{{";
        
        if (!pedestrians.empty())
        {
            std::copy(pedestrians.begin(), pedestrians.end() - 1, std::ostream_iterator<Track>(out, "},{"));
            std::copy(pedestrians.end()-1, pedestrians.end(), std::ostream_iterator<Track>(out, ""));
        }
        out << "}}\"";out << ",";
    // Enclose sequence in double quotations and curly brackets to keep CSV line length uniform
        
        out << "\"{{";
        
        if (!bicyclists.empty())
        {
            std::copy(bicyclists.begin(), bicyclists.end() - 1, std::ostream_iterator<Track>(out, "},{"));
            std::copy(bicyclists.end()-1, bicyclists.end(), std::ostream_iterator<Track>(out, ""));
        }
        out << "}}\"";
    
}

std::stringstream tracked_objects_avt::toStringstream() const
{
    std::stringstream out;
    toCsv(out);
    return out;
}

template<class Archive>
    void tracked_objects_avt::serialize(Archive& ar, [[maybe_unused]] unsigned int version)
    {

        ar& CEREAL_NVP(utime_microseconds);
        ar& CEREAL_NVP(frame_id);
        ar& CEREAL_NVP(vehicles);
        ar& CEREAL_NVP(pedestrians);
        ar& CEREAL_NVP(bicyclists);
    }


template void tracked_objects_avt::serialize(cereal::BinaryInputArchive& ar, unsigned int version);
template void tracked_objects_avt::serialize(cereal::BinaryOutputArchive& ar, unsigned int version);
template void tracked_objects_avt::serialize(cereal::IntrospectionArchive& ar, unsigned int version);
template void tracked_objects_avt::serialize(cereal::JSONInputArchive& ar, unsigned int version);
template void tracked_objects_avt::serialize(cereal::JSONOutputArchive& ar, unsigned int version);
template void tracked_objects_avt::serialize(cereal::PortableBinaryInputArchive& ar, unsigned int version);
template void tracked_objects_avt::serialize(cereal::PortableBinaryOutputArchive& ar, unsigned int version);

std::ostream& operator<< (std::ostream& os, const tracked_objects_avt& tracked_objects_avt)
{
    tracked_objects_avt.toCsv(os);
    return os;
}

bool operator== (const tracked_objects_avt &lhs, const tracked_objects_avt &rhs)
{
    return (  lhs.utime_microseconds== rhs.utime_microseconds &&
     lhs.frame_id== rhs.frame_id &&
     lhs.vehicles== rhs.vehicles &&
     lhs.pedestrians== rhs.pedestrians &&
     lhs.bicyclists== rhs.bicyclists 
    );
}

std::size_t tracked_objects_avt::hash() const noexcept
{
    std::size_t result = 17;
    result = result * 31 + std::hash<int64_t>()(utime_microseconds);
    result = result * 31 + std::hash<std::string>()(frame_id);
    result = result * 31 + hashVector(vehicles);
    result = result * 31 + hashVector(pedestrians);
    result = result * 31 + hashVector(bicyclists);
    return result;
}

 } // namespace av
 } // namespace msg
 } // namespace tracker


CEREAL_BIND_TO_ARCHIVES(av::msg::tracker::tracked_objects_avt)

